<document>

@* <a href=@talk-bpsim.html target=_top>Support for Business Process Redesign</a>

<include file=navigate>

%%%-- Simulation, Hypermedia and the Web</h1>


A major success factor of business process redesign projects
is the extent to which the organisation members themselves - both management
and staff - are actively involved in the redesign project.
One way to assess different redesign alternatives with respect to the
project goals is to model these alternatives and subject them
to a simulation study.
Visualization and animation of the simulation models can provide
insight into both the structure and the dynamics of the modelled alternatives.
Effectively presented, visualization and animation enhance
the involvement of the project members in the assessment
and validation of the proposed redesigns.
<p>
To support communication among all participants in the BPR project, we
advocate
an integrated presentation of redesign alternatives, visualized simulation
experiments and the results of simulation studies in a single active document.
In this paper we describe a framework enabling such integrated
presentations by allowing the embedding of simulation models and experiments
in hypermedia documents.
<p>
<i>Keywords and phrases</i>:
Business Process Redesign, Logistics-based Business Modelling,
Discrete Event Simulation, Hypermedia, Active Documents, Web.


<control>
<h4><a name=contents><a href=plan.txt>Contents</a></h4>
<menu>
<li> <a href=intro.html> Introduction </a>
<li> <a href=bpm.html>Business Process Modelling</a>
<li> <a href=soft.html>Software Support for BPR</a>
<li> <a href=model.html>Example -- Requests for Loans</a>
<li> <a href=hush.html>Embedding BPR in Hypermedia</a>
<li> <a href=conc.html> Conclusions </a>
<li> <a href=refs.html> References </a>
<li> <a href=appendix.html>Appendix</a>
</menu>
</control>

@* Introduction

Business Process Redesign (BPR) is a method to perform organizational change.
Instead of functions and departments it takes business processes as the main
subject to redesign the way work is done in a particular organization. 
[_Davenport90,Davenport93,Davenport94,Hammer90,Wastell94]
Davenport and Short [_Davenport90] argue that business process redesign 
should be combined with the capabilities of Information Technology (IT). 
They show that IT can play two roles in BPR:
on the one hand it is an enabler of new process structures. An example
would be the use of expert systems to assist bank clerks handling loan
requests. On the other hand IT can provide 
support for the implementation of new process designs.
By, for example, 
simulating the logistic aspects of the redesigned loan request process.
<p>
By modelling different redesign alternatives
and subjecting them to simulation
studies the participants in the business process redesign project
can assess
the alternatives with respect to the redesign project goals. Possible goals
include cost reduction, lead time reduction, raising output quality and
improving the quality of worklife. Visualization and animation of 
the simulation models can provide insight into both the structure and 
dynamics of the modelled alternatives and thus improve decision making.
<p>
In this paper we advocate an integrated presentation of redesign 
alternatives, visualized simulation experiments and the results of 
simulation studies in a single active document.
Our approach allows authors to write hypermedia
documents with embedded simulation models and experiments. These documents 
can be viewed on the Internet, or on a company Intranet, using an 
adapted Internet Web browser. 
<p>
Our system has been implemented using the 
<i>hush</i> [_Hush] hypermedia framework developed at the Vrije 
Universiteit. This framework provides support for multimedia user 
interfaces, a discrete event simulation library [_SIM] and embedding of 
applications in the Web [_Applications,Animate]. To model business 
processes we use Logistics-based Business Modelling (LBM) [_Gerrits95]. 
The class library BPSIM (Business Process Simulation) facilitates the 
modelling of business processes using LBM and provides additional support 
for embedding and running simulations on the Web.

@ Structure
In the following sections we will introduce
the concepts underlying logistics-based business modelling
and we will describe the software support for
simulating business processes, in particular the 
classes provided by the BPSIM library.
We will present an example illustrating the
construction of a business process model and its visualisation
in a hypermedia environment.
Then, we will briefly characterize the hypermedia environment
and its relation to the World Wide Web.
And, finally, we will discuss the merits and limitations of our 
approach.

@+ Business Modeling

Dramatic improvements in business performance 
and productivity may be achieved by critically
examining some of the rules 
that govern a business process.
Business Process Redesign (BPR) is the 
generic label for many emerging methodologies aimed at producing these 
improvements. Re-design 
implies that the current state of affairs is no longer acceptable and 
can no longer be refined or evolved.

The BPR perspective demands that its users understand the nature of the 
business they are in, and the processes that can be re-engineered. 
In order to achieve this understanding, 
part of the BPR approach is to map out an 
organisations processes. 
The model created during this phase is used to 
communicate the BPR team's ideas
and will also be used to plan changes to the current processes.

Following Davenport and Short [_Davenport90]
we define a <i>business process</i>
as <i>"a set of logically related tasks
performed to achieve some well-defined business
outcome"</i>.
It is important to note that business processes
have <i>customers</i> that, somehow, receive
the outcome of these processes.
Also, to stress the functional nature of
business processes, it may be remarked that business
processes often transgress departmental boundaries
within an organization.
<p>
Despite the importance of qualitative assessments
in BPR [_Hammer90], for modelling
we favor a more quantitative approach
for which we provide support by means of
a simulation library (BPSIM) based
on the Logistics-based Business Modelling method (LBM)
presented in [_Gerrits95].
<p>
Logistics-based modelling 
allows for analysing
the time spent in executing a business process.
The <i>product lead time</i> is defined as the time
that passes between the moment a customer orders
a product and the moment a product is delivered.
In more detail, we can distinguish between
<i>processing time</i>
(the time actually worked on a job or operation),
<i>queue time</i>
(the time a jobs waits for a resource to become available),
<i>setup time</i>
(the time that passes between the moment
a resource becomes available and the moment
work on the job is started),
<i>wait time</i>
(the time that is spent waiting for another job
to complete), and
<i>transport time</i>
(the time that is needed to move a job from
a resource at a certain location to a resource
at another location).
<p>
For a particular model,
measurements may be obtained by
running a series of simulations.
Based on an analysis of the simulation results
alternative models may be proposed.
For example, when the setup time for a job is
relatively large, combining jobs into a single task
for an employee may be more efficient.

@ Modelling Business Processes
The LBM method offers a number of primitives,
with associated graphic icons,
from which a business process model may be constructed
as a network of resources 
connected by transport arcs.
The primitive entities offered by LBM are:
<ul>
<li> <i>operation</i> -- 
processing component
<li> <i>task</i> -- a series of consecutive operations
<li> <i>transport</i> -- transport of information
<li> <i>choice</i> --  affects flow of information
</ul>
Operations are atomic in the sense that wait time,
queue time and transport time may not be part 
of an operation.
Only setup time and process time are part of an operation.

Tasks are introduced to allow for a series of jobs 
or operations
to be processed, for example by one employee,
in order to reduce the setup time needed.

Transport entities represent the time it takes
for information to flow from one resource 
(that is operation or task) to another.
Transport implicitly defines the 
sequential structure of a process.
However, duplications of information, and consequently
parallel operations, are allowed.
<p>
In addition to the primitives mentioned above,
LBM allows us to characterize
<ul>
<li><i>organizational units</i> --
to represent departmental boundaries, 
<li> <i>external agents</i> --
to represent opaque information producing or
consuming entities,
and
<li><i>archives</i> --
to represent paper-based storage facilities.
</ul>
Also, employees may need additional
<i>means</i>
to engage in an operation or task.

@+ Process Simulation

The BPSIM library is an extension of the simulation library SIM [_SIM].
SIM is a C++ library offering classes supporting
discrete event simulation, based on standard simulation techniques
[_Watkins93].

In discrete event simulation,
the components of the model consists of events,
which are
activated at certain points in time and in this way affect the overall state
of the system.
The simulation library consists of the following classes:
<ul>
<li><i>simulation</i> -- the scheduler
<li><i>event</i> -- representing the events
<li><i>entity</i> -- process consisting of events
<li><i>generator</i> -- random distributions 
<li><i>resource</i> -- to model passive objects
<li><i>queue</i> -- to hold waiting events 
<li><i>histogram</i> -- to plot the results 
<li><i>analysis</i> -- for statistical analyses
</ul>
The SIM library is integrated with the <i>hush</i> library,
which may be used for defining a script interface to the simulation
package, for developing a graphical user interface
and for visualising simulation models.


@ The library BPSIM
The classes provided by the business process simulation
library BPSIM reflect the entities provided by the
logistics-based business modelling method LBM.
On a somewhat more abstract level,
a business process simulation  consists of <i>data</i>,
flowing through the process and different handlers -- 
<i>datahandlers</i> -- performing some action on the data,
such as transportation or specific operations.
<p>
Accordingly, BPSIM provides two  base classes underlying
the classes corresponding to LBM entities:
<ul>
<li> <i>data</i> -- which represents the product 
or case, i.e.
the information, that flows through the process. 
<li> <i>datahandler</i>  -- which is the basis 
for all classes that
handle information.
</ul>
Note that LBM contains no symbol for data as it only
depicts the objects that handle the information.

The following classes are derived from 
the <i>datahandler</i> class:
<em>operation</em>,
<em>transport</em>,
<em>waitqueue</em>,
<em>choice</em>,
<em>archive</em>, and
<em>external agent</em>,
which realize the corresponding entities in LBM.

<p>
An
<i>operation</i> takes time, and is executed by an employee. Sometimes 
an operation results in more than one outgoing dataflow,
for instance when it 
issues a request for additional information from a different 
department.
<p>
A <em>waitqueue</em> functions as a regular queue if 
it has one incoming dataflow, i.e. transport.
When there are more incoming
flows, it functions as a synchronized queue.
Data from one flow is not passed on to the
next datahandler until the data from the other flow has arrived.
This happens for instance when work on a case cannot continue
until additional information has arrived.
<p>
An external <i>agent</i> can either generate data
according to some random number distribution, or take
data, process it and pass it on to the next datahandler. In the latter 
case the agent functions as a black box: we only
care how long the processing takes, not how it is exactly performed.


<p>
In addition, BPSIM offers
the classes
<i>means</i> (which can be used to model resources 
that are necessary to perform certain operations), and
<em>employee</em> (which models the 
different people that perform the operations).

<p>
As one can see, the entities <em>task</em> and 
<em>organization unit</em> 
from LBM have no specific counterpart in BPSIM.
The reason for this is that having no other <i>datahandler</i>
between two operations already implies those
operations belong to the same task. Consequently they are
executed with no time in between and by the same employee. 
Also, the fact 
that tasks are executed in
different <em>organization units</em> does not add any 
information that changes
the behaviour of the simulation. If it takes time to transport
information between different units, 
then that time can be represented by 
the object of class <em>transport</em> between those units. 
<p>
For the gathering and analyzing of results, the SIM classes 
<em>histogram</em> and
<em>analysis</em> are used.
The class <em>agent</em>, for example, 
can be given a histogram to track the lead time
of data it has generated. 

@ A script interface for BPSIM
Employing the facilities of the <i>hush</i> library
a script interface has been defined for BPSIM
that allows the user, that is the designer of business models,
to construct and run business simulation models
with a short turn around time.
<p>
Each class in the BPSIM library corresponds to
a command in the script language.
As illustrated in the next section,
script commands result in a graphic
representation of model, which may be displayed
in a Web page as an applet.

@+ Requests for loans

Now let us take a look at an example business process model based on the objects made available in BPSIM. The example will detail possible situations before and after business process redesign has taken place.

<dl>
<dt>Current Situation:
<dd>Our first model, which represents the current situation,
consists of a client producing,
for example requests for a loan at a bank. The requests are initially handled by a clerk whose task is to send requests for amounts greater than 10000 dollars to the boss. The boss, whose task is to approve or denigh the loan, sends the result back to the clerk for processing. If the amount is less than 10000 dollars, the clerk has authority to process the request himself.


<dt>Redesign Alternative:
<dd>In our second model, which is the result of
a redesign effort, requests are handled by a clerk
who enters the data into a computer. The computer now makes the decision as to whether the loan is improved - for loans of value less than and greater than 10000 dollars. The clerk then passes the results on to another clerk for the task of dispatching loans.
</dl>

<slide id=model cap="Presenting a Redesign Alternative">
%%<!--\begin{figure*}[ht]-->
%%<center>

<image name=local/bpsim/combi2>
%%<!--
%%\epsfig{file=local/combi2.eps,width=12cm}
%%\caption{Presenting a Redesign Alternative}\label{fi:bm4}
%%-->
</slide>


<p>
Figure 1 shows the visual representation of
the model resulting from the redesign effort,
embedded in a Web page.
In addition to the model, the page contains
some results obtained by running a simulation.
For example, the text indicates that
an employee is busy handling incoming requests
only 33\% of the time.
The histogram depicts the distribution
of the lead times of incoming loan requests,
that is the time that passes between receiving
a request and giving an answer.
The Web page further contains a button to
start a simulation run,
a description of the model
and links to alternative models.


@ The script
Below we show some fragments of the script.

<program>
histogram leadtimes
agent client generate leadtimes
client -duration 60 poisson
transport t1 
t1 -duration 20
employee clerk
operation receive clerk
receive -duration 20.0 5.0 normal 
</program>
In this fragment a histogram for recording leadtimes
is created as well as an
agent generating requests and a transport to a 
clerk handling the requests.
The duration of the agent is specified
using a poisson distribution with $\lambda = 60$.
The clerk is declared to perform a receive operation,
the duration of which takes values from a 
normal distribution with $\mbox{\it means} = 20$ and 
$\mbox{\it standard deviation} = 5.0$.

<program>
client -next t1
t1 -next clerk
</program>
The fragment above shows how the various components
of a model (for this fragment the agent, transport and
employee) may be connected.
<p>
After defining the components of the model
and their connection a simulation may be started by
pressing the start button.
If desired, users may change the parameters concerning
for example setup and wait time, to explore
the various scenarios a model has to satisfy.

@+ The  environment

Given the importance of participant communication in
a business process redesign effort,
embedding business simulation models  in hypermedia
provides in a natural way the information context
needed for exploring alternatives and taking decisions.
With Intranets becoming more popular, the World Wide Web
has proven to be a popular access point for many applications.
<p>
The <i>hush</i> library and its extensions offer
a rich environment for developing
hypermedia applications and dynamic Web documents
[_Applications].
<p>
The <i>hush</i> library originated as a C++
interface to the GUI scripting environment
provided by Tcl/Tk [_Hush].
Tcl/Tk offers an extensible script language
with powerful graphics and window programming
utilities.
A number of extensions to <i>hush</i> have been written,
including the SIM and BPSIM libraries
as well as libraries encapsulating multimedia
facilities.
The <i>hush</i> Web component provides a
collection of (inline) data viewers for the <i>hush</i>
Web browser.
It supports full SGML and style sheets
to specify dynamic layout properties in a declarative way [_Animate].
Also <i>applets</i> are supported,
which are Tcl/Tk scripts augmented with the functionality
of <i>hush</i> applications.
Applets are the building blocks of active Web documents.
<p>
By defining a script interface for application-specific
C++ classes, application functionality may be embedded
in a Web page as an applet.
As illustrated in the example,
our approach to providing hypermedia and Web support
employs this feature,
both to reduce the time involved in modelling and
to present a graphical representation of the model
and its execution to the user.

@+ Conclusions

As observed by Wastell [_Wastell94]
organisational change is a ``highly threatening and stressful experience for many participants and that high levels of stress can have a pernicious effect on individuals, group processes and organizational learning''.
Hence,
directly involving the users in the modelling phase of the BPR project may be an 
important step towards capturing the human aspects that are necessary for the production of an optimal model. It may also help to lessen the anxiety of employees.  
<p>
Our contribution in this area is to provide
support for an information context
that encompasses both technical and social dimensions
of the re-engineering effort.
On a technological level, we advocate the use of
business process simulation.
Nevertheless, to accomodate the social aspects,
we support the visualisation of such models
and their integration in an arbitrarily complex
information context, such as the World Wide Web.
<p>
As concerns the limitations of our approach,
we wish to remark that
we have only realized part of the visualisation and
animation facilities that we consider desirable.
Furthermore, in our current realization
the user is required to use the <i>hush</i>
Web browser, instead of a popular browser such as Netscape.
<p>
Developing visualisation and animation support
for simulation models is a topic
of ongoing research.
To deal with the problem of accessibility
we have to adopt a new technology,
that is use for example Java, or develop
a Netscape plugin for <i>hush</i> applications.

</document>
