<document>
<h1>Example -- simulation on the Web</h1>
<include file=navigate>
As an immediate benefit of our component-based approach,
client-side extensions to the Web, such as inline MPEG, 
interactive games and music synthesis [_WWW95b] can be easily realized.
See [_DoornEl95b] for more illustrations and a discussion of safety issues.
<h4>Embedding simulations</h4>
From the perspective of our simulation package,
we can use client-side computing techniques to provide
hypermedia interfaces for existing simulation 
applications. For instance, in figure <ref>Dining</ref>, a
simulation of the "Dining Philosophers" problem
has been embedded into an HTML page to demonstrate
our multi-lingual approach.


<slide cap="Dining Philosophers">
<image name=local/sim/webdinner><br>
</slide>

<p>
Figure <a href=#Dining><ref>Dining</ref></a> shows the corresponding HTML markup for the first page.
We use two <tt>&lt;app&gt;</tt> tags to include inline
applets written in Tcl/Tk. Note that text inside the open
and close tag will be ignored by our browser
and may be used to display warnings if the
page is browsed by non-hush browsers as Mosaic or Netscape.
<pre>
  &lt;title&gt;The Dining Philosophers&lt;title&gt;
  &lt;h1&gt;The Dining Philosophers&lt;h1&gt;
  &lt;h2&gt;The problem&lt;h2&gt;
    Five philosophers sit around a table, with five chopsticks in between.
    ...
    We are interested in &lt;a href="results.html"&gt;the percentage of the time&lt;a&gt; a 
    philosopher actually thinks. 
  &lt;app class="sim-setup"&gt;
    Error: Dining table deleted: use our hush browser!
  &lt;/app&gt; &lt;hr&gt;
  Press this button: &lt;app class="sim-button"&gt;
    Error: Run button deleted: use our hush browser!
  &lt;/app&gt;
</pre><br>
<!--
%%Figure [YYY]. HTML markup with additional hush tag.
-->
Since hush programmers can easily define Tcl interfaces 
for functionality provided by C functions or C++ classes, 
applications written in these languages  can be embedded 
in Web pages as well.
<p>
The Tcl/Tk applets used in the example above 
are in fact only responsible for the graphical interface,
the simulation itself is modeled in C++, by refining 
the <tt>entity</tt> class (modeling a philosopher) and
the <tt>resource</tt> class (modeling a chopstick)
of the simulation library.
<p>
The simulation is initialized by scheduling five
philosopher entity objects in a "waiting" phase at time 
<tt>t=0.0</tt>.
The "simulation init" command will be executed by first applet.
The simulation can be started by executing the "simulation run" command,
which is bound on a push button by the sim-button applet.
<p>
When a philosopher is due to be activated, 
the scheduler will call its application operator which will
perform an explicit dispatch on the current phase:
<pre>
  int philosopher::operator()() {
      switch (phase()){
        case EATING   : eat();   return OK;
        case THINKING : think(); return OK;
        case WAITING  : await(); return OK;
        default: return FALSE; // undefined phase
      } 
  }
</pre>
<p>
Suppose the philosopher is still in a waiting phase.
The await() member will be called and the philosopher
will see whether his chopsticks are available.
If so, she will acquire them and starts eating.
If not, the philosopher will remain in a waiting
state, but is put on the scheduler's list of conditional
events.
  <pre>
  void philosopher::await() {
    if (chopsticks_available()) {
        acquirechopsticks();
  <!-- %% sim->passivate(this); -->
        phase(EATING);
  <!-- %% sim->activate(this); -->
    } else if (!conditional()) {
        waitonchopsticks();
        sim->hold(this);
    }
  }
</pre>
<p>
However, if the philosopher is invoked while she is in a eating
phase, she will continue with eating for some (exponentially distributed)
random time interval after which she will move to a "thinking" phase.
<pre>
  void philosopher::eat() {
    double t = gen->exponential(EAT_TIME);
    phase(THINKING);
    sim->wait(t);
  }
</pre>

<h4>Multimedia Scheduling</h4>
From the perspective of our hypermedia framework,
we benefit from the simulation package by
having an efficient scheduler at our disposal
and hence a convenient way to employ application-defined events.
<p>
In addition, we extended the simulation package with a soft-real time option,
that allows us to schedule clock-synchronized events.
For example, we implemented a simple slide show, 
by the repetitive scheduling of HTML pages with a fixed time interval. 
<p>
However, general hypermedia applications require support
for far more complex synchronization relations and temporal dependencies
between the components of a hypermedia document.

Such high-level temporal alignment cannot be (easily) expressed 
using the basic simulation primitives, nor can the
corresponding documents be expressed
by (text oriented) markup languages as HTML.
<p>
Therefor, we are currently developing components supporting 
more complex scheduling primitives.
One of our goals is to add
a (subset of) the event scheduling mechanisms of HyTime [REFXXX]
to our new, SGML-based Web widget. 
</document>
