<document>

@* Animating the Web -- an SGML-based approach


The DejaVu framework offers access to the Web as
one of the facilities in a collection
of software components for developing 
multimedia user interfaces and hypermedia
applications.
It provides a vehicle
for experimenting with true hypermedia extensions
to the Web.

Our approach, which is essentially object-oriented,
allows for integrating the Web in applications
and vice versa, for composing a Web browser with support
for sound synthesis, digital video,
3D graphics and virtual reality.

On a software level this is realized by extending
HTML to allow for embedded scripts,
written in a script language that is extensible
with application-defined functionality.
However, in addition to this rather low-level
provision for active documents, our Web browser
component allows for SGML-defined document types
capturing the syntax and semantics of multimedia
extensions.

This paper gives a brief overview of the various
components constituting the DejaVu framework
and discusses briefly the architectural notions underlying our framework.
We will show
how to employ these components to develop
a Web browser with (client-side) support
for animation, multimedia and virtual reality.
By giving some examples, we will illustrate
our solution to defining the (browsing)
semantics of arbitrary document types by means of script-based
style sheets.
As a conclusion we will discuss the requirements
that must be met for developing
a HyTime compliant browser for the Web.


@+  Introduction
    
 <p>
In a relatively short period of time the World Wide Web has
become a prominent vehicle for disseminating information.  The
reason for its popularity is not so much its functionality per
se, but rather the convenience with which it provides access to a
variety of information sources.  In fact, the functionality of
the Web, especially with respect to multimedia, is rather
disappointing with regard to the research in this area [_AHM].
Another point of criticism is that the HTML document encoding
language provides a poor vehicle for structuring information,
since it is primarily geared towards defining the (graphical)
layout of small documents.

    <p>
Clearly there is a need to enhance the Web with dynamic
capabilities (for example to display multimedia and to allow for
interaction with the user) and with facilities for structuring
information, as advocated by the SGML standard [_SGML].  Our
research is meant to provide an answer to both problems.  We
allow for extending a browser with dynamic capabilities by
support for client-side computing, that is ``active documents''.
New document structures are supported by allowing for arbitrary
SGML-based document types, including extensions to HTML.

    <p>
Another point of concern in the Web community is that with the
increasing number of document formats, traditional monolithic
browsers (such as Netscape) become too large and too inflexible
to adapt to the needs of an individual user.  In this respect our
project takes a radically different approach, by allowing for a
close integration between application-functionality and
Web-functionality.

    <p>
The DejaVu framework offers access to the Web as one of the
facilities in a collection of software components for developing
multimedia user interfaces and hypermedia applications.  It
provides a vehicle for experimenting with true hypermedia
extensions to the Web.  Our approach, which is essentially
object-oriented, allows for integrating the Web in applications
and vice versa, for composing a Web browser with support for
sound synthesis, digital video, 3D graphics and virtual reality
[_Eliens95].
On a software level this is realized by extending HTML to allow
for embedded scripts, written in a script language that is
extensible with application-defined functionality.  However, in
addition to this rather low-level provision for active documents,
our Web browser component allows for SGML-defined document types
capturing the syntax and semantics of multimedia extensions.

    <h4>Related work</h4>

    <p>With the announcement of Java [_Java], and the realization of
the HotJava browser, client-side computing has been accepted as a
powerful and flexible means to enhance the dynamic capabilities
of Web-browsers.  Another example of such an approach is realized
by the Python-based Grail browser [_Python].  SGML-based browsers
are offered by a number of commercial browser vendors
[_DynaText,Panorama], some stemming from long-standing academic
research in electronic document processing [_Grif].  We believe our
approach to be unique in the sense that it realizes a software
engineering framework that allows for a component-based approach
to integrating application-functionality and Web-functionality.
    
    <h4>Structure</h4>

    <p>We will first give an overview of the various components
constituting the DejaVu framework and discuss briefly the
architectural notions underlying our framework.  Then, we will
show how to employ these components to develop a Web browser with
(client-side) support for animation, multimedia and virtual
reality. Next, we will describe the concepts of encoding
documents with SGML, and by giving some examples, we will
illustrate our solution to defining the (browsing) semantics of
arbitrary document types by means of script-based style sheets.
And finally, we will discuss the requirements that must be met
for developing a HyTime compliant browser for the Web.
    
    

@+ DejaVU -- The Framework
    <!-- \label{dejavu} -->
    
    <p>The problem addressed in the DejaVU project is essentially to
provide support for the software engineering of (moderately complex)
applications that require, apart from other functionality, multimedia
interfaces and associative online help.  Such support is primarily
offered in the form of class libraries, most of which are written in
C++.  The language C++ was chosen for pragmatic reasons, since it
allows for easily wrapping public domain software written in C in
classes with a more user-friendly interface.

    <p>
Application development generally encompasses a variety of programming
tasks, including system-level software development (for example for
networking or multimedia functionality), programming the user
interface (including the definition of screen layout and the
responsivity of the interface widgets to user actions), and the
definition of (high-level) application-specific functionality.  Each
of these kinds of tasks may require a different approach and possibly
a different application programming language.  For example, the
development of the user interface is often more conveniently done
using a scripting language, to avoid the waiting times involved in
compiling and linking.  Similarly, defining knowledge-level
application-specific functionality may benefit from the use of a
declarative or logic programming language &nbsp;[<cite><a href=refs.html#Eliens92>Eliens92</a></cite>].

    <p>
In our project, we decided from the start to support a multi-paradigm
approach to software development and consequently we had to define the
mutual interaction between the various language paradigms, as for
example the interaction between C++ and a scripting language, such as
Tcl &nbsp;[<cite><a href=refs.html#Tcl>Tcl</a></cite>].  Current scripting languages, including Python &nbsp;[<cite><a href=refs.html#Python>Python</a></cite>]
and Tcl, provide facilities for being embedded in C and C++, but
extending and employing these languages in C or C++ is rather
cumbersome.  The <i>hush</i> library &nbsp;[<cite><a href=refs.html#Eliens95b>Eliens95b</a></cite>] offers a uniform
interface to a number of script languages, and in addition, it offers
a variety of widgets and multimedia extensions, that are accessible
through any of the script interpreters as well as the C++ interface.

@- Idioms and Patterns
    
    <p>To realize the functionality of <i>hush</i> and its extensions, we
employed some basic idioms and patterns &nbsp;[<cite><a href=refs.html#GOF94>GOF94</a></cite>]
derived from the <i>handle/body</i> idiom
that was originally introduced in &nbsp;[<cite><a href=refs.html#Coplien92>Coplien92</a></cite>], notably the <i>virtual
 self-reference</i> idiom and the <i>dynamic role-switching</i> idiom.

    <h4>Virtual self-reference</h4>
    
    <p>The <i>virtual self-reference</i> idiom has been introduced to
support the development of compound widgets.  It allows for (re)using
the script and C++ interface of possibly compound widgets, by
employing explicit redirection to an inner or primary component.
Every widget can delegate part of its functionality to an
inner component. It is common practice to derive a compound widget
from another widget by using interface inheritance only, and to
delegate functionality to an inner component by explicit redirection.

    <h4>Dynamic role-switching</h4>
    
    <p>The <i>dynamic role-switching</i> idiom provides a means to offer a
multitude of functional modes simultaneously. For example, a single
kit object gives access to multiple (embedded) script interpreters, as
well as a remote kit.  The idiom is essential in supporting
the multi-paradigm nature of the DejaVu framework. In our description
of the design of the Web components in section <ref><a href=#Design>Design</a></ref>, we
will show how dynamic role-switching is employed for using various
network protocols via the same (net)client class. The idiom is also
used to define a (single) viewer class that is capable of displaying
documents of various MIME-types (including SGML, HTML and VRML).

 @+ The Components
    
    <p>The <i>hush</i> library contains the classes providing the
interface to the embedded script interpreter(s) and the classes
allowing for a smooth interaction with the underlying window system,
including the <i>kit</i>, <i>event</i>, <i>handler</i>, <i>widget</i> and
<i>item</i> classes.  In addition to these, <i>hush</i> provides the class
<i>session</i>.  Application programmers are minimally required to
derive a class, say <i>application</i>, from <i>session</i> and
redefine the function <i>main</i>.  When the program is to be used as
a windowing shell, the function <i>prelude</i> may be redefined to
allow for initializing external packages or other components of the
DejaVu framework.
The extensions of <i>hush</i> include
    <slide id=frame-1 cap="The <i>hush</i> extensions">
    <ul>
<li> <i>widgets</i> -- for creating graphical user interfaces
<li> <i>sim</i>   -- for discrete event simulation
<li> <i>hymne</i> -- for sound synthesis and music
<li> <i>video</i> -- for digital video
<li> <i>vr</i> -- for the OpenGL-based VRML extension
<li> <i>web</i>   -- for connecting to the Web
    </ul>
    </slide>
    
    
    <h4>User interface widgets</h4>
    
    <p>The <i>hush</i> widget library contains classes for a variety of
user interface widgets, including the classes <i>button</i>,
<i>menu</i>, <i>text</i>, <i>canvas</i>, <i>filechooser</i> and many
more &nbsp;[<cite><a href=refs.html#Eliens95b>Eliens95b</a></cite>].  A widget allows for binding an independently
defined handler to user actions.  Alternatively, a widget may declare
itself to be its own handler.  Allowing for delegating to an external
handler helps to avoid cluttering the class name space.  On the other
hand, employing inheritance may result in a significant reuse of
initialization code.

    <h4>Simulation modeling support</h4>
    
    <p>Our interest in discrete event simulation was accidentally aroused
by the need to provide support for Business Mathematics courses
&nbsp;[<cite><a href=refs.html#BolEl94>BolEl94</a></cite>].  The simulation package we developed is fully integrated
with <i>hush</i>, which allows the simulation developer to use the
animation and hypertextual facilities offered by <i>hush</i>.  The
package provides the following classes
    <slide id=frame-2 cap="Simulation classes">
    <ul>
<li><i>simulation</i> -- containing the scheduler
<li><i>event</i> -- representing the events
<li><i>entity</i> -- representing processes consisting of events
<li><i>generator</i> -- providing a variety of random
 distribution functions 
<li><i>resource</i> -- to model passive objects conveniently
<li><i>histogram</i> -- to plot the results of a simulation
<li><i>analysis</i> -- to perform statistical analyses on the
 results
 
    </ul>
    </slide>
    The class <i>event</i> (used in the simulation package) contains
    functions to determine the status of an event.  For example, an event
    may be <i>conditional</i> (which means that it will be tried for each
    cycle of the scheduler) or <i>pending</i> (which means that its
    activation will be delayed until it becomes active or conditional).
    The library also contains a class <i>entity</i> representing
    processes, or active objects.  An entity may be regarded as a
    ``structured event'', that is as an active object with phases in its
    life-cycle.  In our approach, however, an explicit switch on the
    entity's phases must be included in the definition of the event.
    
    <h4>Software sound facilities</h4>
    
    <p>Sound facilities are an essential ingredient of a framework
supporting multimedia.  Sound may be produced from a stored audio file
or, provided that real-time sound synthesis facilities are available,
from some high level representation in a sound or music description
language.  Apart from a reduction in the storage capacity needed (and
correspondingly, the network traffic load in a distributed
environment), employing a high level music/sound description language
has as an additional advantage that improved synchronization and
linking facilities may be offered.  The DejaVu framework supports both
MIDI and the sound synthesis facilities offered by Csound &nbsp;[<cite><a href=refs.html#Csound>Csound</a></cite>].
When creating an instance of the <i>icsound</i> class a process is
created which is capable of receiving sound events, as defined by the
Csound numerical score language, and converting these events to data
for the audio device.  (The <i>icsound</i> class inherits from a
<i>wrapper</i> class providing the functionality for spawning of a new
process that communicates with the original process via pipes.)  Care
is taken that the audio device is connected only once.  However,
multiple instances of the <i>icsound</i> class may exist.  In addition
to the <i>icsound</i> class, which accepts only low level sound
events, instances of <i>player</i> may be used which accept musical
events defined in the high level music description language Scot, that
comes with Csound.  The <i>player</i> class is also a <i>handler</i>.
This allows instances of <i>player</i> to be embedded in scripts, as
well as being activated by user-defined events.

    <h4>Digital video</h4>
    
    <p>Digital video may be considered as another standard ingredient of
hypermedia and multimedia frameworks.  To employ digital video, our
framework offers the class <i>video</i>.  Apart from the usual widget
creation commands, the class <i>video</i> has functions for reading in
a video file, to play (forward) and to rewind.  In addition, it offers
a function to bind a frame number to an event.  Binding frame numbers
to events allows for both synchronization and hyperlinking, albeit in
a low level fashion.  Our video extension is based on the public
domain <i>xanim</i> package and includes support for MPEG, AVI and
Quicktime.

    <h4>3D graphics and VRML</h4>
    
    <p>Another exciting development is the introduction of VRML &nbsp;[<cite><a href=refs.html#VRML>VRML</a></cite>] as
a standard for developing 3D graphics and virtual reality on the Web.
We have extended <i>hush</i> with widgets for displaying 3D graphics
(based on the public domain Mesa &nbsp;[<cite><a href=refs.html#Mesa>Mesa</a></cite>] library for OpenGL) and VRML
scenes (based on an extension of the public domain Qvlib &nbsp;[<cite><a href=refs.html#Qvlib>Qvlib</a></cite>]
parser).  Due to the extensibility of our Web (browser) components,
these widgets can easily be incorporated as special (Web-aware)
viewers in the browser.


@+ Viewing the Web -- the Web Components
    <!-- \label{Design} -->
   
    <p>The <i>hush</i> class library offers two ways to connect
applications with the Web.  Pages on the Web can be used as part
of an application by employing the <i>web</i> component, for
example for online help. On the other hand, these pages may
contain embedded <i>applets</i>, defined by the application.
Since most <i>hush</i> applications have a script interface as
well, the application itself may be (recursively) embedded within
such HTML documents [_DoornEl95].

    @- Requirements of the Web Components

    <p>The design of the Web-related components of the <i>hush</i>
library has been influenced by requirements such as extensibility
with new data formats, data transfer protocols and support for
active documents.

    <h4>Data formats</h4>

    <p>
The world of hyper- and multimedia abounds with data formats.
For each type of media, such as text, sound and video, a few data
formats play a prominent role.  A Web browser should provide the
means to show these data formats.  However, viewer applications,
separate from the browser, have some major drawbacks. For
example, users have to use different interfaces and following
links specified in a separate document is more difficult because
of the loose coupling between the viewer application and the main
browser.

    <p>A more elegant solution is to provide mechanisms to include the
viewers into the browser itself, since it allows for a more
uniform interface and for inlining the display of information, so
that the user does not have to switch to another toplevel window.
For example, it is possible to insert video fragments into an
HTML-page or even to insert an HTML-page into another HTML-page.

    <p>The ability to support multiple data formats has been a major
influence on the design of our Web components.  At the moment our
browser supports HTML, SGML, VRML, video (including MPEG,
Quicktime, AVI) and a variety of image types.  Developing new
viewers and including them as integrated components into the
framework is possible without much effort.

    <h4>Data transfer protocols</h4>
    
    <p>The dominant communication protocol of the Web is currently
HTTP, but new or enhanced communication and network transfer
protocols will undoubtedly be needed. Examples include real-time
protocols to incorporate continuous multimedia streams or
protocols supporting safe data transfer.  The DejaVu framework
provides a testbed to experiment with these new protocols by
plugging them into a Web environment.

    <h4>Active documents -- client-side computing</h4>

    <p>A simple but powerful approach to active documents is to add
the ability to run applets (mini-applications).  At the moment
our browser can execute Tcl-scripts (applets) that can be
inserted into HTML or SGML documents, and the design is
sufficiently open to add support for Java or Python applets as
well.


    @- Architecture of the Web Components
    
    <p>To satisfy our requirements, we needed a flexible and open
design of the Web components.  To achieve such flexibility and
extensibility, we made use of the idioms mentioned in section
<ref>dejavu</ref>. Recall that the virtual self-reference idiom
is used to realize nested components and the dynamic
role-switching idiom allows for multiple modes, for example to
display a variety of document formats. 

<slide id=fig1 cap=Overview>
<image name=local/animate/overview>
</slide>

Figure <ref>fig:overview</ref> shows an overview of the basic
Web-related components of the <i>hush</i> library.  The
<i>browser</i> provides the toplevel user interface for all Web
components, including a viewer, a scrollbar, navigation buttons
(back, forward, home, reload) and an entry box to enter URLs.
The <i>client</i>, <i>web</i> and <i>MIMEviewer</i> components
form the conceptual base of our approach of connecting to the
Web:
    <slide id=view-components cap=Components>
    <ul>
<li><i>client</i> -- the interface to the World Wide Web,
 supporting several protocols
<li><i>viewer</i> -- a widget to display several MIME types, such
 as HTML and VRML
<li><i>web</i> -- an extension of the <i>MIMEviewer</i> with
 connection to the Web, history of followed links and caching
    </ul>
    </slide>
    Each component can be extended in a simple way.
    
    <h4>The client component</h4>
    
    <p>The <i>client</i> component builds the bridge between the local
<i>web</i> widget and the World Wide Web by providing an abstract
and uniform interface to network (file) access and transport
protocols. Figure <ref>fig:client</ref> shows the design of the
<i>client</i> component.  

<slide id=view-3 cap="The client component">
<image name=local/animate/client>
</slide>

To be able to add specific clients for new
protocols, the dynamic role-switching idiom is used to implement
the <i>client</i> class. The <i>client</i> implicitly changes its
role, depending on the protocol required.  The <i>web</i> widget
only knows about the (abstract) <i>client</i> class while the
actual functionality is implemented in several concrete
<i>client</i> classes, one per protocol.

    <p>
When the <i>client</i> gets the instruction to retrieve a URL,
it inspects the URL and chooses the right protocol. In other
words, it changes its role to the appropriate protocol
implementation. The addition of new protocols can be done
without changing the <i>web</i> widget.

    <h4>The viewer component</h4>
    
    <p>In the realization of the <i>MIMEviewer</i> components we have
employed the dynamic role-switching idiom in the same way as in
the implementation of the <i>client</i> component. The
<i>MIMEviewer</i> class offers a uniform interface to the
different viewers (see figure <ref>fig:viewer</ref>).

<slide id=view-2 cap=Viewers>
<image name=local/animate/viewer>
</slide>

The user creates
a <i>MIMEviewer</i> object and tells which role it should play
(e.g. SGMLviewer or VRMLviewer). This role can be changed during
the lifetime of a single <i>MIMEviewer</i> object by calling a
method to change its role.  A browser typically uses only one
<i>MIMEviewer</i> that changes its role according to the type of
data that should be displayed.

    <p>Since the <i>MIMEviewer</i> provides no network functionality
at all, it generates an event whenever it needs to retrieve data
pointed to by a URL. Such events are generated as a response to
user interaction (e.g. clicking an anchor) or to fetch inline
data during the parsing process. These events (realized by the
hidden <i>urlevent</i> class) are typically handled by a
<i>web</i> component.

    <h4>The web component</h4>

    <p>
A central role in our approach is played by the <i>web</i>
widget.  It combines the functionality of the <i>MIMEviewer</i>
and the <i>client</i> components. It is able to follow links by
retrieving data via several protocols, a task delegated to its
<i>client</i> component. Additionally, the <i>web</i> component
adds a history and caching mechanism to the <i>MIMEviewer</i>.
The <i>web</i> component's behavior is similar to the standard
widgets of the framework, and can be conveniently used as a part
of an application's GUI.  Because the <i>web</i> widget has both
a C++ class interface and a script interface, it is easy to
create, or extend, applications with Web functionality.  Note
that the SGML viewer already allows for a rich variety of data
formats, possibly extending HTML.

    @+ Declarative and Operational Document Encodings
    <!-- \label{encoding} -->
    
    <p>Document markup languages have slowly evolved from procedural to
declarative markup. Powerful formatters such as troff and TeX merely
allow the text of the document to be interspersed with formatting
commands.  If the document has to be formatted by another
system, or for a different medium, (all) the markup has to be
changed.  Macro facilities may solve part of the problem because
part of the (low-level) formatting decisions can be localized in
the macro definition. Macro packages such as LaTeX provide high
level markup constructs which allow for a more declarative style
by emphasizing the structure of the text, instead of the physical
document layout. 

    @- Content, Structure and Layout
    
    <p>To enhance inter-operability, that is the ability to edit, browse and
typeset documents for different media on different platforms, one
has to make a clear distinction between a document's contents,
how these contents are structured,
and the mapping of structure to format instructions.

    <h4>Content data</h4>
    
    <p>Encoding mechanisms used to represent the content data largely
depend on the medium used.  Text encoding schemes, for example,
have to be able to deal with multiple (international) character
sets. Similarly, mechanisms used to represent graphics, images,
audio and video data differ on the resolution and compression
technique used. Because continuous media streams are encoded by
sequences of digital samples, they have an inherent structure
representing the temporal dimension that static media lack.
     
    <h4>Declarative markup -- structure</h4>

    <p>Mechanisms to encode the logical structure of a document should
be able to describe several relations between the various (data)
parts of the document.  Common examples are a part-whole
relation, used to build a hierarchical structure (e.g article,
section, subsection, paragraph), references, used to make
bibliographical references, footnotes, annotations and other
links.  For text-based applications, <em>tagging</em> has proved
to be an effective mechanism, but even if other media types are
involved (e.g. images in HTML) tagging can still be very useful.
     
    <h4>Style sheets -- layout</h4>
    
    <p>The information provided by the content data and declarative
markup needs to be augmented with information about how the
various elements should be mapped on an output device.  Rather
than embedding such information in the document or processing
application, it is often better to store this kind of information
independently, in a separate <em>style sheet</em>.

    <p>While the concept of a style sheet can be found in many of
today's word-processors, style sheets are only slowly finding
their way into the Web.  At the moment, various schemes
and style sheet languages are being discussed by a W3C
[_W3-Style] working group in order to find a mechanism which
meets the needs of both authors and readers.

    <p>Style sheets have been an issue for SGML-based document
processing as well. While the SGML standard was originally
published back in 1986, there still is no definitive language
addressing the needs for standardized style sheets. The Document
Style Semantics and Specification Language [_DSSSL] is still a
draft international standard.  As a result, many SGML vendors
developed their own style sheet language, e.g.. SoftQuad's Panorama
Pro [_Panorama].

    @+ SGML -- basic concepts
   
    <p>SGML (Standard Generalized Markup Language) [_SGML] is an
ISO standard which uses tagging to encode the logical structure
of a document.  
The set of tags needed to
describe a specific document structure depends on the type of
documents involved.  For example, the tags needed to markup a
mathematical article are in general not suitable to describe the
structure of a telephone book.  As a consequence, SGML does not
describe a set of tags, but a way to define an appropriate set of
tags and the order these tags should appear in the document
instance. Such a definition is called a <em>document type
definition</em> or DTD.  Note that the idea of defining a
fine-tuned markup language for each document type is the exact
opposite of the ``one size, fits all'' approach of current
HTML-based web browsers.  Many other advantages of having SGML
aware Web browsers (and servers) are discussed in [_SperGold94].

    <p>An SGML document essentially consists of two parts: a prolog,
containing the document type declaration, followed by the
document instance, containing the data interspersed with markup.

    <h4>Document instance</h4>
    
    <p>A document instance is a hierarchical structure of (possibly
empty) <i>elements</i>, where each non-empty element contains
other elements or character data. Each element has a name (the
<i>general identifier</i>) and the start and end of an element
are indicated by tags (typically
&lt;name&gt;content&lt;/name&gt). Begin or end tags may be
mandatory or optional.  Moreover, elements contain zero or more
<i>attributes</i>.  Consider the example document below. 

<program>
  <memo security=confidential>
   <to>Anton
   <to>Bastiaan
   <from>Jacco
   <subject>SGML
   <body>
     Dear Anton and Bastiaan,
     I hope this example clarifies 
     the use of the Standard Generalized Markup Language.
   </body>
  </memo>

</program>
The root element <i>memo</i> has
one attribute, indicating the security level of the document.
The memo element contains five other elements. A double <i>to</i>
element stating the addressees, a <i>from</i> element specifying
the sender, a <i>subject</i> and a <i>body</i> field.  All
elements of memo contain character data and no other elements.
Note that the tags emphasize the logical structure of the
document rather than stating how it should be formatted.

    <h4>Document type declaration</h4>
    
    <p>The document instance above must be preceded by a <i>doctype</i>
declaration.  The main part of the document type declaration is
the <i>document type definition</i> or DTD. It defines the
elements of a document and the required order of their
sub-elements.  The elements and their contents are defined by the
use of <i>element declarations</i>.  <!--

<program>
  <!doctype memo [ 
  <!element memo                   O O (to+,from,subject,body)>
  <!element (to|from|subject|body) - O (#pcdata)              >
  <!attlist memo security (low|confidential|topsecret) low >
  ]>

</program>
The second line declares the
memo element, and defines its content as a sequence of one or
more <i>to</i> elements, a <i>from</i>, a <i>subject</i> and a
<i>body</i> element.  The two `O' characters stand for ``omit'',
indicating that the begin and end tag may be omitted.  The third
line defines the elements containing character data only.  Their
start tags are mandatory, indicated by the `-'.  The list of
attributes of each element is declared by an <i>attlist</i>
declaration. Attributes can be
of different types, and be mandatory or optional.  The DTD can
specify a default value, as is shown in the case of the security
attribute. The DTD declaration may be contained within the
<i>doctype</i> declaration, but are typically defined by a
separate file. In that case, the <i>doctype</i> declaration
contains a reference to that file.

    <h4>Processing instructions</h4>
    
    <p> Processing instructions are used to pass system dependent
information to the application to tell how the document is to be
processed. Processing instructions are contained within `&lt?'
and `&gt;' characters and can appear on arbitrary places within
the document. In the following examples, we employ processing
instructions to indicate the URL of a style sheet.

    <h4>Entities</h4>
    
    <p>Fragments of markup and character data can be given a name
using an <i>entity</i> declaration.  The declaration of an entity
is part of the document type declaration, but the entity may be
used within the document instance. The contents of an entity may
be defined by a string, or may be contained in an external file,
in which case the entity declaration contains a reference to the
file.  External entities may be referenced by a <i>system
identifier</i>. Support for these identifiers is system dependent
and may include filenames, URLs and database queries. Consider a
variant of the previous example:

<program>
  <!doctype memo system "http://www.cs.vu.nl/~bastiaan/memo.dtd" [
  <!entity sgml "the Standard Generalized Markup Language"> ]>
  <?stylesheet lang=Tcl src="http://www.cs.vu.nl/~bastiaan/memo.sty">

     ...
     Dear Anton and Bastiaan,
     I hope this example clearifies the use of &sgml;
    </body>
  </memo>

</program>
The first line references an external DTD by means of a
system identifier, and the second line defines an entity for
later usage. Note that the entity definition is enclosed within
the square brackets of the <i>doctype</i> declaration.  The third
line contains a processing instruction to define the location of the
style sheet. In contrast to the URL of the DTD, which is resolved
by the parser, the URL of the style sheet is passed to
the application without further processing by the parser.

<p>To avoid system dependent identifiers such as filenames, an
extra indirection is provided by the concept of <i>public
identifiers</i>.  These identifiers are assumed to be publicly
known, and the SGML parser of the target application is expected
to be able to resolve them. Typically, a local catalog file is
used to map public identifiers onto system dependent ones.
<i>Formal</i> public identifiers have a standardized and meaningful inner structure,
to facilitate automatic resolving without the use of catalogs.
In the following examples, we refer to the HTML 3.0 DTD by means of
a formal public identifier.



    <h4>Encoding Hypermedia Documents</h4>
    
    <p>The separation of content, structure and layout has proven to
be effective for text-oriented document processing, but it is not
clear whether this approach scales to time-dependent and highly
interactive hypermedia documents.  Synchronization constraints,
user interaction, hyperlink traversal are important
characteristics of such documents, but it depends on the
application whether these issues are part of the style, structure
or even the document contents.

    <p>While many of these issues can be defined in the structure of a
document, SGML lacks standard mechanisms to do so.  The
Hypermedia/Time-Based Structuring Language (HyTime) [_HyTime] is
a relatively new ISO standard defining syntactic SGML conventions
representing semantics for hyperlinking, addressing, aligning and
scheduling of multimedia documents. We will further discuss these
issues in section <ref>issues</ref>.


@+ Active Web Documents -- an example
<!-- \label{example} -->


    <p>As an example, we will illustrate how to deploy the SGML
related components used to parse and display SGML encoded
documents.  The <i>SGMLviewer</i> is based on SP, James Clark's
conforming SGML parser [_SP].  Web applications
benefit from the use of such a parser in a number of ways:
    <ul>

<li><i>Validation</i> -- 
 Unlike most Web browsers, the <i>hush</i> browser uses a validating
 parser.  The document instance is validated against a document
 type definition.  As a result, the <i>hush</i> browser prints
 warnings in case of illegal markup.
 
<li><i>Extra SGML features</i> -- SGML provides some standard
 features such as file inclusion and macro substitution, which
 are processed by the parser in a fully transparent way.
 Moreover, the parser output describes a complete document (that
 is, all optional begin and end tags are added, and default
 attribute values are filled in), which simplifies formatting and
 display.
 
<li><i>Support for new document types</i> -- New document types
 and extended versions of existing ones can be parsed by defining
 a new document type definition describing the structure of the
 document.  Such document types can be made to fit the
 application domain in a precise manner.
    </ul>
    
    @- Extending Document Types 
    
    <p>We will start by illustrating how to extend the default HTML
document type definition.  The document instance given below is
specified in HTML but employs in addition two extensions for
active documents.  The first extension is a <i>video</i> tag,
which is used to display inline video fragments. The other is an
<i>applet</i> tag, used to embed small applications written in a
scripting language. We use Tcl in the examples.  In the example
below, the <i>applet</i> displays some notes that are played when
the user clicks on the image.

<program>
  <!doctype demo system "http://www.cs.vu.nl/~bastiaan/demo/demo.dtd" [
  <!entity title "Hush browser inline video and music demo"> ]>
  <?stylesheet lang=Tcl src="http://www.cs.vu.nl/~bastiaan/demo/demo.style">
  
  <demo>
    <title>&title;</title>
    <h1>&title;</h1>
    <h2> Inline video fragment </h2>
    <p>This page shows the ability to inline video 
       (vision and sound) into an SGML-page:
    <p><video src="http://pmwww.cs.vu.nl/archive/animations/mpeg/moonflag.sgml">
    <hr>
  
    <h2> Inline music </h2>
    <p>If you click on the image of the notes, the music starts playing:
    <p><applet class=music data="http://www.cs.vu.nl/~bastiaan/demo/music/">
    <hr>
    <p>Bastiaan Sch&ouml;nhage, Vrije Universiteit Amsterdam, 1996.
  </demo>

</program>
The first line of the example defines the type of the document
(demo).  It is specified in a separate document type definition
(DTD) that we will describe below.  The parser automatically
retrieves the DTD from the Web if its location is specified by a
URL.  The next line illustrates the use of an <i>entity</i>
declaration, an SGML mechanism used here as a primitive macro
facility defining the title of the document. The title entity is
used twice, in the <i>title</i> and <i>h1</i> tag, and will be
expanded by the parser.  The third line specifies the style sheet
that is needed to display the contents of the document. The use
of style sheets will be explained later.  The <i>video</i>
element requires a <i>src</i> attribute defining the location of
the video file.  Finally, the <i>applet</i> tag is used to inline
embedded script code defining a small musical applet. The image
and the note files are located in the directory specified in the
<i>data</i> attribute of the <i>applet</i> tag.

    <p>Whether active document elements are to be defined by a new,
specific tag or by the more general applet mechanism is a matter
of taste. A new tag requires modification of the DTD and style
sheet but describes the element in a more declarative way,
which gives the application more freedom in displaying
the contents. For example,
a browser may decide to display a text alternative if the
local platform does not support video.  

<slide id=view-xx cap="View">
<image name=local/animate/browser>

</slide>


<h4>Extending the DTD</h4> 
    
<p>
Recall that a document type definition defines the structure of a
document by describing the <i>elements</i> and <i>attributes</i>
that can be interspersed as tags with the document content.  The
following DTD extends the (draft) HTML 3.0 DTD [_HTML3] with the
<i>video</i> and <i>applet</i> elements required for the example
above.  


<program>
  <!entity % special "tab | math | a | img | br | applet | video ">
  <!entity % htmldtd public "-//IETF//DTD HTML 3.0//EN"> 
  %htmldtd; <!-- include standard HTML 3.0 DTD -->

  <!element demo O O (%html.content;) >

  <!element (applet|video) - O empty>

  <!attlist applet        
        class cdata     #required   -- class name of applet        --
        lang (Tcl|Java|Python)  Tcl -- Default script language     -- 
                                    -- URI is defined by HTML DTD: --
        src   %URI;     #implied    -- defaults to base URL        --
        data  %URI;     #implied    -- location of additional data -- >

  <!attlist video         
        src %URI;       #required   -- URI of video fragment       --
        alt cdata       #implied    -- optional text alternative   -- >

</program>

The first line extends
the <i>special</i> parameter entity defined by the HTML DTD with
<i>video</i> and <i>applet</i> elements, enabling the use of
<i>video</i> and <i>applet</i> tags where image or anchor tags
are allowed. The second definition of <i>special</i> (in the HTML
DTD) will be ignored by the parser.  In the second line, the SGML
parameter entity mechanism is used to include the draft HTML 3.0
document type definition, which is referenced by a formal public identifier.  
The <i>demo</i> element consists of
the sub-elements defined by the HTML DTD.  The two <i>O</i>
characters in the element declaration indicate that begin and end
tag of the <i>demo</i> element may be omitted, since the begin
and end of the document can be derived by the parser.  In
contrast, the begin tag of a <i>video</i> or <i>applet</i>
element is mandatory, the end tag optional.  The video and applet
elements have optional (<i>implied</i>), mandatory
(<i>required</i>) and default attributes (e.g. the <i>lang</i>
attribute defaults to <i>Tcl</i>).

    <p>The information contained in the DTD is used by the parser to
generate a complete and validated document instance. Note that
this task could be performed by an HTTP-server as well, which
would significantly simplify the design and implementation of web
clients. Therefore, there are strong arguments to add SGML
functionality to servers as well [_SperGold94].

    <h4>Style sheet</h4>
    
    <p>Style sheets define how the various elements should be
processed.  The <i>hush</i> browser (see figure
<ref>fig:browser</ref>) defines a default style for HTML
elements.  However, these styles can be redefined and extended by
a document instance using a special processing instruction,
notated as <tt>&lt;?stylesheet&nbsp;url&gt;</tt>.  The browser
retrieves the URL specified in the processing instruction and
uses it to display the contents of the document.  The example
specifies a URL to a style sheet that describes how to process
the new <i>video</i> tag. Recall that processing instructions are
application dependent, so the parser passes the text in a processing
instruction directly to the application.  At the moment, we use
an experimental style sheet language based on Tcl. An example of
a style sheet fragment that specifies how the <i>title</i> tag
should be processed, is given below. 

<program>
  ## Fragment of experimental Tcl style for HTML
  proc TITLE   { atts data } { wm title . \$data } 
  proc /TITLE  { } { }
  ## $...$ etc

</program>
While the style sheet mechanism
needs some refinement, our approach supports the extension of
existing document types and allows for extensive experimenting
with the (many) new tags proposed for the HTML 3.0 standard.

    @+ Creating New Document Types 
    
    <p>The document instance of the previous example was very similar
to plain HTML documents, which made it worthwhile to reuse the
original HTML DTD and style sheet. However, for some applications
HTML is not suited at all and a completely new document structure
is needed. The next example shows a document
instance of a simple musical application.  


<program>
  <!doctype score system "music.dtd" [ 
  <!entity G7 "<note name=keynote>g<note>b<note>f" -- entity def-- > ]>
  <?stylesheet lang=Tcl src="music.style">

  <score>                               <!-- document instance -->
    <chord id=chord1>&G7;               <!-- entity usage      -->
    <chord id=chord2>&G7;
    <chord id=chord3 name=Cmajor>
      <note name=keynote>c<note>e<note>g
  </score>

</program>

The first line defines the (filename of) a new document type
definition. The next line contains an entity definition
describing a G7 chord. 
An SGML processing instruction is used to specify the filename of
the style sheet.  The root of the document hierarchy is the
<i>score</i> element, consisting of several chords. Chords are
build of notes, which are described by single characters. The
first two chords use the entity defined before, specifying the
notes of a G7 chord.  The third occurrence of
<i>chord</i> describes a C major
chord. 

    <h4>A new DTD</h4>
    
    <p>The DTD corresponding to the simple musical document 
given above defines the structural elements and their
attributes. When the application  processes the 
document, the parser will fill in the
default duration for all notes, resolve the
entity definition and add the missing end tags for the notes and
chords.


<program>
  <!-- Simple music DTD --> 

  <!element score - O (chord)+    >
  <!element chord - O (note)+     >
  <!element note  - O (#pcdata)   >

  <!attlist (chord|note)        -- all have same attributes  --
     id       id    #implied    -- optional identifier       --
     name     cdata #implied    -- optional name attribute   --
     duration cdata "4"         -- default duration: quarter -- >

</program>

    <p>The DTD defines three structural elements: a <i>score</i>
containing several <i>chords</i>, a <i>chord</i> containing
several <i>notes</i> and a <i>note</i> consisting of data. Both
chord and note have three attributes: <i>id</i>, <i>name</i> and
<i>duration</i>. The first two are optional and the last has a
default value of 4.  Note the difference between the <i>id</i>
attribute, which has to be a unique identifier, and the
<i>name</i> attribute, which can be an arbitrary string.


    <h4>Style sheet</h4>
    
    <p>Playing the document by a web browser, does not
necessarily involve displaying the data visually.
The simple style sheet shown below
simply collects notes and chords and plays 
them by using the <i>play</i> command.
Note that most of the timing relations are implicit
in the document. For example, the notes within a single chord are
to be played in parallel, and the chords themselves are to be
played sequentially. However, this is not explicitly defined by
the document instance or DTD and can only be intuitively derived
from the element names.  Even in the style sheet below, these
timing relations remain implicit.


<program>
  ## Simple Tcl style sheet for music example

  set score  ""; set chord  ""

  proc SCORE { atts  data } { global score; set score "t120 " } 
  proc CHORD { atts  data } { global chord; set chord "(" }
  proc NOTE  { atts  data } { global chord; set chord "$chord$data<" }
  
  proc /SCORE { } { global score; play \$score } 
  proc /CHORD { } { global score chord; 
                    set score "$score$chord r)" ; set chord "" }
  proc /NOTE  { } { }
</program>

The procedures corresponding to the open and close tags build a
string representation of the score. At the opening of the score
element the string is initialized with a command defining the tempo
at 120 beats per minute. During the parsing process the string is
extended with the parsed notes. After the last chord has been
parsed the resulting string is:
<tt>t120 (g&lt;b&lt;f&lt; r)(g&lt;b&lt;f&lt; r)(c&lt;e&lt;g&lt; r)</tt>.
This string is played after the score end tag has
been encountered. The Tcl command <i>play</i> used to play the notes is
provided by the <i>hymne</i> extension of <i>hush</i>
[_OssEl94,OssEl95].



    @+ Towards Full Hypermedia Support
    <!-- \label{issues} -->
    
    <p>
Although the style sheet mechanism illustrated in the previous
examples provides full operational support, it must be considered
too low level to support more complex hypermedia document
formats.  A more generic, declarative approach is needed, to
allow for interoperability between hypermedia applications.  Such
a generic, declarative approach has been proposed by the HyTime
standard [_HyTime], which defines syntactic conventions (known as
<i>architectural forms</i>) to describe hypertextual features of
SGML encoded documents.  The actual look-and-feel of a HyTime
encoded document, that is the operational realization of these
features, is taken to be application dependent.  While a full
implementation of the HyTime standard is yet to be realized, a
limited subset of the architectural forms is supported by some
commercial document processing systems [_Panorama].  Issues
addressed by the HyTime standard include addressing, linking
facilities and spatial and temporal alignment.  Not addressed by
the HyTime standard are facilities for user interaction, and
synchronization issues must be considered to be only weakly
addressed by HyTime.  In this section we will discuss to what
extent the HyTime standard and other hypertext or hypermedia
models such as the Dexter Hypertext Reference model [_Dexter] or
the Amsterdam Hypermedia Model [_AHM] may be taken as a point of
reference for extending the Web with full hypermedia
functionality.  Another standard of interest is proposed by the
MHEG [_MHEG] group. However, whereas MHEG only provides a
solution for final form documents (with the presentation
information built-in), HyTime (and both other models) are of a
more generic nature, since they abstract from detailed
presentation issues. 

    <h4>Addressing</h4>
    
    <p>The ability to identify (ranges of) elements within an
arbitrary document is a critical issue for all hypermedia
systems, since addressing is a prerequisite for hyperlinking and
alignment.  In the Web, addressing is based on the location of
the data on a specific server. Research to make the addressing
mechanism independent of the location of the data is still in its
infancy. Given a certain document, one must be able to address
fragments within the document.  In the Dexter Hypertext Reference
model [_Dexter], the notion of <i>anchors</i> has been introduced
to address items within a document without requiring knowledge of
their inner structure.  In text documents, anchors are typically
encoded within the document structure (e.g. HTML). To identify
items in audio, video or read-only text documents (e.g. on
CD-ROM), mechanisms are needed to define and store extrinsic
anchors independently of the document itself. To allow for both
forms of addressing, HyTime provides a plethora of syntactic
constructs representing (aggregate) pointers to data.
Standardizing these addressing schemes is required for exchanging
hypermedia documents between different hypermedia applications.
To prevent that several applications need to re-implement these
mechanisms, a hypermedia framework needs to support them in a
generic way.

    <h4>Linking</h4>

    <p>Having powerful addressing mechanisms is not sufficient to
fully support hyperlinking. To facilitate document exchange, a
hypermedia framework needs to support standardized syntactic
elements defining both <i>contextual links</i> (i.e. links
defined by the source anchor) as well as <i>independent links</i>
(i.e. the definition of the link is independent from the
definition of its anchors). Independent links may be stored in
another document or link base.  HyTime provides architectural
forms for both type of links. These links can be bidirectional
(allowing traversal from source to destination and vice versa)
and multi-headed (allowing multiple source and destination
anchors). On the Web, support for bidirectional links will
require the development of some kind of (globally) distributed
link service, since links can no longer be embodied in the
(source) document.  The Amsterdam Hypermedia Model [_AHM]
introduces the notion of <i>link context</i> to specify the scope
of both source and destination of hyperlinks in hierarchically
structured documents.  While such a concept would be useful for
most HyTime-encoded documents as well, the HyTime standard does
not seem to address this problem.  
For text-based applications, the style of link
presentation can be defined by a style sheet 
(e.g. by specifying how anchors should be marked).
However, for image, audio and video, the presentation of links and anchors
seems to be too complicated to be solved on that level.

    <h4>Spatial and temporal alignment</h4>

    <p>From a HyTime perspective, presenting a document requires the
ability to position multimedia components in a (n-dimensional)
coordinate space. Note that for text-based documents, the exact
positions of the document's elements can usually be calculated by
combining the logical structure (e.g. a title) and the declarative
rendering information in the style sheet (e.g. which font and
indentation to use for titles).  This mechanism relies on the
fact that the basic logical structure of the common document
formats (e.g. letters, articles, books) is well understood by
both application developers and authors. For hypermedia
documents, there is still little understanding of such fundamental structural
elements.  Another issue is the temporal
synchronization between the time dependent elements of a
hypermedia presentation.  These dependencies are often hard to
express in a style sheet.  In contrast to the
specification of fonts and indentation, knowledge of the temporal
dependencies is often required to understand the semantics of the
document. Such important dependencies should be
defined by the content and structure, and not by an accompanying
style sheet.  HyTime provides mechanisms to wrap multimedia
objects in an <i>event</i>.  Each event has an <i>extent</i>
(i.e. an n-dimensional bounding box) which can be positioned in a
<i>finite coordinate space</i>.  Although these generic
mechanisms can be used to specify spatial alignment and temporal
scheduling in a uniform manner, the resulting document structures
tend to contain (too) long, complex and detailed extent
specifications. Examples of temporal constraint specification in
HyTime can be found in [_Erfle].

    <h4>User interaction</h4>

    <p>Apart from textual anchors to retrieve documents by URL, the Web
provides a limited number of facilities for user interaction,
including clickable maps and forms.  Both these facilities,
however, depend on server-side processing.  Adding such
facilities as client-side extensions is easy to do, and allows
for using more appealing metaphors for interaction, for example
by employing 3D objects or virtual reality extensions.  However,
fitting these extensions and the enriched repertoire of user
interactions they allow for, in a hypermedia model is very
difficult [_AHM].  For example, considering time-based media, when
we take a time-line model as a starting point, we are forced to
take into account the indeterminacy caused by possible user
interactions.  As already observed, the HyTime proposal seems not
fit for this, nor for expressing indeterminacy caused by 
network and other delays.


    @+ Conclusions
    
    <p>We have illustrated how our approach allows for defining both
the syntactical and operational characteristics of documents.
Moreover, the components provided by our framework, the
<i>hush</i> library and its extensions, support the development
of Web-aware applications in a modular and extensible way.  As
such, our framework provides a starting point for discovering
suitable high-level abstractions for integrating multimedia and
advanced linking support.  However, much more research is needed
for realizing adequate facilities for addressing, linking,
synchronization and scheduling, and user interaction.
Nevertheless, by illustrating the joint development of both
syntactical and operational abstractions, we have indicated the
first steps of what we consider to be a promising route.

</document>
