<document>
:@* Patterns and Idioms in the DejaVu Framework

The DejaVu framework,  in particular  the <i>hush</i> 
(<em>hyper utility shell</em>) library
and its extensions,
aims at providing an easy-to-use
and flexible, multi-paradigm environment
for developing distributed hypermedia and web-based applications.
Our framework is object-oriented in that it allows
for a component-wise approach to developing applications.
Yet, in addition to object class interfaces, it
offers the opportunity to employ a script language
to develop applications and prototypes as well.
It is multi-paradigm, not only by supporting a
multi-lingual approach, but also by providing support
for distributed client/server solutions in a transparent manner.
<p>
In this paper we will discuss our experiences in
developing the framework with regard to the method
of software development employed,
recurrent patterns in the design of components,
and, on an architectural level, the problems in
supporting multiple languages and distribution.
<p>
We will discuss the scope of
our framework and the approach taken in its realization
as a collection of classes.
In particular we will delineate the recurrent 
patterns that we employed in its design and,
on an architectural level, the idioms we employed
for the concrete realization of the basic concepts
underlying the framework.
To illustrate the role of the
patterns and their associated idioms in our framework,
we will discuss the construction of
our Web components. 
<p>
We observe that there 
is a tension between
defining a clean object model and providing the flexibility
needed to support a multiparadigm approach.
We resolved this tension by choosing to differentiate
between the object model (that is class interfaces)
offered to the average user of the framework and
the object model offered to advanced users
and system-level developers.
<p>
In our work, idioms play a central role,
that is we achieved the desired flexibility by
systematically employing a limited number of basic idioms.
We succeeded in hiding these idioms from the average
user of the framework.
However, the simplicity of our original object model
is only apparent.
Advanced or system-level developers who intend to define
extensions to the framework must be well aware
of the patterns underlying the basic concepts,
that is the functionality requirements of the classes involved,
and the idioms employed in realizing these requirements.
<p>
Patterns and their associated idioms have guided
us in defining an architecture that allows for
transparently extending the functionality of our framework.
However, with respect to issues of distribution,
the idioms employed provide only a partial solution.
To realize a framework for distributed applications,
we have felt the need to provide additional language
support as well, that is by extending existing
O-O languages with support for parallelism and distribution.
<p>
Our framework has evolved gradually over the years.
Reflecting on the approach we have taken and the problems 
encountered, 
we will evaluate the relative merits of employing patterns
and indicate the costs, in terms of development efforts,
incurred by using them in the realization
of our framework.
<p>
Keywords and phrases:
<i>
object-oriented framework, multiparadigm,
distributed hypermedia, web-based applications,
patterns, idioms, languages, development costs</i>







<h1>Introduction</h1>

For both users and developers,
delineating the patterns and idioms underlying
the realization of an object-oriented framework
is slowly being recognized to be of critical
importance for understanding the inherent complexity
of the framework.
<p>
The DejaVu framework, that we are developing,
is meant to provide a flexible and
easy-to-use
environment for intelligent,
distributed hypermedia applications.
In particular, the <i>hush</i>
(<i>hyper utility shell</i>) library
and its extensions offer support for the
development of multimedia user interfaces,
and is used in practical CS2 and CS3 software
engineering assignments.
The focus of our research has recently shifted towards
Web-integrated applications,
that allow for including Web-aware
widgets as a (GUI) component in an application.
In order to realize the functionality of <i>hush</i>
in a flexible fashion,
a systematic use has been made of a limited number
of patterns,
namely the <i>nested components</i> pattern and
the <i>actor</i> pattern.
For the actual implementation <i>hush</i> we employed
an improved version of the <i>handle/body</i> idiom,
and the derived
<i>virtual self-reference</i> and
<i>dynamic role-switching</i> idioms that underly
the patterns mentioned.
<p>
To clarify the distinction between patterns and idioms
we emphasize that the patterns
provide a solution to a problem of design,
whereas idioms are to be regarded as a convenient
implementation technique.


<h4>Related work</h4>
Clearly, a rich variety
of patterns and idioms has already been
extracted from the development of GUI toolkits
as testified by the collection in [_GOF],
which is primarily drawn from the experience
of developing Interviews [_Interviews]
and ET++ [_ET].
Nevertheless,
our work shows a flaw in one of the commonly used
idioms, notably the <i>handle/body</i> idiom,
and may further be considered of interest
due to our approach of realizing
the <i>handle/body</i> idiom and idioms derived from it.
Our <i>actor</i> pattern
and associated <i>dynamic role switching</i> idiom,
moreover, seem to be more powerful
than similar patterns known from the literature.

<h4>Structure</h4>
The structure of this paper is as follows.
First, we will give a brief overview of
the DejaVu framework and the basic concepts of
the <i>hush</i> library.
Then we will characterize the patterns and idioms
employed in developing hush and its extensions.
As an illustration of our approach,
we will discuss the design of the Web-related
components of our framework.
And finally, we will discuss the relative merits
of the patterns and idioms employed.

:@ The DejaVu framework


The problem addressed in the DejaVu project
is essentially to provide support for the
software engineering of (moderately complex)
applications that require, apart from other
functionality, multimedia interfaces and associative 
online help.
Such support is primarily offered in the form
of class libraries, most of which are written in C++.
The language C++ was chosen for pragmatic reasons,
since it allows for easily wrapping public
domain software written in C in classes with a 
more user-friendly interface.
Another reason was that C++ seemed to be 
the right vehicle for programming assignments,
with regard to its ever more widespread use.
<p>
Application development
generally encompasses a variety of programming tasks,
including system-level software development
(for example for networking or multimedia functionality),
programming the user interface
(including the definition of screen layout
and the responsivity of the interface widgets to user
actions),
and the definition of (high-level) application-specific
functionality.
Each of these kinds of tasks may require
a different approach and possibly
a different application programming language.
For example, the development of the user interface
is often more conveniently done
using a scripting language,
to avoid the waiting times involved
in compiling and linking.
Similarly, defining knowledge-level
application-specific functionality may
benefit from the use of a declarative
or logic programming language [_VE95].
<p>
In our project, we decided from the start to
support a multiparadigm approach to software
development and consequently we had to
define the mutual interaction between the
various language paradigms, as for example 
the interaction between C++ and a scripting language,
such as Tcl [_Tcl].
Current scripting languages, including Python [_Python] and Tcl,
provide facilities for being embedded in C and C++,
but extending these languages with functionality defined
in C or C++ and employing the language
from within C/C++ is rather cumbersome.
The <i>hush</i> library offers a uniform
interface to a number of script languages,
and
in addition, it
offers a variety of widgets and multimedia
extensions, that are accessible through
any of the script interpreters as well
as the C++ interface.

<h4>Basic concepts</h4>
These concepts are embodied in (pseudo) abstract
classes that are realized by employing patterns
based on idioms
extending the <i>handle/body</i>
idiom, as explained later on.

<ul>
<li> <i>session</i> -- to manage (parts of) the application
<li> <i>kit</i> -- to provide access to the underlying system and interpreter
<li> <i>handler</i> -- to bind C++ functionality to events
<li> <i>event</i> -- stores information concerning user actions or system events
 
<li> <i>action</i> -- stores information of the binding of a handler
 
<li> <i>widget</i> -- to display information on a screen
<li> <i>item</i> -- represents an element of a widget
</ul>

Programming a <i>hush</i> application requires the
definition of an application class
derived from <i>session</i> to initialize
the application and start the (window environment)
main loop.
In addition, one may bind (C++) <i>handler</i> objects
to script commands by invoking the <i>kit::bind</i>
function.
Handler objects are to be considered
an object realization of callback functions,
with the advantage that client data may be
accessed in a type-secure way
(that is either by resources stored when creating the
handler object or by information that is passed via events).
When invoked, a handler object receives a pointer to
an <i>event</i>
(that is, either an X event or an event related to
the evaluation of a script command).
The information that results from binding
a handler to a script command
is stored in an <i>action</i> object,
that is used by both widgets and (graphical) items
to bind window events (resulting from user actions)
to the handler (callback) objects.
Both the <i>widget</i> and <i>item</i> class are
derived from <i>handler</i> to
allow for declaring widgets and items to be their own handler.


<h4>The art of hush programming</h4>
In fact, every class in the <i>hush</i> library
(except for the <i>handler</i> class itself)
is derived from the <i>handler</i> class,
to allow for defining a script interface for
these classes.
In a similar way, the extensions to <i>hush</i> are
required to define appropriate widget classes,
and possibly specialized <i>handler</i> classes and
<i>event</i> classes dealing with particular
events
(such as for example file descriptor events
for communicating via sockets or notification
events resulting from joining a ToolTalk session [_ToolTalk]).
The extensions include
<ul>
<li> <i>sim</i>   -- for discrete event simulation
<li> <i>hymne</i> -- for sound synthesis and music
<li> <i>web</i>   -- for connecting to the Web
<li> <i>video</i> -- for digital video
<li> <i>vr</i> -- for the OpenGL-based VRML extension
</ul>
For the average user, programming in <i>hush</i>
amounts (in general) to instantiating widgets and
appropriate handler classes,
or derived widget classes that define their own
<i>handler</i>.
However, advanced users and system-level programmers
developing extensions are required to comply
with the constraints resulting 
the patterns underlying the design of <i>hush</i>
and the application of their associated idioms
in the realization of the library.
In the next section, we will outline the rules
of the game.

:@+ The rules of the game -- patterns and idioms


The design of <i>hush</i> and its extensions can be understood 
by a consideration of two basic patterns
and their associated idioms,
that is the <i>nested-component</i> pattern
(which allows for nesting components that have a similar interface)
and the <i>actor</i> pattern
(which allows for attributing different modes or roles
to objects).

The realizations of these patterns are based on idioms
extending an improved version of the familiar
<i>handle/body</i> idiom.
Our improvement concerns the introduction of
an <i>explicit invocation context</i> which is needed to
repair the disruption of the virtual function call mechanism
caused by the delegation to 'body implementation' objects.

<p>
In this section, we will first discuss the <i>handle/body</i> idiom
and its improvement.
Then we will discuss the two basic patterns underlying
the design of <i>hush</i>
and we will briefly sketch their realization by extensions of
the (improved) <i>handle/body</i> idiom.


:@- The handle/body idiom -- invocation context

The <i>handle/body</i> idiom 
allows for
separating the definition of a component's
interface (the handle class) from its
realization (the body class). All intelligence is
located in the body, and (most) requests to the handle object
are delegated to its implementation.

In order to optimize the creation and destruction of dynamically
allocated objects, one may use techniques such as
<em>representation or resource sharing</em> 
and <em>reference counting</em>.
However, the use of these techniques can put a heavy burden on
the application programmer.
The <i>handle/body</i> idiom was 
introduced in [_Coplien] as a means to hide the use 
of such low-level issues
from the application programmer,
by adding a level of indirection,
that is delegation to a 'body' implementation
object.
An additional advantage of applying the idiom is that
it allows interface classes to become stable
while improving on the realization.
In practice, by applying the idiom one may avoid
recompilation of dependent code as long as the interface class
does not change.


<h4>Invocation context</h4>
The <i>handle/body</i> idiom is one of the most popular idioms
[_Coplien,Eliens95,GOF,Stroustrup,Pree].
It underlies 
several other idioms and patterns
(e.g. the envelope/letter idiom [_Coplien], the Bridge and Proxy
patterns [_GOF]).
However, despite the fact that it is well-documented
there seems to be a major flaw in its realization. 
Its deficiency lies in the fact that the
dynamic binding mechanism is disrupted by introducing
an additional level of indirection
(by delegating to the 'body' object),
since it is not possible to make calls to member functions which
are refined by subclasses of the (visible) handle class in the
implementation of the (hidden) body class.  We restored the working of
the normal virtual function mechanism by introducing the notion of
<em>explicit invocation context</em>.  In this way, the handle/body
idiom can be applied completely transparently, even for
programmers of subclasses of the handle.
<p>
In our realization of the <i>handle/body</i> idiom
the handle class is used both as the interface class
to access functionality
implemented by its body, and as a semi-abstract base class, defining
the interface of the body classes which are derived from the handle.
The body object is typically instantiated directly by the
constructor of the handle, but this action may be deferred to the
moment of first use, or may be done indirectly by using an abstract
object factory [_GOF].
To be able to make calls to member functions redefined by user-defined
subclasses of the handle class, the body class must make such calls
via the handle class as well. 
See <a href="handle.html">appendix A</a> for a detailed 
treatment of the idiom and its improvement.

<h4>Usage</h4>
The (improved version of) the idiom is frequently used
in the <i>hush</i> class library. 
The widget library is build of a stable
interface hierarchy, offering several common GUI widgets classes like
buttons, menus and scrollbars.  The widget (handle) classes are
implemented by a separate, hidden implementation hierarchy, which
allows for changing the implementation of the widget library, without the
need to recompile dependent applications.  Additionally, the idiom
helps us to ensure that the various widget implementations are used in
a consistent manner (e.g. to prevent simultaneous use of Motif buttons and
OpenLook scrollbars)

:@+ The nested component pattern

The <i>nested component</i> pattern has been introduced
to support the development of compound widgets.
It allows for (re)using the script and C++ interface
of possibly compound widgets, by employing explicit redirection to
an inner or primary component.

<h4>Problem</h4>
Inheritance is not always a suitable technique for code sharing
and object composition. A familiar example is the combination of a
Text object and two scrollbars into a ScrollableText object.
In that case, most of the functionality of ScrollableText will
be equal to that of the Text object.
This problem may be dealt with by employing multiple
inheritance.
Using single inheritance, it may be hard to
inherit this functionality directly and add extra functionality
by attaching the scrollbars,
especially when interface inheritance and implementation inheritance
coincide.

<h4>Background</h4>
The <i>nested component</i> pattern is closely
related to the Decorator pattern [_GOF] and InterViews notion of
MonoGlyph [_Interviews]. Additionally, by using explicit delegation
it provides an alternative form of code sharing than inheritance
as can be found in languages supporting <em>prototypes</em> or 
<em>exemplars</em> such as Self [_Ungar92].

<h4>Realization</h4>
The <i>nested component</i> pattern is realized by
applying the <i>virtual self-reference</i> idiom.
Key to the implementation of that idiom is the virtual
<i>self()</i> member of a component. The <i>self()</i>
member returns a
reference to the object itself (e.g. <i>this</i> in C++) by default,
but returns the inner component if the outer object explicitly
delegated its functionality by using the <i>redirect()</i> member.
Note that chasing for <i>self()</i> is recursive,
that is (widget) components can be nested to arbitrary depth.
The <i>self()</i> member must be used to access the functionality
that may be realized by the inner component.
Note that the realization of the idiom uses inheritance primarily for
interface sharing, and not for code sharing and object composition.
See <a href="widget.html">appendix B</a> for a detailed code example.

<h4>Usage</h4>
The <i>nested component</i> pattern is employed
in designing the <i>hush</i> widget hierarchy.
Every (compound) widget
can delegate part of its functionality to an inner component.
It is common practice to derive a compound widget from another
widget by using interface inheritance only, and to delegate
functionality to an inner component by explicit redirection.

:@- The actor pattern

The <i>actor</i> pattern provides
a means to offer a multitude of functional modes
simultaneously.
For example, a single <i>kit</i> object gives access to
multiple (embedded) script interpreters,
as well as (possibly) a remote kit.


<h4>Problem</h4>
For many applications, static type hierarchies do not provide the
flexibility needed to model dynamically changing roles. For example we
may wish to consider a person as an actor capable of various roles
during his lifetime, some of which may even coexist concurrently.
The characteristic feature of the actor
pattern is that it allows us to regard a particular entity
as being attributed various roles or modes
and that the behavior of that entity changes
accordingly. 

<h4>Background</h4>
Changing roles or modes can be regarded
as some kind of state transition, and
indeed the actor pattern 
(and its associated <i>dynamic role-switching</i> idiom)
is closely related to the
State pattern [_GOF]. In both cases, a single object is used to
access the current role (or state) of a set of several role (or
state) classes.
In combination with the <i>virtual self-reference</i>
idiom, our realization of
the <i>actor</i> pattern allows for changing
the role by installing a new actor.

<h4>Realization</h4> 
The realization of the actor pattern employs the
<i>dynamic role-switching</i> idiom,
which is implemented by extending the handle class
with a set of several bodies instead of only one.
To enable role-switching,
some kind of indexing is needed. Usually, a dictionairy
or a simple array of roles will be sufficient.
See <a href="actor.html">appendix C</a> for a detailed code example.

<h4>Usage</h4>
In the <i>hush</i> library the actor pattern is used
to give access to multiple
interpreters via the same interface class (i.e. the <i>kit</i> class).
The pattern is essential in supporting the multi-paradigm nature
of the DejaVu framework.
In our description of the design of the Web components
in section <ref file=design>Design</ref>, 
we will show how <i>dynamic role-switching</i> is employed
for using
various network protocols via the same <i>(net)client</i> class.
The actor pattern is also used to define a (single)
viewer class that is
capable of displaying documents of various MIME-types
(including SGML, HTML, VRML).
</document>
