impl unit D		%%	@(#)D.i	1.3

use Q O P T U Nodes
include N U T

class System

@+ Simplification
\label{code/D:simplify}

The simplification function, defined below,
is called for instance in the function $eval$, defined in 
section \ref{code/P:eval}, to simplify atoms.
Simplifying a term means replacing that term by another term or itself,
possibly with side-effects,
such as the creation of an object or a process.
As an example, the function $simplify$ is used to replace
non-logical variables by their values, if appropriate.

A call to the simplification function must have
a parameter $q$ referring to an evaluation process
to have access to the object for which the inference takes place.

The  integer $k$ is used to indicate whether the left hand side of
an equality is evaluated ($k=1$), the right hand side ($k=2$),
or an ordinary expression ($k=0$).

The parameter $nvars$, indicating the number of variables in use,
is passed to the method $request$ when creating a new active object.
@]

fn simpl(q:Q,k:N,nvars:N,t0:T): T                           %%% simplify
temp n:N, r,t1,t2:T, o:O, z:A, t := t0
{

@[ 
We first check whether $t$ is a non-logical variable. 
If the process $q$ is evaluating the guard of a conditional
accept expression, then the non-logical value of the object
may not be accessed, since otherwise a deadlock would occur.
@]


if t == nil then %%%% do nothing
elsif iscon(t) & ~ isnil(t) & ~ isint(t) & ~ q@guard then r := q@obj@chvl; 
	if member(t,r) then 
		if k=1 then
			t := Chv(q@obj!chv(t))
		else
			t := q@obj!chv(t)@val
		fi;
	fi;
fi;

@[ Then we continue with the simplification.
Recall that only constants, logical variables and function terms
are considered $symbolic$.
C.f. section \ref{code/T:defs}.
@]

if t==nil then result nil
elsif ischv(t) then
	if chvof(t)@channel then result t else return(k=1,t,chvof(t)@val) fi
elsif ~symbolic(t) then result t

@[ The following constants may be used for allocating objects
and processes.
The expression {P my_node P} stands for the processor node
of the object calling $simplify$.
See further section \ref{code/U:fun} for the definitions of
the functions used.
@]

elsif isconst("here",t) then result Int(my_node)
elsif isconst("branch",t) then result Int(U.branch(q))
elsif isconst("width",t) then result Int(U.width(q)) 
elsif isconst("nproc",t) then result Int(U.nproc(q)) 

@[ The constant $scratch$ refers to the local clauses of an evaluation 
process, to be modified by the use of the predicates $assert$
and $retract$. @]

elsif isconst("scratch",t) then result t_pl(q@clauses)

@[ If the term to be simplified is a function term,
its arguments are simplified first except for certain
special function symbols. @]

elsif isfuncar("quote",2,t) then result a1(t)
elsif ~isfun(t) then result t
elsif isfun(t) then z:=fof(t); n := arity(t);
  if z ~= "##" & z ~= "@" & z ~= "." & z ~= "val" & z ~= "global" then
  a1(t):= simpl(q,k,nvars,a1(t)); a2(t):= simpl(q,k,nvars,a2(t));
  fi; 
  t1:=a1(t); t2:= a2(t);
  if z = "##" then t2 := simpl(q,k,nvars,t2) fi;

@[ A {\it node expression} is converted to an integer.
See section \ref{code/U:fun} for the definition of $node$.
@]

  if z="node" then result Int(node(q,a1(t)))

@[ The tag of a term can be asked for. @]

  elsif z="tagof" then result Int(tagof(t1)) 

@[ A term can be converted to an object, or interpreted
as a local or global non-logical variable, or an option.
These expressions may be used to avoid name clashes between
objects, non-logical variables and options.
@]

  elsif z="obj" then return(isconst("self",t1), Obj(q@obj), Obj(q@super[t1]))
  elsif z="val" then
	return(k=1,Chv(q@obj!chv(t1)), simpl(q,k,nvars,q@obj!chv(t1)@val)) 
  elsif z="global" then
	return(k=1,Chv(knot!chv(t1)), simpl(q,k,nvars,knot!chv(t1)@val)) 
  elsif z="option" then return(k=1,t, q!getopt(t1) ) 

@[ An {\it @-symbol} is meaningful only when its first
argument is an object or a process.
It is used to access the value of the non-logical variable of an object,
or to inspect the value of an option for
a particular process.
@]

  elsif z="@" & k ~= 1 then t1 := simpl(q,k,nvars,t1);
	if iscon(t1) then t1 := Obj(q@super[t1]) fi;
	if isobj(t1) then result simpl(q,k,nvars,objof(t1)!chv(t2)@val) 
	elsif isprc(t1) then result prcof(t1)!getopt(t2)
        else result t
	fi;

@[ A {\it new}-expression results in either a new object,
a new process or a new channel.
For an object also a new constructor process may be started.
C.f. section \ref{impl/proto/D:simplify}.
@]

  elsif z="new" then n := my_node; %%% $new(t)$
	if isfun(t1) & fof(t1)="@" then
		n := node(q,a2(t1)); t1 := a1(t1);
	fi;
	if isconst("this",t1) then result Prc(q!cpy(n))
	elsif isconst("self",t1) then result Obj(q@obj!cpy(n))
	elsif fof(t1)="channel" then
		if iscon(t1) then t2:=knot!num() fi;
		result Chv(X.new(t2)!put_channel(true))
	elsif isfun(t1) then
		o:= q@super[Con(fof(t1))]@ini!cpy(n);
		o@acceptlist:=Con(fof(t1)); o@active := true;
		r:= o!request(nil,0,nvars,t1);
		o@constructor := prcof(r); result Obj(o)
	elsif isobj(t1) then result Obj(objof(t1)!cpy(n))
	elsif ischv(t1) then result Chv(X.new(chvof(t1)@name))
	elsif isprc(t1) then result Prc(prcof(t1)!cpy(n))
	elsif symbolic(t1) then result Obj(q@super[t1]@ini!cpy(n))
 	else result t
	fi

@[ A straightforward interpretation of (syntactic) equality and
inequality is given. @]

  elsif z="==" then return(eq_t(t1,t2),Nil,Fail) 
  elsif z="!=" then return(eq_t(t1,t2),Fail,Nil) 

@[ We have defined some macros to deal respectively with arithmetical
simplifications and the evaluation of comparison operators.
When testing whether $t$ was a function symbol,
we have put the arity of $t$ in $n$.
@]

df a_simpl(O,T1,T2)= if n=2 then ar_simpl(O,T1,T2) else result t fi;
df b_simpl(O,T1,T2)=if n=2 then bl_simpl(O,T1,T2) else result t fi;

df ar_simpl(O,T1,T2)= if isint(T1) & isint(T2) \
      then result Int( intof(T1) O intof(T2) ) \
      else result Op("O",T1,T2) fi;
df bl_simpl(O,T1,T2)= if isint(T1) & isint(T2) \
      then if intof(T1) O intof(T2) then result Nil \
      else result Fail fi else result Op("O",T1,T2) fi;

@[ Finally, we provide an interpretation for
the arithmetic and comparison operators. @]

  elsif z="+"  then a_simpl(+,t1,t2)  %% arithmetic
  elsif z="-"  then a_simpl(-,t1,t2) 
  elsif z="/"  then a_simpl(/,t1,t2) 
  elsif z="*"  then a_simpl(*,t1,t2) 
  elsif z="**" then a_simpl(**,t1,t2) 
  elsif z="//" then a_simpl(//,t1,t2) 
  elsif z="<"  then b_simpl(<,t1,t2) 
  elsif z="<=" then b_simpl(<=,t1,t2) 
  elsif z=">"  then b_simpl(>,t1,t2) 
  elsif z=">=" then b_simpl(>=,t1,t2) 

  else result t
  fi 

else result t
fi
} simpl


@+ Equality and assignment
\label{code/D:assign}

The function $assign$, implemented by the function below,
takes the same parameters as an ordinary database function.
It is called by the $system$ database
to evaluate the binary 'predicates' for equality and assignment.
C.f. section \ref{code/D:system}.
See also section \ref{impl/proto/D:assign}.
@]

fn eq_assign(q:Q, t:T, c:C, m,n:N, s:S) : T  %%% assign
temp k1:=1,k2:=2, t1,t2,ot:T,
	q1:Q, nd:= my_node, z := fof(t)
{

@[ We first simplify both the left hand side and right
hand side of the equality and check whether both parts are
syntactically identical, after simplification. @]

if z="=" then k1 := 2 elsif z="::" then k2 := 2 fi;
t1:= simplify(q,k1,n,a1(t)); t2:= simplify(q,k2,n,a2(t));
if eq_t(t1,t2) then result c(m,n,s)

@[ When the (simplified) left hand side is not a variable,
we interpret the equality statement as assigning
a value to a non-logical variable or
to an option of a process.
Local options can also be given a value, directly.
When no such form can be recognized both terms are simply
unified.
@]

elsif ~isvar(t1) & k1=1 then
   if isfuncar("@",2,t1) then
     if iscon(a1(t1)) then a1(t1) := Obj(q@super[a1(t1)]) fi;
     if isobj(a1(t1)) then objof(a1(t1))!chv(a2(t1))@val:=t2; result c(m,n,s)
     elsif isprc(a1(t1)) then prcof(a1(t1))!putopt(a2(t1),t2); result c(m,n,s)
     else result unify(t1,t2,c,m,n,s)
     fi
   elsif ischv(t1) & ischv(t2) then
	chvof(t1)@val := chvof(t2)@val; result c(m,n,s)
   elsif ischv(t1) then chvof(t1)@val := t2; result c(m,n,s)
   elsif iscon(t1) then
	if member(t1,options) then q!putopt(t1,t2); result c(m,n,s)
	else result unify(t1,t2,c,m,n,s)
	fi
   elsif isfun(t1) & fof(t1) = "option" then q!putopt(t1,t2); result c(m,n,s)
   else result unify(t1,t2,c,m,n,s)
   fi

@[ A new evaluation process is started when an expression of the form 
$Q = O!G$ is encountered. In case $O$ refers to an evaluation
process,
a copy of the process is made to evaluate the goal $G$.
Otherwise $O$ is supposed to refer to an object, which is asked for
a rendez-vous. @]

elsif isfuncar("!",2,t2) then ot:=a1(t2); nd := my_node; %% $Q=O!G$
   if symbolic(ot) then ot := Obj(q@super[ot]) fi;
   if isobj(ot) then
	q1:= objof(ot)!process(nd); 
	if q[COPY]>0 then for i from 1 to MAXOPT do q1[i]:=q[i] od fi;
	objof(ot)!request(q1,m,n,a2(t2)); 
	result unify(t1,Prc(q1),c,m,n,s)
   elsif isprc(ot) then q1 := prcof(ot)!cpy(nd);
        infer(q1,a2(t2),yes(q1,m,a2(t2)),m,n,s);
	result unify(t1,Prc(q1),c,m,n,s) 
   else perror("eq" + qstr(t)); result Error;
   fi


else
   result unify(t1,t2,c,m,n,s)
fi;
} eq_assign


@+ The system database 
\label{code/D:system}.

We have defined a number of primitive predicates in
the database function $system$.
This function is put in composition with the database
of each object, as described in section \ref{code/O:init}.
@]


fn start_d( q:Q, t:T, c:C, m,n:N, s:S) : T         %%% system
temp z:=fof(t), t1,t2,g,r,mt:T,
	qa:Q,n1:N, e:E, b:=false, f:fn(T):B
{
q!tr("D:system",m,t);
case tagof(t) of
CON then
  if z="nl" then q!!wr(Con("\n"));result c(m,n,s) 
  elsif z="!" then q!cuton(m,Cuton(0,"X")); result c(m,n,s) 
  elsif z="push" then q!push(); result c(m,n,s)
  elsif z="pop" then q!pop(); result c(m,n,s)
  else result nil
  fi; or

VAR then result eval(q,maps(s,t),c,m,n,s) or

FUN then t1:=a1(t); t2:=a2(t);

@[ Some familiar {\bf system predicates} are defined below. @]

  if z=":-"|z="consult"|z="assert"|z="retract" then
  	            q!super(t); result c(m,n,s) 
  elsif z="write" then q!!wr(argsof(t));result c(m,n,s) 
  elsif z="unify" then result unify(t1,t2,c,m,n,s) 
  elsif z="\\+" then qa:=q@cpy; r:=q@d(qa,argsof(t),yes(qa,m,t1),m,n,s);
                  if r==nil then result c(m,n,s) else result nil fi; 
  elsif z="="|z=":="|z="::" then result assign(q,t,c,m,n,s) 
  elsif z=","  then result eval(q,t,c,m,n,s) 

@[ For our interpretation of the cut,
to check whether a goal is acceptable, to evaluate an accept statement
and to force an output process waiting for communication over a channel,
we have defined the following predicates. @]

  elsif z="new" then result eval(q,Eq(Var(0),t),c,m,n,s)
  elsif z="acceptable" then
	return(isacceptable(t1,q@obj@acceptlist),c(m,n,s),nil) 
  elsif z="accept" then r := q@obj!accept(q,n,argsof(t)); 
		if r == nil then r := Nil else r := Un("?",r) fi;
		q!tr("accepted",m,t); 
                mt := q@obj@post; n1 := intof(a1(mt)); r := mkc(r,a2(mt));
		%dbx(accepted,qstr(r) + n1@A );
		result eval(q,r,c,m,n1,s) 
  elsif z="cuton"   then q!cuton(m,t); result c(m,n,s) 
  elsif z="free"|z="fail" then
       if ischv(t1) then
          if z="free" then r:=Var(0) else r:=Error fi;
 	  chvof(t1)!force(r); result c(m,n,s)
       else result nil fi
  elsif z="##" then result c(m,n,s) %%%% see section \ref\{app/code/O:accept\}


@[ Additionally, we define the $univ$ predicate for
compatibility with Prolog. @]

  elsif z="=.." then
	    if ~isvar(t1) then t1:= Con(fc(t1));
  	        r:= Op(".",t1,t_pl(argsof(arg(1,t))));
 	        result unify(r,arg(2,t),c,m,n,s)
	    else t1:= maps(s,arg(2,t));
		if isfc(".",t1) then  
		   r:= Un(conof(arg(1,t1)),pl_t(arg(2,t1)));
  	           result unify(arg(1,t),r,c,m,n,s)
		else result nil
		fi
	    fi; 

@[ Synchronous  communication is dealt with by the following
definitions. @]

  elsif z="!" then
	    if ischv(t1) then  %%% $C!t$
	         chvof(t1)!write(t2); r:=chvof(t1)!receive();
		 n1 := maxvar(r); e := fn(nv:V):V { result n+nv };
		 r := mape(e,r); result unify(t2,r,c,m,n+n1,s);
	    elsif isconst("self",t1) then result eval(q,t2,c,m,n,s) 
	    else %%% $O!G$
		 r := Var(n+1);
		 result eval(q,mkc(Eq(r,t),Un("?",r)),c,m,n+1,s);
	    fi 

  elsif z="?"  then 
	    if isobj(t1) then t1 := Prc(objof(t1)@constructor); fi;
	    if isnil(t1) then result c(m,n,s)
	    elsif isprc(t1) then r := prcof(t1)!resume(); %%% $Q?$
		 if t2 ~== nil then r := Fun("unify",t2,r) fi;
		 if prcof(t1)@state ~= BUSY then r := r
		 else r:=Fun(";",r,t)
		 fi; result eval(q,r,c,m,n,s)
	    elsif ischv(t1) then mt:=t2; g:=Nil; %%% $C?t$
		 if isfuncar(":",2,mt) then
			g:=a2(mt); mt:=a1(mt)
		 fi;
		 r:=chvof(t1)!read(mt);
		 n1 := maxvar(r); e := fn(nv:V):V { result n+nv };
		 r := mkc(Fun("unify",mt,mape(e,r)),
				mkc(g,Un("confirm",t1)));
		 result eval(q,r,c,m,n+n1,s);
	    else result nil
	    fi 

@[ When a successful communication (over a channel) is possible,
the goal $confirm$ is called from the input side. @]

  elsif z="confirm"  then chvof(a1(t))!confirm(); result c(m,n,s) 

@[ Finally, we define the interpretation
of the predicates $call$ and $bagof$. @]

  elsif z="call"  then result eval(q,argsof(t),c,m,n,s) 
  elsif z="bagof" then result bagof(q,t,c,m,n,s)  
  else  result nil
  fi;
else result nil
esac;
} start_d


@[ To evaluate the {\bf bagof} predicate we need the function $bagof$. @]

fn bag_of( q:Q, t:T, c:C, m,n:N, s:S) : T %%% bagof
temp qa:Q, g,r:T, a:T 
{
if arity(t) ~= 3 then perror(qstr(t)); result Error
else
   qa := q@cpy; g:= Un("bag",arg(1,t)); g:= mkc(arg(2,t),g);
   infer(qa,g,yes(qa,m,g),m,n,s); r := t_pl( qa!bag() );
   dbx(bag,qstr(Prc(qa)) + "::" + qstr(r));
   result unify(r,arg(3,t),c,m,n,s); 
fi
} bag_of


end System
