		%% @(#)N.h	1.3

@[ As types we have
<I
- $A$ for strings,
- $B$ for booleans,
- $N$ for integers,
- $V$ for logical variables,
- $O$ for objects in the sense of DLP,
- $Q$ for evaluation processes, and
- $X$ for non-logical variables and channels
I>
that function as basic types for the type
<I>- T for terms.
For function terms, we use the type
<I>- $F = A * T * T$
which is a recursive type,
since $F$ is a subtype of $T$. C.f. section \ref{code/T:impl}.

As abbreviations, for implementing the Prolog interpreter
(c.f. section \ref{impl/proto/P}), we employ
<I
- $E = V  ->  V$ for environments or renaming functions,
- $S = V -> T$ for substitutions,
- $C = N * N * S -> T$ for continuations,
- $D = Q * T * C * N * N * S -> T$ for database functions, and
- $K = D -> D$ for (clause) continuations.
I>
Also we use the type
<I>- $P$ for sequential (Prolog) inference processes.
As actual macro definitions we have: @]

df A = String
df B = Bool
df N = Int$_$
df V = N

@[ In other words, the identifiers $A$, $B$, $N$ and $V$
are replaced, in the program text, by their respective definitions.
As a remark, when $A$ occurs in an expression like
$n@A$ for an integer variable $n$ then this must be read,
according to POOL* conventions, as $n!get_String()$, which delivers
the string representation of the integer value of $n$.

@ Abbreviations 

To be able to use conditional result {\bf statements}
we have included the following macro-definitions @]

df return(B,X,Y) = if B then result X; else result Y fi;
df returnil(T,B,X,Y,Z) = if T==nil then result X else return(B,Y,Z) fi; 

@[ The second variant first tests whether the argument T is $nil$.
It is mainly used in the unit U, defining the utilities.
\ln

The most important active objects are those embodying an
evaluation process.
The communication protocol used there makes use of a {\bf state}
variable, that can take the values defined here. @]

df BUSY = 0
df PEND = 1
df WAIT = 2
df STOP = 3
df FREE = 4

df HALT = 5

@[ We define the major {\bf derivation functions} as global functions.
The function $infer$ is used to start up a new inference.
The function $eval$, $unify$, $assign$ and $bagof$ are used respectively to
evaluate a goal, to perform a unification, for interpreting a 
special form involving equality and assignment, and to execute
a $bagof$ goal. @]

df infer = P.new()!!inferQ
df eval = P.evalQ
df unify = P.unifyQ
df assign = System.eq_assign
df bagof = System.bag_of

@[ The definitions of a number of {\bf global functions}
are collected below. @]

df mape(E,T) = U.m_e(E,T)
df maps(S,T) = U.m_s(S,T)
df mapc(M,S,T) = U.m_c(M,S,T)

@[ The function $mapc$ is used to rewrite cuts in a proper form.
The functions $maps$ and $mape$ are used to effect respectively
renamings and substitutions.
These functions, as the functions below, are primarily
used in the unit P. @]

df compile = P.compile_D
df compose = P.composeD
df system = System.start_d

df yes = P.yes_q
df starts  = P.start_s
df nilD = P.nil_D
df nilK = P.nil_K
df failD = P.fail_D
df checkcut = Q.check_cut

@[ To collect the variables of a term  we use $varsin$. @]

df varsin(T) = U.v_in(T)

@[ We use compound terms for lists of terms and define functions for
appending terms, and for checking whether a term is contained in a list. @]

df append(T1,T2)= U.t_app(T1,T2)

df member(T,L) = ( U.t_in(eq_t,T,L) > 0 )
df inlist(T,L) = U.t_in(eq_t,T,L)
df atlist(T,L) = U.t_at(eq_t,T,L)

@[ Both the $inlist$ and the $atlist$ function return the position of
the term in the list, the latter however does not return zero if the
term is not in the list but modifies the list by making it an element 
of the list. 

Further we define 
a function to select a list of terms satisfying
some criterion, a function to add objects defined by a term T to a list
of objects L,
and a function to simplify terms. @]

df select(F,L) = U.sel(F,L)
df addchv(T,L,A) = X.add_chv(T,L,A)
df simplify = System.simpl

@[ For actually allocating POOL* objects on processors
we use the definitions below.
The class $Node$ is defined in the standard unit $Nodes$.
@]

df node(Q,T) = U.nd(Q,0,T)
df my_node = (Node.my_Node()@N)

@[ Finally, we define some macros for debugging and error messages. @] 

df dbx(P,X) = if knot@debug then io!!p("***P:" + X); fi;
df perror(X) = io!!p("###error " + X)
