impl unit P 		%%	@(#)P.i	1.3

use IO Q D T Text
include N U T

@[ We need aliases for the types of environments (or renaming
functions) E, substitutions S, continuations C, database functions D
and clause continuations K. @]

alias E = fn(V):V
alias S = fn(V):T

alias C = fn(N,N,S):T
alias D = fn(Q,T,C,N,N,S):T
alias K = fn(D):D

@[ The code for sequential (Prolog) inference processes is collected
in the following class definition. @]

class P

@[ For creating a new inference process
we have defined a macro $infer$ in section \ref{code/N},
that expands to $P.new()!!inferQ(...)$, with the (asynchronous)
method $inferQ$
as defined below.
@]

method inferQ(q:Q,t:T,c:C,m,n:N,s:S)    { eval(q,t,c,m,n,s) } inferQ %% infer\ftn\{For each method or function definition corresponding with a global macro definition, as defined in the unit N we have given the global name as a comment.\} 


@+ Evaluation
\label{code/P:eval}
The function $eval$ extends the evaluation function described in section
\ref{impl/proto/P:eval}. Among other things
it tests for termination due to a halt statement. @]

fn evalQ(q:Q,t:T,c:C,m,n:N,s:S):T                           %%% eval
temp z,a,g:T, asktail:C
{
if t == nil then result c(m,n,s)
elsif q@state = STOP then result nil
else
  if isc(t) then q!tr("compound",m,t); a:=a1(t); g:=a2(t) else a:=t fi;
  asktail:= fn (m1,n1:N, s1:S) : T 
            temp  mct:= m1 , rx:= maps(s1,a)
	    {
	    if iscut(a) then mct:= mct + 1; fi;
	    q!tr("exit",m,rx); result evalQ(q,g,c,mct,n1,s1)
	    };
  a:= maps(s,a); if fof(a)="new" then a:= Eq(Var(0),a) fi; 
  if simplifiable(fof(a)) & q[NOSIMPLIFY] = 0 then
  	a := simplify(q,0,n,a);
  fi;
  a := q!tr("call",m,a);
  if fof(z)="halt" then result nil
  elsif isnil(a) then result asktail(m,n,s)
  else result q@d(q,a,asktail,m,n,s)
  fi;
fi
} evalQ


@+ Compilation
To be able to trace which database function is used for evaluating
a goal, a function is created that contains, apart from the
compiled database, also a call to the evaluation process to give notice of
the name of the object to which the database belongs. @]

fn compile_D(o,cl:T,d:D,k:K):D                             %%% compile
temp d1:= fn(q:Q,t:T,c:C,m,n:N,s:S):T
	  { q!tr("D:"+str(o),m,t); result d(q,t,c,m,n,s); }
{ io!!p("***compile:"+qstr(o)); result compileD(cl,d1,k); } compile_D

@[ The actual compilation is taken care of by the following function. @]

fn compileD(cl:T,d:D,k:K):D
temp
 newd:D, dotail:K
{
if cl == nil then dbx(compile," empty clause "); result k(d) 
elsif isnil(cl) then return(k==nil,d,k(d))
elsif isc(cl) then
  dotail:= fn (d1:D):D { result compileD(a2(cl) , d1, k) };
  result compileD(a1(cl),d,dotail);
else
  dbx(co_C,str(cl));
  newd := fn(q:Q, t:T, c:C, m,n:N, s:S):T 
	  temp  fs:A, b:B, m1,n1,nvars:N, e:E, s1:S, bdy:C, ru,rd:T
	  {
	  fs:= fof(a1(cl)); rd := d(q,t,c,m,n,s);
	  if fof(t) ~= fs then result rd
	  else b:= checkcut(q,t,m,n);
	    if b then q!tr("cut*",m,t); result rd
	    else
	      m1:=m+1; nvars:=maxvar(cl); n1:= n+nvars;
	      e := fn (v:V):V { result v + n };
	      s1 := fn(v:V):T { return(v>n,Var(v),s(v)) };
	      bdy:= fn (m2,n2:N,s2:S):T
	      { result evalQ(q,mapc(m,fs,mape(e,a2(cl))),c,m2,n2,s2) };
	      q!tr("try",m,cl);
	      ru := unifyQ( mape(e,a1(cl)),t,bdy,m1,n1,s1);
	      result append(rd,ru);
	    fi;
	  fi;
	  }; %%%% newd
   return(k==nil,newd,k(newd))
fi
} compileD

[@
As an optimization, we check whether the predicate name 
of the goal is identical to
the predicate name of the head of the clause.

@+ Unification
We have employed a rather simple unification algorithm, using a direct
recursion on subterms, when necessary.
Notice that whenever we encounter two variables we update the
older variable, the one with the lowest number,
since the younger variable will become unbound first. @] 

fn unifyQ( t1,t2: T, c:C, m,n:N, s:S ):T                    %%% unify
temp
 doa:C
 upd:= fn (v:V,t:T):T
       temp news := fn (v1:V):T { return(v==v1,t,s(v1)) }
       { return(unbound(v,s),c(m,n,news),unifyQ(s(v),t,c,m,n,s)) }
{
if t1==nil | t2==nil then return(t1==t2,c(m,n,s),nil)
elsif isnull(t1) | isnull(t2) then result c(m,n,s)
else
case tagof(t1) of
CON then case tagof(t2) of
	 VAR then result upd(varof(t2),t1)  or
	 CON then return(conof(t1)==conof(t2),c(m,n,s),nil)
	 else result nil esac or
VAR then if isvar(t2) then
    	    if varof(t1) = varof(t2) then result c(m,n,s) 
	    elsif varof(t1) < varof(t2) then result upd(varof(t1),t2) 
	    else result upd(varof(t2),t1) 
	    fi
	 else result upd(varof(t1),t2) fi; or
FUN then case tagof(t2) of
	 VAR then result upd(varof(t2),t1) or
	 FUN then if fc(t1) = fc(t2) then
	             doa:= fn(m1,n1:N,s1:S):T
		        { result unifyQ(a2(t1),a2(t2),c,m1,n1,s1) };
		     result unifyQ(a1(t1),a1(t2),doa,m,n,s)
	             else result nil fi
	 else result nil esac
else result nil esac fi
} unifyQ

@+ Initialization

The function $yes_q$, implementing $yes$, results in a continuation
that notifies the evaluation process $q$ of the answers that
result from evaluating a goal. @]

fn yes_q(q:Q, m:N, t:T):C                   %%% yes 
temp 
  l:= Yes(varsin(t))
  c:= fn(m1,n:N,s:S):T
  temp r:= maps(s,Dot(t,l)) { q!tr("yes",m,r);result a2(r) }
{ result c; q!tr("goal",m,Dot(t,l))  } yes_q

@[ Next we give the definition for the initial substitution $starts$,
the initial database $nilD$ and the initial clause continuation
$nilK$. @]

fn start_s(v:V):T { result Var(v) } start_s                   %%% starts

fn nil_K(d:D):D { result d } nil_K                         %%% nilK
fn nil_D(q:Q,p:T,c:C,m,n:N,s:S):T { result nil } nil_D     %%% nilD

@[ The always failing database function $failD$ results,
apart from notifying the evaluation process of failure, in $nil$. @]

fn fail_D(q:Q,p:T,c:C,m,n:N,s:S):T  { q!tr("fail",m,p); result nil } fail_D   %%% failD

@+ Composition
In the composition function $compose$, the parameter $b$ determines
whether the overwriting variant of composition is used ($over = 1$),
or the backtracking variant (which is the default, $over=0$).  @]

fn composeD(over:N,d1,d2:D):D                                      %%% compose
temp d:= fn(q:Q,t:T,c:C,m,n:N,s:S):T
	 temp r :=d1(q,t,c,m,n,s)
	 { 
	 if over==0 then result append(r,d2(q,t,c,m,n,s))
	 elsif r==nil then result d2(q,t,c,m,n,s)
	 else result r
	 fi;
	 }
{ return(d2==nilD,d1,d) } composeD

end P
