%%	@(#)T.h	1.3

@[ {\bf Tags}
are used to employ a form of dynamic typing for terms.
With the help of these tags we can define the type of a term
by means of a kind of tagged variant record.
We provide $constructors$, $selectors$ and $tests$ for the major
term types.
We have adopted the convention to use capitals for the
first letter of a constructor, to use the postfix $of$ \ for
selectors that give access to the contents of a term
and to use the prefix $is$ for tests. @]

df CON=1
df VAR=2
df FUN=3
df OBJ=4
df PRC=5
df CHV=6
df REC=7

@[ {\bf Constructors}
must be used to create terms from objects of some basic type, or
to create a function term from terms.
In order to be able to employ union types in a convenient way,
we have used a name concatenation facility of our
macro processor, which removes the $ . _ . $ part after applying all
expansions.
See section \ref{impl/comp/proto:lang}.
@]

df mkterm(C) T.new._.C
df Con(X) = mkterm(CON)(X)
df Var(X) = mkterm(VAR)(X)
df Fun(S,T1,T2) = mkterm(FUN)(F.mk(S,T1,T2))
df Obj(X) = mkterm(OBJ)(X)
df Prc(X) = mkterm(PRC)(X)
df Chv(X) = mkterm(CHV)(X)
df Rec(X) = mkterm(REC)(R.new()!X)

@[ {\bf Selectors}
are used to access the constituting part of a term.
The selectors each address one of the fields of the union type.
For instance $conof(t)$ expands to $t@1$, which accesses
the first field of $t$, containing a string.
@]

df conof(T) = ((T) @ CON)
df varof(T) = ((T) @ VAR)
df funof(T) = ((T) @ FUN)
df objof(T) = ((T) @ OBJ)
df prcof(T) = ((T) @ PRC)
df chvof(T) = ((T) @ CHV)
df recof(T) = ((T) @ REC)

@[ We can further analyze a term by using the definitions below. @]

df fof(T) = U.func_of(T)
df fc(T) = T @ FUN @ f
df a1(T) = ( T @ FUN @ arg_1 )
df a2(T) = ( T @ FUN @ arg_2 )
df arg(N,T) = U.get_n(N,argsof(T))
df arity(T) = U.length(argsof(T))

df argsof(T) = U.args_of(T)

@[ {\bf Tests}
are provided for determining the type of a term. @]

df tagof(T) =  ( T @ switch )
df symbolic(T) = (tagof(T) < OBJ)
df iscon(T) = ( tagof(T) == CON )
df isvar(T) = ( tagof(T) == VAR )
df isfun(T) = ( tagof(T) == FUN )
df isobj(T) = ( tagof(T) == OBJ )
df isprc(T) = ( tagof(T) == PRC )
df ischv(T) = ( tagof(T) == CHV )
df isrec(T) = ( tagof(T) == REC )

df isun(T) = ( a2(T) == nil )
df isbin(T) = ~ isun(T)

df isconst(C,T) = U.is_con(C,T)
df isfc(F,T) = U.is_fun(F,T)
df isvarno(N,T) = U.is_var(N,T)

@[ {\bf Compound terms}, 
that are function terms having
a $comma$ as a function symbol, play a special role,
and therefore deserve a special test and constructor. @]

df isc(T) = (isfun(T) & fof(T)=",")
df mkc(A1,A2) = Fun(",",A1,A2)

@[ {\bf Auxiliary constructors, selectors and tests}
are provided for convenience. @]

df unbound(V,S) = isvarno(V,S._.(V))
df isground(T) = U.is_ground(T)
df maxvar(T) = U.max_v(T)

df Int(N) = Con((N)@A)
df isint(T) = U.is_int(T)
df intof(T) = U.int_of(T)

df Error = Con("error")
df Clause(H,B) = Fun(":-",H,B)
df Fact(F) = Clause(F,Nil)
df Query(C) = Un("?-",append(C,Nil))
df Op(O,A1,A2) = Fun(O,A1,A2)
df Un(O,A) = Fun(O,A,nil)
df Dot(T1,T2)= Op(".",T1,T2)
df Nil = Con("[]")
df Fail = Con("fail")
df True  =  Con("true")
df Eq(T1,T2) = Op("=",T1,T2)
df Initial = mkc(Nil,nil)
df Cuton(M,S) = Fun("cuton",Int(M),Con(S))
df Cut(M,T) = Dot(Int(M),T)
df Yes(T) = Un("yes",T)
df Mbr(T,L) = Op("member",T,L)

df isnil(T) = isconst("[]",T)
df isfail(T) = isconst("fail",T)
df iscut(T) = ( isconst("!",T) | fof(T)=="cuton" )
df isnull(T) = isvarno(0,T)
df isunit(T) = isconst("()",T)
df iscollect(T) = ( isfc("bagof",T) | isfc("setof",T) )

@[
Recall that we have implemented logical variables as integers.
We use the variable with number zero as the anonymous or null variable,
that in Prolog is written as an underscore.

To make a copy of a term we use $cpyof$. @]

df cpyof(T) = U.t_cpy(T)

@[ We use $isfuncar$ to check if a term is a function term with
function-symbol S and arity N. @]

df isfuncar(S,N,T) = U.is_funcar(S,N,T)

@[ Further we may like to check if a term is a clause, and if so
what kind of clause. @]

df isfact = U.is_fact
df isclause = U.is_clause
df headof = U.head_of

@[ To extract a name from a term, containing possibly apart from
a name also an initialization part or inheritance information,
we use the function $nameof$. @]

df nameof = U.g_n

@[ Partly the definitions are stated in terms of prior 
definitions and partly as an abbreviation for a function
defined in the unit U.
For a better understanding of the program text it is worth 
the trouble to study the definitions given above with some care
and to inspect the implementation given in the unit
defining the utilities. See section \ref{code/U:fun}.
\ln

For testing the {\bf equality} of two terms we provide two functions,
the latter of which allows a kind of matching. @]

df eq_t = U.t_eq
df match = U.t_mt

@[ In some cases it is required to convert a compound term to
a Prolog list, and vice versa. @]

df t_pl = U.t__pl
df pl_t = U.pl__t

@[ For converting a term to string we provide two functions.
The second differs from the first only in that it puts some
space around the term. @]

df str(T) = U.t_str(T)
df qstr(T)= " "+str(T)+" "
