impl unit X		%%	@(#)X.i	1.3

use IO Q O D P U T, Nodes
include N U T


@[ Objects of type $X$
are used both to store (persistent) values, and to effect
a synchronous communication between processes.
@]

class X
newpar(name:T)
var
	val get : T
	channel put get := false

	confirmed := false
	free := true
	store := Nil
	message := Nil

	me := Chv(self)

@+ Non-logical variables

Non-logical variables may be initialized. @]

init if isfuncar("=",2,name) then val := a2(name) else val := Nil fi; tini

@[
When assigning a value to a non-logical
variable, we take some precautions when the name of the variable 
is nil,
since these variables are used to communicate a substitution between
the process evaluating a method call and the process in which
the (conditional) accept statement occurred
by which the method call got accepted.
These variables may be assigned a value only once.
Moreover, to avoid variable clashes between processes,
all (unbound) logical variables are mapped to the anonymous logical
variable, with number zero.
@]

method put_val(t:T):X 
temp e : fn(V):V
{ 
if name == nil then
	if isnil(val) then 
                e := fn(v:V):V { result 0 };
		val:= mape(e,t);
	fi
else val := t
fi; result self
} put_val

@[
For adding a list of non-logical variables $t$ to a symbol-table pair 
$(l,ar)$ the function
{\it add_chv} is used.
C.f. section \ref{code/O:proto}.
@]

fn add_chv(t,l:T,ar:Array(X)):T
temp n:N
{
if t==nil then ;
elsif isc(t) then add_chv(a1(t),l,ar); add_chv(a2(t),l,ar);
else n := atlist(nameof(t),l); if ar[n]==nil then ar[n]:=X.new(t) fi; 
fi; result l
} add_chv

@[
The name of a non-logical variable,
possibly containing an initialization,
is delivered by the method $get_name$.
@]

method get_name():T { result name } get_name

@+ Channels
\label{code/X:channels}

Non-logical variables can be interpreted as
channels that allow {\bf synchronous communication}. 
See also the discussion in section \ref{impl/proto/X:channel}.

The {\bf protocol} governing synchronous communication over channels
allows the input side to backtrack until a unification
between the input term and the output term is possible.

When involved in a communication,
only a limited number of methods may be answered.
Part of these are collected in the macro ME.
@]

df ME = get_name, get_val, put_val, get_channel, put_channel

body do
if ~confirmed & ~free then answer(ME, read, confirm, force )
elsif ~free & confirmed then answer(ME, receive);
else
	answer any
fi
od ydob

@[ To engage in a communication
the input side must call the method $read$.  @]

method read(t:T):T  
{
store:= t;
while isnil(message) do dbx(read,qstr(t)); answer(ME,write) od;
result message 
} read

@[ The output side is supposed to call the method $write$. @]

method write(t:T):T
{
while ~isnil(message) do dbx(write,qstr(t)); answer(ME,read) od; 
message := t; free:= false; confirmed:= false;
result nil 
} write


@[ For confirming the possibility of
a unification and to collect the input term,
the methods $confirm$ and $receive$ must be used. @]

method confirm():X { confirmed:= true; result self } confirm

method receive():T {
free:= true; confirmed:=false; message:=Nil; result store
} receive

@[ The method  defined below forces the output side to
backtrack or to continue the computation,
dependent on the value of
the argument. @]

method force(t:T):X { store := t; confirmed := true; result self } force


end X
