@+ Syntax and informal semantics

A DLP program consists of a number of object declarations.
The clauses occurring in the object declarations may contain
special statements for updating non-logical (instance)
variables, for creating objects and for communicating with
other objects.

@- Declarations
\label{des/dlp:decl}

An object declaration is of the form
[P
object $name$ {
use $objects$.
var $variables$.

$clauses$
}
P]
Among the clauses may be $constructor$ clauses,
that enable to create active instances of an object.
The clauses of an object act as methods, in that they have exclusive
access to the non-logical variables of an object.
Clauses are ordinary Prolog clauses,
except for the possible occurrence of {\it special forms}
by which DLP extends Prolog.

Inheritance of the non-logical variables of objects
is achieved by including
[P] isa $objects$
in the object declaration.
The declared object then contains a copy of all the
non-logical variables of the objects
mentioned in the $isa$ list.

A declaration of the form
[P] object a:b { $...$ }
must be read as
[P
object a {
isa b.
use b.
$...$
}
P]
Inheritance is static.
It is effected before an object becomes active evaluating a goal.
In other words, there is no interaction between
objects, except on the occasion of an explicit communication.\ftn{
See chapter \ref{des/know} for a more precise characterization
of inheritance, and the resolution of possible name conflicts.
}

@- Statements
\label{des/dlp:stat}

An overview of the special forms by which DLP extends Prolog
is given in table \ref{des/dlp:DLP}.
.so DLP

The core of DLP consists of the special forms
<I
- $v:=t$ - to assign (the value of) the term $t$
to the non-logical variable $v$
- $O=new(c(t))$ - to create an active instance of the object $c$
- $O!m(t)$ - to call the method $m(t)$ for the object $O$
- $accept(m1,...,m_n)$ - to state the willingness to accept methods
$m1,...,m_n$
I>
These are the extensions that play a prominent role
in part \ref{sem},
treating the semantics of DLP.
Moreover, the constructs listed above are sufficient to
solve a number of knowledge representation problems,
as illustrated in chapter \ref{des/know}.
\ln
In addition, however, we have special forms that allow to communicate over
channels.
<I
- $C = new(channel)$ - to create a new channel
- $C!t$ - to put output term $t$ on channel $C$
- $C?t$ - to put $t$ as an input term on channel $C$ 
I>
Apart from the regular accept statement we have introduced
a generalized accept statement, that may contain
{\it conditional accept expressions} of the form
<I>- $method:guard -> goal$
that result in the $goal$ being evaluated,
if $method$ agrees in some way with the call
and moreover the $guard$ holds. (C.f. section \ref{des/ext/accept})

For those wishing to squeeze out the last drop of
parallelism
we have included the primitives
<I
- $ Q = O!G$ - to create a process to evaluate $G$
- $ Q?$ - to ask for the results of process $Q$
I>
These are the primitives that have been used
to implement the synchronous rendez-vous. 

Lastly, since effective parallelism requires some strategy
of allocating resources
we have provided {\it node-expressions}
that can be used to allocate an object or a process, evaluating
a goal.
The special forms
<I
- $O = new(obj@N)$
- $G@N$
I>
where $obj$ is either an object name or a constructor,
respectively allocate an object or a goal at the processor
node denoted by $N$.
\ln
Since we have strived for full compatibility with Prolog,
we have retained the backtracking behavior to the extent possible.
We support both $local$ backtracking, that occurs within
the confines of a process,
and $global$ (or $distributed$) backtracking in which
multiple processes are involved.

We have made a distinction between $active$ objects, which have
own activity that must explicitly be interrupted to
answer method calls; and $passive$ objects that may answer a method 
any time.
Active objects are allowed to have a limited amount of internal
concurrency, in that multiple processes may be active
backtracking over the results of a method call.
Passive objects, on the other hand, display full internal 
concurrency.
Method calls may be evaluated concurrently.
Providing the needed protection 
is in this case the duty of the programmer.
