<document>
@* The ACOI framework
<include file=navigate>

<slide id=acoi type=fig cap="The extended ACOI architecture">
<image name=local/acoi/acoi>
</slide>


The ACOI framework is intended to accomodate
a broad spectrum of classification schemes,
manual as well as (semi) automatic, for the indexing
and retrieval of multimedia objects.
What is stored are not the actual multimedia objects
themselves, but structural descriptions of
these objects (including their location) that may be used for
retrieval.
<p>
The ACOI model is based on the assumption that
indexing an arbitrary multimedia object
is equivalent to deriving a grammatical structure
that provides a namespace to reason about the object
and to access its components.
However there is an important difference with
ordinary parsing in that the lexical and grammatical items
corresponding to the components of the multimedia object
must be created dynamically by inspecting the actual object.
Moreover, in general, there is not a fixed sequence
of lexicals as in the case of natural or formal languages.
To allow for the dynamic creation of lexical
and grammatical items the ACOI framework supports both
<em>black-box</em> ans <em>white-box</em> (feature)
detectors.
Black-box detectors are algorithms, usually 
developed by a specialist in the media domain,
that extract properties from the media object 
by some form of analysis.
White-box detectors, on the other hand,
are created by defining logical
or mathematical expressions over the grammar itself.
In this paper we will focus on black-box detectors only.
<p>
As an example, look at the (simple) feature grammar below,
specifying the structure of a hypothetical community.

<slide type=hide cap="A sample grammar">
<program>
detector world; /// finds the name of the world
detector people; /// checks name, eliminates institutes
detector company; /// looks if there are at least two persons

atom str name;

community: world people company;

world: name;
people: person*;

person: name;
</program>
</slide>

A <em>community</em> consists of people,
and is a community only if it allows for the people
to be in eachothers company.
<p>
A community has a name.
The actual purpose of this grammar is to select the persons
that belong to a particular community from the input,
which consists of names of potential community members.
Note that the grammar specifies three detectors.
These detectors correspond to functions that are invoked
when expanding the corresponding non-terminal in the grammar.
An example of a detector function is the
<em>personDetector</em> function partially specified below.

<slide type=hide cap="A person detector">
<program>
int personDetector(tree *pt, list *tks ){
...
q = query_query("kit=pl src=check.pl");

while (t = next_token(tks)) {
      sprintf(buf,"person(%s)",t);
      query_eval(q,buf);
      if (query_result(q,0)) ///// put name(person) on tokenstream
                putAtom(tks,"name",t);
      }
...
}
</program>
</slide>

The <em>personDetector</em> function checks for each token
on the input tokenstream $tks$ whether the token corresponds
to the name of a person belonging to the community.
The check is performed by an embedded logic component
that contains the information needed to establish whether
a person is a member of the community.
Note that the query for a single token
may result in adding multiple names to the token stream.
<p>
The <em>companyDetector</em> differs from the <em>personDetector</em>
in that it needs to inspect the complete parse tree
to see whether the (implicit) <em>company</em> predicate is satisfied.
<p>
When parsing succeeds and the <em>company</em> predicate is satisfied
a given input may result in a sequence
of updates of the underlying database, as illustrated below.


<slide type=hide cap="Database updates">
<display>
V0 := newoid();
V1 := newoid();
  community_world.insert(oid(V0),oid(V1));
    world_name.insert(oid(V1),"casa");
  community_people.insert(oid(V0),oid(V1));
V2 := newoid();
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"alice");
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"sebastiaan");
    ...
</display>
</slide>

Evidently, the updates correspond to assigning 
appropriate values to the attributes of a structured object,
relecting the properties of the given community.

<p>
The overall architecture of the ACOI framework is depicted in \sliref{acoi}.
Taking a feature grammar specification, such as the simple
community grammar, as a point of reference, we see
that it is related to an actual feature detector
(possibly containing an embedded logic component)
taht is invoked by the Feature detector Engine
when an appropriate media object is presented for indexing.
The feature grammar and its associated detector
further result in updating respectively the data schemas
and tye actual information stored in the (Monet) database.
The Monet database, which underlies the ACOI framework,
is a customizable, high-performance, main-memory database
developed at the CWI and the University of Amsterdam [_MONET].
<p>
At the user end, a feature grammar is related to
a <em>View</em>, <em>Query</em>
and <em>Report</em> component,
that respectively allow for inspecting a feature grammar,
expressing a query, and delivering a response
to a query.
Some examples of these components are currently implemented as applets
in Java 1.1 with Swing. See [_ACOI].
<p>
:@ Formal specification
Formally, a feature grammar $G$ may be defined
as $G = (V,T,P,S)$, where $V$ is a collection of
variables or non-terminals, 
$T$ a collection of terminals,
$P$ a collection of productions of the form $V -> (V \union T)$
and $S$ a start symbol.
A token sequence $ts$ belongs to the
the language $L(G)$ if $S -*-> ts$.
Sentential token sequences, those belonging to
$L(G)$ or its sublanguages $L(G_v) = (V_v,T_v,P_v,v)$
for $v \e (T \union V)$, correspond to a complex object
$C_v$,, which is the object corresponding to the parse tree 
for $v$, as illustrated in the community example.
The parse tree defines a hierarchical structure
that may be used to access and manipulate the components
of the multimedia object subjected to the detector.

</document>
