<document>

@* The anatomy of a MIDI feature detector
<include file=navigate>

Automatic indexing for musical data
is an inherently difficult problem.
Existing systems rely on hand-crafted solutions,
geared towards a particular group of users,
such as for example composers of film music [_MM].
In this section, we will look at a simple feature detector for
MIDI encoded musical data.
It provides a skeleton for future experimentation.

<slide id=midi-structure type=fig cap="MIDI features">
<image name=local/acoi/grammar>
</slide>

The hierarchical information structure that we consider
is depicted in \sliref{midi-structure}.
It contains only a limited number of basic
properties and must be extended with information
along the lines of the musical ontology proposed
in [_AI].
However, the detector presented here provides
a skeleton solution that accomodates an extensio with
arbitrary predicates over the musical structure in a transparent manner.
<p>
The grammar given below corresponds in an obvious way with
the structure depicted in \sliref{mid-structure}.

<slide type=hide cap="A simple feature grammar for MIDI files">
<display>

detector song; ### to get the filename
detector lyrics; ### extracts lyrics
detector melody; ### extracts melody
detector check;  ### to walk the tree

atom str name;
atom str text;
atom str note;  

midi: song;

song: file lyrics melody check;

file: name;

lyrics: text*;
melody: note*;

</display>
</slide>

The start symbol is a <em>song</em>.
The detector that is associated with <em>song</em>
reads in a midifile.
The musical information contained in the midifile is then stored
a a collection of Prolog facts.
This translation is very direct.
In effect the midifile header information is stored, and
events are recorded as facts, as illustrated below
for a <em>note_on</em> and <em>note_off</em> event.
<slide mode=hide id=midi-event>
<display>
event('kortjakje',2,time=384, note_on:[chan=2,pitch=72,vol=111]).
event('kortjakje',2,time=768, note_off:[chan=2,pitch=72,vol=100]).
</display>
</slide>

After translating the midifile into a Prolog format, the other detectors
will be invoked, that is the <em>composer</em>,
<em>lyrics</em> and <em>melody</em> detector,
to extract the information related to these properties.

<slide id=midi-processing type=fig cap="Processing MIDI file">
<image name=local/acoi/process>
</slide>

The actual processing is depicted in \sliref{midi-processing}.
The input is a midifile.
As indicated in the top line, the midifile itself may
be generated from a <em>lilypond</em> file.
Lilypond is a \LaTeX-like formatting language for
musical scores that also supports the generation of MIDI [_Lily].
As indicated on the bottom line, processing a midifile
results in a collection of features as well as in a midifile
and lilypond file.
The (result) midifile contains an extract
of the original (input) midifile and the lilypond file
contains a score for this extract,
that may be presented to the (end) user as the result of a query.
This setup allows us to verify whether our extract or
abstraction of the origanal musical structure is effective,
simply by comparing the input (midi or lilypond)
musical structure with the output (midi or lilypond) extract. 
<p>
To extract relevant fragments of the melody we
use the melody detector, of which a partial listing is given below.

<slide type=hide cap="The melody detector">
<program>
int melodyDetector(tree *pt, list *tks ){
char buf[1024]; char* _result;
void* q = _query;
int idq = 0; 

  idq = query_eval(q,"X:melody(X)");
  while ((_result = query_result(q,idq)) ) {
         printf("note: %s\n",_result);
         putAtom(tks,"note",_result);
         }
  return SUCCESS;
} 
</program>
</slide>

The embedded logic component is given the
query {P X:melody(X) P}, which results in the notes
that constitute the (relevant fragment of the) melody.
These notes are then added to the tokenstream.
A similar detector is available for the lyrics.
<p>
Parsing a given midifile, for example <em>kortjakje.mid</em>,
results in updating the Monet database as indicated below.

<ignore>
<slide mode=hide id=token-api cap="The tokenstream API">
<display>
given type AtomName, AtomValue;

interface tokenstream {
	void putAtom(in AtomName, in AtomValue);
};
</display>
</slide>
</ignore>

<slide type=hide cap="Update of Monet database">
@ds fd-trace.txt
</slide>

The updates clearly reflect the structure of
the musical information object that corresponds to
the properties defined in the grammar.

:@ Implementation status
Currently, we have a running prototype of the MIDI
feature detector.
It uses an adapted version of public domain
midi processing software.
The embedded logic component is part of the <em>hush</em>
framework. It uses an object extension of Prolog
that allows for the definition of native objects
to interface with the midi processing software.
A description of the logic component is beyond the
scope of this paper, but will be provided in [_OO].
The logic component, however, allows for
the definition of arbitrary predicates to extract 
the musical information, such as the melody and the lyrics.
As stated before,
the current detector may be regarded as
a skeleton implementation that provides
the basis for further experimentation.

</document>
