<document>
<!--
\title{Midi Feature Detection in ACOI\\ combining grammars and logic}
\author{Anton Eliens
and Martin Kersten\\
{\footnotesize \sl CWI,}
{\footnotesize \sl Kruislaan ...}\\ 
{\footnotesize \sl email: \{eliens,mk\}@cwi.nl}
} \date{}\maketitle
\def\thefootnote{}
\setcounter{page}{1}

\date{}\maketitle
\def\thefootnote{}
\setcounter{page}{1}
-->

<control>

<slide id=title mode=hide>
<center>
<h1>
<a href=@talk-paper.html target=_top>Musical feature detection in ACOI</a>
</h1>
<h2>
<i>combining grammars and logic</i>
</h2>
</center>

<center>
Anton Eliens &amp;
Martin Kersten
<br>
CWI
email: eliens@cs.vu.nl, M.Kersten@cwi.nl
</center>
</slide>
</control>


<latex><document>
<abstract>
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


The indexing and retrieval problem for musical data
is basically unsolved.
Existing systems rely on hand-crafted solutions,
geared towards a particular group of users,
such as for example composers of film music.
This paper describes an approach to the detection
of musical features which combines the use of feature
grammars (as developed in the ACOI framework)
to describe the structural properties of musical data,
and logic to extract content-related properties of
that data. In this way we hope to be able to improve the
indexing and retrieval of musical information.
This work is part of the ACOI project, which provides
a large-scale experimentation platform to study the indexing
and retrieval of multimedia objects.
Central in the design of the ACOI architecture is
the <em>feature detector engine</em> model (FDE),
which allows for employing black-box feature
detectors for arbitrary multimedia objects.
Our research indicates how to extend feature detector grammars
with content-related knowledge in a transparent manner.
It moreover provides a concrete implementation of feature
detectors for MIDI encoded material.
</abstract>
</document>
</latex>
<p>
<document>
@* Introduction
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


</document>


<latex>
<document>

@* The ACOI framework
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


<slide type=fig cap="The extended ACOI architecture">
<image name=local/acoi/acoi>
</slide>
See [_ACOI]


</document>
<document>

@* MIDI feature grammar
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>



<slide type=fig cap="MIDI features">
<image name=local/acoi/grammar>
</slide>


@ MIDI feature grammar

<slide cap="A simple feature grammar for MIDI files">
<display>

detector song; ### to get the filename
detector lyrics; ### extracts lyrics
detector melody; ### extracts melody
detector check;  ### to walk the tree

atom str name;
atom str text;
atom str note;  

midi: song;

song: file lyrics melody check;

file: name;

lyrics: text*;
melody: note*;

</display>
</slide>

</document>
<document>

@* A simple MIDI feature detector
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


<slide type=fig cap="Processing MODI file">
<image name=local/acoi/process>
</slide>

@ The melody detector

<slide type=hide cap="The melody detector">
<program>
int melodyDetector(tree *pt, list *tks ){
        char buf[1024]; char* _result;
        void* q = _query;
        int idq = 0; /// imax is a hack, why??
        int i = 0; int imax = 12; // 36;

        idq = query_eval(q,"X:melody(X)");
        while ((_result = query_result(q,idq)) ) {
                   if ( i++ < imax) {
                          printf("note: %s\n",_result);
                          putAtom(tks,"note",_result);
                          }
                    }
        return SUCCESS;
} 
</program>
</slide>

<slide mode=hide id=token-api cap="The tokenstream API">
<display>
given type AtomName, AtomValue;

interface tokenstream {
	void putAtom(in AtomName, in AtomValue);
};
</display>
</slide>

:@ Update of Monet database

<slide type=hide cap="Update of Monet database">
<program>
V1 := newoid();
  midi_song.insert(oid(V0),oid(V1));
V2 := newoid();
    song_file.insert(oid(V1),oid(V2));
      file_name.insert(oid(V2),"kortjakje");
    song_lyrics.insert(oid(V1),oid(V2));
      lyrics_text.insert(oid(V2),"e");
      lyrics_text.insert(oid(V2),"per-");
      lyrics_text.insert(oid(V2),"sonne");
      lyrics_text.insert(oid(V2),"Moi");
      lyrics_text.insert(oid(V2),"je");
      lyrics_text.insert(oid(V2),"dis");
      lyrics_text.insert(oid(V2),"que");
      lyrics_text.insert(oid(V2),"les");
      lyrics_text.insert(oid(V2),"bon-");
      lyrics_text.insert(oid(V2),"bons");
      lyrics_text.insert(oid(V2),"Val-");
      lyrics_text.insert(oid(V2),"ent");
    song_melody.insert(oid(V1),oid(V2));
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"c-2");

</program>

</slide>

</document>
<document>

@* Queries -- the user interface
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


<slide type=fig cap="User Query Processing">
<image name=local/acoi/query>
</slide>

<slide mode=hide cap="Query interface">
<ul>
<li> fragment of a melody
<li> fragment of the lyrics
<li> other information
</ul>
</slide>

<slide type=fig cap="Keyboard interface">
<image name=local/jamming/keyboard>
</slide>

See [_Jamming]

</document>
<document>
@* Conclusions
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 
<a href=grammars.html>grammars</a> 
<a href=detector.html>midi</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


</document>

</latex>

</document>
