@* Native C++ Objects in Java

<slide id=java cap="Native Objects in Java">
@ Native Objects in Java
<ul>
<li> representation if C++ peer: self pointer
<li> object/method function call: native functions  
<li> cast self pointer to object: smart vm
</ul>
</slide>

<slide id=java-self cap="Peer Object Pointer">
@ Peer Object Pointer
<display>

package hush.dv.api;

class obscure {

public int _self;

...
};


public class handler extends obscure {

public static kit tk;
protected Event _event;


public Event dispatch(event ev) {
	_event = (Event) ev;
	operator();
	return _event;
	}

public int operator() {
	...
	return OK;
	}
};
</display>
@ Example
<display>
package hush.dv.api;

public class kit extends handler {

public kit() { _self = init(); }
protected kit(int x) { }
private native int init();

public native void trace();

public native int load(String cmd);

public native void source(String cmd);

public native void eval(String cmd);
public native String evaluate(String cmd);

public String result() { 
	String _result = getresult();
	if (_result.equals("-")) return null;
	else return _result;
	}
public void result(String res) { setresult(res); }

private native String getresult();
private native void setresult(String res);

public void assert(String cl) { assertz(cl); }
public void asserta(String cl) { nativeassert(cl,"a"); }
public void assertz(String cl) { nativeassert(cl,"z"); }

public void retract(String cl) { nativeretract(cl,""); }
public void retractall(String cl) { nativeretract(cl,"all"); }

private native void nativeassert(String res, String opts);
private native void nativeretract(String res, String opts);

public native void bind(String cmd, handler h);

public native void wait(int msecs);
public native void after(int msecs, handler h);

public native void update();

public Widget root() { return new Widget(rootid()); }
public native void pack(String s);

public native void quit();

private native int rootid();
};
</display.
</slide>

<slide id=java-native cap="Native Functions">
@ Native Functions
<display>
#include <hush/java.h>
#include <hush/kit.h>
#include <hush/session.h>


#include <native/hush_dv_api_kit.h>

#define method(X) Java_hush_dv_api_kit_##X

JNIEXPORT jint JNICALL method(init)(JNIEnv *env, jobject obj)
{
  if (env && obj); // Wall
  jint result = (jint) kit::_default; // (jint) new kit();
  if (!result) {
	kit* x = new kit("tk");
	session::_default->_register(x);
	result = (jint) x;
	}
  return result;
}

JNIEXPORT void JNICALL method(eval)(JNIEnv *env, jobject obj, jstring s)
{
  java_vm<kit> vm(env,obj);
  const char *str = vm.get(s);
  vm->eval(str);
  vm.release(s);
}

JNIEXPORT jstring JNICALL method(getresult)(JNIEnv *env, jobject obj)
{
  java_vm<kit> vm(env,obj);
  char *s = vm->result();
  if (s) return vm.string(s);
  else return vm.string("-");
}

JNIEXPORT void JNICALL method(setresult)(JNIEnv *env, jobject obj, jstring s)
{
  char str[1024];
  java_vm<kit> vm(env,obj);
  char *p = vm.get(s);
  strcpy(str,p);
  vm->result(p);
  //vm.release(s);
}

JNIEXPORT void JNICALL method(bind)(JNIEnv *env, jobject obj,
	       jstring s, jobject o)
{
  java_vm<kit> vm(env,obj);
  java_vm<handler>* vmp = new java_vm<handler>(env,o,"Handler");
  const char *str = vm.get(s);
  handler* h = vmp->self();
  if (h) {
	cerr << "Setting (kit) VM" << endl;
	if (!h->_vmp) h->_vmp = vmp;
  } else {
	cerr << "Creating (kit) VM" << endl;
	h = new handler();
	session::_default->_register(h);
	h->_vmp = vmp;
  }
  h->_register(vmp);
  vm->bind(str,h);
  vm.release(s, str);
}
</display>
</slide>

<slide id=java-vm cap="Java Smart Pointer">
@ Java Smart Pointer
<display>

#include <hush/vm.h> 

#include <jni.h>

template<class T>
class java_vm : public vm<T> {
public:


java_vm(JNIEnv* env_, jobject obj_, char* clazz_ = 0) { 
	_jvm = 0;
	_global = 0;
	_env = env_; 
	_obj = obj_;
	_cls = _env->GetObjectClass(_obj);
	if (clazz_) {
		strcpy(_clazz,clazz_); 
		_global = 1;
		_obj = _env->NewGlobalRef(_obj);
		}
	else _clazz[0]='\0';
	_self = self();
	}

~java_vm() {
	if (_global && _obj)
		env()->DeleteGlobalRef(_obj);
	}

JNIEnv* env() { return _env?_env:((JNIEnv*)vm_type::env); }

JavaVM* jvm() { return _jvm?_jvm:((JavaVM*)vm_type::jvm); }

event* dispatch(event* e) { /// java dispatch
	callIV("dispatch",(int)e);
	return e;
	}

public: /// self

T* operator->() { return _global?self():_self; }

T* self() {
	jfieldID fid = fieldID("_self","I");
	return (T*) env()->GetIntField( object(), fid);
	}

public: /// ID

jfieldID fieldID(const char* fd, const char* sig) {
	if (_global) _cls = env()->GetObjectClass(object());
	return env()->GetFieldID( _jclass(), fd, sig);
	}

jmethodID methodID(const char* md, const char* sig) {
	//cerr << "methodID " << md << " " << sig << endl;
	if (_global) _cls = env()->GetObjectClass(object());
	return env()->GetMethodID( _jclass(), md, sig);
	}

public: /// method call

void callV(const char* md) { // void (*)()
	jmethodID mid = methodID(md,"()V");
	env()->CallVoidMethod(object(), mid);
	}

void callIV(const char* md, int i) { // void (*)(int)
	jmethodID mid = methodID(md,"(I)V");
	env()->CallVoidMethod(object(), mid, i);
	}

int callVI(const char* md) { ///int (*)()
	jmethodID mid = methodID(md,"()I");
	return env()->CallIntMethod(object(), mid);
	}

int call(const char* md) { return callVI(md); }

// convenience functions

jstring string(const char* s) {
	return env()->NewStringUTF(s);
	}

char* get(jstring js) {
	return hstr = (char*) env()->GetStringUTFChars(js, 0);
	}

void release(jstring js, const char* str = 0) {
	char* p = str?(char*)str:hstr;
	env()->ReleaseStringUTFChars(js, p);
	}

private: //@ ...
char* hstr; // to hold a string

int _global;
char _clazz[128];
JNIEnv* _env;
jobject _obj;
jclass _cls;
JavaVM* _jvm;
T* _self;
};
</display>
</slide>
