%%@ native.pl -- binding objects to handlers

:- use(library(objects)).

native(H,X) :- unwrap(X,C), hush(H,C).
native(H,X,R) :-
	%%cerr(native(H,X,R)),
	unwrap(X,C),
	%%cerr(native(H,C,R)),
	hush(H,C,R).


obsolete_native(H,M,A,R) :-
	optify(1,A,Args),
	native(H,[method=M|Args],R).

native(H,ID,Q,R) :-
	atom(ID),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	object(H,Class,ID,REF,_,_,_),
	native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,REF,Q,R) :-
	atom(REF),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	object(H,Class,ID,REF,_,_,_),
	native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,X,Q,R) :-
	var(X),
	not var(H),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[method=G],R).

native(H,X,Q,R) :-
	not var(H),
	X = [Class,REF|_],
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[class=Class, ref=REF, method=G],R).

native(H,X,Q,R) :-
	not var(H),
	atom(X),
	not object(H,_,X,_,_,_,_),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[objectid=X, method=G],R).



unwrap(method=G,S) :-
	G =.. [F|Args],
	not (Args = []),!,
	%%cerr(method=G),
	optify(1,Args,L),
	unwrap([method=F,L],S).

unwrap(X=Y,X=Y):- !. %% :- not (X=method). %% :- unwrap(Y,Z).
unwrap(ID:G,ID:C) :- atomic(ID), not atomic(G), unwrap(G,C).
unwrap([H|T],S) :- list_stringify([H|T],S).
unwrap(G,C) :- G =.. [F|Args],
	       not Args = [],
	       not member(F,['=','.']), !,
	       argify([F|Args],C).
unwrap(X,X) :- not X = (_=_), not X = [_|_], not X = []. 

optify(_,[],[]).
optify(N,[H|T],[N=X|R]) :-
	term_to_atom(H,A),
	list_concat(['"',A,'"'],X),
	%%cerr(optify(N=X)),
	N1 is N + 1,
        optify(N1,T,R).

stringify([],'').
stringify([H|T],R) :-
        stringify(T,C),
	term_to_atom(H,A),
        list_concat([A,' ',C],R).

argify([H],H).
argify([H|T],H:R) :-
        argify(T,R).

list_stringify([],'').
list_stringify([H|T],R) :-
        list_stringify(T,C),
        head_stringify(H,C,R).

head_stringify(H,L,R) :-
	unwrap(H,T),
	term_to_atom(T,A),
	list_concat([A,' ',L],R).
