@* Native Objects in Prolog

<slide id=native cap="Native Objects in Prolog">
@ Native Objects in Prolog
<ul>
<li> method call - check ID is a REF 
<li> declaration - call (native) constructor
<li> mapping creation and method calls -- through native support
<li> native support -- binding: object_handler and query_event
<li> native support -- classes: the usual (hush) hierarchy 
e.g. <a href=obscure.pl.html>obscure</a>
and <a href=kit.pl.html>kit</a>
</ul>
</slide>

<slide id=native-declare cap="Object declarations">
@ Object declarations
<display>
declare(H,new(O),R) :- declare(H,new(O),[],[],[],R).
declare(H,new(O),I,R) :- declare(H,new(O),I,[],[],R).
declare(H,new(O),I,P,R) :- declare(H,new(O),I,P,[],R).
declare(H,new(O),I,P,Q,R) :-
        O =.. [F,A|Args],
        objectname(F,A,N),
        ancestors(F,I,Anc),
        attributes(F,P,Att),
        qualifiers(F,Q,Qual),
        (not var(H), var(R) ->
                native(H,N,F:Args,R);R=R),   /// possibly native
        assert(object(H,F,N,R,Anc,Att,Qual)),
        cerr(asserted((object(H,F,N,R,Anc,Att,Qual)))).
</display>
</slide>

<slide id=native-support cap="Native support">
@ Native support
<display>
native(H,X) :- /// hush provides principal handler 
	unwrap(X,C), 
	hush(H,C). // H is secondary handler
native(H,X,R) :-   /// with result R
        unwrap(X,C),
        hush(H,C,R). // H is secondary handler


native(H,ID,Q,R) :- /// we know the ID
        atom(ID),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        object(H,Class,ID,REF,_,_,_),
        native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,REF,Q,R) :- /// we know the REF
        atom(REF),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        object(H,Class,ID,REF,_,_,_),
        native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,X,Q,R) :- /// we know the handler
        var(X),
        not var(H),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[method=G],R).

native(H,X,Q,R) :- /// there is an unidentified REF
        not var(H),
        X = [Class,REF|_],
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[class=Class, ref=REF, method=G],R).

native(H,X,Q,R) :- /// we know the handler and the ID
        not var(H),
        atom(X),
        not object(H,_,X,_,_,_,_),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[objectid=X, method=G],R).
</display>
</slide>

<slide id=native-unwrap cap="Argument unwrapping">
@ Argument unwrapping -- key-value pairs
<display>
unwrap(method=G,S) :- /// method=F 1=Arg1 2=Arg2 ...
        G =.. [F|Args],
        not (Args = []),!,
        optify(1,Args,L),
        unwrap([method=F,L],S).

unwrap(X=Y,X=Y):- !. 
unwrap(ID:G,ID:C) :- atomic(ID), not atomic(G), unwrap(G,C).
unwrap([H|T],S) :- list_stringify([H|T],S).

unwrap(X,X) :- not X = (_=_), not X = [_|_], not X = [].

optify(_,[],[]).       
optify(N,[H|T],[N=X|R]) :-   /// 1=Arg1 2=Arg2 ...
        term_to_atom(H,A),
        list_concat(['"',A,'"'],X),
        N1 is N + 1,
        optify(N1,T,R).

list_stringify([],''). /// create a space delimited string
list_stringify([H|T],R) :-
        list_stringify(T,C),
        head_stringify(H,C,R).

head_stringify(H,L,R) :-
        unwrap(H,T),
        term_to_atom(T,A),
        list_concat([A,' ',L],R).

</display>
</slide>
