<slide id=title caption="Object+ Talk">
<center>
<p>
Native Objects in Prolog
<p>
Anton Eliens
<p>
VU/CWI, Amsterdam
<p>
2/11/98
</center>
<hr>
<p>
http://www.cs.vu.nl/~eliens/online/experiments/native
</slide>

@* Introduction

<slide color=yellow id=intro cap="Objects in Prolog">
@ Objects in Prolog
<ul>
<li> Many flavors: Sicstus, ...
<li> Modules do not suffice - ...
<li> Requirements: low overhead, natural syntax
<li> Additional requirements:
<b>binding with native objects</b>
</ul>
</slide>

<slide color=yellow id=intro-native cap="Native Objects">
@ Native Objects
<ul>
<li> native bindings are mostly for functions only
<li> Java, however, provides an example
<li> Let's first look at <i>objects in Prolog</i>,
and then their possibly native realization
</ul>
</slide>

@* Objects in Prolog

<slide id=objects cap="Objects in Prolog">
@ Objects in Prolog
<ul>
<li> representation -- object(Handler,Class,ID,REF,Anc,Attr,Qual)
<li> object creation -- class_method(This,...)
<li> object invocation -- self(This):method(...)
<li> state variables representation -- value(ID,Key,Value)
<li> state variables access -- assign(key,Value), value(Key,Value)
<li> operator overloading -- var(Key) = Value, Var = value(key)
<li> operator overloading -- =, &lt;, &lt;=, &gt;, &gt;= 
<li> native binding - native(Handler,Method,Result)
</ul>
<p> Let's give an example first
</slide>

<slide id=object-example cap="Objects - example">
@ Objects - <a href=try.pl.html>examples</a>
<display>
run(4):-
        cerr(start(4)),
        midi(This):midi,  /// create midi object
        Self = midi(This),
        Self:open('a.mid'),
        Self:header(0,1,480),
        Self:track(start),
        Self:melody([48,50,51,53,55]), // c d es f g, minor indeed
        Self:track(end), /// end track
        cerr(end(4)).
</display>
<p>
The midi object, naturally has a native realization
that complements its prolog definition
</slide>

<slide id=object-midi cap="The midi object realization">
@ The midi <a href=midi.pl.html>Prolog object realization</a>
<display>

// from midi.pl

:- use(library(midi:[midi,lily,music,process])).

:- declare(midi:object,class(midi),[handler],[]). /// again, almost everything is a handler


midi_midi(This) :- /// constructor
        midi(This):handler(H),   %% gets Handler from class
        declare(H,new(midi(This)),[],[],_).

midi_read(This,F) :- native(_,This,read(F),_). /// natice methods
midi_analyse(This,I,O) :- native(_,This,analyse(I,O),_).
midi_open(This,F) :- native(_,This,open(F),_).
midi_header(This,M) :- native(_,This,header(M,0,480),_).
midi_track(This,X) :- native(_,This,track(X),_).
midi_tempo(This,X) :- native(_,This,tempo(X),_).
midi_event(This,D,C,M,T,V) :- native(_,This,event(D,C,M,T,V),_).


midi_note(This,D,C,T,V) :- /// other methods
        Self = midi(This),
        Self:event(D,C,note_on,T,V),
        Self:event(D,C,note_off,T,V).

midi_melody(This,L) :- midi(This):melody(480,1,L,64).

midi_melody(_This,_,_,[],_).

midi_melody(This,D,C,[X|R],V) :-
        Self = self(This),
        Self:note(D,C,X,V),  %% possibly virtual
        midi_melody(This,D,C,R,V).   %% direct invocation, not virtual
</display>
</slide>


<slide id=objects-realization cap="Object realization in Prolog">
@ <a href=objects.pl.html>Object realization in Prolog</a> -- method call
<display>

method(self(ID),M,C) :- /// get real self
        atom(ID),
        (object(_,Class,ID,_,_,_,_);
         object(_,Class,_,ID,_,_,_)),!,   /// ID is a REF !
        O =.. [Class,ID],
        method(O,M,C).

method(O,M,C):-  /// yes, it is an object
        object(O),
        O =.. [Class,N],
        method_translate(M,Fun),
        method(Class,N,Fun,C).

method(O,M,C) :-  /// there is no object, but 
        not object(O),
        O =.. [F,ID|_],
        method_translate(M,Fun),
        method(F,ID,Fun,C).

method(Class,ID,M,C):-  /// try Class_Method(ID,...
        M =.. [Method|Args],
        list_concat([Class,'_',Method],Predicate),
        (Args = [] -> C =.. [Predicate,ID];
        append([Predicate,ID],Args,L), C =.. L),
        (not clause(C,_) -> fail; true,! ).

method(Class,ID,M,C):-  /// look for Ancestor, go up
        (object(_,Class,ID,_,Anc,_,_);
         object(_,Class,nil,_,Anc,_,_)),!,
        member(Type,Anc),
        method(Type,ID,M,C).

</display>
</slide>


<slide id=objects-eval cap="Eval and its abbreviation">
@ <a href=objects.pl.html>Eval and its abbreviation</a>
<display>
O:G :-
        eval(O:G).

eval(_:[]) :- !.
        eval(O:H),
        eval(O:T).

eval(O:G) :-
        not G = [],
        not G = [_|_],
        method(O,G,B),
        call(B).
</display>
</slide>

@* Native Objects in Prolog

<slide id=native cap="Native Objects in Prolog">
@ Native Objects in Prolog
<ul>
<li> method call - check ID is a REF 
<li> declaration - call (native) constructor
<li> mapping creation and method calls -- through native support
<li> native support -- binding: object_handler and query_event
<li> native support -- classes: the usual (hush) hierarchy 
e.g. <a href=obscure.pl.html>obscure</a>
and <a href=kit.pl.html>kit</a>
</ul>
</slide>

<slide id=native-declare cap="Object declarations">
@ Object declarations
<display>
declare(H,new(O),R) :- declare(H,new(O),[],[],[],R).
declare(H,new(O),I,R) :- declare(H,new(O),I,[],[],R).
declare(H,new(O),I,P,R) :- declare(H,new(O),I,P,[],R).
declare(H,new(O),I,P,Q,R) :-
        O =.. [F,A|Args],
        objectname(F,A,N),
        ancestors(F,I,Anc),
        attributes(F,P,Att),
        qualifiers(F,Q,Qual),
        (not var(H), var(R) ->
                native(H,N,F:Args,R);R=R),   /// possibly native
        assert(object(H,F,N,R,Anc,Att,Qual)),
        cerr(asserted((object(H,F,N,R,Anc,Att,Qual)))).
</display>
</slide>

<slide id=native-support cap="Native support">
@ Native support
<display>
native(H,X) :- /// hush provides principal handler 
	unwrap(X,C), 
	hush(H,C). // H is secondary handler
native(H,X,R) :-   /// with result R
        unwrap(X,C),
        hush(H,C,R). // H is secondary handler


native(H,ID,Q,R) :- /// we know the ID
        atom(ID),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        object(H,Class,ID,REF,_,_,_),
        native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,REF,Q,R) :- /// we know the REF
        atom(REF),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        object(H,Class,ID,REF,_,_,_),
        native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,X,Q,R) :- /// we know the handler
        var(X),
        not var(H),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[method=G],R).

native(H,X,Q,R) :- /// there is an unidentified REF
        not var(H),
        X = [Class,REF|_],
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[class=Class, ref=REF, method=G],R).

native(H,X,Q,R) :- /// we know the handler and the ID
        not var(H),
        atom(X),
        not object(H,_,X,_,_,_,_),
        (Q = (C:A) -> G =.. [C|A]; G = Q),
        native(H,[objectid=X, method=G],R).
</display>
</slide>

<slide id=native-unwrap cap="Argument unwrapping">
@ Argument unwrapping -- key-value pairs
<display>
unwrap(method=G,S) :- /// method=F 1=Arg1 2=Arg2 ...
        G =.. [F|Args],
        not (Args = []),!,
        optify(1,Args,L),
        unwrap([method=F,L],S).

unwrap(X=Y,X=Y):- !. 
unwrap(ID:G,ID:C) :- atomic(ID), not atomic(G), unwrap(G,C).
unwrap([H|T],S) :- list_stringify([H|T],S).

unwrap(X,X) :- not X = (_=_), not X = [_|_], not X = [].

optify(_,[],[]).       
optify(N,[H|T],[N=X|R]) :-   /// 1=Arg1 2=Arg2 ...
        term_to_atom(H,A),
        list_concat(['"',A,'"'],X),
        N1 is N + 1,
        optify(N1,T,R).

list_stringify([],''). /// create a space delimited string
list_stringify([H|T],R) :-
        list_stringify(T,C),
        head_stringify(H,C,R).

head_stringify(H,L,R) :-
        unwrap(H,T),
        term_to_atom(T,A),
        list_concat([A,' ',L],R).

</display>
</slide>

@* C++ bindings

<slide id=c cap="C++ bindings">
@ C++ bindings -- <a href=@online/hush/sources/dv/objects/contents.html>objects</a>
<ul>
<li> generic binding for (pl) kit - with secondary handler dictionary
<li> handler hierarchy -- obscure, handler, event, kit, query
<li> downcall - via smart pointer  vm<type>
<li> upcall   - via embedded vm in handler
</ul>
</slide>

<slide id=support-query cap="A query handler">>
<display>
class query_object : public kit_object {
public:
query_object(kit* q, char* n = "query:object") : kit_object(q,n) { }
object_handler* _create(query* q, char* n = "query:object") {
        return new query_object(q,n);
        }
int operator()();
};
</display>
@ The kit implementation
<display>


int kit_object::operator()() {
        event* e = _event;

        vm<kit> self(e);  /// smart pointer
        string method = e->_method();

        if (method == "kit") { /// constructor
                kit* q = new kit(e->arg(1));
                _register(q);
                result( reference((void*)q) );
        } else if (method == "eval") {
                long res = self->eval(e->arg(1));
                result( itoa(res) );
        } else if (method == "result") {
                char* res = self->result( atoi(e->arg(1)) );
                result(res);
        } else if (method == "evaluate") {
                char* res = self->evaluate(e->arg(1));
                result( res );
        } else { /// dispatch up in the hierarchy
                return handler_object::operator()();
        }

        return 0;
        }


</display>
</slide>


<slide id=support cap="Up calls through VM">
@ Up calls through <a href=@online/hush/sources/hush/vm.h.html>VM</a>
<display>
// in <a href=@online/hush/sources/hush/api/handler.c.html>handler.c</a>

event* handler::dispatch(event* e) {
_event = e;
if (_vmp) {
        return ((vm<handler>*)_vmp)->dispatch(e);
} else {

         if (e->thekit()) tk = e->thekit(); // AE

         int result = this->operator()();

         if (result != OK) return 0; // jrvosse
         else return _event;
         }
}
</display>
</slide>

<slide id=support-vm cap="The native object VM">
@ The native object VM
<display>
template < class T >
class vm : public vm_type {
public:

vm(event* e) {
        int p = 0;
        char* id = e->option("ref");
        if (id) {
                p = atoi(id);
                }
        kit* k = e?e->thekit():0;
        _pointer = p;
        _self = (T*) _pointer;
        }

event* dispatch(event* e) {  /// upcall - refined in java_vm
        char buf[512];
        sprintf(buf,":handler(%d):dispatch(event(%d))",
                        (int)_pointer,(int)e);
        eval(buf);
        return e;
        }

virtual int call(const char* cmd) { /// upcall - refined in java_vm
        char buf[512]; int n = 0;
        sprintf(buf,"some(X):event(%d):%s(X)",(int)_pointer,cmd);
        char* res = evaluate(buf);
        sscanf(res,"%d",&n);
        return n;
        }

virtual inline T* operator->() { return self(); }
virtual inline T* self() { return _self; }

private:
T* _self;
long _pointer;
kit* _kit;
};


</display>
</slide>

@* Native C++ Objects in Java

<slide id=java cap="Native Objects in Java">
@ Native Objects in Java
<ul>
<li> representation if C++ peer: self pointer
<li> object/method function call: native functions  
<li> cast self pointer to object: smart vm
</ul>
</slide>

<slide id=java-self cap="Peer Object Pointer">
@ Peer Object Pointer
<display>

package hush.dv.api;

class obscure {

public int _self;

...
};


public class handler extends obscure {

public static kit tk;
protected Event _event;


public Event dispatch(event ev) {
	_event = (Event) ev;
	operator();
	return _event;
	}

public int operator() {
	...
	return OK;
	}
};
</display>
@ Example
<display>
package hush.dv.api;

public class kit extends handler {

public kit() { _self = init(); }
protected kit(int x) { }
private native int init();

public native void trace();

public native int load(String cmd);

public native void source(String cmd);

public native void eval(String cmd);
public native String evaluate(String cmd);

public String result() { 
	String _result = getresult();
	if (_result.equals("-")) return null;
	else return _result;
	}
public void result(String res) { setresult(res); }

private native String getresult();
private native void setresult(String res);

public void assert(String cl) { assertz(cl); }
public void asserta(String cl) { nativeassert(cl,"a"); }
public void assertz(String cl) { nativeassert(cl,"z"); }

public void retract(String cl) { nativeretract(cl,""); }
public void retractall(String cl) { nativeretract(cl,"all"); }

private native void nativeassert(String res, String opts);
private native void nativeretract(String res, String opts);

public native void bind(String cmd, handler h);

public native void wait(int msecs);
public native void after(int msecs, handler h);

public native void update();

public Widget root() { return new Widget(rootid()); }
public native void pack(String s);

public native void quit();

private native int rootid();
};
</display.
</slide>

<slide id=java-native cap="Native Functions">
@ Native Functions
<display>
#include <hush/java.h>
#include <hush/kit.h>
#include <hush/session.h>


#include <native/hush_dv_api_kit.h>

#define method(X) Java_hush_dv_api_kit_##X

JNIEXPORT jint JNICALL method(init)(JNIEnv *env, jobject obj)
{
  if (env && obj); // Wall
  jint result = (jint) kit::_default; // (jint) new kit();
  if (!result) {
	kit* x = new kit("tk");
	session::_default->_register(x);
	result = (jint) x;
	}
  return result;
}

JNIEXPORT void JNICALL method(eval)(JNIEnv *env, jobject obj, jstring s)
{
  java_vm<kit> vm(env,obj);
  const char *str = vm.get(s);
  vm->eval(str);
  vm.release(s);
}

JNIEXPORT jstring JNICALL method(getresult)(JNIEnv *env, jobject obj)
{
  java_vm<kit> vm(env,obj);
  char *s = vm->result();
  if (s) return vm.string(s);
  else return vm.string("-");
}

JNIEXPORT void JNICALL method(setresult)(JNIEnv *env, jobject obj, jstring s)
{
  char str[1024];
  java_vm<kit> vm(env,obj);
  char *p = vm.get(s);
  strcpy(str,p);
  vm->result(p);
  //vm.release(s);
}

JNIEXPORT void JNICALL method(bind)(JNIEnv *env, jobject obj,
	       jstring s, jobject o)
{
  java_vm<kit> vm(env,obj);
  java_vm<handler>* vmp = new java_vm<handler>(env,o,"Handler");
  const char *str = vm.get(s);
  handler* h = vmp->self();
  if (h) {
	cerr << "Setting (kit) VM" << endl;
	if (!h->_vmp) h->_vmp = vmp;
  } else {
	cerr << "Creating (kit) VM" << endl;
	h = new handler();
	session::_default->_register(h);
	h->_vmp = vmp;
  }
  h->_register(vmp);
  vm->bind(str,h);
  vm.release(s, str);
}
</display>
</slide>

<slide id=java-vm cap="Java Smart Pointer">
@ Java Smart Pointer
<display>

#include <hush/vm.h> 

#include <jni.h>

template<class T>
class java_vm : public vm<T> {
public:


java_vm(JNIEnv* env_, jobject obj_, char* clazz_ = 0) { 
	_jvm = 0;
	_global = 0;
	_env = env_; 
	_obj = obj_;
	_cls = _env->GetObjectClass(_obj);
	if (clazz_) {
		strcpy(_clazz,clazz_); 
		_global = 1;
		_obj = _env->NewGlobalRef(_obj);
		}
	else _clazz[0]='\0';
	_self = self();
	}

~java_vm() {
	if (_global && _obj)
		env()->DeleteGlobalRef(_obj);
	}

JNIEnv* env() { return _env?_env:((JNIEnv*)vm_type::env); }

JavaVM* jvm() { return _jvm?_jvm:((JavaVM*)vm_type::jvm); }

event* dispatch(event* e) { /// java dispatch
	callIV("dispatch",(int)e);
	return e;
	}

public: /// self

T* operator->() { return _global?self():_self; }

T* self() {
	jfieldID fid = fieldID("_self","I");
	return (T*) env()->GetIntField( object(), fid);
	}

public: /// ID

jfieldID fieldID(const char* fd, const char* sig) {
	if (_global) _cls = env()->GetObjectClass(object());
	return env()->GetFieldID( _jclass(), fd, sig);
	}

jmethodID methodID(const char* md, const char* sig) {
	//cerr << "methodID " << md << " " << sig << endl;
	if (_global) _cls = env()->GetObjectClass(object());
	return env()->GetMethodID( _jclass(), md, sig);
	}

public: /// method call

void callV(const char* md) { // void (*)()
	jmethodID mid = methodID(md,"()V");
	env()->CallVoidMethod(object(), mid);
	}

void callIV(const char* md, int i) { // void (*)(int)
	jmethodID mid = methodID(md,"(I)V");
	env()->CallVoidMethod(object(), mid, i);
	}

int callVI(const char* md) { ///int (*)()
	jmethodID mid = methodID(md,"()I");
	return env()->CallIntMethod(object(), mid);
	}

int call(const char* md) { return callVI(md); }

// convenience functions

jstring string(const char* s) {
	return env()->NewStringUTF(s);
	}

char* get(jstring js) {
	return hstr = (char*) env()->GetStringUTFChars(js, 0);
	}

void release(jstring js, const char* str = 0) {
	char* p = str?(char*)str:hstr;
	env()->ReleaseStringUTFChars(js, p);
	}

private: //@ ...
char* hstr; // to hold a string

int _global;
char _clazz[128];
JNIEnv* _env;
jobject _obj;
jclass _cls;
JavaVM* _jvm;
T* _self;
};
</display>
</slide>

@* Conclusions


<slide id=con cap="Conclusions">
@ Conclusions
<ul>
<li> See some more examples
<a href=pointer.c.html>pointer.c</a>
and <a href=pointer.pl.html>pointer.pl</a>.
<li> Objects - in a mix of styles
<li> With native support - similar to hush-java support
<li> key-value transfer of native calls
<li> virtuality, operator overloading, nested calls, native support
<li> now we need concurrency ...
<li> remember query: we do have distribution already
</ul>
</slide>

@* Appendix


@* Objects
<program>


:- use(library(native)).

:- dynamic object/7, value/3.
:- multifile declare/3, declare/4, declare/5, declare/6.

object(nil,nil,nil,0,[],[],_). %% the generic object
value(nil,nil,nil).



new(O) :- %%% Class_new
        O =.. [F|A],
        list_concat([F,'_new'],M),
        G =.. [M|A],
        %%cerr(trying(G)),
        clause(G,_) -> call(G),!.

new(O) :- %%% Class_Class
        O =.. [F|A],
        list_concat([F,'_',F],M),
        G =.. [M|A],
        %%cerr(trying(G)),
        clause(G,_) -> call(G),!.

new(O) :- %%% default 
        O =.. [F|_],
	C =.. [F,_],
	eval(C:handler(H)),
	%%cerr(default(new(O))),
	declare(H,new(O),[handler],_),!.

delete(O) :- %%% optional: Class_delete
        O =.. [F,A],
	(eval(O:delete);true),
	((atom(A) -> retractall(value(A,_,_)));true),
	((atom(A) -> retract(object(_,_,A,_,_,_,_)));true),
	cerr(deleted(O)),
	!.



object(O) :- object(O,_).
object(O,N) :-
	%%cerr(asking(object(O,N))),
	O =.. [F,A],
	object(_,F,A,N,_,_,_),
	not A = nil,
	not N = 0.
	%%cerr(trying(object(_,F,A,N,_,_,_))).
	
object(O,N,Anc) :- O =.. [F,A], object(_,F,A,N,Anc,_,_).

objectname(C,A,N) :- var(A),!, gensym(C,N),A=N.
objectname(_,N,N).



ancestors(F,P,R) :- 
	ancestors(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

ancestors(F,R):- not var(F), object(_,F,nil,0,R,_,_),!.
ancestors(_,[]).



attributes(F,P,R) :- 
	attributes(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

attributes(F,R):- not var(F), object(_,F,nil,0,_,R,_),!.
attributes(_,[]).



qualifiers(F,P,R) :- 
	qualifiers(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

qualifiers(F,R):- not var(F), object(_,F,nil,0,_,_,R),!.
qualifiers(_,[]).



declare(H,class(C)) :- declare(H,class(C),[obscure],[],[]).
declare(H,class(C),A) :- declare(H,class(C),A,[],[]).
declare(H,class(C),A,P) :- declare(H,class(C),A,P,[]).
declare(H,class(C),A,P,Q) :-
	assert(object(H,C,nil,0,A,P,Q)).





declare(H,new(O),R) :- declare(H,new(O),[],[],[],R).
declare(H,new(O),I,R) :- declare(H,new(O),I,[],[],R).
declare(H,new(O),I,P,R) :- declare(H,new(O),I,P,[],R).
declare(H,new(O),I,P,Q,R) :-
	O =.. [F,A|Args],
	%%cerr(declare(H,new(O),I,P,R)),
	objectname(F,A,N),
	ancestors(F,I,Anc),
	attributes(F,P,Att),
	qualifiers(F,Q,Qual),
	%%cerr(declare(H,new(O),I,P,Q,R)),
	(not var(H), var(R) ->
		native(H,N,F:Args,R);R=R),
	assert(object(H,F,N,R,Anc,Att,Qual)),
	cerr(asserted((object(H,F,N,R,Anc,Att,Qual)))).




method_translate(var(K)=V,assign(K,V)) :- not var(K),!.
method_translate(value(K)=V,value(K,V)) :- not var(K),!.
method_translate(V=value(K),value(K,V)) :- not var(K),!.

method_translate(X=Y,eq(X,Y)) :- !.
method_translate(X<Y,lt(X,Y)) :- !.
method_translate(X>Y,gt(X,Y)) :- !.
method_translate(X<=Y,leq(X,Y)) :- !.
method_translate(X>=Y,geq(X,Y)) :- !.

method_translate(X:Y,eval(X:Y)) :- !.

method_translate(X,X) :- !.



method(self(ID),M,C) :-
	atom(ID),
	(object(_,Class,ID,_,_,_,_);
	 object(_,Class,_,ID,_,_,_)),!,
	O =.. [Class,ID],
	method(O,M,C).

method(O,M,C):-
	object(O),
	O =.. [Class,N],
	method_translate(M,Fun),
	method(Class,N,Fun,C).

method(O,M,C) :-  %%% anonymous
	not object(O),
	O =.. [F,ID|_],
	%%cerr(anonymous(method(O,M,C))),
	method_translate(M,Fun),
	method(F,ID,Fun,C).

method(Class,ID,M,C):- %%% no default handler
	var(ID),
	M = handler(H),
	(object(H,Class,nil,_,_,_,_);true),
	!,
	C = true.


method(_,ID,M,C):- %%% value(K,V)
	M = value(K,V),
	((atom(ID),object(_,_,ID,_,_,_,_)) -> value(ID,K,V); V = nil), 
	!,
	C = true.

method(_,ID,M,C):- %%% assign(K,V)
	M = assign(K,V),
	((atom(ID),value(ID,K,_)) -> retract(value(ID,K,_)); true),
	(atom(ID) -> assert(value(ID,K,V)); true),
	!,
	C = true.
	
method(Class,ID,M,C):-
	M =.. [Method|Args],
	list_concat([Class,'_',Method],Predicate),
	(Args = [] -> C =.. [Predicate,ID];
	append([Predicate,ID],Args,L), C =.. L),
	%%cerr(trying:C),
	(not clause(C,_) -> fail; true,! ).

method(Class,ID,M,C):-
	%%cerr(method(Type,ID,M,C)),
	(object(_,Class,ID,_,Anc,_,_);
	 object(_,Class,nil,_,Anc,_,_)),!,
	member(Type,Anc),
	method(Type,ID,M,C).
	


O:G :-
	%%cerr(O:G),
	eval(O:G).

eval(_:[]) :- !.
eval(O:[H|T]) :-
	eval(O:H),
	eval(O:T).

eval(O:G) :-
	not G = [],
	not G = [_|_],
	%%cerr(eval(O:G)),
	method(O,G,B),
	%%cerr(calling(O:G)),
	call(B).

</program>

@* Native
<program>


:- use(library(objects)).

native(H,X) :- unwrap(X,C), hush(H,C).
native(H,X,R) :-
	%%cerr(native(H,X,R)),
	unwrap(X,C),
	%%cerr(native(H,C,R)),
	hush(H,C,R).


obsolete_native(H,M,A,R) :-
	optify(1,A,Args),
	native(H,[method=M|Args],R).

native(H,ID,Q,R) :-
	atom(ID),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	object(H,Class,ID,REF,_,_,_),
	native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,REF,Q,R) :-
	atom(REF),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	object(H,Class,ID,REF,_,_,_),
	native(H,[class=Class,objectid=ID,ref=REF,method=G],R).

native(H,X,Q,R) :-
	var(X),
	not var(H),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[method=G],R).

native(H,X,Q,R) :-
	not var(H),
	X = [Class,REF|_],
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[class=Class, ref=REF, method=G],R).

native(H,X,Q,R) :-
	not var(H),
	atom(X),
	not object(H,_,X,_,_,_,_),
	(Q = (C:A) -> G =.. [C|A]; G = Q),
	native(H,[objectid=X, method=G],R).



unwrap(method=G,S) :-
	G =.. [F|Args],
	not (Args = []),!,
	%%cerr(method=G),
	optify(1,Args,L),
	unwrap([method=F,L],S).

unwrap(X=Y,X=Y):- !. %% :- not (X=method). %% :- unwrap(Y,Z).
unwrap(ID:G,ID:C) :- atomic(ID), not atomic(G), unwrap(G,C).
unwrap([H|T],S) :- list_stringify([H|T],S).
unwrap(G,C) :- G =.. [F|Args],
	       not Args = [],
	       not member(F,['=','.']), !,
	       argify([F|Args],C).
unwrap(X,X) :- not X = (_=_), not X = [_|_], not X = []. 

optify(_,[],[]).
optify(N,[H|T],[N=X|R]) :-
	term_to_atom(H,A),
	list_concat(['"',A,'"'],X),
	%%cerr(optify(N=X)),
	N1 is N + 1,
        optify(N1,T,R).

stringify([],'').
stringify([H|T],R) :-
        stringify(T,C),
	term_to_atom(H,A),
        list_concat([A,' ',C],R).

argify([H],H).
argify([H|T],H:R) :-
        argify(T,R).

list_stringify([],'').
list_stringify([H|T],R) :-
        list_stringify(T,C),
        head_stringify(H,C,R).

head_stringify(H,L,R) :-
	unwrap(H,T),
	term_to_atom(T,A),
	list_concat([A,' ',L],R).
</program>

@* Examples
<program>



:- use(library([objects,native])).
:- use(library([hush/hush,midi/midi])).

:- cerr(consulting(objects)).

go :- cerr(going(hush)),
      run(7).



dummy_new(This,self(This)) :-
	dummy(This):handler(H),
	cerr(calling(dummy_new(This,self(This)))),
	declare(H,new(dummy(This)),[handler],_).

dummy_dummy(This,self(This)) :-
	cerr(dummy_dummy(This,self(This))),
	declare(_,new(dummy(This)),[handler],_).

dummy_delete(This) :-
	cerr(dummy_delete(This)).

try_new(This,self(This)) :-
	declare(_,new(try(This)),[dummy],_).

try_lt(This,X,Y) :-
	X = value(A), Y = value(B),
	self(This):[ R1 = value(A), R2 = value(B), R1 < R2 ].

try_lt(This,X,Y) :- integer(X), integer(Y), X < Y.

try_gt(This,X,Y) :- try(This):(Y<X).



run(7) :-
	cerr(start(7)),
	new(try(This,Self)),
	Self:( var(a(X)) = f(f(X)) ),
	Self:( Z = value(a(2)) ),
	cerr(Self:( Z = value(a(2)) )),
	Self:[ var(a) = 1, var(b) = 2 ],
	(Self:(value(a) < value(b)) -> cerr(ok); cerr(shit)),
	(Self:(value(a) > value(b)) -> cerr(shit); cerr(ok)),
	delete(Self),
	cerr(end(7)).



run(6) :-
	cerr(start(6)),
	new(dummy(This)), %%,Self)),
	Self = self(This),
	%%dummy(_):dummy(Self),
	Self:assign(self,Self),
	Self:value(self,X),
	cerr(Self:value(self,X)),
	Self:(var(self)=2 ),
	Self:(V = value(self)),
	cerr(Self: V = value(self)),
	%%setof(X,Y^Z^value(X,Y,Z),L), cerr(value:L),
	delete(dummy(This)),
	Self:(XX = value(self)),
	cerr(Self: XX = value(self)),
	cerr(end(6)).



run(5) :-
	cerr(start(5)),
	midi(This):midi,
	midi(This):process(a),
	cerr(end(5)).
	


run(4) :-
	cerr(start(4)),
	midi(This):midi,
	Self = midi(This),
	Self:open('a.mid'),
	Self:header(0,1,480),
	Self:track(start),
	Self:melody([48,50,51,53,55]),
	Self:track(end),
	cerr(end(4)).



run(1) :-
	cerr(start(1)),
	query(This):query([kit=pl,src='test.pl']),
	query(This):evaluate(X:try(X,[1,2,3]),R),
	query(This):evaluate(S:setof(X,try(X,[1,2,3]),S),S),
	%%cerr(got(ID)),
	%%query(This):result(ID,R),
	cerr(result(R:S)),
	cerr(end(1)).



run(2) :- %% core ??
	cerr(start(2)),
	query(This):query([kit=pl]),
	query(This):eval(X:try(X,[1,2,3]),ID),
	query(This):result(ID,R),
	cerr(result(R)),
	cerr(end(2)).

try(X,L) :-
	cerr(trying(X,L)),
	member(X,L),
	cerr(giving(X)).



run(3) :-
	cerr(start(3)),
	midi(_):midi,
	cerr(end(3)).


</program>


