<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Mixing C and Prolog</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_10.html">previous</A>, <A HREF="sicstus_12.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC117" HREF="sicstus_toc.html#TOC117">Mixing C and Prolog</A></H1>
<P>
<A NAME="IDX772"></A>
<A NAME="IDX773"></A>
<A NAME="IDX774"></A>
<A NAME="IDX775"></A>
<A NAME="IDX776"></A>

</P>
<P>
SICStus Prolog provides a bi-directional, procedural interface for
program parts written in C and Prolog.  The C side of the interface
defines a number of functions and macros for various operations.  On the
Prolog side, you have to supply declarations specifying the names and
argument/value types of C functions being called as Prolog predicates.
These declarations are used by the predicate
<CODE>load_foreign_resource/1</CODE>, which performs the actual binding of C
functions to Prolog predicates.

</P>
<P>
<A NAME="IDX777"></A>
<A NAME="IDX778"></A>
<A NAME="IDX779"></A>
In most cases, the argument/value type declaration suffice for making
the necessary conversions of data automatically as they are passed
between C and Prolog.  However, it is possible to declare the type of an
argument to be a Prolog term, in which case the receiving function will
see it as a "handle" object, called an <CODE>SP_term_ref</CODE>, for which
access functions are provided.

</P>
<P>
The C support routines are available in a Development System as
well as in Runtime Systems.  The support routines include:

</P>

<UL>
<LI>

Static and dynamic linking of C code into the Prolog environment.

<LI>

Automatic conversion between Prolog terms and C data with
<CODE>foreign/(2-3)</CODE> declarations.

<LI>

Functions for accessing and creating Prolog terms,
and for creating and manipulating SP_term_refs.

<LI>

The Prolog system may call C predicates which may call Prolog back
without limits on recursion.

<LI>

Possibility to create stand-alone applications, <EM>Runtime Systems</EM>.

<LI>

Possibility to create user defined Prolog streams.

<LI>

Functions to read and write on Prolog streams from C.

<LI>

Possibility to install interrupt handlers that can safely call
Prolog.

<LI>

User hooks that can be used to perform user defined actions on a
number of occasions e.g. before reading a character from the standard input stream,
upon reinitialization, etc.

</UL>


<UL>
<LI><A HREF="sicstus_11.html#SEC118">ImplNotes</A>
<LI><A HREF="sicstus_11.html#SEC119">Calling C</A>
<LI><A HREF="sicstus_11.html#SEC127">Support</A>
<LI><A HREF="sicstus_11.html#SEC134">Calling Prolog</A>
<LI><A HREF="sicstus_11.html#SEC139">SICStus Streams</A>
<LI><A HREF="sicstus_11.html#SEC149">Muse Support Functions</A>
<LI><A HREF="sicstus_11.html#SEC150">Hooks</A>
<LI><A HREF="sicstus_11.html#SEC151">Runtime Systems</A>
<LI><A HREF="sicstus_11.html#SEC156">Development Systems</A>
<LI><A HREF="sicstus_11.html#SEC157">Mixing Examples</A>
</UL>



<H2><A NAME="SEC118" HREF="sicstus_toc.html#TOC118">Notes</A></H2>

<DL COMPACT>

<DT><EM>ANSI Conformance</EM>
<DD>
<A NAME="IDX780"></A>
<A NAME="IDX781"></A>
Throughout this chapter, <CODE>void *</CODE> in the function definitions may
be changed to <CODE>char *</CODE> on non ANSI conforming C compilers.

<DT><EM>The SP_PATH variable</EM>
<DD>
<A NAME="IDX782"></A>
It is normally not necessary to set this environment variable, but its
value will be used at runtime if no explicit boot path is given when
initializing a Runtime or Development System.  In this chapter, 
the environment variable <CODE>SP_PATH</CODE> is used as a shorthand
for the SICStus Prolog installation directory, whose default UNIX location is
<TT>`/usr/local/lib/sicstus37'</TT>). See section <A HREF="sicstus_4.html#SEC13">Environment Variables</A>.

<DT><EM>Definitions and declarations</EM>
<DD>
Type definitions and function declarations for the interface
are found in the header file <TT>`&#60;sicstus/sicstus.h&#62;'</TT>.

<DT><EM>Error Codes</EM>
<DD>
<A NAME="IDX783"></A>
<A NAME="IDX784"></A>
<A NAME="IDX785"></A>

The value of many support functions is a return code which is one of
<CODE>SP_SUCCESS</CODE> for success, <CODE>SP_FAILURE</CODE> for failure,
<CODE>SP_ERROR</CODE> if an error condition occurred.  In particular, uncaught
exceptions resulting from calls from C to Prolog raise an error
condition.  In error situations, the macro <CODE>SP_errno</CODE> will return a
value describing the error condition:

<A NAME="IDX786"></A>


<PRE>
int SP_errno
</PRE>

The function <CODE>SP_error_message</CODE> returns a pointer to the diagnostic
message corresponding to a specified error number:

<A NAME="IDX787"></A>


<PRE>
char *SP_error_message(int errno)
</PRE>

</DL>


<UL>
<LI><A HREF="sicstus_11.html#SEC119">Calling C</A>
<LI><A HREF="sicstus_11.html#SEC127">Support</A>
<LI><A HREF="sicstus_11.html#SEC134">Calling Prolog</A>
<LI><A HREF="sicstus_11.html#SEC139">SICStus Streams</A>
<LI><A HREF="sicstus_11.html#SEC149">Muse Support Functions</A>
<LI><A HREF="sicstus_11.html#SEC150">Hooks</A>
<LI><A HREF="sicstus_11.html#SEC151">Runtime Systems</A>
<LI><A HREF="sicstus_11.html#SEC156">Development Systems</A>
<LI><A HREF="sicstus_11.html#SEC157">Mixing Examples</A>
</UL>



<H2><A NAME="SEC119" HREF="sicstus_toc.html#TOC119">Calling C from Prolog</A></H2>
<P>
<A NAME="IDX788"></A>
<A NAME="IDX789"></A>

</P>
<P>
Functions written in the C language may be called from Prolog using an
interface in which automatic type conversions between Prolog terms and
common C types are declared as Prolog facts.  Calling without type
conversion can also be specified, in which case the arguments and values
are passed as SP_term_refs. This interface is partly modeled after
Quintus Prolog.

</P>
<P>
The functions installed using this foreign language interface may invoke
Prolog code and use the support functions described in the other
sections of this chapter.

</P>
<P>
Functions, or their equivalent, in any other language having C
compatible calling conventions may also be interfaced using this
interface. When referring to C functions in the following, we also
include such other language functions. Note however that a C compiler
is needed since a small amount of glue code (in C) must be generated
for interfacing purposes.  Note also that on some platforms different
calling conventions may exist for C functions. Check the Release Notes
to see which one is used by this interface.

</P>

<UL>
<LI><A HREF="sicstus_11.html#SEC120">Foreign Resource</A>
<LI><A HREF="sicstus_11.html#SEC121">Static and Dynamic Linking</A>
<LI><A HREF="sicstus_11.html#SEC122">Conversion Declarations</A>
<LI><A HREF="sicstus_11.html#SEC123">Conversions</A>
<LI><A HREF="sicstus_11.html#SEC124">Interface Predicates</A>
<LI><A HREF="sicstus_11.html#SEC125">Init and Deinit Functions</A>
<LI><A HREF="sicstus_11.html#SEC126">Creating the Linked Foreign Resource</A>
</UL>



<H3><A NAME="SEC120" HREF="sicstus_toc.html#TOC120">Foreign Resource and Conversion Declarations</A></H3>

<P>
<A NAME="IDX790"></A>
<A NAME="IDX791"></A>
<A NAME="IDX792"></A>
<A NAME="IDX793"></A>

</P>
<P>
A <EM>foreign resource</EM> is a set of C functions,
defined in one or more files, installed as an atomic operation.  The
name of a foreign resource, the <EM>resource name</EM>, is an atom, which
should uniquely identify the resource.  Thus, two foreign resources with
the same name cannot be installed at the same time.

</P>
<P>
For each foreign resource, a <CODE>foreign_resource/2</CODE> fact is used to
declare the interfaced functions. For each of these functions, a
<CODE>foreign/(2,3)</CODE> fact is used to specify conversions between predicate
arguments and C-types.  These conversion declarations are used for
creating the necessary interface between Prolog and C. 

</P>
<P>
The functions making up the foreign resource, the automatically
generated interface code, and any libraries, are compiled and linked to
form a <EM>linked foreign resource</EM>, which can be statically or
dynamically linked to a Development or Runtime System. In all cases,
the declared predicates are installed by the built-in predicate
<CODE>load_foreign_resource/1</CODE>. The resource name of a linked foreign
resource is derived from its file specification by deleting any leading
path and the suffix.

</P>


<H3><A NAME="SEC121" HREF="sicstus_toc.html#TOC121">Static and Dynamic linking</A></H3>

<P>
<A NAME="IDX794"></A>
<A NAME="IDX795"></A>
<A NAME="IDX796"></A>
<A NAME="IDX797"></A>

</P>
<P>
The interface can be used in two ways.

</P>
<P>
With <EM>static linking</EM>, the linked foreign resource is
statically linked to a Development or Runtime System.
<CODE>load_foreign_resource/1</CODE> determines if a resource has been statically
linked, in which case only the binding of predicate names to functions
needs to be done.  Otherwise, dynamic linking is attempted.

</P>
<P>
With <EM>dynamic linking</EM>, the linked foreign resource can be linked 
and unlinked to a Development or Runtime System, as needed.

</P>
<P>
A linked foreign resource may be implemented, depending on the hardware
and operating system platform, as e.g. a <EM>shared object</EM> or a
<EM>dynamic link library</EM>.

</P>



<H3><A NAME="SEC122" HREF="sicstus_toc.html#TOC122">Conversion Declarations</A></H3>

<P>
Conversion declaration predicates:

</P>
<DL COMPACT>

<DT><CODE>foreign_resource(<VAR>+ResourceName</VAR>,<VAR>+Functions</VAR>)</CODE>
<DD>
<A NAME="IDX798"></A>
<EM>A hook predicate.</EM> Specifies that a set of foreign functions,
to be called from Prolog, are to be found in the resource named by
<VAR>ResourceName</VAR>.  <VAR>Functions</VAR> is a list of functions exported by
the resource.  Only functions that are to be called from Prolog and
optionally one <EM>init function</EM> and one <EM>deinit function</EM> should
be listed. The init and deinit functions are specified as
<CODE>init(<VAR>Function</VAR>)</CODE> and <CODE>deinit(<VAR>Function</VAR>)</CODE>
respectively (see section <A HREF="sicstus_11.html#SEC125">Init and Deinit Functions</A>). This predicate should
be defined entirely in terms of facts (unit clauses). For example:


<PRE>
foreign_resource('terminal', [scroll,pos_cursor,ask]).
</PRE>

specifies that functions <CODE>scroll()</CODE>, <CODE>pos_cursor()</CODE> and
<CODE>ask()</CODE> are to be found in the resource <TT>`terminal'</TT>.

<DT><CODE>foreign(<VAR>+CFunctionName</VAR>, <VAR>+Predicate</VAR>)</CODE>
<DD>
<DT><CODE>foreign(<VAR>+CFunctionName</VAR>, <VAR>+Language</VAR>, <VAR>+Predicate</VAR>)</CODE>
<DD>
<A NAME="IDX799"></A>
<A NAME="IDX800"></A>
<EM>Hook predicates</EM>, specify the Prolog interface to a C function.
<VAR>Language</VAR> is at present constrained to the atom <CODE>c</CODE>.
<VAR>CFunctionName</VAR> is the name of a C function.  <VAR>Predicate</VAR>
specifies the name of the Prolog predicate that will be used to call
<VAR>CFunction()</VAR>.  <VAR>Predicate</VAR> also specifies how the predicate
arguments are to be translated into the corresponding C arguments. This
predicate should be defined entirely in terms of facts (unit
clauses). For example:


<PRE>
foreign(pos_cursor, c, move_cursor(+integer, +integer)).
</PRE>

The above example says that the C function <CODE>pos_cursor()</CODE> has two
integer value arguments and that we will use the predicate
<CODE>move_cursor/2</CODE> to call this function.  A goal <CODE>move_cursor(5, 23)</CODE>
would translate into the C call <CODE>pos_cursor(5,23);</CODE>.  
The third argument of the predicate <CODE>foreign/3</CODE> specifies how to
translate between Prolog arguments and C arguments. A call to a foreign
predicate will raise an exception if an input arguments is
uninstantiated (<CODE>instantiation_error/2</CODE>) or has the wrong type
(<CODE>type_error/4</CODE>) or domain (<CODE>domain_error/4</CODE>).  The call will
fail upon return from the function if the output arguments do not unify
with the actual arguments.

The available conversions are listed in the next subsection.
</DL>



<H3><A NAME="SEC123" HREF="sicstus_toc.html#TOC123">Conversions between Prolog Arguments and C Types</A></H3>

<P>
The following table lists the possible values for the arguments in the
predicate specification of <CODE>foreign/(2,3)</CODE>. The value declares which
conversion between corresponding Prolog argument and C type will take
place.

</P>
<DL COMPACT>

<DT><CODE>Prolog: +integer</CODE>
<DD>
<DT><CODE>C:      long</CODE>
<DD>
The argument should be a number.  It is converted to a C <CODE>long</CODE> and
passed to the C function.
<DT><CODE>Prolog: +float</CODE>
<DD>
<DT><CODE>C:      double</CODE>
<DD>
The argument should be a number.  It is converted to a C <CODE>double</CODE> and
passed to the C function.
<DT><CODE>Prolog: +atom</CODE>
<DD>
<DT><CODE>C:      unsigned long</CODE>
<DD>
The argument should be an atom.
Its canonical representation is passed to the C function.
<DT><CODE>Prolog: +chars</CODE>
<DD>
<DT><CODE>C:      char *</CODE>
<DD>
The argument should be a list of character codes.
The C function will be passed the address of an array of characters
containing these characters.  The
array is subject to reuse by other support functions, so if the value is
going to be used on a more than temporary basis, it must be moved
elsewhere. <DT><CODE>Prolog: +string</CODE>
<DD>
<DT><CODE>C:      char *</CODE>
<DD>
The argument should be an atom.
The C function will be passed the address of a text string
containing the printed representation of the atom.  The C function
should <EM>not</EM> overwrite the string.  <DT><CODE>Prolog: +string(<VAR>N</VAR>)</CODE>
<DD>
<DT><CODE>C:      char *</CODE>
<DD>
The argument should be an atom.
The printable representation of the string will be copied into a
newly allocated buffer.  The string will be truncated if it is longer
than <VAR>N</VAR> characters.  The string will be blank padded on the right
if it is shorter than <VAR>N</VAR> characters.  The C function will be passed
the address of the buffer.  The C function may overwrite the buffer,
but should not assume that it remains valid after returning.

<DT><CODE>Prolog: +address</CODE>
<DD>
<DT><CODE>C:      void *</CODE>
<DD>
The argument should be an integer which value is constrained according
to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()).  The value passed
will be a <CODE>void *</CODE> pointer.
<DT><CODE>Prolog: +address(<VAR>TypeName</VAR>)</CODE>
<DD>
<DT><CODE>C:      <VAR>TypeName</VAR> *</CODE>
<DD>
The argument should be an integer which value is constrained according
to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()).  The value passed
will be a <CODE><VAR>TypeName</VAR> *</CODE> pointer.
<DT><CODE>Prolog: +term</CODE>
<DD>
<DT><CODE>C:      SP_term_ref</CODE>
<DD>
The argument could be any term. The value passed will be the internal
representation of the term.  <DT><CODE>Prolog: -integer</CODE>
<DD>
<DT><CODE>C:      long *</CODE>
<DD>
The C function is passed a reference to an uninitialized <CODE>long</CODE>.
The value returned will be converted to a Prolog integer. <DT><CODE>Prolog: -float</CODE>
<DD>
<DT><CODE>C:      double *</CODE>
<DD>
The C function is passed a reference to an uninitialized <CODE>double</CODE>.
The value returned will be converted to a Prolog float. <DT><CODE>Prolog: -atom</CODE>
<DD>
<DT><CODE>C:      unsigned long *</CODE>
<DD>
The C function is passed a reference to an uninitialized <CODE>unsigned
long</CODE>.  The value returned should be the canonical representation of a
Prolog atom.  <DT><CODE>Prolog: -chars</CODE>
<DD>
<DT><CODE>C:      char **</CODE>
<DD>
The C function is passed the address of an uninitialized <CODE>char *</CODE>.
The returned string will be converted to a Prolog list of character
codes. <DT><CODE>Prolog: -string</CODE>
<DD>
<DT><CODE>C:      char **</CODE>
<DD>
The C function is passed the address of an uninitialized <CODE>char *</CODE>.
The returned string will be converted to a Prolog atom.  Prolog will copy
the string to a safe place, so the memory occupied by the returned string
may be reused during subsequent calls to foreign code. <DT><CODE>Prolog: -string(<VAR>N</VAR>)</CODE>
<DD>
<DT><CODE>C:      char *</CODE>
<DD>
The C function is passed a reference to a character buffer large enough
to store an <VAR>N</VAR> character string.  The returned string will be
stripped of trailing blanks and converted to a Prolog atom. <DT><CODE>Prolog: -address</CODE>
<DD>
<DT><CODE>C:      void **</CODE>
<DD>
The C function is passed the address of an uninitialized <CODE>void *</CODE>.
The returned value, which is constrained according to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()), will be converted to a Prolog integer.
<DT><CODE>Prolog: -address(<VAR>TypeName</VAR>)</CODE>
<DD>
<DT><CODE>C:      <VAR>TypeName</VAR> **</CODE>
<DD>
The C function is passed the address of an uninitialized
<CODE><VAR>TypeName</VAR> *</CODE>.  The returned value, which is constrained
according to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()), will be
converted to a Prolog integer.
<DT><CODE>Prolog: -term</CODE>
<DD>
<DT><CODE>C:      SP_term_ref</CODE>
<DD>
The C function is passed a new SP_term_ref, and is expected to set its
value to a suitable Prolog term.  Prolog will try to unify the value
with the actual argument. <DT><CODE>Prolog: [-integer]</CODE>
<DD>
<DT><CODE>C:      long <VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>long</CODE>.  The value returned will be
converted to a Prolog integer. <DT><CODE>Prolog: [-float]</CODE>
<DD>
<DT><CODE>C:      double <VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>double</CODE>.  The value returned will
be converted to a Prolog float. <DT><CODE>Prolog: [-atom]</CODE>
<DD>
<DT><CODE>C:      unsigned long <VAR>F</VAR>()</CODE>
<DD>
The C function should return an <CODE>unsigned long</CODE>.  The value
returned must be the canonical representation of a Prolog atom. <DT><CODE>Prolog: [-chars]</CODE>
<DD>
<DT><CODE>C:      char *<VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>char *</CODE>.  The returned string will
be converted to a Prolog list of character codes. <DT><CODE>Prolog: [-string]</CODE>
<DD>
<DT><CODE>C:      char *<VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>char *</CODE>.  The returned string will
be converted to a Prolog atom.  Prolog will copy
the string to a safe place, so the memory occupied by the returned string
may be reused during subsequent calls to foreign code. <DT><CODE>Prolog: [-string(<VAR>N</VAR>)]</CODE>
<DD>
<DT><CODE>C:      char *<VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>char *</CODE>.  The first <VAR>N</VAR>
characters of the string will be copied and the copied string will be
stripped of trailing blanks.  The stripped string will be converted to a
Prolog atom. C may reuse or destroy the string buffer during 
later calls. <DT><CODE>Prolog: [-address]</CODE>
<DD>
<DT><CODE>C:      void *<VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE>void *</CODE>.  The returned value, which
is constrained according to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>,
SP_put_address()), will be converted to a Prolog integer. <DT><CODE>Prolog: [-address(<VAR>TypeName</VAR>)]</CODE>
<DD>
<DT><CODE>C:      <VAR>TypeName</VAR> *<VAR>F</VAR>()</CODE>
<DD>
The C function should return a <CODE><VAR>TypeName</VAR> *</CODE>.  The returned
value, which is constrained according to (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>,
SP_put_address()), will be converted to a Prolog integer.
<DT><CODE>Prolog: [-term]</CODE>
<DD>
<DT><CODE>C:      SP_term_ref <VAR>F</VAR>()</CODE>
<DD>
The C function should return an SP_term_ref.
Prolog will try to unify its value with the actual argument.  </DL>



<H3><A NAME="SEC124" HREF="sicstus_toc.html#TOC124">Interface Predicates</A></H3>

<P>
In the following descriptions, the <VAR>Resource</VAR> argument is an
unsuffixed file specification of a linked foreign resource.

</P>
<DL COMPACT>

<DT><CODE>link_foreign_resource(<VAR>+Resource</VAR>,<VAR>+SourceFile</VAR>,<VAR>+Option</VAR>,<VAR>+CFiles</VAR>,<VAR>+ObjectFiles</VAR>,<VAR>+Libraries</VAR>)</CODE>
<DD>
<A NAME="IDX801"></A>

Builds a linked foreign resource. <VAR>Option</VAR> can be either
<CODE>dynamic</CODE> or <CODE>static</CODE> as described in the alternative form
shown below.  This predicate is not available in Runtime Systems.

The work of this predicate is done by a command located in
<TT>`$SP_PATH/bin'</TT>. This command is configurable as described in the
Release Notes. It can also be invoked
directly from a shell as an alternative to running it from Prolog:

<DT><CODE>splfr <VAR>Resource</VAR> <VAR>SourceFile</VAR> [+dynamic | +static] [+c <VAR>CFiles</VAR>] [+o <VAR>ObjectFiles</VAR>] [+l <VAR>Libraries</VAR>] [+copt <VAR>COptions</VAR>]</CODE>
<DD>
<A NAME="IDX802"></A>

Reads terms from <VAR>SourceFile</VAR>, applying term expansion and
extracting any <CODE>foreign_resource/2</CODE> fact with first argument
matching the name of <VAR>Resource</VAR> and all <CODE>foreign/(2,3)</CODE> facts.
Compiles <VAR>CFiles</VAR> and the interface code derived from these facts
and links the resulting object files with <VAR>ObjectFiles</VAR> and
<VAR>Libraries</VAR> to form the linked foreign resource <VAR>Resource</VAR>. The
option <CODE>+dynamic</CODE> (<CODE>+static</CODE>) specifies that the linked
foreign resource is suitable for dynamic (static)
linking. The default is dynamic. The option <CODE>+copt</CODE> can be used to
pass options to the C-compiler.

<DT><CODE>load_foreign_resource(<VAR>:Resource</VAR>)</CODE>
<DD>
<A NAME="IDX803"></A>

Unless a foreign resource with the same name as <VAR>Resource</VAR> has been
statically linked, the linked foreign resource specified by
<VAR>Resource</VAR> is linked into the Prolog load image.  In both cases, the
predicates defined by <VAR>Resource</VAR> are installed, and any init
function is called. Dynamic linking is not possible if the foreign
resource was linked using the <CODE>+static</CODE> option.

If a resource with the same name has been previously loaded, it will be
unloaded as if <CODE>unload_foreign_resource(<VAR>Resource</VAR>)</CODE> was
called, before <VAR>Resource</VAR> is loaded.

In Muse, the system is required to be adjusted to one worker
first, and the predicates defined become <EM>cavalier</EM>
(see section <A HREF="sicstus_6.html#SEC63">Programming Considerations</A>).

<DT><CODE>unload_foreign_resource(<VAR>:Resource</VAR>)</CODE>
<DD>
<A NAME="IDX804"></A>
Any deinit function associated with <VAR>Resource</VAR> is called, and
the predicates defined by <VAR>Resource</VAR> are uninstalled.
If <VAR>Resource</VAR> has been dynamically linked, it is unlinked
from the Prolog load image.

In Muse, the system is required to be adjusted to one worker
first, and the predicates defined become <EM>cavalier</EM>
</DL>

<P>
The following predicates are provided for backwards compatibility and
should be avoided in new code:

</P>
<DL COMPACT>

<DT><CODE>foreign_file(<VAR>+File</VAR>,<VAR>+Functions</VAR>)</CODE>
<DD>
<A NAME="IDX805"></A>
<EM>A hook predicate.</EM> Specifies that a set of foreign functions,
to be called from Prolog, are to be found in <VAR>File</VAR>. This predicate
is only called from <CODE>load_foreign_files/2</CODE>.

<DT><CODE>load_foreign_files(<VAR>:ObjectFiles</VAR>,<VAR>+Libraries</VAR>)</CODE>
<DD>
<A NAME="IDX806"></A>
<EM>Hookable</EM>, a resource name is derived from the first file name in
<VAR>ObjectFiles</VAR> by stripping off the suffix. If this resource has been
statically linked, the predicates defined by it are installed;
otherwise, a linked foreign resource containing the declared functions
is created and loaded.  Not available in Runtime Systems.
</DL>



<H3><A NAME="SEC125" HREF="sicstus_toc.html#TOC125">Init and Deinit Functions</A></H3>

<P>
An <EM>init function</EM> and/or a <EM>deinit function</EM> can be declared by
<CODE>foreign_resource/2</CODE>. If this is the case, these functions should
have the prototype:

</P>
<DL COMPACT>

<DT><CODE>void <VAR>FunctionName</VAR> (int when)</CODE>
<DD>
</DL>

<P>
The init function is called by <CODE>load_foreign_resource/1</CODE> after the
resource has been loaded and the interfaced predicates have been
installed.

</P>
<P>
The deinit function is called by <CODE>unload_foreign_resource/1</CODE> before
the interfaced predicates have been uninstalled and the resource has
been unloaded.

</P>
<P>
The init and deinit functions may use the C-interface to call Prolog
etc.

</P>
<P>
Foreign resources are unloaded before saving states, and reloaded
afterwards or when the saved state is restored;
see section <A HREF="sicstus_4.html#SEC24">Saving and Restoring Program States</A>.  Foreign resources are also unloaded when
exiting Prolog execution. The parameter <CODE>when</CODE> reflects the context
of the <CODE>(un)load_foreign_resource/1</CODE> and is set as follows for
init functions:

</P>
<DL COMPACT>

<DT><CODE>SP_WHEN_EXPLICIT</CODE>
<DD>
Explicit call to <CODE>load_foreign_resource/1</CODE>.
<DT><CODE>SP_WHEN_RESTORE</CODE>
<DD>
Resource is reloaded after save or restore.
</DL>

<P>
For deinit functions:

</P>
<DL COMPACT>

<DT><CODE>SP_WHEN_EXPLICIT</CODE>
<DD>
Explicit call to <CODE>unload_foreign_resource/1</CODE>.
<DT><CODE>SP_WHEN_SAVE</CODE>
<DD>
Resource is unloaded before save.
<DT><CODE>SP_WHEN_EXIT</CODE>
<DD>
Resource is unloaded before exiting Prolog.
</DL>



<H3><A NAME="SEC126" HREF="sicstus_toc.html#TOC126">Creating the Linked Foreign Resource</A></H3>

<P>
<A NAME="IDX807"></A>
<A NAME="IDX808"></A>
Suppose we have a Prolog source file <CODE>ex.pl</CODE> containing:

</P>

<PRE>
foreign(f1, p1(+integer,[-integer])).
foreign(f2, p2(+integer,[-integer])).
foreign_resource(ex, [f1,f2]).
:- load_foreign_resource(ex).
</PRE>

<P>
and a C source file <CODE>ex.c</CODE> with definitions of the functions
<CODE>f1</CODE> and <CODE>f2</CODE>, both returning <CODE>long</CODE> and having a
<CODE>long</CODE> as only parameter. The conversion declarations in
<TT>`ex.pl'</TT> state that these functions form the foreign resource
<CODE>ex</CODE>.

</P>
<P>
To create the linked foreign resource, simply type (to Prolog):

</P>

<PRE>
| ?- link_foreign_resource(ex,'ex.pl',dynamic,['ex.c'],[],[]).
</PRE>

<P>
or alternatively (to the Shell):

</P>

<PRE>
% splfr ex ex.pl +c ex.c
</PRE>

<P>
The linked foreign resource <TT>`ex.so'</TT> (file suffix <TT>`.so'</TT> is
system dependent) has been created. It will be dynamically linked by the
directive <CODE>:- load_foreign_resource(ex).</CODE> when the file <TT>`ex.pl'</TT>
is loaded.  Linked foreign resources can also be created manually
(see section <A HREF="sicstus_41.html#SEC363">Glue Code Generator</A>).

</P>
<P>
Dynamic linking of foreign resources can also be used by Runtime
Systems.  On some platforms, however, the
executable must not be <EM>stripped</EM> for dynamic linking to work,
i.e. its symbol table must remain.

</P>



<H2><A NAME="SEC127" HREF="sicstus_toc.html#TOC127">Support Functions</A></H2>
<P>
<A NAME="IDX809"></A>
<A NAME="IDX810"></A>

</P>
<P>
The support functions include functions to manipulate SP_term_refs,
functions to convert data between the
basic C types and Prolog terms, functions to test whether a term can be
converted to a specific C type, and functions to unify or compare two
terms.

</P>

<UL>
<LI><A HREF="sicstus_11.html#SEC128">Creating and Manipulating SP_term_refs</A>
<LI><A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>
<LI><A HREF="sicstus_11.html#SEC130">Accessing Prolog Terms</A>
<LI><A HREF="sicstus_11.html#SEC131">Testing Prolog Terms</A>
<LI><A HREF="sicstus_11.html#SEC132">Unifying and Comparing Terms</A>
<LI><A HREF="sicstus_11.html#SEC133">Memory Allocation</A>
</UL>



<H3><A NAME="SEC128" HREF="sicstus_toc.html#TOC128">Creating and Manipulating SP_term_refs</A></H3>

<P>
<A NAME="IDX811"></A>
Normally, C functions only have indirect access to Prolog terms via 
SP_term_refs.  C functions may receive arguments as 
unconverted Prolog terms, in which case the actual arguments received will
have the type <CODE>SP_term_ref</CODE>.  Also, a C function may return an
unconverted Prolog term, in which case it must create an SP_term_ref.
Finally, any temporary Prolog terms created by C code must be handled
as SP_term_refs.

</P>
<P>
SP_term_refs are motivated by the fact that SICStus Prolog's memory manager
must have a means of reaching all live Prolog terms for memory management
purposes, including such terms that are being manipulated
by the user's C code.  Previous releases of SICStus Prolog provided direct
access to Prolog terms and the ability to tell the memory manager that
a given memory address points to a Prolog term, but this approach was
too low level and highly error-prone.  The current design is modeled after
and largely compatible with Quintus Prolog release 3.

</P>
<P>
SP_term_refs are created dynamically.  At any given time, an
SP_term_ref has a value (a Prolog term).  This value can be examined,
accessed, and updated by the support functions described in this section.

</P>
<P>
It is important to understand the rules governing the scope of
SP_term_refs in conjunction with calls from Prolog to C and
vice versa:

</P>

<UL>
<LI>

When a C function called from Prolog returns, all SP_term_refs 
passed to the function or dynamically created by the function become invalid.

<LI>

When terms are passed to C as a result of calling Prolog, those terms
and any SP_term_refs created since the start of the query are only valid
up to the request for the next solution.  They also become invalid when
the query is closed.
</UL>

<P>
A new SP_term_ref whose value is <CODE>[]</CODE> is created by calling

</P>
<P>
<A NAME="IDX812"></A>

</P>

<PRE>
SP_term_ref SP_new_term_ref(void)
</PRE>

<P>
The value of the SP_term_ref <CODE>to</CODE> is set to the value of the
SP_term_ref <CODE>from</CODE> by calling <CODE>SP_put_term(to,from)</CODE>.  The
previous value of <CODE>to</CODE> is lost:

</P>
<P>
<A NAME="IDX813"></A>

</P>

<PRE>
void SP_put_term(SP_term_ref to, SP_term_ref from)
</PRE>

<P>
Each Prolog atom is represented internally by a unique integer,
represented in C as an <CODE>unsigned long</CODE>.  This mapping between atoms
and integers depends on the execution history.  Certain functions
require this representation as opposed to an SP_pred_ref.  It can be
obtain by a special argument type declaration when calling C from
Prolog, by calling <CODE>SP_get_atom()</CODE>, or by looking up a string
<CODE>s</CODE> in the Prolog symbol table by calling
<CODE>SP_atom_from_string(s)</CODE>:

</P>
<P>
<A NAME="IDX814"></A>

</P>

<PRE>
unsigned long SP_atom_from_string(char *s)
</PRE>

<P>
The print name of a Prolog atom <CODE>a</CODE> can be obtained by calling:

</P>
<P>
<A NAME="IDX815"></A>

</P>

<PRE>
char *SP_string_from_atom(unsigned long a)
</PRE>

<P>
The print name length of a Prolog atom <CODE>a</CODE> can be obtained by calling:

</P>
<P>
<A NAME="IDX816"></A>

</P>

<PRE>
int SP_atom_length(unsigned long a)
</PRE>

<P>
Same as <CODE>strlen(SP_string_from_atom(a))</CODE> but runs in O(1) time.

</P>
<P>
Prolog atoms, and the space occupied by their print names, are subject
to garbage collection when the number of atoms has reached a certain
threshold, under the control of the <CODE>agc_margin</CODE> Prolog flag
(see section <A HREF="sicstus_10.html#SEC105">Information about the State of the Program</A>), or when the atom garbage collector is called
explicitly.  The atom garbage collector will find all references to
atoms from the Prolog specific memory areas, including SP_term_refs and
arguments passed from Prolog to foreign language functions.  However, atoms
created by <CODE>SP_atom_from_string</CODE> and merely stored in a local variable
are endangered by garbage collection.  The following functions make it
possible to protect an atom while it is in use.  The operations are
implemented using reference counters to cater for multiple, independent
use of the same atom in different foreign resources:

</P>
<P>
<A NAME="IDX817"></A>

</P>

<PRE>
int SP_register_atom(unsigned long a)
</PRE>

<P>
Registers the atom <CODE>a</CODE> with the Prolog memory manager by incrementing
its reference counter.  Returns a nonzero value if the operation succeeds.

</P>
<P>
<A NAME="IDX818"></A>

</P>

<PRE>
int SP_unregister_atom(unsigned long a)
</PRE>

<P>
Unregisters the atom <CODE>a</CODE> with the Prolog memory manager by decrementing
its reference counter.  Returns a nonzero value if the operation succeeds.

</P>



<H3><A NAME="SEC129" HREF="sicstus_toc.html#TOC129">Creating Prolog Terms</A></H3>

<P>
These functions create a term and store it as the value of an
SP_term_ref, which must exist prior to the call.  They return zero if
the conversion fails (as far as failure can be detected), and a nonzero
value otherwise, assigning to <CODE>t</CODE> the converted value.

</P>
<P>
<A NAME="IDX819"></A>
<A NAME="IDX820"></A>
<A NAME="IDX821"></A>
<A NAME="IDX822"></A>
<A NAME="IDX823"></A>
<A NAME="IDX824"></A>
<A NAME="IDX825"></A>
<A NAME="IDX826"></A>
<A NAME="IDX827"></A>
<A NAME="IDX828"></A>
<A NAME="IDX829"></A>
<A NAME="IDX830"></A>
<A NAME="IDX831"></A>

</P>
<DL COMPACT>

<DT><CODE>int SP_put_variable(SP_term_ref t)</CODE>
<DD>
Assigns to <CODE>t</CODE> a new Prolog variable.

<DT><CODE>int SP_put_integer(SP_term_ref t, long l)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog integer from a C long integer.

<DT><CODE>int SP_put_float(SP_term_ref t, double d)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog float from a C double.

<DT><CODE>int SP_put_atom(SP_term_ref t, unsigned long a)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog atom from <CODE>a</CODE>,
which must be the canonical representation of a Prolog atom.
(see section <A HREF="sicstus_11.html#SEC119">Calling C from Prolog</A>).

<DT><CODE>int SP_put_string(SP_term_ref t, char *name)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog atom from a C string.

<DT><CODE>int SP_put_address(SP_term_ref t, void *pointer)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog integer from a C pointer.

The pointer must be <CODE>NULL</CODE> or an address having the four most
significant bits in common with addresses returned by
<CODE>malloc()</CODE>. (This excludes pointers to automatic C variables,
i.e. addresses to the C stack).  Furthermore, the address must be aligned
on a four bytes boundary.

<CODE>NULL</CODE> is converted to the integer <CODE>0</CODE>.

<DT><CODE>int SP_put_list_chars(SP_term_ref t, SP_term_ref tail, char *s)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog list of the character codes in <CODE>s</CODE>.
The list is terminated by the value of <CODE>tail</CODE>.

<DT><CODE>int SP_put_list_n_chars(SP_term_ref t, SP_term_ref tail, long n, char *s)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog list of the first <CODE>n</CODE> character codes
or zeroes in <CODE>s</CODE>.  The list is terminated by the value of
<CODE>tail</CODE>.

<DT><CODE>int SP_put_number_chars(SP_term_ref t, char *s)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog number by parsing the string in <CODE>s</CODE>.

<DT><CODE>int SP_put_functor(SP_term_ref t, unsigned long name, int arity)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog compound term with all the arguments
unbound variables.  If <CODE>arity</CODE> is 0, assigns the Prolog atom whose
canonical representation is <CODE>name</CODE> to <CODE>t</CODE>.  This is similar to
calling <CODE>functor/3</CODE> with the first argument unbound and the second
and third arguments bound to an atom and an integer, respectively.

<DT><CODE>int SP_put_list(SP_term_ref t)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog list whose head and tail are both unbound
variables.

<DT><CODE>int SP_cons_functor(SP_term_ref t, unsigned long name, int arity, SP_term_ref arg, ...)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog compound term whose arguments are the
values of <CODE>arg</CODE>...  If <CODE>arity</CODE> is 0, assigns the Prolog atom
whose canonical representation is <CODE>name</CODE> to <CODE>t</CODE>.  This is
similar to calling <CODE>=../2</CODE> with the first argument unbound and the
second argument bound.

<DT><CODE>int SP_cons_list(SP_term_ref t, SP_term_ref head, SP_term_ref tail)</CODE>
<DD>
Assigns to <CODE>t</CODE> a Prolog list whose head and tail are the values of
<CODE>head</CODE> and <CODE>tail</CODE>.

</DL>



<H3><A NAME="SEC130" HREF="sicstus_toc.html#TOC130">Accessing Prolog Terms</A></H3>

<P>
These functions will take an SP_term_ref and convert it to C data.
They return zero if the conversion fails, and a nonzero value otherwise,
and (except the last one) store the C data in output arguments.

</P>
<P>
<A NAME="IDX832"></A>
<A NAME="IDX833"></A>
<A NAME="IDX834"></A>
<A NAME="IDX835"></A>
<A NAME="IDX836"></A>
<A NAME="IDX837"></A>
<A NAME="IDX838"></A>
<A NAME="IDX839"></A>
<A NAME="IDX840"></A>
<A NAME="IDX841"></A>
<A NAME="IDX842"></A>

</P>
<DL COMPACT>

<DT><CODE>int SP_get_integer(SP_term_ref t, long *l)</CODE>
<DD>
Assigns to <CODE>*l</CODE> the C <CODE>long</CODE> corresponding to a Prolog
number.  The value must fit in <CODE>*l</CODE> for the operation to succeed.

<DT><CODE>int SP_get_float(SP_term_ref t, double *d)</CODE>
<DD>
Assigns to <CODE>*d</CODE> the C <CODE>double</CODE> corresponding to a Prolog
number.

<DT><CODE>int SP_get_atom(SP_term_ref t, unsigned long *a)</CODE>
<DD>
Assigns to <CODE>*a</CODE> the canonical representation of a Prolog atom.

<DT><CODE>int SP_get_string(SP_term_ref t, char **name)</CODE>
<DD>
Assigns to <CODE>*name</CODE> a pointer to the string that is the name of a
Prolog atom.  This string must <EM>not</EM> be modified.

<DT><CODE>int SP_get_address(SP_term_ref t, void **pointer)</CODE>
<DD>
Assigns to <CODE>*pointer</CODE> a C pointer from a Prolog term. The term
should be an integer which value is constrained according to
(see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()).

<DT><CODE>int SP_get_list_chars(SP_term_ref t, char **s)</CODE>
<DD>
Assigns to <CODE>*s</CODE> a zero-terminated array of characters
corresponding to a Prolog list of character codes.  The array is subject
to reuse by other support functions, so if the value is going to be used
on a more than temporary basis, it must be moved elsewhere.

<DT><CODE>int SP_get_list_n_chars(SP_term_ref t, SP_term_ref tail, long n, long *w, char *s)</CODE>
<DD>
Copies into <CODE>s</CODE> at most <CODE>n</CODE> character codes or zeroes from the
list <CODE>t</CODE>.  The number of character codes actually written is
assigned to <CODE>*w</CODE>.  <CODE>tail</CODE> is set to the remainder of the list.
The array <CODE>s</CODE> must have room for at least <CODE>n</CODE> characters.

<DT><CODE>int SP_get_number_chars(SP_term_ref t, char **s)</CODE>
<DD>
Assigns to <CODE>*s</CODE> a zero-terminated array of characters
corresponding to the printed representation of a Prolog number.  The
array is subject to reuse by other support functions, so if the value is
going to be used on a more than temporary basis, it must be moved
elsewhere.

<DT><CODE>int SP_get_functor(SP_term_ref t, unsigned long *name, int *arity)</CODE>
<DD>
Assigns to <CODE>*name</CODE> and <CODE>*arity</CODE> the canonical representation
and arity of the principal functor of a Prolog compound term.  If the
value of <CODE>t</CODE> is an atom, then that atom is assigned to
<CODE>*name</CODE> and 0 is assigned to <CODE>*arity</CODE>.  This is similar to
calling <CODE>functor/3</CODE> with the first argument bound to a compound
term or an atom and the second and third arguments unbound.

<DT><CODE>int SP_get_list(SP_term_ref t, SP_term_ref head, SP_term_ref tail)</CODE>
<DD>
Assigns to <CODE>head</CODE> and <CODE>tail</CODE> the head and tail of a Prolog list.

<DT><CODE>int SP_get_arg(int i, SP_term_ref t, SP_term_ref arg)</CODE>
<DD>
Assigns to <CODE>arg</CODE> the <CODE>i</CODE>:th argument of a Prolog compound
term.  This is similar to calling <CODE>arg/3</CODE> with the third argument
unbound.
</DL>



<H3><A NAME="SEC131" HREF="sicstus_toc.html#TOC131">Testing Prolog Terms</A></H3>

<P>
There is one general function for type testing of Prolog terms and a set
of specialized, more efficient, functions, one for each term type.

</P>
<P>
<A NAME="IDX843"></A>
<A NAME="IDX844"></A>
<A NAME="IDX845"></A>
<A NAME="IDX846"></A>
<A NAME="IDX847"></A>
<A NAME="IDX848"></A>
<A NAME="IDX849"></A>
<A NAME="IDX850"></A>
<A NAME="IDX851"></A>

</P>
<P>
<A NAME="IDX852"></A>
<A NAME="IDX853"></A>
<A NAME="IDX854"></A>
<A NAME="IDX855"></A>
<A NAME="IDX856"></A>

</P>
<DL COMPACT>

<DT><CODE>int SP_term_type(SP_term_ref t)</CODE>
<DD>
Depending on the type of the term <VAR>t</VAR>, one of
<CODE>SP_TYPE_VARIABLE</CODE>, <CODE>SP_TYPE_INTEGER</CODE>, <CODE>SP_TYPE_FLOAT</CODE>,
<CODE>SP_TYPE_ATOM</CODE>, or <CODE>SP_TYPE_COMPOUND</CODE> is returned.

<DT><CODE>int SP_is_variable(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog variable, zero otherwise.

<DT><CODE>int SP_is_integer(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog integer, zero otherwise.

<DT><CODE>int SP_is_float(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog float, zero otherwise.

<DT><CODE>int SP_is_atom(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog atom, zero otherwise.

<DT><CODE>int SP_is_compound(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog compound term, zero otherwise.

<DT><CODE>int SP_is_list(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog list, zero otherwise.

<DT><CODE>int SP_is_atomic(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is an atomic Prolog term, zero otherwise.

<DT><CODE>int SP_is_number(SP_term_ref t)</CODE>
<DD>
Returns nonzero if the term is a Prolog number, zero otherwise.
</DL>



<H3><A NAME="SEC132" HREF="sicstus_toc.html#TOC132">Unifying and Comparing Terms</A></H3>

<P>
<A NAME="IDX857"></A>
<A NAME="IDX858"></A>

</P>
<DL COMPACT>

<DT><CODE>int SP_unify(SP_term_ref x, SP_term_ref y)</CODE>
<DD>
Unifies two terms, returning zero on failure and nonzero on success.

<DT><CODE>int SP_compare(SP_term_ref x, SP_term_ref y)</CODE>
<DD>
Returns <CODE>-1</CODE> if <CODE>x</CODE> @&#60; <CODE>y</CODE>, <CODE>0</CODE> if <CODE>x</CODE> ==
<CODE>y</CODE> and <CODE>1</CODE> if <CODE>x</CODE> @&#62; <CODE>y</CODE>
</DL>



<H3><A NAME="SEC133" HREF="sicstus_toc.html#TOC133">Memory Allocation</A></H3>

<P>
The usual C library memory allocation functions (<CODE>malloc</CODE>, <CODE>realloc</CODE>,
and <CODE>free</CODE>) may not work properly in foreign code.  The following
functions provide these services from SICStus Prolog's memory manager:

</P>
<DL COMPACT>

<DT><CODE>void *SP_malloc(unsigned int size)</CODE>
<DD>
<A NAME="IDX859"></A>
Returns a properly aligned pointer to a block of at least <CODE>size</CODE>
bytes.  In Muse, the blocks are allocated in shared memory.

<DT><CODE>void *SP_realloc(void *ptr, unsigned int size)</CODE>
<DD>
<A NAME="IDX860"></A>
Changes the size of the block referenced by <CODE>ptr</CODE> to <CODE>size</CODE>
bytes and returns a pointer to the (possibly moved) block.  The contents
will be unchanged up to the lesser of the new and old sizes.  The block
referenced by <CODE>ptr</CODE> must have been obtained by a call to
<CODE>SP_malloc</CODE> or <CODE>SP_realloc</CODE>, and must not have been released
by a call to <CODE>SP_free</CODE> or <CODE>SP_realloc</CODE>.
In Muse, the blocks are allocated in shared memory.

<DT><CODE>void SP_free(void *ptr)</CODE>
<DD>
<A NAME="IDX861"></A>
Releases the block referenced by <CODE>ptr</CODE>, which must have been
obtained by a call to <CODE>SP_malloc</CODE> or <CODE>SP_realloc</CODE>, and must
not have been released by a call to <CODE>SP_free</CODE> or <CODE>SP_realloc</CODE>.
In Muse, the blocks are allocated in shared memory.
</DL>



<H2><A NAME="SEC134" HREF="sicstus_toc.html#TOC134">Calling Prolog from C</A></H2>
<P>
<A NAME="IDX862"></A>

</P>
<P>
In the sequential Development System and in Runtime Systems,
Prolog and C code may call each other to arbitrary depths.  The ability
to call Prolog from C is not available in the Muse Development System
in this release.

</P>
<P>
Before calling a predicate from C you must look up the predicate
definition by module, name, and arity.  The function
<CODE>SP_predicate()</CODE> will return a pointer to this definition or return
<CODE>NULL</CODE> if the predicate is not visible in the module.  This
definition could be used in more than one call to the same predicate.
The module specification is optional.  If <CODE>NULL</CODE> or <CODE>""</CODE> (the
empty string) is given then the default <EM>type-in</EM> module
(see section <A HREF="sicstus_7.html#SEC66">Module Prefixing</A>) is assumed:
<A NAME="IDX863"></A>

</P>

<PRE>
SP_pred_ref SP_predicate(char *name_string,
                         long arity,
                         char *module_string)
</PRE>

<P>
The function <CODE>SP_pred()</CODE> may be used as an alternative to the
above.  The only difference is that the name and module arguments are
passed as Prolog atoms rather than strings, and the module argument is
mandatory.  This saves the cost of looking up the two arguments in the
Prolog symbol table.  This cost dominates the cost of
<CODE>SP_predicate()</CODE>.
<A NAME="IDX864"></A>

</P>

<PRE>
SP_pred_ref SP_pred(unsigned long name_atom,
                    long arity,
                    unsigned long module_atom)
</PRE>


<UL>
<LI><A HREF="sicstus_11.html#SEC135">Finding One Solution of a Call</A>
<LI><A HREF="sicstus_11.html#SEC136">Finding Multiple Solutions of a Call</A>
<LI><A HREF="sicstus_11.html#SEC137">Calling Prolog Asynchronously</A>
<LI><A HREF="sicstus_11.html#SEC138">Exception Handling in C</A>
</UL>



<H3><A NAME="SEC135" HREF="sicstus_toc.html#TOC135">Finding One Solution of a Call</A></H3>

<P>
The easiest way to call a predicate if you are only interested in the
first solution is to call the function <CODE>SP_query()</CODE>.  It will
create a goal from the predicate definition and the arguments, and try
to prove it.  The call will return <CODE>SP_SUCCESS</CODE> if the goal
succeeded, <CODE>SP_FAILURE</CODE> if it failed, and <CODE>SP_ERROR</CODE> if an
error condition occurred.

</P>
<P>
<A NAME="IDX865"></A>

</P>

<PRE>
int SP_query(SP_pred_ref predicate, SP_term_ref arg1, ...)
</PRE>

<P>
If you are only interested in the side effects of a predicate you can
call <CODE>SP_query_cut_fail()</CODE>.  It will try to prove the predicate,
cut away the rest of the solutions, and finally fail.  This will
reclaim the storage used after the call:
<A NAME="IDX866"></A>

</P>

<PRE>
int SP_query_cut_fail(SP_pred_ref predicate, SP_term_ref arg1, ...)
</PRE>



<H3><A NAME="SEC136" HREF="sicstus_toc.html#TOC136">Finding Multiple Solutions of a Call</A></H3>

<P>
If you are interested in more than one solution a more complicated
scheme is used.  You find the predicate definition as above but you
don't call the predicate directly.

</P>

<OL>
<LI>

Set up a call with <CODE>SP_open_query()</CODE>
<LI>

Call <CODE>SP_next_solution()</CODE> to find a solution.  Call this predicate
again to find more solutions if there are any.
<LI>

Terminate the call with <CODE>SP_close_query()</CODE> or <CODE>SP_cut_query()</CODE>
</OL>

<P>
The function <CODE>SP_open_query()</CODE> will return an identifier of type
<CODE>SP_qid</CODE> that you use in successive calls, or <CODE>NULL</CODE>, if given
an invalid predicate reference.  Note that if a new query is opened
while another is already open, the new query must be terminated before
performing any action on the old one.  That is, queries must be
strictly nested:
<A NAME="IDX867"></A>
<A NAME="IDX868"></A>

</P>

<PRE>
SP_qid SP_open_query(SP_pred_ref predicate, SP_term_ref arg1, ...)
</PRE>

<P>
The function <CODE>SP_next_solution()</CODE> will cause the Prolog engine to
find solutions of the open query.  The SP_term_refs that you sent with
the call to <CODE>SP_open_query()</CODE> will be assigned new values.
<CODE>SP_next_solution()</CODE> will return <CODE>SP_SUCCESS</CODE> for success,
<CODE>SP_FAILURE</CODE> for failure, <CODE>SP_ERROR</CODE> if an error condition
occurred.

</P>
<P>
<A NAME="IDX869"></A>

</P>

<PRE>
int SP_next_solution(SP_qid query)
</PRE>

<P>
You can terminate a query in two ways.  The function
<CODE>SP_cut_query()</CODE> will only take away the choices created since the
corresponding <CODE>SP_open_query()</CODE>.  The data created in the call are
still valid and could be passed to another call.  The function
<CODE>SP_close_query()</CODE> will restore the state to what it was before the
call to <CODE>SP_open_query()</CODE>.  If the call created data that you want
to keep, it must be converted to C data before calling
<CODE>SP_close_query()</CODE>.  Both functions return <CODE>SP_SUCCESS</CODE> for
success and <CODE>SP_ERROR</CODE> for invalid usage:

</P>
<P>
<A NAME="IDX870"></A>
<A NAME="IDX871"></A>

</P>

<PRE>
int SP_cut_query(SP_qid query)
</PRE>


<PRE>
int SP_close_query(SP_qid query)
</PRE>



<H3><A NAME="SEC137" HREF="sicstus_toc.html#TOC137">Calling Prolog Asynchronously</A></H3>
<P>
<A NAME="IDX872"></A>
<A NAME="IDX873"></A>
<A NAME="IDX874"></A>
<A NAME="IDX875"></A>
<A NAME="IDX876"></A>
<A NAME="IDX877"></A>

</P>
<P>
A Prolog execution may be interrupted by signals or similar
asynchronous events.  If you wish to call Prolog back from a signal
handler you cannot use <CODE>SP_query()</CODE> etc. directly.  The call to
Prolog has to be delayed until a time when the Prolog execution can
accept an interrupt.  The function <CODE>SP_event()</CODE> serves this
purpose, and installs the function <CODE>func</CODE> to be called from Prolog
when the execution can accept a callback.  Returns non-zero iff
installation succeeded.  <CODE>func</CODE> is called with <CODE>arg</CODE> as first
argument.

</P>
<P>
A queue of functions, with corresponding arguments, is maintained; that
is, if several calls to <CODE>SP_event()</CODE> occur before Prolog can a
accept an interrupt, the functions are queued and executed in turn at
the next possible opportunity. Note that the queuing facility is only
safe for signal handlers installed using <CODE>SP_signal()</CODE> (see below).

</P>
<P>
Depending on the value returned from <CODE>func</CODE>, the interrupted Prolog
execution will just continue (<CODE>SP_SUCCESS</CODE>) or backtrack
(<CODE>SP_FAILURE</CODE> or <CODE>SP_ERROR</CODE>). An exception raised by
<CODE>func</CODE> will be processed in the interrupted Prolog execution. In
case of fail or exception the event queue is flushed:
<A NAME="IDX878"></A>

</P>

<PRE>
int SP_event(int (*func)(), void *arg)
</PRE>

<P>
A signal handler having called <CODE>SP_event()</CODE> should call
<CODE>SP_continue()</CODE> as its last action, to ensure that the interrupt is
processed as early as possible:
<A NAME="IDX879"></A>

</P>

<PRE>
void SP_continue()
</PRE>

<P>
To install a function, <CODE>func</CODE>, as a handler for the signal
<CODE>sig</CODE>, call:
<A NAME="IDX880"></A>

</P>

<PRE>
void (*SP_signal (int sig, void (*func)()))()
</PRE>

<P>
<CODE>SP_signal()</CODE> will also, if permitted by the operating system, add
<CODE>sig</CODE> to a set of signals which are all blocked during the handling
of the event queue. Some operating systems require that:
<A NAME="IDX881"></A>

</P>

<PRE>
void (*SP_reinstall_signal (int sig, void (*func)()))()
</PRE>

<P>
be called from a signal handler to unblock or reinstall the handler.
This function should be called <EM>before</EM> <CODE>SP_continue()</CODE>.

</P>
<P>
The following piece of C code illustrates these facilities.  The
function <CODE>signal_init()</CODE> installs the function
<CODE>signal_handler()</CODE> as the primary signal handler for the
signals <CODE>USR1</CODE> and <CODE>USR2</CODE>.  That function invokes the
predicate <CODE>prolog_handler/1</CODE> as the actual signal handler, passing
the signal number as an argument to the predicate.

</P>

<PRE>
SP_pred_ref event_pred;

static int signal_event(signal_no)
     void *signal_no;
{
  SP_term_ref x=SP_new_term_ref();
  int rc;

  SP_put_integer(x, (int)signal_no);
  rc = SP_query(event_pred, x);
  if (rc == SP_ERROR &#38;&#38; SP_exception_term(x))
    SP_raise_exception(x);        /* Propagate any raised exception */
  return rc;
}

static void signal_handler(sig)
     int sig;
{
  SP_event(signal_event, (void *)sig);
  SP_reinstall_signal(sig, signal_handler);
  SP_continue();
}

void signal_init()
{
  event_pred = SP_predicate("prolog_handler",1,"");
  
  SP_signal(SIGUSR1, signal_handler);
  SP_signal(SIGUSR2, signal_handler);
}

</PRE>



<H3><A NAME="SEC138" HREF="sicstus_toc.html#TOC138">Exception Handling in C</A></H3>
<P>
<A NAME="IDX882"></A>

</P>
<P>
When an exception has been raised, the functions <CODE>SP_query()</CODE>,
<CODE>SP_query_cut_fail()</CODE> and <CODE>SP_next_solution()</CODE> return
<CODE>SP_ERROR</CODE>.  To access the <VAR>exception term</VAR> (the argument of
the call to <CODE>raise_exception/1</CODE>), which is asserted when the
exception is raised, the function <CODE>SP_exception_term()</CODE> is used.
As a side effect, the exception term is retracted, so if your code wants
to pass the exception term back to Prolog, it must use the
<CODE>SP_raise_exception()</CODE> function below.  If an exception term exists,
<CODE>SP_exception_term()</CODE> retracts it and stores it as the value of an
SP_term_ref which must exist prior to the call and returns nonzero.
Otherwise, it returns zero.

</P>

<P>
<A NAME="IDX883"></A>

</P>

<PRE>
int SP_exception_term(SP_term_ref t)
</PRE>

<P>
To raise an exception from a C function called from Prolog, 
just call <CODE>SP_raise_exception(t)</CODE> where <CODE>t</CODE>
is the SP_term_ref whose value is the exception term.  The glue code
will detect that an exception has been raised, any value returned from
the function will be ignored, and the exception will be passed back to
Prolog.

</P>
<P>
<A NAME="IDX884"></A>

</P>

<PRE>
void SP_raise_exception(SP_term_ref t)
</PRE>



<H2><A NAME="SEC139" HREF="sicstus_toc.html#TOC139">SICStus Streams</A></H2>

<P>
With the SICStus Prolog C interface, the user can define his/her own
streams as well as from C read or write on the predefined streams.  The
stream interface is modeled after Quintus Prolog release 2.  It
provides:

</P>

<UL>
<LI>

C functions to perform I/O on Prolog streams.
This way you can use the same stream from Prolog and C code.

<LI>

User defined streams. You can define your own Prolog streams in C.

<LI>

Bidirectional streams. A SICStus stream supports reading or 
writing or both.

<LI>

Hookable standard input/output/error streams.
</UL>


<UL>
<LI><A HREF="sicstus_11.html#SEC140">Prolog Streams</A>
<LI><A HREF="sicstus_11.html#SEC141">Defining a New Stream</A>
<LI><A HREF="sicstus_11.html#SEC145">Hookable Standard Streams</A>
</UL>



<H3><A NAME="SEC140" HREF="sicstus_toc.html#TOC140">Prolog Streams</A></H3>

<P>
From the Prolog level there is a unique number that identifies
a stream. This identifier can be converted from/to a Prolog stream:

</P>
<DL COMPACT>

<DT><CODE>stream_code(<VAR>+Stream</VAR>,<VAR>?StreamCode</VAR>)</CODE>
<DD>
<DT><CODE>stream_code(<VAR>?Stream</VAR>,<VAR>+StreamCode</VAR>)</CODE>
<DD>
<A NAME="IDX885"></A>
<VAR>StreamCode</VAR> is the C stream identifier (an integer) corresponding to
the Prolog stream <VAR>Stream</VAR>.  This predicate is only useful when
streams are passed between Prolog and C.  
Note that <CODE><VAR>StreamCode</VAR></CODE> no longer has any relation to the 
file descriptor.
</DL>

<P>
<A NAME="IDX886"></A>
The <CODE><VAR>StreamCode</VAR></CODE> is a Prolog integer representing a
<CODE>SP_stream *</CODE> pointer as described in (see section <A HREF="sicstus_11.html#SEC129">Creating Prolog Terms</A>, SP_put_address()).

</P>
<P>
To write on a Prolog stream from C, special versions of the 
most common standard C I/O functions are used:

</P>
<P>
<A NAME="IDX887"></A>
<A NAME="IDX888"></A>
<A NAME="IDX889"></A>
<A NAME="IDX890"></A>
<A NAME="IDX891"></A>
<A NAME="IDX892"></A>
<A NAME="IDX893"></A>
<A NAME="IDX894"></A>
<A NAME="IDX895"></A>
<A NAME="IDX896"></A>

</P>
<DL COMPACT>

<DT><CODE>int  SP_getc(void)</CODE>
<DD>
<DT><CODE>int  SP_fgetc(SP_stream *s)</CODE>
<DD>
<DT><CODE>void SP_putc(int c)</CODE>
<DD>
<DT><CODE>void SP_fputc(int c, SP_stream *s)</CODE>
<DD>
<DT><CODE>void SP_puts(char *string)</CODE>
<DD>
<DT><CODE>void SP_fputs(char *string, SP_stream *s)</CODE>
<DD>
<DT><CODE>int  SP_printf(char *format, ...)</CODE>
<DD>
<DT><CODE>int  SP_fprintf(SP_stream *s, char *format, ...)</CODE>
<DD>
<DT><CODE>int  SP_fflush(SP_stream *s)</CODE>
<DD>
<DT><CODE>int  SP_fclose(SP_stream *s)</CODE>
<DD>
</DL>

<P>
There are three predefined streams accessible from C:

</P>
<DL COMPACT>

<DT><CODE>SP_stdin</CODE>
<DD>
Standard input. Refers to the same stream as <CODE>user_input</CODE> in
Prolog. Which stream is referenced by <CODE>user_input</CODE> is controlled by the
flag <CODE>user_input</CODE> (see <CODE>prolog_flag/3</CODE>) .

<DT><CODE>SP_stdout</CODE>
<DD>
Standard output. Refers to the same stream as <CODE>user_output</CODE> in
Prolog. Which stream is referenced by <CODE>user_output</CODE> is controlled by
the flag <CODE>user_output</CODE> (see <CODE>prolog_flag/3</CODE>).

<DT><CODE>SP_stderr</CODE>
<DD>
Standard error. Refers to the same stream as <CODE>user_error</CODE> in
Prolog. Which stream is referenced by <CODE>user_error</CODE> is controlled by
the flag <CODE>user_error</CODE> (see <CODE>prolog_flag/3</CODE>).

<DT><CODE>SP_curin</CODE>
<DD>
Current input. It is initially set equal to <CODE>SP_stdin</CODE>.
It can be changed with the predicates <CODE>see/1</CODE> and <CODE>set_input/1</CODE>.

<DT><CODE>SP_curout</CODE>
<DD>
Current output. It is initially set equal to <CODE>SP_stdout</CODE>.
It can be changed with the predicates <CODE>tell/1</CODE> and <CODE>set_output/1</CODE>.
</DL>

<P>
Note that these variables are read only. They are set but never
read by the stream handling.

</P>



<H3><A NAME="SEC141" HREF="sicstus_toc.html#TOC141">Defining a New Stream</A></H3>

<P>
The following steps are required to define a new stream in C:

</P>

<UL>
<LI>

Define low level functions (character reading, writing etc).

<LI>

Initialize and open your stream.

<LI>

Allocate memory needed for your particular stream. 

<LI>

Initialize and install a Prolog stream with <CODE>SP_make_stream()</CODE>.

<LI>

Initialize additional fields.
Some streams may require additional changes to the fields in the 
<CODE>SP_stream</CODE> structure than the default values set by 
<CODE>SP_make_stream()</CODE>.
</UL>


<UL>
<LI><A HREF="sicstus_11.html#SEC142">Low Level I/O Functions</A>
<LI><A HREF="sicstus_11.html#SEC143">Installing a New Stream</A>
<LI><A HREF="sicstus_11.html#SEC144">The Internal Representation of a Prolog Stream</A>
</UL>



<H4><A NAME="SEC142" HREF="sicstus_toc.html#TOC142">Low Level I/O Functions</A></H4>

<P>
For each new stream the appropriate low level I/O functions have to be
defined.  Error handling, prompt handling and character counting is
handled in a layer above these functions.  They all operate on a user
defined private data structure pointed out by <CODE>user_handle</CODE> in
<CODE>SP_stream</CODE>.

</P>
<P>
User defined low level I/O functions may invoke Prolog code and use the
support functions described in the other sections of this chapter.

</P>
<DL COMPACT>

<DT><CODE>int <VAR>my_fgetc</VAR>(void *handle)</CODE>
<DD>
Should return the character read or -1 on end of file.

<DT><CODE>int <VAR>my_fputc</VAR>(char c, int handle)</CODE>
<DD>
Should write the character <CODE>c</CODE> and return the character written.

<DT><CODE>int <VAR>my_flush</VAR>(void *handle)</CODE>
<DD>
Flush the stream. Should return 0 on success, EOF on error.

<DT><CODE>int <VAR>my_eof</VAR>(void *handle)</CODE>
<DD>
Should return 1 on end of file, else 0.

<DT><CODE>void <VAR>my_clrerr</VAR>(void *handle)</CODE>
<DD>
Clear the error level.

<DT><CODE>int <VAR>my_close</VAR>(void *handle)</CODE>
<DD>
Close the stream. Should return zero.
</DL>



<H4><A NAME="SEC143" HREF="sicstus_toc.html#TOC143">Installing a New Stream</A></H4>

<P>
A new stream is made accessible to Prolog with the function 
<CODE>SP_make_stream()</CODE>.
<A NAME="IDX897"></A>

<PRE>
int SP_make_stream(
        void *handle,
        int (*sgetc)(),
        int (*sputc)(),
        int (*sflush)(),
        int (*seof)(), 
        void (*sclrerr)(),
        int (*sclose)(),
        SP_stream **stream)
</PRE>

<P>
The function will:

</P>

<UL>
<LI>

Allocate a <CODE>SP_stream</CODE> structure 
<LI>

Install your low level I/O functions. For those not supplied
default functions are installed. 
<LI>

Determine if the stream is for input or output or both from 
the functions supplied.
<LI>

Fill in fields in the <CODE>SP_stream</CODE> structure with default values
</UL>

<P>
The <CODE>handle</CODE> pointer will be supplied as the <CODE>handle</CODE>
argument in the calls to the low level functions.

</P>
<P>
A stream without a close function will be treated as not closable i.e.
<CODE>close/1</CODE> will not have any effect on it.

</P>



<H4><A NAME="SEC144" HREF="sicstus_toc.html#TOC144">Internal Representation</A></H4>

<P>
For most streams you don't have to know anything about the internal
representation but there may be occasions when you have to set some fields
manually or do some processing on all streams of a particular type.
SICStus Prolog maintain a circular list of stream objects of type <CODE>SP_stream</CODE>.

</P>
<DL COMPACT>

<DT><CODE>SP_stream *backward;</CODE>
<DD>
<DT><CODE>SP_stream *forward;</CODE>
<DD>
Used for linking streams together. The insertion is done
by <CODE>SP_make_stream()</CODE> and the deletion is done from the Prolog
predicate <CODE>close/1</CODE>.

<DT><CODE>char *filename;</CODE>
<DD>
This field is set to the empty string, "", by <CODE>SP_make_stream()</CODE>.
May be set to a suitable string, provided the string will not be
overwritten until the stream is closed.

<DT><CODE>unsigned long mode;</CODE>
<DD>
A bit vector that contains information about the access modes supported,
if the stream is a TTY stream etc.  It is not available to the user 
but the TTY property can be set by the function:
<A NAME="IDX898"></A>

<DL COMPACT>

<DT><CODE>void SP_set_tty(SP_stream *s)</CODE>
<DD>
</DL>

<DT><CODE>int fd;</CODE>
<DD>
The I/O descriptor if the stream is associated with a file, socket
etc. Otherwise a negative number.

<DT><CODE>void *user_handle;</CODE>
<DD>
This is the pointer to the user supplied private data for the stream.
In the case of SICStus Prolog predefined file streams the
<CODE>user_handle</CODE> could be a pointer to the standard I/O <CODE>FILE</CODE>.
</DL>

<P>
There is no standard way to tell if a stream is user defined.  You have
to save pointers to the streams created or check if one of the stream
functions installed is user defined, i.e:

</P>

<PRE>
int is_my_stream(SP_stream *s)
{
  return (s-&#62;sclose == my_close);
}
</PRE>



<H3><A NAME="SEC145" HREF="sicstus_toc.html#TOC145">Hookable Standard Streams</A></H3>

<P>
As of SICStus release 3.7, the standard I/O streams (input, output, and
error) are hookable, i.e. the streams can be redefined by the
user.

</P>
<DL COMPACT>

<DT><CODE>SP_UserStreamHook *SP_set_user_stream_hook(SP_UserStreamHook *hook)</CODE>
<DD>
<A NAME="IDX899"></A>
Sets the user-stream hook to <CODE>hook</CODE>.

<DT><CODE>SP_UserStreamPostHook *SP_set_user_stream_post_hook(SP_UserStreamPostHook *hook)</CODE>
<DD>
<A NAME="IDX900"></A>
Sets the user-stream post-hook to <CODE>hook</CODE>.
</DL>


<UL>
<LI><A HREF="sicstus_11.html#SEC146">Writing User-stream Hooks</A>
<LI><A HREF="sicstus_11.html#SEC147">Writing User-stream Post-hooks</A>
<LI><A HREF="sicstus_11.html#SEC148">User-stream Hook Example</A>
</UL>



<H4><A NAME="SEC146" HREF="sicstus_toc.html#TOC146">Writing User-stream Hooks</A></H4>

<P>
The user-stream hook is, if defined, called during
<CODE>SP_initialize()</CODE>. <CODE>SP_set_user_stream_hook()</CODE> and
<CODE>SP_set_user_stream_post_hook()</CODE> must therefore be called before
<CODE>SP_initialize()</CODE> is called. It has the following prototype:

</P>

<PRE>
SP_stream *user_stream_hook(int which)
</PRE>

<P>
If the hook is not defined, SICStus will attempt to open the standard
TTY/console versions of these streams. If they are unavailable (such as
for windowed applications under Windows), the result is undefined.

</P>
<P>
It is called three times, one for each stream. The <CODE>which</CODE> argument
indicates which stream it is called for. The value of <CODE>which</CODE>
is one of:

</P>
<DL COMPACT>

<DT><CODE>SP_STREAMHOOK_STDIN</CODE>
<DD>
Create stream for standard input.
<DT><CODE>SP_STREAMHOOK_STDOUT</CODE>
<DD>
Create stream for standard output.
<DT><CODE>SP_STREAMHOOK_STDERR</CODE>
<DD>
Create stream for standard error.
</DL>

<P>
The hook should return a standard SICStus I/O stream, as described in
section <A HREF="sicstus_11.html#SEC141">Defining a New Stream</A>.

</P>


<H4><A NAME="SEC147" HREF="sicstus_toc.html#TOC147">Writing User-stream Post-hooks</A></H4>

<P>
The user-stream post-hook is, if defined, called after all the streams
have been defined, once for each of the three standard streams. It has a
slightly different prototype:

</P>

<PRE>
void user_stream_post_hook(int which, SP_stream *str)
</PRE>

<P>
where <CODE>str</CODE> is a pointer to the corresponding <CODE>SP_stream</CODE>
structure. There are no requirements as to what this hook must do; the
default behavior is to do nothing at all.

</P>
<P>
The post-hook is intended to be used to do things which may require that
all streams have been created.

</P>


<H4><A NAME="SEC148" HREF="sicstus_toc.html#TOC148">User-stream Hook Example</A></H4>

<P>
This section contains an example of how to create and install a set of
user-streams.

</P>
<P>
The hook is set by calling <CODE>SP_set_user_stream_hook()</CODE> in the main
program like this:

</P>

<PRE>
SP_set_user_stream_hook((SP_UserStreamHook *)user_strhook);
</PRE>

<P>
<STRONG>Remember: </STRONG> <CODE>SP_set_user_stream_hook()</CODE> and
<CODE>SP_set_user_stream_post_hook()</CODE> must be called <EM>before</EM>
<CODE>SP_initialize()</CODE>. 

</P>
<P>
The hook <CODE>user_strhook()</CODE> is defined like this:

</P>

<PRE>
SP_stream *user_strhook(int std_fd)
{
  SP_stream *s;

  SP_make_stream(NULL, my_getc, my_putc, my_flush, my_eof, my_clrerr, NULL, &#38;s);

  return s;
}
</PRE>

<P>
See section <A HREF="sicstus_11.html#SEC143">Installing a New Stream</A> for a description on the parameters to
<CODE>SP_make_stream()</CODE>.

</P>



<H2><A NAME="SEC149" HREF="sicstus_toc.html#TOC149">Muse Support Functions</A></H2>

<P>
Dynamic loading of foreign language functions must be done when the
system is adjusted to one worker.  If you use dynamic linking, it is
not recommended to use C-global or static local variables.  It is
platform dependent if the variables are shared among workers or
private.  If static linking is used, the variables become private.

</P>
<P>
Some useful Muse C-functions:

</P>
<DL COMPACT>

<DT><CODE>int muse_max_workers(void)</CODE>
<DD>
<A NAME="IDX901"></A>
Returns the maximum number of workers.  Also available as a read-only
Muse flag.

<DT><CODE>int muse_num_workers(void)</CODE>
<DD>
<A NAME="IDX902"></A>
Returns the actual number of allocated workers.  Also available as
a writable Muse flag.

<DT><CODE>int muse_worker_id(void)</CODE>
<DD>
<A NAME="IDX903"></A>
Returns the worker identity as a non-negative integer.
Also available as a read-only Muse flag.

<DT><CODE>void muse_init_lock(int *lock)</CODE>
<DD>
<DT><CODE>void muse_lock(int *lock)</CODE>
<DD>
<DT><CODE>void muse_un_lock(int *lock)</CODE>
<DD>
<A NAME="IDX904"></A>
<A NAME="IDX905"></A>
<A NAME="IDX906"></A>
The <CODE>muse_lock</CODE> and <CODE>muse_un_lock</CODE> functions is used to
implement mutual exclusion regions.  <STRONG>Note:</STRONG> the actual locks in
Muse may not be <CODE>int</CODE> but they are of equal or smaller size.  A
Muse lock must be initialized with <CODE>muse_init_lock()</CODE> before it can
be used.
</DL>

<P>
In Muse, dynamic memory allocation is performed in shared memory.  To
allocate worker private memory, it is recommended (as
<CODE>worker_counters</CODE> in section <A HREF="sicstus_13.html#SEC187">Muse FLI Example</A>) to
allocate an array of size <CODE>muse_max_workers()</CODE> that is indexed by
the worker id (<CODE>muse_worker_id()</CODE>).

</P>



<H2><A NAME="SEC150" HREF="sicstus_toc.html#TOC150">Hooks</A></H2>
<P>
<A NAME="IDX907"></A>
<A NAME="IDX908"></A>
<A NAME="IDX909"></A>
<A NAME="IDX910"></A>

</P>
<P>
The user may define functions to be called at certain occasions by the
Prolog system. This is accomplished by passing the functions as
arguments to the following set-hook-functions. The functions can be
removed by passing a <CODE>NULL</CODE>.

</P>
<P>
<A NAME="IDX911"></A>
<A NAME="IDX912"></A>
<A NAME="IDX913"></A>

</P>
<DL COMPACT>

<DT><CODE>typedef int (SP_ReadHookProc) (int fd)</CODE>
<DD>
<DT><CODE>SP_ReadHookProc SP_set_read_hook (SP_ReadHookProc *)</CODE>
<DD>
The installed function is called before reading a character from
<CODE>fd</CODE> provided it is associated with a terminal device. This
function shall return nonzero when there is input available at
<CODE>fd</CODE>. It is called repeatedly until it returns nonzero.  Not
available in Muse.

<DT><CODE>typedef void (SP_VoidFun) (void)</CODE>
<DD>
<DT><CODE>SP_VoidFun * SP_set_reinit_hook (SP_VoidFun *)</CODE>
<DD>
The installed function is called upon abort and reinitialization. The
call is made after SICStus Prolog's signal handler installation but
before the built-in predicates <CODE>version/0</CODE> and
<CODE>initialization/0</CODE> are called.  Calling Prolog from functions
invoked through this hook is not supported. (This hook is not available
in Runtime Systems.)

<DT><CODE>typedef void (SP_VoidFun) (void);</CODE>
<DD>
<DT><CODE>SP_VoidFun * SP_set_interrupt_hook (SP_VoidFun *)</CODE>
<DD>
The installed function is called on occasions like expansion of stacks,
garbage collection and printouts, in order to yield control to special
consoles etc. for interrupt checking.  Calling Prolog from functions
invoked through this hook is not supported. (This hook is not available
in Runtime Systems.)

</DL>



<H2><A NAME="SEC151" HREF="sicstus_toc.html#TOC151">Runtime Systems</A></H2>

<P>
<A NAME="IDX914"></A>
<A NAME="IDX915"></A>
It is possible to mix C and Prolog code to create stand-alone
applications, <EM>Runtime Systems</EM>. 
In a Runtime System there are some differences in the behavior of 
the Prolog engine and many built-in predicates are omitted or have
limited functionality:

</P>

<UL>

<LI>

No top level

<LI>

No debugger

<LI>

No compiler

<LI>

No progress or error messages while loading files

<LI>

No profiling

<LI>

No OR-parallelism

<LI>

No redefinition warnings

<LI>

No signal handling except as installed by <CODE>SP_signal()</CODE>

<LI>

Uncaught exceptions are returned to C

<LI>

The predicates <CODE>halt/0</CODE>, <CODE>abort/0</CODE> and <CODE>reinitialise/0</CODE> 
will return to C

<LI>

You have to supply a main program and explicitly initialize the Prolog
engine with <CODE>SP_initialize()</CODE>.
</UL>


<UL>
<LI><A HREF="sicstus_11.html#SEC152">Initializing the Prolog Engine</A>
<LI><A HREF="sicstus_11.html#SEC153">Loading Prolog Code</A>
<LI><A HREF="sicstus_11.html#SEC154">Creating the Executable</A>
<LI><A HREF="sicstus_11.html#SEC155">Running the Executable</A>
</UL>



<H3><A NAME="SEC152" HREF="sicstus_toc.html#TOC152">Initializing the Prolog Engine</A></H3>

<P>
You must call <CODE>SP_initialize()</CODE> before calling any other interface
function.  The function will allocate data areas used by Prolog,
initialize command line arguments so that they can be accessed by the
<CODE>argv</CODE> Prolog flag, and load the Runtime Library:

</P>
<P>
<A NAME="IDX916"></A>

</P>

<PRE>
int SP_initialize(int argc, char **argv, char *boot_path)
</PRE>

<P>
<CODE>boot_path</CODE> should be the name of a directory, equivalent to
<TT>`$SP_PATH/bin'</TT>.  If <CODE>boot_path</CODE> is <CODE>NULL</CODE>,
<CODE>SP_initialize()</CODE> will look up the value of the environment
variable <CODE>SP_PATH</CODE> and look for the file
<TT>`$SP_PATH/bin/sprt.sav'</TT> which contains the Runtime Library.

</P>
<P>
It returns <CODE>SP_SUCCESS</CODE> if initialization was successful, and
<CODE>SP_ERROR</CODE> otherwise.

</P>
<P>
Optionally, you may also call <CODE>SP_force_interactive()</CODE> before
calling <CODE>SP_initialize()</CODE>.  This will force the I/O built-in
predicates to treat the standard input stream as a terminal, even if it
does not appear to be a terminal.  Same as the <SAMP>`-i'</SAMP> option in
Development Systems. (see section <A HREF="sicstus_4.html#SEC11">Getting Started</A>).

</P>
<P>
<A NAME="IDX917"></A>

</P>

<PRE>
void SP_force_interactive(void)
</PRE>

<P>
Optionally, you may also call <CODE>SP_set_memalloc_hooks()</CODE> before
calling <CODE>SP_initialize()</CODE>.  This will define the bottom layer of
Prolog's memory manager, in case your application has special
requirements.

</P>
<P>
<A NAME="IDX918"></A>

</P>

<PRE>
typedef void *(SP_AllocHook)(unsigned int size, 
                             unsigned int align, 
                             unsigned int *actual_sizep);
typedef void *(SP_ReAllocHook)(void *ptr, 
                               unsigned int oldsize, 
                               unsigned int newsize, 
                               unsigned int align, 
                               unsigned int *actual_sizep);
typedef int (SP_FreeHook)(void *ptr, 
                          unsigned int size);

void SP_set_memalloc_hooks(int usage,
                           SP_AllocHook *init_alloc_hook,
                           SP_AllocHook *alloc_hook,
                           SP_ReAllocHook *realloc_hook,
                           SP_FreeHook *free_hook)
</PRE>

<P>
The effect of <CODE>SP_set_memalloc_hooks</CODE> is controlled by the value of
<CODE>usage</CODE>, which should be one of:

</P>
<DL COMPACT>

<DT><CODE>MM_USE_MALLOC</CODE>
<DD>
The bottom layer will be based on <CODE>malloc()</CODE>/<CODE>free()</CODE>.
The other arguments are ignored.
Same as the <SAMP>`-m'</SAMP> option in Development Systems. (see section <A HREF="sicstus_4.html#SEC11">Getting Started</A>).

<DT><CODE>MM_USE_SBRK</CODE>
<DD>
The bottom layer will be based on <CODE>sbrk()</CODE>.
The default for UNIX; not available for Windows.
The other arguments are ignored.

<DT><CODE>MM_USE_SPARSE</CODE>
<DD>
The bottom layer will be based on <CODE>VirtualAlloc()</CODE>/<CODE>VirtualFree()</CODE>.
The default for Windows; not available for UNIX.
The other arguments are ignored.

<DT><CODE>MM_USE_OTHER</CODE>
<DD>
The bottom layer will be based on the other arguments.
Their meaning is explained below.
</DL>

<P>
In the latter case, the other arguments should be functions as specified
below:

</P>
<DL COMPACT>

<DT><CODE>alloc_hook</CODE>
<DD>
must allocate and return a pointer to a piece of memory that has at
least <CODE>size</CODE> bytes aligned at <CODE>align</CODE> in it.  <CODE>align</CODE> is
guaranteed to be a power of 2.  The actual size of the piece of memory
should be returned in <CODE>*actual_sizep</CODE>.  Should return <CODE>NULL</CODE>
if it cannot allocate any more memory.

<DT><CODE>init_alloc_hook</CODE>
<DD>
is a special case of <CODE>alloc_hook</CODE>.  It will be called initially
whereas <CODE>alloc_hook</CODE> will be called subsequently. It can do
whatever initialization that this layer of memory management wants to do.

<DT><CODE>realloc_hook</CODE>
<DD>
is called with a piece of memory to be resized and possibly moved.
<CODE>ptr</CODE> is the pointer, <CODE>oldsize</CODE> its current size.  The
function must allocate and return a pointer to a piece of memory that
has at least <CODE>newsize</CODE> bytes aligned at <CODE>align</CODE> in it, and
that has the same contents as the old block up to the lesser of
<CODE>oldsize</CODE> and <CODE>newsize</CODE>.  <CODE>align</CODE> is guaranteed to be a
power of 2.  The actual size of the piece of memory should be returned
in <CODE>*actual_sizep</CODE>.  Should return <CODE>NULL</CODE> if it cannot
allocate any more memory, or if it cannot reclaim the old block in a
meaningful way.  In that case, Prolog will use the other functions.

<DT><CODE>free_hook</CODE>
<DD>
is called with a pointer to the piece of memory to be freed and its
size.  Should return non-zero iff the function was able to free this
piece of memory.  Otherwise, Prolog will keep using the memory as if it
were not freed.
</DL>

<P>
The default bottom layers look at the environment variables
<CODE>PROLOGINITSIZE</CODE>, <CODE>PROLOGINCSIZE</CODE>, <CODE>PROLOGKEEPSIZE</CODE> and
<CODE>PROLOGMAXSIZE</CODE>.  They are useful to customize the default memory
manager.  If users redefine the bottom layer, they can choose to ignore
these environment variables.  See section <A HREF="sicstus_4.html#SEC13">Environment Variables</A>.

</P>


<H3><A NAME="SEC153" HREF="sicstus_toc.html#TOC153">Loading Prolog Code</A></H3>

<P>
You can load your Prolog code compiled to Prolog object files into the
system with the call <CODE>SP_load()</CODE>.  This is the C equivalent of the
Prolog predicate <CODE>load/1</CODE>:

</P>
<P>
<A NAME="IDX919"></A>

</P>

<PRE>
int SP_load(char *filename)
</PRE>

<P>
Alternatively, you can restore a saved state with the call
<CODE>SP_restore()</CODE>, which is the C equivalent of the Prolog predicate
<CODE>restore/1</CODE>:

</P>
<P>
<A NAME="IDX920"></A>

</P>

<PRE>
int SP_restore(char *filename)
</PRE>

<P>
<CODE>SP_load()</CODE> and <CODE>SP_restore()</CODE> return <CODE>SP_SUCCESS</CODE>
for success or <CODE>SP_ERROR</CODE> if an error condition occurred.

</P>
<P>
Prolog error handling is mostly done by raising and catching exceptions.
However, some <EM>faults</EM> are of a nature such that when they occur,
the internal program state may be corrupted, and it is not safe to
merely raise an exception.  Memory allocation failures are examples of
faults.  In Runtime Systems, the following C macro provides an
environment for handling faults:

</P>
<P>
<A NAME="IDX921"></A>

</P>

<PRE>
int SP_on_fault(<VAR>Stmt</VAR>, <VAR>Message</VAR>, <VAR>Cleanup</VAR>)
</PRE>

<P>
which should occur in the scope of a <CODE>char *<VAR>Message</VAR></CODE>
declaration.  <VAR>Stmt</VAR> is run, and if a fault occurs, <VAR>Stmt</VAR> is
aborted, <VAR>Message</VAR> gets assigned a value explaining the fault, the
Prolog internal state is cleaned, all queries and SP_term_refs become
invalid, and <VAR>Cleanup</VAR> run.  If <VAR>Stmt</VAR> terminates normally,
<VAR>Message</VAR> is left unchanged.  For example, a "fault-proof"
Runtime System could have the structure:

</P>

<PRE>
int main(int argc, char **argv)
{
  char *message;

  SP_initialize(argc, argv, "/usr/local/lib/sicstus37/bin");
loop:
  SP_on_fault(main_loop(), message, 
              {printf("ERROR: %s\n",message); goto loop;});
  exit(0);
}

main_loop()
{...}
</PRE>

<P>
Faults that occur outside the scope of <CODE>SP_on_fault()</CODE> cause the
Runtime System to halt with an error message.

</P>
<P>
The following function can be used to raise a fault.  For example, it
can be used in a signal handler for SIGSEGV to prevent the
program from dumping core in the event of a segmentation violation
(Runtime Systems have no predefined signal handling):

</P>
<P>
<A NAME="IDX922"></A>

</P>

<PRE>
void SP_raise_fault(char *message)
</PRE>



<H3><A NAME="SEC154" HREF="sicstus_toc.html#TOC154">Creating the Executable</A></H3>

<P>
<A NAME="IDX923"></A>
<A NAME="IDX924"></A>
This describes how to create a Runtime System with a set of statically
linked foreign resources.  For the SICStus library modules containing
C-code, such linked foreign resources already exist in
<TT>`$SP_PATH/library'</TT>.  For user foreign code, you must first create
linked foreign resources for them.

</P>
<P>
A utility command is provided for the generation of such systems.  This
command, located in <TT>`$SP_PATH/bin'</TT>, is configurable as described in
the Release Notes.

</P>
<DL COMPACT>

<DT><CODE>spmkrs [--help] [-S] <VAR>Executable</VAR> [+c <VAR>SourceFiles</VAR>] [+o <VAR>ObjectFiles</VAR>] [+l <VAR>Libraries</VAR>] [+r <VAR>Resources</VAR>]</CODE>
<DD>
<A NAME="IDX925"></A>
Generates a Runtime System with any foreign resources mentioned in
<VAR>Resources</VAR> statically linked.  The resources must be created with
the <CODE>+static</CODE> option.  Note however that a statically linked
dynamic resource is not, in general, statically linked from the
operating system's point of view. It must therefore be accessible at
runtime as well as at build-time.

Compiles <VAR>SourceFiles</VAR> and links the resulting object files with
<VAR>ObjectFiles</VAR>, <VAR>Libraries</VAR>, and the foreign resources to produce
<VAR>Executable</VAR>.  The main program of the Runtime System must be
provided by <VAR>SourceFiles</VAR> or <VAR>ObjectFiles</VAR>.

The option <CODE>-S</CODE> causes the SICStus Runtime Kernel to be statically
linked into the executable, thus eliminating the need to load these at
runtime. This produces a significantly larger executable. Observe that
this option has no effect on the resources and libraries spefified after
by <CODE>+r</CODE> and <CODE>+l</CODE>.  <STRONG>Note:</STRONG> This option is not available
on Win32.

The option <CODE>--help</CODE> prints out usage info.
</DL>

<P>
For example, to compile and link a Runtime System <TT>`ex_rt'</TT> with the
linked foreign resource <CODE>ex</CODE> statically linked, first create the
linked foreign resource (see section <A HREF="sicstus_11.html#SEC126">Creating the Linked Foreign Resource</A>)
using the <CODE>+static</CODE> option.  Assuming the Runtime System's main
program is in the file <TT>`main.c'</TT>, then type:

</P>

<PRE>
% spmkrs ex_rt +c main.c +r ex
</PRE>

<P>
If you prefer not to use the utility commands, an alternative method is
described in section <A HREF="sicstus_41.html#SEC363">Glue Code Generator</A>.

</P>


<H3><A NAME="SEC155" HREF="sicstus_toc.html#TOC155">Running the Executable</A></H3>

<P>
To execute a Runtime System on a target machine you need:

</P>
<DL COMPACT>

<DT>the executable
<DD>
This is your application program created as described above.

<DT>the runtime kernel
<DD>
This is a shared object or a DLL, usually
<TT>`$SP_PATH/../libsprt37.so'</TT> under UNIX, or
<TT>`%SP_PATH%\..\sprt37.dll'</TT> under Windows.

<DT>the runtime library
<DD>
The saved state <TT>`$SP_PATH/bin/sprt.sav'</TT>
contains the built-in predicates written in Prolog.
It is restored into the program at runtime by the function
<CODE>SP_initialize()</CODE>.

<DT>your Prolog code
<DD>
Either as a saved state, object code (<SAMP>`.ql'</SAMP> files), or source code
(<SAMP>`.pl'</SAMP> files).  They must be explicitly loaded by the program at runtime.

<DT>your linked foreign resources
<DD>
Any linked foreign resources which are not statically linked to the
executable, including any linked foreign resources for library modules
which are located in <TT>`$SP_PATH/library'</TT>.

</DL>

<P>
<A NAME="IDX926"></A>
Also, on most UNIX platforms, it is necessary to set an environment
variable, typically <CODE>LD_LIBRARY_PATH</CODE>, for shared objects to be
found at runtime.  See section <A HREF="sicstus_4.html#SEC12">Finding Libraries at Runtime</A>.

</P>



<H2><A NAME="SEC156" HREF="sicstus_toc.html#TOC156">Development Systems</A></H2>

<P>
<A NAME="IDX927"></A>
This describes how to create a Development System with a set of linked
foreign resources.  For the SICStus library modules containing C-code,
such linked foreign resources already exist in <TT>`$SP_PATH/library'</TT>.
For user foreign code, you must first create linked foreign resources
for them.

</P>
<P>
A utility command is provided for the generation of such systems.  This
command, located in <TT>`$SP_PATH/bin'</TT>, is configurable as described in
the Release Notes.

</P>
<DL COMPACT>

<DT><CODE>spmkds [--help] [-S] <VAR>Executable</VAR> [<VAR>Resources</VAR>]</CODE>
<DD>
<A NAME="IDX928"></A>
Generates a Development System with any foreign resources mentioned in
<VAR>Resources</VAR> statically linked. The resources must be created with
the <CODE>+static</CODE> option.  Note however that a statically linked
dynamic resource is not, in general, statically linked from the
operating system's point of view. It must therefore be accessible at
runtime as well as at build-time.

The option <CODE>-S</CODE> causes the SICStus Runtime Kernel and development
system extensions to be statically linked into the executable, thus
eliminating the need to load these at runtime. This produces a
significantly larger executable. Observe that this option has no effect
on the resources. <STRONG>Note:</STRONG> This option is not available on Win32.

The option <CODE>--help</CODE> prints out usage info.
</DL>

<P>
For example, to compile and link a Development System <TT>`ex_stat'</TT>
with the linked foreign resource <CODE>ex</CODE> statically linked, first
create the linked foreign resource (see section <A HREF="sicstus_11.html#SEC126">Creating the Linked Foreign Resource</A>) using the <CODE>+static</CODE> option.  Then type:

</P>

<PRE>
% spmkds ex_stat ex
</PRE>

<P>
The executable can then be run as:

</P>

<PRE>
% sicstus -base ./ex_stat
</PRE>

<P>
<A NAME="IDX929"></A>
and optionally more command-line options as for the standard
<CODE>sicstus</CODE> command.

</P>
<P>
The generated executable will run the usual Prolog top level, and the
full develoment environment is available as usual.

</P>


<H2><A NAME="SEC157" HREF="sicstus_toc.html#TOC157">Examples</A></H2>


<UL>
<LI><A HREF="sicstus_11.html#SEC158">Train</A>
<LI><A HREF="sicstus_11.html#SEC159">I/O on Lists of Character Codes</A>
<LI><A HREF="sicstus_11.html#SEC160">Exceptions from C</A>
<LI><A HREF="sicstus_11.html#SEC161">Stream Example</A>
</UL>



<H3><A NAME="SEC158" HREF="sicstus_toc.html#TOC158">Train Example (connections)</A></H3>

<P>
This is an example of how to create a Runtime System.  The Prolog
program <TT>`train.pl'</TT> will display a route from one train station to
another.  The C program <TT>`train.c'</TT> calls the Prolog code and writes
out all the routes found between two stations:

</P>


<PRE>
% train.pl

connected(From, To, Path) :-
        connected(From, To, Path, [From]).

connected(To, To, [To], _).
connected(From, To, [From|Path], Visited) :-
        (   connection(From, Via)
        ;   connection(Via, From)
        ),
        not_visited(Visited, Via),
        connected(Via, To, Path, [Via|Visited]).

connection('Stockholm', 'Katrineholm').
connection('Stockholm', 'Vasteras').
connection('Stockholm', 'Uppsala').
connection('Uppsala', 'Vasteras').
connection('Katrineholm', 'Hallsberg').
connection('Katrineholm', 'Linkoping').
connection('Hallsberg', 'Kumla').
connection('Hallsberg', 'Goteborg').
connection('Orebro', 'Vasteras').
connection('Orebro', 'Kumla').

not_visited([], _).
not_visited([X|Visited], Y) :- X \== Y, not_visited(Visited, Y).
</PRE>


<PRE>
/* train.c */

#include &#60;stdio.h&#62;
#include &#60;sicstus/sicstus.h&#62;

void write_path(SP_term_ref path)
{
  char *text = NULL;
  SP_term_ref 
    tail = SP_new_term_ref(),
    via = SP_new_term_ref();

  SP_put_term(tail,path);

  while (SP_get_list(tail,via,tail))
  {
    if (text) 
      printf(" -&#62; ");

    SP_get_string(via, &#38;text);
    printf("%s",text);
  }
  printf("\n");  
}

int main(int argc, char **argv)
{
  int rval;
  SP_pred_ref pred;
  SP_qid goal;
  SP_term_ref from, to, path;
  
  /* Initialize Prolog engine. This call looks up SP_PATH in order to
   * find the Runtime Library.  */
  if (SP_FAILURE == SP_initialize(argc, argv, NULL))
    {
      fprintf(stderr, "SP_initialize failed: %s\n", SP_error_message(SP_errno));
      exit(1);
    }

  rval = SP_restore("train.sav");

  if (rval == SP_ERROR || rval == SP_FAILURE)
    {
      fprintf(stderr, "Could not restore \"train.sav\".\n");
      exit(1);
    }

  /* Look up connected/3. */
  if (!(pred = SP_predicate("connected",3,"")))
    {
      fprintf(stderr, "Could not find connected/3.\n");
      exit(1);
    }

  /* Create the three arguments to connected/3. */
  SP_put_string(from = SP_new_term_ref(), "Stockholm");
  SP_put_string(to = SP_new_term_ref(), "Orebro");
  SP_put_variable(path = SP_new_term_ref());

  /* Open the query. In a development system, the query would look like:
   * 
   * | ?- connected('Stockholm','Orebro',X).
   */
  if (!(goal = SP_open_query(pred,from,to,path)))
    {
      fprintf(stderr, "Failed to open query.\n");
      exit(1);
    }

  /* 
   * Loop through all the solutions.
   */
  while (SP_next_solution(goal))
    {
      printf("Path: ");
      write_path(path);
    }
  
  SP_close_query(goal);

  exit(0);
}
</PRE>

<P>
Create the saved-state containing the Prolog code:

<PRE>
% sicstus
SICStus 3.7: Tue Jun 02 12:29:02 MET DST 1998
| ?- compile(train).
{compiling /home/jojo/sicstus/examples/train.pl...}
{/home/jojo/sicstus/examples/train.pl compiled, 40 msec 2848 bytes}

yes
| ?- save_program('train.sav').
{SICStus state saved in /home/jojo/sicstus/examples/train.sav}

yes
| ?- halt.
</PRE>

<P>
Create the executable using <CODE>spmkrs</CODE>:

</P>

<PRE>
% spmkrs train +c train.c
SICStus 3.7: Tue Jun 02 12:29:02 MET DST 1998
{/var/tmp/aaa0ch9LZ.c generated, 0 msec}

yes
</PRE>

<P>
And finally, run the executable:

</P>

<PRE>
% ./train
Path: Stockholm -&#62; Katrineholm -&#62; Hallsberg -&#62; Kumla -&#62; Orebro
Path: Stockholm -&#62; Vasteras -&#62; Orebro
Path: Stockholm -&#62; Uppsala -&#62; Vasteras -&#62; Orebro
</PRE>



<H3><A NAME="SEC159" HREF="sicstus_toc.html#TOC159">I/O on Lists of Character Codes</A></H3>

<P>
This example is taken from the SICStus Prolog library (simplified, but
operational). A stream for writing is opened where the written
characters are placed in a buffer. When the stream is closed a list of
character codes is made from the contents of the buffer. The example
illustrates the use of user definable streams.

</P>
<P>
The <CODE>open_buf_stream()</CODE> function opens a stream where the
characters are put in a buffer. The stream is closed by
<CODE>stream_to_chars()</CODE> which returns the list constructed on the
heap.

</P>
<P>
The Prolog code (simplified):

</P>

<PRE>
foreign(open_buf_stream, '$open_buf_stream'(-address('SP_stream'))).
foreign(stream_to_chars, '$stream_to_chars'(+address('SP_stream'),
                                            -term)).

foreign_resource(example, [open_buf_stream,stream_to_chars]).

:- load_foreign_resource(example).

%% with_output_to_chars(+Goal, -Chars)
%% runs Goal with current_output set to a list of characters

with_output_to_chars(Goal, Chars) :-
        '$open_buf_stream'(StreamCode),
        stream_code(Stream, StreamCode),
        current_output(CurrOut),
        set_output(Stream),
        call_and_reset(Goal, Stream, CurrOut, StreamCode, Chars).

call_and_reset(Goal, Stream, CurrOut, StreamCode, Chars) :-
        call(Goal), !,
        put(0),
        '$stream_to_chars'(StreamCode, Chars),
        reset_stream(Stream, CurrOut).
call_and_reset(_, Stream, CurrOut, _, _) :-
        reset_stream(Stream, CurrOut).

reset_stream(Stream, CurrOut) :-
        set_output(CurrOut),
        close(Stream).
</PRE>

<P>
The C code:

</P>

<PRE>
#include &#60;sicstus/sicstus.h&#62;

struct open_chars {
  char *chars;       /* character buffer */
  int index;         /* current insertion point */
  int size;
};

#define INIT_BUFSIZE 512

static int lputc(c, buf)
     int c;
     struct open_chars *buf;
{
  if (buf-&#62;index == buf-&#62;size)  /* grow buffer if necessary */
    {
      buf-&#62;size *= 2;
      buf-&#62;chars = (char *)realloc(buf-&#62;chars, buf-&#62;size);
    }
  return (buf-&#62;chars[buf-&#62;index++] = c);   
}

static int lwclose(buf)
     struct open_chars *buf;
{
  free(buf-&#62;chars);
  free(buf);
  return 0;
}

void open_buf_stream(streamp)
     SP_stream **streamp;
{
  struct open_chars *buf;

  /* Allocate buffer, create stream &#38; return stream code */

  buf = (struct open_chars *)malloc(sizeof(struct open_chars));
  SP_make_stream(buf, NULL, lputc, NULL, NULL, NULL, lwclose, 
                 streamp);
  
  buf-&#62;chars = (char *)malloc(INIT_BUFSIZE);
  buf-&#62;size = INIT_BUFSIZE;
  buf-&#62;index = 0;
}

void stream_to_chars(streamp, head)
     SP_stream *streamp;
     SP_term_ref head;
{
  SP_term_ref tail = SP_new_term_ref();
  struct open_chars *buf = (struct open_chars *)streamp-&#62;user_handle;

  /* Make a list of character codes out of the buffer */

  SP_put_string(tail, "[]");  
  SP_put_list_chars(head, tail, buf-&#62;chars);
}
</PRE>



<H3><A NAME="SEC160" HREF="sicstus_toc.html#TOC160">Exceptions from C</A></H3>

<P>
Consider, for example, a function which returns the square root of its
argument after checking that the argument is valid.  If the argument is 
invalid, the function should raise an exception instead.

</P>

<PRE>
/* math.c */

#include &#60;math.h&#62;
#include &#60;stdio.h&#62;
#include &#60;sicstus/sicstus.h&#62;

double sqrt_check(d)
     double d;
{
  if (d &#60; 0.0)
    {    /* build a domain_error/4 exception term */
      SP_term_ref culprit=SP_new_term_ref();
      SP_term_ref argno=SP_new_term_ref();
      SP_term_ref expdomain=SP_new_term_ref();
      SP_term_ref t1=SP_new_term_ref();

      SP_put_float(culprit, d);
      SP_put_integer(argno, 1);
      SP_put_string(expdomain, "&#62;=0.0");
      SP_cons_functor(t1, SP_atom_from_string("sqrt"), 1, culprit);
      SP_cons_functor(t1, SP_atom_from_string("domain_error"), 4, 
                      t1, argno, expdomain, culprit);
      SP_raise_exception(t1);    /* raise the exception */
      return 0.0;
    }  
  return sqrt(d);
}
</PRE>

<P>
The Prolog interface to this function is defined in a file
<TT>`math.pl'</TT>.  The function uses the <CODE>sqrt()</CODE> library function,
and so the math library <SAMP>`-lm'</SAMP> has to be included:

</P>

<PRE>
/* math.pl */

foreign_resource(math, [sqrt]).

foreign(sqrt_check, c, sqrt(+float, [-float])).

:- load_foreign_resource(math).
</PRE>

<P>
A linked foreign resource is created:

<PRE>
% splfr math math.pl +c math.c +l -lm
</PRE>

<P>
A simple session using this function could be:

</P>

<PRE>
% sicstus
SICStus 3  #5: Sat Feb 24 00:35:37 MET 1996
| ?- [math].
{consulting /home/san/pl/math.pl...}
{/home/san/pl/math.pl consulted, 10 msec 816 bytes}

yes
| ?- sqrt(5.0,X).

X = 2.23606797749979 ? 

yes
| ?- sqrt(a,X).
{TYPE ERROR: sqrt(a,_30) - arg 1: expected number, found a}
| ?- sqrt(-5,X).
{DOMAIN ERROR: sqrt(-5.0) - arg 1: expected '&#62;=0.0', found -5.0}
</PRE>

<P>
The above example used the foreign language interface with dynamic
linking.  To statically link <TT>`math.o'</TT> with the Prolog emulator,
the following steps would have been taken:

</P>

<PRE>
% splfr math math.pl +static +c math.c +l -lm
SICStus 3  #5: Sat Feb 24 00:35:37 MET 1996
{/tmp/00249aaa.c generated, 10 msec}

yes
% spmkds mathsp math               
SICStus 3  #5: Sat Feb 24 00:35:37 MET 1996
{/tmp/00274aaa.c generated, 0 msec}

yes
% sicstus -base ./mathsp
SICStus 3  #5: Sat Feb 24 00:35:37 MET 1996
| ?- [math].
{consulting /home/san/pl/math.pl...}
{/home/san/pl/math.pl consulted, 0 msec 864 bytes}

yes
| ?- sqrt(5.0,X).

X = 2.23606797749979 ? 

yes
</PRE>



<H3><A NAME="SEC161" HREF="sicstus_toc.html#TOC161">Stream Example</A></H3>

<P>
This is a small example how to initialize a bidirectional socket
stream (error handling omitted):

</P>

<PRE>

typedef struct {
  int fd;                       /* socket number */
  FILE *r_stream;               /* For reading */
  FILE *w_stream;               /* For writing */
} SocketData;

int socket_sgetc(SocketData *socket)
{
  return fgetc(socket-&#62;r_stream);
}

int socket_sputc(char c, SocketData *socket)
{
  return fputc(c, socket-&#62;w_stream);
}

int socket_sflush(SocketData *socket)
{
  return fflush(socket-&#62;w_stream);
}

int socket_seof(SocketData *socket)
{
  return feof(socket-&#62;r_stream);
}

void socket_sclrerr(SocketData *socket)
{
  clearerr(socket-&#62;r_stream);
  clearerr(socket-&#62;w_stream);
}

int socket_sclose(SocketData *socket) 
{
  fclose(socket-&#62;r_stream);
  fclose(socket-&#62;w_stream);
  close(socket-&#62;fd);
  free(socket);
  return 0;
}

SP_stream *new_socket_stream(int fd)
{
  SP_stream *stream;
  SocketData *socket;
  
/* Allocate and initialize data local to socket */

  socket = (SocketData *)malloc(sizeof(SocketData));
  socket-&#62;fd = fd;
  socket-&#62;r_stream = fdopen(fd,"r");
  socket-&#62;w_stream = fdopen(fd,"w");

/* Allocate and initialize Prolog stream */

  SP_make_stream(
                 socket,
                 socket_sgetc,
                 socket_sputc,
                 socket_sflush,
                 socket_seof,
                 socket_sclrerr,
                 socket_sclose,
                 &#38;stream);

/* Allocate and copy string */

  stream-&#62;filename = "socket";
  stream-&#62;fd = fd;
  
  return stream;
}
</PRE>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_10.html">previous</A>, <A HREF="sicstus_12.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
