<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Mixing Java and Prolog</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_11.html">previous</A>, <A HREF="sicstus_13.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC162" HREF="sicstus_toc.html#TOC162">Mixing Java and Prolog</A></H1>

<P>
<EM>Jasper</EM> is a bi-directional interface between programs written in
Java and programs written in Prolog. The Java-side of the interface
constists of a Java package (<CODE>jasper</CODE>) containing classes
representing the SICStus emulator.  The Prolog part is designed as an
extension to the foreign language interface, which means that
<CODE>foreign/3</CODE> declarations are used to map Java-methods on Prolog
predicates and provide automatic conversion of arguments.

</P>
<P>
Jasper can be operated in two different ways, depending on which
emulator is used as top-level application; the Java Runtime System or
the SICStus Development System.  When the Java Runtime System is used as
a top-level application, the SICStus runtime kernel is loaded into
the Java Runtime System. When the Java Runtime System is loaded into the
SICStus Development System, it is loaded as any other foreign resource,
i.e. by using <CODE>load_foreign_resource/1</CODE>. This distinction is not
visible to the programmer, but it is useful to have a picture of how the
interface works.

</P>
<P>
<STRONG>Note:</STRONG> Some of the information in this chapter is a
recapitulation of the information in the chapter section <A HREF="sicstus_11.html#SEC117">Mixing C and Prolog</A>. The intention is that this chapter should be possible to read
fairly independently.

</P>

<UL>
<LI><A HREF="sicstus_12.html#SEC163">Prerequisites</A>
<LI><A HREF="sicstus_12.html#SEC164">Calling Java from Prolog</A>
<LI><A HREF="sicstus_12.html#SEC167">Conversions between Prolog Arguments and Java Types</A>
<LI><A HREF="sicstus_12.html#SEC169">Calling Prolog from Java</A>
<LI><A HREF="sicstus_12.html#SEC170">Jasper Package Class Reference</A>
<LI><A HREF="sicstus_12.html#SEC171">Exception Handling</A>
<LI><A HREF="sicstus_12.html#SEC172">Java Resources Online</A>: Useful links to online Java resources
</UL>



<H2><A NAME="SEC163" HREF="sicstus_toc.html#TOC163">Prerequisites</A></H2>

<P>
The low-level interface uses the JNI (Java Native Interface) to call C
functions from Java. The JNI is a native interface standard developed by
Sun Microsystems. See section <A HREF="sicstus_12.html#SEC172">Resources</A>. Since the interface uses
the JNI, it is necessary that the Java VM which is to be used has
support for JNI. Even though the JNI is intended as a standard, not all
vendors support it. If your Java installation does not support the JNI,
Sun's <EM>Java Runtime Environment</EM> provides a minimal execution
enviroment for running Java code. It can be downloaded from

</P>

<PRE>
http://java.sun.com/products/jdk/1.1/jre/index.html
</PRE>

<P>
The rest of this chapter assumes that there is a Java installation with
JNI support available. 

</P>


<H2><A NAME="SEC164" HREF="sicstus_toc.html#TOC164">Calling Java from Prolog</A></H2>

<P>
Java methods are called from Prolog much in the same way as C functions
are called (see section <A HREF="sicstus_11.html#SEC119">Calling C from Prolog</A>); by creating a <EM>foreign
resource</EM>. When loaded, this resource installs a set of predicates which
are mapped onto Java-methods and when invoked, converts the Prolog
arguments to the corresponding Java-types before calling the Java
method itself.

</P>
<P>
In fact, a foreign resource (as defined in section <A HREF="sicstus_11.html#SEC120">Foreign Resource and Conversion Declarations</A>) is
not language specific itself. The language is instead specified in the
second argument to the <CODE>foreign/3</CODE> fact and it is possible to mix
foreign C functions with foreign Java methods.

</P>
<P>
How a foreign resource is created in general is described in detail in
section <A HREF="sicstus_11.html#SEC126">Creating the Linked Foreign Resource</A>. The following section(s) will
focus on the Java-specific parts of foreign resources.

</P>

<UL>
<LI><A HREF="sicstus_12.html#SEC165">Static and Dynamic Linking for Jasper</A>
<LI><A HREF="sicstus_12.html#SEC166">Declarating Java-methods</A>
</UL>



<H3><A NAME="SEC165" HREF="sicstus_toc.html#TOC165">Static and Dynamic Linking</A></H3>

<P>
There is no support for static linking of foreign resources containing
Java declarations, since Java implementations usually do not support
static linking.

</P>


<H3><A NAME="SEC166" HREF="sicstus_toc.html#TOC166">Declarating Java-methods</A></H3>

<P>
Java-methods are declared similarly to C-functions. There are two
major differences. The first is how methods are identified. It is not
enough to simply use an atom as the C interface does. Instead, a term
<CODE>method/3</CODE> is introduced:

</P>
<DL COMPACT>

<DT><CODE>method(+<VAR>ClassName</VAR>,+<VAR>MethodName</VAR>,+<VAR>Flags</VAR>)</CODE>
<DD>
<A NAME="IDX930"></A>
Used as first argument to <CODE>foreign/3</CODE> when declaring Java
methods. The first argument is an atom containing the <EM>Fully
Qualified Classname</EM> of the class. The second argument is the method
name. The third argument is a list of flags. Possible flags are
<CODE>instance</CODE> or <CODE>static</CODE>, indicating whether or not the method
is static or non-static. Non-static methods <EM>must</EM> have an
object-reference as their first argument. This is a reference to the
object on which the method will be invoked.

This term is then used to identify the method in the
<CODE>foreign_resource/2</CODE> predicate. So, to define a foreign resource
exporting the non-static Java method <CODE>getFactors</CODE> in the class
<CODE>PrimeNumber</CODE> in the package <CODE>numbers</CODE>, the <CODE>method/3</CODE>
term would look like

<PRE>
method('numbers/PrimeNumber','getFactors',[instance])
</PRE>

</DL>

<P>
The syntax for <CODE>foreign/3</CODE> is the basically the same as for
C-functions:

</P>
<DL COMPACT>

<DT><CODE>foreign(<VAR>+MethodIdentifier</VAR>, <CODE>java</CODE>, <VAR>+Predicate</VAR>)</CODE>
<DD>
<A NAME="IDX931"></A>
A <EM>hook predicate</EM>, specifies the Prolog interface to a Java method.
<VAR>MethodIdentifier</VAR> is <CODE>method/3</CODE> term as described
above. <VAR>Predicate</VAR> specifies the name of the Prolog predicate that
will be used to call <VAR>MethodIdentifier</VAR>.  <VAR>Predicate</VAR> also specifies
how the predicate arguments are to be translated into the corresponding
Java arguments.
</DL>



<H2><A NAME="SEC167" HREF="sicstus_toc.html#TOC167">Conversions between Prolog Arguments and Java Types</A></H2>

<P>
The following table lists the possible values of arguments of the
predicate specification to <CODE>foreign/3</CODE>. The value declares which
conversion between corresponding Prolog argument and Java type will take
place.

</P>
<P>
<STRONG>Note:</STRONG> The conversion declarations (composed of the declarators
specified below) together with the <CODE>method/3</CODE> term are used by the
glue-code generator to create the method's <EM>type signature</EM>, i.e. a
string which can uniquely identify a method within a class. This means
that unlike the C interface, the conversion declarations for a Java
method will affect the lookup of the method-name (in the C interface,
only the function <EM>name</EM> is relevant). So, if a method is declared as
<CODE>foo(+integer)</CODE>, there must be a method which has the name
<CODE>foo</CODE> and takes one argument of type <CODE>int</CODE>, or an argument
which can be automatically converted to an <CODE>int</CODE> (a <CODE>short</CODE>,
for example).

</P>
<DL COMPACT>

<DT><CODE>Prolog: +integer</CODE>
<DD>
<DT><CODE>Java:   int</CODE>
<DD>
The argument should be a number. It is converted to a Java <CODE>int</CODE>.

<DT><CODE>Prolog: +byte</CODE>
<DD>
<DT><CODE>Java:   byte</CODE>
<DD>
The argument should be a number. It is converted to a Java <CODE>byte</CODE>.

<DT><CODE>Prolog: +short</CODE>
<DD>
<DT><CODE>Java:   short</CODE>
<DD>
The argument should be a number. It is converted to a Java <CODE>short</CODE>.

<DT><CODE>Prolog: +long</CODE>
<DD>
<DT><CODE>Java:   long</CODE>
<DD>
The argument should be a number. It is converted to a Java
<CODE>long</CODE>. 

<STRONG>Note:</STRONG> Since Java's <CODE>long</CODE> type is 64 bits
wide and there is no standardized support for 64 bits integers in C, the
value will be truncated. So, this declaration is really only useful in
order to indicate which method should be used. For example:


<PRE>
class Bar
{
  void foo(int x)
    { ... }

  void foo(long x)
    { ... }
}
</PRE>

In order to be able to indicate that the latter of the <CODE>foo</CODE>
methods should be called, a <CODE>+long</CODE> declaration must be used, even
if the value itself will be truncated in the call.

<DT><CODE>Prolog: +float</CODE>
<DD>
<DT><CODE>Java:   float</CODE>
<DD>
The argument should be a number. It is converted to a Java <CODE>float</CODE>.

<DT><CODE>Prolog: +double</CODE>
<DD>
<DT><CODE>Java:   double</CODE>
<DD>
The argument should be a number. It is converted to a Java <CODE>double</CODE>.

<DT><CODE>Prolog: +term</CODE>
<DD>
<DT><CODE>Java:   SPTerm</CODE>
<DD>
The argument can be any term. It is passed to Java as an object of the
class <CODE>SPTerm</CODE>.

<DT><CODE>Prolog: +object(<VAR>Class</VAR>)</CODE>
<DD>
<DT><CODE>Java:   SPTerm</CODE>
<DD>
The argument should be the Prolog representation of a Java object of
class <VAR>Class</VAR>. Unless it is the first argument in a non-static method
(in which case is it treated as the object on which the method should be
invoked), it is passed to the Java method as an object of class 
<CODE>Class</CODE>.

<DT><CODE>Prolog: +atom</CODE>
<DD>
<DT><CODE>Java:   SPTerm</CODE>
<DD>
The argument should be an atom. The Java method will be passed an object
of class <CODE>SPTerm</CODE>.

<DT><CODE>Prolog: +boolean</CODE>
<DD>
<DT><CODE>Java:   boolean</CODE>
<DD>
The argument should be an atom in {<CODE>true</CODE>,<CODE>false</CODE>}. The Java
method will receive a <CODE>boolean</CODE>.

<DT><CODE>Prolog: +chars</CODE>
<DD>
<DT><CODE>Java:   String</CODE>
<DD>
The argument should be a list of character codes. The Java method will
receive an object of class <CODE>String</CODE>.

<DT><CODE>Prolog: +string</CODE>
<DD>
<DT><CODE>Java:   String</CODE>
<DD>
The argument should be an atom. The Java method will receive an object of
class <CODE>String</CODE>.

<DT><CODE>Prolog: -atom</CODE>
<DD>
<DT><CODE>Java:   SPTerm</CODE>
<DD>
The argument should be an unbound variable. The Java method will receive
an atom of class <CODE>SPTerm</CODE> which can be modified. The argument will
be bound to the value of the atom when the method returns.

<DT><CODE>Prolog: -chars</CODE>
<DD>
<DT><CODE>Java:   StringBuffer</CODE>
<DD>
The argument should be an unbound variable. The Java method will
receive an object of type <CODE>StringBuffer</CODE> which can be modified. The
argument will be bound to a list of the character codes of the
<CODE>StringBuffer</CODE> object. 

<DT><CODE>Prolog: -string</CODE>
<DD>
<DT><CODE>Java:   StringBuffer</CODE>
<DD>
The argument should be an unbound variable. The Java method will receive
an object of type <CODE>StringBuffer</CODE> which can be modified. The
argument will be bound to an atom converted from the <CODE>StringBuffer</CODE>
object.

<DT><CODE>Prolog: [-integer]</CODE>
<DD>
<DT><CODE>Java:   int <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an <CODE>int</CODE>. The value will be converted
to a Prolog integer.

<DT><CODE>Prolog: [-byte]</CODE>
<DD>
<DT><CODE>Java:   byte <VAR>M</VAR>()</CODE>
<DD>
The Java method should return a <CODE>byte</CODE>. The value will be converted
to a Prolog integer.

<DT><CODE>Prolog: [-short]</CODE>
<DD>
<DT><CODE>Java:   short <VAR>M</VAR>()</CODE>
<DD>
The Java method should return a <CODE>short</CODE>. The value will be converted
to a Prolog integer.

<DT><CODE>Prolog: [-long]</CODE>
<DD>
<DT><CODE>Java:   long <VAR>M</VAR>()</CODE>
<DD>
The Java method should return a <CODE>long</CODE>. The value will be converted
and possibly truncated to a Prolog integer.

<DT><CODE>Prolog: [-float]</CODE>
<DD>
<DT><CODE>Java:   float <VAR>M</VAR>()</CODE>
<DD>
The Java method should return a <CODE>float</CODE>. The value will be converted
to a Prolog float.

<DT><CODE>Prolog: [-double]</CODE>
<DD>
<DT><CODE>Java:   double <VAR>M</VAR>()</CODE>
<DD>
The Java method should return a <CODE>double</CODE>. The value will be converted
to a Prolog float.

<DT><CODE>Prolog: [-term]</CODE>
<DD>
<DT><CODE>Java:   SPTerm <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an object of class <CODE>SPTerm</CODE> which
will be converted to a Prolog term.

<DT><CODE>Prolog: [-object(<VAR>Class</VAR>)]</CODE>
<DD>
<DT><CODE>Java:   SPTerm <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an object of class <VAR>Class</VAR> which
will be converted to the internal Prolog representation of the Java
object.

<DT><CODE>Prolog: [-atom]</CODE>
<DD>
<DT><CODE>Java:   SPTerm <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an object of class <CODE>SPTerm</CODE> which
will be converted to a Prolog atom.

<DT><CODE>Prolog: [-boolean]</CODE>
<DD>
<DT><CODE>Java:   boolean <VAR>M</VAR>()</CODE>
<DD>
The Java should return a <CODE>boolean</CODE>. The value will be converted to
a Prolog atom in {<CODE>true</CODE>,<CODE>false</CODE>}.

<DT><CODE>Prolog: [-chars]</CODE>
<DD>
<DT><CODE>Java:   String <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an object of class <CODE>String</CODE> which
will be converted to a list of character codes.

<DT><CODE>Prolog: [-string]</CODE>
<DD>
<DT><CODE>Java:   String <VAR>M</VAR>()</CODE>
<DD>
The Java method should return an object of class <CODE>String</CODE> which
will be converted to an atom.

</DL>


<UL>
<LI><A HREF="sicstus_12.html#SEC168">Java-Prolog Example</A>
</UL>



<H3><A NAME="SEC168" HREF="sicstus_toc.html#TOC168">Calling Java from Prolog: An Example</A></H3>

<P>
The following is an simple, but complete example of how a Java method can be called
from Prolog.

</P>
<P>
First, we must write the resource file. Let us call it <TT>`simple.pl'</TT>.

</P>

<PRE>
% File: simple.pl

:- module(simple, [simple/2]).

:- use_module(library(jasper)).
:- load_foreign_resource(simple).

foreign(method('Simple', 'simpleMethod', [static]), java,
        simple(+integer,[-integer])).

foreign_resource(simple,
                 [
                  method('Simple', 'simpleMethod', [static])
                 ]).
</PRE>

<P>
This file is the processed with the script <CODE>splfr</CODE>
(see section <A HREF="sicstus_11.html#SEC124">Interface Predicates</A>) to produce a foreign resource:

</P>

<PRE>
% splfr simple simple.pl
SICStus  3.7: Mon Mar  2 19:22:44 MET 1998
{/var/tmp/aaaa004Cd.c generated, 20 msec}

yes
</PRE>

<P>
Note that we do not specify any Java files to <CODE>splfr</CODE> as we would
specify C files when building foreign resources for C code. This is
because the C code can be compiled into the resource itself, while the
Java code must be loaded at runtime into the JVM. This means that the
resource will only contain the glue-code for calling the JVM, and no
actual Java code. Hence, these resources are usually quite small.

</P>
<P>
Note also that the use of <CODE>+dynamic</CODE> is implicit and obligatory;
resources with Java-calls <EM>must</EM> be dynamic.

</P>
<P>
Now, we need some Java code to call:

</P>

<PRE>
Simple.java:

public class Simple
{
  static int simpleMethod(int value)
  {
    return value*42;
  }
}
</PRE>

<P>
This Java code must now be compiled. Refer to the documentation of your
Java implementation exactly how to do this. On Solaris, this might look like:

</P>

<PRE>
% javac Simple.java
</PRE>

<P>
Now we are ready to call the method <CODE>simple/2</CODE> from inside SICStus.

</P>

<PRE>
% sicstus
SICStus  3.7: Mon Mar  2 19:22:44 MET 1998
| ?- compile(simple).
{compiling ...}
[...]
{compiled ... simple.pl in module simple, 160 msec 48640 bytes}

yes
| ?- simple(17,X).

X = 714 ? 

yes
| ?- 
</PRE>

<P>
What has happened is that the predicate <CODE>simple/2</CODE> has been
installed as a predicate defined in Java (this is not exactly true; the
predicate is defined as a C-function which calls the Java method). When
we load the <CODE>simple</CODE> module, we will first load the <CODE>jasper</CODE>
module (and thereby the JVM) and then load the <CODE>simple</CODE> foreign
resource, which defines the <CODE>simple/2</CODE> predicate.

</P>


<H2><A NAME="SEC169" HREF="sicstus_toc.html#TOC169">Calling Prolog from Java</A></H2>

<P>
Calling Prolog from Java is done by using the Java package
<CODE>jasper</CODE>. This package contains a set of Java classes which can be
used to create and manipulate terms, ask queries and request one or more
solutions. The functionality provided by this set of classes is
basically the same as the functionality provided by the C-Prolog
interface (see section <A HREF="sicstus_11.html#SEC117">Mixing C and Prolog</A>).

</P>
<P>
The usage is easiest described by an example. The following is a Java
version of the <CODE>train</CODE> example. See section <A HREF="sicstus_11.html#SEC158">Train Example (connections)</A>.

</P>

<PRE>
import jasper.*;

public class Simple
{
  public static void main(String argv[]) {

  SICStus sp;
  SPPredicate pred;
  SPTerm from, to, way;
  SPQuery query;
  int i;
    
  try 
    {
      sp = new SICStus(argv,null);

      sp.load("train.ql");
    
      pred = new SPPredicate(sp, "connected", 4, "");
      to = new SPTerm(sp, "Orebro");
      from = new SPTerm(sp, "Stockholm");
      way = new SPTerm(sp).putVariable();

      query = sp.openQuery(pred, new SPTerm[] { from, to, way, way });
    
      while (query.nextSolution())
        {
          System.out.println(way.toString());
        }
    }
  catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
}
</PRE>

<P>
This is how it works:

</P>

<OL>
<LI>

First, we import all the <CODE>jasper</CODE>-classes. To find them, the
class-path should be set to contain the path to the SICStus libraries
(<CODE>$SP_PATH/library</CODE>). How this is done depends on the operating
system and which Java implementation is used, but typically there is a
environment variable <CODE>CLASSPATH</CODE> which contains the path. The path
can often also be set on the commandline using an option such as
<SAMP>`-classpath'</SAMP>. Refer to your Java documentation for more details.

<LI>

Before any predicates can be called, the SICStus emulator must be
initialized. This is done by instantiating the <CODE>SICStus</CODE>
class. NOTE: This class must only be instantiated once per Java
process. Multiple SICStus-objects are not supported.

Most methods take a reference to this object as their first
argument. This is implicit in the rest of this chapter, unless otherwise
stated.

<LI>

The next step is to load the Prolog code. This is done by the method
<CODE>load</CODE>. Corresponds to <CODE>SP_load()</CODE> in the
C-interface. See section <A HREF="sicstus_11.html#SEC153">Loading Prolog Code</A>.

<LI>

Now, everything is set up to start making queries. In order to make a
query, the actual query term must be created. This is done by creating
an object of the <CODE>SPPredicate</CODE> class:


<PRE>
SPPredicate pred = new SPPredicate(sp, "connected", 4, "");
</PRE>

<LI>

At this point, we have created a predicate object for the predicate
<CODE>connected/4</CODE>. It is now time to create the arguments for the
query. The arguments are placed in an array which is passed to a
suitable method to make the query.

The arguments consist of objects of the class <CODE>SPTerm</CODE>. For
example, if we need two atoms and a variable for the query

<PRE>
| ?- connected('Stockholm', 'Orebro', X, X).
</PRE>

the following Java code will do it for us:

<PRE>
to = new SPTerm(sp, "Orebro");
from = new SPTerm(sp, "Stockholm");
way = new SPTerm(sp).putVariable();
</PRE>

<LI>

Now it is time to make the query. As in the C-Prolog interface, there
are three ways of making a query.

<DL COMPACT>

<DT><CODE>query(pred, args)</CODE>
<DD>
This method is useful if you are only interested in finding the first
solution to a goal. In the case of <CODE>connected/4</CODE> this is not the
case; there are more than one solution.

<DT><CODE>queryCutFail(pred, args)</CODE>
<DD>
This method is useful if you are only interested in the side-effects of
the query. As <CODE>query()</CODE> it only finds the first solution, and
then it cuts away all other solutions and fails.

<DT><CODE>openQuery(pred, args)</CODE>
<DD>
This method is useful when you are interested in some or all solutions to the
query. Since the <CODE>connected/4</CODE> may give us multiple solution, this
is what we will use.


<PRE>
SPQuery query;

query = sp.openQuery(pred, new SPTerm[] { from, to, way, way });
    
while (query.nextSolution())
  System.out.println(way.toString());
</PRE>

The <CODE>openQuery</CODE> method returns a reference to the query, an object
of the <CODE>SPQuery</CODE> class. To obtain solutions, the method
<CODE>nextSolution()</CODE> is called with no arguments. <CODE>nextSolution</CODE>
returns <CODE>true</CODE> as long as there are more solutions, the example
above will print the value of <CODE>way</CODE> until there are no more
solutions.

</DL>
</OL>



<H2><A NAME="SEC170" HREF="sicstus_toc.html#TOC170">Jasper Package Class Reference</A></H2>

<P>
Detailed documentation of the classes in the <CODE>jasper</CODE> package can
be found at

</P>

<PRE>
http://www.sics.se/isl/sicstus/jasper/Package-jasper.html
</PRE>

Or follow this <a
href="http://www.sics.se/isl/sicstus/jasper/Package-jasper.html">hyperlink</a>.

<P>
This documentation will most probably be included in a future version of
the manual.

</P>


<H2><A NAME="SEC171" HREF="sicstus_toc.html#TOC171">Exception Handling</A></H2>

<P>
Exceptions are handled seamlessly between Java and Prolog. This means
that exceptions can be thrown in Prolog and caught in Java and the other
way around. For example, if a predicate called from Java raises an
exception by <CODE>raise_exception/1</CODE> and the predicate itself does not
catch the exception, the Java-method which performed the query,
<CODE>queryCutFail()</CODE> for example, will throw an exception containing
the exception term. Symmetrically, a Java-exception thrown (and not
caught) in a method called from Prolog will cause the corresponding
predicate (<CODE>simple/2</CODE> in the example above) to raise an exception
containing the exception object (in the internal Prolog representation
of a Java object).

</P>


<H2><A NAME="SEC172" HREF="sicstus_toc.html#TOC172">Resources</A></H2>

<P>
There are almost infinitely many Java resources on the Internet. Here is a
list of a few which are related to Jasper:

</P>

<UL>
<LI>

The SICStus Prolog Release Notes contains information useful in order to
get Jasper working. 

<LI>

There is a collection of Java-to-Prolog examples in


<PRE>
<CODE>library('jasper/examples')</CODE>
</PRE>

See the file <TT>`README'</TT> for details on how to use the examples.

<LI>

The main Java resource is Sun's own Java homepage:

<PRE>
http://java.sun.com
</PRE>

<LI>

Yahoo:

<PRE>
http://www.yahoo.com/Computers_and_Internet/Programming_Languages/Java/
</PRE>

<LI>

Information about the JNI can be found at:

<PRE>
http://java.sun.com/products/jdk/1.1/docs/guide/jni/index.html
</PRE>

<LI>

The ACM student magazine <EM>Crossroads</EM> has published an article on
the JNI, available online at:


<PRE>
http://www1.acm.org:82/crossroads/xrds4-2/jni.html
</PRE>

</UL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_11.html">previous</A>, <A HREF="sicstus_13.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
