<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Programming Tips and Examples</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_12.html">previous</A>, <A HREF="sicstus_14.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC173" HREF="sicstus_toc.html#TOC173">Programming Tips and Examples</A></H1>

<P>
This chapter describes how to write clean programs that will execute
efficiently.  To some extent, writing efficient code in any language
requires basic knowledge of its compiler, and we will mention some
important properties of the SICStus Prolog compiler.  A number of simple
examples of Prolog programming are also given.

</P>


<UL>
<LI><A HREF="sicstus_13.html#SEC174">Programming Guidelines</A>
<LI><A HREF="sicstus_13.html#SEC175">Indexing</A>
<LI><A HREF="sicstus_13.html#SEC176">Tail</A>
<LI><A HREF="sicstus_13.html#SEC177">If-Then-Else</A>
<LI><A HREF="sicstus_13.html#SEC178">Programming Examples</A>
</UL>



<H2><A NAME="SEC174" HREF="sicstus_toc.html#TOC174">Programming Guidelines</A></H2>

<P>
A lot of clarity and efficiency is gained by sticking to a few basic
rules.  This list is necessarily very incomplete.  The reader is
referred to textbooks such as [O'Keefe 90] for a thorough exposition of
the elements of Prolog programming style and techniques.

</P>

<UL>
<LI>

Don't write code in the first place if there is a library predicate that
will do the job.

<LI>

Write clauses representing base case before clauses representing recursive
cases.

<LI>

Input arguments before output arguments in clause heads and goals.

<LI>

Use pure data structures instead of data base changes.

<LI>

Use cuts sparingly, and <EM>only</EM> at proper places (see section <A HREF="sicstus_5.html#SEC48">The Cut Symbol</A>).  A
cut should be placed at the exact point that it is known that the
current choice is the correct one: no sooner, no later.

<LI>

Make cuts as local in their effect as possible.  If a predicate is
intended to be deterministic, define <EM>it</EM> as such; do not rely on
its callers to prevent unintended backtracking.

<LI>

Binding output arguments before a cut is a common source of programming
errors, so don't do it.

<LI>

Replace cuts by if-then-else constructs if the test is simple enough
(see section <A HREF="sicstus_13.html#SEC177">If-Then-Else Compilation</A>).

<LI>

Use disjunctions sparingly, <EM>always</EM> put parentheses around
them, <EM>never</EM> put parentheses around the individual disjuncts,
<EM>never</EM> put the <SAMP>`;'</SAMP> at the end of a line.

<LI>

Write the clauses of a predicate so that
they discriminate on the principal functor of the first argument (see
below).  For maximum efficiency, avoid "defaulty" programming
("catch-all" clauses).

<LI>

Don't use lists (<CODE>[...]</CODE>), "round lists" (<CODE>(...)</CODE>), or
braces (<CODE>{...}</CODE>) to represent compound terms, or "tuples", of
some fixed arity.  The name of a compound term comes for free.
</UL>



<H2><A NAME="SEC175" HREF="sicstus_toc.html#TOC175">Indexing</A></H2>
<P>
<A NAME="IDX932"></A>

</P>
<P>
The clauses of any predicate are <EM>indexed</EM> according to the
principal functor of the first argument in the head of the clause.  This
means that the subset of clauses which match a given goal, as far as the
first step of unification is concerned, is found very quickly, in
practically constant time.  This can be very important where there is a
large number of clauses for a predicate.  Indexing also improves the
Prolog system's ability to detect determinacy--important for conserving
working storage, and strongly related to last call optimization (see below).

</P>
<P>
Indexing applies to interpreted clauses as well as to compiled clauses.

</P>


<H2><A NAME="SEC176" HREF="sicstus_toc.html#TOC176">Last Call Optimization</A></H2>
<P>
<A NAME="IDX933"></A>
<A NAME="IDX934"></A>
<A NAME="IDX935"></A>
<A NAME="IDX936"></A>

</P>
<P>
The compiler incorporates <EM>last call optimization</EM> to improve the
speed and space efficiency of determinate predicates.

</P>
<P>
When execution reaches the last goal in a clause belonging to some
predicate, and provided there are no remaining backtrack points in the
execution so far of that predicate, all of the predicate's local working
storage is reclaimed <EM>before</EM> the final call, and any structures it
has created become eligible for garbage collection.  This means that
programs can now recurse to arbitrary depths without necessarily
exceeding core limits.  For example:

</P>

<PRE>
cycle(State) :- transform(State, State1), cycle(State1).
</PRE>

<P>
where <CODE>transform/2</CODE> is a determinate predicate, can continue
executing indefinitely, provided each individual structure, <VAR>State</VAR>,
is not too large.  The predicate cycle is equivalent to an iterative
loop in a conventional language.  
</P>
<P>
To take advantage of last call optimization one must ensure that
the Prolog system can recognize that the predicate is determinate at the
point where the recursive call takes place.  That is, the system must be
able to detect that there are no other solutions to the current goal to
be found by subsequent backtracking.  In general this involves reliance
on the Prolog compiler's indexing and/or use of cut; see section <A HREF="sicstus_5.html#SEC48">The Cut Symbol</A>.

</P>


<H2><A NAME="SEC177" HREF="sicstus_toc.html#TOC177">If-Then-Else Compilation</A></H2>
<P>
<A NAME="IDX937"></A>
<A NAME="IDX938"></A>
<A NAME="IDX939"></A>
<A NAME="IDX940"></A>
<A NAME="IDX941"></A>

</P>
<P>
Ordinary disjunction, <CODE>(<VAR>P</VAR>;<VAR>Q</VAR>)</CODE>, is treated by the
compiler as an anonymous predicate with two clauses, and the execution
of a disjunction relies on backtracking to explore the two disjuncts.

</P>
<P>
If-then-else statements of the form:

</P>

<PRE>
(<VAR>If</VAR> -&#62; <VAR>Then</VAR>; <VAR>Else</VAR>)
</PRE>

<P>
are recognized by the compiler and are under certain conditions compiled
to code that is much more efficient than the corresponding disjunction,
essentially turning the <VAR>If</VAR> test to a conditional jump and often
avoiding costly backtracking altogether.

</P>
<P>
For this optimization to be effective, the test must be a conjunction of
a restricted set of built-in predicates (roughly, arithmetic tests, type
tests and term comparisons).

</P>
<P>
<A NAME="IDX942"></A>
<A NAME="IDX943"></A>
This optimization is actually somewhat more general than what is
described above.  A sequence of <EM>guarded clauses</EM>:

</P>

<PRE>
<VAR>Head1</VAR> :- <VAR>Guard1</VAR>, !, <VAR>Body1</VAR>.
...
<VAR>Headm</VAR> :- <VAR>Guardm</VAR>, !, <VAR>Bodym</VAR>.
<VAR>Headn</VAR> :- <VAR>Bodym</VAR>.
</PRE>

<P>
is eligible for the same optimization, provided that the arguments of
the clause heads are all unique variables and that the "guards" are
simple tests as described above.

</P>


<H2><A NAME="SEC178" HREF="sicstus_toc.html#TOC178">Programming Examples</A></H2>

<P>
The rest of this chapter contains a number of simple examples of Prolog
programming, illustrating some of the techniques described above.

</P>


<UL>
<LI><A HREF="sicstus_13.html#SEC179">Simple List</A>
<LI><A HREF="sicstus_13.html#SEC180">Family Example</A>
<LI><A HREF="sicstus_13.html#SEC181">Association Lists</A>
<LI><A HREF="sicstus_13.html#SEC182">Derivative</A>
<LI><A HREF="sicstus_13.html#SEC183">Use Of Meta</A>
<LI><A HREF="sicstus_13.html#SEC184">Use Of Term Exp</A>
<LI><A HREF="sicstus_13.html#SEC185">Interpreter</A>
<LI><A HREF="sicstus_13.html#SEC186">Translate</A>
<LI><A HREF="sicstus_13.html#SEC187">Muse FLI Example</A>
</UL>



<H3><A NAME="SEC179" HREF="sicstus_toc.html#TOC179">Simple List Processing</A></H3>

<P>
The goal <CODE>concatenate(<VAR>L1</VAR>,<VAR>L2</VAR>,<VAR>L3</VAR>)</CODE> is true if list
<VAR>L3</VAR> consists of the elements of list <VAR>L1</VAR> concatenated with the
elements of list <VAR>L2</VAR>.  The goal <CODE>member(<VAR>X</VAR>,<VAR>L</VAR>)</CODE> is
true if <VAR>X</VAR> is one of the elements of list <VAR>L</VAR>.  The goal
<CODE>reverse(<VAR>L1</VAR>,<VAR>L2</VAR>)</CODE> is true if list <VAR>L2</VAR> consists of
the elements of list <VAR>L1</VAR> in reverse order.  
</P>

<PRE>
concatenate([], L, L).
concatenate([X|L1], L2, [X|L3]) :- concatenate(L1, L2, L3).

member(X, [X|_]).
member(X, [_|L]) :- member(X, L).

reverse(L, L1) :- reverse_concatenate(L, [], L1).

reverse_concatenate([], L, L).
reverse_concatenate([X|L1], L2, L3) :-
        reverse_concatenate(L1, [X|L2], L3).
</PRE>



<H3><A NAME="SEC180" HREF="sicstus_toc.html#TOC180">Family Example (descendants)</A></H3>

<P>
The goal <CODE>descendant(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE> is true if <VAR>Y</VAR> is a
descendant of <VAR>X</VAR>.  
</P>

<PRE>
descendant(X, Y) :- offspring(X, Y).
descendant(X, Z) :- offspring(X, Y), descendant(Y, Z).

offspring(abraham, ishmael).
offspring(abraham, isaac).
offspring(isaac, esau).
offspring(isaac, jacob).
</PRE>

<P>
If for example the query

</P>

<PRE>
| ?- descendant(abraham, X).
</PRE>

<P>
is executed, Prolog's backtracking results in different descendants of
Abraham being returned as successive instances of the variable <VAR>X</VAR>,
i.e.

</P>

<PRE>
X = ishmael
X = isaac
X = esau
X = jacob
</PRE>



<H3><A NAME="SEC181" HREF="sicstus_toc.html#TOC181">Association List Primitives</A></H3>

<P>
These predicates implement "association list" primitives.  They use a
binary tree representation.  Thus the time complexity for these
predicates is <VAR>O(lg N)</VAR>, where <VAR>N</VAR> is the number of keys.  These
predicates also illustrate the use of <CODE>compare/3</CODE> (see section <A HREF="sicstus_10.html#SEC102">Comparison of Terms</A>) for case analysis.  
</P>
<P>
The goal <CODE>get_assoc(<VAR>Key</VAR>, <VAR>Assoc</VAR>, <VAR>Value</VAR>)</CODE> is true
when <VAR>Key</VAR> is identical to one of the keys in <VAR>Assoc</VAR>, and
<VAR>Value</VAR> unifies with the associated value.  
</P>

<PRE>
get_assoc(Key, t(K,V,L,R), Val) :-
        compare(Rel, Key, K),
        get_assoc(Rel, Key, V, L, R, Val).

get_assoc(=, _, Val, _, _, Val).
get_assoc(&#60;, Key, _, Tree, _, Val) :-
        get_assoc(Key, Tree, Val).
get_assoc(&#62;, Key, _, _, Tree, Val) :-
        get_assoc(Key, Tree, Val).
</PRE>

<P>
The goal <CODE>put_assoc(<VAR>Key</VAR>, <VAR>OldAssoc</VAR>, <VAR>Val</VAR>,
<VAR>NewAssoc</VAR>)</CODE> is true when <VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> define
the same mapping for all keys other than <VAR>Key</VAR>, and
<CODE>get_assoc(<VAR>Key</VAR>, <VAR>NewAssoc</VAR>, <VAR>Val</VAR>)</CODE> is true.  
</P>

<PRE>
put_assoc(Key, t, Val, Tree) :- !, Tree = t(Key,Val,t,t).
put_assoc(Key, t(K,V,L,R), Val, New) :-
        compare(Rel, Key, K),
        put_assoc(Rel, Key, K, V, L, R, Val, New).

put_assoc(=, Key, _, _, L, R, Val, t(Key,Val,L,R)).
put_assoc(&#60;, Key, K, V, L, R, Val, t(K,V,Tree,R)) :-
        put_assoc(Key, L, Val, Tree).
put_assoc(&#62;, Key, K, V, L, R, Val, t(K,V,L,Tree)) :-
        put_assoc(Key, R, Val, Tree).
</PRE>



<H3><A NAME="SEC182" HREF="sicstus_toc.html#TOC182">Differentiation</A></H3>

<P>
The goal <CODE>d(<VAR>E1</VAR>, <VAR>X</VAR>, <VAR>E2</VAR>)</CODE> is true if expression
<VAR>E2</VAR> is a possible form for the derivative of expression <VAR>E1</VAR>
with respect to <VAR>X</VAR>.  
</P>

<PRE>
:- op(300, xfy, **). /* binds tighter than * */

d(X, X, D) :- atomic(X), !, D = 1.
d(C, X, D) :- atomic(C), !, D = 0.
d(U+V, X, DU+DV) :- d(U, X, DU), d(V, X, DV).
d(U-V, X, DU-DV) :- d(U, X, DU), d(V, X, DV).
d(U*V, X, DU*V+U*DV) :- d(U, X, DU), d(V, X, DV).
d(U**N, X, N*U**N1*DU) :- integer(N), N1 is N-1, d(U, X, DU).
d(-U, X, -DU) :- d(U, X, DU).
</PRE>



<H3><A NAME="SEC183" HREF="sicstus_toc.html#TOC183">Use of Meta-Logical Predicates</A></H3>

<P>
This example illustrates the use of the meta-logical predicates
<CODE>var/1</CODE>, <CODE>arg/3</CODE>, and <CODE>functor/3</CODE> (see section <A HREF="sicstus_10.html#SEC106">Meta-Logic</A>).
The procedure call <CODE>variables(<VAR>Term</VAR>, <VAR>L</VAR>, [])</CODE>
instantiates variable <VAR>L</VAR> to a list of all the variable occurrences
in the term <VAR>Term</VAR>.  e.g.

</P>

<PRE>
| ?- variables(d(U*V, X, DU*V+U*DV), L, []).

L = [U,V,X,DU,V,U,DV]
</PRE>


<PRE>
variables(X, [X|L0], L) :- var(X), !, L = L0.
variables(T, L0, L) :- 
%       nonvar(T),
        functor(T, _, A), 
        variables(0, A, T, L0, L).

variables(A, A, _, L0, L) :- !, L = L0.
variables(A0, A, T, L0, L) :- 
%       A0&#60;A,
        A1 is A0+1,
        arg(A1, T, X),
        variables(X, L0, L1),
        variables(A1, A, T, L1, L).
</PRE>



<H3><A NAME="SEC184" HREF="sicstus_toc.html#TOC184">Use of Term Expansion</A></H3>

<P>
This example illustrates the use of <CODE>user:term_expansion/(2,4)</CODE> to
augment the built-in predicate <CODE>expand_term/2</CODE> which works as a
filter on the input to compile and consult.  The code below will allow
the declaration <SAMP>`:- wait f/3'</SAMP> as an alias for <SAMP>`:- block
f(-,?,?)'</SAMP>.  Wait declarations were used in previous versions of SICStus
Prolog.

</P>
<P>
Note the <CODE>multifile</CODE> declaration, which prevents this
<CODE>user:term_expansion/(2,4)</CODE> clause from erasing any other clauses for
the same predicate that might have been loaded.

</P>

<PRE>
:- op(1150, fx, [wait]).

:- multifile user:term_expansion/2.
user:term_expansion((:- wait F/N), (:- block Head)) :-
        functor(Head, F, N), 
        wb_args(N, Head).

wb_args(1, Head) :- !, arg(1, Head, -).
wb_args(N, Head) :-
%       N&#62;1,
        arg(N, Head, ?),
        N1 is N-1,
        wb_args(N1, Head).
</PRE>



<H3><A NAME="SEC185" HREF="sicstus_toc.html#TOC185">Prolog in Prolog</A></H3>

<P>
This example shows how simple it is to write a Prolog interpreter in
Prolog, and illustrates the use of a variable goal.  In this
mini-interpreter, goals and clauses are represented as ordinary Prolog
data structures (i.e. terms).  Terms representing clauses are specified
using the predicate <CODE>my_clause/1</CODE>, e.g.

</P>

<PRE>
my_clause( (grandparent(X, Z) :- parent(X, Y), parent(Y, Z)) ).
</PRE>

<P>
A unit clause will be represented by a term such as

</P>

<PRE>
my_clause( (parent(john, mary) :- true) ).
</PRE>

<P>
The mini-interpreter consists of three clauses:  

</P>

<PRE>
execute((P,Q)) :- !, execute(P), execute(Q).
execute(P) :- predicate_property(P, built_in), !, P.
execute(P) :- my_clause((P :- Q)), execute(Q).
</PRE>

<P>
The second clause enables the mini-interpreter to cope with calls to
ordinary Prolog predicates, e.g. built-in predicates.  The
mini-interpreter needs to be extended to cope with the other control
structures, i.e. <CODE>!</CODE>, <CODE>(P;Q)</CODE>, <CODE>(P-&#62;Q)</CODE>,
<CODE>(P-&#62;Q;R)</CODE>, <CODE>(\+ P)</CODE>, and <CODE>if(P,Q,R)</CODE>.  
</P>


<H3><A NAME="SEC186" HREF="sicstus_toc.html#TOC186">Translating English Sentences into Logic Formulae</A></H3>

<P>
The following example of a definite clause grammar defines in a formal way
the traditional mapping of simple English sentences into formulae of
classical logic.  By way of illustration, if the sentence

</P>

<BLOCKQUOTE>
<P>
Every man that lives loves a woman.
</BLOCKQUOTE>

<P>
is parsed as a sentence by the call

</P>

<PRE>
| ?- phrase(sentence(<VAR>P</VAR>), [every,man,that,lives,loves,a,woman]).
</PRE>

<P>
then <VAR>P</VAR> will get instantiated to

</P>

<PRE>
all(X):(man(X)&#38;lives(X) =&#62; exists(Y):(woman(Y)&#38;loves(X,Y)))
</PRE>

<P>
where <CODE>:</CODE>, <CODE>&#38;</CODE> and <CODE>=&#62;</CODE> are infix operators defined by 

</P>

<PRE>
:- op(900, xfx, =&#62;).
:- op(800, xfy, &#38;).
:- op(300, xfx, :).
</PRE>

<P>
The grammar follows:

</P>

<PRE>
sentence(P) --&#62; noun_phrase(X, P1, P), verb_phrase(X, P1).

noun_phrase(X, P1, P) --&#62;
        determiner(X, P2, P1, P), noun(X, P3), rel_clause(X, P3, P2).
noun_phrase(X, P, P) --&#62; name(X).

verb_phrase(X, P) --&#62; trans_verb(X, Y, P1), noun_phrase(Y, P1, P).
verb_phrase(X, P) --&#62; intrans_verb(X, P).

rel_clause(X, P1, P1&#38;P2) --&#62; [that], verb_phrase(X, P2).
rel_clause(_, P, P) --&#62; [].

determiner(X, P1, P2, all(X):(P1=&#62;P2)) --&#62; [every].
determiner(X, P1, P2, exists(X):(P1&#38;P2)) --&#62; [a].

noun(X, man(X)) --&#62; [man].
noun(X, woman(X)) --&#62; [woman].

name(john) --&#62; [john].

trans_verb(X, Y, loves(X,Y)) --&#62; [loves].
intrans_verb(X, lives(X)) --&#62; [lives].
</PRE>



<H3><A NAME="SEC187" HREF="sicstus_toc.html#TOC187">Muse FLI Example</A></H3>

<P>
This example illustrates the techniques used in foreign language
interface programming in Muse.

</P>
<P>
Let us assume we want to count the number of solutions of some problem,
as well as the
number of solutions produced by each worker. The following C file,
<TT>`count.c'</TT>,
defines some procedures that can be used for this purpose.  
</P>

<PRE>
#include &#60;sicstus/sicstus.h&#62;

static int *overall_counter_lock;
static int *worker_counters, *overall_counter;

void allocate_counters()
{
  overall_counter_lock = (int *)SP_malloc(sizeof(int));
  overall_counter = (int *)SP_malloc(sizeof(int));
  worker_counters = (int *)SP_malloc(muse_max_workers()*sizeof(int));
}

void init_counters()
{
  int i;

  muse_init_lock(overall_counter_lock);
  *overall_counter = 0;

  for (i=0; i&#60;muse_num_workers(); i++)
    worker_counters[i]=0;
}

long get_counter(c)
     long c;
{
  if (c &#62;= 0) return worker_counters[c];
  else        return *overall_counter;
}

long incr_counter()
{
  int i;

  muse_lock(overall_counter_lock);
  i = ++(*overall_counter);
  muse_un_lock(overall_counter_lock);
  ++(worker_counters[muse_worker_id()]);

  return i;
}
</PRE>

<P>
We have defined here an <CODE>overall_counter</CODE> and an array of
<CODE>worker_counters</CODE>. There is a lock, <CODE>overall_counter_lock</CODE>,
ensuring the atomicity of incrementing the <CODE>overall_counter</CODE>.  The
code <CODE>allocate_counter</CODE> allocates the counters in shared memory.
The <CODE>init_counter</CODE> procedure initializes all counters, while
<CODE>get_counters</CODE> retrieves the current value of a counter associated
with a worker (non-negative argument), or the value of the
<CODE>overall_counter</CODE> (negative argument). The <CODE>incr_counter</CODE>
procedure increments the counter for the worker, as well as the overall
counter.

</P>
<P>
The following Muse code, <TT>`count.pl'</TT>, makes use of the above C
routines to accomplish the task of counting the solutions of some goal.

</P>

<PRE>
foreign_resource(count, [allocate_counters,init_counters,
                         incr_counter,get_counter]).

foreign(allocate_counters, c, allocate_counters).
foreign(init_counters, c, init_counters).
foreign(incr_counter, c, incr_counter([-integer])).
foreign(get_counter, c, get_counter(+integer, [-integer])).

:- load_foreign_resource(count).
:- allocate_counters.

count(Goal) :-
        statistics(walltime, [Start,_]),
        count_solutions(Goal),
        statistics(walltime, [End,_]),
        display_counts(N),
        Time is End - Start,
        format('Total ~d solutions in ~3d seconds.~n', [N,Time]).

count_solutions(Goal) :-
        init_counters,
        Goal,
        incr_counter(_),
        fail.
count_solutions(_).

display_counts(N) :-
        get_counter(-1, N),
        muse_num_workers(M),
        display_worker_counts(0, M).

display_worker_counts(M, M) :- !.
display_worker_counts(W, M) :-
        get_counter(W, C),
        (   C==0 -&#62; true ;
            format('Solutions by worker ~d: ~d ~n', [W,C])
        ),
        W1 is W+1, display_worker_counts(W1, M).
</PRE>

<P>
This example is compiled and executed as follows to count the number
of solutions to the 8 queens problem (see section <A HREF="sicstus_6.html#SEC54">Running a Parallel Example</A>):

</P>

<PRE>
% gcc -c count.c
% sicstus -P
| ?- compile([count,queens]).
| ?- muse_flag(num_workers,_,3).
| ?- count(queens(8,_)).
Solutions by worker 0: 39
Solutions by worker 1: 31
Solutions by worker 2: 22
Total 92 solutions in 0.070 seconds.

yes
| ?-
</PRE>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_12.html">previous</A>, <A HREF="sicstus_14.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
