<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Association Lists</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_15.html">previous</A>, <A HREF="sicstus_17.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC190" HREF="sicstus_toc.html#TOC190">Association Lists</A></H1>
<P>
<A NAME="IDX954"></A>

</P>
<P>
In this package, finite mappings ("association lists") are represented
by AVL trees, i.e. they are subject to the Adelson-Velskii-Landis
balance criterion:

</P>

<BLOCKQUOTE>
<P>
  A tree is balanced iff for every node the heights of its
  two subtrees differ by at most 1.
</BLOCKQUOTE>

<P>
The empty tree is represented as <CODE>t</CODE>.  A tree with key <CODE>K</CODE>,
value <CODE>V</CODE>, and left and right subtrees <CODE>L</CODE> and <CODE>R</CODE> is
represented as <CODE>t(K,V,|R|-|L|,L,R)</CODE>, where <CODE>|T|</CODE> denotes the
height of <CODE>T</CODE>.

</P>
<P>
The advantage of this representation is that lookup, insertion and
deletion all become--in the worst case---<VAR>O(log n)</VAR> operations.

</P>
<P>
The algorithms are from [Wirth 76], section 4.4.6--4.4.8.

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(assoc)).
</PRE>

<DL COMPACT>

<DT><CODE>empty_assoc(<VAR>?Assoc</VAR>)</CODE>
<DD>
<A NAME="IDX955"></A>
<VAR>Assoc</VAR> is an empty AVL tree.

<DT><CODE>assoc_to_list(<VAR>+Assoc</VAR>, <VAR>?List</VAR>)</CODE>
<DD>
<A NAME="IDX956"></A>
<VAR>List</VAR> is a list of <VAR>Key-Value</VAR> pairs in ascending order with no
duplicate <VAR>Keys</VAR> specifying the same finite function as the
association tree <VAR>Assoc</VAR>.  Use this to convert an association tree
to a list.

<DT><CODE>is_assoc(<VAR>+Assoc</VAR>)</CODE>
<DD>
<A NAME="IDX957"></A>
<VAR>Assoc</VAR> is a (proper) AVL tree.  It checks both that the keys are in
ascending order and that <VAR>Assoc</VAR> is properly balanced.

<DT><CODE>min_assoc(<VAR>+Assoc</VAR>, <VAR>?Key</VAR>, <VAR>?Val</VAR>)</CODE>
<DD>
<A NAME="IDX958"></A>
<VAR>Key</VAR> is the smallest key in <VAR>Assoc</VAR> and <VAR>Val</VAR> is its value.

<DT><CODE>max_assoc(<VAR>+Assoc</VAR>, <VAR>?Key</VAR>, <VAR>?Val</VAR>)</CODE>
<DD>
<A NAME="IDX959"></A>
<VAR>Key</VAR> is the greatest key in <VAR>Assoc</VAR> and <VAR>Val</VAR> is its value.

<DT><CODE>gen_assoc(<VAR>?Key</VAR>, <VAR>+Assoc</VAR>, <VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX960"></A>
<VAR>Key</VAR> is associated with <VAR>Value</VAR> in the association tree
<VAR>Assoc</VAR>.  Can be used to enumerate all <VAR>Values</VAR> by ascending
<VAR>Keys</VAR>.

<DT><CODE>get_assoc(<VAR>+Key</VAR>, <VAR>+Assoc</VAR>, <VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX961"></A>
<VAR>Key</VAR> is identical (==) to one of the keys in the association tree
<VAR>Assoc</VAR>, and <VAR>Value</VAR> unifies with the associated value.

<DT><CODE>get_assoc(<VAR>+Key</VAR>, <VAR>+OldAssoc</VAR>, <VAR>?OldValue</VAR>, <VAR>?NewAssoc</VAR>, <VAR>?NewValue</VAR>)</CODE>
<DD>
<A NAME="IDX962"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> are association trees of the same
shape having the same elements except that the value for <VAR>Key</VAR> in
<VAR>OldAssoc</VAR> is <VAR>OldValue</VAR> and the value for <VAR>Key</VAR> in
<VAR>NewAssoc</VAR> is <VAR>NewValue</VAR>.

<DT><CODE>get_next_assoc(<VAR>+Key</VAR>, <VAR>+Assoc</VAR>, <VAR>?Knext</VAR>, <VAR>?Vnext</VAR>)</CODE>
<DD>
<A NAME="IDX963"></A>
<VAR>Knext</VAR> and <VAR>Vnext</VAR> is the next key and associated value after
<VAR>Key</VAR> in <VAR>Assoc</VAR>.

<DT><CODE>get_prev_assoc(<VAR>+Key</VAR>, <VAR>+Assoc</VAR>, <VAR>?Kprev</VAR>, <VAR>?Vprev</VAR>)</CODE>
<DD>
<A NAME="IDX964"></A>
<VAR>Kprev</VAR> and <VAR>Vprev</VAR> is the previous key and associated value
after <VAR>Key</VAR> in <VAR>Assoc</VAR>.

<DT><CODE>list_to_assoc(<VAR>+List</VAR>, <VAR>?Assoc</VAR>)</CODE>
<DD>
<A NAME="IDX965"></A>
<VAR>List</VAR> is a proper list of <VAR>Key-Value</VAR> pairs (in any order) and
<VAR>Assoc</VAR> is an association tree specifying the same finite function
from <VAR>Keys</VAR> to <VAR>Values</VAR>.

<DT><CODE>ord_list_to_assoc(<VAR>+List</VAR>, <VAR>?Assoc</VAR>)</CODE>
<DD>
<A NAME="IDX966"></A>
<VAR>List</VAR> is a proper list of <VAR>Key-Value</VAR> pairs (keysorted) and
<VAR>Assoc</VAR> is an association tree specifying the same finite function
from <VAR>Keys</VAR> to <VAR>Values</VAR>.

<DT><CODE>map_assoc(<VAR>:Pred</VAR>, <VAR>?Assoc</VAR>)</CODE>
<DD>
<A NAME="IDX967"></A>
<VAR>Assoc</VAR> is an association tree, and for each <VAR>Key</VAR>, if <VAR>Key</VAR>
is associated with <VAR>Value</VAR> in <VAR>Assoc</VAR>, <VAR>Pred(Value)</VAR> is
true.

<DT><CODE>map_assoc(<VAR>:Pred</VAR>, <VAR>?OldAssoc</VAR>, <VAR>?NewAssoc</VAR>)</CODE>
<DD>
<A NAME="IDX968"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> are association trees of the same
shape, and for each <VAR>Key</VAR>, if <VAR>Key</VAR> is associated with <VAR>Old</VAR>
in <VAR>OldAssoc</VAR> and with <VAR>New</VAR> in <VAR>NewAssoc</VAR>,
<VAR>Pred(Old,New)</VAR> is true.

<DT><CODE>put_assoc(<VAR>+Key</VAR>, <VAR>+OldAssoc</VAR>, <VAR>+Val</VAR>, <VAR>?NewAssoc</VAR>)</CODE>
<DD>
<A NAME="IDX969"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> define the same finite function,
except that <VAR>NewAssoc</VAR> associates Val with <VAR>Key</VAR>.
<VAR>OldAssoc</VAR> need not have associated any value at all with <VAR>Key</VAR>.

<DT><CODE>del_assoc(<VAR>+Key</VAR>, <VAR>+OldAssoc</VAR>, <VAR>?Val</VAR>, <VAR>?NewAssoc</VAR>)</CODE>
<DD>
<A NAME="IDX970"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> define the same finite function except
that <VAR>OldAssoc</VAR> associates <VAR>Key</VAR> with <VAR>Val</VAR> and
<VAR>NewAssoc</VAR> doesn't associate <VAR>Key</VAR> with any value.

<DT><CODE>del_min_assoc(<VAR>+OldAssoc</VAR>, <VAR>?Key</VAR>, <VAR>?Val</VAR>, <VAR>?NewAssoc</VAR>)</CODE>
<DD>
<A NAME="IDX971"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> define the same finite function except
that <VAR>OldAssoc</VAR> associates <VAR>Key</VAR> with <VAR>Val</VAR> and
<VAR>NewAssoc</VAR> doesn't associate <VAR>Key</VAR> with any value and <VAR>Key</VAR>
precedes all other keys in <VAR>OldAssoc</VAR>.

<DT><CODE>del_max_assoc(<VAR>+OldAssoc</VAR>, <VAR>?Key</VAR>, <VAR>?Val</VAR>, <VAR>-NewAssoc</VAR>)</CODE>
<DD>
<A NAME="IDX972"></A>
<VAR>OldAssoc</VAR> and <VAR>NewAssoc</VAR> define the same finite function except
that <VAR>OldAssoc</VAR> associates <VAR>Key</VAR> with <VAR>Val</VAR> and
<VAR>NewAssoc</VAR> doesn't associate <VAR>Key</VAR> with any value and <VAR>Key</VAR>
is preceded by all other keys in <VAR>OldAssoc</VAR>.
</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_15.html">previous</A>, <A HREF="sicstus_17.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
