<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Attributed Variables</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_16.html">previous</A>, <A HREF="sicstus_18.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC191" HREF="sicstus_toc.html#TOC191">Attributed Variables</A></H1>
<P>
<A NAME="IDX973"></A>
<A NAME="IDX974"></A>

</P>
<P>
This package implements attributed variables.  It provides a means of
associating with variables arbitrary attributes, i.e. named properties
that can be used as storage locations as well as to extend the default
unification algorithm when such variables are unified with other terms
or with each other.
This facility was primarily designed as a clean interface between Prolog
and constraint solvers, but has a number of other uses as well.  The
basic idea is due to Christian Holzbaur and he was actively involved in
the final design.  For background material, see the dissertation
[Holzbaur 90].

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(atts)).
</PRE>

<P>
The package provides a means to declare and access named attributes of
variables.  The attributes are compound terms whose arguments are
the actual attribute values.  The attribute names are
<EM>private</EM> to the module in which they are defined.  They are
defined with a declaration

</P>
<P>
<A NAME="IDX975"></A>
<A NAME="IDX976"></A>
<A NAME="IDX977"></A>

</P>

<PRE>
:- attribute <VAR>AttributeSpec</VAR>, ..., <VAR>AttributeSpec</VAR>.
</PRE>

<P>
where each <VAR>AttributeSpec</VAR> has the form (<VAR>Name</VAR>/<VAR>Arity</VAR>).
There must be at most one such declaration in a module <VAR>Module</VAR>.

</P>
<P>
Having declared some attribute names, these attributes can now be added,
updated and deleted from unbound variables.  For each declared attribute
name, any variable can have at most one such attribute (initially it has
none).  

</P>
<P>
The declaration causes the following two access predicates to become
defined by means of the <CODE>user:goal_expansion/3</CODE> mechanism.  They take a
variable and an <VAR>AccessSpec</VAR> as arguments where an <VAR>AccessSpec</VAR>
is either <CODE>+(<VAR>Attribute</VAR>)</CODE>, <CODE>-(<VAR>Attribute</VAR>)</CODE>,
or a list of such.
The <KBD>+</KBD> prefix may be dropped for convenience.
The meaning of the <KBD>+</KBD>/<KBD>-</KBD> prefix is documented below:

</P>
<DL COMPACT>

<DT><CODE><VAR>Module</VAR>:get_atts(<VAR>-Var</VAR>, <VAR>?AccessSpec</VAR>)</CODE>
<DD>
<A NAME="IDX978"></A>
 
Gets the attributes of <VAR>Var</VAR> according to <VAR>AccessSpec</VAR>.  If
<VAR>AccessSpec</VAR> is unbound, it will be bound to a list of all set
attributes of <VAR>Var</VAR>.  Non-variable terms cause a type error to be
raised.  The prefixes in the <VAR>AccessSpec</VAR> have the following meaning:
<DL COMPACT>

<DT><CODE>+(<VAR>Attribute</VAR>)</CODE>
<DD>
The corresponding actual attribute must be present and is unified with
<VAR>Attribute</VAR>.

<DT><CODE>-(<VAR>Attribute</VAR>)</CODE>
<DD>
The corresponding actual attribute must be absent.  The arguments
of <VAR>Attribute</VAR> are ignored, only the name and arity are relevant.
</DL>

<A NAME="IDX979"></A>
<DT><CODE><VAR>Module</VAR>:put_atts(<VAR>-Var</VAR>, <VAR>+AccessSpec</VAR>)</CODE>
<DD>
Sets the attributes of <VAR>Var</VAR> according to <VAR>AccessSpec</VAR>.
Non-variable terms cause a type error to be raised.
The effects of <CODE>put_atts/2</CODE> are undone on backtracking.
<DL COMPACT>

<DT><CODE>+(<VAR>Attribute</VAR>)</CODE>
<DD>
The corresponding actual attribute is set to <VAR>Attribute</VAR>.
If the actual attribute was already present, it is simply replaced.

<DT><CODE>-(<VAR>Attribute</VAR>)</CODE>
<DD>
The corresponding actual attribute is removed.
If the actual attribute was already absent, nothing happens.
</DL>
</DL>

<P>
A module that contains an attribute declaration has an opportunity to
extend the default unification algorithm by defining the following
predicate:

</P>
<DL COMPACT>

<DT><CODE><VAR>Module</VAR>:verify_attributes(<VAR>-Var</VAR>, <VAR>+Value</VAR>, <VAR>-Goals</VAR>)</CODE>
<DD>
<A NAME="IDX980"></A>
 
<EM>A hook predicate.</EM> 
This predicate is called whenever a variable <VAR>Var</VAR> that might have
attributes in <VAR>Module</VAR> is about to be bound to <VAR>Value</VAR> (it might
have none).  The unification resumes after the call to
<CODE>verify_attributes/3</CODE>.  <VAR>Value</VAR> is a non-variable term, or
another attributed variable.  <VAR>Var</VAR> might have no attributes present
in <VAR>Module</VAR>; the unification extension mechanism is not
sophisticated enough to filter out exactly the variables that are
relevant for <VAR>Module</VAR>.

<CODE>verify_attributes/3</CODE> is called <EM>before</EM> <VAR>Var</VAR> has
actually been bound to <VAR>Value</VAR>.  If it fails, the unification is
deemed to have failed.  It may succeed non-deterministically, in which
case the unification might backtrack to give another answer.  It is
expected to return, in <VAR>Goals</VAR>, a list of goals to be called
<EM>after</EM> <VAR>Var</VAR> has been bound to <VAR>Value</VAR>.

<CODE>verify_attributes/3</CODE> may invoke arbitrary Prolog goals, but
<VAR>Var</VAR> should <EM>not</EM> be bound by it.  Binding <VAR>Var</VAR> will result
in undefined behavior.

If <VAR>Value</VAR> is a non-variable term, <CODE>verify_attributes/3</CODE> will
typically inspect the attributes of <VAR>Var</VAR> and check that they are
compatible with <VAR>Value</VAR> and fail otherwise.  If <VAR>Value</VAR> is
another attributed variable, <CODE>verify_attributes/3</CODE> will typically
copy the attributes of <VAR>Var</VAR> over to <VAR>Value</VAR>, or merge them with
<VAR>Value</VAR>'s, in preparation for <VAR>Var</VAR> to be bound to <VAR>Value</VAR>.
In either case, <CODE>verify_attributes/3</CODE> may determine <VAR>Var</VAR>'s
current attributes by calling <CODE>get_atts(<VAR>Var</VAR>,<VAR>List</VAR>)</CODE> with
an unbound <VAR>List</VAR>.
</DL>

<P>
An important use for attributed variables is in implementing coroutining
facilities as an alternative or complement to the built-in coroutining
mechanisms.  In this context it might be useful to be able to interpret
some of the attributes of a variable as a goal that is blocked on that
variable.  Certain built-in predicates (<CODE>frozen/2</CODE>,
<CODE>call_residue/2</CODE>) and the Prolog top level need to access blocked
goals, and so need a means of getting the goal interpretation of
attributed variables by calling:

</P>
<DL COMPACT>

<DT><CODE><VAR>Module</VAR>:attribute_goal(<VAR>-Var</VAR>, <VAR>-Goal</VAR>)</CODE>
<DD>
<A NAME="IDX981"></A>
 
<EM>A hook predicate.</EM> This predicate is called in each module that
contains an attribute declaration, when an interpretation of the
attributes as a goal is needed.  It should unify <VAR>Goal</VAR> with the
interpretation, or merely fail if no such interpretation is
available.
</DL>

<P>
An important use for attributed variables is to provide an interface to
constraint solvers.  An important function for a constraint solver in
the constraint logic programming paradigm is to be able to perform
projection of the residual constraints onto the variables that occurred
in the top-level query.  
A module that contains an attribute declaration has an opportunity to
perform such projection of its residual constraints by defining the
following predicate:

</P>
<DL COMPACT>

<DT><CODE><VAR>Module</VAR>:project_attributes(<VAR>+QueryVars</VAR>, <VAR>+AttrVars</VAR>)</CODE>
<DD>
<A NAME="IDX982"></A>
 
<EM>A hook predicate.</EM> This predicate is called by the Prolog top
level and by the built-in predicate <CODE>call_residue/2</CODE> in each module
that contains an attribute declaration.  <VAR>QueryVars</VAR> is the list of
variables occurring in the query, or in terms bound to such variables,
and <VAR>AttrVars</VAR> is a list of possibly attributed variables created
during the execution of the query.  The two lists of variables may or
may not be disjoint.

If the attributes on <VAR>AttrVars</VAR> can be interpreted as constraints,
this predicate will typically "project" those constraints onto the
relevant <VAR>QueryVars</VAR>.  Ideally, the residual constraints will be
expressed entirely in terms of the <VAR>QueryVars</VAR>, treating all other
variables as existentially quantified.  Operationally,
<CODE>project_attributes/2</CODE> must remove all attributes from
<VAR>AttrVars</VAR>, and add transformed attributes representing the
projected constraints to some of the <VAR>QueryVars</VAR>.

Projection has the following effect on the Prolog top level.  When the
top level query has succeeded, <CODE>project_attributes/2</CODE> is called
first.  The top level then prints the answer substition and residual
constraints.  While doing so, it searches for attributed variables
created during the execution of the query.  For each such variable, it
calls <CODE>attribute_goal/2</CODE> to get a printable representation of the
constraint encoded by the attribute.  Thus, <CODE>project_attributes/2</CODE>
is a mechanism for controlling how the residual constraints should
be displayed at top level.

Similarly during the execution of
<CODE>call_residue(<VAR>Goal</VAR>,<VAR>Residue</VAR>)</CODE>, when <VAR>Goal</VAR> has
succeeded, <CODE>project_attributes/2</CODE> is called.  After that, all
attributed variables created during the execution of <VAR>Goal</VAR> are
located.  For each such variable, <CODE>attribute_goal/2</CODE> produces a
term representing the constraint encoded by the attribute, and <VAR>Residue</VAR>
is unified with the list of all such terms.

The exact definition of <CODE>project_attributes/2</CODE> is constraint system
dependent, but see section <A HREF="sicstus_32.html#SEC233">Projection and Redundancy Elimination</A> for details about projection in clp(Q,R).
</DL>

<P>
In the following example we sketch the implementation of a finite
domain "solver".  Note that an industrial strength solver would have
to provide a wider range of functionality and that it quite likely
would utilize a more efficient representation for the domains proper.
The module exports a single predicate
<CODE>domain(<VAR>-Var</VAR>,<VAR>?Domain</VAR>)</CODE> which associates <VAR>Domain</VAR>
(a list of terms) with <VAR>Var</VAR>.  A variable can be queried for its
domain by leaving <VAR>Domain</VAR> unbound.

</P>
<P>
We do not present here a 
definition for <CODE>project_attributes/2</CODE>.  Projecting finite domain
constraints happens to be difficult.

</P>


<PRE>
:- module(domain, [domain/2]).

:- use_module(library(atts)).
:- use_module(library(ordsets), [
        ord_intersection/3,
        ord_intersect/2,
        list_to_ord_set/2
   ]).

:- attribute dom/1.

verify_attributes(Var, Other, Goals) :-
        get_atts(Var, dom(Da)), !,          % are we involved?
        (   var(Other) -&#62;                   % must be attributed then
            (   get_atts(Other, dom(Db)) -&#62; %   has a domain?
                ord_intersection(Da, Db, Dc),
                Dc = [El|Els],              % at least one element
                (   Els = [] -&#62;             % exactly one element
                    Goals = [Other=El]      % implied binding
                ;   Goals = [],
                    put_atts(Other, dom(Dc))% rescue intersection
                )
            ;   Goals = [],
                put_atts(Other, dom(Da))    % rescue the domain
            )
        ;   Goals = [],
            ord_intersect([Other], Da)      % value in domain?
        ).
verify_attributes(_, _, []).                % unification triggered
                                            % because of attributes
                                            % in other modules

attribute_goal(Var, domain(Var,Dom)) :-     % interpretation as goal
        get_atts(Var, dom(Dom)).

domain(X, Dom) :-
        var(Dom), !,
        get_atts(X, dom(Dom)).
domain(X, List) :-
        list_to_ord_set(List, Set),
        Set = [El|Els],                     % at least one element
        (   Els = [] -&#62;                     % exactly one element
            X = El                          % implied binding
        ;   put_atts(Fresh, dom(Set)),
            X = Fresh                       % may call
                                            % verify_attributes/3
        ).
</PRE>

<P>
Note that the "implied binding" <CODE>Other=El</CODE> was deferred until after
the completion of <CODE>verify_attribute/3</CODE>.  Otherwise, there might be a
danger of recursively invoke <CODE>verify_attribute/3</CODE>, which might bind
<CODE>Var</CODE>, which is not allowed inside the scope of <CODE>verify_attribute/3</CODE>.
Deferring unifications into the third argument of <CODE>verify_attribute/3</CODE>
effectively serializes th calls to <CODE>verify_attribute/3</CODE>.

</P>
<P>
Assuming that the code resides in the file <TT>`domain.pl'</TT>, we
can use it via:

</P>

<PRE>
| ?- use_module(domain).
</PRE>

<P>
Let's test it:

</P>

<PRE>
| ?- domain(X,[5,6,7,1]), domain(Y,[3,4,5,6]), domain(Z,[1,6,7,8]).

domain(X,[1,5,6,7]),
domain(Y,[3,4,5,6]),
domain(Z,[1,6,7,8]) ? 

yes
| ?- domain(X,[5,6,7,1]), domain(Y,[3,4,5,6]), domain(Z,[1,6,7,8]), 
     X=Y.

Y = X,
domain(X,[5,6]),
domain(Z,[1,6,7,8]) ? 

yes
| ?- domain(X,[5,6,7,1]), domain(Y,[3,4,5,6]), domain(Z,[1,6,7,8]),
     X=Y, Y=Z.

X = 6,
Y = 6,
Z = 6
</PRE>

<P>
To demonstrate the use of the <VAR>Goals</VAR> argument of
<CODE>verify_attributes/3</CODE>, we give an implementation of
<CODE>freeze/2</CODE>.  We have to name it <CODE>myfreeze/2</CODE> in order to
avoid a name clash with the built-in predicate of the same name.

</P>

<PRE>
:- module(myfreeze, [myfreeze/2]).

:- use_module(library(atts)).

:- attribute frozen/1.

verify_attributes(Var, Other, Goals) :-
        get_atts(Var, frozen(Fa)), !,       % are we involved?
        (   var(Other) -&#62;                   % must be attributed then
            (   get_atts(Other, frozen(Fb)) % has a pending goal?
            -&#62;  put_atts(Other, frozen((Fa,Fb))) % rescue conjunction
            ;   put_atts(Other, frozen(Fa)) % rescue the pending goal
            ),
            Goals = []
        ;   Goals = [Fa]
        ).
verify_attributes(_, _, []).

attribute_goal(Var, Goal) :-                % interpretation as goal
        get_atts(Var, frozen(Goal)).

myfreeze(X, Goal) :-
        put_atts(Fresh, frozen(Goal)),
        Fresh = X.
</PRE>

<P>
Assuming that this code lives in file <TT>`myfreeze.pl'</TT>,
we would use it via:

</P>

<PRE>
| ?- use_module(myfreeze).
| ?- myfreeze(X,print(bound(x,X))), X=2.

bound(x,2)                      % side effect
X = 2                           % bindings
</PRE>

<P>
The two solvers even work together:

</P>

<PRE>
| ?- myfreeze(X,print(bound(x,X))), domain(X,[1,2,3]),
     domain(Y,[2,10]), X=Y.

bound(x,2)                      % side effect
X = 2,                          % bindings
Y = 2
</PRE>

<P>
The two example solvers interact via bindings to shared attributed
variables only.  More complicated interactions are likely to be found
in more sophisticated solvers.  The corresponding
<CODE>verify_attributes/3</CODE> predicates would typically refer to the
attributes from other known solvers/modules via the module prefix in
<CODE><VAR>Module</VAR>:get_atts/2</CODE>.

</P>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_16.html">previous</A>, <A HREF="sicstus_18.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
