<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Heap Operations</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_17.html">previous</A>, <A HREF="sicstus_19.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC192" HREF="sicstus_toc.html#TOC192">Heap Operations</A></H1>
<P>
<A NAME="IDX983"></A>
<A NAME="IDX984"></A>

</P>
<P>
A binary heap is a tree with keys and associated values that satisfies
the <EM>heap codition</EM>: the key of every node is greater than or
equal to the key of its parent, if it has one.  The main application
of binary heaps are priority queues.  To load the package, enter the
query

</P>

<PRE>
| ?- use_module(library(heaps)).
</PRE>

<DL COMPACT>

<DT><CODE>add_to_heap(<VAR>+OldHeap</VAR>, <VAR>+Key</VAR>, <VAR>+Datum</VAR>, <VAR>?NewHeap</VAR>)</CODE>
<DD>
<A NAME="IDX985"></A>
Inserts the new <VAR>Key</VAR>-<VAR>Datum</VAR> pair into the current heap
<VAR>OldHeap</VAR> producing the new heap <VAR>NewHeap</VAR>.  The insertion is
not stable, that is, if you insert several pairs with the same
<VAR>Key</VAR> it is not defined which of them will come out first, and it
is possible for any of them to come out first depending on the 
history of the heap.  Example:

<PRE>
| ?- add_to_heap(t(0,[],t),3,678,N).

N = t(1,[],t(3,678,t,t)) ? 

yes
</PRE>

<DT><CODE>get_from_heap(<VAR>+OldHeap</VAR>, <VAR>?Key</VAR>, <VAR>?Datum</VAR>, <VAR>?NewHeap</VAR>)</CODE>
<DD>
<A NAME="IDX986"></A>
Returns the <VAR>Key</VAR>-<VAR>Datum</VAR> pair in <VAR>OldHeap</VAR> with the
smallest <VAR>Key</VAR>, and also a <VAR>NewHeap</VAR> which is the <VAR>OldHeap</VAR>
with that pair deleted.  Example:

<PRE>
get_from_heap(t(1,[],t(1,543,t,t)),K,D,N).

D = 543,
K = 1,
N = t(0,[1],t) ? 

yes
</PRE>

<DT><CODE>empty_heap(<VAR>?Heap</VAR>)</CODE>
<DD>
<A NAME="IDX987"></A>
is true when <VAR>Heap</VAR> is the empty heap.

<DT><CODE>heap_size(<VAR>+Heap</VAR>, <VAR>?Size</VAR>)</CODE>
<DD>
<A NAME="IDX988"></A>
<VAR>Size</VAR> is the number of elements in the heap <VAR>Heap</VAR>.

<DT><CODE>heap_to_list(<VAR>+Heap</VAR>, <VAR>-List</VAR>)</CODE>
<DD>
<A NAME="IDX989"></A>
Returns the current set of <VAR>Key</VAR>-<VAR>Datum</VAR> pairs in the <VAR>Heap</VAR> as a
keysorted <VAR>List</VAR>.

<DT><CODE>is_heap(<VAR>+Heap</VAR>)</CODE>
<DD>
<A NAME="IDX990"></A>
is true when <VAR>Heap</VAR> is a valid heap.

<DT><CODE>list_to_heap(<VAR>+List</VAR>, <VAR>-Heap</VAR>)</CODE>
<DD>
<A NAME="IDX991"></A>
Takes a list <VAR>List</VAR> of <VAR>Key</VAR>-<VAR>Datum</VAR> pairs and forms them
into a heap <VAR>Heap</VAR>.  Example:

<PRE>
| ?- list_to_heap([1-34,2-345,5-678],H).

H = t(3,[],t(1,34,t(2,345,t,t),t(5,678,t,t))) ? 

yes
</PRE>

<DT><CODE>min_of_heap(<VAR>+Heap</VAR>, <VAR>?Key</VAR>, <VAR>?Datum</VAR>)</CODE>
<DD>
<A NAME="IDX992"></A>
Returns the <VAR>Key</VAR>-<VAR>Datum</VAR> pair at the top of the heap <VAR>Heap</VAR>
without removing it.  Fails if the heap is empty.

 
<DT><CODE>min_of_heap(<VAR>+Heap</VAR>, <VAR>?Key1</VAR>, <VAR>?Datum1</VAR>, <VAR>?Key2</VAR>, <VAR>?Datum2</VAR>)</CODE>
<DD>
<A NAME="IDX993"></A>
Returns the smallest (<VAR>Key1</VAR>-<VAR>Datum1</VAR>) and second smallest
(<VAR>Key2</VAR>-<VAR>Datum2</VAR>) pairs in the <VAR>Heap</VAR>, without deleting
them.  It fails if the heap does not have at least two elements.

<DT><CODE>delete_from_heap(<VAR>+OldHeap</VAR>, <VAR>+Key</VAR>, <VAR>?Datum</VAR>, <VAR>?NewHeap</VAR>)</CODE>
<DD>
<A NAME="IDX994"></A>
deletes a single <VAR>Key</VAR>-<VAR>Datum</VAR> pair in <VAR>OldHeap</VAR> producing
<VAR>NewHeap</VAR>.  This is useful if you want to e.g. change
the priority of <VAR>Datum</VAR>.  Beware: this operation needs to search
the whole heap in the worst case.

</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_17.html">previous</A>, <A HREF="sicstus_19.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
