<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Term Utilities</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_19.html">previous</A>, <A HREF="sicstus_21.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC194" HREF="sicstus_toc.html#TOC194">Term Utilities</A></H1>
<P>
<A NAME="IDX1022"></A>
<A NAME="IDX1023"></A>
<A NAME="IDX1024"></A>
<A NAME="IDX1025"></A>

</P>
<P>
This package defines operations on terms for subsumption checking,
"anti-unification", unification with occurs-check, testing
acyclicity, and getting the variables.

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(terms)).
</PRE>

<DL COMPACT>

<DT><CODE>subsumes_chk(<VAR>?General</VAR>, <VAR>?Specific</VAR>)</CODE>
<DD>
<A NAME="IDX1026"></A>
<VAR>Specific</VAR> is an instance of <VAR>General</VAR>, i.e. if there
is a substitution that leaves <VAR>Specific</VAR> unchanged and makes
<VAR>General</VAR> identical to <VAR>Specific</VAR>.  It doesn't bind any
variables.

<PRE>
subsumes_chk(f(X), f(a)).

true

| ?- subsumes_chk(f(a), f(X)).

no

| ?- subsumes_chk(A-A, B-C).

no

| ?- subsumes_chk(A-B, C-C).

true
</PRE>

<DT><CODE>subsumes(<VAR>?General</VAR>, <VAR>?Specific</VAR>)</CODE>
<DD>
<A NAME="IDX1027"></A>
<VAR>Specific</VAR> is an instance of <VAR>General</VAR>.  It will bind
variables in <VAR>General</VAR> (but not those in <VAR>Specific</VAR>) so that
<VAR>General</VAR> becomes identical to <VAR>Specific</VAR>.

<DT><CODE>variant(<VAR>?Term</VAR>, <VAR>?Variant</VAR>)</CODE>
<DD>
<A NAME="IDX1028"></A>
<VAR>Term</VAR> and <VAR>Variant</VAR> are identical modulo renaming of variables,
provided <VAR>Term</VAR> and <VAR>Variant</VAR> have no variables in common.

<DT><CODE>term_subsumer(<VAR>?Term1</VAR>, <VAR>?Term2</VAR>, <VAR>?General</VAR>)</CODE>
<DD>
<A NAME="IDX1029"></A>
<VAR>General</VAR> is the most specific term that generalizes
<VAR>Term1</VAR> and <VAR>Term2</VAR>.  This process is sometimes called
<EM>anti-unification</EM>, as it is the dual of unification.

<PRE>
| ?- term_subsumer(f(g(1,h(_))), f(g(_,h(1))), T).

T = f(g(_B,h(_A)))

| ?- term_subsumer(f(1+2,2+1), f(3+4,4+3), T).

T = f(_A+_B,_B+_A)

</PRE>

<DT><CODE>term_hash(<VAR>?Term</VAR>, <VAR>?Hash</VAR>)</CODE>
<DD>
<DT><CODE>term_hash(<VAR>?Term</VAR>, <VAR>+Depth</VAR>, <VAR>+Range</VAR>, <VAR>?Hash</VAR>)</CODE>
<DD>
<A NAME="IDX1030"></A>
<A NAME="IDX1031"></A>
If <VAR>Term</VAR> is instantiated up to the given <VAR>Depth</VAR>, 
an integer hash value in the range [0,<VAR>Range</VAR>) as a function
of <VAR>Term</VAR> is unified with <VAR>Hash</VAR>.  Otherwise, the goal
just succeeds, leaving <VAR>Hash</VAR> uninstantiated.

If <VAR>Term</VAR> contains floats or integers outside the range
<CODE>[-33554432,33554431]</CODE>, the hash value will be platform dependent.
Otherwise, the hash value will be identical across runs and platforms.

The <EM>depth</EM> of a term is defined as follows: the (principal functor
of) the term itself has depth 1, and an argument of a term with depth
<VAR>i</VAR> has depth <VAR>i+1</VAR>.

<VAR>Depth</VAR> should be an integer &#62;= -1.  If <VAR>Depth</VAR> = -1 (the default),
<VAR>Term</VAR> must be ground, and all subterms of <VAR>Term</VAR> are relevant
in computing <VAR>Hash</VAR>.  Otherwise, only the subterms up to depth
<VAR>Depth</VAR> of <VAR>Term</VAR> are used in the computation.

<VAR>Range</VAR> should be an integer &#62;= 1.  The default will give hash values
in a range appropriate for all platforms.


<PRE>
| ?- term_hash([a,b,_], 3, 4, H).

H = 2

| ?- term_hash([a,b,_], 4, 4, H).

true

| ?- term_hash(f(a,f(b,f(_,[]))), 2, 4, H).

H = 2
</PRE>

<CODE>term_hash/(2,4)</CODE> is provided primarily as a tool for the construction
of sophisticated Prolog clause access schemes.  Its intended use is to
generate hash values for terms that will be used with first argument
clause indexing, yielding compact and efficient multi-argument or deep
argument indexing.

<DT><CODE>term_variables(<VAR>?Term</VAR>, <VAR>?Variables</VAR>)</CODE>
<DD>
<A NAME="IDX1032"></A>
<VAR>Variables</VAR> is the set of variables occurring in <VAR>Term</VAR>.

<DT><CODE>unify_with_occurs_check(<VAR>?X</VAR>, <VAR>?Y</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX1033"></A>
True if <VAR>X</VAR> and <VAR>Y</VAR> unify to a finite (acyclic) term.
Runs in almost linear time.

<DT><CODE>acyclic_term(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX1034"></A>
True if <VAR>X</VAR> is finite (acyclic).  Runs in linear time.

<DT><CODE>cyclic_term(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX1035"></A>
True if <VAR>X</VAR> is infinite (cyclic).  Runs in linear time.
</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_19.html">previous</A>, <A HREF="sicstus_21.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
