<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Ordered Set Operations</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_20.html">previous</A>, <A HREF="sicstus_22.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC195" HREF="sicstus_toc.html#TOC195">Ordered Set Operations</A></H1>
<P>
<A NAME="IDX1036"></A>
<A NAME="IDX1037"></A>

</P>
<P>
This package defines operations on ordered sets.  Ordered sets are sets
represented as lists with the elements ordered in a standard order.
The ordering is defined by the <CODE>@&#60;</CODE> family of term comparison
predicates and it is the ordering produced by the built-in predicate
<CODE>sort/2</CODE> (see section <A HREF="sicstus_10.html#SEC102">Comparison of Terms</A>).

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(ordsets)).
</PRE>

<DL COMPACT>

<DT><CODE>is_ordset(<VAR>+Set</VAR>)</CODE>
<DD>
<A NAME="IDX1038"></A>
<VAR>Set</VAR> is an ordered set.

<DT><CODE>list_to_ord_set(<VAR>+List</VAR>, <VAR>?Set</VAR>)</CODE>
<DD>
<A NAME="IDX1039"></A>
<VAR>Set</VAR> is the ordered representation of the set denoted by the
unordered representation <VAR>List</VAR>.  Example:

<PRE>
| ?- list_to_ord_set([p,r,o,l,o,g], P).

P = [g,l,o,p,r] ? 

yes

</PRE>

<DT><CODE>ord_add_element(<VAR>+Set1</VAR>, <VAR>+Element</VAR> <VAR>?Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1040"></A>
<VAR>Set2</VAR> is <VAR>Set1</VAR> with <VAR>Element</VAR> inserted in it, preserving
the order.  Example:

<PRE>
| ?- ord_add_element([a,c,d,e,f], b, N).

N = [a,b,c,d,e,f] ? 

yes
</PRE>

<DT><CODE>ord_del_element(<VAR>+Set1</VAR>, <VAR>+Element</VAR>, <VAR>?Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1041"></A>
<VAR>Set2</VAR> is like <VAR>Set1</VAR> but with <VAR>Element</VAR> removed.

<DT><CODE>ord_disjoint(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1042"></A>
The two ordered sets have no elements in common.  

<DT><CODE>ord_intersect(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1043"></A>
The two ordered sets have at least one element in common.

<DT><CODE>ord_intersection(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Intersect</VAR>)</CODE>
<DD>
<A NAME="IDX1044"></A>
<VAR>Intersect</VAR> is the ordered set representation of the intersection
between <VAR>Set1</VAR> and <VAR>Set2</VAR>.

<DT><CODE>ord_intersection(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Intersect</VAR>, <VAR>?Diff</VAR>)</CODE>
<DD>
<A NAME="IDX1045"></A>
<VAR>Intersect</VAR> is the intersection between <VAR>Set1</VAR> and <VAR>Set2</VAR>,
and <VAR>Diff</VAR> is the difference between <VAR>Set2</VAR> and <VAR>Set1</VAR>.

<DT><CODE>ord_intersection(<VAR>+Sets</VAR>, <VAR>?Intersection</VAR>)</CODE>
<DD>
<A NAME="IDX1046"></A>
<VAR>Intersection</VAR> is the ordered set representation of the intersection
of all the sets in <VAR>Sets</VAR>.  Example:

<PRE>
| ?- ord_intersection([[1,2,3],[2,3,4],[3,4,5]], I).

I = [3] ? 

yes

</PRE>

<DT><CODE>ord_member(<VAR>+Elt</VAR>, <VAR>+Set</VAR>)</CODE>
<DD>
<A NAME="IDX1047"></A>
is true when <VAR>Elt</VAR> is a member of <VAR>Set</VAR>.

<DT><CODE>ord_seteq(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1048"></A>
Is true when the two arguments represent the same set.  Since they
are assumed to be ordered representations, they must be identical.

<DT><CODE>ord_setproduct(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?SetProduct</VAR>)</CODE>
<DD>
<A NAME="IDX1049"></A>
<VAR>SetProduct</VAR> is the Cartesian Product of the two Sets.  The product
is represented as pairs: Elem1-Elem2 where Elem1 is an element from
<VAR>Set1</VAR> and Elem2 is an element from <VAR>Set2</VAR>.  Example

<PRE>
| ?- ord_setproduct([1,2,3], [4,5,6], P).

P = [1-4,1-5,1-6,2-4,2-5,2-6,3-4,3-5,3-6] ? 

yes
</PRE>

<DT><CODE>ord_subset(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1050"></A>
Every element of the ordered set <VAR>Set1</VAR> appears in the ordered set
<VAR>Set2</VAR>.

<DT><CODE>ord_subtract(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Difference</VAR>)</CODE>
<DD>
<A NAME="IDX1051"></A>
<VAR>Difference</VAR> contains all and only the elements of <VAR>Set1</VAR> which
are not also in <VAR>Set2</VAR>.  Example:

<PRE>
| ?- ord_subtract([1,2,3,4], [3,4,5,6], S).

S = [1,2] ? 

yes 
</PRE>

<DT><CODE>ord_symdiff(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Difference</VAR>)</CODE>
<DD>
<A NAME="IDX1052"></A>
<VAR>Difference</VAR> is the symmetric difference of <VAR>Set1</VAR> and
<VAR>Set2</VAR>.  Example:

<PRE>
| ?- ord_symdiff([1,2,3,4], [3,4,5,6], D).

D = [1,2,5,6] ? 

yes
</PRE>

<DT><CODE>ord_union(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Union</VAR>)</CODE>
<DD>
<A NAME="IDX1053"></A>
<VAR>Union</VAR> is the union of <VAR>Set1</VAR> and <VAR>Set2</VAR>.  

<DT><CODE>ord_union(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>?Union</VAR>, <VAR>?New</VAR>)</CODE>
<DD>
<A NAME="IDX1054"></A>
<VAR>Union</VAR> is the union of <VAR>Set1</VAR> and <VAR>Set2</VAR>, and <VAR>New</VAR> is
the difference between <VAR>Set2</VAR> and <VAR>Set1</VAR>.  This is useful if you
are accumulating members of a set and you want to process new elements
as they are added to the set.

<DT><CODE>ord_union(<VAR>+Sets</VAR>, <VAR>?Union</VAR>)</CODE>
<DD>
<A NAME="IDX1055"></A>
<VAR>Union</VAR> is the union of all the sets in <VAR>Sets</VAR>.  Example:

<PRE>
| ?- ord_union([[1,2,3],[2,3,4],[3,4,5]], U).

U = [1,2,3,4,5] ? 

yes

</PRE>

</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_20.html">previous</A>, <A HREF="sicstus_22.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
