<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Unweighted Graph Operations</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_25.html">previous</A>, <A HREF="sicstus_27.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC200" HREF="sicstus_toc.html#TOC200">Unweighted Graph Operations</A></H1>
<P>
<A NAME="IDX1112"></A>
<A NAME="IDX1113"></A>
<A NAME="IDX1114"></A>

</P>
<P>
Directed and undirected graphs are fundamental data structures
representing arbitrary relationships between data objects.  This package
provides a Prolog implementation of directed graphs, undirected graphs
being a special case of directed graphs.

</P>
<P>
An unweighted directed graph (ugraph) is represented as a list of
(vertex-neighbors) pairs, where the pairs are in standard order (as
produced by <CODE>keysort</CODE> with unique keys) and the neighbors of each
vertex are also in standard order (as produced by <CODE>sort</CODE>),
every neighbor appears as a vertex even if it has no neighbors itself,
and no vertex is a neighbor to itself.

</P>
<P>
An undirected graph is represented as a directed graph where for
each edge <VAR>(U,V)</VAR> there is a symmetric edge <VAR>(V,U)</VAR>.

</P>
<P>
An edge <VAR>(U,V)</VAR> is represented as the term <VAR>U-V</VAR>.
<VAR>U</VAR> and <VAR>V</VAR> must be distinct.

</P>
<P>
A vertex can be any term.  Two vertices are distinct iff they are
not identical (<CODE>==</CODE>).

</P>
<P>
A path from <VAR>u</VAR> to <VAR>v</VAR> is represented as a list of vertices,
beginning with <VAR>u</VAR> and ending with <VAR>v</VAR>.  A vertex cannot appear
twice in a path.  A path is maximal in a graph if it cannot be extended.

</P>
<P>
A tree is a tree-shaped directed graph (all vertices have a single
predecessor, except the root node, which has none).

</P>
<P>
A strongly connected component of a graph is a maximal set of vertices
where each vertex has a path in the graph to every other vertex.

</P>
<P>
Sets are represented as ordered lists (see section <A HREF="sicstus_21.html#SEC195">Ordered Set Operations</A>).

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(ugraphs)).
</PRE>

<P>
The following predicates are defined for directed graphs.

</P>
<DL COMPACT>

<DT><CODE>vertices_edges_to_ugraph(<VAR>+Vertices</VAR>, <VAR>+Edges</VAR>, <VAR>-Graph</VAR>)</CODE>
<DD>
<A NAME="IDX1115"></A>
 
Is true if <VAR>Vertices</VAR> is a list of vertices, <VAR>Edges</VAR> is a list
of edges, and <VAR>Graph</VAR> is a graph built from <VAR>Vertices</VAR> and
<VAR>Edges</VAR>.  <VAR>Vertices</VAR> and <VAR>Edges</VAR> may be in any order.  The
vertices mentioned in <VAR>Edges</VAR> do not have to occur explicitly in
<VAR>Vertices</VAR>.  <VAR>Vertices</VAR> may be used to specify vertices that are
not connected by any edges.

<A NAME="IDX1116"></A>
<DT><CODE>vertices(<VAR>+Graph</VAR>, <VAR>-Vertices</VAR>)</CODE>
<DD>
Unifies <VAR>Vertices</VAR> with the vertices in <VAR>Graph</VAR>.

<A NAME="IDX1117"></A>
<DT><CODE>edges(<VAR>+Graph</VAR>, <VAR>-Edges</VAR>)</CODE>
<DD>
Unifies <VAR>Edges</VAR> with the edges in <VAR>Graph</VAR>.

<A NAME="IDX1118"></A>
<DT><CODE>add_vertices(<VAR>+Graph1</VAR>, <VAR>+Vertices</VAR>, <VAR>-Graph2</VAR>)</CODE>
<DD>
<VAR>Graph2</VAR> is <VAR>Graph1</VAR> with <VAR>Vertices</VAR> added to it.

<A NAME="IDX1119"></A>
<DT><CODE>del_vertices(<VAR>+Graph1</VAR>, <VAR>+Vertices</VAR>, <VAR>-Graph2</VAR>)</CODE>
<DD>
<VAR>Graph2</VAR> is <VAR>Graph1</VAR> with <VAR>Vertices</VAR> and all edges to and
from them removed from it.

<A NAME="IDX1120"></A>
<DT><CODE>add_edges(<VAR>+Graph1</VAR>, <VAR>+Edges</VAR>, <VAR>-Graph2</VAR>)</CODE>
<DD>
<VAR>Graph2</VAR> is <VAR>Graph1</VAR> with <VAR>Edges</VAR> and their "to" and
"from" vertices added to it.

<A NAME="IDX1121"></A>
<DT><CODE>del_edges(<VAR>+Graph1</VAR>, <VAR>+Edges</VAR>, <VAR>-Graph2</VAR>)</CODE>
<DD>
<VAR>Graph2</VAR> is <VAR>Graph1</VAR> with <VAR>Edges</VAR> removed from it.

<A NAME="IDX1122"></A>
<DT><CODE>transpose(<VAR>+Graph</VAR>, <VAR>-Transpose</VAR>)</CODE>
<DD>
<VAR>Transpose</VAR> is the graph computed by replacing each edge <VAR>(u,v)</VAR>
in <VAR>Graph</VAR> by its symmetric edge <VAR>(v,u)</VAR>.  Takes O(N^2) time.

<A NAME="IDX1123"></A>
<A NAME="IDX1124"></A>
<DT><CODE>neighbors(<VAR>+Vertex</VAR>, <VAR>+Graph</VAR>, <VAR>-Neighbors</VAR>)</CODE>
<DD>
<DT><CODE>neighbours(<VAR>+Vertex</VAR>, <VAR>+Graph</VAR>, <VAR>-Neighbors</VAR>)</CODE>
<DD>
<VAR>Vertex</VAR> is a vertex in <VAR>Graph</VAR> and <VAR>Neighbors</VAR> are its
neighbors.

<A NAME="IDX1125"></A>
<DT><CODE>complement(<VAR>+Graph</VAR>, <VAR>-Complement</VAR>)</CODE>
<DD>
<VAR>Complement</VAR> is the complement graph of <VAR>Graph</VAR>, i.e. the graph
that has the same vertices as <VAR>Graph</VAR> but only the edges that are
not in <VAR>Graph</VAR>.

<A NAME="IDX1126"></A>
<DT><CODE>compose(<VAR>+G1</VAR>, <VAR>+G2</VAR>, <VAR>-Composition</VAR>)</CODE>
<DD>
Computes <VAR>Composition</VAR> as the composition of two graphs, which need
not have the same set of vertices.

<A NAME="IDX1127"></A>
<DT><CODE>transitive_closure(<VAR>+Graph</VAR>, <VAR>-Closure</VAR>)</CODE>
<DD>
Computes <VAR>Closure</VAR> as the transitive closure of <VAR>Graph</VAR> in
O(N^3) time.

<A NAME="IDX1128"></A>
<DT><CODE>symmetric_closure(<VAR>+Graph</VAR>, <VAR>-Closure</VAR>)</CODE>
<DD>
Computes <VAR>Closure</VAR> as the symmetric closure of <VAR>Graph</VAR>, i.e.
for each edge <VAR>(u,v)</VAR> in <VAR>Graph</VAR>, add its symmetric edge
<VAR>(v,u)</VAR>.  Takes O(N^2) time.  This is useful for making a directed
graph undirected.

<A NAME="IDX1129"></A>
<DT><CODE>top_sort(<VAR>+Graph</VAR>, <VAR>-Sorted</VAR>)</CODE>
<DD>
Finds a topological ordering of a <VAR>Graph</VAR> and returns the ordering
as a list of <VAR>Sorted</VAR> vertices.  Fails iff no ordering exists, i.e.
iff the graph contains cycles.  Takes O(N^2) time.

<A NAME="IDX1130"></A>
<DT><CODE>max_path(<VAR>+V1</VAR>, <VAR>+V2</VAR>, <VAR>+Graph</VAR>, <VAR>-Path</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<VAR>Path</VAR> is a longest path of cost <VAR>Cost</VAR> from <VAR>V1</VAR> to
<VAR>V2</VAR> in <VAR>Graph</VAR>, there being no cyclic paths from <VAR>V1</VAR> to
<VAR>V2</VAR>.  Takes O(N^2) time.

<A NAME="IDX1131"></A>
<DT><CODE>min_path(<VAR>+V1</VAR>, <VAR>+V2</VAR>, <VAR>+Graph</VAR>, <VAR>-Path</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<VAR>Path</VAR> is a shortest path of cost <VAR>Cost</VAR> from <VAR>V1</VAR> to
<VAR>V2</VAR> in <VAR>Graph</VAR>.  Takes O(N^2) time.

<A NAME="IDX1132"></A>
<DT><CODE>min_paths(<VAR>+Vertex</VAR>, <VAR>+Graph</VAR>, <VAR>-Tree</VAR>)</CODE>
<DD>
<VAR>Tree</VAR> is a tree of all the shortest paths from <VAR>Vertex</VAR> to
every other vertex in <VAR>Graph</VAR>.  This is the single-source shortest
paths problem.

<A NAME="IDX1133"></A>
<DT><CODE>path(<VAR>+Vertex</VAR>, <VAR>+Graph</VAR>, <VAR>-Path</VAR>)</CODE>
<DD>
Given a <VAR>Graph</VAR> and a <VAR>Vertex</VAR> of <VAR>Graph</VAR>, returns a maximal
<VAR>Path</VAR> rooted at <VAR>Vertex</VAR>, enumerating more paths on
backtracking.

<A NAME="IDX1134"></A>
<DT><CODE>reduce(<VAR>+Graph</VAR>, <VAR>-Reduced</VAR>)</CODE>
<DD>
<VAR>Reduced</VAR> is the reduced graph for <VAR>Graph</VAR>.  The vertices of the
reduced graph are the strongly connected components of <VAR>Graph</VAR>.
There is an edge in <VAR>Reduced</VAR> from <VAR>u</VAR> to <VAR>v</VAR> iff there is
an edge in <VAR>Graph</VAR> from one of the vertices in <VAR>u</VAR> to one of the
vertices in <VAR>v</VAR>.

<A NAME="IDX1135"></A>
<DT><CODE>reachable(<VAR>+Vertex</VAR>, <VAR>+Graph</VAR>, <VAR>-Reachable</VAR>)</CODE>
<DD>
Given a <VAR>Graph</VAR> and a <VAR>Vertex</VAR> of <VAR>Graph</VAR>, returns the set
of vertices that are <EM>reachable</EM> from that <VAR>Vertex</VAR>, including
<VAR>Vertex</VAR> itself.  Takes O(N^2) time.

<A NAME="IDX1136"></A>
<DT><CODE>random_ugraph(<VAR>+P</VAR>, <VAR>+N</VAR>, <VAR>-Graph</VAR>)</CODE>
<DD>
Where <VAR>P</VAR> is a probability, unifies <VAR>Graph</VAR> with a random graph
of vertices <VAR>1..N</VAR> where each possible edge is included with
probability <VAR>P</VAR>.
</DL>

<P>
The following predicates are defined for undirected graphs only.

</P>
<DL COMPACT>

<DT><CODE>min_tree(<VAR>+Graph</VAR>, <VAR>-Tree</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<A NAME="IDX1137"></A>
 
<VAR>Tree</VAR> is a spanning tree of <VAR>Graph</VAR> with cost
<VAR>Cost</VAR>, if it exists.

<A NAME="IDX1138"></A>
<DT><CODE>clique(<VAR>+Graph</VAR>, <VAR>+K</VAR>, <VAR>-Clique</VAR>)</CODE>
<DD>
<VAR>Clique</VAR> is a maximal clique (complete subgraph) of <VAR>N</VAR> vertices
of <VAR>Graph</VAR>, where <VAR>N&#62;=K</VAR>.  <VAR>N</VAR> is not necessarily maximal.

<A NAME="IDX1139"></A>
<DT><CODE>independent_set(<VAR>+Graph</VAR>, <VAR>+K</VAR>, <VAR>-Set</VAR>)</CODE>
<DD>
<VAR>Set</VAR> is a maximal independent (unconnected) set of <VAR>N</VAR> vertices
of <VAR>Graph</VAR>, where <VAR>N&#62;=K</VAR>.  <VAR>N</VAR> is not necessarily maximal.

<A NAME="IDX1140"></A>
<A NAME="IDX1141"></A>
<DT><CODE>coloring(<VAR>+Graph</VAR>, <VAR>+K</VAR>, <VAR>-Coloring</VAR>)</CODE>
<DD>
<DT><CODE>colouring(<VAR>+Graph</VAR>, <VAR>+K</VAR>, <VAR>-Coloring</VAR>)</CODE>
<DD>
<VAR>Coloring</VAR> is a mapping from vertices to colors <VAR>1..N</VAR> of
<VAR>Graph</VAR> such that all edges have distinct end colors, where
<VAR>N=&#60;K</VAR>.  The mapping is represented as an ordered list of
<VAR>Vertex</VAR>-<VAR>Color</VAR> pairs.  <VAR>N</VAR> is not necessarily minimal.
</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_25.html">previous</A>, <A HREF="sicstus_27.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
