<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Weighted Graph Operations</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_26.html">previous</A>, <A HREF="sicstus_28.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC201" HREF="sicstus_toc.html#TOC201">Weighted Graph Operations</A></H1>
<P>
<A NAME="IDX1142"></A>
<A NAME="IDX1143"></A>
<A NAME="IDX1144"></A>

</P>
<P>
A weighted directed graph (wgraph) is represented as a list of
(vertex-edgelist) pairs, where the pairs are in standard order (as
produced by <CODE>keysort</CODE> with unique keys), the edgelist is a list of
(neighbor-weight) pair also in standard order (as produced by
<CODE>keysort</CODE> with unique keys), every weight is a nonnegative integer,
every neighbor appears as a vertex even if it has no neighbors itself,
and no vertex is a neighbor to itself.

</P>
<P>
An undirected graph is represented as a directed graph where for
each edge <VAR>(U,V)</VAR> there is a symmetric edge <VAR>(V,U)</VAR>.

</P>
<P>
An edge <VAR>(U,V)</VAR> with weight <VAR>W</VAR> is represented as the term
<VAR>U-(V-W)</VAR>.  <VAR>U</VAR> and <VAR>V</VAR> must be distinct.

</P>
<P>
A vertex can be any term.  Two vertices are distinct iff they are
not identical (<CODE>==</CODE>).

</P>
<P>
A path from <VAR>u</VAR> to <VAR>v</VAR> is represented as a list of vertices,
beginning with <VAR>u</VAR> and ending with <VAR>v</VAR>.  A vertex cannot appear
twice in a path.  A path is maximal in a graph if it cannot be extended.

</P>
<P>
A tree is a tree-shaped directed graph (all vertices have a single
predecessor, except the root node, which has none).

</P>
<P>
A strongly connected component of a graph is a maximal set of vertices
where each vertex has a path in the graph to every other vertex.

</P>
<P>
Sets are represented as ordered lists (see section <A HREF="sicstus_21.html#SEC195">Ordered Set Operations</A>).

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(wgraphs)).
</PRE>

<P>
The following predicates are defined for directed graphs.

</P>
<DL COMPACT>

<DT><CODE>wgraph_to_ugraph(<VAR>+WeightedGraph</VAR>, <VAR>-Graph</VAR>)</CODE>
<DD>
<A NAME="IDX1145"></A>
 
<VAR>Graph</VAR> has the same vertices and edges as <VAR>WeightedGraph</VAR>,
except the edges of <VAR>Graph</VAR> are unweighted.

<A NAME="IDX1146"></A>
<DT><CODE>ugraph_to_wgraph(<VAR>+Graph</VAR>, <VAR>-WeightedGraph</VAR>)</CODE>
<DD>
<VAR>WeightedGraph</VAR> has the same vertices and edges as <VAR>Graph</VAR>,
except the edges of <VAR>WeightedGraph</VAR> all have weight 1.

<A NAME="IDX1147"></A>
<DT><CODE>vertices_edges_to_wgraph(<VAR>+Vertices</VAR>, <VAR>+Edges</VAR>, <VAR>-WeightedGraph</VAR>)</CODE>
<DD>
<VAR>Vertices</VAR> is a list of vertices, <VAR>Edges</VAR> is a list of edges,
and <VAR>WeightedGraph</VAR> is a graph built from <VAR>Vertices</VAR> and
<VAR>Edges</VAR>.  <VAR>Vertices</VAR> and <VAR>Edges</VAR> may be in any order.  The
vertices mentioned in <VAR>Edges</VAR> do not have to occur explicitly in
<VAR>Vertices</VAR>.  <VAR>Vertices</VAR> may be used to specify vertices that are
not connected by any edges.

<A NAME="IDX1148"></A>
<DT><CODE>vertices(<VAR>+WeightedGraph</VAR>, <VAR>-Vertices</VAR>)</CODE>
<DD>
Unifies <VAR>Vertices</VAR> with the vertices in <VAR>WeightedGraph</VAR>.

<A NAME="IDX1149"></A>
<DT><CODE>edges(<VAR>+WeightedGraph</VAR>, <VAR>-Edges</VAR>)</CODE>
<DD>
Unifies <VAR>Edges</VAR> with the edges in <VAR>WeightedGraph</VAR>.

<A NAME="IDX1150"></A>
<DT><CODE>add_vertices(<VAR>+WeightedGraph1</VAR>, <VAR>+Vertices</VAR>, <VAR>-WeightedGraph2</VAR>)</CODE>
<DD>
<VAR>WeightedGraph2</VAR> is <VAR>WeightedGraph1</VAR> with <VAR>Vertices</VAR> added
to it.

<A NAME="IDX1151"></A>
<DT><CODE>del_vertices(<VAR>+WeightedGraph1</VAR>, <VAR>+Vertices</VAR>, <VAR>-WeightedGraph2</VAR>)</CODE>
<DD>
<VAR>WeightedGraph2</VAR> is <VAR>WeightedGraph1</VAR> with <VAR>Vertices</VAR> and all
edges to and from them removed from it.

<A NAME="IDX1152"></A>
<DT><CODE>add_edges(<VAR>+WeightedGraph1</VAR>, <VAR>+Edges</VAR>, <VAR>-WeightedGraph2</VAR>)</CODE>
<DD>
<VAR>WeightedGraph2</VAR> is <VAR>WeightedGraph1</VAR> with <VAR>Edges</VAR> and their
"to" and "from" vertices added to it.

<A NAME="IDX1153"></A>
<DT><CODE>del_edges(<VAR>+WeightedGraph1</VAR>, <VAR>+Edges</VAR>, <VAR>-WeightedGraph2</VAR>)</CODE>
<DD>
<VAR>WeightedGraph2</VAR> is <VAR>WeightedGraph1</VAR> with <VAR>Edges</VAR> removed
from it.

<A NAME="IDX1154"></A>
<DT><CODE>transpose(<VAR>+WeightedGraph</VAR>, <VAR>-Transpose</VAR>)</CODE>
<DD>
<VAR>Transpose</VAR> is the graph computed by replacing each edge <VAR>(u,v)</VAR>
in <VAR>WeightedGraph</VAR> by its symmetric edge <VAR>(v,u)</VAR>.  It can only
be used one way around.  Takes O(N^2) time.

<A NAME="IDX1155"></A>
<A NAME="IDX1156"></A>
<DT><CODE>neighbors(<VAR>+Vertex</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Neighbors</VAR>)</CODE>
<DD>
<DT><CODE>neighbours(<VAR>+Vertex</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Neighbors</VAR>)</CODE>
<DD>
<VAR>Vertex</VAR> is a vertex in <VAR>WeightedGraph</VAR> and <VAR>Neighbors</VAR> are
its weighted neighbors.

<A NAME="IDX1157"></A>
<DT><CODE>transitive_closure(<VAR>+WeightedGraph</VAR>, <VAR>-Closure</VAR>)</CODE>
<DD>
Computes <VAR>Closure</VAR> as the transitive closure of <VAR>WeightedGraph</VAR>
in O(N^3) time.

<A NAME="IDX1158"></A>
<DT><CODE>symmetric_closure(<VAR>+WeightedGraph</VAR>, <VAR>-Closure</VAR>)</CODE>
<DD>
Computes <VAR>Closure</VAR> as the symmetric closure of <VAR>WeightedGraph</VAR>,
i.e. for each edge <VAR>(u,v)</VAR> in <VAR>WeightedGraph</VAR>, add its symmetric
edge <VAR>(v,u)</VAR>.  Takes O(N^2) time.  This is useful for making a
directed graph undirected.

<A NAME="IDX1159"></A>
<DT><CODE>top_sort(<VAR>+WeightedGraph</VAR>, <VAR>-Sorted</VAR>)</CODE>
<DD>
Finds a topological ordering of a <VAR>WeightedGraph</VAR> and returns the
ordering as a list of <VAR>Sorted</VAR> vertices.  Fails iff no ordering
exists, i.e. iff the graph contains cycles.  Takes O(N^2) time.

<A NAME="IDX1160"></A>
<DT><CODE>max_path(<VAR>+V1</VAR>, <VAR>+V2</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Path</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<VAR>Path</VAR> is a maximum-cost path of cost <VAR>Cost</VAR> from <VAR>V1</VAR> to
<VAR>V2</VAR> in <VAR>WeightedGraph</VAR>, there being no cyclic paths from
<VAR>V1</VAR> to <VAR>V2</VAR>.  Takes O(N^2) time.

<A NAME="IDX1161"></A>
<DT><CODE>min_path(<VAR>+V1</VAR>, <VAR>+V2</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Path</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<VAR>Path</VAR> is a minimum-cost path of cost <VAR>Cost</VAR> from <VAR>V1</VAR> to
<VAR>V2</VAR> in <VAR>WeightedGraph</VAR>.  Takes O(N^2) time.

<A NAME="IDX1162"></A>
<DT><CODE>min_paths(<VAR>+Vertex</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Tree</VAR>)</CODE>
<DD>
 <VAR>Tree</VAR> is a tree of all the minimum-cost paths from <VAR>Vertex</VAR> to
every other vertex in <VAR>WeightedGraph</VAR>.  This is the single-source
minimum-cost paths problem.

<A NAME="IDX1163"></A>
<DT><CODE>path(<VAR>+Vertex</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Path</VAR>)</CODE>
<DD>
Given a <VAR>WeightedGraph</VAR> and a <VAR>Vertex</VAR> of <VAR>WeightedGraph</VAR>,
returns a maximal <VAR>Path</VAR> rooted at <VAR>Vertex</VAR>, enumerating more
paths on backtracking.

<A NAME="IDX1164"></A>
<DT><CODE>reduce(<VAR>+WeightedGraph</VAR>, <VAR>-Reduced</VAR>)</CODE>
<DD>
<VAR>Reduced</VAR> is the reduced graph for <VAR>WeightedGraph</VAR>.  The
vertices of the reduced graph are the strongly connected components of
<VAR>WeightedGraph</VAR>.  There is an edge in <VAR>Reduced</VAR> from <VAR>u</VAR> to
<VAR>v</VAR> iff there is an edge in <VAR>WeightedGraph</VAR> from one of the
vertices in <VAR>u</VAR> to one of the vertices in <VAR>v</VAR>.

<A NAME="IDX1165"></A>
<DT><CODE>reachable(<VAR>+Vertex</VAR>, <VAR>+WeightedGraph</VAR>, <VAR>-Reachable</VAR>)</CODE>
<DD>
Given a <VAR>WeightedGraph</VAR> and a <VAR>Vertex</VAR> of <VAR>WeightedGraph</VAR>,
returns the set of vertices that are <EM>reachable</EM> from that
<VAR>Vertex</VAR>.  Takes O(N^2) time.

<A NAME="IDX1166"></A>
<DT><CODE>random_wgraph(<VAR>+P</VAR>, <VAR>+N</VAR>, <VAR>+W</VAR>, <VAR>-WeightedGraph</VAR>)</CODE>
<DD>
Where <VAR>P</VAR> is a probability, unifies <VAR>WeightedGraph</VAR> with a
random graph of vertices <VAR>1..N</VAR> where each possible edge is included
with probability <VAR>P</VAR> and random weight in <VAR>1..W</VAR>.
</DL>

<P>
The following predicate is defined for undirected graphs only.

</P>
<DL COMPACT>

<DT><CODE>min_tree(<VAR>+WeightedGraph</VAR>, <VAR>-Tree</VAR>, <VAR>-Cost</VAR>)</CODE>
<DD>
<A NAME="IDX1167"></A>
 
<VAR>Tree</VAR> is a minimum-cost spanning tree of <VAR>WeightedGraph</VAR> with
cost <VAR>Cost</VAR>, if it exists.
</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_26.html">previous</A>, <A HREF="sicstus_28.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
