<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Socket I/O</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_27.html">previous</A>, <A HREF="sicstus_29.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC202" HREF="sicstus_toc.html#TOC202">Socket I/O</A></H1>
<P>
<A NAME="IDX1168"></A>
<A NAME="IDX1169"></A>

</P>
<P>
This library package defines a number of predicates manipulating
sockets.  They are all rather straight-forward interfaces to the
corresponding BSD-type socket functions with the same name (except
<CODE>current_host/1</CODE>).  The reader should therefore study the
appropriate documents for a deeper description.

</P>
<P>
The <VAR>Domain</VAR> is either the atom <CODE>'AF_INET'</CODE> or
<CODE>'AF_UNIX'</CODE>. They correspond directly to the same domains in
BSD-type sockets. <CODE>'AF_UNIX'</CODE> may not be available on non-UNIX
platforms.

</P>
<P>
An <VAR>Address</VAR> is either <CODE>'AF_INET'(Host,Port)</CODE> or
<CODE>'AF_UNIX'(SocketName)</CODE>.  <VAR>Host</VAR> is an atomic hostname,
<VAR>Port</VAR> is a portnumber and <VAR>SocketName</VAR> is an atom denoting a
socket.  A reader familiar with BSD sockets will understand this
immediately.

</P>
<P>
All streams below can be both read from and written on.  All I/O-predicates
operating on streams can be used, for example <CODE>read/2</CODE>, <CODE>write/2</CODE>,
<CODE>format/3</CODE>, <CODE>current_stream/3</CODE>, etc.  Socket streams are unbuffered
on reads and block buffered on writes.

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library(sockets)).
</PRE>

<DL COMPACT>

<DT><CODE>socket(<VAR>+Domain</VAR>, <VAR>-Socket</VAR>)</CODE>
<DD>
<A NAME="IDX1170"></A>
A socket <VAR>Socket</VAR> in the domain <VAR>Domain</VAR> is created.  
<DT><CODE>socket_close(<VAR>+Socket</VAR>)</CODE>
<DD>
<A NAME="IDX1171"></A>
<VAR>Socket</VAR> is closed. Sockets used in <CODE>socket_connect/2</CODE> should
not be closed by <CODE>socket_close/1</CODE> as they will be closed when the
corresponding stream is closed. 
<DT><CODE>socket_bind(<VAR>+Socket</VAR>, <CODE>'AF_UNIX'(<VAR>+SocketName</VAR>)</CODE>)</CODE>
<DD>
<DT><CODE>socket_bind(<VAR>+Socket</VAR>, <CODE>'AF_INET'(<VAR>?Host</VAR>,<VAR>?Port</VAR>)</CODE>)</CODE>
<DD>
<A NAME="IDX1172"></A>
The socket <VAR>Socket</VAR> is bound to the address. If <VAR>Port</VAR> is
uninstantiated, the operative system picks a port number to which
<VAR>Port</VAR> is bound. 
<DT><CODE>socket_connect(<VAR>+Socket</VAR>, <CODE>'AF_UNIX'(<VAR>+SocketName</VAR>)</CODE>, <VAR>-Stream</VAR>)</CODE>
<DD>
<DT><CODE>socket_connect(<VAR>+Socket</VAR>, <CODE>'AF_INET'(<VAR>+Host</VAR>,<VAR>+Port</VAR>)</CODE>, <VAR>-Stream</VAR>)</CODE>
<DD>
<A NAME="IDX1173"></A>
The socket <VAR>Socket</VAR> is connected to the address.  <VAR>Stream</VAR> is
a special stream on which items can be both read and written.  
<DT><CODE>socket_listen(<VAR>+Socket</VAR>, <VAR>+Length</VAR>)</CODE>
<DD>
<A NAME="IDX1174"></A>
The socket <VAR>Socket</VAR> is defined to have a maximum backlog queue of 
<VAR>Length</VAR> pending connections.  
<DT><CODE>socket_accept(<VAR>+Socket</VAR>, <VAR>-Stream</VAR>)</CODE>
<DD>
<DT><CODE>socket_accept(<VAR>+Socket</VAR>, <VAR>-Client</VAR>, <VAR>-Stream</VAR>)</CODE>
<DD>
<A NAME="IDX1175"></A>
<A NAME="IDX1176"></A>
The first connection to socket <VAR>Socket</VAR> is extracted.  The stream
<VAR>Stream</VAR> is opened for read and write on this connection. For the
<CODE>'AF_INET'</CODE> domain, <VAR>Client</VAR> will unified with an atom
containing the Internet host address of the connecting entity in
numbers-and-dots notation. For other domains, <VAR>Client</VAR> will not be
used.

<DT><CODE>socket_select(<VAR>+TermsSockets</VAR>, <VAR>-NewTermsStreams</VAR>, <VAR>+TimeOut</VAR>, <VAR>+Streams</VAR>, <VAR>-ReadStreams</VAR>)</CODE>
<DD>
<A NAME="IDX1177"></A>
The list of streams in <VAR>Streams</VAR> is checked for readable characters.
A stream can be any stream associated with an I/O descriptor.  The list
<VAR>ReadStreams</VAR> returns the streams with readable data.

<CODE>socket_select/5</CODE> also waits for connections to the sockets
specified by <VAR>TermsSockets</VAR>. This argument should be a list of
<VAR>Term<CODE>-</CODE>Socket</VAR> pairs, where <VAR>Term</VAR>, which can be any term,
is used as an identifier. <VAR>NewTermsStreams</VAR> is a
list of <VAR>Term</VAR><CODE>-connection(<VAR>Client</VAR>,<VAR>Stream</VAR>)</CODE> pairs,
where <VAR>Stream</VAR> is a new stream open for communicating with a process
connecting to the socket identified with <VAR>Term</VAR>, <VAR>Client</VAR> is the
client host address (see <CODE>socket_accept/3</CODE>).

If <VAR>TimeOut</VAR> is instantiated to <CODE>off</CODE>, the predicate waits
until something is available.  If <VAR>TimeOut</VAR> is <CODE>S:U</CODE> the
predicate waits at most <CODE>S</CODE> seconds and <CODE>U</CODE> microseconds.
Both <CODE>S</CODE> and <CODE>U</CODE> must be integers &#62;=0.  If there is a timeout,
<VAR>ReadStreams</VAR> and <VAR>NewTermsStreams</VAR> are <CODE>[]</CODE>.

<DT><CODE>socket_select(<VAR>+Sockets</VAR>, <VAR>-NewStreams</VAR>, <VAR>+TimeOut</VAR>, <VAR>+Streams</VAR>, <VAR>-ReadStreams</VAR>)</CODE>
<DD>
<DT><CODE>socket_select(<VAR>+Socket</VAR>, <VAR>-NewStream</VAR>, <VAR>+TimeOut</VAR>, <VAR>+Streams</VAR>, <VAR>-ReadStreams</VAR>)</CODE>
<DD>
<DT><CODE>socket_select(<VAR>+Sockets</VAR>, <VAR>-NewStreams</VAR>, <VAR>-NewClients</VAR>, <VAR>+TimeOut</VAR>, <VAR>+Streams</VAR>, <VAR>-ReadStreams</VAR>)</CODE>
<DD>
<DT><CODE>socket_select(<VAR>+Socket</VAR>, <VAR>-NewStream</VAR>, <VAR>-NewClient</VAR>, <VAR>+TimeOut</VAR>, <VAR>+Streams</VAR>, <VAR>-ReadStreams</VAR>)</CODE>
<DD>
<A NAME="IDX1178"></A>
<A NAME="IDX1179"></A>
These forms, which are provided for backward compatibility only, differs in how sockets are specified and new streams returned.

<CODE>socket_select/(5,6)</CODE> also wait for connections to the sockets in
the list <VAR>Sockets</VAR>.  <VAR>NewStreams</VAR> is the list of new streams
opened for communicating with the connecting processes. <VAR>NewClients</VAR>
is the corresponding list of client host addresses (see
<CODE>socket_accept/3</CODE>).

The second form requires one socket (not a list) for the first argument
and returns a stream, <VAR>NewStream</VAR>, if a connection is made.

<DT><CODE>current_host(<VAR>?HostName</VAR>)</CODE>
<DD>
<A NAME="IDX1180"></A>
<VAR>HostName</VAR> is unified with the fully qualified name of the machine
the process is executing on. The call will also succeed if
<VAR>HostName</VAR> is instantiated to the unqualified name of the machine.

<DT><CODE>hostname_address(<VAR>+HostName</VAR>, <VAR>-HostAddress</VAR>)</CODE>
<DD>
<DT><CODE>hostname_address(<VAR>-HostName</VAR>, <VAR>+HostAddress</VAR>)</CODE>
<DD>
<A NAME="IDX1181"></A>
The Internet host is resolved given either the host name or
address. <VAR>HostAddress</VAR> should be an atom containing the Internet
host address in numbers-and-dots notation. The predicate will fail if
the host name or address cannot be resolved.

</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_27.html">previous</A>, <A HREF="sicstus_29.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
