<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Linda--Process Communication</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_28.html">previous</A>, <A HREF="sicstus_30.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC203" HREF="sicstus_toc.html#TOC203">Linda--Process Communication</A></H1>
<P>
<A NAME="IDX1182"></A>
<A NAME="IDX1183"></A>
<A NAME="IDX1184"></A>
<A NAME="IDX1185"></A>
Linda is a concept for process communication.

</P>
<P>
For an introduction and a deeper description, see [Carreiro &#38; Gelernter
89a] or [Carreiro &#38; Gelernter 89b], respectively.

</P>
<P>
One process is running as a server and one or more processes are running
as clients.  The processes are communicating with sockets and
supports networks.

</P>
<P>
The server is in principle a blackboard on which the clients can write
(<CODE>out/1</CODE>), read (<CODE>rd/1</CODE>) and remove (<CODE>in/1</CODE>) data.  If the
data is not present on the blackboard, the predicates suspend the process
until they are available.

</P>
<P>
There are some more predicates besides the basic <CODE>out/1</CODE>, <CODE>rd/1</CODE>
and <CODE>in/1</CODE>.  The <CODE>in_noblock/1</CODE> and <CODE>rd_noblock/1</CODE> does not
suspend if the data is not available--they fail instead.  A blocking fetch
of a conjunction of data can be done with <CODE>in/2</CODE> or
<CODE>rd/2</CODE>.

</P>

<P>
<B>Example</B>: A simple producer-consumer.  In client 1:

<PRE>
producer :-
       produce(X),
       out(p(X)),
       producer.

produce(X) :- .....
</PRE>

<P>
In client 2:

</P>

<PRE>
consumer :- 
       in(p(A)),
       consume(A),
       consumer.

consume(A) :- .....
</PRE>

<P>
<B>Example</B>: Synchronization

</P>

<PRE>
       ...,
       in(ready),  %Waits here until someone does out(ready)
       ...,
</PRE>

<P>
<B>Example</B>: A critical region

</P>

<PRE>
       ...,
       in(region_free),  % wait for region to be free
       critical_part,
       out(region_free), % let next one in
       ...,
</PRE>

<P>
<B>Example</B>: Reading global data

</P>

<PRE>
       ...,
       rd(data(Data)),
       ...,

or, without blocking:
       ...,
       rd_noblock(data(Data)) -&#62;
             do_something(Data)
       ;     write('Data not available!'),nl
       ),
       ...,
</PRE>

<P>
<B>Example</B>: Waiting for one of several events

</P>

<PRE>
       ...,
       in([e(1),e(2),...,e(n)], E),
%  Here is E instantiated to the first tuple that became available
       ...,
</PRE>


<UL>
<LI><A HREF="sicstus_29.html#SEC204">Server</A>
<LI><A HREF="sicstus_29.html#SEC205">Client</A>
</UL>



<H2><A NAME="SEC204" HREF="sicstus_toc.html#TOC204">Server</A></H2>
<P>
The server is the process running the "blackboard process".  It is an
ordinary SICStus process which can be run on a separate machine if
necessary.

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library('linda/server')).
</PRE>

<P>
and start the server with <CODE>linda/0</CODE> or <CODE>linda/1</CODE>.

</P>
<DL COMPACT>

<DT><CODE>linda</CODE>
<DD>
<A NAME="IDX1186"></A>
Starts a Linda-server in this SICStus.  The network address is written
to current output stream as <VAR>Host:PortNumber</VAR>.

<DT><CODE>linda(<VAR>+Hook</VAR>)</CODE>
<DD>
<A NAME="IDX1187"></A>
Starts a Linda-server in this SICStus.  When it is started, a goal passed
in <VAR>Hook</VAR> is evaluated. <VAR>Hook</VAR> must have the form 
<VAR>Address-Goal</VAR> where <VAR>Address</VAR> must be unifiable with <VAR>Host:Port</VAR>
and <VAR>Goal</VAR> must be instantiated to a goal.
<B>Example</B>:

<PRE>
| ?- linda((Host:Port)-(my_module:mypred(Host,Port))).
</PRE>

will call <CODE>mypred/2</CODE> in module <CODE>my_module</CODE> when the server is
started.  <CODE>mypred/2</CODE> could start the client-processes, save the
address for the clients etc.  Note that the module must be present in
<VAR>Goal</VAR>.

</DL>



<H2><A NAME="SEC205" HREF="sicstus_toc.html#TOC205">Client</A></H2>
<P>
The clients are one or more sicstus processes which have connection(s) to the
server.

</P>
<P>
To load the package, enter the query

</P>

<PRE>
| ?- use_module(library('linda/client')).
</PRE>

<P>
Some of the following predicates fail if they don't receive an answer
from the Linda-server in a reasonable amount of time.  That time is set
with the predicate <CODE>linda_timeout/2</CODE>.

</P>
<DL COMPACT>

<DT><CODE>linda_client(<VAR>+Address</VAR>)</CODE>
<DD>
<A NAME="IDX1188"></A>
Establishes a connection to a Linda-server specified by <VAR>Address</VAR>.
The <VAR>Address</VAR> is of the format <VAR>Host:PortNumber</VAR> as given by
<CODE>linda/0</CODE> and <CODE>linda/1</CODE>.

It is not possible to be connected to two Linda-servers in the same time.

This predicate can fail due to a timeout.

<DT><CODE>close_client</CODE>
<DD>
<A NAME="IDX1189"></A>
Closes the connection to the Linda-server.

<DT><CODE>linda_timeout(<VAR>?OldTime</VAR>, <VAR>?NewTime</VAR>)</CODE>
<DD>
<A NAME="IDX1190"></A>
This predicate controls Linda's timeout. <VAR>OldTime</VAR> is unified with
the old timeout and then timeout is set to <VAR>NewTime</VAR>.  The value is
either <CODE>off</CODE> or of the form <VAR>Seconds:Milliseconds</VAR>.  The former
value indicates that the timeout mechanism is disabled, that is, eternal
waiting.  The latter form is the <VAR>timeout</VAR>-<VAR>time</VAR>. 
<DT><CODE>out(<VAR>+Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1191"></A>
Places the tuple <VAR>Tuple</VAR> in Linda's tuple-space.

<DT><CODE>in(<VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1192"></A>
Removes the tuple <VAR>Tuple</VAR> from Linda's tuple-space if it is there.
If not, the predicate blocks until it is available (that is, someone
performs an <CODE>out/1</CODE>).

<DT><CODE>in_noblock(<VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1193"></A>
Removes the tuple <VAR>Tuple</VAR> from Linda's tuple-space if it is there.
If not, the predicate fails.

This predicate can fail due to a timeout.

<DT><CODE>in(<VAR>+TupleList</VAR>, <VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1194"></A>
As <CODE>in/1</CODE> but succeeds when either of the tuples in <VAR>TupleList</VAR>
is available. <VAR>Tuple</VAR> is unified with the fetched tuple.  If that
unification fails, the tuple is <EM>not</EM> reinserted in the tuple-space.

<DT><CODE>rd(<VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1195"></A>
Succeeds if <VAR>Tuple</VAR> is available in the tuple-space, suspends otherwise
until it is available.
Compare this with <CODE>in/1</CODE>: the tuple is <EM>not</EM> removed.

<DT><CODE>rd_noblock(<VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1196"></A>
Succeeds if <VAR>Tuple</VAR> is available in the tuple-space, fails otherwise.

This predicate can fail due to a timeout.

<DT><CODE>rd(<VAR>+TupleList</VAR>, <VAR>?Tuple</VAR>)</CODE>
<DD>
<A NAME="IDX1197"></A>
As <CODE>in/2</CODE> but does not remove any tuples.

<DT><CODE>bagof_rd_noblock(<VAR>?Template</VAR>, <VAR>+Tuple</VAR>, <VAR>?Bag</VAR>)</CODE>
<DD>
<A NAME="IDX1198"></A>
<VAR>Bag</VAR> is the list of all instances of <VAR>Template</VAR> such that
<VAR>Tuple</VAR> exists in the tuple-space.

The behavior of variables in <VAR>Tuple</VAR> and <VAR>Template</VAR> is as in
<CODE>bagof/3</CODE>.  The variables could be existentially quantified with
<CODE>^/2</CODE> as in <CODE>bagof/3</CODE>.

The operation is performed as an atomic operation.

This predicate can fail due to a timeout.

<B>Example</B>:
Assume that only one client is connected to the server and that the tuple-space
initially is empty.

<PRE>
| ?- out(x(a,3)), out(x(a,4)), out(x(b,3)), out(x(c,3)).

yes
| ?- bagof_rd_noblock(C-N, x(C,N), L).

C = _32,
L = [a-3,a-4,b-3,c-3],
N = _52 ?

yes
| ?- bagof_rd_noblock(C, N^x(C,N), L).

C = _32,
L = [a,a,b,c],
N = _48 ?

yes
</PRE>

</DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_28.html">previous</A>, <A HREF="sicstus_30.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
