<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Notational Conventions</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_2.html">previous</A>, <A HREF="sicstus_4.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC3" HREF="sicstus_toc.html#TOC3">Notational Conventions</A></H1>
<P>
<A NAME="IDX4"></A>

</P>

<UL>
<LI><A HREF="sicstus_3.html#SEC4">Keyboard Characters</A>
<LI><A HREF="sicstus_3.html#SEC5">Mode Spec</A>
<LI><A HREF="sicstus_3.html#SEC6">Predicate Spec</A>
<LI><A HREF="sicstus_3.html#SEC7">Development and Runtime Systems</A>
<LI><A HREF="sicstus_3.html#SEC8">Function Prototypes</A>
<LI><A HREF="sicstus_3.html#SEC9">ISO Compliance</A>
</UL>



<H2><A NAME="SEC4" HREF="sicstus_toc.html#TOC4">Keyboard Characters</A></H2>

<P>
<A NAME="IDX5"></A>
<A NAME="IDX6"></A>
<A NAME="IDX7"></A>
When referring to keyboard characters, printing characters are written
thus: <KBD>a</KBD>, while control characters are written like this: ^A.
Thus ^C is the character you get by holding down the <KBD>CTL</KBD> key
while you type <KBD>c</KBD>.  Finally, the special control characters
carriage-return, line-feed and space are often abbreviated to <KBD>RET</KBD>,
<KBD>LFD</KBD> and <KBD>SPC</KBD> respectively.

</P>


<H2><A NAME="SEC5" HREF="sicstus_toc.html#TOC5">Mode Spec</A></H2>

<P>
<A NAME="IDX8"></A>
When introducing a built-in predicate, we shall present its usage with a
<EM>mode spec</EM> which has the form <VAR>name(arg, ..., arg)</VAR> where each
<VAR>arg</VAR> denotes how that argument should be instantiated in goals, and
has one of the following forms: 

</P>
<DL COMPACT>

<DT><VAR>:ArgName</VAR>
<DD>
This argument should be instantiated to a term denoting a goal or a
clause or a predicate name, or which otherwise needs special handling
of module prefixes.

<DT><VAR>+ArgName</VAR>
<DD>
This argument should be instantiated to a non-variable term.

<DT><VAR>-ArgName</VAR>
<DD>
This argument should be uninstantiated.

<DT><VAR>?ArgName</VAR>
<DD>
This argument may or may not be instantiated.
</DL>

<P>
Mode specs are not only used in the manual, but are part of the
syntax of the language as well.  When used in the source code, however,
the <VAR>ArgName</VAR> part must be omitted.  That is, <VAR>arg</VAR> must be
either <KBD>:</KBD>, <KBD>+</KBD>, <KBD>-</KBD>, or <KBD>?</KBD>.

</P>


<H2><A NAME="SEC6" HREF="sicstus_toc.html#TOC6">Predicate Spec</A></H2>

<P>
<A NAME="IDX9"></A>
In the context of certain compiler declarations, we shall need the
following notation: Predicates in Prolog are distinguished by their name
<EM>and</EM> their arity.  The notation <CODE><VAR>name</VAR>/<VAR>arity</VAR></CODE> is
therefore used when it is necessary to refer to a predicate
unambiguously; e.g. <CODE>concatenate/3</CODE> specifies the predicate which
is named "concatenate" and which takes 3 arguments.  In general, a
<EM>predicate spec</EM> takes the form <CODE><VAR>name</VAR>/<VAR>arity</VAR></CODE>.  With
the introduction of the module system, this spec is only defined
relative to the "current" module.  An absolute predicate spec must
include a module prefix: <CODE><VAR>module</VAR>:<VAR>name</VAR>/<VAR>arity</VAR></CODE>.

</P>


<H2><A NAME="SEC7" HREF="sicstus_toc.html#TOC7">Development and Runtime Systems</A></H2>

<P>
<A NAME="IDX10"></A>
<A NAME="IDX11"></A>
<A NAME="IDX12"></A>
<A NAME="IDX13"></A>
The full Prolog system with top-level, compiler, debugger etc.
is known as the <EM>Development System</EM>.  On some platforms, both
a sequential and an OR-parallel Development System can be built.
The OR-parallel Development System is also known as <EM>Muse</EM>.

</P>
<P>
OR-parallelism executes clauses and disjunctions of a predicate in
parallel and is useful for search and all-solutions style programming.
No change or annotations of the programs are necessary but sometimes
more parallelism may be utilized if the program is modified.  The main
issue is to avoid side effects that might serialize the search.  Except
for some few exceptions Muse preserves the full functionality of SICStus
Prolog.  Muse is still an experimental system and is not formally
supported.

</P>
<P>
It is possible to link user-written C code with a subset of SICStus
Prolog to create stand-alone applications, called <EM>Runtime Systems</EM>.
Only sequential Runtime Systems are available.
When introducing a built-in predicate, any limitations on its use in
Runtime Systems will be mentioned.

</P>


<H2><A NAME="SEC8" HREF="sicstus_toc.html#TOC8">Function Prototypes</A></H2>

<P>
<A NAME="IDX14"></A>
<A NAME="IDX15"></A>
<A NAME="IDX16"></A>
<A NAME="IDX17"></A>
Whenever this manual documents a C function as part of SICStus Prolog's
foreign language interface, the function prototype will be displayed in
ANSI C syntax.

</P>



<H2><A NAME="SEC9" HREF="sicstus_toc.html#TOC9">ISO Compliance</A></H2>
<P>
SICStus Prolog complies in spirit, if not in detail, with the Draft
International Standard ISO/IEC 13211-1 (PROLOG: Part 1--General Core).
In particular, SICStus Prolog does not offer a strictly conforming mode
which rejects uses of implementation specific features.  Minor
deviations also concern e.g. the syntax, the arithmetic functions, and
the error system.

</P>
<P>
To aid programmers who wish to write standard compliant programs,
built-in predicates that have a counterpart in the ISO Prolog Standard
are annotated with <EM>[ISO]</EM> in this manual, where appropriate with a
comment clarifying the difference between the SICStus and the prescribed
ISO Prolog versions.

</P>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_2.html">previous</A>, <A HREF="sicstus_4.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
