<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - External Storage of Terms (External Database)</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_29.html">previous</A>, <A HREF="sicstus_31.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC206" HREF="sicstus_toc.html#TOC206">External Storage of Terms (External Database)</A></H1>
<P>
<A NAME="IDX1199"></A>
<A NAME="IDX1200"></A>
<A NAME="IDX1201"></A>

</P>
<P>
This library handles storage and retrieval of terms on files.  By using
indexing, the store/retrieve operations are efficient also for large
data sets.

</P>
<P>
The package is loaded by the query

</P>

<PRE>
| ?- use_module(library(db)).
</PRE>


<UL>
<LI><A HREF="sicstus_30.html#SEC207">Basics</A>
<LI><A HREF="sicstus_30.html#SEC208">Current Limitations</A>
<LI><A HREF="sicstus_30.html#SEC209">The DB-Spec Informal Description</A>
<LI><A HREF="sicstus_30.html#SEC210">Predicates</A>
<LI><A HREF="sicstus_30.html#SEC211">An Example Session</A>
<LI><A HREF="sicstus_30.html#SEC212">The Db-Spec</A>
</UL>



<H2><A NAME="SEC207" HREF="sicstus_toc.html#TOC207">Basics</A></H2>

<P>
The idea is to get a behavior similar to <CODE>assert/1</CODE>,
<CODE>retract/1</CODE> and <CODE>clause/2</CODE> but the terms are stored on files
instead of in primary memory.

</P>
<P>
The differences compared with the internal database are:

</P>

<UL>
<LI>

<A NAME="IDX1202"></A>
A <EM>database</EM> must be opened before any access and closed after the
last access. (There are special predicates for this: <CODE>db_open</CODE> and
<CODE>db_close</CODE>.)

<LI>

The indexing is specified when the database is created.  It is possible
to index on other parts of the term than just the functor and first
argument.

<LI>

Changes affect the database immediately.

<LI>

The database will store variables with frozen goals as ordinary variables.
</UL>

<P>
Some commercial databases can't store non-ground terms or more than one
instance of a term.  The SICStus database can however store terms of
either kind.

</P>
<P>
The database is kept in a secure state and the last stored term is safe
even if the SICStus process dies (machine rebooted, process killed,
<CODE>halt/0</CODE>, power failure...)  (but see section <A HREF="sicstus_30.html#SEC208">Current Limitations</A>).

</P>



<H2><A NAME="SEC208" HREF="sicstus_toc.html#TOC208">Current Limitations</A></H2>


<UL>
<LI>

Only one process can open the same database.

<LI>

The terms are not necessarily fetched in the same order as they were
stored.

<LI>

If the process dies <EM>during</EM> the <CODE>db_store</CODE> the database
<EM>could</EM> be inconsistent.

<LI>

At the user's option, a database can be operated in an insecure
state where modified pages are not immediately flushed to disk.
This is useful e.g. when a database is created by storing a large
number of terms.  See <CODE>db_buffering/(2-3)</CODE>.
</UL>



<H2><A NAME="SEC209" HREF="sicstus_toc.html#TOC209">The DB-Spec--Informal Description</A></H2>

<P>
<A NAME="IDX1203"></A>
The <EM>db-spec</EM> defines which parts of a term that is used for
indexing in a database.  It is a structure with the functor <CODE>on</CODE> or
<CODE>off</CODE>.  The arguments are <CODE>on</CODE>, <CODE>off</CODE> or the same kind of
structure.

</P>
<P>
<A NAME="IDX1204"></A>
<A NAME="IDX1205"></A>
The db-spec is compared with the <EM>indexed term</EM> and every argument
where there is an <CODE>on</CODE> in the db-spec is indexed.

</P>
<P>
If the db-spec is of lower arity than the indexed term,
the last part of the indexed term is skipped or vice versa. 

</P>
<P>
The idea of a db-spec are illustrated with a few examples.
(A section further down explains the db-spec in a more formal
way).

</P>

<PRE>
<EM>DB-Spec</EM>    <EM>Term (the parts with indexing are underlined)</EM> 

on(on)           a(b)   a(b,c)  a   [a,b(c),d]    /* as Prolog */
                 - -    - -     -   --

on(on,on)        a(b)   a(b,c)  a(b,c,d)  a(b,c(d))   [a,b(c),d]
                 - -    - - -   - - -     - - -       ---

on(off,on(on))   a(b)   a(b,c)  a(b,c,d)  a(b,c(d)) [a,b(c),d]
                 -      -   -   -   -     -   - -   - --
</PRE>



<H2><A NAME="SEC210" HREF="sicstus_toc.html#TOC210">Predicates</A></H2>
<P>
The following conventions are used in the predicate descriptions below.

<UL>
<LI>

<VAR>Mode</VAR> is either <CODE>update</CODE> or <CODE>read</CODE>

<LI>

<VAR>DBRef</VAR> is a reference to an open database.  The reference is
returned when the database is opened.  The reference becomes invalid
after the database has been closed.

<LI>

<VAR>TermRef</VAR> is a reference to a term in a given database.  The
reference is returned when a term is stored.  The reference stays valid
even after the database is closed and hence can be stored permanently as
part of another term.  

A term generally corresponds to several alias references.
<CODE>db_canonical/(2,3)</CODE> can be used to check whether two
<VAR>TermRef</VAR>s refer to the same term.

<LI>

<VAR>Spec</VAR> is a description of the indexing, see the description of
db-specs below.

<LI>

<VAR>Term</VAR> is any Prolog term.

</UL>

<DL COMPACT>

<DT><CODE>db_open(<VAR>+Name</VAR>, <VAR>+Mode</VAR>, <VAR>-DBref</VAR>)</CODE>
<DD>
<DT><CODE>db_open(<VAR>+Name</VAR>, <VAR>+Mode</VAR>, <VAR>?Spec</VAR>, <VAR>-DBref</VAR>)</CODE>
<DD>
<A NAME="IDX1206"></A>
<A NAME="IDX1207"></A>
Opens a database with the name <VAR>Name</VAR>.  The database physically
consists of a subdirectory with the same name, containing the files that
make up the database.  If the subdirectory does not exist, it is created.
In that case <VAR>Mode</VAR> must be <CODE>update</CODE>.

<DL COMPACT>

<DT><CODE>db_open/4:</CODE>
<DD>
The db-spec <VAR>Spec</VAR> must be
ground if opening a new database.  If an existing database
is opened, <VAR>Spec</VAR> is unified with the db-spec given
when the database was created.  If the unification fails the predicate fails
and an error is raised.

<DT><CODE>db_open/3:</CODE>
<DD>
On creating a new database, the db-spec is <CODE>on(on)</CODE>, the same
kind of indexing as in the internal database.  When opening an existing
database, any db-spec from the database is accepted.
</DL>

<DT><CODE>db_close</CODE>
<DD>
<DT><CODE>db_close(<VAR>+DBref</VAR>)</CODE>
<DD>
<A NAME="IDX1208"></A>
<A NAME="IDX1209"></A>
<A NAME="IDX1210"></A>
<A NAME="IDX1211"></A>
Closes a database. <CODE>db_close/0</CODE> closes the <EM>default database</EM>.
Note that after <CODE>db_close/0</CODE> there is no default database.
<CODE>abort/0</CODE> does not close databases.

<DT><CODE>set_default_db(<VAR>+DBref</VAR>)</CODE>
<DD>
<DT><CODE>set_default_db(<VAR>+Name</VAR>)</CODE>
<DD>
<A NAME="IDX1212"></A>
Sets the database <VAR>Name</VAR> or <VAR>DBref</VAR> to be the default database
regardless of whether there already is one or not. 

<DT><CODE>get_default_db(<VAR>?DBref</VAR>)</CODE>
<DD>
<A NAME="IDX1213"></A>
Unifies <VAR>DBref</VAR> with the default database.

<DT><CODE>current_db(<VAR>?Name</VAR>, <VAR>?Mode</VAR>, <VAR>?Spec</VAR>, <VAR>?DBref</VAR>)</CODE>
<DD>
<A NAME="IDX1214"></A>
Unifies the arguments with the open databases. 
This predicate can be
used for enumerating all currently open databases through backtracking.

<DT><CODE>db_store(<VAR>+Term</VAR>, <VAR>-TermRef</VAR>)</CODE>
<DD>
<DT><CODE>db_store(<VAR>+DBref</VAR>, <VAR>+Term</VAR>, <VAR>-TermRef</VAR>)</CODE>
<DD>
<A NAME="IDX1215"></A>
<A NAME="IDX1216"></A>
Stores <VAR>Term</VAR> in the database <VAR>DBref</VAR>, which defaults to the
default database.  <VAR>TermRef</VAR> is unified with a corresponding term
reference.

<DT><CODE>db_fetch(<VAR>?Term</VAR>, <VAR>?TermRef</VAR>)</CODE>
<DD>
<DT><CODE>db_fetch(<VAR>+DBref</VAR>, <VAR>?Term</VAR>, <VAR>?TermRef</VAR>)</CODE>
<DD>
<A NAME="IDX1217"></A>
<A NAME="IDX1218"></A>
Unifies <VAR>Term</VAR> with a term from the database <VAR>DBref</VAR>, which
defaults to the default database.  At the same time,
<VAR>TermRef</VAR> is unified with a corresponding term reference.
Backtracking over the predicate unifies with all terms matching
<VAR>Term</VAR>.  If you simply want to find all matching terms,
it is more efficient to use <CODE>db_findall/(2-3)</CODE>.

If <VAR>TermRef</VAR> and <VAR>DBref</VAR> are instantiated (and the referenced
term is not erased), the referenced term is read and unified with
<VAR>Term</VAR>.

<DT><CODE>db_findall(<VAR>?Term</VAR>, <VAR>?TermList</VAR>)</CODE>
<DD>
<DT><CODE>db_findall(<VAR>+DBref</VAR>, <VAR>?Term</VAR>, <VAR>?TermList</VAR>)</CODE>
<DD>
<A NAME="IDX1219"></A>
<A NAME="IDX1220"></A>
Unifies <VAR>TermList</VAR> with the list of all terms matching <VAR>Term</VAR>
from the database <VAR>DBref</VAR>, which defaults to the default database.
The list is guaranteed to be free of duplicates,
except in the cases where the same term has actually been stored more
than once.

<DT><CODE>db_erase(<VAR>+TermRef</VAR>)</CODE>
<DD>
<DT><CODE>db_erase(<VAR>+DBref</VAR>, <VAR>+TermRef</VAR>)</CODE>
<DD>
<A NAME="IDX1221"></A>
<A NAME="IDX1222"></A>
Deletes the term in the database <VAR>DBref</VAR>, which defaults to the
default database, that is referenced by <VAR>TermRef</VAR> if
it is not already deleted.

<DT><CODE>db_canonical(<VAR>+TermRef</VAR>, <VAR>-TermID</VAR>)</CODE>
<DD>
<DT><CODE>db_canonical(<VAR>+DBref</VAR>, <VAR>+TermRef</VAR>, <VAR>-TermID</VAR>)</CODE>
<DD>
<A NAME="IDX1223"></A>
<A NAME="IDX1224"></A>
Returns the canonical term identifier for the term in <VAR>DBref</VAR>, which
defaults to the default database, that is referenced by <VAR>TermRef</VAR>.

<DT><CODE>db_buffering(<VAR>?Old</VAR>, <VAR>?New</VAR>)</CODE>
<DD>
<DT><CODE>db_buffering(<VAR>+DBref</VAR>, <VAR>?Old</VAR>, <VAR>?New</VAR>)</CODE>
<DD>
<A NAME="IDX1225"></A>
<A NAME="IDX1226"></A>
Unifies <VAR>Old</VAR> with the current buffering mode of <VAR>DBref</VAR>,
which defaults to the defaults database,
and sets its buffering mode to <VAR>New</VAR>, which must be <CODE>on</CODE>
or <CODE>off</CODE>.  Buffering is initially <CODE>off</CODE>.  When it is <CODE>on</CODE>,
modified pages are not immediately flushed to disk, enabling faster execution
but with a higher risk of inconsistencies if a crash occurs.  <CODE>db_close</CODE>
always flushes any modified pages.
</DL>



<H2><A NAME="SEC211" HREF="sicstus_toc.html#TOC211">An Example Session</A></H2>

<PRE>
| ?- db_open(my_db,update,on(on),R), set_default_db(R).

R = '$db'(1411928) ?

yes
| ?- db_store(a(b),_).

yes
| ?- db_store(a(c),_).

yes
| ?- db_fetch(X,_).

X = a(b) ? ;

X = a(c) ? ;

no
| ?- current_db(A,B,C,D).

A = my_db,
B = update,
C = on(on),
D = '$db'(1411928) ? ;

no
| ?- db_close.

yes
</PRE>



<H2><A NAME="SEC212" HREF="sicstus_toc.html#TOC212">The Db-Spec</A></H2>

<P>
A db-spec is on of the following terms:

</P>

<UL>
<LI>

the atom <CODE>on</CODE>, or

<LI>

the atom <CODE>off</CODE>, or

<LI>

a structure with functor <CODE>on</CODE> or <CODE>off</CODE> and every argument 
being a db-spec.
</UL>

<P>
The following table defines the way indices are calculated a bit more
formally.  The table defines the index as a function <VAR>INDEX</VAR> of the
db-spec and the indexed term.

</P>
<P>
The column <EM>Index Calculation</EM> describes the procedure: a "yes"
means that some primitive function is used, such as a hash function;
"<VAR>INDEX(s,t)</VAR>" means simply that the function definition (the
table) is applied again, but with a new db-spec <VAR>s</VAR> and a new term
<VAR>t</VAR>; <VAR>I</VAR> means <CODE>on</CODE> or <CODE>off</CODE>.

</P>

<PRE>
<EM>DB-Spec</EM>      <EM>Indexed Term</EM>    <EM>Index Calculation</EM>
<CODE>off</CODE>             any             no
<CODE>on</CODE>              any             yes (on principal functor)
<CODE><VAR>I</VAR>(...)</CODE>          atomic          yes
<CODE><VAR>I</VAR>(...)</CODE>          variable        yes
<CODE><VAR>I</VAR>(<VAR>S1</VAR>,<VAR>S2</VAR>...<VAR>Sn</VAR>)</CODE>    <VAR>F</VAR>(<VAR>A1</VAR>,<VAR>A2</VAR>...<VAR>Am</VAR>)  yes (on principal functor)
                                <VAR>INDEX(Si,Ai)</VAR> for 1 =&#60; <VAR>i</VAR> =&#60; min(<VAR>n</VAR>,<VAR>m</VAR>)
</PRE>

<P>
Every term is stored together with a set of "keywords" for indexing
purposes.  The space overhead is approximately 16 bytes per keyword
per term.  The number of keywords stored depends on the db-spec
and on the term being indexed.  The following table defines the
number of keywords as a function <VAR>K</VAR> of the db-spec and
the indexed term.

</P>

<PRE>
<EM>DB-Spec</EM>      <EM>Indexed Term</EM>    <EM>Number of keywords</EM>
<CODE>off</CODE>             any             0
<CODE>on</CODE>              any             2
<CODE><VAR>I</VAR>(...)</CODE>          atomic          2
<CODE><VAR>I</VAR>(...)</CODE>          variable        2
<CODE><VAR>I</VAR>(<VAR>S1</VAR>,...,<VAR>Sn</VAR>)</CODE>    <VAR>F</VAR>(<VAR>A1</VAR>,...,<VAR>Am</VAR>)  1+<VAR>K(S1,A1)*...*K(Sj,Aj)</VAR>, j=min(<VAR>n</VAR>,<VAR>m</VAR>)
</PRE>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_29.html">previous</A>, <A HREF="sicstus_31.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
