<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Boolean Constraint Solver</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_30.html">previous</A>, <A HREF="sicstus_32.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC213" HREF="sicstus_toc.html#TOC213">Boolean Constraint Solver</A></H1>

<P>
The clp(B) system provided by this library module is an instance of the
general Constraint Logic Programming scheme introduced in [Jaffar &#38;
Michaylov 87].  It is a solver for constraints over the Boolean domain,
i.e. the values 0 and 1.  This domain is particularly useful for
modeling digital circuits, and the constraint solver can be used for
verification, design, optimization etc. of such circuits.

</P>
<P>
To load the solver, enter the query:

</P>

<PRE>
| ?- use_module(library(clpb)).
</PRE>

<P>
The solver contains predicates for checking the consistency and
entailment of a constraint wrt. previous constraints, and for
computing particular solutions to the set of previous constraints.

</P>
<P>
The underlying representation of Boolean functions is based on Boolean
Decision Diagrams [Bryant 86].
This representation is very efficient, and allows many combinatorial
problems to be solved with good performance.

</P>
<P>
Boolean expressions are composed from the following operands: the
constants 0 and 1 (<CODE>FALSE</CODE> and <VAR>TRUE</VAR>), logical variables, and
symbolic constants, and from the following connectives.  <VAR>P</VAR> and
<VAR>Q</VAR> are Boolean expressions, <VAR>X</VAR> is a logical variable, <VAR>Is</VAR>
is a list of integers or integer ranges, and <VAR>Es</VAR> is a list of
Boolean expressions:

</P>
<DL COMPACT>

<DT><CODE>~ <VAR>P</VAR></CODE>
<DD>
<A NAME="IDX1227"></A>
True if <VAR>P</VAR> is false.

<DT><CODE><VAR>P</VAR> * <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1228"></A>
True if <VAR>P</VAR> and <VAR>Q</VAR> are both true.

<DT><CODE><VAR>P</VAR> + <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1229"></A>
True if at least one of <VAR>P</VAR> and <VAR>Q</VAR> is true.

<DT><CODE><VAR>P</VAR> # <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1230"></A>
True if exactly one of <VAR>P</VAR> and <VAR>Q</VAR> is true.

<DT><CODE><VAR>X</VAR> ^ <VAR>P</VAR></CODE>
<DD>
<A NAME="IDX1231"></A>
True if there exists an <VAR>X</VAR> such that <VAR>P</VAR> is true.  Same as
<CODE><VAR>P</VAR>[<VAR>X</VAR>/0] + <VAR>P</VAR>[<VAR>X</VAR>/1]</CODE>.

<DT><CODE><VAR>P</VAR> =:= <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1232"></A>
Same as <CODE>~<VAR>P</VAR> # <VAR>Q</VAR></CODE>.

<DT><CODE><VAR>P</VAR> =\= <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1233"></A>
Same as <CODE><VAR>P</VAR> # <VAR>Q</VAR></CODE>.

<DT><CODE><VAR>P</VAR> =&#60; <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1234"></A>
Same as <CODE>~<VAR>P</VAR> + <VAR>Q</VAR></CODE>.

<DT><CODE><VAR>P</VAR> &#62;= <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1235"></A>
Same as <CODE><VAR>P</VAR> + ~<VAR>Q</VAR></CODE>.

<DT><CODE><VAR>P</VAR> &#60; <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1236"></A>
Same as <CODE>~<VAR>P</VAR> * <VAR>Q</VAR></CODE>.

<DT><CODE><VAR>P</VAR> &#62; <VAR>Q</VAR></CODE>
<DD>
<A NAME="IDX1237"></A>
Same as <CODE><VAR>P</VAR> * ~<VAR>Q</VAR></CODE>.

<DT><CODE>card(<VAR>Is</VAR>, <VAR>Es</VAR>)</CODE>
<DD>
<A NAME="IDX1238"></A>
True if the number of true expressions in <VAR>Es</VAR> is a member of the
set denoted by <VAR>Is</VAR>.
</DL>

<P>
Symbolic constants (Prolog atoms) denote parametric values and can be
viewed as all-quantified variables whose quantifiers are placed
outside the entire expression.  They are useful for forcing certain
variables of an equation to be treated as input parameters.

</P>

<UL>
<LI><A HREF="sicstus_31.html#SEC214">CLPB Interface</A>
<LI><A HREF="sicstus_31.html#SEC215">CLPB Examples</A>
</UL>



<H2><A NAME="SEC214" HREF="sicstus_toc.html#TOC214">Solver Interface</A></H2>

<P>
The following predicates are defined:

</P>
<DL COMPACT>

<DT><CODE>sat(<VAR>+Expression</VAR>)</CODE>
<DD>
<A NAME="IDX1239"></A>
<VAR>Expression</VAR> is a Boolean expression.  This checks the consistency
of the expression wrt. the accumulated constraints, and, if the check
succeeds, <EM>tells</EM> the constraint that the expression be true.

If a variable <VAR>X</VAR>, occurring in the expression, is subsequently
unified with some term <VAR>T</VAR>, this is treated as a shorthand for the
constraint

<PRE>
?- sat(X=:=T).
</PRE>

<DT><CODE>taut(<VAR>+Expression</VAR>, <VAR>?Truth</VAR>)</CODE>
<DD>
<A NAME="IDX1240"></A>
<VAR>Expression</VAR> is a Boolean expression.  This <EM>asks</EM> whether the
expression is now entailed by the accumulated constraints
(<VAR>Truth</VAR>=1), or whether its negation is entailed by the accumulated
constraints (<VAR>Truth</VAR>=0).  Otherwise, it fails.

<DT><CODE>labeling(<VAR>+Variables</VAR>)</CODE>
<DD>
<A NAME="IDX1241"></A>
<VAR>Variables</VAR> is a list of variables.  The variables are instantiated
to a list of 0s and 1s, in a way that satisfies any accumulated
constraints.  Enumerates all solutions by backtracking, but creates
choicepoints only if necessary.
</DL>



<H2><A NAME="SEC215" HREF="sicstus_toc.html#TOC215">Examples</A></H2>

<UL>
<LI><A HREF="sicstus_31.html#SEC216">Example 1</A>
<LI><A HREF="sicstus_31.html#SEC217">Example 2</A>
<LI><A HREF="sicstus_31.html#SEC218">Example 3</A>: A one-bit adder circuit.
<LI><A HREF="sicstus_31.html#SEC219">Example 4</A>: Fault detection.
</UL>



<H3><A NAME="SEC216" HREF="sicstus_toc.html#TOC216">Example 1</A></H3>


<PRE>
| ?- sat(X + Y).

sat(X=\=_A*Y#Y) ? 
</PRE>

<P>
illustrates three facts.  First, any accumulated constraints affecting the
top-level variables are displayed as floundered goals, since the query
is not true for all <CODE>X</CODE> and <CODE>Y</CODE>.  Secondly, accumulated
constraints are displayed as <CODE>sat(<VAR>V</VAR>=:=<VAR>Expr</VAR>)</CODE>
or <CODE>sat(<VAR>V</VAR>=\=<VAR>Expr</VAR>)</CODE> where <VAR>V</VAR> is a variable and
<VAR>Expr</VAR> is a "polynomial", i.e. an exclusive or of conjunctions of
variables and constants.  Thirdly, <CODE>_A</CODE> had to be introduced as an
artificial variable, since <CODE>Y</CODE> cannot be expressed as a function of
<CODE>X</CODE>.  That is, <CODE>X + Y</CODE> is true iff there exists an <CODE>_A</CODE>
such that <CODE>X=\=_A*Y#Y</CODE>.  Let's check it!

</P>

<PRE>
| ?- taut(_A ^ (X=\=_A*Y#Y) =:= X + Y, T).

T = 1 ?
</PRE>

<P>
verifies the above answer.  Notice that the formula in this query is a
tautology, and so it is entailed by an empty set of constraints.

</P>


<H3><A NAME="SEC217" HREF="sicstus_toc.html#TOC217">Example 2</A></H3>


<PRE>
| ?- taut(A =&#60; C, T).

no
| ?- sat(A =&#60; B), sat(B =&#60; C), taut(A =&#60; C, T).

T = 1,
sat(A=:=_A*_B*C),
sat(B=:=_B*C) ? 

| ?- taut(a, T).

T = 0 ?

yes
| ?- taut(~a, T).

T = 0 ?
</PRE>

<P>
illustrates the entailment predicate.  In the first query, the
expression "A implies C" is neither known to be true nor false, so the
query fails.  In the second query, the system is told that "A implies
B" and "B implies C", so "A implies C" is entailed.  The
expressions in the third and fourth queries are to be read "for each a,
a is true" and "for each a, a is false", respectively, and so <CODE>T
= 0</CODE> in both cases since both are unsatisfiable.  This illustrates the
fact that the implicit universal quantifiers introduced by symbolic
constants are placed in front of the entire expression.

</P>


<H3><A NAME="SEC218" HREF="sicstus_toc.html#TOC218">Example 3</A></H3>


<PRE>
| ?- [user].
| adder(X, Y, Sum, Cin, Cout) :-
     sat(Sum =:= card([1,3],[X,Y,Cin])),
     sat(Cout =:= card([2-3],[X,Y,Cin])).
| {user consulted, 40 msec 576 bytes}

yes
| ?- adder(x, y, Sum, cin, Cout).

sat(Sum=:=cin#x#y),
sat(Cout=:=x*cin#x*y#y*cin) ?

yes
| ?- adder(x, y, Sum, 0, Cout).

sat(Sum=:=x#y),
sat(Cout=:=x*y) ?

yes
| ?- adder(X, Y, 0, Cin, 1), labeling([X,Y,Cin]).

Cin = 0,
X = 1,
Y = 1 ? ;

Cin = 1,
X = 0,
Y = 1 ? ;

Cin = 1,
X = 1,
Y = 0 ? ;
</PRE>

<P>
illustrates the use of cardinality constraints and models a one-bit
adder circuit.  The first query illustrates how representing the input
signals by symbolic constants forces the output signals to be displayed
as functions of the inputs and not vice versa.  The second query
computes the simplified functions obtained by setting carry-in to 0.
The third query asks for particular input values satisfying sum and
carry-out being 0 and 1, respectively.

</P>


<H3><A NAME="SEC219" HREF="sicstus_toc.html#TOC219">Example 4</A></H3>

<P>
The predicate <CODE>fault/3</CODE> below describes a 1-bit adder consisting of
five gates, with at most one faulty gate.  If one of the variables
<CODE>Fi</CODE> is equal to 1, the corresponding gate is faulty, and its
output signal is undefined (i.e., the constraint representing the gate
is relaxed).

</P>
<P>
Assuming that we have found some incorrect output from a circuit, we
are interesting in finding the faulty gate.  Two instances of incorrect
output are listed in <CODE>fault_ex/2</CODE>:

</P>

<PRE>
fault([F1,F2,F3,F4,F5], [X,Y,Cin], [Sum,Cout]) :-
        sat(
                    card([0-1],[F1,F2,F3,F4,F5]) *
                    (F1 + (U1 =:= X * Cin)) *
                    (F2 + (U2 =:= Y * U3)) *
                    (F3 + (Cout =:= U1 + U2)) *
                    (F4 + (U3 =:= X # Cin)) *
                    (F5 + (Sum =:= Y # U3))
                ).

fault_ex(1, Faults) :- fault(Faults, [1,1,0], [1,0]).
fault_ex(2, Faults) :- fault(Faults, [1,0,1], [0,0]).
</PRE>

<P>
To find the faulty gates, we run the query

</P>

<PRE>
| ?- fault_ex(I,L), labeling(L).

I = 1,
L = [0,0,0,1,0] ? ;

I = 2,
L = [1,0,0,0,0] ? ;

I = 2,
L = [0,0,1,0,0] ? ;

no
</PRE>

<P>
Thus for input data <CODE>[1,1,0]</CODE>, gate 4 must be faulty.
For input data <CODE>[1,0,1]</CODE>, either gate 1 or gate 3 must be faulty.

</P>
<P>
To get a symbolic representation of the outputs interms of the input,
we run the query

</P>

<PRE>
| ?- fault([0,0,0,0,0], [x,y,cin], [Sum,Cout]).

sat(Cout=:=x*cin#x*y#y*cin),
sat(Sum=:=cin#x#y)
</PRE>

<P>
which shows that the sum and carry out signals indeed compute the
intended functions if no gate is faulty.

</P>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_30.html">previous</A>, <A HREF="sicstus_32.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
