<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Constraint Logic Programming over Finite Domains</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_32.html">previous</A>, <A HREF="sicstus_34.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC248" HREF="sicstus_toc.html#TOC248">Constraint Logic Programming over Finite Domains</A></H1>


<UL>
<LI><A HREF="sicstus_33.html#SEC249">CLPFD Intro</A>
<LI><A HREF="sicstus_33.html#SEC252">CLPFD Interface</A>
<LI><A HREF="sicstus_33.html#SEC256">Available Constraints</A>
<LI><A HREF="sicstus_33.html#SEC262">Enumeration Predicates</A>
<LI><A HREF="sicstus_33.html#SEC263">Statistics Predicates</A>
<LI><A HREF="sicstus_33.html#SEC264">Answer Constraints</A>
<LI><A HREF="sicstus_33.html#SEC265">The Constraint System</A>
<LI><A HREF="sicstus_33.html#SEC268">Defining Global Constraints</A>
<LI><A HREF="sicstus_33.html#SEC273">Defining Primitive Constraints</A>
<LI><A HREF="sicstus_33.html#SEC282">Example Programs</A>
<LI><A HREF="sicstus_33.html#SEC286">Syntax Summary</A>
</UL>



<H2><A NAME="SEC249" HREF="sicstus_toc.html#TOC249">Introduction</A></H2>

<P>
The clp(FD) solver described in this chapter is an instance of the general
Constraint Logic Programming scheme introduced in [Jaffar &#38; Michaylov
87].  This constraint domain is particularly useful for modeling
discrete optimization and verification problems such as scheduling,
planning, packing, timetabling etc.  The treatise [Van Hentenryck 89]
is an excellent exposition of the theoretical and practical framework
behind constraint solving in finite domains, and summarizes the work
up to 1989.

</P>
<P>
This solver has the following highlights:

</P>

<UL>
<LI>

Two classes of constraints are handled internally: primitive constraints
and global constraints.
<LI>

The constraints described in this chapter are automatically translated
to conjunctions of primitive and global library constraints.
<LI>

The truth value of a primitive constraint can be reflected into a 0/1-variable
(reification).
<LI>

New primitive constraints can be added by writing so-called indexicals.
<LI>

New global constraints can be written in Prolog, by means of a
programming interface.
</UL>


<UL>
<LI><A HREF="sicstus_33.html#SEC250">Referencing CLPFD</A>
<LI><A HREF="sicstus_33.html#SEC251">Acknowledgments CLPFD</A>
</UL>

<P>
The rest of this chapter is organized as follows:
How to load the solver and how to write simple programs is explained in
section <A HREF="sicstus_33.html#SEC252">Solver Interface</A>.
A description of all constraints that the solver provides is contained
in section <A HREF="sicstus_33.html#SEC256">Available Constraints</A>.
The predicates for searching for solution are documented in
section <A HREF="sicstus_33.html#SEC262">Enumeration Predicates</A>.
The predicates for getting execution statistics are documented in
section <A HREF="sicstus_33.html#SEC263">Statistics Predicates</A>.
A few example programs are given in section <A HREF="sicstus_33.html#SEC282">Example Programs</A>.
Finally, section <A HREF="sicstus_33.html#SEC286">Syntax Summary</A> contains syntax rules for all expressions.

</P>
<P>
The following sections discuss advanced features and are probably only
relevant to experienced users:
How to control the amount of information presented in answers to queries
is explained in section <A HREF="sicstus_33.html#SEC264">Answer Constraints</A>.
The solver's execution mechanism and primitives are described in
section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.
How to add new global constraints via a programming interface
is described in section <A HREF="sicstus_33.html#SEC268">Defining Global Constraints</A>.
How to define new primitive constraints with indexicals is described in
section <A HREF="sicstus_33.html#SEC273">Defining Primitive Constraints</A>.

</P>



<H3><A NAME="SEC250" HREF="sicstus_toc.html#TOC250">Referencing this Software</A></H3>

<P>
When referring to this implementation of clp(FD) in publications,
please use the following reference:

<BLOCKQUOTE>
<P>
Carlsson M., Ottosson G., Carlson B.
"An Open-Ended Finite Domain Constraint Solver"
Proc. Programming Languages: Implementations, Logics, and Programs, 1997. 
</BLOCKQUOTE>



<H3><A NAME="SEC251" HREF="sicstus_toc.html#TOC251">Acknowledgments</A></H3>

<P>
The first version of this solver was written as part of Key
Hyckenberg's MSc thesis in 1995.  The code was later rewritten by Mats
Carlsson, with contributions from Greger Ottosson at the Computing
Science Department, Uppsala University.  Peter Szeredi contributed
material for this manual chapter.

</P>
<P>
The development of this software was supported by the Swedish National
Board for Technical and Industrial Development (NUTEK) under the
auspices of Advanced Software Technology (ASTEC) Center of Competence
at Uppsala University.

</P>
<P>
We include a collection of examples, some of which have been
distributed with the INRIA implementation of clp(FD) [Diaz &#38; Codognet 93].

</P>


<H2><A NAME="SEC252" HREF="sicstus_toc.html#TOC252">Solver Interface</A></H2>

<P>
The solver is available as a library module and can be loaded with a query

</P>

<PRE>
:- use_module(library(clpfd)).
</PRE>

<P>
The solver contains predicates for checking the consistency and
entailment of finite domain constraints, as well as solving for solution
values for your problem variables. 

</P>
<P>
<A NAME="IDX1257"></A>
<A NAME="IDX1258"></A>
<A NAME="IDX1259"></A>
<A NAME="IDX1260"></A>
<A NAME="IDX1261"></A>
In the context of this constraint solver, a <EM>finite domain</EM> is a
subset of the integers, and a <EM>finite domain constraint</EM>
denotes a relation over one or more finite domains.
All <EM>domain variables</EM>, i.e. 
variables that occur as arguments to finite domain constraints get
associated with a finite domain, either explicitly declared by the
program, or implicitly imposed by the constraint solver.  Temporarily,
the domain of a variable may actually be infinite, if it does not have
a finite lower or upper bound.

</P>
<P>
The domain of all variables gets narrower and narrower as more
constraints are added.  If a domain becomes empty, the accumulated
constraints are unsatisfiable, and the current computation branch
fails.  At the end of a successful computation, all domains have
usually become singletons, i.e. the domain variables have become
assigned.

</P>
<P>
The domains do not become singletons automatically.  Usually, it takes
some amount of search to find an assignment that satisfies all
constraints.  It is the programmer's responsibility to do so.  If some
domain variables are left unassigned in a computation, the garbage
collector will preserve all constraint data that is attached to them.
Furthermore, the behavior of the predicates <CODE>assert/1</CODE>,
<CODE>findall/3</CODE>, <CODE>copy_term/2</CODE> and friends is undefined on
non-ground terms containing domain variables.

</P>

<P>
The heart of the constraint solver is a scheduler for indexicals [Van
Hentenryck et al. 92] and global constraints.  Both entities act as
coroutines performing incremental constraint solving or entailment
checking.  They wake up by changes in the domains of its arguments.  All
constraints provided by this package are implemented as indexicals or
global constraints.  New constraints can be defined by the user.

</P>
<P>
Indexicals are reactive functional rules that take part in the solver's
basic constraint solving algorithm, whereas each global constraint is
associated with its particular constraint solving algorithm.  The
solver maintains two scheduling queues, giving priority to the queue of
indexicals.

</P>
<P>
The feasibility of integrating the indexical approach with a Prolog
based on the WAM was clearly demonstrated by Diaz's clp(FD)
implementation [Diaz &#38; Codognet 93], one of the fastest finite domains
solvers around.

</P>


<UL>
<LI><A HREF="sicstus_33.html#SEC253">Posting Constraints</A>
<LI><A HREF="sicstus_33.html#SEC254">A Constraint Satisfaction Problem</A>
<LI><A HREF="sicstus_33.html#SEC255">Reified Constraints</A>
</UL>



<H3><A NAME="SEC253" HREF="sicstus_toc.html#TOC253">Posting Constraints</A></H3>

<P>
<A NAME="IDX1262"></A>
A constraint is called as any other Prolog predicate.  When called,
the constraint is <EM>posted</EM> to the store.  For example:

</P>

<PRE>
| ?- X in 1..5, Y in 2..8, X+Y #= T.

X in 1..5,
Y in 2..8,
T in 3..13 ? 

yes
| ?- X in 1..5, T in 3..13, X+Y #= T.

X in 1..5,
T in 3..13,
Y in -2..12 ? 

yes
</PRE>

<P>
Note that the answer constraint shows the domains of nonground query variables,
but not any constraints that may be attached to them.

</P>


<H3><A NAME="SEC254" HREF="sicstus_toc.html#TOC254">A Constraint Satisfaction Problem</A></H3>

<P>
Constraint satisfaction problems (CSPs) are a major class of problems
for which this solver is ideally suited.  In a CSP, the goal is to
pick values from pre-defined domains for certain variables so that
the given constraints on the variables are all satisfied.

</P>
<P>
As a simple CSP example, let us consider the Send More Money puzzle.
In this problem, the variables are the letters S, E, N, D, M, O, R,
and Y.  Each letter represents a digit between 0 and 9.  The problem
is to assign a value to each digit, such that SEND + MORE equals
MONEY.

</P>
<P>
A program which solves the puzzle is given below.  The program contains
the typical three steps of a clp(FD) program:

</P>

<OL>
<LI>

declare the domains of the variables
<LI>

post the problem constraints
<LI>

look for a feasible solution via backtrack search, or
look for an optimal solution via branch-and-bound search
</OL>

<P>
Sometimes, an extra step precedes the search for a solution: the posting of
surrogate constraints to break symmetries or to otherwise help prune
the search space.  No surrogate constraints are used in this example.

</P>
<P>
The domains of this puzzle are stated via the <CODE>domain/3</CODE> goal
and by requiring that S and M be greater than zero.
The two problem constraint of this puzzle are the equation (<CODE>sum/8</CODE>)
and the constraint that all letters take distinct values
(<CODE>all_different/1</CODE>).
Finally, the backtrack search is performed by <CODE>labeling/2</CODE>.  Different
search strategies can be encoded in the <CODE>Type</CODE> parameter.  In the example
query, the default search strategy is used (select the leftmost variable,
try values in ascending order).

</P>

<PRE>
:- use_module(library(clpfd)).

mm([S,E,N,D,M,O,R,Y], Type) :-
     domain([S,E,N,D,M,O,R,Y], 0, 9),      % step 1
     S#&#62;0, M#&#62;0,
     all_different([S,E,N,D,M,O,R,Y]),     % step 2
     sum(S,E,N,D,M,O,R,Y),
     labeling(Type, [S,E,N,D,M,O,R,Y]).    % step 3

sum(S, E, N, D, M, O, R, Y) :-
                  1000*S + 100*E + 10*N + D 
     +            1000*M + 100*O + 10*R + E
     #= 10000*M + 1000*O + 100*N + 10*E + Y.

| ?- mm([S,E,N,D,M,O,R,Y], []).

D = 7,
E = 5,
M = 1,
N = 6,
O = 0,
R = 8,
S = 9,
Y = 2 ? 
</PRE>



<H3><A NAME="SEC255" HREF="sicstus_toc.html#TOC255">Reified Constraints</A></H3>

<P>
Instead of merely posting constraints it is often useful to 
reflect its truth value into a 0/1-variable <VAR>B</VAR>, so that:

</P>

<UL>
<LI>the constraint is posted if <VAR>B</VAR> is set to 1

<LI>the negation of the constraint is posted if <VAR>B</VAR> is set to 0

<LI><VAR>B</VAR> is set to 1 if the constraint becomes entailed

<LI><VAR>B</VAR> is set to 0 if the constraint becomes disentailed

</UL>

<P>
<A NAME="IDX1263"></A>
<A NAME="IDX1264"></A>
This mechanism is known as <EM>reification</EM>.  Several frequently used
operations can be defined in terms of reified constraints, such as
blocking implication [Saraswat 90] and the cardinality operator [Van
Hentenryck &#38; Deville 91], to name a few.  A reified constraint is
written:

</P>

<PRE>
| ?- <VAR>Constraint</VAR> #&#60;=&#62; <VAR>B</VAR>.
</PRE>

<P>
where <VAR>Constraint</VAR> is reifiable.  As an example of a constraint
that uses reification, consider <CODE>exactly(X,L,N)</CODE> which is true if <CODE>X</CODE>
occurs exactly <CODE>N</CODE> times in the list <CODE>L</CODE>.  It can be defined thus:

</P>

<PRE>
exactly(_, [], 0).
exactly(X, [Y|L], N) :-
    X #= Y #&#60;=&#62; B,
    N #= M+B,
    exactly(X, L, M).
</PRE>



<H2><A NAME="SEC256" HREF="sicstus_toc.html#TOC256">Available Constraints</A></H2>

<P>
This section describes the classes of constraints that can be used with this
solver.

</P>


<UL>
<LI><A HREF="sicstus_33.html#SEC257">Arithmetic Constraints</A>
<LI><A HREF="sicstus_33.html#SEC258">Membership Constraints</A>
<LI><A HREF="sicstus_33.html#SEC259">Propositional Constraints</A>
<LI><A HREF="sicstus_33.html#SEC260">Combinatorial Constraints</A>
<LI><A HREF="sicstus_33.html#SEC261">User-Defined Constraints</A>
</UL>



<H3><A NAME="SEC257" HREF="sicstus_toc.html#TOC257">Arithmetic Constraints</A></H3>

<DL COMPACT>

<DT><CODE><VAR>?Expr</VAR> <VAR>RelOp</VAR> <VAR>?Expr</VAR></CODE>
<DD>
<A NAME="IDX1265"></A>
<A NAME="IDX1266"></A>
<A NAME="IDX1267"></A>
<A NAME="IDX1268"></A>
<A NAME="IDX1269"></A>
<A NAME="IDX1270"></A>
defines an arithmetic constraint.  The syntax for <VAR>Expr</VAR> and
<VAR>RelOp</VAR> is defined by a grammar (see section <A HREF="sicstus_33.html#SEC288">Syntax of Arithmetic Expressions</A>).  Note that the expressions are not restricted to being
linear.  Constraints over non-linear expressions, however, will usually
yield less constraint propagation than constraints over linear expressions.

Arithmetic constraints can be reified as e.g.


<PRE>
| ?- X in 1..2, Y in 3..5, X#=&#60;Y #&#60;=&#62; B.

B = 1,
X in 1..2,
Y in 3..5 ? 
</PRE>

</DL>

<P>
Linear arithmetic constraints maintain (at least) interval-consistency and
their reified versions detect (at least) interval-entailment and
-disentailment; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.

</P>
<P>
The following constraints are among the library constraints that general
arithmetic constraints compile to.  They express a relation between a
sum or a scalar product and a value, using a dedicated algorithm that
avoids creating any temporary variables holding intermediate values.  If
you are computing a sum or a scalar product, it can be much more
efficient to compute lists of coefficients and variables and post a
single sum or scalar product constraint than to post a sequence of
elementary constraints.

</P>
<DL COMPACT>

<DT><CODE>sum(<VAR>+Xs</VAR>, <VAR>+RelOp</VAR>, <VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX1271"></A>
where <VAR>Xs</VAR> is a list of integers or domain
variables, <VAR>RelOp</VAR> is a relational symbol as above, and
<VAR>Value</VAR> is an integer or a domain variable.  True if
<VAR>Xs RelOp Value</VAR>.  Cannot be reified.

<DT><CODE>scalar_product(<VAR>+Coeffs</VAR>, <VAR>+Xs</VAR>, <VAR>+RelOp</VAR>, <VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX1272"></A>
where <VAR>Coeffs</VAR> is a list of length <VAR>n</VAR> of integers,
<VAR>Xs</VAR> is a list of length <VAR>n</VAR> of integers or domain
variables, <VAR>RelOp</VAR> is a relational symbol as above, and
<VAR>Value</VAR> is an integer or a domain variable.  True if
<VAR>Coeffs*Xs RelOp Value</VAR>.  Cannot be reified.

</DL>



<H3><A NAME="SEC258" HREF="sicstus_toc.html#TOC258">Membership Constraints</A></H3>

<DL COMPACT>

<DT><CODE>domain(<VAR>+Variables</VAR>, <VAR>+Min</VAR>, <VAR>+Max</VAR>)</CODE>
<DD>
<A NAME="IDX1273"></A>
where <VAR>Variables</VAR> is a list of domain variables or integers,
<VAR>Min</VAR> is an integer or the atom <CODE>inf</CODE> (minus infinity), and
<VAR>Max</VAR> is an integer or the atom <CODE>sup</CODE> (plus infinity).  True if
the variables all are elements of the range <CODE><VAR>Min</VAR>..<VAR>Max</VAR></CODE>.
Cannot be reified.

<DT><CODE><VAR>?X</VAR> in <VAR>+Range</VAR></CODE>
<DD>
<A NAME="IDX1274"></A>
defines a membership constraint.  <VAR>X</VAR> is an integer or a domain
variable and <VAR>Range</VAR> is a <VAR>ConstantRange</VAR> (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>).  True if <VAR>X</VAR> is an element
of the range.

<DT><CODE><VAR>?X</VAR> in_set <VAR>+FDSet</VAR></CODE>
<DD>
<A NAME="IDX1275"></A>
defines a membership constraint.
<VAR>X</VAR> is an integer or a domain variable and <VAR>FDSet</VAR> is an
FD set term (see section <A HREF="sicstus_33.html#SEC271">FD Set Operations</A>).
True if <VAR>X</VAR> is an element of the FD set.
</DL>

<P>
<CODE>in/2</CODE> and <CODE>in_set/2</CODE> constraints can be reified.  They
maintain domain-consistency and their reified versions detect
domain-entailment and -disentailment; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.

</P>


<H3><A NAME="SEC259" HREF="sicstus_toc.html#TOC259">Propositional Constraints</A></H3>

<P>
Propositional combinators can be used to combine reifiable constraints
into propositional formulae over such constraints.  Such formulae are
goal expanded by the system into sequences of reified
constraints and arithmetic constraints.  For example,

</P>

<PRE>
X #= 4 #\/ Y #= 6
</PRE>

<P>
expresses the disjunction of two equality constraints.

</P>
<P>
The leaves of propositional formulae can be reifiable constraints, the
constants 0 and 1, or 0/1-variables.  New primitive, reifiable
constraints can be defined with indexicals as described in section <A HREF="sicstus_33.html#SEC273">Defining Primitive Constraints</A>.  The following propositional combinators are
available:

</P>
<DL COMPACT>

<DT><CODE>#\ <VAR>:Q</VAR></CODE>
<DD>
<A NAME="IDX1276"></A>
True if the constraint <VAR>Q</VAR> is false.

<DT><CODE><VAR>:P</VAR> #/\ <VAR>:Q</VAR></CODE>
<DD>
<A NAME="IDX1277"></A>
True if the constraints <VAR>P</VAR> and <VAR>Q</VAR> are both true.

<DT><CODE><VAR>:P</VAR> #\ <VAR>:Q</VAR></CODE>
<DD>
<A NAME="IDX1278"></A>
True if exactly one of the constraints <VAR>P</VAR> and <VAR>Q</VAR> is true.

<DT><CODE><VAR>:P</VAR> #\/ <VAR>:Q</VAR></CODE>
<DD>
<A NAME="IDX1279"></A>
True if at least one of the constraints <VAR>P</VAR> and <VAR>Q</VAR> is true.

<DT><CODE><VAR>:P</VAR> #=&#62; <VAR>:Q</VAR></CODE>
<DD>
<DT><CODE><VAR>:Q</VAR> #&#60;= <VAR>:P</VAR></CODE>
<DD>
<A NAME="IDX1280"></A>
<A NAME="IDX1281"></A>
True if the constraint <VAR>Q</VAR> is true or the constraint <VAR>P</VAR> is false.

<DT><CODE><VAR>:P</VAR> #&#60;=&#62; <VAR>:Q</VAR></CODE>
<DD>
<A NAME="IDX1282"></A>
True if the constraints <VAR>P</VAR> and <VAR>Q</VAR> are both true or both false.
</DL>

<P>
Note that the reification scheme introduced in section <A HREF="sicstus_33.html#SEC255">Reified Constraints</A>
is a special case of a propositional constraint.

</P>


<H3><A NAME="SEC260" HREF="sicstus_toc.html#TOC260">Combinatorial Constraints</A></H3>

<P>
The constraints listed here are sometimes called symbolic constraints.
They are currently not reifiable.

</P>
<DL COMPACT>

<DT><CODE>count(<VAR>+Val</VAR>,<VAR>+List</VAR>,<VAR>+RelOp</VAR>,<VAR>?Count</VAR>)</CODE>
<DD>
<A NAME="IDX1283"></A>
where <VAR>Val</VAR> is an integer, <VAR>List</VAR> is a list of integers or
domain variables, <VAR>Count</VAR> an integer or a domain variable, and
<VAR>RelOp</VAR> is a relational symbol as in section <A HREF="sicstus_33.html#SEC257">Arithmetic Constraints</A>.
True if <VAR>N</VAR> is the number of elements of <VAR>List</VAR> that are
equal to <VAR>Val</VAR> and <VAR>N RelOp Count</VAR>.  Thus, <CODE>count/4</CODE> is
a generalization of <CODE>exactly/3</CODE> (not an exported constraint) that was
used in an example earlier.

<CODE>count/4</CODE> maintains domain-consistency; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.

<DT><CODE>element(<VAR>?X</VAR>,<VAR>+List</VAR>,<VAR>?Y</VAR>)</CODE>
<DD>
<A NAME="IDX1284"></A>
where <VAR>X</VAR> and <VAR>Y</VAR> are integers or domain variables and
<VAR>List</VAR> is a list of integers or domain variables.  True if the
<VAR>X</VAR>:th element of <VAR>List</VAR> is <VAR>Y</VAR>.  Operationally, the
domains of <VAR>X</VAR> and <VAR>Y</VAR> are constrained so that for every element
in the domain of <VAR>X</VAR>, there is a compatible element in the domain of
<VAR>Y</VAR>, and vice versa.

This constraint uses an optimized algorithm for the special case where
<VAR>List</VAR> is ground.

<CODE>element/3</CODE> maintains domain-consistency; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.

<DT><CODE>relation(<VAR>?X</VAR>,<VAR>+MapList</VAR>,<VAR>?Y</VAR>)</CODE>
<DD>
<A NAME="IDX1285"></A>
where <VAR>X</VAR> and <VAR>Y</VAR> are integers or domain variables and
<VAR>MapList</VAR> is a list of <CODE><VAR>integer</VAR>-<VAR>ConstantRange</VAR></CODE> pairs,
where the integer keys occur uniquely (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>).
True if <VAR>MapList</VAR> contains a pair <CODE><VAR>X</VAR>-<VAR>R</VAR></CODE> and
<VAR>Y</VAR> is in the range denoted by <VAR>R</VAR>.

Operationally, the domains of <VAR>X</VAR> and <VAR>Y</VAR> are constrained
so that for every element in the domain of <VAR>X</VAR>, there is a
compatible element in the domain of <VAR>Y</VAR>, and vice versa.

If <VAR>MapList</VAR> is not ground, the constraint must be wrapped in <CODE>call/1</CODE>
to postpone goal expansion until runtime.

An arbitrary binary constraint can be defined with <CODE>relation/3</CODE>.
<CODE>relation/3</CODE> maintains domain-consistency; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.

<DT><CODE>all_different(<VAR>+Variables</VAR>)</CODE>
<DD>
<A NAME="IDX1286"></A>
where <VAR>Variables</VAR> is a list of domain variables or integers.  Each
variable is constrained to take a value that is unique among the
variables.  Operationally, this is equivalent to posting an inequality
constraint for each pair of variables.

This constraint wakes up each time a variable becomes ground, pruning
the domains of the other variables.  The implementation is incomplete,
i.e. domain elements for <VAR>Variables</VAR> for which the constraint has
no solutions are not always eliminated.  A complete version is provided by
the <CODE>all_distinct/1</CODE> constraint (see below).
</DL>

<P>
The following constraint is a complete version of the
<CODE>all_different/1</CODE> constraint.  The algorithm is due to Regin [Regin
94].

</P>
<DL COMPACT>

<DT><CODE>all_distinct(<VAR>+Variables</VAR>)</CODE>
<DD>
<A NAME="IDX1287"></A>
where <VAR>Variables</VAR> is a list of domain variables or integers.  Each
variable is constrained to take a value that is unique among the
variables.  The implementation is complete, i.e. it maintains
domain-consistency; see section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>.
</DL>

<P>
The following is a constraint over two lists of length <VAR>n</VAR> of
variables.  Each variable is constrained to take a value in
1,...,<VAR>n</VAR> that is unique for its list.  Furthermore, the lists
are dual in a sense described below.

</P>
<DL COMPACT>

<DT><CODE>assignment(<VAR>+Xs</VAR>, <VAR>+Ys</VAR>)</CODE>
<DD>
<A NAME="IDX1288"></A>
where <VAR>Xs</VAR> and <VAR>Ys</VAR> are lists of domain variables or integers,
both of length <VAR>n</VAR>.  True if all <VAR>Xi</VAR>, <VAR>Yi</VAR> in 1,...,<VAR>n</VAR>
and <VAR>Xi=j</VAR> iff <VAR>Yj=i</VAR>.
</DL>

<P>
The following constraint can be thought of as constraining <VAR>n</VAR>
nodes in a graph to form a Hamiltonian circuit.  The nodes are
numbered from 1 to <VAR>n</VAR>.  The circuit starts in node 1, visits each
node, and returns to the origin.

</P>
<DL COMPACT>

<DT><CODE>circuit(<VAR>+Succ</VAR>)</CODE>
<DD>
<DT><CODE>circuit(<VAR>+Succ</VAR>, <VAR>+Pred</VAR>)</CODE>
<DD>
<A NAME="IDX1289"></A>
<A NAME="IDX1290"></A>
where <VAR>Succ</VAR> is a list of length <VAR>n</VAR> of domain variables or
integers.  The <VAR>i</VAR>:th element of <VAR>Succ</VAR> (<VAR>Pred</VAR>) is the
successor (predecessor) of <VAR>i</VAR> in the graph.
True if the values form a Hamiltonian circuit.
</DL>

<P>
The following four constraints can be thought of as constraining <VAR>n</VAR>
tasks, each with a start time <VAR>Sj</VAR> and a duration <VAR>Dj</VAR>, so that
no tasks ever overlap.  The tasks can be seen as competing for some
exclusive resource.

</P>
<DL COMPACT>

<DT><CODE>serialized(<VAR>+Starts</VAR>,<VAR>+Durations</VAR>)</CODE>
<DD>
<A NAME="IDX1291"></A>
where <VAR>Starts</VAR> = [<VAR>S1</VAR>,...,<VAR>Sn</VAR>] and
<VAR>Durations</VAR> = [<VAR>D1</VAR>,...,<VAR>Dn</VAR>] are lists of
domain variables with finite bounds or integers. True if:


<PRE>
<VAR>Si</VAR>+<VAR>Di</VAR> =&#60; <VAR>Sj</VAR> OR <VAR>Sj</VAR>+<VAR>Dj</VAR> =&#60; <VAR>Si</VAR>, for all 1 =&#60; <VAR>i</VAR>&#60;<VAR>j</VAR> =&#60; <VAR>n</VAR>
</PRE>

The <CODE>serialized/2</CODE> constraint is merely a special case of
<CODE>cumulative/4</CODE> (see below).

<DT><CODE>serialized_resource(<VAR>+Starts</VAR>,<VAR>+Durations</VAR>,<VAR>-Resource</VAR>)</CODE>
<DD>
<A NAME="IDX1292"></A>
expresses the same constraint, returning in <VAR>Resource</VAR> a term which
can be passed to <CODE>order_resource/2</CODE> (see section <A HREF="sicstus_33.html#SEC262">Enumeration Predicates</A>) in order to find a consistent ordering of the tasks.

<DT><CODE>serialized_precedence(<VAR>+Starts</VAR>,<VAR>+Durations</VAR>,<VAR>+Precedences</VAR>)</CODE>
<DD>
<A NAME="IDX1293"></A>
<DT><CODE>serialized_precedence_resource(<VAR>+Starts</VAR>,<VAR>+Durations</VAR>,<VAR>+Precedences</VAR>,<VAR>-Resource</VAR>)</CODE>
<DD>
<A NAME="IDX1294"></A>
express the same constraint, combined with some extra precedence
constraint as encoded in <VAR>Precedences</VAR> which should be a list of
terms of the form <CODE>d(<VAR>i</VAR>,<VAR>j</VAR>,<VAR>d</VAR>)</CODE>.  Here, <VAR>i</VAR> and
<VAR>j</VAR> should be task numbers, and <VAR>d</VAR> should be a positive integer
or <CODE>sup</CODE>.  Each term adds the constraint:


<PRE>
<VAR>Si</VAR>+<VAR>d</VAR> =&#60; <VAR>Sj</VAR> OR <VAR>Sj</VAR> =&#60; <VAR>Si</VAR>, if <VAR>d</VAR> is an integer
<VAR>Sj</VAR> =&#60; <VAR>Si</VAR>, if <VAR>d</VAR> is sup
</PRE>

<CODE>serialized_precedence/3</CODE> can model a set of tasks to be serialized
with sequence-dependent setup times.  For example, the following
constraint models three tasks, all with duration 5, where task 1 must
precede task 2 and task 3 must either complete before task 2 or start
at least 10 time units after task 2 started:


<PRE>
?- domain([S1,S2,S3], 0, 20),
   serialized_precedence([S1,S2,S3], [5,5,5], [d(2,1,sup),d(2,3,10)]).

S1 in 0..15,
S2 in 5..20,
S3 in 0..20 ? 
</PRE>

The bounds of <CODE>S1</CODE> and <CODE>S2</CODE> changed because of the precedence
constraint.  Setting <CODE>S2</CODE> to 5 will propagate <CODE>S1=0</CODE> and <CODE>S3
in 15..20</CODE>.
</DL>

<P>
The following constraint
can be thought of as constraining <VAR>n</VAR> tasks, each
with a start time <VAR>Sj</VAR>, a duration <VAR>Dj</VAR>, and a resource amount
<VAR>Rj</VAR>, so that the total resource consumption does not exceed
<VAR>Limit</VAR> at any time:

</P>
<DL COMPACT>

<DT><CODE>cumulative(<VAR>+Starts</VAR>,<VAR>+Durations</VAR>,<VAR>+Resources</VAR>,<VAR>?Limit</VAR>)</CODE>
<DD>
<A NAME="IDX1295"></A>
where <VAR>Starts</VAR> = [<VAR>S1</VAR>,...,<VAR>Sn</VAR>],
<VAR>Durations</VAR> = [<VAR>D1</VAR>,...,<VAR>Dn</VAR>],
<VAR>Resource</VAR> = [<VAR>R1</VAR>,...,<VAR>Rn</VAR>] are lists of
domain variables with finite bounds or integers, and <VAR>Limit</VAR> is a
domain variable with finite bounds or an integer.  Let


<PRE>
<VAR>a</VAR> = min(<VAR>S1</VAR>,...,<VAR>Sn</VAR>),
<VAR>b</VAR> = max(<VAR>S1</VAR>+<VAR>D1</VAR>,...,<VAR>Sn</VAR>+<VAR>Dn</VAR>)
<VAR>Rij</VAR> = <VAR>Rj</VAR>, if <VAR>Sj</VAR> =&#60; <VAR>i</VAR> &#60; <VAR>Sj</VAR>+<VAR>Dj</VAR>
<VAR>Rij</VAR> = 0 otherwise
</PRE>

The constraint holds if:


<PRE>
<VAR>Ri1</VAR>+...+<VAR>Rin</VAR> =&#60; <VAR>Limit</VAR>, for all <VAR>a</VAR> =&#60; <VAR>i</VAR> &#60; <VAR>b</VAR>
</PRE>

The <CODE>cumulative/4</CODE> constraint is due to Aggoun and Beldiceanu
[Aggoun &#38; Beldiceanu 92].  The current implementation is incomplete
and takes <VAR>O(n^2)</VAR> space.  
</DL>



<H3><A NAME="SEC261" HREF="sicstus_toc.html#TOC261">User-Defined Constraints</A></H3>

<P>
New, primitive constraints can be added defined by the user on two
different levels.  On a higher level, constraints can be defined using
the global constraint programming interface; see section <A HREF="sicstus_33.html#SEC268">Defining Global Constraints</A>.  Such constraints can embody specialized algorithms and
use the full power of Prolog.  They cannot be reified.

</P>
<P>
On a lower level, new primitive constraints can be defined with
indexicals.  In this case, they take part in the basic constraint
solving algorithm and express custom designed rules for special cases of
the overall local propagation scheme.  Such constraints are called
<EM>FD predicates</EM>; see section <A HREF="sicstus_33.html#SEC273">Defining Primitive Constraints</A>.  They can
optionally be reified.

</P>


<H2><A NAME="SEC262" HREF="sicstus_toc.html#TOC262">Enumeration Predicates</A></H2>

<P>
As is usually the case with finite domain constraint solvers,
this solver is not <EM>complete</EM>.  That is, it does not ensure that
the set of posted constraints is satisfiable.  One must resort to search
(enumeration) to check satisfiability and get particular solutions.

</P>
<P>
The following predicates provide several variants of search:

</P>
<DL COMPACT>

<DT><CODE>indomain(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX1296"></A>
where <VAR>X</VAR> is a domain variable with finite bounds or an integer.
Assigns, in increasing order via backtracking, a feasible value to <VAR>X</VAR>.

<DT><CODE>minimize(<VAR>:Goal</VAR>,<VAR>?X</VAR>)</CODE>
<DD>
<DT><CODE>maximize(<VAR>:Goal</VAR>,<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX1297"></A>
<A NAME="IDX1298"></A>
Uses a branch-and-bound algorithm with restart to find an assignment
that minimizes (maximizes) the domain variable <VAR>X</VAR>.  <VAR>Goal</VAR>
should be a Prolog goal that constrains <VAR>X</VAR> to become assigned,
and could be a <CODE>labeling/2</CODE> goal.  The algorithm calls <VAR>Goal</VAR>
repeatedly with a progressively tighter upper (lower) bound on <VAR>X</VAR>
until a proof of optimality is obtained, at which time <VAR>Goal</VAR> and
<VAR>X</VAR> are unified with values corresponding to the optimal solution.

<DT><CODE>labeling(<VAR>+Options</VAR>, <VAR>+Variables</VAR>)</CODE>
<DD>
<A NAME="IDX1299"></A>
where <VAR>Variables</VAR> is a list of domain variables or integers and
<VAR>Options</VAR> is a list of search options.  True if an assignment of
the variables can be found which satisfies the posted constraints.
</DL>

<P>
The search options control the order in which variables are selected for
assignment (variable choice heuristic), the way in which choices are
made for the selected variable (value choice heuristic), and whether
all solutions or a single, optimal solution should be found.  The search
options are divided into four groups.  One option may be selected per
group.  Finally, the number of assumptions (choices) made during the search
can be collected.

</P>
<DL COMPACT>

The following options control the order in which the next variable is
selected for assignment.  None of these methods will select a variable
with an unbounded domain.  Thus <CODE>labeling/2</CODE> may succeed with
such variables still unassigned.

<DL COMPACT>

<DT><CODE>leftmost</CODE>
<DD>
The leftmost variable is selected.  This is the default.

<DT><CODE>min</CODE>
<DD>
The leftmost variable with the smallest lower bound is selected.

<DT><CODE>max</CODE>
<DD>
The leftmost variable with the greatest upper bound is selected.

<DT><CODE>ff</CODE>
<DD>
The first-fail principle is used: the leftmost variable with the
smallest domain is selected.

<DT><CODE>ffc</CODE>
<DD>
The most constrained heuristic is used: a variable with the
smallest domain is selected, breaking ties by (a) selecting the variable
that has the most constraints suspended on it and (b) selecting the
leftmost one.
</DL>

The following options control the way in which choices are made for
the selected variable <VAR>X</VAR>:

<DL COMPACT>

<DT><CODE>step</CODE>
<DD>
Makes a binary choice between <CODE><VAR>X</VAR> #= <VAR>B</VAR></CODE>
and <CODE><VAR>X</VAR> #\= <VAR>B</VAR></CODE>, where <VAR>B</VAR> is the lower or upper
bound of <VAR>X</VAR>.  This is the default.

<DT><CODE>enum</CODE>
<DD>
Makes a multiple choice for <VAR>X</VAR> corresponding to the values in its
domain.

<DT><CODE>bisect</CODE>
<DD>
Makes a binary choice between <CODE><VAR>X</VAR> #=&#60; <VAR>M</VAR></CODE>
and <CODE><VAR>X</VAR> #&#62; <VAR>M</VAR></CODE>, where <VAR>M</VAR> is the midpoint
of the domain of <VAR>X</VAR>.  This strategy is also known as domain
splitting.
</DL>

The following options control the order in which the choices
are made for the selected variable <VAR>X</VAR>:

<DL COMPACT>

<DT><CODE>up</CODE>
<DD>
The domain is explored in ascending order.
This is the default.

<DT><CODE>down</CODE>
<DD>
The domain is explored in descending order.
</DL>

The following options control whether all solutions should be
enumerated by backtracking or whether a single solution that minimizes
(maximizes) <VAR>X</VAR> is returned, if one exists.

<DL COMPACT>

<DT><CODE>all</CODE>
<DD>
All solutions are enumerated.  This is the default.

<DT><CODE>minimize(<VAR>X</VAR>)</CODE>
<DD>
<DT><CODE>maximize(<VAR>X</VAR>)</CODE>
<DD>
Uses a branch-and-bound algorithm to find an assignment that minimizes
(maximizes) the domain variable <VAR>X</VAR>.  The
labeling should constrain <VAR>X</VAR> to become assigned for all assignments
of <VAR>Variables</VAR>.
</DL>

Finally, the following option counts the number of assumptions
(choices) made during the search:

<DL COMPACT>

<DT><CODE>statistics(<VAR>K</VAR>)</CODE>
<DD>
When a solution is found, <VAR>K</VAR> is unified with the number of choices made.
</DL>
</DL>

<P>
For example, to enumerate solutions using a static variable ordering,
use:

</P>

<PRE>
| ?- constraints(Variables),
     labeling([], Variables).
     %same as [leftmost,step,up,all]
</PRE>

<P>
To minimize a cost function using branch-and-bound search,
a dynamic variable ordering using the first-fail principle,
and domain splitting exploring the upper part of domains first, use:

</P>

<PRE>
| ?- constraints(Variables, Cost),
     labeling([ff,bisect,down,minimize(Cost)], Variables).
</PRE>

<P>
As opposed to the predicates above which search for consistent assignments
to domain variables, the following predicate searches for a consistent
ordering among tasks competing for an exclusive resource, without
necessarily fixing their start times:

</P>
<DL COMPACT>

<DT><CODE>order_resource(<VAR>+Options</VAR>, <VAR>+Resource</VAR>)</CODE>
<DD>
<A NAME="IDX1300"></A>
where <VAR>Options</VAR> is a list of search options and <VAR>Resource</VAR>
represents a resource as returned by <CODE>serialized_resource/3</CODE>
(see section <A HREF="sicstus_33.html#SEC260">Combinatorial Constraints</A>) on which tasks must be serialized.
True if a total ordering can be imposed on the tasks, enumerating all
such orderings via backtracking.

The search options control the construction of the total ordering.
It may contain at most one of the following atoms, selecting a strategy:

<DL COMPACT>

<DT><CODE>first</CODE>
<DD>
The ordering is built by repetitively selecting some task to be placed
before all others.
<DT><CODE>last</CODE>
<DD>
The ordering is built by repetitively selecting some task to be placed
after all others.
</DL>

and at most one of the following atoms, controlling which task to select
at each step.  If <CODE>first</CODE> is chosen (the default), the task with
the smallest value is selected, otherwise the task with the greatest
value is selected.

<DL COMPACT>

<DT><CODE>est</CODE>
<DD>
The tasks are ordered by earliest start time.
<DT><CODE>lst</CODE>
<DD>
The tasks are ordered by latest start time.
<DT><CODE>ect</CODE>
<DD>
The tasks are ordered by earliest completion time.
<DT><CODE>lct</CODE>
<DD>
The tasks are ordered by latest completion time.
</DL>

<CODE>[first,est]</CODE> (the default) and <CODE>[last,lct]</CODE> can be good heuristics.
</DL>



<H2><A NAME="SEC263" HREF="sicstus_toc.html#TOC263">Statistics Predicates</A></H2>

<P>
The following predicates can be used to get execution statistics.

</P>
<DL COMPACT>

<DT><CODE>fd_statistics(<VAR>?Key</VAR>, <VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX1301"></A>
This allows a program to access execution statistics specific to this
solver.  General statistics about CPU time and memory consumption etc.
is available from the built-in predicate <CODE>statistics/2</CODE>.

For each of the possible keys <VAR>Key</VAR>, <VAR>Value</VAR> is unified with the
current value of a counter which is simultaneously zeroed.  The
following counters are maintained.  See section <A HREF="sicstus_33.html#SEC265">The Constraint System</A>, for
details of what they all mean:

<DL COMPACT>

<DT><CODE>resumptions</CODE>
<DD>
The number of times a constraint was resumed.
<DT><CODE>entailments</CODE>
<DD>
The number of times a (dis)entailment was detected by a constraint.
<DT><CODE>prunings</CODE>
<DD>
The number of times a domain was pruned.
<DT><CODE>backtracks</CODE>
<DD>
The number of times a contradiction was found by a domain being wiped out,
or by a global constraint signalling failure.
Other causes of backtracking, such as failed Prolog tests, are not
covered by this counter.
<DT><CODE>constraints</CODE>
<DD>
The number of constraints created.
</DL>

<DT><CODE>fd_statistics</CODE>
<DD>
<A NAME="IDX1302"></A>
Displays on the <CODE>user_error</CODE> stream a summary of the above statistics.
All counters are zeroed.
</DL>



<H2><A NAME="SEC264" HREF="sicstus_toc.html#TOC264">Answer Constraints</A></H2>

<P>
By default, the answer constraint only shows the projection of the
store onto the variables that occur in the query, but not any
constraints that may be attached to these variables, nor any domains
or constraints attached to other variables.  This is a conscious
decision, as no efficient algorithm for projecting answer constraints
onto the query variables is known for this constraint system.

</P>
<P>
It is possible, however, to get a complete answer constraint including
all variables that took part in the computation and their domains
and attached constraints.  This is done by asserting a clause for
the following predicate:

</P>
<DL COMPACT>

<DT><CODE>clpfd:full_answer</CODE>
<DD>
<A NAME="IDX1303"></A>
A dynamic hook predicate.  If false (the default), the answer constraint
only contains the domains of the query variables.  If true, the
answer constraints contains the domains and any attached constraints of all
variables.
</DL>



<H2><A NAME="SEC265" HREF="sicstus_toc.html#TOC265">The Constraint System</A></H2>


<UL>
<LI><A HREF="sicstus_33.html#SEC266">Definitions</A>
<LI><A HREF="sicstus_33.html#SEC267">Pitfalls of Interval Reasoning</A>
</UL>



<H3><A NAME="SEC266" HREF="sicstus_toc.html#TOC266">Definitions</A></H3>

<P>
The constraint system is based on domain constraints and indexicals.  A
<EM>domain constraint</EM> is an expression <CODE><VAR>X</VAR>::<VAR>I</VAR></CODE>, where
<VAR>X</VAR> is a domain variable and
<VAR>I</VAR> is a nonempty set of integers.

</P>
<P>
<A NAME="IDX1304"></A>
<A NAME="IDX1305"></A>
<A NAME="IDX1306"></A>
<A NAME="IDX1307"></A>
<A NAME="IDX1308"></A>
<A NAME="IDX1309"></A>
A set <VAR>S</VAR> of domain constraints is called a <EM>store</EM>. <VAR>D(X,S)</VAR>,
the <EM>domain</EM> of <VAR>X</VAR> in <VAR>S</VAR>, is defined as the intersection
of all <VAR>I</VAR> such that <CODE><VAR>X</VAR>::<VAR>I</VAR></CODE> belongs to <VAR>S</VAR>.
The store is <EM>contradictory</EM> if the domain of some variable is empty;
otherwise, it is <EM>consistent</EM>.  A consistent store <VAR>S'</VAR> is an
<EM>extension</EM> of a store <VAR>S</VAR> iff, for all variables <VAR>X</VAR>,
<VAR>D(X,S')</VAR> is contained in <VAR>D(X,S)</VAR>.

</P>
<P>
The following definitions, adapted from [Van Hentenryck et al. 95],
define important notions of consistency and entailment of constraints
wrt. stores.

</P>
<P>
A ground constraint is <EM>true</EM> if it holds and <EM>false</EM> otherwise.

</P>
<P>
<A NAME="IDX1310"></A>
A constraint <VAR>C</VAR> is <EM>domain-consistent wrt. <VAR>S</VAR></EM> iff, for
each variable <VAR>Xi</VAR> and value <VAR>Vi</VAR> in <VAR>D(Xi,S)</VAR>, there
exist values <VAR>Vj</VAR> in <VAR>D(Xj,S)</VAR>, <VAR>1 =&#60; j =&#60; n, i\=j</VAR>, such
that <VAR>C(V1,...,Vn)</VAR> is true.

</P>
<P>
<A NAME="IDX1311"></A>
<A NAME="IDX1312"></A>
A constraint <VAR>C</VAR> is <EM>domain-entailed by <VAR>S</VAR></EM> iff, for
all values <VAR>Vj</VAR> in <VAR>D(Xj,S)</VAR>, <VAR>1 =&#60; j =&#60; n</VAR>,
<VAR>C(V1,...,Vn)</VAR> is true.

</P>
<P>
Let <VAR>D'(X,S)</VAR> denote the interval <VAR>min(D(X,S))..max(D(X,S))</VAR>.

</P>
<P>
<A NAME="IDX1313"></A>
A constraint <VAR>C</VAR> is <EM>interval-consistent wrt. <VAR>S</VAR></EM> iff,
for each variable <VAR>Xi</VAR> there exist values <VAR>Vj</VAR> in
<VAR>D'(Xj,S)</VAR>, <VAR>1 =&#60; j =&#60; n, i\=j</VAR>, such that
<VAR>C(V1,...,min(D(Xi,S)),...,Vn)</VAR> and
<VAR>C(V1,...,max(D(Xi,S)),...,Vn)</VAR> are both true.

</P>
<P>
<A NAME="IDX1314"></A>
<A NAME="IDX1315"></A>
A constraint <VAR>C</VAR> is <EM>interval-entailed by <VAR>S</VAR></EM> iff, for
all values <VAR>Vj</VAR> in <VAR>D'(Xj,S)</VAR>, <VAR>1 =&#60; j =&#60; n</VAR>,
<VAR>C(V1,...,Vn)</VAR> is true.

</P>
<P>
Finally, a constraint is <EM>domain-disentailed (interval-disentailed) by <VAR>S</VAR></EM>
iff its negation is domain-entailed (interval-entailed) by <VAR>S</VAR>.

</P>



<H3><A NAME="SEC267" HREF="sicstus_toc.html#TOC267">Pitfalls of Interval Reasoning</A></H3>

<P>
In most circumstances, arithmetic constraints only maintain
interval-consistency and only detect interval-entailment and
-disentailment.  Note that there are cases where an interval-consistency
maintaining constraint may detect a contradiction when the constraint
is not yet interval-disentailed, as the following example illustrates.
Note that <CODE><VAR>X</VAR> #\= <VAR>Y</VAR></CODE> maintains domain consistency if
both arguments are constants or variables:

</P>

<PRE>
| ?- X+Y #= Z, X=1, Z=6, Y in 1..10, Y #\= 5.

no
| ?- X+Y #= Z #&#60;=&#62; B, X=1, Z=6, Y in 1..10, Y #\= 5.

X = 1,
Z = 6,
Y in(1..4)\/(6..10),
B in 0..1
</PRE>

<P>
Since <CODE>1+5#=6</CODE> holds, <CODE>X+Y #= Z</CODE> is not interval-disentailed,
although any attempt to make it interval-consistent wrt. the store
results in a contradictory store.

</P>



<H2><A NAME="SEC268" HREF="sicstus_toc.html#TOC268">Defining Global Constraints</A></H2>


<UL>
<LI><A HREF="sicstus_33.html#SEC269">The Global Constraint Programming Interface</A>
<LI><A HREF="sicstus_33.html#SEC270">Reflection Predicates</A>
<LI><A HREF="sicstus_33.html#SEC271">FD Set Operations</A>
<LI><A HREF="sicstus_33.html#SEC272">A Global Constraint Example</A>
</UL>



<H3><A NAME="SEC269" HREF="sicstus_toc.html#TOC269">The Global Constraint Programming Interface</A></H3>

<P>
<A NAME="IDX1316"></A>
<A NAME="IDX1317"></A>
This section describes a programming interface by means of which new
constraints can be written.  The interface consists of a set of
predicates provided by this library module.  Constraints defined in this
way can take arbitrary arguments and may use any constraint solving
algorithm, provided it makes sense.  Reification cannot be expressed
in this interface; instead, reification may be achieved by explicitly
passing a 0/1-variable to the constraint in question.

</P>
<P>
Global constraints have state which may be updated each time the
constraint is resumed.  The state information may be used e.g. in
incremental constraint solving.

</P>
<P>
The following two predicates are the principal entrypoints for defining
and posting new global constraints:

</P>
<DL COMPACT>

<DT><CODE>clpfd:dispatch_global(<VAR>+Constraint</VAR>, <VAR>+State0</VAR>, <VAR>-State</VAR>, <VAR>-Actions</VAR>)</CODE>
<DD>
<A NAME="IDX1318"></A>
<EM>A multifile hook predicate,</EM> telling the solver how to solve constraints of
the form <VAR>Constraint</VAR>.

When defining a new constraint, a clause of this predicate must be
added. Its body defines a constraint solving method and should always
succeed deterministically.  When a global constraint is called or
resumed, the solver will call this predicate to deal with the
constraint.

<VAR>State0</VAR> and <VAR>State</VAR> are the old and new state respectively.

The constraint solving method must not invoke the constraint solver
recursively e.g. by binding variables or posting new constraints;
instead, <VAR>Actions</VAR> should be unified with a list of requests to the
solver.  Each request should be of the following form:

<DL COMPACT>

<DT><CODE>exit</CODE>
<DD>
The constraint has become entailed, and ceases to exist.
<DT><CODE>fail</CODE>
<DD>
The constraint has become disentailed, causing the solver to backtrack.
<DT><CODE><VAR>X</VAR> = <VAR>V</VAR></CODE>
<DD>
The solver binds <VAR>X</VAR> to <VAR>V</VAR>.
<DT><CODE><VAR>X</VAR> in <VAR>R</VAR></CODE>
<DD>
The solver constrains <VAR>X</VAR> to be a member of the 
<VAR>ConstantRange</VAR> <VAR>R</VAR> (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>).
<DT><CODE><VAR>X</VAR> in_set <VAR>S</VAR></CODE>
<DD>
The solver constrains <VAR>X</VAR> to be a member of the 
FD set <VAR>S</VAR> (see section <A HREF="sicstus_33.html#SEC271">FD Set Operations</A>).
<DT><CODE>call(<VAR>Goal</VAR>)</CODE>
<DD>
The solver calls the goal or constraint <VAR>Goal</VAR>, which should be module
prefixed unless it is a built-in predicate or an exported predicate
of the <CODE>clpfd</CODE> module.
</DL>

<DT><CODE>fd_global(<VAR>+Constraint</VAR>, <VAR>+State</VAR>, <VAR>+Susp</VAR>)</CODE>
<DD>
<A NAME="IDX1319"></A>
where <VAR>Constraint</VAR> is a constraint goal, <VAR>State</VAR> is its initial
state, and <VAR>Susp</VAR> is a term encoding how the constraint should wake
up in response to domain changes.  This posts the constraint.

<VAR>Susp</VAR> is a list of <VAR>F(Var)</VAR> terms where <VAR>Var</VAR> is a variable
to suspend on and <VAR>F</VAR> is a functor encoding when to wake up:

<DL COMPACT>

<DT><CODE>dom(<VAR>X</VAR>)</CODE>
<DD>
wake up when the domain of <VAR>X</VAR> has changed
<DT><CODE>min(<VAR>X</VAR>)</CODE>
<DD>
wake up when the lower bound of <VAR>X</VAR> has changed
<DT><CODE>max(<VAR>X</VAR>)</CODE>
<DD>
wake up when the upper bound of <VAR>X</VAR> has changed
<DT><CODE>minmax(<VAR>X</VAR>)</CODE>
<DD>
wake up when the lower or upper of <VAR>X</VAR> has changed
<DT><CODE>val(<VAR>X</VAR>)</CODE>
<DD>
wake up when <VAR>X</VAR> has become ground
</DL>
</DL>

<P>
For an example of usage, see section <A HREF="sicstus_33.html#SEC272">A Global Constraint Example</A>.

</P>


<H3><A NAME="SEC270" HREF="sicstus_toc.html#TOC270">Reflection Predicates</A></H3>

<P>
The constraint solving method needs access to information about the
current domains of variables.  This is provided by the following
predicates, which are all constant time operations.

</P>
<DL COMPACT>

<DT><CODE>fd_min(<VAR>?X</VAR>, <VAR>?Min</VAR>)</CODE>
<DD>
<A NAME="IDX1320"></A>
where <VAR>X</VAR> is a domain variable (or an integer).  <VAR>Min</VAR> is
unified with the smallest value in the current domain of <VAR>X</VAR>,
i.e. an integer or the atom <CODE>inf</CODE> denoting minus infinity.

<DT><CODE>fd_max(<VAR>?X</VAR>, <VAR>?Max</VAR>)</CODE>
<DD>
<A NAME="IDX1321"></A>
where <VAR>X</VAR> is a domain variable (or an integer).  <VAR>Max</VAR> is
unified with the upper bound of the current domain of <VAR>X</VAR>,
i.e. an integer or the atom <CODE>sup</CODE> denoting infinity.

<DT><CODE>fd_size(<VAR>?X</VAR>, <VAR>?Size</VAR>)</CODE>
<DD>
<A NAME="IDX1322"></A>
where <VAR>X</VAR> is a domain variable (or an integer).  <VAR>Size</VAR> is
unified with the size of the current domain of <VAR>X</VAR>, if the domain is
finite, or the atom <CODE>sup</CODE> otherwise.

<DT><CODE>fd_set(<VAR>?X</VAR>, <VAR>?Set</VAR>)</CODE>
<DD>
<A NAME="IDX1323"></A>
where <VAR>X</VAR> is a domain variable (or an integer).  <VAR>Set</VAR> is
unified with an FD set term denoting the internal representation of
the current domain of <VAR>X</VAR>; see below.  

<DT><CODE>fd_dom(<VAR>?X</VAR>, <VAR>?Range</VAR>)</CODE>
<DD>
<A NAME="IDX1324"></A>
where <VAR>X</VAR> is a domain variable (or an integer).  <VAR>Range</VAR> is
unified with a <VAR>ConstantRange</VAR> (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>)
denoting the the current domain of <VAR>X</VAR>.
</DL>

<P>
The following predicate can be used for computing the set of variables
that are transitively connected via constraints to some given variables.

</P>
<DL COMPACT>

<DT><CODE>fd_closure(<VAR>+Vars</VAR>, <VAR>-Closure</VAR>)</CODE>
<DD>
<A NAME="IDX1325"></A>
Given a list <VAR>Vars</VAR> of domain variables, <VAR>Closure</VAR> is the set of
variables (including <VAR>Vars</VAR>) that can be transitively reached via
constraints posted so far.
</DL>



<H3><A NAME="SEC271" HREF="sicstus_toc.html#TOC271">FD Set Operations</A></H3>
<P>
<A NAME="IDX1326"></A>

</P>
<P>
The domains of variables are internally represented compactly as
<EM>FD set</EM> terms.  The details of this representation are subject to
change and should not be relied on.  Therefore, 
a number of operations on FD sets
are provided, as such terms play an important role in the interface.
The following operations are the primitive ones:

</P>
<DL COMPACT>

<DT><CODE>is_fdset(<VAR>+Set</VAR>)</CODE>
<DD>
<A NAME="IDX1327"></A>
<VAR>Set</VAR> is a valid FD set.

<DT><CODE>empty_fdset(<VAR>?Set</VAR>)</CODE>
<DD>
<A NAME="IDX1328"></A>
<VAR>Set</VAR> is the empty FD set.

<DT><CODE>fdset_parts(<VAR>?Set</VAR>, <VAR>?Min</VAR>, <VAR>?Max</VAR>, <VAR>?Rest</VAR>)</CODE>
<DD>
<A NAME="IDX1329"></A>
<VAR>Set</VAR> is an FD set which is a union of the non-empty interval
<VAR>Min</VAR>..<VAR>Max</VAR> and the FD set <VAR>Rest</VAR>, and all elements of
<VAR>Rest</VAR> are greater than <VAR>Max</VAR>+1.
<VAR>Min</VAR> and <VAR>Max</VAR> are both integers or the atoms <CODE>inf</CODE>
and <CODE>sup</CODE>, denoting minus and plus infinity, respectively.
Either <VAR>Set</VAR> or all the other arguments must be ground.
</DL>

<P>
The following operations can all be defined in terms of the primitive
ones, but in most cases, a more efficient implementation is used:

</P>
<DL COMPACT>

<DT><CODE>empty_interval(<VAR>+Min</VAR>, <VAR>+Max</VAR>)</CODE>
<DD>
<A NAME="IDX1330"></A>
<VAR>Min</VAR>..<VAR>Max</VAR> is an empty interval.

<DT><CODE>fdset_interval(<VAR>?Set</VAR>, <VAR>?Min</VAR>, <VAR>?Max</VAR>)</CODE>
<DD>
<A NAME="IDX1331"></A>
<VAR>Set</VAR> is an fdset which is the non-empty interval <VAR>Min</VAR>..<VAR>Max</VAR>.

<DT><CODE>fdset_singleton(<VAR>?Set</VAR>, <VAR>?Elt</VAR>)</CODE>
<DD>
<A NAME="IDX1332"></A>
<VAR>Set</VAR> is an FD set containing <VAR>Elt</VAR> only.
At least one of the arguments must be ground.

<DT><CODE>fdset_min(<VAR>+Set</VAR>, <VAR>-Min</VAR>)</CODE>
<DD>
<A NAME="IDX1333"></A>
<VAR>Min</VAR> is the lower bound of <VAR>Set</VAR>.

<DT><CODE>fdset_max(<VAR>+Set</VAR>, <VAR>-Min</VAR>)</CODE>
<DD>
<A NAME="IDX1334"></A>
<VAR>Max</VAR> is the upper bound of <VAR>Set</VAR>.
This operation is linear in the number of intervals of <VAR>Set</VAR>.

<DT><CODE>fdset_size(<VAR>+Set</VAR>, <VAR>-Size</VAR>)</CODE>
<DD>
<A NAME="IDX1335"></A>
<VAR>Size</VAR> is the cardinality of <VAR>Set</VAR>, represented as <CODE>sup</CODE>
if <VAR>Set</VAR> is infinite.
This operation is linear in the number of intervals of <VAR>Set</VAR>.

<DT><CODE>list_to_fdset(<VAR>+List</VAR>, <VAR>-Set</VAR>)</CODE>
<DD>
<A NAME="IDX1336"></A>
<VAR>Set</VAR> is the FD set containing the elements of <VAR>List</VAR>.
Slightly more efficient if <VAR>List</VAR> is ordered.

<DT><CODE>fdset_to_list(<VAR>+Set</VAR>, <VAR>-List</VAR>)</CODE>
<DD>
<A NAME="IDX1337"></A>
<VAR>List</VAR> is an ordered list of the elements of <VAR>Set</VAR>,
which must be finite.

<DT><CODE>range_to_fdset(<VAR>+Range</VAR>, <VAR>-Set</VAR>)</CODE>
<DD>
<A NAME="IDX1338"></A>
<VAR>Set</VAR> is the FD set containing the elements of the
<VAR>ConstantRange</VAR> (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>) <VAR>Range</VAR>.

<DT><CODE>fdset_to_range(<VAR>+Set</VAR>, <VAR>-Range</VAR>)</CODE>
<DD>
<A NAME="IDX1339"></A>
<VAR>Range</VAR> is a constant interval, a singleton constant set, or a
union of such, denoting the same set as <VAR>Set</VAR>.

<DT><CODE>fdset_add_element(<VAR>+Set1</VAR>, <VAR>+Elt</VAR> <VAR>-Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1340"></A>
<VAR>Set2</VAR> is <VAR>Set1</VAR> with <VAR>Elt</VAR> inserted in it.

<DT><CODE>fdset_del_element(<VAR>+Set1</VAR>, <VAR>+Elt</VAR>, <VAR>-Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1341"></A>
<VAR>Set2</VAR> is like <VAR>Set1</VAR> but with <VAR>Elt</VAR> removed.

<DT><CODE>fdset_disjoint(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1342"></A>
The two FD sets have no elements in common.  

<DT><CODE>fdset_intersect(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1343"></A>
The two FD sets have at least one element in common.

<DT><CODE>fdset_intersection(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>-Intersection</VAR>)</CODE>
<DD>
<A NAME="IDX1344"></A>
<VAR>Intersection</VAR> is the intersection between <VAR>Set1</VAR> and <VAR>Set2</VAR>.

<DT><CODE>fdset_intersection(<VAR>+Sets</VAR>, <VAR>-Intersection</VAR>)</CODE>
<DD>
<A NAME="IDX1345"></A>
<VAR>Intersection</VAR> is the intersection
of all the sets in <VAR>Sets</VAR>.

<DT><CODE>fdset_member(<VAR>?Elt</VAR>, <VAR>+Set</VAR>)</CODE>
<DD>
<A NAME="IDX1346"></A>
is true when <VAR>Elt</VAR> is a member of <VAR>Set</VAR>.
If <VAR>Elt</VAR> is unbound, <VAR>Set</VAR> must be finite.

<DT><CODE>fdset_eq(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1347"></A>
Is true when the two arguments represent the same set i.e. they are
identical.

<DT><CODE>fdset_subset(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>)</CODE>
<DD>
<A NAME="IDX1348"></A>
Every element of <VAR>Set1</VAR> appears in <VAR>Set2</VAR>.

<DT><CODE>fdset_subtract(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>-Difference</VAR>)</CODE>
<DD>
<A NAME="IDX1349"></A>
<VAR>Difference</VAR> contains all and only the elements of <VAR>Set1</VAR> which
are not also in <VAR>Set2</VAR>. 

<DT><CODE>fdset_union(<VAR>+Set1</VAR>, <VAR>+Set2</VAR>, <VAR>-Union</VAR>)</CODE>
<DD>
<A NAME="IDX1350"></A>
<VAR>Union</VAR> is the union of <VAR>Set1</VAR> and <VAR>Set2</VAR>.  

<DT><CODE>fdset_union(<VAR>+Sets</VAR>, <VAR>-Union</VAR>)</CODE>
<DD>
<A NAME="IDX1351"></A>
<VAR>Union</VAR> is the union of all the sets in <VAR>Sets</VAR>.  

<DT><CODE>fdset_complement(<VAR>+Set</VAR>, <VAR>-Complement</VAR>)</CODE>
<DD>
<A NAME="IDX1352"></A>
<VAR>Complement</VAR> is the complement of <VAR>Set</VAR> wrt. <CODE>inf..sup</CODE>.
</DL>



<H3><A NAME="SEC272" HREF="sicstus_toc.html#TOC272">A Global Constraint Example</A></H3>

<P>
The following example defines a new global constraint
<CODE>exactly(<VAR>X</VAR>,<VAR>L</VAR>,<VAR>N</VAR>)</CODE> which is true if <VAR>X</VAR> occurs
exactly <VAR>N</VAR> times in the list <VAR>L</VAR> of integers and domain
variables.  A version defined in terms of reified equalities was
presented earlier; see section <A HREF="sicstus_33.html#SEC255">Reified Constraints</A>.

</P>
<P>
This example illustrates the use of state information.  The state has
two components: the list of variables that could still be <VAR>X</VAR>, and
the number of variables still required to be <VAR>X</VAR>.

</P>
<P>
The constraint is defined to wake up on any domain change.

</P>

<PRE>
/*
An implementation of exactly(I, X[1]...X[m], N):

Necessary condition: 0 =&#60; N =&#60; m.
Rewrite rules:

[1] |= X[i]=I  ==&#62; exactly(I, X[1]...X[i-1],X[i+1]...X[m], N-1):
[2] |= X[i]\=I ==&#62; exactly(I, X[1]...X[i-1],X[i+1]...X[m], N):
[3] |= N=0     ==&#62; X[1]\=I ... X[m]\=I
[4] |= N=m     ==&#62; X[1]=I  ... X[m]=I
*/
:- use_module(library(clpfd)).

% the entrypoint
exactly(I, Xs, N) :-
        dom_suspensions(Xs, Susp),
        fd_global(exactly(I,Xs,N), state(Xs,N), Susp).

dom_suspensions([], []).
dom_suspensions([X|Xs], [dom(X)|Susp]) :- 
        dom_suspensions(Xs, Susp).

% the solver method
:- multifile clpfd:dispatch_global/4.
clpfd:dispatch_global(exactly(I,_,_), state(Xs0,N0), state(Xs,N), Actions) :-
        exactly_solver(I, Xs0, Xs, N0, N, Actions).

exactly_solver(I, Xs0, Xs, N0, N, Actions) :-
        ex_filter(Xs0, Xs, N0, N, I),
        length(Xs, M),
        (   N=:=0 -&#62; Actions = [exit|Ps], ex_neq(Xs, I, Ps)
        ;   N=:=M -&#62; Actions = [exit|Ps], ex_eq(Xs, I, Ps)
        ;   N&#62;0, N&#60;M -&#62; Actions = []
        ;   Actions = [fail]
        ).

% rules [1,2]: filter the X's, decrementing N
ex_filter([], [], N, N, _).
ex_filter([X|Xs], Ys, L, N, I) :- X==I, !,
        M is L-1,
        ex_filter(Xs, Ys, M, N, I).
ex_filter([X|Xs], Ys0, L, N, I) :-
        fd_set(X, Set),
        fdset_member(I, Set), !,
        Ys0 = [X|Ys],
        ex_filter(Xs, Ys, L, N, I).
ex_filter([_|Xs], Ys, L, N, I) :-
        ex_filter(Xs, Ys, L, N, I).

% rule [3]: all must be neq I
ex_neq(Xs, I, Ps) :-
        fdset_singleton(Set0, I),
        fdset_complement(Set0, Set),
        eq_all(Xs, Set, Ps).

% rule [4]: all must be eq I
ex_eq(Xs, I, Ps) :-
        fdset_singleton(Set, I),
        eq_all(Xs, Set, Ps).

eq_all([], _, []).
eq_all([X|Xs], Set, [X in_set Set|Ps]) :- 
        eq_all(Xs, Set, Ps).

end_of_file.

% sample queries:
| ?- exactly(5,[A,B,C],1), A=5.

A = 5,
B in(inf..4)\/(6..sup),
C in(inf..4)\/(6..sup)

| ?- exactly(5,[A,B,C],1), A in 1..2, B in 3..4.

C = 5,
A in 1..2,
B in 3..4
</PRE>



<H2><A NAME="SEC273" HREF="sicstus_toc.html#TOC273">Defining Primitive Constraints</A></H2>

<P>
Indexicals are the principal means of defining constraints, but it is
usually not necessary to resort to this level of programming--most
commonly used constraints are available in a library and/or via
macro-expansion.
The key feature about indexicals is that they give the
programmer precise control over aspects of the operational semantics of
the constraints. Trade-offs can be made between the computational cost
of the constraints and their pruning power.  The indexical language
provides many degrees of freedom for the user to select the level of
consistency to be maintained depending on application-specific needs.

</P>


<UL>
<LI><A HREF="sicstus_33.html#SEC274">Indexicals</A>
<LI><A HREF="sicstus_33.html#SEC275">Range Expressions</A>
<LI><A HREF="sicstus_33.html#SEC276">Term Expressions</A>
<LI><A HREF="sicstus_33.html#SEC277">Monotonicity of Indexicals</A>
<LI><A HREF="sicstus_33.html#SEC278">FD predicates</A>
<LI><A HREF="sicstus_33.html#SEC279">Execution of Propagating Indexicals</A>
<LI><A HREF="sicstus_33.html#SEC280">Execution of Checking Indexicals</A>
<LI><A HREF="sicstus_33.html#SEC281">Goal Expanded Constraints</A>
</UL>



<H3><A NAME="SEC274" HREF="sicstus_toc.html#TOC274">Indexicals</A></H3>

<P>
<A NAME="IDX1353"></A>
An <EM>indexical</EM> is a reactive functional rule of the form
<CODE><VAR>X</VAR> in <VAR>R</VAR></CODE>, where <VAR>R</VAR> is a set valued <EM>range
expression</EM> (see below).  See section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>, for a grammar defining
indexicals and range expressions.

</P>
<P>
Indexicals can play one of two roles: <EM>propagating indexicals</EM> are
used for constraint solving, and <EM>checking indexicals</EM> are used for
entailment checking.  When a propagating indexical fires, <VAR>R</VAR> is
evaluated in the current store <VAR>S</VAR>, which is extended by adding the
new domain constraint <CODE><VAR>X</VAR>::<VAR>S(R)</VAR></CODE> to the store, where
<VAR>S(R)</VAR> denotes the value of <VAR>R</VAR> in <VAR>S</VAR>.  When a checking
indexical fires, it checks if <VAR>D(X,S)</VAR> is contained in <VAR>S(R)</VAR>,
and if so, the constraint corresponding to the indexical is detected as
entailed.

</P>


<H3><A NAME="SEC275" HREF="sicstus_toc.html#TOC275">Range Expressions</A></H3>

<P>
A range expression has one of the following forms, where <VAR>Ri</VAR>
denote range expressions, <VAR>Ti</VAR> denote integer valued <EM>term
expressions</EM>, <VAR>S(Ti)</VAR> denotes the integer value of <VAR>Ti</VAR> in
<VAR>S</VAR>, <VAR>X</VAR> denotes a variable, <VAR>I</VAR> denotes an integer, and
<VAR>S</VAR> denotes the current store.

</P>
<DL COMPACT>

<DT><CODE>dom(<VAR>X</VAR>)</CODE>
<DD>
evaluates to <VAR>D(X,S)</VAR>

<DT><CODE>{<VAR>T1</VAR>,...,<VAR>Tn</VAR>}</CODE>
<DD>
evaluates to <CODE>{S(<VAR>T1</VAR>),...,S(<VAR>Tn</VAR>)}</CODE>.
Any term expression containing a
subexpression which is a variable that is not "quantified" by
<CODE>unionof/3</CODE> will only be evaluated when this variable has been assigned.

<DT><CODE><VAR>T1</VAR>..<VAR>T2</VAR></CODE>
<DD>
evaluates to the interval between <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>.

<DT><CODE><VAR>R1</VAR>/\<VAR>R2</VAR></CODE>
<DD>
evaluates to the intersection of <VAR>S(R1)</VAR> and <VAR>S(R2)</VAR>

<DT><CODE><VAR>R1</VAR>\/<VAR>R2</VAR></CODE>
<DD>
evaluates to the union of <VAR>S(R1)</VAR> and <VAR>S(R2)</VAR>

<DT><CODE>\<VAR>R2</VAR></CODE>
<DD>
evaluates to the complement of <VAR>S(R2)</VAR>

<DT><CODE><VAR>R1</VAR>+<VAR>R2</VAR></CODE>
<DD>
<DT><CODE><VAR>R1</VAR>+<VAR>T2</VAR></CODE>
<DD>
evaluates to <VAR>S(R2)</VAR> or <VAR>S(T2)</VAR> added pointwise to <VAR>S(R1)</VAR>

<DT><CODE>-<VAR>R2</VAR></CODE>
<DD>
evaluates to <VAR>S(R2)</VAR> negated pointwise

<DT><CODE><VAR>R1</VAR>-<VAR>R2</VAR></CODE>
<DD>
<DT><CODE><VAR>R1</VAR>-<VAR>T2</VAR></CODE>
<DD>
<DT><CODE><VAR>T1</VAR>-<VAR>R2</VAR></CODE>
<DD>
evaluates to <VAR>S(R2)</VAR> or <VAR>S(T2)</VAR> subtracted pointwise from
<VAR>S(R1)</VAR> or <VAR>S(T1)</VAR>

<DT><CODE><VAR>R1</VAR> mod <VAR>R2</VAR></CODE>
<DD>
<DT><CODE><VAR>R1</VAR> mod <VAR>T2</VAR></CODE>
<DD>
evaluates to <VAR>S(R1)</VAR> pointwise modulo <VAR>S(R2)</VAR> or <VAR>S(T2)</VAR>

<DT><CODE><VAR>R1</VAR> ? <VAR>R2</VAR></CODE>
<DD>
evaluates to <VAR>S(R2)</VAR> if <VAR>S(R1)</VAR> is a non-empty set; otherwise,
evaluates to the empty set.  This expression is commonly used in the
context <CODE>(<VAR>R1</VAR> ? (inf..sup) \/ <VAR>R3</VAR>)</CODE>, which evaluates to
<VAR>S(R3)</VAR> if <VAR>S(R1)</VAR> is an empty set; otherwise, evaluates to
<CODE>inf..sup</CODE>.  As an optimization, <VAR>R3</VAR> is not evaluated
while the value of <VAR>R1</VAR> is a non-empty set.

<DT><CODE>unionof(<VAR>X</VAR>,<VAR>R1</VAR>,<VAR>R2</VAR>)</CODE>
<DD>
evaluates to the union of <VAR>S(Expr_1)...S(Expr_N)</VAR>, where each
<VAR>Expr_I</VAR> has been formed by substituting <VAR>K</VAR> for <VAR>X</VAR> in
<VAR>R2</VAR>, where <VAR>K</VAR> is the <VAR>I</VAR>:th element of
<VAR>S(R1)</VAR>.  See section <A HREF="sicstus_33.html#SEC284">N Queens</A>, for an example of usage.
<B>N.B.</B> If <VAR>S(R1)</VAR> is infinite, the evaluation of
the indexical will be abandoned, and the indexical will simply suspend.

<DT><CODE>switch(<VAR>T1</VAR>,<VAR>MapList</VAR>)</CODE>
<DD>
evaluates to <VAR>S(Expr)</VAR> if <VAR>S(T1)</VAR> equals <VAR>Key</VAR> and
<VAR>MapList</VAR> contains a pair <CODE><VAR>Key</VAR>-<VAR>Expr</VAR></CODE>.  Otherwise,
evaluates to the empty set.
</DL>

<P>
When used in the body of an FD predicate (see section <A HREF="sicstus_33.html#SEC281">Goal Expanded Constraints</A>),
a <CODE>relation/3</CODE> expression expands to two 
indexicals, each consisting of a <CODE>switch/2</CODE> expression nested
inside a <CODE>unionof/3</CODE> expression.  Thus, the following constraints
are equivalent:

</P>

<PRE>
p(X, Y) +: relation(X, [1-{1},2-{1,2},3-{1,2,3}], Y).

q(X, Y) +: 
        X in unionof(B,dom(Y),switch(B,[1-{1,2,3},2-{2,3},3-{3}])),
        Y in unionof(B,dom(X),switch(B,[1-{1},2-{1,2},3-{1,2,3}])).
</PRE>



<H3><A NAME="SEC276" HREF="sicstus_toc.html#TOC276">Term Expressions</A></H3>

<P>
A term expression has one of the following forms,
where <VAR>T1</VAR> and <VAR>T2</VAR> denote term expressions,
<VAR>X</VAR> denotes a variable,
<VAR>I</VAR> denotes an integer, and
<VAR>S</VAR> denotes the current store.

</P>
<DL COMPACT>

<DT><CODE>min(<VAR>X</VAR>)</CODE>
<DD>
evaluates to the minimum of <VAR>D(X,S)</VAR>

<DT><CODE>max(<VAR>X</VAR>)</CODE>
<DD>
evaluates to the maximum of <VAR>D(X,S)</VAR>

<DT><CODE>card(<VAR>X</VAR>)</CODE>
<DD>
evaluates to the size of <VAR>D(X,S)</VAR>

<DT><CODE><VAR>X</VAR></CODE>
<DD>
evaluates to the integer value of <VAR>X</VAR>.
A subexpression of this form, not "quantified" by <CODE>unionof/3</CODE>,
will cause the evaluation to suspend until the variable is assigned.

<DT><CODE><VAR>I</VAR></CODE>
<DD>
an integer

<DT><CODE>inf</CODE>
<DD>
minus infinity

<DT><CODE>sup</CODE>
<DD>
plus infinity

<DT><CODE>-<VAR>T1</VAR></CODE>
<DD>
evaluates to <VAR>S(T1)</VAR> negated

<DT><CODE><VAR>T1</VAR>+<VAR>T2</VAR></CODE>
<DD>
evaluates to the sum of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>

<DT><CODE><VAR>T1</VAR>-<VAR>T2</VAR></CODE>
<DD>
evaluates to the difference of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>

<DT><CODE><VAR>T1</VAR>*<VAR>T2</VAR></CODE>
<DD>
evaluates to the product of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>,
where <VAR>S(T2)</VAR> must not be negative

<DT><CODE><VAR>T1</VAR>/&#62;<VAR>T2</VAR></CODE>
<DD>
evaluates to the quotient of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>, rounded up,
where <VAR>S(T2)</VAR> must be positive

<DT><CODE><VAR>T1</VAR>/&#60;<VAR>T2</VAR></CODE>
<DD>
evaluates to the quotient of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>, rounded down,
where <VAR>S(T2)</VAR> must be positive

<DT><CODE><VAR>T1</VAR> mod <VAR>T2</VAR></CODE>
<DD>
evaluates to the modulo of <VAR>S(T1)</VAR> and <VAR>S(T2)</VAR>
</DL>



<H3><A NAME="SEC277" HREF="sicstus_toc.html#TOC277">Monotonicity of Indexicals</A></H3>

<P>
A range <VAR>R</VAR> is <EM>monotone in <VAR>S</VAR></EM> iff the value of <VAR>R</VAR> in
<VAR>S'</VAR> is contained in the value of <VAR>R</VAR> in <VAR>S</VAR>, for every
extension <VAR>S'</VAR> of <VAR>S</VAR>.  A range <VAR>R</VAR> is <EM>anti-monotone in
<VAR>S</VAR></EM> iff the value of <VAR>R</VAR> in <VAR>S</VAR> is contained in the value
of <VAR>R</VAR> in <VAR>S'</VAR>, for every extension <VAR>S'</VAR> of <VAR>S</VAR>.  By
abuse of notation, we will say that <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE> is
(anti-)monotone iff <VAR>R</VAR> is (anti-)monotone.

</P>
<P>
The consistency or entailment of a constraint <VAR>C</VAR> expressed as
indexicals <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE> in a store <VAR>S</VAR> is checked by
considering the relationship between <VAR>D(X,S)</VAR> and <VAR>S(R)</VAR>,
together with the (anti-)monotonicity of <VAR>R</VAR> in <VAR>S</VAR>.  The
details are given in section <A HREF="sicstus_33.html#SEC279">Execution of Propagating Indexicals</A> and
section <A HREF="sicstus_33.html#SEC280">Execution of Checking Indexicals</A>.

</P>
<P>
The solver checks (anti-)monotonicity by requiring that certain
variables occurring in the indexical be ground.  This sufficient
condition can sometimes be false for an (anti-)monotone indexical, but
such situations are rare in practice.

</P>


<H3><A NAME="SEC278" HREF="sicstus_toc.html#TOC278">FD predicates</A></H3>
<P>
<A NAME="IDX1354"></A>
<A NAME="IDX1355"></A>
The following example defines the constraint <VAR>X+Y=T</VAR> as an <EM>FD
predicate</EM> in terms of three indexicals.  Each indexical is a rule
responsible for removing values detected as incompatible from one
particular constraint argument.  Indexicals are <EM>not</EM> Prolog goals;
thus, the example does not express a conjunction.  However, an indexical
may make the store contradictory, in which case backtracking is
triggered:

</P>

<PRE>
plus(X,Y,T) +:
        X in min(T) - max(Y) .. max(T) - min(Y),
        Y in min(T) - max(X) .. max(T) - min(X),
        T in min(X) + min(Y) .. max(X) + max(Y).
</PRE>

<P>
The above definition contains a single clause used for constraint
solving.  The first indexical wakes up whenever the bounds of
<VAR>S(T)</VAR> or <VAR>S(Y)</VAR> are updated, and removes from <VAR>D(X,S)</VAR>
any values that are not compatible with the new bounds of <VAR>T</VAR> and
<VAR>Y</VAR>.  Note that in the event of "holes" in the domains of
<VAR>T</VAR> or <VAR>Y</VAR>, <VAR>D(X,S)</VAR> may contain some values that are
incompatible with <VAR>X+Y=T</VAR> but go undetected.  Like most built-in
arithmetic constraints, the above definition maintains
interval-consistency, which is significantly cheaper to maintain than
domain-consistency and suffices in most cases.  The constraint could for
example be used as follows:

</P>

<PRE>
| ?- X in 1..5, Y in 2..8, plus(X,Y,T).

X in 1..5,
Y in 2..8,
T in 3..13 ? 

yes
</PRE>

<P>
Thus, when an FD predicate is called, the <SAMP>`+:'</SAMP> clause is activated.

</P>
<P>
The definition of a user constraint has to specify what domain
constraints should be added to the constraint store when the constraint
is posted.  Therefore the FD predicate contains a set of indexicals,
each representing a domain constraint to be added to the constraint
store.  The actual domain constraint depends on the constraint store
itself.  For example, the third indexical in the above FD predicate
prescribes the domain constraint <SAMP>`T :: 3..13'</SAMP> if the store contains
<SAMP>`X :: 1..5, Y :: 2..8'</SAMP>.  As the domain of some variables gets
narrower, the indexical may enforce a new, stricter constraint on some
other variables.  Therefore such an indexical (called a propagating
indexical) can be viewed as an agent reacting to the changes in the
store by enforcing further changes in the store.

</P>
<P>
In general there are three stages in the lifetime of a propagating
indexical.  When it is posted it may not be evaluated immediately
(e.g. has to wait until some variables are ground before being able to
modify the store).  Until the preconditions for the evaluation are
satisfied, the agent does not enforce any constraints.  When the
indexical becomes evaluable the resulting domain constraint is added to
the store.  The agent then waits and reacts to changes in the domains of
variables occurring in the indexical by re-evaluating it and adding the
new, stricter constraint to the store.  Eventually the computation
reaches a phase when no further refinement of the store can result in a
more precise constraint (the indexical is entailed by the store), and
then the agent can cease to exist.

</P>
<P>
A necessary condition for the FD predicate to be correctly defined is the
following: for any store mapping each variable to a singleton domain the
execution of the indexicals should succeed without contradiction exactly
when the predicate is intended to be true.

</P>
<P>
There can be several alternative definitions for the same user
constraint with different strengths in propagation.  For example, the
definition of <CODE>plusd</CODE> below encodes the same <CODE>X+Y=T</CODE> constraint as
the <CODE>plus</CODE> predicate above, but maintaining domain consistency:

</P>

<PRE>
plusd(X,Y,T) +:
        X in dom(T) - dom(Y),
        Y in dom(T) - dom(X),
        T in dom(X) + dom(Y).

| ?- X in {1}\/{3}, Y in {10}\/{20}, plusd(X, Y, T).

X in{1}\/{3},
Y in{10}\/{20},
T in{11}\/{13}\/{21}\/{23} ? 

yes
</PRE>

<P>
This costs more in terms of execution time, but gives more precise
results. For singleton domains <CODE>plus</CODE> and <CODE>plusd</CODE> behave in
the same way.

</P>
<P>
In our design, general indexicals can only appear in the context of FD
predicate definitions.
The rationale for this restriction is the need for general indexicals
to be able to suspend and resume, and this ability is only provided by the
FD predicate mechanism.

</P>
<P>
If the program merely posts a constraint, it suffices for the definition
to contain a single clause for solving the constraint.  If a constraint
is reified or occurs in a propositional formula, the definition must
contain four clauses for solving and checking entailment of the
constraint and its negation.  The role of each clause is reflected in
the "neck" operator.  The following table summarizes the different
forms of indexical clauses corresponding to a constraint <VAR>C</VAR>.  In
all cases, <VAR>Head</VAR> should be a compound term with all arguments being
distinct variables:

</P>
<DL COMPACT>

<DT><CODE><VAR>Head</VAR> +: <VAR>Indexicals</VAR>.</CODE>
<DD>
The clause consists of propagating indexicals for solving <VAR>C</VAR>.

<DT><CODE><VAR>Head</VAR> -: <VAR>Indexicals</VAR>.</CODE>
<DD>
The clause consists of propagating indexicals for solving the negation of <VAR>C</VAR>.

<DT><CODE><VAR>Head</VAR> +? <VAR>Indexical</VAR>.</CODE>
<DD>
The clause consists of a single checking indexical for testing
entailment of <VAR>C</VAR>.

<DT><CODE><VAR>Head</VAR> -? <VAR>Indexical</VAR>.</CODE>
<DD>
The clause consists of a single checking indexical for testing
entailment of the negation of <VAR>C</VAR>.
</DL>

<P>
When a constraint is reified,
the solver spawns two reactive agents corresponding to
detecting entailment and disentailment.  Eventually, one of them will
succeed in this and consequently will bind <VAR>B</VAR> to 0 or 1.  A third
agent is spawned, waiting for <VAR>B</VAR> to become assigned, at which time
the constraint (or its negation) is posted.  In the mean time, the
constraint may have been detected as (dis)entailed, in which case the
third agent is dismissed.  The waiting is implemented by means of the
coroutining facilities of SICStus Prolog.

</P>
<P>
As an example of a constraint with all methods defined, consider the following
library constraint defining a disequation between two domain variables:

</P>

<PRE>
'x\\=y'(X,Y) +:
        X in \{Y},
        Y in \{X}.
'x\\=y'(X,Y) -:
        X in dom(Y),
        Y in dom(X).
'x\\=y'(X,Y) +?
        X in \dom(Y).
'x\\=y'(X,Y) -?
        X in {Y}.
</PRE>

<P>
The following sections provide more precise coding rules and operational
details for indexicals.  <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE> denotes an indexical
corresponding to a constraint <VAR>C</VAR>.  <VAR>S</VAR> denotes the current store.

</P>


<H3><A NAME="SEC279" HREF="sicstus_toc.html#TOC279">Execution of Propagating Indexicals</A></H3>

<P>
Consider the definition of a constraint <VAR>C</VAR> containing a propagating
indexical <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE>.  Let <VAR>TV(X,C,S)</VAR> denote the set of
values for <VAR>X</VAR> that can make <VAR>C</VAR> true in some ground extension of
the store <VAR>S</VAR>.  Then the indexical should obey the following coding rules:

</P>

<UL>
<LI>all arguments of <VAR>C</VAR> except <VAR>X</VAR> should occur in <VAR>R</VAR>

<LI>if <VAR>R</VAR> is ground in <VAR>S</VAR>, <VAR>S(R) = TV(X,C,S)</VAR>

</UL>

<P>
If the coding rules are observed, <VAR>S(R)</VAR> can be proven to contain
<VAR>TV(X,C,S)</VAR> for all stores in which <VAR>R</VAR> is monotone.  Hence it is
natural for the implementation to wait until <VAR>R</VAR> becomes monotone
before admitting the propagating indexical for execution.
The execution of <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE> thus involves the following:

</P>

<UL>
<LI>

If <VAR>D(X,S)</VAR> is disjoint from <VAR>S(R)</VAR>, a contradiction is detected.

<LI>

If <VAR>D(X,S)</VAR> is contained in <VAR>S(R)</VAR>, <VAR>D(X,S)</VAR> does not
contain any values known to be incompatible with <VAR>C</VAR>, and the
indexical suspends, unless <VAR>R</VAR> is ground in <VAR>S</VAR>, in which
case <VAR>C</VAR> is detected as entailed.

<LI>

Otherwise, <VAR>D(X,S)</VAR> contains some values that are known to be
incompatible with <VAR>C</VAR>.  Hence, <CODE><VAR>X</VAR>::<VAR>S(R)</VAR></CODE> is added
to the store (<VAR>X</VAR> is <EM>pruned</EM>), and the indexical suspends,
unless <VAR>R</VAR> is ground in <VAR>S</VAR>, in which case <VAR>C</VAR> is detected as entailed.
</UL>

<P>
A propagating indexical is scheduled for execution as follows:

</P>

<UL>
<LI>it is evaluated initially as soon as it has become monotone

<LI>it is re-evaluated when one of the following conditions occurs:


<OL>
<LI>the domain of a variable <VAR>Y</VAR> that occurs as <CODE>dom(<VAR>Y</VAR>)</CODE>

      or <CODE>card(<VAR>Y</VAR>)</CODE> in <VAR>R</VAR> has been updated
<LI>the lower bound of a variable <VAR>Y</VAR> that occurs as <CODE>min(<VAR>Y</VAR>)</CODE>

      in <VAR>R</VAR> has been updated
<LI>the upper bound of a variable <VAR>Y</VAR> that occurs as <CODE>max(<VAR>Y</VAR>)</CODE>

      in <VAR>R</VAR> has been updated
</OL>

</UL>



<H3><A NAME="SEC280" HREF="sicstus_toc.html#TOC280">Execution of Checking Indexicals</A></H3>

<P>
Consider the definition of a constraint <VAR>C</VAR> containing a checking
indexical <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE>.  Let <VAR>FV(X,C,S)</VAR> denote the set of
values for <VAR>X</VAR> that can make <VAR>C</VAR> false in some ground extension of
the store <VAR>S</VAR>.  Then the indexical should obey the following coding rules:

</P>


<UL>
<LI>all arguments of <VAR>C</VAR> except <VAR>X</VAR> should occur in <VAR>R</VAR>

<LI>if <VAR>R</VAR> is ground in <VAR>S</VAR>, <VAR>S(R) = TV(X,C,S)</VAR>

</UL>

<P>
If the coding rules are observed, <VAR>S(R)</VAR> can be proven to exclude
<VAR>FV(X,C,S)</VAR> for all stores in which <VAR>R</VAR> is anti-monotone.  Hence it is
natural for the implementation to wait until <VAR>R</VAR> becomes anti-monotone
before admitting the checking indexical for execution.
The execution of <CODE><VAR>X</VAR> in <VAR>R</VAR></CODE> thus involves the following:

</P>

<UL>
<LI>

If <VAR>D(X,S)</VAR> is contained in <VAR>S(R)</VAR>, none of the possible values for
<VAR>X</VAR> can make <VAR>C</VAR> false, and so <VAR>C</VAR> is detected as entailed.

<LI>

Otherwise, if <VAR>D(X,S)</VAR> is disjoint from <VAR>S(R)</VAR> and <VAR>R</VAR> is
ground in <VAR>S</VAR>, all possible values for <VAR>X</VAR> will make <VAR>C</VAR>
false, and so <VAR>C</VAR> is detected as disentailed.

<LI>

Otherwise, <VAR>D(X,S)</VAR> contains some values that could make <VAR>C</VAR> true
and some that could make <VAR>C</VAR> false, and the indexical suspends.
</UL>

<P>
A checking indexical is scheduled for execution as follows:

</P>

<UL>
<LI>it is evaluated initially as soon as it has become anti-monotone

<LI>it is re-evaluated when one of the following conditions occurs:


<OL>
<LI>the domain of <VAR>X</VAR> has been pruned, or <VAR>X</VAR> has been assigned

<LI>the domain of a variable <VAR>Y</VAR> that occurs as <CODE>dom(<VAR>Y</VAR>)</CODE>

      or <CODE>card(<VAR>Y</VAR>)</CODE> in <VAR>R</VAR> has been pruned
<LI>the lower bound of a variable <VAR>Y</VAR> that occurs as <CODE>min(<VAR>Y</VAR>)</CODE>

      in <VAR>R</VAR> has been increased
<LI>the upper bound of a variable <VAR>Y</VAR> that occurs as <CODE>max(<VAR>Y</VAR>)</CODE>

      in <VAR>R</VAR> has been decreased
</OL>

</UL>



<H3><A NAME="SEC281" HREF="sicstus_toc.html#TOC281">Goal Expanded Constraints</A></H3>

<P>
The arithmetic, membership, and propositional constraints described
earlier are transformed at compile time to conjunctions of goals of
library constraints.

</P>
<P>
Sometimes it is necessary to postpone the expansion of a constraint
until runtime, e.g. if the arguments are not instantiated enough.
This can be achieved by wrapping <CODE>call/1</CODE> around the constraint.

</P>
<P>
Although space economic (linear in the size of the source code), the
expansion of a constraint to library goals can have an overhead compared
to expressing the constraint in terms of indexicals.  Temporary
variables holding intermediate values may have to be introduced, and the
grain size of the constraint solver invocations can be rather small.
The translation of constraints to library goals has been greatly
improved in the current version, so these problems have virtually
disappeared.  However, for backward compatibility, an implementation by
compilation to indexicals of the same constraints is also provided.  An
FD predicate may be defined by a single clause:

</P>

<PRE>
    <VAR>Head</VAR> +: <VAR>Constraint</VAR>.
</PRE>

<P>
where <VAR>Constraint</VAR> is an arithmetic constraint or an <CODE>element/3</CODE> or
a <CODE>relation/3</CODE> constraint.  This translation is only available for
<SAMP>`+:'</SAMP> clauses; thus, <VAR>Head</VAR> cannot be reified.

</P>
<P>
In the case of arithmetic constraints, the constraint must be over
linear terms (see section <A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>).  The memory consumption of
the FD predicate will be quadratic in the size of the source code.  The
alternative version of <CODE>sum/8</CODE> in section <A HREF="sicstus_33.html#SEC283">Send More Money</A> illustrates
this technique.

</P>
<P>
In the case of <CODE>element(X,L,Y)</CODE> or <CODE>relation(X,L,Y)</CODE>, the
memory consumption of the FD predicate will be linear in the size of the
source code.  The execution time of the initial evaluation of the FD
predicate will be linear in the size of the initial domains for <VAR>X</VAR>
and <VAR>Y</VAR>; if these domains are infinite, no propagation will
take place.

</P>


<H2><A NAME="SEC282" HREF="sicstus_toc.html#TOC282">Example Programs</A></H2>

<UL>
<LI><A HREF="sicstus_33.html#SEC283">Send More Money</A>
<LI><A HREF="sicstus_33.html#SEC284">N Queens</A>
<LI><A HREF="sicstus_33.html#SEC285">Cumulative Scheduling</A>
</UL>

<P>
This section contains a few example programs.  The first two programs
are included in a benchmark suite that comes with the distribution.
The benchmark suite is run by typing:

</P>

<PRE>
| ?- compile(library('clpfd/examples/bench')).
| ?- bench.
</PRE>



<H3><A NAME="SEC283" HREF="sicstus_toc.html#TOC283">Send More Money</A></H3>

<P>
Let us return briefly to the Send More Money problem 
(see section <A HREF="sicstus_33.html#SEC254">A Constraint Satisfaction Problem</A>).  Its <CODE>sum/8</CODE> predicate
will expand to a space-efficient conjunction of library constraints.
A faster but more memory consuming version is defined simply by changing
the neck symbol of <CODE>sum/8</CODE> from <SAMP>`:-'</SAMP> to <SAMP>`+:'</SAMP>, thus
turning it into an FD predicate:

</P>

<PRE>
sum(S, E, N, D, M, O, R, Y) +:
                  1000*S + 100*E + 10*N + D 
     +            1000*M + 100*O + 10*R + E
     #= 10000*M + 1000*O + 100*N + 10*E + Y.
</PRE>



<H3><A NAME="SEC284" HREF="sicstus_toc.html#TOC284">N Queens</A></H3>

<P>
The problem is to place N queens on an NxN chess board so that no queen
is threatened by another queen.

</P>
<P>
The variables of this problem are the N queens.  Each queen has a designated
row.  The problem is to select a column for it.

</P>
<P>
The main constraint of this problem is that no queen threaten another.
This is encoded by the <CODE>no_threat/3</CODE> constraint and holds between
all pairs <CODE>(X,Y)</CODE> of queens.   It could be defined as

</P>

<PRE>
no_threat(X, Y, I) :-
        X   #\= Y,
        X+I #\= Y,
        X-I #\= Y.
</PRE>

<P>
However, this formulation introduces new temporary domain variables and
creates twelve fine-grained indexicals.  Worse, the arithmetic constraints
are only guaranteed to maintain interval-consistency and so may
miss some opportunities for pruning elements in the middle of domains.

</P>
<P>
A better idea is to formulate <CODE>no_threat/3</CODE> as an FD predicate with
two indexicals, as shown in the program below.  This constraint will not
fire until one of the queens has been assigned (the corresponding
indexical does not become monotone until then).  Hence, the constraint
is still not as strong as it could be.

</P>
<P>
For example, if the domain of one queen is (2..3), then it will
threaten any queen placed in column 2 or 3 on an adjacent row, no
matter which of the two open positions is chosen for the first queen.
The commented out formulation of the constraint captures this
reasoning, and illustrates the use of the <CODE>unionof/3</CODE> operator.
This stronger version of the constraint indeed gives less
backtracking, but is computationally more expensive and does not pay
off in terms of execution time, except possibly for very large chess
boards.

</P>
<P>
It is clear that <CODE>no_threat/3</CODE> cannot detect any incompatible values
for a queen with domain of size greater than three.  This observation is
exploited in the third version of the constraint.

</P>
<P>
The first-fail principle is appropriate in the enumeration part of
this problem.

</P>

<PRE>
:- use_module(library(clpfd)).

queens(N, L, LabelingType) :-
     length(L, N),
     domain(L, 1, N),
     constrain_all(L),
     labeling(LabelingType, L).

constrain_all([]).
constrain_all([X|Xs]) :-
     constrain_between(X, Xs, 1),
     constrain_all(Xs).

constrain_between(_X, [], _N).
constrain_between(X, [Y|Ys], N) :-
     no_threat(X, Y, N),
     N1 is N+1,
     constrain_between(X, Ys, N1).

% version 1: weak but efficient
no_threat(X, Y, I) +:
     X in \({Y} \/ {Y+I} \/ {Y-I}),
     Y in \({X} \/ {X+I} \/ {X-I}).

/*
% version 2: strong but very inefficient version
no_threat(X, Y, I) +:
    X in unionof(B,dom(Y),\({B} \/ {B+I} \/ {B-I})),
    Y in unionof(B,dom(X),\({B} \/ {B+I} \/ {B-I})).

% version 3: strong but somewhat inefficient version
no_threat(X, Y, I) +:
    X in (4..card(Y)) ? (inf..sup) \/ 
          unionof(B,dom(Y),\({B} \/ {B+I} \/ {B-I})),
    Y in (4..card(X)) ? (inf..sup) \/ 
          unionof(B,dom(X),\({B} \/ {B+I} \/ {B-I})).
*/

| ?- queens(8, L, [ff]).

L = [1,5,8,6,3,7,2,4] ? 

</PRE>



<H3><A NAME="SEC285" HREF="sicstus_toc.html#TOC285">Cumulative Scheduling</A></H3>

<P>
This example is a very small scheduling problem.  We consider seven tasks
where each task has a fixed duration and a fixed amount of used resource:

</P>

<PRE>
TASK    DURATION        RESOURCE
====    ========        ========
t1            16               2
t2             6               9
t3            13               3
t4             7               7
t5             5              10
t6            18               1
t7             4              11
</PRE>

<P>
The goal is to find a schedule that minimizes the completion time for
the schedule while not exceeding the capacity 13 of the resource.  The
resource constraint is succinctly captured by a <CODE>cumulative/4</CODE>
constraint.  Branch-and-bound search is used to find the minimal
completion time.

</P>
<P>
This example was borrowed from [Beldiceanu &#38; Contejean 94].

</P>

<PRE>
:- use_module(library(clpfd)).

schedule(Ss, End) :-
        length(Ss, 7),
        Ds = [16, 6,13, 7, 5,18, 4],
        Rs = [ 2, 9, 3, 7,10, 1,11],
        domain(Ss, 1, 30),
        domain([End], 1, 50),
        after(Ss, Ds, End),
        cumulative(Ss, Ds, Rs, 13),
        labeling([minimize(End)], [End|Ss]).

after([], [], _).
after([S|Ss], [D|Ds], E) :- E #&#62;= S+D, after(Ss, Ds, E).

%% End of file

| ?- schedule(Ss, End).

Ss = [1,17,10,10,5,5,1],
End = 23 ? 
</PRE>



<H2><A NAME="SEC286" HREF="sicstus_toc.html#TOC286">Syntax Summary</A></H2>


<UL>
<LI><A HREF="sicstus_33.html#SEC287">Syntax of Indexicals</A>
<LI><A HREF="sicstus_33.html#SEC288">Syntax of Arithmetic Expressions</A>
<LI><A HREF="sicstus_33.html#SEC289">Operator Declarations</A>
</UL>



<H3><A NAME="SEC287" HREF="sicstus_toc.html#TOC287">Syntax of Indexicals</A></H3>


<PRE>
<VAR>X</VAR> --&#62; <VAR>variable</VAR>                { domain variable }

<VAR>Constant</VAR> --&#62; <VAR>integer</VAR>
  |   inf                         { minus infinity }
  |   sup                         { plus infinity }

<VAR>Term</VAR> --&#62; <VAR>Constant</VAR>
  |   <VAR>X</VAR>                     { suspend until assigned }
  |   min(<VAR>X</VAR>)                { min. of domain of X }
  |   max(<VAR>X</VAR>)                { max. of domain of X }
  |   card(<VAR>X</VAR>)               { size of domain of X }
  |   - <VAR>Term</VAR>
  |   <VAR>Term</VAR> + <VAR>Term</VAR>
  |   <VAR>Term</VAR> - <VAR>Term</VAR>
  |   <VAR>Term</VAR> * <VAR>Term</VAR>
  |   <VAR>Term</VAR> /&#62; <VAR>Term</VAR>               { division rounded up }
  |   <VAR>Term</VAR> /&#60; <VAR>Term</VAR>               { division rounded down }
  |   <VAR>Term</VAR> mod <VAR>Term</VAR>

<VAR>TermSet</VAR> --&#62; {<VAR>Term</VAR>,...,<VAR>Term</VAR>}

<VAR>Range</VAR> --&#62; <VAR>TermSet</VAR>
  |   dom(<VAR>X</VAR>)                          { domain of X }
  |   <VAR>Term</VAR>..<VAR>Term</VAR>                      { interval }
  |   <VAR>Range</VAR>/\<VAR>Range</VAR>                     { intersection }
  |   <VAR>Range</VAR>\/<VAR>Range</VAR>                     { union }
  |   \<VAR>Range</VAR>                          { complement }
  |   - <VAR>Range</VAR>                         { pointwise negation }
  |   <VAR>Range</VAR> + <VAR>Range</VAR>                   { pointwise addition }
  |   <VAR>Range</VAR> - <VAR>Range</VAR>                   { pointwise subtraction }
  |   <VAR>Range</VAR> mod <VAR>Range</VAR>                 { pointwise modulo }
  |   <VAR>Range</VAR> + <VAR>Term</VAR>                    { pointwise addition }
  |   <VAR>Range</VAR> - <VAR>Term</VAR>                    { pointwise subtraction }
  |   <VAR>Term</VAR> - <VAR>Range</VAR>                    { pointwise subtraction }
  |   <VAR>Range</VAR> mod <VAR>Term</VAR>                  { pointwise modulo }
  |   <VAR>Range</VAR> ? <VAR>Range</VAR>
  |   unionof(<VAR>X</VAR>,<VAR>Range</VAR>,<VAR>Range</VAR>)
  |   switch(<VAR>Term</VAR>,<VAR>MapList</VAR>)

<VAR>ConstantSet</VAR> --&#62; {<VAR>integer</VAR>,...,<VAR>integer</VAR>}

<VAR>ConstantRange</VAR> --&#62; <VAR>ConstantSet</VAR>
  |   <VAR>Constant</VAR>..<VAR>Constant</VAR>
  |   <VAR>ConstantRange</VAR>/\<VAR>ConstantRange</VAR>
  |   <VAR>ConstantRange</VAR>\/<VAR>ConstantRange</VAR>
  |   \<VAR>ConstantRange</VAR>

<VAR>MapList</VAR> --&#62; []
  |   [<VAR>integer</VAR>-<VAR>ConstantRange</VAR>|<VAR>MapList</VAR>]

<VAR>CList</VAR> --&#62; []
  |   [<VAR>integer</VAR>|<VAR>CList</VAR>]

<VAR>Indexical</VAR> --&#62; <VAR>X</VAR> in <VAR>Range</VAR>

<VAR>Indexicals</VAR> --&#62; <VAR>Indexical</VAR>
  |   <VAR>Indexical</VAR>, <VAR>Indexicals</VAR>

<VAR>ConstraintBody</VAR> --&#62; <VAR>Indexicals</VAR>
   |  <VAR>LinExpr</VAR> <VAR>RelOp</VAR> <VAR>LinExpr</VAR>
   |  element(<VAR>X</VAR>,<VAR>CList</VAR>,<VAR>X</VAR>)
   |  relation(<VAR>X</VAR>,<VAR>MapList</VAR>,<VAR>X</VAR>)

<VAR>Head</VAR> --&#62; <VAR>term</VAR>  { a compound term with unique variable args }

<VAR>TellPos</VAR> --&#62; <VAR>Head</VAR> +: <VAR>ConstraintBody</VAR>.
<VAR>TellNeg</VAR> --&#62; <VAR>Head</VAR> -: <VAR>ConstraintBody</VAR>.
<VAR>AskPos</VAR> --&#62; <VAR>Head</VAR> +? <VAR>Indexical</VAR>.
<VAR>AskNeg</VAR> --&#62; <VAR>Head</VAR> -? <VAR>Indexical</VAR>.

<VAR>ConstraintDef</VAR> --&#62;
      <VAR>TellPos</VAR>. [<VAR>TellNeg</VAR>.] [<VAR>AskPos</VAR>.] [<VAR>AskNeg</VAR>.] 
</PRE>



<H3><A NAME="SEC288" HREF="sicstus_toc.html#TOC288">Syntax of Arithmetic Expressions</A></H3>


<PRE>
<VAR>X</VAR> --&#62; <VAR>variable</VAR>                { domain variable }

<VAR>N</VAR> --&#62; <VAR>integer</VAR>

<VAR>LinExpr</VAR> --&#62; <VAR>N</VAR>                { linear expression }
  |   <VAR>X</VAR>
  |   <VAR>N</VAR> * <VAR>X</VAR>
  |   <VAR>N</VAR> * <VAR>N</VAR>
  |   <VAR>LinExpr</VAR> + <VAR>LinExpr</VAR>
  |   <VAR>LinExpr</VAR> - <VAR>LinExpr</VAR>

<VAR>Expr</VAR> --&#62; <VAR>LinExpr</VAR>
  |   <VAR>Expr</VAR> + <VAR>Expr</VAR>
  |   <VAR>Expr</VAR> - <VAR>Expr</VAR>
  |   <VAR>Expr</VAR> * <VAR>Expr</VAR>
  |   <VAR>Expr</VAR> / <VAR>Expr</VAR>                { integer division }
  |   <VAR>Expr</VAR> mod <VAR>Expr</VAR>
  |   min(<VAR>Expr</VAR>,<VAR>Expr</VAR>)
  |   max(<VAR>Expr</VAR>,<VAR>Expr</VAR>)
  |   abs(<VAR>Expr</VAR>)

<VAR>RelOp</VAR> --&#62; #= | #\= | #&#60; | #=&#60; | #&#62; | #&#62;=
</PRE>



<H3><A NAME="SEC289" HREF="sicstus_toc.html#TOC289">Operator Declarations</A></H3>


<PRE>
:- op(1200, xfx, [+:,-:,+?,-?]).
:- op(760,  yfx, #&#60;=&#62;).
:- op(750,  xfy, #=&#62;).
:- op(750,  yfx, #&#60;=).
:- op(740,  yfx, #\/).
:- op(730,  yfx, #\).
:- op(720,  yfx, #/\).
:- op(710,   fy, #\).
:- op(700,  xfx, [in,in_set]).
:- op(700,  xfx, [#=,#\=,#&#60;,#=&#60;,#&#62;,#&#62;=]).
:- op(550,  xfx, ..).
:- op(500,   fy, \).
:- op(490,  yfx, ?).
:- op(400,  yfx, [/&#62;,/&#60;]).
</PRE>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_32.html">previous</A>, <A HREF="sicstus_34.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
