<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Constraint Handling Rules</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_33.html">previous</A>, <A HREF="sicstus_35.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC290" HREF="sicstus_toc.html#TOC290">Constraint Handling Rules</A></H1>


<UL>
<LI><A HREF="sicstus_34.html#SEC291">CHR Copyright</A>
<LI><A HREF="sicstus_34.html#SEC292">CHR Introduction</A>
<LI><A HREF="sicstus_34.html#SEC293">CHR Introductory Examples</A>
<LI><A HREF="sicstus_34.html#SEC294">CHR Library</A>
<LI><A HREF="sicstus_34.html#SEC306">CHR Debugging</A>
<LI><A HREF="sicstus_34.html#SEC312">CHR Programming Hints</A>
<LI><A HREF="sicstus_34.html#SEC313">CHR Constraint Handlers</A>
<LI><A HREF="sicstus_34.html#SEC314">CHR Backward Compatibility</A>
</UL>



<H2><A NAME="SEC291" HREF="sicstus_toc.html#TOC291">Copyright</A></H2>

<P>
This chapter is Copyright (C) 1996-98 LMU

</P>
<P>
LMU (Ludwig-Maximilians-University)<BR>
Munich, Germany

</P>
<P>
Permission is granted to make and distribute verbatim copies of
this chapter provided the copyright notice and this permission notice
are preserved on all copies.

</P>
<P>
Permission is granted to copy and distribute modified versions of this
chapter under the conditions for verbatim copying, provided that the entire
resulting derived work is distributed under the terms of a permission
notice identical to this one.

</P>
<P>
Permission is granted to copy and distribute translations of this chapter
into another language, under the above conditions for modified versions,
except that this permission notice may be stated in a translation approved
by LMU.

</P>


<H2><A NAME="SEC292" HREF="sicstus_toc.html#TOC292">Introduction</A></H2>

<P>
Experience from real-life applications using constraint-based
programming has shown that typically, one is confronted with a
heterogeneous mix of different types of constraints. To be able to
express constraints as they appear in the application and to write and
combine constraint systems, a special purpose language for writing
constraint systems called <EM>constraint handling rules</EM> (CHR) was
developed.  CHR have been used to encode a wide range of constraint
handlers (solvers), including new domains such as terminological and
temporal reasoning.  Several CHR libraries exist in declarative
languages such as Prolog and LISP, worldwide more than 20 projects use
CHR. You can find more information about CHR at URL:
<CODE>http://www.pst.informatik.uni-muenchen.de/personen/fruehwir/chr-intro.html</CODE>

</P>
<P>
The high-level CHR are an excellent tool for rapid prototyping
and implementation of constraint handlers. The usual abstract formalism
to describe a constraint system, i.e. inference rules, rewrite rules,
sequents, formulas expressing axioms and theorems, can be written as
CHR in a straightforward way.  Starting from this executable
specification, the rules can be refined and adapted to the specifics of
the application.

</P>
<P>
The CHR library includes a compiler, which translates
CHR programs into Prolog programs on the fly, and a runtime system,
which includes a stepper for debugging.  Many constraint
handlers are provided in the example directory of the library.

</P>
<P>
CHR are essentially a committed-choice language consisting of guarded
rules that rewrite constraints into simpler ones until they are solved.
CHR define both <EM>simplification</EM> of and <EM>propagation</EM> over
constraints. Simplification replaces constraints by
simpler constraints while preserving logical equivalence (e.g.  
<CODE>X&#62;Y,Y&#62;X &#60;=&#62; fail</CODE>).  Propagation adds new constraints which are
logically redundant but may cause further simplification (e.g. 
<CODE>X&#62;Y,Y&#62;Z ==&#62; X&#62;Z</CODE>).  Repeatedly applying CHR incrementally simplifies
and finally solves constraints (e.g. <CODE>A&#62;B,B&#62;C,C&#62;A</CODE>
leads to <CODE>fail</CODE>.

</P>
<P>
With multiple heads and propagation rules, CHR provide two features
which are essential for non-trivial constraint handling.  The
declarative reading of CHR as formulas of first order logic allows
one to reason about their correctness.  On the other hand, regarding
CHR as a rewrite system on logical formulas allows one to reason
about their termination and confluence.

</P>
<P>
In case the implementation of CHR disagrees with your expectations
based on this chapter, drop a line to the current maintainer:
<CODE>christian@ai.univie.ac.at</CODE> (Christian Holzbaur).

</P>


<H2><A NAME="SEC293" HREF="sicstus_toc.html#TOC293">Introductory Examples</A></H2>

<P>
We define a CHR constraint for less-than-or-equal, <CODE>leq</CODE>, that can
handle variable arguments. This handler can be found in the library as
the file <CODE>leq.pl</CODE>.  (The code works regardless of options switched
on or off.)

<PRE>
:- use_module(library(chr)).

handler leq.
constraints leq/2.
:- op(500, xfx, leq).

reflexivity  @ X leq Y &#60;=&#62; X=Y | true.
antisymmetry @ X leq Y , Y leq X &#60;=&#62; X=Y.
idempotence  @ X leq Y \ X leq Y &#60;=&#62; true.
transitivity @ X leq Y , Y leq Z ==&#62; X leq Z.
</PRE>

<P>
The CHR specify how <CODE>leq</CODE> simplifies and propagates as a
constraint.  They implement reflexivity, idempotence, antisymmetry and
transitivity in a straightforward way.  CHR <CODE>reflexivity</CODE> states
that <CODE>X leq Y</CODE> simplifies to <CODE>true</CODE>, provided it is the case that
<CODE>X=Y</CODE>. This test forms the (optional) guard of a rule, a
precondition on the applicability of the rule.  Hence, whenever we see a
constraint of the form <CODE>A leq A</CODE> we can simplify it to <CODE>true</CODE>.

</P>
<P>
The rule <CODE>antisymmetry</CODE> means that if we find <CODE>X leq Y</CODE> as well as
<CODE>Y leq X</CODE> in the constraint store, we can replace it by the
logically equivalent <CODE>X=Y</CODE>. Note the different use of <CODE>X=Y</CODE> in
the two rules: In the <CODE>reflexivity</CODE> rule the equality is a
precondition (test) on the rule, while in the <CODE>antisymmetry</CODE> rule
it is enforced when the rule fires. (The reflexivity rule could also have 
been written as <CODE>reflexivity   X leq X &#60;=&#62; true</CODE>.)

</P>
<P>
The rules <CODE>reflexivity</CODE> and <CODE>antisymmetry</CODE> are
<EM>simplification CHR</EM>. In such rules, the constraints found are
removed when the rule applies and fires.  The rule <CODE>idempotence</CODE> is
a <EM>simpagation CHR</EM>, only the constraints right of <CODE>'\'</CODE> will
be removed. The rule  says that if we find <CODE>X leq Y</CODE> and another <CODE>X
leq Y</CODE> in the constraint store, we can remove one.

</P>
<P>
Finally, the rule <CODE>transitivity</CODE> states that the conjunction
<CODE>X leq Y, Y leq Z</CODE> implies <CODE>X leq Z</CODE>.  Operationally, we add
<CODE>X leq Z</CODE> as (redundant) constraint, without removing the
constraints <CODE>X leq Y, Y leq Z</CODE>. This kind of CHR is called
<EM>propagation CHR</EM>.

</P>
<P>
Propagation CHR are useful, as the query <CODE>A leq B,C leq A,B leq C</CODE>
illustrates: The first two constraints cause CHR <CODE>transitivity</CODE> to
fire and add <CODE>C leq B</CODE> to the query. This new constraint together
with <CODE>B leq C</CODE> matches the head of CHR <CODE>antisymmetry</CODE>, <CODE>X
leq Y, Y leq X</CODE>.  So the two constraints are replaced by
<CODE>B=C</CODE>. Since <CODE>B=C</CODE> makes <CODE>B</CODE> and <CODE>C</CODE> equivalent, CHR
<CODE>antisymmetry</CODE> applies to the constraints <CODE>A leq B, C leq A</CODE>,
resulting in <CODE>A=B</CODE>. The query contains no more CHR constraints, the
simplification stops.  The constraint handler we built has solved
<CODE>A leq B, C leq A, B leq C</CODE> and produced the answer <CODE>A=B,
B=C</CODE>:

</P>

<PRE>
A leq B,C leq A,B leq C. 
% C leq A, A leq B propagates C leq B by transitivity.
% C leq B, B leq C simplifies to B=C by antisymmetry.
% A leq B, C leq A simplifies to A=B by antisymmetry since B=C.
A=B,B=C.
</PRE>

<P>
Note that multiple heads of rules are essential in solving these
constraints.  Also note that this handler implements a (partial) order
constraint over any constraint domain, this generality is only possible
with CHR.

</P>
<P>
As another example, we can implement the sieve of Eratosthenes to
compute primes simply as (for variations see the handler
<TT>`primes.pl'</TT>):

</P>

<PRE>
:- use_module(library(chr)).
handler eratosthenes.
constraints primes/1,prime/1.

primes(1) &#60;=&#62; true.
primes(N) &#60;=&#62; N&#62;1 | M is N-1,prime(N),primes(M). % generate candidates

absorb(J) @ prime(I) \ prime(J) &#60;=&#62; J mod I =:= 0 | true. 
</PRE>

<P>
The constraint <CODE>primes(N)</CODE> generates candidates for prime numbers,
<CODE>prime(M)</CODE>, where <CODE>M</CODE> is between <CODE>1</CODE> and <CODE>N</CODE>.
The candidates react with each other such that each
number absorbs multiples of itself. In the end, only prime numbers remain.

</P>
<P>
Looking at the two rules defining <CODE>primes/1</CODE>, note that head
matching is used in CHR, so the first rule will only apply to
<CODE>primes(1)</CODE>. The test <CODE>N&#62;1</CODE> is a guard (precondition) on the
second rule.  A call with a free variable, like <CODE>primes(X)</CODE>,
will delay (suspend).  The third, multi-headed rule <CODE>absorb(J)</CODE> 
reads as follows:
If there is a constraint <CODE>prime(I)</CODE> and some other constraint
<CODE>prime(J)</CODE> such that <CODE>J mod I =:= 0</CODE> holds, i.e. <CODE>J</CODE> is a
multiple of <CODE>I</CODE>, then keep <CODE>prime(I)</CODE> but remove
<CODE>prime(J)</CODE> and execute the body of the  rule,
<CODE>true</CODE>.

</P>


<H2><A NAME="SEC294" HREF="sicstus_toc.html#TOC294">CHR Library</A></H2>

<P>
CHR extend the Prolog syntax by a few constructs introduced in the next
sections. Technically, the extension is achieved through the
<CODE>user:term_expansion/2</CODE> mechanism. A file that contains a constraint
handler may also contain arbitrary Prolog code.  Constraint handling
rules can be scattered across a file. Declarations and options should
precede rules. There can only be at most one constraint handler per module.

</P>

<UL>
<LI><A HREF="sicstus_34.html#SEC295">CHR Loading the Library</A>
<LI><A HREF="sicstus_34.html#SEC296">CHR Declarations</A>
<LI><A HREF="sicstus_34.html#SEC297">CHR Syntax</A>
<LI><A HREF="sicstus_34.html#SEC298">How CHR work</A>
<LI><A HREF="sicstus_34.html#SEC299">CHR Pragmas</A>
<LI><A HREF="sicstus_34.html#SEC300">CHR Options</A>
<LI><A HREF="sicstus_34.html#SEC301">CHR Built-In Predicates</A>
<LI><A HREF="sicstus_34.html#SEC302">CHR Consulting and Compiling</A>
<LI><A HREF="sicstus_34.html#SEC303">CHR Compiler-generated Predicates</A>
<LI><A HREF="sicstus_34.html#SEC304">CHR Operator Declarations</A>
<LI><A HREF="sicstus_34.html#SEC305">CHR Exceptions</A>
</UL>



<H3><A NAME="SEC295" HREF="sicstus_toc.html#TOC295">Loading the Library</A></H3>

<P>
Before you can load or compile any file containing a
constraint handler (solver) written in CHR, the <CODE>chr</CODE> library
module has to be imported:

<PRE>
| ?- use_module(library(chr)).
</PRE>

<P>
It is recommended to include the corresponding directive at the 
start of your files containing handlers:

<PRE>
  :- use_module(library(chr)).
</PRE>



<H3><A NAME="SEC296" HREF="sicstus_toc.html#TOC296">Declarations</A></H3>

<P>
Declarations in files containing CHR affect the compilation and thus
the behavior of the rules at runtime.

</P>
<P>
The mandatory handler declaration precedes any other CHR specific
code.
Example:

<PRE>
handler minmax.
</PRE>

<P>
A handler name must be a valid Prolog <CODE>atom</CODE>.
Per module, only one constraint handler can be defined.

</P>
<P>
The constraints must be declared before they are used by rules.
With this mandatory declaration one lists the constraints 
the rules will later talk about. The declaration can be used more
than once per handler.
Example:

<PRE>
constraints  leq/2, minimum/3, maximum/3.
</PRE>

<P>
The following
optional declaration allows for conditional rule compilation.
Only the rules mentioned get compiled. Rules are referred to by their 
names (see section <A HREF="sicstus_34.html#SEC297">Constraint Handling Rules, Syntax</A>). The latest occurrence takes precedence if used
more than once per handler. Although it can be put anywhere in the handler
file, it makes sense, as with other declarations, to use it early.
Example:

<PRE>
rules antisymmetry, transitivity.
</PRE>

<P>
To simplify the handling of operator declarations, in particular
during <CODE>fcompile/1</CODE>, <CODE>operator/3</CODE> declarations with the
same denotation as <CODE>op/3</CODE>, but 
taking effect during compilation and loading, are helpful.
Example:

<PRE>
operator(700, xfx, ::).
operator(600, xfx, :).
</PRE>



<H3><A NAME="SEC297" HREF="sicstus_toc.html#TOC297">Constraint Handling Rules, Syntax</A></H3>

<P>
A constraint handling rule has one or more heads, an optional guard, a
body and an optional name.  A <EM>Head</EM> is a <EM>Constraint</EM>. A
constraint is a callable Prolog term,
whose functor is a declared constraint.
The <EM>Guard</EM> is a Prolog goal.  
The <EM>Body</EM> of a rule is a Prolog
goal (including constraints).  
A rule can be named with
a <EM>Name</EM> which can be any Prolog term (including variables
from the rule). 

</P>
<P>
There are three kinds of constraint handling rules:

<PRE>
Rule           --&#62; [Name @] 
                     (Simplification | Propagation | Simpagation) 
                        [pragma Pragma].

Simplification --&#62; Heads         &#60;=&#62; [Guard '|'] Body
Propagation    --&#62; Heads         ==&#62; [Guard '|'] Body
Simpagation    --&#62; Heads \ Heads &#60;=&#62; [Guard '|'] Body

Heads          --&#62; Head | Head, Heads
Head           --&#62; Constraint | Constraint # Id
Constraint     --&#62; a callable term declared as constraint
Id             --&#62; a unique variable

Guard          --&#62; Ask | Ask &#38; Tell
Ask            --&#62; Goal
Tell           --&#62; Goal
Goal           --&#62; a callable term, including conjunction and disjunction etc.

Body           --&#62; Goal

Pragma         --&#62; a conjunction of terms usually referring to 
                   one or more heads identified via #/2
</PRE>

<P>
The symbol <SAMP>`|'</SAMP> separates the guard (if present) from the body of a
rule.  Since <SAMP>`|'</SAMP> is read as <SAMP>`;'</SAMP> (disjunction) by the  
reader, care has to be taken when using disjunction in the guard
or body of the rule. The top level disjunction will always be
interpreted as guard-body separator <SAMP>`|'</SAMP>, so proper bracketing has
to be used, e.g. <CODE>a &#60;=&#62; (b;c) | (d;e)</CODE> instead of <CODE>a &#60;=&#62; b;c |
d;e</CODE> and <CODE>a &#60;=&#62; true | (d;e)</CODE> instead of <CODE>a &#60;=&#62; (d;e)</CODE>.

</P>
<P>
In simpagation rules, <SAMP>`\'</SAMP> separates the heads of the rule into two parts.

</P>
<P>
Individual head constraints may be tagged with variables via
<SAMP>`#'</SAMP>, which may be used as identifiers in pragma declarations, for
example. Constraint identifiers must be distinct variables, not
occurring elsewhere in the heads.

</P>
<P>
Guards test the applicability of a rule.  Guards come in two
parts, tell and ask, separated by <SAMP>`&#38;'</SAMP>. 
If the <SAMP>`&#38;'</SAMP> operator is not present, the whole guard is
assumed to be of the ask type.

</P>
<P>
Declaratively, a rule relates heads and body <EM>provided the guard is
true</EM>.  A simplification rule means that the heads are true if and only
if the body is true.  A propagation rule means that the body is true if
the heads are true.  A simpagation rule combines a simplification and a
propagation rule.  The rule <CODE>Heads1 \ Heads2 &#60;=&#62; Body</CODE> is
equivalent to the simplification rule <CODE>Heads1, Heads2 &#60;=&#62; Heads1,
Body</CODE>. However, the simpagation rule is more compact to write, more
efficient to execute and has better termination behavior than the
corresponding simplification rule, since the constraints comprising
<CODE>Heads1</CODE> will not be removed and inserted again.

</P>


<H3><A NAME="SEC298" HREF="sicstus_toc.html#TOC298">How CHR work</A></H3>

<P>
Each CHR constraint is associated with all rules in whose heads it
occurs by the CHR compiler. Every time a CHR constraint is executed
(called) or woken and reconsidered, it checks itself the applicability
of its associated CHR by <EM>trying</EM> each CHR.  By default, the rules
are tried in textual order, i.e. in the order they occur in the defining
file.  To try a CHR, one of its heads is matched against the constraint.
Matching succeeds if the constraint is an instance of the head.
If a CHR has more than one head, the constraint store is searched for
<EM>partner</EM> constraints that match the other heads.  Heads are tried
from left to right, except that in simpagation rules, the heads to be
removed are tried before the head constraints to be kept (this is done
for efficiency reasons).  If the matching succeeds, the guard is
executed. Otherwise the next rule is tried.

</P>
<P>
The guard either succeeds or fails.  A guard succeeds if the execution
of its Ask and Tell parts succeeds and in the ask part no variable that
occurs also in the heads was touched or the cause of an instantiation
error. The ask guard will fail otherwise.  A variable is <EM>touched</EM>
if it is unified with a term (including other variables from other
constraints) different from itself.  Tell guards, on the contrary, are
trusted and not checked for that property.  If the guard succeeds, the
rule applies.  Otherwise the next rule is tried.

</P>
<P>
If the firing CHR is a simplification rule, the matched constraints are
removed from the store and the body of the CHR is executed.  Similarly
for a firing simpagation rule, except that the constraints that matched
the heads preceding <SAMP>`\'</SAMP> are kept.  If the firing CHR is a
propagation rule the body of the CHR is executed without removing any
constraints.  It is remembered that the propagation rule fired, so it
will not fire again with the same constraints if the constraint is woken
and reconsidered.  If the currently active constraint has not been
removed, the next rule is tried.

</P>
<P>
If the current constraint has not been removed and all rules have been
tried, it delays until a variable occurring in the constraint is
touched.  Delaying means that the constraint is inserted into the
constraint store. When a constraint is woken, all its rules are tried
again. (This process can be watched and inspected with the CHR debugger,
see below.)

</P>



<H3><A NAME="SEC299" HREF="sicstus_toc.html#TOC299">Pragmas</A></H3>

<P>
Pragmas are annotations to rules and constraints that enable the 
compiler to generate more specific, more optimized code. A pragma
can be a conjunction of the following terms:

</P>
<DL COMPACT>

<DT><CODE>already_in_heads</CODE>
<DD>
The intention of simplification and simpagation rules is often
to combine the heads into a stronger version of one of them.
Depending on the strength of the guard, the new constraint may
be identical to one of the heads to removed by the rule.
This removal followed by addition is inefficient and may even cause
termination problems. If the pragma is used, this situation is
detected and the corresponding problems are avoided.
The pragma applies to all constraints removed by the rule.

<DT><CODE>already_in_head(Id)</CODE>
<DD>
Shares the intention of the previous pragma, but affects only 
the constraint indicated via <VAR>Id</VAR>. Note that one can use more
than one pragma per rule.

<DT><CODE>passive(Id)</CODE>
<DD>
No code will be generated for the specified constraint in the particular
head position. This means that the constraint will not see the rule, it
is passive in that rule. This changes the behavior of the CHR system,
because normally, a rule can be entered starting from each head constraint.
Usually this pragma will improve the efficiency of the
constraint handler, but care has to be taken in order not to lose
completeness.

For example, in the handler <CODE>leq</CODE>, any pair of constraints, say
<CODE>A leq B, B leq A</CODE>, that matches the head <CODE>X leq Y , Y leq X</CODE>
of the <CODE>antisymmetry</CODE> rule, will also match it when the constraints
are exchanged, <CODE>B leq A, A leq B</CODE>. Therefore it is enough if a
currently active constraint enters this rule in the first head only,
the second head can be declared to be passive. Similarly for the
<CODE>idempotence</CODE> rule.  For this rule, it is more efficient to declare
the first head passive, so that the currently active constraint will be
removed when the rule fires (instead of removing the older constraint
and redoing all the propagation with the currently active constraint).
Note that the compiler itself detects the symmetry of the two head
constraints in the simplification rule <CODE>antisymmetry</CODE>, thus it is
automatically declared passive and the compiler outputs <CODE>CHR
eliminated code for head 2 in antisymmetry</CODE>.

<PRE>
antisymmetry  X leq Y , Y leq X # Id &#60;=&#62; X=Y pragma passive(Id).
idempotence   X leq Y # Id \ X leq Y &#60;=&#62; true pragma passive(Id).
transitivity  X leq Y # Id , Y leq Z ==&#62; X leq Z pragma passive(Id).
</PRE>

Declaring the first head of rule <CODE>transitivity</CODE> passive changes the
behavior of the handler. It will propagate less depending on the order in
which the constraints arrive:

<PRE>
?- X leq Y, Y leq Z.
X leq Y,
Y leq Z,
X leq Z ? 

?- Y leq Z, X leq Y.
Y leq Z,
X leq Y ? 

?- Y leq Z, X leq Y, Z leq X.
Y = X,
Z = X ?
</PRE>

The last query shows that the handler is still complete in the sense
that all circular chains of leq-relations are collapsed into equalities.

</DL>



<H3><A NAME="SEC300" HREF="sicstus_toc.html#TOC300">Options</A></H3>

<P>
Options parametrise the rule compilation process.
Thus they should precede the rule definitions. Example:

<PRE>
option(check_guard_bindings, off).
</PRE>

<P>
The format below lists the names of the recognized options together with
the acceptable values. The first entry in the lists is the default
value.

</P>
<DL COMPACT>

<DT><CODE>option(debug_compile,         [off,on]).</CODE>
<DD>
Instruments the generated code such that the execution
of the rules may be traced (see section <A HREF="sicstus_34.html#SEC306">Debugging CHR Programs</A>).

<DT><CODE>option(check_guard_bindings,  [on,off]).</CODE>
<DD>
Per default, for guards of type ask the CHR runtime system makes sure
that no variables are touched or the cause of an instantiation
error. These checks may be turned off with this option, i.e. all guards
are treated as if they were of the tell variety. The option was
kept for backward compatibility. Tell and ask guards offer better
granularity.

<DT><CODE>option(already_in_store,      [off,on]).</CODE>
<DD>
If this option is on, the CHR runtime system checks for the presence of
an identical constraint upon the insertion into the store. If present,
the attempted insertion has no effect. Since checking for duplicates for
all constraints costs, duplicate removal specific to individual
constraints, using a few simpagation rules of the following form instead,
may be a better solution.

<PRE>
Constraint \ Constraint &#60;=&#62; true.
</PRE>

<DT><CODE>option(already_in_heads,      [off,on]).</CODE>
<DD>
The intention of simplification and simpagation rules is often
to combine the heads into a stronger version of one of them.
Depending on the strength of the guard, the new constraint may
be identical to one of the heads removed by the rule.
This removal followed by addition is inefficient and may even cause
termination problems. If the option is enabled, this situation is
detected and the corresponding problems are avoided.
This option applies to all constraints and is provided mainly
for backward compatibility. Better grained control can be achieved
with corresponding pragmas. (see section <A HREF="sicstus_34.html#SEC299">Pragmas</A>).

</DL>

<P>
The remaining options are meant for CHR implementors only:
<DL COMPACT>

<DT><CODE>option(flatten,               [on,off]).</CODE>
<DD>
<DT><CODE>option(rule_ordering,         [canonical,heuristic]).</CODE>
<DD>
<DT><CODE>option(simpagation_scheme,    [single,multi]).</CODE>
<DD>
<DT><CODE>option(revive_scheme,         [new,old]).</CODE>
<DD>
<DT><CODE>option(dead_code_elimination, [on,off]).</CODE>
<DD>
</DL>



<H3><A NAME="SEC301" HREF="sicstus_toc.html#TOC301">Built-In Predicates</A></H3>

<P>
This table lists the predicates made available by the CHR library.  They
are meant for advanced users, who want to tailor the CHR system 
towards their specific needs.
<DL COMPACT>

<DT><CODE>current_handler(?Handler, ?Module)</CODE>
<DD>
<A NAME="IDX1356"></A>
Nondeterministically enumerates the defined handlers
with the module they are defined in.

<DT><CODE>current_constraint(?Handler, ?Constraint)</CODE>
<DD>
<A NAME="IDX1357"></A>
Nondeterministically enumerates the defined constraints
in the form <VAR>Functor/Arity</VAR> and the handlers they are
defined in.

<DT><CODE>insert_constraint(+Constraint, -Id)</CODE>
<DD>
<A NAME="IDX1358"></A>
Inserts <VAR>Constraint</VAR> into the constraint store
without executing any rules. The constraint will be woken and reconsidered
when one of the variables in <VAR>Constraint</VAR> is touched. <VAR>Id</VAR> is
unified with an internal object representing the constraint.
This predicate only gets defined when a handler and constraints are
declared (see section <A HREF="sicstus_34.html#SEC296">Declarations</A>).

<DT><CODE>insert_constraint(+Constraint, -Id, ?Term)</CODE>
<DD>
<A NAME="IDX1359"></A>
Inserts <VAR>Constraint</VAR> into the constraint store without executing
any rules. The constraint will be woken and reconsidered when one of the
variables in <VAR>Term</VAR> is touched. <VAR>Id</VAR> is
unified with an internal object representing the constraint.
This predicate only gets defined when a handler and constraints are
declared (see section <A HREF="sicstus_34.html#SEC296">Declarations</A>).

<DT><CODE>find_constraint(?Pattern, -Id)</CODE>
<DD>
<A NAME="IDX1360"></A>
Nondeterministically enumerates constraints from the constraint
store that match <VAR>Pattern</VAR>, i.e. which are instances of
<VAR>Pattern</VAR>. <VAR>Id</VAR> is
unified with an internal object representing the constraint.

<DT><CODE>find_constraint(-Var, ?Pattern, -Id)</CODE>
<DD>
<A NAME="IDX1361"></A>
Nondeterministically enumerates constraints from the constraint
store that delay on <VAR>Var</VAR> and match <VAR>Pattern</VAR>, 
i.e. which are instances of
<VAR>Pattern</VAR>. The identifier <VAR>Id</VAR> can be used to
refer to the constraint later, e.g. for removal.

<DT><CODE>findall_constraints(?Pattern, ?List)</CODE>
<DD>
<A NAME="IDX1362"></A>
Unifies <VAR>List</VAR> with a list of <CODE>Constraint # Id</CODE> pairs
from the constraint store that match <VAR>Pattern</VAR>.

<DT><CODE>findall_constraints(-Var, ?Pattern, ?List)</CODE>
<DD>
<A NAME="IDX1363"></A>
Unifies <VAR>List</VAR> with a list of <CODE>Constraint # Id</CODE> pairs
from the constraint store that delay on <VAR>Var</VAR> and
match <VAR>Pattern</VAR>.

<DT><CODE>remove_constraint(+Id)</CODE>
<DD>
<A NAME="IDX1364"></A>
Removes the constraint <VAR>Id</VAR>, obtained with one of the previous predicates,
from the constraint store.

<DT><CODE>unconstrained(?Var)</CODE>
<DD>
<A NAME="IDX1365"></A>
Succeeds if no CHR constraint delays on <VAR>Var</VAR>. Defined as:

<PRE>
unconstrained(X) :- 
    find_constraint(X, _, _), !, fail.
unconstrained(_).
</PRE>

<DT><CODE>notify_constrained(?Var)</CODE>
<DD>
<A NAME="IDX1366"></A>
Leads to the reconsideration of the constraints associated with <VAR>Var</VAR>.
This mechanism allows solvers to communicate reductions on the set of
possible values of variables prior to making bindings.
</DL>



<H3><A NAME="SEC302" HREF="sicstus_toc.html#TOC302">Consulting and Compiling Constraint Handlers</A></H3>

<P>
The CHR compilation process has been made as transparent as possible.
The user deals with files containing CHR just as with files containing
ordinary Prolog predicates. Thus CHR may be consulted, compiled with
various compilation modes, and compiled to file (see section <A HREF="sicstus_8.html#SEC71">Loading Programs</A>).

</P>


<H3><A NAME="SEC303" HREF="sicstus_toc.html#TOC303">Compiler-generated Predicates</A></H3>

<P>
Besides predicates for the defined constraints,
the CHR compiler generates some support predicates in the
module containing the handler. To avoid naming conflicts,
the  following predicates must not be defined or referred to
by user code in the same module:
<DL COMPACT>

<DT><CODE>verify_attributes/3</CODE>
<DD>
<DT><CODE>attribute_goal/2</CODE>
<DD>
<DT><CODE>attach_increment/2</CODE>
<DD>
<DT><CODE>'attach_F/A'/2</CODE>
<DD>
for every defined constraint F/A.
<DT><CODE>'F/A_N_M_...'/Arity</CODE>
<DD>
for every defined constraint F/A. N,M is are integers, Arity &#62; A.
</DL>

<P>
For the prime number example that is:

<PRE>
        attach_increment/2
        attach_prime/1/2
        attach_primes/1/2
        attribute_goal/2
        goal_expansion/3
        prime/1
        prime/1_1/2
        prime/1_1_0/3
        prime/1_2/2
        primes/1
        primes/1_1/2
        verify_attributes/3
</PRE>

<P>
If an author of a handler wants to avoid naming conflicts with the
code that uses the handler, it is easy to encapsulate the handler.
The module declaration below puts the handler into module <CODE>primes</CODE>,
which exports only selected predicates - the constraints in our example.

<PRE>
        :- module(primes, [primes/1,prime/1]).

        :- use_module(library(chr)).

        handler eratosthenes.
        constraints primes/1,prime/1.
        ...
</PRE>



<H3><A NAME="SEC304" HREF="sicstus_toc.html#TOC304">Operator Declarations</A></H3>

<P>
This table lists the operators as used by the CHR library:

<PRE>
:- op(1200, xfx, @).
:- op(1190, xfx, pragma).
:- op(1180, xfx, [==&#62;,&#60;=&#62;]).
:- op(1180, fy,  chr_spy).
:- op(1180, fy,  chr_nospy).
:- op(1150, fx,  handler).
:- op(1150, fx,  constraints).
:- op(1150, fx,  rules).
:- op(1100, xfx, '|').
:- op(1100, xfx, \ ).
:- op(1050, xfx, &#38;).
:- op( 500, yfx, #).
</PRE>



<H3><A NAME="SEC305" HREF="sicstus_toc.html#TOC305">Exceptions</A></H3>

<P>
The CHR runtime system reports instantiation and type errors for the predicates:
<DL COMPACT>

<DT><CODE>find_constraint/2</CODE>
<DD>
<DT><CODE>findall_constraints/3</CODE>
<DD>
<DT><CODE>insert_constraint/2</CODE>
<DD>
<DT><CODE>remove_constraint/1</CODE>
<DD>
<DT><CODE>notify_constrained/1</CODE>
<DD>
</DL>

<P>
The only other CHR specific runtime error is:
<DL COMPACT>

<DT><CODE>{CHR ERROR: registering &#60;New&#62;, module &#60;Module&#62; already hosts &#60;Old&#62;}</CODE>
<DD>
An attempt to load a second handler New into module &#60;Module&#62; already 
hosting handler &#60;Old&#62; was made.
</DL>

<P>
The following exceptional conditions are detected by the CHR
compiler:

</P>
<DL COMPACT>

<DT><CODE>{CHR Compiler ERROR: syntax rule &#60;N&#62;: &#60;Term&#62;}</CODE>
<DD>
If the N-th &#60;Term&#62; in the file being loaded violates the CHR syntax (see section <A HREF="sicstus_34.html#SEC297">Constraint Handling Rules, Syntax</A>).

<DT><CODE>{CHR Compiler ERROR: too many general heads in &#60;Name&#62;}</CODE>
<DD>
Unspecific heads in definitions like
<CODE>C \ C &#60;=&#62; true</CODE>
must not be combined with other heads in rule &#60;Name&#62;.

<DT><CODE>{CHR Compiler ERROR: bad pragma &#60;Pragma&#62; in &#60;Name&#62;}</CODE>
<DD>
The pragma &#60;Pragma&#62; used in rule &#60;Name&#62; does not qualify. Currently this only happens if
&#60;Pragma&#62; is unbound.

<DT><CODE>{CHR Compiler ERROR: found head &#60;F/A&#62; in &#60;Name&#62;, expected one of: &#60;F/A list&#62;}</CODE>
<DD>
Rule &#60;Name&#62; has a head of given F/A which is not among the defined constraints.

<DT><CODE>{CHR Compiler ERROR: head identifiers in &#60;Name&#62; are not unique variables}</CODE>
<DD>
The identifiers to refer to individual constraints (heads) via <SAMP>`#'</SAMP> in
rule &#60;Name&#62; do not meet the indicated requirements.

<DT><CODE>{CHR Compiler ERROR: no handler defined}</CODE>
<DD>
CHR specific language elements, declarations or rules for example, are
used before a handler was defined. This error is usually reported a couple
of times, i.e. as often as there are CHR forms in the file expecting the missing
definition.

<DT><CODE>{CHR Compiler ERROR: compilation failed}</CODE>
<DD>
Not your fault. Send us a bug report.
</DL>



<H2><A NAME="SEC306" HREF="sicstus_toc.html#TOC306">Debugging CHR Programs</A></H2>

<P>
Use <CODE>option(debug_compile,on)</CODE> preceding any rules
in the file containing the handler to enable CHR debugging. 
The CHR debugging mechanism works by instrumenting the code
generated by the CHR compiler.
Basically, the CHR debugger works like the Prolog debugger. 
The main differences are: there are extra ports specific to CHR,
and the CHR debugger provides no means for the user to change
the flow of control, i.e. there are currently no <VAR>retry</VAR> and <VAR>fail</VAR>
options available.

</P>

<UL>
<LI><A HREF="sicstus_34.html#SEC307">CHR Control Flow Model</A>
<LI><A HREF="sicstus_34.html#SEC308">CHR Debugging Predicates</A>
<LI><A HREF="sicstus_34.html#SEC309">CHR Spy-points</A>
<LI><A HREF="sicstus_34.html#SEC310">CHR Debugging Messages</A>
<LI><A HREF="sicstus_34.html#SEC311">CHR Debugging Options</A>
</UL>



<H3><A NAME="SEC307" HREF="sicstus_toc.html#TOC307">Control Flow Model</A></H3>
<P>
<A NAME="IDX1367"></A>

</P>
<P>
The entities reflected by the CHR debugger are constraints
and rules. Constraints are treated like ordinary Prolog goals
with the usual ports: <CODE>[call,exit,redo,fail]</CODE>.
In addition, constraints may get inserted into or removed from the 
constraint store (ports: <CODE>insert,remove</CODE>), and stored
constraints containing variables will be woken and reconsidered
(port: <CODE>wake</CODE>) when variables are touched.

</P>
<P>
The execution of a constraint consists of trying to apply the rules
mentioning the constraint in their heads.  Two ports for rules reflect this
process: At a <CODE>try</CODE> port the active constraint matches one of the
heads of the rule, and matching constraints for the remaining heads of
the rule, if any, have been found as well.  The transition from a
<CODE>try</CODE> port to an <CODE>apply</CODE> port takes place when the guard has
been successfully evaluated, i.e. when the rule commits. At the
<CODE>apply</CODE> port, the body of the rule is just about to be executed. The
body is a Prolog goal transparent to the CHR debugger. If the
rule body contains CHR constraints, the CHR debugger will track
them again.  If the rules were consulted, the Prolog debugger can be
used to study the evaluations of the other predicates in the body.

</P>


<H3><A NAME="SEC308" HREF="sicstus_toc.html#TOC308">CHR Debugging Predicates</A></H3>
<P>
<A NAME="IDX1368"></A>

</P>
<P>
The following predicates control the operation of the CHR debugger:

</P>
<DL COMPACT>

<DT><CODE>chr_trace</CODE>
<DD>
<A NAME="IDX1369"></A>
Switches the CHR debugger on and ensures that the
next time control enters a CHR port, a message will be produced and
you will be asked to interact. 
At this point you have a number of options.  See section <A HREF="sicstus_34.html#SEC311">CHR Debugging Options</A>.  In
particular, you can just type <KBD>cr</KBD> (Return) to <EM>creep</EM> (or single-step)
into your program. You will
notice that the CHR debugger stops at many ports.  If this is not
what you want, the predicate <CODE>chr_leash</CODE> gives full control over
the ports at which you are prompted. 
<DT><CODE>chr_debug</CODE>
<DD>
<A NAME="IDX1370"></A>
Switches the CHR debugger on and ensures that the next time
control enters a CHR port with a spy-point set, a message will be produced
and you will be asked to interact. 
<DT><CODE>chr_nodebug</CODE>
<DD>
<A NAME="IDX1371"></A>
Switches the CHR debugger off.  If there are any spy-points set then they
will be kept.  
<DT><CODE>chr_notrace</CODE>
<DD>
<A NAME="IDX1372"></A>
Equivalent to <CODE>chr_nodebug</CODE>.  
<DT><CODE>chr_debugging</CODE>
<DD>
<A NAME="IDX1373"></A>
Prints onto the standard error stream information about the current
CHR debugging state.  This will show: 

<OL>
<LI>

Whether the CHR debugger is switched on.
<LI>

What spy-points have been set (see below).
<LI>

What mode of leashing is in force (see below).
</OL>

<DT><CODE>chr_leash(<VAR>+Mode</VAR>)</CODE>
<DD>
<A NAME="IDX1374"></A>
The leashing mode is set to <VAR>Mode</VAR>. It determines the CHR ports at
which you are to be prompted when you <EM>creep</EM> through your program.
At unleashed ports a tracing message is still output, but program
execution does not stop to allow user interaction.  Note that the ports
of spy-points are always leashed (and cannot be unleashed).  <VAR>Mode</VAR>
is a list containing none, one or more of the following port names:

<DL COMPACT>

<DT><CODE>call</CODE>
<DD>
Prompt when a constraint is executed for the first time.

<DT><CODE>exit</CODE>
<DD>
Prompt when the constraint is successfully processed, i.e. the applicable
rules have applied.

<DT><CODE>redo</CODE>
<DD>
Prompt at subsequent exits generated by non-deterministic rule bodies.

<DT><CODE>fail</CODE>
<DD>
Prompt when a constraint fails.

<DT><CODE>wake</CODE>
<DD>
Prompt when a constraint from the constraint store is woken and
reconsidered because one of its variables has been touched.

<DT><CODE>try</CODE>
<DD>
Prompt just before the guard evaluation of a rule, after constraints
matching the heads have been found.

<DT><CODE>apply</CODE>
<DD>
Prompt upon the application of a rule, after the successful guard evaluation,
when the rule commits and fires, just before evaluating the body.

<DT><CODE>insert</CODE>
<DD>
Prompt when a constraint gets inserted into the constraint store,
i.e. after all rules have been tried.

<DT><CODE>remove</CODE>
<DD>
Prompt when a constraint gets removed from the constraint store, e.g.
when a simplification rule applies.

</DL>

The initial value of the CHR leashing mode is
<CODE>[call,exit,fail,wake,apply]</CODE>. Predefined shortcuts are:
<DL COMPACT>

<DT><CODE>chr_leash(none), chr_leash(off)</CODE>
<DD>
To turn leashing off.

<DT><CODE>chr_leash(all)</CODE>
<DD>
To prompt at every port.

<DT><CODE>chr_leash(default)</CODE>
<DD>
Same as <CODE>chr_leash([call,exit,fail,wake,apply])</CODE>.

<DT><CODE>chr_leash(call)</CODE>
<DD>
No need to use a list if only a singular port is to be leashed.
</DL>

</DL>



<H3><A NAME="SEC309" HREF="sicstus_toc.html#TOC309">CHR Spy-points</A></H3>
<P>
<A NAME="IDX1375"></A>

</P>
<P>
For CHR programs of any size, it is clearly impractical to creep through the
entire program.  <EM>Spy-points</EM> make it possible to stop the program
upon an event of interest.  Once
there, one can set further spy-points in order to catch the control flow
a bit further on, or one can start creeping.

</P>
<P>
Setting a spy-point on a constraint or a rule indicates that you wish to
see all control flow through the various ports involved, except during
skips.  When control passes through any port with a spy-point set on it,
a message is output and the user is asked to interact.  Note that the
current mode of leashing does not affect spy-points: user interaction is
requested on <EM>every</EM> port.

</P>
<P>
Spy-points are set and removed by the following predicates,
which are declared as prefix operators:

</P>
<DL COMPACT>

<DT><CODE>chr_spy <VAR>Spec</VAR></CODE>
<DD>
<A NAME="IDX1376"></A>
Sets spy-points on constraints and rules given by <VAR>Spec</VAR>, which is
is of the form:

<DL COMPACT>

<DT><VAR>_ (variable)</VAR>
<DD>
denoting all constraints and rules, or:

<DT><VAR>constraints Cs</VAR>
<DD>
where <VAR>Cs</VAR> is one of
<DL COMPACT>

<DT><VAR>_ (variable)</VAR>
<DD>
denoting all constraints

<DT><VAR>C,...,C</VAR>
<DD>
denoting a list of constraints <VAR>C</VAR>

<DT><VAR>Name</VAR>
<DD>
denoting all constraints with this functor, regardless of arity

<DT><VAR>Name/Arity</VAR>
<DD>
denoting the constraint of that name and arity
</DL>

<DT><VAR>rules Rs</VAR>
<DD>
where <VAR>Rs</VAR> is one of:
<DL COMPACT>

<DT><VAR>_ (variable)</VAR>
<DD>
denoting all rules

<DT><VAR>R,...,R</VAR>
<DD>
denoting a list of rules <VAR>R</VAR>

<DT><VAR>Name</VAR>
<DD>
where <VAR>Name</VAR> is the name of a rule in any handler.

<DT><VAR>already_in_store</VAR>
<DD>
The name of a rule implicitly defined by the system when
the option <CODE>already_in_store</CODE> is in effect.

<DT><VAR>already_in_heads</VAR>
<DD>
The name of a rule implicitly defined by the system when
the option <CODE>already_in_heads</CODE> or the corresponding 
pragmas are in effect.

<DT><VAR>Handler:Name</VAR>
<DD>
where <VAR>Handler</VAR> is the name of a constraint handler
and <VAR>Name</VAR> is the name of a rule in that handler

</DL>
</DL>

Examples:


<PRE>
| ?- chr_spy rules rule(3), transitivity, already_in_store.
| ?- chr_spy constraints prime/1.
</PRE>

If you set spy-points, the CHR debugger will be switched on.

<DT><CODE>chr_nospy <VAR>Spec</VAR></CODE>
<DD>
<A NAME="IDX1377"></A>
Removes spy-points on constraints and rules given by <VAR>Spec</VAR>, where
<VAR>Spec</VAR> is of the form as described for <CODE>chr_spy <VAR>Spec</VAR></CODE>.
There is no <CODE>chr_nospyall/0</CODE>.  To remove all CHR spy-points use
<CODE>chr_nospy _</CODE>.

</DL>

<P>
The options available when you arrive at a spy-point are described later.
See section <A HREF="sicstus_34.html#SEC311">CHR Debugging Options</A>.

</P>


<H3><A NAME="SEC310" HREF="sicstus_toc.html#TOC310">CHR Debugging Messages</A></H3>
<P>
<A NAME="IDX1378"></A>

</P>
<P>
All trace messages are output to the standard error
stream.  This allows you to trace programs while they are performing
file I/O.  The basic format is as follows:

</P>

<PRE>
<VAR>S</VAR> 3   1 try     eratosthenes:absorb(10) @ prime(9)#&#60;c4&#62;, prime(10)#&#60;c2&#62; ?
</PRE>

<P>
<VAR>S</VAR> is a spy-point indicator. It is printed as <SAMP>` '</SAMP> if there is 
no spy-point, as <SAMP>`r'</SAMP>, indicating that there is a spy-point on this
rule, or as <SAMP>`c'</SAMP> if one of the involved constraints has a spy-point.

</P>
<P>
The first number indicates the current depth of the execution; i.e.
the number of direct <EM>ancestors</EM> the currently active constraint
has.

</P>
<P>
The second number indicates the head position of the currently active
constraint at rule ports.

</P>
<P>
The next item tells you which port is currently traced.

</P>
<P>
A constraint or a matching rule are printed next.
Constraints print as <CODE>Term#Id</CODE>, where <VAR>Id</VAR> is a unique
identifier pointing into the constraint store.
Rules are printed as <CODE>Handler:Name @</CODE>, followed by the constraints
matching the heads.

</P>
<P>
The final <SAMP>`?'</SAMP> is the prompt indicating that you should type in one of the
debug options (see section <A HREF="sicstus_34.html#SEC311">CHR Debugging Options</A>). 
</P>


<H3><A NAME="SEC311" HREF="sicstus_toc.html#TOC311">CHR Debugging Options</A></H3>
<P>
<A NAME="IDX1379"></A>

</P>
<P>
This section describes the options available when the system prompts you
after printing out a debugging message.  Most of them you know from the
standard Prolog debugger. All the options are one letter mnemonics, some
of which can be optionally followed by a decimal integer.  They are read
from the standard input stream up to the end of the line
(Return, <KBD>&#60;cr&#62;</KBD>). Blanks will be ignored.

</P>
<P>
The only option which you really have to remember is <SAMP>`h'</SAMP>.  This
provides help in the form of the following list of available options.

</P>

<PRE>
CHR debugging options:
   &#60;cr&#62;   creep            c      creep
    l     leap                         
    s     skip             s &#60;i&#62;  skip (ancestor i)
    g     ancestors                   
    &#38;     constraints      &#38; &#60;i&#62;  constraints (details)
    n     nodebug          =      debugging
    +     spy this                    
    -     nospy this       .      show rule
    &#60;     reset printdepth &#60; &#60;n&#62;  set printdepth
    a     abort            b      break
    ?     help             h      help
</PRE>

<DL COMPACT>

<DT><KBD>c</KBD>
<DD>
<DT><KBD><KBD>&#60;cr&#62;</KBD></KBD>
<DD>
<EM>creep</EM> causes the debugger to single-step to the very next port
and print a message.  Then if the port is leashed, the
user is prompted for further interaction.  Otherwise, it continues creeping.
If leashing is off, creep is the same as <EM>leap</EM> (see below) except that a
complete trace is printed on the standard error stream.  
<DT><KBD>l</KBD>
<DD>
<EM>leap</EM> causes the debugger to resume running your program, only
stopping when a spy-point is reached (or when the program terminates).
Leaping can thus be used to follow the execution at a higher level than
exhaustive tracing. 
<DT><KBD>s</KBD>
<DD>
<DT><KBD>s i</KBD>
<DD>
<EM>skip</EM> over the entire execution of the constraint.  That is, you
will not see anything until control comes back to this constraint (at
either the <CODE>exit</CODE> port or the <CODE>fail</CODE> port). This includes
ports with spy-points set; they will be masked out during the skip.
The command can be used with a numeric argument to skip the execution
up to and including the ancestor indicated by the argument.
Example:

<PRE>
      ...
      4   - exit    prime(8)#&#60;c6&#62; ? g
Ancestors:
      1   1 apply   eratosthenes:rule(2) @ primes(10)#&#60;c1&#62;
      2   1 apply   eratosthenes:rule(2) @ primes(9)#&#60;c3&#62;
      3   1 apply   eratosthenes:rule(2) @ primes(8)#&#60;c5&#62;
      4   - call    prime(8)#&#60;c6&#62;

      4   - exit    prime(8)#&#60;c6&#62; ? s 2
      2   - exit    primes(9)#&#60;c3&#62; ? 
</PRE>

<DT><KBD>g</KBD>
<DD>
<EM>print ancestors</EM> provides you with a list of ancestors to the
currently active constraint, i.e. all constraints not yet exited that led 
to the current constraint in the derivation sequence.
The format is the same as with trace messages. 
Constraints start with <CODE>call</CODE> entries in the stack.
The subsequent application of a rule replaces the call entry
in the stack with an <CODE>apply</CODE> entry. Later the constraint
shows again as <CODE>redo</CODE> or <CODE>fail</CODE> entry.
Example:

<PRE>
      0   - call    primes(10)#&#60;c1&#62; ? 
      1   1 try     eratosthenes:rule(2) @ primes(10)#&#60;c1&#62; ? g

Ancestors:
      1   - call    primes(10)#&#60;c1&#62;

      1   1 try     eratosthenes:rule(2) @ primes(10)#&#60;c1&#62; ? 
      1   1 apply   eratosthenes:rule(2) @ primes(10)#&#60;c1&#62; ? 
      1   - call    prime(10)#&#60;c2&#62; ? 
      2   - insert  prime(10)#&#60;c2&#62;
      2   - exit    prime(10)#&#60;c2&#62; ? g

Ancestors:
      1   1 apply   eratosthenes:rule(2) @ primes(10)#&#60;c1&#62;
      2   - call    prime(10)#&#60;c2&#62;
</PRE>

<DT><KBD>&#38;</KBD>
<DD>
<EM>print constraints</EM> prints a list of the constraints in the
constraint store. With a numeric argument, details relevant primarily to
CHR implementors are shown. 
<DT><KBD>n</KBD>
<DD>
<EM>nodebug</EM> switches the CHR debugger off. 
<DT><KBD>=</KBD>
<DD>
<EM>debugging</EM> outputs information concerning the status of the CHR debugger
as via <CODE>chr_debugging/0</CODE>

<DT><KBD>+</KBD>
<DD>
<EM>spy this</EM> sets a spy-point on the current constraint or rule.

<DT><KBD>-</KBD>
<DD>
<EM>nospy this</EM> removes the spy-point from the current constraint or
rule, if it exists.

<DT><KBD>.</KBD>
<DD>
<EM>show rule</EM>
prints the current rule instantiated by the matched constraints. Example:

<PRE>
 8   1 apply   era:absorb(8) @ prime(4)#&#60;c14&#62; \ prime(8)#&#60;c6&#62; ? .

 absorb(8) @
  prime(4)#&#60;c14&#62; \ 
    prime(8)#&#60;c6&#62; &#60;=&#62;

    8 mod 4=:=0
    |
    true.
</PRE>

<DT><KBD>&#60;</KBD>
<DD>
<DT><KBD>&#60; n</KBD>
<DD>
While in the debugger, a <EM>printdepth</EM> is in effect for limiting
the subterm nesting level when printing rules and constraints.
The limit is initially 10.  This command, without arguments, resets the
limit to 10.  With an argument of <VAR>n</VAR>, the limit is set to <VAR>n</VAR>,
treating 0 as infinity.

<DT><KBD>a</KBD>
<DD>
<EM>abort</EM> calls the built-in predicate <CODE>abort/0</CODE>.

<DT><KBD>b</KBD>
<DD>
<EM>break</EM> calls the built-in predicate <CODE>break/0</CODE>, thus putting
you at a recursive top-level.  When you end the break 
(entering ^D) you will be
re-prompted at the port at which you broke. The CHR debugger is
temporarily switched off as you call the break and will be switched
on again when you finish the break and go back to the old execution.
Any changes to the CHR leashing or to spy-points during the break
will remain in effect.

<DT><KBD>?</KBD>
<DD>
<DT><KBD>h</KBD>
<DD>
<EM>help</EM> displays the table of options given above.  
</DL>



<H2><A NAME="SEC312" HREF="sicstus_toc.html#TOC312">Programming Hints</A></H2>

<P>
This section gives you some programming hints for CHR. For maximum
efficiency of your constraint handler, see also the previous subsections
on declarations and options.

</P>
<P>
Constraint handling rules for a given constraint system can often be
derived from its definition in formalisms such as inference rules,
rewrite rules, sequents, formulas expressing axioms and theorems.
CHR can also be found by first considering
special cases of each constraint and then looking at interactions of
pairs of constraints sharing a variable. Cases that do not occur in the
application can be ignored.

</P>
<P>
It is important to find the right <EM>granularity</EM> of the constraints.
Assume one wants to express that <EM>n</EM> variables are different from
each other.  It is more efficient to have a single constraint
<CODE>all_different(List_of_n_Vars)</CODE> than <EM>n*n</EM> inequality
constraints between each pair of different variables.  However, the
extreme case of having a single constraint modeling the whole constraint
store will usually be inefficient.

</P>
<P>
Starting from an executable specification, the rules can then be refined
and adapted to the specifics of the application.  Efficiency can be
improved by weakening the guards to perform simplification as early as
needed and by strengthening the guards to do the <EM>just right</EM> amount
of propagation.  Propagation rules can be expensive, because no
constraints are removed. 

</P>
<P>
The more heads a rule has, the more expensive it is.  <EM>Rules with
several heads</EM> are more efficient, if the heads of the rule share a
variable (which is usually the case). Then the search for a partner
constraint has to consider less candidates.  In the current
implementation, constraints are indexed by their functors, so that the
search is only performed among the constraints containing the shared
variable. Moreover, two rules with identical
(or sufficiently similar) heads can be merged into one rule so that the
search for a partner constraint is only performed once instead of twice.

</P>
<P>
As <EM>guards</EM> are tried frequently, they should be simple <EM>tests</EM>
not involving side-effects.  Head matching is more efficient than
explicitly checking equalities in the ask-part of the guard.  In the
tell part of a guard, it should be made sure that variables from the
head are never touched (e.g. by using <CODE>nonvar</CODE> or <CODE>ground</CODE> if
necessary). For efficiency and clarity reasons, one should also avoid
using constraints in guards.  Besides conjunctions, disjunctions are
allowed in the guard, but they should be used with care.  The use of
other control built-in predicates in the guard is
discouraged.  Negation and if-then-else in the ask part of a guard can
give wrong results, since e.g. failure of the negated goal may be due to
touching its variables.

</P>
<P>
<EM>Several handlers can be used simultaneously if</EM> they do not share
constraints with the same name. The implementation will not work
correctly if the same constraint is defined in rules of different
handlers that have been compiled separately. In such a case, the
handlers must be merged <EM>by hand</EM>. This means that the source code
has to be edited so that the rules for the shared constraint are
together (in one module). Changes may be necessary (like
strengthening guards) to avoid divergence or loops in the computation.

</P>



<H2><A NAME="SEC313" HREF="sicstus_toc.html#TOC313">Constraint Handlers</A></H2>

<P>
The CHR library comes with plenty of constraint handlers written in CHR.
The most recent versions of these are maintained at:

</P>

<PRE>
http://www.pst.informatik.uni-muenchen.de/~fruehwir/chr-solver.html
</PRE>

<DL COMPACT>

<DT><TT>`arc.pl'</TT>
<DD>
classical arc-consistency over finite domains

<DT><TT>`bool.pl'</TT>
<DD>
simple Boolean constraints

<DT><TT>`cft.pl'</TT>
<DD>
feature term constraints according to the CFT theory

<DT><TT>`domain.pl'</TT>
<DD>
finite domains over arbitrary ground terms and interval domains over integers and reals, but without arithmetic functions

<DT><TT>`gcd.pl'</TT>
<DD>
elegant two-liner for the greatest common divisor

<DT><TT>`interval.pl'</TT>
<DD>
straightforward interval domains over integers and reals, with arithmetic functions

<DT><TT>`kl-one.pl'</TT>
<DD>
terminological reasoning similar to KL-ONE or feature trees

<DT><TT>`leq.pl'</TT>
<DD>
standard introductory CHR example handler for less-than-or-equal

<DT><TT>`list.pl'</TT>
<DD>
equality constraints over concatenations of lists (or strings)

<DT><TT>`listdom.pl'</TT>
<DD>
a straightforward finite enumeration list domains over integers, similar to <TT>`interval.pl'</TT>

<DT><TT>`math-elim.pl'</TT>
<DD>
solves linear polynomial equations and inequations using variable elimination, several variations possible

<DT><TT>`math-fougau.pl'</TT>
<DD>
solves linear polynomial equations and inequations by combining variable elimination for equations with Fourier's algorithm for inequations, several variations possible

<DT><TT>`math-fourier.pl'</TT>
<DD>
a straightforward Fouriers algorithm to solve polynomial inequations
over the real or rational numbers

<DT><TT>`math-gauss.pl'</TT>
<DD>
a straightforward, elegant implementation of variable elimination for equations in one rule

<DT><TT>`minmax.pl'</TT>
<DD>
simple less-than and less-than-or-equal ordering constraints together with minimum and maximum constraints

<DT><TT>`modelgenerator.pl'</TT>
<DD>
example of how to use CHR for model generation in theorem proving

<DT><TT>`monkey.pl'</TT>
<DD>
classical monkey and banana problem, illustrates how CHR can be used as
a fairly efficient production rule system

<DT><TT>`osf.pl'</TT>
<DD>
constraints over order sorted feature terms according to the OSF theory

<DT><TT>`oztype.pl'</TT>
<DD>
rational trees with disequality and OZ type constraint with intersection

<DT><TT>`pathc.pl'</TT>
<DD>
the most simple example of a handler for path consistency - two rules

<DT><TT>`primes.pl'</TT>
<DD>
elegant implementations of the sieve of Eratosthenes reminiscent of the
chemical abstract machine model, also illustrates use of CHR as a
general purpose concurrent constraint language

<DT><TT>`scheduling.pl'</TT>
<DD>
simple classical constraint logic programming scheduling example on building a house

<DT><TT>`tarski.pl'</TT>
<DD>
most of Tarski's axiomatization of geometry as constraint system

<DT><TT>`term.pl'</TT>
<DD>
Prolog term manipulation built-in predicates <CODE>functor/3, arg/3, =../2</CODE> as constraints

<DT><TT>`time-pc.pl'</TT>
<DD>
grand generic handler for path-consistency over arbitrary constraints,
load via <TT>`time.pl'</TT> to get a powerful solver for temporal constraints based on Meiri's unifying framework. <TT>`time-rnd.pl'</TT> contains a generator for random test problems.

<DT><TT>`time-point.pl'</TT>
<DD>
quantitative temporal constraints over time points using path-consistency

<DT><TT>`tree.pl'</TT>
<DD>
equality and disequality over finite and infinite trees (terms)

<DT><TT>`type.pl'</TT>
<DD>
equalities and type constraints over finite and infinite trees (terms)

</DL>

<P>
You can consult or compile a constraint handler from the CHR library using
e.g.: 

<PRE>
?- [library('chr/examples/gcd')].
?- compile(library('chr/examples/gcd')).
</PRE>

<P>
If you want to learn more about the handlers, 
look at their documented source code.

</P>
<P>
In addition, there are files with example queries for some handlers,
their file name starts with <TT>`examples-'</TT> and the file extension
indicates the handler, e.g. <TT>`.bool'</TT>:

</P>

<PRE>
examples-adder.bool
examples-benchmark.math
examples-deussen.bool
examples-diaz.bool
examples-fourier.math
examples-holzbaur.math
examples-lim1.math
examples-lim2.math
examples-lim3.math
examples-puzzle.bool
examples-queens.bool
examples-queens.domain
examples-stuckey.math
examples-thom.math
</PRE>



<H2><A NAME="SEC314" HREF="sicstus_toc.html#TOC314">Backward Compatibility</A></H2>

<P>
In this section, we discuss backward compatibility with the CHR library
of Eclipse Prolog.

</P>

<OL>

<LI>

The restriction on at most two heads in a rule has been abandoned.  A
rule can have as many heads as you like. Note however, that searching
for partner constraints can be expensive.

<LI>

By default, rules are compiled in textual order. This gives the
programmer more control over the constraint handling process. In the
Eclipse library of CHR, the compiler was optimizing the order of
rules. Therefore, when porting a handler, rules may have to
be reordered. A good heuristic is to prefer simplification to
simpagation and propagation and to prefer rules with single heads to
rules with several heads. Instead of manually rearranging an old handler
one may also use the following combination of options to get the
corresponding effect:

<PRE>
option(rule_ordering,heuristic).
option(revive_scheme,old).
</PRE>

<LI>

For backward compatibility, the <CODE>already_in_store</CODE>,
<CODE>already_in_head</CODE> and <CODE>guard_bindings</CODE> options
are still around, but there are CHR syntax extensions: section <A HREF="sicstus_34.html#SEC297">Constraint Handling Rules, Syntax</A> 
and pragmas section <A HREF="sicstus_34.html#SEC299">Pragmas</A>
offering better grained control.

<LI>

The Eclipse library of CHR provided automatic built-in labeling
through the <CODE>label_with</CODE> declaration. Since it was not widely used
and can be easily simulated, built-in labeling was dropped.  
The same effect can be achieved
by replacing the declaration <CODE>label_with Constraint
if Guard</CODE> by the simplification rule <CODE>chr_labeling, Constraint &#60;=&#62;
Guard | Constraint', chr_labeling</CODE> and by renaming the head in each
clause <CODE>Constraint :- Body</CODE> into <CODE>Constraint' :- Body</CODE> where
<CODE>Constraint'</CODE> is a new predicate.  Efficiency can be improved by
declaring <CODE>Constraint</CODE> to be passive: <CODE>chr_labeling,
Constraint#Id &#60;=&#62; Guard | Constraint', chr_labeling pragma passive(Id)</CODE>.
This translation will not work if <CODE>option(already_in_heads,on)</CODE>.
In that case use e.g. <CODE>chr_labeling(_), Constraint &#60;=&#62; Guard |
Constraint', chr_labeling(_)</CODE> to make the new call to
<CODE>chr_labeling</CODE> differ from the head occurrence.

<LI>

The set of built-in predicates for advanced CHR users is now larger
and better designed. Also the debugger has been improved. The Opium
debugging environment is not available in SICStus Prolog.
</OL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_33.html">previous</A>, <A HREF="sicstus_35.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
