<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Prolog Objects</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_34.html">previous</A>, <A HREF="sicstus_36.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC315" HREF="sicstus_toc.html#TOC315">Prolog Objects</A></H1>

<P>
<A NAME="IDX1380"></A>
<A NAME="IDX1381"></A>
<A NAME="IDX1382"></A>
<A NAME="IDX1383"></A>
<A NAME="IDX1384"></A>

</P>
<P>
Prolog Objects is an extension to SICStus Prolog for flexible structuring,
sharing and reuse of knowledge in large logic programming applications. 
Prolog Objects enhances Prolog with an expressive and efficient 
object-oriented programming component.

</P>
<P>
Prolog Objects is based on the notion of prototypes.  In object-oriented
programming a <EM>prototype</EM> is an object that represents a typical behavior
of a certain concept.  A prototype can be used as is or as a model to
construct other objects that share some of the characteristics of the
prototypical object.  These specialized objects can themselves become
prototypes used to construct other objects and so forth.  The basic mechanism
for sharing is by <EM>inheritance</EM> and <EM>delegation</EM>.  Inheritance is
known for most readers. By using the delegation mechanism an object can
forward a message to another object to invoke a method defined by the
recipient but interpreted in the context of the sender.

</P>
<P>
<A NAME="IDX1385"></A>
<A NAME="IDX1386"></A>
<A NAME="IDX1387"></A>
<A NAME="IDX1388"></A>
<A NAME="IDX1389"></A>
<A NAME="IDX1390"></A>
<A NAME="IDX1391"></A>
<A NAME="IDX1392"></A>
<A NAME="IDX1393"></A>

</P>
<P>
In Prolog Objects, an <EM>object</EM> is a named collection of predicate
definitions.  In this sense an object is similar to a Prolog module.  The
object system can be seen as an extension of SICStus Prolog's module system.
In addition an object may have attributes that are modifiable.  Predicate
definitions belonging to an object are called <EM>methods</EM>.  So, an object is
conceptually a named collection of methods and attributes.  Some of the
methods defined for an object need not be stored explicitly within the object,
but are rather shared with other objects by the inheritance mechanism.

</P>
<P>
The Object system allows objects to be defined in a file, or dynamically
created during the execution of a program.  Objects defined in a file are
integrated into SICStus Prolog in a way similar to definite clause grammars.
That is to say, objects have a specific syntax as Prolog terms, and can be
loaded and expanded into Prolog code.  When an object is created, during
load-time, or run-time, it inherits the methods and attributes of its
prototypical object(s).  Objects defined in a file can be either <EM>static</EM>
or <EM>dynamic</EM>.  Also, methods can be either dynamic or static. these
properties are inherited by sub-objects.  Objects created during execution are
dynamic. 

</P>
<P>
The inheritance mechanism is implemented using the importation mechanism of
the module system.  The default inheritance is an <EM>inheritance by
overriding</EM> mechanism, which means that if a method is defined locally, and
the same method is defined in a super-object, then the clauses of the
super-method are not part of the definition of the local one.  As usual in
Prolog, methods can be non-determinately defined, and alternative answers can
be retrieved through backtracking.  Using the delegation mechanism, other
methods for knowledge sharing can be implemented by the user.  In 
Objects, there is an initial prototypical proto-object called <CODE>object</CODE>,
from which other objects may be constructed, directly or indirectly.

</P>

<UL>
<LI><A HREF="sicstus_35.html#SEC316">Obj Getting Started</A>: Getting Started.
<LI><A HREF="sicstus_35.html#SEC317">Obj Declared Objects</A>
<LI><A HREF="sicstus_35.html#SEC321">Obj Self</A>
<LI><A HREF="sicstus_35.html#SEC322">Obj Hierarchies</A>: Object Hierarchies, Inheritance, and Modules.
<LI><A HREF="sicstus_35.html#SEC332">Obj Dynamic</A>: Creating Objects Dynamically.
<LI><A HREF="sicstus_35.html#SEC336">Obj Daemons</A>: Access Driven Programming--Daemons.
<LI><A HREF="sicstus_35.html#SEC337">Instances</A>
<LI><A HREF="sicstus_35.html#SEC338">Obj Built-In</A>: Built-In Objects and Methods.
<LI><A HREF="sicstus_35.html#SEC343">Obj Expand</A>: Expansion to Prolog Code.
<LI><A HREF="sicstus_35.html#SEC350">Obj Examples</A>
</UL>



<H2><A NAME="SEC316" HREF="sicstus_toc.html#TOC316">Getting Started</A></H2>

<P>
To load the Prolog Objects library, enter the query:

</P>

<PRE>
| ?- use_module(library(objects)).
</PRE>

<P>
Prolog Objects defines some new infix and prefix operators, and redefines some
of the built-in ones.  The following operators become installed:

</P>

<PRE>
:- op(1200, xfy, [ &#38; ]).
:- op(1198, xfx, [ :- ]).
:- op(1198,  fx, [ :- ]).
:- op(550,  xfx, [ ::, &#60;: ]).
:- op(550,   fx, [ ::, &#60;: ]).
</PRE>



<H2><A NAME="SEC317" HREF="sicstus_toc.html#TOC317">Declared Objects</A></H2>
<P>
Declared objects are created when the files defining them are loaded into the
system.

</P>

<UL>
<LI><A HREF="sicstus_35.html#SEC318">Object Declaration</A>
<LI><A HREF="sicstus_35.html#SEC319">Method Declarations</A>
<LI><A HREF="sicstus_35.html#SEC320">Generic Objects for Easy Reuse</A>
</UL>



<H3><A NAME="SEC318" HREF="sicstus_toc.html#TOC318">Object Declaration</A></H3>
<P>
An object <VAR>object-identifier</VAR> is declared by writing it in the
following form:

</P>

<PRE>
<VAR>object-identifier</VAR> :: {
        <VAR>sentence-1</VAR> &#38;
        <VAR>sentence-2</VAR> &#38;
        :
        <VAR>sentence-n</VAR>
        }.
</PRE>

<P>
where <VAR>object-identifier</VAR> is a Prolog term that is either an
atom or a compound term of the form <VAR>functor(V1,...,Vn)</VAR>, where
<VAR>V1,...,Vn</VAR> are distinct variables.  The object body consists of a number
of <EM>sentences</EM>, possibly none, surrounded by braces, where each sentence
is either a <EM>method-directive</EM>, to be executed when the object is created,
or a <EM>method-clause</EM>.  A <EM>method</EM> is a number of method-clauses with
the same principal functor.  A method-clause has a clausal syntax similar to
that of Prolog, but instead of usual predicate calls in the body of a clause
there are <EM>method-calls</EM>. Ordinary Prolog goals are also allowed in a
prefixed form, using ':' as a prefix.  A method-directive is a directive which
contains method-calls.

</P>
<P>
All sentences are subject to term expansion (see section <A HREF="sicstus_10.html#SEC95">Term and Goal Expansion</A>, built-in
<CODE>expand_term/2</CODE>) before further processing, so in particular
definite clause grammar syntax can be used in method-clauses. In
addition, before <CODE>expand_term/2</CODE>, sentences are expanded by the
predicate <CODE>user:method_expansion/3</CODE>.

</P>
<DL COMPACT>

<DT><CODE>method_expansion(<VAR>+Term1</VAR>,<VAR>+ObjectIdentifier</VAR>,<VAR>?Term2</VAR>)</CODE>
<DD>
<DT><CODE>user:method_expansion(<VAR>+Term1</VAR>,<VAR>+ObjectIdentifier</VAR>,<VAR>?Term2</VAR>)</CODE>
<DD>
<A NAME="IDX1394"></A>
<A NAME="IDX1395"></A>
<EM>A hook predicate</EM>, which defines transformations on methods
similarly as <CODE>user:term_expansion/(2,4)</CODE>. At the end of an object
definition, <CODE>user:method_expansion/3</CODE> is called with
<CODE>end_of_object</CODE>.

</DL>



<H3><A NAME="SEC319" HREF="sicstus_toc.html#TOC319">Method Declarations</A></H3>

<P>
Method-clauses are declared similarly to Prolog clauses.  Thus a
method-clause can be either a unit-clause or a non-unit-clause.  We also
allow a default catch-all method-clause as the last clause in an object
body.  The catch-all clause has as its head a Prolog variable, in order
to match messages that are not previously defined or inherited in the
object. It can be used to implement alternative inheritance mechanisms.

</P>
<P>
Goals in the body of a non-unit clause have the normal control
structures of Prolog:

</P>
<DL COMPACT>

<DT><CODE><VAR>:P</VAR>, <VAR>:Q</VAR></CODE>
<DD>
Conjunction
<DT><CODE><VAR>:P</VAR>; <VAR>:Q</VAR></CODE>
<DD>
Disjunction
<DT><CODE>!</CODE>
<DD>
Cut
<DT><CODE>\+ <VAR>:P</VAR></CODE>
<DD>
Negation
<DT><CODE><VAR>:P</VAR> -&#62; <VAR>:Q</VAR></CODE>
<DD>
<DT><CODE><VAR>:P</VAR> -&#62; <VAR>:Q</VAR>; <VAR>:R</VAR></CODE>
<DD>
<DT><CODE>if(<VAR>:P</VAR>, <VAR>:Q</VAR>, <VAR>:R</VAR>)</CODE>
<DD>
If-then[-else]
<DT><CODE><VAR>?A</VAR> = <VAR>?B</VAR></CODE>
<DD>
Unification
</DL>

<P>
Atomic goals in the body of a method-clause may be one of the following:

</P>
<DL COMPACT>

<DT><VAR>:goal</VAR>
<DD>
to call the Prolog predicate <VAR>goal</VAR> in the source module.

<DT><VAR>m:goal</VAR>
<DD>
to call the Prolog predicate <VAR>goal</VAR> in module <VAR>m</VAR>.

<DT><VAR>goal</VAR>
<DD>
to send the message <VAR>goal</VAR> to the object <VAR>Self</VAR>.

<DT><VAR>::goal</VAR>
<DD>
to send the message <VAR>goal</VAR> to a method that may be defined
locally or inherited by the object.

<DT><VAR>&#60;:goal</VAR>
<DD>
to delegate the message <VAR>goal</VAR> to a method that may be defined
locally or inherited by the object.

<DT><VAR>object::goal</VAR>
<DD>
to send the message <VAR>goal</VAR> to object <VAR>object</VAR>.

<DT><VAR>object&#60;:goal</VAR>
<DD>
to delegate the message <VAR>goal</VAR> to object <VAR>object</VAR>.
</DL>

<P>
Message sending and delegation will be explained later (see section <A HREF="sicstus_35.html#SEC321">Self, Message Sending, and Message Delegation</A>).

</P>
<P>
The following is a definition for the object <CODE>list_object</CODE>.  It is
constructed from three methods: <CODE>append/3</CODE>, <CODE>member/2</CODE>, and
<CODE>length/2</CODE>.  Note that the calls to <CODE>append/3</CODE> and
<CODE>length/2</CODE> are to the local definition, whereas the <CODE>member/2</CODE>
call is to the predicate imported from the Prolog library module <CODE>lists</CODE>.

</P>

<PRE>
list_object :: {
        :- :use_module(library(lists), [append/3,member/2]) &#38;
        
        append([], L, L) &#38;
        append([X|L1], L2, [X|L3]) :-
                :: append(L1, L2, L3) &#38;

        member(X, L) :- 
                :member(X,L) &#38;

        length([], 0) &#38;
        length([_|L], N) :-
                :: length(L, N1),
                :(N is N1+1)
        }.
</PRE>

<P>
The following object <CODE>apt_1</CODE> could be part of a larger database
about free apartments in a real-estate agency:

</P>

<PRE>
apt_1 :: {
        super(apartment) &#38;

        street_name('York') &#38;
        street_number(100) &#38;
        wall_color(white) &#38;
        floor_surface(wood)
        }.
</PRE>

<P>
Another way to define <CODE>apt_1</CODE> is by using attributes. These can be
retrieved and modified efficiently by the methods <CODE>get/1</CODE> and
<CODE>set/1</CODE> respectively.

</P>

<PRE>
apt_1 :: {
        super(apartment) &#38;

        attributes([
               street_name('York'),
               street_number(100),
               wall_color(white),
               floor_surface(wood)])
        }.
</PRE>



<H3><A NAME="SEC320" HREF="sicstus_toc.html#TOC320">Generic Objects for Easy Reuse</A></H3>
<P>
<A NAME="IDX1396"></A>
<A NAME="IDX1397"></A>
<A NAME="IDX1398"></A>
<A NAME="IDX1399"></A>

</P>
<P>
Defining objects for easy reuse is a very important property for
reducing the cost of large projects.  One important technique is to
define prototypes in a parameterized way, so that various instantiations
of a prototype correspond to different uses.  Parameterized or
<EM>generic objects</EM> have been used for this purpose in other
object-oriented systems.  An object-identifier can be a compound term.
The arguments of the term are parameters that are visible in the
object-body.  Here we show one example.  Other examples and techniques
that use this facility has been investigated extensively in 
[McCabe 92].

</P>
<P>
The following is an object <CODE>sort</CODE> that sorts lists of different
types.  <CODE>sort</CODE> has a parameter that defines the type of the
elements of the list.  Notice that <VAR>Type</VAR> is visible to all methods
in the body of <CODE>sort</CODE>, and is used in the method
<CODE>partition/4</CODE>.  In the query, we use <CODE>sort(rat)</CODE> to sort a
list of terms denoting rational numbers.  We must therefore define a
<CODE>rat</CODE> object and its <CODE> &#60; </CODE> method also:

</P>

<PRE>
rat :: {
        (P/Q &#60; R/S) :- :(P*S &#60; Q*R)
        }.

sort(Type) :: {
        :- :use_module(library(lists), [append/3]) &#38;

        qsort([], []) &#38;
        qsort([P|L], S) :-
                partition(L, P, Small, Large),
                qsort(Small, S0),
                qsort(Large, S1),
                :append(S0, [P|S1], S) &#38;

        partition([], _P, [], []) &#38;
        partition([X|L1], P, Small, Large) :-
                (   Type :: (X &#60; P) -&#62;
                    Small = [X|Small1], Large = Large1
                ;   Small = Small1, Large = [X|Large1]
                ),
                partition(L1, P, Small1, Large1)
        }.

| ?- sort(rat) :: qsort([23/3, 34/11, 45/17], L).

L = [45/17,34/11,23/3]
</PRE>

<P>
Parameterized objects are interesting in their own right in Prolog even
if one is not interested in the object-oriented paradigm.  They provide
global context variables in a Prolog program without having to add such
variables as additional context arguments to each clause that
potentially uses the context.

</P>


<H2><A NAME="SEC321" HREF="sicstus_toc.html#TOC321">Self, Message Sending, and Message Delegation</A></H2>
<P>
<A NAME="IDX1400"></A>
<A NAME="IDX1401"></A>
<A NAME="IDX1402"></A>
<A NAME="IDX1403"></A>
<A NAME="IDX1404"></A>

</P>
<P>
In Prolog Objects, each method is executed in the context of an object.
This object may not be the static object where the method is declared.
The current contextual object is used to determine dynamically which
attributes are accessed, and which methods are called. This leads to a
mechanism known as dynamic binding. This object can be retrieved using
the universal method <CODE>self(<VAR>S</VAR>)</CODE>, where <VAR>S</VAR> will be bound
to the current contextual object.

</P>
<P>
When a message is sent to an object, the corresponding method will 
be executed in the context of the target object. A message delegated
to an object will invoke a method that is executed in the context
of the message-delegation operation.

</P>
<DL COMPACT>

<DT><VAR>object :: message</VAR>
<DD>
<DT><VAR>:: message</VAR>
<DD>
<A NAME="IDX1405"></A>
<A NAME="IDX1406"></A>
Message sending.  Sends <VAR>message</VAR> to <VAR>object</VAR>, setting <VAR>Self</VAR> of
the recipient to the recipient, i.e. <VAR>object</VAR>.  If <VAR>object</VAR> is
omitted, the recipient is the object in which the goal textually appears.

<DT><VAR>object &#60;: message</VAR>
<DD>
<DT><VAR>&#60;: message</VAR>
<DD>
<A NAME="IDX1407"></A>
<A NAME="IDX1408"></A>
Message delegation.  Sends <VAR>message</VAR> to <VAR>object</VAR>, setting <VAR>Self</VAR> of
the recipient to <VAR>Self</VAR> of the sender.  If <VAR>object</VAR> is omitted, the
recipient is the object in which the goal textually appears.  <EM>Delegation
preserves <VAR>Self</VAR></EM>.
</DL>

<P>
The following objects <CODE>physical_object</CODE>, <CODE>a</CODE>, and <CODE>b</CODE> are
written using the default notations for sending and delegation, hiding the
contextual variable <VAR>Self</VAR>:

</P>

<PRE>
physical_object :: {
        volume(50) &#38;
        density(100) &#38;
        weight(X) :- 
                volume(V),
                density(D),
                :(X is V*D) 
        }.

a :: {
        volume(5) &#38;
        density(10) &#38;

        Method :- 
                physical_object &#60;: Method
        }.

b :: {
        volume(5) &#38;
        density(10) &#38;

        Method :- 
                physical_object :: Method
        }.
</PRE>

<P>
Notice that the difference between the objects <CODE>a</CODE> and <CODE>b</CODE> is
that <CODE>a</CODE> <EM>delegates</EM> any message except <CODE>volume(_)</CODE> and
<CODE>density(_)</CODE> to <CODE>physical_object</CODE> while <CODE>b</CODE> <EM>sends</EM>
the message to <CODE>physical_object</CODE>.  We may now ask

</P>

<PRE>
| ?- a :: weight(X), b :: weight(Y).

X = 50
Y = 5000
</PRE>

<P>
To get hold of the current contextual object, the universal method
<CODE>self(<VAR>S</VAR>)</CODE> is provided. Another way to send a message to
<VAR>Self</VAR> is to use the constant <CODE>self</CODE>. So the following two
alternative definition of <CODE>physical_object</CODE> are equivalent to the
previous one:

</P>

<PRE>
physical_object :: {
        volume(50) &#38;
        density(100) &#38;
        weight(X) :- 
                self(S),
                S::volume(V),
                S::density(D),
                :(X is V*D) 
        }.

physical_object :: {
        volume(50) &#38;
        density(100) &#38;
        weight(X) :- 
                self::volume(V),
                self::density(D),
                :(X is V*D) 
        }.
</PRE>



<H2><A NAME="SEC322" HREF="sicstus_toc.html#TOC322">Object Hierarchies, Inheritance, and Modules</A></H2>
<P>
<A NAME="IDX1409"></A>
<A NAME="IDX1410"></A>
<A NAME="IDX1411"></A>

</P>
<P>
The Prolog Objects system implements a default inheritance mechanism.  By
declaring within an object which objects are super-objects, the hierarchy of
objects are maintained.  The system also maintains for each object its
immediate sub-objects (i.e. immediate children).  Each object may also call
Prolog predicates.  At the top of the hierarchy, the proto-object
<CODE>object</CODE> provides various services for other objects. If <CODE>object</CODE> is
not used at the top of the hierarchy many services will not be available for
other objects (check what methods are available in <CODE>object</CODE> by sending
the message <CODE>method/1</CODE> to <CODE>object</CODE>).

</P>

<UL>
<LI><A HREF="sicstus_35.html#SEC323">Inheritance</A>
<LI><A HREF="sicstus_35.html#SEC324">Differential Inheritance</A>
<LI><A HREF="sicstus_35.html#SEC325">Use of Modules</A>
<LI><A HREF="sicstus_35.html#SEC326">Super and Sub</A>
<LI><A HREF="sicstus_35.html#SEC327">The Keyword Super</A>
<LI><A HREF="sicstus_35.html#SEC328">Semantic Links to Other Objects</A>
<LI><A HREF="sicstus_35.html#SEC329">Dynamically Declared Objects</A>
<LI><A HREF="sicstus_35.html#SEC330">Dynamic Methods</A>
<LI><A HREF="sicstus_35.html#SEC331">Inheritance of Dynamic Behavior</A>
</UL>



<H3><A NAME="SEC323" HREF="sicstus_toc.html#TOC323">Inheritance</A></H3>
<P>
<A NAME="IDX1412"></A>
<A NAME="IDX1413"></A>

</P>
<P>
Immediate super-objects are declared by defining the method
<CODE>super/2</CODE> within the object. (Any definition
<CODE>super(<VAR>Super</VAR>)</CODE> is transformed to
<CODE>super(<VAR>Super</VAR>,[])</CODE>). The objects declared by <CODE>super/2</CODE>
are the immediate objects from which a method is inherited if not
defined within the object.  This implies that the inheritance mechanism
is an overriding one.  One could possibly have a union inheritance,
whereby all clauses defining a method are collected from the super
hierarchy and executed in a Prolog fashion.  This can easily be
programmed in Prolog Objects, using delegation to super objects.

</P>
<P>
The following example shows some objects used for animal classification.

</P>

<PRE>
animal :: {}.

bird :: {
        super(animal) &#38;
        skin(feather) &#38;
        habitat(tree) &#38;
        motions(fly)
        }.

penguin :: {
        super(bird) &#38;
        habitat(land) &#38;
        motions(walk) &#38;
        motions(swim) &#38;
        size(medium)
        }.

| ?- penguin :: motions(M).
M = walk ;
M = swim ;
no

| ?- penguin :: skin(S).
S = feather ;
no
</PRE>

<P>
<A NAME="IDX1414"></A>
<A NAME="IDX1415"></A>

</P>
<P>
The following is an example of multiple inheritance: an object <CODE>john</CODE> is
both a sportsman and a professor:

</P>

<PRE>
john :: {
        super(sportsman) &#38;
        super(professor) &#38;
        :
        }.
</PRE>

<P>
Inheritance will give priority to the super-objects by the order defined in
the <CODE>super/2</CODE> method.  Therefore in the above example John's
characteristics of being a sportsman will dominate those of being professor.
Other kinds of hierarchy traversal can be programmed explicitly using the
delegation mechanism.

</P>


<H3><A NAME="SEC324" HREF="sicstus_toc.html#TOC324">Differential Inheritance</A></H3>
<P>
<A NAME="IDX1416"></A>
<A NAME="IDX1417"></A>

</P>
<P>
It is possible to be selective about what is inherited by
using the method <CODE>super/2</CODE>. Its first argument is the
super object, and its second is a list of the methods that
will not be inherited from the super object.

</P>


<H3><A NAME="SEC325" HREF="sicstus_toc.html#TOC325">Use of Modules</A></H3>

<P>
In Prolog Objects, the visible predicates of the source module
(context) for the object definition may be called in the body of a
method. (The <CODE>:</CODE> prefix is used to distinguish such calls from
method calls.)  Any (<CODE>:</CODE> prefixed) directives occurring among
the method-clauses are also executed in the same source module.  For
example, to import into the source module and call the public
predicates of a module, the built-in predicate <CODE>use_module/2</CODE> and
its variants may be used:

</P>

<PRE>
some_object :: {
        :- :use_module(library(lists), [append/3]) &#38; 
        double_list(X, XX) :- :append(X,X,XX)
        }.
</PRE>



<H3><A NAME="SEC326" HREF="sicstus_toc.html#TOC326">Super and Sub</A></H3>
<P>
<A NAME="IDX1418"></A>

</P>
<P>
Two methods provided by the initial object <CODE>object</CODE> are
<CODE>super/1</CODE> and <CODE>sub/1</CODE>.
(Note that any definition of <CODE>super/1</CODE>, except the one in
<CODE>object</CODE>, is transformed to <CODE>super/2</CODE>).
<CODE>super/1</CODE> if sent to an object will return the immediate parents of
the object. <CODE>sub/1</CODE> will return the immediate children of the
object if any. It is important to note that this service is provided
only for objects that have <CODE>object</CODE> as their initial ancestor.

</P>


<PRE>
| ?- john :: super(S), S :: sub(john).
S = sportsman ;
S = professor ;
no
</PRE>

<P>
The <CODE>sub/1</CODE> property allows programs to traverse object hierarchies from a
root object <CODE>object</CODE> down to the leaves.

</P>


<H3><A NAME="SEC327" HREF="sicstus_toc.html#TOC327">The Keyword Super</A></H3>
<P>
<A NAME="IDX1419"></A>
To be able to send or delegate messages to the super-objects in a convenient
way while following the inheritance protocol, the keyword <CODE>super</CODE> is
provided.  The calls:

</P>

<PRE>
        super :: method, or
        super &#60;: method
</PRE>

<P>
mean: send or delegate (respectively) <VAR>method</VAR> to the super-objects
according to the inheritance protocol.  A simple example illustrates this
concept: assume that <CODE>john</CODE> in the above example has three id-cards, one
stored in his sportsman prototype identifying the club he is member of, one
stored in his professor prototype identifying the university he works in, and
finally one stored locally identifying his social-security number.  Given the
following methods in the object <CODE>john</CODE>:

</P>

<PRE>
m1(X) :-
        super &#60;: id_card(X) &#38;
m2(X) :-
        super(S), S &#60;: id_card(X) &#38;
</PRE>

<P>
one may ask the following:

</P>

<PRE>
| ?- john :: m1(X).
     % will follow the default inheritance and returns:
X = johns_club ;

| ?- john :: m2(X).
     % will backtrack through the possible supers returning:
X = johns_club ;
X = johns_university ;
</PRE>



<H3><A NAME="SEC328" HREF="sicstus_toc.html#TOC328">Semantic Links to Other Objects</A></H3>
<P>
Some object-oriented languages have syntactic constructs for redirecting the
inheritance chain for certain methods to completely other objects which are not
defined in the object's inheritance hierarchy.  This is not needed in Prolog
Objects due to delegation.  Assume that the method <CODE>m/<VAR>n</VAR></CODE> is linked
to object <CODE>some_object</CODE>, we just add a method for this:

</P>

<PRE>
        m(X1, ..., Xn) :- some_object &#60;: m(X1, ..., Xn) &#38;
</PRE>



<H3><A NAME="SEC329" HREF="sicstus_toc.html#TOC329">Dynamically Declared Objects</A></H3>
<P>
<A NAME="IDX1420"></A>
<A NAME="IDX1421"></A>
<A NAME="IDX1422"></A>
<A NAME="IDX1423"></A>
<A NAME="IDX1424"></A>

</P>
<P>
When an object is declared and compiled into Prolog Objects, its methods
cannot be changed during execution.  Such an object is said to be
<EM>static</EM>.  To be able to update any method in an object, the object has to
be declared <EM>dynamic</EM>.  There is one exception, the inheritance hierarchy
declared by <CODE>super/(1-2)</CODE> cannot be changed.  By including the fact
<CODE>dynamic</CODE> as part of the object body, the object becomes dynamic:

</P>

<PRE>
dynamic_object :: {
        dynamic &#38;
        :
        }.
</PRE>



<H3><A NAME="SEC330" HREF="sicstus_toc.html#TOC330">Dynamic Methods</A></H3>
<P>
<A NAME="IDX1425"></A>
<A NAME="IDX1426"></A>
<A NAME="IDX1427"></A>
To be able to change a method with functor <VAR>F</VAR> and arity <VAR>N</VAR> in a
static object, the method has to be declared dynamic by storing the following
fact in the object:

</P>

<PRE>
some_object :: {
        dynamic <VAR>F/N</VAR> &#38;
        :
        }.
</PRE>

<P>
Each book in a library can be represented as an object, in which the
name of the book is stored, the authors, and a borrowing history
indicating when a book is borrowed and when it is returned.  A history
item may have the form
<CODE>history_item(<VAR>Person</VAR>,<VAR>Status</VAR>,<VAR>Date</VAR>)</CODE> where
<VAR>Status</VAR> is either <CODE>borrowed</CODE> or <CODE>returned</CODE>, and
<VAR>Date</VAR> has the form YY-MM-DD, for YY year, MM month, DD day.

</P>
<P>
A typical book <CODE>book_12</CODE> could have the following status.  Note that
<CODE>history_item/3</CODE> is dynamic:

</P>

<PRE>
book_12 :: {
        super(book) &#38;

        title('The Art of Prolog') &#38;
        authors(['Leon Sterling', 'Ehud Shapiro']) &#38;

        dynamic history_item/3 &#38;
        
        history_item('Dan Sahlin', returned, 92-01-10) &#38;
        history_item('Dan Sahlin', borrowed, 91-06-10) &#38;
        :
        }.
</PRE>

<P>
Dynamic methods that are stored in an object can be updated, as in usual
Prolog programs, by sending <CODE>assert</CODE> and <CODE>retract</CODE> messages
directly to the object.

</P>
<P>
For example, to borrow a book the following method could be defined in the
object <CODE>book</CODE>.  We assume that the top most <CODE>history_item</CODE> fact is
the latest transaction, and there is an object <CODE>date</CODE> from which we can
get the current date.

</P>

<PRE>
borrow(Person) :-
        history_item(_Person0, Status, _Date0), !,
        (   Status = returned -&#62;
            date::current(Date),
            asserta(history_item(Person, borrowed, Date))
        ;   :display('book not available'), :ttynl
        ) &#38;
</PRE>



<H3><A NAME="SEC331" HREF="sicstus_toc.html#TOC331">Inheritance of Dynamic Behavior</A></H3>

<P>
When an object is created, it will inherit from its parents their
dynamic behavior. Methods that are declared dynamic in a parent, 
will be copied into the object, and its dynamic behavior preserved.

</P>

<PRE>
a:: {
        super(object) &#38;
        dynamic p/1 &#38;
        p(1) &#38;
        p(2)
     }

b :: {
        super(a)
     }

| ?- b::p(X).
X = 1 ? ;
X = 2 ? ;
no
| ?- b::asserta(p(3)).
yes
| ?- b::p(X).         
X = 3 ? ;
X = 1 ? ;
X = 2 ? ;
no

</PRE>

<P>
Notice that by redeclaring a method to be dynamic in a sub-object, 
amounts to redefining the method, and overriding of the parent definition
will take effect.

</P>

<PRE>
c :: {
        super(a) &#38;
        dynamic p/1 
     }

| ?- c::p(X).
no

</PRE>



<H2><A NAME="SEC332" HREF="sicstus_toc.html#TOC332">Creating Objects Dynamically</A></H2>
<P>
<A NAME="IDX1428"></A>
<A NAME="IDX1429"></A>

</P>
<P>
As with dynamically declared objects, the full flexibility of Prolog
Objects is achieved when objects are created at runtime.  Anything,
except the inheritance hierarchy, can be changed: methods can be added
or deleted.  The services for object creation, destruction, and method
modification are defined in the proto-object <CODE>object</CODE>.

</P>

<UL>
<LI><A HREF="sicstus_35.html#SEC333">Object Creation</A>
<LI><A HREF="sicstus_35.html#SEC334">Method Additions</A>
<LI><A HREF="sicstus_35.html#SEC335">Parameter Passing to New Objects</A>
</UL>



<H3><A NAME="SEC333" HREF="sicstus_toc.html#TOC333">Object Creation</A></H3>
<DL COMPACT>

<DT><CODE><VAR>+SomeObject</VAR> :: new(<VAR>?NewObject</VAR>)</CODE>
<DD>
<A NAME="IDX1430"></A>
 
<VAR>NewObject</VAR> is created with <VAR>SomeObject</VAR> as super.
<VAR>NewObject</VAR> could be an atom, variable, or compound term whose
arguments are distinct variables.

<A NAME="IDX1431"></A>
<DT><CODE><VAR>+SomeObject</VAR> :: new(<VAR>?NewObject</VAR>,<VAR>+Supers</VAR>)</CODE>
<DD>
<VAR>NewObject</VAR> is created with <VAR>Supers</VAR> specifying the super
objects (prototypes). <VAR>Supers</VAR> is a list containing super
specifications. A super specification is either an object identifier or
a pair <VAR>Object-NotInheritList</VAR> where <VAR>NotInheritList</VAR> specifies
methods not to inherit from <VAR>Object</VAR>. <VAR>NewObject</VAR> could be an
atom, variable, or compound term whose arguments are distinct variables.
</DL>

<P>
The object <CODE>vehicle</CODE> is created having the proto-object <CODE>object</CODE> as
super, followed by creating <CODE>moving_van</CODE> with <CODE>vehicle</CODE> as super,
followed by creating <CODE>truck</CODE>.

</P>

<PRE>
| ?- object :: new(vehicle), 
     vehicle :: new(moving_van), 
     moving_van :: new(truck).
yes

| ?- truck :: super(X), vehicle :: sub(X).
X = moving_van ;
no
</PRE>



<H3><A NAME="SEC334" HREF="sicstus_toc.html#TOC334">Method Additions</A></H3>
<DL COMPACT>

<DT><CODE><VAR>+SomeObject</VAR> :: asserta(<VAR>+SomeMethod</VAR>)</CODE>
<DD>
<A NAME="IDX1432"></A>
 <A NAME="IDX1433"></A>
 <A NAME="IDX1434"></A>
 
<DT><CODE><VAR>+SomeObject</VAR> :: assertz(<VAR>+SomeMethod</VAR>)</CODE>
<DD>
<DT><CODE><VAR>+SomeObject</VAR> :: assert(<VAR>+SomeMethod</VAR>)</CODE>
<DD>
Asserts <VAR>SomeMethod</VAR> in <VAR>SomeObject</VAR> with normal Prolog semantics.
</DL>

<P>
Add some facts to <CODE>vehicle</CODE> and <CODE>truck</CODE> with initial value equal to
<CODE>[]</CODE>.

</P>

<PRE>
| ?- vehicle :: assert(fuel_level([])),
     vehicle :: assert(oil_level([])),
     vehicle :: assert(location([])),
     truck :: assert(capacity([])),
     truck :: assert(total_weight([])).
yes
</PRE>



<H3><A NAME="SEC335" HREF="sicstus_toc.html#TOC335">Parameter Passing to New Objects</A></H3>

<P>
When new objects are created, it is possible to pass parameters.  The following
example shows:

</P>

<UL>
<LI>

How general methods are asserted
</UL>

<P>
In the previous examples one could pass parameters to an object as follows,
using the method <CODE>augment/1</CODE>.

</P>

<PRE>
| ?- vehicle :: augment({
        new_attrs(Instance, Attribute_list) :-
                self :: new(Instance),
                :: assign_list(Attribute_list, Instance) &#38;

        assign_list([], Instance) &#38;
        assign_list([Att|List], Instance) :-
                :: assign(Att, Instance),
                :: assign_list(List, Instance) &#38;
        
        assign(P, Instance) :-
                Instance :: assert(P) 
        }).
yes

% create a new 'truck'
| ?- vehicle :: new_attrs(truck, [capacity([]),total_weight([])]).
yes
</PRE>



<H2><A NAME="SEC336" HREF="sicstus_toc.html#TOC336">Access Driven Programming--Daemons</A></H2>
<P>
<A NAME="IDX1435"></A>
<A NAME="IDX1436"></A>
<A NAME="IDX1437"></A>
Access based programming is a paradigm where certain actions are
performed, or some constraints are checked, when "access operations"
are invoked.  Access operations for updates (i.e. assert, retract) can
be redefined in an object by redefining these operations and delegating
the same operation to <CODE>super</CODE>.  Notice that without a delegation
mechanism this would not be possible, since the <VAR>Self</VAR> would have
changed.
So assume that we want to print on
the screen "p is augmented" whenever the fact <CODE>p(X)</CODE> is asserted
in an object <CODE>foo</CODE>, we just redefine <CODE>assert/1</CODE>:

</P>

<PRE>
foo :: {
        super(object) &#38;

        dynamic p/1 &#38;

        p(0) &#38;
        p(1) &#38;

        assert(p(X)) :- !,   /* assert/1 is redefined for p(X) */
                super &#60;: assert(p(X)),
                :display('p is augmented'), :ttynl &#38;
        assert(M) :-         /* delegating assert(_) messages */
                super &#60;: assert(M) &#38;
        :
        }.
</PRE>



<H2><A NAME="SEC337" HREF="sicstus_toc.html#TOC337">Instances</A></H2>
<P>
<A NAME="IDX1438"></A>

</P>
<P>
Objects are relatively heavy weight. To be able to create efficiently
light weight objects, we introduce the notion of <EM>instances</EM>.  An
instance is an object with restricted capability. It is created from
an object that is considered its class. It gets a copy of the
attributes of its class. These can be modified by <CODE>get/1</CODE> and
<CODE>set/1</CODE>. An instance cannot be a class for other
instances. Instances are in general very efficient, both in space and
access/modification time.  The attribute <CODE>'$class'/1</CODE> will store
the identity of the class of the instance including parameters.

</P>



<H2><A NAME="SEC338" HREF="sicstus_toc.html#TOC338">Built-In Objects and Methods</A></H2>


<UL>
<LI><A HREF="sicstus_35.html#SEC339">Universal Methods</A>
<LI><A HREF="sicstus_35.html#SEC340">Inlined Methods</A>
<LI><A HREF="sicstus_35.html#SEC341">The Proto-Object</A>
<LI><A HREF="sicstus_35.html#SEC342">The Utility Object</A>
</UL>



<H3><A NAME="SEC339" HREF="sicstus_toc.html#TOC339">Universal Methods</A></H3>
<P>
The following methods are "universal", i.e. they are defined locally, if
appropriate, for every object:
<DL COMPACT>

<DT><CODE>super(<VAR>?Object</VAR>,<VAR>?NotInheritList</VAR>)</CODE>
<DD>
<A NAME="IDX1439"></A>
 
<VAR>Object</VAR> is a parent (a super-object) of <VAR>Self</VAR>.
<VAR>NotInheritList</VAR> specifies methods of <VAR>Object</VAR> explicitly not
inherited by <VAR>Self</VAR>.  The definition <CODE>super(<VAR>Object</VAR>)</CODE> is
translated to <CODE>super(<VAR>Object</VAR>,[])</CODE>.

<A NAME="IDX1440"></A>
<DT><CODE>attributes(<VAR>+Attributes</VAR>)</CODE>
<DD>
<VAR>Attributes</VAR> is a list of compound terms specifying the local
attributes of <VAR>Self</VAR> and the initial values.

</DL>



<H3><A NAME="SEC340" HREF="sicstus_toc.html#TOC340">Inlined Methods</A></H3>
<P>
The following methods are compiled inline i.e. calls are replaced by
definitions. This implies (in the current implementation) that they have
a fixed semantics an can not be redefined. There are also definitions
for these methods in <CODE>object</CODE> covering the cases of unexpanded
calls.
<DL COMPACT>

<DT><CODE>self(<VAR>?Self</VAR>)</CODE>
<DD>
<A NAME="IDX1441"></A>
 
Unifies <VAR>Self</VAR> with "self".

<A NAME="IDX1442"></A>
<DT><CODE>get(<VAR>+Attribute</VAR>)</CODE>
<DD>
Gets the attribute value(s) of the attribute specified by the head
functor of <VAR>Attribute</VAR>. The value(s) are unified with the
argument(s) of <VAR>Attribute</VAR>.

<A NAME="IDX1443"></A>
<DT><CODE>set(<VAR>+Attribute</VAR>)</CODE>
<DD>
Sets the attribute value(s) of the attribute specified by the head
functor of <VAR>Attribute</VAR>. The value(s) are taken from the argument(s)
of <VAR>Attribute</VAR>.

</DL>



<H3><A NAME="SEC341" HREF="sicstus_toc.html#TOC341">The Proto-Object "object"</A></H3>
<P>
<A NAME="IDX1444"></A>
The proto-object <CODE>object</CODE> provides basic methods that are available
to all other objects by delegation:

</P>
<DL COMPACT>

<DT><CODE>super(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1445"></A>
 
<VAR>Object</VAR> is a parent (a super-object) of <VAR>Self</VAR>. Note that any
other definition of <CODE>super(<VAR>Object</VAR>)</CODE> are translated to the
universal method <CODE>super/2</CODE>.

<A NAME="IDX1446"></A>
<DT><CODE>sub(<VAR>?Object</VAR>)</CODE>
<DD>
<VAR>Object</VAR> is a child (a sub-object) of <VAR>Self</VAR>.

<A NAME="IDX1447"></A>
<DT><CODE>self(<VAR>?Self</VAR>)</CODE>
<DD>
Unifies <VAR>Self</VAR> with "self". <STRONG>Note:</STRONG> this method is inlined when
possible.

<DT><CODE>object(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1448"></A>
One of the defined objects in the system is <VAR>Object</VAR>.

<DT><CODE>dynamic</CODE>
<DD>
<A NAME="IDX1449"></A>
<VAR>Self</VAR> is a dynamic object.

<DT><CODE>static</CODE>
<DD>
<A NAME="IDX1450"></A>
<VAR>Self</VAR> is a static object.

<DT><CODE>dynamic <VAR>?Name/?Arity</VAR></CODE>
<DD>
<A NAME="IDX1451"></A>
<VAR>Name/Arity</VAR> is a dynamic method of <VAR>Self</VAR>.

<DT><CODE>static <VAR>?Name/?Arity</VAR></CODE>
<DD>
<A NAME="IDX1452"></A>
<VAR>Name/Arity</VAR> is a static method of <VAR>Self</VAR>.

<DT><CODE>new(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1453"></A>
Creates a new dynamic <VAR>Object</VAR>. <VAR>Self</VAR> will be the prototype of
<VAR>Object</VAR>.  <VAR>Object</VAR> can be a compound term, an atom, or a
variable. In the last case the method generates a unique name for
<VAR>Object</VAR>.

<DT><CODE><VAR>+SomeObject</VAR> :: new(<VAR>?NewObject</VAR>,<VAR>+Supers</VAR>)</CODE>
<DD>
<A NAME="IDX1454"></A>
<VAR>NewObject</VAR> is created with <VAR>Supers</VAR> specifying the super
objects (prototypes). <VAR>Supers</VAR> is a list containing super
specifications. A super specification is either an object identifier or
a pair <VAR>Object-NotInheritList</VAR> where <VAR>NotInheritList</VAR> specifies
methods not to inherit from <VAR>Object</VAR>. <VAR>NewObject</VAR> could be an
atom, variable, or compound term whose arguments are distinct variables.

<DT><CODE>instance(<VAR>?Instance</VAR>)</CODE>
<DD>
<A NAME="IDX1455"></A>
Creates a new instance <VAR>Instance</VAR>.  <VAR>Self</VAR> will be the class of
<VAR>Instance</VAR>.  <VAR>Instance</VAR> can be a compound term, an atom, or a
variable. In the last case the method generates a unique name for
<VAR>Instance</VAR>.

<DT><CODE>has_instance(<VAR>?Instance</VAR>)</CODE>
<DD>
<A NAME="IDX1456"></A>
<VAR>Self</VAR> has the instance <VAR>Instance</VAR>.

<DT><CODE>has_attribute(<VAR>?AttributeSpec</VAR>)</CODE>
<DD>
<A NAME="IDX1457"></A>
<VAR>Self</VAR> has the attribute <VAR>AttributeSpec</VAR>, locally defined or
inherited. <VAR>AttributeSpec</VAR> is on the format
<CODE><VAR>Name</VAR>/<VAR>Arity</VAR></CODE>.

<A NAME="IDX1458"></A>
<DT><CODE>get(<VAR>+Attribute</VAR>)</CODE>
<DD>
Gets the attribute value(s) of the attribute specified by the head
functor of <VAR>Attribute</VAR>. The value(s) are unified with the
argument(s) of <VAR>Attribute</VAR>. <STRONG>Note:</STRONG> this method is inlined when
possible.

<A NAME="IDX1459"></A>
<DT><CODE>set(<VAR>+Attribute</VAR>)</CODE>
<DD>
Sets the attribute value(s) of the attribute specified by the head
functor of <VAR>Attribute</VAR>. The value(s) are taken from the
argument(s) of <VAR>Attribute</VAR>. <STRONG>Note:</STRONG> this method is inlined when
possible.

<DT><CODE>assert(<VAR>+Fact</VAR>)</CODE>
<DD>
<DT><CODE>assert(<VAR>+Fact</VAR>, <VAR>-Ref</VAR>)</CODE>
<DD>
<DT><CODE>asserta(<VAR>+Fact</VAR>)</CODE>
<DD>
<DT><CODE>asserta(<VAR>+Fact</VAR>, <VAR>-Ref</VAR>)</CODE>
<DD>
<DT><CODE>assertz(<VAR>+Fact</VAR>)</CODE>
<DD>
<DT><CODE>assertz(<VAR>+Fact</VAR>, <VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX1460"></A>
<A NAME="IDX1461"></A>
<A NAME="IDX1462"></A>
<A NAME="IDX1463"></A>
<A NAME="IDX1464"></A>
<A NAME="IDX1465"></A>
Asserts a new <VAR>Fact</VAR> in <VAR>Self</VAR>.  If <VAR>Self</VAR> is static, the
name and arity of <VAR>Fact</VAR> must be declared as a dynamic method.
<CODE>asserta</CODE> places <VAR>Fact</VAR> before any old facts.  The other forms
place it after any old facts.  A pointer to the asserted fact is
returned in the optional argument <VAR>Ref</VAR>, and can be used by the
Prolog built-in predicates <CODE>erase/1</CODE> and <CODE>instance/2</CODE>.

<DT><CODE>retract(<VAR>+Fact</VAR>)</CODE>
<DD>
<A NAME="IDX1466"></A>
Retracts a <VAR>Fact</VAR> from <VAR>Self</VAR>.  If <VAR>Self</VAR> is static,
the name and arity of <VAR>Fact</VAR> must be declared as a dynamic method.

<DT><CODE>update(<VAR>+Fact</VAR>)</CODE>
<DD>
<A NAME="IDX1467"></A>
Replaces the first fact with the same name and arity as <VAR>Fact</VAR> in
<VAR>Self</VAR> by <VAR>Fact</VAR>.  If <VAR>Self</VAR> is static, the name and arity
of <VAR>Fact</VAR> must be declared as a dynamic method.

<DT><CODE>retractall(<VAR>?Head</VAR>)</CODE>
<DD>
<A NAME="IDX1468"></A>
Removes facts from <VAR>Self</VAR> that unify with <VAR>Head</VAR>.  If
<VAR>Self</VAR> is static, the name and arity of <VAR>Fact</VAR> must be declared
as a dynamic method.

<DT><CODE>abolish</CODE>
<DD>
<A NAME="IDX1469"></A>
Abolishes <VAR>Self</VAR> if dynamic. 

<DT><CODE>augment(<VAR>?ObjectBody</VAR>)</CODE>
<DD>
<DT><CODE>augmenta(<VAR>?ObjectBody</VAR>)</CODE>
<DD>
<DT><CODE>augmentz(<VAR>?ObjectBody</VAR>)</CODE>
<DD>
<A NAME="IDX1470"></A>
<A NAME="IDX1471"></A>
<A NAME="IDX1472"></A>
<VAR>ObjectBody</VAR>, having the form <VAR>{ sentence-1 &#38; ... &#38; sentence-n
}</VAR>, is added to <VAR>Self</VAR>.  <CODE>augmenta</CODE> places the new clauses
before any old clauses.  The other forms place it after any old clauses.
</DL>



<H3><A NAME="SEC342" HREF="sicstus_toc.html#TOC342">The built-in object "utility"</A></H3>
<P>
<A NAME="IDX1473"></A>
The base object <CODE>utility</CODE> provides methods that could be used in user
programs.  <CODE>utility</CODE> has <CODE>object</CODE> as its super-object.

</P>
<DL COMPACT>

<DT><CODE>subs(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1474"></A>
Gives a list of all the children of <VAR>Self</VAR>.

<DT><CODE>supers(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1475"></A>
Gives a list of all parents of <VAR>Self</VAR>.

<DT><CODE>objects(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1476"></A>
Gives a list of all objects.

<DT><CODE>dynamic_objects(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1477"></A>
Gives a list of all dynamic objects.

<DT><CODE>static_objects(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1478"></A>
Gives a list of all static objects.

<DT><CODE>methods(<VAR>?Methods</VAR>)</CODE>
<DD>
<A NAME="IDX1479"></A>
Gives a list of all the methods of <VAR>Self</VAR>.

<DT><CODE>dynamic_methods(<VAR>?Methods</VAR>)</CODE>
<DD>
<A NAME="IDX1480"></A>
Gives a list of all dynamic methods of <VAR>Self</VAR>.

<DT><CODE>static_methods(<VAR>?Methods</VAR>)</CODE>
<DD>
<A NAME="IDX1481"></A>
Gives a list of all static methods of <VAR>Self</VAR>.

<DT><CODE>descendant(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1482"></A>
One of the descendants of <VAR>Self</VAR> is <VAR>Object</VAR>.

<DT><CODE>descendant(<VAR>?Object</VAR>, <VAR>?Level</VAR>)</CODE>
<DD>
<A NAME="IDX1483"></A>
<VAR>Object</VAR> a descendant at depth <VAR>Level</VAR> of <VAR>Self</VAR>.  A child of
<VAR>Self</VAR> is at level 1.

<DT><CODE>descendants(<VAR>?Objects</VAR>)</CODE>
<DD>
<A NAME="IDX1484"></A>
The list of all  descendants of <VAR>Self</VAR> is <VAR>Objects</VAR>.

<DT><CODE>descendants(<VAR>?Objects</VAR>, <VAR>?Level</VAR>)</CODE>
<DD>
<A NAME="IDX1485"></A>
<VAR>Objects</VAR> is the list of descendants at depth <VAR>Level</VAR> of
<VAR>Self</VAR>.

<DT><CODE>ancestor(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1486"></A>
One of the ancestors of <VAR>Self</VAR> is <VAR>Object</VAR>.

<DT><CODE>ancestor(<VAR>?Object</VAR>, <VAR>?Level</VAR>)</CODE>
<DD>
<A NAME="IDX1487"></A>
<VAR>Object</VAR> is an ancestor of <VAR>Self</VAR> at height <VAR>Level</VAR>.  A
super-object of <VAR>Self</VAR> has level 1.

<DT><CODE>ancestors(<VAR>?Object</VAR>)</CODE>
<DD>
<A NAME="IDX1488"></A>
The list of all  ancestors of <VAR>Self</VAR> is <VAR>Objects</VAR>.

<DT><CODE>ancestors(<VAR>?Object</VAR>, <VAR>?Level</VAR>)</CODE>
<DD>
<A NAME="IDX1489"></A>
<VAR>Objects</VAR> is the list of ancestors at height <VAR>Level</VAR> of
<VAR>Self</VAR>.

<DT><CODE>restart</CODE>
<DD>
<A NAME="IDX1490"></A>
Removes all dynamic objects.  Note that dynamic methods added to static
objects are not removed.

<DT><CODE>and_cast(<VAR>+Objects</VAR>, <VAR>?Message</VAR>)</CODE>
<DD>
<A NAME="IDX1491"></A>
Sends the same message <VAR>Message</VAR> to all objects in the list
<VAR>Objects</VAR>.

<DT><CODE>or_cast(<VAR>+Objects</VAR>, <VAR>?Message</VAR>)</CODE>
<DD>
<A NAME="IDX1492"></A>
Sends the same message <VAR>Message</VAR> to one of the objects in the list
<VAR>Objects</VAR>, backtracking through the alternative objects.
</DL>



<H2><A NAME="SEC343" HREF="sicstus_toc.html#TOC343">Expansion to Prolog Code</A></H2>

<P>
As already mentioned, object definitions are expanded to Prolog clauses
much as definite clause grammars.  This expansion is usually transparent
to the user.  While debugging a Prolog Objects program, however, the
expanded representation may become exposed.  This section will explain
in detail the source expansion, so as to give the user the possibility
to relate back to the source code during a debugging session. The
inheritance mechanism, based on module importation, is also described.

</P>
<P>
<A NAME="IDX1493"></A>
<A NAME="IDX1494"></A>
First of all, every statically defined object will translate to several
Prolog clauses belonging to a unique <EM>object module</EM> with the same
identity as the <VAR>object-identifier</VAR>. Object modules are
significantly cheaper to create than ordinary modules, as they do not
import the built-in Prolog predicates.

</P>
<P>
The module will contain predicates implementing an object declaration,
 the method code, imported methods and parameter transfer predicates.
These predicates will be described in detail below, using the
notational convention that variable names in italics are syntactic
variables that will be replaced by something else in the translation
process.

</P>

<UL>
<LI><A HREF="sicstus_35.html#SEC344">The Inheritance Mechanism</A>
<LI><A HREF="sicstus_35.html#SEC345">Object Attributes</A>
<LI><A HREF="sicstus_35.html#SEC346">Object Instances</A>
<LI><A HREF="sicstus_35.html#SEC347">The Object Declaration</A>
<LI><A HREF="sicstus_35.html#SEC348">The Method Code</A>
<LI><A HREF="sicstus_35.html#SEC349">Parameter Transfer</A>
</UL>



<H3><A NAME="SEC344" HREF="sicstus_toc.html#TOC344">The Inheritance Mechanism</A></H3>

<P>
The inheritance mechanism is based on the importation mechanism of the
Prolog module system. When an object is created, whether loaded from
file or at runtime by <CODE>new/(1-2)</CODE>, the method predicates
(i.e. predicates implementing the methods) visible in the immediate
supers are collected. After subtracting from this set the method
predicates which are locally defined, and those that are specified in
the <EM>don't-inherit-list</EM> , the resulting set is made visible in the
module of the inheriting object by means of importation. This implies
that inherited methods are shared, expect dynamic methods.

</P>
<P>
Dynamic methods are inherited in a similar way with the big difference
that they are not imported but copied. Even dynamic declarations
(methods without clauses) are inherited.

</P>
<P>
Inheritance from dynamic objects differs in one aspect: Static
predicates visible in a dynamic object are not imported directly
from the dynamic object but from the static object from where it was
imported to the dynamic object. This makes an inheriting object
independent of any dynamic ancestor object after its creation.

</P>



<H3><A NAME="SEC345" HREF="sicstus_toc.html#TOC345">Object Attributes</A></H3>
<P>
<A NAME="IDX1495"></A>

</P>
<P>
Attributes are based on an efficient term storage associated to
modules. The attributes for an object is collected from its ancestors
and itself at compile time and used for initialization at load
time. The methods for accessing attributes, <CODE>get/1</CODE> and
<CODE>set/1</CODE>, are inlined to primitive calls whenever possible. They
should hence not be redefined.

</P>



<H3><A NAME="SEC346" HREF="sicstus_toc.html#TOC346">Object Instances</A></H3>
<P>
<A NAME="IDX1496"></A>

</P>
<P>
Instances are different from other objects in that they do not
inherit. Instead they share the predicate name space with its class
object. They do however have their own attributes. At creation, an
instance gets a copy of its class objects attributes. The reserved
attribute <CODE>'$class'/1</CODE>, which is present in any object, is used for
an instance to hold its class object identifier. The purpose of this
is mainly to store the parameters of the class object when the
instance is created.

</P>


<H3><A NAME="SEC347" HREF="sicstus_toc.html#TOC347">The Object Declaration</A></H3>

<P>
The object declaration is only used by certain meta-programming operations.
It consists of a fact

</P>

<PRE>
'$so_type'(<VAR>Object</VAR>, <VAR>Type</VAR>).
</PRE>

<P>
where <VAR>Object</VAR> is the <VAR>object-identifier</VAR>, and <VAR>Type</VAR> is
either <CODE>static</CODE> or <CODE>dynamic</CODE>.  If the type is <CODE>static</CODE>,
the other generated predicates will be static, otherwise they will be
dynamic.

</P>



<H3><A NAME="SEC348" HREF="sicstus_toc.html#TOC348">The Method Code</A></H3>

<P>
Each method clause translates to a Prolog clause with two extra
arguments: <VAR>Self</VAR> (a variable) and <VAR>Myself</VAR>. The latter
argument is needed to cater for passing object parameters to the
method body which is desribed further in next section.

</P>
<P>
The method body is translated to a Prolog-clause body as follows.
The code is traversed, and the goals are transformed according to the
following transformation patterns and rules.  In the transformation
rules, the notation <CODE><VAR>Msg</VAR>(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE> denotes the term
produced by augmenting <VAR>Msg</VAR> by the two arguments <VAR>X</VAR> and
<VAR>Y</VAR>:

</P>
<DL COMPACT>

<DT><CODE><VAR>Goal</VAR></CODE>
<DD>
where <VAR>Goal</VAR> is a variable, is translated to <BR>
<CODE>objects:call_from_body(<VAR>Goal</VAR>,<VAR>Self</VAR>,<VAR>Myself</VAR>,<VAR>Src</VAR>)</CODE>
where <VAR>Src</VAR> is the source module. <CODE>objects:call_from_body/4</CODE>
will meta-interpret <CODE>Goal</CODE> at runtime.

<DT><CODE>:: <VAR>Msg</VAR></CODE>
<DD>
is translated to <CODE><VAR>Myself</VAR>:<VAR>Msg</VAR>(<VAR>Myself</VAR>,<VAR>Myself</VAR>)</CODE>
if <VAR>Msg</VAR> is a non variable.  Otherwise, it is translated to
<CODE>objects:call_object(<VAR>Myself</VAR>, <VAR>Msg</VAR>, <VAR>Myself</VAR>)</CODE>. 

<DT><CODE>&#60;: <VAR>Msg</VAR></CODE>
<DD>
is translated to <CODE><VAR>Myself</VAR>:<VAR>Msg</VAR>(<VAR>Self</VAR>,<VAR>Myself</VAR>)</CODE> if
<VAR>Msg</VAR> is a non variable. Otherwise, it is translated to
<CODE>objects:call_object(<VAR>Myself</VAR>, <VAR>Msg</VAR>, <VAR>Self</VAR>)</CODE>. 

<DT><CODE>super :: <VAR>Msg</VAR> is translated to</CODE>
<DD>
<CODE>objects:call_super_exp(<VAR>Myself</VAR>,<VAR>Msg</VAR>(<VAR>Super</VAR>,<VAR>Myself</VAR>),<VAR>Super</VAR>)</CODE>
if <VAR>Msg</VAR> is a non variable. <CODE>call_super_exp/3</CODE> searches the
supers of <VAR>Myself</VAR>. <VAR>Super</VAR> is bound to the super object where
the method is found. If <VAR>Msg</VAR> is a variable, the goal is
translated to
<CODE>objects:call_super(<VAR>Myself</VAR>,<VAR>Msg</VAR>,<VAR>Super</VAR>,<VAR>Super</VAR>)</CODE>
which expands <VAR>Msg</VAR> and performs otherwise the same actions as
<CODE>call_super_exp/3</CODE>.

<DT><CODE>super &#60;: <VAR>Msg</VAR></CODE>
<DD>
is translated to
<CODE>objects:call_super_exp(<VAR>Myself</VAR>,<VAR>Msg</VAR>(<VAR>Self</VAR>,<VAR>Myself</VAR>),<VAR>Super</VAR>)</CODE>
if <VAR>Msg</VAR> is a non variable. <CODE>call_super_exp/3</CODE> searches the
supers of <VAR>Myself</VAR>. <VAR>Super</VAR> is bound to the super object where
the method is found. If <VAR>Msg</VAR> is a variable, the goal is
translated to
<CODE>objects:call_super(<VAR>Myself</VAR>,<VAR>Msg</VAR>,<VAR>Self</VAR>,<VAR>Super</VAR>)</CODE>
which expands <VAR>Msg</VAR> and performs otherwise the same actions as
<CODE>call_super_exp/3</CODE>.

<DT><CODE><VAR>Obj</VAR> :: <VAR>Msg</VAR></CODE>
<DD>

<UL>
<LI>

If <VAR>Msg</VAR> is non-variable, this is translated to
<CODE><VAR>Obj</VAR>:<VAR>Msg</VAR>(<VAR>Obj</VAR>,<VAR>Obj</VAR>)</CODE>.

<LI>

Otherwise, it is translated to
<CODE>objects:call_object(<VAR>Obj</VAR>,<VAR>Msg</VAR>,<VAR>Obj</VAR>)</CODE>.  
</UL>

<DT><CODE><VAR>Obj</VAR> &#60;: <VAR>Msg</VAR></CODE>
<DD>

<UL>
<LI>

If <VAR>Msg</VAR> is non-variable, this is translated to
<CODE><VAR>Obj</VAR>:<VAR>Msg</VAR>(<VAR>Self</VAR>,<VAR>Obj</VAR>)</CODE>.

<LI>

Otherwise, if <VAR>Msg</VAR> is a non-variable, it is translated to
        <CODE>functor(<VAR>Obj</VAR>,<VAR>O</VAR>,<VAR>_</VAR>),
        <VAR>O</VAR>:<VAR>Msg</VAR>(<VAR>Self</VAR>,<VAR>Obj</VAR>)</CODE>.

<LI>

Otherwise, it is translated to
<CODE>objects:call_object(<VAR>Obj</VAR>,<VAR>Msg</VAR>,<VAR>Self</VAR>)</CODE>.  
</UL>

<DT><CODE><CODE>self</CODE> &#60;: <VAR>Msg</VAR></CODE>
<DD>
<DT><CODE><CODE>self</CODE> :: <VAR>Msg</VAR></CODE>
<DD>
<DT><CODE><VAR>Msg</VAR></CODE>
<DD>
are all translated like <CODE><VAR>Self</VAR> :: <VAR>Msg</VAR></CODE>.

<DT><CODE><CODE><VAR>Module</VAR>:<VAR>Goal</VAR></CODE></CODE>
<DD>
is translated to <CODE><VAR>Module</VAR>:<VAR>Goal</VAR></CODE>.

<DT><CODE><CODE>:<VAR>Goal</VAR></CODE></CODE>
<DD>
is translated to <CODE><VAR>Src</VAR>:<VAR>Goal</VAR></CODE> where <VAR>Src</VAR> is the
source module.

</DL>

<P>
To illustrate the expansion, consider the object <CODE>history_point</CODE>
directives, all executed in the <CODE>history_point</CODE> module:

</P>

<PRE>
:-objects:create_object(history_point,
  [point-[]],
  [attributes/3,display/3,move/4,new/4,print_history/3,super/4],
  [],
  [y(0),x(0),history([])],
  tree(history_point,[tree(point,[tree(object,[])])])).

history_point:super(point, [], _, history_point).

history_point:attributes([history([])], _, _).

history_point:display(A, B, _) :-
        objects:call_super_exp(history_point, display(A,B,C), C),
        history_point:print_history(A, B, history_point).

history_point:'$so_type'(history_point, static).

history_point:move(A, B, C, _) :-
        objects:call_super_exp(history_point, move(A,B,C,E), E),
        prolog:'$get_module_data'(C, history, D),
        prolog:'$set_module_data'(C, history, [(A,B)|D]).

history_point:print_history(A, B, _) :-
        prolog:'$get_module_data'(B, history, C),
        A:format('with location history ~w~n', [C], A, A).

history_point:new(A, xy(D,E), B, _) :-
        objects:call_super_exp(history_point, new(A,xy(D,E),B,C), C),
        prolog:'$set_module_data'(A, history, [(D,E)]).
</PRE>

<P>
The directive <CODE>create_object/6</CODE> creates the object, performs the
inheritance by importation, and initializes attributes. The last
argument is a tree representing the ancestor hierarchy during
compilation. It is used to check that the load time and compile time
environments are consistent.

</P>



<H3><A NAME="SEC349" HREF="sicstus_toc.html#TOC349">Parameter Transfer</A></H3>
<P>
<A NAME="IDX1497"></A>
<A NAME="IDX1498"></A>
<A NAME="IDX1499"></A>
<A NAME="IDX1500"></A>

</P>
<P>
As can be seen in the expanded methods above, the second additional
argument is simply ignored if the object has no parameter. In contrast
regard the following objects:

</P>

<PRE>
ellipse(RX,RY,Color) :: {
        color(Color) &#38;
        area(A) :-
                :(A is RX*RY*3.14159265)
                        }.

circle(R,Color) :: {
        super(ellipse(R,R,Color))
                   }.

red_circle(R) :: {
        super(circle(R,red))
                 }.
</PRE>

<P>
... and their expansions:

</P>

<PRE>

ellipse(_, _, _):'$so_type'(ellipse(_,_,_), static).

ellipse(_, _, _):area(A, _, B) :-
        B:'$fix_param'(ellipse(C,D,_), B),
        user:(A is C*D*3.14159265).

ellipse(_, _, _):color(A, _, B) :-
        B:'$fix_param'(ellipse(_,_,A), B).

ellipse(_, _, _):'$fix_param'(ellipse(B,C,D), A) :-
        objects:object_class(ellipse(B,C,D), A).

circle(_, _):'$so_type'(circle(_,_), static).

circle(_, _):super(ellipse(A,A,B), [], _, circle(A,B)).

circle(_, _):'$fix_param'(circle(B,C), A) :-
        objects:object_class(circle(B,C), A).
circle(_, _):'$fix_param'(ellipse(B,B,C), A) :-
        objects:object_class(circle(B,C), A).

red_circle(_):'$so_type'(red_circle(_), static).

red_circle(_):super(circle(A,red), [], _, red_circle(A)).

red_circle(_):'$fix_param'(red_circle(B), A) :-
        objects:object_class(red_circle(B), A).
red_circle(_):'$fix_param'(circle(B,red), A) :-
        objects:object_class(red_circle(B), A).
red_circle(_):'$fix_param'(ellipse(B,B,red), A) :-
        objects:object_class(red_circle(B), A).
</PRE>

<P>
The second additional argument contains the receiver of a method
call. If the method makes use of any parameter of the object where it is
defined, it places a call to the reserved predicate <CODE>$fix_param/2</CODE>
in the module of the receiver. The purpose of this call is to bind the
parameters used in the method to appropriate values given by the
receiver. The receiver may be the object where the method is defined or
any of its subs. In order to service these calls, a clause of
<CODE>$fix_param/2</CODE> is generated for each ancestor having
parameters. Such a clause may be regarded as the collapsed chain of
<CODE>super/(1-2)</CODE> definitions leading up to the ancestor.

</P>
<P>
The call <CODE>objects:object_class(<VAR>Class</VAR>,<VAR>Object</VAR>)</CODE> serves to
pick up the <CODE>'$class'/1</CODE> attribute if <VAR>Object</VAR> is an instance,
otherwise <VAR>Class</VAR> is unified with <VAR>Object</VAR>.

</P>
<P>
The following trace illustrates how parameters are transfered:

</P>

<PRE>
| ?- red_circle(2.5)::area(A).
1  1  Call: red_circle(2.5)::area(_A) ? 
2  2  Call: ellipse(_,_,_):area(_A,red_circle(2.5),red_circle(2.5)) ? 
3  3  Call: red_circle(_):$fix_param(ellipse(_B,_,_),red_circle(2.5)) ? 
4  4  Call: objects:object_class(red_circle(_B),red_circle(2.5)) ? 
4  4  Exit: objects:object_class(red_circle(2.5),red_circle(2.5)) ? 
3  3  Exit: red_circle(_):$fix_param(ellipse(2.5,2.5,red),red_circle(2.5)) ? 
5  3  Call: _A is 2.5*2.5*3.14159265 ? 
5  3  Exit: 19.6349540625 is 2.5*2.5*3.14159265 ? 
2  2  Exit: ellipse(_,_,_):area(19.6349540625,red_circle(2.5),red_circle(2.5)) ? 
1  1  Exit: red_circle(2.5)::area(19.6349540625) ? 

A = 19.6349540625 ? 
</PRE>



<H2><A NAME="SEC350" HREF="sicstus_toc.html#TOC350">Examples</A></H2>


<UL>
<LI><A HREF="sicstus_35.html#SEC351">Classification of Birds</A>
<LI><A HREF="sicstus_35.html#SEC352">Inheritance and Delegation</A>
<LI><A HREF="sicstus_35.html#SEC353">PrologPlusPlus Pgms</A>
</UL>



<H3><A NAME="SEC351" HREF="sicstus_toc.html#TOC351">Classification of Birds</A></H3>
<P>
This example illustrates how Prolog object can be used in classification of
certain concepts.  This style is common in expert system application for
describing its domain.

</P>

<PRE>
animal :: {
        super(object) &#38;
        relative_size(S) :-
                size(Obj_size),
                super(Obj_prototype),
                Obj_prototype :: size(Prototype_size),
                :(S is Obj_size/Prototype_size * 100)
                }.

bird :: {
        super(animal) &#38;
        moving_method(fly) &#38;
        active_at(daylight)
        }.

albatross :: {
        super(bird) &#38;
        color(black_and_white) &#38;
        size(115)
        }.

kiwi :: {
        super(bird) &#38;
        moving_method(walk) &#38;
        active_at(night) &#38;
        size(40) &#38;
        color(brown)
        }.

albert :: {
        super(albatross) &#38;
        size(120)
        }.

ross :: {
        super(albatross) &#38;
        size(40)
        }.

| ?- ross :: relative_size(R).

R = 34.78
</PRE>



<H3><A NAME="SEC352" HREF="sicstus_toc.html#TOC352">Inheritance and Delegation</A></H3>
<P>
The following example illustrates a number of concepts.  Firstly, how to use
Prolog Objects for defining traditional classes a la Smalltalk, or
other traditional object oriented languages.  Secondly, how to create
instances of these classes.  Finally, how to access <EM>instance variables</EM>.

</P>
<P>
<A NAME="IDX1501"></A>
<A NAME="IDX1502"></A>
The concept of instance variables is readily available as the variables
belonging to the instances created dynamically and not to the class of
the instances.  For example, each instance of the class <CODE>point</CODE>
will have two instance variables, <CODE>x</CODE> and <CODE>y</CODE>, represented by
the attributes <CODE>x/1</CODE> and <CODE>y/1</CODE>. The traditional class
variables are easily available by accessing the same attributes in the
associated class.

</P>
<P>
Another issue is the pattern used to create new instances.  For example,
to create an instance of the class <CODE>history_point</CODE>, the following
code is used:

</P>

<PRE>
        new(Instance, xy(IX,IY)) :-
                super &#60;: new(Instance, xy(IX,IY)),
                Instance :: set(history([(IX,IY)])) &#38;
</PRE>

<P>
Note that the delegation of <CODE>new/2</CODE> to <CODE>super</CODE> is necessary in
order to create an object whose super is <CODE>history_point</CODE> and not
<CODE>point</CODE>.

</P>
<P>
The example shows how delegation can be effective as a tool for flexible
sharing of concepts in multiple inheritance.  Four prototypes 
are defined: <CODE>point</CODE>, <CODE>history_point</CODE>, <CODE>bounded_point</CODE>,
and <CODE>bh_point</CODE>.  The latter is a bounded history point.

</P>
<P>
An instance of the <CODE>point</CODE> class is a point that moves in 2-D space
and that can be displayed.  An instance of the <CODE>history_point</CODE>
class is similar to an instance of the <CODE>point</CODE> class but also keeps
a history of all the moves made so far.  An instance of
<CODE>bounded_point</CODE> is similar to an instance of <CODE>point</CODE> but moves
only in a region of the 2-D space.  Finally an instance of
<CODE>bh_point</CODE> inherits most of the features of a <CODE>bounded_point</CODE>
and a <CODE>history_point</CODE>.

</P>
<P>
The default inheritance does not work for the methods <CODE>display/1</CODE>
and <CODE>move/2</CODE> in <CODE>bh_point</CODE>.  Inheritance by delegating
messages to both supers of <CODE>bh_point</CODE> results in redundant actions,
(moving and displaying the point twice).  Selective delegation solves
the problem.  Taken from [Elshiewy 90].

</P>

<PRE>
point :: {
        super(object) &#38;

        attributes([x(0),y(0)]) &#38;

        xy(X, Y) :- get(x(X)), get(y(Y)) &#38;

        new(Instance, xy(IX,IY)) :-
                super &#60;: instance(Instance),
                Instance :: set(x(IX)),
                Instance :: set(y(IY)) &#38;

        location((X,Y)) :- &#60;: xy(X,Y) &#38;

        move_horizontal(X) :-
                set(x(X)) &#38;

        move_vertical(Y) :-
                set(y(Y)) &#38;

        move(X, Y) :-
                &#60;: move_horizontal(X),
                &#60;: move_vertical(Y) &#38;

        display(Terminal) :-
                &#60;: xy(X, Y),
                Terminal :: format('point at (~d,~d)~n',[X,Y]) 
        }.

history_point :: {
        super(point) &#38;

        attributes([history([])]) &#38;

        new(Instance, xy(IX,IY)) :-
                super &#60;: new(Instance, xy(IX,IY)),
                Instance :: set(history([(IX,IY)])) &#38;
        
        move(X, Y) :-
                super &#60;: move(X, Y),
                get(history(History)),
                set(history([(X,Y)|History])) &#38;
        
        display(Terminal) :-
                super &#60;: display(Terminal),
                &#60;: print_history(Terminal) &#38;

        print_history(Terminal) :-
                get(history(History)),
                Terminal :: format('with location history ~w~n', 
                                   [History])
        }.

bounded_point :: {
        super(point) &#38;

        attributes([bounds(0,0,0,0)]) &#38;
        
        new(Instance, Coords, Bounds) :-
                super &#60;: new(Instance, Coords),
                Instance :: set_bounds(Bounds) &#38;

        set_bounds(Bounds) :-
                set(Bounds) &#38;

        move(X, Y) :-
                &#60;: bound_constraint(X, Y), !,
                super &#60;: move(X, Y) &#38;
        move(_, _) &#38;

        bound_constraint(X, Y) :-
                get(bounds(X0, X1, Y0, Y1)),
                :(X &#62;= X0),
                :(X =&#60; X1),
                :(Y &#62;= Y0),
                :(Y =&#60; Y1) &#38;
        
        display(Terminal) :-
                super &#60;: display(Terminal),
                &#60;: print_bounds(Terminal) &#38;

        print_bounds(Terminal) :- 
                get(bounds(X0, X1, Y0, Y1)),
                Terminal :: format('xbounds=(~d,~d), \c
                                    ybounds=(~d,~d)~n',
                                   [X0,X1,Y0,Y1])
        }.

bh_point :: {
        super(history_point) &#38;
        super(bounded_point) &#38;

        new(Instance, Coords, Bounds) :-
                history_point &#60;: new(Instance, Coords),
                Instance :: set_bounds(Bounds) &#38;

        move(X, Y) :-
                bounded_point &#60;: bound_constraint(X, Y), !,
                history_point &#60;: move(X, Y) &#38;
        move(_, _) &#38;

        display(Terminal) :-
                bounded_point &#60;: display(Terminal),
                history_point &#60;: print_history(Terminal)
        }.

tty :: {
        format(X, Y) :- :format(X, Y)
       }.

point at (8,12)
xbounds=(5,15), ybounds=(5,15)
with location history [(8,12),(9,11)]
</PRE>



<H3><A NAME="SEC353" HREF="sicstus_toc.html#TOC353">Prolog++ programs</A></H3>
<P>
Prolog++ is a product by LPA Associates for object-oriented programming
extensions of LPA Prolog.  Most Prolog++ programs can be easily
converted into Prolog Objects programs.  The following is a translation
of a program for fault diagnosis in LPA's Prolog++ manual, page 83.  The
program illustrates a top-down diagnosis method starting from general
objects to more specific objects.  The problem is fault diagnosis for
car maintenance. The objects have the following structure:

</P>

<PRE>
- faults
        - electrical
        |       - lights
        |       - starting
        |               - starter_motor
        |               - sparking
        |                       - plugs
        |                       - distributer
        - fuel_system
        - mechanical
</PRE>

<P>
The general diagnosis method is defined in the object <CODE>faults</CODE>,
whereas the cause-effect relationships are defined in the specific
objects e.g. the object <CODE>distributor</CODE>.

</P>
<P>
This program heavily uses the <CODE>sub/1</CODE> method.  We have tried to be
as close as possible to the original formulation.

</P>

<PRE>

faults :: {

        super(utility) &#38;
        dynamic(told/2) &#38;

        /* no fault is the default */
        fault(_, _) :- :fail &#38;

        findall :-
                &#60;: restart,
                :: sub(Sub),
                Sub :: find(Where, Fault),
                &#60;: print(Where, Fault),
                :fail &#38;
        findall &#38;

        print(Where, Fault) :-
                :writeseqnl('Location       : ', [Where]),
                :writeseqnl('Possible Fault : ', [Fault]),
                :nl &#38;

        find(Where, Fault) :-
                self(Where),
                fault(FaultNum, Fault),
                \+ (effect(FaultNum, S),
                    contrary(S, S1),
                    exhibited(S1)
                   ),
                \+ (effect(FaultNum, SymptomNum),
                    \+ exhibited(SymptomNum)) &#38;

        find(Where, Fault) :-
                sub(Sub),
                Sub :: find(Where, Fault) &#38;     

        exhibited(S) :-
                :: told(S, R), !,
                R = yes &#38;
        exhibited(S) :-
                symptom(S,Text),
                (   :yesno([Text]) -&#62; R = yes
                ;   R = no
                ),
                :: asserta(told(S,R)),
                R = yes &#38;

        restart :-
                :: retractall(told(_,_))

        }.

electrical :: {
        super(faults)
        }.

fuel_system :: {
        super(faults)
        }.

mechanical :: {
        super(faults)
        }.

lights :: {
        super(electrical)
        }.

sparking :: {
        super(electrical)
        }.

starting :: {
        super(electrical)
        }.

starter_motor :: {
        super(electrical)
        }.

plugs :: {
        super(sparking)
        }.

engine :: {
        super(mechanical)
        }.

cylinders :: {
        super(engine)
        }.

distributor :: {
        super(sparking) &#38;

        /* faults */
        fault('F1001', 'Condensation in distributor cap') &#38;
        fault('F1002', 'Faulty distributor arm') &#38;
        fault('F1003', 'Worn distributor brushes') &#38;

        /* symptoms */
        symptom('S1001', 'Starter turns, but engine does not fire') &#38;
        symptom('S1002', 'Engine has difficulty starting') &#38;
        symptom('S1003', 'Engine cuts out shortly after starting') &#38;
        symptom('S1004', 'Engine cuts out at speed') &#38;

        /* symptoms contrary to each other */
        contrary('S1002', 'S1001') &#38;
        contrary('S1003', 'S1001') &#38;

        /* causal-effect relationship */
        effect('F1001', 'S1001') &#38;
        effect('F1002', 'S1001') &#38;
        effect('F1002', 'S1004') &#38;
        effect('F1003', 'S1002') &#38;
        effect('F1003', 'S1003')

        }.

yesno(Value) :- write(Value), nl, read(yes).
        
writeseqnl(Prompt, L) :- write(Prompt), write_seq(L).
        
write_seq([]).
write_seq([X|L]) :- write(X), write(' '), write_seq(L), nl.

faults :- faults :: findall.

| ?- faults.
[Starter turns, but engine does not fire]
|: yes.
Location       : distributor 
Possible Fault : Condensation in distributor cap 

[Engine cuts out at speed]
|: yes.
Location       : distributor 
Possible Fault : Faulty distributor arm 

yes
| ?- faults.
[Starter turns, but engine does not fire]
|: no.
[Engine has difficulty starting]
|: yes.
[Engine cuts out shortly after starting]
|: yes.
Location       : distributor 
Possible Fault : Worn distributor brushes
</PRE>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_34.html">previous</A>, <A HREF="sicstus_36.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
