<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Tcl/Tk Interface</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_36.html">previous</A>, <A HREF="sicstus_38.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC355" HREF="sicstus_toc.html#TOC355">Tcl/Tk Interface</A></H1>

<P>
The <CODE>tcltk</CODE> library package is a bidirectional interface to the
<EM>Tcl</EM> (pronounced Tickle) language and the <EM>Tk</EM> toolkit. Tcl is
an interpreted scripting language with many extension packages, in
particular the graphical interface toolkit Tk.

</P>
<P>
You can read about Tcl/Tk in [Ousterhout 94] or in various articles.
The articles and the Tcl/Tk system can be found by anonymous FTP, see
the Release Notes for up-to-date information. This
file also contains information on how to include Tcl/Tk extensions.

</P>
<P>
To load the package, enter the query:

</P>

<PRE>
| ?- use_module(library(tcltk)).
</PRE>


<UL>
<LI><A HREF="sicstus_37.html#SEC356">Prolog to Tcl</A>
<LI><A HREF="sicstus_37.html#SEC358">Tcl to Prolog</A>
<LI><A HREF="sicstus_37.html#SEC359">Tk</A>
</UL>



<H2><A NAME="SEC356" HREF="sicstus_toc.html#TOC356">Prolog to Tcl</A></H2>

<P>
To use Tcl, you must create a Tcl interpreter object and send commands
to it.  The following predicate executes a single Tcl command:

</P>

<PRE>
test_command(Command, Result) :-
        tcl_new(Interp),
        tcl_eval(Interp, Command, Result),
        tcl_delete(Interp).
</PRE>

<P>
The Tcl command and its arguments is specified in <VAR>Command</VAR>
(see section <A HREF="sicstus_37.html#SEC357">Command Format</A>), and the result will be returned as a string
(list of character codes) in <VAR>Result</VAR>.

</P>
<P>
These are the predicates to use Tcl/Tk from Prolog:

</P>
<DL COMPACT>

<DT><CODE>tcl_new(<VAR>-TclInterpreter</VAR>)</CODE>
<DD>
<A NAME="IDX1505"></A>
Creates a new interpreter, initializes it and returns a reference to
it. If you don't have a standard Tcl installation you can tell Tcl where
to look for the initialization files by setting the environment variable
<CODE>TCL_LIBRARY</CODE>.

<DT><CODE>tcl_delete(<VAR>+TclInterpreter</VAR>)</CODE>
<DD>
<A NAME="IDX1506"></A>
Deletes an interpreter and the memory used by it.

<DT><CODE>tcl_eval(<VAR>+TclInterpreter</VAR>, <VAR>+Command</VAR>, <VAR>-Result</VAR>)</CODE>
<DD>
<A NAME="IDX1507"></A>
Lets <VAR>TclInterpreter</VAR> interpret the command specified by
<VAR>Command</VAR>.  The result will be stored as a string (list of character
codes) in <VAR>Result</VAR>.

<DT><CODE>tcl_event(<VAR>+TclInterpreter</VAR>, <VAR>+Command</VAR>, <VAR>-Events</VAR>)</CODE>
<DD>
<A NAME="IDX1508"></A>
This call is similar to <CODE>tcl_eval/3</CODE>. <VAR>Events</VAR> is a list of
terms stored from Tcl by the <CODE>prolog_event</CODE>
command (see section <A HREF="sicstus_37.html#SEC358">Tcl to Prolog</A>).

</DL>


<UL>
<LI><A HREF="sicstus_37.html#SEC357">Command Format</A>
</UL>



<H3><A NAME="SEC357" HREF="sicstus_toc.html#TOC357">Command Format</A></H3>

<P>
A Tcl command is specified as follows:

</P>

<PRE>
<VAR>Command</VAR>           --&#62; <VAR>Atom</VAR>           { other than <KBD>[]</KBD> }
                          | <VAR>Number</VAR>
                          | <CODE>chars(<VAR>PrologString</VAR>)</CODE>
                          | <CODE>write(<VAR>Term</VAR>)</CODE>
                          | <CODE>format(<VAR>Fmt</VAR>,<VAR>Args</VAR>)</CODE>
                          | <CODE>dq(<VAR>Command</VAR>)</CODE>
                          | <CODE>br(<VAR>Command</VAR>)</CODE>
                          | <CODE>sqb(<VAR>Command</VAR>)</CODE>
                          | <CODE>min(<VAR>Command</VAR>)</CODE>
                          | <VAR>ListOfCommands</VAR>

<VAR>ListOfCommands</VAR>    --&#62; <CODE>[]</CODE>
                          | <CODE>[<VAR>ListExpr</VAR>]</CODE>

<VAR>ListExpr</VAR>          --&#62; <VAR>Command</VAR>
                          | <CODE><VAR>Command</VAR>,<VAR>ListExpr</VAR></CODE>
                          | <CODE><VAR>Command</VAR>|<VAR>ListOfCommands</VAR></CODE>
</PRE>

<P>
where

</P>
<DL COMPACT>

<DT><CODE><VAR>Atom</VAR></CODE>
<DD>
<DT><CODE><VAR>Number</VAR></CODE>
<DD>
denote their printed representations.

<DT><CODE>chars(<VAR>PrologString</VAR>)</CODE>
<DD>
denotes the string represented by <VAR>PrologString</VAR> (a list of
character codes).

<DT><CODE>write(<VAR>Term</VAR>)</CODE>
<DD>
denotes the string that is printed by the corresponding built-in
predicate.

<DT><CODE>format(<VAR>Fmt</VAR>,<VAR>Args</VAR>)</CODE>
<DD>
denotes the string that is printed by the corresponding built-in
predicate.

<DT><CODE>dq(<VAR>Command</VAR>)</CODE>
<DD>
denotes the string specified by <VAR>Command</VAR>, enclosed in double
quotes.

<DT><CODE>br(<VAR>Command</VAR>)</CODE>
<DD>
denotes the string specified by <VAR>Command</VAR>, enclosed in braces.

<DT><CODE>sqb(<VAR>Command</VAR>)</CODE>
<DD>
denotes the string specified by <VAR>Command</VAR>, enclosed in square
brackets.

<DT><CODE>min(<VAR>Command</VAR>)</CODE>
<DD>
denotes the string specified by <VAR>Command</VAR>, immediately preceded by
a hyphen.

<DT><CODE><VAR>ListOfCommands</VAR></CODE>
<DD>
denotes the strings denoted by each element, separated by spaces.
</DL>

<P>
Examples of command specifications and corresponding translations:

</P>

<PRE>
'set x 32'
        ==&#62; set x 32

[set,x,br([a,b,c])]
        ==&#62; set x {a b c}

['.panel.value_info.name',configure,min(text),br(write('$display'/1))]
        ==&#62; .panel.value_info.name configure -text {$display/1}
</PRE>



<H2><A NAME="SEC358" HREF="sicstus_toc.html#TOC358">Tcl to Prolog</A></H2>

<P>
There are two new Tcl commands defined in this library:

</P>
<DL COMPACT>

<DT><CODE>prolog <VAR>Goal</VAR></CODE>
<DD>
<VAR>Goal</VAR> is a string containing the printed representation of a Prolog
goal. The goal will be called in the <CODE>user</CODE> module unless it is
prefixed with another module name. The call is always deterministic.
The result of the call is returned as:

<DL COMPACT>

<DT><CODE>"1"</CODE>
<DD>
if execution succeeds. The value of any of the variables in the goal
that is bound to a term will be returned to Tcl in the array
<CODE>prolog_variables</CODE> with the variable name as index.

The term is converted to Tcl using the same conversion as used for Tcl
commands (see section <A HREF="sicstus_37.html#SEC357">Command Format</A>). 

<DT><CODE>"0"</CODE>
<DD>
if the execution fails.

<DT><CODE><VAR>Tcl exception</VAR></CODE>
<DD>
if a Prolog exception is raised. The error message will be "<CODE>Prolog
Exception: </CODE>" appended with a string representation of the Prolog
exception.
</DL>

An example:


<PRE>
test_callback(Result) :-
        tcl_new(Interp),
        tcl_eval(Interp,
                 'if {[prolog "foo(X,Y,Z)"] == 1} \\
                     {lappend res $prolog_variables(X) \\
                                  $prolog_variables(Y) \\
                                  $prolog_variables(Z)}',
                 Result),
        tcl_delete(Interp).

foo(1, bar, [a,b,c]).
</PRE>

The query

<PRE>
| ?- test_callback(Result).
</PRE>

will succeed, binding <VAR>Result</VAR> to: 


<PRE>
        "1 bar {a b c}"
</PRE>

<DT><CODE>prolog_event <VAR>Terms...</VAR></CODE>
<DD>
<VAR>Terms</VAR> are strings that contain the printed representation of
Prolog terms. These are stored in a queue and retrieved as Prolog terms
by <CODE>tcl_event/3</CODE> or <CODE>tk_next_event/(2,3)</CODE>.
</DL>

<P>
An example:

</P>

<PRE>
test_event(Event) :-
        tcl_new(Interp),
        tcl_event(Interp, [prolog_event,dq(write(zap(42)))], Event),
        tcl_delete(Interp).
</PRE>

<P>
The query

<PRE>
| ?- test_event(Event).
</PRE>

<P>
will succeed, binding <VAR>Event</VAR> to the list <CODE>[zap(42)]</CODE>

</P>



<H2><A NAME="SEC359" HREF="sicstus_toc.html#TOC359">Tk</A></H2>

<P>
The following example is a simple "Hello, world" program:

</P>

<PRE>
tk_hello_world :-
        tk_new([], Interp),
        tcl_eval(Interp, 
                 'button .b -text "Hello" -command "destroy ."', _),
        tcl_eval(Interp, 'pack .b', _),
        tk_main_loop,
        tcl_delete(Interp).
</PRE>

<P>
The program brings up a tiny window with a button labeled "Hello" and
is terminated by pressing the button.

</P>
<P>
These are the predicates to use Tk from Prolog:

</P>
<DL COMPACT>

<DT><CODE>tk_new(<VAR>+Options</VAR>, <VAR>-TclInterpreter</VAR>)</CODE>
<DD>
<A NAME="IDX1509"></A>
Performs basic Tcl and Tk initialization and creates the main window of
a Tk application. If you don't have a standard Tk installation you can
tell Tk where to look for the initialization files by setting the
environment variable <CODE>TK_LIBRARY</CODE>. <VAR>Options</VAR> is a list of
optional elements according to:

<DL COMPACT>

<DT><CODE>top_level_events</CODE>
<DD>
Event handling will be installed in the Prolog top-level loop. This
means that Tk windows and the Prolog prompt can be simultaneously
active.

<DT><CODE>name(<VAR>+ApplicationName</VAR>)</CODE>
<DD>
Sets the Tk application name. The application name will be displayed in
the main window and is also used for communicating between applications
in Tk. Default name is an empty string.

<DT><CODE>display(<VAR>+Display</VAR>)</CODE>
<DD>
Gives the name of the screen on which to create the main window. Default
is normally determined by the <CODE>DISPLAY</CODE> environment variable.

</DL>

<DT><CODE>tk_main_window(<VAR>+TclInterpreter</VAR>, <VAR>-TkWindow</VAR>)</CODE>
<DD>
<A NAME="IDX1510"></A>
Gets a reference to the main window of the application
using this interpreter.

<DT><CODE>tk_make_window_exist(<VAR>+TkWindow</VAR>)</CODE>
<DD>
<A NAME="IDX1511"></A>
This will force a window to be mapped to the screen,
i.e. it will force it to show up immediately.
Normally, mapping is delayed as long as possible.

<DT><CODE>tk_destroy_window(<VAR>+TkWindow</VAR>)</CODE>
<DD>
<A NAME="IDX1512"></A>
Destroys a window or widget.

<DT><CODE>tk_num_main_windows(<VAR>-NumberOfWindows</VAR>)</CODE>
<DD>
<A NAME="IDX1513"></A>
Returns the number of main windows currently in use.

<DT><CODE>tk_do_one_event</CODE>
<DD>
<DT><CODE>tk_do_one_event(<VAR>+ListOrBitmask</VAR>)</CODE>
<DD>
<A NAME="IDX1514"></A>
<A NAME="IDX1515"></A>
Passes control to Tk for handling a single event.  This predicate may be
called to give Tk time for handling the windows. You can specify what
events to handle in a <VAR>List</VAR> of event flags.  Alternatively a
<VAR>Bitmask</VAR> as specified in the Tcl/Tk documentation can be used but
should be avoided for portability between Tcl/Tk versions. The flags you
can have in <VAR>List</VAR> are:
<DL COMPACT>

<DT><CODE>tk_dont_wait</CODE>
<DD>
<A NAME="IDX1516"></A>
Don't wait for new events, process only events that are ready.

<DT><CODE>tk_x_events</CODE>
<DD>
<DT><CODE>tk_window_events</CODE>
<DD>
<A NAME="IDX1517"></A>
<A NAME="IDX1518"></A>
Process window events.

<DT><CODE>tk_file_events</CODE>
<DD>
<A NAME="IDX1519"></A>
Process file events.

<DT><CODE>tk_timer_events</CODE>
<DD>
<A NAME="IDX1520"></A>
Process timer events.

<DT><CODE>tk_idle_events</CODE>
<DD>
<A NAME="IDX1521"></A>
Process Tk_DoWhenIdle callbacks.

<DT><CODE>tk_all_events</CODE>
<DD>
<A NAME="IDX1522"></A>
Process any event.

</DL>

<CODE>tk_do_one_event/0</CODE> is equivalent to <CODE>tk_do_one_event/1</CODE> with
all flags set. If the <CODE>tk_dont_wait</CODE> flag is set and there is no
event to handle, the call will fail.

It is straight-forward to define a predicate which handles
all events in the queue and then returns:


<PRE>
tk_do_all_events :-
        tk_do_one_event, !,
        tk_do_all_events.
tk_do_all_events.
</PRE>

<CODE>tk_do_one_event/(0,1)</CODE> is an interface to the C-function
<CODE>TkDoOneEvent()</CODE> (or <CODE>TclDoOneEvent()</CODE> in later Tcl/Tk
versions).

<DT><CODE>tk_next_event(<VAR>+TclInterpreter</VAR>, <VAR>-Event</VAR>)</CODE>
<DD>
<DT><CODE>tk_next_event(<VAR>+ListOrBitmask</VAR>, <VAR>+TclInterpreter</VAR>, <VAR>-Event</VAR>)</CODE>
<DD>
<A NAME="IDX1523"></A>
<A NAME="IDX1524"></A>
Processes events until there is at least one Prolog event associated
with <VAR>TclInterpreter</VAR>. <VAR>Event</VAR> is the term corresponding to the
head of a queue of events (strings representing terms) stored from Tcl
with the <CODE>prolog_event</CODE> command (see section <A HREF="sicstus_37.html#SEC358">Tcl to Prolog</A>). Only
events associated with <VAR>TclInterpreter</VAR> are returned; other events
remain in the queue. If there are no windows left the term <CODE>[]</CODE>
will be returned.  This predicate does not correspond directly to any
Tcl/Tk C function.

<DT><CODE>tk_main_loop</CODE>
<DD>
<A NAME="IDX1525"></A>
Passes control to Tk until all windows are gone.

</DL>

<P>
There are basically two alternatives for invoking Prolog actions on user
events. The first is to directly call Prolog by means of the Tcl command
<CODE>prolog</CODE>. Tcl/Tk must then be repeatedly invoked, either by calling
<CODE>tk_main_loop/0</CODE> or using the option <CODE>top_level_events</CODE>,
possibly in conjunction with calling <CODE>tk_do_one_event/(0,1)</CODE> in
lengthy computations.

</P>
<P>
The second alternative is using the Tcl command <CODE>prolog_event</CODE>. The
user program then, by calling <CODE>tk_next_event/(2,3)</CODE>, passes control
to Tcl/Tk until one or more invocations of <CODE>prolog_event</CODE> has
occurred. The Prolog event determines the Prolog action to take. This
approach has the advantage that context variables might be passed around
in the event loop.

</P>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_36.html">previous</A>, <A HREF="sicstus_38.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
