<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Full Prolog Syntax</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_44.html">previous</A>, <A HREF="sicstus_46.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC367" HREF="sicstus_toc.html#TOC367">Full Prolog Syntax</A></H1>

<P>
A Prolog program consists of a sequence of <EM>sentences</EM> or lists of
sentences.  Each sentence is a Prolog <EM>term</EM>.  How terms are
interpreted as sentences is defined below (see section <A HREF="sicstus_45.html#SEC369">Syntax of Sentences as Terms</A>).  Note that
a term representing a sentence may be written in any of its equivalent
syntactic forms.  For example, the 2-ary functor <SAMP>`:-'</SAMP> could be
written in standard prefix notation instead of as the usual infix
operator.  
</P>
<P>
Terms are written as sequences of <EM>tokens</EM>.  Tokens are sequences of
characters which are treated as separate symbols.  Tokens include the
symbols for variables, constants and functors, as well as punctuation
characters such as brackets and commas.

</P>
<P>
We define below how lists of tokens are interpreted as terms
(see section <A HREF="sicstus_45.html#SEC370">Syntax of Terms as Tokens</A>).  Each list of tokens which is read in (for
interpretation as a term or sentence) has to be terminated by a
full-stop token.  Two tokens must be separated by a layout-text token if
they could otherwise be interpreted as a single token.  Layout-text
tokens are ignored when interpreting the token list as a term, and may
appear at any point in the token list.  
</P>
<P>
We define below defines how tokens are represented as strings of
characters (see section <A HREF="sicstus_45.html#SEC371">Syntax of Tokens as Character Strings</A>).  But we start by describing the
notation used in the formal definition of Prolog syntax (see section <A HREF="sicstus_45.html#SEC368">Notation</A>).  
</P>

<UL>
<LI><A HREF="sicstus_45.html#SEC368">Syntax Notation</A>
<LI><A HREF="sicstus_45.html#SEC369">Sentence</A>
<LI><A HREF="sicstus_45.html#SEC370">Term Token</A>
<LI><A HREF="sicstus_45.html#SEC371">Token String</A>
<LI><A HREF="sicstus_45.html#SEC372">Escape Sequences</A>
<LI><A HREF="sicstus_45.html#SEC373">Syntax Notes</A>
</UL>



<H2><A NAME="SEC368" HREF="sicstus_toc.html#TOC368">Notation</A></H2>


<OL>
<LI>

Syntactic categories (or <EM>non-terminals</EM>) are written thus:
<VAR>item</VAR>.  Depending on the section, a category may represent a class
of either terms, token lists, or character strings.  <LI>

A syntactic rule takes the general form 


<PRE>
<VAR>C</VAR> --&#62; <VAR>F1</VAR> | <VAR>F2</VAR> | <VAR>F3</VAR>
</PRE>

which states that an entity of category <VAR>C</VAR> may take any of the
alternative forms <VAR>F1</VAR>, <VAR>F2</VAR>, <VAR>F3</VAR>, etc.  <LI>

Certain definitions and restrictions are given in ordinary English,
enclosed in { } brackets.  <LI>

A category written as <VAR>C...</VAR> denotes a sequence of one or more
<VAR>C</VAR>s.  <LI>

A category written as <VAR>?C</VAR> denotes an optional <VAR>C</VAR>.
Therefore <VAR>?C...</VAR> denotes a sequence of zero or more
<VAR>C</VAR>s.<LI>

A few syntactic categories have names with arguments, and rules in which
they appear may contain meta-variables looking thus: <VAR>X</VAR>.  The meaning
of such rules should be clear from analogy with the definite clause
grammars (see section <A HREF="sicstus_10.html#SEC95">Term and Goal Expansion</A>).  <LI>

In the section describing the syntax of terms and tokens (see section <A HREF="sicstus_45.html#SEC370">Syntax of Terms as Tokens</A>) particular tokens of the category name are written thus:
<VAR>name</VAR>, while tokens which are individual punctuation characters are
written literally.  </OL>



<H2><A NAME="SEC369" HREF="sicstus_toc.html#TOC369">Syntax of Sentences as Terms</A></H2>


<PRE>
<VAR>sentence</VAR>          --&#62; <VAR>module</VAR> <KBD>:</KBD> <VAR>sentence</VAR>
                   |  <VAR>list</VAR>
                         { where <VAR>list</VAR> is a list of <VAR>sentence</VAR> }
                   |  <VAR>clause</VAR>
                   |  <VAR>directive</VAR>
                   |  <VAR>grammar-rule</VAR>

<VAR>clause</VAR>            --&#62; <VAR>non-unit-clause</VAR> | <VAR>unit-clause</VAR>

<VAR>directive</VAR>         --&#62; <VAR>command</VAR> | <VAR>query</VAR>

<VAR>non-unit-clause</VAR>   --&#62; <VAR>head</VAR> <KBD>:-</KBD> <VAR>body</VAR>

<VAR>unit-clause</VAR>       --&#62; <VAR>head</VAR>
                         { where <VAR>head</VAR> is not otherwise a <VAR>sentence</VAR> }

<VAR>command</VAR>           --&#62; <KBD>:-</KBD> <VAR>body</VAR>

<VAR>query</VAR>             --&#62; <KBD>?-</KBD> <VAR>body</VAR>

<VAR>head</VAR>              --&#62; <VAR>module</VAR> <KBD>:</KBD> <VAR>head</VAR>
                   |  <VAR>goal</VAR>
                         { where <VAR>goal</VAR> is not a <VAR>variable</VAR> }

<VAR>body</VAR>             --&#62; <VAR>module</VAR> <KBD>:</KBD> <VAR>body</VAR>
                   |  <VAR>body</VAR> <KBD>-&#62;</KBD> <VAR>body</VAR> <KBD>;</KBD> <VAR>body</VAR>
                   |  <VAR>body</VAR> <KBD>-&#62;</KBD> <VAR>body</VAR>
                   |  <KBD>\+</KBD> <VAR>body</VAR>
                   |  <VAR>body</VAR> <KBD>;</KBD> <VAR>body</VAR>
                   |  <VAR>body</VAR> <KBD>,</KBD> <VAR>body</VAR>
                   |  <VAR>goal</VAR>

<VAR>goal</VAR>              --&#62; <VAR>term</VAR>
                         { where <VAR>term</VAR> is not otherwise a <VAR>body</VAR> }

<VAR>grammar-rule</VAR>      --&#62; <VAR>gr-head</VAR> <KBD>--&#62;</KBD> <VAR>gr-body</VAR>

<VAR>gr-head</VAR>           --&#62; <VAR>module</VAR> <KBD>:</KBD> <VAR>gr-head</VAR>
                   |  <VAR>gr-head</VAR> <KBD>,</KBD> <VAR>terminals</VAR>
                   |  <VAR>non-terminal</VAR>
                         { where <VAR>non-terminal</VAR> is not a <VAR>variable</VAR> }

<VAR>gr-body</VAR>           --&#62; <VAR>module</VAR> <KBD>:</KBD> <VAR>gr-body</VAR>
                   |  <VAR>gr-body</VAR> <KBD>-&#62;</KBD> <VAR>gr-body</VAR> <KBD>;</KBD> <VAR>gr-body</VAR>
                   |  <VAR>gr-body</VAR> <KBD>-&#62;</KBD> <VAR>gr-body</VAR>
                   |  <KBD>\+</KBD> <VAR>gr-body</VAR>
                   |  <VAR>gr-body</VAR> <KBD>;</KBD> <VAR>gr-body</VAR>
                   |  <VAR>gr-body</VAR> <KBD>,</KBD> <VAR>gr-body</VAR>
                   |  <VAR>non-terminal</VAR>
                   |  <VAR>terminals</VAR>
                   |  <VAR>gr-condition</VAR>

<VAR>non-terminal</VAR>      --&#62; <VAR>term</VAR>
                         { where <VAR>term</VAR> is not otherwise a <VAR>gr-body</VAR> }

<VAR>terminals</VAR>         --&#62; <VAR>list</VAR> | <VAR>string</VAR>

<VAR>gr-condition</VAR>      --&#62; <KBD>!</KBD> | <KBD>{</KBD> <VAR>body</VAR> <KBD>}</KBD>

<VAR>module</VAR>            --&#62; <VAR>atom</VAR>
</PRE>



<H2><A NAME="SEC370" HREF="sicstus_toc.html#TOC370">Syntax of Terms as Tokens</A></H2>


<PRE>
<VAR>term-read-in</VAR>      --&#62; <VAR>subterm(1200)</VAR> <VAR>full-stop</VAR>

<VAR>subterm(N)</VAR>        --&#62; <VAR>term(M)</VAR>
                         { where <VAR>M</VAR> is less than or equal to <VAR>N</VAR> }

<VAR>term(N)</VAR>           --&#62; <VAR>op(N,fx)</VAR> <VAR>subterm(N-1)</VAR>
                         { except in the case of a <VAR>number</VAR> }
                         { if <VAR>subterm</VAR> starts with a <KBD>(</KBD>,
                           <VAR>op</VAR> must be followed by <VAR>layout-text</VAR> }
                   |  <VAR>op(N,fy)</VAR> <VAR>subterm(N)</VAR>
                         { if <VAR>subterm</VAR> starts with a <KBD>(</KBD>,
                           <VAR>op</VAR> must be followed by <VAR>layout-text</VAR> }
                   |  <VAR>subterm(N-1)</VAR> <VAR>op(N,xfx)</VAR> <VAR>subterm(N-1)</VAR>
                   |  <VAR>subterm(N-1)</VAR> <VAR>op(N,xfy)</VAR> <VAR>subterm(N)</VAR>
                   |  <VAR>subterm(N)</VAR> <VAR>op(N,yfx)</VAR> <VAR>subterm(N-1)</VAR>
                   |  <VAR>subterm(N-1)</VAR> <VAR>op(N,xf)</VAR>
                   |  <VAR>subterm(N)</VAR> <VAR>op(N,yf)</VAR>

<VAR>term(1000)</VAR>        --&#62; <VAR>subterm(999)</VAR> <KBD>,</KBD> <VAR>subterm(1000)</VAR>

<VAR>term(0)</VAR>           --&#62; <VAR>functor</VAR> <KBD>(</KBD> <VAR>arguments</VAR> <KBD>)</KBD>
                         { provided there is no <VAR>layout-text</VAR> between
                           the <VAR>functor</VAR> and the <KBD>(</KBD> }
                   |  <KBD>(</KBD> <VAR>subterm(1200)</VAR> <KBD>)</KBD>
                   |  <KBD>{</KBD> <VAR>subterm(1200)</VAR> <KBD>}</KBD>
                   |  <VAR>list</VAR>
                   |  <VAR>string</VAR>
                   |  <VAR>constant</VAR>
                   |  <VAR>variable</VAR>

<VAR>op(N,T)</VAR>           --&#62; <VAR>name</VAR>
                         { where <VAR>name</VAR> has been declared as an
                           operator of type <VAR>T</VAR> and precedence <VAR>N</VAR> }

<VAR>arguments</VAR>         --&#62; <VAR>subterm(999)</VAR>
                   |  <VAR>subterm(999)</VAR> <KBD>,</KBD> <VAR>arguments</VAR>

<VAR>list</VAR>              --&#62; <KBD>[]</KBD>
                   |  <KBD>[</KBD> <VAR>listexpr</VAR> <KBD>]</KBD>

<VAR>listexpr</VAR>          --&#62; <VAR>subterm(999)</VAR>
                   |  <VAR>subterm(999)</VAR> <KBD>,</KBD> <VAR>listexpr</VAR>
                   |  <VAR>subterm(999)</VAR> <KBD>|</KBD> <VAR>subterm(999)</VAR>

<VAR>constant</VAR>          --&#62; <VAR>atom</VAR> | <VAR>number</VAR>

<VAR>number</VAR>            --&#62; <VAR>unsigned-number</VAR>
                   |  <VAR>sign</VAR> <VAR>unsigned-number</VAR>
                   |  <VAR>sign</VAR> <KBD>inf</KBD>
                   |  <VAR>sign</VAR> <KBD>nan</KBD>

<VAR>unsigned-number</VAR>   --&#62; <VAR>natural-number</VAR> | <VAR>unsigned-float</VAR>

<VAR>atom</VAR>              --&#62; <VAR>name</VAR>

<VAR>functor</VAR>           --&#62; <VAR>name</VAR>
</PRE>



<H2><A NAME="SEC371" HREF="sicstus_toc.html#TOC371">Syntax of Tokens as Character Strings</A></H2>

<P>
<A NAME="IDX1553"></A>
By default, SICStus Prolog uses the ISO 8859/1 character set standard, but will
alternatively support the EUC (Extended UNIX Code) standard.  This is
governed by the value of the environment variable <CODE>SP_CTYPE</CODE>
(see section <A HREF="sicstus_4.html#SEC11">Getting Started</A>).

</P>
<P>
The character categories used below are defined as follows in the two
standards:

</P>
<DL COMPACT>

<DT><VAR>layout-char</VAR>
<DD>
In ISO 8859/1, these are character codes 0..32 and 127..159.  In EUC,
these are character codes 0..32 and 127.  The common subset includes
characters such as <KBD>TAB</KBD>, <KBD>LFD</KBD>, and <KBD>SPC</KBD>.

<DT><VAR>small-letter</VAR>
<DD>
In ISO 8859/1, these are character codes 97..122, 223..246, and 248..255.
In EUC, these are character codes 97..122 and 128..255.  The common subset
includes the letters <KBD>a</KBD> through <KBD>z</KBD>.

<DT><VAR>capital-letter</VAR>
<DD>
In ISO 8859/1, these are character codes 65..90, 192..214, and 216..222.  In
EUC, these are character codes 65..90.  The common subset is the letters
<KBD>A</KBD> through <KBD>Z</KBD>.

<DT><VAR>digit</VAR>
<DD>
In both standards, these are character codes 48..57, i.e. the digits
<KBD>0</KBD> through <KBD>9</KBD>.

<DT><VAR>symbol-char</VAR>
<DD>
In ISO 8859/1, these are character codes 35, 36, 38, 42, 43, 45..47, 58,
60..64, 92, 94, 96, 126, 160..191, 215, and 247.  In EUC, these are
character codes 35, 36, 38, 42, 43, 45..47, 58, 60..64, 92, 94, 96, and 126.
The common subset is<BR>
<KBD>+</KBD><KBD>-</KBD><KBD>*</KBD><KBD>/</KBD><KBD>\</KBD><KBD>^</KBD><KBD>&#60;</KBD><KBD>&#62;</KBD><KBD>=</KBD><KBD>`</KBD><KBD>~</KBD><KBD>:</KBD><KBD>.</KBD><KBD>?</KBD><KBD>@</KBD><KBD>#</KBD><KBD>$</KBD><KBD>&#38;</KBD>.

<DT><VAR>solo-char</VAR>
<DD>
In both standards, these are character codes 33 and 59 i.e. the
characters <KBD>!</KBD> and <KBD>;</KBD>.

<DT><VAR>punctuation-char</VAR>
<DD>
In both standards, these are character codes 37, 40, 41, 44, 91, 93, and
123..125, i.e. the characters <KBD>%(),[]{|}</KBD>.

<DT><VAR>quote-char</VAR>
<DD>
In both standards, these are character codes 34 and 39 i.e. the
characters <KBD>"</KBD> and <KBD>'</KBD>.

<DT><VAR>underline</VAR>
<DD>
In both standards, this is character code 95 i.e. the character
<KBD>_</KBD>.
</DL>


<PRE>
<VAR>token</VAR>             --&#62; <VAR>name</VAR>
                   |  <VAR>natural-number</VAR>
                   |  <VAR>unsigned-float</VAR>
                   |  <VAR>variable</VAR>
                   |  <VAR>string</VAR>
                   |  <VAR>punctuation-char</VAR>
                   |  <VAR>layout-text</VAR>
                   |  <VAR>full-stop</VAR>

<VAR>name</VAR>              --&#62; <VAR>quoted-name</VAR>
                   |  <VAR>word</VAR>
                   |  <VAR>symbol</VAR>
                   |  <VAR>solo-char</VAR>
                   |  <KBD>[</KBD> <VAR>?layout-text</VAR> <KBD>]</KBD>
                   |  <KBD>{</KBD> <VAR>?layout-text</VAR> <KBD>}</KBD>

<VAR>quoted-name</VAR>       --&#62; <KBD>'</KBD> <VAR>?quoted-item...</VAR> <KBD>'</KBD>

<VAR>quoted-item</VAR>       --&#62; <VAR>char</VAR>  { other than <KBD>'</KBD> or <KBD>\</KBD> }
                   |  <KBD>''</KBD>
                   |  <KBD>\</KBD> <VAR>escape-sequence</VAR>

<VAR>word</VAR>              --&#62; <VAR>small-letter</VAR> <VAR>?alpha...</VAR>

<VAR>symbol</VAR>            --&#62; <VAR>symbol-char...</VAR>
                         { except in the case of a <VAR>full-stop</VAR>
                           or where the first 2 chars are <KBD>/*</KBD> }

<VAR>natural-number</VAR>    --&#62; <VAR>digit...</VAR>
                   |  <VAR>base</VAR> <KBD>'</KBD> <VAR>alpha...</VAR>
                         { where each <VAR>alpha</VAR> must be less than the <VAR>base</VAR>,
                         treating a,b,... and A,B,... as 10,11,... }
                   |  <KBD>0</KBD> <KBD>'</KBD> <VAR>char-item</VAR>
                         { yielding the character code for <VAR>char</VAR> }

<VAR>char-item</VAR>         --&#62; <VAR>char</VAR>  { other than <KBD>\</KBD> }
                   |  <KBD>\</KBD> <VAR>escape-sequence</VAR>
  
<VAR>base</VAR>              --&#62; <VAR>digit...</VAR>  { in the range [2..36] }

<VAR>unsigned-float</VAR>    --&#62; <VAR>simple-float</VAR>
                   |  <VAR>simple-float</VAR> <VAR>exp</VAR> <VAR>exponent</VAR>

<VAR>simple-float</VAR>      --&#62; <VAR>digit...</VAR> <KBD>.</KBD> <VAR>digit...</VAR>

<VAR>exp</VAR>               --&#62; <KBD>e</KBD>  |  <KBD>E</KBD>

<VAR>exponent</VAR>          --&#62; <VAR>digit...</VAR> | <VAR>sign</VAR> <VAR>digit...</VAR>

<VAR>sign</VAR>              --&#62; <KBD>-</KBD> | <KBD>+</KBD>

<VAR>variable</VAR>          --&#62; <VAR>underline</VAR> <VAR>?alpha...</VAR>
                   |  <VAR>capital-letter</VAR> <VAR>?alpha...</VAR>

<VAR>string</VAR>            --&#62; <KBD>"</KBD> <VAR>?string-item...</VAR> <KBD>"</KBD>

<VAR>string-item</VAR>       --&#62; <VAR>char</VAR>  { other than <KBD>"</KBD> or <KBD>\</KBD> }
                   |  <KBD>""</KBD>
                   |  <KBD>\</KBD> <VAR>escape-sequence</VAR>

<VAR>layout-text</VAR>             --&#62; <VAR>layout-text-item...</VAR>

<VAR>layout-text-item</VAR>        --&#62; <VAR>layout-char</VAR> | <VAR>comment</VAR>

<VAR>comment</VAR>           --&#62; <KBD>/*</KBD> <VAR>?char...</VAR> <KBD>*/</KBD>
                         { where <VAR>?char...</VAR> must not contain <KBD>*/</KBD> }
                   |  <KBD>%</KBD> <VAR>?char...</VAR> <KBD>LFD</KBD>
                         { where <VAR>?char...</VAR> must not contain <KBD>LFD</KBD> }

<VAR>full-stop</VAR>         --&#62; <KBD>.</KBD>
                         { the following token, if any, must be <VAR>layout-text</VAR>}

<VAR>char</VAR>              --&#62; { any character, i.e. }
                      <VAR>layout-char</VAR>
                   |  <VAR>alpha</VAR>
                   |  <VAR>symbol-char</VAR>
                   |  <VAR>solo-char</VAR>
                   |  <VAR>punctuation-char</VAR>
                   |  <VAR>quote-char</VAR>

<VAR>alpha</VAR>             --&#62; <VAR>capital-letter</VAR> | <VAR>small-letter</VAR> | <VAR>digit</VAR> | <VAR>underline</VAR>

<VAR>escape-sequence</VAR>   --&#62; b        { backspace, character code 8 }
                   |  t        { horizontal tab, character code 9 }
                   |  n        { newline, character code 10 }
                   |  v        { vertical tab, character code 11 }
                   |  f        { form feed, character code 12 }
                   |  r        { carriage return, character code 13 }
                   |  e        { escape, character code 27 }
                   |  d        { delete, character code 127 }
                   |  a        { alarm, character code 7 }
                   |  x <VAR>alpha</VAR> <VAR>alpha</VAR>
                         {treating a,b,... and A,B,... as 10,11,... }
                               { in the range [0..15], hex character code }
                   |  <VAR>digit</VAR> <VAR>?digit</VAR> <VAR>?digit</VAR> 
                               { in the range [0..7], octal character code }
                   |  ^ ?      { delete, character code 127 }
                   |  ^ <VAR>capital-letter</VAR>
                   |  ^ <VAR>small-letter</VAR>
                               { the control character <VAR>alpha</VAR> mod 32 }
                   |  c <VAR>?layout-char</VAR>... { ignored }
                   |  <VAR>layout-char</VAR>  { ignored }
                   |  <VAR>char</VAR>    { other than the above, represents itself }
</PRE>



<H2><A NAME="SEC372" HREF="sicstus_toc.html#TOC372">Escape Sequences</A></H2>
<P>
<A NAME="IDX1554"></A>
<A NAME="IDX1555"></A>

</P>
<P>
A backslash occurring inside integers in <SAMP>`0''</SAMP> notation or inside
quoted atoms or strings has special meaning, and indicates the start
of an escape sequence.
Character escaping can be turned off for compatibility with old code.
The following escape sequences exist:

</P>
<DL COMPACT>

<DT><CODE>\b</CODE>
<DD>
backspace (character code 8)

<DT><CODE>\t</CODE>
<DD>
horizontal tab (character code 9)

<DT><CODE>\n</CODE>
<DD>
newline (character code 10)

<DT><CODE>\v</CODE>
<DD>
vertical tab (character code 11)

<DT><CODE>\f</CODE>
<DD>
form feed (character code 12)

<DT><CODE>\r</CODE>
<DD>
carriage return (character code 13)

<DT><CODE>\e</CODE>
<DD>
escape (character code 27)

<DT><CODE>\d</CODE>
<DD>
<DT><CODE>\^?</CODE>
<DD>
delete (character code 127)

<DT><CODE>\a</CODE>
<DD>
alarm (character code 7)

<DT><CODE>\x<VAR>CD</VAR></CODE>
<DD>
the character code <VAR>CD</VAR> (two hexadecimal digits)

<DT><CODE>\<VAR>octal</VAR></CODE>
<DD>
the character code <VAR>octal</VAR> base 8, where <VAR>octal</VAR> is up to 3 octal digits

<DT><CODE>\^<VAR>char</VAR></CODE>
<DD>
the character code <CODE><VAR>char</VAR> mod 32</CODE>, where <VAR>char</VAR> is a letter.

<DT><CODE>\<VAR>layout-char</VAR></CODE>
<DD>
A single layout character, for example a newline, is ignored.

<DT><CODE>\c</CODE>
<DD>
All characters up to, but not including, the next non-layout character are ignored.

<DT><CODE>\<VAR>other</VAR></CODE>
<DD>
A character not mentioned in this table stands for itself.  For example,
<SAMP>`\\'</SAMP> inserts a single backslash and <SAMP>`\''</SAMP> inserts a single
quote.
</DL>



<H2><A NAME="SEC373" HREF="sicstus_toc.html#TOC373">Notes</A></H2>


<OL>
<LI>

The expression of precedence 1000 (i.e. belonging to syntactic category
<VAR>term(1000)</VAR>) which is written


<PRE>
<VAR>X</VAR>,<VAR>Y</VAR>
</PRE>

denotes the term <CODE>','(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE> in standard syntax.

<LI>

The parenthesized expression (belonging to syntactic category
<VAR>term(0)</VAR>) 

<PRE>
(<VAR>X</VAR>)
</PRE>

denotes simply the term <CODE><VAR>X</VAR></CODE>.

<LI>

The curly-bracketed expression (belonging to syntactic category
<VAR>term(0)</VAR>) 

<PRE>
{<VAR>X</VAR>}
</PRE>

denotes the term <CODE>{}(<VAR>X</VAR>)</CODE> in standard syntax.
<LI>

Note that, for example, <CODE>-3</CODE> denotes a number whereas <CODE>-(3)</CODE>
denotes a compound term which has the 1-ary functor <CODE>-</CODE> as its
principal functor.  
<LI>

The character <KBD>"</KBD> within a string must be written duplicated.
Similarly for the character <KBD>'</KBD> within a quoted atom.  
<LI>

Backslashes in strings, quoted atoms, and integers written in <SAMP>`0''</SAMP>
notation denote escape sequences. 
<LI>

A name token declared to be a prefix operator will be treated as an atom
only if no <VAR>term-read-in</VAR> can be read by treating it as a prefix
operator.

<LI>

A name token declared to be both an infix and a postfix operator will be
treated as a postfix operator only if no <VAR>term-read-in</VAR> can be read
by treating it as an infix operator.
</OL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_44.html">previous</A>, <A HREF="sicstus_46.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
