<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - The Module System</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_6.html">previous</A>, <A HREF="sicstus_8.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC64" HREF="sicstus_toc.html#TOC64">The Module System</A></H1>
<P>
<A NAME="IDX182"></A>

</P>
<P>
By making use of the module systems facilities, programs can be divided
into different modules.  Each module has its own independent predicate
name space.  This is an important feature for the development of larger
programs.  The module system of SICStus Prolog is procedure based.  This
means that only the predicates are local to a module, whereas terms are
global.  The module system is flat, not hierarchical, so all modules are
visible to one another.  It is non-strict, i.e. the normal visibility
rules can be overridden by special syntax.  No overhead is incurred on
compiled calls to predicates in other modules.  It is modeled after and
compatible with the Quintus Prolog module system.  Finally, using the
module system is optional, and SICStus Prolog may be used without the
user being aware of the module system at all.

</P>
<P>
Modules in SICStus Prolog can also be used for object-oriented
programming.  See section <A HREF="sicstus_35.html#SEC315">Prolog Objects</A>, for details.

</P>

<UL>
<LI><A HREF="sicstus_7.html#SEC65">Basic Concepts</A>
<LI><A HREF="sicstus_7.html#SEC66">Module Spec</A>
<LI><A HREF="sicstus_7.html#SEC67">Def Modules</A>
<LI><A HREF="sicstus_7.html#SEC68">Importation</A>
<LI><A HREF="sicstus_7.html#SEC69">Meta Exp</A>
<LI><A HREF="sicstus_7.html#SEC70">Meta Decl</A>
</UL>



<H2><A NAME="SEC65" HREF="sicstus_toc.html#TOC65">Basic Concepts</A></H2>

<P>
Each predicate in the Prolog system, whether built-in or user defined,
belongs to a module.  A predicate is generally only visible in the
module where it is defined.  However a predicate may be <EM>imported</EM>
by another module.  It is thereby made visible in that module too.
Built-in predicates are visible in every module.
Predicates declared as public in a module declaration (see below) are
<EM>exported</EM>.  Normally only public predicates may be imported
by another module.
<A NAME="IDX183"></A>
<A NAME="IDX184"></A>
<A NAME="IDX185"></A>
<A NAME="IDX186"></A>
<A NAME="IDX187"></A>
<A NAME="IDX188"></A>
<A NAME="IDX189"></A>
<A NAME="IDX190"></A>

</P>
<P>
<A NAME="IDX191"></A>
<A NAME="IDX192"></A>
For any given goal, the <EM>source module</EM> is the module in which the
corresponding predicate must be visible.  Similarly, for any given
clause, the source module of its head is the module into which the
clause is loaded.

</P>
<P>
<A NAME="IDX193"></A>
<A NAME="IDX194"></A>
For goals occurring in a source file with a module declaration, the
source module is the declared module.  For goals occurring in a source
file without a module declaration, the source module is the module that
the file is being loaded into.  For goals typed at the top level, the
source module is the <EM>type-in module</EM>.  The type-in module is by
default the <CODE>user</CODE> module but may be changed by the built-in
predicate <CODE>module/1</CODE>.

</P>
<P>
The other predefined module is the <CODE>prolog</CODE> module where all the
built-in predicates reside.  The exported built-in predicates are
automatically imported into each new module as it is created.

</P>


<H2><A NAME="SEC66" HREF="sicstus_toc.html#TOC66">Module Prefixing</A></H2>

<P>
Notwithstanding the visibility rules, any predicate can be called from
any other module by prefixing the goal with the module name and the
colon operator, thus overriding the source module of the goal:

</P>

<PRE>
| ?- foo:bar(X).
</PRE>

<P>
This feature is intended mainly for debugging purposes, since it defies
the purposes of the module system.  If the prefixed goal is a
<EM>meta-predicate</EM>, however, the prefixed module name may affect the
module name expansion of the goal (see section <A HREF="sicstus_7.html#SEC69">Module Name Expansion</A>).

</P>
<P>
It is also possible to override the source module of clauses and
directives by module prefixing.  For example,

</P>

<PRE>
:- dynamic mod:p/1.
p(X) :- mod:(q(X), r(X)).
mod:(q(X) :- r(X)).
mod:s(X) :- t(X).
</PRE>

<P>
declares <CODE>mod:p/1</CODE> as dynamic, whatever the source module is;
defines <CODE>p/1</CODE> in the source module as calling <CODE>mod:q/1</CODE> and
<CODE>mod:r/1</CODE>; defines <CODE>mod:q/1</CODE> as calling <CODE>mod:r/1</CODE>; and
defines <CODE>mod:s/1</CODE> as calling <CODE>t/1</CODE> in the source module.
The latter technique is particularly useful when the prefix is
<CODE>user</CODE> and the predicate is a "hook" predicate such as
<CODE>user:portray/1</CODE> which must be defined in the <CODE>user</CODE> module,
but the rest of the file consists of predicates belonging to some other
module.

</P>


<H2><A NAME="SEC67" HREF="sicstus_toc.html#TOC67">Defining Modules</A></H2>

<P>
A module is normally defined by putting a module declaration first in
a source file.  A module declaration has the form:
<A NAME="IDX195"></A>
<A NAME="IDX196"></A>

</P>

<PRE>
:- module(<VAR>ModuleName</VAR>, <VAR>ExportList</VAR>[, <VAR>Options</VAR>]).
</PRE>

<P>
<A NAME="IDX197"></A>
<A NAME="IDX198"></A>
When the file is loaded, all predicates in the file go into
<VAR>ModuleName</VAR> and the predicates of the <VAR>ExportList</VAR> are
exported.  When a module declaration is processed, all existing
predicates in the module are erased before the new ones are loaded.  A
file which contains a module declaration is henceforth called a
<EM>module file</EM>.

</P>
<P>
<A NAME="IDX199"></A>
<A NAME="IDX200"></A>
<VAR>Options</VAR> is an optional argument, and should be a list.
The only available option is <CODE>hidden(<VAR>Bool</VAR>)</CODE>, where
<VAR>Bool</VAR> is <CODE>false</CODE> (the default) or <VAR>true</VAR>.
In the latter case, tracing of the predicates of the modules
is disabled (although spy-points can be set), and no source information
is generated at compile time.

</P>
<P>
A module can also be defined dynamically by asserting or loading
predicates to it:

</P>

<PRE>
| ?- assert(m:p(x)).
</PRE>

<P>
creates the module <CODE>m</CODE>, if it does not already exists, and asserts
<CODE>p(x)</CODE> to it.

</P>

<PRE>
| ?- compile(m:f).
</PRE>

<P>
creates the module <CODE>m</CODE> and loads <CODE>f</CODE> into <CODE>m</CODE>.

</P>
<P>
Dynamically created modules have no public predicates.

</P>


<H2><A NAME="SEC68" HREF="sicstus_toc.html#TOC68">Importation</A></H2>
<P>
<A NAME="IDX201"></A>

</P>
<P>
When a module file is loaded by <CODE>load_files/(1,2)</CODE> or one of its
shorthands (see section <A HREF="sicstus_10.html#SEC94">Reading-in Programs</A>), by default all the public predicates of the
module file are imported by the receiving module.  An explicit list of
predicates to import may also be specified.

</P>
<P>
Clashes with already existing predicates, local or imported from other
modules, are handled in two different ways: If the receiving module is
the <CODE>user</CODE> module, the user is asked for redefinition of the
predicate.  For other receiving modules, a warning is issued and the
importation is canceled. In the first case redefinition silently takes
place if the flag <CODE>redefine_warnings</CODE> has the value <CODE>off</CODE> (see
<CODE>prolog_flag/3</CODE>). The binding of an imported predicate remains,
even if the origin is reloaded or deleted.  However, <CODE>abolish/1</CODE>
breaks up the importation binding.  When a module file is reloaded, a
check is made that the predicates imported by other modules are still in
the public list.  If that is not the case, a warning is issued.  Note
that an imported predicate may be re-exported.

</P>


<H2><A NAME="SEC69" HREF="sicstus_toc.html#TOC69">Module Name Expansion</A></H2>
<P>
<A NAME="IDX202"></A>
<A NAME="IDX203"></A>

</P>
<P>
Some predicates take goals as arguments (i.e. meta-predicates).
These arguments must include a module specification stating which
module the goal refers.  Some other predicates also need module
information i.e. <CODE>compile/1</CODE>.  The property of needing module
information is declared with a <EM>meta-predicate declaration</EM>
(see section <A HREF="sicstus_7.html#SEC70">Meta-Predicate Declarations</A>).  Goals for these predicates are module name
expanded to ensure the module information.  Goals issued at top level
and goals appearing as directives are expanded prior to execution
while goals in the bodies of clauses are expanded at compile time.
The expansion is made by preceding the relevant argument with
<SAMP>`<VAR>Module</VAR>:'</SAMP>.  If the goal is prefixed by
<SAMP>`<VAR>Module</VAR>:'</SAMP>, <VAR>Module</VAR> is used for the expansion,
otherwise the source/type-in module is used.  An argument is not
expanded if:

</P>

<PRE>
1. It already has a module prefix.
2. It is a variable which appears in an expandable position in the head of the clause.
</PRE>

<P>
Some examples:

</P>

<PRE>
| ?- [user].
| :- meta_predicate p(:), q(:).
| r(X) :- p(X).
| q(X) :- p(X).
| ^D
{user consulted, 40 msec 1088 bytes}

yes
| ?- listing.

r(A) :-
        p(user:A).

q(A) :-
        p(A).

yes
</PRE>

<P>
Here, <CODE>p/1</CODE> and <CODE>q/1</CODE> are declared as meta-predicates while
<CODE>r/1</CODE> is not.  Thus the clause <CODE>r(X) :- p(X)</CODE> will be
transformed to <CODE>r(X) :- p(<VAR>M</VAR>:X)</CODE>, by item 2 above, where
<VAR>M</VAR> is the type-in module, whereas <CODE>q(X) :- p(X)</CODE> will not.

</P>

<PRE>
| ?- m:assert(f(1)).
</PRE>

<P>
Here, <CODE>assert/1</CODE> is called in the module <CODE>m</CODE>.  However, this
does not ensure that <CODE>f(1)</CODE> is asserted into <CODE>m</CODE>.  The fact
that <CODE>assert/1</CODE> is a meta-predicate makes the system module name
expand the goal, transforming it to <CODE>m:assert(m:f(1))</CODE> before
execution.  This way, <CODE>assert/1</CODE> is supplied the correct module
information.

</P>


<H2><A NAME="SEC70" HREF="sicstus_toc.html#TOC70">Meta-Predicate Declarations</A></H2>
<P>
<A NAME="IDX204"></A>
<A NAME="IDX205"></A>
<A NAME="IDX206"></A>

</P>
<P>
The fact that a predicate needs module name expansion is declared in a
<VAR>meta-predicate declaration</VAR>:

</P>

<PRE>
:- meta_predicate <VAR>MetaPredSpec</VAR>, ..., <VAR>MetaPredSpec</VAR>.
</PRE>

<P>
where each <VAR>MetaPredSpec</VAR> is a mode spec. E.g.

</P>

<PRE>
:- meta_predicate p(:, +).
</PRE>

<P>
which means that the first argument of <CODE>p/2</CODE> shall be module name
expanded.  The arguments in the mode spec are interpreted as:

</P>
<DL COMPACT>

<DT><CODE>:</CODE>
<DD>
<DT><CODE><VAR>An integer</VAR></CODE>
<DD>
This argument, in any call to the declared predicate, shall be
expanded. (Integers are allowed for compatibility reasons).

<DT><CODE><VAR>Anything else e.g. <CODE>+</CODE>, <CODE>-</CODE> or <CODE>?</CODE></VAR></CODE>
<DD>
This argument shall not be expanded
</DL>

<P>
A number of built-in predicates have predefined meta-predicate
declarations, as indicated by the mode specs in this manual,
e.g. <CODE>call(<VAR>:Term</VAR>)</CODE>.

</P>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_6.html">previous</A>, <A HREF="sicstus_8.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
