<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Debugging</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_8.html">previous</A>, <A HREF="sicstus_10.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC84" HREF="sicstus_toc.html#TOC84">Debugging</A></H1>
<P>
<A NAME="IDX262"></A>

</P>
<P>
This chapter describes the debugging facilities that are available in
Development Systems.  The purpose of these facilities is to provide
information concerning the control flow of your program.

</P>
<P>
In the Muse Development System, the debugger effectively sequentializes 
the execution.  Other tools (see section <A HREF="sicstus_6.html#SEC59">Visualization Tools</A>) must be used
to debug the parallel aspects of execution.

</P>
<P>
The main features of the debugging package are as follows:

</P>

<UL>
<LI>

The <EM>Procedure Box</EM> model of Prolog execution which provides a
simple way of visualizing control flow, especially during backtracking.
Control flow is viewed at the predicate level, rather than at the level
of individual clauses.  
<LI>

The ability to exhaustively trace your program or to selectively set
<EM>spy-points</EM>.  Spy-points allow you to nominate interesting
predicates at which the program is to pause so that you can interact.

<LI>

The wide choice of control and information options available during
debugging.  </UL>

<P>
The Procedure Box model of execution is also called the Byrd Box model
after its inventor, Lawrence Byrd.

</P>
<P>
Much of the information in this chapter is also in Chapter eight of
[Clocksin &#38; Mellish 81] which is recommended as an introduction.

</P>
<P>
Unless otherwise stated, the debugger print goals using
<CODE>write_term/3</CODE> with the value of the Prolog flag
<CODE>debugger_print_options</CODE> (see section <A HREF="sicstus_10.html#SEC105">Information about the State of the Program</A>).

</P>
<P>
The debugger is not available in Runtime Systems and the predicates
defined in this chapter are undefined; see section <A HREF="sicstus_11.html#SEC151">Runtime Systems</A>.

</P>


<UL>
<LI><A HREF="sicstus_9.html#SEC85">Procedure Box</A>
<LI><A HREF="sicstus_9.html#SEC86">Basic Debug</A>
<LI><A HREF="sicstus_9.html#SEC87">Spy-Point</A>
<LI><A HREF="sicstus_9.html#SEC88">Debug Format</A>
<LI><A HREF="sicstus_9.html#SEC89">Debug Options</A>
<LI><A HREF="sicstus_9.html#SEC90">Consult</A>
<LI><A HREF="sicstus_9.html#SEC91">Advanced Debug</A>
</UL>



<H2><A NAME="SEC85" HREF="sicstus_toc.html#TOC85">The Procedure Box Control Flow Model</A></H2>
<P>
<A NAME="IDX263"></A>

</P>
<P>
During debugging, the debugger prints out a sequence of goals in
various states of instantiation in order to show the state the program
has reached in its execution.  However, in order to understand what is
occurring it is necessary to understand when and why the debugger
prints out goals.  As in other programming languages, key points of
interest are predicate entry and return, but in Prolog there is the
additional complexity of backtracking.  One of the major confusions that
novice Prolog programmers have to face is the question of what actually
happens when a goal fails and the system suddenly starts backtracking.
The Procedure Box model of Prolog execution views program control flow
in terms of movement about the program text.  This model provides a
basis for the debugging mechanism in Development Systems, and enables
the user to view the behavior of the program in a consistent way.

</P>
<P>
Let us look at an example Prolog predicate :

</P>

<PRE>
           *--------------------------------------*
   Call    |                                      |    Exit
---------&#62; +  descendant(X,Y) :- offspring(X,Y).  + ---------&#62;
           |                                      |
           |  descendant(X,Z) :-                  |
&#60;--------- +     offspring(X,Y), descendant(Y,Z). + &#60;---------
   Fail    |                                      |    Redo
           *-------------------+------------------*
                               |
&#60;------------------------------+
   Exception

</PRE>

<P>
The first clause states that <VAR>Y</VAR> is a descendant of <VAR>X</VAR> if
<VAR>Y</VAR> is an offspring of <VAR>X</VAR>, and the second clause states that
<VAR>Z</VAR> is a descendant of <VAR>X</VAR> if <VAR>Y</VAR> is an offspring of <VAR>X</VAR>
and if <VAR>Z</VAR> is a descendant of <VAR>Y</VAR>.  In the diagram a box has been
drawn around the whole predicate and labeled arrows indicate the
control flow in and out of this box.  There are five such arrows which
we shall look at in turn.  
</P>
<DL COMPACT>

<DT><VAR>Call</VAR>
<DD>
This arrow represents initial invocation of the predicate.  When a goal
of the form <CODE>descendant(X,Y)</CODE> is required to be satisfied, control
passes through the <VAR>Call</VAR> port of the descendant box with the
intention of matching a component clause and then satisfying any
subgoals in the body of that clause.  Note that this is independent of
whether such a match is possible; i.e. first the box is called, and then
the attempt to match takes place.  Textually we can imagine moving to
the code for descendant when meeting a call to descendant in some other
part of the code.  
<DT><VAR>Exit</VAR>
<DD>
This arrow represents a successful return from the predicate.  This
occurs when the initial goal has been unified with one of the component
clauses and any subgoals have been satisfied.  Control now passes out of
the <VAR>Exit</VAR> port of the descendant box.  Textually we stop following
the code for descendant and go back to the place we came from.  
<DT><VAR>Redo</VAR>
<DD>
This arrow indicates that a subsequent goal has failed and that the system
is backtracking in an attempt to find alternatives to previous solutions.
Control passes through the <VAR>Redo</VAR> port of the descendant box.  An attempt
will now be made to resatisfy one of the component subgoals in the body of
the clause that last succeeded; or, if that fails, to completely rematch
the original goal with an alternative clause and then try to satisfy any
subgoals in the body of this new clause.  Textually we follow the code
backwards up the way we came looking for new ways of succeeding, possibly
dropping down on to another clause and following that if necessary.  
<DT><VAR>Fail</VAR>
<DD>
This arrow represents a failure of the initial goal, which might occur
if no clause is matched, or if subgoals are never satisfied, or if any
solution produced is always rejected by later processing.  Control now
passes out of the <VAR>Fail</VAR> port of the descendant box and the system
continues to backtrack.  Textually we move back to the code which called
this predicate and keep moving backwards up the code looking for choice
points.  
<DT><VAR>Exception</VAR>
<DD>
This arrow represents an exception which was raised in the initial goal,
either by a call to <CODE>raise_exception/1</CODE> or by an error in a
built-in predicate.  See section <A HREF="sicstus_10.html#SEC104">Error and Exception Handling</A>.  Control now passes out of the
<VAR>Exception</VAR> port of the descendant box and the systems continues to
pass the exception to outer levels.  Textually we move back to the code
which called this predicate and keep moving backwards up the code
looking for a call to <CODE>on_exception/3</CODE>.
</DL>

<P>
In terms of this model, the information we get about the procedure box
is only the control flow through these five ports.  This means that at
this level we are not concerned with which clause matches, and how any
subgoals are satisfied, but rather we only wish to know the initial goal
and the final outcome.  However, it can be seen that whenever we are
trying to satisfy subgoals, what we are actually doing is passing
through the ports of <EM>their</EM> respective boxes.  If we were to
follow this, then we would have complete information about the control
flow inside the procedure box.  
</P>
<P>
Note that the box we have drawn round the predicate should really be
seen as an <EM>invocation box</EM>.  That is, there will be a different box
for each different invocation of the predicate.  Obviously, with
something like a recursive predicate, there will be many different
<VAR>Calls</VAR> and <VAR>Exits</VAR> in the control flow, but these will be for
different invocations.  Since this might get confusing each invocation
box is given a unique integer identifier.  
</P>



<H2><A NAME="SEC86" HREF="sicstus_toc.html#TOC86">Basic Debugging Predicates</A></H2>
<P>
<A NAME="IDX264"></A>

</P>
<P>
Development Systems provide a range of built-in predicates for control
of the debugging facilities.  The most basic predicates are as follows:

</P>
<DL COMPACT>

<DT><CODE>debug</CODE>
<DD>
<A NAME="IDX265"></A>
Switches the debugger on, and ensures that the next time control reaches
a spy-point, a message will be produced and you will be prompted for a
command.  In order for the full range of control flow information to be
available it is necessary to have this on from the start.  When it is
off the system does not remember invocations that are being executed.
(This is because it is expensive and not required for normal running of
programs.)  You can switch <VAR>Debug Mode</VAR> on in the middle of
execution, either from within your program or after a ^C (see
<CODE>trace/0</CODE> below), but information prior to this will just be
unavailable.

<DT><CODE>zip</CODE>
<DD>
<A NAME="IDX266"></A>
Same as <CODE>debug/0</CODE>, except no debugging information is being
collected, and so is almost as fast as running with the debugger
switched off.

<DT><CODE>trace</CODE>
<DD>
<A NAME="IDX267"></A>
Switches the debugger on, and ensures that the
next time control enters an invocation box, a message will be produced and
you will be prompted for a command.  The effect of trace can also be achieved
by typing <KBD>t</KBD> after a ^C interruption of a program.  
At this point you have a number of options.  See section <A HREF="sicstus_9.html#SEC89">Options Available during Debugging</A>.  In
particular, you can just type <KBD>RET</KBD> to <EM>creep</EM> (or single-step)
into your program.  If you continue to creep through your program you
will see every entry and exit to/from every invocation box, including
compiled code, except for code belonging to hidden modules
(see section <A HREF="sicstus_7.html#SEC67">Defining Modules</A>).  You will
notice that the debugger stops at all ports.  However, if this is not
what you want, the following built-in predicate gives full control over
the ports at which you are prompted: 
<DT><CODE>leash(<VAR>+Mode</VAR>)</CODE>
<DD>
<A NAME="IDX268"></A>
Leashing Mode is set to <VAR>Mode</VAR>.  Leashing Mode determines the ports
of invocation boxes at which you are to be prompted when you Creep
through your program.  At unleashed ports a tracing message is still
output, but program execution does not stop to allow user interaction.
Note that the ports of spy-points are always leashed (and cannot be
unleashed).  <VAR>Mode</VAR> can be a subset of the following, specified as a
list: 
<DL COMPACT>

<DT><CODE>call</CODE>
<DD>
Prompt on Call.

<DT><CODE>exit</CODE>
<DD>
Prompt on Exit.

<DT><CODE>redo</CODE>
<DD>
Prompt on Redo.

<DT><CODE>fail</CODE>
<DD>
Prompt on Fail.

<DT><CODE>exception</CODE>
<DD>
Prompt on Exception.
</DL>

The initial value of <VAR>Leashing Mode</VAR> is
<CODE>[call,exit,redo,fail,exception]</CODE> (full leashing).

<DT><CODE>nodebug</CODE>
<DD>
<DT><CODE>notrace</CODE>
<DD>
<DT><CODE>nozip</CODE>
<DD>
<A NAME="IDX269"></A>
<A NAME="IDX270"></A>
<A NAME="IDX271"></A>
Switches the debugger off.  If there are any spy-points set then they
will be kept but disabled.  
<DT><CODE>debugging</CODE>
<DD>
<A NAME="IDX272"></A>
Prints information about the current
debugging state.  This will show: 

<OL>
<LI>

Whether undefined predicates are being trapped.
<LI>

Whether the debugger is switched on.
<LI>

What spy-points have been set (see below).
<LI>

What mode of leashing is in force (see below).
</OL>

</DL>



<H2><A NAME="SEC87" HREF="sicstus_toc.html#TOC87">Spy-points</A></H2>
<P>
<A NAME="IDX273"></A>

</P>
<P>
For programs of any size, it is clearly impractical to creep through the
entire program.  <EM>Spy-points</EM> make it possible to stop the program
whenever it gets to a particular predicate which is of interest.  Once
there, one can set further spy-points in order to catch the control flow
a bit further on, or one can start creeping.

</P>
<P>
Setting a spy-point on a predicate indicates that you wish to see all
control flow through the various ports of its invocation boxes, except
during skips.  When control passes through any port of a invocation box
with a spy-point set on it, a message is output and the user is asked to
interact.  Note that the current mode of leashing does not affect
spy-points: user interaction is requested on <EM>every</EM> port.

</P>
<P>
Spy-points are set and removed by the following built-in predicates.
The first two are also standard operators:

</P>
<DL COMPACT>

<DT><CODE>spy <VAR>:Spec</VAR></CODE>
<DD>
<A NAME="IDX274"></A>
Sets spy-points on all the predicates given by <VAR>Spec</VAR>, which is
is of the form:

<DL COMPACT>

<DT><VAR>Name</VAR>
<DD>
all predicates called <VAR>Name</VAR> no matter what arity,
where <VAR>Name</VAR> is an atom for a specific name or a variable for all names, or

<DT><VAR>Name/Arity</VAR>
<DD>
the predicate of that name and arity, or

<DT><VAR>Name/Low-High</VAR>
<DD>
<DT><VAR>Name/(Low-High)</VAR>
<DD>
the predicates of that name with arity in the range <VAR>Low-High</VAR>, or

<DT><VAR>Module:Spec</VAR>
<DD>
specifying a particular module (see section <A HREF="sicstus_7.html#SEC66">Module Prefixing</A>) instead of the
default type-in module, where <VAR>Module</VAR> is an atom for a specific
module or a variable for all modules, or

<DT>[Spec,...,Spec]
<DD>
the set of all predicates covered by the <VAR>Specs</VAR>.
</DL>

You cannot place a spy-point on an undefined predicate.  If you set some
spy-points when the debugger is switched off then it will be
automatically switched on.  Examples:


<PRE>
| ?- spy [user:p, m:q/(2-3)].
| ?- spy m:[p/1, q/1].
</PRE>

<DT><CODE>nospy <VAR>:Spec</VAR></CODE>
<DD>
<A NAME="IDX275"></A>
Similar to <CODE>spy <VAR>Spec</VAR></CODE> except that all the predicates given
by <VAR>Spec</VAR> will have previously set spy-points removed from them.

<DT><CODE>nospyall</CODE>
<DD>
<A NAME="IDX276"></A>
Removes all the spy-points that have been set.  
<DT><CODE>spypoint_condition(<VAR>:Goal</VAR>,<VAR>?Port</VAR>,<VAR>+Test</VAR>)</CODE>
<DD>
<A NAME="IDX277"></A>
Sets a <EM>conditional</EM> spy-point on the predicate for <VAR>Goal</VAR>.
When the debugger reaches the spy-point, three conditions must be met
for the spy-point to apply:  The actual goal must match <VAR>Goal</VAR>, the
actual port must match <VAR>Port</VAR>, and <VAR>Test</VAR> (an arbitrary Prolog
goal) must succeed.
</DL>

<P>
The options available when you arrive at a spy-point are described later.
See section <A HREF="sicstus_9.html#SEC89">Options Available during Debugging</A>.

</P>


<H2><A NAME="SEC88" HREF="sicstus_toc.html#TOC88">Format of Debugging messages</A></H2>
<P>
<A NAME="IDX278"></A>

</P>
<P>
We shall now look at the exact format of the message output by the
system at a port.  All trace messages are output to the standard error
stream.  This allows you to trace programs while they are performing
file I/O.  The basic format is as follows:

</P>

<PRE>
<VAR>E</VAR> <VAR>N</VAR> <VAR>S</VAR>    23     6 Call: <VAR>T</VAR> foo(hello,there,_123) ?
</PRE>

<P>
<VAR>E</VAR> is only used at Exception ports and displays a pending exception.

</P>
<P>
<VAR>N</VAR> is only used at Exit ports and indicates whether the invocation
could backtrack and find alternative solutions.  Unintended
non-determinism is a source of inefficiency, and this annotation can
help spot such efficiency bugs.  It is printed as <SAMP>`?'</SAMP>, indicating
that <CODE>foo/3</CODE> could backtrack and find alternative solutions, or
<SAMP>` '</SAMP> otherwise.

</P>
<P>
<VAR>S</VAR> is a spy-point indicator.  It is printed as <SAMP>`+'</SAMP>, indicating
that there is a spy-point on <CODE>foo/3</CODE>, or <SAMP>` '</SAMP>, denoting no
spy-point.

</P>
<P>
The first number is the unique invocation identifier.  It is
nondecreasing regardless of whether or not you are actually seeing the
invocations (provided that the debugger is switched on).  This number can
be used to cross correlate the trace messages for the various ports,
since it is unique for every invocation.  It will also give an
indication of the number of procedure calls made since the start of the
execution.  The invocation counter starts again for every fresh
execution of a command, and it is also reset when retries (see later)
are performed.  
</P>
<P>
<A NAME="IDX279"></A>
<A NAME="IDX280"></A>
The number following this is the <EM>current depth</EM>; i.e. the number
of direct <EM>ancestors</EM> this goal has.  
</P>
<P>
The next word specifies the particular port (Call, Exit, Redo, Fail, or
Exception).

</P>
<P>
<VAR>T</VAR> is a subterm trace.  This is used in conjunction with the
<SAMP>`^'</SAMP> command (set subterm), described below.  If a subterm has been
selected, <VAR>T</VAR> is printed as the sequence of commands used to select
the subterm.  Normally, however, <VAR>T</VAR> is printed as <SAMP>` '</SAMP>,
indicating that no subterm has been selected.

</P>
<P>
The goal is then printed so that you can inspect its current instantiation
state.

</P>
<P>
The final <SAMP>`?'</SAMP> is the prompt indicating that you should type in one of the
option codes allowed (see section <A HREF="sicstus_9.html#SEC89">Options Available during Debugging</A>).  If this particular port is
unleashed then you will obviously not get this prompt since you have
specified that you do not wish to interact at this point.  
</P>
<P>
Note that calls that are compiled in-line and built-in predicates that
are called directly from the top level are not traced.
  
There are two exceptions to the above debugger message format.  A message

</P>

<PRE>
       <VAR>S</VAR> -  -  Block: p(_133)
</PRE>

<P>
<A NAME="IDX281"></A>
<A NAME="IDX282"></A>
indicates that the debugger has encountered a <EM>blocked</EM> goal, i.e.
one which is temporarily suspended due to insufficiently instantiated
arguments (see section <A HREF="sicstus_5.html#SEC46">Procedural Semantics</A>).  No interaction takes place at this
point, and the debugger simply proceeds to the next goal in the
execution stream.  The suspended goal will be eligible for execution
once the blocking condition ceases to exist, at which time a message

</P>

<PRE>
       <VAR>S</VAR> -  -  Unblock: p(_133)
</PRE>

<P>
is printed.

</P>


<H2><A NAME="SEC89" HREF="sicstus_toc.html#TOC89">Options Available during Debugging</A></H2>
<P>
<A NAME="IDX283"></A>

</P>
<P>
This section describes the particular options that are available when
the system prompts you after printing out a debugging message.  All the
options are one letter mnemonics, some of which can be optionally
followed by a decimal integer.  They are read from the standard input stream with any
blanks being completely ignored up to the end of the line (<KBD>RET</KBD>).
Some options only actually require the terminator; e.g. the creep
option, as we have already seen, only requires <KBD>RET</KBD>.

</P>
<P>
The only option which you really have to remember is <SAMP>`h'</SAMP> (followed
by <KBD>RET</KBD>).  This provides help in the form of the following list of
available options.

</P>

<PRE>
RET    creep            c      creep
 l     leap             z      zip
 s     skip             s &#60;i&#62;  skip i
 o     out              o &#60;n&#62;  out n
 q     q-skip           q &#60;i&#62;  q-skip i
 r     retry            r &#60;i&#62;  retry i
 f     fail             f &#60;i&#62;  fail i
 d     display          w      write
 p     print            p &#60;i&#62;  print partial
 g     ancestors        g &#60;n&#62;  ancestors n
 t     backtrace        t &#60;n&#62;  backtrace n
 &#38;     blocked goals    &#38; &#60;n&#62;  nth blocked goal
 n     nodebug          =      debugging
 +     spy this         + &#60;i&#62;  spy conditionally
 -     nospy this       .      find this
 a     abort            b      break
 @     command          u      unify
 e     raise exception
 &#60;     reset printdepth &#60; &#60;n&#62;  set printdepth
 ^     reset subterm    ^ &#60;n&#62;  set subterm
 ?     help             h      help
</PRE>

<DL COMPACT>

<DT><KBD>c</KBD>
<DD>
<DT><KBD><KBD>RET</KBD></KBD>
<DD>
<EM>creep</EM> causes the debugger to single-step to the very next port
and print a message.  Then if the port is leashed (see section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>), the
user is prompted for further interaction.  Otherwise, it continues creeping.
If leashing is off, creep is the same as <EM>leap</EM> (see below) except that a
complete trace is printed on the standard error stream.  <A NAME="IDX284"></A>

<DT><KBD>l</KBD>
<DD>
<EM>leap</EM> causes the debugger to resume running your program, only
stopping when a spy-point is reached (or when the program terminates).
Leaping can thus be used to follow the execution at a higher level than
exhaustive tracing.  All you need to do is to set spy-points on an
evenly spread set of pertinent predicates, and then follow the control
flow through these by leaping from one to the other.  Debugging
information is collected while leaping, so when a spy-points is
reached, it is possible to inspect the ancestor goals, or creep into
them upon entry to Redo ports. <A NAME="IDX285"></A>

<DT><KBD>z</KBD>
<DD>
<EM>zip</EM> is like <EM>leap</EM>, except no debugging information is
being collected while zipping, resulting in significant
savings in memory and execution time. <A NAME="IDX286"></A>

<DT><KBD>s</KBD>
<DD>
<EM>skip</EM> is only valid for Call and Redo ports.  It skips over the
entire execution of the predicate.  That is, you will not see anything
until control comes back to this predicate (at either the Exit port or
the Fail port).  Skip is particularly useful while creeping since it
guarantees that control will be returned after the (possibly complex)
execution within the box.  If you skip then no message at all will
appear until control returns.  This includes calls to predicates with
spy-points set; they will be masked out during the skip.  No debugging
information is being collected while skipping. 
If you supply an integer argument, then this should denote an
invocation number of an ancestral goal.  The system tries to get you
to the Exit or Fail port of the invocation box you have specified.
<A NAME="IDX287"></A>

<DT><KBD>o</KBD>
<DD>
<EM>out</EM> is a shorthand for skipping to the Exit or Fail port of the
immediate ancestor goal.  If you supply an integer argument <VAR>n</VAR>,
it denotes skipping to the Exit or Fail port of the <VAR>n</VAR>th ancestor goal.
<A NAME="IDX288"></A>

<DT><KBD>q</KBD>
<DD>
<EM>quasi-skip</EM> is like a combination of <EM>leap</EM> and <EM>skip</EM>:
execution stops when either control comes back to this predicate, or
a spy-point is reached.  No debugging information is
being collected while quasi-skipping. 
An integer argument can be supplied as for <EM>skip</EM>.
<A NAME="IDX289"></A>

<DT><KBD>r</KBD>
<DD>
<EM>retry</EM> can be used at any of the four ports (although at the Call
port it has no effect).  It transfers control back to the Call port of
the box.  This allows you to restart an invocation when, for example,
you find yourself leaving with some weird result.  The state of
execution is exactly the same as when you originally called, (unless you
use side effects in your program; i.e. asserts etc. will not be undone).
When a retry is performed the invocation counter is reset so that
counting will continue from the current invocation number regardless of
what happened before the retry.  This is in accord with the fact that
you have, in executional terms, returned to the state before anything
else was called.  
If you supply an integer argument, then this should denote an
invocation number of an ancestral goal.  
The system tries to get you to the Call port of the box you have
specified.  It does this by continuously failing until it reaches the
right place.  Unfortunately this process cannot be guaranteed: it may be
the case that the invocation you are looking for has been cut out of the
search space by cuts (<CODE>!</CODE>) in your program.  In this case the
system fails to the latest surviving Call port before the correct one.
<A NAME="IDX290"></A>

<DT><KBD>f</KBD>
<DD>
<EM>fail</EM> can be used at any of the four ports (although at the Fail
port it has no effect).  It transfers control to the Fail port of the
box, forcing the invocation to fail prematurely.  
If you supply an integer after the command, then this is taken as
specifying an invocation number and the system tries to get you to the
Fail port of the invocation box you have specified.  It does this by
continuously failing until it reaches the right place.  Unfortunately
this process cannot be guaranteed: it may be the case that the
invocation you are looking for has been cut out of the search space by
cuts (<CODE>!</CODE>) in your program.  In this case the system fails to the
latest surviving Fail port before the correct one.  <A NAME="IDX291"></A>

<DT><KBD>d</KBD>
<DD>
<EM>display goal</EM> displays the current goal using <CODE>display/1</CODE>.
See Write (below).  <A NAME="IDX292"></A>

<DT><KBD>p</KBD>
<DD>
<EM>print goal</EM> re-prints the current goal.
An argument will override the default printdepth, treating 0 as infinity.
<A NAME="IDX293"></A>

<DT><KBD>w</KBD>
<DD>
<EM>write goal</EM> writes the current goal using <CODE>write/1</CODE>.  <A NAME="IDX294"></A>

<DT><KBD>g</KBD>
<DD>
<EM>print ancestor goals</EM> provides you with a list of ancestors to the
current goal, i.e. all goals that are hierarchically above the current
goal in the calling sequence.  You can always be sure of jumping to any
goal in the ancestor list (by using retry etc).  If you supply an integer
<VAR>n</VAR>, then only that number of ancestors will be printed.  That is to
say, the last <VAR>n</VAR> ancestors will be printed counting back from the
current goal.  Each entry is displayed just as they would be in a trace
message.  <A NAME="IDX295"></A>

<DT><KBD>t</KBD>
<DD>
<EM>print backtrace</EM> is the same as the above, but also shows any goals
that have exited non-deterministically and their ancestors.  This
information shows where there are outstanding choices that the program
could backtrack to.  If you supply an integer
<VAR>n</VAR>, then only that number of goals will be printed.  

Ancestors to the current goal are annotated with the <SAMP>`Call:'</SAMP> port,
as they have not yet exited, whereas goals that have exited are
annotated with the <SAMP>`Exit:'</SAMP> port.  The backtrace is a tree rather
than a stack: to find the parent of a given goal with depth indicator
<VAR>d</VAR>, look for the closest goal above it with depth indicator
<VAR>d-1</VAR>. <A NAME="IDX296"></A>

<DT><KBD>&#38;</KBD>
<DD>
<EM>print blocked goals</EM> prints a list of the goals which are currently
blocked in the current debugging session together with the variable that
each such goal is blocked on (see section <A HREF="sicstus_5.html#SEC46">Procedural Semantics</A>).  The goals are
enumerated from 1 and up.  If you supply an integer <VAR>n</VAR>, then only
that goal will be printed.  Each entry is preceded by the goal number
followed by the variable name.  <A NAME="IDX297"></A>

<DT><KBD>n</KBD>
<DD>
<EM>nodebug</EM> switches the debugger off.  Note that this is the correct way
to switch debugging off at a trace point.  You cannot use the <KBD>@</KBD> or
<KBD>b</KBD> options because they always return to the debugger.  <A NAME="IDX298"></A>

<DT><KBD>=</KBD>
<DD>
<EM>debugging</EM> outputs information concerning the status of the debugging
package.  See section <A HREF="sicstus_10.html#SEC113">Debugging</A>, the built-in <CODE>debugging/0</CODE>.
<A NAME="IDX299"></A>

<DT><KBD>+</KBD>
<DD>
<EM>spy this</EM> sets a spy-point on the current goal.  With an
argument, prompts for a condition to be tested each time the debugger
reaches the spy-point.  Conditions consist of three parts: a <VAR>port</VAR>,
a <VAR>goal</VAR> template, and a <VAR>test</VAR> (arbitrary Prolog goal), usually
sharing some variables with the <VAR>test</VAR> part.  For the spy-point to
apply, the <VAR>port</VAR> and <VAR>goal</VAR> parts must match, and the
<VAR>test</VAR> must succeed, otherwise the spy-point is simply disabled.  The
following example illustrates how to define a spy-point to only apply at
Call ports when a test is satisfied:

<PRE>
   4  2  Call: nr([0,1,2,3,4,5,6,7,8,9,...],_267) ? + 1
Goal, Port, Cond: nr(L,_), call, (length(L,N), N&#60;3).
{Spy-point placed on user:nr/2}
 + 4  2  Call: nr([0,1,2,3,4,5,6,7,8,9,...],_267) ? l
 + 32  30  Call: nr([8,9],_2771) ? s
</PRE>

<A NAME="IDX300"></A>

<DT><KBD>-</KBD>
<DD>
<EM>nospy this</EM> removes the spy-point from the current goal, if it exists,
and any condition for that spy-point.
<A NAME="IDX301"></A>

<DT><KBD>.</KBD>
<DD>
<EM>find this</EM> outputs information about where the predicate being called
is defined.
<A NAME="IDX302"></A>

<DT><KBD>a</KBD>
<DD>
<EM>abort</EM> causes an abort of the current execution.  All the execution
states built so far are destroyed and you are put right back at the
top-level.  (This is the same as the built-in predicate <CODE>abort/0</CODE>.)
<A NAME="IDX303"></A>

<DT><KBD>b</KBD>
<DD>
<EM>break</EM> calls the built-in predicate <CODE>break/0</CODE>, thus putting you
at a recursive top-level with the execution so far sitting underneath you.
When you end the break (^D) you will be reprompted at the port at
which you broke.  The new execution is completely separate from the
suspended one; the invocation numbers will start again from 1 during the
break.  The debugger is temporarily switched off as you call the break and
will be re-switched on when you finish the break and go back to the old
execution.  However, any changes to the leashing or to spy-points will
remain in effect.  <A NAME="IDX304"></A>

<DT><KBD>@</KBD>
<DD>
<EM>command</EM> gives you the ability to call arbitrary Prolog goals.  It
is effectively a one-off <EM>break</EM> (see above).  The initial message
<SAMP>`| :- '</SAMP> will be output on the standard error stream, and a command is then read
from the standard input stream and executed as if you were at top level.  <A NAME="IDX305"></A>

<DT><KBD>u</KBD>
<DD>
<EM>unify</EM> is available at the Call port and gives you the option of
providing a solution to the goal from the standard input stream rather than executing
the goal.  This is convenient e.g. for providing a "stub" for a
predicate that has not yet been written.  A prompt will be
output on the standard error stream, and the solution is then read from
the standard input stream and unified with the goal.  <A NAME="IDX306"></A>

<DT><KBD>e</KBD>
<DD>
<EM>raise exception</EM> is available at all ports.  A prompt will be
output on the standard error stream, and an exception term is then read from
the standard input stream and raised in the program being debugged. <A NAME="IDX307"></A>

<DT><KBD>&#60;</KBD>
<DD>
This command, without arguments, resets the printdepth to 10.  With an
argument of <VAR>n</VAR>, the printdepth is set to <VAR>n</VAR>, treating 0 as
infinity.
<A NAME="IDX308"></A>
<A NAME="IDX309"></A>

<DT><KBD>^</KBD>
<DD>
While at a particular port, a current <EM>subterm</EM> of the current goal
is maintained.  It is the current subterm which is displayed, printed,
or written when prompting for a debugger command.  Used in combination
with the printdepth, this provides a means for navigating in the
current goal for focusing on the part which is of interest.
The current subterm is set to the current goal when arriving at a new port.
This command, without arguments, resets the current subterm to the current 
goal.  With an argument of <VAR>n</VAR> (&#62; 0), the current subterm is replaced
by its <VAR>n</VAR>:th subterm.  With an argument of <VAR>0</VAR>, the current subterm
is replaced by its parent term.  With a list of arguments, the arguments are
applied from left to right. <A NAME="IDX310"></A>
<A NAME="IDX311"></A>

<DT><KBD>?</KBD>
<DD>
<DT><KBD>h</KBD>
<DD>
<EM>help</EM> displays the table of options given above.  <A NAME="IDX312"></A>
</DL>



<H2><A NAME="SEC90" HREF="sicstus_toc.html#TOC90">Consulting during Debugging</A></H2>
<P>
<A NAME="IDX313"></A>

</P>
<P>
It is possible, and sometimes useful, to consult a file whilst in the
middle of program execution.  Predicates, which have been successfully
executed and are subsequently redefined by a consult and are later
reactivated by backtracking, will not notice the change of their
definitions.  In other words, it is as if every predicate, when called,
creates a virtual copy of its definition for backtracking purposes.

</P>
<P>
If SICStus Prolog is run via the Emacs interface, the commands for
loading code (such as <KBD>C-c C-p</KBD>, consulting the current predicate)
are not directly available when the system prompts you after printing
out a debugging message.  Press <KBD>b</KBD> followed by <KBD>RET</KBD> to get a
recursive top-level, ready to accept the Emacs commands.  Type ^D
to return to the debugging port.

</P>


<H2><A NAME="SEC91" HREF="sicstus_toc.html#TOC91">Catching Exceptions</A></H2>

<P>
<A NAME="IDX314"></A>
<A NAME="IDX315"></A>
Usually, exceptions that occur during debugging sessions are displayed
only in trace mode and for invocation boxes for predicates with
spy-points on them, and not during skips.  However, it is sometimes
useful to make exceptions trap to the debugger at the earliest
opportunity instead.  The following predicate provides such a
possibility

</P>
<DL COMPACT>

<DT><CODE>error_exception(<VAR>+Exception</VAR>)</CODE>
<DD>
<DT><CODE>user:error_exception(<VAR>+Exception</VAR>)</CODE>
<DD>
<A NAME="IDX316"></A>
<A NAME="IDX317"></A>
<EM>A hook predicate.</EM> This predicate is called at all
exception ports.  If it succeeds, the debugger enters trace mode and
prints an exception port message.  Otherwise, the debugger mode is
unchanged and a message is printed only in trace mode or if a spy-point
is reached, and not during skips.  </DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_8.html">previous</A>, <A HREF="sicstus_10.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
