<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 2.11</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-2.10.html">
<LINK REL=next HREF="sec-2.12.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-2.10.html">Previous</A>
<A HREF="sec-2.12.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:2.11">2.11 Syntax Notes</A></H2>

<P>SWI-Prolog uses standard `Edinburgh' syntax. A description of this 
syntax can be found in the Prolog books referenced in the introduction. 
Below are some non-standard or non-common constructs that are accepted 
by SWI-Prolog:

<P><UL>
<LI><I><CODE>0'&lt;<VAR>char</VAR>&gt;</CODE></I><BR>
This construct is not accepted by all Prolog systems that claim to have 
Edinburgh compatible syntax. It describes the ASCII value of &lt;<VAR>char</VAR>&gt;. 
To test whether <CODE>C</CODE> is a lower case character one can use
<CODE>between(0'a, 0'z, C)</CODE>.
<LI><I><CODE>/* ... /* ... */ ... */</CODE></I><BR>
The <CODE>/* ... */</CODE> comment statement can be nested. This is 
useful if some code with <CODE>/* ... */</CODE> comment statements in it 
should be commented out.
</UL>

<P>

<H3><A NAME="sec:2.11.1">2.11.1 ISO Syntax Support</A></H3>

<P>SWI-Prolog offers ISO compatible extensions to the Edinburgh syntax.

<P>

<H4><A NAME="sec:2.11.1.1">2.11.1.1 Character Escape Syntax</A></H4>

<A NAME="sec:charescapes"></A>

<P>Within quoted atoms (using single quotes: <CODE>'&lt;<VAR>atom</VAR>&gt;'</CODE> 
special characters are represented using escape-sequences. An escape 
sequence is lead in by the backslash (<CODE><CODE>\</CODE></CODE>) 
character. The list of escape sequences is compatible with the ISO 
standard, but contains one extension and the interpretation of 
numerically specified characters is slightly more flexible to improve 
compatibility.

<P>

<DL>

<P>
<DT><CODE>\a</CODE><DD>
Alert character. Normally the ASCII character 7 (beep).

<P>
<DT><CODE>\b</CODE><DD>
Backspace character.

<P>
<DT><CODE>\c</CODE><DD>
No output. All input characters upto but not including the first 
non-layout character are skipped. This allows for the specification of 
pretty-looking long lines. For compatibility with Quintus Prolog. Nor 
supported by ISO. Example:

<XMP>
format('This is a long line that would look better if it was \c
       split across multiple physical lines in the input')
</XMP>

<P>

<P>
<DT><CODE>\&lt;<VAR></EM><font size=-1>RETURN</font></VAR>&gt;</CODE><DD>
No output. Skips input till the next non-layout character or to the end 
of the next line. Same intention as <CODE>\c</CODE> but ISO compatible.

<P>
<DT><CODE>\f</CODE><DD>
Form-feed character.

<P>
<DT><CODE>\n</CODE><DD>
Next-line character.

<P>
<DT><CODE>\r</CODE><DD>
Carriage-return only (i.e. go back to the start of the line).

<P>
<DT><CODE>\t</CODE><DD>
Horizontal tab-character.

<P>
<DT><CODE>\v</CODE><DD>
Vertical tab-character (ASCII 11).

<P>
<DT><CODE>\x23</CODE><DD>
Hexadecimal specification of a character. <CODE>23</CODE> is just an 
example. The `x' may be followed by a maximum of 2 hexadecimal digits. 
The closing <CODE>\</CODE> is optional. The code <CODE>\xa\3</CODE> 
emits the character 10 (hexadecimal `a') followed by `3'. The code <CODE>\x201</CODE> 
emits 32 (hexadecimal `20') followed by `1'. According to ISO, the 
closing
<CODE>\</CODE> is obligatory and the number of digits is unlimited. The 
SWI-Prolog definition allows for ISO compatible specification, but is 
compatible with other implementations.

<P>
<DT><CODE>\40</CODE><DD>
Octal character specification. The rules and remarks for hexadecimal 
specifications apply to octal specifications too, but the maximum 
allowed number of octal digits is 3.

<P>
<DT><CODE>\&lt;<VAR>character</VAR>&gt;</CODE><DD>
Any character immediately preceded by a <CODE><CODE>\</CODE></CODE> is 
copied verbatim. Thus, <CODE>'\\'</CODE> is an atom consisting of a 
single <CODE><CODE>\</CODE></CODE> and
<CODE>'\''</CODE> and <CODE>''''</CODE> both describe the atom with a 
single&nbsp;<CODE>'</CODE>.
</DL>

<P>Character escaping is only available if the
<CODE>feature(character_escapes, true)</CODE> is active (default). See
<A NAME="idx:feature/2:136"></A><A HREF="sec-2.8.html#feature/2">feature/2</A>. 
Character escapes conflict with <A NAME="idx:writef/2:137"></A><A HREF="sec-3.32.html#writef/2">writef/2</A> 
in two ways:
<CODE>\40</CODE> is interpreted as decimal 40 by <A NAME="idx:writef/2:138"></A><A HREF="sec-3.32.html#writef/2">writef/2</A>, 
but character escapes handling by read has already interpreted as 32 (40 
octal). Also,
<CODE>\l</CODE> is translated to a single `l'. Double the <CODE><CODE>\</CODE></CODE> 
(e.g. <CODE>\\</CODE>, use the above escape sequences or use <A NAME="idx:format/2:139"></A><A HREF="sec-3.32.html#format/2">format/2</A>.

<P>

<H4><A NAME="sec:2.11.1.2">2.11.1.2 Syntax for Non-Decimal Numbers</A></H4>

<P>SWI-Prolog implements both Edinburgh and ISO representations for 
non-decimal numbers. According to Edinburgh syntax, such numbers are 
written as <CODE>&lt;<VAR>radix</VAR>&gt;'&lt;<VAR>number</VAR>&gt;</CODE>, 
where &lt;<VAR>radix</VAR>&gt; is a number between 2 and 36. ISO defines 
binary, octal and hexadecimal numbers using
<CODE>0</EM><EM>[bxo]</EM>&lt;<VAR>number</VAR>&gt;</CODE>. For example: <CODE>A is 0b100 \/ 0xf00</CODE> 
is a valid expression. Such numbers are always unsigned.

