<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 2.12</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-2.11.html">
<LINK REL=next HREF="sec-3.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-2.11.html">Previous</A>
<A HREF="sec-3.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:2.12">2.12 System Limits</A></H2>

<P>

<H3><A NAME="sec:2.12.1">2.12.1 Limits on Memory Areas</A></H3>

<P>SWI-Prolog has a number of memory areas which are only enlarged to a 
certain limit. The default sizes for these areas should suffice for most 
applications, but big applications may require larger ones. They are 
modified by command line options. The table below shows these areas. The 
first column gives the option name to modify the size of the area. The 
option character is immediately followed by a number and optionally by a
<CODE>k</CODE> or <CODE>m</CODE>. With <CODE>k</CODE> or no unit 
indicator, the value is interpreted in Kbytes (1024 bytes), with <CODE>m</CODE>, 
the value is interpreted in Mbytes (<VAR>1024 &times; 1024</VAR> bytes).

<P>The local-, global- and trail-stack are limited to 64 Mbytes on 32 
bit processors, or more in general to <VAR>2 ** bits-per-long - 6</VAR> 
bytes.

<P>
<CENTER>
<TABLE BORDER=2 COLS=4 FRAME=border RULES=group>
<TR VALIGN=top><TD ALIGN=center>Option<TD ALIGN=center>Default<TD>Area 
name<TD>Description </TR>
<TBODY>
<TR VALIGN=top><TD ALIGN=center><STRONG>-L</STRONG> <TD ALIGN=center>2M<TD></EM><B>local 
stack</B><TD>The local stack is used to store the execution environments 
of procedure invocations. The space for an environment is reclaimed when 
it fails, exits without leaving choice points, the alternatives are cut 
of with the !/0 predicate or no choice points have been created since 
the invocation and the last subclause is started (tail recursion 
optimisation). </TR>
<TR VALIGN=top><TD ALIGN=center><STRONG>-G</STRONG> <TD ALIGN=center>4M<TD></EM><B>global 
stack</B><TD>The global stack is used to store terms created during 
Prolog's execution. Terms on this stack will be reclaimed by 
backtracking to a point before the term was created or by garbage 
collection (provided the term is no longer referenced). </TR>
<TR VALIGN=top><TD ALIGN=center><STRONG>-T</STRONG> <TD ALIGN=center>4M<TD></EM><B>trail 
stack</B><TD>The trail stack is used to store assignments during 
execution. Entries on this stack remain alive until backtracking before 
the point of creation or the garbage collector determines they are nor 
needed any longer. </TR>
<TR VALIGN=top><TD ALIGN=center><STRONG>-A</STRONG> <TD ALIGN=center>1M<TD></EM><B>argument 
stack</B><TD>The argument stack is used to store one of the intermediate 
code interpreter's registers. The amount of space needed on this stack 
is determined entirely by the depth in which terms are nested in the 
clauses that constitute the program. Overflow is most likely when using 
long strings in a clause. </TR>
</TABLE>

</CENTER>
<CENTER><B>Table 2 : Memory areas</B></CENTER>
<A NAME="tab:areas"></A>

<P>

<H4><A NAME="sec:2.12.1.1">2.12.1.1 The heap</A></H4>

<A NAME="sec:heap"></A>

<P><A NAME="idx:stack,memory management:140"></A><A NAME="idx:memory,layout:141"></A>With 
the heap, we refer to the memory area used by <B>malloc()</B> and 
friends. SWI-Prolog uses the area to store atoms, functors, predicates 
and their clauses, records and other dynamic data. As of SWI-Prolog 
2.8.5, no limits are imposed on the addresses returned by
<B>malloc()</B> and friends.

<P>On some machines, the runtime stacks described above are allocated 
using `sparse allocation'. Virtual space upto the limit is claimed at 
startup and committed and released while the area grows and shrinks. On 
Win32 platform this is realised using <B>VirtualAlloc()</B> and friends. 
On Unix systems this is realised using <B>mmap()</B>, either mapping
<CODE>/dev/zero</CODE> or a temporary file created in <CODE>/tmp</CODE>.

<P>As Unix provides no way to reserve part of the address space, this 
process may lead to conflicts. By default, SWI-Prolog computes the 
required virtual address space for the runtime stacks. If available, it 
uses <B>getrlimit()</B> to determine the top of the virtual space 
reserved for <B>malloc()</B> usage and locates the stacks in the top of 
this area. It assumes no other <B>mmap()</B> activity such as mapping 
shared libraries or <B>mmap()</B> by foreign modules will use the area 
reserved for the heap and <B>malloc()</B>,
<B>malloc()</B> will grow the heap from low to high addresses and will 
notice the existence of the Prolog stacks.

<P>These assumptions appear to hold. The user may using the
<STRONG>-H</STRONG> to specify the maximum heap-size. In this case, the 
Prolog stacks will be allocated at the indicated size from the current 
top of the heap. On these system, <A NAME="idx:statistics/[0,2]:142"></A><A HREF="sec-3.39.html#statistics/0">statistics/[0,2]</A> 
reports
<A NAME="idx:heaplimit:143">heaplimit</A> and <A NAME="idx:heap:144">heap</A>. 
The <A NAME="idx:heaplimit:145">heaplimit</A> value is the distance 
between the </EM><EM>break</EM> and the first Prolog stack. The <A NAME="idx:heap:146">heap</A> 
value is the distance between what Prolog assumes to be the base of the 
heap<A NAME=back-to-note-6 HREF="Notes.html#note-6"> (6)</A> and the 
current location of the break.

<P>

<H3><A NAME="sec:2.12.2">2.12.2 Other Limits</A></H3>

<P>

<DL>
<DT><B>Clauses</B><DD>
Currently the following limitations apply to clauses. The arity may not 
be more than 1024 and the number of variables should be less than 65536.
<DT><B>Atoms and Strings</B><DD>
SWI-Prolog has no limits on the sizes of atoms and strings. <A NAME="idx:read/1:147"></A><A HREF="sec-3.18.html#read/1">read/1</A> 
and its derivatives however normally limit the number of newlines in an 
atom or string to 5 to improve error detection and recovery. This can be 
switched off with <A NAME="idx:style_check/1:148"></A><A HREF="sec-3.38.html#style_check/1">style_check/1</A>.
<DT><B>Address space</B><DD>
SWI-Prolog data is packed in a 32-bit word, which contains both type and 
value information. The size of the various memory areas is limited to 
128 Mb for each of the areas. With some redesign, the program area could 
be split into data that should be within this range and the rest of the 
data, virtually unlimiting the program size.
<DT><B>Integers</B><DD>
Integers are 32-bit to the user, but integers upto the value of the
</EM><TT>max_tagged_integer</TT> feature are represented more 
efficiently.
<DT><B>Floats</B><DD>
Floating point numbers are represented as native double precision 
floats, 64 bit IEEE on most machines.
</DL>

<P>

<H3><A NAME="sec:2.12.3">2.12.3 Reserved Names</A></H3>

<P>The boot compiler (see <STRONG>-b</STRONG> option) does not support 
the module system (yet). As large parts of the system are written in 
Prolog itself we need some way to avoid name clashes with the user's 
predicates, database keys, etc. Like Edinburgh C-Prolog <CITE><A HREF="sec-C.html#CPROLOG:manual">Pereira, 
1986</A></CITE> all predicates, database keys, etc. that should be 
hidden from the user start with a dollar (<CODE>$</CODE>) sign (see <A NAME="idx:style_check/1:149"></A><A HREF="sec-3.38.html#style_check/1">style_check/1</A>).

<P>The compiler uses the special functor <CODE>$VAR$/1</CODE> while 
analysing the clause to compile. Using this functor in a program causes 
unpredictable behaviour of the compiler and resulting program.
