<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 2.7</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-2.6.html">
<LINK REL=next HREF="sec-2.8.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-2.6.html">Previous</A>
<A HREF="sec-2.8.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:2.7">2.7 Compilation</A></H2>

<A NAME="sec:compilation"></A>

<P>

<H3><A NAME="sec:2.7.1">2.7.1 During program development</A></H3>

<P>During program development, programs are normally loaded using
<A NAME="idx:consult/1:78"></A><A HREF="sec-3.2.html#consult/1">consult/1</A>, 
or the list abreviation. It is common practice to organise a project as 
a collection of source-files and a <EM>load-file</EM>, a Prolog file 
containing only <A NAME="idx:use_module/[1,2]:79"></A><A HREF="sec-4.4.html#use_module/1">use_module/[1,2]</A> 
or <A NAME="idx:ensure_loaded/1:80"></A><A HREF="sec-3.2.html#ensure_loaded/1">ensure_loaded/1</A> 
directives, possibly with a definition of the <EM>entry-point</EM> of 
the program, the predicate that is normally used to start the program. 
This file is often called <CODE>load.pl</CODE>. If the entry-point is 
called
</EM><EM>go</EM>, a typical session starts as:

<P>

<XMP>
% pl
<banner>

1 ?- [load].
<compilation messages>

Yes
2 ?- go.
<program interaction>
</XMP>

<P>When using Windows, the user may open <CODE>load.pl</CODE> from the 
Windows explorer, which will cause <B>plwin.exe</B> to be started in the 
directory holding <CODE>load.pl</CODE>. Prolog loads <CODE>load.pl</CODE> 
before entering the toplevel.

<P>

<H3><A NAME="sec:2.7.2">2.7.2 For running the result</A></H3>

<P>There are various options if you want to make your program ready for 
real usage. The best choice depends on whether the program is to be used 
only on machines holding the SWI-Prolog development system, the size of 
the program and the operating system (Unix vs. Windows).

<P>

<H4><A NAME="sec:2.7.2.1">2.7.2.1 Creating a shell-script</A></H4>

<P>Especially on Unix systems and not-too-large applications, writing a 
shell-script that simply loads your application and calls the 
entry-point is often a good choice. A skeleton for the script is given 
below, followed by the Prolog code to obtain the program arguments.

<P>

<XMP>
#!/bin/sh

base=<absolute-path-to-source>
PL=pl

exec $PL -f none -g "load_files(['$base/load'],[silent(true)])" -t go -- $*
</XMP>

<P>

<XMP>
go :-
        unix(argv(Arguments)),
        append(_SytemArgs, [--|Args], Arguments), !,
        go(Args).

go(Args) :-
        ...
</XMP>

<P>On Windows systems, similar behaviour can be achieved by creating a 
shortcut to Prolog, passing the proper options or writing a <CODE>.bat</CODE> 
file.

<P>

<H4><A NAME="sec:2.7.2.2">2.7.2.2 Creating a saved-state</A></H4>

<P>For larger programs, as well as for programs that are required run on 
systems that do not have the SWI-Prolog development system installed, 
creating a saved state is the best solution. A saved state is created 
using <A NAME="idx:qsave_program/[1,2]:81"></A><A HREF="sec-6.html#qsave_program/1">qsave_program/[1,2]</A> 
or using the linker plld(1). A saved state is a file containing 
machine-independent intermediate code in a format dedicated for fast 
loading. Optionally, the emulator may be integrated in the saved state, 
creating a single-file, but machine-dependent, exectable. This process 
is descriped in <A HREF="sec-6.html">chapter 6</A>.

<P>

<H4><A NAME="sec:2.7.2.3">2.7.2.3 Compililation using the -c commandline 
option</A></H4>

<P>This mechanism is mostly for backward compatibility. It creates files 
in the same format as saved-states created using <A NAME="idx:qsave_program/[1,2]:82"></A><A HREF="sec-6.html#qsave_program/1">qsave_program/[1,2]</A>, 
but the resulting file is a translation of the source-files read, rather 
than a translation of the state of the machine. Unlike saved states, 
programs created using <STRONG>-c</STRONG> <VAR>file</VAR> do not 
include the Prolog part of the development system. The result is very 
dependent on the local SWI-Prolog installation.

<P>The command below is used to compile one or more source-files.

<P>

<XMP>
pl [options] [-o output] -c file ...
</XMP>

<P>The individual source files may include other files using the 
standard list notation, <A NAME="idx:consult/1:83"></A><A HREF="sec-3.2.html#consult/1">consult/1</A>, <A NAME="idx:ensure_loaded/1:84"></A><A HREF="sec-3.2.html#ensure_loaded/1">ensure_loaded/1</A> 
and <A NAME="idx:use_module/[1,2]:85"></A><A HREF="sec-4.4.html#use_module/1">use_module/[1,2]</A>. 
When the
<STRONG>-o</STRONG> <VAR>file</VAR> option is omitted a file named <CODE>a.out</CODE> 
is created that holds the intermediate code file.

<P>Intermediate code files start with the Unix magic code <CODE><CODE>#!</CODE></CODE> 
and are executable. This implies they can be started as a command:

<P>

<XMP>
% pl -o my_program -c ...
...
% my_program [options]
</XMP>

<P>Alternatively, </EM><TT>my_program</TT> can be started with

<XMP>
% pl -x my_program [options]
</XMP>

<P>The following restrictions apply to source files that are to be 
compiled with `<STRONG>-c</STRONG>':

<P><UL>
<LI><A NAME="idx:term_expansion/2:86"></A><A HREF="sec-3.2.html#term_expansion/2">term_expansion/2</A> 
should not use <A NAME="idx:assert/1:87"></A><A HREF="sec-3.12.html#assert/1">assert/1</A> 
and or <A NAME="idx:retract/1:88"></A><A HREF="sec-3.12.html#retract/1">retract/1</A> 
other than for local computational purposes.
<LI>Files can only be included by the standard include directives: 
[...],
<A NAME="idx:consult/1:89"></A><A HREF="sec-3.2.html#consult/1">consult/1</A>, <A NAME="idx:ensure_loaded/1:90"></A><A HREF="sec-3.2.html#ensure_loaded/1">ensure_loaded/1</A> 
and <A NAME="idx:use_module/[1,2]:91"></A><A HREF="sec-4.4.html#use_module/1">use_module/[1,2]</A>. 
User defined loading predicate invocations will not be compiled.
</UL>

<P>Directives are executed both when compiling the program and when 
loading the intermediate code file.

