<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.13</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.12.html">
<LINK REL=next HREF="sec-3.14.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.12.html">Previous</A>
<A HREF="sec-3.14.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.13">3.13 Declaring Properties of Predicates</A></H2>

<A NAME="ch:dynamic"></A>
<A NAME="sec:declare"></A>

<P>This section describes directives which manipulate attributes of 
predicate definitions. The functors <A NAME="idx:dynamic/1:305"></A><A HREF="sec-3.13.html#dynamic/1">dynamic/1</A>, <A NAME="idx:multifile/1:306"></A><A HREF="sec-3.13.html#multifile/1">multifile/1</A> 
and
<A NAME="idx:discontiguous/1:307"></A><A HREF="sec-3.13.html#discontiguous/1">discontiguous/1</A> 
are operators of priority 1150 (see <A NAME="idx:op/3:308"></A><A HREF="sec-3.22.html#op/3">op/3</A>), 
which implies the list of predicates they involve can just be a comma 
separated list:

<P>

<XMP>
:- dynamic
        foo/0, 
        baz/2.
</XMP>

<P>On SWI-Prolog all these directives are just predicates. This implies 
they can also be called by a program. Do not rely on this feature if you 
want to maintain portability to other Prolog implementations.

<P>

<DL>

<P>
<DT><A NAME="dynamic/1"><STRONG>dynamic</STRONG> <VAR>+Functor/+Arity, \ldots</VAR></A><DD>
Informs the interpreter that the definition of the predicate(s) may 
change during execution (using <A NAME="idx:assert/1:309"></A><A HREF="sec-3.12.html#assert/1">assert/1</A> 
and/or <A NAME="idx:retract/1:310"></A><A HREF="sec-3.12.html#retract/1">retract/1</A>). 
Currently <A NAME="idx:dynamic/1:311"></A><A HREF="sec-3.13.html#dynamic/1">dynamic/1</A> 
only stops the interpreter from complaining about undefined predicates 
(see
<A NAME="idx:unknown/2:312"></A><A HREF="sec-3.38.html#unknown/2">unknown/2</A>). 
Future releases might prohibit <A NAME="idx:assert/1:313"></A><A HREF="sec-3.12.html#assert/1">assert/1</A> 
and <A NAME="idx:retract/1:314"></A><A HREF="sec-3.12.html#retract/1">retract/1</A> 
for not-dynamic declared procedures.

<P>
<DT><A NAME="multifile/1"><STRONG>multifile</STRONG> <VAR>+Functor/+Arity, \ldots</VAR></A><DD>
Informs the system that the specified predicate(s) may be defined over 
more than one file. This stops <A NAME="idx:consult/1:315"></A><A HREF="sec-3.2.html#consult/1">consult/1</A> 
from redefining a predicate when a new definition is found.

<P>
<DT><A NAME="discontiguous/1"><STRONG>discontiguous</STRONG> <VAR>+Functor/+Arity, \ldots</VAR></A><DD>
Informs the system that the clauses of the specified predicate(s) might 
not be together in the source file. See also <A NAME="idx:style_check/1:316"></A><A HREF="sec-3.38.html#style_check/1">style_check/1</A>.

<P>
<DT><A NAME="index/1"><STRONG>index</STRONG>(<VAR>+Head</VAR>)</A><DD>
Index the clauses of the predicate with the same name and arity as
<VAR>Head</VAR> on the specified arguments. <VAR>Head</VAR> is a term of 
which all arguments are either `1' (denoting `index this argument') or 
`0' (denoting `do not index this argument'). Indexing has no 
implications for the semantics of a predicate, only on its performance. 
If indexing is enabled on a predicate a special purpose algorithm is 
used to select candidate clauses based on the actual arguments of the 
goal. This algorithm checks whether indexed arguments might unify in the 
clause head. Only atoms, integers and functors (e.g. name and arity of a 
term) are considered. Indexing is very useful for predicates with many 
clauses representing facts.

<P>Due to the representation technique used at most 4 arguments can be 
indexed. All indexed arguments should be in the first 32 arguments of 
the predicate. If more than 4 arguments are specified for indexing only 
the first 4 will be accepted. Arguments above 32 are ignored for 
indexing.

<P>By default all predicates with <VAR>&lt;<VAR>arity</VAR>&gt; &gt;= 1</VAR> 
are indexed on their first argument. It is possible to redefine indexing 
on predicates that already have clauses attached to them. This will 
initiate a scan through the predicates clause list to update the index 
summary information stored with each clause.

<P>If---for example---one wants to represents sub-types using a fact 
list `sub_type(Sub, Super)' that should be used both to determine sub- 
and super types one should declare sub_type/2 as follows:

<P>

<XMP>
:- index(sub_type(1, 1)).

sub_type(horse, animal).
...
...
</XMP>

<P>
</DL>

