<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.15</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.14.html">
<LINK REL=next HREF="sec-3.16.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.14.html">Previous</A>
<A HREF="sec-3.16.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.15">3.15 Input and Output</A></H2>

<P>SWI-Prolog provides two different packages for input and output. One 
confirms to the Edinburgh standard. This package has a notion of 
`current-input' and `current-output'. The reading and writing predicates 
implicitly refer to these streams. In the second package, streams are 
opened explicitly and the resulting handle is used as an argument to the 
reading and writing predicate to specify the source or destination. Both 
packages are fully integrated; the user may switch freely between them.

<P>

<H3><A NAME="sec:3.15.1">3.15.1 Input and Output Using Implicit Source 
and Destination</A></H3>

<P>The package for implicit input and output destination is upwards 
compatible to DEC-10 and C-Prolog. The reading and writing predicates 
refer to resp. the current input- and output stream. Initially these 
streams are connected to the terminal. The current output stream is 
changed using <A NAME="idx:tell/1:335"></A><A HREF="sec-3.15.html#tell/1">tell/1</A> 
or <A NAME="idx:append/1:336"></A><A HREF="sec-3.15.html#append/1">append/1</A>. 
The current input stream is changed using <A NAME="idx:see/1:337"></A><A HREF="sec-3.15.html#see/1">see/1</A>. 
The streams current value can be obtained using <A NAME="idx:telling/1:338"></A><A HREF="sec-3.15.html#telling/1">telling/1</A> 
for output- and <A NAME="idx:seeing/1:339"></A><A HREF="sec-3.15.html#seeing/1">seeing/1</A> 
for input streams. The table below shows the valid stream 
specifications. The reserved names <CODE>user_input</CODE>,
<CODE>user_output</CODE> and <CODE>user_error</CODE> are for neat 
integration with the explicit streams.

<P><CENTER>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>user</CODE> <TD>This reserved name refers to 
the terminal </TR>
<TR VALIGN=top><TD><CODE>user_input</CODE> <TD>Input from the terminal </TR>
<TR VALIGN=top><TD><CODE>user_output</CODE> <TD>Output to the terminal </TR>
<TR VALIGN=top><TD><CODE>user_error</CODE> <TD>Unix error stream (output 
only) </TR>
<TR VALIGN=top><TD>&lt;<VAR>Atom</VAR>&gt; <TD>Name of a Unix file </TR>
<TR VALIGN=top><TD><CODE>pipe(&lt;<VAR>Atom</VAR>&gt;)</CODE> <TD>Name 
of a Unix command </TR>
</TABLE>

</CENTER>

<P>Source and destination are either a file, one of the reserved words 
above, or a term `pipe(<VAR>Command</VAR>)'. In the predicate 
descriptions below we will call the source/destination argument `<VAR>SrcDest</VAR>'. 
Below are some examples of source/destination specifications.

<P><CENTER>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD><CODE>?- see(data).</CODE> <TD>% Start reading from 
file `data'. </TR>
<TR VALIGN=top><TD><CODE>?- tell(stderr).</CODE> <TD>% Start writing on 
the error stream. </TR>
<TR VALIGN=top><TD><CODE>?- tell(pipe(lpr)).</CODE> <TD>% Start writing 
to the printer.</TR>
</TABLE>

</CENTER>

<P>Another example of using the pipe/1 construct is shown below. Note 
that the pipe/1 construct is not part of Prolog's standard I/O 
repertoire.

<P>

<XMP>
getwd(Wd) :-
        seeing(Old), see(pipe(pwd)), 
        collect_wd(String), 
        seen, see(Old), 
        atom_chars(Wd, String).

collect_wd([C|R]) :-
        get0(C), C \== -1, !, 
        collect_wd(R).
collect_wd([]).
</XMP>

<P>

<DL>

<P>
<DT><A NAME="see/1"><STRONG>see</STRONG>(<VAR>+SrcDest</VAR>)</A><DD>
Make <VAR>SrcDest</VAR> the current input stream. If <VAR>SrcDest</VAR> 
was already opened for reading with <A NAME="idx:see/1:340"></A><A HREF="sec-3.15.html#see/1">see/1</A> 
and has not been closed since, reading will be resumed. Otherwise <VAR>SrcDest</VAR> 
will be opened and the file pointer is positioned at the start of the 
file.

<P>
<DT><A NAME="tell/1"><STRONG>tell</STRONG>(<VAR>+SrcDest</VAR>)</A><DD>
Make <VAR>SrcDest</VAR> the current output stream. If <VAR>SrcDest</VAR> 
was already opened for writing with <A NAME="idx:tell/1:341"></A><A HREF="sec-3.15.html#tell/1">tell/1</A> 
or <A NAME="idx:append/1:342"></A><A HREF="sec-3.15.html#append/1">append/1</A> 
and has not been closed since, writing will be resumed. Otherwise the 
file is created or---when existing---truncated. See also <A NAME="idx:append/1:343"></A><A HREF="sec-3.15.html#append/1">append/1</A>.

<P>
<DT><A NAME="append/1"><STRONG>append</STRONG>(<VAR>+File</VAR>)</A><DD>
Similar to <A NAME="idx:tell/1:344"></A><A HREF="sec-3.15.html#tell/1">tell/1</A>, 
but positions the file pointer at the end of <VAR>File</VAR> rather than 
truncating an existing file. The pipe construct is not accepted by this 
predicate.

<P>
<DT><A NAME="seeing/1"><STRONG>seeing</STRONG>(<VAR>?SrcDest</VAR>)</A><DD>
Unify the name of the current input stream with <VAR>SrcDest</VAR>.

<P>
<DT><A NAME="telling/1"><STRONG>telling</STRONG>(<VAR>?SrcDest</VAR>)</A><DD>
Unify the name of the current output stream with <VAR>SrcDest</VAR>.

<P>
<DT><A NAME="seen/0"><STRONG>seen</STRONG></A><DD>
Close the current input stream. The new input stream becomes
<VAR>user</VAR>.

<P>
<DT><A NAME="told/0"><STRONG>told</STRONG></A><DD>
Close the current output stream. The new output stream becomes
<VAR>user</VAR>.
</DL>

<P>

<H3><A NAME="sec:3.15.2">3.15.2 Explicit Input and Output Streams</A></H3>

<P>The predicates below are part of the Quintus compatible stream-based 
I/O package. In this package streams are explicitly created using the 
predicate <A NAME="idx:open/3:345"></A><A HREF="sec-3.15.html#open/3">open/3</A>. 
The resulting stream identifier is then passed as a parameter to the 
reading and writing predicates to specify the source or destination of 
the data.

<P>

<DL>

<P>
<DT><A NAME="open/4"><STRONG>open</STRONG>(<VAR>+SrcDest, +Mode, 
-Stream, +Options</VAR>)</A><DD>
ISO compliant predicate to open a stream. <VAR>SrcDes</VAR> is either an 
atom, specifying a Unix file, or a term `<CODE>pipe(<VAR>Command</VAR>)</CODE>', 
just like <A NAME="idx:see/1:346"></A><A HREF="sec-3.15.html#see/1">see/1</A> 
and <A NAME="idx:tell/1:347"></A><A HREF="sec-3.15.html#tell/1">tell/1</A>. <VAR>Mode</VAR> 
is one of <CODE>read</CODE>,
<CODE>write</CODE>, <CODE>append</CODE> or <CODE>update</CODE>. Mode <CODE>append</CODE> 
opens the file for writing, positioning the file-pointer at the end. 
Mode <CODE>update</CODE> opens the file for writing, positioning the 
file-pointer at the beginning of the file without truncating the file. 
See also <A NAME="idx:stream_position/3:348"></A><A HREF="sec-3.15.html#stream_position/3">stream_position/3</A>. <VAR>Stream</VAR> 
is either a variable, in which case it is bound to an integer 
identifying the stream, or an atom, in which case this atom will be the 
stream identifier. The <VAR>Options</VAR> list can contain the following 
options:

<P>

<DL>

<P>
<DT><STRONG>type</STRONG>(<VAR>Type</VAR>)<DD>
Using type <CODE>text</CODE> (default), Prolog will write a text-file in 
an operating-system compatible way. Using type <CODE>binary</CODE> the 
bytes will be read or written without any translation. Note there is no 
difference between the two on Unix systems.

<P>

<P>
<DT><STRONG>alias</STRONG>(<VAR>Atom</VAR>)<DD>
Gives the stream a name. The following two calls are identical, but only 
the latter is allowed in ISO Prolog.

<XMP>
?- open(foo, read, in, []).
?- open(foo, read, S, [alias(in)]).
</XMP>

<P>

<P>
<DT><STRONG>eof_action</STRONG>(<VAR>Action</VAR>)<DD>
Defines what happens if the end of the input stream is reached. Action
<CODE>eof_code</CODE> makes <A NAME="idx:get0/1:349"></A><A HREF="sec-3.17.html#get0/1">get0/1</A> 
and friends return -1 and <A NAME="idx:read/1:350"></A><A HREF="sec-3.18.html#read/1">read/1</A> 
and friends return the atom <CODE>end_of_file</CODE>. Repetitive reading 
keeps yielding the same result. Action <CODE>error</CODE> is like <CODE>eof_code</CODE>, 
but repetitive reading will raise an error. With action <CODE>reset</CODE>, 
Prolog will examine the file again and return more data if the file has 
grown.

<P>

<P>
<DT><STRONG>buffer</STRONG>(<VAR>Buffering</VAR>)<DD>
Defines output buffering. The atom <CODE>fullf</CODE> (default) defines 
full buffering, <CODE>line</CODE> buffering by line, and <CODE>false</CODE> 
implies the stream is fully unbuffered. Smaller buffering is useful if 
another process or the user is waiting for the output as it is being 
produced. See also <A NAME="idx:flush/0:351"></A><A HREF="sec-3.17.html#flush/0">flush/0</A> 
and <A NAME="idx:flush_output/1:352"></A><A HREF="sec-3.17.html#flush_output/1">flush_output/1</A>. 
This option is not an ISO option.

<P>

<P>
<DT><STRONG>close_on_abort</STRONG>(<VAR>Bool</VAR>)<DD>
If <CODE>true</CODE> (default), the stream is closed on an abort (see
<A NAME="idx:abort/0:353"></A><A HREF="sec-3.36.html#abort/0">abort/0</A>). 
If <CODE>false</CODE>, the stream is not closed. If it is an output 
stream, it will be flushed however. Useful for logfiles and if the 
stream is associated to a process (using the pipe/1 construct).
</DL>

<P>The option <CODE>reposition</CODE> is not supported in SWI-Prolog. 
All streams connected to a file may be repositioned.

<P>
<DT><A NAME="open/3"><STRONG>open</STRONG>(<VAR>+SrcDest, +Mode, ?Stream</VAR>)</A><DD>
Equivalent to <A NAME="idx:open/4:354"></A><A HREF="sec-3.15.html#open/4">open/4</A> 
with an empty option-list.

<P>
<DT><A NAME="open_null_stream/1"><STRONG>open_null_stream</STRONG>(<VAR>?Stream</VAR>)</A><DD>
Open a stream that produces no output. All counting functions are 
enabled on such a stream. An attempt to read from a null-stream will 
immediately signal end-of-file. Similar to Unix <CODE>/dev/null</CODE>.
<VAR>Stream</VAR> can be an atom, giving the null-stream an alias name.

<P>
<DT><A NAME="close/1"><STRONG>close</STRONG>(<VAR>+Stream</VAR>)</A><DD>

<P>Close the specified stream. If <VAR>Stream</VAR> is not open an error 
message is displayed. If the closed stream is the current input or 
output stream the terminal is made the current input or output.

<P>
<DT><A NAME="current_stream/3"><STRONG>current_stream</STRONG>(<VAR>?File, 
?Mode, ?Stream</VAR>)</A><DD>
Is true if a stream with file specification <VAR>File</VAR>, mode <VAR>Mode</VAR> 
and stream identifier <VAR>Stream</VAR> is open. The reserved streams
<CODE>user</CODE> and <CODE>user_error</CODE> are not generated by this 
predicate. If a stream has been opened with mode <CODE>append</CODE> 
this predicate will generate mode <CODE>write</CODE>.

<P>
<DT><A NAME="stream_position/3"><STRONG>stream_position</STRONG>(<VAR>+Stream, 
-Old, +New</VAR>)</A><DD>
Unify the position parameters of <VAR>Stream</VAR> with <VAR>Old</VAR> 
and set them to
<VAR>New</VAR>. A position is represented by the following term:

<XMP>
'$stream_position'(CharNo, LineNo, LinePos).
</XMP>

<P>It is only possible to change the position parameters if the stream 
is connected to a disk file. If the position is changed, the <VAR>CharNo</VAR> 
field determines the new position in the file. The <VAR>LineNo</VAR> and
<VAR>LinePos</VAR> are copied in the stream administration. See also <A NAME="idx:seek/4:355"></A><A HREF="sec-3.15.html#seek/4">seek/4</A>.

<P>
<DT><A NAME="seek/4"><STRONG>seek</STRONG>(<VAR>+Stream, +Offset, 
+Method, -NewLocation</VAR>)</A><DD>
Reposition the current point of the given <VAR>Stream</VAR>. <VAR>Method</VAR> 
is one of <CODE>bof</CODE>, <VAR>current</VAR> or <VAR>eof</VAR>, 
indicating positioning relative to the start, current point or end of 
the underlying object. <VAR>NewLocation</VAR> is unified with the new 
offset, relative to the start of the stream.

<P>If the seek modifies the current location, the line number and 
character position in the line are set to 0.

<P>If the stream cannot be repostioned, a <CODE>reposition</CODE> error 
is raised. The predicate <A NAME="idx:seek/4:356"></A><A HREF="sec-3.15.html#seek/4">seek/4</A> 
is compatible to Quintus Prolog, though the error conditions and 
signalling is ISO compliant. See also
<A NAME="idx:stream_position/3:357"></A><A HREF="sec-3.15.html#stream_position/3">stream_position/3</A>.
</DL>

<P>

<H3><A NAME="sec:3.15.3">3.15.3 Switching Between Implicit and Explicit 
I/O</A></H3>

<P>The predicates below can be used for switching between the implicit- 
and the explicit stream based I/O predicates.

<P>

<DL>

<P>
<DT><A NAME="set_input/1"><STRONG>set_input</STRONG>(<VAR>+Stream</VAR>)</A><DD>
Set the current input stream to become <VAR>Stream</VAR>. Thus, 
open(file, read, Stream), set_input(Stream) is equivalent to see(file).

<P>
<DT><A NAME="set_output/1"><STRONG>set_output</STRONG>(<VAR>+Stream</VAR>)</A><DD>
Set the current output stream to become <VAR>Stream</VAR>.

<P>
<DT><A NAME="current_input/1"><STRONG>current_input</STRONG>(<VAR>-Stream</VAR>)</A><DD>
Get the current input stream. Useful to get access to the status 
predicates associated with streams.

<P>
<DT><A NAME="current_output/1"><STRONG>current_output</STRONG>(<VAR>-Stream</VAR>)</A><DD>
Get the current output stream.

<P>
<DT><A NAME="dup_stream/2"><STRONG>dup_stream</STRONG>(<VAR>+From, +To</VAR>)</A><DD>
Duplicate the underlying data from stream <VAR>From</VAR> to streamTo, 
so actions performed on either stream have the same effect. The primary 
goal of this predicate is to facilitate redirection of the user 
interaction to allow for `interactor' windows. For example, the 
following code will redirect output to <CODE>user_output</CODE> and
<CODE>user_error</CODE> to an XPCE text window:

<P>

<XMP>
        ...,
        pce_open(Window, append, Fd),
        dup_stream(user_output, Fd),
        dup_stream(user_error, Fd),
        ...
</XMP>

<P>The old status of a stream can be stored by duplicating to a 
null-stream as obtained using <A NAME="idx:open_null_stream/1:358"></A><A HREF="sec-3.15.html#open_null_stream/1">open_null_stream/1</A>.

<P>This predicate is SWI-Prolog specific.
</DL>

