<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.23</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.22.html">
<LINK REL=next HREF="sec-3.24.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.22.html">Previous</A>
<A HREF="sec-3.24.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.23">3.23 Arithmetic</A></H2>

<A NAME="sec:arith"></A>

<P>Arithmetic can be divided into some special purpose integer 
predicates and a series of general predicates for floating point and 
integer arithmetic as appropriate. The integer predicates are as 
``logical'' as possible. Their usage is recommended whenever applicable, 
resulting in faster and more ``logical'' programs.

<P>The general arithmetic predicates are optionally compiled now (see
<A NAME="idx:please/3:430"></A><A HREF="sec-2.8.html#please/3">please/3</A> 
and the <STRONG>-O</STRONG> command line option). Compiled arithmetic 
reduces global stack requirements and improves performance. 
Unfortunately compiled arithmetic cannot be traced, which is why it is 
optional.

<P>The general arithmetic predicates all handle <VAR>expressions</VAR>. 
An expression is either a simple number or a <VAR>function</VAR>. The 
arguments of a function are expressions. The functions are described in
<A HREF="sec-3.24.html">section 3.24</A>.

<P>

<DL>

<P>
<DT><A NAME="between/3"><STRONG>between</STRONG>(<VAR>+Low, +High, 
?Value</VAR>)</A><DD>
<VAR>Low</VAR> and <VAR>High</VAR> are integers, <VAR><VAR>High</VAR> &gt;=<VAR>Low</VAR></VAR>. 
If
<VAR>Value</VAR> is an integer, <VAR><VAR>Low</VAR> =&lt;<VAR>Value</VAR> 
=&lt;<VAR>High</VAR></VAR>. When <VAR>Value</VAR> is a variable it is 
successively bound to all integers between <VAR>Low</VAR> and <VAR>High</VAR>.

<P>
<DT><A NAME="succ/2"><STRONG>succ</STRONG>(<VAR>?Int1, ?Int2</VAR>)</A><DD>
Succeeds if <VAR><VAR>Int2</VAR> = <VAR>Int1</VAR> + 1</VAR>. At least 
one of the arguments must be instantiated to an integer.

<P>
<DT><A NAME="plus/3"><STRONG>plus</STRONG>(<VAR>?Int1, ?Int2, ?Int3</VAR>)</A><DD>
Succeeds if <VAR><VAR>Int3</VAR> = <VAR>Int1</VAR> + <VAR>Int2</VAR></VAR>. 
At least two of the three arguments must be instantiated to integers.

<P>
<DT><A NAME=">/2"><VAR>+Expr1</VAR> <STRONG>&gt;</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a larger number 
than <VAR>Expr2</VAR>.

<P>
<DT><A NAME="</2"><VAR>+Expr1</VAR> <STRONG>&lt;</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a smaller number 
than <VAR>Expr2</VAR>.

<P>
<DT><A NAME="=</2"><VAR>+Expr1</VAR> <STRONG>=&lt;</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a smaller or 
equal number to <VAR>Expr2</VAR>.

<P>
<DT><A NAME=">=/2"><VAR>+Expr1</VAR> <STRONG>&gt;=</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a larger or equal 
number to <VAR>Expr2</VAR>.

<P>
<DT><A NAME="=\=/2"><VAR>+Expr1</VAR> <STRONG>=\=</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a number 
non-equal to
<VAR>Expr2</VAR>.

<P>
<DT><A NAME="=:=/2"><VAR>+Expr1</VAR> <STRONG>=:=</STRONG> <VAR>+Expr2</VAR></A><DD>
Succeeds when expression <VAR>Expr1</VAR> evaluates to a number equal to <VAR> 
Expr2</VAR>.

<P>
<DT><A NAME="is/2"><VAR>-Number</VAR> <STRONG>is</STRONG> <VAR>+Expr</VAR></A><DD>
Succeeds when <VAR>Number</VAR> has successfully been unified with the 
number
<VAR>Expr</VAR> evaluates to. If <VAR>Expr</VAR> evaluates to a float 
that can be represented using an integer (i.e. the value is integer and 
within the range that can be described by Prolog's integer 
representation), <VAR> Expr</VAR> is unified with the integer value.

<P>Note that normally, <A NAME="idx:is/2:431"></A><A HREF="sec-3.23.html#is/2">is/2</A> 
will be used with unbound left operand. If equality is to be tested, 
=:=/2 should be used. For example:

<P><CENTER>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD><CODE>?- 1.0 is sin(pi/2).</CODE> <TD>Fails!. sin(pi/2) 
evaluates to 1.0, but <A NAME="idx:is/2:432"></A><A HREF="sec-3.23.html#is/2">is/2</A> 
will represent this as the integer 1, after which unify will fail. </TR>
<TR VALIGN=top><TD><CODE>?- 1.0 is float(sin(pi/2)).</CODE> <TD>Succeeds, 
as the <A NAME="idx:float/1:433"></A><A HREF="sec-3.4.html#float/1">float/1</A> 
function forces the result to be float. </TR>
<TR VALIGN=top><TD><CODE>?- 1.0 =:= sin(pi/2).</CODE> <TD>Succeeds as 
expected.</TR>
</TABLE>

</CENTER>
</DL>

