<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.24</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.23.html">
<LINK REL=next HREF="sec-3.25.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.23.html">Previous</A>
<A HREF="sec-3.25.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.24">3.24 Arithmetic Functions</A></H2>

<A NAME="sec:functions"></A>

<P>Arithmetic functions are terms which are evaluated by the arithmetic 
predicates described above. SWI-Prolog tries to hide the difference 
between integer arithmetic and floating point arithmetic from the Prolog 
user. Arithmetic is done as integer arithmetic as long as possible and 
converted to floating point arithmetic whenever one of the arguments or 
the combination of them requires it. If a function returns a floating 
point value which is whole it is automatically transformed into an 
integer. There are three types of arguments to functions:

<P><CENTER>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD><VAR>Expr</VAR> <TD>Arbitrary expression, returning 
either a floating point value or an integer. </TR>
<TR VALIGN=top><TD><VAR>IntExpr</VAR> <TD>Arbitrary expression that 
should evaluate into an integer. </TR>
<TR VALIGN=top><TD><VAR>Int</VAR> <TD>An integer.</TR>
</TABLE>

</CENTER>

<P>In case integer addition, subtraction and multiplication would lead 
to an integer overflow the operands are automatically converted to 
floating point numbers. The floating point functions (<A NAME="idx:sin/1:434"></A><A HREF="sec-3.24.html#sin/1">sin/1</A>, <A NAME="idx:exp/1:435"></A><A HREF="sec-3.24.html#exp/1">exp/1</A>, 
etc.) form a direct interface to the corresponding C library functions 
used to compile SWI-Prolog. Please refer to the C library documentation 
for details on precision, error handling, etc.

<P>

<DL>

<P>
<DT><A NAME="-/1"><STRONG>-</STRONG> <VAR>+Expr</VAR></A><DD>
<VAR><VAR>Result</VAR> = -<VAR>Expr</VAR></VAR>

<P>
<DT><A NAME="+/2"><VAR>+Expr1</VAR> <STRONG>+</STRONG> <VAR>+Expr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> + <VAR>Expr2</VAR></VAR>

<P>
<DT><A NAME="-/2"><VAR>+Expr1</VAR> <STRONG>-</STRONG> <VAR>+Expr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> - <VAR>Expr2</VAR></VAR>

<P>
<DT><A NAME="*/2"><VAR>+Expr1</VAR> <STRONG>*</STRONG> <VAR>+Expr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> &times; <VAR>Expr2</VAR></VAR>

<P>
<DT><A NAME="//2"><VAR>+Expr1</VAR> <STRONG>/</STRONG> <VAR>+Expr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR>/<VAR>Expr2</VAR></VAR>

<P>
<DT><A NAME="mod/2"><VAR>+IntExpr1</VAR> <STRONG>mod</STRONG> <VAR>+IntExpr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> mod <VAR>Expr2</VAR></VAR> 
(remainder of division).

<P>
<DT><A NAME="rem/2"><VAR>+IntExpr1</VAR> <STRONG>rem</STRONG> <VAR>+IntExpr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> rem <VAR>Expr2</VAR></VAR> 
(remainder of division).

<P>
<DT><A NAME="///2"><VAR>+IntExpr1</VAR> <STRONG>//</STRONG> <VAR>+IntExpr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR> div<VAR>Expr2</VAR></VAR> 
(integer division).

<P>
<DT><A NAME="abs/1"><STRONG>abs</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Evaluate <VAR>Expr</VAR> and return the absolute value of it.

<P>
<DT><A NAME="sign/1"><STRONG>sign</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Evaluate to -1 if <VAR><VAR>Expr</VAR> &lt; 0</VAR>, 1 if <VAR><VAR>Expr</VAR> 
&gt; 0</VAR> and 0 if
<VAR><VAR>Expr</VAR> = 0</VAR>.

<P>
<DT><A NAME="max/2"><STRONG>max</STRONG>(<VAR>+Expr1, +Expr2</VAR>)</A><DD>
Evaluates to the largest of both <VAR>Expr1</VAR> and <VAR>Expr2</VAR>.

<P>
<DT><A NAME="min/2"><STRONG>min</STRONG>(<VAR>+Expr1, +Expr2</VAR>)</A><DD>
Evaluates to the smallest of both <VAR>Expr1</VAR> and <VAR>Expr2</VAR>.

<P>
<DT><A NAME="./2"><STRONG>.</STRONG>(<VAR>+Int,</VAR>)</A><DD>
A list of one element evaluates to the element. This implies <CODE>"a"</CODE> 
evaluates to the ASCII value of the letter `a' (97). This option is 
available for compatibility only. It will not work if `<CODE>style_check(+string)</CODE>' 
is active as <CODE>"a"</CODE> will then be transformed into a string 
object. The recommended way to specify the ASCII value of the letter `a' 
is <CODE>0'a</CODE>.

<P>
<DT><A NAME="random/1"><STRONG>random</STRONG>(<VAR>+Int</VAR>)</A><DD>
Evaluates to a random integer <VAR>i</VAR> for which <VAR>0 =&lt; i &lt; <VAR>Int</VAR></VAR>. 
The seed of this random generator is determined by the system clock when 
SWI-Prolog was started.

<P>
<DT><A NAME="round/1"><STRONG>round</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Evaluates <VAR>Expr</VAR> and rounds the result to the nearest integer.

<P>
<DT><A NAME="integer/1"><STRONG>integer</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Same as <A NAME="idx:round/1:436"></A><A HREF="sec-3.24.html#round/1">round/1</A> 
(backward compatibility).

<P>
<DT><A NAME="float/1"><STRONG>float</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Translate the result to a floating point number. Normally, Prolog will 
use integers whenever possible. When used around the 2nd argument of
<A NAME="idx:is/2:437"></A><A HREF="sec-3.23.html#is/2">is/2</A>, the 
result will be returned as a floating point number. In other contexts, 
the operation has no effect.

<P>
<DT><A NAME="float_fractional_part/1"><STRONG>float_fractional_part</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Fractional part of a floating-point number. Negative if <VAR>Expr</VAR> 
is negative, 0 if <VAR>Expr</VAR> is integer.

<P>
<DT><A NAME="float_integer_part/1"><STRONG>float_integer_part</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Integer part of floating-point number. Negative if <VAR>Expr</VAR> is 
negative, <VAR>Expr</VAR> if <VAR>Expr</VAR> is integer.

<P>
<DT><A NAME="truncate/1"><STRONG>truncate</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Truncate <VAR>Expr</VAR> to an integer. Same as <A NAME="idx:float_integer_part/1:438"></A><A HREF="sec-3.24.html#float_integer_part/1">float_integer_part/1</A>.

<P>
<DT><A NAME="floor/1"><STRONG>floor</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Evaluates <VAR>Expr</VAR> and returns the largest integer smaller or 
equal to the result of the evaluation.

<P>
<DT><A NAME="ceiling/1"><STRONG>ceiling</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Evaluates <VAR>Expr</VAR> and returns the smallest integer larger or 
equal to the result of the evaluation.

<P>
<DT><A NAME="ceil/1"><STRONG>ceil</STRONG>(<VAR>+Expr</VAR>)</A><DD>
Same as <A NAME="idx:ceiling/1:439"></A><A HREF="sec-3.24.html#ceiling/1">ceiling/1</A> 
(backward compatibility).

<P>
<DT><A NAME=">>/2"><VAR>+IntExpr</VAR> <STRONG>&gt;&gt;</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise shift <VAR>IntExpr1</VAR> by <VAR>IntExpr2</VAR> bits to the 
right.

<P>
<DT><A NAME="<</2"><VAR>+IntExpr</VAR> <STRONG>&lt;&lt;</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise shift <VAR>IntExpr1</VAR> by <VAR>IntExpr2</VAR> bits to the 
left.

<P>
<DT><A NAME="\//2"><VAR>+IntExpr</VAR> <STRONG>\/</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise `or' <VAR>IntExpr1</VAR> and <VAR>IntExpr2</VAR>.

<P>
<DT><A NAME="/\/2"><VAR>+IntExpr</VAR> <STRONG>/\</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise `and' <VAR>IntExpr1</VAR> and <VAR>IntExpr2</VAR>.

<P>
<DT><A NAME="xor/2"><VAR>+IntExpr</VAR> <STRONG>xor</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise `exclusive or' <VAR>IntExpr1</VAR> and <VAR>IntExpr2</VAR>.

<P>
<DT><A NAME="\/1"><STRONG>\</STRONG> <VAR>+IntExpr</VAR></A><DD>
Bitwise negation.

<P>
<DT><A NAME="sqrt/1"><STRONG>sqrt</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = sqrt(<VAR>Expr</VAR>)</VAR>

<P>
<DT><A NAME="sin/1"><STRONG>sin</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = sin(<VAR>Expr</VAR>)</VAR>. <VAR>Expr</VAR> is 
the angle in radians.

<P>
<DT><A NAME="cos/1"><STRONG>cos</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = cos(<VAR>Expr</VAR>)</VAR>. <VAR>Expr</VAR> is 
the angle in radians.

<P>
<DT><A NAME="tan/1"><STRONG>tan</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = tan(<VAR>Expr</VAR>)</VAR>. <VAR>Expr</VAR> is 
the angle in radians.

<P>
<DT><A NAME="asin/1"><STRONG>asin</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = arcsin(<VAR>Expr</VAR>)</VAR>. <VAR>Result</VAR> 
is the angle in radians.

<P>
<DT><A NAME="acos/1"><STRONG>acos</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = arccos(<VAR>Expr</VAR>)</VAR>. <VAR>Result</VAR> 
is the angle in radians.

<P>
<DT><A NAME="atan/1"><STRONG>atan</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = arctan(<VAR>Expr</VAR>)</VAR>. <VAR>Result</VAR> 
is the angle in radians.

<P>
<DT><A NAME="atan/2"><STRONG>atan</STRONG>(<VAR>+YExpr, +XExpr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = arctan(<VAR>YExpr</VAR>/<VAR>XExpr</VAR>)</VAR>. <VAR>Result</VAR> 
is the angle in radians. The return value is in the range <VAR>[- pi ... 
pi ]</VAR>. Used to convert between rectangular and polar coordinate 
system.

<P>
<DT><A NAME="log/1"><STRONG>log</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = ln(<VAR>Expr</VAR>)</VAR>

<P>
<DT><A NAME="log10/1"><STRONG>log10</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = log10(<VAR>Expr</VAR>)</VAR>

<P>
<DT><A NAME="exp/1"><STRONG>exp</STRONG>(<VAR>+Expr</VAR>)</A><DD>
<VAR><VAR>Result</VAR> = e **<VAR>Expr</VAR></VAR>

<P>
<DT><A NAME="**/2"><VAR>+Expr1</VAR> <STRONG>**</STRONG> <VAR>+Expr2</VAR></A><DD>
<VAR><VAR>Result</VAR> = <VAR>Expr1</VAR>**<VAR>Expr2</VAR></VAR>

<P>
<DT><A NAME="^/2"><VAR>+Expr1</VAR> <STRONG>^</STRONG> <VAR>+Expr2</VAR></A><DD>
Same as **/2. (backward compatibility).

<P>
<DT><A NAME="pi/0"><STRONG>pi</STRONG></A><DD>
Evaluates to the mathematical constant <VAR>pi</VAR> (3.141593).

<P>
<DT><A NAME="e/0"><STRONG>e</STRONG></A><DD>
Evaluates to the mathematical constant <VAR>e</VAR> (2.718282).

<P>
<DT><A NAME="cputime/0"><STRONG>cputime</STRONG></A><DD>
Evaluates to a floating point number expressing the </EM><font size=-1>CPU</font> 
time (in seconds) used by Prolog up till now. See also <A NAME="idx:statistics/2:440"></A><A HREF="sec-3.39.html#statistics/2">statistics/2</A> 
and <A NAME="idx:time/1:441"></A><A HREF="sec-3.39.html#time/1">time/1</A>.
</DL>

