<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.42</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.41.html">
<LINK REL=next HREF="sec-3.43.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.41.html">Previous</A>
<A HREF="sec-3.43.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.42">3.42 Windows DDE interface</A></H2>

<A NAME="sec:DDE"></A>

<P>The predicates in this section deal with MS-Windows `Dynamic Data 
Exchange' or DDE protocol.<A NAME=back-to-note-21 HREF="Notes.html#note-21"> (21)</A> 
A Windows DDE conversation is a form of interprocess communication based 
on sending reserved window-events between the communicating processes.

<P>See also <A HREF="sec-5.4.html">section 5.4</A> for loading Windows 
DLL's into SWI-Prolog.

<P>

<H3><A NAME="sec:3.42.1">3.42.1 DDE client interface</A></H3>

<P>The DDE client interface allows Prolog to talk to DDE server 
programs. We will demonstrate the use of the DDE interface using the 
Windows PROGMAN (Program Manager) application:

<P>

<XMP>
1 ?- open_dde_conversation(progman, progman, C).

C = 0
2 ?- dde_request(0, groups, X)

--> Unifies X with description of groups

3 ?- dde_execute(0, '[CreateGroup("DDE Demo")]').

Yes

4 ?- close_dde_conversation(0).

Yes
</XMP>

<P>For details on interacting with <B>progman</B>, use the SDK online 
manual section on the Shell DDE interface. See also the Prolog
<CODE>library(progman)</CODE>, which may be used to write simple Windows 
setup scripts in Prolog.

<P>

<DL>

<P>
<DT><A NAME="open_dde_conversation/3"><STRONG>open_dde_conversation</STRONG>(<VAR>+Service, 
+Topic, -Handle</VAR>)</A><DD>
Open a conversation with a server supporting the given service name and 
topic (atoms). If successful, <VAR>Handle</VAR> may be used to send 
transactions to the server. If no willing server is found this predicate 
fails silently.

<P>
<DT><A NAME="close_dde_conversation/1"><STRONG>close_dde_conversation</STRONG>(<VAR>+Handle</VAR>)</A><DD>
Close the conversation associated with <VAR>Handle</VAR>. All opened 
conversations should be closed when they're no longer needed, although 
the system will close any that remain open on process termination.

<P>
<DT><A NAME="dde_request/3"><STRONG>dde_request</STRONG>(<VAR>+Handle, 
+Item, -Value</VAR>)</A><DD>
Request a value from the server. <VAR>Item</VAR> is an atom that 
identifies the requested data, and <VAR>Value</VAR> will be a string (<CODE>CF_TEXT</CODE> 
data in DDE parlance) representing that data, if the request is 
successful. If unsuccessful, <VAR>Value</VAR> will be unified with a 
term of form
</EM><TT>error(&lt;<VAR>Reason</VAR>&gt;)</TT>, identifying the problem. 
This call uses SWI-Prolog string objects to return the value rather then 
atoms to reduce the load on the atom-space. See <A HREF="sec-3.21.html">section 
3.21</A> for a discussion on this data type.

<P>
<DT><A NAME="dde_execute/2"><STRONG>dde_execute</STRONG>(<VAR>+Handle, 
+Command</VAR>)</A><DD>
Request the DDE server to execute the given command-string. Succeeds if 
the command could be executed and fails with error message otherwise.

<P>
<DT><A NAME="dde_poke/4"><STRONG>dde_poke</STRONG>(<VAR>+Handle, +Item, 
+Command</VAR>)</A><DD>
Issue a <CODE>POKE</CODE> command to the server on the specified <VAR>Item</VAR>. 
Command is passed as data of type <CODE>CF_TEXT</CODE>.
</DL>

<P>

<H3><A NAME="sec:3.42.2">3.42.2 DDE server mode</A></H3>

<P>The (autoload) <CODE>library(dde)</CODE> defines primitives to 
realise simple DDE server applications in SWI-Prolog. These features are 
provided as of version 2.0.6 and should be regarded prototypes. The 
C-part of the DDE server can handle some more primitives, so if you need 
features not provided by this interface, please study <CODE>library(dde)</CODE>.

<P>

<DL>

<P>
<DT><A NAME="dde_register_service/2"><STRONG>dde_register_service</STRONG>(<VAR>+Template, 
+Goal</VAR>)</A><DD>
Register a server to handle DDE request or DDE execute requests from 
other applications. To register a service for a DDE request, <VAR>Template</VAR> 
is of the form:
<BLOCKQUOTE> +Service(+Topic, +Item, +Value)
</BLOCKQUOTE>
<VAR>Service</VAR> is the name of the DDE service provided (like
<B>progman</B> in the client example above). <VAR>Topic</VAR> is either 
an atom, indicating <VAR>Goal</VAR> only handles requests on this topic 
or a variable that also appears in <VAR>Goal</VAR>. <VAR>Item</VAR> and <VAR>Value</VAR> 
are variables that also appear in <VAR>Goal</VAR>.

<P>The example below registers the Prolog <A NAME="idx:feature/2:544"></A><A HREF="sec-2.8.html#feature/2">feature/2</A> 
predicate to be accessible from other applications. The request may be 
given from the same Prolog as well as from another application.

<P>

<XMP>
?- dde_register_service(prolog(feature, F, V),
                        feature(F, V)).

?- open_dde_conversation(prolog, feature, Handle),
   dde_request(Handle, home, Home),
   close_dde_conversation(Handle).

Home = '/usr/local/lib/pl-2.0.6/'
</XMP>

<P>Handling DDE <CODE>execute</CODE> requests is very similar. In this 
case the template is of the form:

<P><BLOCKQUOTE> +Service(+Topic, +Item)
</BLOCKQUOTE>

<P>Passing a <VAR>Value</VAR> argument is not needed as execute requests 
either succeed or fail. If <VAR>Goal</VAR> fails, a `not processed' is 
passed back to the caller of the DDE request.

<P>
<DT><A NAME="dde_unregister_service/1"><STRONG>dde_unregister_service</STRONG>(<VAR>+Service</VAR>)</A><DD>
Stop responding to <VAR>Service</VAR>. If Prolog is halted, it will 
automatically call this on all open services.

<P>
<DT><A NAME="dde_current_service/2"><STRONG>dde_current_service</STRONG>(<VAR>-Service, 
-Topic</VAR>)</A><DD>
Find currently registered services and the topics served on them.

<P>
<DT><A NAME="dde_current_connection/2"><STRONG>dde_current_connection</STRONG>(<VAR>-Service, 
-Topic</VAR>)</A><DD>
Find currently open conversations.
</DL>

