<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.6</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.5.html">
<LINK REL=next HREF="sec-3.7.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.5.html">Previous</A>
<A HREF="sec-3.7.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.6">3.6 Control Predicates</A></H2>

<P>The predicates of this section implement control structures. Normally 
these constructs are translated into virtual machine instructions by the 
compiler. It is still necessary to implement these constructs as true 
predicates to support meta-calls, as demonstrated in the example below. 
The predicate finds all currently defined atoms of 1 character long. 
Note that the cut has no effect when called via one of these predicates 
(see !/0).

<P>

<XMP>
one_character_atoms(As) :-
        findall(A, (current_atom(A), atom_length(A, 1)), As).
</XMP>

<P>

<DL>

<P>
<DT><A NAME="fail/0"><STRONG>fail</STRONG></A><DD>
Always fail. The predicate <A NAME="idx:fail/0:230"></A><A HREF="sec-3.6.html#fail/0">fail/0</A> 
is translated into a single virtual machine instruction.

<P>
<DT><A NAME="true/0"><STRONG>true</STRONG></A><DD>
Always succeed. The predicate <A NAME="idx:true/0:231"></A><A HREF="sec-3.6.html#true/0">true/0</A> 
is translated into a single virtual machine instruction.

<P>
<DT><A NAME="repeat/0"><STRONG>repeat</STRONG></A><DD>
Always succeed, provide an infinite number of choice points.

<P>
<DT><A NAME="!/0"><STRONG>!</STRONG></A><DD>
Cut. Discard choice points of parent frame and frames created after the 
parent frame. Note that the control structures <A HREF="sec-3.6.html#;/2">;/2</A>,
<A HREF="sec-3.6.html#|/2">|/2</A>, <A HREF="sec-3.6.html#->/2">-&gt;/2</A> 
and <A HREF="sec-3.6.html#\+/1">\+/1</A> are normally handled by the 
compiler and do not create a frame, which implies the cut operates 
through these predicates. Some examples are given below. Note the 
difference between t3/1 and t4/1. Also note the effect of <A NAME="idx:call/1:232"></A><A HREF="sec-3.7.html#call/1">call/1</A> 
in t5/0. As the argument of <A NAME="idx:call/1:233"></A><A HREF="sec-3.7.html#call/1">call/1</A> 
is evaluated by predicates rather than the compiler the cut has no 
effect.

<P><CENTER>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD><CODE>t1 :- (a, !, fail ; b).</CODE> <TD>% cuts a/0 
and t1/0 </TR>
<TR VALIGN=top><TD><CODE>t2 :- (a -&gt; b, ! ; c).</CODE> <TD>% cuts b/0 
and t2/0 </TR>
<TR VALIGN=top><TD><CODE>t3(G) :- a, G, fail.</CODE> <TD>% if `G = !' 
cuts a/0 and t1/1 </TR>
<TR VALIGN=top><TD><CODE>t4(G) :- a, call(G), fail.</CODE> <TD>% if `G = 
!' cut has no effect </TR>
<TR VALIGN=top><TD><CODE>t5 :- call((a, !, fail ; b)).</CODE> <TD>% Cut 
has no effect </TR>
<TR VALIGN=top><TD><CODE>t6 :- <CODE>\+</CODE>(a, !, fail ; b).</CODE><TD>% 
cuts a/0 and t6/0 </TR>
</TABLE>

</CENTER>

<P>
<DT><A NAME=",/2"><VAR>+Goal1</VAR> <STRONG>,</STRONG> <VAR>+Goal2</VAR></A><DD>
Conjunction. Succeeds if both `Goal1' and `Goal2' can be proved. It is 
defined as (this definition does not lead to a loop as the second comma 
is handled by the compiler):

<XMP>
Goal1, Goal2 :- Goal1, Goal2.
</XMP>

<P>

<P>
<DT><A NAME=";/2"><VAR>+Goal1</VAR> <STRONG>;</STRONG> <VAR>+Goal2</VAR></A><DD>
The `or' predicate is defined as:

<XMP>
Goal1 ; _Goal2 :- Goal1.
_Goal1 ; Goal2 :- Goal2.
</XMP>

<P>

<P>
<DT><A NAME="|/2"><VAR>+Goal1</VAR> <STRONG>|</STRONG> <VAR>+Goal2</VAR></A><DD>
Equivalent to <A HREF="sec-3.6.html#;/2">;/2</A>. Retained for 
compatibility only. New code should use <A HREF="sec-3.6.html#;/2">;/2</A>. 
Still nice though for grammar rules.

<P>
<DT><A NAME="->/2"><VAR>+Condition</VAR> <STRONG>-&gt;</STRONG> <VAR>+Action</VAR></A><DD>
If-then and If-Then-Else. The <A HREF="sec-3.6.html#->/2">-&gt;/2</A> 
construct commits to the choices made at its left-hand side, destroying 
choice-points created inside the clause (by <A HREF="sec-3.6.html#;/2">;/2</A>), 
or by goals called by this clause. Unlike <A HREF="sec-3.6.html#!/0">!/0</A>, 
the choicepoint of the predicate as a whole (due to multiple clauses) is <STRONG>not</STRONG> 
destroyed. The combination <A HREF="sec-3.6.html#;/2">;/2</A> and <A HREF="sec-3.6.html#->/2">-&gt;/2</A> 
is defines as:

<XMP>
If -> Then; _Else :- If, !, Then.
If -> _Then; Else :- !, Else.
If -> Then :- If, !, Then.
</XMP>

<P>Note that the operator precedence relation between <CODE><CODE>;</CODE></CODE> 
and
<CODE><CODE>-&gt;</CODE></CODE> ensure <CODE>If -&gt; Then ; Else</CODE> 
is actually a term of the form <CODE>;(-&gt;(If, Then), Else)</CODE>. 
The first two clauses belong to the definition of <A HREF="sec-3.6.html#;/2">;/2</A>), 
while only the last defines
<A HREF="sec-3.6.html#->/2">-&gt;/2</A>.

<P>
<DT><A NAME="*->/2"><VAR>+Condition</VAR> <STRONG>*-&gt;</STRONG> <VAR>+Action ; +Else</VAR></A><DD>
This construct implements the so-called `soft-cut'. The control is 
defined as follows: If <VAR>Condition</VAR> succeeds at least once, the 
semantics is the same as (<VAR>Condition</VAR>, <VAR>Action</VAR>). If
<VAR>Condition</VAR> does not succeed, the semantics is that of (<VAR>Condition</VAR>, <VAR>Else</VAR>). 
In other words, If <VAR>Condition</VAR> succeeds at least once, simply 
behave as the conjunction of
<VAR>Condition</VAR> and <VAR>Action</VAR>, otherwise execute <VAR>Else</VAR>.

<P>
<DT><A NAME="\+/1"><STRONG>\+</STRONG> <VAR>+Goal</VAR></A><DD>
Succeeds if `Goal' cannot be proven (mnemonic: <CODE><CODE>+</CODE></CODE> 
refers to </EM><EM>provable</EM> and the backslash (<CODE><CODE>\</CODE></CODE>) 
is normally used to indicate negation).
</DL>

