<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.7</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.6.html">
<LINK REL=next HREF="sec-3.8.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.6.html">Previous</A>
<A HREF="sec-3.8.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.7">3.7 Meta-Call Predicates</A></H2>

<A NAME="sec:metacall"></A>

<P>Meta call predicates are used to call terms constructed at run time. 
The basic meta-call mechanism offered by SWI-Prolog is to use variables 
as a subclause (which should of course be bound to a valid goal at 
runtime). A meta-call is slower than a normal call as it involves 
actually searching the database at runtime for the predicate, while for 
normal calls this search is done at compile time.

<P>

<DL>

<P>
<DT><A NAME="call/1"><STRONG>call</STRONG>(<VAR>+Goal</VAR>)</A><DD>
Invoke <VAR>Goal</VAR> as a goal. Note that clauses may have variables 
as subclauses, which is identical to <A NAME="idx:call/1:234"></A><A HREF="sec-3.7.html#call/1">call/1</A>, 
except when the argument is bound to the cut. See <A HREF="sec-3.6.html#!/0">!/0</A>.

<P>
<DT><A NAME="call/2"><STRONG>call</STRONG>(<VAR>+Goal, +ExtraArg1, ...</VAR>)</A><DD>
Append <VAR>ExtraArg1, ExtraArg2, ...</VAR> to the argument list of
<VAR>Goal</VAR> and call the result. For example, <CODE>call(plus(1), 2, 
X)</CODE> will call <A NAME="idx:plus/3:235"></A><A HREF="sec-3.23.html#plus/3">plus/3</A>, 
binding <VAR>X</VAR> to 3.

<P>The call/[2..] construct is handled by the compiler, which implies 
that redefinition as a predicate has no effect. The predicates <A NAME="idx:call/[2-6]:236"></A><A HREF="sec-3.7.html#call/2">call/[2-6]</A> 
are defined as true predicates, so they can be handled by interpreted 
code.

<P>
<DT><A NAME="apply/2"><STRONG>apply</STRONG>(<VAR>+Term, +List</VAR>)</A><DD>
Append the members of <VAR>List</VAR> to the arguments of <VAR>Term</VAR> 
and call the resulting term. For example: <CODE>apply(plus(1), [2, X])</CODE> 
will call <CODE>plus(1, 2, X)</CODE>. <A NAME="idx:apply/2:237"></A><A HREF="sec-3.7.html#apply/2">apply/2</A> 
is incorporated in the virtual machine of SWI-Prolog. This implies that 
the overhead can be compared to the overhead of <A NAME="idx:call/1:238"></A><A HREF="sec-3.7.html#call/1">call/1</A>. 
New code should use call/[2..] if the length of
<VAR>List</VAR> is fixed, which is more widely supported and faster 
because there is no need to build and examine the argument list.

<P>
<DT><A NAME="not/1"><STRONG>not</STRONG>(<VAR>+Goal</VAR>)</A><DD>
Succeeds when <VAR>Goal</VAR> cannot be proven. Retained for 
compatibility only. New code should use <A HREF="sec-3.6.html#\+/1">\+/1</A>.

<P>
<DT><A NAME="once/1"><STRONG>once</STRONG>(<VAR>+Goal</VAR>)</A><DD>
Defined as:

<XMP>
once(Goal) :-
        Goal, !.
</XMP>

<P><A NAME="idx:once/1:239"></A><A HREF="sec-3.7.html#once/1">once/1</A> 
can in many cases be replaced with <A HREF="sec-3.6.html#->/2">-&gt;/2</A>. 
The only difference is how the cut behaves (see !/0). The following two 
clauses are identical:

<XMP>
1) a :- once((b, c)), d.
2) a :- b, c -> d.
</XMP>

<P>

<P>
<DT><A NAME="ignore/1"><STRONG>ignore</STRONG>(<VAR>+Goal</VAR>)</A><DD>
Calls <VAR>Goal</VAR> as <A NAME="idx:once/1:240"></A><A HREF="sec-3.7.html#once/1">once/1</A>, 
but succeeds, regardless of whether
<VAR>Goal</VAR> succeeded or not. Defined as:

<P>

<XMP>
ignore(Goal) :-
        Goal, !.
ignore(_).
</XMP>

<P>

<P>
<DT><A NAME="call_with_depth_limit/3"><STRONG>call_with_depth_limit</STRONG>(<VAR>+Goal, 
+Limit, -Result</VAR>)</A><DD>
If <VAR>Goal</VAR> can be proven without recursion deeper than <VAR>Limit</VAR> 
levels, <A NAME="idx:call_with_depth_limit/3:241"></A><A HREF="sec-3.7.html#call_with_depth_limit/3">call_with_depth_limit/3</A> 
succeeds, binding <VAR>Result</VAR> to the deepest recursion level used 
during the proof. Otherwise, <VAR>Result</VAR> is unified with <CODE>depth_limit_exceeded</CODE> 
if the limit was exceeded during the proof, or the entire predicate 
fails if <VAR>Goal</VAR> fails without exceeding <VAR>Limit</VAR>.

<P>The depth-limit is guarded by the internal machinery. This differ 
from the depth computed based on a theoretical model. For example, <A NAME="idx:true/0:242"></A><A HREF="sec-3.6.html#true/0">true/0</A> 
is translated into an inlined virtual machine instruction. Also, <A NAME="idx:repeat/0:243"></A><A HREF="sec-3.6.html#repeat/0">repeat/0</A> 
is not implemented as below, but as a non-deterministic foreign 
predicate.

<P>

<XMP>
repeat.
repeat :-
        repeat.
</XMP>

<P>As a result, <A NAME="idx:call_with_depth_limit/3:244"></A><A HREF="sec-3.7.html#call_with_depth_limit/3">call_with_depth_limit/3</A> 
may still loop inifitly on programs that should theoretically finish in 
finite time. This problem can be cured by using Prolog equivalents to 
such built-in predicates.

<P>This predicate may be used for theorem-provers to realise techniques 
like <EM>iterrative deepening</EM>. It was implemented after discussion 
with Steve Moyle <A HREF="mailto:smoyle@ermine.ox.ac.uk"">smoyle@ermine.ox.ac.uk</A>.
</DL>

