<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.8</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.7.html">
<LINK REL=next HREF="sec-3.9.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.7.html">Previous</A>
<A HREF="sec-3.9.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.8">3.8 ISO compliant Exception handling</A></H2>

<A NAME="sec:exception"></A>

<P>SWI-Prolog defines the predicates <A NAME="idx:catch/3:245"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
and <A NAME="idx:throw/1:246"></A><A HREF="sec-3.8.html#throw/1">throw/1</A> 
for ISO compliant raising and catching of exceptions. In the current 
implementation (2.9.0), only part of the built-in predicates generate 
exceptions. In general, exceptions are implemented for I/O and 
arithmetic.

<P>

<DL>

<P>
<DT><A NAME="catch/3"><STRONG>catch</STRONG>(<VAR>:Goal, +Catcher, 
:Recover</VAR>)</A><DD>
Behaves as <A NAME="idx:call/1:247"></A><A HREF="sec-3.7.html#call/1">call/1</A> 
if no exception is raised when executing <VAR>Goal</VAR>. If a exception 
is raised using <A NAME="idx:throw/1:248"></A><A HREF="sec-3.8.html#throw/1">throw/1</A> 
while <VAR>Goal</VAR> executes, and the <VAR>Goal</VAR> is the innermost 
goal for which <VAR>Catcher</VAR> unifies with the argument of <A NAME="idx:throw/1:249"></A><A HREF="sec-3.8.html#throw/1">throw/1</A>, 
all choicepoints generated by <VAR>Goal</VAR> are cut, and <VAR>Recover</VAR> 
is called as in <A NAME="idx:call/1:250"></A><A HREF="sec-3.7.html#call/1">call/1</A>.

<P>The overhead of calling a goal through <A NAME="idx:catch/3:251"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
is very comparable to
<A NAME="idx:call/1:252"></A><A HREF="sec-3.7.html#call/1">call/1</A>. 
Recovery from an exception has a similar overhead.

<P>
<DT><A NAME="throw/1"><STRONG>throw</STRONG>(<VAR>+Exception</VAR>)</A><DD>
Raise an exception. The system will look for the innermost <A NAME="idx:catch/3:253"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
ancestor for which <VAR>Exception</VAR> unifies with the <VAR>Catcher</VAR> 
argument of the <A NAME="idx:catch/3:254"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
call. See <A NAME="idx:catch/3:255"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
for details.

<P>If there is no <A NAME="idx:catch/3:256"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
willing to catch the error in the current Prolog context, the toplevel (<A NAME="idx:prolog/0:257"></A><A HREF="sec-3.36.html#prolog/0">prolog/0</A>) 
catches the error and prints a warning message. If an exception was 
raised in a callback from C (see
<A HREF="sec-5.html">chapter 5</A>), <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> 
will fail and the exception context can be retrieved using <A HREF="sec-5.6.html#PL_exception()">PL_exception()</A>.
</DL>

<P>

<H3><A NAME="sec:3.8.1">3.8.1 Debugging and exceptions</A></H3>

<P><A NAME="idx:exceptions,debugging:258"></A><A NAME="idx:debugging,exceptions:259"></A>Before 
the introduction of exceptions in SWI-Prolog a runtime error was handled 
by printing an error message, after which the predicate failed. If the 
feature (see <A NAME="idx:feature/2:260"></A><A HREF="sec-2.8.html#feature/2">feature/2</A>) <CODE>debug_on_error</CODE> 
was in effect (default), the tracer was switched on. The combination of 
the error message and trace information is generally sufficient to 
locate the error.

<P>With exception handling, things are different. A programmer may wish 
to trap an exception using <A NAME="idx:catch/3:261"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
to avoid it reaching the user. If the exception is not handled by 
user-code, the interactive toplevel will trap it to prevent termination.

<P>If we do not take special precautions, the context information 
associated with an unexpected exception (i.e. a programming error) is 
lost. Therefore, if an exception is raised, which is not caught using
<A NAME="idx:catch/3:262"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
and the toplevel is running, the error will be printed, and the system 
will enter trace mode.

<P>If the system is in an non-interactive callback from foreign code and 
there is no <A NAME="idx:catch/3:263"></A><A HREF="sec-3.8.html#catch/3">catch/3</A> 
active in the current context, it cannot determine whether or not the 
exception will be caught by the external routine calling Prolog. It will 
then base its behaviour on the feature debug_on_error:

<P><UL>
<LI><I>feature(debug_on_error, false)</I><BR>
The exception does not trap the debugger and is returned to the foreign 
routine calling Prolog, where it can be accessed using <A HREF="sec-5.6.html#PL_exception()">PL_exception()</A>. 
This is the default.
<LI><I>feature(debug_on_error, true)</I><BR>
If the exception is not caught by Prolog in the current context, it will 
trap the tracer to help analysing the context of the error.
</UL>

<P>While looking for the context in which an exception takes place, it 
is adviced to switch on debug mode using the predicate <A NAME="idx:debug/0:264"></A><A HREF="sec-3.38.html#debug/0">debug/0</A>.

<P>

<H3><A NAME="sec:3.8.2">3.8.2 The exception term</A></H3>

<A NAME="sec:exceptterm"></A>

<P>Builtin predicates generates exceptions using a term
<CODE>error(Formal, Context)</CODE>. The first argument is the `formal' 
description of the error, specifying the class and generic defined 
context information. When applicable, the ISO error-term definition is 
used. The second part describes some additional context to help the 
programmer while debugging. In its most generic form this is a term of 
the form <CODE>context(Name/Arity, Message)</CODE>, where
<VAR>Name</VAR>/<VAR>Arity</VAR> describes the built-in predicate that 
raised the error, and <VAR>Message</VAR> provides an additional 
description of the error. Any part of this structure may be a variable 
if no information was present.

<P>

<H3><A NAME="sec:3.8.3">3.8.3 Printing a message from an exception</A></H3>

<P>The predicate <A NAME="idx:print_message/2:265"></A><A HREF="sec-3.8.html#print_message/2">print_message/2</A> 
may be used to print an exception term in a human readable format:

<P>

<DL>

<P>
<DT><A NAME="print_message/2"><STRONG>print_message</STRONG>(<VAR>+Kind, 
+Term</VAR>)</A><DD>
This predicate is modelled after the Quintus predicate with the same 
name, though its current implementation is incomplete. It is used only 
for printing messages from exceptions from built-in predicates.
<VAR>Kind</VAR> is one of <CODE>informational</CODE>, <CODE>warning</CODE>, 
consterror, <CODE>help</CODE> or <CODE>silent</CODE>. Currently only
<CODE>error</CODE> is defined. <VAR>Term</VAR> is an <B>error(2)</B> 
term described in <A HREF="sec-3.8.html">section 3.8.2</A>. A 
human-readable message is printed to the stream <CODE>user_error</CODE>.

<P>This predicate first obtains the `human translation' of <VAR>Term</VAR> 
and then calls <A NAME="idx:message_hook/3:266"></A><A HREF="sec-3.8.html#message_hook/3">message_hook/3</A>. 
If this fails the message is printed to the stream <CODE>user_error</CODE>.

<P>The <A NAME="idx:print_message/2:267"></A><A HREF="sec-3.8.html#print_message/2">print_message/2</A> 
predicate and its rules are in the file
<CODE>&lt;<VAR>plhome</VAR>&gt;/boot/messages.pl</CODE>, which may be 
inspected for more information on the error messages and related error 
terms.

<P>
<DT><A NAME="message_hook/3"><STRONG>message_hook</STRONG>(<VAR>+Term, 
+Kind, +Message</VAR>)</A><DD>
Hook predicate that may be define in the module <CODE>user</CODE> to 
intercept messages from <A NAME="idx:print_message/2:268"></A><A HREF="sec-3.8.html#print_message/2">print_message/2</A>. <VAR>Term</VAR> 
and <VAR>Kind</VAR> are the same as passed to <A NAME="idx:print_message/2:269"></A><A HREF="sec-3.8.html#print_message/2">print_message/2</A>. <VAR>Message</VAR> 
is a string containing the human readable translation of the message. If 
this predicate succeeds, <A NAME="idx:print_message/2:270"></A><A HREF="sec-3.8.html#print_message/2">print_message/2</A> 
considers the message printed.

<P>This predicate should be defined dynamic and multifile to allow other 
modules defining clauses for it too.
</DL>

