<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 3.9</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-3.8.html">
<LINK REL=next HREF="sec-3.10.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-3.8.html">Previous</A>
<A HREF="sec-3.10.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:3.9">3.9 Handling signals</A></H2>

<A NAME="sec:signal"></A>

<P>As of version 3.1.0, SWI-Prolog is capable to handle software 
interrupts (signals) in Prolog as well as in foreign (C) code (see <A HREF="sec-5.6.html">section 
5.6.11</A>).

<P>Signals are used to handle internal errors (execution of a 
non-existing CPU intruction, arithmetic domain errors, illegal memory 
access, resource overflow, etc.), as well as for dealing asynchronous 
inter-process communication.

<P>Signals are defined by the Posix standard and part of all Unix 
machines. The MS-Windows Win32 provides a subset of the signal handling 
routines, lacking the vital funtionality to raise a signal in another 
thread for achieving asynchronous inter-process (or inter-thread) 
communication (Unix kill() function).

<P>

<DL>

<P>
<DT><A NAME="on_signal/3"><STRONG>on_signal</STRONG>(<VAR>+Signal, -Old, 
:New</VAR>)</A><DD>
Determines the reaction on <VAR>Signal</VAR>. <VAR>Old</VAR> is unified 
with the old behaviour, while the behaviour is switched to <VAR>New</VAR>. 
As with similar environment-control predicates, the current value is 
retrieved using <CODE>on_signal(Signal, Current, Current)</CODE>.

<P>The action description is an atom denoting the name of the predicate 
that will be called if <VAR>Signal</VAR> arrives. <A NAME="idx:on_signal/3:271"></A><A HREF="sec-3.9.html#on_signal/3">on_signal/3</A> 
is a meta predicate, which implies that &lt;<VAR>Module</VAR>&gt;:&lt;<VAR>Name</VAR>&gt; 
refers the &lt;<VAR>Name</VAR>&gt;/1 in the module &lt;<VAR>Module</VAR>&gt;.

<P>Two predicate-names have special meaning. <CODE>throw</CODE> implies 
Prolog will map the signal onto a Prolog exception as described in
<A HREF="sec-3.8.html">section 3.8</A>. <CODE>default</CODE> resets the 
handler to the settings active before SWI-Prolog manipulated the 
handler.

<P>After receiving a signal mapped to <CODE>throw</CODE>, the exception 
raised has the structure

<P><BLOCKQUOTE></EM><TT>error(signal(&lt;<VAR>SigName</VAR>&gt;, &lt;<VAR>SigNum</VAR>&gt;), &lt;<VAR>Context</VAR>&gt;)
</TT></BLOCKQUOTE>

<P>One possible usage of this is, for example, to limit the time spent 
on proving a goal. This requires a little C-code for setting the alarm 
timer (see <A HREF="sec-5.html">chapter 5</A>):

<P>

<XMP>
#include <SWI-Prolog.h>
#include <unistd.h>

foreign_t
pl_alarm(term_t time)
{ double t;

  if ( PL_get_float(time, &t) )
  { alarm((long)(t+0.5));

    PL_succeed;
  }

  PL_fail;
}


install_t
install()
{ PL_register_foreign("alarm", 1, pl_alarm, 0);
}
</XMP>

<P>Next, we can define the following Prolog code:

<P>

<XMP>
:- load_foreign_library(alarm).

:- on_signal(alrm, throw).

:- module_transparent
        call_with_time_limit/2.

call_with_time_limit(Goal, MaxTime) :-
        alarm(MaxTime),
        catch(Goal, signal(alrm, _), fail), !,
        alarm(0).
call_with_time_limit(_, _) :-
        alarm(0),
        fail.
</XMP>

<P>The signal names are defined by the C-Posix standards as symbols of 
the form </EM><TT>SIG_</TT>&lt;<VAR>SIGNAME</VAR>&gt;. The Prolog name 
for a signal is the lowercase version of &lt;<VAR>SIGNAME</VAR>&gt;. The 
predicate <A NAME="idx:current_signal/3:272"></A><A HREF="sec-3.9.html#current_signal/3">current_signal/3</A> 
may be used to map between names and signals.

<P>Initially, some signals are mapped to <CODE>throw</CODE>, while all 
other signals are <CODE>default</CODE>. The following signals throw an 
exception: <CODE>ill</CODE>, <CODE>fpe</CODE>, <CODE>segv</CODE>, <CODE>pipe</CODE>,
<CODE>alrm</CODE>, <CODE>bus</CODE>, <CODE>xcpu</CODE>, <CODE>xfsz</CODE> 
and
<CODE>vtalrm</CODE>.

<P>
<DT><A NAME="current_signal/3"><STRONG>current_signal</STRONG>(<VAR>?Name, 
?Id, ?Handler</VAR>)</A><DD>
Enumerate the currently defined signal handling. <VAR>Name</VAR> is the 
signal name, <VAR>Id</VAR> is the numerical identifier and <VAR>Handler</VAR> 
is the currently defined handler (see <A NAME="idx:on_signal/3:273"></A><A HREF="sec-3.9.html#on_signal/3">on_signal/3</A>).
</DL>

<P>

<H3><A NAME="sec:3.9.1">3.9.1 Notes on signal handling</A></H3>

<P>Before deciding to deal with signals in your application, please 
consider the following:

<P><UL>
<LI><I>Portibility</I><BR>
On MS-Windows, the signal interface is severely limited. Different Unix 
brands support different sets of signals, and the relation between 
signal name and number may vary.
<LI><I>Safety</I><BR>
Signal handling is not completely safe in the current implementation, 
especially if <CODE>throw</CODE> is used in combination with external 
foreign code. The system will use the C longjmp() construct to direct 
control to the innermost <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>, 
thus forcing an external procedure to be abandoned at an arbitrary 
moment. Most likely not all SWI-Prologs own foreign code is (yet) safe 
too.
<LI><I>Garbage Collection</I><BR>
The garbage collector will block all signals that are handled by Prolog. 
While handling a signal, the garbage-collector is disabled.
<LI><I>Time of delivery</I><BR>
Normally delivery is immediate (or as defined by the operating system 
used). Signals are blocked with the garbage collector is active, and 
internally delayed if they occur will in a `critical section'. The 
critical sections are generally very short.
</UL>

