<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 4.2</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-4.1.html">
<LINK REL=next HREF="sec-4.3.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-4.1.html">Previous</A>
<A HREF="sec-4.3.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:4.2">4.2 Name-based versus Predicate-based Modules</A></H2>

<P>Two approaches to realize a module system are commonly used in Prolog 
and other languages. The first one is the <EM>name based</EM> module 
system. In these systems, each atom read is tagged (normally prefixed) 
with the module name, with the exception of those atoms that are defined
<EM>public</EM>. In the second approach, each module actually implements 
its own predicate space.

<P>A critical problem with using modules in Prolog is introduced by the 
meta-predicates that transform between Prolog data and Prolog 
predicates. Consider the case where we write:

<P>

<XMP>
:- module(extend, [add_extension/3]).

add_extension(Extension, Plain, Extended) :-
        maplist(extend_atom(Extension), Plain, Extended).

extend_atom(Extension, Plain, Extended) :-
        concat(Plain, Extension, Extended).
</XMP>

<P>In this case we would like maplist to call extend_atom/3 in the 
module
<CODE>extend</CODE>. A name based module system will do this correctly. 
It will tag the atom <CODE>extend_atom</CODE> with the module and 
maplist will use this to construct the tagged term extend_atom/3. A name 
based module however, will not only tag the atoms that will eventually 
be used to refer to a predicate, but <STRONG>all</STRONG> atoms that are 
not declared public. So, with a name based module system also data is 
local to the module. This introduces another serious problem:

<P>

<XMP>
:- module(action, [action/3]).

action(Object, sleep, Arg) :- ....
action(Object, awake, Arg) :- ....

:- module(process, [awake_process/2]).

awake_process(Process, Arg) :-
        action(Process, awake, Arg).
</XMP>

<P>This code uses a simple object-oriented implementation technique were 
atoms are used as method selectors. Using a name based module system, 
this code will not work, unless we declare the selectors public atoms in 
all modules that use them. Predicate based module systems do not require 
particular precautions for handling this case.

<P>It appears we have to choose either to have local data, or to have 
trouble with meta-predicates. Probably it is best to choose for the 
predicate based approach as novice users will not often write generic 
meta-predicates that have to be used across multiple modules, but are 
likely to write programs that pass data around across modules. 
Experienced Prolog programmers should be able to deal with the 
complexities of meta-predicates in a predicate based module system.

