<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 4.5</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-4.4.html">
<LINK REL=next HREF="sec-4.6.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-4.4.html">Previous</A>
<A HREF="sec-4.6.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:4.5">4.5 Using the Module System</A></H2>

<P>The current structure of the module system has been designed with 
some specific organisations for large programs in mind. Many large 
programs define a basic library layer on top of which the actual program 
itself is defined. The module <CODE>user</CODE>, acting as the default 
module for all other modules of the program can be used to distribute 
these definitions over all program module without introducing the need 
to import this common layer each time explicitly. It can also be used to 
redefine built-in predicates if this is required to maintain 
compatibility to some other Prolog implementation. Typically, the 
loadfile of a large application looks like this:

<P>

<XMP>
:- use_module(compatibility).   % load XYZ prolog compatibility

:- use_module(                  % load generic parts
        [ error                 % errors and warnings
        , goodies               % general goodies (library extensions)
        , debug                 % application specific debugging
        , virtual_machine       % virtual machine of application
        , ...                   % more generic stuff
        ]).

:- ensure_loaded(
        [ ...                   % the application itself
        ]).
</XMP>

<P>The `use_module' declarations will import the public predicates from 
the generic modules into the <CODE>user</CODE> module. The 
`ensure_loaded' directive loads the modules that constitute the actual 
application. It is assumed these modules import predicates from each 
other using
<A NAME="idx:use_module/[1,2]:556"></A><A HREF="sec-4.4.html#use_module/1">use_module/[1,2]</A> 
as far as necessary.

<P>In combination with the object-oriented schema described below it is 
possible to define a neat modular architecture. The generic code defines 
general utilities and the message passing predicates (invoke/3 in the 
example below). The application modules define classes that communicate 
using the message passing predicates.

<P>

<H3><A NAME="sec:4.5.1">4.5.1 Object Oriented Programming</A></H3>

<P>Another typical way to use the module system is for defining classes 
within an object oriented paradigm. The class structure and the methods 
of a class can be defined in a module and the explicit module-boundary 
overruling describes in <A HREF="sec-4.6.html">section 4.6.2</A> can by 
used by the message passing code to invoke the behaviour. An outline of 
this mechanism is given below.

<P>

<XMP>
%       Define class point

:- module(point, []).           % class point, no exports

%        name           type,           default access
%                                       value

variable(x,             integer,        0,      both).
variable(y,             integer,        0,      both).

%         method name   predicate name  arguments

behaviour(mirror,       mirror,         []).

mirror(P) :-
        fetch(P, x, X),
        fetch(P, y, Y),
        store(P, y, X),
        store(P, x, Y).
</XMP>

<P>The predicates fetch/3 and store/3 are predicates that change 
instance variables of instances. The figure below indicates how message 
passing can easily be implemented:

<P>

<XMP>
%       invoke(+Instance, +Selector, ?ArgumentList)
%       send a message to an instance

invoke(I, S, Args) :-
        class_of_instance(I, Class),
        Class:behaviour(S, P, ArgCheck), !,
        convert_arguments(ArgCheck, Args, ConvArgs),
        Goal =.. [P|ConvArgs],
        Class:Goal.

</XMP>

<P>The construct &lt;<VAR>Module</VAR>&gt;:&lt;<VAR>Goal</VAR>&gt; 
explicitly calls <VAR>Goal</VAR> in module <VAR>Module</VAR>. It is 
discussed in more detail in
<A HREF="sec-3.7.html">section 3.7</A>.

