<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 4.6</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-4.5.html">
<LINK REL=next HREF="sec-4.7.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-4.5.html">Previous</A>
<A HREF="sec-4.7.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:4.6">4.6 Meta-Predicates in Modules</A></H2>

<P>As indicated in the introduction, the problem with a predicate based 
module system lies in the difficulty to find the correct predicate from 
a Prolog term. The predicate `solution(Solution)' can exist in more than 
one module, but `assert(solution(4))' in some module is supposed to 
refer to the correct version of solution/1.

<P>Various approaches are possible to solve this problem. One is to add 
an extra argument to all predicates (e.g. `assert(Module, Term)'). 
Another is to tag the term somehow to indicate which module is desired 
(e.g. `assert(Module:Term)'). Both approaches are not very attractive as 
they make the user responsible for choosing the correct module, inviting 
unclear programming by asserting in other modules. The predicate <A NAME="idx:assert/1:557"></A><A HREF="sec-3.12.html#assert/1">assert/1</A> 
is supposed to assert in the module it is called from and should do so 
without being told explicitly. For this reason, the notion <EM>context 
module</EM> has been introduced.

<P>

<H3><A NAME="sec:4.6.1">4.6.1 Definition and Context Module</A></H3>

<P>Each predicate of the program is assigned a module, called it's
<EM>definition module</EM>. The definition module of a predicate is 
always the module in which the predicate was originally defined. Each 
active goal in the Prolog system has a <EM>context module</EM> assigned 
to it.

<P>The context module is used to find predicates from a Prolog term. By 
default, this module is the definition module of the predicate running 
the goal. For meta-predicates however, this is the context module of the 
goal that invoked them. We call this <EM>module_transparent</EM> in 
SWI-Prolog. In the `using maplist' example above, the predicate
<A NAME="idx:maplist/3:558"></A><A HREF="sec-3.30.html#maplist/3">maplist/3</A> 
is declared module_transparent. This implies the context module remains <CODE>extend</CODE>, 
the context module of add_extension/3. This way <A NAME="idx:maplist/3:559"></A><A HREF="sec-3.30.html#maplist/3">maplist/3</A> 
can decide to call extend_atom in module
<CODE>extend</CODE> rather than in it's own definition module.

<P>All built-in predicates that refer to predicates via a Prolog term 
are declared module_transparent. Below is the code defining maplist.

<P>

<XMP>
:- module(maplist, maplist/3).

:- module_transparent maplist/3.

%       maplist(+Goal, +List1, ?List2)
%       True if Goal can successfully be applied to all successive pairs
%       of elements of List1 and List2.

maplist(_, [], []).
maplist(Goal, [Elem1|Tail1], [Elem2|Tail2]) :-
        apply(Goal, [Elem1, Elem2]), 
        maplist(Goal, Tail1, Tail2).
</XMP>

<P>

<H3><A NAME="sec:4.6.2">4.6.2 Overruling Module Boundaries</A></H3>

<A NAME="sec:overrule"></A>

<P>The mechanism above is sufficient to create an acceptable module 
system. There are however cases in which we would like to be able to 
overrule this schema and explicitly call a predicate in some module or 
assert explicitly in some module. The first is useful to invoke goals in 
some module from the user's toplevel or to implement a object-oriented 
system (see above). The latter is useful to create and modify <EM>dynamic 
modules</EM> (see <A HREF="sec-4.7.html">section 4.7</A>).

<P>For this purpose, the reserved term :/2 has been introduced. All 
built-in predicates that transform a term into a predicate reference 
will check whether this term is of the form `&lt;<VAR>Module</VAR>&gt;:&lt;<VAR>Term</VAR>&gt;' 
. If so, the predicate is searched for in <VAR>Module</VAR> instead of 
the goal's context module. The
<STRONG><CODE>:</CODE></STRONG> operator may be nested, in which case 
the inner-most module is used.

<P>The special calling construct &lt;<VAR>Module</VAR>&gt;:&lt;<VAR>Goal</VAR>&gt; 
pretends
<VAR>Goal</VAR> is called from <VAR>Module</VAR> instead of the context 
module. Examples:

<P>

<XMP>
?- assert(world:done).  % asserts done/0 into module world
?- world:assert(done).  % the same
?- world:done.          % calls done/0 in module world
</XMP>

