<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 4.9</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-4.8.html">
<LINK REL=next HREF="sec-5.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-4.8.html">Previous</A>
<A HREF="sec-5.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:4.9">4.9 Compatibility of the Module System</A></H2>

<P>The principles behind the module system of SWI-Prolog differ in a 
number of aspects from the Quintus Prolog module system.

<P><UL>
<LI>The SWI-Prolog module system allows the user to redefine system 
predicates.
<LI>All predicates that are available in the <CODE>system</CODE> and <CODE>user</CODE> 
modules are visible in all other modules as well.
<LI>Quintus has the `<A NAME="idx:meta_predicate/1:571"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A>' 
declaration were SWI-Prolog has the
<A NAME="idx:module_transparent/1:572"></A><A HREF="sec-4.8.html#module_transparent/1">module_transparent/1</A> 
declaration.
</UL>

<P>The <A NAME="idx:meta_predicate/1:573"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A> 
declaration causes the compiler to tag arguments that pass module 
sensitive information with the module using the :/2 operator. This 
approach has some disadvantages:

<P><UL>
<LI>Changing a meta_predicate declaration implies all predicates
<STRONG>calling</STRONG> the predicate need to be reloaded. This can 
cause serious consistency problems.
<LI>It does not help for dynamically defined predicates calling module 
sensitive predicates.
<LI>It slows down the compiler (at least in the SWI-Prolog 
architecture).
<LI>At least within the SWI-Prolog architecture the run-time overhead is 
larger than the overhead introduced by the transparent mechanism.
</UL>

<P>Unfortunately the transparent predicate approach also has some 
disadvantages. If a predicate <VAR>A</VAR> passes module sensitive 
information to a predicate <VAR>B</VAR>, passing the same information to 
a module sensitive system predicate both <VAR>A</VAR> and <VAR>B</VAR> 
should be declared transparent. Using the Quintus approach only <VAR>A</VAR> 
needs to be treated special (i.e. declared with <A NAME="idx:meta_predicate/1:574"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A>)<A NAME=back-to-note-23 HREF="Notes.html#note-23"> (23)</A>. 
A second problem arises if the body of a transparent predicate uses 
module sensitive predicates for which it wants to refer to its own 
module. Suppose we want to define <A NAME="idx:findall/3:575"></A><A HREF="sec-3.29.html#findall/3">findall/3</A> 
using <A NAME="idx:assert/1:576"></A><A HREF="sec-3.12.html#assert/1">assert/1</A> 
and
<A NAME="idx:retract/1:577"></A><A HREF="sec-3.12.html#retract/1">retract/1</A><A NAME=back-to-note-24 HREF="Notes.html#note-24"> (24)</A>. 
The example in <A HREF="sec-4.9.html#fig:findall">figure 4</A> gives the 
solution.

<P>

<XMP>
:- module(findall, [findall/3]).

:- dynamic
        solution/1.

:- module_transparent
        findall/3, 
        store/2.

findall(Var, Goal, Bag) :-
        assert(findall:solution('$mark')), 
        store(Var, Goal), 
        collect(Bag).

store(Var, Goal) :-
        Goal,                   % refers to context module of
                                % caller of findall/3
        assert(findall:solution(Var)), 
        fail.
store(_, _).

collect(Bag) :-
        ..., 
</XMP>

<P><CENTER><B>Figure 4 : <A NAME="idx:findall/3:578"></A><A HREF="sec-3.29.html#findall/3">findall/3</A> 
using modules</B></CENTER>
<A NAME="fig:findall"></A>

<P>

<H3><A NAME="sec:4.9.1">4.9.1 Emulating <A NAME="idx:meta_predicate/1:579"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A></A></H3>

<A NAME="sec:metapredicate"></A>

<P>The Quintus <A NAME="idx:meta_predicate/1:580"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A> 
directive can in many cases be replaced by the transparent declaration. 
Below is the definition of <A NAME="idx:meta_predicate/1:581"></A><A HREF="sec-4.8.html#meta_predicate/1">meta_predicate/1</A> 
as available from <CODE>library(quintus)</CODE>.

<P>

<XMP>
:- op(1150, fx, (meta_predicate)).

meta_predicate((Head, More)) :- !, 
        meta_predicate1(Head), 
        meta_predicate(More).
meta_predicate(Head) :-
        meta_predicate1(Head).

meta_predicate1(Head) :-
        Head =.. [Name|Arguments], 
        member(Arg, Arguments), 
        module_expansion_argument(Arg), !, 
        functor(Head, Name, Arity), 
        module_transparent(Name/Arity).
meta_predicate1(_).             % just a mode declaration

module_expansion_argument(:).
module_expansion_argument(N) :- integer(N).
</XMP>

<P>The discussion above about the problems with the transparent 
mechanism show the two cases in which this simple transformation does 
not work.
