<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 5.5</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-5.4.html">
<LINK REL=next HREF="sec-5.6.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-5.4.html">Previous</A>
<A HREF="sec-5.6.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:5.5">5.5 Interface Data types</A></H2>

<P>

<H3><A NAME="sec:5.5.1">5.5.1 Type <CODE>term_t</CODE>: a reference to a 
Prolog term</A></H3>

<P>The principal data-type is <CODE>term_t</CODE>. Type <CODE>term_t</CODE> 
is what Quintus calls <CODE>QP_term_ref</CODE>. This name indicates 
better what the type represents: it is a <EM>handle</EM> for a term 
rather than the term itself. Terms can only be represented and 
manipulated using this type, as this is the only safe way to ensure the 
Prolog kernel is aware of all terms referenced by foreign code and thus 
allows the kernel to perform garbage-collection and/or stack-shifts 
while foreign code is active, for example during a callback from C.

<P>A term reference is a C unsigned long, representing the offset of a 
variable on the Prolog environment-stack. A foreign function is passed 
term references for the predicate-arguments, one for each argument. If 
references for intermediate results are needed, such references may be 
created using <A HREF="sec-5.5.html#PL_new_term_ref()">PL_new_term_ref()</A> 
or <A HREF="sec-5.5.html#PL_new_term_refs()">PL_new_term_refs()</A>. 
These references normally live till the foreign function returns control 
back to Prolog. Their scope can be explicitly limited using <A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A> 
and <A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A>/<A HREF="sec-5.6.html#PL_discard_foreign_frame()">PL_discard_foreign_frame()</A>.

<P>A term_t always refers to a valid Prolog term (variable, atom, 
integer, float or compound term). A term lives either until backtracking 
takes us back to a point before the term was created, the garbage 
collector has collected the term or the term was created after a
<A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A> 
and <A HREF="sec-5.6.html#PL_discard_foreign_frame()">PL_discard_foreign_frame()</A> 
has been called.

<P>The foreign-interface functions can either </EM><EM>read</EM>, </EM><EM>unify</EM> 
or
</EM><EM>write</EM> to term-references. In the this document we use the 
following notation for arguments of type term_t:

<P><BLOCKQUOTE>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD></EM><TT>term_t +t</TT><TD>Accessed in read-mode. The 
`+' indicates the argument is `input'. </TR>
<TR VALIGN=top><TD></EM><TT>term_t -t</TT><TD>Accessed in write-mode. </TR>
<TR VALIGN=top><TD></EM><TT>term_t ?t</TT><TD>Accessed in unify-mode. </TR>
</TABLE>

</BLOCKQUOTE>

<P>Term references are obtained in any of the following ways.

<P><UL>
<LI><I>Passed as argument</I><BR>
The C-functions implementing foreign predicates are passed their 
arguments as term-references. These references may be read or unified. 
Writing to these variables causes undefined behaviour.
<LI><I>Created by <A HREF="sec-5.5.html#PL_new_term_ref()">PL_new_term_ref()</A></I><BR>
A term created by <A HREF="sec-5.5.html#PL_new_term_ref()">PL_new_term_ref()</A> 
is normally used to build temporary terms or be written by one of the 
interface functions. For example, <A HREF="sec-5.6.html#PL_get_arg()">PL_get_arg()</A> 
writes a reference to the term-argument in its last argument.
<LI><I>Created by <A HREF="sec-5.5.html#PL_new_term_refs()">PL_new_term_refs(int 
n)</A></I><BR>
This function returns a set of term refs with the same characteristics 
as <A HREF="sec-5.5.html#PL_new_term_ref()">PL_new_term_ref()</A>. See <A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A>.
<LI><I>Created by <A HREF="sec-5.5.html#PL_copy_term_ref()">PL_copy_term_ref(term_t 
t)</A></I><BR>
Creates a new term-reference to the same term as the argument. The term 
may be written to. See <A HREF="sec-5.6.html#fig:pl-display">figure 7</A>.
</UL>

<P>Term-references can safely be copied to other C-variables of type 
term_t, but all copies will always refer to the same term.

<P>

<DL>

<P>
<DT><A NAME="PL_new_term_ref()"><VAR>term_t</VAR> <STRONG>PL_new_term_ref</STRONG>(<VAR></VAR>)</A><DD>
Return a fresh reference to a term. The reference is allocated on the
<EM>local</EM> stack. Allocating a term-reference may trigger a 
stack-shift on machines that cannot use sparse-memory management for 
allocation the Prolog stacks. The returned reference describes a 
variable.

<P>
<DT><A NAME="PL_new_term_refs()"><VAR>term_t</VAR> <STRONG>PL_new_term_refs</STRONG>(<VAR>int 
n</VAR>)</A><DD>
Return <VAR>n</VAR> new term references. The first term-reference is 
returned. The others are <VAR><VAR>t</VAR>+1</VAR>, <VAR><VAR>t</VAR>+2</VAR>, 
etc. There are two reasons for using this function. <A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A> 
expects the arguments as a set of consecutive term references and </EM><EM>very</EM> 
time-critical code requiring a number of term-references can be written 
as:

<P>

<XMP>
pl_mypredicate(term_t a0, term_t a1)
{ term_t t0 = PL_new_term_refs(2);
  term_t t1 = t0+1;

  ...
}
</XMP>

<P>

<P>
<DT><A NAME="PL_copy_term_ref()"><VAR>term_t</VAR> <STRONG>PL_copy_term_ref</STRONG>(<VAR>term_t 
from</VAR>)</A><DD>
Create a new term reference and make it point initially to the same term 
as <VAR>from</VAR>. This function is commonly used to copy a predicate 
argument to a term reference that may be written.

<P>
<DT><A NAME="PL_reset_term_refs()"><VAR>void</VAR> <STRONG>PL_reset_term_refs</STRONG>(<VAR>term_t 
after</VAR>)</A><DD>
Destroy all term references that have been created after <VAR>after</VAR>, 
including <VAR>after</VAR> itself. Any reference to the invalidated term 
references after this call results in undefined behaviour.

<P>Note that returning from the foreign context to Prolog will reclaim 
all references used in the foreign context. This call is only necessary 
if references are created inside a loop that never exits back to Prolog. 
See also <A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A>, <A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A> 
and
<A HREF="sec-5.6.html#PL_discard_foreign_frame()">PL_discard_foreign_frame()</A>.
</DL>

<P>

<H4><A NAME="sec:5.5.1.1">5.5.1.1 Interaction with the garbage collector 
and stack-shifter</A></H4>

<P>Prolog implements two mechanisms for avoiding stack overflow: garbage 
collection and stack expansion. On machines that allow for it, Prolog 
will use virtual memory management to detect stack overflow and expand 
the runtime stacks. On other machines Prolog will reallocate the stacks 
and update all pointers to them. To do so, Prolog needs to know which 
data is referenced by C-code. As all Prolog data known by C is 
referenced through term references (<CODE>term_t</CODE>), Prolog has all 
information necessary to perform its memory management without special 
precautions from the C-programmer.

<P>

<H3><A NAME="sec:5.5.2">5.5.2 Other foreign interface types</A></H3>

<P>

<DL>
<DT><B>atom_t</B><DD>
An atom in Prologs internal representation. Atoms are pointers to an 
opaque structure. They are a unique representation for represented text, 
which implies that atom <VAR>A</VAR> represents the same text as atom <VAR>B</VAR> 
if-and-only-if <VAR>A</VAR> and <VAR>B</VAR> are the same pointer.

<P>Atoms are the central representation for textual constants in Prolog 
The transformation of C a character string to an atom implies a 
hash-table lookup. If the same atom is needed often, it is advised to 
store its reference in a global variable to avoid repeated lookup.
<DT><B>functor_t</B><DD>
A functor is the internal representation of a name/arity pair. They are 
used to find the name and arity of a compound term as well as to 
construct new compound terms. Like atoms they live for the whole Prolog 
session and are unique.
<DT><B>predicate_t</B><DD>
Handle to a Prolog predicate. Predicate handles live forever (although 
they can loose their definition).
<DT><B>qid_t</B><DD>
Query Identifier. Used by
<A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A>/<A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>/<A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A> 
to handle backtracking from C.
<DT><B>fid_t</B><DD>
Frame Identifier. Used by
<A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A>/<A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A>.
<DT><B>module_t</B><DD>
A module is a unique handle to a Prolog module. Modules are used only to 
call predicates in a specific module.
<DT><B>foreign_t</B><DD>
Return type for a C-function implementing a Prolog predicate.
<DT><B>control_t</B><DD>
Passed as additional argument to non-deterministic foreign functions. 
See PL_retry*() and PL_foreign_context*().
<DT><B>install_t</B><DD>
Type for the install() and uninstall() functions of shared or dynamic 
link libraries. See secrefshlib.
</DL>

