<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 5.6</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-5.5.html">
<LINK REL=next HREF="sec-5.7.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-5.5.html">Previous</A>
<A HREF="sec-5.7.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:5.6">5.6 The Foreign Include File</A></H2>

<P>

<H3><A NAME="sec:5.6.1">5.6.1 Argument Passing and Control</A></H3>

<P>If Prolog encounters a foreign predicate at run time it will call a 
function specified in the predicate definition of the foreign predicate. 
The arguments <VAR>1, ... , &lt;<VAR>arity</VAR>&gt;</VAR> pass the 
Prolog arguments to the goal as Prolog terms. Foreign functions should 
be declared of type
<CODE>foreign_t</CODE>. Deterministic foreign functions have two 
alternatives to return control back to Prolog:

<P>

<DL>

<P>
<DT><A NAME="PL_succeed()"><VAR>void</VAR> <STRONG>PL_succeed</STRONG>(<VAR></VAR>)</A><DD>
Succeed deterministically. PL_succeed is defined as
<CODE>return <CODE>TRUE</CODE></CODE>.

<P>
<DT><A NAME="PL_fail()"><VAR>void</VAR> <STRONG>PL_fail</STRONG>(<VAR></VAR>)</A><DD>
Fail and start Prolog backtracking. PL_fail is defined as <CODE>return <CODE>FALSE</CODE></CODE>.
</DL>

<P>

<H4><A NAME="sec:5.6.1.1">5.6.1.1 Non-deterministic Foreign Predicates</A></H4>

<A NAME="sec:foreignnondet"></A>

<P>By default foreign predicates are deterministic. Using the
<CODE>PL_FA_NONDETERMINISTIC</CODE> attribute (see <A HREF="sec-5.6.html#PL_register_foreign()">PL_register_foreign()</A>) 
it is possible to register a predicate as a non-deterministic predicate. 
Writing non-deterministic foreign predicates is slightly more 
complicated as the foreign function needs context information for 
generating the next solution. Note that the same foreign function should 
be prepared to be simultaneously active in more than one goal. Suppose 
the natural_number_below_n/2 is a non-deterministic foreign predicate, 
backtracking over all natural numbers lower than the first argument. Now 
consider the following predicate:

<P>

<XMP>
quotient_below_n(Q, N) :-
        natural_number_below_n(N, N1),
        natural_number_below_n(N, N2),
        Q =:= N1 / N2, !.
</XMP>

<P>In this predicate the function natural_number_below_n/2 
simultaneously generates solutions for both its invocations.

<P>Non-deterministic foreign functions should be prepared to handle 
three different calls from Prolog:

<P><UL>
<LI><I>Initial call (<CODE>PL_FIRST_CALL</CODE>)</I><BR>
Prolog has just created a frame for the foreign function and asks it to 
produce the first answer.
<LI><I>Redo call (<CODE>PL_REDO</CODE>)</I><BR>
The previous invocation of the foreign function associated with the 
current goal indicated it was possible to backtrack. The foreign 
function should produce the next solution.
<LI><I>Terminate call (<CODE>PL_CUTTED</CODE>)</I><BR>
The choice point left by the foreign function has been destroyed by a 
cut. The foreign function is given the opportunity to clean the 
environment.
</UL>

<P>Both the context information and the type of call is provided by an 
argument of type <CODE>control_t</CODE> appended to the argument list 
for deterministic foreign functions. The macro <A HREF="sec-5.6.html#PL_foreign_control()">PL_foreign_control()</A> 
extracts the type of call from the control argument. The foreign 
function can pass a context handle using the </EM><TT>PL_retry*()</TT> 
macros and extract the handle from the extra argument using the
</EM><TT>PL_foreign_context*()</TT> macro.

<P>

<DL>

<P>
<DT><A NAME="PL_retry()"><VAR>void</VAR> <STRONG>PL_retry</STRONG>(<VAR>long</VAR>)</A><DD>
The foreign function succeeds while leaving a choice point. On 
backtracking over this goal the foreign function will be called again, 
but the control argument now indicates it is a `Redo' call and the macro <A HREF="sec-5.6.html#PL_foreign_context()">PL_foreign_context()</A> 
will return the handle passed via
<A HREF="sec-5.6.html#PL_retry()">PL_retry()</A>. This handle is a 30 
bits signed value (two bits are used for status indication).

<P>

<P>
<DT><A NAME="PL_retry_address()"><VAR>void</VAR> <STRONG>PL_retry_address</STRONG>(<VAR>void 
*</VAR>)</A><DD>
As <A HREF="sec-5.6.html#PL_retry()">PL_retry()</A>, but ensures an 
address as returned by malloc() is correctly recovered by <A HREF="sec-5.6.html#PL_foreign_context_address()">PL_foreign_context_address()</A>.

<P>

<P>
<DT><A NAME="PL_foreign_control()"><VAR>int</VAR> <STRONG>PL_foreign_control</STRONG>(<VAR>control_t</VAR>)</A><DD>
Extracts the type of call from the control argument. The return values 
are described above. Note that the function should be prepared to handle 
the <CODE>PL_CUTTED</CODE> case and should be aware that the other 
arguments are not valid in this case.

<P>

<P>
<DT><A NAME="PL_foreign_context()"><VAR>long</VAR> <STRONG>PL_foreign_context</STRONG>(<VAR>control_t</VAR>)</A><DD>
Extracts the context from the context argument. In the call type is
<CODE>PL_FIRST_CALL</CODE> the context value is 0L. Otherwise it is the 
value returned by the last <A HREF="sec-5.6.html#PL_retry()">PL_retry()</A> 
associated with this goal (both if the call type is <CODE>PL_REDO</CODE> 
as <CODE>PL_CUTTED</CODE>).

<P>

<P>
<DT><A NAME="PL_foreign_context_address()"><VAR>void *</VAR> <STRONG>PL_foreign_context_address</STRONG>(<VAR>control_t</VAR>)</A><DD>
Extracts an address as passed in by <A HREF="sec-5.6.html#PL_retry_address()">PL_retry_address()</A>.
</DL>

<P>Note: If a non-deterministic foreign function returns using 
PL_succeed or PL_fail, Prolog assumes the foreign function has cleaned 
its environment. </EM><B>No</B> call with control argument <CODE>PL_CUTTED</CODE> 
will follow.

<P>The code of <A HREF="sec-5.6.html#fig:nondetermf">figure 6</A> shows 
a skeleton for a non-deterministic foreign predicate definition.

<P>

<XMP>
typedef struct                  /* define a context structure */
{ ...
} context;

foreign_t
my_function(term_t a0, term_t a1, foreign_t handle)
{ struct context * ctxt;

  switch( PL_foreign_control(handle) )
  { case PL_FIRST_CALL:
        ctxt = malloc(sizeof(struct context));
        ...
        PL_retry_address(ctxt);
    case PL_REDO:
        ctxt = PL_foreign_context_address(handle);
        ...
        PL_retry_address(ctxt);
    case PL_CUTTED:
        free(ctxt);
        PL_succeed;
  }
}
</XMP>

<P><CENTER><B>Figure 6 : Skeleton for non-deterministic foreign 
functions</B></CENTER>
<A NAME="fig:nondetermf"></A>

<P>

<H3><A NAME="sec:5.6.2">5.6.2 Atoms and functors</A></H3>

<P>The following functions provide for communication using atoms and 
functors.

<P>

<DL>

<P>
<DT><A NAME="PL_new_atom()"><VAR>atom_t</VAR> <STRONG>PL_new_atom</STRONG>(<VAR>const 
char *</VAR>)</A><DD>
Return an atom handle for the given C-string. This function always 
succeeds. The returned handle is valid for the entire session.

<P>
<DT><A NAME="PL_atom_chars()"><VAR>const char *</VAR> <STRONG>PL_atom_chars</STRONG>(<VAR>atom_t 
atom</VAR>)</A><DD>
Return a C-string for the text represented by the given atom. The 
returned text will not be changed by Prolog. It is not allowed to modify 
the contents, not even `temporary' as the string may reside in read-only 
memory.

<P>
<DT><A NAME="PL_new_functor()"><VAR>functor_t</VAR> <STRONG>PL_new_functor</STRONG>(<VAR>atom_t 
name, int arity</VAR>)</A><DD>
Returns a </EM><EM>functor identifier</EM>, a handle for the name/arity 
pair. The returned handle is valid for the entire Prolog session.

<P>
<DT><A NAME="PL_functor_name()"><VAR>atom_t</VAR> <STRONG>PL_functor_name</STRONG>(<VAR>functor_t 
f</VAR>)</A><DD>
Return an atom representing the name of the given functor.

<P>
<DT><A NAME="PL_functor_arity()"><VAR>int</VAR> <STRONG>PL_functor_arity</STRONG>(<VAR>functor_t 
f</VAR>)</A><DD>
Return the arity of the given functor.
</DL>

<P>

<H3><A NAME="sec:5.6.3">5.6.3 Analysing Terms via the Foreign Interface</A></H3>

<P>Each argument of a foreign function (except for the control argument) 
is of type <CODE>term_t</CODE>, an opaque handle to a Prolog term. Three 
groups of functions are available for the analysis of terms. The first 
just validates the type, like the Prolog predicates <A NAME="idx:var/1:599"></A><A HREF="sec-3.4.html#var/1">var/1</A>, <A NAME="idx:atom/1:600"></A><A HREF="sec-3.4.html#atom/1">atom/1</A>, 
etc and are called </EM><TT>PL_is_*()</TT>. The second group attempts to 
translate the argument into a C primitive type. These predicates take a <CODE>term_t</CODE> 
and a pointer to the appropriate C-type and return <CODE>TRUE</CODE> or
<CODE>FALSE</CODE> depending on successful or unsuccessful translation. 
If the translation fails, the pointed-to data is never modified.

<P>

<H4><A NAME="sec:5.6.3.1">5.6.3.1 Testing the type of a term</A></H4>

<P>

<DL>

<P>
<DT><A NAME="PL_term_type()"><VAR>int</VAR> <STRONG>PL_term_type</STRONG>(<VAR>term_t</VAR>)</A><DD>
Obtain the type of a term, which should be a term returned by one of the 
other interface predicates or passed as an argument. The function 
returns the type of the Prolog term. The type identifiers are listed 
below. Note that the extraction functions </EM><TT>PL_ge_t*()</TT> also 
validate the type and thus the two sections below are equivalent.

<P>

<XMP>
        if ( PL_is_atom(t) )
        { char *s;

          PL_get_atom_chars(t, &s);
          ...;
        }

or

        char *s;
        if ( PL_get_atom_chars(t, &s) )
        { ...;
        }
</XMP>

<P>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>PL_VARIABLE</CODE> <TD>An unbound variable. The 
value of term as such is a unique identifier for the variable. </TR>
<TR VALIGN=top><TD><CODE>PL_ATOM</CODE> <TD>A Prolog atom. </TR>
<TR VALIGN=top><TD><CODE>PL_STRING</CODE> <TD>A Prolog string. </TR>
<TR VALIGN=top><TD><CODE>PL_INTEGER</CODE> <TD>A Prolog integer. </TR>
<TR VALIGN=top><TD><CODE>PL_FLOAT</CODE> <TD>A Prolog floating point 
number. </TR>
<TR VALIGN=top><TD><CODE>PL_TERM</CODE> <TD>A compound term. Note that a 
list is a compound term ./2 . </TR>
</TABLE>

</DL>

<P>The functions PL_is_&lt;<VAR>type</VAR>&gt; are an alternative to <A HREF="sec-5.6.html#PL_term_type()">PL_term_type()</A>. 
The test <CODE><A HREF="sec-5.6.html#PL_is_variable()">PL_is_variable(term)</A></CODE> 
is equivalent to
<CODE><A HREF="sec-5.6.html#PL_term_type()">PL_term_type(term)</A> == 
PL_VARIABLE</CODE>, but the first is considerably faster. On the other 
hand, using a switch over <A HREF="sec-5.6.html#PL_term_type()">PL_term_type()</A> 
is faster and more readable then using an if-then-else using the 
functions below. All these functions return either <CODE>TRUE</CODE> or <CODE>FALSE</CODE>.

<P>

<DL>

<P>
<DT><A NAME="PL_is_variable()"><VAR>int</VAR> <STRONG>PL_is_variable</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is a variable.

<P>
<DT><A NAME="PL_is_atom()"><VAR>int</VAR> <STRONG>PL_is_atom</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is an atom.

<P>
<DT><A NAME="PL_is_string()"><VAR>int</VAR> <STRONG>PL_is_string</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is a string.

<P>
<DT><A NAME="PL_is_integer()"><VAR>int</VAR> <STRONG>PL_is_integer</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is an integer.

<P>
<DT><A NAME="PL_is_float()"><VAR>int</VAR> <STRONG>PL_is_float</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is a float.

<P>
<DT><A NAME="PL_is_compound()"><VAR>int</VAR> <STRONG>PL_is_compound</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is a compound term.

<P>
<DT><A NAME="PL_is_functor()"><VAR>int</VAR> <STRONG>PL_is_functor</STRONG>(<VAR>term_t, 
functor_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is compound and its functor is <VAR>functor</VAR>. 
This test is equivalent to <A HREF="sec-5.6.html#PL_get_functor()">PL_get_functor()</A>, 
followed by testing the functor, but easier to write and faster.

<P>
<DT><A NAME="PL_is_list()"><VAR>int</VAR> <STRONG>PL_is_list</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is a compound term with functor ./2 
or the atom <CODE></CODE>.

<P>
<DT><A NAME="PL_is_atomic()"><VAR>int</VAR> <STRONG>PL_is_atomic</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is atomic (not variable or 
compound).

<P>
<DT><A NAME="PL_is_number()"><VAR>int</VAR> <STRONG>PL_is_number</STRONG>(<VAR>term_t</VAR>)</A><DD>
Returns non-zero if <VAR>term</VAR> is an integer or float.
</DL>

<P>

<H4><A NAME="sec:5.6.3.2">5.6.3.2 Reading data from a term</A></H4>

<P>The functions </EM><TT>PL_get_*()</TT> read information from a Prolog 
term. Most of them take two arguments. The first is the input term and 
the second is a pointer to the output value or a term-reference.

<P>

<DL>

<P>
<DT><A NAME="PL_get_atom()"><VAR>int</VAR> <STRONG>PL_get_atom</STRONG>(<VAR>term_t 
+t, atom_t *a</VAR>)</A><DD>
If <VAR>t</VAR> is an atom, store the unique atom identifier over <VAR>a</VAR>. 
See also <A HREF="sec-5.6.html#PL_atom_chars()">PL_atom_chars()</A> and <A HREF="sec-5.6.html#PL_new_atom()">PL_new_atom()</A>. 
If there is no need to access the data (characters) of an atom, it is 
advised to manipulate atoms using their handle.

<P>
<DT><A NAME="PL_get_atom_chars()"><VAR>int</VAR> <STRONG>PL_get_atom_chars</STRONG>(<VAR>term_t 
+t, char **s</VAR>)</A><DD>
If <VAR>t</VAR> is an atom, store a pointer to a 0-terminated C-string 
in
<VAR>s</VAR>. It is explicitly <STRONG>not</STRONG> allowed to modify 
the contents of this string. Some built-in atoms may have the string 
allocated in read-only memory, so `temporary manipulation' can cause an 
error.

<P>
<DT><A NAME="PL_get_string()"><VAR>int</VAR> <STRONG>PL_get_string</STRONG>(<VAR>term_t 
+t, char **s, int *len</VAR>)</A><DD>
If <VAR>t</VAR> is a string object, store a pointer to a 0-terminated 
C-string in <VAR>s</VAR> and the length of the string in <VAR>len</VAR>. 
Note that this pointer is invalidated by backtracking, 
garbage-collection and stack-shifts, so generally the only save 
operations are to pass it immediately to a C-function that doesn't 
involve Prolog.

<P>
<DT><A NAME="PL_get_chars()"><VAR>int</VAR> <STRONG>PL_get_chars</STRONG>(<VAR>term_t 
+t, char **s, unsigned flags</VAR>)</A><DD>
Convert the argument term <VAR>t</VAR> to a 0-terminated C-string. </EM><EM>flags</EM> 
is a bitwise disjunction from two groups of constants. The first 
specifies which term-types should converted and the second how the 
argument is stored. Below is a specification of these constants. <CODE>BUF_RING</CODE> 
implies, if the data is not static (as from an atom), the data is copied 
to the next buffer from a ring of four (4) buffers. This is a convenient 
way of converting multiple arguments passed to a foreign predicate to 
C-strings. If BUF_MALLOC is used, the data must be freed using free() 
when not needed any longer.

<P>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>CVT_ATOM</CODE> <TD>Convert if term is an atom </TR>
<TR VALIGN=top><TD><CODE>CVT_STRING</CODE> <TD>Convert if term is a 
string </TR>
<TR VALIGN=top><TD><CODE>CVT_LIST</CODE> <TD>Convert if term is a list 
of integers between 1 and 255 </TR>
<TR VALIGN=top><TD><CODE>CVT_INTEGER</CODE> <TD>Convert if term is an 
integer (using <CODE>%d</CODE>) </TR>
<TR VALIGN=top><TD><CODE>CVT_FLOAT</CODE> <TD>Convert if term is a float 
(using <CODE>%f</CODE>) </TR>
<TR VALIGN=top><TD><CODE>CVT_NUMBER</CODE> <TD>Convert if term is a 
integer or float </TR>
<TR VALIGN=top><TD><CODE>CVT_ATOMIC</CODE> <TD>Convert if term is atomic </TR>
<TR VALIGN=top><TD><CODE>CVT_VARIABLE</CODE><TD>Convert variable to 
print-name </TR>
<TR VALIGN=top><TD><CODE>CVT_ALL</CODE> <TD>Convert if term is any of 
the above, except for variables </TR>
<TBODY>
<TR VALIGN=top><TD><CODE>BUF_DISCARDABLE</CODE> <TD>Data must copied 
immediately </TR>
<TR VALIGN=top><TD><CODE>BUF_RING</CODE> <TD>Data is stored in a ring of 
buffers </TR>
<TR VALIGN=top><TD><CODE>BUF_MALLOC</CODE> <TD>Data is copied to a new 
buffer returned by
<STRONG>malloc</STRONG>(3) </TR>
</TABLE>

<P>
<DT><A NAME="PL_get_list_chars()"><VAR>int</VAR> <STRONG>PL_get_list_chars</STRONG>(<VAR>+term_t 
l, char **s, unsigned flags</VAR>)</A><DD>
Same as <CODE><A HREF="sec-5.6.html#PL_get_chars()">PL_get_chars(<VAR>l</VAR>, <VAR>s</VAR>, 
CVT_LIST|<VAR>flags</VAR>)</A></CODE>, provided <VAR>flags</VAR> 
contains no of the </EM><TT>CVT_*</TT> flags.

<P>
<DT><A NAME="PL_get_integer()"><VAR>int</VAR> <STRONG>PL_get_integer</STRONG>(<VAR>+term_t 
t, int *i</VAR>)</A><DD>
If <VAR>t</VAR> is a Prolog integer, assign its value over <VAR>i</VAR>. 
On 32-bit machines, this is the same as <A HREF="sec-5.6.html#PL_get_long()">PL_get_long()</A>, 
but avoids a warning from the compiler. See also <A HREF="sec-5.6.html#PL_get_long()">PL_get_long()</A>.

<P>
<DT><A NAME="PL_get_long()"><VAR>int</VAR> <STRONG>PL_get_long</STRONG>(<VAR>term_t 
+t, long *i</VAR>)</A><DD>
If <VAR>t</VAR> is a Prolog integer, assign its value over <VAR>i</VAR>. 
Note that Prolog integers have limited value-range. If <VAR>t</VAR> is a 
floating point number that can be represented as a long, this function 
succeeds as well.

<P>
<DT><A NAME="PL_get_pointer()"><VAR>int</VAR> <STRONG>PL_get_pointer</STRONG>(<VAR>term_t 
+t, void **ptr</VAR>)</A><DD>
In the current system, pointers are represented by Prolog integers, but 
need some manipulation to make sure they do not get truncated due to the 
limited Prolog integer range. <A HREF="sec-5.6.html#PL_put_pointer()">PL_put_pointer()</A>/<A HREF="sec-5.6.html#PL_get_pointer()">PL_get_pointer()</A> 
guarantees pointers in the range of malloc() are handled without 
truncating.

<P>
<DT><A NAME="PL_get_float()"><VAR>int</VAR> <STRONG>PL_get_float</STRONG>(<VAR>term_t 
+t, double *f</VAR>)</A><DD>
If <VAR>t</VAR> is a float or integer, its value is assigned over <VAR>f</VAR>.

<P>
<DT><A NAME="PL_get_functor()"><VAR>int</VAR> <STRONG>PL_get_functor</STRONG>(<VAR>term_t 
+t, functor_t *f</VAR>)</A><DD>
If <VAR>t</VAR> is compound or an atom, the Prolog representation of the 
name-arity pair will be assigned over <VAR>f</VAR>. See also
<A HREF="sec-5.6.html#PL_get_name_arity()">PL_get_name_arity()</A> and <A HREF="sec-5.6.html#PL_is_functor()">PL_is_functor()</A>.

<P>
<DT><A NAME="PL_get_name_arity()"><VAR>int</VAR> <STRONG>PL_get_name_arity</STRONG>(<VAR>term_t 
+t, atom_t *name, int *arity</VAR>)</A><DD>
If <VAR>t</VAR> is compound or an atom, the functor-name will be 
assigned over <VAR>name</VAR> and the arity over <VAR>arity</VAR>. See 
also
<A HREF="sec-5.6.html#PL_get_functor()">PL_get_functor()</A> and <A HREF="sec-5.6.html#PL_is_functor()">PL_is_functor()</A>.

<P>
<DT><A NAME="PL_get_module()"><VAR>int</VAR> <STRONG>PL_get_module</STRONG>(<VAR>term_t 
+t, module_t *module</VAR>)</A><DD>
If <VAR>t</VAR> is an atom, the system will lookup or create the 
corresponding module and assign an opaque pointer to it over </EM><EM>module</EM>,.

<P>
<DT><A NAME="PL_get_arg()"><VAR>int</VAR> <STRONG>PL_get_arg</STRONG>(<VAR>int 
index, term_t +t, term_t -a</VAR>)</A><DD>
If <VAR>t</VAR> is compound and index is between 1 and arity 
(including), assign <VAR>a</VAR> with a term-reference to the argument.
</DL>

<P>

<H4><A NAME="sec:5.6.3.3">5.6.3.3 Reading a list</A></H4>

<P>The functions from this section are intended to read a Prolog list 
from C. Suppose we expect a list of atoms, the following code will print 
the atoms, each on a line:

<P>

<XMP>
foreign_t
pl_write_atoms(term_t l)
{ term_t head = PL_new_term_ref();      /* variable for the elements */
  term_t list = PL_copy_term_ref(l);    /* copy as we need to write */

  while( PL_get_list(list, head, list) )
  { char *s;

    if ( PL_get_atom_chars(head, &s) )
      Sprintf("%s\n", s);
    else
      PL_fail;
  }

  return PL_get_nil(list);              /* test end for [] */
}
</XMP>

<P>

<DL>

<P>
<DT><A NAME="PL_get_list()"><VAR>int</VAR> <STRONG>PL_get_list</STRONG>(<VAR>term_t 
+l, term_t -h, term_t -t</VAR>)</A><DD>
If <VAR>l</VAR> is a list and not <CODE></CODE> assign a term-reference 
to the head to <VAR>h</VAR> and to the tail to <VAR>t</VAR>.

<P>
<DT><A NAME="PL_get_head()"><VAR>int</VAR> <STRONG>PL_get_head</STRONG>(<VAR>term_t 
+l, term_t -h</VAR>)</A><DD>
If <VAR>l</VAR> is a list and not <CODE></CODE> assign a term-reference 
to the head to <VAR>h</VAR>.

<P>
<DT><A NAME="PL_get_tail()"><VAR>int</VAR> <STRONG>PL_get_tail</STRONG>(<VAR>term_t 
+l, term_t -t</VAR>)</A><DD>
If <VAR>l</VAR> is a list and not <CODE></CODE> assign a term-reference 
to the tail to <VAR>t</VAR>.

<P>
<DT><A NAME="PL_get_nil()"><VAR>int</VAR> <STRONG>PL_get_nil</STRONG>(<VAR>term_t 
+l</VAR>)</A><DD>
Succeeds if </EM><EM></EM> represents the atom <CODE></CODE>.
</DL>

<P>

<H4><A NAME="sec:5.6.3.4">5.6.3.4 An example: defining <A NAME="idx:write/1:601"></A><A HREF="sec-3.18.html#write/1">write/1</A> 
in C</A></H4>

<P><A HREF="sec-5.6.html#fig:pl-display">Figure 7</A> shows a simplified 
definition of <A NAME="idx:write/1:602"></A><A HREF="sec-3.18.html#write/1">write/1</A> 
to illustrate the described functions. This simplified version does not 
deal with operators. It is called <A NAME="idx:display/1:603"></A><B>display/1</B>, 
because it mimics closely the behaviour of this Edinburgh predicate.

<P>

<XMP>
foreign_t
pl_display(term_t t)
{ functor_t functor;
  int arity, len, n;
  char *s;

  switch( PL_term_type(t) )
  { case PL_VARIABLE:
    case PL_ATOM:
    case PL_INTEGER:
    case PL_FLOAT:
      PL_get_chars(t, &s, CVT_ALL);
      Sprintf("%s", s);
      break;
    case PL_STRING:
      PL_get_string_chars(t, &s, &len);
      Sprintf("\"%s\"", s);
      break;
    case PL_TERM:
    { term_t a = PL_new_term_ref();

      PL_get_name_arity(t, &name, &arity);
      Sprintf("%s(", PL_atom_chars(name));
      for(n=1; n<=arity; n++)
      { PL_get_arg(n, t, a);
        if ( n > 1 )
          Sprintf(", ");
        pl_display(a);
      }
      Sprintf(")");
      break;
    default:
      PL_fail;                          /* should not happen */
  }

  PL_succeed;
}
</XMP>

<P><CENTER><B>Figure 7 : A Foreign definition of <A NAME="idx:display/1:604"></A><B>display/1</B></B></CENTER>
<A NAME="fig:pl-display"></A>

<P>

<H3><A NAME="sec:5.6.4">5.6.4 Constructing Terms</A></H3>

<P>Terms can be constructed using functions from the </EM><TT>PL_put_*()</TT> 
and
</EM><TT>PL_cons_*()</TT> families. This approach builds the term 
`inside-out', starting at the leaves and subsequently creating compound 
terms. Alternatively, terms may be created `top-down', first creating a 
compound holding only variables and subsequently unifying the arguments. 
This section discusses functions for the first approach. This approach 
is generally used for creating arguments for <A HREF="sec-5.6.html#PL_call()">PL_call()</A> 
and PL_open_query.

<P>

<DL>

<P>
<DT><A NAME="PL_put_variable()"><VAR>void</VAR> <STRONG>PL_put_variable</STRONG>(<VAR>term_t 
-t</VAR>)</A><DD>
Put a fresh variable in the term. The new variable lives on the global 
stack. Note that the initial variable lives on the local stack and is 
lost after a write to the term-references. After using this function, 
the variable will continue to live.

<P>
<DT><A NAME="PL_put_atom()"><VAR>void</VAR> <STRONG>PL_put_atom</STRONG>(<VAR>term_t 
-t, atom_t a</VAR>)</A><DD>
Put an atom in the term reference from a handle. See also
<A HREF="sec-5.6.html#PL_new_atom()">PL_new_atom()</A> and <A HREF="sec-5.6.html#PL_atom_chars()">PL_atom_chars()</A>.

<P>
<DT><A NAME="PL_put_atom_chars()"><VAR>void</VAR> <STRONG>PL_put_atom_chars</STRONG>(<VAR>term_t 
-t, const char *chars</VAR>)</A><DD>
Put an atom in the term-reference constructed from the 0-terminated 
string. The string itself will never be references by Prolog after this 
function.

<P>
<DT><A NAME="PL_put_string_chars()"><VAR>void</VAR> <STRONG>PL_put_string_chars</STRONG>(<VAR>term_t 
-t, const char *chars</VAR>)</A><DD>
Put a zero-terminated string in the term-reference. The data will be 
copied. See also <A HREF="sec-5.6.html#PL_put_string_nchars()">PL_put_string_nchars()</A>.

<P>
<DT><A NAME="PL_put_string_nchars()"><VAR>void</VAR> <STRONG>PL_put_string_nchars</STRONG>(<VAR>term_t 
-t, unsigned int len, const char *chars</VAR>)</A><DD>

<P>Put a string, represented by a length/start pointer pair in the 
term-reference. The data will be copied. This interface can deal with 
0-bytes in the string. See also <A HREF="sec-5.6.html">section 5.6.17</A>.

<P>
<DT><A NAME="PL_put_list_chars()"><VAR>void</VAR> <STRONG>PL_put_list_chars</STRONG>(<VAR>term_t 
-t, const char *chars</VAR>)</A><DD>
Put a list of ASCII values in the term-reference.

<P>
<DT><A NAME="PL_put_integer()"><VAR>void</VAR> <STRONG>PL_put_integer</STRONG>(<VAR>term_t 
-t, long i</VAR>)</A><DD>
Put a Prolog integer in the term reference.

<P>
<DT><A NAME="PL_put_pointer()"><VAR>void</VAR> <STRONG>PL_put_pointer</STRONG>(<VAR>term_t 
-t, void *ptr</VAR>)</A><DD>
Put a Prolog integer in the term-reference. Provided ptr is in the 
`malloc()-area', <A HREF="sec-5.6.html#PL_get_pointer()">PL_get_pointer()</A> 
will get the pointer back.

<P>
<DT><A NAME="PL_put_float()"><VAR>void</VAR> <STRONG>PL_put_float</STRONG>(<VAR>term_t 
-t, double f</VAR>)</A><DD>
Put a floating-point value in the term-reference.

<P>
<DT><A NAME="PL_put_functor()"><VAR>void</VAR> <STRONG>PL_put_functor</STRONG>(<VAR>term_t 
-t, functor_t functor</VAR>)</A><DD>
Create a new compound term from <VAR>functor</VAR> and bind <VAR>t</VAR> 
to this term. All arguments of the term will be variables. To create a 
term with instantiated arguments, either instantiate the arguments using 
the </EM><TT>PL_unify_*()</TT> functions or use <A HREF="sec-5.6.html#PL_cons_functor()">PL_cons_functor()</A>.

<P>
<DT><A NAME="PL_put_list()"><VAR>void</VAR> <STRONG>PL_put_list</STRONG>(<VAR>term_t 
-l</VAR>)</A><DD>
Same as <CODE><A HREF="sec-5.6.html#PL_put_functor()">PL_put_functor(l, 
PL_new_functor(PL_new_atom(".")</A>, 2))</CODE>.

<P>
<DT><A NAME="PL_put_nil()"><VAR>void</VAR> <STRONG>PL_put_nil</STRONG>(<VAR>term_t 
-l</VAR>)</A><DD>
Same as <CODE><A HREF="sec-5.6.html#PL_put_atom_chars()">PL_put_atom_chars("[]")</A></CODE>.

<P>
<DT><A NAME="PL_put_term()"><VAR>void</VAR> <STRONG>PL_put_term</STRONG>(<VAR>term_t 
-t1, term_t +t2</VAR>)</A><DD>
Make <VAR>t1</VAR> point to the same term as <VAR>t2</VAR>.

<P>
<DT><A NAME="PL_cons_functor()"><VAR>void</VAR> <STRONG>PL_cons_functor</STRONG>(<VAR>term_t 
-h, functor_t f, ...</VAR>)</A><DD>
Create a term, whose arguments are filled from variable argument list 
holding the same number of term_t objects as the arity of the functor. 
To create the term <CODE>animal(gnu, 50)</CODE>, use:

<P>

<XMP>
        term_t a1 = PL_new_term_ref();
        term_t a2 = PL_new_term_ref();
        term_t t  = PL_new_term_ref();

        PL_put_atom_chars(a1, "gnu");
        PL_put_integer(a2, 50);
        PL_cons_functor(t, PL_new_functor(PL_new_atom("animal"), 2),
                        a1, a2);
</XMP>

<P>After this sequence, the term-references <VAR>a1</VAR> and <VAR>a2</VAR> 
may be used for other purposes.

<P>
<DT><A NAME="PL_cons_functor_v()"><VAR>void</VAR> <STRONG>PL_cons_functor_v</STRONG>(<VAR>term_t 
-h, functor_t f, term_t a0</VAR>)</A><DD>
Creates a compound term like <A HREF="sec-5.6.html#PL_cons_functor()">PL_cons_functor()</A>, 
but <VAR>a0</VAR> is an array of term references as returned by <A HREF="sec-5.5.html#PL_new_term_refs()">PL_new_term_refs()</A>. 
The length of this array should match the number of arguments required 
by the functor.

<P>
<DT><A NAME="PL_cons_list()"><VAR>void</VAR> <STRONG>PL_cons_list</STRONG>(<VAR>term_t 
-l, term_t +h, term_t +t</VAR>)</A><DD>
Create a list (cons-) cell in <VAR>l</VAR> from the head and tail. The 
code below creates a list of atoms from a <CODE>char **</CODE>. The list 
is built tail-to-head. The </EM><TT>PL_unify_*()</TT> functions can be 
used to build a list head-to-tail.

<P>

<XMP>
void
put_list(term_t l, int n, char **words)
{ term_t a = PL_new_term_ref();

  PL_put_nil(l);
  while( --n >= 0 )
  { PL_put_atom_chars(a, words[n]);
    PL_cons_list(l, a, l);
  }
}
</XMP>

<P>Note that <VAR>l</VAR> can be redefined within a </EM><TT>PL_cons_list</TT> 
call as shown here because operationally its old value is consumed 
before its new value is set.
</DL>

<P>

<H3><A NAME="sec:5.6.5">5.6.5 Unifying data</A></H3>

<P>The functions of this sections <EM>unify</EM> terms with other terms 
or translated C-data structures. Except for <A HREF="sec-5.6.html#PL_unify()">PL_unify()</A>, 
the functions of this section are specific to SWI-Prolog. They have been 
introduced to make translation of old code easier, but also because they 
provide for a faster mechanism for returning data to Prolog that 
requires less term-references. Consider the case where we want a foreign 
function to return the host name of the machine Prolog is running on. 
Using the </EM><TT>PL_get_*()</TT> and </EM><TT>PL_put_*()</TT> 
functions, the code becomes:

<P>

<XMP>
foreign_t
pl_hostname(term_t name)
{ char buf[100];
  
  if ( gethostname(buf, sizeof(buf)) )
  { term_t tmp = PL_new_term_ref();

    PL_put_atom_chars(tmp, buf);
    return PL_unify(name, buf);
  }

  PL_fail;
}
</XMP>

<P>Using <A HREF="sec-5.6.html#PL_unify_atom_chars()">PL_unify_atom_chars()</A>, 
this becomes:

<P>

<XMP>
foreign_t
pl_hostname(term_t name)
{ char buf[100];
  
  if ( gethostname(buf, sizeof(buf)) )
    return PL_unify_atom_chars(name, buf);

  PL_fail;
}
</XMP>

<P>

<DL>

<P>
<DT><A NAME="PL_unify()"><VAR>int</VAR> <STRONG>PL_unify</STRONG>(<VAR>term_t 
?t1, term_t ?t2</VAR>)</A><DD>
Unify two Prolog terms and return non-zero on success.

<P>
<DT><A NAME="PL_unify_atom()"><VAR>int</VAR> <STRONG>PL_unify_atom</STRONG>(<VAR>term_t 
?t, atom_t a</VAR>)</A><DD>
Unify <VAR>t</VAR> with the atom <VAR>a</VAR> and return non-zero on 
success.

<P>
<DT><A NAME="PL_unify_atom_chars()"><VAR>int</VAR> <STRONG>PL_unify_atom_chars</STRONG>(<VAR>term_t 
?t, const char *chars</VAR>)</A><DD>
Unify <VAR>t</VAR> with an atom created from <VAR>chars</VAR> and return 
non-zero on success.

<P>
<DT><A NAME="PL_unify_list_chars()"><VAR>int</VAR> <STRONG>PL_unify_list_chars</STRONG>(<VAR>term_t 
?t, const char *chars</VAR>)</A><DD>
Unify <VAR>t</VAR> with a list of ASCII characters constructed from
<VAR>chars</VAR>.

<P>
<DT><A NAME="PL_unisy_string_chars()"><VAR>void</VAR> <STRONG>PL_unisy_string_chars</STRONG>(<VAR>term_t 
?t, const char *chars</VAR>)</A><DD>
Unify <VAR>t</VAR> with a Prolog string object created from the 
zero-terminated string <VAR>chars</VAR>. The data will be copied. See 
also <A HREF="sec-5.6.html#PL_put_string_nchars()">PL_put_string_nchars()</A>.

<P>
<DT><A NAME="PL_put_string_nchars()"><VAR>void</VAR> <STRONG>PL_put_string_nchars</STRONG>(<VAR>term_t 
?t, unsigned int len, const char *chars</VAR>)</A><DD>
Unify <VAR>t</VAR> with a Prolog string object created from the string 
created from the <VAR>len</VAR>/<VAR>chars</VAR> pair. The data will be 
copied. This interface can deal with 0-bytes in the string. See also
<A HREF="sec-5.6.html">section 5.6.17</A>.

<P>
<DT><A NAME="PL_unify_integer()"><VAR>int</VAR> <STRONG>PL_unify_integer</STRONG>(<VAR>term_t 
?t, long n</VAR>)</A><DD>
Unify <VAR>t</VAR> with a Prolog integer from <VAR>n</VAR>.

<P>
<DT><A NAME="PL_unify_float()"><VAR>int</VAR> <STRONG>PL_unify_float</STRONG>(<VAR>term_t 
?t, double f</VAR>)</A><DD>
Unify <VAR>t</VAR> with a Prolog float from <VAR>f</VAR>.

<P>
<DT><A NAME="PL_unify_pointer()"><VAR>int</VAR> <STRONG>PL_unify_pointer</STRONG>(<VAR>term_t 
?t, void *ptr</VAR>)</A><DD>
Unify <VAR>t</VAR> with a Prolog integer describing the pointer. See 
also
<A HREF="sec-5.6.html#PL_put_pointer()">PL_put_pointer()</A> and <A HREF="sec-5.6.html#PL_get_pointer()">PL_get_pointer()</A>.

<P>
<DT><A NAME="PL_unify_functor()"><VAR>int</VAR> <STRONG>PL_unify_functor</STRONG>(<VAR>term_t 
?t, functor_t f</VAR>)</A><DD>
If <VAR>t</VAR> is a compound term with the given functor, just succeed. 
If it is unbound, create a term and bind the variable, else fails. Not 
that this function does not create a term if the argument is already 
instantiated.

<P>
<DT><A NAME="PL_unify_list()"><VAR>int</VAR> <STRONG>PL_unify_list</STRONG>(<VAR>term_t 
?l, term_t -h, term_t -t</VAR>)</A><DD>
Unify <VAR>l</VAR> with a list-cell (</EM><TT>./2</TT>). If successful, 
write a reference to the head of the list to <VAR>h</VAR> and a 
reference to the tail of the list in <VAR>t</VAR>. This reference may be 
used for subsequent calls to this function. Suppose we want to return a 
list of atoms from a <CODE>char **</CODE>. We could use the example 
described by
<A HREF="sec-5.6.html#PL_put_list()">PL_put_list()</A>, followed by a 
call to <A HREF="sec-5.6.html#PL_unify()">PL_unify()</A>, or we can use 
the code below. If the predicate argument is unbound, the difference is 
minimal (the code based on <A HREF="sec-5.6.html#PL_put_list()">PL_put_list()</A> 
is probably slightly faster). If the argument is bound, the code below 
may fail before reaching the end of the word-list, but even if the 
unification succeeds, this code avoids a duplicate (garbage) list and a 
deep unification.

<P>

<XMP>
foreign_t
pl_get_environ(term_t env)
{ term_t l = PL_copy_term_ref(env);
  term_t a = PL_new_term_ref();
  extern char **environ;

  while(*environ)
  { if ( !PL_unify_list(l, a, l) ||
         !PL_unify_atom_chars(a, *environ) )
      PL_fail;
  }

  return PL_unify_nil(l);
}
</XMP>

<P>

<P>
<DT><A NAME="PL_unify_nil()"><VAR>int</VAR> <STRONG>PL_unify_nil</STRONG>(<VAR>term_t 
?l</VAR>)</A><DD>
Unify <VAR>l</VAR> with the atom <CODE></CODE>.

<P>
<DT><A NAME="PL_unify_arg()"><VAR>int</VAR> <STRONG>PL_unify_arg</STRONG>(<VAR>int 
index, term_t ?t, term_t ?a</VAR>)</A><DD>
Unifies the </EM><EM>index-th</EM> argument (1-based) of <VAR>t</VAR> 
with
<VAR>a</VAR>.

<P>
<DT><A NAME="PL_unify_term()"><VAR>int</VAR> <STRONG>PL_unify_term</STRONG>(<VAR>term_t 
?t, ...</VAR>)</A><DD>
Unify <VAR>t</VAR> with a (normally) compound term. The remaining 
arguments is a sequence of a type identifier, followed by the required 
arguments. This predicate is an extension to the Quintus and SICStus 
foreign interface from which the SWI-Prolog foreign interface has been 
derived, but has proved to be a powerful and comfortable way to create 
compound terms from C. Due to the vararg packing/unpacking and the 
required type-switching this interface is slightly slower than using the 
primitives. Please note that some bad C-compilers have fairly low limits 
on the number of arguments that may be passed to a function.

<P>The type identifiers are:

<P>

<DL>

<P>
<DT><CODE>PL_VARIABLE</CODE> <VAR>none</VAR><DD>
No op. Used in arguments of <CODE>PL_FUNCTOR</CODE>.

<P>
<DT><CODE>PL_ATOM</CODE> <VAR>atom_t</VAR><DD>
Unify the argument with an atom, as in <A HREF="sec-5.6.html#PL_unify_atom()">PL_unify_atom()</A>.

<P>
<DT><CODE>PL_INTEGER</CODE> <VAR>long</VAR><DD>
Unify the argument with an integer, as in <A HREF="sec-5.6.html#PL_unify_integer()">PL_unify_integer()</A>.

<P>
<DT><CODE>PL_FLOAT</CODE> <VAR>double</VAR><DD>
Unify the argument with a float, as in <A HREF="sec-5.6.html#PL_unify_float()">PL_unify_float()</A>. 
Note that, as the argument is passed using the C vararg conventions, a 
float must be casted to a double explicitly.

<P>
<DT><CODE>PL_STRING</CODE> <VAR>const char *</VAR><DD>
Unify the argument with a string object, as in <B>PL_unify_string_chars()</B>.

<P>
<DT><CODE>PL_TERM</CODE> <VAR>term_t</VAR><DD>
Unify a subterm. Note this may the return value of a <A HREF="sec-5.5.html#PL_new_term_ref()">PL_new_term_ref()</A> 
call to get access to a variable.

<P>
<DT><CODE>PL_CHARS</CODE> <VAR>const char *</VAR><DD>
Unify the argument with an atom, constructed from the C <CODE>char *</CODE>, 
as in <A HREF="sec-5.6.html#PL_unify_atom_chars()">PL_unify_atom_chars()</A>.

<P>
<DT><CODE>PL_FUNCTOR</CODE> <VAR>functor_t, ...</VAR><DD>
Unify the argument with a compound term. This specification should be 
followed by exactly as many specifications as the number of arguments of 
the compound term.

<P>
<DT><CODE>PL_LIST</CODE> <VAR>int length, ...</VAR><DD>
Create a list of the indicated length. The following arguments contain 
the elements of the list.
</DL>

<P>For example, to unify an argument with the term <CODE>language(dutch)</CODE>, 
the following skeleton may be used:

<P>

<XMP>
static functor_t FUNCTOR_language1;

static void
init_constants()
{ FUNCTOR_language1 = PL_new_functor(PL_new_atom("language"), 1);
}

foreign_t
pl_get_lang(term_t r)
{ return PL_unify_term(r,
                       PL_FUNCTOR, FUNCTOR_language1,
                           PL_CHARS, "dutch");
}

install_t
install()
{ PL_register_foreign("get_lang", 1, pl_get_lang, 0);
  init_constants();
}
</XMP>

<P>
</DL>

<P>

<H3><A NAME="sec:5.6.6">5.6.6 Calling Prolog from C</A></H3>

<P>The Prolog engine can be called from C. There are to interfaces for 
this. For the first, a term is created that could be used as an argument 
to <A NAME="idx:call/1:605"></A><A HREF="sec-3.7.html#call/1">call/1</A> 
and next <A HREF="sec-5.6.html#PL_call()">PL_call()</A> is used to call 
Prolog. This system is simple, but does not allow to inspect the 
different answers to a non-deterministic goal and is relatively slow as 
the runtime system needs to find the predicate. The other interface is 
based on
<A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A>, <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> 
and <A HREF="sec-5.6.html#PL_cut_query()">PL_cut_query()</A> or
<A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A>. This 
mechanism is more powerful, but also more complicated to use.

<P>

<H4><A NAME="sec:5.6.6.1">5.6.6.1 Predicate references</A></H4>

<P>This section discusses the functions used to communicate about 
predicates. Though a Prolog predicate may defined or not, redefined, 
etc., a Prolog predicate has a handle that is not destroyed, nor moved. 
This handle is known by the type <CODE>predicate_t</CODE>.

<P>

<DL>

<P>
<DT><A NAME="PL_pred()"><VAR>predicate_t</VAR> <STRONG>PL_pred</STRONG>(<VAR>functor_t 
f, module_t m</VAR>)</A><DD>
Return a handle to a predicate for the specified name/arity in the given 
module. This function always succeeds, creating a handle for an 
undefined predicate if no handle was available.

<P>
<DT><A NAME="PL_predicate()"><VAR>predicate_t</VAR> <STRONG>PL_predicate</STRONG>(<VAR>const 
char *name, int arity, const char* module</VAR>)</A><DD>
Same a <A HREF="sec-5.6.html#PL_pred()">PL_pred()</A>, but provides a 
more convenient interface to the C-programmer.

<P>
<DT><A NAME="PL_predicate_info()"><VAR>void</VAR> <STRONG>PL_predicate_info</STRONG>(<VAR>predicate_t 
p, atom_t *n, int *a, module_t *m</VAR>)</A><DD>
Return information on the predicate <VAR>p</VAR>. The name is stored 
over <VAR>n</VAR>, the arity over <VAR>a</VAR>, while <VAR>m</VAR> 
receives the definition module. Note that the latter need not be the 
same as specified with <A HREF="sec-5.6.html#PL_predicate()">PL_predicate()</A>. 
If the predicate was imported into the module given to <A HREF="sec-5.6.html#PL_predicate()">PL_predicate()</A>, 
this function will return the module where the predicate was defined.
</DL>

<P>

<H4><A NAME="sec:5.6.6.2">5.6.6.2 Initiating a query from C</A></H4>

<P>This section discusses the functions for creating and manipulating 
queries from C. Note that a foreign context can have at most one active 
query. This implies it is allowed to make strictly nested calls between 
C and Prolog (Prolog calls C, calls Prolog, calls C, etc., but it is <STRONG>not</STRONG> 
allowed to open multiple queries and start generating solutions for each 
of them by calling <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>. 
Be sure to call <A HREF="sec-5.6.html#PL_cut_query()">PL_cut_query()</A> 
or <A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A> on any 
query you opened before opening the next or returning control back to 
Prolog.

<P>

<DL>

<P>
<DT><A NAME="PL_open_query()"><VAR>qid_t</VAR> <STRONG>PL_open_query</STRONG>(<VAR>module_t 
ctx, int flags, predicate_t p, term_t +t0</VAR>)</A><DD>

<P>Opens a query and returns an identifier for it. This function always 
succeeds, regardless whether the predicate is defined or not. <VAR>ctx</VAR> 
is the </EM><EM>context module</EM> of the goal. When <CODE>NULL</CODE>, 
the context module of the calling context will be used, or <CODE>user</CODE> 
if there is no calling context (as may happen in embedded systems). Note 
that the context module only matters for <EM>module_transparent</EM> 
predicates. See <A NAME="idx:context_module/1:606"></A><A HREF="sec-4.8.html#context_module/1">context_module/1</A> 
and <A NAME="idx:module_transparent/1:607"></A><A HREF="sec-4.8.html#module_transparent/1">module_transparent/1</A>. 
The <VAR>p</VAR> argument specifies the predicate, and should be the 
result of a call to <A HREF="sec-5.6.html#PL_pred()">PL_pred()</A> or <A HREF="sec-5.6.html#PL_predicate()">PL_predicate()</A>. 
Note that it is allowed to store this handle as global data and reuse it 
for future queries. The term-reference <VAR>t0</VAR> is the first of a 
vector of term-references as returned by
<A HREF="sec-5.5.html#PL_new_term_refs()">PL_new_term_refs(n)</A>.

<P>The <VAR>flags</VAR> arguments provides some additional options 
concerning debugging and exception handling. It is a bitwise or of the 
following values:

<P>

<DL>

<P>
<DT><CODE>PL_Q_NORMAL</CODE><DD>
Normal operation. The debugger inherits its settings from the 
environment. If an exception occurs that is not handled in Prolog, a 
message is printed and the tracer is started to debug the error.<A NAME=back-to-note-26 HREF="Notes.html#note-26"> (26)</A>

<P>
<DT><CODE>PL_Q_NODEBUG</CODE><DD>
Switch off the debugger while executing the goal. This option is used by 
many calls to hook-predicates to avoid tracing the hooks. An example is <A NAME="idx:print/1:608"></A><A HREF="sec-3.18.html#print/1">print/1</A> 
calling <A NAME="idx:portray/1:609"></A><A HREF="sec-3.18.html#portray/1">portray/1</A> 
from foreign code.

<P>
<DT><CODE>PL_Q_CATCH_EXCEPTION</CODE><DD>
If an exception is raised while executing the goal, do not report it, 
but make it available for <A HREF="sec-5.6.html#PL_exception()">PL_exception()</A>.

<P>
<DT><CODE>PL_Q_PASS_EXCEPTION</CODE><DD>
As <CODE>PL_Q_CATCH_EXCEPTION</CODE>, but do not invalidate the 
exception-term while calling <A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A>. 
This option is experimental.
</DL>

<P>The example below opens a query to the predicate is_a/2 to find the 
ancestor of for some name.

<P>

<XMP>
char *
ancestor(const char *me)
{ term_t a0 = PL_new_term_refs(2);
  static predicate_t p;

  if ( !p )
    p = PL_predicate("is_a", 2, "database");

  PL_put_atom_chars(a0, me);
  PL_open_query(NULL, PL_Q_NORMAL, p, a0);
  ...
}
</XMP>

<P>

<P>
<DT><A NAME="PL_next_solution()"><VAR>int</VAR> <STRONG>PL_next_solution</STRONG>(<VAR>qid_t 
qid</VAR>)</A><DD>
Generate the first (next) solution for the given query. The return value 
is <CODE>TRUE</CODE> if a solution was found, or <CODE>FALSE</CODE> to 
indicate the query could not be proven. This function may be called 
repeatedly until it fails to generate all solutions to the query.

<P>
<DT><A NAME="PL_cut_query()"><VAR>void</VAR> <STRONG>PL_cut_query</STRONG>(<VAR>qid</VAR>)</A><DD>
Discards the query, but does not delete any of the data created by the 
query. It just invalidate <VAR>qid</VAR>, allowing for a new call to
<A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A> in this 
context.

<P>
<DT><A NAME="PL_close_query()"><VAR>void</VAR> <STRONG>PL_close_query</STRONG>(<VAR>qid</VAR>)</A><DD>
As <A HREF="sec-5.6.html#PL_cut_query()">PL_cut_query()</A>, but all 
data and bindings created by the query are destroyed.

<P>
<DT><A NAME="PL_call_predicate()"><VAR>int</VAR> <STRONG>PL_call_predicate</STRONG>(<VAR>module_t 
m, int debug, predicate_t pred, term_t +t0</VAR>)</A><DD>
Shorthand for <A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A>, <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>, <A HREF="sec-5.6.html#PL_cut_query()">PL_cut_query()</A>, 
generating a single solution. The arguments are the same as for
<A HREF="sec-5.6.html#PL_open_query()">PL_open_query()</A>, the return 
value is the same as <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>.

<P>
<DT><A NAME="PL_call()"><VAR>int</VAR> <STRONG>PL_call</STRONG>(<VAR>term_t, 
module_t</VAR>)</A><DD>
Call term just like the Prolog predicate <A NAME="idx:once/1:610"></A><A HREF="sec-3.7.html#once/1">once/1</A>. <VAR>Term</VAR> 
is called in the specified module, or in the context module if module_t 
= NULL. Returns <CODE>TRUE</CODE> if the call succeeds, <CODE>FALSE</CODE> 
otherwise.
<A HREF="sec-5.6.html#fig:calling">Figure 8</A> shows an example to 
obtain the number of defined atoms. All checks are omitted to improve 
readability.
</DL>

<P>

<H3><A NAME="sec:5.6.7">5.6.7 Discarding Data</A></H3>

<P>The Prolog data created and term-references needed to setup the call 
and/or analyse the result can in most cases be discarded right after the 
call. <A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A> 
allows for destructing the data, while leaving the term-references. The 
calls below may be used to destroy term-references and data. See <A HREF="sec-5.6.html#fig:calling">figure 
8</A> for an example.

<P>

<DL>

<P>
<DT><A NAME="PL_open_foreign_frame()"><VAR>fid_t</VAR> <STRONG>PL_open_foreign_frame</STRONG>(<VAR></VAR>)</A><DD>
Created a foreign frame, holding a mark that allows the system to undo 
bindings and destroy data created after it as well as providing the 
environment for creating term-references. This function is called by the 
kernel before calling a foreign predicate.

<P>
<DT><A NAME="PL_close_foreign_frame()"><VAR>void</VAR> <STRONG>PL_close_foreign_frame</STRONG>(<VAR>fid_t 
id</VAR>)</A><DD>
Discard all term-references created after the frame was opened. All 
other Prolog data is retained. This function is called by the kernel 
whenever a foreign function returns control back to Prolog.

<P>
<DT><A NAME="PL_discard_foreign_frame()"><VAR>void</VAR> <STRONG>PL_discard_foreign_frame</STRONG>(<VAR>fid_t 
id</VAR>)</A><DD>
Same as <A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A>, 
but also undo all bindings made since the open and destroy all Prolog 
data.
</DL>

<P>It is obligatory to call either of the two closing functions to 
discard a foreign frame. Foreign frames may be nested.

<P>

<P>

<XMP>
int
count_atoms()
{ fid_t fid = PL_open_foreign_frame();
  term_t goal  = PL_new_term_ref();
  term_t a1    = PL_new_term_ref();
  term_t a2    = PL_new_term_ref();
  functor_t s2 = PL_new_functor(PL_new_atom("statistics"), 2);
  int atoms;
  
  PL_put_atom_chars(a1, "atoms");
  PL_cons_functor(goal, s2, a1, a2);
  PL_call(goal, NULL);         /* call it in current module */
  
  PL_get_integer(a2, &atoms);
  PL_discard_foreign_frame(fid);
  
  return atoms;
}
</XMP>

<P><CENTER><B>Figure 8 : Calling Prolog</B></CENTER>
<A NAME="fig:calling"></A>

<P>

<H3><A NAME="sec:5.6.8">5.6.8 Foreign Code and Modules</A></H3>

<P>Modules are identified via a unique handle. The following functions 
are available to query and manipulate modules.

<P>

<DL>

<P>
<DT><A NAME="PL_context()"><VAR>module_t</VAR> <STRONG>PL_context</STRONG>(<VAR></VAR>)</A><DD>
Return the module identifier of the context module of the currently 
active foreign predicate.

<P>
<DT><A NAME="PL_strip_module()"><VAR>int</VAR> <STRONG>PL_strip_module</STRONG>(<VAR>term_t 
+raw, module_t *m, term_t -plain</VAR>)</A><DD>
Utility function. If <VAR>raw</VAR> is a term, possibly holding the 
module construct &lt;<VAR>module</VAR>&gt;</EM><TT>:</TT>&lt;<VAR>rest</VAR>&gt; 
this function will make
<VAR>plain</VAR> a reference to &lt;<VAR>rest</VAR>&gt; and fill <VAR>module 
*</VAR> with &lt;<VAR>module</VAR>&gt;. For further nested module 
constructs the inner most module is returned via <VAR>module *</VAR>. If <VAR>raw</VAR> 
is not a module construct <VAR>arg</VAR> will simply be put in <VAR>plain</VAR>. 
If <VAR>module *</VAR> is <CODE>NULL</CODE> it will be set to the 
context module. Otherwise it will be left untouched. The following 
example shows how to obtain the plain term and module if the default 
module is the user module:

<P>

<XMP>
{ module m = PL_new_module(PL_new_atom("user"));
  term_t plain = PL_new_term_ref();

  PL_strip_module(term, &m, plain);
  ...
</XMP>

<P>

<P>
<DT><A NAME="PL_module_name()"><VAR>atom_t</VAR> <STRONG>PL_module_name</STRONG>(<VAR>module_t</VAR>)</A><DD>
Return the name of <VAR>module</VAR> as an atom.

<P>
<DT><A NAME="PL_new_module()"><VAR>module_t</VAR> <STRONG>PL_new_module</STRONG>(<VAR>atom_t 
name</VAR>)</A><DD>
Find an existing or create a new module with name specified by the atom
<VAR>name</VAR>.
</DL>

<P>

<H3><A NAME="sec:5.6.9">5.6.9 Prolog exceptions in foreign code</A></H3>

<P>This section discusses <A HREF="sec-5.6.html#PL_exception()">PL_exception()</A>, <A HREF="sec-5.6.html#PL_throw()">PL_throw()</A> 
and
<A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A>, 
the interface functions to detect and generate Prolog exceptions from 
C-code. <A HREF="sec-5.6.html#PL_throw()">PL_throw()</A> and <A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A> 
from the C-interface to raise an exception from foreign code. <A HREF="sec-5.6.html#PL_throw()">PL_throw()</A> 
exploits the C-function longjmp() to return immediately to the innermost
<A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>. <A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A> 
registers the exception term and returns <CODE>FALSE</CODE>. If a 
foreign predicate returns FALSE, while and exception-term is registered 
a Prolog exception will be raised by the virtual machine.

<P>Calling these functions outside the context of a function 
implementing a foreign predicate results in undefined behaviour.

<P><A HREF="sec-5.6.html#PL_exception()">PL_exception()</A> may be used 
after a call to <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> 
fails, and returns a term reference to an exception term if an exception 
was raised, and 0 otherwise.

<P>If a C-function, implementing a predicate calls Prolog and detects an 
exception using <A HREF="sec-5.6.html#PL_exception()">PL_exception()</A>, 
it can handle this exception, or return with the exception. Some caution 
is required though. It is
<STRONG>not</STRONG> allowed to call <A HREF="sec-5.6.html#PL_close_query()">PL_close_query()</A> 
or
<A HREF="sec-5.6.html#PL_discard_foreign_frame()">PL_discard_foreign_frame()</A> 
afterwards, as this will invalidate the exception term. Below is the 
code that calls a Prolog defined arithmetic function (see <A NAME="idx:arithmethic_function/1:611"></A><B>arithmethic_function/1</B>).

<P>If <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> 
succeeds, the result is analysed and translated to a number, after which 
the query is closed and all Prolog data created after <A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A> 
is destroyed. On the other hand, if
<A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> fails 
and if an exception was raised, just pass it. Otherwise generate an 
exception (<B>PL_error()</B> is an internal call for building the 
standard error terms and calling <A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A>). 
After this, the Prolog environment should be discarded using
<A HREF="sec-5.6.html#PL_cut_query()">PL_cut_query()</A> and <A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A> 
to avoid invalidating the exception term.

<P>

<XMP>
static int
prologFunction(ArithFunction f, term_t av, Number r)
{ int arity = f->proc->definition->functor->arity;
  fid_t fid = PL_open_foreign_frame();
  qid_t qid;
  int rval;

  qid = PL_open_query(NULL, PL_Q_NORMAL, f->proc, av);

  if ( PL_next_solution(qid) )
  { rval = valueExpression(av+arity-1, r);
    PL_close_query(qid);
    PL_discard_foreign_frame(fid);
  } else
  { term_t except;

    if ( (except = PL_exception(qid)) )
    { rval = PL_throw(except);          /* pass exception */
    } else
    { char *name = stringAtom(f->proc->definition->functor->name);

                                        /* generate exception */
      rval = PL_error(name, arity-1, NULL, ERR_FAILED, f->proc);
    }

    PL_cut_query(qid);                  /* donot destroy data */
    PL_close_foreign_frame(fid);        /* same */
  }

  return rval;
}
</XMP>

<P>

<DL>

<P>
<DT><A NAME="PL_raise_exception()"><VAR>int</VAR> <STRONG>PL_raise_exception</STRONG>(<VAR>term_t 
exception</VAR>)</A><DD>
Generate an exception (as <A NAME="idx:throw/1:612"></A><A HREF="sec-3.8.html#throw/1">throw/1</A>) 
and return <CODE>FALSE</CODE>. Below is an example returning an 
exception from foreign predicate:

<P>

<XMP>
foreign_t
pl_hello(term_t to)
{ char *s;

  if ( PL_get_atom_chars(to, &s) )
  { Sprintf("Hello \"%s\"\n", s);

    PL_succeed;
  } else
  { term_t except = PL_new_term_ref();

    PL_unify_term(except,
                  PL_FUNCTOR, PL_new_functor(PL_new_atom("type_error"), 2),
                    PL_CHARS, "atom",
                    PL_TERM,  to);

    return PL_raise_exception(except);
  }
}
</XMP>

<P>

<P>
<DT><A NAME="PL_throw()"><VAR>int</VAR> <STRONG>PL_throw</STRONG>(<VAR>term_t 
exception</VAR>)</A><DD>
Similar to <A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A>, 
but returns using the C longjmp() function to the innermost <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A>.

<P>

<P>
<DT><A NAME="PL_exception()"><VAR>term_t</VAR> <STRONG>PL_exception</STRONG>(<VAR>qid_t 
qid</VAR>)</A><DD>
If <A HREF="sec-5.6.html#PL_next_solution()">PL_next_solution()</A> 
fails, this can be due to normal failure of the Prolog call, or because 
an exception was raised using <A NAME="idx:throw/1:613"></A><A HREF="sec-3.8.html#throw/1">throw/1</A>. 
This function returns a handle to the exception term if an exception was 
raised, or 0 if the Prolog goal simply failed.<A NAME=back-to-note-27 HREF="Notes.html#note-27"> (27)</A>.
</DL>

<P>

<H3><A NAME="sec:5.6.10">5.6.10 Miscellaneous</A></H3>

<P>

<DL>

<P>
<DT><A NAME="PL_compare()"><VAR>int</VAR> <STRONG>PL_compare</STRONG>(<VAR>term_t 
t1, term_t t2</VAR>)</A><DD>
Compares two terms using the standard order of terms and returns -1, 0 
or 1. See also <A NAME="idx:compare/3:614"></A><A HREF="sec-3.5.html#compare/3">compare/3</A>.
</DL>

<P>

<H3><A NAME="sec:5.6.11">5.6.11 Catching Signals (Software Interrupts)</A></H3>

<A NAME="sec:csignal"></A>

<P>SWI-Prolog offers both a C and Prolog interface to deal with software 
interrupts (signals). The Prolog mapping is defined in
<B>section ??</B>. This subsection deals with handling signals from C.

<P>If a signal is not used by Prolog and the handler does not call 
Prolog in any way, the native signal interface routines may be used.

<P>Some versions of SWI-Prolog, notably running on popular Unix 
platforms, handle <CODE>SIG_SEGV</CODE> for guarding the Prolog stacks. 
If the application whishes to handle this signal too, it should use <A HREF="sec-5.6.html#PL_signal()">PL_signal()</A> 
to install its handler after initialisating Prolog. SWI-Prolog will pass
<CODE>SIG_SEGV</CODE> to the user code if it detected the signal is not 
related to a Prolog stack overflow.

<P>Any handler that wishes to call one of the Prolog interface functions 
should call <A HREF="sec-5.6.html#PL_signal()">PL_signal()</A> for its 
installation.

<P>

<DL>

<P>
<DT><A NAME="PL_signal()"><VAR>void (*)()</VAR> <STRONG>PL_signal</STRONG>(<VAR>sig, 
func</VAR>)</A><DD>
This function is equivalent to the BSD-Unix signal() function, 
regardless of the platform used. The signal handler is blocked while the 
signal routine is active, and automatically reactivated after the 
handler returns.

<P>After a signal handler is registered using this function, the native 
signal interface redirects the signal to a generic signal handler inside 
SWI-Prolog. This generic handler validates the environment, creates a 
suitable environment for calling the interface functions described in 
this chapter and finally calls the registered user-handler.
</DL>

<P>

<H3><A NAME="sec:5.6.12">5.6.12 Errors and warnings</A></H3>

<P><A HREF="sec-5.6.html#PL_warning()">PL_warning()</A> prints a 
standard Prolog warning message to the standard error (<CODE>user_error</CODE>) 
stream. Please note that new code should consider using <A HREF="sec-5.6.html#PL_raise_exception()">PL_raise_exception()</A> 
to raise a Prolog exception. See also <A HREF="sec-3.8.html">section 3.8</A>.

<P>

<DL>

<P>
<DT><A NAME="PL_warning()"><VAR>int</VAR> <STRONG>PL_warning</STRONG>(<VAR>format, 
a1, ...</VAR>)</A><DD>
Print an error message starting with `</EM><TT>[WARNING: </TT>', 
followed by the output from <VAR>format</VAR>, followed by a `<CODE>]</CODE>' 
and a newline. Then start the tracer. <VAR>format</VAR> and the 
arguments are the same as for <STRONG>printf</STRONG>(2). Always returns <CODE>FALSE</CODE>.
</DL>

<P>

<H3><A NAME="sec:5.6.13">5.6.13 Environment Control from Foreign Code</A></H3>

<P>

<DL>

<P>
<DT><A NAME="PL_action()"><VAR>int</VAR> <STRONG>PL_action</STRONG>(<VAR>int, 
C_type</VAR>)</A><DD>
Perform some action on the Prolog system. <VAR>int</VAR> describes the 
action, <VAR>C_type</VAR> provides the argument if necessary. The 
actions are listed in <A HREF="sec-5.6.html#tab:action">table 5</A>.

<P>
<BLOCKQUOTE>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>PL_ACTION_TRACE</CODE> <TD>Start Prolog tracer </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_DEBUG</CODE> <TD>Switch on Prolog 
debug mode </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_BACKTRACE</CODE> <TD>Print backtrace 
on current output stream. The argument (an int) is the number of frames 
printed. </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_HALT</CODE> <TD>Halt Prolog 
execution. This action should be called rather than Unix exit() to give 
Prolog the opportunity to clean up. This call does not return. </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_ABORT</CODE> <TD>Generate a Prolog 
abort. This call does not return. </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_BREAK</CODE> <TD>Create a standard 
Prolog break environment. Returns after the user types control-D. </TR>
<TR VALIGN=top><TD><CODE>PL_ACTION_SYMBOLFILE</CODE><TD>The argument (a 
char *) is considered to be hold the symbolfile for further incremental 
loading. Should be called by user applications that perform incremental 
loading as well and want to inform Prolog of the new symbol table. </TR>
</TABLE>

</BLOCKQUOTE>

<P><CENTER><B>Table 5 : <A HREF="sec-5.6.html#PL_action()">PL_action()</A> 
options</B></CENTER>
<A NAME="tab:action"></A>
</DL>

<P>

<H3><A NAME="sec:5.6.14">5.6.14 Querying Prolog</A></H3>

<P>

<DL>

<P>
<DT><A NAME="PL_query()"><VAR>C_type</VAR> <STRONG>PL_query</STRONG>(<VAR>int</VAR>)</A><DD>
Obtain status information on the Prolog system. The actual argument type 
depends on the information required. <VAR>int</VAR> describes what 
information is wanted. The options are given in <A HREF="sec-5.6.html#tab:query">table 
6</A>.

<P>
<BLOCKQUOTE>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>PL_QUERY_ARGC</CODE> <TD>Return an integer 
holding the number of arguments given to Prolog from Unix. </TR>
<TR VALIGN=top><TD><CODE>PL_QUERY_ARGV</CODE> <TD>Return a char ** 
holding the argument vector given to Prolog from Unix. </TR>
<TR VALIGN=top><TD><CODE>PL_QUERY_SYMBOLFILE</CODE> <TD>Return a char * 
holding the current symbol file of the running process. </TR>
<TR VALIGN=top><TD><CODE>PL_QUERY_ORGSYMBOLFILE</CODE><TD>Return the 
initial symbol file (before loading) of Prolog. By setting the symbol 
file to this value no name clashes can occur with previously loaded 
foreign files (but no symbols can be shared with earlier loaded modules 
as well).</TR>
<TR VALIGN=top><TD><CODE>PL_MAX_INTEGER</CODE> <TD>Return a long, 
representing the maximal integer value represented by Prolog's tagged 
integers. </TR>
<TR VALIGN=top><TD><CODE>PL_MIN_INTEGER</CODE> <TD>Return a long, 
represented the minimal integer value. </TR>
<TR VALIGN=top><TD><CODE>PL_QUERY_VERSION</CODE> <TD>Return a long, 
representing the version as
<VAR>10,000 &times; M + 100 &times; m + p</VAR>, where
<VAR>M</VAR> is the major, <VAR>m</VAR> the minor version number and <VAR>p</VAR> 
the patch-level. For example,
<CODE>20717</CODE> means <CODE>2.7.17</CODE>. </TR>
</TABLE>

</BLOCKQUOTE>

<P><CENTER><B>Table 6 : <A HREF="sec-5.6.html#PL_query()">PL_query()</A> 
options</B></CENTER>
<A NAME="tab:query"></A>
</DL>

<P>

<H3><A NAME="sec:5.6.15">5.6.15 Registering Foreign Predicates</A></H3>

<DL>

<P>
<DT><A NAME="PL_register_foreign()"><VAR>int</VAR> <STRONG>PL_register_foreign</STRONG>(<VAR>name, 
arity, function, flags</VAR>)</A><DD>
Register a C-function to implement a Prolog predicate. After this call 
returns successfully a predicate with name <VAR>name</VAR> (a char *) 
and arity <VAR>arity</VAR> (a C int) is created. When called in Prolog, 
Prolog will call <VAR>function</VAR>. <VAR>flags</VAR> forms bitwise 
or'ed list of options for the installation. These are:

<P>
<TABLE BORDER=2 COLS=2 FRAME=border RULES=group>
<TR VALIGN=top><TD><CODE>PL_FA_NOTRACE</CODE> <TD>Predicate cannot be 
seen in the tracer </TR>
<TR VALIGN=top><TD><CODE>PL_FA_TRANSPARENT</CODE> <TD>Predicate is 
module transparent </TR>
<TR VALIGN=top><TD><CODE>PL_FA_NONDETERMINISTIC</CODE> <TD>Predicate is 
non-deterministic. See also <A HREF="sec-5.6.html#PL_retry()">PL_retry()</A>. </TR>
</TABLE>

<P>
<DT><A NAME="PL_register_extensions()"><VAR>void</VAR> <STRONG>PL_register_extensions</STRONG>(<VAR>PL_extension 
*e</VAR>)</A><DD>
Register foreign predicates from a table of structures. The type
<CODE>PL_extension</CODE> is defined as:

<P>

<XMP>
typedef struct _PL_extension
{ char          *predicate_name; /* Name of the predicate */
  short         arity;           /* Arity of the predicate */
  pl_function_t function;        /* Implementing functions */
  short         flags;           /* Or of PL_FA_... */
} PL_extension;
</XMP>

<P>Here is an example of its usage:

<P>

<XMP>
static PL_extension predicates[] = {
{ "foo",        1,      pl_foo, 0 },
{ "bar",        2,      pl_bar, PL_FA_NONDETERMINISTIC },
{ NULL,         0,      NULL,   0 }
};

main(int argc, char **argv)
{ PL_register_extensions(predicates);

  if ( !PL_initialise(argc, argv) )
    PL_halt(1);

  ...
}
</XMP>

<P>The function <A HREF="sec-5.6.html#PL_register_extensions()">PL_register_extensions()</A> 
is the only PL_* function that may be called <STRONG>before</STRONG> <A HREF="sec-5.6.html#PL_initialise()">PL_initialise()</A>. 
The functions are registered after registration of the SWI-Prolog 
builtin foreign predicates and before loading the initial saved state. 
This implies that
<A NAME="idx:initialization/1:615"></A><A HREF="sec-3.2.html#initialization/1">initialization/1</A> 
directives can refer to them.
</DL>

<P>

<H3><A NAME="sec:5.6.16">5.6.16 Foreign Code Hooks</A></H3>

<P>For various specific applications some hooks re provided.

<P>

<DL>

<P>
<DT><A NAME="PL_dispatch_hook()"><VAR>PL_dispatch_hook_t</VAR> <STRONG>PL_dispatch_hook</STRONG>(<VAR>PL_dispatch_hook_t</VAR>)</A><DD>
If this hook is not NULL, this function is called when reading from the 
terminal. It is supposed to dispatch events when SWI-Prolog is connected 
to a window environment. It can return two values:
<CODE>PL_DISPATCH_INPUT</CODE> indicates Prolog input is available on 
file descriptor 0 or <CODE>PL_DISPATCH_TIMEOUT</CODE> to indicate a 
timeout. The old hook is returned. The type <CODE>PL_dispatch_hook_t</CODE> 
is defined as:

<P>

<XMP>
typedef int  (*PL_dispatch_hook_t)(void);
</XMP>

<P>

<P>
<DT><A NAME="PL_abort_hook()"><VAR>void</VAR> <STRONG>PL_abort_hook</STRONG>(<VAR>PL_abort_hook_t</VAR>)</A><DD>
Install a hook when <A NAME="idx:abort/0:616"></A><A HREF="sec-3.36.html#abort/0">abort/0</A> 
is executed. SWI-Prolog <A NAME="idx:abort/0:617"></A><A HREF="sec-3.36.html#abort/0">abort/0</A> 
is implemented using C setjmp()/longjmp() construct. The hooks are 
executed in the reverse order of their registration after the longjmp() 
took place and before the Prolog toplevel is reinvoked. The type
<CODE>PL_abort_hook_t</CODE> is defined as:

<XMP>
typedef void (*PL_abort_hook_t)(void);
</XMP>

<P>

<P>
<DT><A NAME="PL_abort_unhook()"><VAR>int</VAR> <STRONG>PL_abort_unhook</STRONG>(<VAR>PL_abort_hook_t</VAR>)</A><DD>
Remove a hook installed with <A HREF="sec-5.6.html#PL_abort_hook()">PL_abort_hook()</A>. 
Returns <CODE>FALSE</CODE> if no such hook is found, <CODE>TRUE</CODE> 
otherwise.
</DL>

<P>

<H3><A NAME="sec:5.6.17">5.6.17 Storing foreign data</A></H3>

<A NAME="sec:foreigndata"></A>

<P>This section provides some hints for handling foreign data in Prolog. 
With foreign data, we refer to data that is used by foreign language 
predicates and needs to be passed around in Prolog. Excluding 
combinations, there are three principal options for storing such data

<P><UL>
<LI><I>Natural Prolog data</I><BR>
E.i. using the representation one would choose if there was no foreign 
interface required.
<LI><I>Opaque packed Prolog data</I><BR>
Data can also be represetented in a foreign structure and stored on the 
Prolog stacks using <A HREF="sec-5.6.html#PL_put_string_nchars()">PL_put_string_nchars()</A> 
and retrieved using
<B>PL_get_string_chars()</B>. It is generally good practice to wrap the 
string in a compound term with arity 1, so Prolog can identify the type. <A NAME="idx:portray/1:618"></A><A HREF="sec-3.18.html#portray/1">portray/1</A> 
rules may be used to streamline printing such terms during development.
<LI><I>Natural foreign data, passing a pointer</I><BR>
An alternative is to pass a pointer to the foreign data. Again, this 
functor may be wrapped in a compound term.
</UL>

<P>The choice may be guided using the following distinctions

<P><UL>
<LI><I>Is the data opaque to Prolog</I><BR>
With `opaque' data, we refer to data handled in foreign functions, 
passed around in Prolog, but of which Prolog never examines the contents 
of the data itself. If the data is opaque to Prolog, the choosen 
representation does not depend on simple analysis by Prolog, and the 
selection will be driven solely by simplicity of the interface and 
performance (both in time and space).
<LI><I>How big is the data</I><BR>
Is effient encoding required? For examine, a boolean aray may be 
expressed as a compound term, holding integers each of which contains a 
number of bits, or as a list of <CODE>true</CODE> and <CODE>false</CODE>.
<LI><I>What is the nature of the data</I><BR>
For examples in C, constants are often expressed using `enum' or
#define'd integer values. If prolog needs to handle this data, atoms are 
a more logical choice. Whether or not this mapping is used depends on 
whether Prolog needs to interpret the data, how important debugging is 
and how important performance is.
<LI><I>What is the lifetime of the data</I><BR>
We can distinguish three cases.
<OL>
<LI>The lifetime is dictated by the accesibility of the data on the 
Prolog stacks. Their is no way by which the foreign code when the data 
becomes `garbage', and the data thus needs to be represented on the 
Prolog stacks using Prolog data-types. (2),
<LI>The data lives on the `heap' and is explicitly allocated and 
deallocated. In this case, representing the data using native foreign 
representation and passing a pointer to it is a sensible choice.
<LI>The data lives as during the lifetime of a foreign predicate. If the 
predicate is deterministic, foreign automatic variables are suitable. if 
the predicate is non-deterministic, the data may be allocated using 
malloc() and a pointer may be passed. See <A HREF="sec-5.6.html">section 
5.6.1.1</A>.
</OL>
</UL>

<P>

<H4><A NAME="sec:5.6.17.1">5.6.17.1 Examples for storing foreign data</A></H4>

<P>In this section, we wull outline some examples, covering typical 
cases. In the first example, we will deal with extending Prolog's data 
representation with integer-sets, represented as bit-vectors. In the 
second example, we look at handling a `netmask'. Finally, we discuss the 
outline of the DDE interface.

<P><B>Integer sets</B> with not-to-far-apart upper- and lower-bounds can 
be represented using bit-vectors. Common set operations, such as union, 
intersection, etc. are reduced to simple and'ing and or'ing the 
bitvectors. This can be done in Prolog, using a compound term holding 
integer arguments. Especially if the integers are kept below the maximum 
tagged integer value (see <A NAME="idx:feature/2:619"></A><A HREF="sec-2.8.html#feature/2">feature/2</A>), 
this representation is fairly space-efficient (wasting 1 word for the 
functor and and 7 bits per integer for the tags). Arithmetic can all be 
performed in Prolog too.

<P>For really demanding applications, foreign representation will 
perform better, especially time-wise. Bit-vectors are natrually 
expressed using string objects. If the string is wrapped in bitvector/1 
, lower-bound of the vector is 0, and the upperbound is not defined, an 
implementation for getting and putting the setes as well as the union 
predicate for it is below.

<P>

<XMP>
#include <SWI-Prolog.h>

#define max(a, b) ((a) > (b) ? (a) : (b))
#define min(a, b) ((a) < (b) ? (a) : (b))

static functor_t FUNCTOR_bitvector1;

static int
get_bitvector(term_t in, int *len, unsigned char **data)
{ if ( PL_is_functor(in, FUNCTOR_bitvector1) )
  { term_t a = PL_new_term_ref();

    PL_get_arg(1, in, a);
    return PL_get_string(a, (char **)data, len);
  }

  PL_fail;
}

static int
unify_bitvector(term_t out, int len, const unsigned char *data)
{ if ( PL_unify_functor(out, FUNCTOR_bitvector1) )
  { term_t a = PL_new_term_ref();

    PL_get_arg(1, out, a);

    return PL_unify_string_nchars(a, len, (const char *)data);
  }

  PL_fail;
}

static foreign_t
pl_bitvector_union(term_t t1, term_t t2, term_t u)
{ unsigned char *s1, *s2;
  int l1, l2;

  if ( get_bitvector(t1, &l1, &s1) &&
       get_bitvector(t2, &l2, &s2) )
  { int l = max(l1, l2);
    unsigned char *s3 = alloca(l);
    
    if ( s3 )
    { int n;
      int ml = min(l1, l2);

      for(n=0; n<ml; n++)
        s3[n] = s1[n] | s2[n];
      for( ; n < l1; n++)
        s3[n] = s1[n];
      for( ; n < l2; n++)
        s3[n] = s2[n];

      return unify_bitvector(u, l, s3);
    }

    return PL_warning("Not enough memory");
  }

  PL_fail;
}


install_t
install()
{ PL_register_foreign("bitvector_union", 3, pl_bitvector_union, 0);

  FUNCTOR_bitvector1 = PL_new_functor(PL_new_atom("bitvector"), 1);
}
</XMP>

<P><B>Netmask's</B> are used with TCP/IP configuration. Suppose we have 
an application dealing with reasoning about a network configuration. 
Such an application requires communicating netmask structures from the 
operating system, reasoning about them and possibly communicate them to 
the user. A netmask consists of 4 bitmasks between 0 and 255. 
C-application normally see them as an 4-byte wide unsigned integer. 
SWI-Prolog cannot do that, as integers are always signed.

<P>We could use the string approach outlined above, but this makes it 
hard to handle these terms in Prolog. A better choice is a compound term <A NAME="idx:netmask/4:620"></A><B>netmask/4</B>, 
holding the 4 submasks as integer arguments.

<P>As the implementation is trivial, we will omit this here.

<P><B>The DDE interface</B> (see <A HREF="sec-3.42.html">section 3.42</A>) 
represents another common usage of the foreign interface: providing 
communication to new operating system features. The DDE interface 
requires knowledge about active DDE server and client channels. These 
channels contains various foreign data-types. Such an interface is 
normally achieved using an open/close protocol that creates and destroys 
a <EM>handle</EM>. The handle is a reference to a foreign data-structure 
containing the relevant information.

<P>There are a couple of possibilities for representing the handle. The 
choice depends on responsibilities and debugging facilities. The 
simplest aproach is to using <A HREF="sec-5.6.html#PL_unify_pointer()">PL_unify_pointer()</A> 
and <A HREF="sec-5.6.html#PL_get_pointer()">PL_get_pointer()</A>. This 
approach is fast and easy, but has the drawbacks of (untyped) pointers: 
there is no reliable way to detect the validity of the pointer, not to 
verify it is pointing to a structure of the desired type. The pointer 
may be wrapped into a compound term with arity 1 (i.e. <CODE>dde_channel(&lt;<VAR>Pointer</VAR>&gt;)</CODE>), 
making the type-problem less serious.

<P>Alternatively (used in the DDE interface), the interface code can 
maintain a (preferably variable length) array of pointers and return the 
index in this array. This provides better protection. Especially for 
debugging purposes, wrapping the handle in a compound is a good 
suggestion.

<P>

<H3><A NAME="sec:5.6.18">5.6.18 Embedding SWI-Prolog in a C-program</A></H3>

<P>As of version 2.1.0, SWI-Prolog may be embedded in a C-program. To 
reach at a compiled C-program with SWI-Prolog as an embedded application 
is very similar to creating a statically linked SWI-Prolog executable as 
described in <A HREF="sec-5.4.html">section 5.4.1</A>.

<P>The file <CODE>... /pl/include/stub.c</CODE> defines SWI-Prologs 
default main program:

<P>

<XMP>
int
main(int argc, char **argv)
{ if ( !PL_initialise(argc, argv) )
    PL_halt(1);

  PL_install_readline();        /* delete if you don't want readline */

  PL_halt(PL_toplevel() ? 0 : 1);
}
</XMP>

<P>This may be replaced with your own main C-program. The interface 
function <A HREF="sec-5.6.html#PL_initialise()">PL_initialise()</A> </EM><B>must</B> 
be called before any of the other SWI-Prolog foreign language functions 
described in this chapter.
<A HREF="sec-5.6.html#PL_initialise()">PL_initialise()</A> interprets 
all the command-line arguments, except for the <STRONG>-t</STRONG> <VAR>toplevel</VAR> 
flag that is interpreted by <A HREF="sec-5.6.html#PL_toplevel()">PL_toplevel()</A>.

<P>

<DL>

<P>
<DT><A NAME="PL_initialise()"><VAR>int</VAR> <STRONG>PL_initialise</STRONG>(<VAR>int 
argc, char **argv, char **environ</VAR>)</A><DD>
Initialises the SWI-Prolog heap and stacks, restores the boot QLF file, 
loads the system and personal initialisation files, runs the <A NAME="idx:at_initialization/1:621"></A><A HREF="sec-3.2.html#at_initialization/1">at_initialization/1</A> 
hooks and finally runs the <STRONG>-g</STRONG> <VAR>goal</VAR> hook.

<P><A HREF="sec-5.6.html#PL_initialise()">PL_initialise()</A> returns 1 
if all initialisation succeeded and 0 otherwise. Various fatal errors 
may cause PL_initialise to call
<A HREF="sec-5.6.html#PL_halt()">PL_halt(1)</A>, preventing it from 
returning at all.

<P>
<DT><A NAME="PL_install_readline()"><VAR>void</VAR> <STRONG>PL_install_readline</STRONG>(<VAR></VAR>)</A><DD>
Installs the GNU-readline line-editor. Embedded applications that do not 
use the Prolog toplevel should normally delete this line, shrinking the 
Prolog kernel significantly.

<P>
<DT><A NAME="PL_toplevel()"><VAR>int</VAR> <STRONG>PL_toplevel</STRONG>(<VAR></VAR>)</A><DD>
Runs the goal of the <STRONG>-t</STRONG> <VAR>toplevel</VAR> switch 
(default <A NAME="idx:prolog/0:622"></A><A HREF="sec-3.36.html#prolog/0">prolog/0</A>) 
and returns 1 if successful, 0 otherwise.

<P>
<DT><A NAME="PL_halt()"><VAR>void</VAR> <STRONG>PL_halt</STRONG>(<VAR>int 
status</VAR>)</A><DD>
Cleanup the Prolog environment and calls exit() with the status 
argument.
</DL>

