<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 5.9</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-5.8.html">
<LINK REL=next HREF="sec-6.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-5.8.html">Previous</A>
<A HREF="sec-6.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:5.9">5.9 Notes on Using Foreign Code</A></H2>

<P>

<H3><A NAME="sec:5.9.1">5.9.1 Memory Allocation</A></H3>

<P>SWI-Prolog's memory allocation is based on the <STRONG>malloc</STRONG>(3) 
library routines. Foreign applications can safely use
<STRONG>malloc</STRONG>(3), <STRONG>realloc</STRONG>(3) and <STRONG>free</STRONG>(3). 
Memory allocation using <STRONG>brk</STRONG>(2) or <STRONG>sbrk</STRONG>(2) 
is not allowed as these calls conflict with <STRONG>malloc</STRONG>(3).

<P>

<H3><A NAME="sec:5.9.2">5.9.2 Debugging Foreign Code</A></H3>

<P>Statically linked foreign code or embedded systems can be debugged 
normally. Most modern environments provide debugging tools for 
dynamically loaded shared objects or dynamic load libraries. The 
following example traces the code of lowercase using <STRONG>gdb</STRONG>(1) 
in a Unix environment.

<P>

<XMP>
% gcc -I/usr/local/lib/pl-2.2.0/include -fpic -c -g lowercase.c
% gcc -shared -o lowercase.so lowercase.o
% gdb pl
(gdb) r
Welcome to SWI-Prolog (Version \plversion)
Copyright (c) 1993-1996 University of Amsterdam.  All rights reserved.

For help, use ?- help(Topic). or ?- apropos(Word).

?- load_foreign_library(lowercase).
<type Control-C>
(gdb) shared                    % loads symbols for shared objects
(gdb) break pl_lowercase
(gdb) continue
?- lowercase('HELLO', X).
</XMP>

<P>

<H3><A NAME="sec:5.9.3">5.9.3 Name Conflicts in C modules</A></H3>

<P>In the current version of the system all public C functions of 
SWI-Prolog are in the symbol table. This can lead to name clashes with 
foreign code. Someday I should write a program to strip all these 
symbols from the symbol table (why does Unix not have that?). For now I 
can only suggest to give your function another name. You can do this 
using the C preprocessor. If---for example---your foreign package uses a 
function warning(), which happens to exist in SWI-Prolog as well, the 
following macro should fix the problem.

<P>

<XMP>
#define warning warning_
</XMP>

<P>Note that shared libraries do not have this problem as the shared 
library loader will only look for symbols in the main executable for 
symbols that are not defined in the library itself.

<P>

<H3><A NAME="sec:5.9.4">5.9.4 Compatibility of the Foreign Interface</A></H3>

<P>The term-reference mechanism was first used by Quintus Prolog version 
3. SICStus Prolog version 3 is strongly based on the Quintus interface. 
The described SWI-Prolog interface is similar to using the Quintus or 
SICStus interfaces, defining all foreign-predicate arguments of type
<CODE>+term</CODE>. SWI-Prolog explicitly uses type <CODE>functor_t</CODE>, 
while Quintus and SICStus uses &lt;<VAR>name</VAR>&gt; and &lt;<VAR>arity</VAR>&gt;. 
As the names of the functions differ from Prolog to Prolog, a simple 
macro layer dealing with the names can also deal with this detail. For 
example:

<P>

<XMP>
#define QP_put_functor(t, n, a) PL_put_functor(t, PL_new_functor(n, a))
</XMP>

<P>The </EM><TT>PL_unify_*()</TT> functions are lacking from the Quintus 
and SICStus interface. They can easily be emulated or the put/unify 
approach should be used to write compatible code.

<P>The <A HREF="sec-5.6.html#PL_open_foreign_frame()">PL_open_foreign_frame()</A>/<A HREF="sec-5.6.html#PL_close_foreign_frame()">PL_close_foreign_frame()</A> 
combination is lacking from both other Prologs. SICStus has <A HREF="sec-5.5.html#PL_new_term_refs()">PL_new_term_refs(0)</A>, 
followed by <A HREF="sec-5.5.html#PL_reset_term_refs()">PL_reset_term_refs()</A> 
that allows for discarding term references.

<P>The Prolog interface for the graphical user interface package XPCE 
shares about 90% of the code using a simple macro layer to deal with 
different naming and calling conventions of the interfaces.
