<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 6.2</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-6.1.html">
<LINK REL=next HREF="sec-6.3.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-6.1.html">Previous</A>
<A HREF="sec-6.3.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:6.2">6.2 Runtimes and Foreign Code</A></H2>

<P><A NAME="sec:qforeign"></A> Some applications may need to use the 
foreign language interface. Object code is by definition 
machine-dependent and thus cannot be part of the saved program file.

<P>To complicate the matter even further there are various ways of 
loading foreign code:

<P><UL>
<LI><I>Using the library(shlib) predicates</I><BR>
This is the preferred way of dealing with foreign code. It loads quickly 
and ensures an acceptable level of independence between the versions of 
the emulator and the foreign code loaded. It works on Unix machines 
supporting shared libraries and library functions to load them. Most 
modern Unixes, as well as Win32 (Windows 95/NT) satisfy this constraint.
<LI><I>Static linking</I><BR>
This mechanism works on all machines, but generally requires the same 
C-compiler and linker to be used for the external code as is used to 
build SWI-Prolog itself.
</UL>

<P>To make a runtime executable that can run on multiple platforms one 
must make runtime checks to find the correct way of linking. Suppose we 
have a source-file </EM><TT>myextension</TT> defining the installation 
function <B>install()</B>.

<P>If this file is compiled into a shared library, <A NAME="idx:load_foreign_library/1:641"></A><A HREF="sec-5.4.html#load_foreign_library/1">load_foreign_library/1</A> 
will load this library and call the installation function to initialise 
the foreign code. If it is loaded as a static extension, define
<B>install()</B> as the predicate <A NAME="idx:install/0:642"></A><B>install/0</B>:

<P>

<XMP>
static foreign_t
pl_install()
{ install();

  PL_succeed;
}

PL_extension PL_extensions [] =
{
/*{ "name",     arity,  function,       PL_FA_<flags> },*/

  { "install",  0,      pl_install,     0 },
  { NULL,       0,      NULL,           0 }     /* terminating line */
};
</XMP>

<P>Now, use the following Prolog code to load the foreign library:

<P>

<XMP>
load_foreign_extensions :-
        current_predicate(install, install), !, % static loaded
        install.
load_foreign_extensions :-                      % shared library
        load_foreign_library(foreign(myextension)).

:- initialization load_foreign_extensions.
</XMP>

<P>The path alias <CODE>foreign</CODE> is defined by <A NAME="idx:file_search_path/2:643"></A><A HREF="sec-3.2.html#file_search_path/2">file_search_path/2</A>. 
By default it searches the directories <CODE>&lt;<VAR>home</VAR>&gt;/lib/&lt;<VAR>arch</VAR>&gt;</CODE> 
and
<CODE>&lt;<VAR>home</VAR>&gt;/lib</CODE>. The application can specify 
additional rules for
<A NAME="idx:file_search_path/2:644"></A><A HREF="sec-3.2.html#file_search_path/2">file_search_path/2</A>.

