%
% CWI general style
%
% File		: cwigeneral.sty
% Version	: 1.9
% Modified	: 7/29/93 16:59:59
% Author	: Jan van der Steen (jansteen@cwi.nl)
% Based on	: Peter de Waal/Rob van der Horst
%

%
% Paragraphs
%
\parindent 1em				% Length of indent
\parskip   5pt				% Space between paragraphs
\parsep    5pt				% Space between paragraphs in list

\input cwiheading.sty\relax		% Headers and footers
\input cwisection.sty\relax		% Sectioning
\input cwicontents.sty\relax		% Table of Contents
\input cwimath.sty\relax		% Mathematics
\input cwibiblio.sty\relax		% Bibliography
\input cwifonts.sty\relax		% Fonts
\input cwixfonts.sty\relax		% Open characters
\font\egtsfi=cmssi8\relax

%
% CWI specific macros for report classification and abstract fields
% like "Keywords" and "Note"
%
\def\subjclass#1{%
    \vskip1em
    \noindent{\it AMS Subject Classification (1991):} #1\par}
\def\subjclasscr#1{%
    \noindent{\it CR Subject Classification (1991):} #1\par}
\def\keywords#1{%
    \noindent{\it Keywords \& Phrases:} #1\par}
\def\note#1{%
    \noindent{\it Note:} #1\par}

%
% CWI specific strings
%
\def\Cy{{\sc \ifnum\language=\english Corollary	\else Gevolg		\fi}}
\def\Dn{{\sc \ifnum\language=\english Definition\else Definitie		\fi}}
\def\Ee{{\sc \ifnum\language=\english Example	\else Voorbeeld		\fi}}
\def\Fe{{\sc \ifnum\language=\english Figure	\else Figuur		\fi}}
\def\La{{\sc \ifnum\language=\english Lemma	\else Lemma		\fi}}
\def\Pf{{\sc \ifnum\language=\english Proof	\else Bewijs		\fi}}
\def\Rk{{\sc \ifnum\language=\english Remark	\else Opmerking		\fi}}
\def\Rs{{\sc \ifnum\language=\english References\else Referenties	\fi}}
\def\Te{{\sc \ifnum\language=\english Table	\else Tabel		\fi}}
\def\Tm{{\sc \ifnum\language=\english Theorem	\else Stelling		\fi}}

%
% Title, Subtitle(s), Author(s), Affiliation, Abstract
%
\def\title#1{\gdef\@title{#1}}
\def\author#1{\gdef\@author{#1}}
\def\date#1{\gdef\@date{#1}}
\def\thanks#1{\gdef\@thanks{#1}}
\def\and{%
  \end{tabular}
  \vskip 1em % plus .17fil
  \begin{tabular}[t]{c}%
}
\gdef\@title	{CWI style document}%
\gdef\@author	{CWI publication service}%
\gdef\@date	{}%
\gdef\@thanks	{CWI	\\
		 P.O. Box 94079, 1090 GB Amsterdam, The Netherlands}%

\def\maketitle{%
  \par
  \begingroup
    \def\thefootnote{\fnsymbol{footnote}}
    \def\@makefnmark{\hbox to 0pt{$^{\@thefnmark}$\hss}}
    \if@twocolumn\twocolumn[\@maketitle]
    \else
      \newpage
      \global\@topnum\z@
      \@maketitle
    \fi
    \thispagestyle{empty}
  \endgroup
  \setcounter{footnote}{0}
  \let\maketitle\relax
  \let\@maketitle\relax
  \gdef\@thanks{}%
  \gdef\@author{}%
  \gdef\@title{}%
  \let\thanks\relax
}
\def\@maketitle{%
  \newpage
  \null
  \vskip 2em
  \begin{center}
    {\frtnsf\@title\par}
    \vskip 1.5em
    {%
      \tensf\lineskip.5em
      \begin{tabular}[t]{c}
	\@author
      \end{tabular}\par
      \egtsfi
      \begin{tabular}[t]{c}
	\@thanks
      \end{tabular}\par
    }
    \vskip 1em
    {\egtsf \@date}
  \end{center}
  \par
  \vskip 1.5em
}

\def\quotation{\list{}{%
 \listparindent\parindent
 \rightmargin\leftmargin\parsep 0pt plus 1pt}\item[]}

\def\abstract{\if@twocolumn
\section*{Abstract}
\else
\begin{center}
{\bf Abstract\vspace{-.5em}\vspace{0pt}}
\end{center}
\small\egtsf\quotation\fi}

\def\endabstract{\if@twocolumn\else\endquotation\fi}
