%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%									%
%                       CWI Letter style				%
%									%
%			Jan van der Steen				%
%	      Centre for Mathematics and Computer Science		%
%		      Amsterdam, the Netherlands			%
%									%
%-----------------------------------------------------------------------%
% File    : cwiletter.sty						%
% Purpose : Implement the CWI house style in LaTeX			%
% Version : 1.9								%
% Modified: 7/30/93 11:07:49						%
% Author  : Jan van der Steen (jansteen@cwi.nl)				%
% Thanks  : Joke Blom (gollum@cwi.nl)					%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\nofiles
\input	article.sty
\input	art12.sty
\input	epsf.sty

\newif\ifcwifont
\newdimen\infoy
\newdimen\addrx
\newdimen\addry

\font\infotxt=cmr9
\font\addrfnt=cmr10
\font\infofnt=cmr7
\font\heavyfnt=cmb10 scaled 800
\font\lightfnt=cmr5 scaled 800

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Taken from at.sty  12-Aug-91  Colin Hogben (chah@jet.uk)
%
\newbox\@atbox
\long\def\at(#1,#2)#3{\setbox\@atbox=\hbox
 {\unhbox\@atbox
  \vtop to 0pt{\kern #2\hbox to 0pt{\kern #1\relax #3\hss}\vss}}}
\def\@useat{\vtop to 0pt{\kern-\headsep \kern-\topmargin \kern-1in
            \hbox to 0pt{\kern-\@themargin \kern-1in \unhbox\@atbox \hss}\vss}}
\output{\setbox255=\vbox{\@useat \unvbox255}
  \ifnum\outputpenalty <-\@M\@specialoutput\else
  \@makecol\@opcol\@floatplacement\@startcolumn
  \@whilesw\if@fcolmade \fi{\@opcol\@startcolumn}\fi
  \global\vsize\ifnum\outputpenalty >-\@Miv \@colroom
                  \else \maxdimen\fi}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Page numbering
%
\renewcommand{\thepage}{\ifnum \value{page} >1 \arabic{page} \fi}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Global settings
%
\topmargin	 0pt
%\headheight	12pt
\headsep	 0pt
%\topskip	10pt
\textheight	22.5cm \advance\textheight\topskip
\textwidth	16cm
\parindent	 1em
\oddsidemargin	 0pt
\evensidemargin	 0pt
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Fonts used in CWI letter style
%
\def\cwiinit#1{
    \clearpage
    \thispagestyle{empty}%	% Make the first page empty
    \pagenumbering{arabic}%	% Also resets the page count
    \infoy7.0cm
    \addrx\textwidth\advance\addrx1in\advance\addrx-6.7cm
    \addry5.0cm
    #1
    \ifcwifont
	%
	% pfulrc = Futura-CondensedLight
	% pfuh   = Futura-Heavy
	%
	\font\infofnt  = pfulrc at 8pt
	\font\heavyfnt = pfuh   at 8pt
	\font\lightfnt = pfulrc at 9pt
	\epsfpreviewfalse
    \else
	\font\infofnt  = cmr7
	\font\heavyfnt = cmb10 scaled 800
	\font\lightfnt = cmr5  scaled 800
	\epsfpreviewtrue
    \fi
    \cwigadgets
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Address Area
%
\newcommand{\address}[1]{%
    \at(\addrx,\addry){{\addrfnt #1}}%
    \advance\addry11pt
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Info Area
%
\newcommand{\infofield}[2]{%
    \at(1.0in,\infoy){\makebox[2cm][l]{\infofnt #1}{\infotxt #2}}%
    \advance\infoy9pt
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Salutation
%
\newcommand{\salutation}[1]{%
    \at(1.0in,10.5cm){#1 \\}%
    \vspace*{8cm}
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Signature
%
\newcommand{\signature}[2]{%
    \vskip2.0cm
    \noindent\hfill\parbox{.5\textwidth}{\raggedright#1\vskip1.5cm#2}
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Carbon Copies
%
\newcommand{\carboncopies}[1]{%
    \vskip0.2cm\noindent Cc: \vtop{\hsize.5\textwidth\noindent#1}\hfill
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CWI specific (first page) gadgets
%
\def\cwibar{%
    \hfil\rule[-.25cm]{4pt}{.75cm}\kern-2.25pt\rule[.5cm]{.5pt}{.95cm}\hfil
}
\def\cwigadgets{%
  %
  % Company Logo, Company Slogan and Fold Mark
  %
  \at(2.5cm,1.5cm){\epsfxsize2.2cm\epsfbox{/usr/local/lib/CWIlogo/CWI92.ps}}%
  \at(14.7cm,2.7cm){\epsfbox{/usr/local/lib/CWIlogo/c_sw.ps}}%
  \at(18.4cm,1.5cm){\epsfbox{/usr/local/lib/CWIlogo/c_ne.ps}}%
  \at(15.7cm,2.0cm){\parbox{4cm}{%
      \baselineskip10pt\lightfnt
      \raggedright
      {\heavyfnt CWI} is the			\\
      research institute of the			\\
      Stichting Mathematisch Centrum
  }}%
  \at(19.5cm,10.0cm){\rule{1cm}{.5pt}}%
  %
  % Company address, telphone, fax, E-mail and invoice address
  %
  \at(1.0in,26.0cm){%
    \hbox to\textwidth{%
      \hbox{\vbox{%
	  \hsize2.6cm\baselineskip10pt\lightfnt
	  \hbox{\heavyfnt Mail address}
	  \hbox{CWI}
	  \hbox{P.O.Box 94079}
	  \hbox{1090 GB\ \ Amsterdam (NL)}}}%
      \cwibar\hbox{\vbox{%
	  \hsize2.55cm\baselineskip10pt\lightfnt
	  \hbox{\heavyfnt Address}
	  \hbox{CWI}
	  \hbox{Kruislaan 413}
	  \hbox{1098 SJ\ \ Amsterdam (NL)}}}%
      \cwibar\hbox{\vbox{%
	  \hsize2.7cm\baselineskip10pt\lightfnt
	  \hbox{\heavyfnt Telephone}
	  \hbox{+31 20 592 9333}
	  \hbox{+31 20 592 4199 (telefax)}
	  \hbox{12571 mactr nl (telex)}}}%
      \cwibar\hbox{\vbox{%
	  \hsize1.2cm\baselineskip10pt\lightfnt
	  \hbox{\heavyfnt E-mail}
	  \hbox{secr@cwi.nl}
	  \hbox{}
	  \hbox{}}}%
      \cwibar\hbox{\vbox{%
	  \hsize3.15cm\baselineskip10pt\lightfnt
	  \hbox{\heavyfnt Invoice address}
	  \hbox{Stichting Mathematisch Centrum}
	  \hbox{P.O.Box 94079}
	  \hbox{1090 GB\ \ Amsterdam (NL)}}}%
    }%
  }%
}
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
