%
% CWI sectioning
%
% File		: cwisection.sty
% Version	: 1.5
% Modified	: 5/26/93 16:26:38
% Author	: Jan van der Steen (jansteen@cwi.nl)
%
% \chapter	Original definitions taken from rep10.sty (or rep10.doc)
% \section	Original definitions taken from art10.sty (or art10.doc)
%

%
% Heading for \chapter command
%
% Changes: Font sans-serif, and centered display
%
\def\@makechapterhead#1{%
    \vspace*{50pt}%
    {%
	\parindent0pt
	\ifnum \c@secnumdepth>\m@ne
	    \hfil\frtnsf\@chapapp{} \thechapter\hfil\par
	    \vskip5pt
	\fi
	\@xchapter{#1}
    }%
}

%
% Heading for \chapter* command
%
% Changes: Font sans-serif, and centered display
%
\def\@makeschapterhead#1{%
    \vspace*{50pt}%
    {%
	\parindent0pt
	\@xchapter{#1}
    }%
}

\def\@xchapter#1{%
    \hfil\frtnsf#1\hfil\par
    \nobreak
    \vskip150pt
}

\def\section{\@startsection%
    %
    % 1. We changed the font style from "\Large\bf" to "\normalsize\sc".
    % 2. We *don't* want indent of first paragraph
    %    so we give the skip above a negative value.
    %
    {section}				% name
    {1}					% level
    {\z@}				% indent
    {-12pt plus -1ex minus -.2ex}	% skip above
    {.1pt}				% skip below
    {\normalsize\sc}			% style for title
}
\def\subsection{\@startsection%
    %
    % 1. We changed the font style to "\normalsize\it".
    % 2. We *don't* want indent of first paragraph
    %    so we give the skip above a negative value.
    %
    {subsection}			% name
    {2}					% level
    {\z@}				% indent
    {-11pt plus -1ex minus -.2ex}	% skip above
    {.1pt}				% skip below
    {\normalsize\it}			% style for title
}
\def\subsubsection{\@startsection
    %
    % 1. We changed the font style to be "\normalsize\it".
    % 2. We want the paragraph to continue on the same line
    %    so we give the skip below a negative value.
    %    which equals the hskip after the header.
    %
    {subsubsection}			% name
    {3}					% level
    {\z@}				% indent
    {-11pt plus -1ex minus -.2ex}	% skip above
    {-10pt}				% skip after (indent)
    {\normalsize\it}			% style for title
}
\def\paragraph{\@startsection
    %
    % 1. We changed the font style to be "\normalsize\it".
    % 2. We want the paragraph to continue on the same line
    %    so we give the skip below a negative value.
    %    which equals the hskip after the header.
    %
    {paragraph}				% name
    {4}					% level
    {\z@}				% indent
    {-11pt plus -1ex minus -.2ex}	% skip above
    {-10pt}				% skip after (indent)
    {\normalsize\it}			% style for title
}
\def\subparagraph{\@startsection
    %
    % 1. We left the font style to be "\normalsize\bf".
    %
    {subparagraph}			% name
    {4}					% level
    {\z@}				% indent
    {-11pt plus -1ex minus -.2ex}	% skip above
    {-2pt}				% skip after (indent)
    {\normalsize\bf}			% style for title
}

%
% Original definitions are taken from article.sty
%
\def\thepart		{\Roman{part}}
\def\thesection		{\arabic{section}}
\def\thesubsection	{\thesection.\arabic{subsection}}
\def\thesubsubsection	{\thesubsection.\arabic{subsubsection}}
\def\theparagraph	{\thesubsubsection.\arabic{paragraph}}
\def\thesubparagraph	{\theparagraph.\arabic{subparagraph}}

\newdimen\@oldskip
%
% Original definition taken from latex.tex
% The following macro actually *prints* a sectioning header.
%
\def\@sect#1#2#3#4#5#6[#7]#8{%
    % #1 = {section}%				% name
    % #2 = {1}%					% level
    % #3 = {\z@}%				% indent
    % #4 = {-11pt plus -1ex minus -.2ex}%	% skip above
    % #5 = {.1pt}%				% skip below
    % #6 = {\normalsize\sc}%			% style for title
    % #7 = [contents title]%			% the contents entry title
    % #8 = {Section title}%			% the actual section title
    %
    % Adjustments:
    % 1. Adjusted the space between the section number and title.
    % 2. Added a dot after the \section number.
    % 3. Center "section" headers, lower level headers normal
    %
    \ifnum #2>\c@secnumdepth\relax
	\def\@svsec{}%
    \else
	\refstepcounter{#1}%
	%
	% Modified code starts:
	%
	%	1. Adjust size of white space
	%	2. Add a dot for \section & \subsection
	%
	% Original code: \edef\@svsec{\csname the#1\endcsname\hskip 1em }%
	%
	% -- jansteen@cwi.nl
	%
	\edef\@svsec{\csname the#1\endcsname\ifnum #2=1.\fi\hskip .5em\relax}%
	%
	% The modified code ends
	%
    \fi
    \@tempskipa#5\relax
    \ifdim \@tempskipa>\z@\relax
	\begingroup #6\relax
	    \@hangfrom{\hskip #3\relax\@svsec}{\interlinepenalty \@M #8\par}%
	\endgroup
	\csname #1mark\endcsname{#7}\addcontentsline
	{toc}{#1}{%
	    \ifnum #2>\c@secnumdepth\relax
	    \else
		\protect\numberline{\csname the#1\endcsname}%
	    \fi
	    #7\relax
	}%
    \else
	\def\@svsechd{%
	    #6\hskip #3\@svsec #8\csname #1mark\endcsname
	    {#7}\addcontentsline
	    {toc}{#1}{%
		\ifnum #2>\c@secnumdepth\else
		    \protect\numberline{\csname the#1\endcsname}%
		\fi
		#7\relax
	    }%
	}%
    \fi
    \@xsect{#5\relax}%
}%

\def\@xsect#1{%
    \@tempskipa #1\relax
    \ifdim \@tempskipa>\z@\relax
	\par\nobreak
	\vskip\@tempskipa\relax
	\@afterheading
    \else
	\global\@nobreakfalse
	\global\@noskipsectrue
	\everypar{%
	    \if@noskipsec
		\global\@noskipsecfalse
		\clubpenalty\@M
		\hskip -\parindent
		\begingroup \@svsechd \endgroup
		\unskip
		\hskip -#1
	    \else
		\clubpenalty\@clubpenalty
		\everypar{}%
	    \fi
	}%
    \fi
    \ignorespaces
}

\def\@afterheading{%
    %
    % Adjustments:
    % 1. Remove parskip between the \section header and the paragraph
    %
    \global\@nobreaktrue    
    \everypar\expandafter{%
	\expandafter\parskip\the\parskip\relax
	\if@nobreak
	    \global\@nobreakfalse      
	    \clubpenalty \@M
	    \if@afterindent\else{\setbox0=\lastbox}\fi
	\else
	    \clubpenalty \@clubpenalty
	    \everypar{}%
	\fi
    }\parskip0pt\relax
}
