<document>
@* Multimedia database

\slyname{9}

<slide id=9-intro cap="Multimedia database">
@ Multimedia database
<ul>
<li> multimedia database -- homegrown, third-party and legacy sources
<li> architecture -- common format, native format, hybrid
<li> media abstraction -- inverted indexes, relaxation
</ul>
</slide>

@+ Design and architecture of multimedia database

<slide id=9-design cap="Design and architecture -- issues">
@ Design and architecture <i>-- issues</i>
/// <box>multimedia database</box>
<ul>
<li> organisation of content
<li> physical layout
</ul>
</slide>

<slide id=9-arch cap="Architecture -- content organisation">
@ Architecture 
/// <box>content organisation</box>
<ul>
<li> autonomy -- index per media type
<li> uniformity -- unified index
<li> hybrid -- media indexes + unified index
</ul>
</slide>

<slide id=9-ex-hybrid cap="Example -- hybrid">
:@ Example -- hybrid
<p>
Find all images and audio clips in which John Smith's boss
is seen (image!) and heard (audio!) talking to Dennis Dopeman!
:@ Query
<display>
  SELECT   Name, Image, Audio
  FROM employee E, ImgaeDB I, AudioDB A
  WHERE  E.name = "John Smith" AND
	 I contains E.boss AND
	 A contains E.boss AND
	 A contains "Dennis Dopeman"
</display>
</slide>

<slide id=9-uniform cap="Uniform data organisation">
:@ Principle of Uniformity
/// <box>data organisation</box>
<ul>
<li> from a semantical point of view the <em>content</em>
of a multimedia source is independent of the source itself.
</ul>
:@ Consequence
<ul>
<li> use statements as meta data
</ul>
:@ Formally
<ul>
<li> $md(o)$ -- metadata associated with media object $o$
</ul>
:@ Tradeoffs
<ul>
<li> metadata can be stored using standard relational and OO structures
<li> manipulating metadata is easy
<li> feature extraction is (!) straightforward /// <em>is it?</em>
</ul>
</slide>


\clearpage
@+ Media abstractions

<slide id=9-abstr cap="Media abstractions">
@ Media abstractions
<ul>
<li> formal structure that may be used to capture media content
</ul>
<p> $ (S,fe,ATTR,%l,R,F,Var1,Var2) $
<ul>
<li> $S$ -- set of objects called <em>states</em>
<li> $fe$ -- set of objects called <em>features</em>
<li> $ATTR$ -- set of objects called <em>attribute values</em>
<li> $ %l : S -> 2^{fe} $ -- map from states to sets of features
<li> $R$ -- a set of relations -- $fe^{i} \* ATTR^{j} \* S$, for $ i,j >= 0 $
<li> $F$ -- a set of relations of $S$
<li> $Var1$ -- set of variables over $S$
<li> $Var2$ -- set of variables over $fe$
</ul>
</slide>


<slide id=9-explain cap="Media abstractions -- explanation">
:@ Explanation
/// <box>media abstractions</box>
<ul>
<li> <em>state</em> -- smallest chunk of media data
<li> <em>feature</em> -- any object in a state
<li> <em>attributes</em> characteristics of objects
<li> <em>feature extraction map</em> to identify content
<li> relations -- to capture state-dependent information
<li> relations between 'states' or chunks
</ul>
</slide>

<slide id=9-ex-1 cap="Example -- image database">
:@ Example -- image database
/// <box>abstraction</box>
<ul>
<li> states -- {P { pic1.gif,...,picn.gif } P}
<li> features> -- names of people
<li> extraction -- find people in pictures
<li> relations -- {P left-of, ... P}
</ul>
</slide>

<slide id=9-ex-2 cap="Example -- video database">
:@ Example -- video database
/// <box>abstraction</box>
<ul>
<li> states -- set of frames
<li> features> -- persons and objects
<li> extraction -- gives features per frame
<li> relations -- frame-dependent and frame-independent information
<li> inter-state relation -- specifies sequences of frames
</ul>
</slide>

<slide id=9-def-md cap="Definition -- multimedia database">
:@ Definition -- multimedia database
/// <box>simple</box>
<ul>
<li> a simple multimedia database is a finite set $M$
of media abstractions
</ul>
:@ Problems
<ul>
<li> synonyms and inheritance relationships
</ul>
</slide>

<slide id=9-def-md-str cap="Definition -- multimedia database">
:@ Definition -- multimedia database
/// <box>structured</box>
<ul>
<li> a structured multimedia database is more complex
</ul>
<raw height=300>
<p>$ ( {{ M_1,...,M_n }}, ==, <=, inh, subst ) $
<ul>
<li> $ M_i = ( S^i, fe^i, ATTR^i, R^i, F^i, Var1^i, Var2^i) $ -- media abstraction
<li> $ == $ -- equivalence relation of $ F = \cup_{i=1}^{n} fe^i $
<li> $ <= $ -- partial ordering on $ F / == $
<li> $ inh : F/== -> 2^{ F/== } $ such that $ [f1] \e inh([f2]) $
implies that $ [f1] <= [f2] $
<li> $ subst : \cup_{i=1}^{n} ATTR^i -> 2^{ \cup_{i=1}^{n} ATTR^i } $
</ul>
</raw>
<p><text> See the example on p. 240 </text>
</slide>

\clearpage
@+ Query language for retrieving multimedia data


<slide id=9-query-sql cap="Query language">
@ Query language 
/// <box>multimedia data</box>
<ul>
<li> on top of SQL
</ul>
</slide>

<slide id=9-funs cap="Basic functions">
:@ Basic functions
/// <box>SMDS -- uniform</box>
<ul>
<li> Type: object $ |-> $ type 
<li> ObjectWithFeatures: $ f |-> {{ o | $ object <em>o</em> contains $ f }} $
<li> ObjectWithFeaturesAndAttributes: $ (f,a,v) |-> {{ o | $ <em>o</em> contains <em>f</em> with  $ a=v }} $
<li> FeaturesInObject: $ o |-> {{ f | o $ contains $ f }} $
<li> FeaturesAndAttributesInObject: $ o |-> {{ (f,a,v) | o $ contains $ f $ with $ a=v }} $
</ul>
</slide>

<slide id=9-smds-sql cap="SMDS-SQL">
/// <box>SMDS-SQL</box>
:@ SELECT -- <i>media entities</i>
<ul>
<li> $m$ -- if $m$ is not a continuous media object
<li> $m:[i,j]$ -- $m$ is continuous, $i,j$ integers (segments)
<li> $m.a$ -- $m$ is media entity, $a$ is attribute
</ul>
:@ FROM
<ul>
<li> &lt;media&gt;&lt;source&gt;&ltM&gt;
</ul>
:@ WHERE
<ul>
<li> term IN funcall
</ul>
</slide>


<slide id=9-smds-ex cap="Example -- SMDS-SQL">
:@ Example
/// <box>SMDS-SQL</box>
<display>
  SELECT M
  FROM   smds source1 M
  WHERE  Type(M) = Image AND
	 M IN ObjectWithFeature("Dennis") AND
	 M IN ObjectWithFeature("Jane") AND
	 left("Jane","Dennis",M)
</display>
<ul>
<li> M is a relation in the image database media abstraction
</ul>
</slide>


<slide id=9-q-hybrid cap="Querying hybrid representations">
:@ Querying hybrid representations
/// <box>HM-SQL</box>
<ul>
<li> express queries in specialized language
<li> operations (joins) between SMDS and non-SMDS data
</ul>
@ HM-SQL
<ul>
<li> WHERE -- term in MS:funcall
</ul>
:@ Difference
<ul>
<li> function calls are annotated with media source
<li> queries to non-SMDS data may be embedded 
</ul>
</slide>

<slide id=9-ex-hm-sql cap="Example -- HM-SQL">
:@ Example
/// <box>HM-SQL</box>
<display>
 SELECT M
 FROM smds video1, videodb video2
 WHERE M IN smds:ObjectWithFeature("Dennis") AND
       M IN videodb:VideoWithObject("Dennis")
</display>
</slide>


<so q-9>
\clearpage

</document>
