:@+ <a href=question.html>Chapter 9</a>

<N
- In knowledge representation,
inheritance is primarily applied
to describe taxonomic structures
in a declarative way.
Employing exceptions in inheritance
networks leads to non-monotony.
Non-monotonic inheritance networks
may give rise to inconsistencies.
\see{9-knowledge}

- The meaning of an inheritance lattice
may be expressed as a first order logic
formula.
An example is given in
\sliref{9-logic}

- A type denotes a set of individuals.
The subtyping relation is essentially
the set inclusion relation,
with some additional constraints.
However, the subtype relation is best
defined by means of subtype refinement
rules.

- \see{9-subtypes}

- The contravariant nature of the
function subtype refinement rule
may be explained by relying on the
business service metaphor:
refining a service means better work
for less money.
Or, put differently,
refining a function means imposing 
less constraints on the client,
yet delivering a result that is
more tightly defined.
\see{9-functions}

- The notion of {\em objects as records}
is introduced simply to justify
the interpretation of objects as records or
tuples of values and functions.
Again employing a business metaphor,
regarding an object as a collection 
of services, improving such a collection
means offering more, and possibly
better, services.
\see{9-objects}

- Typed formalisms provide protection
against errors.
Yet, untyped formalisms are generally more
flexible.
In the practice of computer science
and mathematics, untyped formalisms
are surprisingly popular.

- A first distinction may be made between
universal polymorphism and {\it ad hoc} polymorphism,
which accounts for overloading and 
coercion.
Universal polymorphism may be subdivided
into parametric polymorphism,
which covers template classes,
and inclusion polymorphism,
that results from derivation by inheritance.
\see{9-flavors}

- Inheritance allows for
the incremental development of object descriptions.
A child class may be regarded as
modifying the parent base class,
as it may include additional attributes
and methods and may refine inherited
attributes or methods.

- See slides \ref{sli-9-c-subtypes},
\ref{sli-9-c-intersection}
and \ref{sli-9-c-bounded}.

- (a) ${{ a : Int , f : Int -> Int }}$,
(b) $ Int -> Int $,
(c) $ {{ a : Bool, f : Bool -> Int }} -> Int $.

- (a) No, since $ 1..4 \not<= 2..5 $.
(b) No, since
$ f : Bool -> Int \not<= f : Int -> Int $,
because $ Int \not<= Bool $.
(c) Yes, since 
${{ a : Bool, f : Bool -> Int }} <= {{ a : Bool }}$.

- To give an example, 
if you have a record $x$ of
type
$ \E %a. {{ val : %a . op : %a -> Int }}$
then you do not need to know the precise
nature of the (hidden) type $%a$
to be able to type the expression
$ x.op( x.val )$ as $Int$.
\see{9-ADT}

- A possible realization is given by the
record
$ {{ a = 0, f = %l  x.E }}$,
for $ E = if \, even(x) \, then \, true \, else \, false $.
The corresponding package is given
by the expression
$ pack [ %a = Int \, in \, {{ a : %a , f : %a -> Bool }} ] ( 0, %l x.E )$.
Another realization is given by the record
type
${{ a : R, f : R -> Bool }}$
where $R$ stands for
${{ x : Int, y : Int }}$.

- The proof involves unrolling.
Let $ T1 =  %m %a.{{ c : %a, b : %a -> %a }}$
and
$ T2 = %m %a.{{ b : %a -> %a }}$.
Now suppose that $ T1 <= T2 $ then,
by unrolling, we would have
that
$ {{ c : T1, b : T1 -> T1 }} <= {{ b : T2 -> T2 }}$,
and hence, by the function subtyping rule,
that $ T2 <= T1 $ and $ T1 <= T2 $.
This would only hold if $ T1 = T2 $,
which is obviously not the case.

- Let $ %s = %m %a.{{ c : %a, b : %t -> %a }}$
and assume that $ %s <= %t $,
then by unrolling
we have that
${{ c : %s, b : %t -> %s }} <= {{ b : %t -> %t}}$
which clearly holds since
$ b : %t -> %s <= b : %t -> %t$.
And, by applying the refinement rule
for recursive types
(given in \sliref{9-recursion}),
we indeed have that $ %s <= %t $.
N>

