<document>
@* Object-oriented databases

\slyname{3}

<slide id=3-concepts cap="Object-oriented databases">
<p align=right><a href=local/college/2.html>?</a>
@ Object-oriented databases
<ul>
<li> concepts -- object-database, object-relational
<li> abstractions -- objects, encapsulation, methods
<li> techniques -- object definition/query language
<li> technology -- object-relational systems
</ul>
</slide>

<slide id=3-compare cap="relational versus objects">
@ relational model
/// structured
<ul>
<li> flat tuples
<li> static schema
<li> integrity constraints
</ul>
@ object model
/// complex data
<ul>
<li> encapsulation
<li> classes and methods
<li> (inheritance) hierarchy
</ul>
</slide>

<slide id=3-example cap="Example">
:@ Examples
<ul>
<li> documents -- (Word, HTML,...)
<li> museum -- art, science, natural history
</ul>
</slide>

<slide id=3-ovs cap="Objects and values">
@ Object alphabet
<ul>
<li> object-ids, class-ids, attributes
</ul>
%%<raw height=100>
@ Object values -- $v$
<ul>
<li> object-id
<li> <em>nil</em>
<li> <latex>[$\bullet$]</latex>[ $a_1 = c_1;...; a_n = c_n$ ]
<li> &lt;$v_1,..., v_n$&gt; -- tuple value
<li> $ {{ v_1,...,v_n }} $ -- set value
</ul>
%%</raw>
</slide>

<slide id=3-example-mm cap="Example multimedia scenario">
@ Example multimedia scenario
<ul>
<li> object-ids = \{ b1, b2,...,bk \}
<li> class-ids = \{ Html, Image, Text, Word-mac, Word-pc \}
<li> attributes = \{ author, date-created, date-last-updated, related-docs \}
</ul>
%%</slide>
%%<slide id=3-example-mm-2 cap="Example multimedia scenario">
:@ Values
<ul>
<li> <latex>[$\bullet$]</latex>[ author = John Smith ]
<li> <latex>[$\bullet$]</latex>[ author = John Smith; date-created = (15 Jan 1996) ]
<li> \{ [ author = John Smith; date-created = (15 Jan 1996) ] \}
<li> &lt; \{ [ author = John Smith; date-created = (15 Jan 1996) ] \} &gt;
<li> <latex>[]</latex>[ author = John Smith; related-docs = { b2, b3 } ]
</ul>
</slide>

<slide id=3-example-museum cap="Example -- museum">
@ Example -- museum
<ul>
<latex>
<li> object-ids = \{ met, louvre, smiths \}
<li> class-ids = \{ Art, History, Nature \}
<li> attributes = \{ address, director, budget, admission, exhibits \}
</latex>
<c>
<li> object-ids = { met, louvre, smiths }
<li> class-ids = { Art, History, Nature }
<li> attributes = { address, director, budget, admission, exhibits }
</c>
</ul>
:@ Values
<ul>
<li> <latex>[$\bullet$]</latex>[ address = The Hall; Washington DC; director = Rudi Fuchs ]
<latex><li> \{ [ exhibits = Caravaggio ], [ exhibits = manuscripts ] \}</latex>
<c><li> { [ exhibits = Caravaggio ], [ exhibits = manuscripts ] }</c>
</ul>
</slide>

<slide id=3-def-object cap="Definition -- objects">
:@ Definition
/// <box>object = (id, value)</box>
@ OODB
<display>
  declare oid
  values  value
</display>
:@ Example
<display>
  declare b2
  values  [ author = A. Eliens;...; date-modified = (10 Jan 2000) ]
</display>
</slide>

<slide id=3-def-types cap="Definition -- types">
:@ Types
<ul>
<li> simple types: attributes, class-ids, object-ids
<li> record: [ f1:t1;...;fn:tn ]
<latex><li> sets: \{ T \}</latex>
<control><li> sets: { T }</control>
<li> lists: &lt; T &gt;
</ul>
:@ Example
<display>
  [ author : string; url : urltype; ; date-modified : datetype ]
</display>
</slide>

<slide id=3-def-subtypes cap="Definition -- subtypes">
:@ Subtypes
<ul>
<li> <latex>[$\bullet$]</latex>[ f1:t1;..;fk:tk ] subtype of [ f1:t1;...;fn:tn ] for <em>k</em> &lt; <em>n</em>
<latex><li> \{ t1 \} subtype of \{ t2 \} if t1 subtype of t2</latex>
<c><li> { t1 } subtype of { t2 } if t1 subtype of t2</c>
<li> &lt; t1 &gt; subtype of &lt; t2 &gt; if t1 subtype of t2
</ul>
:@ Example -- art-museaum subtype of museum
<display>
  museum = [ address : string; departments : { string } ]
  art-museum = [ ...; modern-art : { string } ]
</display>
</slide>

<slide id=3-def-methods cap="Definition -- methods">
:@ Methods
<ul>
<li> just think of Java
</ul>
</slide>

<slide id=3-language cap="Object Definition Language">
:@ Object Definition Language
<display>
 interface html:documents
   extent html_documents
   keys url : persistent
     { 
	&lt;properties&gt;
	&lt;relationships&gt;
     }
</display>
</slide>

<slide id=3-def-oql cap="Object Query Language">
:@ Object Query Language
/// <box>OQL</box>
<ul>
<li> objects may have a nested structure
</ul>
:@ Example
<display>
  SELECT struct(field1: x.url, filed2: x.link)
  FROM Word x
  WHERE x.author = "John Smith"
</display>
</slide>

<slide id=3-oql-example-1 cap="Example OQL">
:@ Example
/// <box>OQL</box>
<display>
  SELECT x.name, y.name
  FROM documents x,
       x.links y,
       y.links z
  WHERE x.url = z.url
</display>
</slide>

<slide id=3-oql-example-2 cap="Example OQL">
:@ Example -- methods
/// <box>OQL</box>
<display>
  SELECT x.url
  FROM mostrecent(
       SELECT x
       FROM Word
       WHERE x.author = John Smith
       )
</display>
</slide>

<slide id=3-o-r cap="Object-relational systems">
@ Object-relational systems
<display>
  bank = (fname,lname,type,trans,date,PIC)
  criminal = (snn,name,conviction,date,PIC)
</display>
@ Object-relational schema
<ul>
<li> $(A_1:T_1;...;A_n:T_n)$ -- with objects
</ul>
@ Object condition
<ul>
<li> match(image1,image2) &gt; 0.7
</ul>
:@ Example 
/// <box>OQL</box>
<display>
  SELECT name
  FROM crime C
  WHERE match(image1,C.PIC) &gt; 0.9 // image1 from surveillance
</display>
</slide>

<so q-3>
\clearpage
</document>
