<document>
@* Image databases

\slyname{5}

<slide id=5-intro cap="Image databases">
@ Image databases
/// <box>image analysis</box>
<ul>
<li> querying -- different from textual databases 
<li> image content -- description, image processing
<li> implementation -- object-relational, n-dimensional
<li> examples -- faces + similarity query (<em>match</em>)
</ul>
<p>
<text>
sections 5.1, 5.4, 5.5, 5.9; skip 5.2, 5.3, 5.6, 5.7, 5.8
</text>
</slide>

<slide id=5-raw cap="Content-based description">
@ Content-based description
/// <box>raw images</box>
<ul>
<li> objects in image
<li> shape descriptor -- shape/region of object
<li> property description -- cells in image
</ul>
</slide>

<slide id=5-descriptors cap="Descriptors">
/// <box>descriptors</box>
:@ Shape 
<ul> 
<li> bounding box -- (XLB,XUB,YLB,YUB)
</ul>
:@ Property 
/// <box>(Name,Value)</box>
<ul> 
<li> property -- name and value domain
</ul>
:@ Example
<ul>
<li> shape descriptor: XLB=10; XUB=60; YLB=3; YUB=50 (rectangle)
<li> property descriptor: pixel(14,7): R=5; G=1; B=3
</ul>
</slide>

<slide id=5-defs cap="Definitions">
:@ Definitions
<ul>
<li> image grid: $ (m * n) $ cells of equal size
<li> cell property: (Name, Value, Method)
</ul>
:@ Example
<ul>
<li> property: (bwcolor,\{b,w\},bwalgo)
</ul>
</slide>

<slide id=5-more-defs cap="More definitions">
:@ More definitions
<ul>
<li> object shape: set of points P
<li> rectangle: 
$ {{ (x,y) | XLB <= x <= XUB \& YLB <= y <= YUB }} $
</ul>
:@ Definition -- image database
/// (GI,Prop,Rec)
<ul>
<li> GI -- set of gridded images: $ (image,m,n) $
<li> Prop -- set of cell properties
<li> Rec -- mapping giving rectangles denoting objects
</ul>
:@ Remarks
<ul>
<li> images may be large -- compression
<li> feature detection -- segmentation
</ul>
</slide>

<slide id=5-similarity cap="Similarity-based retrieval">
@ Similarity-based retrieval
/// <box>5.4</box>
<ul>
<li> How do we determine whether the content of a segment
(of a segmented image) is similar to another image (or set
of images)?
</ul>
:@ Solutions
<ul>
<li> metric approach -- distance between two image objects
<li> transformation approach -- relative to specification
</ul>
</slide>

<slide id=5-metrics cap="The metric approach">
:@ Metric approach
<ul>
<li> objects with pixel properties $ p_1,...,p_n $
<li> pixels: $ (x,y,v1,...,v_n) $
<li> S(obj) contains w x h (n+2)-tuples
<li> distance $d:X->[0,1]$ is distance measure if:
</ul>
<display>
         d(x,y) = d(y,x)
	 d(x,y) <= d(x,z) + d(z,y)
	 d(x,x) = 0
</display>
:@ Complexity
<ul>
<li> an object considered as a set of points in k-dimensional 
space for k = n + 2 
</ul>
</slide>

<slide id=5-features cap="Feature extraction">
@ Feature extraction
/// <box>reduce complexity</box>
<ul>
<li> maps object into s-dimensional space 
</ul>
:@ Index creation: Obj, index T
/// <box>feature extractor: <em>fe</em></box>
<display>
   while Obj != nil {
      o = Obj.get();  // Obj = Obj - o
      T.insert( fe(o) );
      }
   return T;
</display>
</slide>

<slide id=5-similar cap="Find similar objects">
:@ Find similar objects
<ul>
<li> nearest neighbors of  fe(o)  in index T
</ul>
:@ Similarity range
<ul>
<li> index T, object o, tolerance $ 0 <= e <= 1 $
<li> return pi for which $ d(o, fe-1(pi)) <= e $
</ul>
</slide>

<slide id=5-transformation cap="The transformation approach">
:@ Transformation approach
/// <box>edit distance</box>
<ul>
<li> subsumes the metric approach
</ul>
:@ Principle
<p>Given two objects <em>o1</em> and <em>o2</em>,
the level of <em>dissimilarity</em> is proportional
to the (minimum) cost of transforming object <em>o1</em>
into object <em>o2</em> or vice versa
@ Transformation operators
<ul>
<li> $ to_1,...,to_r $ -- translation, rotation, scaling
</ul>
</slide>

<slide id=5-cost cap="Cost and distance">
%%<raw height=30>
:@ Cost
<ul>
<li> $ cost(TS) = %S_{i=1}^{r} cost(to_{i}) $
</ul>
%%</raw>
:@ Distance
<ul>
<li> $d(o,o') = $ min \{ cost(TS) | TS in $ (TSeq(o,o') + TSeq(o'o)) $ \}
</ul>
:@ Example
<ul>
<li> see page 121, 122
</ul>
:@ Advantages
<ul>
<li> user-defined notion of similarity -- by allowing transformation operators 
<li> user-defined cost-function
</ul>
</slide>

<slide id=5-alt cap="Alternative Image DB paradigms">
:@ Alternative Image DB paradigms
<ul>
<li> representing IDB as relations 
<li> representing IDB with spatial data structure
<li> representing IDB using image transformations
</ul>
</slide>


<slide id=5-techn cap="Implementations IDBs">
:@ Implementations IDBs
/// <box>object-oriented</box>
:@ Operations
<ul>
<li> rotate(image-id,dir,angle)
<li> segment(image-id, predicate)
<li> edit(image-id, edit-op)
</ul>
:@ Conventions
<ul>
<li> unsegmented images /// storage
<li> limited set of features /// description
<li> feature-based index /// index
<li> distance between feature vectors /// retrieval
</ul>
</slide>

<so q-5>
\clearpage
</document>
