<document>
@* Image databases

\slyname{5}

<slide id=5-intro cap="Image databases">
@ Image databases
/// <box>image analysis</box>
<ul>
<li> querying -- different from textual databases  </li>
<li> image content -- description, image processing </li>
<li> implementation -- object-relational, n-dimensional </li>
<li> examples -- faces + similarity query (<em>match</em>) </li>
</ul>
</slide>

<slide id=5-raw cap="Content-based description">
@ Content-based description
/// <box>raw images</box>
<ul>
<li> objects in image </li>
<li> shape descriptor -- shape/region of object </li>
<li> property description -- cells in image </li>
</ul>
</slide>

<slide id=5-descriptors cap="Descriptors">
/// <box>descriptors</box>
:@ Shape 
<ul> 
<li> bounding box -- (XLB,XUB,YLB,YUB) </li>
</ul>
:@ Property 
/// <box>(Name,Value)</box>
<ul> 
<li> property -- name and value domain </li>
</ul>
:@ Example
<ul>
<li> shape descriptor: XLB=10; XUB=60; YLB=3; YUB=50 (rectangle) </li>
<li> property descriptor: pixel(14,7): R=5; G=1; B=3 </li>
</ul>
</slide>

<slide id=5-defs cap="Definitions">
:@ Definitions
<ul>
<li> image grid: $ (m * n) $ cells of equal size </li>
<li> cell property: (Name, Value, Method) </li>
</ul>
:@ Example
<ul>
<li> property: (bwcolor,\{b,w\},bwalgo) </li>
</ul>
</slide>

<slide id=5-more-defs cap="More definitions">
:@ More definitions
<ul>
<li> object shape: set of points P </li>
<li> rectangle: $ {{ (x,y) | XLB <= x <= XUB \& YLB <= y <= YUB }} $</li>
</ul>
:@ Definition -- image database
/// (GI,Prop,Rec)
<ul>
<li> GI -- set of gridded images: $ (image,m,n) $</li>
<li> Prop -- set of cell properties</li>
<li> Rec -- mapping giving rectangles denoting objects</li>
</ul>
:@ Remarks
<ul>
<li> images may be large -- compression</li>
<li> feature detection -- segmentation</li>
</ul>
</slide>

<slide id=5-similarity cap="Similarity-based retrieval">
@ Similarity-based retrieval
/// <box>5.4</box>
<ul>
<li> How do we determine whether the content of a segment
(of a segmented image) is similar to another image (or set
of images)?</li>
</ul>
:@ Solutions
<ul>
<li> metric approach -- distance between two image objects</li>
<li> transformation approach -- relative to specification</li>
</ul>
</slide>

<slide id=5-metrics cap="The metric approach">
:@ Metric approach
<ul>
<li> objects with pixel properties $ p_1,...,p_n $</li>
<li> pixels: $ (x,y,v1,...,v_n) $</li>
<li> S(obj) contains w x h (n+2)-tuples</li>
<li> distance $d:X->[0,1]$ is distance measure if:
</ul>
<display>
         d(x,y) = d(y,x)
	 d(x,y) <= d(x,z) + d(z,y)
	 d(x,x) = 0
</display>
:@ Complexity
<ul>
<li> an object considered as a set of points in k-dimensional space for k = n + 2 </li>
</ul>
</slide>

<slide id=5-features cap="Feature extraction">
@ Feature extraction
/// <box>reduce complexity</box>
<ul>
<li> maps object into s-dimensional space </li>
</ul>
:@ Index creation: Obj, index T
/// <box>feature extractor: <em>fe</em></box>
<display>
   while Obj != nil {
      o = Obj.get();  // Obj = Obj - o
      T.insert( fe(o) );
      }
   return T;
</display>
</slide>

<slide id=5-similar cap="Find similar objects">
:@ Find similar objects
<ul>
<li> nearest neighbors of  fe(o)  in index T</li>
</ul>
:@ Similarity range
<ul>
<li> index T, object o, tolerance $ 0 <= e <= 1 $</li>
<li> return pi for which $ d(o, fe-1(pi)) <= e $</li>
</ul>
</slide>

<slide id=5-transformation cap="The transformation approach">
:@ Transformation approach
/// <box>edit distance</box>
<ul>
<li> subsumes the metric approach</li>
</ul>
:@ Principle
<p>Given two objects <em>o1</em> and <em>o2</em>,
the level of <em>dissimilarity</em> is proportional
to the (minimum) cost of transforming object <em>o1</em>
into object <em>o2</em> or vice versa
@ Transformation operators
<ul>
<li> $ to_1,...,to_r $ -- translation, rotation, scaling</li>
</ul>
</slide>

<slide id=5-cost cap="Cost and distance">
%%<raw height=30>
:@ Cost
<ul>
<li> $ cost(TS) = %S_{i=1}^{r} cost(to_{i}) $</li>
</ul>
%%</raw>
:@ Distance
<ul>
<li> $d(o,o') = $ min \{ cost(TS) | TS in $ (TSeq(o,o') + TSeq(o'o)) $ \}</li>
</ul>
:@ Advantages
<ul>
<li> user-defined notion of similarity -- by allowing transformation operators </li>
<li> user-defined cost-function</li>
</ul>
</slide>

<slide id=5-alt cap="Alternative Image DB paradigms">
:@ Alternative Image DB paradigms
<ul>
<li> representing IDB as relations </li>
<li> representing IDB with spatial data structure</li>
<li> representing IDB using image transformations</li>
</ul>
</slide>


<slide id=5-techn cap="Implementations IDBs">
:@ Implementations IDBs
/// <box>object-oriented</box>
:@ Operations
<ul>
<li> rotate(image-id,dir,angle)</li>
<li> segment(image-id, predicate)</li>
<li> edit(image-id, edit-op)</li>
</ul>
:@ Conventions
<ul>
/// storage
<li> unsegmented images </li>
/// description
<li> limited set of features </li>
/// index
<li> feature-based index </li>
/// retrieval
<li> distance between feature vectors </li>
</ul>
</slide>
</document>
