<document>
:@* Video databases

\slyname{7}

<slide id=7-intro cap="Video databases">
@ Video databases
<ul>
<li> content-based retrieval -- knowledge representation</li>
</ul>
:@ Organizing content of a single video
<ul>
<li> choice of interesting aspects</li>
<li> storage of information</li>
<li> query language</li>
/// <em>reliability</em>
<li> automatic content extraction</li> 
</ul>
</slide>

@+ Video content

<slide id=7-content cap="Video content">
:@ Video content <i>-- which aspects to store</i>
<ul>
<li> video v, frame f </li>
<li> f has associated objects and activities </li>
<li> objects and activities have properties (that may change) </li>
</ul>
</slide>

<slide id=7-properties cap="Properties">
@ Properties
/// <box>name = v</box>
<ul>
<li> property = (name,Values)  </li>
</ul>
@ Object schema
/// <box>(fd,fi)</box>
<ul>
<li> (fd,fi) -- frame-dependent and frame-independent properties </li>
</ul>
@ Object instance
/// <box>(oid,os,ip)</box>
<ul>
<li> object-id -- oid </li>
<li> object-schema -- os = (fd,fi) </li>
<li> set of statements -- ip: name = v  and  name = v IN f </li>
</ul>
</slide>

<slide id=7-example-fd cap="Example -- frame-dependent properties">
:@ Example -- frame-dependent properties
<p><center>
<table><!--{ | c | l | l | }-->
<tr><th>frame</th><th>objects</th><th>frame-dependent properties</th></tr>
<tr><td>1</td><td>Jane</td><td>has(briefcase), at(path)</td></tr>
<tr><td>-</td><td>house</td><td>door(closed)</td></tr>
<tr><td>-</td><td>briefcase</td><td></td></tr>
<tr><td>2</td><td>Jane</td><td>has(briefcase), at(door)</td></tr>
<tr><td>-</td><td>Dennis</td><td>at(door)</td></tr>
<tr><td>-</td><td>house</td><td>door(open)</td></tr>
<tr><td>-</td><td>briefcase</td><td></td></tr>
</table>
</center>
</slide>

<slide id=7-example-fi cap="Example -- frame-independent properties">
:@ Example -- frame-independent properties
<p><center>
<table><!--{ | c | l | l | }-->
<tr><th>object</th><th>frame-independent properties</th><th>value</th></tr>
<tr><td>Jane</td><td>age</td><td>35</td></tr>
<tr><td> </td><td>height</td><td>170cm</td></tr>
<tr><td>house</td><td>address</td><td>...</td></tr>
<tr><td> </td><td>color</td><td>brown</td></tr>
<tr><td>briefcase</td><td>color</td><td>black</td></tr>
<tr><td> </td><td>size</td><td>40 x 31</td></tr>
</table>
</center>
</slide>

<slide id=7-activity cap="Activity scheme">
@ Activity scheme
<ul>
<li> finite set of properties </li>
</ul>
@ Activity 
<ul>
<li> activity name -- id </li>
<li> statements -- <em>name = v</em> </li>
</ul>
:@ Example -- activity: <em>exchange</em>
<ul>
<li> { giver : Person, receiver : person, item : Object } </li>
<li> giver = Jane, receiver = Dennis, object = briefcase </li>
</ul>
</slide>

<slide id=7-cbd-video cap="Content-based video description">
@ Content-based video description
/// <box>(OBJ, AC, <math>\lambda</math>)</box>
<ul>
<li> object instances -- OBJ = { oid1,...,oidn } </li>
<li> activities -- AC = { acid1,...,acidn } </li>
<li> lambda:  { 1 .. framenum(v) } $->$ setof(OBJ and AC) </li>
</ul>
<p>
The mapping $lambda$ tells us which objects and which activities
are associated with any given frame f of the video.
</slide>

\clearpage
@+ Querying content of video libraries

<slide id=7-query cap="Video libraries">
@ Video libraries
<ul>
<li> which videos are in the library </li>
<li> what constitutes the content of each video </li>
<li> what is the location of a particular video </li>
</ul>
</slide>

<slide id=7-lib-def cap="Definition -- video library">
:@ Definition -- video library
<p>
/// <box>{P VidLib = { (Content, Id, #frames, R, plm) | ... } P}</box>
<ul>
<li> Content  -- content of the video </li>
<li> Id -- name </li>
<li> \#frames -- number of frames </li>
<li> R -- set of relations </li>
<li> plm -- placement mappings </li>
</ul>
:@ Example -- <em>relation</em>
<ul>
<li> cast = (Id, Role, Person ) </li>
</ul>
</slide>

<slide id=7-ql cap="Query language for video libraries">
@ Query language for video libraries
/// <box>query</box>
<ul>
<li> segment retrievals -- <em>exchange of briefcase</em> </li>
<li> object retrievals -- all people in v:[s,e] </li>
<li> activity retrieval -- all activities in v:[s,e] </li>
<li> property-based -- find all videos with object oid </li>
</ul>
</slide>

<slide id=7-functions cap="Video functions">
@ Video functions
<p>
/// <box>(v,s,e) : (VideoID, StartFrame, EndFrame)</box>
<ul>
<li> VideoWithObject: $ o |-> {{ (v,s,e) | (v,s,e)$  contains $ o }}$ </li>
<li> VideoWithActivity: $ a |-> {{ (v,s,e) | (v,s,e)$  contains $ a }}$ </li>
<li> VideoWithObjectAndProperty: $ (o,p,z) |-> {{ (v,s,e) | $  with $o$ and $ p = z }}$ </li>
<li> VideoWithActivityAndProperty: $ (a,p,z) |-> {{ (v,s,e) | $  with $a$ and $ p = z }}$ </li>
<li> ObjectsInVideo: $ (v,s,e) |-> {{ o | o $  in frames $ }}$ </li>
<li> ActivitiesInVideo: $ (v,s,e) |-> {{ a | a $  in frames $ }}$ </li>
<li> ObjectsAndPropertiesInVideo: $ (v,s,e) |-> {{ o : E | o $  with $ E }}$ </li>
<li> ActivitiesAndPropertiesInVideo: $ (v,s,e) |-> {{ a : E | a $  with $ E }}$ </li>
</ul>
</slide>

<slide id=7-sql cap="VideoSQL">
@ VideoSQL
<ul>
<li> SELECT -- v:[s,e] </li>
<li> FROM -- video:&lt;source&gt;&lt;V&gt; </li>
<li> WHERE -- term IN funcall </li>
</ul>
</slide>

<slide id=7-ex-1 cap="Example 1">
:@ Example 1
<display>
  SELECT  vid:[s,e]
  FROM video:VidLib
  WHERE (vid,s,e) IN VideoWithObject(Dennis)
</display>
</slide>

<slide id=7-ex-2 cap="Example 2">
:@ Example 2
<display>
  SELECT  vid:[s,e]
  FROM video:VidLib
  WHERE (vid,s,e) IN VideoWithObject(Dennis) AND
	(vid,s,e) IN VideoWithObject(Jane)
</display>
</slide>

<slide id=7-ex-3 cap="Example 3">
:@ Example 3
<display>
  SELECT  vid:[s,e]
  FROM video:VidLib
  WHERE (vid,s,e) IN VideoWithObject(Dennis) AND
	object IN ObjectsInVideo(vid,s,e) AND
	object != Dennis AND
	typeof(object) = Person
</display>
</slide>

\clearpage
:@+ Indexing video content

<slide id=7-index cap="Indexing video content">
@ Indexing video content
/// <box>index</box>
<ul>
<li> video content </li>
<li> video library </li>
<li> SQL + video queries </li>
</ul>
:@ We need
<ul>
<li> compact representations </li>
</ul>
@ Frame segment trees
/// <box>segmentation</box>
<ul>
<li> associate objects with (sets of) frame sequences </li>
</ul>
</slide>

\clearpage

@+ Video standards

<slide id=7-standards cap="Video standards">
@ Compression
<ul>
<li> compression -- intraframe analysis </li>
<li> redundant 'blocks' </li>
</ul>
@ Quality of compression
<ul>
<li> color map fidelity </li>
<li> frame pixel resolution </li>
<li> \#frames/second </li>
</ul>
</slide>

<slide id=7-mpeg cap="MPEG-1">
<display>
  IBBPBBIBBPBBI... /// <box>MPEG-1</box>
  IBBPBBPBBPBBI...
</display>
@ MPEG-1
<ul>
<li> sequence of I,P,B frames
</ul>
:@ Frames
<ul>
<li> I: intra-frames -- independent images </li>
<li> P: computed from closest frame using DCT (or from P frame) </li>
<li> B: computed from two closest P or I frames </li>
</ul>
@ Decoding
<ul>
<li> first I, then P, and finally B </li>
</ul>
:@ Alternatives
<ul>
<li> MPEG-2 -- higher pixel resolution and data rate </li>
<li> MPEG-3 -- to support HDTV </li>
</ul>
</slide>

</document>
