<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.35
     from sC++.texi on 14 Febuary 1997 -->

<TITLE>The sC++ language</TITLE>
</HEAD>
<BODY>
@* <a href=@talk-sC++.html target=_top>The sC++ language</a>


@* <A NAME="SEC1" HREF="sC++_toc.html#SEC1">ACTIVE CLASSES</A>

<P>
  The sC++ language permits the creation of <STRONG>active objects</STRONG> which 
have the same form as passive objects, but can delay the execution of their 
methods: Calling a method of an active object suspends the caller until the
called object decides to accept the call.

</P>
<P>
The active object can possess an internal program, called a <STRONG>body</STRONG>,
which is executed in quasi-parallelism with the internal bodies of all the
other active objects. In this way an active object can play the role of a
process. Interprocess communications are provided by means of method calls
between active objects bodies.

</P>
<P>
The class of an active object forms a kind of barrier which ensures that at 
any moment, one and only one program executes the instructions in the object.
It is not possible to interrupt a method or the program of the object without
the program having terminated or explicitly suspended it's execution. This
suspention is acheived by means of the <EM>accept</EM> instruction
(See section <A HREF="sC++.html#SEC6">Accept instruction</A>). Several <EM>accepts</EM> can be placed in parallel
by means of the <STRONG>select</STRONG> instruction.

</P>


@+ <A NAME="SEC2" HREF="sC++_toc.html#SEC2">Definition, creation and destruction of active objects</A>

<P>
  An object is active if the keyword <EM>active</EM> is place before the keyword 
<EM>class</EM> (line 1). The active objects are defined, created, referenced and
destroyed in the same way as passive objects. The body of an active object is
defined as a method without parameters with the name corresponding to that of
the class, preceeded by the symbol '@' (line 7).
The body is therefore added to the list of members in the same way as a 
constructor or a destructor would be, but it must be <EM>private</EM>. By
instanciating an active object, the instanciator will execute the constructor
of the active object. At the end of the constructor the body of the active
object is launched, running in quasi-parallelism with the instanciator.

</P>
<slide id=sc-ex1 cap="Definition">
<PRE>
 1	<STRONG>active</STRONG> class ACT {
 2	public:
 3	   int i;
 4	   void m ();

 5	   ACT () { ... }
 6	private: 
 7	   <STRONG>@ACT</STRONG>() { ... }
 8	} 

 9	ACT x;
10	void main () { 
11	   ACT*y = new ACT; 
12	   delete y; 
13	}

</PRE>
</slide>

<P>
The lines 9 and 11 create active objects of the class <EM>ACT</EM>.

</P>
<P>
The object <EM>x</EM> of line 9 is activated at the same time as the program
<EM>main</EM> is activated, since it acts as a global object. When the program
<EM>main</EM> terminates, it tries to automatically destroy object <EM>x</EM>
conforming to the definition of  C++, but as the destructor can only be
executed if the object accepts it, the program <EM>main</EM> is suspended until
the object <EM>x</EM> accepts it's own destructor using <EM>accept ~ACT</EM>
statement (See section <A HREF="sC++.html#SEC6">Accept instruction</A>).

</P>
<P>
Similarly, object y is created at line 2. The <EM>delete</EM> of line 12 is
blocked until y internally accepts its own destruction.

</P>
<P>
If we define an active object inside a function, it is created when the program
reaches the first curly bracket which contains it. It is destroyed at the 
moment the program leaves the second curly bracket, assuming once again it 
accepts it's destructor.

</P>


@+ <A NAME="SEC3" HREF="sC++_toc.html#SEC3">Active objects</A>

<P>
Let us assume that two objects <EM>01</EM> and <EM>03</EM> call the methods
(m and n) of object <EM>02</EM>. In the case where object <EM>02</EM> has no body,
an automatic protection, integrated into each active object, assures that when
object <EM>01</EM> has started execution of the method <EM>m()</EM>, it is not
interrupted by the calls from object <EM>03</EM>. Even if the method <EM>m()</EM>
contains system calls or delays, it is not interupted. It is also not possible
to execute another method (e.g. <EM>n()</EM> of <EM>02</EM>) whilst <EM>m()</EM> is
actually executing. The only possibility of interruption is an explicit
suspension by means of an accept, using <EM>accept method</EM> statement.

</P>
<P>
The protection of active objects prevents several objects simultaneously 
manipulating the same data, which avoids the risk of unspecified outcomes.

</P>


@+ <A NAME="SEC4" HREF="sC++_toc.html#SEC4">The body of an active object</A>

<P>
The <STRONG>body</STRONG> of an active object has it's own life and is executed in
parallel with the bodies of other active objects. An example of a body is
declared below (lines 3-5).

</P>
<P>
Whilst executing, a body can call its own methods, but the other objects are
suspended if they call those methods. When a body terminates, the object
preserves it's barrier function, and will then work in the same way as if no
body was ever defined in the class.

</P>
<P>
An active object terminates when it is destroyed: It accepts its destructor
and an external object calls this same destructor, or it executes
<EM>delete this</EM> from it's own body.

</P>
<P>
The <STRONG>constructor</STRONG> of an active object (lines 6 and 7) is executed by
the creator of the class, exactly as in the case of a passive object. The body
is launched at the end of the execution of this constructor. In order to pass
values to a body, the constructor must put them into class member variables.

</P>
<slide id=sc-ex2 cap="Body">
<PRE>
1	active class X {
2	   int local;
3	   @X () {
4	      cout &#60;&#60; local;
5	   }
6	public:
7	   X (int value):local (value) {  ... } 
8	};

</PRE>
</slide>

<P>
In the example above, the parameter passed to the constructor is stored in a
class member variable <EM>local</EM> which the body can then access.

</P>


<H1><A NAME="SEC5" HREF="sC++_toc.html#SEC5">SYNCHRONIZATION INSTRUCTIONS</A></H1>


@+ <A NAME="SEC6" HREF="sC++_toc.html#SEC6">Accept instruction</A>

<P>
   The <EM>accept</EM> instruction allows a method or a body to suspend it's
execution. The object which calls <EM>accept</EM> remains suspended until the
method who's name is indicated after the <EM>accept</EM> has been called by
another object and completely executed.

</P>


<slide id=sc-accept cap="Accept">
<PRE>

 1	active class X { 
 2	   public: 
 3	      void m();
 4	   private: 
 5	      @X () { 
 6	      int i = 0; 
 7	         for (;;) { 
 8	            accept m; 
 9	            i++; 
10	         } 
11	      } 
12	} 

</PRE>
</slide>

<P>
In the above example the method <EM>m</EM> can only be executed when the body has
executed <EM>accept m</EM>, and the body can only continue it's loop when
<EM>m()</EM> has been executed.

</P>
<P>
There is a rendezvous between the caller of the method and the code which has
made the <EM>accept</EM>. That is to say that there is a moment when the first 
participant is suspended before the method call and the second suspended in
the acceptance. The method is executed during the rendezvous. Both participants
continue their execution independantly after the rendezvous.

</P>
<P>
   As already mentioned, the destruction of an object is performed in the 
same way as a passive object: When the curly bracket at the end of the
function <EM>main</EM> is reached, when <EM>delete object</EM> is called from
outside, <EM>delete this</EM> from the inside, or when the <EM>~nameClass</EM> is
called either internally or externally.

</P>
<P>
   However, the destructor has the same execution conditions as the other
methods. It can only be executed if there is no other activity inside the
object. In order that an object can be destroyed externally, all it's methods
and the body must have terminated, or the destructor has been explicitly
accepted by <EM>accept ~ClassName</EM> statement.

</P>
<P>
   Note: An object cannot be destroyed by placing the code that provokes its
destruction inside a method of the object, if this method can be called 
externally, since the object would be destroyed in the middle of the execution
of the method, and a correct return would not be made to the caller. It is not
possible to suspend a method called from another object using
<EM>accept ~nameClass</EM>, for the same reasons.

</P>


@+ <A NAME="SEC7" HREF="sC++_toc.html#SEC7">Select Instruction</A>

<P>
The <EM>select</EM> instruction allows several synchronisations and events to be
waited for in parallel. In this instruction, acceptances, method calls,
delays, and a default execution, in the case of the other options not being
immediately ready, can be placed simultaneously. None are evidently
obligatory, and the cases are tested in the order in which they are written.
Therefore the order corresponds to a from of priority.

</P>

<slide id=sc-select cap="Select">

<PRE>
 1	active class S { 
 2	    public: 
 3	       m () { ... } 
 4	    private: 
 5	       @S () { 
 6	         select { 
 7	            01 -&#62; m(); 
 8	            instructions ...
 9	         || 
10	            accept m; 
11	            instructions ... 
12	         ||
13	            waituntil (date); 
14	            instructions ... 
15	         ||
16	            default 
17	            instructions ... 
18	         } 
19	    } 
20	} 

</PRE>
</slide>

<P>
Note: After the  <EM>select</EM> {}, there is no need for <STRONG>;</STRONG>.

</P>
<P>
The <EM>select</EM> instruction above proposes four simultaneous possible events
to the sC++ kernel (lines 7, 10, 13 and 16). When the <EM>select</EM>
instruction is reached, the kernel memorises the 4 indicated options, and as
soon as one of them becomes executable (this could be immediately) it
reactivates the object. The choice that is satisfied is then memorised and the
others are forgotten. The object then executes the instructions which follow
the satisfied case. <STRONG>Only one case is executed</STRONG>.

</P>
<P>
The four cases are detailed below.

</P>

<UL>
<LI>

<STRONG>01-&#62;m();</STRONG> Call to another <STRONG>active</STRONG> object. The event which
triggers the execution in this case is the rendezvous with the object
<EM>01</EM>. Therefore this case is executed when the object <EM>01</EM>
<EM>accepts</EM> m.
 
<LI>

<STRONG>accept m;</STRONG> Acceptance of a local public method. The event which
triggers this case is the rendezvous with any object that calls <EM>m()</EM>.

<LI>

<STRONG>waituntil (date);</STRONG> Delay. This case is executed if the time counter
reaches or passes the value <EM>date</EM> indicated in the parentheses. This
instruction is given in more detail in (See section <A HREF="sC++.html#SEC9">Waituntil instruction</A>).

<LI>

<STRONG>default;</STRONG> If none of the other cases are satisfied at the moment the
<EM>select</EM> is entered, this is immediately executed. This instruction
allows, for example, the testing of a possible message arrival, but avoids the
blocking of execution if the message is not present when the <EM>select</EM> is
executed.
</UL>



@+ <A NAME="SEC8" HREF="sC++_toc.html#SEC8">Guards</A>

<P>
   It is often useful to repeat a <EM>select</EM> instruction several times with
certain cases always present and others which are not present under certain
conditions. This can be realized using guards, which are boolean conditions
preceded by the keyword <EM>when</EM>, as shown below.

</P>

<slide id=sc-guards cap="Guards">

<PRE>

 1	select { 
 2	   when (n &#62; 0) 
 3	      01 -&#62; m(); 
 4	      actions ... 
 5	||
 6	   when (TimeOut) 
 7	      waituntil (date); 
 8	      actions  ...
 9	||
10	   when (FreePlace) 
11	      accept m; 
12	      actions  ...
13	||
14	   when (NoWait) 
15	      default; 
16	      actions  ...
17	} 
</PRE>
</slide>

<P>
The <EM>select</EM> above represents a <EM>selection</EM> of 0,1,2,3 or 4 cases
following the guard values. The guards are evaluated when the <EM>select</EM> is
reached and only the cases where the guards are true are taken into account
for this execution. If the guard conditions change during the selection of the
true cases, the <EM>select</EM> is not changed. These will only be taken into
account on a new call of <EM>select</EM>.

</P>
<P>
Therefore, both parts of the program shown below are absolutely equivalent.

</P>


<slide id=sc-equiv cap="Equivalence">
<PRE>
 1	select { 
 2	   when (n&#62;0) 
 3	   01 -&#62; m(); 
 4	||
 5	   waituntil (date); 
 6	}

is equivalent to  

 7	if (n&#62;0) 
 8	   select { 
 9	      01 -&#62; m(); 
10	   ||
11	      waituntil (date); 
12	   } 
13	else 
14	   select { waituntil (date);}
 
</PRE>
</slide>

<P>
The <EM>select</EM> instruction on line 14 is not necessary, and a simple
<EM>waituntil(date)</EM> could have been be used.

</P>
<P>
The instruction <EM>when</EM> is therefore similar to an <EM>if</EM> statement.
However, the <EM>whens</EM> are evaluated in parallel, which explains why
another name was chosen.

</P>


<H2><A NAME="SEC9" HREF="sC++_toc.html#SEC9">Waituntil instruction</A></H2>

<P>
The real time kernel on which sC++ is based, manages a variable containing the
number of 1/100ths of seconds that pass by from the time the program is first
launched.

</P>
<P>
   This variable can be read by means of the function <EM>now()</EM> which
returns a value of type <EM>unsigned long</EM>.

</P>
<P>
   It is in fact possible for the counter to loop and go back to zero after a
certain time, but this does not change the calculations of the delays as long
as the delay does not exceed one half of the time value corresponding to the 
maximum value of the variable. In effect if this condition is respected, a 
delay can be calculated in the way indicated on line 5.

</P>


<slide id=sc-loop cap="Loop">
<PRE>
1	unsigned long x, y, delay; 

2	x = now(); 
3	actions to be mesured  ...
4	y = now(); 
5	delay = y - x; 

</PRE>
</slide>

<P>
If the program was required to be suspended for one and a half seconds, line 1
would be used in the diagram shown below.

</P>

<slide id=sc-secs cap="Suspend">
<PRE>
1	waituntil (now() + 150); 

2	FinalEvent = now() + T1;

3	... actions  ...

4	waituntil (FinalEvent);

</PRE>
</slide>

<P>
If the delay must be measured from a precise point in the program, and there 
was still certain things to execute before the call to <EM>waituntil</EM>, the
instant corresponding to the end of the delay can be calculated (line 2), the
remaining executions done (line 3) and the delay waited for. If the actions
take more time than T1, the function <EM>waituntil</EM> on line 4  immediately
returns without blocking.

</P>
<P>
   If the program was realized only having one function to suspend the program
for a certain time,  rather than the function <EM>waituntil</EM> proposed here,
it would nevertheless be necessary to use an absolute time, like that provided
by <EM>now()</EM>, to measure the execution time of the actions in line 3.

</P>
<P>
   Line 1 can be considered as executing a relative delay, whereas line 4 
executes an absolute delay.

</P>


@+ <A NAME="SEC10" HREF="sC++_toc.html#SEC10">Forall Instruction</A>

<P>
In the <EM>select</EM> statement it is possible to automatically generate a
certain number of cases by using the <EM>forall</EM> instruction, the syntax of
which is indicated in line 2 below.

</P>


<slide id=sc-select cap="Select">
<PRE>
1	select { 
2	   <STRONG>forall (i : first ... last)</STRONG> 
3	      when (condition (i)) 
4	         object (i) -&#62; method (data[i]); 
5	            ... instructions (i) ... 

</PRE>
</slide>

<P>
The variable i is of type <EM>int</EM>, <EM>signed</EM> or <EM>unsigned</EM>, long
or <EM>short</EM>. <EM>first</EM> and <EM>last</EM> are expressions of one of these
types as indicated above.

</P>
<P>
The <EM>forall</EM> instruction generates a case for each value of <EM>i</EM> from
<EM>first</EM> up to and including <EM>last</EM>, where <EM>i</EM> is replaced each
time by it's specific value. In fact although the code is not repeated, the
result is the same as if it was.

</P>
<P>
One could furthermore consider that only lines 3 and 4 are repeated and that 
the instructions on line 5 only exist for the executed case. Lines 3 to 5 show
examples of use of <EM>i</EM> but <EM>i</EM> can be used in the same way as any
integer variable.

</P>


@* <A NAME="SEC11" HREF="sC++_toc.html#SEC11">INHERITANCE</A>

<P>
Firstly a passive object cannot inherit an active object. It is therefore 
necessary only to present the inheritance of a passive object into an active
object and an active object into an active object.

</P>
<P>
An active object can only inherit a passive object privately. It is therefore 
not possible to reach the methods of a passive object inherited by subtyping.
This would have an undesirable effect: it wouldn't be known wether to 
synchronize or not since that would depend upon the way in which they were 
reached.

</P>
<P>
It is possible to redefine the members of a passive object inherited into an
active object to the interior of an active object as public. In this case, the
calls to these members provoke a synchronization since it is considered that
they have become the property of the active object.

</P>

<slide id=sc-inheritance cap="Inheritance">

<PRE>
 1	class pass { 
 2	   public; 
 3	      int i; 
 4	      void m(); 
 5	}; 
 6	active class act : pass { 
 7	   public: 
 8	      pass :: m;
 9	      @act { ... accept m;  ... } 
10	}; 

</PRE>
</slide>

<P>
When an active class is inherited into another, two independant embedded 
objects are created, each eventually having a body. The active classes can
be inherited in either public or private modes. If they are inherited in 
public mode, <STRONG>subtyping can be used</STRONG>.

</P>
<P>
When an inherited active object is adressed by subtyping or even through the
inheriting object, the inheriting object doesn't notice anything. The call to
a method of the inherited object can be made even if the inheriting object has
not suspended it's execution.

</P>
<P>
An inheriting object can call the methods of the inherited object. In this case
there is also synchronization and the inherited object should accept the calls
in order to be executed. The execution of a call of a method of an inherited
object naturally block any extra access to the inherited object during the
rendez-vous even those generated by the inheriting class to the inherited
object.

</P>
<P>
The rules of the <EM>public</EM>, <EM>protected</EM> or <EM>private</EM> modes in
active classes are used as in passive classes in that they concern visibility.

</P>
<P>
A protected inherited method called from the inheriting object generates a
synchronization in the same way as public method calls.

</P>
<P>
Of course, there is no question for private methods as they cannot be called.

</P>



@+ <A NAME="SEC12" HREF="sC++_toc.html#SEC12">"Virtual" instruction in active objects</A>

<P>
  The behaviour of an active object is based on it's capacity to impose 
constraints upon the times of execution of it's methods. When an active object
is created, it is sometimes interesting to inherit it and to modify it by 
adding extra constraints to these times of execution. A typical example is a
mail box of infinite size transformed  into one of finite size, preventing any
addition of data when the box already contains a predefined number.

</P>
<P>
To do this in sC++, the virtual instruction can be used. We remark
initially that the redefinitions of these virtual methods are done using the 
same syntax as is used for passive objects. What evidently changes is the group
of conditions under which the methods can be executed. These conditions are 
shown below.

</P>

<slide id=sc-virtual cap="Virtual">

<PRE>
 1	active class X { 
 2	   public: 
 3	      virtual void f() { ... } 
 4	   private: 
 5	      @X() {  ... accept f;  ... } 
 6	}; 

 7	active class Y : public X { 
 8	   public: 
 9	      void f() {  ... } 
10	   private: 
11	      @Y() {  ... accept f;  ... } 
12	};
13	active class Z : public Y { 
14	   public: 
15	      void f() {  ... } 
16	   private: 
17	      @Z() {  ... accept f;  ... } 
18	}; 

</PRE>
</slide>

<P>
The classes <EM>X</EM>, <EM>Y</EM> and <EM>Z</EM>  form a class hierarchy. <EM>f()</EM>
is declared virtual in the class <EM>X</EM> (line 3), and redefined in the
classes <EM>Y</EM> and <EM>Z</EM>  (lines 9 and 15). When the method <EM>f()</EM> is
called by <EM>O-&#62;f()</EM>, it is the code of the last redefinition that is
executed, that is to say <EM>Z::f()</EM> in this case. If one of the forms
<EM>O-&#62;X::f()</EM>, <EM>O-&#62;Y::f()</EM> or <EM>O-&#62;Z::f()</EM> is used, the code used
is that of  the class indicated before the sign ::.

</P>
<P>
  However, in each case there is a triple synchronization, that is to say
<EM>f()</EM> must be accepted by the three classes: Each one of these should not
have body or method executing, or have executed <EM>accept f</EM> from the body
or the method, even if the method is called from the outside. The
synchronization is triple whether <EM>f()</EM> is reached by subtyping or by the
intermediary of <EM>Z</EM>.

</P>
<P>
If the method <EM>X::f()</EM> is not redefined in one of the classes <EM>Y</EM> or
<EM>Z</EM>, it evidently cannot be accepted by this class. In this situation,
<EM>f()</EM> can be executed only when the classes that have redefined <EM>f()</EM>
are accepting it.

</P>
<P>
It is possible to call the function <EM>f()</EM> from inside of one of the
classes in the hierarchy shown above, that is to say from it's body, or from
another method. In this case the other classes must accept the execution of
<EM>f()</EM>. The object from where the call originates has already the right to
work in the object, and no longer has to accept the call. Besides, it could
not execute and accept the same method simultaneously.

</P>

<P>
The following is a special case, but it obeys the same rules. Suppose <EM>X</EM>
and <EM>Y</EM> jointly possess virtual <EM>m()</EM> and that <EM>m</EM> calls
<EM>f()</EM>. In this case, only <EM>Z</EM> must still accept <EM>f()</EM> since the
two other classes already have the right to operate on the object.

</P>
<P>
If we wanted to execute <EM>f()</EM> from each of these classes in turn. A
deadlock could occur if by chance two classes call these methods at the same
time. It is therefore necessary to use the code shown below to avoid risk of
deadlock.

</P>

<slide id=sc-dead cap="Avoid deadlock">

<PRE>
1	select {
2	   f(); 
3	||
4	   accept f;
5	} 

</PRE>
</slide>



@+ <A NAME="SEC13" HREF="sC++_toc.html#SEC13">Friend Functions</A>

<P>
   Friend functions of active classes are treated in practically the same way 
as those of passive classes. When a call is made to a friend function, there 
is no synchronization (line 14). On the other hand, if a method <EM>n()</EM>, 
declared <EM>public</EM> is called from <EM>f()</EM>, then the call to <EM>n()</EM>
(line 11) is synchronized. Finally, if the method <EM>m()</EM>, declared private,
is called from <EM>f()</EM> (line 10), this call is <STRONG>not synchronized</STRONG>.

</P>

<slide id=sc-friends cap="Friends">

<PRE>
 1	active class B; 
 2	active class A { 
 3	   void m() {  } 
 4	public: 
 5	   void n() {  } 
 6	   friend B; 
 7	   friend f(B&#38;); 
 8	}; 
 9	void f (B&#38;b) { 
10	   b.m(); 
11	   b.n(); 
12	} 
13	B bb; 	
14	f (bb);

</PRE>
</slide>

<P>
  In the case of classes declared as friends (line 6 above), there is a
synchronization (lines 7 and 8 below) when the friend class is entered.

</P>
<P>
  On the other hand, as before, at the time of the call to a method of the
class that contains the friend declaration, the presence of a synchronization
depends upon the protection of this method. The call to <EM>m()</EM> on line 3
below is not synchronizing but the call to <EM>n()</EM> in line 4 below is.

</P>

</slide id=sc-fr-2 cap="Friends (2)">

<PRE>

1	active class B { 
2	public: 
3	   void f() { m(); } 
4	   void g() { n(); } 
5	}; 

6	B bb; 

7	bb.f(); 
8	bb.g(); 

</PRE>
</slide>



@+ <A NAME="SEC14" HREF="sC++_toc.html#SEC14">Functional model</A>

<P>
It can be shown that the behaviour of an active object can be modelled with a
finite state machine. In this section modelling is done using a system of 
guards.

</P>
<P>
Firstly we remark that all the <EM>selects</EM> can be replaced by <EM>selects</EM>
that contain an <EM>accept</EM> for each method of the object, but with false
guards for the <EM>accepts</EM> which are not present in the replaced
<EM>select</EM>.

</P>
<P>
As in the following diagram the following <EM>select</EM> can be replaced by a
<EM>select</EM> with 3 cases with the third having a false guard.

</P>

<slide id=sc-model cap="Model">

<PRE>

1	void m1 () {  ... }; 
2	void m2 () {  ... }; 
3	void m3 () {  ... }; 
4	select {
5	   when (guard1) 
6	      accept m1; 
7	||
8	   when (guard2) 
9	      accept m2; 
10	} 

</PRE>
</slide>

<P>
The accepted methods can therefore be defined by means of a set of symbols, 
each of which correspond to the guard of a method. Only the symbols  of the 
methods for which the guard is true appear in the group. The body could thus 
be considered as manipulating a group of guards, which includes the guards of 
the accepted methods and removing  those which are not.
The following refers to a set of guards.

</P>
<P>
An object without a body therefore has a set which contains all the guards.
That set is temporarily empty during the execution of the methods.

</P>
<P>
  This model allows the clear explanation of the functioning of the inheritance
of the acceptance conditions of the classes <EM>X</EM>, <EM>Y</EM> and <EM>Z</EM>
previously described. Each class has its own set of guards. For <EM>f()</EM> to
be executable it must be accepted in each class. According to our model one
could say therefore that for <EM>f()</EM> to be executable, the guard of
<EM>f()</EM> must be included in each set. Naturally, if the method is not
redefined in a class one must consider that the guard is always a part of the
set at that level, since it is accepted by default.

</P>
<P>
<EM>When behaviour is inherited from an active class, the group of guards is 
inherited. The contents of this group vary over time.</EM>

</P>
<P>
The executable methods can be determined at each instant in calculating the 
intersection of the groups of guards. It is therefore clear that if one of 
these classes decides to refuse the call to a method, the others cannot oppose
this. It is not possible to release the constraints on these guards, but only
to augment them, which is reasonable. In effect, if a class has been defined,
taking into account situations in which a given method cannot be accepted, this
cannot be changed from outside.

</P>


@+ <A NAME="SEC15" HREF="sC++_toc.html#SEC15">Creation of sub-states in an inherited automaton</A>

<P>
The mechanism described earlier permits the addition of more sub-states in a
state of an inherited class, but not the change in the order of the method
calls. In the diagram below, an automaton with 3 states and a sub automaton
derived from the latter are shown.

</P>

<slide id=sc-subs cap="Sub-state">

<PRE>
a  -&#62;  b -&#62;  c  -
^                \ 
|                / 
----------------

(a)  -&#62;  b  --&#62;  x  --&#62;  c  -&#62;  z  -
            \       /               \ 
 ^            -&#62; y -                / 
 |                                 / 
  --------------------------------
</PRE>
</slide>

<P>
The methods corresponding to the transitions <EM>a</EM>, <EM>b</EM> and <EM>c</EM>
are defined in an active class. Another class inherits this active class and
adds three other methods <EM>x</EM>, <EM>y</EM> and <EM>z</EM> to it. The method
<EM>c</EM> is only accepted by the inherited object when the method <EM>x</EM> or
the two methods <EM>y</EM> and <EM>z</EM> have been accepted and executed.

</P>
<P>
The two diagrams below show how these sub-states are realized.

</P>


<slide id=sc-realize cap="Realization">
<PRE>
 1	active class ActObj { 
 2	public: 
 3	   virtual void a() {printf("a\ n");} 
 4	   virtual void b() { } 
 5	   virtual void c() { } 
 6	private: 
 7	   @ActObj() { 
 8	      while (1) { 
 9	         accept a; 
10	         accept b; 
11	         accept c; 
12	      } 
13	   } 
14	};      

</PRE>
</slide>

<P>
The method <EM>a</EM> (line 3) is not redefined in the inherited class shown
below. It must therefore only be accepted from the class described above in
order to be executed.

</P>


<slide id=sc-synch cap="Synchronize">
<PRE>
 1	active class ActObjHer: public ActObj { 
 2	public: 
 3	   void b() {printf("b\ n");} 
 4	   void c() {printf("c\ n");} 
 5	   void x() {printf("x\ n");} 
 6	   void y() {printf("y\ n");} 
 7	   void z() {printf("z\ n");} 
 8	private: 
 9	   @ActObjHer() { 
10	      while (1) { 
11	         accept b; 
12	         select { 
13	            accept x; 
14	         || 
15	            accept y; 
16	         } 
17	         accept c;
18	         accept z; 
19	      } 
20	   } 
21	};    

</PRE>
</slide>



@+ <A NAME="SEC16" HREF="sC++_toc.html#SEC16">Synchronization examples</A>

<P>
  The example below taking the class <EM>X</EM> defined earlier, is a critical
case which illustrates what the compiler must do in order to resolve different
cases of synchronization. <EM>X::f()</EM> is called from inside <EM>Z::f()</EM>
(line 3). If <EM>Z::f()</EM> is called externally, there is a double
synchronization at the beginning of the execution of <EM>Z::f()</EM>,
<EM>X::f()</EM> (line 3) must not therefore provoke any new synchronization. On
the other hand, if <EM>Z::f()</EM> is called from the body of <EM>Z</EM> (line 5),
class X must accept the call in order that X::f() can be executed, since the
double synchronization has not taken place, and <EM>X</EM> has probably not yet
accepted the call.

</P>


<slide id=sc-sync-ex cap="Synchronization example">
<PRE>
1	active class Z : public X { 
2	   public: 
3	      void f() {  ... X::f();  ... } 
4	   private: 
5	      @Z() {  ... f();  ... } 
6	}

</PRE>
</slide>

<P>
sC++ knows how to tell the difference between the two calls to <EM>X::f()</EM>
and executes the synchronizations at the right times. For this it codes the
synchronization instructions executed in every case and the kernel tests if
<EM>X</EM> has already accepted <EM>X::f()</EM> before executing it. If not, it
causes wait for the acceptance.

</P>


@* <A NAME="SEC17" HREF="sC++_toc.html#SEC17">TYPES OF PARALLELISM</A>

<P>
   On a single processor computer, real parallelism cannot be performed. 
However, during the conception of a parrallel program, it is necessary to
consider that all the objects are actually executed in parallel, if these 
programs are to be able to be ported from one system to another without
problems.
Noted here are several ways to realize parallelism (or pseudo-parallelism) on
a single processor. This can done by switching between one task and another

</P>

<UL>
<LI>during calls to the real time library and only at these cases.

<LI>during all events, internal (rendezvous), external (hardware interrupts),

addressing the current task or another (pre-emption).

<LI>at the end of a given time slice.

</UL>

<P>
   The first version is evidently the simplest to realize and is often used
when a group of coroutines are available or an environment capable of
executing several applications simultaneously (PC, UNIX, Macintosh).

</P>
<P>
   The second version of parralelism is similar to the first if the tasks have
no relative priority. On the other hand if they do have priorities, external
events waited for by tasks with higher priority than the current task provoke
a switch from the current task to the one waiting for the event. Since this can
happen at any time, it is necessary to design the programs as if each task had
it's own processor.

</P>
<P>
   The third version is similar to the second if the end of a time slice is 
considered as an external event that causes a switch even if the current task 
and the activated task have the same priority. This version is used in the 
case where several tasks of the same priority have to share the processor time
equally. This is the case in multi-user systems. In a one-user system which
executes an editor and a compilation, we are interested in giving priority to 
the editor and not using time slices. The time slicing system is only really 
useful in a multi-user system.

</P>
<P>
   The sC++ language does not specify the type of real time system 
it uses. This depends upon the implementations. A correct program should be 
able to run on either of the three types.

</P>
<P>
   The sC++ libraries offer several execution kernels: real time or
accelerated time, with records of executed rendezvous and random execution.
These kernels are described below.

</P>


@+ <A NAME="SEC18" HREF="sC++_toc.html#SEC18">Real or accelerated time</A>

<P>
  This kernel is the one used normally. It is possible (line 1 below) to put
the kernel into accelerated mode and to put it back to normal mode (line 2).

</P>


<slide id=sc-time cap="Time">
<PRE>
1	SetVirtualTimeMode (); 
2	SetRealTimeMode ();

</PRE>
</slide>

<P>
In virtual mode, the kernel advances the time to the date of the active object
with the smallest timeout possible from which there are no other active 
objects ready to be executed. This mode allows the easy creation of event 
simulators and to integrate parts of real programs into them (protocols, 
controllers etc). The time therefore jumps each time there is nothing to do, 
to the next timeout. It therefore advances faster than in real time.

</P>


@+ <A NAME="SEC19" HREF="sC++_toc.html#SEC19">Debugging kernel</A>

<P>
Note : The kernel do not come with the compiler but exists as an separate
application.

</P>
<P>
   This kernel sends information regarding the progress of the program under 
test to a display and supervision program, through a communication medium, e.g 
TPC/IP. This kernel slows down the execution a little but not so much as if 
displays were made on a screen.

</P>
<P>
   This kernel can be used in a real execution with <EM>breakpoints</EM> added
to survey memory zones and display the list of executed methods. The
surveilance of memory zones allows the determination of which object has
modified data, which is not always possible without such a tool.

</P>
<P>
   It is equally possible to trigger the time counter by putting
<EM>breakpoints</EM> into shared applications to avoid untimely <EM>timeouts</EM>.

</P>


@+ <A NAME="SEC20" HREF="sC++_toc.html#SEC20">Random Execution</A>

<P>
Note: The kernel do not come with the compiler but exists as an separate application.

</P>
<P>
   The third kernel allows testing of programs executing synchronizations, in
a random order. This causes a <EM>random walk</EM> of several active objects.
  
   In the normal case, when the list of synchronization offers defined by a 
<EM>select</EM> is transmitted to the kernel, the kernel determines the first 
rendezvous in the list that can be performed, if there is one, it puts the
object which accepted the rendezvous into the list of objects ready to be
executed and continues the execution of the object that made the call in the
chosen rendezvous.

</P>
<P>
   In the case of a random walk execution, the kernel stores all the
synchronization offers transmitted by a <EM>select</EM> into a global list. Then
it takes a possible rendezvous at random and executes the two objects
concerned. Each one is executed until it's next <EM>select</EM> from which the
synchronization offers are again put into the global list.

</P>
<P>
   If there are any delays (<EM>waituntil</EM>), they are equally put into the
global list and taken at random like with the rendezvous. The value of time is
no longer used.

</P>
<P>
   It is possible to give certain transition probabilities to transitions to
avoid a transition which is present everywhere, being chosen too often.

</P>
<P>
   An assert function defined within the kernel allows 
interuption of the program if it is called with the value <EM>false</EM>. The
kernel memorises the  number of time that each assertion has been executed, so
that the programmer can estimate if all the assertions have been executed at
least once. It could happen that one of the assertions was never actually
executed.

</P>
<P>
   The developper could therefore insert this function at the points where 
errors could be found. The kernel interrupts the execution at this point and
then the trace that led to the error can be analyzed.

</P>
<P>
   This kernel allows the testing ofinteractive programs, such as protocols 
and industrial control processes, in which execution depends on the arrival of
external events. By replacing the event sources by active objects that 
permanently offer these events with an adequate probability, the kernel sends 
them to the program in an equally indifferent order. By letting the program 
run for a certain amount of time, a large number of possible execution 
sequences can therefore be tested. Error cases will be indicated by means of 
the assertions.

</P>
<P>
   As an example, the problem of readers and writers can be tested by creating
a certain number of readers and writers permanently making requests to enter 
the critical sections. Assertions marking the presence of simultaneous reads 
and writes are placed between the demand of entry and exit of the critical 
section, in the readers and writers. The random walk kernel therefore takes 
the entrance demands in a random order. It will stop execution as soon as a 
reader or writer violates the conditions of access to the critical section. 
If after a "certain" time, none of the assertions are emptied, the program can
be considered to be reliable enough. Evidently it is still necessary that all 
the assertions have been executed. 

</P>
<P>
  This test is not exhaustive, but is easy to do. Moreover, a complete 
analysis often demand unfeasable amounts of calculation time. Finally the
studies led by Colin West have shown that a number of errors can be detected
using this method.

</P>


@* <A NAME="SEC21" HREF="sC++_toc.html#SEC21">COMPILER RESTRICTIONS</A>

<P>
The implementation of the sC++ language/compiler is not yet finished. You need
to keep in mind the following points when writing an sC++ application:

</P>

<slide id=sc-cc cap="Compiler">
<UL>

<LI>As the first include file in each sC++ source file you should include

the file 'scxx.h' (include it like you include a system header file). The file
contains definitions from the sC++ kernel (used by the compiler). The compiler
will automatically find it when you compile your program.

<LI>When compiling with gcc you need to link with the sC++ kernel library

'libsc.a' on the command line (specify '-lsc'). You may also need to specify
some supplementary system libraries. On SUN Sparc stations you need to specify
the libraries 'libgen.a'(-lgen), 'libnsl.a'(-lnsl), and 'libsocket.a'
(-lsocket) after the sC++ kernel library. On DEC Alpha you don't need to link
with any supplementary libraries.

</UL>
</slide>

<P>
The following sC++ language constructions is currently producing buggy code
when done on active objects:

</P>

<slide id=sc-bugs cap=Bugs">
@ Bugs
<UL>

<LI>Calling of a method in a base class that has been overidden by another

method defintion/implementation (method calls like this:
object-&#62;baseClass::method()).

<LI>Methods from a base class that has been inherited privately (or

protected) but made public (or protected) due to an explicit visibility
adaptation for each method.

<LI>Calling methods belonging to active objects using pointers to methods.

The compiler do not yet produce code for the synchronisation.

</UL>
</slide>

<P>
<STRONG>Warning</STRONG>: A sC++ specific warning/error may by emitted for the first
statement in a block (usually in a constructor), to avoid this warning put 
a ';' (semi-colon) in front of the first statement in the block. It's due to a
bug in the parser (compiler).

</P>
</BODY>
</HTML>
