<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>

<head>
<meta http-equiv="Content-Type"
content="text/html; charset=iso-8859-1">
<meta name="GENERATOR" content="Microsoft FrontPage 2.0">
<title>Relation to Java</title>
</head>

<body bgcolor="#FFFFFF">

<h1>Relation of sC++ to Java</h1>

<h3>Active object creation</h3>

<p>The main features that distinguishes JAVA from C++ are the possibility of declaring active 
objects, the simplification of the pointers and the use of an interpreted code.
<P>
sC++ has active objects that are very similar to JAVA's, but a simpler communication mechanism. This mechanism is prone to modelling and thus to analysis by the theories and tools that have been developed by logicians (CCS, temporal logic, Buchi automaton...).
<P>
sC++ is not interpreted and retains the same pointers as C++. It can directly call the libraries 
developed for C++.
<P>sC++ active objects are created in the following way:</p>

<pre>active class A {
public:
     void m () { ...... }
     A ()  { ........ }          //  constructor
private:
     @A () { ... accept m; ...}  //  body (runs in parallel with the  
};                               //        bodies of the other objects)</pre>

<pre>A aa, bb[4];</pre>

<p>The line above automatically creates 5 active objects that run
concurrently. It is not possible to create passive objects from a
class that has been declared active.</p>

<h3>Inter-object synchronization</h3>

<p>In sC++, each method that is declared public in an active object is automatically
synchronizing. There is no <em>synchronised</em> keyword. (In
order to call a method without synchronization, one can use the
keyword friend). The method call is thus the means used to communicate
between active objects. It has exactly the same syntax as the
usual call. </p>

<p>Only one method can be executing within an object at any one
time, and only when the body is inactive (suspended or
terminated). </p>

<p>The suspension of the body is made by the statement <em>accept
m; </em>which works like the accept statement of Ada. The method <em>m</em>
is executed when both the caller and the body are suspended on
the call, respectively the accept statement:</p>

<table border="0">
    <tr>
        <td><pre><font size"3">B::@B () { 
       ......	
     accept m; 
       ......	
}	</font></pre>
        </td>
        <td><div align="center"><center><pre>                <font
size="3">
</font>                              
  &lt;------------<font size="3">

   rendezvous</font></pre>
        </center></div></td>
        <td><pre>C::@C () {
<font size="3">      ......	
     bb-&gt;m(); 
      ......	
}	</font></pre>
        </td>
    </tr>
</table>

<h3>Select statement</h3>

<p>An object can enable several rendezvous simultaneously with
the select statement. Calls, as well as accept statements can be
written in a select statement.</p>

<pre>D::@D () { 
    ... 
   select {
      accept m; 
       ...
   ||
      obj-&gt;meth();
       ...
   ||
      waituntil(T);
       ...
   }
    ...
}; </pre>

<p>Any number of calls, accept and delays can be written in a
select statement. The first event that meets its complement is
fired and the other ones are forgotten. After the method
execution, the code that lies under the triggered event is
executed and the select is exited.</p>

<h3>Advantages over JAVA</h3>

<p>In Java, a method can only be suspended inside the object. The
caller is thus blocked if the awaited event is not readily
available. In sC++, on the contrary, the caller is blocked just
before it enters the methods. It can thus handle the other events
of the select if they occur beforehand.</p>

<p>The identity of the method that is accepted is defined by the
accepting object, which avoids rescheduling all methods, and
providing them with the knowledge of what is expected for the
termination of the method.</p>

<p>The semantic of sC++ corresponds exactly to CCS, the calculus
of communicating systems, which allows all kinds of modelling,
verifications, tests and specifications (See sC++ documentation).</p>

<p>sC++ provides an alternative to the widely used
concept designated with the names of <em>callback</em>, <em>inversion
control</em> or <em>event driven</em> programming, and allows a
much more structured programming style. Around sC++, we have
created development and debugging tools, and libraries such as
CORBA that makes it very easy and quick to build interactive or
distributed applications</p>

<p>Many examples have shown us that the concept used by sC++
makes the design, specification, and implementation of programs
very easy.</p>

<p>Note that the active objects of JAVA could be
easily completed with the interprocess communications means
described herewith.</p>
</body>
</html>
