<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">
<html>

<head>
<meta http-equiv="Content-Type"
content="text/html; charset=iso-8859-1">
<meta name="GENERATOR" content="Microsoft FrontPage 2.0">
<title>Postgraduate Course 97-98</title>
</head>

<body bgcolor="#FFFFFF" link="#0000FF" vlink="#7F007F">

@* <a href=@talk-sc-se.html target=_top>Communication Software Engineering with sC++</a>

<p><strong>Keywords: <br>
active objects, concurrency, program modeling, program analysis,
random walking, deadlocks, CORBA</strong></p>

<slide id=sc-se cap="SE">
<blockquote>
    <ul>
        <li>Active objects</li>
        <li>The application in local mode</li>
        <li>Debugging the application (error + solution)</li>
        <li>The application in distributed mode (error +
            solution)</li>
        <li>CORBAlization of the application<br>
            <font size="3">Definition of the IDL interface<br>
            </font>Code segments extracted from the file produced
            by the IDL compiler</li>
        <li>Graphical architecture editor</li>
        <li>Model checking</li>
        <li>Conclusion</li>
    </ul>
</blockquote>
</slide>

<p><font size="3">This exercise shows how to analyze a
distributed application based on active objects with the aim of
detecting potential deadlocks.</font> </p>

<p>The application we consider is distributed. It allows a group
of users to see, on their own display, a shared drawing that they
all edit simultaneously. In order to assure that all commands are
processed in the same order, the system has a central monitor
that collects the commands produced by the users and broadcasts
them back to. Even the commands that come from the local user
pass through the monitor before being applied locally. </p>

<p>In this exercise, we will only analyze the structure of the
application without any internal details because the deadlock
only depends on the ordering of the actions, and not on the
details of the variables. </p>

<p>We will present successively five steps: the first one has a
deadlock in the local mode already, the second one fixes the
problem, the third one simulates a remote method call to reach
the monitor, which again creates the conditions of a potential
deadlock, the fourth one solves this problem and finally the last
one is implemented on top of our own CORBA library. </p>

<h3><b>Active Objects</b> </h3>

<p>This briefe description of active objects should be sufficient
to understand the following sections. An active object has a
special method, the body, defined much like a
constructor/destructor, that is running in parallel with the
bodies of the other objects. A body can call the methods of the
other objects, exactly like passive objects. However, an active
object has the capability to postpone the execution of the calls
of its methods: A call cannot be executed as long as the body of
the called object does not accept the call. In order to avoid
blocking (see below), a select statement allows the parallel
waiting of calls to other objects and acceptations of calls from
the other bodies. Active objects can be compared to processes,
but with method calls as inter-process communication means.</p>

<h3 align="left">The Application in Local Mode</h3>

<p>The first version of the application depicted on the figure
below simulates an application running locally. Its
communications are made by method calls. All objects are active.
Three users get drawing elements from objects <i>Graph</i>, which
get themselves their information from mouse clicks. The drawing
commands are transmitted to the monitor which broadcasts it to
all users, including the initiator of the request. Each user
accepts the calls from the monitor in parallel with the waiting
of elements from the graphical module, in order to be ready to
serve a broadcast if it occurs before the user produces the next
drawing element. </p>

<p>Note: We store the <em>User</em> code within a single object.
The problem analysed below would not appear if the commands
coming back from the monitor would be handled by a second object.
However, this is not always possible. The actions received may
influence the subsequent actions that are to be sent. The problem
as addressed here is thus more general.</p>

<p align="center">&nbsp; <img src="local/Image14.gif" width="451"
height="183"></p>

<p>This system is specified by the following classes: </p>

<blockquote>
    <blockquote>
        <pre><font size="3"><strong>active class User</strong> {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // <b>user definition
</b>&nbsp; int figure;
public:
&nbsp; User() { figure=0 ; }&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// initialization constructor
&nbsp; </font><font color="#FF0000" size="3"><strong>void broadcastCommand(int n) { </strong></font><font
size="3">// entry point offered by the
&nbsp;&nbsp;</font><font color="#FF0000" size="3"><strong>&nbsp; printf (&quot;%d\n&quot;,n);</strong></font><font
size="3">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // user to receive its broadcast
&nbsp;</font><font color="#FF0000" size="3"><strong> }</strong></font><font
size="3">
private:
&nbsp; @User ();&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // body = object's thread
};</font></pre>
        <pre><font size="3"><strong>active class CentralMonitor</strong> {&nbsp;&nbsp;&nbsp; // <b>server definition
</b>&nbsp; int m;
public:
&nbsp; </font><font color="#FF0000" size="3"><strong>void accessMonitor (int m1) {</strong></font><font
size="3">&nbsp; // entry point where the monitor
&nbsp; </font><font color="#FF0000" size="3"><strong>  m = m1;</strong></font><font
size="3">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // awaits the user calls
  </font><font color="#FF0000" size="3"><strong>}</strong></font><font
size="3">
private:
&nbsp; @CentralMonitor ();
};</font></pre>
        <pre><font size="3">CentralMonitor * S;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // the pointers to the objects
User * C1;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // must be placed here because
User * C2;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // they are referenced
User * C3;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // in the bodies
Graph * graphEnv;</font></pre>
        <pre><font size="3">User::@User () {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // <b>user implementation
</b>&nbsp; for (;;) {
&nbsp;&nbsp;&nbsp; <strong>select {</strong>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // the user awaits a local
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <strong>graphEnv-&gt;get(figure);</strong>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // command in parallel with
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <strong>S-&gt;accessMonitor (figure);</strong>
&nbsp;&nbsp;&nbsp; <strong>||</strong>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; <strong>accept broadcastCommand;&nbsp;&nbsp;&nbsp;</strong> // the reception of its own
&nbsp;&nbsp;&nbsp; <strong>}</strong>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // part of the broadcast
&nbsp; }
}</font></pre>
        <pre><font size="3">CentralMonitor::@CentralMonitor(){// <b>monitor implementation
</b>&nbsp; for (;;) {
&nbsp;&nbsp;&nbsp; <strong>accept accessMonitor;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </strong>// the monitor accepts a command<strong>
&nbsp;&nbsp;&nbsp; C1-&gt;broadcastCommand(m);&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </strong>// and sends it to all users<strong>
&nbsp;&nbsp;&nbsp; C2-&gt;broadcastCommand(m);
&nbsp;&nbsp;&nbsp; C3-&gt;broadcastCommand(m);</strong>
&nbsp; }
}</font></pre>
        <pre><font size="3">void main() {
  graphEnv = new Graph;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;  &nbsp; // instanciation of the active
&nbsp; S = new CentralMonitor;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // objects
&nbsp; C1 = new User;
&nbsp; C2 = new User;
&nbsp; C3 = new User;
&nbsp; awaitObjectTermination();
}</font></pre>
    </blockquote>
</blockquote>

<h3>Debugging the Application</h3>

<p><font size="3">This program can be compiled with our sC++
compiler, an extension of the gcc of GNU. It can then be started
under the control of a task level debugger. When the program
above arrives in its deadlock state, the debugger shows the
following picture.</font> </p>

<p><img src="local/monitor1.gif" width="671" height="465"></p>

<p>The monitor can execute the application in normal mode, in
accelerated mode (when the ready queue is empty, time jumps to
the date of the next active object in the timer queue), or in
random mode (the rendezvous and the time transitions are chosen
randomly, independently of the values of the timers).</p>

<p>When the developer clicks on an active object in the window
that lists them, the monitor displays the position where it is
suspended. Not shown here is a possibility to display a history
diagram that displays all past events. Thus it is easy to
understand why the program cannot progress anymore. The <strong>deadlock</strong>
situation is repeated below.</p>

<table border="0">
    <tr>
        <td><pre><font size="3">// <strong>user implementation</strong></font></pre>
        <pre><font size="3">User::@User () {&nbsp;
&nbsp; for (;;) {
&nbsp;&nbsp;&nbsp; select {&nbsp;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; graphEnv-&gt;get(figure);&nbsp;
</font><font color="#FF0000" size="3"><strong>--&gt;</strong></font><font
size="3">&nbsp;&nbsp; S-&gt;accessMonitor (figure);
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;&nbsp;&nbsp;
&nbsp;&nbsp;&nbsp; }
&nbsp; }
}</font></pre>
        </td>
        <td><pre><font size="3">  // <b>monitor implementation</b></font></pre>
        <pre><font size="3">  CentralMonitor::@CentralMonitor(){
    for (;;) {
&nbsp;&nbsp;&nbsp;   accept accessMonitor;
 </font><font color="#FF0000" size="3"><strong>--&gt;</strong></font><font
size="3">&nbsp; C1-&gt;broadcastCommand(m);
&nbsp;&nbsp;&nbsp;   C2-&gt;broadcastCommand(m);
&nbsp;&nbsp;&nbsp;   C3-&gt;broadcastCommand(m);
&nbsp;   }
  }

</font></pre>
        </td>
    </tr>
</table>

<h4><b>Solution</b> </h4>

<p>The following code avoids the deadlock. It uses labels and
gotos, because they are the statements closest to states and next
state jumps. Only the user code is modified to have it
corresponding exactly to the automaton below. </p>

<p align="center"><img src="local/Image12.gif" width="180" height="224"></p>

<blockquote>
    <pre><font size="3">User::@User () {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // <b>user implementation
</b>&nbsp; </font><font color="#FF0000" size="3"><strong>state0</strong></font><font
size="3">:
&nbsp;&nbsp;&nbsp; select {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // the user awaits a local
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; graphEnv-&gt;get(figure);&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // command in parallel with
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </font><font color="#FF0000"
size="3"><strong>goto state1</strong></font><font size="3">;
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;&nbsp;&nbsp;&nbsp; // the reception of its own
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </font><font color="#FF0000"
size="3"><strong>goto state0</strong></font><font size="3">;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // part of the broadcast
&nbsp;&nbsp;&nbsp; }
&nbsp; </font><font color="#FF0000" size="3"><strong>state1</strong></font><font
size="3">:
&nbsp;&nbsp;&nbsp; select {
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; S-&gt;accessMonitor (figure);
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </font><font color="#FF0000"
size="3"><strong>goto state0</strong></font><font size="3">;
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;&nbsp;&nbsp;&nbsp; // the reception of its own
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </font><font color="#FF0000"
size="3"><strong>goto state1</strong></font><font size="3">:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // part of the broadcast
&nbsp;&nbsp;&nbsp; }
&nbsp; }
}</font></pre>
</blockquote>

<p>The above code has been shown to be deadlock free.</p>

<h3><b>The Application in Distributed Mode</b></h3>

<p>We now define the simulation of a distributed version of the
version above, and verify if there are potential deadlocks in
this new version. An object simulating the RPC (remote procedure
calls) can be instantiated from the following class: </p>

<blockquote>
    <blockquote>
        <pre><font size="3"><strong>active class RPC</strong> {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // This class represents
&nbsp; int m;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // the whole channel between
public:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // the user and the monitor,
&nbsp; </font><font color="#FF0000" size="3"><strong>void accessMonitor (int m){</strong></font><font
size="3">&nbsp;&nbsp;&nbsp;&nbsp; // i.e., the stub, the skeleton 
&nbsp;&nbsp;&nbsp; </font><font color="#FF0000" size="3"><strong>S-&gt;accessMonitor(m);</strong>&nbsp;</font><font
size="3">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // and the TCP/IP channels
&nbsp; </font><font color="#FF0000" size="3"><strong>}</strong></font><font
size="3">
};&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;     // Although the object has no body, it accepts
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;     // the call of <i>accessMonitor</i> sequentially
                    // (atomic execution)</font></pre>
    </blockquote>
</blockquote>

<p>The user calls then the RPC object, which in turn calls the
monitor. We should implement the same channel in the reverse
direction, but with one direction already, the program contains a
deadlock state. This situation can again be detected with the
monitor. The deadlock state is shown below. The user is stuck in
the middle of its RPC, which cannot terminate before the monitor
accepts it. But the latter does not accepts it, because it is
actually calling.</p>

<table border="0">
    <tr>
        <td><pre><font size="3">  // <strong>RPC emulation</strong></font></pre>
        <pre><font size="3">  active class RPC {
  &nbsp; int m;
  public:
&nbsp;   void accessMonitor (int m){   
</font><font color="#FF0000" size="3"><strong>--&gt;</strong></font><font
size="3">&nbsp;&nbsp; S-&gt;accessMonitor(m);
&nbsp;   }
  };&nbsp;

</font></pre>
        </td>
        <td><pre><font size="3">  // <b>monitor implementation</b></font></pre>
        <pre><font size="3">  CentralMonitor::@CentralMonitor(){
    for (;;) {
&nbsp;&nbsp;&nbsp;   accept accessMonitor;
 </font><font color="#FF0000" size="3"><strong>--&gt;</strong></font><font
size="3">&nbsp; C1-&gt;broadcastCommand(m);
&nbsp;&nbsp;&nbsp;   C2-&gt;broadcastCommand(m);
&nbsp;&nbsp;&nbsp;   C3-&gt;broadcastCommand(m);
&nbsp;   }
  }
</font></pre>
        </td>
    </tr>
</table>

<h4>Solution</h4>

<p>The solution relies on a deferred remote call. The invocation
of the call is separated from the waiting of the return of its
parameters. The broadcast can thus be awaited in parallel with
the production of the next graphical element, the call invocation
(<em>post_accessMonitor</em>) and the parameter return (<em>ready_accessMonitor</em>)
respectively.</p>

<p>The RPC that emulates the deferred invocation is depicted
below with its code.</p>

<p align="center"><font size="2"><img src="local/Image18.gif"
width="610" height="135"></font></p>

<blockquote>
    <pre><font size="3">// <strong>RPC emulation</strong></font></pre>
    <pre><font size="3">  active class RPC {
  &nbsp; int m;
  public:
&nbsp;   void post_accessMonitor (int m1){ m = m1; }
    void ready_accessMonitor (int m1){ }</font></pre>
    <pre><font size="3">    @RPC() {                          // the body supports the RPC
      for (;;)                        // protocol in parallel with 
        <strong>accept post_accessMonitor;</strong>    // the client's execution<strong>
</strong></font><font color="#FF0000" size="3"><strong>     </strong></font><font
size="3"><strong>&nbsp;&nbsp; S-&gt;accessMonitor(m);
        accept ready_accessMonitor;</strong>
      }
&nbsp;   }
  };&nbsp;</font></pre>
</blockquote>

<p>Note that the server receives exactly the same call as in the
previous situation. The new user code is written below. In this
solution, the broadcast can occur at any time.</p>

<blockquote>
    <blockquote>
        <pre><font size="3">User::@User () {&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; // <b>user implementation
</b>&nbsp; </font><font color="#FF0000" size="3"><strong>state0</strong></font><font
size="3">:
&nbsp;&nbsp;&nbsp; select {
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; graphEnv-&gt;get(figure);
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state1;
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state0;
&nbsp;&nbsp;&nbsp; }
&nbsp; </font><font color="#FF0000" size="3"><strong>state1</strong></font><font
size="3">:
&nbsp;&nbsp;&nbsp; select {
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; S-&gt;post_accessMonitor (figure);
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state2;
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state1:
&nbsp;&nbsp;&nbsp; }
&nbsp; }
&nbsp; </font><font color="#FF0000" size="3"><strong>state2</strong></font><font
size="3">:
&nbsp;&nbsp;&nbsp; select {
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; S-&gt;ready_accessMonitor (figure);
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state0;
&nbsp;&nbsp;&nbsp; ||
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; accept broadcastCommand;
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; goto state2:
&nbsp;&nbsp;&nbsp; }
&nbsp; }
} </font></pre>
    </blockquote>
</blockquote>

<p><font size="4"><b>CORBAlization of the Application</b></font></p>

<p>The application can easily be completed to run in distributed
mode. The sources contained in this <a
href="http://ltiwww.epfl.ch/~petitp/webMonitor/CORBA/">directory</a>
have been generated by our IDL compiler. They contain all what
must be added to the previous program to make it distributed.
However, all stubs are generated in the same file, the skeletons
in another file, etc., which is not what we want, but the problem
is handled in the following. (<a
href="http://diwww.epfl.ch/w3lti/index.html#Publications">Restrepo
A.J., Petitpierre C. - CORBA Implementation: Active Objects
versus Callbacks</a>)</p>

<p><font size="3">The structure of a complete program is shown
below. The user and the central monitor have the same code as
before, but the RPC emulation must now be replaced by the real
CORBA calls.</font></p>

<p align="center"><img src="local/Image8.gif" width="632" height="243"></p>

<p align="center"><font size="3"><strong>Definition of the Corba
interfaces</strong></font><strong> </strong></p>

<h4><font size="3">Definition of the IDL interface</font></h4>

<p><font size="3">The IDL description of the interfaces of the
services that can be called in the monitor and in the user, as
well as the structure that is transmitted during the calls are
defined below. Actually both the monitor and the user have client
and server functioning. We will thus insert all interface classes
used on both side in the same interface, but instantiate on each
side only the needed interfaces. The <i>post_</i> and <i>ready_</i>
versions of <em>accessMonitor</em> are generated automatically.
Note that only the simple version appears on the server side.</font>
</p>

<p><font size="3">IDL description:</font> </p>

<blockquote>
    <pre><font size="3">1 struct remCommand {</font><font
size="4"> 
</font><font size="3">2   long type ;</font><font size="4"> 
</font><font size="3">3   double v1X, v1Y, v2X, v2Y, r;
4 };</font><font size="4"> </font></pre>
    <pre><font size="3">5 interface CentralMonitor {</font><font
size="4"> 
</font><font size="3">6   boolean <strong>connect</strong> (in string userIOR);</font><font
size="4"> 
</font><font size="3">6   void <strong>accessMonitor</strong> (in remCommand comm);
7 };</font><font size="4"> </font></pre>
    <pre><font size="3">8 interface User {</font><font size="4"> 
</font><font size="3">9   void <strong>broadcastCommand</strong> (in remCommand comm);</font><font
size="4"> 
</font><font size="3">10 } ;</font><font size="4"> </font></pre>
</blockquote>

<p><font size="3">Remember that both the user and the monitor are
client and server (in the CORBA sense). The central monitor is
registered in the name server, but the users pass their own
object id (IOR) to the central monitor by means of the <i>connect</i>
RPC, so that it can broadcast the data back to them.</font> </p>

<p><font size="3">The IDL compiler generates the following files
from the definition above, assuming that it is contained in a
file named <i>editorDistr.idl</i>. These files contain the stub
and skeleton classes as well as all the commands that instantiate
and initialize the various modules of the CORBA library.</font></p>

<table border="1">
    <tr>
        <td align="center" width="120"><font size="3">editorDistr.idl</font></td>
        <td><blockquote>
            <p><font size="3"><br>
            editorDistr_CORBA.h</font>&nbsp; <br>
            <font size="3">editorDistrclt_Stub.scxx</font>&nbsp; <br>
            <font size="3">editorDistrclt_Clt_Main.scxx<br>
            editorDistr_Skeleton.scxx</font>&nbsp; <br>
            <font size="3">editorDistr_Impl.h</font>&nbsp; <br>
            <font size="3">editorDistr_Impl.scxx</font>&nbsp; <br>
            <font size="3">editorDistr_Impl_Main.scxx<br>
            </font></p>
        </blockquote>
        </td>
        <td>interfaces and services<br>
        classes defining the stubs<br>
        initialization and calls of the stubs<br>
        classes defining the skeletons<br>
        frames of the object implementations<br>
        <br>
        initialization and calls of the implementations and the
        skeletons</td>
    </tr>
</table>

<p><font size="3">The figure below shows the structure of the
user side of the application.</font></p>

<p align="right">&nbsp; <a name="_964962014"></a><font size="2"><img
src="Image16.gif" width="595" height="340"></font><font size="3"><strong>Structure
of the User</strong></font><strong> </strong></p>

<h4 align="left"><strong>Code segments extracted from the file
produced by the IDL compiler</strong></h4>

<p align="left">The code that realizes the modules displayed on
the figure are given in the following. They represent the frame
of the program. They are extracted from the files produced by the
compiler. Not all error code is shown to avoid hiding the
structure of the application. The code obtained from the analysis
above can simply be inserted in this code to produce the complete
application.</p>

<blockquote>
    <pre><strong>editorDistr_Impl.h</strong></pre>
    <pre>active class User_i : <strong>public __BASE_User</strong>{ // skeleton inheritance
public:
  User_i(CORBA::String);
   ...   
};</pre>
    <pre><strong>editorDistr_Impl.scxx</strong></pre>
    <pre>User_i::User_i(CORBA::String _key) 
     : <font color="#FF0000"><strong>__BASE_User(_key)</strong> </font>// initialization of inherited skeleton
{   ...   }
void User_i::<font color="#FF0000"><strong>broadcastCommand</strong></font>( const remCommand&amp; comm )
{   ...   };             //  User method called by the monitor</pre>
    <pre>//  <strong>The main contain the initialization code</strong>
void <font color="#FF0000"><strong>main</strong></font>(int argc, char **argv){ 
  CORBA::ORB_ptr <font color="#FF0000"><strong>orb_ptr</strong></font>;    // pointer to the orb services
  CORBA::Environment <font color="#FF0000"><strong>env</strong></font>;    // error indications</pre>
    <pre>  // <strong>Initialize the ORB</strong>
  env.clear();
  orb_ptr = CORBA::<font color="#FF0000"><strong>ORB_init</strong></font>(argc, argv, &quot;ior&quot;, env);
  if (env.exception() != 0) error(...);  </pre>
    <pre>  // <strong>Initialize a stub pointing to the naming service</strong> (<em>name_context</em>)
  CORBA::Object_ptr obj_ptr; 
  obj_ptr = orb_ptr-&gt;<font color="#FF0000"><strong>resolve_initial_references</strong></font>((CORBA::String)&quot;NameService&quot;); 
  CosNaming::NamingContext_ptr name_context;
  name_context = CosNaming::NamingContext::<font color="#FF0000"><strong>_narrow</strong></font>(obj_ptr);</pre>
    <pre>  // <strong>Initialize a stub pointing to the monitor</strong> (<em>CentralMonitor_ptr</em>)
  CosNaming::Name name_CentralMonitor;
  name_CentralMonitor.Length(1);
  name_CentralMonitor[0].id = CORBA::string_copy((CORBA::String)&quot;<font
color="#FF0000"><strong>CentralMonitor</strong></font>&quot;);
  CORBA::Object_ptr obj_CentralMonitor_ptr;
  obj_CentralMonitor_ptr = name_context-&gt;<font color="#FF0000"><strong>resolve</strong></font>(name_CentralMonitor);
  CentralMonitor_stub_ptr CentralMonitor_ptr;
  CentralMonitor_ptr = CentralMonitor_stub::<font color="#FF0000"><strong>_narrow</strong></font>(obj_CentralMonitor_ptr);
</pre>
    <pre>  // <strong>Instantiate  User (</strong><em>my_User</em>)<strong>
</strong>  CORBA::String key_User;
  key_User = CORBA::string_copy((CORBA::String)&quot;<font
color="#FF0000"><strong>User</strong></font>&quot;);
  User_i *my_User = <font color="#FF0000"><strong>new User_i</strong></font>(key_User, 0, 0);   // user instantiation</pre>
    <pre>  // <strong>The code shows how to register an object implementation</strong> 
  //  (not needed for the user, as it registers directly in the monitor)
  CosNaming::Name name_User; name_User.Length(1);
  name_User[0].id = CORBA::string_copy(key_User); 
  CORBA::Object_ptr UserRef; UserRef = <font color="#FF0000"><strong>my_User-&gt;_this</strong></font>();
  name_context-&gt;<font color="#FF0000"><strong>bind</strong></font>(name_User, UserRef); </pre>
    <pre>  // <strong>Registration in the monitor</strong>
  CORBA::String myIOR = orb_ptr-&gt;<font color="#FF0000"><strong>object_to_string</strong></font> (UserRef, env);
  int result = CentralMonitor_ptr-&gt;<font color="#FF0000"><strong>connect</strong></font>( myIOR );  // remote call</pre>
    <pre>  StopMain();
  exit(0); };
}</pre>
</blockquote>

<h3>Graphical Architecture Editor</h3>

<p>The IDL compiler has no provision to generate files according
to the location of the client and server interfaces. As this
application has client and server interfaces in all components,
the software developer must pick the various code elements in
several files. The code relating to the client interfaces are
located in the file <em>XXX_Clt_Main.scxx</em>. The interface
code related to the server is located in <em>XXX_Impl.h</em>, <em>XXX_Impl.scxx</em>
and <em>XXX_Impl_Main.scxx</em>. We have thus developed an
architecture editor that saves the structure of the application
and can thus generate the frames of all components of the
application.</p>

<p>A snapshot of a window of the architecture editor is shown
below (click on the window to see it in real size).</p>

<p><a
href="http://ltiwww.epfl.ch/~petitp/webMonitor/editwind.html"><img
src="editWindow.gif" border="0" width="341" height="247"></a></p>

<p>&nbsp;</p>

<h3>Model Checking</h3>

<p>The last part of our environment is not terminated. It will
allow the modeling and the analysis of an application written in
a subset of sC++ (sufficient to describe the program written in
the first three design steps of the current presentation). The
analyses that we will be capable of performing are the following
ones:</p>

<ul>
    <li>exhaustive reachability analysis</li>
    <li>deadlock detection</li>
    <li>checking properties described by expressions written in
        temporal logic</li>
    <li>state reduction</li>
    <li>verification of the equivalence multi-threaded
        applications</li>
</ul>

<h3>Conclusion</h3>

<p>The whole approach described above supports all libraries
supported by C++, as we use the very same compiler. The use of
TCP sockets or GUI interfaces is greatly simplified thanks to our
interfaces based on active objects. On the other hand, we have as
much support for analysis as competitive environments such as
Promela/SPIN or SDT.</p>

<p>This situation is not dependent on sC++, and it may also
happen in a program based on an event loop. For example, the
program may execute a blocking remote invocation, which leaves
the main stack busy until the remote function returns. If an
external call is accepted at this moment and needs a resource
already obtained by the program before the remote invocation, the
system deadlocks. </p>

<p>However, in event loop programs, it is not possible to
automatically identify deadlocks, as there are many ways to
enable/disable the acceptance of an event: rebinding an event to
another function, disable the event source (mask a window
button), modify the dispatching conditions... Of course, if
deadlocks are difficult to identify automatically, they are also
very difficult to identify by the programmer. </p>

<p>Threads can be added to systems based on event loops, but it
is a very difficult task, as several experiences we have heard of
has shown. The problem is that the two concepts are opposite. It
can even be considered that an event loop program is an inversion
of a multi-thread program (<a
href="http://diwww.epfl.ch/w3lti/index.html#Publications">Petitpierre
C. An Event-Driven Programming Paradigm Compatible with
OO-Programming</a>)</p>
</body>
</html>
