/* This is the output produced when running this program:

Show's when you need Coplien's Canonical form:
        Base default ctor called
        Base::m called
        Base copy ctor called
        Base default ctor called
        Base::m called
        Base assignment operator called
        Base dtor called
        Base dtor called
        Base dtor called
No late binding in base constructors: 
        Base default ctor called
        Base::m called
        Subclass ctor called
But we do have late binding in other members: 
        Subclass::m called
Note the error in destructor calls...
... Subclass dtor is not called!
This is why you need virtual dtors!
        Base dtor called

Make sure you undersntd this, otherwise you will easily 
get in trouble during your OOP assignment.

*/

#include <string.h>
#include <iostream.h>

class Base {
  public:
    Base();                    // default ctor
    Base(const char* s);       // normal ctor
    Base(const Base&);         // copy ctor
    ~Base();           	      //  ERROR: should be virtual dtor
    Base& operator=(const Base&); // assignment 
    virtual void m();
    virtual void m2();
  private:
   char* s;
};

Base::Base() { 
	cout << "\tBase default ctor called" << endl;
	s = new char[12345]; 
	strcpy(s, "");  

	// The following will always call Base::m(), even when m() is redefined
	// by a sub class. Calling m() in a normal (non-ctor) member
	// will result in a call to the sub class' member (if any)
	// because of the late bidning concept.
	m(); 
}
Base::Base(const char* _s) { 
	cout << "\tBase ctor with argument called" << endl;
	s = new char[12345]; 
	strcpy(s, _s);  
}
Base::Base(const Base& b) { 
	cout << "\tBase copy ctor called" << endl;
	s = new char[12345]; 
	strcpy(s, b.s);
}

Base::~Base() { 
	cout << "\tBase dtor called" << endl;
	delete[] s; 
}

Base& Base::operator=(const Base& b) { 
	cout << "\tBase assignment operator called" << endl;
	strcpy(s, b.s); return *this; 
}

void Base::m() { cout << "\tBase::m called" << endl; }
void Base::m2() { m(); }

class Subclass: public Base {
  public:
    Subclass()  { cout << "\tSubclass ctor called" << endl; };
    ~Subclass() { cout << "\tSubclass dtor called" << endl; };
    virtual void m() { cout << "\tSubclass::m called" << endl; }
};

// Show's when you need Coplien's Canonical form:
void f(Base b1) { 		// needs copy ctor to copy argument
  Base b2;			// needs default ctor 
  b2 = b1;			// needs assignment
  /* ...
     ... */
} 				// dtor called to destroy b1(!) and b2

void canonical () {
  cout << "Show's when you need Coplien's Canonical form:" << endl;
  Base b1;			// needs default ctor
  f(b1);			// needs copy ctor to copy argument

  /* ...
     ... */

} 				// dtor called to destroy b1

void no_late_binding_in_ctor() {
  cout << "No late binding in base constructors: " << endl;
  Base* s = new Subclass;
  cout << "But we do have late binding in other members: " << endl;
  s->m2();
  cout << "Note the error in destructor calls..." << endl;
  cout << "... Subclass dtor is not called!" << endl;
  cout << "This is why you need virtual dtors!" << endl;
  delete s;
}

int main() {
   canonical();
   no_late_binding_in_ctor();
}

