  <document>
  <h1>OOP Style guide</h1>

  (under construction)
  <include file=navigate>
  <h3>Design Specification</h3>
  
  <p>
   A minimal article contains an abstract, introduction, body, conclusion
   and reference list.
   
  <p>
   Identify the most important classes of your application and their
   responsibilities, describe the evolution of your design, and the pros
   and cons of the current design (E.g.  "The original implementation
   contained a single object controlling the entire application. We
   changed the design and distributed the functionality of the control
   object over the objects X, Y, and Z. The main reason for this was that
   the control object needed knowledge about every single class in our
   application, which was a maintenance nightmare.  Additional advantages
   include ... However, the disadvantage is ..."  or "We moved
   functionality from classes X, Y, and Z to a common base class. We
   defined this base class as an abstract base class (it has pure virtual
   member function f1 and f2) because ..." or "We used the MVC paradigm
   (or Document/View) to separate the GUI from the rest of the application", 
   etc)
   
  <h3><a name=implementation>Implementation</a></h3> 
  
  <p>
   
  <h4>Header files and class definitions</h4>
  
  <p>It is a good custom to create a header file for each class
   you define. Try to move as much implementation as possible
   from the header file into the separate
   corresponding implementation file.
   
  <p>Try to keep your header files as small as possible.
   Do not include other files in your header unless:
  <ul>
   <li>The class has a base class. Include the file defining the base class.
   <li>The class has a data member which is not a pointer or a build-in type.
    Include the file defining the class of the data member.
   <li>The class has a function member which needs a parameter
    which is not a pointer or a build-in type 
    (note: this may represent an error!). Include the
    file defining the class of the parameter.
  </ul>
  <p>Use forward declarations for all other class names used in the header,
   and include the corresponding class definitions in the implementation
   file.
   This style avoids "recursive include" errors and unnecessary re-compilation
   in combination with makedepend. 
   Example:
   
  <pre>
   // file example.h:
   #ifndef	_EXAMPLE_H_
   #define	_EXAMPLE_H_
   
   #include &lt;widgets/canvas.h&gt;	// base class
   #include &lt;list.h&gt;		// data member
   
   class button;			// forward declarations
   class widget;			// ... included in example.cc
   
   class example: public canvas {
     public:
       example(widget* w, ...);
       virtual ~example();
       ...
     private:
       button* Cancel;
       list&lt;int&gt;	score;
   };
   
   #endif
  </pre>
  
  <h4>Constructors, destructors, operators and other member functions</h4>
  <ul>
   <li>Virtual constructors do not exist in C++. Make sure you know
       in what case you'd like to have a ``virtual constructor'' and
      how to get the desired effect.
   <li>Make all destructors virtual unless you really know what your doing.
   <li>Be sure you know what <a href="FAQ/FAQ.html">Coplien's 
     Orthodox Canonical Class Form</a> is.
   <li>Make sure you know how to use the ``const'' keyword.
   <li>Try to avoid function names for which equivalent operator names
    are available. E.g. use counter::operator++() instead of
    counter::incr(), use mylist::operator==() instead of mylist::equals() etc.
   <li>Do not repeat the class name in your member functions name (e.g.
    do not use list::sort_list())
  </ul>
  </document>
  
