<document>
@* Paradigms of programming
\label{paradigms}

<include file=navigate>

<c>
<slide color=silver mode=hide id=1.2-sub>
:@ Subsections:
<ul>
<li> <a href=paradigms-procedure.html>Procedural programming</a>
<li> <a href=paradigms-data.html>Data abstraction</a>
<li> <a href=paradigms-object.html>Object-oriented programming</a>
</ul>
</slide>
<hr>
</c>

<ignore>
Since the introduction of programmable devices, our conception of
programming has undergone radical changes.
New insights with respect to programming have been reflected in a succession of
high level programming languages, such as Algol-60, Algol-68, Simula,
Pascal, C, Modula-2, Modula-3, CLU, Ada, Ada9X and numerous others,
the latest of which is, as you will probably know, Java.

<p>
These programming languages were all introduced to support
some specific style of programming.
All of them have in some way set an example of a {\em good} programming style.
This is exactly what we mean by the phrase {\em paradigm of programming},
a style of programming that may somehow be considered mandatory
for developing (a particular category of) applications.
As an example, consider the use of Pascal as a vehicle for teaching
structured programming, or the use of Modula-2 as a vehicle for 
teaching the employment of abstract data types.
</ignore>

In a landmark paper with the title <em>`What is object-oriented programming?' </em>
Bjarne Stroustrup raises the question of
when a language may be considered to
support a particular style of programming, [_St88].
\see{1-styles}

\vstr{2}
%%\vfig{1.5}
<slide id=1-styles caption="Styles of programming">
<h4>Object-oriented programming</h4>
*- {\em high tech synonym for good }
\footnotesize
@ Styles of programming
<I
<li> A language {\em supports} a style of programming
if it provides facilities that make it convenient (easy, safe and efficient)
to use that style
I>
*- compile/runtime checks
*- clean interpretation/ orthogonal / efficient / minimal
</slide>


In general, one can say that a language supports a particular style 
of programming if it provides facilities, both syntactic and semantic,
that makes it convenient (that is easy, safe and efficient) to use that style.
The crucial distinction that must be made
in this context is that between allowing a certain style
and providing support for that style.
Allowing means that it is possible to program in that style.
To support a given style, however, requires in addition that suitable 
compile and runtime checks are provided to enforce a proper
use of the relevant language constructs.
With these considerations in mind, one could question the assertion
that {\em Ada is object-oriented} or that {\em Modula supports abstract data types}.
Naturally, this attitude backfires with C++.
Does C++ support abstract data types
and is it really object-oriented?



<slide id=1-paradigms caption="Paradigms of programming">
@ Procedural programming
*- procedures, use the optimal algorithms
@ Modules
*- hide the data, provide functional abstractions
@ Data abstraction
*- types, provide a sufficiently complete set of operations
:@ Object-oriented <i> -- organize your types</i>
*- make commonality explicit <c>by using inheritance</c>
<!--\ifsli{\vspace{-1cm}}{}-->
</slide>


It is equally important to establish whether a language allows a clean
interpretation of the constructs introduced, whether the constructs supporting
object orientation are {\em orthogonal} to (that is independent of)
the other constructs of the language, whether an {\em efficient} implementation
of these constructs is possible, and whether the language is kept {\em minimal},
that is without superfluous constructs.


Before establishing 
%%in more detail 
what the main ingredients of
object-orientation are, let us briefly look at some of the styles of programming
that may be considered as leading to an object-oriented style.
\see{1-paradigms}


In his article, Stroustrup (1988) 
stresses the continuity between the respective styles
of programming pictured in \sliref{1-paradigms}.
Each style is captured by a short phrase stating its principal concern,
that is guidelines for developing {\em good} programs.

<so paradigms-procedure>
<so paradigms-data>
<so paradigms-object>

</document>
