<document>
@+ <a href=@talk-soft-design.html target=_top>Design</a>
<include file=navigate>

In an object-oriented approach,
the distinction between {\em analysis}
and {\em design}
is primarily one of emphasis;
emphasis on modeling the reality of
the problem domain versus
emphasis on providing an architectural
model of a system that lends itself
to implementation.

One of the attractive features of such
an approach is the opportunity of a seamless
transition between the respective phases
of the software product in development.
The classical waterfall model can no
longer be considered as appropriate
for such an approach. 
An alternative model, the
{\em fountain model}, is proposed by [_Hend92].
This model allows for a more autonomous
development of software components,
within the constraints of a unifying framework.
The end goal of such a development process
may be viewed as a repository of reusable components.
A similar viewpoint has originally been
proposed by [_Cox86] and [_Meyer88].


<slide color=red id=1-design caption="Object-oriented design">
:@ O<rm>bject</rm>-O<rm>riented</rm> D<rm>esign</rm>
<ul>
<li> design for maintenance and reuse!
</ul>
:@ Software quality
<ul>
<li> correctness, robustness, extensibility, compatibility
</ul>
:@ Design projects
<ul>
<li> IDA -- <a href=ida.html>Interior Design Assistant</a>
<li> MASS -- <a href=mass.html>Multi-user Agenda Support System</a>
</ul>
</slide>


In examining the primary goals of design,
[_Meyer88] distinguishes between {\em reusability},
{\em quality} and {\em ease of maintenance}.
Naturally, reusable software presupposes quality,
hence both quality and maintainability
are important design goals.
See \sliref{1-design}.
In [_Meyer88] a rough estimate is given of the shift in effort
between the phases of the software life-cycle,
brought about by an object-oriented approach.
Essentially, these figures show an increase
in the effort needed for design.
This is an immediate consequence
of the observation that the development
of reusable code is intrinsically more difficult.
<p>
To my mind, there is yet another reason for the extra
effort involved in design.
In practice it appears to be difficult and time
consuming to arrive at the appropriate
abstract data types for a given application.
The implementation of these structures,
on the other hand, is usually straightforward.
This is another indication that the unit of
reuse should perhaps not be small pieces of code, but
rather (the design of) components that fit into
a larger framework.

%%@ Software quality

<ignore>
\see{1-quality}


<slide id=1-quality caption="Software quality">
%%@ Software quality
*- correctness, robustness, extendibility, reusability, compatibility
%%:@ Maintenance <em> -- 70\% of costs</em>
[P
Requirements change --  41.8\% 
Data format change --  17.4\% 
Emergency fixes --  12.4\% 
Routine debugging --  9\% 
Hardware changes --  6.2\%
Documentation --  5.5\% 
Efficiency improvements -- 4\%
P]
\ifsli{\vspace{-1cm}}{} 
</slide>
</ignore>

From the perspective of software quality and maintenance,
these mechanisms of {\em encapsulation}
and {\em inheritance} may be characterized as powerful means
to control the complexity of the code needed to realize a system.
In [_Meyer88] it is estimated that maintenance accounts for
<math>70 \% </math>
of the actual cost of software.
Moreover, {\em adaptive maintenance},
which is the adaptation to changing requirements,
accounts for a disproportionately large part
of the cost.
Of primary importance for maintenance,
in the sense of the correction of errors,
is the {\em principle of locality}
supported by encapsulation, data abstraction and hiding.
In contrast, inheritance is a feature that may
interfere with maintenance, since it often breaks
down the protection offered by encapsulation.
However, to cope with changing requirements,
inheritance provides both a convenient and relatively
safe mechanism.


:@- Design assignments
Actually designing systems is a complex activity,
about which a lot can be said.
Nevertheless, to get a good feeling for what is involved
in designing a system it is best to gain some experience first.
In the remainder of this subsection, you will find the
descriptions of actual software engineering assignments.
The assignments have been given, in subsequent years, to groups consisting
of four or five  CS2 students.
The groups had to accomplish the assignments in five weeks,
a total of 1000 man-hours.
That includes formulating the requirements, writing the design specification
and  coding the implementation.
(For the first of the assignments, IDA, C++ was used with the 
<em>hush</em> GUI library.
For the second, MASS, Java with Swing was used.) 
In both cases we allowed for an iterative development cycle,
inspired by a Rapid Application Development (RAD) approach. 
These assignments will be taken as a running example,
in the sense that most examples presented in the book
solve in one way or another the problems that may occur
when realizing the systems described in the assignments.

<so ida mass>
</document>
