<document>
@* <a href=@talk-soft.html target=_top>The object-oriented software life-cycle</a>
\label{life}

<include file=navigate>

<c>
<slide color=silver mode=hide id=1.3-sub>
:@ Subsections:
<ul>
<li> <a href=soft-analysis.html>Analysis</a>
<li> <a href=soft-design.html>Design</a>
<li> <a href=soft-implement.html>Implementation</a>
</ul>
</slide>
<hr>
</c>



No approach to software development is likely to survive unless it solves
some of the real problems encountered in software engineering practice.
In this section we will examine how the object-oriented approach is 
related to the conceptions of the life-cycle of software
and what factors may motivate
the adoption of an object-oriented approach to
software development.


Despite some variations in terminology,
there is a generally agreed-on
conception of the various phases in the development of a software product.
Roughly, a distinction can be made between a phase of {\em analysis},
which aims at specifying the requirements a product must meet,
a phase of {\em design}, which must result in a conceptual view of the 
architecture of the intended system, and a phase of {\em implementation},
covering coding, testing and, to some extent, also maintenance
activities. 
\see{1-lifecycle}

No such consensus exists with respect to the exact relation between
these phases.
More specifically, there is a considerable variation in methods and guidelines
describing how to make the transition from one phase to another.
Another important issue is to determine what the products are exactly,
in terms of software and documentation, that must result from each phase.

\vstr{3}
<slide color=blue id=1-lifecycle caption="The software life-cycle">
:@ The software life-cycle
<ul>
<li> Analysis -- Conceptual Model, System Requirements  
<li> Design -- System Design, Detailed Design 
<li> Implementation -- Coding, Testing
</ul>
<p>
<em><p> With an increase in the number of software products
not satisfying user needs, prototyping
has become quite popular!</em>
</slide>

The traditional conception of the software life-cycle is
known as the {\em waterfall model}, which prescribes a strictly sequential transition between
the successive phases, possibly in an iterative manner.
Strict regulations with respect to validation of the products
resulting from each phase may be imposed to avoid the risk of backtracking.
Such a rigid approach, however, may cause severe problems,
since it does not easily allow for modifying decisions taken earlier.

One important problem in this respect is that the needs of the users of a system
may change over time, invalidating the requirements laid down in an earlier
phase.
To some extent this problem may be avoided by better techniques
of evoking the user requirements in the analysis phase,
for instance by developing a prototype.
Unfortunately, the problem of accommodating changing user needs
and adapting to changing circumstances (such as hardware)
seems to be of a more persistent nature,
which provides good reason to look at alternative software development models.

:@ Software development models
The software engineering literature abounds
with descriptions of failing software projects and remedies proposed
to solve the problem of software not meeting user expectations.

User expectations may be succinctly characterized by the RAMP
requirements listed in
\sliref{1-requirements}.
Reliability, adaptability, maintainability 
and performance are not unreasonable demands in themselves.
However, opinions on how to satisfy these criteria
clearly diverge.

\vfig{0.5}
<slide color=red id=1-requirements caption="Requirements -- RAMP">
@ Requirements <em> -- user needs <c>are constantly evolving</c></em>
<ul>
<li> {\bf R}eliability -- <em> incremental development, reuse, synthesis</em>
<li> {\bf A}daptability -- <em> evolutionary prototyping</em>
<li> {\bf M}aintainability -- <em> incremental development, synthesis</em>
<li> {\bf P}erformance -- <em> incremental development, reuse</em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>
\vstr{1}


[_Bersoff91] and  [_Davis88] explain
how the choice of a particular
software development model may influence the 
chances of successfully completing a software project.
As already mentioned, {\em rapid throwaway prototyping}
may help to evoke user needs at an early stage,
but does not help much in adapting to 
evolving user requirements.
A better solution in this respect is to adopt a 
method of {\em evolutionary prototyping}.
Dependent on the technology used, however, 
this may cause severe problems in
maintaining the integrity and robustness 
of the system.
Less flexible but more reliable is an approach of
{\em incremental development},
which proceeds by realizing those
parts of a system for which the user requirements
can be clearly specified.


Another means of adapting to changing user
requirements is to use a technique of
{\em automated software synthesis}.
However, such an approach works only if
the user requirements can be formalized easily.
This is not always very likely, unless the
application domain is sufficiently restricted.
A similar constraint adheres to the {\em reuse of software}.
Only in familiar application domains is it 
possible to anticipate how user requirements
may change and how to adapt the system appropriately.
Nevertheless, the reuse of software seems
a very promising technique with which to reduce the cost and time
involved in software products
without (in principle) sacrificing reliability
and performance.
\see{1-development}

<slide id=1-development caption="Software development models">
@ Software development models

<I
- rapid throwaway prototyping -- {\em quick and dirty}
- incremental development -- {\em slowly evolving}
- evolutionary prototyping -- {\em evolving requirements}
- reusable software -- {\em reduces cost and time}
- automated software synthesis -- {\em one level of abstraction higher}
I>
\ifsli{\vspace{-1cm}}{}
</slide>


Two of the early advocates of
object-oriented technology, Cox and Meyer,
regard the reuse of software as the ultimate
solution to the software crisis.
However, the true solution is in my
opinion not so straightforward.
One problem is that tools and technologies
are needed to store and retrieve reusable
components.
That simple solutions do not suffice is illustrated
by an anecdote reported by Alan Kay
telling how difficult it was to find his way
in the Smalltalk class structure after
a significant change, despite the browsing
facilities offered by the Smalltalk system.

Another problem lies in the area of human factors.
The incentives for programmer productivity have too long
been directed at the number of lines of code
to make software reuse attractive.
This attitude is also encouraged in universities.
Moreover, the reuse of other students' work is
usually (not unjustifiably) punished instead of encouraged.

However, having a sufficiently
large store of reusable software at our disposal
will allow us to build software
meeting the RAMP requirements stated above,
only if we have arrived at sufficiently
stable abstractions of the application domain.

In the following, we will explore how object-oriented
technology is motivated by problems
occurring in the respective phases of the
software life-cycle and how it contributes
to solving these problems.


<so soft-analysis>
<so soft-design>
<so soft-implement>
</document>
