<document>
@+ Dynamic role-switching

<include file=navigate>

For many applications, static type hierarchies
do not provide the flexibility needed
to model dynamically changing roles.
For example we may wish to consider a person
as an actor capable of various roles during his lifetime,
some of which may even coexist concurrently.
The characteristic feature of the
<i>dynamic role switching</i> idiom underlying the
<i>actor</i> pattern is that it
allows us to regard a particular entity from multiple
perspectives
and to see that the behavior of that entity changes accordingly.
We will look at a possible realization of the idiom below.
<p>
Taking our view of a person as an actor
as a starting point, 
we need first to establish the repertoire of possible behavior.

<slide type=hide cap="actor.java">
@ds local/lib/java/actor/actor.java
</slide>

Apart from the repertoire of possible behavior,
which consists of the ability to <i>walk</i>, <i>talk</i>,
<i>think</i> and <i>act</i>,
an actor has the ability to establish its own
identity (<i>self</i>) and to check whether it exists
as an actor, which is true only if it has become another self.
However, an actor is not able to assume a different role
or to become another self. We need a <i>person</i> for that!
<p>
Next, we may wish to refine the behavior of an actor
for certain roles, such as for example the <i>student</i>
and <i>employer</i> roles, which are among the many roles
a person can play.

<slide type=hide cap="Students and Employers">
@ds local/lib/java/actor/student.java
@ds local/lib/java/actor/employer.java
</slide>

Only a <i>person</i> has the ability
to assume a different role or to assume a different identity.
Apart from becoming a <i>Student</i> or <i>Employer</i>,
a person may for example become an <i>adult_person</i>
and in that capacity again assume a variety of roles.

<slide type=hide cap="person.java">
@ds local/lib/java/actor/person.java
</slide>

A person may check whether he exists as a <i>Person</i>,
that is whether the <i>Person</i> role differs from the person's
own identity.
A person's <i>self</i> may be characterized as the actor
belonging to the role the person is playing,
taking a possible change of identity into account.

When a person is created, his repertoire is still empty.
Only when a person changes identity by becoming
a different actor (or person) or by assuming
one of his (fixed) roles, is he capable of displaying
actual behavior.
<p>
Assuming or `becoming' a role results in creating
a role instance if none exists and setting the <i>_role</i>
instance variable to that particular role.
When a person's identity has been changed,
assuming a role affects the actor that
replaced the person's original identity.
(However, only a person can change roles!)
<p>
The ability to become an actor allows us to model
the various phases of a person's lifetime
by different classes, as illustrated by
the <i>adult</i> class.

<slide type=hide cap="adult.java">
@ds local/lib/java/actor/adult.java
</slide>

In the example code below we have a person
talking while assuming different roles.
Note that the person's identity may be restored
by letting the person become its original self.

<slide type=hide cap="go.java">
@ds local/lib/java/actor/go.java
</slide>

The <i>dynamic role switching</i> idiom
can be used in any situation where we wish
to change the functionality of
an object dynamically.
It may for example be used to incorporate
a variety of tools in a drawing editor,
as illustrated in chapter 4.
</document>
