<document>
@* <a href=@talk-idioms.html target=_top>Idioms in <em>hush</em></a>
\label{hush-idioms}

<include file=navigate>

<control>
<slide color=silver mode=hide id=2.2-sub>
:@ Subsections
<ul>
<li> <a href=hush-handle.html>The <em>handle/body</em> idiom</a>
<li> <a href=hush-self.html>Virtual self-reference</a>
<li> <a href=hush-role.html>Dynamic role switching</a>
<li> <a href=hush-patterns.html>The art of <em>hush</em> programming</a>
</ul>
</slide>
<hr>
</control>



The <i>hush</i> framework, developed by the author and his colleagues, 
aims at providing an easy-to-use
and flexible, multi-paradigm environment
for developing distributed hypermedia and web-based applications.
Actually <em>hush</em>, which stands for 
<em>hyper utility shell</em>, is a part of the DejaVU framework
which has been developed at the Free University in Amsterdam over
the last five years.
The DejaVU framework is meant as an umbrella for our
research in object-oriented applications and architectures.
Many of the examples in this book are in some way derived from
<em>hush</em> or applications developed within the DejaVU project.
The <em>hush</em> library was originally developed in C++,
but parts of it have been ported to Java using the Java native
runtime interface. You will see examples of <em>hush</em>
in chapters <ref file=@online/oo/I/4/4>4</ref>,
<ref file=@online/oo/II/6/6>6</ref>,
<ref file=@online/oo/II/7/7>7</ref>,
<ref file=@online/oo/IV/11/11>11</ref> and
<ref file=@online/oo/IV/12/12>12</ref>.
<p>
In this section a brief overview will be given of the basic
concepts underlying <em>hush</em>.
Then we will discuss the idioms used in realizing <em>hush</em> and
its extensions, in particular an adapted version of
the <em>handle/body</em> idiom originally introduced in [_Coplien92],
the <em>virtual self-reference</em> idioms and the
<em>dynamic role switching</em> idiom.
At the end of this section we will discuss the implications
these idioms have for developing <em>hush</em> applications.
Readers not interested in <em>hush</em> may safely skip the
introduction that follows and the discussion at the end of this section.

<p>
The <em>hush</em> framework is object-oriented in that it allows
for a component-wise approach to developing applications.
Yet, in addition to object class interfaces, it
offers the opportunity to employ a script language, such as Tcl and Prolog,
to develop applications and prototypes.
The <em>hush</em> framework is a multi-paradigm framework,
not only by supporting a
multi-lingual approach, but also by providing support
for distributed client/server solutions in a transparent (read CORBA) manner.
<p>
In this section we will look at 
the idioms employed
for the realization of the framework.
In developing <em>hush</em>
we observed that there 
is a tension between
defining a clean object model and providing the flexibility
needed to support a multiparadigm approach.
We resolved this tension by choosing to differentiate
between the object model (that is class interfaces)
offered to the average user of the framework and
the object model offered to advanced users
and system-level developers.
<p>
In this approach, idioms play a central role.
We achieved the desired flexibility by
systematically employing a limited number of basic idioms.
We succeeded in hiding these idioms from the average
user of the framework.
However, the simplicity of our original object model
is only apparent.
Advanced or system-level developers who intend to define
extensions to the framework must be well aware
of the patterns underlying the basic concepts,
that is the functionality requirements of the classes involved,
and the idioms employed in realizing these requirements.

:@+ The <em>hush</em> framework -- basic concepts
Application development
generally encompasses a variety of programming tasks,
including system-level software development
(for example for networking or multi-media functionality),
programming the user interface
(including the definition of screen layout
and the responsivity of the interface widgets to user
actions),
and the definition of (high-level) application-specific
functionality.
Each of these kinds of tasks may require
a different approach and possibly
a different application programming language.
For example, the development of the user interface
is often more conveniently done
using a scripting language,
to avoid the waiting times involved
in compiling and linking.
Similarly, defining knowledge-level
application-specific functionality may
benefit from the use of a declarative
or logic programming language.
<p>
In developing <em>hush</em>, we decided from the start to
support a multiparadigm approach to software
development and consequently we had to
define the mutual interaction between the
various language paradigms, as for example 
the interaction between C++ and a scripting language,
such as Tcl.
Current scripting languages, including Python and Tcl,
provide facilities for being embedded in C and C++,
but extending these languages with functionality defined
in C or C++ and employing the language
from within C/C++ is rather cumbersome.
The <i>hush</i> library offers a uniform
interface to a number of script languages
and,
in addition, it
offers a variety of widgets and multimedia
extensions, which are accessible through
any of the script interpreters as well
as the C++ interface.

%%<h4>Basic concepts</h4>
These concepts are embodied in (pseudo) abstract
classes that are realized by employing 
idioms
extending the <i>handle/body</i>
idiom, as explained later on.

<slide cap="Basic <em>hush</em> classes">
:@ Basic <em>hush</em> classes
<ul>
<li> <i>session</i> -- to manage (parts of) the application
<li> <i>kit</i> -- to provide access to the underlying system and interpreter
<li> <i>handler</i> -- to bind C++ functionality to events
<li> <i>event</i> -- stores information concerning user actions or system events
 
 
<li> <i>widget</i> -- to display information on a screen
<li> <i>item</i> -- represents an element of a widget
</ul>
</slide>

Programming a <i>hush</i> application requires the
definition of an application class
derived from <i>session</i> to initialize
the application and start the (window environment)
main loop.
In addition, one may bind Java or C+= <i>handler</i> objects
to script commands by invoking the <i>kit::bind</i>
function.
Handler objects are to be considered
an object realization of callback functions,
with the advantage that client data may be
accessed in a type-secure way
(that is either by resources stored when creating the
handler object or by information that is passed via events).
When invoked, a handler object receives a pointer to
an <i>event</i>
(that is, either an X event or an event related to
the evaluation of a script command).
Both the <i>widget</i> and (graphical) <i>item</i> class are
derived from <i>handler</i> to
allow for declaring widgets and items to be their own handler.
<p>
<h4>Embedding script interpreters</h4>
The <i>hush</i> framework offers a generic <i>kit</i>
that may be used as the interface to any embedded
interpreter.
The public interface of the <i>kit</i> class looks
as follows:
<slide mode=hide cap="kit interface">
<program>
  interface kit { /// <box>kit</box>

	void eval(string cmd);
	string result();

	void bind(string name, handler h);
   };
</program>
</slide>
The function <i>eval</i> is used for evaluating
(script) commands, and <i>result</i> may be used to
communicate data back.
The limitation of this approach, obviously,
is that it is purely string based.
In practice, however, this proves to be flexible
and sufficiently powerful.
The <i>bind</i> function may be used to
define new commands and associate it with
functionality defined in <i>handler</i> objects,
which are introduced below.

<h4>Handler objects</h4>
The problem of extending the script language
with functionality defined by the application,
is (as already indicated above) addressed by defining
a generic <i>handler</i> object class.
Handler objects may be regarded as
a generalization of callback functions,
in the sense that they are activated whenever the
corresponding script command is evaluated.
The advantage of using objects for
callbacks instead of functions, obviously,
is that we no longer need type-insecure casts,
or static or global variables
to pass information around.
The public interface of the <i>handler</i> class looks as follows:
<slide mode=hide cap="interface handler">
<program>
   interface handler { /// <box>handler</box>
	int dispatch( event e );   ///// to dispatch events
	int operator();
   };
</program>
</slide>
The <i>dispatch</i> function is called by the underlying system.
The <i>dispatch</i> function receives a pointer to
an event which encodes the information
relevant for that particular callback.
In its turn <i>dispatch</i> calls the {P operator() P} function.
Classes derived from <i>handler</i> need only redefine
the {P operator() P} function.
Information needed when activating a handler
object must be provided when creating the object,
or obtained from the event for which the handler is activated.
<p>
The use of handler objects is closely connected
to the paradigm of event-driven computation.
An <i>event</i>, conceptually speaking,
is an entity that is characterized by two significant
moments,
the moment of its <i>creation</i> and the
moment of its <i>activation</i>, its occurrence.
Naturally, an event may be activated multiple times
and even record a history of its activation,
but the basic principle underlying the use of events
is that all the information that is needed is
stored at creation time and, subsequently,
activation may proceed blindly. See section <a href=reactor.html><ref>reactor</ref></a>.

<h4>User actions</h4>
Another use of handler objects (in <i>hush</i>)
is for defining what must be done
in response to user events, resulting from
actions such as moving the mouse, or pressing
a button, or selecting an entry from a menu.
This is illustrated by the public interface of
the generic <i>widget</i> class:
<slide id=hush-widget mode=hide> 
<display>
   interface widget : handler { /// <box>widget</box>
	...
	void bind( handler h );
	void bind( string action, handler h );
	...
   };
</display>
</slide>
The first member function <i>bind</i> may be 
used for installing a handler for the default bindings
of the widget,
whereas the second <i>bind</i> function is to be used
for overriding any specific bindings.
(Recall that the class <i>widget</i> is derived from
<i>handler</i> class to allow the widget to
be its own handler.
In this way inheritance or the delegation to
a separate handler object may be used 
to define the functionality of a widget.)
<p>
In addition to the <i>widget</i> class, the <i>hush</i> library
also provides the class <i>item</i>, representing graphical
items.
Graphical items, however, are to be placed within
a canvas widget, and
may be tagged to allow for the groupwise manipulation
of a collection of items, as for example moving
them in response to dragging the mouse pointer.

<h4>Programmer-defined events</h4>
User interface events occur in response
to actions by the user.
They are scheduled by the underlying window system,
which invokes the handler whenever it is convenient
or necessary.
When getting used to event-driven computation,
system designers and programmers may feel
the need to have events at their disposal that may
be scheduled at will, under the programmer's control.
It will come as no surprise that another use
of handler objects is to allow for programmer-defined
events.
The public interface of the class <i>event</i>
looks as follows:
<slide id=hush-event mode=hide>
<display>
   interface event : handler { /// <box>event</box>
	operator();
   };
</display>
</slide>
Actual event classes are derived from the generic class
<i>event</i>,
and a scheduler is provided to activate events
at the appropriate time.
(In effect, we provide a fully functional discrete
event simulation library,
including facilities for generating
random distributions and analysing the outcome of experiments.
Business process simulations done with this library
are discussed in Chapter 11.)
Note that there is an important difference between
programmer-defined events and system-defined events.
System-defined events are delivered to the user
by activating a handler callback.
In contrast, programmer-defined events are (directly)
activated by a scheduler. They contain, so to speak,
their own handler.

<h4>Discussion</h4>
What benefits do we derive from employing
handler objects and their derivatives?
One advantage is that we have a uniform way
to define the functionality of script commands,
callbacks to user actions
and programmer-controlled events.
Another, less apparent advantage,
is that it allows us to incorporate
a variety of functionality 
(including sound synthesis facilities,
digital video and active documents)
in a relatively straightforward fashion.
%%of which we will give an illustration later.

<so hush-handle>
<so hush-self>
<so hush-role>
<so hush-patterns>

</document>
