<document>
@+ Behavioral patterns
\label{Behavioral}

<include file=navigate>

Our final category of patterns, <em>behavioral patterns</em>,
concern the construction of algorithms and the assignment
of responsibilities to the objects that cooperate in achieving
some goal.

<slide color=yellow id=behavioral cap="Behavioral patterns">
@ Behavioral patterns
<font color=green><p align=right><box><em>cooperation</em></box></p></font>
<ul>
<font color=red><p>
<li> algorithms and the assignment of responsibilities between objects
</font>
</ul>
@ class
<ul>
<li> Template Method -- <font color=green>the skeleton of an algorithm</font>
<li> Interpreter -- <font color=green>to evaluate expressions</font>
</ul>
@ object  
<font color=red><p align=right><box><em>composition</em></box></p></font>
<ul>
<li> Mediator -- <font color=green>provides indirection</font>
<li> Chain of Responsibility -- <font color=green>connect objects to interact</font>
<li> Observer -- <font color=green>to handle dependencies</font>
</ul>
</slide>

A first distinction can be made between patterns that
involve the composition of classes (using inheritance)
and patterns that rely
on object composition.
<p>
As an example of the <em>Template Method</em> pattern,
think of a compiler class that offers methods for
scanning and the creation of a parse tree.
Each of these methods may be refined without affecting the
structure of the compilation itself.
<p>
An <em>interpreter</em> allows for evaluating
expressions, for example mathematical formula.
Expressions may be organised in a hierarchy. 
new kinds of expressions can be inserted simply by filling
in details of syntax and (semantic) evaluation.  
<p>
Object composition, which employs the <em>handle/body</em>
idiom and delegation, is employed in 
the <em>Mediator</em> pattern,
the <em>Chain of Responsibility</em> pattern
and the <em>Observer</em> pattern.
The actual task, such as for example updating
the display of information when the actual information
has changed, is delegated to a more specialized object,
to achieve a loose coupling between components.
The difference between a <em>mediator</em> and 
<em>chain of responsibility</em> is primarily the complexity
of co-ordinating the tasks.
For example, changing the format of a single image component
from one image type to another image type may be done
simply by using an image converter (<em>mediator</em>),
whereas
exporting the complete document to a particular format
such as HTML may involve delegating control to a
specialized converter that itself needs access to
the original components (<em>chain of responsibility</em>).
We will discuss the <em>Observer</em> pattern in more detail later.

<slide id=objectify cap="Encapsulating behavior">
@ Encapsulating behavior
<font color=red><p align=right><box>objectify!</box></p></font>
<ul>
<li> Command -- <font color=green>action + undo</font>
<li> Strategy -- <font color=green>choice of algorithms</font>
<li> Visitor -- <font color=green>decouple traversal and operations</font>
<li> Iterator -- <font color=green>access and traversal</font>
<li> State -- <font color=green>object state <math>-></math> behavioral change</font>
</ul>
</slide>

A common characteristic of the patterns listed in \sliref{objectify}
is that functional behavior is realized as an object.
Semantically, objects are more powerful than functions,
since objects can carry a state.
Hence, the imperative <em>objectify</em> pays off
when we need functions that must know their
invocation history. 
<p>
As an example of the <em>Command</em> pattern,
think of how you would realize insertion
and deletion commands in an interactive editor,
with undo!
Turning these commands into an object in which the
information necessary for undoing the command can be stored,
for example having a snapshot of the state stored in 
a <em>Memento</em>, it suffices to stack the actual
command objects. To undo a command, pop the stack and invoke
the undo method.
<p>
The <em>Strategy</em> pattern may be used 
to hide the details of the various layout algorithms
that are available. For example, you may use a straightforward
algorithm that formats the text line by line, or you may 
use the much more advanced formatting algorithm of \TeX,
which involves the minimalization of penalties.
These alternatives can be collected in a formatting <em>strategy</em>
hierarchy, that hides the implementation details from the client
by a common interface.
<p>
When doing the formatting, you may wish
to separate the traversal of the component tree structure
from the actual formatting operations.
This may be accomplished by employing the <em>Visitor</em>
pattern.
In general it is recommended to abstract
from the data structure and use a more abstract
way, such as an <em>Iterator</em> or <em>Visitor</em>
to access and traverse it.
<p>
The <em>State</em> pattern is similar to the <em>dynamic role switching</em> idiom
that has been discussed in section <ref file=hush-patterns>hush-patterns</ref>.
As an example, think of providing viewers for alternative document formats,
such as VRML or PDF, in your application.
Using the <em>State</em> pattern, it suffices to
have a single viewer that changes itself according to the 
format of the document viewed.

:@+ The Observer pattern
The <em>Observer</em> pattern is a variant of the
famous <em>Model-View-Control</em> (MVC) pattern,
that governed the creation of the graphical user interface
of the Smalltalk environment and many Smalltalk applications.


<slide color=green id=observer cap="Observer pattern">
@ Observer
<ul>
<li> <font color=red><p>one-to-many dependencies and notification</p></font>
</ul>
:@ Consequences
<ul>
<li> abstract coupling between subject and observer
<li> constraint propagation
<li> deals with unexpected updates
</ul>
</slide>

The basic idea is simple, to decouple information
management and the display of information.
In other words, a distinction is made between
the <em>model</em> or <em>subject</em>,
that carries the information, and the <em>views</em>
or <em>observers</em>, that present that information
in some format.
As a consequence, when a change occurs, the <em>viewers</em>
or <em>observers</em> have only to be notified to
update their presentation.
<p>
In effect, MVC or the <em>Observer</em> pattern can be regarded
as a simple method for constraint propagation. 
An advantage is that unexpected updates can be easily dealt
with.

<slide type=fig id=figure-observer cap="The Observer pattern">
<center>
<image name=@figures/patterns/observer>
</center>
</slide>

The objects involved in realizing the <em>Observer</em>
pattern are depicted in \sliref{figure-observer}.
The <em>subject</em> object must allow for <em>observers</em>
to be attached and detached. Note that <em>observers</em>
must also have a reference to the <em>subject</em>.
In particular, concrete observers must know how to obtain
information about the state of the <em>subject</em>,
to be able to update their view.
What the abstract <em>subject</em> and <em>observer</em> classes
supply are the facilities for attachment and
mechanisms for notification and updates.
%%AE-classes?
<p>
In the implementation of the <em>Observer</em> pattern
there are a number of problems and tradeoffs that must
be considered.
For example, do we allow one <em>observer</em> to
be attached to more than one <em>subject</em>?
Do we allow for alternative update semantics,
for example observer-pull instead of subject-push?
Do we provide facilities for specifying
aspects of interest, so that updates only need
to concern those aspects?
And finally, how do we guarantee mutual consistency
between subjects and observers when we do allow for
alternative update semantics?
</document>
