<document>
@+ Structural patterns

<include file=navigate>

Objects rarely live in isolation.
In \sliref{structural-patterns} a selection of the
structural patterns treated in [_GOF94] is collected.
Structural patterns indicate how classes and objects may be composed
to form larger structures.


<slide color=yellow id=structural-patterns cap="Structural patterns">
@ Structural patterns
*- object and class composition
%%<font color=red><p>are concerned with how classes and objects are composed to form larger structures
%%</font>
\vspace{-0.5cm}
<latex>
\ln\begin{center}
\begin{tabular}{ l  c  l }
<b>Pattern</b> & <b>Alias</b> & <b>Remarks</b> \\ %%\hline
Composite & part/whole & collections of components \\ %%\hline
Flyweight & handle/body & extrinsic state, many objects \\ %%\hline
Adaptor & wrapper & resolve inconsistency between interfaces \\ %%\hline
Bridge & handle/body & relate abstraction to implementation \\ %%\hline
Decorator & handle/body & to introduce additional functionality \\ %%\hline
Facade & handle/body & provides unified interface \\ %%\hline
Proxy & handle/body & to defer ... remote, virtual, protection \\ %%\hline
\end{tabular}
\end{center}
</latex>
<control>
<style type=text/css>
TH { color: red ;  background: lightgreen; }
TD { color: blue ; background: yellow; }
</style>
<script language=javascript src=row.js></script>
<table cellspacing=5 border=0>
<th>Pattern</th><th>Alias</th><th>Remarks</th>
<script language=javascript>

pattern="Composite";
alias="part/whole";
remarks="collections of components (see <a href=local/patterns/composite.gif>...</a>)";
project();

pattern="Flyweight";
alias="part/whole*";
remarks="extrinsic state, many objects";
project();

pattern="Adaptor";
alias="wrapper";
remarks="resolves inconsistencies between interfaces";
project();

pattern="Bridge";
alias="handle/body";
remarks="relate abstraction to implementation";
project();

pattern="Decorator";
alias="wrapper";
remarks="to introduce additional functionality";
project();

pattern="Facade";
alias="wrapper*";
remarks="provides unified interface";
project();

pattern="Proxy";
alias="surrogate";
remarks="to defer  ... remote, virtual, protection";
project();

</script>
</table>
</control>
</slide>

Imagine, for example, an application for interactive text processing. 
Now, the <em>Composite</em> pattern may be used
to combine text, images and also compound components, that
may itself consist of other components.
<p>
Closely related to the <em>Composite</em> pattern is 
the <em>Flyweight</em> pattern, which is needed when
the number of components grows very large.
In that case, the components themselves must for obvious reasons
carry as little information as possible.
Context or state information must then be passed as a parameter.
<p>
To give some more examples, suppose there exists a
nice library for formatting text and images, but unfortunately
with only a procedural interface.
Then the <em>Adaptor</em> pattern may be used to provide
a interface that suits you, by wrapping the original library.
<p>
The <em>Bridge</em> pattern is in some sense related to the
<em>Factory</em>. In order to work with a platform-independent
widget library, you need, as has been explained, a <em>factory</em>
to hide the creation of widgets, but you also need to
bridge a hierarchy of platform-dependent implementation 
classes to the more abstract platform-independent widget set.
<p>
When creating widgets to display text or images it may be very 
inconvenient to create a separate class, for example
when adding scrolling functionality.
The <em>Decorator</em> pattern 
allows you to insert additional functionality without subclassing.
%%See <ref file=hush-patterns>hush-patterns</ref>.
<p>
Now think of a networked application, for example to be able
to incorporate components that are delivered by a server.
The library may provide a number of networking classes
that deal with all possible communication protocols. 
To simplify access to these classes a <em>Facade</em>
may be built, hiding the complexity of the original
class interfaces.
<p>
Alternatively, remote components may be available
through a <em>proxy</em>.
The <em>Proxy</em> pattern describes how access may be regulated
by an object that acts as a surrogate for the real object.
Like <em>composites</em> and <em>decorators</em>, <em>proxies</em>
may be used for recursive composition.
However, <em>proxies</em> primarily regulate access,
whereas <em>decorators</em> add responsibilities,
and composites represent structure.

<control>
<style type=text/css>
TD { color: black ; background: lightyellow; }
</style>
</control>
</document>
