<document>
@* <a href=@talk-patterns.html target=_top>A catalogue of design patterns</a>
\label{Patterns}

<include file=navigate>
<control>
<slide color=silver mode=hide id=patterns-sections>
:@ Subsections:
<ul>
<li> <a href=patterns-creational.html>Creational patterns</a>
<li> <a href=patterns-structural.html>Structural patterns</a>
<li> <a href=patterns-behavioral.html>Behavioral patterns</a>
</ul>
</slide>
<hr>
</control>

Why patterns, you may wonder.
Why patterns and why not a method of object-oriented design
and an introduction in one or more object-oriented languages?
The answer is simple.
Patterns bookmark effective design.
They fill the gap between the almost infinite possibilities of
object-oriented programming languages and tools
and the rigor of methodical design.
As Brian Foote expressed it in [_POPL3], patterns
are the footprints of design, paving the way for future designs.
They provide a common design vocabulary and are also helpful
in documenting a framework.
And, as we will see later, patterns may also act as a target
for redesign, that is when the current design no longer offers the
desired functionality and flexibility.

<slide color=gold id=patterns-why cap="A catalogue of design patterns">
:@ A catalogue of design patterns
<ul>
<li> a common design vocabulary
<li> documentation and learning aid
<li> an adjunct to existing methods
<li> a target for redesign
</ul>
/// [_GOF94]
%%<c>
%%<p>see 
%%<a href=http://hillside.net/patterns/books/#Gamma>GOF Design Patterns</a>
%%the <a href=http://hillside.net/patterns/patterns.html>Patterns Homepage</a></c>
</slide>

The Gang of Four book, <em>Design Patterns</em> by [_GOF94],
was immediately recognized as an important
contribution to object-oriented software development.
Not only because of the actual patterns presented,
but also because of the style in which they were presented,
crisp problem-oriented descriptions of actual solutions
to real design problems, written with scientific rigor and accuracy.
As Brian Foote remarked, 
actual design became a legitimate subject of computer science
research.
<p>
The <em>pattern schema</em>, or rather a simplified version thereof,
is depicted in \sliref{pattern-schema}.
Each pattern must have a <em>name</em>, which acts as a handle
in discussions about the design. Being able to speak about
specific pattern solutions, such as a <em>factory</em>,
greatly facilitates discussions about design.

<slide id=pattern-schema cap="The pattern schema">
%%:@ The Pattern Schema
/// <box> pattern schema </box>
%%%<p align=right><font color=red>structure</font>
:@ Name -- <font color=green>handle</font> 
*- increases design <i>vocabulary</i>
:@ Problem -- <font color=green>when to apply</font>
*- explains the <i>problem</i> and the <i>conflict</i>
:@ Solution -- <font color=green>general arrangement</font>
*- design, <i>responsibilities</i>, <i>collaborations</i>
:@ Consequences -- <font color=green>tradeoffs</font>
*- to understand the costs and benefits
</slide>

Other important entries in the pattern schema are,
the <em>problem</em> indicating what the patterns is all about,
the <em>solution</em> describing the general arrangment
of the  classes and objects involved,
and the <em>consequences</em> or tradeoffs that a particular
solution entails.



The actual patterns presented in [_GOF94] are the result of
the authors' involvement in developing various GUI toolkits,
in particular Interviews, [_Interviews], and ET++, [_ET],
and applications such as, for example, interactive text and image editors.
In the course of developing a toolkit or application there
are many occasions for redesign.
Reasons why you may need to redesign are listed in
\sliref{redesign-patterns}, along with an appropriate selection
of patterns from [_GOF94].

<slide color=yellow id=redesign-patterns cap="Causes for redesign">
@ Causes for redesign
<p align=right><font color=red><box><em>design for change</em></box></font>
<ul>
<li> creating an object by specifying a class explicitly
<font color=blue>-- Abstract Factory, Factory Method, Prototype</font>
<li> dependence on specific operations
<font color=blue>-- Chain of Responsibilty, Command</font>
<li> dependence on hardware &amp; software platforms
<font color=blue>-- Abstract Factory, Bridge</font>
<li> dependence on object implementation or representation
<font color=blue>--Abstract Factory, Bridge, Memento, Proxy</font>
<li> algorithm dependence
<font color=blue>-- Iterator, Strategy, Template Method,
Visitor</font>
<li> extending functionality by subclassing
<font color=blue>-- Bridge, Composite, Decorator, Observer</font> 
<li> tight coupling 
<font color=blue>-- Abstract Factory, Bridge, Chain of Responsibilities,
Command, Facade, Mediator, Observer</font>
<li> inability to alter classes conveniently
<font color=blue>-- Adaptor, Decorator, Visitor</font>
</ul>
</slide>

Following [_GOF94], we may distinguish between
<em>creational</em> patterns that govern the construction
and management of objects,
<em>structural</em> patterns that define the static relationships
between objects,
and <em>behavioral</em> patterns that characterize the
dynamic aspects of the interaction between objects.
<p>
In this section we will look at a brief overview
of the classification as originally presented in [_GOF94].
The patterns themselves will be treated only briefly.
The reader is invited to consult the original source
and the many publications that followed: [_POPL1], [_POPL2], [_POPL3].


%%<?latex>
<so patterns-creational>
<so patterns-structural>
<so patterns-behavioral>
%%<?/latex>

<latex>\marksection{Event-driven computation}</latex>
</document>
