<document>
@+ Inheritance and delegation in Java 
\label{delegation-in-Java}

<include file=navigate>

Consider the example below,
an <em>envelope</em> class
that offers a <em>message</em> method.
In this form it is nothing but a variation on the <em>hello world</em>
example presented in the <a href=@online/oo/appendix/java/objects.html>appendix</a>.

<slide type=hide cap="Hello World">
@ds local/lib/java/naive/envelope.java
%%:@ Driver
%%@ds local/lib/java/naive/go.java
</slide>

To illustrate the idea underlying idioms and patterns in
its most simple form, we will refine the
<em>envelope</em> class into the collection of classes
depicted in \sliref{2-envelope-letter}.
<p>
We will proceed in three steps:
(1) The <em>envelope</em> class will be redesigned
so that it acts only as an interface to the <em>letter</em>
implementation class.
(2) Then we introduce a <em>factory</em> object,
that is used to create <em>envelope</em> and <em>letter</em>
instances.
(3) Finally, we refine the <em>letter</em> class into
a <em>singleton</em> class, that prevents
the creation of multiple <em>letter</em> instances.

<slide type=fig id=2-envelope-letter cap="Envelope/Letter Factory">
<image name=@figures/2/envelope>
</slide>

:@ Envelope/Letter
The <em>Envelope/Letter</em> idiom was introduced in
[_Coplien92] as a means to separate interface aspects
from implementation aspects.
Here the call to <em>message</em> is simply forwarded to
the <em>letter</em> object.

<slide type=hide cap="Envelope/Letter">
%%:@ Envelope
@ds local/lib/java/envelope/envelope.java
%%:@ Letter
@ds local/lib/java/envelope/letter.java
</slide>

Admittedly, there is no need here to
make such a distinction, but the idea speaks for itself.
As you will see, this distinction allows us to change
the implementation without modifying the <em>envelope</em>
or interface class.

:@ Factory
In the next refinement, we introduce a <em>factory</em> object,
that allows us to create <em>envelope</em> and <em>letter</em>
instances without invoking a constructor.

<slide type=hide cap="Factory">
%%:@ Factory
@ds local/lib/java/factory/factory.java
%%:@ Envelope
@ds local/lib/java/factory/envelope.java
</slide>

The <em>factory</em> object is used in the <em>envelope</em>
class to create a <em>letter</em>.
The advantage here, as will be shown shortly, is that the <em>envelope</em>
class does not need to have any information about the 
actual type of the <em>letter</em>.

:@ Singleton letter
Finally, we refine the <em>letter</em> class into a <em>singleton</em>
class. When you inspect the implementation, you will see
that only one instance of a <em>letter</em> will be created.

<slide type=hide cap="Singleton letter">
%%:@ Singleton Letter
@ds local/lib/java/singleton/singleton.java
</slide>

Note that the <em>factory</em> object must be modified
so that the static method <em>instance</em> of
<em>singleton</em> is invoked
instead of the original constructor of <em>letter</em>.

:@ Discussion
This example, however simple, demonstrates the implementation
of some of the idioms and patterns that will be discussed
in the rest of this chapter.
It shows that the basic mechanisms of inheritance and simple
delegation or forwarding are sufficient to implement
these idioms and patterns.
We have not discussed yet why we need idioms and patterns,
but this will hopefully become clear later on.
</document>
