<document>

@+ Polymorphism in C++
\label{1-poly}
\label{polymorphism}

<include file=navigate>

Polymorphism essentially characterizes the type
of a variable, function or object.
Polymorphism may be due to overloading, parametrized types or inheritance.
Polymorphism due to inheritance is often considered
as the greatest contribution of object-oriented languages.
This may be true, but the importance of generic (template) types
and overloading should not be overlooked.

<slide id=1-polymorphism caption="Polymorphic type declarations">
@ Overloading
/// <box>print</box>
<program>
extern void print(int);
extern void print(float);
</program>
<h4> Generic class <em> -- templates</em></h4>
/// <box> {P list< T > P} </box>
<program>
template< class T > class list { ... }
list<int>* alist; 
</program>
@ Polymorphism by inheritance
/// <box> shape </box> 
<program>
class shape { ... };
class circle : public shape { ... } 
shape* s = new circle;
</program>
</slide>

In \sliref{1-polymorphism} some examples are given of
declarations involving polymorphic types.
The function $print$ is separately defined for $int$ and $float$.
Also, a generic $list$ class is defined by means by employing
$templates$.
The list may be used for any kind of objects, for example integers.
Finally, a $shape$ class is defined from which a $circle$ class
is derived.
An instance of the $circle$ may be referred to by using a $shape$
pointer, because the type $shape$ encompasses $circle$ objects.


@ Inheritance and virtual functions
The power of inheritance in C++ comes from virtual
functions and dynamic binding.
Dynamic binding realizes the polymorphism inherent
in the static type structure of the inheritance 
hierarchy in a dynamic way.
Dynamic binding is illustrated 
by the example shown in \sliref{7-dispatching}.

<slide type=hide id=7-dispatching caption="Virtual functions -- dispatching">
<h4> Virtual functions <em> -- dispatching</em></h4>
<program>
class A { /// <box>$A$</box>
public:
virtual void operator()() {
	cout << "A"; 
	}
};

class B : public A { /// <box>$B$</box>
public:
virtual void operator()() {
	 cout << "B";
	 }
};
</program>
</slide>

The class $A$ defines a virtual member function
(that results in printing $A$) which is redefined
by a similar function in class $B$ (which results
in printing $B$).
As an example of using the classes defined above,
look at the following program fragment:
<slide mode=hide id=poly-cc-ex-1>
[P
A* a = new B(); (*a)(); /// produces B
P]
</slide>
In case the function would not have been defined as virtual,
the outcome of applying it to (a pointer to) a $B$ object
would have been $A$, instead of $B$.

:@ Explicit scoping
Virtual functions that are redefined in derived classes
may still access the original version defined in the base
class, as illustrated in \sliref{7-scoping}.

<slide type=hide id=7-scoping caption="Virtual functions -- scoping">
<v>
%%<h4> Scoping <em> -- explicit </em></h4>
<program>
class B : public A {  /// <box>$B'$</box>
public:
virtual void operator()() {
	cout << "B";
	cout << "B";
        A::operator()(); 
	}
};
</program>
</slide>

Scoping may be used within a member function of the
class as well as by a client (when invoking a member
function) as illustrated below.

<slide mode=hide id=poly-cc-abba>
[P
 A* a = new B(); a->A::operator()(); (*a)(); /// produces ABBA
P]
The outcome of this statement is  $ABBA$.
</slide>

Such a scoping mechanism is certainly not unique
to C++, although the notation for it is.
In Smalltalk, the expression {P super P} may be used
to access methods defined in the ancestor class,
and in Eiffel one must use a combination of
redefining and renaming to achieve this.
As a remark, I prefer the use of
{P operator()() P} when any other method name would
be arbitrary.
The attractiveness of {P operator()() P} is
that it is used as a function application
operator, as in {P (*a)() P}.
</document>
