<document>
@* <a href=@talk-guidelines.html target=_top>Design guidelines</a>
\label{3-style}
\label{guidelines}
%%\comment{See [_LH89].}


<include file=navigate>

Computing is a relatively young discipline.
Despite its short history, a number of styles
and schools promoting a particular style have emerged.
However, in contrast to other disciplines
such as the fine arts (including architecture)
and musical composition, there is no
well-established tradition of what is to be considered
as {\em good taste} with respect to software design.
There is even a debate whether software design
must be looked at as an {\em art} or must be promoted
into a {\em science}. See for example [_Knuth92] and [_Gries].
The debate has certainly resulted in new technology
but has not, I am afraid, resulted in 
universally valid design guidelines.


The notion of {\em good design} in the other
disciplines is usually implicitly defined by
a collection of examples of good design,
as preserved in museums or (art or music) historian works.
For software design, we are still a long way from
anything like a museum, setting the standards of good design.
Nevertheless, a compendium of examples of object
oriented applications such as [_Pinson90] and [_Harmon93],
if perhaps not setting the standards
for good design, may certainly be instructive.

<slide id=3-design-criteria cap="Criteria for design">
@ Development process {\em -- cognitive factors}
*- model -->  realize  -->  refine
@ Design criteria <em> -- natural, flexible, reusable</em>
<I
- abstraction {\em -- types}
- modularity {\em -- strong cohesion} (class)
- structure {\em -- subtyping}
- information hiding {\em -- narrow interfaces}
- complexity {\em -- weak coupling}
I>
</slide>

The software engineering literature abounds
with advice and tools to measure the quality of
good design.
In \sliref{3-design-criteria}, a number of the criteria commonly found
in software engineering texts is listed.
In software design, we evidently
strive for a high level of abstraction
(as enabled by a notion of types and a corresponding
notion of {\em contracts}),
a modular structure with strongly cohesive units
(as supported by the class construct),
with units interrelated in a precisely
defined way (for instance by a client/server or subtype
relation).
Other desirable properties are,
a high degree of information hiding (that is narrowly
defined and yet complete interfaces),
and a low level of complexity (which may be achieved
with units that have only weak coupling, as supported
by the client/server model).
An impressive list, indeed.
In practice, however, we see that (object oriented)
software design is more a matter of {\em give and take}.

Design is a human process, in which {\em cognitive factors}
play a critical role.
The role of cognitive factors is reflected
in the so-called {\em fractal design process model}
introduced in [_JF88], which describes object oriented
development as a triangle with bases labeled by
the phrases $model$, $realize$ and $refine$.
This triangle may be iterated at each of the bases,
and so on.
The iterative view of software development
does justice to the importance of human understanding,
since it allows for a simultaneous understanding
of the problem domain and the mechanisms needed
to model the domain and the system architecture.

Good design involves taste.
My personal definition of good design would
certainly also involve cognitive factors {\em --
is the design understandable --}, including
subjective criteria such as {\em -- is it pleasant
to read or study the design --}.
But rather than expressing these criteria explicitly,
I would like to point at a collection of examples
of good (object oriented) software design.

Lacking such a collection, we will instead discuss some
of the issues that may arise in developing a design.
First, we will look at the distinction between
structural and behavioral encapsulation
(which rests primarily on a difference in background
rather than goals).
Next, we will introduce the notion of {\em abstract systems}
and we will present a methodology based on
event-driven programming to employ
abstract systems in modeling interactions
between objects.
And, we will conclude this section by discussing the
tradeoffs involved
in designing for reuse and individual class design.
We will present an example illustrating the tradeoffs
of using either ones of the mechanisms of
inheritance, delegation and templates in developing
(user-defined) abstract data types,
and we will give some guidelines for
detailed class design.

<document>
@+ Structural versus behavioral encapsulation
\label{ood-semantic}

Object-oriented modeling has clearly been inspired
by or, to be more careful, shows significant
similarity to the method of {\em semantic modeling}
that has become popular for developing information systems.
In an amusing paper, [_Ki89] discusses how semantic
modeling and object-oriented modeling are related.
Apart from a difference in terminology,
semantic modeling differs from object-oriented
modeling primarily by its focus on {\em structural aspects},
whereas object-oriented modeling is more concerned with
{\em behavioral} aspects, as characterized
by the notion of {\em responsibilities}.


\vfig{1}
\index{behavioral encapsulation}
\index{structural encapsulation}
%%\comment{See [_Ki89].}
<slide id=3-semantic cap="Semantic modeling">
:@ Structural versus behavioral \c{encapsulation}
<latex>
\ln\hspace{1cm}\begin{tabular}{r | c | c |}
& {\em semantic model} & {\em object-oriented} \\ \hline
\c{{\em abstraction}} & structural & behavioral \\
\c{{\em inheritance}} & subtypes & subclasses \\ \hline
\end{tabular}
</latex>
@ Semantic modeling {\em -- constructing types}
*- {\it aggregation, grouping by association}
\ifsli{\vspace{-1cm}}{}
</slide>


Typically, semantic modeling techniques provide
a richer repertoire for constructing types,
including a variety of methods for aggregation
and a notion of grouping by association.
\see{3-semantic}
The object-oriented counterpart of aggregation
may be characterized as the {\em has-a} or {\em part-of}
relation, that is usually expressed by
including the (part) object as a data member.

Associations between objects cannot be expressed
directly in an object-oriented framework.
On an implementation level, the association
relation corresponds to membership of a common collection,
or being stored in the same container.
However, the absence of an explicit association
relation makes it hard to express general {\em m-n relations},
as, for example, the relation between students and courses.


\vstr{1}
<slide id=3-challenges cap="Relations between objects">
@ Object-oriented modeling
<I
- {\em is-a} -- inheritance
- {\em has-a, uses} -- delegation
- {\em uses} -- templates
I>
:@ Challenges
<I
- {\it Clearly, the trend is for database researchers
to view object-oriented models
as having both structural and behavioral encapsulation facilities} 
I>
\ifsli{\vspace{-1cm}}{}
</slide>


The influence of a semantic modeling background can be clearly
felt in the OMT method.
The object model of OMT is a rather direct generalization
of the {\em entity-relationship} model.
Entities in the {\em entity-relationship} model
may only contain (non-object) data members,
which are called attributes.

In contrast, {\em objects} (in the more general sense)
usually hide object and non-object data members,
and instead provide a method interface.
Moreover, object-oriented modeling focuses on
behavioral properties, whereas semantic modeling has been
more concerned with (non-behavioral) data types
and (in the presence of inheritance) data subtypes.

Relations, as may be expressed in the {\em entity-relationship}
model, can partly be expressed directly in terms
of the mechanisms supported by object-oriented
languages.
For instance, the {\em is-a} relation corresponds
closely (although not completely) with
the inheritance relation.
\see{3-challenges}
Both the {\em has-a} and {\em uses} relation is 
usually implemented by including (a pointer to)
an object as a data member.
Another important relation is the {\em is-like} relation,
which may exist between objects that are neither related
by the inheritance relation nor by the subtype relation,
but yet have a similar interface and hence may be regarded
as being of analogous types.
The {\em is-like} relation may be enforced by parametrized
types that require the presence of particular methods,
such as a {\em compare} operator in the case of a generic
list supporting a {\em sort} method.
%%See section \ref{1-nextXX} for an example.

Due, partially, to what may be regarded as
the natural dynamics of research (which includes financing),
there seems to be a trend of convergence between
the interests of the object-oriented community
and the proponents of semantic modeling.
The main challenge, from the perspective of
object-oriented programming, is to provide efficient
support for persistent objects and (explicit)
support for aggregation and association relations.
Within the community of database researchers,
an interesting debate is going on about whether to
extend the relational model with object-oriented notions
or whether to start anew from fresh premises.
The main challenge, from the perspective of
database management systems, is clearly to provide
efficient support for the behavioral aspects
of objects.
One approach is to extend the object model
of object-oriented programming languages with
{\em persistence} as proposed by the Object Database Management
Group (ODMG) in [_Cattell94]. See section \ref{ODMG}.
To meet the demands of future applications (featuring graphics
and animation),
support for persistence should include {\em active objects},
having activity of their own, as well.

From the perspective of object-oriented modeling,
the main issue is to combine both structural
and behavioral aspects in a 
coherent way.
</document>

<document>
@+ Abstract systems and events
\label{systems}



User actions may require complex interactions
between the objects constituting
the object model of a system.
Such interactions are often of
an {\it ad hoc} character in the sense that they embody one
of the many possible ways in which the functionality
of objects may be used.
What we need is a methodology or paradigm that
allows us to express these interactions in a concise
yet pragmatically amenable way.
In [_Henderson93], a notion of {\em abstract systems}
is introduced that seems to meet our needs to
a large extent.
\see{3-abstract}


Abstract systems
extend the notion of abstract data types 
to capture the
(possible) interactions between collections of objects.


\clearfalse
%%\vfig{2}
<slide id=3-abstract cap="Abstract systems and events">
:@ Abstract systems <em> -- design methodology</em>
<I>- abstract system = abstract data types + protocol
:@ Events <em> -- high level glue</em>
<I>- realization of the interaction protocol
\ifsli{\vspace{-0.5cm}}{}
</slide>

The idea underlying the notion of
an {\em abstract system} is to collect
the commands available for the client
or user of the {\em system}.
The collection of commands comprising
an abstract system are usually 
a (strict) subset of the commands 
available in the combined interface
of the abstract data types involved.
In other words, an abstract system
provides a restricted interface,
restricted to safeguard the user
from breaking the protocol of interaction
implicitly defined by the collection 
of abstract data types of which the system consists.

An abstract system in itself merely provides 
a guideline on how a collection of objects is to be used,
but does not offer a formal means to check whether
a user plays by the rules.
After presenting an example of an abstract
system, we will look at how {\em events}
may be used to protect the user against breaking
the (implicit) laws governing the interaction.


:@ Example -- the library
The abstract system comprising a library may be characterized as
in \sliref{3-library}.
In essence, it provides an exemplary interface, that is,
it lists the statements that are
typically used by a client of the library software.
We use typical identifiers to denote objects
of the various types involved.

%%\clearfalse
<slide id=3-library cap="The library system">
@ Abstract system {\em -- exemplary interface}
\zline{\fbox{\fbox{library}}}
[P
p = new person();
b = new book();
p = b->borrower;
s = p->books;
tf = b->inlibrary();
b->borrow(p);
p->allocate(b);
p->deallocate(b);
b->_return(p);
P]
\ifsli{\nl}{For {P person* p; book* b; set<book>* s; bool tf; P}}
</slide>

The commands available to the user of the
library software are constructors for a $person$
and a $book$,
an instruction to get access to the $borrower$
of a particular book, an instruction to ask what books
a particular person has borrowed,
an instruction to query whether a particular book
is in the library,
and instructions for a person to $borrow$ or $return$ a book.

To realize the abstract system $library$,
we evidently need the classes $book$ and $person$.
The class $book$ may be defined as in \sliref{3-book}.

\clearfalse
<slide id=3-book cap="The book class">
\slilarge
\ifsli{}{\vspace{0.1cm}}
[P
class book { /// \fbox{book}
public:
person* borrower;
book() {}
void borrow( person* p ) { borrower = p;  }
void _return( person* p ) { borrower = 0; }
bool inlibrary() { return !borrower; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

It consists of a constructor, functions to $borrow$
and $return$ a book, a function to test
whether the book is in the library
and an instance variable containing the $borrower$ of the book.
Naturally, the class $book$ may be improved
with respect to encapsulation
(by providing a method to access the borrower)
and may further be extended to store additional
information, such as the title and publisher of the book.



%%\clearfalse
<slide id=3-person cap="The person class">
\slilarge
\ifsli{}{\vspace{0.1cm}}
[P
class person { /// \fbox{person}
public:
person() { books = new set<book>(); }
void allocate( book* b ) { books->insert(b); }
void deallocate( book* b ) { books->remove(b); }
set<book>* books;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The next class involved in the $library$ system
is the class $person$, which is shown in \sliref{3-person}.
The class $person$ offers a constructor,
an instance variable to store the set of books borrowed
by the person and the functions $allocate$ and $deallocate$
to respectively insert and remove the books
from the person's collection.
A typical example of using the library system is given below.
[P
book* Stroustrup = new book();
book* ChandyMisra = new book();
book* Smalltalk80 = new book();

person* Hans = new person();
person* Cees = new person();

Stroustrup->borrow(Hans);
Hans->allocate(Stroustrup);
ChandyMisra->borrow(Cees);
Cees->allocate(ChandyMisra);
Smalltalk80->borrow(Cees);
Cees->allocate(Smalltalk80);
P]
\noid
First, a number of books are defined,
then a number of persons,
and finally (some of) the books that are borrowed
by (some of) the persons.

Note that lending a book involves
both the invocation of $book::borrow$
and $person::allocate$.
This could easily be simplified by extending the function
$book::borrow$ and $book::\_return$
with the statements $p->allocate(this)$
and $p->deallocate(this)$ respectively.
However, I would rather take the opportunity to illustrate the use
of $events$, providing a generic solution
to the interaction problem noted.
}

@ Events
%%{\em -- high level glue}
[_Henderson93] introduces $events$
as a means by which to control the complexity of relating
a user interface to the functionality provided
by the classes comprising the library system.
The idea underlying the use of events is that for
every kind of interaction with the user
a specific event class is defined  that captures
the details of the interaction between the user
and the various object classes.
Abstractly, we may define an event as an entity
with only two significant moments in its life-span,
the moment of
its {\em creation} (and initialization)
and the moment of its {\em activation}
(that is when it actually happens).
As a  class we may define an $event$ as in \sliref{3-event}.

%%\clearfalse
\ifsli{\mbox{}\vspace{3cm}}{}
<slide id=3-event cap="The Event class">
\ifsli{}{\vspace{0.1cm}}
[P
class Event { /// \fbox{\fbox{Event}}
public:
virtual void operator()() = 0;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The class $Event$ is an abstract class,
since the application operator that may
be used to activate the event is defined as zero.

\clearfalse
<slide id=3-borrow cap="The Borrow event class">
\slilarge
\ifsli{}{\vspace{0.1cm}}
[P
class Borrow : public Event { /// \c{\fbox{Borrow}}
public:
Borrow( person* _p, book* _b ) { _b = b; _p = p; }
void operator()() {
	require( _b && _p ); /// \c{// \_b and \_p exist}
	_b->borrow(p);
	_p->allocate(b);
	}
private:
person* _p; book* _b;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

For the $library$ system defined above we may conceive of two
actual events (that is, possible refinements
of the $Event$ class),
namely a $Borrow$ event and a $Return$ event.
See slides  \ref{sli-3-borrow} and \ref{sli-3-return}.


The $Borrow$ event class provides a controlled
way in which to effect the borrowing of a book.
In a similar way, a $Return$ event class may be defined
as in \sliref{3-return}.

\cleartrue
<slide id=3-return cap="The Return event class">
\ifsli{}{\vspace{0.1cm}}
[P
class Return : public Event { /// \c{\fbox{Return}}
public:
Return( person* _p, book* _b ) { _b = b; _p = p; }
void operator()() {
	require( _b && _p );
	_b->_return(p);
	_p->deallocate(b);
        }
private:
person* _p; book* _b;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The operation {\em Has} specified in the
previous section has an immediate counterpart
in the $person::books$ data member and need
not be implemented by a separate event.

Events are primarily used
as intermediate between the user (interface)
and the objects comprising the library system.
For the application at hand, using events
may seem to be somewhat of an overkill.
However, as we will further illustrate
in section \ref{events},
events not only give a precise characterization of
the interactions involved but, equally as important,
allow for extending the repertoire
of interactions without disrupting
the structure of the application
simply by introducing additional event types.
</document>


@+ Designing for reuse

:@ Class design
We have nearly completed a first tour around
the various landmarks of object oriented design.
Identifying objects, expressing the interaction
between objects by means of client/server contracts
and describing the collaboration between objects
in terms of behavioral compositions, belong
to a craft that will only be learned in the
practice of developing real systems.
Ideally, the design document should present
a complete and formal description of the
structural, functional and dynamic aspects of the system,
including an argument showing that the various models
are consistent.
However, in practice, this will seldom be realized.
Partly, because object oriented design techniques
are as yet not sufficiently matured to allow
a completely formal treatment and partly because
most designers will be satisfied with a non-formal
rendering of the architecture of their system.
Admittedly, the task of designing is already
sufficiently complex, even without the additional
complexity of a completely formal treatment.
Nevertheless, studying the formal underpinnings
of object oriented modeling based on types and polymorphism
is still worthwhile, since it will sharpen the
intuition with respect to the notion of behavioral
conformance and the refinement of contracts,
which are both essential for
developing reliable object models.
And reliability is the key to reuse!
}

\c{
."@ Class design
We will conclude this chapter by looking at some
informal, pragmatic guidelines for individual class design.
}

\nop{
@ Class design
{\em -- reducing name conflicts}
<I
- faithful model of a single concept
- a reusable plug-compatible component
- robust, well-designed
- integrable, extensible
I>
}

\nop{
Above, we have rephrased the goals that should be kept
in mind when developing a class.
}

\c{
Ideally, a class should represent a faithful model of a single concept,
and be a reusable, plug-compatible, component,
that is robust, well-designed and extensible.
In \sliref{3-individual}, we list a number of suggestions
put forward in [_McGregor92].
}

\slide{3-individual}{Individual class design}{
@ Class design
{\em -- guidelines}
\ifsli{\vspace{-0.5cm}}{}
<I
- only methods public
{\em -- information hiding}
- do not expose implementation details
- public members available to all classes
{\em -- strong cohesion}
- as few dependencies as possible
{\em -- weak coupling}
- explicit information passing
- root class should be abstract model
{\em -- abstraction}
I>
\ifsli{\vspace{-1.0cm}}{}
}

\c{
The first two guidelines enforce the principle of 
{\em information hiding},
advising to make only methods public and to
hide all implementation details.
The third guideline states a principle
of {\em strong cohesion}, by requiring that
classes implement a single protocol
that is valid for all potential clients.
A principle of {\em weak coupling} is enforced by
requiring a class to have as few dependencies as possible,
and to employ explicit information passing
using messages instead of inheritance
(except when inheritance may be used in a type
consistent fashion).
When using inheritance, the root class should be
an abstract model of its derived classes,
whether inheritance is used to realize 
a partial type or to define a specialization
in a conceptual hierarchy.

The list given above can be used as a checklist
to verify whether a class is well-designed.
In section \ref{2-metrics} we will explore
metrics that capture the guidelines given in
a more quantitative manner.
Such metrics may be an aid in the software engineering
of object oriented systems and may possibly
also be used to measure the productivity of 
object oriented programmers.
}


</document>
