<document>
@+ <a href=@talk-methods-compare.html target=_top>Methods for analysis and design  -- a comparative study</a>
\label{Compare}

<include file=navigate>

In [_Fichman] a comparative review of a selected number of
object-oriented analysis and design methods is given.
Criteria for selection were the availability
of documentation and acceptance in the object-oriented
community,
measured in terms of refereed articles.

Paraphrasing [_Fichman] again:
<em>As with traditional analysis, the primary goal of
object-oriented analysis is the development of an
accurate and complete description of the problem
domain.</em>

The three analysis models described in [_Fichman]
share a number of diagram techniques
with both structured methods and methods for
object-oriented design.
However, the method proposed
in [_Shlaer88] in particular reflects the domain-oriented focus
of analysis.

A similar focus on domain requirements
and analysis may be found in the
Objectory method.
\see{11-compar-1}
Objectory is one of the methods that
has inspired Fusion,
in particular because it presents a systematic
approach to the process of software development.
The Objectory method centers around {\em use case}
analysis.
Use case analysis involves a precise description
of the interaction between the user of a system
and the components representing domain-specific
functionality.
The Objectory method gives precise guidelines
on how to proceed from the identification of
{\em use cases}, which include user interface aspects,
to their realization in the subsequent phases of
design and implementation.
Objects are called blocks in Objectory.
Use case analysis corresponds in a loose way with
the identification of system operations in Fusion.



<slide id=11-compar-1 cap="Comparison of methods (1)">
:@ Objectory <em> -- systematic process</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- requirements
{\em -- use cases, domain object model, user interface}
- analysis
{\em -- subsystems}
- design, implementation
{\em -- block model, interaction diagrams}
I>
:@ OMT <em> -- few rules for <c>discovering</c> inconsistencies</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- analysis
{\em -- object model, dynamic model, functional model}
- design, implementation
{\em -- heuristics to implement analysis models}
I>
:@ Booch <em> -- descriptive</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- diagrams 
{\em -- class, object, timing, state, module, process}
I>
:@ CRC <em> -- exploratory</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- analysis, design
{\em -- class, responsibilities, collaborators}
I>
:@ Formal methods
\ifsli{\vspace{-0.5cm}}{}
<I
- operations
{\em -- pre- and post-conditions}
I>
\ifsli{\vspace{-1cm}}{}
</slide>


There is a close correspondence between the OMT
object model and the analysis object model of Fusion.
Both OMT and Fusion employ extended entity-relationship
diagrams.
Also, the dynamic model of OMT reoccurs in the
Fusion method, albeit in a later phase.
The functional model of OMT, which has the form of a 
dataflow diagram, is generally considered to be 
inappropriate for object-oriented analysis.
Instead, Fusion employs a model in which
the semantics of system operations are captured
by means of formal pre- and post-conditions.
In [_Fusion], OMT is characterized as a very loose
method, giving few rules for discovering
inconsistencies between the various models
and lacking a clear view with respect to the
process of development.
OMT is strongly focused on analysis,
giving nothing but heuristics to implement
the models that result from analysis.
However, what is called the {\em light-weight Fusion method}
almost coincides with OMT.

A lack of detailed guidelines for the
process of software development is also characteristic
of the Booch OOD method.
Booch offers a wealth of descriptive diagrams,
giving detailed information on the various
aspects of a system,
but offers merely heuristics for the actual
process of development.

The CRC method must be regarded primarily as a means
to explore the interaction between the various
objects of a domain.
It is powerful in generating ideas, but offers
poor support for documenting the decisions
with respect to the objects and how they interact.

Formal methods have been another
important source of inspiration for the Fusion method.
The description of system operations
during analysis employs a characterization
of the functionality of operations that
is directly related to
the specification of operations in model-based
specification methods such as VDM and Z. See section \ref{formal-coop}.

The Fusion method may be regarded as being composed of
elements of the methods mentioned above.
It shares  its object model with OMT,
its approach to the characterization of system
operations with formal methods,
its focus on object interaction with CRC
and its explicit description of classes and
their relations with Booch.
\see{11-compar-2}




<slide id=11-compar-2 cap="Comparison of methods (2)">
<control>
:@ Comparison - as a systematic approach
<table border=0 cellpadding=10>
<tr><td></td>
<td>Objectory</td>
<td>OMT</td>
<td>Booch</td>
<td>CRC</td>
<td>Fusion</td>
</tr>
<tr><td>development</td><td align=center>+</td><td align=center>+-</td><td align=center>-</td><td align=center>x</td><td align=center>+</td></tr>
<tr><td align=left>maintenance</td><td align=center>+</td><td align=center>+-</td><td align=center>+</td><td align=center>-</td><td align=center>+</td></tr>
<tr><td align=left>structure</td><td align=center>+-</td><td align=center>+-</td><td align=center>+</td><td align=center>+</td><td align=center>+</td></tr>
<tr><td align=left>management</td><td align=center>+</td><td align=center>+-</td><td align=center>+-</td><td align=center>-</td><td align=center>+</td></tr>
<tr><td align=left>tool support</td><td align=center>+-</td><td align=center>+-</td><td align=center>+-</td><td align=center>-</td><td align=center>+</td></tr>
</table>
</control>
<latex>
<h4>Comparison <em> -- as a systematic approach</em></h4>
\zline{\fbox{Fusion}}
\ln
\begin{tabular}{ l c c c c c } 
 & Objectory & OMT  & Booch & CRC & Fusion \\
 \hline
development & 
    $ + $      & $\pm$ & $-$   & $\oplus$ & $ + $ \\
maintenance &
    $ + $      & $\pm$ & $ + $   & $-$ & $ + $ \\
structure   &
    $\pm$      & $\pm$ & $ + $   & $ + $ & $ + $ \\
management &
    $ + $      & $\pm$ & $\pm$   & $-$ & $ + $ \\
tool support &
    $\pm$      & $\pm$ & $\pm$   & $-$ & $ + $ 
\end{tabular}
<?/latex>
</slide>

In comparison with these methods, however,
it provides a much more systematic approach
to the process of development
and, moreover,
is explicitly concerned with issues of validation
and consistency between models.
In addition, [_Fusion] claim
to provide explicit semantics for their various models,
whereas the other methods fail to do so.
However, it must be remarked that the Fusion method
remains somewhat obscure about the nature of system operations.
System operations are characterized as {\em asynchronous}.
Yet, if they are to be taken as methods, such operations
may return a result, which is quite hard to reconcile
with their asynchronous nature.
The claim that the models have a precise semantics,
which is essential for tool support, must be
substantiated by providing an explicit semantics
in a formal manner!

With regard to the process of development,
both Objectory and Fusion provide precise
guidelines.
The CRC method may be valuable
as an additional exploratory device.
For maintenance, the extent to which
a method enforces the documentation of design
decisions is of utmost importance.
Both the Objectory and Booch method  satisfys
this criterion, as does the Fusion method.
OMT is lacking in this respect, and CRC is clearly inadequate.

Whether a method leads to a {\em good} object-oriented design
of the system architecture, depends to a large extent upon the
ability and experience of the development team.
Apart from Fusion, both the Booch method and CRC
may be characterized as purely object-oriented,
whereas Objectory and OMT are considered to be impure.

A strongly systematic approach to the process of
development is important in particular from
the point of view of project management.
Project management support entails a precise definition
of the deliverables associated with each phase,
as well as an indication of the timing
of their deliverance and validation.
Both the OMT method and Booch are lacking in this
respect, since they primarily provide techniques
to develop descriptive models.
Clearly, CRC lacks any support for project management.

Tool support is dependent on the existence of
a well-defined semantics for the models employed.
For both Objectory and OMT commercial tools
are available, despite their loosely specified semantics.
The Fusion diagramming techniques are also supported.
%%for example by Paradigm Plus, which also
%%supports the diagraming techniques of OMT. See section \ref{pde}.
For CRC, tool support is considered
to be useless.
The success of the method depends upon flexibility,
the ease with which new ideas can be tried,
a flexibility which even hypertext cannot offer,
according to its authors.

<ignore>
An important question, when considering methods
for software development,
is what means we wish to have to
represent the decisions taken during design.


%%@ Design representation
As phrased in [_Fichman],
{\em design is the process of mapping system requirements defined
during analysis to an abstract representation of a specific
system-based implementation meeting cost and performance constraints}.
In other words, design is concerned with architectural issues,
whereas analysis is concerned with requirements and constraints.
Although different in objective, in an object-oriented approach
analysis and design may share a common object model.
Both in the method proposed by [_CY91b] and the method proposed in [_Booch91],
the difference between analysis and design lies primarily in the level of detail
in which the respective components of the system are specified.
Many of the diagram techniques employed, however, have no clear
semantics, and rely on an informal description
of common cases only.
In \sliref{11-diagrams}
a number of the diagram types proposed in [_Booch91] are listed.

<slide id=11-diagrams cap="Diagrams for design">
%%@ Diagrams 
\zline{\fbox{Booch}}
<I
- class diagram/template
- object diagram/template <c> {\em -- dynamics} </c>
- module diagram/template
- process diagram/template <c> {\em -- physical design} </c>
- state transition diagram
- timing diagram
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Apart from {\em class diagrams} and {\em templates}
to describe the functionality
of classes and their (static) relations to other classes,
the methods also uses {\em object diagrams} (and templates) to
characterize the dynamic behavior of (collections of) objects.
Moreover, the method provides for 
{\em module diagrams}
to depict
the relations between modules and to describe the physical architecture
of distributed systems.
As additional techniques to describe the behavioral characteristics of object,
[_Booch91] employs {\em state-transition diagrams} and {\em timing diagrams}
(expressing constraints on the order in which methods may be invoked).

These diagram techniques have as a common objective to represent the
important properties and aspects of a design.
Along the dimensions by which we may classify design representation
technologies, {\em expressiveness} and {\em formality},
object-oriented technology may be ranked as being midway between non-formal,
highly expressive techniques such as hypertext, and highly formal (logic-based)
techniques which require a lot of expertise to express relationships
and properties.
However, object-oriented techniques cover a broad range.
For example, {\em collaboration graphs} (Wirfs-Brock {\it et al.}, 1990) and {\em contracts}
([_Meyer88]) offer (in principle)
a high degree of formality, whereas many of the diagram techniques
seem to be amenable to an extension with hypertext features
to support browsing and the retrieval of information.
See [_CY91b], [_Bigelow88] and [_Webster].

A major problem that is still not satisfactorily solved by any of the
existing methods is to support what may be called the {\em harvesting of reuse},
that is the actual exploitation of the reuse potential that
is intrinsic to object-oriented technology.
Both at the level of analysis and design,
a repository-based approach to managing reuse (in combination with appropriate
retrieval techniques) seems to be the most promising.
Another issue about which opinions diverge is by what principles objects
should be clustered, and what techniques must be provided to support  system partitioning.
Perhaps the solution to these problems is not strictly a matter of
technology, but instead requires us to wait for appropriate conventions
and styles to emerge within each particular application domain.
</ignore>
</document>
