<document>
@+ <a href=@talk-objects-respons.html target=_top>Assigning responsibilities</a>
\label{CRC}

<include file=navigate>

Design is to a large extent a matter of creative thinking.
Heuristics such as performing a linguistic scan
on the requirements document for finding objects
(nouns), methods (verbs) and attributes (adjectives)
may be helpful, but will hopelessly fail
when not applied with good taste and judgement.
Not surprisingly, one of the classical techniques
of creative writing, namely the {\em shoe-box method},
has reappeared in the guise of an object-oriented
development method.
The shoe-box method consists of writing fragments
and ideas on note cards and storing them in a (shoe) box,
so that they may later be retrieved and manipulated
to find a suitable ordering for the
presentation of the material.
To find a proper decomposition into objects,
the method creates for each potential
(object) class a so-called CRC card,
which lists the $Class$ name, the $Responsibilities$
and the possible $Collaborators$ of the proposed class.
In a number of iterations, a collection of cards
will result that more or less reflects the
structure of the intended system.

According to the authors (see Beck and Cunningham, 1989), the method
effectively supports the early stages of design,
especially when working in small groups.
An intrinsic part of the method consists
of what the authors call {\em dynamic simulation}.
To test whether a given collection of cards
adequately characterizes the functionality of the intended
system, the cards may be used to simulate
the behavior of the system.
When working in a group, the cards may be distributed
among the members of the group, who participate
in the simulation game according to their cards.
\see{3-crc}


%%\comment{See [_BC89].}
<slide id=3-crc cap="The CRC method">
:@ Object-oriented thinking
<p align=right><box>CRC</box>
<ul>
<li> Immerse the reader in the {\em object-ness} of the material
<li> Give up global knowledge of control
<li> Rely on the local knowledge of objects
</ul>
@ OO design with CRC cards
<ul>
<li> Class, Responsibility, Collaborators
</ul>
%%:@ Regressions:
%%*- global variables, pointers, reliance on the implementation of objects.
</slide>

A number of authors have adopted this method,
or developed a very similar method,
for identifying objects and characterizing
their functionality in an abstract way.
It is doubtful, however, whether the method has
any significance beyond the early stages of analysis and design.
Without any more formal means to verify whether the
responsibilities listed adequately characterize
the intended functionality of the system,
the method amounts to not much more than brainstorming.
Clearly, the method needs to be complemented
by more formal means to establish whether the (implicit)
protocols of interaction between the objects
satisfy the behavioral requirements
of the system.

Nevertheless, the elegant simplicity of the method
is appealing, and the card format lends itself to
easy incorporation in an on-line documentation system.
Moreover, since the method imposes no strict order,
and has relatively little overhead, it is indeed
a good way to get an initial idea of
what objects the system will comprise.


:@ Example -- ATM (2)
\label{ATM-2}
Actually, the ATM example is an interesting example
for comparing the various approaches,
since it is used by many authors to illustrate
their methods.
In [_WWW90] the example is used for spelling out
all the steps that must be taken.
In [_Rum91] it is extensively described
to illustrate the various 
modeling techniques employed by the method.
And in [_BC89] the CRC cards method is illustrated
by sketching the design of an automated teller machine.

The approaches presented in [_BC89] and [_WWW90]
are actually very closely related.
Both may be characterized as {\em responsibility-driven},
in that they concentrate on {\em responsibilities}
and {\em collaboration} relations
to model the interaction between objects.
However, the method described in [_WWW90]
is much more detailed, and to some extent includes
means to formally characterize the behavior of an object
and its interaction with other objects.
To this end it employs an informal notion of
{\em contracts} as originally introduced in [_Meyer88].

In section <ref file=objects-heuristics>ATM-1</ref>
a number of candidate
classes have been suggested for our ATM.
Now, with the use of CRC cards, we will delineate
the functionality of (a number of) these classes
more precisely.
Also we will establish how the various object classes
must collaborate to perform their duties.
At the highest level of the design, we may distinguish between
two groups of classes:
the classes representing the  {\em banking model}
(comprising the class $account$ and the class $transaction$),
and the classes that model the interaction with the user
(comprising the class {\em card-reader} and the class
{\em cash-dispenser}).
At a lower level, we also need a class modeling
the {\em database} that provides persistent
storage for the user's account
and the information needed for authorization.
For each of these classes we will use a CRC card
to indicate their responsibilities and the classes
with which they need to collaborate.

The {\em banking model}, depicted in \sliref{3-atm-2a},
consists of the classes $account$
and $transaction$.
The class $account$ keeps a record of the actual $balance$
of the $account$ and must allow a user to $deposit$
or $withdraw$ money.
However, for safety reasons, these operations are
never carried out directly, but are performed
by an intermediary {\em transaction} object.
%%\see{3-atm-2a}

<slide id=3-atm-2a cap="The ATM example (2a)">
:@ Banking model
\zline{\fbox{ATM}}
<c><p></c>
<raw file=banking heigth=200>
\footnotesize
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf account} & {\it transaction  } \\
keeps balance & {\it database}  \\
deposit money & \\
withdraw money & \\
\hline
\end{tabular}}
</raw>
<c><p></c>
<raw height=200>
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf transaction} & {\it card-reader } \\
validation & {\it cash-dispenser} \\
performs transfer & {\it account} \\
keeps audit info & {\it database} \\
\hline
\end{tabular}}
</raw>
</slide>

The responsibilities of the {\em transaction} class
may be summarized as: the validation of user requests
and the execution of money transfers.
The responsibility for maintaining audit information is also
assigned to the {\em transaction} class.
To act as required, a {\em transaction} object
needs to communicate with a number of other objects.
It must acquire information from both the {\em card-reader}
and the {\em database} to check, for example,
whether the user has entered the right PIN code.
To validate a request, it must check whether the
account will be overdrawn or not.
To pay the requested money, it must instruct the
<em>cash-dispenser</em> to do so.
And it must contact the database to log the appropriate
audit information.
In contrast, an {\em account} only needs to respond
to the requests it receives from a {\em transaction}.
Apart from that, it must participate in committing the
transaction to the bank's {\em database}.
Note that the CRC method is non-specific about how
the collaborations are actually realized;
it is unclear which object will take the initiative.
To model these aspects we will need a more precise
notion of {\em control}
that tells us how the potential behavior
(or responsibility) of an object is activated.

The second group of classes may be called
{\em interaction classes},
since these are meant to communicate with
entities in the outside world, outside from
the perspective of the system.
Also the bank's {\em database} may be considered
as belonging to the outside world,
since it stores the information concerning the
account and the authorization of customers
in a system-independent manner.
\see{3-atm-2b}

<slide id=3-atm-2b cap="The ATM example (2b)">
:@ Interaction classes
\zline{\fbox{ATM}}
<c><p></c>
<raw height=200>
\ifsli{\vspace{-0.5cm}}{}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf cardreader} & {\it event  } \\
signals insertion  & {\it transaction}  \\
decodes strip & \\
\hline
\end{tabular}}
</raw>
<c><p></c>
<raw height=200>
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf \c{cash}dispenser} & {\it event  } \\
emits cash  & {\it transaction}  \\
\hline
\end{tabular}}
</raw>
<c><p></c>
<raw height=200>
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf database} & {\it event } \\
retrieves account & {\it transaction} \\
records transaction & {\it account} \\
authorization & {\it database} \\
\hline
\end{tabular}}
</raw>
</slide>


Both the {\em card-reader} and the {\em cash-dispenser}
rely on a class called {\em event},
which is needed to model the actions of the user.
For example, when a user inserts a bankcard,
we expect a transaction to start.
For this to happen, we must presuppose an underlying system
that dispatches the event to the {\em card-reader},
which in turn notifies the {\em teller machine}
that a new transaction is to take place.
%%See also section \ref{cooperation} for a comparative study of
%%such mechanisms.
The flow of control between a {\em transaction} object
and the {\em cash-dispenser} is far more straightforward,
since a {\em transaction} object only needs to
issue the appropriate instruction.
However, the actual interaction between the
{\em cash-dispenser} and the underlying hardware,
that turns out the money, may be quite intricate.

The {\em database} may either respond directly to the request
coming from the {\em account} or {\em transaction} object
or it may respond to {\em events}
by taking the initiative to call the appropriate methods
of the {\em account} and {\em transaction} objects.
Whether the {\em database} may be accessed directly
or will only react to
events is actually dependent on the
control model we assume when developing the system model.
</document>
