<document>

@* <a href=@talk-contracts.html target=_top>Contracts</a>
\label{contracts}

<include file=navigate>

To establish the interaction between objects in a more
precise way, we need a notion of {\em contracts},
specifying the requirements a client must comply with
when requesting a service from a server object.
Our notion of contracts will be based on the notion of {\em types}.

In the universe of programming,
{\em types} are above all a means to create
order and regularity.
Also, in an object-oriented approach,
types may play an important role in
organizing classes and their relationships.
As observed in [_HOB87],
the notion of types gives a natural criterion
for {\em modularization},
perhaps not so much as a guideline to arrive at a particular
object decomposition,
but as a means to judge whether the modular
structure of a system is consistently defined,
that is technically well-typed.
\see{3-types}


\clearfalse
\vstr{1}
<slide id=3-obligations cap="Contractual obligations">
@ Contractual obligations
<center>
<control>
<table cellpadding=10 border=1>
<tr><td></td><td>client</td><td>supplier</td></tr>
<tr><td>pre-condition</td><td>obligation</td><td>benefit</td>
<tr><td>post-condition</td><td>benefit<td>obligation</td>
</table>
</control>
<latex>
\ifsli{\vspace{-1cm}}{}
\mbox{}\nl\mbox{}\hspace{1cm}\begin{tabular}{r | c c |} & {\it client} & {\it supplier} \\ \hline
{\it pre-condition} & obligation & benefit \\
{\it post-condition} & benefit & obligation \\ \hline
\end{tabular}\nl
</latex>
</center>
</slide>

The meaning of a type must be understood as
a formal characterization of the behavior of the
elements belonging to the type.
A type consists of a (possibly infinite)
collection of elements which is characterized by the definition
of the type.
For example, a {\em class} defines such a collection,
namely the instances of the class, whose behavior
is constrained by the specification of the class.
<p>
A formal specification of the behavior of an object may
be given by defining a {\em pre-condition}
and {\em post-condition} for each method.
The pre-condition of a method specifies in a logical
manner what
restrictions the client invoking a particular method
is obliged to comply with.
When the client fails to meet these requirements
the result of the method will be undefined.
In effect, after the violation of a pre-condition
anything can happen.
Usually, this means that the computation
may be aborted or that some other means of error-handling
may be started.
For instance, when the implementation language
supports exceptions an exception handler may be invoked.

The post-condition of a method states what
obligations the server object has when executing the
method, provided that the client's request satisfies
the method's pre-condition.

Apart from specifying a pre-condition and post-condition
for each method publicly supported by the class,
the designer of the class may also specify a {\em class invariant},
to define the invariant properties of the state of
each instance of the class.

A class annotated with an invariant
and pre- and post-conditions for
the methods may be regarded as a $contract$,
since it specifies precisely (in an abstract way)
the behavioral conformance conditions of the object
and
the constraints imposed on the interactions
between the object and its clients.
\see{3-obligations}


\clearfalse
<slide id=3-formal cap="Formal specification of contracts">
@ Assertions <em> -- formal specification</em>
<I
- {\bf require} -- method call pre-condition
- {\bf ensure, promise} -- post-condition
- {\bf invariant} -- object invariance
I>
</slide>

Intuitively, {\em contracts} have a clear analogy
to our business affairs in everyday life.
For instance, when buying audio equipment, as
a client you wish to know what you get for
the price you pay, whereas the dealer may require
that you pay in cash.
Following this metaphor through, we see that
the supplier may actually benefit from
imposing a (reasonable) pre-condition and that
the client has an interest in a well-stated post-condition.
Most people are not willing to pay without knowing
what they will get for their money.

The use of $contracts$ was originally
proposed by [_Meyer88], and is directly
supported by the language Eiffel,
which offers the keywords $require$
(to indicate a pre-condition),
$ensure$ (to indicate a post-condition)
and $invariant$ (to indicate the invariance condition).
\see{3-formal}
The Eiffel environment has options to dynamically
check any of the three kinds of assertions,
even selectively per class.
The assertions, except for the invariance condition,
are directly embedded in the code.
Although less elegant, the same functionality
can be achieved in C++ by using the $assert$
macro defined in {\tt assert.h} as explained
in section \ref{ASSERT}, which also introduced 
the $require$ and $promise$ macros for C++.


For dynamically checking the invariance condition,
a test should be executed when evaluating the
constructor
and before and after each method
invocation.
While a method is being executed, the invariant
need not necessarily hold,
but it is the responsibility of a method to
restore the invariant when it is disrupted.
In case object methods are recursively applied,
the invariant must be restored when returning
to the original caller.

An alternative approach to incorporating assertions
in a class description is presented in [_Cline],
which introduces an extension of C++ called Annotated C++.
Instead of directly embedding assertions in the
code, Annotated C++ requires the user to
specify separately the axioms characterizing
the functionality of the methods and their
effect on the state of the object.

Important from a design perspective is
that the specification, even without an implementation
of the methods, may be understood as specifying
a contract.
In practice, they may only be stated as comments.

@ Conformance
Having looked at a precise definition
of contracts, we may well reflect (again)
on the nature of object-oriented computation
to draw implications with respect to system
development.
Each object, so to speak, carries a state and behavior.
In principle, the state is not directly accessible
but is encapsulated by a well-defined method
interface.
Computation in an object-oriented system, then,
amounts to sending messages that may
be dispatched by the dynamic binding mechanism
to invoke the appropriate procedure.
As a client of a server object, what you need to know
is what you may expect as the result of invoking a method,
irrespective of what actual procedure the message
is dispatched to.

\clearfalse
<slide id=3-conformance cap="The substitutability requirement">
:@ Object = state + behavior
*- {\em computation is sending messages}
:@ Subtyping
*- if B is a subtype of A then B may be used wherever A is used
:@ Inheritance ( type hierarchy graph ) 
*- {\em code sharing} (storage representation + operations)
\ifsli{\vspace{-1cm}}{}
</slide>

In many cases it is sufficient to
characterize the desired behavior in terms of
(behavioral) {\em conformance}.
Operationally, (the behavior of) an object $B$
conforms to (the behavior of) an object $A$
if the behavior of $B$ is in some sense equivalent to
the behavior of object $A$.
More formally, this may be expressed in terms of
the subtype relation, by saying that type $B$
is a subtype of $A$ if any object of type $B$
may be used anywhere that an object of type $A$
may be used.
Behavioral conformance
amounts to the requirement of substitutability.
\see{3-conformance}
If we know what a type does, we may substitute
any object that realizes the type as long
as it does what it is expected to do.
In other words, conformance to a type means
that the behavior of the object respects
the constraints imposed by the type.

A direct corollary of our notion of {\em conformance}
is that instances of derived classes must
conform to the type of their base classes.
We will study how this affects the use of contracts
to characterize the behavioral properties of a class.


@ System development
From the perspective of system development,
the notion of contracts has some interesting consequences.
Most importantly, {\em contracts} may be used to
document the method interface of a class.
Pre- and post-conditions allow the class designer
to specify in a concise manner the functional
characteristics of a method,
whereas the use of natural language often leads to
lengthy (and imprecise) descriptions.
Below, an example is given of a contract specifying an account.


<slide type=hide id=3-acc-1 cap="The $account$ contract">
%%@ Contract <em> -- interface</em>
%%/// <box>account</box>
[P
class account {  /// <box>account</box>
public:
account();
// assert( invariant() );

virtual float balance() { return _balance; }

void deposit(float x); /// to deposit money
// require( x >= 0 );
// promise( balance() == old_balance + x  && invariant() );

void withdraw(float x); /// to withdraw money
// require( x <= balance() );
// promise( balance() == old_balance - x && invariant() );

bool invariant() { return balance() >= 0; }
protected:
float _balance;
};
P]
\ifsli{\ln\vspace{-1cm}}{}
</slide>

The interface for the $account$ class given in \sliref{3-acc-1},
specifies in an abstract way what the user
expects of an account.
From the perspective of design, the behavioral
abstraction expressed by the axioms is exactly
what we need, in principle.
The implementation must guarantee that these constraints
are met.



With regard to encapsulation, the obvious disadvantage
of using assertions is that detailed knowledge
of the class, including knowledge of the existence
of private and protected instance variables, is needed
to be able to understand the meaning of these assertions.
However, this may not be so odd as it appears,
since we do regard an object as having a state
and operations that possibly modify this state.
From this point of view, encapsulation may be seen
as merely prohibiting access to the state,
not as a dictum not to look at or reason about the state.
Nevertheless, despite the advantages the use of assertions
offers from a formal point of view, the cognitive
complexity of logical assertions will usually
necessitate additional comments (in natural language)
for a user to be able to understand the functionality
of an object and the services it offers.



<slide id=3-limits cap="System development with contracts">
@ System development
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em violated pre-condition} -- bug in client
- {\em violated post-condition} -- bug in supplier
I>
<p>
<em>A pre-condition limits the cases that 
a supplier must handle</em>!
</slide>


Assertions may be used to decide
who is responsible for any erroneous behavior of the system.
\see{3-limits}
For example, imagine that you are using a software
library to implement a system for financial transactions
and that your company suffers a number of losses
due to bugs in the system.
How would you find out whether the loss is your own fault
or whether it is caused by some bug in the library?

Perhaps surprisingly, the use of assertions allows
you to determine exactly whether to sue the library
vendor or not.
Assume that the classes in the library are
all annotated with assertions that can be checked
dynamically at runtime.
Now, when you replay the examples that resulted
in a loss for your company with the option
for checking pre- and post-conditions on,
it can easily be decided who is in error.
In the case a pre-condition of a method
signals violation, you, as a client of a library
object, are in error.
However, when no pre-condition violation is signaled,
but instead a post-condition is violated, then the library
object as the supplier of a service is in error;
and you may proceed to go to court,
or do something less dramatic as asking the
software vendor to correct the bug.

:@ Realization
The contract specified in the class interface
as given in \sliref{3-acc-1} may actually be enforced
in the code as illustrated in \sliref{3-acc-2}.

<slide type=hide id=3-acc-2 cap="The realization of the $account$ contract">
%%:@ Realization
[P
class account { /// <box><i>account</i></box>
public:
account() { _balance = 0; assert(invariant()); }

virtual float balance() { return _balance; }


void deposit(float x) {
	require( x >= 0 );  /// check precondition
	hold(); /// to save the old state
	_balance += x; 
	promise( balance() == old_balance + x );
	promise( invariant() );
	}

void withdraw(float x) {
	require( x <= _balance );  /// check precondition
	hold(); /// to save the old state
	_balance -= x; 
	promise( balance() == old_balance - x );
	promise( invariant() );
	}

virtual bool invariant() { return balance() >= 0; }
protected:
float _balance;
float old_balance; /// additional variable
virtual void hold() { old_balance = _balance; }
};
P]
</slide>

The additional variable {\em old\_balance} is needed
to compare the state preceding an operation with 
the state that results afterwards.
The old state must explicitly be copied by calling $hold$.
In this respect, Eiffel offers better support
than C++.

Whenever $balance()$ proves to be less than zero,
the procedure sketched above can be used to determine whether
the error is caused by an erroneous method invocation,
for example when calling $withdraw(x)$ with $x >= balance()$,
or whether the implementation code contains a bug.

For the developer of the software, pre-conditions
offer a means to limit the number of cases that a method
must be able to handle.
Often, programmers tend to anticipate all possible uses.
For instance, many programs or systems have options that
may be learned only when inspecting the source code
but are otherwise undocumented. \nop{See for example [UndocDos].}
Rather than providing all possible options,
for now and the future, it is more sensible to delineate
in a precise manner what input will be processed and what
input is considered illegal.
For the developer, this may significantly reduce
the effort of producing the software.
It is important that what is and what is not supported
is in principle negotiable whenever the class
interface explicitly states the requirements
imposed on the user.

@ Contracts and inheritance
Contracts provide a means to specify the behavior of an object
in a formal way by using logical assertions.
In particular, a contract specifies the constraints involved
in the interaction between a server object and a client invoking a method
for that object.
When developing a refinement subtype hierarchy we need to establish
that the derived types satisfy the constraints imposed by
the contract associated with the base type.

To establish that the contract of a derived class refines the contract
of the base class it suffices to verify that the following rules
are satisfied.
\see{3-inheritance}

<slide id=3-inheritance cap="Contracts and inheritance">
@ Refining a contract <em> -- state responsibilities and obligations</em> 
<I
- {\em invariance} -- the invariants of all the parents of a class apply to the class itself
- {\em methods} -- services may be added or refined 
I>
@ Improving a service <em> -- do only more than expected </em>
<I
- accept weaker pre-conditions
- guarantee stronger post-conditions
I>
</slide>


First, the invariant of the base class must apply to all instances
of the derived class.
In other words, the invariance assertions of the derived class must
be logically equal to or stronger than the assertions characterizing the 
invariant properties of the base class.
This requirement may be verified by checking that the invariance properties
of the base class can be logically derived from the statement asserting
the invariance properties of the derived class.
The intuition underlying this requirement is that the behavior
of the derived class is more tightly defined and hence subject
to stronger invariance conditions.

Secondly, each method occurring in the base class must occur
in the derived class, possibly in a refined form.
Note that from a type theoretical point of view it is perfectly
all right to add methods but strictly forbidden to delete methods,
since deleting a method would violate the requirement of
behavioral conformance that adheres to the subtype relation.
Apart from adding a method, we may also refine existing methods.
Refining a method involves strengthening the post-condition and
weakening the pre-condition.
Suppose that we have a class $C$ derived from a base class $P$,
to verify that the method $m_C$ refines the method $m_P$ defined 
for the base class $P$, we must check, assuming that the
signatures of $m_C$ and $m_P$ are compatible, that the post-condition
of $m_C$ is not weaker than the post-condition of $m_P$,
and also that the pre-condition of $m_C$ is not stronger than the pre-condition of $m_P$.
\see{3-refining}


<slide id=3-refining cap="Refining a method">
@ Refining a method <em> -- like improving a business contract</em>
[P
class C : public P {
	virtual void m();
	}
P]
<I
- <em>pre( m_C ) <math> >= </math> pre(m_P)</em>  /// weaken pre-condition
- <em>post( m_C ) <math> <= </math> post(m_P)</em> /// strengthen post-condition
I>
</slide>

This rule may at first sight be surprising, because
of the asymmetric way in which post-conditions and pre-conditions are treated.
But reflecting on what it means to improve
a service, the intuition underlying this rule,
and in particular the contra-variant relation
between
the pre-conditions involved, is quite
straightforward.
To improve or refine a service, in our
common sense notion of a service,
means that the quality of the product or
the result delivered becomes better.
Alternatively, a service may be considered as
improved when, even with the result remaining
the same, the cost of the service is decreased.
In other words, a service is improved if either
the client may have higher expectations of the result
or the requirements on the client becomes less
stringent.
The {\em or} is non-exclusive.
A derived class may improve a service while
at the same time imposing fewer constraints on the 
clients.

:@ Example
As an example of improving a contract,
consider the refinement of the class $account$
into a class {\em credit\_account}, which allows
a consumer to overdraw an account to a limit
of some $maximum$ amount.
\see{3-credit-account}

<slide type=hide id=3-credit-account cap="Refining the account contract">
%%@ Refining the <em>account</em> contract
[P
class credit_account : public account { /// <box>credit_account</box>
public:
credit_account(float x) { _maxcredit = x; _credit = 0; }

float balance() { return _balance + _credit; }

float credit(float x) { 
	require( x + _credit <= _maxcredit );
	hold();
	_credit += x;
	promise( _credit = old_credit + x );
	promise( _balance = old_balance);
	promise( invariant() );
	}

void reduce(float x) {
	require( 0 <= x && x <= _credit );
	hold();
	_credit -= x;
	promise( _credit = old_credit - x );
	promise( _balance = old_balance );
	promise( invariant() );
	}


bool invariant() {
	return _credit <= _maxcredit && account::invariant();
	}
protected:
float _maxcredit, _credit;
float old_credit;
void hold() { old_credit = _credit; account::hold(); }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

As a first observation, we may note that the invariant
of $account$ immediately follows from the invariant
of <em>credit_account</em>.
Also, we may easily establish that the pre-condition
of $withdraw$  has (implicitly) been weakened, since we are allowed
to overdraw the {\em credit\_account} by the amount
given by $credit$.
Note, however, that this is implied by the
virtual definition of $balance()$.
To manage the $credit$ given, the methods $credit$
and $reduce$ are supplied.
This allows us to leave the methods $deposit$
and $withdraw$ unmodified.


</document>
