<document>
@+ <a href=@talk-develop-invariants.html target=_top>Inheritance and invariance</a>
\label{class-invariants}

<include file=navigate>

When developing complex systems or class libraries,
reliability is of critical importance.
As shown in section <ref file=../3/contracts>contracts</ref>,
assertions provide a means by which to check the runtime
consistency of objects.
In particular, assertions may be used to check that
the requirements for behavioral conformance
of derived classes are met.

\cleartrue
<slide id=object-invariant cap="Invariant properties as algebraic laws">
:@ Invariant properties <em> -- algebraic laws</em>
[P
class employee { /// <box>employee</box>
public:
employee( int n = 0 ) : sal(n) { }
employee* salary(int n) { sal = n; return this; }
virtual long salary() { return sal; }
protected:
int sal;
};
P]
:@ Invariant
[P
   k == (e->salary(k))->salary() 
P]
</slide>


%%@ Maintaining invariants
Invariant properties are often conveniently
expressed in the form of algebraic laws that must
hold for an object.
%%See section \ref{validation}.
Naturally, when extending a class by inheritance
(to define a specialization or refinement)
the invariants pertaining to the base class
should not be disrupted.
Although we cannot give a general guideline
to prevent disruption, the example discussed here clearly
suggests that {\em hidden features} should be carefully
checked with respect to the invariance properties
of the (derived) class.
The example is taken from [_Bar92].
<p>
In \sliref{object-invariant}, we have defined a class $employee$.
The main features of an $employee$ are the (protected)
attribute $sal$ (storing the salary of an employee)
and the methods to access and modify the salary
attribute.
For $employee$ objects,
the invariant (expressing that any amount $k$
is equal to the salary of an employee whose
salary has been set to $k$)
clearly holds.
<p>
Now imagine that we distinguish between ordinary employees
and managers by adding a permanent
bonus when paying the salary of a manager,
as shown in \sliref{hidden-bonus}.
The reader may judge whether this example is realistic or not.

\vstr{1}
<slide id=hidden-bonus cap="Violating the invariant">
:@ Problem <em> -- hidden bonus</em>
[P
class manager : public employee { /// <box>manager</box>
public:
long salary() { return sal + 1000; }
};
P]
:@ Invariant
[P
    k =?= (m->salary(k))->salary() 
P]
</slide>

Then, perhaps somewhat to our surprise, we find that
the invariant stated for employees no longer
holds for managers.
From the perspective of predictable object behavior
this is definitely undesirable, since invariants
are the cornerstone of reliable software.
The solution to this anomaly is to make the assignment
of a bonus explicit, as shown in \sliref{explicit-bonus}.

\cleartrue
\vstr{1}
<slide id=explicit-bonus cap="Restoring the invariant">
:@ Solution <em> -- explicit bonus</em>
[P
class manager : public employee { /// <box>manager'</box>
public:
manager* bonus(int n) { sal += n; return this; }
};
P]
:@ Invariant <em> -- restored</em>
[P
     k + n == ((m->salary(k))->bonus(n))->salary() 
P]
</slide>

Now, the invariant pertaining to managers may be strengthened
by including the effects of assigning a bonus.
As a consequence, the difference in salary
no longer occurs as if by magic but is directly
visible in the interaction with a manager object, as it should be.

</document>
