<document>
:@+ <a href=@talk-draw-tool.html target=_top>The <em>drawtool</em> application</a>
\label{DRawtool}
<include file=navigate>


<slide type=fig id=eps-drawtool cap="The <em>drawtool</em> interface">
<image name=@figures/hush/drawtool>
</slide>

In this section we will look at the realization of simple drawing tool.
The example illustrates how to use the
{\em hush} library widgets, and
serves to illustrate
in particular
%%how handlers may be attached
%%to widgets, either by declaration or
%%by inheritance, and
how to construct
compound widgets.
<p>
A structural view of the <em>drawtool</em> application is given
in \sliref{drawtool-structure}.

<slide id=drawtool-structure type=fig cap="A (partial) class diagram">
<image name=@figures/hush/tool>
</slide>

<p>
Usually, the various widgets constituting the user
interface are (hierarchically) related to each other,
such as in the $drawtool$ application which contains
a canvas to display graphic elements,
a button toolbox for selecting the graphic
items and a menubar offering various options
such as saving the drawing in a file.

Widgets in Tk are identified by a {\em path name}.
The path name of a widget reflects its possible
subordination to another widget.
See \sliref{widget-hierarchy}.

<slide id=widget-hierarchy type=fig cap="Widget containment">
<image name=@figures/hush/paths>
</slide>

Pathnames consist of strings separated by dots.
The first character of a path must be a dot.
The first letter of a path must be lower case.
The format of a path name may be expressed in BNF form as
<slide id=paths mode=hide>
\vspace{0.15cm}
<display>
 $path$ ::= '.' <math>|</math> '.'$string$ <math>|</math> $path$'.'$string$
</display>
\vspace{-0.15cm}
</slide>
\noindent
For example `.' is the path name of the root widget,
whereas {P `.quit' P} is the path name of a widget
subordinate to the root widget.
A widget subordinate to another widget
must have the path name of that widget
as part of its own path name.
For example, the widget {P `.f.m' P} may have a widget {P `.f.m.h' P}
as a subordinate widget.
Note that the widget hierarchy induced by the path names
is completely orthogonal to the widget class inheritance
hierarchy.
With respect to the path name hierarchy, when speaking of
ancestors we simply mean superordinate widgets.
<ignore>
<p>
Pathnames are treated somewhat more liberally in {\em hush}.
For example, widget path names may simply be defined
or extended by a string.
The missing dot is then automatically inserted.
</ignore>

<ignore>
It must be pointed out beforehand that the main difficulty
in defining compound or mega widgets is not the construction
of the components themselves, but delegating the configuration
and binding instructions to the appropriate components. 
Section \ref{new} shows
how a compound widget
defined in C++ may be made to correspond
to a widget command that may be used
in a Tcl script.
Ideally, defining a new widget
includes both the definition
of a C++ class interface and
the definition of the corresponding Tcl
command.
</ignore>


Our drawing tool consists of a $tablet$,
which is a canvas with scrollbars
to allow for a large size canvas
of which only a part is displayed,
a {\em menubar}, having a {\em File}
and an {\em Edit} menu, and a {\em toolbox},
which is a collection of buttons for
selecting from among the drawing facilities.
In addition, a help facility is offered.


%%<slide type=hide id=draw-tool-x cap="The drawing tool">
%%@ds drawtool/go.java
%%</slide>


<ignore>
Look at the application class for {\em drawtool}
above.
Before the main event loop is started,
the components of the drawing tool are
created and packed to the root widget.
</ignore>



<slide id=drawtool-interact type=fig cap="An interaction diagram">
<image name=@figures/hush/draw-circle>
</slide>

A typical interaction (or <em>use case</em>) with 
<em>drawtool</em> is depicted in \sliref{drawtool-interact}.
On selecting the <em>circle</em> menu entry (or toolbox button),
the <em>circle handler</em> is activated to assist
in the drawing of a circle. Details will be given
when discussing the <em>tablet</em> widget.

<ignore>
In addition to the {\em tablet}, {\em menubar} and {\em toolbox},
a $frame$ widget is created to pack the
menubar and tablet together.
This is needed to ensure that the geometrical
layout of the widget comes out right.
Each of the component widgets is given
a pointer to the root widget.
In addition, a pointer to the tablet
is given to the {\em toolbox} and a pointer to
the {\em toolbox} is given to the {\em menu\_bar}.
The reason for this will become clear
when discussing the {\em toolbox} and
{\em menubar} in sections \ref{buttons} and \ref{menus},
respectively.
</ignore>

\label{buttons}

:@ The <em>toolbox</em> component
As the first component of <em>drawtool</em>,
we will look at the {\em toolbox}.
The {\em toolbox} is a collection of buttons
packed in a frame. 
%%See \sliref{draw-toolbox}.

<slide type=hide id=draw-toolbox cap="The toolbox class">
@ds drawtool/toolbox.java
</slide>

\noindent
Each button is an instance of
the class {\em toolbutton}.

<slide type=hide id=draw-toolbutton cap="The toolbutton class">
@ds drawtool/toolbutton.java
</slide>

When a toolbutton is created,
the actual button is given the name of the button
as its path.
Next, the button is given the name as its text,
the ancestor widget $w$ is declared to be the 
handler for the button and the button is packed.
The function {\em text} is a member function
of the class {\em button}, whereas both
{\em handler} and {\em pack} are common widget functions.
Note that the parameter {\em name} is used as 
a path name, as the text to display, and as an argument
for the handler, that will be passed
as a parameter when invoking the handler object.


The $toolbox$ class inherits from the $frame$ widget class,
and creates a frame widget with a path relative to the widget parameter
provided by the constructor.
The constructor further creates the five toolbuttons.


The $toolbox$ is both the superordinate widget
and handler for each $toolbutton$.
When the  $operator()$ function of the $toolbox$ is invoked
in response to pressing a button,
the call is delegated to the $mode$ function
of the $tablet$.
The argument given to $mode$ corresponds to
the name of the button pressed.

The definition of the $toolbutton$ and $toolbox$
illustrates that a widget need not necessarily be its
own handler.
The decision, whether to define a subclass which is made
its own handler or
to install an external handler
depends upon what is considered the most convenient
way in which to access the resources needed.
As a guideline, exploit the regularity of the application.

\label{menus}
:@ The <em>menubar</em> component
The second component of our drawing tool is the {\em menubar}.

\cleartrue
<slide type=hide id=draw-menubar cap="The <em> menubar</em>">
@ds drawtool/menubar.java
</slide>

The class {\em menubar}, given above,
is derived from the {\em hush} widget $menubar$.
Its constructor requires an ancestor widget,
a {\em tablet} and a {\em toolbox}.
The tablet is passed as a parameter to the {\em file\_menu},
and the $toolbox$ to the {\em edit\_menu}.
In addition, a {\em help\_button} is created, which provides
online help in a hypertext format when pressed.
%%The help facility will be discussed in section \ref{hyper}.
<p>
A menubar consists of menubuttons to which actual menus are attached.
Each menu consists of a number of entries,
which may possibly lead to cascaded menus.


%%\cleartrue
%%<slide type=hide id=draw-file cap="The <em> EditMenu</em>">
%%@ds drawtool/EditMenu.java
%%</slide>


The second button of the {\em menubar} is defined by the
{\em EditMenu}.
The {\em EditMenu} requires a $toolbox$ and creates
a menubutton.
It configures the button and
defines a menu containing two entries,
one of which is a cascaded menu.
Both the main menu and the cascaded menu
are given the $toolbox$ as a handler.
This makes sense only because 
for our simple application
the functionality offered by
the $toolbox$ and {\it EditMenu} coincide.

\vspace{0.5cm}
<slide type=fig cap="Tablet">
<image name=@figures/hush/tablet>
</slide>

<latex>\mbox{}\vspace{-1.0cm}\mbox{}</latex>
:@ The <em>tablet</em> component
The most important component of our {\em drawtool}
application is defined by the {\em tablet} widget class
given below.
%%depicted in \sliref{draw-tablet}.

\vfig{1}
<slide type=hide id=draw-tablet cap="The {\em tablet}">
@ds drawtool/tablet.java
</slide>

The various modes supported by the drawing tool are
enumerated as final constants.
The {\em tablet} class itself inherits from the $canvas$
widget class.
This has the advantage that it offers the full functionality
of a canvas.
In addition to the constructor and $operator()$ function,
which delegates the incoming event to the appropriate
handler
according to the {\em \_mode} variable,
it offers a function $mode$, which sets the mode of the
canvas as indicated by its string argument,
and a function {\em init} that determines the creation
and geometrical layout of the component widgets.
As instance variables, it contains an integer {\em \_mode}
variable and an array of handlers that contains
the handlers corresponding to the modes supported.
%%See section \ref{canvas} for an example of a typical
%%canvas handler.


%%@ Dispatching
Although the $tablet$ must act as a canvas,
the actual $tablet$ widget is nothing but a $frame$
that contains a canvas widget as one of its components.
This is reflected in the invocation of the
canvas constructor ($super$).
By convention, when the options parameter is $*$
instead of the empty string, no actual widget
is created but only an abstract widget,
as happens when calling the $widget$ class constructor.
Instead of creating a canvas right away,
the $tablet$ constructor creates a top frame,
initializes the actual component widgets,
and redirects the $eval$, $configure$ and $bind$
invocations to the subordinate $canvas$
widget.
It then
binds itself to be its own handler,
which results in binding itself to be the handler for the
canvas component.
Note that reversing the order of calling $redirect$
and $bind$ would be disastrous.
After that it creates the handlers for the various modes
and sets the initial mode to $move$.

The $operator()$ function takes care of dispatching calls
to the appropriate handler.
The $dispatch$ function must be called to pass
the $tk$, $argc$ and $argv$ parameters.

:@ The <em>drawtool</em> class
Having taken care of the basic components
of the drawing tool, that is the {\em toolbox},
{\em menubar} and {\em tablet} widgets, all that remains
to be done is to define
a suitable {\em file\_handler},
appropriate handlers for the
various drawing modes
and a {\em help\_handler}.

%%<slide id=drawtool-class type=fig cap="A (partial) class diagram">
%%<image name=@figures/hush/tool>
%%</slide>

We will skip these, but look at the
definition of the <em>drawtool</em> class instead.
In particular, it will be shown how we may grant
the {\em drawtool} the status of a veritable
Tk widget, by defining a
$drawtool$ handler class
and a corresponding $drawtool$ widget command.

<slide type=hide id=drawtool-code cap="The <em>drawtool</em> class">
@ds drawtool/drawtool.java
</slide>

Defining a widget command involves three steps: (I) 
the declaration of the binding between
a command and a handler,
(II) the definition of the $operator()$
function, which actually defines a mini-interpreter,
and (III) the definition of the actual
creation of the widget and its declaration
as a Tcl/Tk command.

Step (I) is straightforward.
We need to define an empty handler, which will
be associated with the $drawtool$ command
when starting the application.
%%See \sliref{new-command} (a).
The functionality offered by the
interpreter defined by the $operator()$
function in (II) is kept quite simple,
but may easily be extended.
When the first argument of the call is $drawtool$,
a new $drawtool$ widget is created as specified in (III),
except when the second argument is $self$.
In that case, 
the virtual
path of the widget is returned, which is actually
the path of the $tablet$'s canvas.
It is the responsibility of the writer of the script
that the $self$ command is not addressed to the
empty handler.
If neither of these cases apply,
the function $eval$ is invoked
for $self()$, with the remaining arguments flattened
to a string.
This allows for using the drawtool almost as an ordinary
canvas. 
%%See the example hypertext script shown in section \ref{hyper}.
<ignore>
The creation of the actual widget and
declaration of the corresponding Tcl command,
according to the Tk convention,
is somewhat more involved (III).


Recall that each Tk widget is identified by
its path, which simultaneously
defines a command that may be used
to configure the widget or,
as for a canvas,
to draw figures on the screen.
Hence, the function $create$ must 
create a new widget and declare the widget
to be the handler of the command
corresponding to its path name.
</ignore>


<slide type=hide id=new-command cap="The drawtool application">
<program>
%%public int main() {   // in the Application class
%%	debug("Application.main (Java) ...");
%%	tk.trace();
	Canvas c = new DrawTool("draw","");
	tk.bind("drawtool",c);
	c.circle(20,20,20,"-fill red");
	c.rectangle(30,30,70,70,"-fill blue");
	c.pack();
%%
%%	tk.pack("quit");
%%	return OK;
%%	}
</program>
</slide>


In the program fragment above,
the Tcl command $drawtool$ is declared,
with an instance of {\em drawtool} as its handler.
(It is assumed that the $tk$ variable refers
to an instance of $kit$.)
In this way, the $drawtool$ widget
is made available as a command when
the program is used as an interpreter.
In this case, the actual $drawtool$ widget
is made the handler of the command,
to allow for a script to
address the $drawtool$ by calling
{\em drawtool self}.
</document>
