<document>
@+ <a href=@talk-dimensions-ortho.html target=_top>Towards an orthogonal approach -- type extensions</a>
\label{orthogonal}

<include file=navigate>

According to [_Wegner87], much of the confusion
around the various features of object-oriented
programming languages arises from the fact that
these features are largely interdependent,
as for instance the notion of object and class on 
the one hand,
and the notion of class and inheritance on the other.

To resolve this confusion, [_Wegner87] proposes
a more orthogonal approach to characterize the
various features of object-oriented languages,
according to dimensions that are to a large extent independent.
\see{5-orthogonal}


\vstr{1}
<slide id=5-orthogonal cap="Orthogonal dimensions">
@ Orthogonal approach
<ul>
<li> {\bf objects} -- {\em modular computing agents}
<li> {\bf types} -- {\em expression classification}
<li> {\bf delegation} -- {\em resource sharing}
<li> {\bf abstraction} -- {\em interface specification}
%%<hr>
%%<li> {\bf type}: behavior specification\n
%%<li> {\bf delegation}: ancestors (with dynamic self-reference)
</ul>
</slide>

The features that constitute an object-oriented
programming language in an orthogonal way are,
according to [_Wegner87]: {\em objects}, {\em types},
{\em delegation} and {\em abstraction}.

@ Objects
are in essence {\em modular computing agents}.
They correspond to the need for encapsulation in design,
that is the construction of modular units to
which a principle of locality applies (due to
combining data and operations).

Object-oriented languages may, however, differ in
the degree to which they support encapsulation.
For example, in a distributed environment a 
high degree of encapsulation must be offered,
prohibiting attempts to alter global variables
(from within an object) or local instance
variables (from without).
Moreover, the runtime object support system must
allow for what may best be called {\em remote method invocation}.

As far as parallel activity is concerned, only a few languages provide
constructs to define concurrently active objects.
%%See chapter \ref{communication} for a more detailed discussion.
See section <ref file=dimensions-active>Active</ref> for a more detailed discussion.

Whether objects support {\em reactiveness},
that is sufficient flexibility to respond safely
to a message, depends largely upon (program) design.
[_Meyer88], for instance, advocates a {\em shopping list
approach} to designing the interface of an object,
to allow for a high degree of (temporal) independence
between method calls.

@ Types
may be understood as a mechanism for {\em expression classification}.
From this perspective, Smalltalk may be regarded
as having a dynamic typing system:
dynamic, in the sense that the inability to evaluate an expression
will lead to a runtime error.
The existence of types obviates the need to have classes,
since a type may be considered as a more abstract description of the
behavior of an object.
Furthermore, subclasses (as may be derived through inheritance)
are more safely defined as subtypes in a polymorphic
type system. See section \ref{flavors}.

At the opposite side of the type dimension we find the statically typed languages,
which allow us to determine the type of 
the designation of a variable at compile-time.
In that case, the runtime support system need not carry any type information,
except a dispatch table to locate virtual functions.

@ Delegation
(in its most generic sense)
is a mechanism for {\em resource sharing}.
As has been shown in [_Lieber], delegation subsumes inheritance,
since the resource sharing effected by inheritance may easily be mimicked
by delegating messages to the object's ancestors
by means of an appropriate dispatching mechanism.

In a narrower sense, delegation is usually understood as a more
dynamic mechanism that allows the redirection of control dynamically.
In addition, languages supporting dynamic delegation
(such as Self) do
not sacrifice dynamic self-reference.
This means that when the object executing
a method refers to itself, the actual context
will be the delegating object.
See section \ref{prototypes} for a more detailed discussion.

In contrast, inheritance (as usually understood in the context of classes)
is a far more static mechanism.
Inheritance may be understood as (statically) copying the code 
from an ancestor class to the inheriting class
(with perhaps some modifications),
whereas delegation is truly dynamic in that messages are dispatched
to objects that have a life-span independent of the dispatching object.

@ Abstraction
(although to some extent related to types) 
is a mechanism that may be independently applied to provide
an {\em interface specification} for an object.
For example, in the presence of active objects
(that may execute in parallel)
we may need to be able to restrict dynamically the interface of an object as specified by its type
in order to maintain the object in a consistent state.
Also for purely sequential objects we may impose a particular protocol of interaction
(as may, for example, be expressed by a {\em contract}) to be able to
guarantee correct behavior.

Another important aspect of abstraction is {\em protection}.
Object-oriented languages may provide (at least) two kinds
of protection.
First, a language may have facilities to
protect the object from illegal access by a client (from without).
This is effected by annotations such as {\em private} and {\em protected}.
And secondly, a language may have facilities to protect the object
(as it were from within) from illegal access through delegation
(that is by instances of derived object classes).
Most languages support the first kind of protection.
Only few languages, among which are C++ and Java, support the second kind too. 

The independence of abstraction and typing may further
be argued by pointing out that languages
supporting strong typing need not
enforce the use of abstract data types having a well-defined
behavior.



<ignore>
%%:@ Discussion
Currently there is no single language providing
what in any sense can be called an ideal or
optimal combination of features.
At least there is no consensus as to whether
Smalltalk, Eiffel, C++ or any of the other
languages mentioned is the likely candidate
for this position.
A critique on many of these languages is that they
contain too many features or try to serve too many goals.
Think of Smalltalk's meta classes (see section \ref{meta}),
Eiffel's attempt to be satisfying both from
a practical and formal point of view
(which severely limits the power of the assertion mechanism)
and the intricate type construction and coercion
operators of C++ (which make it flexible, but complex).
An interesting example of what may be called
{\em minimal language design}
is presented by the language Oberon
that features so-called {\em type extensions}
encompassing (in a {\em minimal} way) both the notion
of object and class inheritance.

."@ Type extensions
The Oberon language has been developed
as the (object-oriented) successor of Modula-2.
According to [_BPS89], simplicity was the prime goal
of its design.
Oberon is a procedural language (like Modula-2)
with strong static type checking.
It provides only simple data types, arrays and records.
A class concept is lacking.

Oberon, however, supports records that may
contain (so-called {\em installed}) procedures.
And, more importantly, records can be extended.
Extension works like inheritance, 
in that a record type can be defined that
includes all the components of the extended
(read inherited) record type.
In Oberon a record that is the result
of extending a given record type
is called {\em compatible} with the original
record type,
which means that it may be assigned to a variable holding
a record of that (base) type.
When such an assignment is made, the extended
record is projected onto the original
record (type),
ignoring the fields that are added due to the extension.

In a similar way as Modula-2,
Oberon supports separate compilation by requiring
a definition part (containing an interface specification)
and an implementation part.
Modules may have private classes (that is records),
just by omitting them from the interface specification.
Oberon allows the dynamic creation of (record) objects.
A garbage collector takes care of their elimination.
As far as inheritance is concerned, Oberon only supports single
inheritance.
In other words, record extension only operates 
on a single (base) record.
Moreover, inheritance in Oberon is strict,
due to the restriction that no overriding is allowed.
Viewed from the principle {\em small is beautiful},
Oberon indeed is a very nice language, containing a
minimal set of features to support object-oriented
programming.
However, from a programming perspective,
as a vehicle to implement concrete data types
(see section \ref{canonical}) it leaves
much to be desired.
Not surprising, then, that even purported
{\em object-oriented} extensions to Oberon have been proposed.
See [_BPS89] for a further discussion.
</ignore>
</document>
