<document>
@+ <a href=@talk-paradigm-ext.html target=_top>Object extensions of Lisp, C and Prolog</a>
<include file=navigate>

The notion of object is to a certain extent orthogonal
to, that is independent of, language constructs around
which programming languages may be constructed,
such as expressions, functions and procedures.
Hence, it should come as no surprise that a number of (popular)
object-oriented programming languages were originally
developed as extensions of existing languages or
language implementations.
\see{5-extensions}

<slide id=5-extensions cap="Object-oriented languages">
@ Object extensions
<I
- Lisp -- LOOPS, FLAVORS, CLOS, FOOPS
- C -- Objective C, C++
- Prolog -- SPOOL, VULCAN, DLP
I>
:@ Commercial products <em> -- languages</em>
*- Smalltalk, Eiffel, C++, Objective C, Object Pascal, Java
</slide>

The advantage of extending an existing language with
object-oriented constructs, from the point of view
of the user, is that the object-oriented approach can be
gradually learned.
However, at the same time this may be a disadvantage, since a hybrid
approach to software development may give rise to sloppy design.
Many proponents of an object-oriented approach, therefore, believe
that learning to use object-oriented constructs
is best done in an environment as offered by Smalltalk,
where classes and objects are the sole means of developing 
an application.

It is noteworthy that, with the exception of Smalltalk,
Eiffel and Java, many commercially available languages
are actually extensions of existing languages such
as Lisp, C and (to some extent) Prolog. 


@ Lisp-based extensions
In [_Sau89], ten Lisp-based object-oriented languages
are mentioned, among which are
LOOPS (introducing a variety of object-oriented constructs,
see [_LOOPS]),
Flavors (which extends Lisp by adding generic functions
that operate on objects, see [_Moon]),
and CLOS (which is actually a standardization effort
of the ANSI X3J13 group to define the Common Lisp Object Standard).
CLOS is a widely used system containing some
non-trivial extensions to the object model
and the way in which polymorphic methods may be defined.
%%See section \ref{adt-theory} for further discussion.

@ C-based extensions
Another very important class of object extensions
is those of C-based object-oriented languages,
of which the most well-known are Objective-C and C++.

The concepts underlying these two extensions
are radically different.
Objective-C introduces objects as an add-on to
the constructs (including {\it struct}s) available
in C,
whereas C++ realizes a close (and efficient)
coupling between the {\it struct} (record)
notion of C and the concept of a class.

In other words, in Objective-C there is a clear
distinction between conventional C values and data types
such as $int$, $float$ and $struct$ on the one hand, and
objects on the other hand.
Objects have a special data type ($id$) which allows
them to be treated as first class elements.
To define an object class, both an {\em interface}
and {\em implementation} description must be given.
These descriptions are preceded by a special sign 
to designate Objective-C specific code.
Also, method declarations (in the interface description)
and method definitions (which are to be put in the
implementation section) must be preceded by a special sign
to designate them as methods available
for clients of object instances of that class.

The object model of Objective-C
%%closely resembles
is similar to
the object model of Smalltalk.
In contrast, C++ quite radically departs from this object 
model in order to achieve an as efficient as possible implementation of
objects.
The key to an efficient implementation lies
in the integration of the {\em struct} (record) construct
originally provided by C with the class concept,
by allowing functions to be members of a {\it struct}.

As explained in [_St91], the equivalences depicted in
\sliref{5-structure} hold.


\cleartrue
<slide color=red id=5-structure cap="The equivalence between <em>class</em> and <em>struct</em>">
@ Object structure <em> -- efficient mapping</em> 
<p align=right><box>C++</box>
\vspace{0.3cm}
<program>
struct A { ... }  ==  class   A { public: ...  } 
class   A { ... }  ==  struct A { private: ... } 
</program>
\vspace{-0.3cm}
</slide>

This interpretation allows an efficient mapping
of object structures to the memory of a computer,
provided that the compiler is clever enough.

Nevertheless, the efficiency of C++ comes
at a price. C++ does support micro-efficiency
but does not necessarily lead to the design of
efficient code.
In particular, hand-crafted memory management
will not necessarily offer the most efficient
solution when compared with built-in support,
but is almost certainly detrimental to the quality of the code.

<ignore>
As argued in [_ES90], the design of C++ is intended to
allow an efficient compilation of C++ into C,
that is efficient C code.
Yet, the efficiency of C++ is not
due to its being compiled into C,
but comes from an efficient mapping of object (type)
structures and class member functions to analogous C structures
and functions that require a minimum of overhead.


In effect, C is increasingly being used as a target
language for the compilation of high level languages.
The reason for this is, however, primarily {\em portability},
and not efficiency.
For example, Eiffel compilers (both under Unix and MS-DOS)
produce C code and also allow Eiffel programs to
interface with programs written in C.
However, the overhead due to the exception handling primitives
of Eiffel and the overhead in calling functions defined 
in C (still) results in quite inefficient code,
despite the translation into C.

As portability is concerned,
many authors optimistically claim
the portability of their C++ code
(across platforms and compilers).
However, portability has not been realized
for C++ as it has for ANSI-C.
Currently available compilers (including
AT\&T 3.0 and those of Borland, GNU and Zortech)
definitely disagree on the handling of, for
instance, multiple inheritance, pure virtual functions,
templates and overloading.
Hopefully, the standardization efforts of the C++ ANSI committee
will help to reduce these problems.
</ignore>

@ Prolog-based extensions
A quite different class of object-oriented extensions,
used primarily in research laboratories,
consists of attempts to incorporate object-oriented
features in (high-level) logic-based languages, such as Prolog.
Among these are languages such as SPOOL (developed
in the context of the Japanese fifth-generation computing
project, see [_Fukanaga]),
Vulcan (that provides a preprocessor giving syntactic support for embedding objects in
concurrent logic programming languages, see [_Kahn])
and DLP (a language combining logic programming
with object-oriented features and parallelism developed by the author,
see appendix <ref file=@online/oo/appendix/dlp/dlp>DLP</ref>).
The list of research articles covering the subject of combining
logic programming and object-oriented programming
is quite extensive.
An overview and discussion of the various approaches
is given in [_Davison93] and also in [_Eliens92].
</document>
