<document>

@* Crush -- extending <em>hush</em> with CORBA
\label{Crush}

<include file=navigate>


This section describes how the <i>hush</i> toolkit
has been extended with CORBA functionality.
The nickname for this effort was <em>crush</em>.
The major problem that arises when extending
a given toolkit or framework, such as <i>hush</i>,
with CORBA IDL interfaces and classes to
implement these interfaces is to provide for
a seamless integration of the already existing
code with the CORBA-based extensions.
In <i>crush</i> we have included facilities
for object creation and access, as well as client-side
adaptors for the <i>hush</i> CORBA objects,
to resolve the type clash between the original <i>hush</i> 
class hierarchy and the <i>hush</i> CORBA object classes.

<slide cap="Extending a framework with CORBA">
:@ Extending a framework with CORBA
<ul>
<li> the legacy problem -- <em>granularity of wrappers</em>
<li> object creation and access -- <em>factories and tables</em>
<li> client-side adaptors -- <em>to fit within native type system</em>
<li> events versus objects -- <em>natural interfaces</em>
</ul>
</slide>


Extending a given framework with CORBA is not as straightforward
as it may seem.
First of all, one has to decide which interfaces
may become public, that is may be exported as
IDL interfaces.
Secondly,
one has to decide how object references become known
to clients, and what rights clients have
to create objects within a particular server.
The most important problem, however, concerns
the type clash between the CORBA classes
implementing the IDL interfaces
and the `native' class hierarchy
offered by the framework itself.

:@ The legacy problem -- integrating CORBA
CORBA technology is well suited to develop
distributed applications.
For new projects, the restrictions imposed by
CORBA can be taken into consideration from the start.
For projects that carry the legacy of existing code,
a decision must be made to what extent the
CORBA functionality is integrated with the legacy code.
On one side of the spectrum,
CORBA technology can be used simply for wrapping the legacy
code.
For example a database may be embedded
in a CORBA server, without affecting the database itself.
However, for an object-oriented framework such as <i>hush</i>
such a solution is not very satisfying.
Instead, one would like to have the basic
interfaces of <i>hush</i> available to develop
distributed components of arbitrary granularity.

:@ Object creation and access
The CORBA Naming Service may be used to provide access to
an object residing somewhere on a server.
Alternatively, the server may export a reference to
a factory object that allows the client to create objects within the
server.
<p>
For giving access to objects within a particular <i>hush</i>
component, we have provided <i>dots</i> (distributed object tables)
for both <i>hush</i> and the <i>widgets</i> components.
Using the <i>dot</i> the client can access an object of a given type by
the name it is given by the server.
The object must already exist in the server.
<p>
In case clients are allowed to create objects within
the server, a <i>factory</i> is provided for creating <i>hush</i>
or <i>widget</i> objects.

:@ Client-side adaptors
The intermediary between clients and servers in a CORBA-based system
are the CORBA IDL classes generated by the <i>idl</i> compiler
from the IDL interfaces.
These classes (using the C++ language binding)
inherit directly from the CORBA::Object class and hence 
do not fit within the given class hierarchy.
<p>
To allow clients the use of CORBA IDL classes wherever one
of the original <i>hush</i> classes is expected,
client-side adaptors have been provided for each of the <i>hush</i>
or <i>widgets</i> IDL classes.
An additional advantage of client-side adaptors is that
they allow for overcoming the `weaknesses' of IDL with respect to
overloading member functions,
parametrized types and operator definitions.
<p>
Typically, client-side adaptors have their corresponding <i>hush</i> class
as a base class and simply delegate method invocations
to the CORBA object they encapsulate.

:@ Events versus object method invocation
Since GUI components are in some way typically
event-driven, one may be inclined to limit
the communication between such components to exchanging events.
The CORBA Event Service would suffice for such communications.
<p>
Nevertheless, in our opinion events should be used in a very restricted manner.
Events tend to break the `crisp' object interfaces that
are one of the benefits of an object-oriented
approach to design.
<p>
For the <i>hush</i> CORBA extensions, we have chosen for retaining the original <i>hush</i>
object interfaces.
Note however that the IDL interfaces are somewhat more
abstract than the corresponding C++ interfaces.
Nevertheless, the <i>event</i> interface is part
of the <i>hush</i> module.
Together with the <i>dispatch</i> function of 
the <i>handler</i> interface incoming events resulting from user 
actions may be dispatched directly to
remote components.
%%See the <ref file=examples>Canvas</ref> example.

:@+ <a href=@talk-idl.html target=_top>Interfaces</a>


The IDL interfaces reflect to a large extent the functionality of
the original <i>hush</i> and <i>widgets</i> interfaces.
In this section a partial listing of the interfaces will be given.
In comparison with the corresponding C++/Java classes,
the IDL interfaces are much more abstract in the sense that
many member functions required 
for the actual implementation of the <i>hush</i> framework
may be omitted.

:@- The <i>hush</i> module

The <i>hush</i> module contains interfaces corresponding to
the basic <i>hush</i> classes,
<i>handler</i>, <i>event</i>, <i>kit</i>, <i>widget</i>
and <i>item</i>, as well as the auxiliary classes
for iterators and containers.

<slide type=hide id=idl-handler cap="handler">
<program>
interface handler { /// <box>handler</box>
        event dispatch( in event data );
	};
</program>
</slide>

The <i>handler</i> interface provides only a method
for dispatching events. It may be extended in the
future though.
In <i>hush</i> almost every class is derived from <i>handler</i>.
This is directly reflected in the <i>hush</i> IDL interfaces.

<slide type=hide id=idl-event cap="event">
<program>
interface event : handler { /// <box>event</box>

	attribute long type;

	attribute long x;
	attribute long y;
	};
</program>
</slide>

The <i>event</i> interface offers attributes to
determine the type of event and its location.
Also the <i>event</i> interface will very likely be extended in
the future, to allow for a greater variety of events.

<slide type=hide id=idl-kit cap="kit">
<program>
interface kit : handler { /// <box>kit</box>

	void source(in string file);
	void eval(in string command);
	string result();

	widget root();
	};
</program>
</slide>

In <i>hush</i>, a <i>kit</i> provides an interface to
some embedded interpreter, such as a Tcl interpreter
or a logic engine.
The <i>kit</i> also gives access to the underlying window environment;
in particular it may be asked to provide a reference to
the root window.

<slide type=hide id=idl-widget cap="widget">
<program>
interface widget : handler { /// <box>widget</box>

   string path();

   void eval( in string cmd );

   void configure( in string options );
   void pack( in string options );
};
</program>
</slide>

A widget is a user interface gadget.
The <i>widget</i> interface collects the functions that
all these gadgets have in common.

<slide type=hide id=idl-item cap="item">
<program>
interface item : handler { /// <box>item</box>

        void move( in long x, in long y );
	};
</program>
</slide>

An <i>item</i> is obtained when creating a graphical
object for a canvas. Subsequently, the item reference
suffices to manipulate such objects.
Also the <i>item</i> interface will very likely
be extended in the future.


:@ Iterators and lists
As an alternative for
CORBA arrays and sequences, the <i>hush</i>
module offers interfaces for iterators and containers.
%%syntactic objects called <i>terms</i>.

<slide type=hide id=idl-iter cap="iterator">
<program>
interface iterator { /// <box>iterator</box>
        Object next();
	};
</program>
</slide>

From a client's perspective, an <i>iterator</i>
is a data generator.
To deal with typed iterators, the <i>hush</i>
C++ library offers template client-side adaptor classes
encapsulating the untyped CORBA iterators.
%%See section <ref file=examples>Items</ref> and the code
%%in <ref file=code>Iter</ref>.

<slide type=hide id=idl-container cap="container">
<program>
interface container { /// <box>container</box>

        long length();

        Object first();
        Object next();
        Object current();

        iterator walk();
	};
</program>
</slide>

The <i>container</i> interface offers access
to the <i>hush</i> list class.
It offers functions for cursor-based list traversal
as well as the <i>walk</i> function that may be used to obtain an iterator.

:@ Factories and distributed object tables
To obtain references to objects, clients may use
either factory object or distributed object tables.

<slide type=hide id=idl-factory cap="factory">
<program>
interface factory { /// <box>factory</box>

        hush::kit kit(in string name);
        hush::event event(in long type);
};
</program>
</slide>

The <i>factory</i> interface allows only for creating 
a <i>kit</i> and for creating an <i>event</i>.
Note that <i>handler</i> objects may not be created directly.

<slide type=hide id=idl-dot cap="dot">
<program>
interface dot {  /// <box>dot</box>

        hush::kit kit(in string name);

        hush::container container(in string name);
        hush::iterator iterator(in string name);

        hush::factory hush(in string name);
};
</program>
</slide>

Apart from giving access to a <i>hush</i> factory, 
the <i>dot</i> interface allows for getting access to
a <i>kit</i>, a <i>container</i> and
an <i>iterator</i>.
When obtaining references through a <i>dot</i> object,
these objects are assumed to exist within the server.


:@- The <i>widgets</i> module

The <i>widgets</i> module provides the actual user interface
gadgets for <i>hush</i>.
Below we have  included only the (partial) interfaces for
a <i>canvas</i> and a <i>message</i> widget.

<slide type=hide id=idl-widgets cap="module widgets">
<program>
module widgets {

interface canvas : hush::widget { /// <box>canvas</box>

	canvas create( in hush::widget anc, in string path );

	hush::item circle( in long x, in long y, in long radius, in string options );
	// other items ...
	};


interface message : hush::widget { /// <box>message</box>

	message create( in hush::widget anc, in string path );
	void text(in string txt);
	};


interface factory : hush::factory { /// <box>factory</box>

 	widgets::canvas  canvas(in string name, in string options);
 	widgets::message message(in string name, in string options);
	};

interface dot : hush::dot {  /// <box>dot</box>

	widgets::canvas  canvas(in string name);
	widgets::message message(in string name);

	widgets::factory widgets(in string name);
	};


};
</program>
</slide>

Note that each widget type has a method <i>create</i>,
with which  an actual widget of that type can be created.
In effect this means that each widget object may act as a factory
for widget objects of that type.
(The server may however refuse to create such objects!)
In addition to the specific gadget interfaces, the <i>widgets</i> module
provides a <i>factory</i> and <i>dot</i> interface,
extending the respective <i>hush</i> interfaces.

:@+ Examples

The <i>hush</i> CORBA extensions may be used in a number of ways.
For example, the client does not need to be linked with <i>hush</i>
when only the server side is given a graphical user interface.
In the case that also the client has a graphical user interface,
the client side may dispatch incoming events to the server,
as illustrated in the canvas example.
The communication between server and clients can be arbitrarily complex.
The final example shows how to employ iterators and containers
to give clients accesses to collections of information.

:@- A remote interpreter kit

This example uses a remote kit. It shows how to get access to a message widget and a kit via a dot
(which is a distributed object table). The client access the kit and the message widget by using a name
(hello for the message widget and tk for the kit). 
The client can send Tcl/Tk interpreter commands to the
kit. 

<slide type=hide id=exampel-tk-client cap="A tk client">
%%@ A <i>tk</i> client

/// <box>client</box>
<program>
    hush::dot*      hush;      // (distributed) object tables
    widgets::dot*   widgets;   // widgets contains hush

    hush::kit* tk;             // remote kit object
    widgets::message* banner;

    try {
        hush = widgets = widgets::dot::_bind (SERVER, argv[1]);

        tk = hush->kit("tk");
        banner = widgets->message("hello"); // must exist

    } catch (...) {
         cerr << "Unexpected exception ..." << endl;
         return -1;
    }

    while (1) {
	char text = readtext(); // from stdin
	
	banner->text( text );  // display text
	tk->eval(text);
        }
</program>
</slide>

This fragment shows how a distributed object table is obtained
via the <i>bind</i> function.
From this table, the client obtains a kit and a message area.
Queries are read in from standard input, displayed in the message area and 
evaluated.
Queries may be arbitrary Tcl/Tk commands.
In this way the client may even construct a complete user interface
through Tk commands.

<slide type=hide id=example-tk-server cap="The <i>tk</i> server">
%%@ The <i>tk</i> server

/// <box>server</box>
<program>
class application : public session {
public:
application(int argc, char* argv[]) : session(argc,argv,"hello") {
}
void corba();
int main() {
    tk->trace();
    kit::declare("tk",tk);

    message* m = new hello(".hello"); 
    m->pack();

    message::declare("hello",m);

    corba(); // make yourself available as a server

    return OK;
    }
};
</program>
</slide>

The server is realized as a standard <i>hush</i> program,
except for the call to <i>corba</i> (for which the code is given below).
Note that the calls to <i>declare</i> for both the <i>kit</i>
and <i>message</i> objects is needed to make these objects
accessible via the <i>dot</i>.


<slide type=hide id=example-tk-application cap="A tk application">
%%@ A tk application
<program>
void application::corba() {

widgets::dot* dw = new widgets_dot_srv(); // create dot for widgets

try {
 CORBA::Orbix.registerIOCallback(it_orbix_fd_open, FD_OPEN_CALLBACK);
 CORBA::Orbix.registerIOCallback(it_orbix_fd_close, FD_CLOSE_CALLBACK);

 CORBA::Orbix.impl_is_ready(SERVER,0);
 CORBA::Orbix.processEvents(0);
 }
catch (...) {
     cout << "apparently something went wrong" << endl;
   }
</program>
</slide>

In <i>application::corba()</i> a distributed object table is created.
This object is exported as a server by a call to
<i>Orbix.impl_is_ready(SERVER,0)</i>,
where SERVER is a macro defining the name of the server.
Calling <i>registerIOCallback</i> is needed to
merge the (Orbix) CORBA server event loop with the window event loop
for <i>hush</i>.


:@- Evaluating logical queries
With a few minor changes, the client program can be adapted
for accessing a logical query evaluator. 

<slide type=hide id=example-logical cap="Evaluating logical queries">
%%@ Evaluating logical queries
/// <box>client</box>
<program>
    try {

        tk = hush->kit("bp");       // A kit for BinProlog
        tk->eval("consult(facts)");
        }
    catch(...) {
        cout << "An exception ... " << endl;
    }

    while (1) {
        char* text = readtext();
        tk->eval(text);
        char* q = 0;
        while ( (q = tk->result()) )
                cout << "Result: " << q << endl;
	}
</program>
</slide>

This fragment show how to obtain a kit for BinProlog and
consult a <i>facts</i> database.
Since queries may produce multiple answers
the client must iterate over the term resulting
from the query.

:@- A remote canvas
<a name=Canvas>

This example shows how a client canvas can be used to draw on a remote canvas.

<slide type=hide id=example-canvas-client cap="A (remote) canvas client">
%%@ A canvas client

<program>
class draw_clt : public canvas {  /// <box>draw_clt</box>
public:

void plug(widgets::canvas* x) { draw = x; }

int operator()() {
        hush::event* e = hush->event(_event->type());
        cerr << "Getting event " << e->type() << endl;
        e->x(_event->x()+10);
        e->y(_event->y()+10);
        hush::event::_duplicate(e);  // CORBA 2.0
        hush::event* res = draw->dispatch(e);
        return canvas::operator()();
        }

draw_clt(const widget* w,  char* path ) : canvas(w,path) {  
        configure("-background white");
        geometry(200,100);
        self()->bind(this);
        dragging = 0;
}

draw_clt(char* path ) : canvas(path) { 
        configure("-background white");
        geometry(200,100);
        self()->bind(this);
        dragging = 0;
}

void press( event& ) { dragging = 1; }

void motion( event& e) { 
        if (dragging) {
                self()->circle(e.x(),e.y(),2,"-fill black");
                draw->circle(e.x(),e.y(),3,"-fill yellow");
                }
}

void release( event&  ) { dragging = 0; }

protected:
int dragging;
widgets::canvas* draw;
};
</program>
</slide>

This fragment shows the implementation of
a canvas which is simultaneously the client side of a remote canvas. 
The method <i>plug</i> allows for declaring the remote canvas,
which is accessed via the instance variable <i>draw</i> in
both the <i>operator</i> method and
the <i>motion</i> method (when dragging).
In the <i>operator</i> method an event is created which is dispatched to
the remote canvas.
Note that this is possible since a canvas is a handler.
In the <i>motion</i> method, a large yellow dot is drawn
on the remote canvas, whereas the local canvas draws a black dot.
Combined, the actions on the remote canvas result in 
drawing parallel yellow and black dots.


%%@ The server

<slide type=hide id=example-canvas-server cap="A canvas server">
%%@ A canvas server
<program>
class draw_srv : public canvas { /// <box>draw_srv</box>  
public:

draw_srv( const widget* w,  char* path ) : canvas(w,path) {  
        geometry(200,100);
        self()->bind(this);
        dragging = 0;
}

void press( event& ) { dragging = 1; }

void motion( event& e) { 
        if (dragging) circle(e.x(),e.y(),10,"-fill black");
}

void release( event&  ) { dragging = 0; }

protected:
int dragging;
};
</program>
</slide>

The canvas implementation on the server side straightforwardly
implements a <i>hush</i> canvas.
It is embedded in a CORBA server when an object reference
is given to it via the distributed object table.

:@- Moving items
<a name=Items>

This example is similar to the canvas example, but shows some additional features, such as how to
manipulate a list of items.

<slide type=hide id=example-items cap="Moving items">
%%@ Moving items
/// <box>server</box>
<program>
    list<hush::item>* rlist =  new list<hush::item>;
    item* it = draw->circle(40,40,10,"-fill yellow");
    hush::item* rit = new item_srv(it);
    rlist->insert(rit);
    it = draw->circle(30,30,10,"-fill red");
    rit = new item_srv(it);
    rlist->insert(rit);

    hush::container* rx = new list_srv<hush::item>(rlist);
    list<hush::item>::declare("items",rx); // store server

    iter<hush::item>* riter = rlist->walk();
    iter<hush::item>::declare("riter",riter);
</program>
</slide>

The fragment above illustrates the creation of a list
of items.
In addition it shows how to obtain an iterator and 
how the iterator may be declared to make it accessible
via the distributed object table.

:@ Discussion
From a design point of view,
the <i>hush</i> framework proved to be
sufficiently abstract to allow for
recapturing its design by means of IDL interfaces.
Lacking in the current realization of <i>crush</i> though,
are proper exceptions to indicate possible error conditions.
<p>
This work 
shows how to integrate CORBA functionality
with an already existing framework.
In particular the need for client-side adaptors
for resolving the type clash between the `native' classes
and the CORBA IDL classes has been amply demonstrated.
Enriching <i>hush</i> with CORBA makes <i>crush</i>
a potential competitor of Fresco, the CORBA based GUI toolkit
derived from the Interviews library.

</document>
