<document>
@* <a href=@talk-dot.html target=_top>Elements of architecture</a>
\label{Elements}

<include file=navigate>

Software architecture has become an area of research
in its own right.
The seminal work of [_Shaw96] introduced the notion
of <em>software architecture</em> as a means to describe how
the various elements of a software system
interact to achieve some computational goal.
For example, at a high level we can distinguish
between a pipe-lined architecture, common to many compilers,
and event-driven computation, as it occurs for example in
GUI-based systems.

<slide id=Elements cap="Elements of architecture -- models and views">
:@ Elements of architecture 
/// [_Wolf]
<ul>
<li> processing elements -- transformation on data
<li> data elements -- contain information
<li> connections -- glue that holds elements together
</ul>
:@ Models and views
/// [_Kruchten95]
<ul>
<li> logical -- functional requirements
<li> process -- performance, availability, distribution
<li> physical -- scalability, configuration
<li> development -- organization of software modules
<li> scenarios -- instances of use cases
</ul>
:@ Definitions 
<ul>
<li> http://www.sei.cmu.edu/architecture/definitions.html
</ul>
</slide>

In the definition given in [_Wolf],
a software architecture is described as
consisting of <em>processing elements</em>,
which operate on data,
and <em>data elements</em>,
which somehow contain the information being processed.
In addition there are <em>connection elements</em>
that glue the processing and data elements together.
Such an abstract view allows for
describing a software system at a high level of 
abstraction and to indicate choice points
and alternatives. 
<p>
A later definition, given in [_Kruchten95],
makes a distinction between the levels of
abstraction, or points of view,
from which a description of a system is possible.
It distinguishes between a <em>logical view</em>,
which captures the functional requirements,
a <em>process view</em>, which indicates non-functional aspects
such as performance, availability and distribution,
a <em>physical view</em>,
which deals with issues such as scalability and configuration,
and a <em>development view</em>,
which describes the organization of the software modules.
In addition, [_Kruchten95] distinguishes a <em>scenario view</em>,
which may be used for formulating tests based on
properly instantiated use cases.
The scenario view may be regarded as orthogonal to
the logical, process, physical and development models
since it does not affect the structure of
the system itself, but rather the way the structure
is validated against proper usage tests.
<p>
An exhaustive list of definitions of
the notion of <em>software architecture</em> is given
at the Web site of the Software Engineering Institute (SEI),
of which the url is given in \sliref{Elements}.
<p>
At the time of writing, the most comprehensive book
concerning software architectures is [_Practice].
As a definition it proposes:

<p>
<slide color=blue id=arch-def mode=hide>
<quote><em><c><p></c>
The software architecture of a program or computing system
is the structure of the system, which comprises
software components, the externally visible properties
of those components, and their interrelationships.
</em></quote>
</slide>

\noindent
Note the stress on <em>externally visible properties</em>
here.
It is meant to express that both components and their
relations must be described at a suitable level of
abstraction.
Also note that the phrase <em>relationships between components</em>
may cover quite a lot.
For example, when considering the architecture of a 
Web application,
issues such as communication protocols and
document standards must be considered as well.
In addition, the technological infrastructure,
elements of which are given in \sliref{infrastructure},
must also be taken into account.

<slide id=infrastructure cap="Technological infrastucture">
:@ Technological infrastructure
/// [_CS2001]
<ul>
<li> client-platform -- hardware, OS
<li> presentation services -- windows, multimedia
<li> application software -- code, business logic
<li> network -- communication support
<li> middleware -- distribution, (object) brokers
<li> server platform -- hardware, OS
<li> database -- data management system
</ul>
</slide>


One may wonder whether the architecture metaphor,
which is derived from the construction of buildings,
is really appropriate for software systems.
Software systems are much more dynamic than buildings,
so it might perhaps be more appropriate to focus on dynamic, 
behavioral aspects rather than structural aspects.
As a metaphor, I would prefer
for example one related to an ecological system,
to stress the dynamic growth that seems to be characteristic
of software systems nowadays.
<p>
In the definition or rather collection of definitions, given
by the IEEE Architecture Working Group,
for the terms <em>architect</em>,
<em>architectural description</em>, 
<em>stakeholder</em> and <em>viewpoint</em>,
utmost care is taken to suppress the phrase <em>structure</em>.
Instead, the notion of <em>architecting</em> is defined
as defining, maintaining, improving and
certifying proper implementation of an architecture,
and an <em>architecture</em> as a collection of views 
relevant to the stakeholders of a system.
<ignore>
This brings us back to the definition
of [_Kruchten95] given in \sliref{elements},
which defines an architecture as a collection of
views or models describing the system at a suitable
level of abstraction.
</ignore>


@ Distributed object architectures
When considering the architecture of a system,
invariably the technological infrastructure
plays a role.
In particular, when considering client/server
or distributed object systems the choice for 
respectively a particular
client and server platform, middleware and
communication infrastructure
may to a large extent determine the characteristics
of the software architecture.
<p>
Explicit attention for the architecture of a system becomes
increasingly relevant as the complexity of the system grows.
As argued in [_CorbaPatterns],
an architecture is an abstraction that allows for
mastering complexity and managing change.


<slide id=dop cap="Distributed object patterns">
@ <a href=@online/lectures/oo/patterns/dot.html>Distributed object patterns</a>
/// [_CorbaPatterns] <!--\mbox{}\hspace{-0.2cm}\mbox{}-->
<ul>
%%<li> Objects/Classes
%%<li> MicroArchitectures
<li> Framework /// <red>(class hierarchies)</red>
<li> Applications /// <red>(wrappers)</red>
<li> System /// <red>(horizontal, vertical, metadata)</red>
<li> Enterprise /// <red>(reference models, infrastructure, policies)</red>
<li> Intra/Internet /// <red>(standards)</red> <!--\mbox{}\hspace{-0.2cm}\mbox{}-->
</ul>
</slide>

[_CorbaPatterns] present a number of patterns
based on the Common Object Request Broker Architecture (CORBA).
The patterns differ in scale, ranging from frameworks and
systems to
enterprise-level and intra/Internet-level infrastructures.
According to [_CorbaPatterns], software problems
are due to inadequate definition and transfer of software
boundaries.
They criticize traditional object-oriented analysis
and design methods for not paying sufficient attention to
the actual interfaces that define these boundaries
which may be regarded as a contract between the supplier of a service
and its clients.
At the higher enterprise and intra/internet levels,
policies and standards are perhaps more important 
than interfaces <i>per se</i>.
However, at the framework and system level
interface definitions delineate stable boundaries
between the components that constitute the system.

%%:@ Discussion
<p>
In business applications a distinction can be made
between <em>horizontal components</em>
(covering general functionality, such as GUI-aspects
and document interoperability),
<em>vertical components</em>
(covering domain-specific functionality for one
area of business, such as finance),
and <em>meta-data</em>,
representing the more volatile, knowledge-level
aspects of a system.
[_CorbaPatterns] observe that each of these component types
may cover one third of a system.
When to consider information or a service as
part of the meta-data must be determined
by the extent to which that particular information or
service may be considered stable.
Architectural decisions must strive for an ecology of change,
that is a flexible arrangement of components
to promote changes in business-logic and adaptiveness to
a changing environment.

</document>
