<document>
@+ <a href=@talk-embed.html target=_top>Embedded logic -- crossing the paradigm boundary</a>
\label{Logic}


<include file=navigate>

Knowledge is a substantial ingredient in many applications.
By knowledge we mean information and rules
operating on that information, to obtain derived information.
As in any (software) engineering effort, maintenance,
that is knowledge maintenance, is of crucial importance.
When we do not avoid the dispersion of knowledge and information
in the actual code of the system, maintenance will
be difficult.
Put differently, for reasons of flexibility and maintenance
we need to factor out the (volatile) knowledge and information
components.
<p>
Traditionally, the information components are often
taken care of by a database that allows
for the formulation of views to obtain (possibly aggregate)
information.
Logic or logic programming is a strictly more powerful
mechanism to deal with information and knowledge.
In our group, we have been studying the use of logic programming
in knowledge-intensive software engineering applications.


<ignore>
<slide mode=hide id=embed cap="Embedded Logic">
%%@ Embedded Logic
<ul>
<li> (SG/X)ML-like syntax for queries
<li> logic engine in filter
<li> Note: this is an offline approach
<li> Rationale: only knowledge maintenance
</ul>
</slide>

<slide mode=hide id=embed-query cap="Querying from a document">
%%@ Querying from a document
<ul>
<li> a simple knowledge base: <a href=people.pl.html>people</a>
<li> some rules: concerning the <a href=institute.pl.html>institute</a>
<li> a query: to collect <a href=mailinglist.in.html>email adresses</a>
</ul>
<hr>
<p>Motivation: e.g the <a href=http://www.cs.vu.nl/~eliens/imse>
IM&amp;SE Colloquium</a>.
</slide>
</ignore>


%%:@ Querying a knowledge base


<slide mode=hide id=local-example cap="Obtaining the mailinglist">
<display>
/// <box><a href=@online/experiments/embed/contents.html>embedded logic</a></box>

&lt;query kit=pl src=local.pl cmd=X:email_address(X)>
&lt;param format=" %s">
&lt;param result="">
&lt;param display="<h4>The query</h4>">
&lt;param header="<h4>The adresses</h4> <ul>">
&lt;param footer="</ul>">
email_address(E) :-
	person(X),
	property(X,name:N),
	property(X,familyname:F),
	email(X,E),
	cout(['<li>', N,' ',F,' has email adress ']),
	cout([ '&lt;a href=mailto:', E, '>', E, '&lt;/a>',nl]).
&lt;/query>
</display>
</slide>

As an example, consider the query above,
which is expressed in an SGML/XML like syntax.
The query command {P X:email_address(X) P} asks
for all {P X P} for which the predicate {P email_address(X) P}
holds. The predicate {P email_address P} is defined between
the query begin and end tags.
<p>
The <em>query</em> tag is an element of one of the
text processing filters to provide
<a href=@online/papers/htf4/contents.html>hypermedia
support for software engineering</a> described
in [_HypermediaSE].
Processing the fragment above results in an HTML list
of names and email addresses.
The collection of filters itself is written in
<em>lex</em>, <em>yacc</em> and C++.
To process the query, an embedded logic programming interpreter
is invoked.
To merge the output from the query, a handler is installed
for the <em>cout</em> command.
<p>
The query example was motivated by the need to maintain
Web pages for the administration of a colloquium
within our group.
The actual knowledge base consists of a list of people
and some rules to determine their affiliations
and email addresses.
The knowledge base is made available by
consulting the file {P local.pl P}.
<ignore>
I admit, there is some overkill in using a logic programming approach here.
But nevertheless, the example served its goal, since using a different
query with the same knowledge-base was sufficient
to generate a group mailing-list as well.
</ignore>
<p>
As concerns the implementation,
the Java fragment below indicates how to access the logic
programming interpreter from a (Java) program.

%%:@+ <a href=@talk-remote.html target=_top>Remote knowledge</a>

<slide id=java mode=hide>
<display>
query pl = new query("kit=pl src=remote.pl"); /// <box><a href=logic.java.html>logic.java</a></box>
pl.eval("X:assistant(X)");
String res = null;
while ( (res = pl.result()) != null ) {
            System.out.println("<li> " + res);
            }
</display>
</slide>

\noindent
After creating a <em>query</em> object,
the goal {P X:assistant(X) P} is invoked,
which can be taken to mean, give me every {P X P} for
which the predicate {P assistant(X) P} holds.
The final output is obtained by iterating over the results
of the evaluation of that goal.
As a comment, multiple results may be obtained
in Prolog by backtracking over the possible choice points.
%%Similar code, although in C++, is used in the collection
%%of filters mentioned above.

:@+ Distributed knowledge servers

Maintaining knowledge is difficult.
As a rule of thumb, avoid the replication of knowledge
as much as possible.
However, this means that we may need to access
knowledge from remote sources.
One (obvious) solution that presents itself is to allow
for url-enabled consults,
as illustrated in the fragment below.

<slide id=remote mode=hide>
<display>
/// <box><a href=@online/experiments/embed/remote.pl.html>remote.pl</a></box>
:- source('http://www.cs.vu.nl/~eliens/db/se/people.pl').
:- source('http://www.cs.vu.nl/~eliens/db/se/institute.pl').
:- source('http://www.cs.vu.nl/~eliens/db/se/property.pl').
:- source('http://www.cs.vu.nl/~eliens/db/se/query.pl').
</display>
</slide>

\noindent
This solution has (indeed) be implemented in our filters,
since the url addressing scheme is straightforward
and easy to implement.
<p>
However, processing the information accessed by url
is still done locally.
So, the next step that may be suggested is to
distribute the knowledge processing itself,
for example by using CORBA.

<ignore>
<slide mode=hide id=remote cap="Distributed knowledge">
%%@ Remote (distributed) knowledge
<ul>
<li> maintaining knowledge is difficult
<li> don't replicate, otherwise
<li> so we need to access knowledge from remote sources
<li> and possibly mix remote and local knowledge
<li> Solution: <a href=remote.pl.html>url-enabled consults</a>
<li> Problem: availability of the (VU) server
</ul>
</slide>
</ignore>

<ignore>
<slide id=remote-remarks cap="Remarks">
%%@ Remarks
<ul>
<li> queries may clash, we need seperate 'engines' 
<li> this is static, the next step is dynamic: signed applets
<li> this is research code, not production code
</ul>
</slide>
</ignore>


<slide id=query mode=hide>
<display>
interface query { /// <box><a href=@online/hush/sources/hush/dv/query/hush.idl.html>query.idl</a></box>

        void source(in string file);
        long eval(in string cmd);
        string result(in long id);

        oneway void halt();
        };
</display>
</slide>

Exploiting the integration of CORBA and <em>hush</em>,
we have defined an interface for <em>query</em>
in IDL and implemented query client and query server objects.
These objects may be created by giving appropriate
parameters to the <em>query</em> constructor invocation.
This approach allows for embedding remote knowledge processing
transparently in our collection of filters.
Nevertheless, although we showed that this approach is feasible,
we have not addressed the problems that may occur
due to the unavailability or faults of the server.


%%:@+ Distributed knowledge servers

<ignore>
<slide id=corba cap="Distributed knowledge servers">
%%@ Distributed knowledge servers
<ul>
<li> The next step is to distribute knowledge processing
<li> A simple CORBA framework: 
<a href=client.c.html>client</a>
and <a href=server.c.html>server</a>
<li> Both <a href=client.pl.html>client</a>
and <a href=server.pl.html>server</a> can be logic-enabled.
<li> The <a href=@online/hush/examples/experimental/universe/index.html>universe</a>
is more logical than it appeared at first sight.
</ul>
</slide>
</ignore>

<ignore>
<slide id=corba-test-0 cap="Distributed knowledge processing - example">
%%@ Example - distributed knowledge processing
<hr><p>
Look at the <a href=@debug-corba.html>debug information</a>

</slide>

<slide id=corba-test-1 cap="Distributed knowledge processing - example (2)">
%%@ Example - distributed knowledge processing
<hr><p>
Look at the <a href=@debug-corba.html>debug information</a>
</slide>
</ignore>



%%:@ Conclusions

<ignore>
<slide id=conclusions cap="Conclusions">
%%@ Conclusions
<ul>
<li> knowledge management with logic is feasible
<li> has applications in document engineering and ...
<li> embedding is relatively efficient
<li> can be done in multiple ways
<li> corba enabled logic services are possible
<li> true logic is hard to debug
</ul>
%%@ Future developments
<ul>
<li> dynamic: signed applets
</ul>
</slide>
</ignore>

</document>
