<document>
@* <a href=@talk-midi.html target=_top>Case study --  multimedia feature detection</a>
\label{Midi}

<include file=navigate>


In this section, we will look at the indexing and retrieval
of musical fragments.
This study is primarily aimed at establishing the
architectural requirements for the detection of musical features
and to indicate directions for exploring the 
inherently difficult problem of finding proper discriminating
features and similarity measures in the musical domain.
In this study we have limited ourselves to the analysis
of music encoded in MIDI, to avoid the technical difficulties
involved in extracting basic musical properties
from raw sound material.
Currently we have a simple running prototype for
extracting higher level features from MIDI files.
In our approach to musical feature detection, 
we extended the basic grammar-based ACOI framework
with an embedded logic component to facilitate
the formulation of predicates and constraints over
the musical structure obtained from the input.


<slide id=acoi type=fig cap="The extended ACOI architecture">
<image name=@figures/acoi/acoi>
</slide>


:@+ The ACOI framework

The ACOI framework is intended to accomodate
a broad spectrum of classification schemes,
manual as well as (semi) automatic, for the indexing
and retrieval of multimedia objects, [_ACOI].
<p>
What are stored are not the actual multimedia objects
themselves, but structural descriptions of
these objects (including their location) that may be used for
retrieval.
<p>
The ACOI model is based on the assumption that
indexing an arbitrary multimedia object
is equivalent to deriving a grammatical structure
that provides a namespace to reason about the object
and to access its components.
However, there is an important difference with
ordinary parsing in that the lexical and grammatical items
corresponding to the components of the multimedia object
must be created dynamically by inspecting the actual object.
Moreover, in general, there is not a fixed sequence
of lexicals as in the case of natural or formal languages.
To allow for the dynamic creation of lexical
and grammatical items the ACOI framework supports both
<em>black-box</em> and <em>white-box</em> (feature)
detectors.
Black-box detectors are algorithms, usually 
developed by a specialist in the media domain,
that extract properties from the media object 
by some form of analysis.
White-box detectors, on the other hand,
are created by defining logical
or mathematical expressions over the grammar itself.
In this paper we will focus on black-box detectors only.
<p>
As an example, look at the (simple) feature grammar below,
specifying the structure of a hypothetical community.

<slide type=hide cap="A sample grammar">
<program>
detector world; /// finds the name of the world
detector people; /// checks name, eliminates institutes
detector company; /// looks if there are at least two persons

atom str name;

community: world people company;

world: name;
people: person*;

person: name;
</program>
</slide>

A <em>community</em> consists of people,
and is a community only if it allows for the people
to be in each other's company.
<p>
A community has a name.
The actual purpose of this grammar is to select the persons
that belong to a particular community from the input,
which consists of names of potential community members.
Note that the grammar specifies three detectors.
These detectors correspond to functions that are invoked
when expanding the corresponding non-terminal in the grammar.
An example of a detector function is the
<em>personDetector</em> function partially specified below.

<slide type=hide cap="A person detector">
<program>
int personDetector(tree *pt, list *tks ){
...
q = query_query("kit=pl src=check.pl");

while (t = next_token(tks)) {
      sprintf(buf,"person(%s)",t);
      query_eval(q,buf);
      if (query_result(q,0)) ///// put name(person) on tokenstream
                putAtom(tks,"name",t);
      }
...
}
</program>
</slide>

The <em>personDetector</em> function checks for each token
on the input tokenstream $tks$ as to whether the token corresponds
to the name of a person belonging to the community.
The check is performed by an embedded logic component
that contains the information needed to establish whether
a person is a member of the community.
Note that the query for a single token
may result in adding multiple names to the token stream.
<p>
The <em>companyDetector</em> differs from the <em>personDetector</em>
in that it needs to inspect the complete parse tree
to see whether the (implicit) <em>company</em> predicate is satisfied.
<p>
When parsing succeeds and the <em>company</em> predicate is satisfied
a given input may result in a sequence
of updates of the underlying database, as illustrated below.


<slide type=hide cap="Database updates">
<display>
V0 := newoid();
V1 := newoid();
  community_world.insert(oid(V0),oid(V1));
    world_name.insert(oid(V1),"casa");
  community_people.insert(oid(V0),oid(V1));
V2 := newoid();
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"alice");
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"sebastiaan");
    ...
</display>
</slide>

Evidently, the updates correspond to assigning 
appropriate values to the attributes of a structured object,
reflecting the properties of the given community.

<p>
The overall architecture of the ACOI framework is depicted in \sliref{acoi}.
Taking a feature grammar specification, such as the simple
community grammar, as a point of reference, we see
that it is related to an actual feature detector
(possibly containing an embedded logic component)
that is invoked by the Feature Detector Engine (FDE)
when an appropriate media object is presented for indexing.
The feature grammar and its associated detector
further result in updating respectively the data schemas
and the actual information stored in the (Monet) database.
The Monet database, which underlies the ACOI framework,
is a customizable, high-performance, main-memory database
developed at the CWI and the University of Amsterdam, see [_MONET].
<p>
At the user end, a feature grammar is related to
a <em>View</em>, <em>Query</em>
and <em>Report</em> component,
that respectively allow for inspecting a feature grammar,
expressing a query, and delivering a response
to a query.
Some examples of these components are currently implemented as applets
in Java 1.1 with Swing, as described in [_ACOI].
<p>
:@ Formal specification
Formally, a feature grammar $G$ may be defined
as $G = (V,T,P,S)$, where $V$ is a collection of
variables or non-terminals, 
$T$ a collection of terminals,
$P$ a collection of productions of the form $V -> (V \union T)$
and $S$ a start symbol.
A token sequence $ts$ belongs to the
language $L(G)$ if $S -*-> ts$.
Sentential token sequences, those belonging to
$L(G)$ or its sublanguages $L(G_v) = (V_v,T_v,P_v,v)$
for $v \e (T \union V)$, correspond to a complex object
$C_v$, which is the object corresponding to the parse tree 
for $v$, as illustrated in the community example.
The parse tree defines a hierarchical structure
that may be used to access and manipulate the components
of the multimedia object subjected to the detector.

:@+ The anatomy of a MIDI feature detector


Automatic indexing for musical data
is an inherently difficult problem.
Existing systems rely on hand-crafted solutions,
geared towards a particular group of users,
such as for example composers of film music, see [_MM].
In this section, we will look at a simple feature detector for
MIDI-encoded musical data.
It provides a skeleton for future experimentation.

<slide id=midi-structure type=fig cap="MIDI features">
<image name=@figures/acoi/grammar>
</slide>

The hierarchical information structure that we consider
is depicted in \sliref{midi-structure}.
It contains only a limited number of basic
properties and must be extended with information
along the lines of a musical ontology including genre,
mood and the like.
%%in [_AI].
However, the detector presented here provides
a skeleton solution that accommodates an extension with
arbitrary predicates over the musical structure in a transparent manner.
<p>
The grammar given below corresponds in an obvious way with
the structure depicted in \sliref{midi-structure}.

<slide type=hide cap="A simple feature grammar for MIDI files">
<display>

detector song; ### to get the filename
detector lyrics; ### extracts lyrics
detector melody; ### extracts melody

atom str name;
atom str text;
atom str note;  

song: file lyrics melody;

file: name;
lyrics: text*;
melody: note*;

</display>
</slide>

The start symbol is a <em>song</em>.
The detector that is associated with <em>song</em>
reads in a MIDI file.
The musical information contained in the MIDI file is then stored
a a collection of Prolog facts.
This translation is very direct.
In effect the MIDI file header information is stored, and
events are recorded as facts, as illustrated below
for a <em>note_on</em> and <em>note_off</em> event.
<slide mode=hide id=midi-event>
<display>
event('kortjakje',2,time=384, note_on:[chan=2,pitch=72,vol=111]).
event('kortjakje',2,time=768, note_off:[chan=2,pitch=72,vol=100]).
</display>
</slide>
\noindent
After translating the MIDI file into a Prolog format, the other detectors
will be invoked, that is the <em>composer</em>,
<em>lyrics</em> and <em>melody</em> detector,
to extract the information related to these properties.

<slide id=midi-processing type=fig cap="Processing MIDI file">
<image name=@figures/acoi/process>
</slide>

The actual processing is depicted in \sliref{midi-processing}.
The input is a MIDI file.
As indicated in the top line, the MIDI file itself may
be generated from a <em>lilypond</em> file.
Lilypond is a \LaTeX-like formatting language for
musical scores that also supports the generation of MIDI, described in [_Lily].
As indicated on the bottom line, processing a MIDI file
results in a collection of features as well as in a MIDI file
and lilypond file.
The (result) MIDI file contains an extract
of the original (input) MIDI file and the lilypond file
contains a score for this extract,
which may be presented to the (end) user as the result of a query.
This setup allows us to verify whether our extract or
abstraction of the original musical structure is effective,
simply by comparing the input (midi or lilypond)
musical structure with the output (midi or lilypond) extract. 
<p>
To extract relevant fragments of the melody we
use the melody detector, of which a partial listing is given below.

<slide type=hide cap="The melody detector">
<program>
int melodyDetector(tree *pt, list *tks ){
char buf[1024]; char* _result;
void* q = _query;
int idq = 0; 

  idq = query_eval(q,"X:melody(X)");
  while ((_result = query_result(q,idq)) ) {
         printf("note: %s",_result);
         putAtom(tks,"note",_result);
         }
  return SUCCESS;
} 
</program>
</slide>

The embedded logic component is given the
query {P X:melody(X) P}, which results in the notes
that constitute the (relevant fragment of the) melody.
These notes are then added to the tokenstream.
A similar detector is available for the lyrics.
<p>
Parsing a given MIDI file, for example <em>kortjakje.mid</em>,
results in updating the Monet database.
%%as indicated below.

<ignore>
<slide mode=hide id=token-api cap="The tokenstream API">
<display>
given type AtomName, AtomValue;

interface tokenstream {
	void putAtom(in AtomName, in AtomValue);
};
</display>
</slide>
</ignore>

<ignore>
<slide type=hide cap="Update of Monet database">
<program>
V1 := newoid();
  midi_song.insert(oid(V0),oid(V1));
V2 := newoid();
    song_file.insert(oid(V1),oid(V2));
      file_name.insert(oid(V2),"kortjakje");
    song_lyrics.insert(oid(V1),oid(V2));
      lyrics_text.insert(oid(V2),"e");
      lyrics_text.insert(oid(V2),"per-");
      lyrics_text.insert(oid(V2),"sonne");
      lyrics_text.insert(oid(V2),"Moi");
      lyrics_text.insert(oid(V2),"je");
      lyrics_text.insert(oid(V2),"dis");
      lyrics_text.insert(oid(V2),"que");
      lyrics_text.insert(oid(V2),"les");
      lyrics_text.insert(oid(V2),"bon-");
      lyrics_text.insert(oid(V2),"bons");
      lyrics_text.insert(oid(V2),"Val-");
      lyrics_text.insert(oid(V2),"ent");
    song_melody.insert(oid(V1),oid(V2));
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"c-2");

</program>

</slide>
</ignore>

The updates reflect the structure of
the musical information object that corresponds to
the properties defined in the grammar.

:@ Implementation status
Currently, we have a running prototype of the MIDI
feature detector.
It uses an adapted version of public domain
MIDI processing software.
The embedded logic component is part of the <em>hush</em>
framework. It uses an object extension of Prolog
that allows for the definition of native objects
to interface with the midi processing software.
The logic component allows for
the definition of arbitrary predicates to extract 
the musical information, such as the melody and the lyrics.

:@+ Queries -- the user interface

Assuming that we have an adequate solution
for indexing musical data, we need to define how end users may access these data, 
that is search for musical objects in the information space
represented by the database, for the ACOI project the World Wide Web.

<slide id=midi-keyboard type=fig cap="Keyboard interface">
<image name=@figures/jamming/keyboard>
</slide>

For a limited category of users, those with some musical skills,
a direct interface such as a keyboard
or a score editor, as provided by the <em>hush</em> framework,
might provide a suitable interface for
querying the musical database.
Yet, for many others, a textual description, or a form-based
query will be more appropriate.

<slide type=fig cap="User Query Processing">
<image name=@figures/acoi/query>
</slide>

In processing a query, we may in some cases
derive a partial melody or rhythmic structure from the query,
as well as some additional features or criteria.
As explained, the output of indexing MIDI files
consists of both information concerning features
as well as a musical rendering of some of these features.
These features can be used to match against
the criteria formulated in the query.
The musical renderings, which include a partial score,
may be presented to the user in response to a query,
to establish whether the result is acceptable.

</document>
