<document>
<a name=correspond>
@+ The correspondence relation
\label{correspondence}
<include file=navigate>

@$=raw

Behavioral refinement is not restricted 
to the realization of abstract specifications.
We will now look at a definition of behavioral
refinement, following [_Liskov93],
that may serve as a guideline for programmers
to define behavioral subtypes, both abstract and concrete,
including subtypes extending the behavioral
repertoire of their supertypes.

In [_Liskov93] the relation between behavioral
types is explained by means of a so-called
{\em correspondence mapping}, that relates
a subtype to its (abstract) supertype.

<slide id=10-correspondence cap="The subtype correspondence mapping">
<raw height=200>
@ Correspondence 
\zline{\fbox{$<< %a, %r, %x >>$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $%a$ abstraction -- maps $%s$ values to $%t$ values
- $%r$ renaming -- maps subtype to supertype methods
- $%x$ extension -- explains effects of extra methods
I>
\ifsli{\vspace{-1cm}}{}
</raw>
\vspace{-0.15cm}
</slide>

A {\em correspondence mapping} is a triple
consisting of an {\em abstraction} function $%a$
(that projects the values of the subtype
on the value domain of the supertype),
a {\em renaming} $%r$
(that defines the relation between methods defined
in both types)
and an {\em extension} map $%x$ (that defines the 
meaning of additional methods).
\see{10-correspondence}
Technically, the function $%a$ must be $onto$, that is
each value of the supertype domain must be
representable by one or more values of the subtype domain.
Generally, when applying the abstraction function,
we loose information (which is irrelevant from
the perspective of the supertype), for
example the specific ordering of items in a container.


<slide id=10-subtyping cap="Behavioral subtyping constraints">
<raw height=400>
@ Signature
\zline{\fbox{$ %s <= %t $}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $dom( m_{%t} ) <= dom( m_{%s} )$
- $ran( m_{%s} ) <= ran( m_{%t} )$
I>
\vspace{-0.1cm}
:@ Behavior
\ifsli{\vspace{-0.5cm}}{}
<I
- $pre( m_{%t} )[ x_{%t} := %a( x_{%s} )] => pre( m_{ %s } )$
- $post( m_{ %s } ) => post( m_{ %t } )[ x_{ %t } := %a( x_{ %s } ) ]$
- $invariant( %s )$ $ => $
\ifsli{\n\mbox{}\hspace{3.0cm}}{}
$invariant( %t )[ x_{ %t } := %a( x_{ %s } ) ]$
I>
@ Extension <em> -- diamond rule</em>
\zline{\fbox{$%x(x.m(a)) = %p_{m}$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %f \apijl{ x.m(a) } %f' /\ %f \apijl{ %p_{m} } %Q $
  $ => $ $%a( x_{ %f } ) = %a( x_{ %Q } )$
I>
\ifsli{\vspace{-1cm}}{}
\vspace{-0.15cm}
</raw>
</slide>

To determine whether a type $%s$ is a
(behavioral) subtype of a type $%t$, one 
has to define a correspondence mapping
$<< %a, %r, %x >>$
and check the issues listed in \sliref{10-subtyping}.
First, syntactically, we must check that the
signature of $%s$ and $%t$ satisfy the (signature)
subtyping relation defined in the previous chapter.
In other words, for each method $m$ associated 
with the object type $%t$ (which we call $m_{%t}$),
and corresponding method $m_{%s}$
(which is determined by applying the renaming $%r$)
we must check the (contravariant) function subtyping
rule, that is
$dom( m_{%t} ) <= dom( m_{ %s } )$
and
$ran( m_{%s} ) <= ran( m_{ %t } )$,
where $ran$ is the range or result type of $m$.

Secondly, we must check that the behavioral
properties of $%s$ respect those of $%t$.
In other words, for each method $m$
occurring in $%t$ we must check
that $pre( m_{%t} )[ x_{%t} := %a( x_{%s} )] => pre( m_{%s} )$
and that 
$post( m_{ %s } ) => post( m_{ %t } )[ x_{ %t } := %a( x_{ %s } ) ]$.
Moreover, the invariant characterizing $%s$
must respect the invariant
characterizing $%t$,
that is
$invariant( %s ) => invariant( %t )[ x_{ %t } := %a( x_{ %s } ) ]$.
The substitutions $[ x_{ %t } := %a( x_{ %s } )]$
occurring in the behavioral rules are meant
to indicate that each variable of
type $%t$ must be replaced by a corresponding variable
of type $%s$
to which the abstraction function is applied
(in order to obtain a value in the (abstract) domain of $%t$).

And thirdly, in the final place,
it must be shown that the extension map
$%x$ is well-defined.
The extension map must be defined in such a way
that each method call for an object
$x$ of type $%s$,
which we write as $x.m(a)$ where $a$ represents
the arguments to the call,
is mapped to a program $%p_{m}$ in
which only calls appear to methods shared by
$%s$ and $%t$ (modulo renaming)
or external function or method calls.
In addition the {\em diamond rule}
must be satisfied,
which means that the states $%f'$ and $%q$ resulting from
applying respectively $x.m(a)$ and $%p_{m}$
in state $%f$ must deliver identical 
values for $x$ from the perspective of $%t$,
that is after applying the abstraction function.
In other words, extension maps allow us to understand
the effect of adding new methods and to establish
whether they endanger behavioral compatibility.

In [_Liskov93] a distinction is made between
{\em constructors} (by which objects are created),
{\em mutators}
(that modify the state or value of an object)
and {\em observers}
(that leave the state of an object unaffected).
Extension maps are only needed for mutator methods.
Clearly, for {\em observer} methods the result
of $%x$ is empty, and {\em constructors}
are taken care of by the abstraction function.

@ Behavioral subtypes
The behavioral subtyping rules defined above
are applicable to arbitrary (sub)types, and not only
to (sub)types defined by inheritance.
As an example, we will sketch (still
following [_Liskov93]) that a $stack$ may be defined as
a behavioral subtype of the type $bag$.
Recall that a $bag$ is a $set$ allowing duplicates.
\see{10-ex-subtype}

<slide id=10-ex-subtype cap="Behavioral subtypes -- example">
<raw height=400>
:@ Behavioral subtypes
\zline{\fbox{$stack <= bag$}}
<ul> 
<li> $bag$ -- $put$, $get$
<li> $stack$ -- $push$, $pop$, $settop$
</ul>
:@ Representation
<ul>
<li> $bag$ -- $<< elems, bound >>$ \zline{multiset}
<li> $stack$ -- $<< items, limit >>$ \zline{sequence}
</ul>
:@ Behavior <em> -- put/push</em>
[P
$put(i):$
  $require \: size( b.elems ) < b.bound$
  $promise \: b' = << b. elems \uplus {{ i }}, b.bound >>$

$push(i):$
  $require \: length( s.items ) < s.limit$
  $promise \: s' = << s.items \. i, s.limit >>$
P]
\ifsli{\ln\vspace{-1cm}}{}
\vspace{-0.2cm}
</raw>
</slide>

Let the type $bag$ support the methods
$put(i:Int)$ and $get():Int$ and assume
that the type $stack$ supports
the methods $push(i:Int)$, $pop():Int$ and
in addition a method $settop(i:Int)$
that replaces the top element of
the stack with its argument.
Now, assume that a $bag$ is represented by
a pair $<<elems, bound>>$, where
$elems$ is a multiset (which is a set
which may contain multiple elements of the same value)
and $bound$ is an integer indicating the maximal
number of elements that may be in the {\it bag}.
Further, we assume that a $stack$ is represented as
a pair $<< items, limit >>$,
where $items$ is a sequence and $limit$ is
the maximal length of the sequence.
For example $<< {{ 1,2,7,1 }}, 12 >>$
is a legal value of $bag$
and $<< 1 \. 2 \. 7 \. 1, 12 >>$
is a legal value of $stack$.

The behavioral constraints for respectively the 
method $put$  
for $bag$ 
and $push$
for $stack$
are given as pre- and post-conditions in \sliref{10-ex-subtype}.
To apply $put$, we require that the size
of the multiset is strictly smaller than the bound
and we ensure that the element $i$ is inserted
when that pre-condition is satisfied.
The multi-set union operator $\uplus$
is employed to add the new element to
the bag.
Similarly, for $push$ we require the length
of  the sequence to be smaller than the limit
of the  stack
and we then ensure that the element is appended
to the sequence.
As before, we use the primed variables $b'$ and $s'$
to denote the value of respectively 
the bag $b$ and the stack $s$ after
applying the operations, respectively $put$ and $push$.

Proceeding from the characterization of $bag$
and $stack$ we may define the correspondence 
mapping $<< %a, %r, %x >>$ as in \sliref{10-ex-correspondence}.

<slide id=10-ex-correspondence cap="Behavioral subtypes -- correspondence">
<raw height=400>
@ Correspondence
\zline{\fbox{$stack -> bag$}}
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> $%a(<<items, limit>>) = $ 
\ifsli{\n\mbox{}\hspace{2cm}}{}
$ << mk\_set( items ), limit >>$ \n 
where
[P
$mk\_set( %e ) = \0$
$mk\_set( e \. s ) = mk\_set(s) \uplus {{ e }}$
P]
\ifsli{\vspace{1.0cm}}{}
<li> $%r( push ) = put$, $%r( pop ) = get$
<li> $%x(s.settop(i))$ $ = $
\ifsli{\n\mbox{}\hspace{4cm}}{}
$s.pop(); s.push(i);$
</ul>
\ifsli{\vspace{-1.5cm}}{}
</raw>
</slide>

To map the representation of a stack to
the bag representation we use the function
$mk\_set$ (which is inductively defined to map
the empty sequence to the empty set and to
transform a non-empty sequence into the union
of the one-element multiset of its first element
and the result of applying $mk\_set$ to the remaining
part).
The stack $limit$ is left unchanged, since
it directly corresponds with the bound of the bag.

The renaming function $%r$ maps $push$ to $put$
and $pop$ to $get$, straightforwardly.
And, the extension map describes the result of $settop(i)$
as the application of (subsequently) $pop()$ and $push(i)$.

\cleartrue
<slide id=10-ex-proof cap="Behavioral subtypes -- proof obligations">
<raw height=400>
:@ Proof obligations <em> -- push/put</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $size( %a(s). elems ) < %a(s).bound $ \n
  $ => $ \n
  $length( s.items ) < s.limit$
- $s' = << s.items \. i, s.limit >>$ \n
  $ => $ \n
  $%a(s') $ $ = $ 
  \ifsli{\n}{}
  $ << %a(s).elems \uplus {{ i }}, %a(s).bound >>$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>


With respect to the behavioral definitions given for
$push$ and $put$ we have to verify that
%%<display>
$pre( put(i) )[ b := %a(s) ] => pre( push(i) )$
%%</display>
and that
%%<display>
$post( push(i) ) => post( put(i) )[ b := %a(s) ]$.
%%</display>
These conditions, written out fully in \sliref{10-ex-proof},
are easy to verify.

Generally, a formal proof is not really necessary
to check  that two types satisfy the behavioral
subtype relation.
As argued in [_Liskov93],
the definition of the appropriate behavioral
constraints and the formulation of
a correspondence mapping is already a significant 
step towards verifying that the types have the
desired behavioral properties.
</document>
