<a name=state>
@+ State transformers
<include file=navigate>
@$=raw

Proving the correctness of (imperative)
programs is based on the notion of $states$
and the interpretation of programs
as {\em state transformers}.
A $state$, in a mathematical sense, is simply a function
that records a value for each variable in the program.
For example, having a program $S$ in which the (integer)
variable $i$ occurs, and a state $%f$, we may have
$%f(i) = 3$.
States may be modified by actions that result from
executing the program, such as by an assignment 
of a value to a variable.
We employ {\em substitutions} to modify a state.
As before, substitutions may be defined by an equation, as 
given in \sliref{10-subst}.

\vfig{1}
<slide id=10-subst cap="Substitution">
<raw height=200>
@ Substitutions
\ifsli{\vspace{-0.5cm}}{}
<p> \hspace{2.5cm} \eqcurl{$%f[x:=v](y)$}{$v$}{if $x = y$}{$%f(y)$}{otherwise}
\ifsli{\vspace{-1cm}}{\vspace{-0.5cm}}
</raw>
</slide>

A substitution $%f[x:=v](y)$
states that modifying $%f$ by
assigning the value $v$ to the variable $x$
then, for a variable $y$, the state $%f$ will
deliver $v$ whenever $y$ is identical to $x$ and $%f(y)$
otherwise.
<p>
When we have, for example, an assignment $i = 5$
then we have as the corresponding transition
%%<display>
$ %f -{i=5}-> %f' $
%%</display>
where $%f' = %f[i:=5]$,
that is $%f'$ is like $%f$ except 
for the variable $i$ for which the value $5$
will now be delivered.

Whenever we have a sequence of actions $a_1,...,a_n$
then, starting from a state $%f_0$ we have corresponding
state transformations resulting in states
$%f_1,...,%f_{n-1}$ as intermediary states and
$%f_n$ as the final state.
Often the states $%f_0$ and $%f_n$ are referred to
as respectively the {\em input} and {\em output} state
and the program that results in the actions $a_1,...,a_n$
as the {\em state transformer} modifying $%f_0$ into $%f_n$.

\vstr{1}
<slide id=10-verification cap="The verification of state transformations">
<raw height=400>
@ Program state 
<em> -- $%f$</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- $%f \e %S : Var -> Value$
@ State transformations
<em> -- operations $a_1,a_2,...,a_n$</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- $%f_{0} -{a_1}-> %f_{1} ... -{a_n}-> %f_n$
@ Correctness formulae <em> -- Hoare logic</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- ${{ P }} S {{ Q }}$
@ Verification
\ifsli{\vspace{-0.5cm}}{}
<I>- $ %f_i -{a}-> %f_j /\ %f_i |= P => %f_j |= Q $
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

To characterize the actions that result from
executing a program, we need an operational semantics
that relates the programming constructs to 
the dynamic behavior of a program.
We will study such a semantics in section \ref{10:behavior}.

The requirements a program (fragment)
has to meet may be expressed by using predicates
characterizing certain properties
of a program state.
Then, all we need to do is check whether the final state 
of a computation satisfies these requirements.

Predicates characterizing the properties of a state
before and after executing a program (fragment)
may be conveniently stated by
correctness formulae of the form
%%<display>
${{ P }} S {{ Q }}$
%%</display>
where $S$ denotes a program (fragment)
and $P$ and $Q$ respectively the pre-condition
and post-condition associated with $S$.

A formula of the form ${{ P }} S {{ Q }}$ is true if, for every
initial state $%f$ that satisfies $P$ and for which
the computation characterized by $S$ terminates,
the final state $%f'$ satisfies $Q$.
This interpretation of ${{ P }} S {{ Q }}$
characterizes {\em partial correctness}, partial since
the truth of the formula is dependent on the termination of
$S$ (which may, for example, for a {\em while statement},
not always be guaranteed).
When termination can be guaranteed, then we
may use the stronger notion of {\em total correctness},
which makes the truth of ${{ P }} S {{ Q }}$
no longer dependent on the termination of $S$.

Pre- and post-conditions may also be used to check
invariance properties.
As an example, consider the following
correctness formula:
%%<display>
${{ s = i*(i+1)/2 }} i = i + 1; s = s + i; {{ s = i*(i+1)/2 }}$
%%</display>
It states that the begin and end state of the 
computation characterized by $i = i + 1; s = s + i$
is invariant with respect to the condition $s = i*(i+1)/2$.
As an exercise, try to establish the correctness of
this formula!

To verify whether for a particular program fragment
$S$ and (initial) state $%f_i$ satisfying $P$ 
the correctness formula ${{ P }} S {{ Q }}$
holds,
we need to compute the (final) state $%f_j$
and check that $Q$ is true for $%f_j$.
In general, for example in the case of non-deterministic
programs, there may be multiple (final) states
resulting from the execution of $S$.
For each of these states we have to establish that
it satisfies (the post-condition) $Q$.
We call the collection of possible
computation sequences of a program fragment $S$
the $traces$ of $S$.
Traces characterize the (operational) behavior
of a program.

