<a name=Initial>
@+ Initial algebra semantics
<include file=navigate>

@$=raw

In the previous section we have given a rather operational
characterization of the equivalence relation induced by 
the equational theory and the process of term rewriting
that enables us to purge the generator universe of a type,
by eliminating redundant elements.
However, what we actually strive for is a mathematical
model that captures the meaning of an algebraic
specification.
Such a model is provided (or rather a class of such models)
by the mathematical structures known as
(not surprisingly) algebras.


A {\em single sorted} algebra $|A$ is a structure $ (A,%S) $
where $A$ is a set of values, and $%S$ specifies the signature
of the functions operating on $A$.
A {\em multi-sorted} algebra is a structure $ |A = ( {{ A_s }}_{s \e S }, %S)$
where $S$ is a set of sort names and $A_s$ the set of values belonging
to the sort $s$.
The set $S$ may be ordered (in which case the ordering
indicates the subtyping relationships between the sorts).
We call the (multi-sorted) structure $|A$ a <raw> <math>%S</math>-algebra</raw>.

%%\ifsli{\mbox{}\vspace{3cm}}{}
<slide id=8-algebra cap="Interpretations and models">
<raw height=150>
@ Mathematical model <em> -- algebra</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $%S$-algebra -- $ |A = ( {{ A_s }}_{s \e S }, %S )$
- interpretation -- $ eval : T_{%S} -> |A $
- adequacy -- $ |A |= t_1 = t_2 <=> E |- t_1 = t_2 $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>



Having a notion of algebras, we need to have a way in which
to relate an algebraic specification to
such a structure.
To this end we define an interpretation $eval : T_{%S} -> |A$
which maps closed terms formed by following the rules
given in the specification to elements of the structure $|A$.
We may extend the interpretation $eval$ to include
variables as well (which we write
as <raw>$ eval : T_{%S}(X) -> |A$</raw>), but then we also need to assume
that an assignment <raw>$%h : X -> T_{%S}(X)$</raw> is given,
such that when applying $%h$ to a term $t$ the result
is free of variables, otherwise no interpretation in <raw>$|A$</raw> exists.
\see{8-algebra}

@ Interpretations
As an example, consider the interpretations
of the specification of $Bool$ and
the specification of $Nat$, given in \sliref{8-B-N}.

<slide id=8-B-N cap="Interpretations of <em>Bool</em> and <em>Nat</em>">
<raw height=300>
@ Booleans
\ifsli{\vspace{-0.5cm}}{}
<I
- $ |B = ( {{ tt, ff }}, {{ \neg, /\, \/ }} )$
- $ eval_{|B} : T_{Bool} -> |B = {{ or |-> \/, and |-> /\, not |-> \neg }}$
I>
@ Natural numbers
\ifsli{\vspace{-0.5cm}}{}
<I
- $ |N = ( \nat , {{ ++ , + , \star }} ) $
- $ eval_{|N} : T_{Nat} -> |N = {{ S |-> ++ , mul |-> \star, plus |-> + }}$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The structure <raw>$|B$</raw> given above is simply a boolean algebra, with 
the operators <raw>$\neg$</raw>, <raw>$ /\ $</raw> and <raw>$ \/ $</raw>.
The functions $not$, $and$ and $or$ naturally map to their semantic counterparts.
In addition, we assume that the constants $true$ and $false$ map
to the elements $tt$ and $ff$.

As another example, look at the structure <raw>$|N$</raw> and the
interpretation <raw>$eval_{|N}$</raw>, which maps
the functions $S$, $mul$ and $plus$ specified
in $Nat$ in a natural way.
However, since we have also given equations for $Nat$
(specifying how to eliminate the functions $mul$ and $plus$)
we must take precautions such that the requirement
<display>
   <raw>$ |N |= eval_{|N}(t_1) =_{|N} eval_{|N}(t_2) <=> E_{Nat} |- t_1 = t_2 $</raw>
</display>
\vspace{-0.4cm}
is satisfied if the structure <raw>$ |N $</raw> is to count as an adequate
model of $Nat$.
The requirement above states
that whenever equality holds for two interpreted
terms (in <raw>$ |N $</raw>) then these terms must also be provably equal
(by using the equations given in the specification of {\it Nat}),
and vice versa.

As we will see illustrated later, many models may exist
for a single specification, all satisfying the requirement
of adequacy.
The question is, do we have a means to select one of these 
models as (in a certain sense) the best model.
The answer is yes.
These are the models called {\em initial models}.

@ Initial models
A model (in a mathematical sense) represents the meaning
of a specification in a precise way.
A model may be regarded as stating a commitment with
respect to the interpretation of the specification.
An initial model is intuitively the least committing
model, least committing in the sense
that it imposes only identifications made necessary by
the equational theory of a specification.
Technically, an initial model is a model from which
every other model can be derived by an algebraic mapping
which is a homomorphism.


<slide id=8-initial cap="Initial models">
<raw height=400>
@ Initial algebra
\ifsli{\vspace{-0.5cm}}{}
<I>- $%S E$-algebra -- $|M = ( T_{%S}/=, %S/=)$
:@ Properties
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em no junk} -- $ \A a : T_{%S}/= \E t \dot eval_{|M}(t) = a $
- {\em no confusion} -- $ |M |= t_1 = t_2 <=> E |- t_1 = t_2 $ 
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The starting point for the construction of an initial model
for a given specification with signature $%S$ is to
construct a term algebra $T_{%S}$ with 
the terms that may be generated from the signature $%S$
as elements.
The next step is then to factor the universe of generated
terms into equivalence classes, such that
two terms belong to the same class if they can be proven equivalent
with respect to the equational theory of the specification.
We will denote the representative of the equivalence class to
which a term $t$ belongs by $[t]$.
Hence $t_1 = t_2$ (in the model) {\em iff} $[t_1] = [t_2]$.

So assume that we have constructed a structure
$ |M = (T_{%S}/=, %S ) $  then; finally, we must define an interpretation,
say $eval_{|M} : T_{%S} -> |M$, that assigns closed
terms to appropriate terms in the term model
(namely the representatives of the equivalence class of that term).
Hence, the interpretation of a function $f$ in the structure $|M$
is such that
<display>
<raw> $f_{|M}([t_1],...,[t_n]) = [ f(t_1,...,t_n) ] $</raw>
</display>
\vspace{-0.3cm}
where <raw>$f_{|M}$</raw> is the interpretation of $f$ in <raw>$|M$</raw>.
In other words, the result of applying <raw>$f$</raw> to terms <raw>$t_1,...,t_n$</raw>
belongs to the same equivalence class as the result
of applying <raw>$f_{|M}$</raw> to the representatives of the
equivalence classes of <raw>$t1,...,t_n$</raw>.
\see{8-initial}

An initial algebra model has two important properties,
known respectively as the {\em no junk} and {\em no confusion}
properties.
The {\em no junk} property states that for each element of the
model there is some term for which the interpretation in $|M$
is equal to that element.
(For the $T_{%S}/=$ model this is simply a representative of
the equivalence class corresponding with the element.)
The {\em no confusion} property states that if equality of two 
terms can be proven in the equational theory of the specification,
then the equality also holds (semantically) in the model,
and vice versa.
The {\em no confusion} property means, in other words, that
sufficiently many identifications are made
(namely those that may be proven to hold),
but no more  than that (that is, no other than those
for which a proof exists).
The latter property is why we may speak of an initial model
as the least committing model; it simply gives
no more meaning than is strictly needed.

The initial model constructed from the term algebra
of a signature <raw>$%S$</raw> is intuitively a very natural model
since it corresponds directly with (a subset of) the
generator universe of <raw>$%S$</raw>.
Given such a model, other models may be derived from it
simply by specifying an appropriate interpretation.
For example, when we construct a model for the natural
numbers (as specified by $Nat$) consisting of the 
generator universe <raw>${{ 0, S 0, SS 0, ... }}$</raw>
and the operators <raw>${{ ++, +, \star }}$</raw>
(which are defined as <raw>$S^n ++ = S^{n+1}$</raw>,
<raw>${ S^n } \ast { S^m } = S^{ {n \ast m} }$</raw>
and <raw>$S^n + S^m = S^{n + m}$</raw>)
we may simply derive from this model
the structure $ ({{ 0,1,2,... }}, {{ ++, +, \star }}) $
for which the operations have their standard arithmetical
meaning.
Actually, this structure is also an initial model for
$Nat$, since we may also make the inverse transformation.

More generally, when defining an initial model only
the structural aspects
(characterizing the behavior of the operators) are
important, not the actual contents.
Technically, this means that initial models
are defined up to isomorphism, that is a mapping to
equivalent models with perhaps different contents
but an identical structure.
Not in all cases is a structure derived from an initial model itself
also an initial model, as shown in the example below.


:@ Example
Consider the specification of $Bool$ as given before.
For this specification we have given the structure
$ |B $ and the interpretation $ eval_{|B} $ which defines
an initial model for $Bool$. (Check this!)


<slide id=8-structure cap="Structure and interpretation">
<raw height=400>
:@ Structure
{\em -- $ |B = ( {{ tt, ff }}, {{ \neg, /\, \/ }} )$ }
\zline{\fbox{$|B$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ eval_{|B} : T_{%S_{Bool}} -> |B = {{ or |-> \/, not |-> \neg }}$
- $ eval_{|B} : T_{%S_{Nat}} -> |B = {{ S |-> \neg, mul |-> /\,  plus |-> xor }}$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

We may, however, also use the structure <raw>$|B$</raw> to
define an interpretation of $Nat$.
\see{8-structure}
The interpretation <raw>$eval_{|B} : T_{Nat} -> |B$</raw> is such
that
<raw>$eval_{|B}(0) = ff$</raw>,
<raw>$eval_{|B}(Sx) = \neg eval_{|B}(x)$</raw>,
<raw>$eval_{|B}(mul(x,y)) $ $ = eval_{|B}(x) $\hspace{0.2cm} $ /\ $ \hspace{0.2cm} $ eval_{|B}(y)$\hspace{0.3cm}</raw> and <!-- \hspace{0.2cm} -->
<raw>$eval_{|B}(plus(x,y)) = xor(eval_{|B}(x),eval_{|B}(y))$</raw>,
where <raw>$xor(p,q) = (p \/ q) /\ (\neg ( p /\ q ))$</raw>.
The reader may wish to ponder on what this interpretation
effects.
The answer is that it interprets $Nat$ as
specifying the naturals modulo 2, which discriminates
only between odd and even numbers.
Clearly, this interpretation defines not an initial model,
since it identifies all odd numbers with $ff$ and all even numbers with $tt$.
Even if we replace $ff$ by $0$ and $tt$ by $1$, this is not what we 
generally would like to commit ourselves to when we speak 
about the natural numbers,
simply because it assigns too much meaning.
