<a name=Signatures>
@+ Signatures -- generators and observers
<include file=navigate>

@$=raw

Abstract data types may be considered as modules
specifying the values and functions belonging
to the type.
In [_Dahl92], a type $T$ is characterized as a tuple specifying
the set of elements constituting the type $T$
and the collection of functions related
to the type $T$.
Since constants may be regarded as zero-ary functions
(having no arguments), we will speak of a {\em signature}
$%S$ or $%S_T$ defining a particular type $T$.
Also, in accord with common parlance, we will speak
of the sorts $s \e %S$, which are the sorts (or types)
occurring in the declaration of the functions in $%S$.
\see{8-signature}

<slide id=8-signature cap="Algebraic specification">
<raw height=400>
@ Signature <em> -- names and profiles</em>
\zline{\fbox{$%S$}}
\ifsli{\vspace{-0.5cm}}{}
<I>- $ f : s_1 \* ... \* s_n -> s $
\ifsli{\vspace{-0.5cm}}{}
:@ Functions <em> -- for $T$</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- constants -- $c : \:  -> T$ \zline{\fbox{$C$}}
- producers -- $g : s_1 \* ... \* s_n -> T$
\zline{\fbox{$P$}}
- observers -- $f : T -> s_i$ \zline{\fbox{$O$}}
I>
\ifsli{\vspace{-0.5cm}}{}
@ Type {\em -- generators}
\ifsli{\vspace{-0.5cm}}{}
<I>- $ %S_T = P_T \cup O_T$, $C_T \subset P_T$, $P_T \cap O_T = \emptyset$
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

A $signature$ specifies the names and (function) profiles
of the constants and functions of a data type.
In general, the profile of a function is specified as
<I>- $ f : s_1 \* ... \* s_n -> s $
where $ s_i (i=1..n) $ are the sorts defining
the domain (that is the types of the arguments)
of the function $f$, and $s$ is the sort defining the codomain
(or result type) of $f$.
In the case $ n=0 $ the function $f$ may be regarded as
a constant. More generally, when $ s_1,...,s_n $ are all
unrelated to the type $T$ being defined, we may regard
$f$ as a relative constant.
Relative constants are values that are assumed to
be defined in the context
where the specification is being employed.

The functions related to a data type $T$ may be discriminated
according to their role in defining $T$.
We distinguish between <em>producers</em> $ g \e P_T $,
that have the type $T$ under definition as their result
type, and $observers$ $f \e O_T$, that have $T$
as their argument type and deliver a result of
a type different from $T$.
In other words, producer functions define how
elements of $T$ may be constructed.
(In the literature one often speaks of $constructors$,
but we avoid this term because it already has a
precisely defined meaning in the object-oriented
programming language C++.)
In contrast, observer functions do not produce
values of $T$, but give instead information
on some particular aspect of $T$.

The signature $ %S_T $ of a type <em>T</em> is uniquely defined
by the union of producer functions $ P_T $ and
observer functions $ O_T $.
Constants of type $T$ are regarded as a subset of
the producer functions $P_T$ defining $T$.
Further, we require that the collection of producers is
disjoint from the collection of observers for $T$,
that is $P_T \cap O_T = \emptyset$.

@ Generators
The producer functions actually defining the
values of a data type $T$ are called the
{\em generator basis} of $T$, or generators of $T$.
The generators of $T$ may be used to enumerate the
elements of $T$, resulting in the collection of $T$ values
that is called the {\em generator universe} in [_Dahl92].
\see{8-basis}


<slide id=8-basis cap="Generators -- basis and universe">
<raw height=400>
:@ Generators <em> -- values of $T$ </em>
\zline{\fbox{$T$}}
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> generator basis -- $G_T = {{ g \e P_T }}$
<li> generator universe --
\ifsli{\n\mbox{}\hspace{4.3cm}}{}
$GU_T = {{ v_1, v_2, ... }}$
</ul>
\ifsli{\vspace{-0.5cm}}{}
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> $G_{Bool} = {{ t, f }}$,
\ifsli{\n}{}
$GU_{Bool} = {{ t, f }}$
<li> $G_{Nat} = {{ 0, S }}$, 
\ifsli{\n}{}
$GU_{Nat} = {{ 0, S 0, SS 0, ... }}$
<li> $G_{Set_{A} } = {{ \emptyset, add }}$,
\ifsli{\n}{}
$GU_{Set_{A}} = {{ \emptyset, add(\emptyset,a), ... }}$
</ul>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The generator universe of a type $T$ consists of the
closed (that is variable-free) terms that may be constructed
using either constants or producer functions of $T$.
As an example, consider the data type $Bool$ with
generators $t$ and $f$.
Obviously, the value domain of $Bool$, the
generator universe $GU_{Bool}$ consists only of the values
$t$ and $f$.

As another example, consider the data type $Nat$
(representing the natural numbers) with generator basis
$G_{Nat} = {{ 0, S }}$, consisting of the constant $0$
and the successor function $S : Nat -> Nat$
(that delivers the successor of its argument).
The terms that may be constructed by $G_{Nat}$ is the
set $GU_{Nat} = {{ 0, S 0, SS 0, ... }}$,
which uniquely corresponds to the natural numbers 
${{ 0, 1, 2, ... }}$.
(More precisely, 
the natural numbers are isomorphic with $GU_{Nat}$.)


In contrast, given a type $A$ 
with element $a$, $b$, ..., the generators of $Set_{A}$ result in
a universe that contains terms such as
$add(\emptyset,a)$ and $add(add(\emptyset,a),a)$
which we would like
to identify, based on our conception of a set
as containing only one exemplar of a particular value.
To effect this we need additional equations imposing
constraints expressing what we consider as the
desired shape (or {\em normal form}) of the values
contained in the universe of $T$.
However, before we look at how to extend a signature
$%S$ defining $T$ with equations defining the 
(behavioral) properties of $T$ we will look
at another example illustrating how the choice
of a generator basis may affect the structure
of the value domain of a data type.

In the example presented in \sliref{8-Seq},
the profiles are given of the functions
that may occur in the signature specifying sequences.
(The notation <em>\_</em> is used to indicate parameter positions.)

<slide id=8-Seq cap="The ADT <em>Seq</em>">
<raw height=600>
@ Sequences
\zline{\fbox{$Seq$}}
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> $ %e : seq T$ \zline{empty}
<li> $ \_ |> \_ : seq T \* T -> seq T$ /// right append
<li> $ \_ <| \_ : T \* seq T -> seq T$ /// left append
<li> $ \_ \. \_ : seq T \* seq T -> seq T$ /// concatenation
<li> $<< \_ >> : T -> seq T$ /// lifting
<li> $<< \_,...,\_ >> : T^{n} -> seq T$ /// multiple arguments <!--\hspace{-0.2cm}\mbox{} -->
</ul>
:@ Generator basis 
<em> -- preferably one-to-one</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $G_{seq T } = {{ %e, |> }}$, \ifsli{\n}{}
$GU_{seq T } = {{ %e, %e |> a, %e |> b, ..., %e |> a |> b, ... }}$
- $G'_{seq T } = {{ %e, <| }}$, \ifsli{\n}{}
$GU'_{seq T } = {{ %e, a <| %e , b <| %e, ..., b <| a <| %e, ... }}$
- $G''_{seq T } = {{ %e,  \. , << \_ >> }}$, %%\ifsli{\n}{}
$GU''_{seq T } = {{ %e, <<a>>, <<b>>, ,..., %e \. %e, ..., %e \. <<a>> , ... }}$ 
I>
:@ Infinite generator basis
\ifsli{\vspace{-0.5cm}}{}
<I
- $G'''_{seq T } = {{ %e, << \_ >>, << \_ , \_ >>, ... }}$, \ifsli{\n}{}
$GU'''_{seq T } = {{ %e, <<a>>, <<b>>, ,..., <<a,a>> , ... }}$ 
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

Dependent on which producer functions are selected to
generate the universe of $T$, the correspondence
between the generated universe and the intended domain
is either {\em one-to-one}
(as for $G$ and $G'$) or {\em many-to-one}
(as for $G''$).
Since we require our specification to be first-order
and finite, infinite generator bases (such as $G'''$)
must be disallowed, even if they result
in a one-to-one correspondence.
See [_Dahl92] for further details.
