<document>
<a name=abstract>
@+ Abstract interfaces
<include file=navigate>

\ifsli{\def\baselinestretch{1.2}}{}

When choosing for the module realization of
the data abstraction $list$ in C style, we are likely
to have an abstract functional interface as specified
in \sliref{mod-face}.


\vstr{1}
<slide id=mod-face cap="Modules -- a functional interface">
:@ Modules <em> -- a functional interface</em>
/// <box>ADT</box>
<program>
typedef int element; 
struct list;

extern list* nil();
extern list* cons(element e, list* l);
extern element head(list* l);
extern list* tail(list* l);
extern bool equal(list* l, list* m);
</program>

%%\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

\noindent
For convenience, the $list$ has been restricted to
contain integer elements only.
However, at the expense of additional notation,
we could also easily define a generic list
by employing template functions as provided 
by C++.
This is left as an exercise for the reader.

The interface of the abstract class $list$
given in \sliref{obj-face} has been defined 
generically by employing templates.


\vstr{1}
<slide id=obj-face cap="Objects -- a method interface">
:@ Objects <em> -- a method interface</em>
/// <box>OOP</box>
<program>
template< class E > 
class list {
public:
list() { }
virtual ~list() { }
virtual bool empty() = 0;
virtual E head()  = 0;
virtual list<E>* tail() = 0;
virtual bool operator==(list<E>* m) = 0;
};
</program>

\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Note that the $equal$ function in the ADT
interface takes two arguments,
whereas the {P operator== P} function in the OOP
interface takes only one,
since the other is implicitly provided by the 
object itself.

\ifsli{\def\baselinestretch{0.7}}{}
</document>
