<a name=obs>
@+ Adding new observers
<include file=navigate>

Now, for the complementary case, what happens
when we add new observers to the specification of a
data type?
Somewhat surprisingly, the object-oriented approach
now seems to be at a disadvantage.

Since in a module realization of an abstract data type
the code is organized around observers, adding a new
observer function amounts simply to adding a new operation
with a case for each of the possible generator
types, as shown in \sliref{8-mod-obs}.


\ifsli{\mbox{}\vspace{-1cm}}{}
<slide id=8-mod-obs cap="Modules and observers">
:@ Adding new observers
/// <box>ADT</box>
<program>
int length( list* l ) { ///<box>length</box>
      switch( l->tag ) {
      case NIL: return 0;
      case CONS: return 1 + length(l->next);
      case INTERVAL: return l->z - l->e + 1;
      };
}
</program>
</slide>

When we look at how we may extend a given object 
realization of an abstract data type
with a new observer we are facing a problem.

The obvious solution is to modify the source
code and add the $length$ function to the $list$
interface class and each of the generator classes.
This is, however, against the spirit of object
orientation and may not always be feasible.

Another, rather awkward solution,
is to extend the collection of
possible generator subtypes with a number of new
generator subtypes that explicitly incorporate the new
observer function.
However, this also means redefining the $tail$
function since it must deliver an instance
of a {\em list with length} class.
%%(This solution does not compile with
%%the AT\&T 3.0 compiler, which does not allow one
%%to change the result type of virtual function, but it will
%%be accepted by compilers conforming
%%to the ANSI/ISO standard.)


As a workaround, one may
define a function $length$
and an extended version of the
$list$ template class supporting only the
$length$ (observer) member function
as depicted in
\sliref{8-oop-obs}.


\cleartrue
<slide id=8-oop-obs cap="Objects and observers">
:@ Adding new observers
/// <box>OOP</box>
<program>
template< class E >  
int length(list< E >* l) { ///<box>length</box>
   return l->empty() ? 0 : 1 + length( l->tail() );
 }

template< class E >
class listWL : public list&lt;E&gt; {  ///<box>listWL</box>
public:
int length() { return ::length( this ); }
};
</program>
</slide>

A program fragment illustrating the use of the 
$listWL$ class is given below.
[P
list&lt;int>* r = new cons&lt;int>(1,new cons&lt;int>(2,new interval(3,7))); 
while (! r->empty()) { 
	cout << ((listWL< int >*)r)->length() << endl;
	r = r->tail();
	}
delete r;
P]

\noid
Evidently, we need to 
employ a cast whenever we wish to
apply the $length$ observer function.
Hence, this seems not to be the right solution.

Alternatively, we may use the function $length$ directly.
However, we are then forced to mix method
syntax of the form $ref->op(args)$ with function syntax
of the form $fun(ref,args)$, which may easily lead
to confusion.


:@ Discussion
%%To my  mind, this looks rather clumsy at best.
We may wonder why an object-oriented approach,
that is supposed to support extensibility, is
at a disadvantage here when compared to a more
traditional module-based approach.

As observed in [_Cook90], the problem lies in the fact
that neither of the two approaches reflect
the full potential and flexibility of
the matrix specification of an abstract data type.
Each of the approaches represents a particular choice
with respect to the decomposition of the matrix, into
either an {\em operations-oriented} (horizontal) decomposition
or a {\em data-oriented} (vertical) decomposition.

The apparent misbehavior of an object realization
with respect to extending the specification with observer
functions explains why in some cases we prefer
the use of overloaded functions rather than methods,
since overloaded functions allow for implicit dispatching
to take place on multiple arguments,
whereas method dispatching behavior is determined only
by the type of the object.

However, it must be noted that the dispatching behavior
of overloaded functions in C++ is of a purely syntactic nature.
This means that we cannot exploit the information
specific for a class type as we can when using
virtual functions.
Hence, to employ this information we would be required
to write as many variants of overloaded functions
as there are combinations of argument
types.

Dynamic dispatching on multiple arguments is
supported by
{\em multi-methods} in CLOS,
see [_Paepcke93].
According to [_Cook90], the need for such methods
might be taken as a hint that objects
only partially realize the true potential of {\em data abstraction}.
