<document>
@* Decomposition -- modules versus objects
\label{adt-modules}

<include file=navigate>

<c>
<slide color=silver id=modules-sections mode=hide>
:@ Subsections:
<ul>
<li> <a href=modules-abstract.html>Abstract interfaces</a>
<li> <a href=modules-represent.html>Representation and implementation</a>
<li> <a href=modules-gen.html>Adding new generators</a>
<li> <a href=modules-obs.html>Adding new observers</a>
</ul>
</slide>
<hr>
</c>

@$=P

Abstract data types allow the programmer to define a complex
data structure and an associated collection of functions,
operating on that structure, in a consistent way.
Historically, the idea of data abstraction was originally
not type-oriented but arose from a more pragmatic concern with
information hiding and representation abstraction,
see [_Parnas72b].
The first realization of the idea of data abstraction was
in the form of modules grouping a collection of functions
and allowing the actual representation
of the data structures underlying the values of
the (abstract) type domain to be hidden,
see also [_Parnas72a].

In [_Cook90], a comparison is made between the way in which
abstract
data types are realized traditionally (as modules)
and the way abstract data types
may be realized using object-oriented programming techniques.
According to [_Cook90], these approaches must be regarded as
being orthogonal to one another and, being to some extent
complementary, deserve to be integrated in
a common framework.

After presenting an example highlighting the differences between
the two approaches, we will further explore these differences and
study the trade-offs with respect to possible extensions
and reuse of code.


\cleartrue
\vstr{1}
<slide id=8-decomposition cap="Decomposition and data abstraction">
@ Decomposition <em> -- matrix</em>
<p align=right><box>data abstraction</box>
<escape>
<raw height=300 width=400>
\ln
\begin{center}
\mbox{}\hspace{-2.5cm}
\begin{tabular}{| r | c c |} \hline
 & nil & cons(h,t) \\ \cline{1-3}
 empty(l) & true & false \\
 head(l) & error & h \\
 tail(l) & error & t \\ \hline
\end{tabular}
\end{center}\ln
</raw>
</escape>
@ Modules <em> -- operation oriented</em>
<p align=right><box>ADT</box>
<ul>
<li> organized around observers <em> -- representation hiding</em>
</ul>
@ Objects <em> -- data oriented</em>
<p align=right><box>OOP</box>
<ul>
<li> organized around generators <em> -- method interface</em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

Recall that abstract data types may be completely characterized
by a finite collection of generators and a number of
observer functions that are defined with respect to each possible
generator.
Following this idea, we may approach the specification of a data
abstraction by constructing a {\em matrix}
listing the generators column-wise and the observers
row-wise, which for each {\em observer/generator} pair
specifies
the value of the observer for that particular generator.
Incidentally, the definition of such a matrix allows us
to check
in an easy way
whether we have given a complete characterization of the data type.
Above, an example is given of the specification of a $list$,
with generators $nil$ and $cons$, and observers $empty$, $head$
and $tail$.
(Note that we group the secondary producer $tail$ with the observers.)

Now, the traditional way of realizing abstract data types
as modules may be characterized as {\em operation oriented},
in the sense that the module realization of the type is organized
around the observers, resulting in a horizontal decomposition
of the matrix.

On the other hand, an object-oriented approach may be characterized
as {\em data oriented}, since the object realization of a type
is based on specifying a method interface for each
possible generator (sub)type,
resulting in a vertical decomposition of the matrix.
\see{8-decomposition}

Note, however, that in practice, different generators need
not necessarily correspond to different (sub)classes.
Behavior may be subsumed in variables, as an object
cannot change its class/type.
%%See section \ref{roles}.

<so modules-abstract>
<so modules-represent>
<so modules-gen>
<so modules-obs>

</document>
