@+ Equations <em> -- specifying constraints</em>

The specification of the signature of a type
(which lists the {\em syntactic constraints}
to which a specification must comply)
is in general not sufficient to characterize
the properties of the values of the type.
In addition, we need to impose {\em semantic constraints}
(in the form of equations) to define the meaning
of the observer functions and (very importantly) 
to identify the elements of the type domain
that are considered equivalent (based on
the intuitions one has of that particular type).

\index{equivalence}
\index{congruence}
\ifsli{\mbox{}\vspace{3cm}}{}
<slide id=8-equivalence cap="Equivalence">
<raw height=400>
:@ The equivalence relation <em> -- congruence</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $x = x$ \zline{reflexivity}
- $x = y => y = x$ \zline{symmetry}
- $x = y /\ y = z => x = z$ \zline{transitivity}
- $x = y => f(...,x,...) = f(...,y,...)$
I>
@ Equivalence classes <em> -- representatives</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- abstract elements <em> -- $GU_T /= $ </em>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

Mathematically, the equality predicate may
be characterized by the properties listed above,
including $reflexivity$ (stating that an element 
is equal to itself), $symmetry$ (stating that 
the orientation of the formula is not important)
and $transitivity$ (stating that if one element
is equal to another and that element is equal to
yet another, then the first element is also equal to
the latter).
In addition, we have the property that, given
that two elements are equal, the results
of the function applied to them (separately) are
also equal.
(Technically, the latter property makes a {\em congruence}
of the equality relation, lifting 
equality between elements to the function level.)
\see{8-equivalence}

Given a suitable set of equations, in addition to
a signature,
we may identify the elements that can be proved
identical by applying the equality relation.
In other words, given an equational theory
(of which the properties stated above must be a part),
we can divide the generator universe of a type $T$
into one or more subsets,
each consisting of elements that are equal
according to our theory.
The subsets of $GU/=$, that is $GU$
factored with respect to equivalence,
may be regarded as the abstract elements constituting the type 
$T$,
and from each subset we may choose a concrete
element acting as a {\em representative} for the subset
which is the equivalence class of the element.

%%@ Abstract evaluation
Operationally, equations may be regarded as 
{\em rewrite rules} (oriented from left to right),
that allow us to transform a term in which
a term $t_1$ occurs as a subterm into a term
in which $t_1$ is replaced by $t_2$ if $t_1 = t_2$.
For this procedure to be terminating,
some technical restrictions must be met,
amounting (intuitively) to the requirement that the right-hand side
must in some sense be simpler than the left-hand side.

Also, when defining an observer function, we must specify for
each possible generator case an appropriate rewriting rule.
That is, each observer must be able to give
a result for each generator.
The example of the natural numbers, given below, will make
this clear.
Identifying spurious elements by rewriting a term into
a canonical form is somewhat more complex,
as we will see for the example of sets.

@ Equational theories
To illustrate the notions introduced above,
we will look at specifications  of some familiar
types, namely the natural numbers and sets.

%%@ Natural numbers
In \sliref{8-Nat}, an algebraic specification is given of the natural
numbers (as first axiomatized by Peano).

<slide id=8-Nat cap="The ADT Nat">
@ Natural numbers
/// <box>Nat</box>
[P
functions
0 : Nat
S : Nat -> Nat
mul : Nat * Nat -> Nat
plus : Nat * Nat -> Nat
axioms
[1] plus(x,0) = x
[2] plus(x,Sy) = S(plus(x,y))
[3] mul(x,0) = 0
[4] mul(x,Sy) = plus(mul(x,y),x)
end
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>
 

In addition to the constant $0$ and successor function
$S$ we also introduce a function $mul$ for
multiplication and a function $plus$ for addition.
(The notation $Sy$ stands for application by 
juxtaposition; its meaning is simply $S(y)$.)
The reader who does not immediately accept
the specification in \sliref{8-Nat} as an adequate
axiomatization of the natural numbers must
try to unravel the computation depicted in \sliref{8-symbolic}.

<slide id=8-symbolic cap="Symbolic evaluation">
\ifsli{}{\vspace{0.1cm}}
[P
mul(plus(S 0,S 0),S 0) -[2]-> 
mul(S(plus(S 0,0)), S 0) -[1]-> 
mul(SS 0,S 0) -[4]->
plus(mul(SS0,0),SS0) -[3]->
plus(0,SS0) -[2*]-> SS0
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Admittedly, not an easy way to compute with
natural numbers, but fortunately term rewriting
may, to a large extent, be automated (and actual calculations
may be mimicked by semantics preserving primitives).

Using the equational theory expressing the properties
of natural numbers, we may eliminate
the occurrences of the functions $mul$ and $plus$
to arrive (through symbolic evaluation) at something
of the form $S^n 0$ (where $n$ corresponds 
to the magnitude of the natural number denoted by the term).

The opportunity of symbolic evaluation by term
rewriting is exactly what has made the algebraic
approach so popular for the specification of software,
since it allows (under some restrictions)
for executable specifications.

Since they do not reappear in what may be considered
the {\em normal forms} of terms denoting the naturals
(that are obtained by applying the evaluations induced by the
equality theory),
the functions $plus$ and $mul$ may be regarded as {\em secondary}
producers.
They are not part of the generator basis of the type $Nat$.

Since we may consider $mul$ and $plus$ as secondary
producers at best, we can easily see that when we define 
$mul$ and $plus$ for the case $0$ and $Sx$
for arbitrary $x$, that we have covered all possible (generator) cases.
Technically, this allows us to prove properties of these
functions by using structural induction on the possible generator cases.
The proof obligation (in the case of the naturals) then is to
prove that the property holds for the function applied
to 0 and assuming that the property 
holds for applying the function to $x$, it also
holds for $Sx$.

%%@ Sets
As our next example, consider the algebraic
specification of the type <raw>$Set_{A}$</raw> in \sliref{8-Set}.

<slide id=8-Set cap="The ADT Set">
<raw height=400>
@ Sets
\zline{\fbox{$Set$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $G_{Set_{A}} = {{ \emptyset, add }}$
- $GU_{Set_{A} }$ $ = $
\ifsli{\n\mbox{}\hspace{1cm}}{}
${{ 0, add(0,a), ..., add(add(0,a),a), ... }}$
I>
</raw>
:@ Axioms
\ifsli{\vspace{-0.5cm}}{}
[P
[S1] $add(add(s,x),y) = add(add(s,y),x)$ /// commutativity
[S2] $add(add(s,x),x) = add(s,x)$ /// idempotence
P]
\ifsli{\vspace{1.0cm}}{}
</slide>


In the case of sets we have the problem that
we do not start with a one-to-one generator base
as we had with the natural numbers.
Instead, we have a many-to-one generator base,
so we need equality axioms to eliminate spurious
elements from the (generator) universe of sets.

<slide id=8-set-equi cap="Equivalence classes for Set">
<raw height=300>
:@ Equivalence classes
/// <box>$ GU_{ Set _{A} }/= $</box>
\ifsli{\vspace{-0.5cm}}{\vspace{0.1cm}}
<ul>
<li> ${{ \emptyset }}$
<li> ${{ add(0,a), add(add(0,a),a), ... }}$
<li> $...$
<li> ${{ add(add(0,a),b), add(add(0,b),a), \ifsli{..}{...} }}$
</ul>
\ifsli{\vspace{-1cm}}{\vspace{-0.2cm}}
</raw>
</slide>


The equivalence classes of <raw>$GU_{Set _{A} }/=$</raw> 
(which is <raw>$GU_{Set _{A} }$</raw> factored by the equivalence
relation), each have multiple elements (except
the class representing the empty set).
To select an appropriate representative
from each of these classes (representing the abstract
elements of the type <raw>$Set_{A}$</raw>) we need
an ordering on terms, so that we can take the smaller 
term as its canonical representation.
\see{8-set-equi}
