<a name=refine>
@+ The subtype refinement relation
<include file=navigate>

In determining whether a given type is a subtype of another type, we must make 
a distinction between simple (or basic) types built into the language
and compound (or user-defined) types explicitly declared by the programmer.
Compound types, such as {\em integer subranges}, {\em functions},
{\em records} and {\em variant records}, themselves make use
of other (basic or compound) types.
Basic types are (in principle) only a subtype of themselves,
although many languages allow for an implicit subtyping relation
between for example integers and reals.
The rules given in \sliref{9-subtypes}
characterize the subtyping relation for the 
compound types mentioned. 

<slide id=9-subtypes cap="The subtype refinement relation">
<raw height=500>
@ Sub-range inclusion
\zline{\fbox{$ <= $ }}
\ifsli{\vspace{-0.5cm}}{}
*- \ffrac{$n <= n'$ and $m' <= m$}{$n'..m' <= n..m$} 
:@ Functions
\zline{\fbox{contravariance}}
\ifsli{\vspace{-0.5cm}}{}
*- \ffrac{$%s <= %s'$ and $%t' <= %t$}{$%s' -> %t' <= %s -> %t$} 
@ Records
<I
- \ffrac{
$%s_i <= %t_i$ for $i = 1..m$ ($m <= n$)}{ 
  ${{a_1: %s_1, ... , a_n: %s_n}} <= {{a_1: %t_1 , ... , a_m: %t_m}}$
  } 
I>
@ Variants
<I
- \ffrac{$%s_i <= %t_i$ for $i = 1..m$ ($m <= n$)}{
$[a_1:%s_1 \/ ... \/ a_m:%s_m] <= [a_1:%t_1 \/ ... \/ a_n:%t_n]$} 
I>
</raw>
</slide>

@$=raw

We use the relation symbol $ <= $ to denote the subtype relation.
Types (both basic and compound) are denoted by $%s$ and $%t$.
For subranges, a given (integer) subrange $%s$ is a subtype
of another subrange $%t$ if $%s$ is (strictly) included in $%t$
as a subset.
In other words, if $%s = n'..m'$ and $%t = n..m$ then the subtyping
condition is $n <= n'$ and $m' <= m$.
We may also write $n'..m' \subseteq n..m$ in this case.

For functions we have a somewhat similar rule,
a function $f' : %s' -> %t'$ (with domain $%s'$
and range or codomain $%t'$) is a subtype of a function
$f : %s -> %t$ (with domain $%s$ and codomain $%t$)
if the subtype condition $%s <= %s'$ and $%t' <= %t$
is satisfied.
Note that the relation between the domains is contravariant,
whereas the relation between the ranges is covariant.
We will discuss this phenomenon of contravariance below.

Records may be regarded as a collection of
labels (the record fields) that may have values of
a particular type.
The subtyping rule for records expresses that a given record
(type)
may be extended to a (record) subtype by adding
new labels, provided that the types for labels which
occur in both records are refined in the subtype.
The intuition underlying this rule is that by extending a record
we add, so to speak, more information concerning
the individuals described by such a record,
and hence we constrain the set of possible elements
belonging to that (sub)type.

Variants are (a kind of) record that leave the choice
between a (finite) number of possible values, each
represented by a label.
The subtyping rules for variants states that we may
create a subtype of a given variant record if we
reduce the choice by eliminating one or more 
possibilities.
This is in accord with our notion of refinement
as improving our knowledge, since by reducing
the choice we constrain the set of possible individuals
described by the variant record.

The subtyping rules given above specify what
checks to perform in order to determine whether
a given (compound) type is a subtype of another type.
In the following we will look in more detail at
the justification underlying these rules, and also hint
at some of the restrictions and problems implied.
However, let us first look at some examples.
\see{9-ex-subtyping}


<slide id=9-ex-subtyping cap="Examples of subtyping">
<raw height=400>
:@ Examples
\zline{\fbox{subtyping}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ 8..12 -> 3..5 <= 9..11 -> 2..6$
- $ {{ age : int, speed : int, fuel : int }}  <=  {{ age : int,  speed : int }} $
- $ [ yellow  \/ blue ]  <  [ yellow \/ blue  \/ green  ] $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

As a first example, 
when we define a function $f' : 8..12 -> 3..5$ and a function
$f : 9..11 -> 2..6$ then, according to our rules,
we have $f' <= f$.
Recall that we required subtypes to be compatible
with their supertypes, compatible in the sense that an instance
of the subtype may be used at all places where an instance 
of the supertype may be used.
With regard to its signature, obviously, $f'$ may be used everywhere
where $f$ may be used, since $f'$ will deliver a result
that falls within the range of the results expected
from $f$ and, further, any valid argument for $f$ will
also be accepted by $f'$ (since the domain of
$f'$ is larger, due contravariance, than the domain of $f$).

As another example, look at the relation between
the record types 
$ {{ age : int, speed : int, fuel : int }}$ and
$ {{ age : int, speed : int }}$.
Since the former has an additional field $fuel$
it delimits so to speak the possible entities falling under
its description and hence may be regarded as a subtype
of the latter.

Finally, look at the relation between the variant records
$ [yellow : color \/ blue : color ]$ and
$ [yellow : color \/ blue : color \/ green : color ]$.
The former leaves us the choice between the colors
$yellow$ and $blue$, whereas the latter also allows
for $green$ objects
and, hence, encompasses the set associated with
$ [yellow : color \/ blue : color ]$.

@ Contravariance rule
The subtyping rules given above are all rather intuitive,
except possibly for the $function$ subtyping rule.
Actually, the {\em contravariance} 
expressed in the function subtyping rule is somewhat of an embarrassment
since it reduces the opportunities for specializing functions
to particular types.
\see{9-functions}

%%\ifsli{\mbox{}\vspace{3cm}}{}
<slide id=9-functions cap="The function subtype relation">
<raw height=300>
@ Function refinement
\ifsli{\vspace{-0.5cm}}{}
<I>- $ f' : Nat -> Nat \not<= f : Int -> Int $
:@ Functions <em> -- as a service</em>
\zline{\fbox{contravariance}}
\ifsli{\vspace{-0.5cm}}{}
<I
- domain {\em -- restrictions for the client}
- range  {\em -- obligations for server}
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

Consider, for example, that we have a function
$f : Int -> Int$, then it seems quite natural to specialize
this function into a function $f' : Nat -> Nat$ 
(which may make use of the fact that $Nat$ only contains
the positive elements of $Int$).
However, according to our subtyping rule $ f' \not<= f$,
since the domain of $f'$ is smaller than the 
domain of $f$.

For an intuitive understanding of the function subtyping rule,
it may be helpful to regard a function as a {\em service}.
The domain of the function may then be interpreted as 
characterizing the restrictions imposed on the
client of the service (the caller of the function)
and the codomain of the function as somehow expressing
the benefits for the client
and the obligations for the (implementor of the) function.
Now, as we have already indicated,
to refine or improve on a service
means to relax the restrictions imposed on the client
and to strengthen the obligations of the server.
This, albeit in a syntactic way, is precisely
what is expressed by the contravariance rule for
function subtyping.


