<document>
<a name=bound>
@+ Bounded polymorphism
<include file=navigate>
@$=raw

Our next extension, which we call $F_{ <= }$, involves
(bounded) universal quantification.
For technical reasons we need to introduce  a
primitive type $Top$, which may be considered as
the supertype of all types (including itself).
Also we need type abstraction variables, that we will
write as $%a$ and $%b$.
Our notation for a universally quantified (bounded) type
is $\A %a <= %s . %t$, which denotes the type $%t$
with the type variable $%a$ replaced by any subtype $%s'$ of $%s$.
In a number of cases, we will simply write $\A %a. %t$,
which must be read as $\A %a <= Top. %t$.
Recall that any type is a subtype of $Top$.
Observe that, in contrast to $%l_{ <= }$ and $%l_{ /\ }$,
the calculus $F_{ <= }$ is second order (due to the quantification
over types).

In addition to the (value) expressions found in the two
previous calculi, $F_{ <= }$ introduces a {\em type abstraction}
expression of the form $%L %a <= %t.e$ and a {\em type instantiation}
expression of the form $e[%t]$.
The type abstraction expression $%L %a <= %t.e$
is used in a similar way as the function abstraction expression,
although the abstraction involves types and not values.
Similar to the corresponding type expression,
we write $%L %a.e$ as an abbreviation for $ %L %a <= Top.e $.
The (complementary) type instantiation statement is written
as $e[%t]$, which denotes the expression $e$ in which
the type identifier $%t$ is substituted for the type variable
bound by the first type abstractor.

<slide id=9-c-bounded cap="The bounded type calculus">
<raw height=400>
@ Bounded polymorphism <em> -- abstraction</em>
\zline{\fbox{ $F_{ <= }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %t ::= Top | %a | %r | %t_1 -> %t_2 | \A %a <= %t_1. %t_2$
- $ e ::= x | %l x:%t.e | e_1  e_2 | %L %a <= %t.e | e [ %t ]$
I>
:@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$%G, %a <= %s  |- e : %t$   }{
    $%G |- %L %a <= %s. e \e \A %a <= %s . %t$
    }
- \ffrac{$%G, e : \A  %a <= %s . %t$ \hspace{1cm} $ %G  |- %s' <= %s $   }{
    $%G |- e [ %s' ] \e %t [ %a := %s' ]$ 
    }
I>
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{ $%G |- %s <= %s'$ \hspace{1cm} $ %G |- %t' <= %t$}{
   $%G |- \A %a <= %s'.%t' <= \A %a <= %s.%t $
   }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The type assignment rule for type abstraction states
that, when we may type an expression $e$ as being of
type $%t$ (under the assumption that $%a <= %s$),
then we may type $%L %a <= %s.e$ as being of type $\A %a <= %s.%t$.

The type assignment rule for type instantiation
characterizes the relation between type instantiation
and substitution (which is notationally very similar).
When we have an expression $e$ of type $\A %a <= %s.%t$
and we have that $%s' <= %s$, then $e[%s']$ is of type 
$ %t [ %a := %s' ]$, which is $%t$ with $%s'$ substituted for
$%a$.
\see{9-c-bounded}

The refinement rule for bounded types states
the subtyping relation between two bounded types.
We have that $\A %a <= %s' . %t'$ is
a subtype of $\A %a <= %s.%t$ whenever 
$%s <= %s'$ and $%t' <= %t$.
Notice that the relation is contravariant with respect
to the types bounding the abstraction, similar
as for the domains of function subtypes in
the function subtyping rule.

In contrast to the polymorphism due to object type
extensions and overloading, bounded polymorphism
(employing type quantifiers) is an example
of what we have called parametric polymorphism.
In effect, this means that we must explicitly give a type
parameter to instantiate an object or function of
a bounded (parametric) type, similar to when we use
a template in C++.

The examples given in
\sliref{9-ex-parameters}
illustrate how we may define and subsequently
type parametric functions.
In these examples, we employ the convention
that in the absence of a bounding type we assume $Top$
as an upper limit.
The examples are taken from [_CW85].

<slide id=9-ex-parameters cap="Parametrized types -- examples">
<raw height=400>
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<I
- $id = %L %a. %l x:%a.x$ \n
$id : \A %a. %a -> %a $ 
- $twice1 = %L %a.%l f: %L %b. %b -> %b. %l x:%a. f[%a](f[%a](x))$ \n
$twice1 : \A %a. \A %b. (%b -> %b) -> %a -> %b$ 
- $twice2 = %L %a.%l f: %a -> %a. %l x:%a. f(f(x))$ \n
$twice2 : \A %a. (%a -> %a) -> %a -> %a$ 
I>
:@ Applications
\ifsli{\vspace{-0.5cm}}{}
<I
- $id [ Int ] ( 3 ) = 3$
- $twice1 [ Int ] ( id )( 3 ) = 3$
- $twice1 [ Int ] ( S ) = illegal$
- $twice2 [ Int ] ( S )( 3 ) = 5$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The (generic) identity function $id$
is defined as $ %L %a.%l x:%a.x$, which states
that when we supply a particular type, say $Int$,
then we obtain the function $%l x:Int. x$.
Since the actual type used to instantiate $id$ is not important,
we may type $id$ as being of type $\A %a.%a -> %a$.
In a similar way, we may define and type the two
(generic) variants of the function $twice$.
Notice the difference between the two definitions of $twice$.
The first variant requires the function argument itself
to be of a generic type, and fails (is incorrectly typed)
for the successor function $S$ which is (non generic)
of type $Int -> Int$.
In contrast, the second variant accepts $S$, and we may
rely on the automatic conversion
of $id : \A %a. %a -> %a$ to $ id [ Int ] : Int -> Int$
(based on the second type assignment rule) to accept $id$
as well.

The interplay between {\em parametric} and {\em inclusion}
polymorphism is illustrated in the examples presented
in \sliref{9-ex-quantification}.
Recall that inclusion polymorphism is based
on the subtyping relation between records
(which states that refinement of a record type involves the addition
of components and/or refinement of components that
already belong to the super type).

<slide id=9-ex-quantification cap="Bounded quantification -- examples">
<raw height=400>
@ Bounded quantification
\ifsli{\vspace{-0.5cm}}{}
<I
- $g = %L %a <= {{ one : Int }}. %l x: %a. (x.one)$ \n
$ g : \A %a <= {{ one : int }}. %a -> Int $
- $g' = %L %b . %L %a <= {{ one : %b }}. %l x: %a. (x.one)$ \n
$ g' : \A %b . \A %a <= {{ one : %b }}. %a -> %b $
- $ move = %L %a <= Point. %l p:%a. %l d : Int .(p.x := p.x + d); p$ \n
$move : \A %a <= Point. \, %a -> Int -> %a$
I>
:@ Application
\ifsli{\vspace{-0.5cm}}{}
<I
- $g' [ Int ][ {{ one:Int, two : Bool }}]({{ one = 3, two = true }}) = 3$
- $move [{{ x : Int, y : Int }}]( {{ x = 0, y = 0 }} )(1) = {{ x = 1, y = 0 }}$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The first example defines a function $g$ that works on a record
with at least one component $one$ and delivers as a result
the value of the component $one$ of the argument record.
The function $g'$ is a generalized version of $g$
that abstracts from the particular type of the $one$
component.
Notice that both $g$ and $g'$ may be applied to any record
that conforms to the requirement stated in the bound,
such as the record $ {{ one = 3, two = true }} $.

As another example of employing bounds to impose requirements,
look at the function $move$ that is defined for subtypes of $Point$
(which we assume to be a record containing $x$ and $y$ coordinates).
It expects a record (that is similar to or extends $Point$)
and an (integer) distance, and as a result delivers the modified
record.


:@ Discussion
Parametric polymorphism is an important means to incorporate
subtyping in a coherent fashion.
Apart from [_Pierce93], from which we have taken most of the
material presented here, we may mention
[_PA93] as a reference for further study.
In [_Pierce93] a calculus $F_{ /\ }$ is also introduced
in which intersection polymorphism is expressed by
means of an explicit type variable.
The resulting type may be written as $\A %a \e {{ ... }}$,
where ${{ ... }}$ denotes a finite collection of types.
As already mentioned, intersection types may also be used
to model inclusion polymorphism
(see Castagna {\it et al.}, 1993).\index{Castagna {\it et al.} (1993)}

It is an interesting research issue to explore
the relation between parametric polymorphism and 
inclusion polymorphism further along this line.
However, we will not pursue this line here.
Instead, in the next section we will look
at another application
of parametric polymorphism, namely existential types
that allow us to abstract from hidden component types.
This treatment is based on [_CW85].
In the last section of this chapter, we will look in
more detail at the role of self-reference
in defining (recursive) object types,
following [_CoHC90].
We will conclude this chapter with some
observations concerning the relevance of
such type theories for actual programming languages.
In particular, we will show that Eiffel is
not type consistent.

@$=tt

:@ Type abstraction in C++
Type abstraction in C++ may occur in various
guises.
One important means of type abstraction
is to employ
what we have called polymorphic base class hierarchies.
For example, the function $move$,
which was somewhat loosely characterized in
\sliref{9-ex-quantification},
may be defined in C++ as follows:
<slide type=hide id=c3-x1 cap="example move">
[P
Point* move(Point* p, int d); /// require <tt>int Point::x</tt>
Point* move(Point* p, int d) { p.x += d; return p; }
P]
</slide>

\noid
In effect, the function $move$ accepts a pointer
to an instance of $Point$, or
any class derived from $Point$, satisfying the requirement
that it has a public integer data member $x$.

Similar restrictions generally hold when
instantiating a template class,
but in contrast to base class subtyping requirements,
these restrictions will only be verified at link time.


<slide id=9-cc-abs cap="Type abstraction in C++">
\ifsli{}{\vspace{0.1cm}}
<program>
template< class T > /// requires <tt>T::value()</tt>
class P {
public:
P(T& r) : t(r) {}
int operator==( P<T>& p) {
	return t.value() == p.t.value();
	}
private:
T& t;
};
</program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Consider the template class definition given
in \sliref{9-cc-abs}.
Evidently, for the comparison function to
operate properly, each instantiation type
substituted for the type parameter $T$
must satisfy the requirement that it has a public member function
$value$.

<slide id=9-cc-abs-2 cap="Type instantiation">
\ifsli{}{\vspace{0.1cm}}
[P
template< class T >
class A { /// <box> A&lt;T&gt; </box>
public: 
virtual T value() = 0;
};

class Int : public A&lt;int&gt; { /// <box> Int $ <= $ A&lt;int&gt; </box>
public:
Int(int n = 0) : _n(n) {}
int value() { return _n; }
private:
int _n;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Such a requirement may also be expressed by defining
an abstract class $A$ defining a pure
virtual member function $value$.
See \sliref{9-cc-abs-2}.
The restrictions on instantiating $P$
may then be stated informally as
the requirement that each instantiation type
$T$ must be a subtype of $A<X>$ for arbitrary type $X$.
The class $Int$ is an example of a type complying with
the implicit requirements imposed by the definition of $P$.
An example of using $P$ is given below
\cleartrue
<slide id=c3-x-2 type=hide cap="Example P">
[P
Int i1, i2;
P&lt;Int&gt; p1(i1), p2(i2);
if ( p1 == p2 ) cout << "OK" << endl; /// OK
P]
</slide>

\noid
Note, however, that
the derivation of {P A&lt;int&gt; P} is by no means
necessary or in any way enforced by C++.
</document>

