<document>
<a name=simple>
@+ A simple type calculus
\label{calculi}
<include file=navigate>

@$=raw

In our first version of a type calculus
we will restrict ourselves to a given set
of basic types (indicated by the 
letter $%r$) and function types (written
$%s -> %t$, where $%s$ stands for the domain
and $%t$ for the range or codomain).
This version of the typed lambda calculus
(with subtyping) is called $%l_{ <= }$ in
[_Pierce93] from which most of the material is taken.
The $%l_{<=}$ calculus is a first order calculus, since it
does not involve quantification over types.
\see{9-c-subtypes}

The structure of type expressions is given by the 
definition
[P]  $ %t ::= %r | %t_1 -> %t_2$
where we use $%t$ as a type identifier and $%r$
as a meta variable for basic types.
The expressions of our language, that we indicate with 
the letter $e$, are similar to lambda terms,
except for the typing of the abstraction variable in
$%l x: %t.e$.


\index{type calculus}
<slide id=9-c-subtypes cap="The subtype calculus">
<raw height=400>
:@ A simple type calculus <em> -- subtypes</em>
\zline{\fbox{ $%l_{ <= }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %t ::= %r | %t_1 -> %t_2$
- $ e ::= x | %l x : %t . e | e_1 e_2 $
I>
\ifsli{\vspace{-0.3cm}}{}
@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$%G |- x : %s$ \hspace{1cm} $%G |- e : %t $}{
    $%G |- %l x : %s . e \e %s -> %t$
    }
- \ffrac{$%G |- e_1 : %s -> %t, e_2 : %s $}{
    $%G |- e_1 e_2 \e %t $
    }
I>
\ifsli{\vspace{-0.2cm}}{}
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{ $ %G |- e : %s$ \hspace{1cm}  $%G |- %s <= %t$}{
   $%G |- e : %t$
   }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>


To determine whether an expression $e$
is correctly typed (with some type expression $%t$)
we need type assignment rules, as given above.
Typing is usually based on a collection
of assumptions $%G$, that contains the typing of
expressions occurring in the expression
for which we are determining the type.
In the type assignment rules and the (subtyping)
refinement rules, the phrase $ %G |- e : %t$ means that
the expression $e$ has type $%t$, under the assumption that
the type assignments in $%G$ are valid.
When $%G$ is empty, as in $ |- e : %t $, the type assignment
holds unconditionally.
Occasionally, we write $ %G |- e \e %t$ instead of
$ %G |- e : %t$ for readability.
These two expressions have identical meaning.

The premises of a type assignment rule
are given above the line. The type assignment
given below the line states the assignment
that may be made on the basis of these premises.

For example, the first type assignment rule states that,
assuming $ %G |- x : %s $ ($x$ has type $%s$) and
$ %G |- e : %t $ ($e$ has type $%t$)
then $ %G |- %l x : %s. e \e %s -> %t$, in other words
the abstraction $ %l x:%s.e$ may be validly typed
as $ %s -> %t $.

Similarly, the second type assignment rule
states that applying a function $ e_1 : %s -> %t$
to an expression $ e_2 $ of type $%s$ results
in an (application) expression $ e_1 e_2 $ of type $ %t $.

We may assume the basic types denoted by $%r$
to include (integer) subranges, records and variants.
As a consequence, we may employ the subtyping rules
given in section \ref{subtypes} to determine the subtyping
relation between these types.
The (subtyping) refinement rule repeated here
expresses the substitutability property of subtypes,
which allows us to consider an expression
$e$ of type $%s$, with $%s <= %t$, as being
of type $%t$.

In \sliref{9-ex-subtypes}, some examples are given illustrating
the assignment of types to expressions.
Type assignment may to a certain extent be done
automatically, by type inference, as for example in ML,
see [_ML90].
%%(Milner {\it et al.}, 1990).\index{Milner {\it et al.} (1990)} 
However, in general, typing is not decidable
when we include the more powerful type expressions
treated later.
In those cases the programmer is required to
provide sufficient type information to enable the
type checker to determine the types.

<slide id=9-ex-subtypes cap="Subtypes -- examples">
<raw height=400>
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<I
- $ S = %l x:Int. x + 1$ \n
$S : Int -> Int $
- $ twice = %l f:Int -> Int. %l y : Int . f ( f ( y ) )$ \n
$ twice : ( Int -> Int ) -> Int -> Int$
I>
\ifsli{\vspace{-0.3cm}}{}
:@ Application
\ifsli{\vspace{-0.5cm}}{}
<I
- $ S 0 = 1 \e Int $
- $ twice ( S ) = %l x. S S x \e Int -> Int $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

When we define the successor function $S$ as
$%l x : Int. x + 1$ then we may type $S$
straightforwardly as being of type $Int -> Int$.
Similarly, we may type the (higher order) function
$twice$ as being of type $ (Int -> Int) -> Int -> Int$.
Note that the first argument to $twice$ must be a function.
Applying $twice$ to a function argument only results
in a function.
When applied to $S$ it results in a function
of type $Int -> Int$ that results in applying $S$
twice to its (integer) argument.
The subtyping rules (partly imported from section \ref{subtypes})
work as expected.
We may define, for example, a function
$ + : Real \* Real -> Int $ as a subtype of
$ + : Int \* Int -> Int $ (according
to the contra-variant subtyping rule for functions).

@$=tt

:@ Subtyping in C++
Subtyping is supported in C++ only to
a very limited extent.
Function subtypes are completely absent.
However, class subtypes due to derivation by
inheritance may be employed.
Also, built-in conversions are provided,
some of which are in accordance with the subtyping
requirements,
and some of which, unfortunately,
violate the subtyping requirements.
Built-in conversions exist, for example,
between $double$ and $int$, in both ways.
However, whereas the conversion from $int$ to $double$
is safe, the other way around may cause loss of
information by truncation.

The type system sketched in \sliref{9-c-subtypes}
is quite easily mapped to a C++ context.
For example, we may mimic the functions $S$
and $twice$ as given in \sliref{9-ex-subtypes}
in C++ as:
<slide type=hide id=cc-types cap="Types in C++">
<program>
int S(int x) { return x+1; }
int twice(int f(int), int y) { return f(f(y)); }
int twice_S(int y) { return twice(S,y); }
</program>
</slide>

\noid
Nevertheless, the type system of C++ imposes
some severe restrictions.
For example, functions may not be returned
as a value from functions.
(Although we may provide a workaround, when we employ
the $operator()$ function for objects.)

The absence of function subtyping becomes clear
when, for example, we call the function $twice$
with the function $SD$, which is defined as:
<slide type=hide cap="SD example">
\ifsli{\vspace{-1cm}}{}
[P
int SD(double x) { return x+1; }
P]
</slide>

\noid
According to the subtyping rules and in accordance
with the substitutability requirement,
we employ $SD$ whenever we may employ $S$.
But not so in C++.

We run into similar limitations when we try
to refine an object class descriptions following
the object subtype refinement rules.


<slide id=9-cc-sub cap="Subtyping in C++">
<display>
class P { /// <box>$P$</box>
public:
P() { _self = 0; }
virtual P* self() {
	return _self?_self->self():this;
	}
virtual void attach(C* p) {
	_self = p;
	}
private:
P* _self;
};

class C : public P { /// <box>$ C <= P$</box>
public:
C() : P(this) { }
C* self() { /// ANSI/ISO
	return _self?_self->self():this;
	}
void attach(P* p) {  /// rejected
	p->attach(self()); 
	} 
void redirect(C* c) { _self = c; }
private:
C* _self;
};
</display>
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Suppose we have a parent class $P$ which offers
the member functions $self$ and $attach$,
as in \sliref{9-cc-sub}.
The meaning of the function $self$ is that
it de-references the <em> _self</em> variable if
it is non-zero and delivers $this$ otherwise.
(See section \ref{hush} for an example of its use.)
The function $attach$ may be used to connect an
instance of $C$ to the <em> _self</em> variable.

The class $C$ in its turn inherits from $P$
and redefines $self$ and $attach$.
Syntactically, both refinements are allowed,
due to the function subtype refinements rules.
The function $self$ is redefined to deliver
a more tightly specified result,
and the $attach$ function is allowed to take a wider
range of arguments.

In a number of compilers for C++,
both redefinitions are considered illegal.
However, in the ANSI/ISO standard of C++,
redefining a member function to deliver
a subtype (that is, derived class) pointer will be allowed.
Redefining $attach$, as has been done for $C$
is probably not a wise thing to do,
since it changes the semantics of $attach$
as defined for the parent class $P$.
In effect, it allows us to write
$c->attach(p)$ instead of
$p->attach(c->self())$, for $P\!*\:p$ and $C\!*\:c$.
Nevertheless, from a type theoretical perspective,
there seem to be no grounds for forbidding it.
</document>
