<document>
@+ Example -- requests for loans
\label{Loans}

<include file=navigate>

Now let us take a look at an example business process model based on the objects made available in BPSIM. The example will detail possible situations before and after business process redesign has taken place.

<slide id=models type=fig cap="Processing alternatives">
<image name=@figures/bpsim/request>
</slide>

<slide mode=hide id=model-scenarios>
<dl>
<dt>Current situation:
<dd>Our first model, \sliref{models} (left),
which represents the current situation,
consists of a client producing,
for example, requests for a loan at a bank. The requests are initially handled by a clerk whose task is to send requests for amounts greater than \$10,000 to the boss. The boss, whose task is to approve or deny the loan, sends the result back to the clerk for processing. If the amount is less than \$10,000, the clerk has authority to process the request himself.

<dt>Redesign alternative:
<dd>In our second model, \sliref{models}(right),
which is the result of
a redesign effort, requests are handled by a clerk
who enters the data into a computer. The computer now makes the decision as to whether the loan is improved -- for loans of value
less than and greater than 10,000. The clerk then passes the results on to another clerk for the task of dispatching loans.
</dl>
</slide>

<slide id=present-models type=fig cap="Presenting a redesign alternative">
<image name=@figures/bpsim/combi2>
</slide>


<p>
In \sliref{present-models}
the visual representation of
the model resulting from the redesign effort is shown,
embedded in a Web page.
In addition to the model, the page contains
some results obtained by running a simulation.
For example, the text indicates that
an employee is busy handling incoming requests
only 33\% of the time.
The histogram depicts the distribution
of the lead times of incoming loan requests,
that is the time that passes between receiving
a request and giving an answer.
The Web page further contains a button to
start a simulation run,
a description of the model
and links to alternative models.


:@ The script
In the fragment below, a histogram for recording leadtimes
is created as well as an
agent generating requests and a transport to a 
clerk handling the requests.
<slide type=hide cap="The script">
<program>
histogram leadtimes
agent client generate leadtimes
client -duration 60 poisson
transport t1 
t1 -duration 20
employee clerk
operation receive clerk
receive -duration 20.0 5.0 normal 
</program>
</slide>
The duration of the agent is specified
using a poisson distribution with $\lambda = 60$.
The clerk is declared to perform a receive operation,
the duration of which takes values from a 
normal distribution with $\mbox{\it means} = 20$ and 
$\mbox{\it standard deviation} = 5.0$.
<ignore>
<slide mode=hide cap="component connection">
<program>
client -next t1
t1 -next clerk
</program>
</slide>
The fragment above shows how the various components
of a model (for this fragment the agent, transport and
employee) may be connected.
<p>
</ignore>
After defining the components of the model
and their connection a simulation may be started by
pressing the start button.
If desired, users may change the parameters concerning
for example setup and wait time, to explore
the various scenarios a model has to satisfy.

<ignore>
%%:@+ The  environment
Given the importance of participant communication in
a business process redesign effort,
embedding business simulation models  in hypermedia
provides in a natural way the information context
needed for exploring alternatives and taking decisions.
With Intranets becoming more popular, the World Wide Web
has proven to be a popular access point for many applications.
<p>
The <i>hush</i> library and its extensions offer
a rich environment for developing
hypermedia applications and dynamic Web documents
[_Applications].
<p>
The <i>hush</i> library originated as a C++
interface to the GUI scripting environment
provided by Tcl/Tk [_HUSH].
Tcl/Tk offers an extensible script language
with powerful graphics and window programming
utilities.
A number of extensions to <i>hush</i> have been written,
including the SIM and BPSIM libraries
as well as libraries encapsulating multimedia
facilities.
The <i>hush</i> Web component provides a
collection of (inline) data viewers for the <i>hush</i>
Web browser.
It supports full SGML and style sheets
to specify dynamic layout properties in a declarative way [_Animate].
Also <i>applets</i> are supported,
which are Tcl/Tk scripts augmented with the functionality
of <i>hush</i> applications.
Applets are the building blocks of active Web documents.
<p>
By defining a script interface for application-specific
C++ classes, application functionality may be embedded
in a Web page as an applet.
As illustrated in the example,
our approach to providing hypermedia and Web support
employs this feature,
both to reduce the time involved in modelling and
to present a graphical representation of the model
and its execution to the user.
</ignore>

%%<ignore>
:@ Discussion
As observed in [_Wastell94],
organizational change is a `highly threatening and stressful experience for many participants and ... high levels of stress can have a pernicious effect on individuals, group processes and organizational learning'.
Hence,
directly involving the users in the modeling phase of the BPR project may be an 
important step towards capturing the human aspects that are necessary for the production of an optimal model. It may also help to lessen the anxiety of employees.  
<p>
On a technological level, we advocate the use of
business process simulation.
Nevertheless, to accomodate the social aspects,
it is important to support the visualization of such models
and their integration in an arbitrarily complex
information context, such as the World Wide Web.
Developing visualisation and animation support
for simulation models is a topic
of ongoing research. See section <ref file=visualize.html>DIVA</ref>.
%%</ignore>
</document>
