<document>
@+ Logistics-based modeling

<include file=navigate>

Improvements in business performance 
and productivity may be achieved by critically
examining some of the rules 
that govern a business process.
Business process redesign (BPR) is the 
generic label for many emerging methodologies aimed at producing these 
improvements. Re-design 
implies that the current state of affairs is no longer acceptable and 
can no longer be refined or evolved.

<p>
Despite the importance of qualitative assessments
in BPR, as noted in [_Hammer90], for modeling
we favor a more quantitative approach
for which we provide support by means of
a simulation library (BPSIM) based
on the logistics-based business modeling method (LBM)
presented in [_Gerrits95].
<p>
<slide cap="Logistics-based business models">
:@ Logistics-based modeling -- criteria for redesign  
*- the time spent in executing a business process.
:@ Product lead time -- time between order and delivery
<ul>
<li> <i>processing time</i> -- actual working time
<li> <i>queue time</i> -- waiting for a resource
<li> <i>setup time</i> -- for the job to get started
<li> <i>wait time</i> -- waiting for a job to complete
<li> <i>transport time</i> -- between resources or operations
</ul>
</slide>
<p>
Logistics-based modeling 
allows for analysing
the time spent in executing a business process.
The <i>product lead time</i> is defined as the time
that passes between the moment a customer orders
a product and the moment a product is delivered.
In more detail, we can distinguish between
<i>processing time</i>
(the time actually worked on a job or operation),
<i>queue time</i>
(the time a jobs waits for a resource to become available),
<i>setup time</i>
(the time that passes between the moment
a resource becomes available and the moment
work on the job is started),
<i>wait time</i>
(the time that is spent waiting for another job
to complete), and
<i>transport time</i>
(the time that is needed to move a job from
a resource at a certain location to a resource
at another location).
<p>
For a particular model,
measurements may be obtained by
running a series of simulations.
Based on an analysis of the simulation results,
alternative models may be proposed.
For example, when the setup time for a job is
relatively large, combining jobs into a single task
for an employee may be more efficient.

%%@ Modeling Business Processes
The LBM method offers a number of primitives,
with associated graphic icons,
from which a business process model may be constructed
as a network of resources 
connected by transport arcs.

<slide id=lbm-prim cap="Logistics-based modeling primitives">
:@ Logistics-based modeling primitives
<ul>
<li> <i>operation</i> -- 
processing component
<li> <i>task</i> -- a series of consecutive operations
<li> <i>transport</i> -- transport of information
<li> <i>choice</i> --  affects flow of information
<li><i>organizational units</i> --
to represent departmental boundaries 
<li> <i>external agents</i> --
opaque
%%information producing or consuming
entities
<li><i>archives</i> --
paper-based storage facilities
</ul>
</slide>

The primitive entities offered by LBM are listed in \sliref{lbm-prim}.
Operations are atomic in the sense that wait time,
queue time and transport time may not be part 
of an operation.
Only setup time and process time are part of an operation.
Tasks are introduced to allow for a series of jobs 
or operations
to be processed, for example by one employee,
in order to reduce the setup time needed.
Transport entities represent the time it takes
for information to flow from one resource 
(that is operation or task) to another.
Transport implicitly defines the 
sequential structure of a process.
However, duplications of information, and consequently
parallel operations, are allowed.
In addition to the primitives mentioned above,
LBM allows us to characterize
<i>organizational units</i> 
to represent departmental boundaries, 
<i>external agents</i> 
to represent opaque information producing or
consuming entities,
and
<i>archives</i> 
to represent paper-based storage facilities.
Also, employees may need additional
<i>means</i>
to engage in an operation or task.
</document>
