<document>
@+ <a href=@talk-model-sim.html target=_top>Business process simulation</a>

<include file=navigate>

%%@ The library BPSIM
The classes provided by the business process simulation
library <a href=@online/tutorials/hush/bpsim/index.html>BPSIM</a> reflect the entities provided by the
logistics-based business modeling method LBM.
The BPSIM library is an extension of the simulation library
<a href=@online/tutorials/hush/sim/sim.html>SIM</a>, described in [_SIM].
SIM is a C++ library, which is part of the <em>hush</em>
framework, offering classes supporting
discrete event simulation, based on standard simulation techniques,
developed in 
[_Watkins93]. It will be described in the next section.
<p>
On a somewhat more abstract level,
one may regard 
a business process simulation as  consisting of <i>data</i>,
flowing through the process, and  handlers -- 
<i>datahandlers</i> -- performing some action on the data,
such as transportation or specific operations.
<p>
Accordingly, BPSIM provides two  base classes underlying
the classes corresponding to LBM entities:
<slide mode=hide id=bp-sim-data>
<ul>
<li> <i>data</i> -- which represents the product 
or case, i.e.
the information, that flows through the process. 
<li> <i>datahandler</i>  -- which is the basis 
for all classes that
handle information.
</ul>
</slide>
As an aside, LBM contains no symbol for data as it only
depicts the objects that handle the information.

<slide id=bpsim-classes cap="Classes derived from <em>datahandler</em>">
:@ Derived from <i>datahandler</i> class:
<ul>
<li> <em>operation</em> -- executed by employee
<li> <em>transport</em> -- connects datahandlers
<li> <em>waitqueue</em> -- wait for processing
<li> <em>choice</em> -- to model alternatives
<li> <em>archive</em> -- for storage of resources
<li> <em>external agent</em> -- blackbox entity
</ul>
</slide>

The classes depicted in \sliref{bpsim-classes} are derived from 
the <em>datahandler</em> class.
They  realize the corresponding entities in LBM.
<p>
An
<i>operation</i> takes time, and is executed by an employee. Sometimes 
an operation results in more than one outgoing dataflow,
for instance when it 
issues a request for additional information from a different 
department.
<p>
A <em>waitqueue</em> functions as a regular queue if 
it has one incoming dataflow, i.e. transport.
When there are more incoming
flows, it functions as a synchronized queue.
Data from one flow is not passed on to the
next datahandler until the data from the other flow has arrived.
This happens for instance when work on a case cannot continue
until additional information has arrived.
<p>
An external <i>agent</i> can either generate data
according to some random number distribution, or take
data, process it and pass it on to the next datahandler. In the latter 
case the agent functions as a black box: we only
care how long the processing takes, not how it is exactly performed.


<p>
In addition, BPSIM offers
the classes
<i>means</i> (which can be used to model resources 
that are necessary to perform certain operations), and
<em>employee</em> (which models the 
different people that perform the operations).

<p>
As one can see, the entities <em>task</em> and 
<em>organization unit</em> 
from LBM have no specific counterpart in BPSIM.
The reason for this is that having no other <i>datahandler</i>
between two operations already implies that those
operations belong to the same task. Consequently they are
executed with no time in between and by the same employee. 
Also, the fact 
that tasks are executed in
different <em>organization units</em> does not add any 
information that changes
the behaviour of the simulation. If it takes time to transport
information between different units, 
then that time can be represented by 
the object of class <em>transport</em> between those units. 
<p>
For the gathering and analyzing of results, the SIM classes 
<em>histogram</em> and
<em>analysis</em> are used.
The class <em>agent</em>, for example, 
can be given a histogram to track the lead time
of data it has generated. 

:@ A script interface for BPSIM
Employing the facilities of the <i>hush</i> library
a script interface has been defined for BPSIM
that allows the user, that is the designer of business models,
to construct and run business simulation models
with a short turnaround time.
%%<p>
Each class in the BPSIM library corresponds to
a command in the script language.
As illustrated in the next section,
script commands allow for a graphic
representation of the model, which may be displayed
in a Web page as an applet.
</document>
