<document>
@* <a href=@talk-simulate.html target=_top>Object-oriented simulation</a>

<include file=navigate>

Historically, there is a close connection between 
simulation and object-orientation.
The first object-oriented language, SIMULA,
was a programming language meant for discrete event simulation,
see [_DaNy66].
<p>
In [_Hill96], an overview is given of what is required
for complex system modeling,
and more in particular how object-oriented analysis
and design may aid in defining models
that lend themselves to performance evaluation
by means of simulation studies.
Performance evaluation belongs traditionally
to the field of statistics and operations research.
However, according to [_Hill96], for a clear understanding
of the behavior of complex systems and the interpretation
of measurement results, we need to derive our
action models, which are used to perform the simulations,
from domain and system object models,
describing the structure of the system
and its relation to reality.
In other words, an object-oriented approach may help
us in arriving at better models,
provided that we have the stochastic support needed
to perform reliable performance studies.
<p>
In this section, we will briefly discuss the 
SIM library, as an example of object-oriented support
for discrete event simulation.
See [_Hill96] for a great many alternatives.


:@+ The SIM library

SIM is a C++ library offering classes supporting
discrete event simulation, based on standard simulation techniques
described in [_Watkins93].
In discrete event simulation,
the components of the model consists of events,
which are
activated at certain points in time and in this way affect the overall state
of the system.
The simulation library consists of the classes as listed in \sliref{sim-classes}.

<slide id=sim-classes cap="Simulation classes">
:@ <a href=@online/tutorials/hush/sim/index.html>Simulation classes</a>
<ul>
<li><i>simulation</i> -- the scheduler
<li><i>event</i> -- representing the events
<li><i>entity</i> -- process consisting of events
<li><i>generator</i> -- random distributions 
<li><i>resource</i> -- to model passive objects
<li><i>queue</i> -- to hold waiting events 
<li><i>histogram</i> -- to plot the results 
<li><i>analysis</i> -- for statistical analyses
</ul>
</slide>
The SIM library is integrated with the <i>hush</i> library,
which may be used for defining a script interface to the simulation
package, for developing a graphical user interface
and for visualizing simulation models.

:@ The event scheduling strategy
In its most simple form, a simulation runs 
until there are no events left.
Events are user-defined objects that represent
the functionality of the system to be modeled.
At the time an event is due to be activated, 
it is extracted from the scheduler and the main simulation routine
executes the code from the {P operator() P} method of that event. 
<p>
<slide id=sim-event type=fig cap="State diagram for event">
<image name=@figures/sim/event>
</slide>
<p>
The scheduling algorithm extracts all events with the same activation time. It
activates the events in priority order with the highest scheduling priority first. Before executing the events, the
simulation clock is updated to the activation time of the current events. Furthermore, a FIFO conditional list (containing events that
occur when some condition is met)
is traversed in priority order with highest scheduling priority first. 
Events that can run now are executed.
Events that are not used anymore should be terminated to prevent the system from overflow. 
<p>
<slide id=sim-states cap="Event states">
:@ Event states
<ul>
<li> <em>passive</em> - currently not available for any processing 
<li> <em>active</em> - this is the event currently being processed 
<li> <em>queued</em> - the event is in a queue 
<li> <em>pending</em> - the event is in the scheduler 
<li> <em>conditional</em> - the event is on the conditional list 
<li> <em>closed</em> - on the conditional list but unavailable 
</ul>
</slide>
<p>
In \sliref{sim-event} a state-diagram is given, 
which depicts the states (see \sliref{sim-states}) an event object can be in.
The state of an event can be affected both by the scheduler
as well as from within the user-defined methods of the event.
The labels on the arrows indicate the methods that
result in a state transition. (As a convention, the methods 
above the arrows indicate a transition from left to right,
the methods below the arrow a transition from right to left,
in case of a bi-directional relation between states.)

%%<so sim-event>
<so simulate-dining>
</document>
