<document>
@* <a href=@talk-design.html target=_top>Library design</a>
\label{library-design}

<include file=navigate>

@$=P

One of the great promises of object orientation
is what has been called the {\em industrial reuse of software} 
in [_Meyer90].
The idea of objects as reusable components
was originally put forward in [_Cox86],
who coined the phrase {\em software IC}
to indicate the analogy with building electronic circuits
from {\em off-the-shelf components}.

Software reuse is certainly not an exclusive interest
of object-oriented developers,
as testified for instance in [_Biggerstaff89].
However, object orientation holds strong cards,
as
the mechanisms of encapsulation and
inheritance directly support the reuse of software.
Encapsulation may be used to define
{\em wrapper classes} for existing software,
and inheritance may be used to refine
existing classes to meet specific requirements.
Essentially, a well-designed class library
may act as a repository of
reusable components.
The {\em unit of reuse} is, however,
generally not a single class but
rather a collection of classes providing
a framework for application development.
In this section, we will look
at the design decisions underlying the 
Eiffel libraries, and 
discuss the development of template classes
for abstract data structures in C++.

@+ Taxonomic classification <em> -- the Eiffel libraries</em>



A good example of a well-designed and well-documented
collection of class libraries is given in [_Meyer90],
who describes the design and implementation
of the standard Eiffel libraries.
(See also Meyer, 1994.)

\vfig{0.1}
<slide id=11-eiffel cap="The Eiffel libraries">
@ Library design
{\em -- industrial reuse \c{of software}}
*- {\em unit of reuse} -- class/cluster
:@ The Eiffel libraries
{\em -- contracts}
<I
- {\em Kernel} -- basic system needs\c{, array, strings, io ...}
- {\em Support} -- browsing, \c{persistent} storage\c{, debugging} 
- {\em Data Structures} -- lists, trees\c{, stacks},...
- {\em Lexical/Parsing} -- scanners \c{and parsers}
- {\em Graphics} -- windows, \c{mouse handling,} figures
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The Eiffel libraries include {\em kernel} classes
(for arrays, strings and io),
{\em support} classes (for browsing, persistent storage
and debugging),
{\em data structures} (such as lists, trees and stacks),
classes for {\em lexical analysis} and {\em parsing},
and {\em graphics} classes (for windows, mouse handling
and figures).
\see{11-eiffel}

\vstr{0.5}
<slide id=11-obsolete cap="The problem of consistency">
:@ Consistency <em> -- support tools</em>
The great Tempter of Perfection exhorts: ``Correct it here and now,
before it is to late.''
But, here the Guardian Angel of the Installed Base is really a frontman
for the hideous Devil of Eternal Compatibility with the Horrors of the Past,
whose nefarious influence is also visible in Computer Science.
</slide>


The Eiffel libraries may serve as an example
of rational and consistent library design.
Each class interface is described by a {\em contract}
stating the class invariant and the
pre- and post-conditions for the services provided.
An interesting feature of the Eiffel libraries
is that the class interface documentation has
been extracted automatically from the class
implementations, thus guaranteeing consistency between
the actual library classes and their description.

Like any software system, libraries tend to evolve
over time.
As the quotation taken from [_Meyer90] in \sliref{11-obsolete}
indicates,
the developer is faced with the dilemma
of keeping the library clean and consistent
or satisfying the user of earlier versions
by maintaining the original class interfaces.
As an intermediate solution, the Eiffel
language provides the keyword {\em obsolete}
which may be used to indicate that particular
methods may eventually be removed from the 
class interface.
The user acquiring a new version of the library
is then urged to adapt the code without
being confronted with a non-functioning system.


\label{eiffel-tax}

Good library design is aimed at the development
of a stable collection of classes with stable interfaces.
Stable interfaces mean that the user of the library
need not worry about any changes in the implementation
of the library, since as long as class interfaces
remain the same user code is not affected.


\vstr{1}
<slide id=11-naming cap="Naming conventions">
:@ Naming
*- favor consistency \c{over specificity}
:@ Conventions 
<I
- {\em verbs} -- procedures
- {\em nouns} -- attributes, functions
- {\em adjectives \c{, question-verbs}} -- boolean queries
I>
\ifsli{\vspace{-1cm}}{}
</slide>

An important aspect of developing library classes
is to arrive at a consistent and understandable
(read predictable) naming scheme for the methods
listed in the class interface.
The advice given in [_Meyer90] is to
use {\em verbs} for (state modifying) procedures,
{\em nouns} for (non-modifying) functions and attributes,
and {\em adjectives} or {\em question verbs} for
boolean queries.
Note that this advice is not without ambiguity,
since words such as {\em empty} and {\em count}
may be used both as nouns and verbs.
In addition, rather than adhering to specific
method names that are conventionally used
(such as $push$ and $pop$ for stacks and $insert$ and $extract$
for queues), [_Meyer90] advises resorting to
standard feature or method names that apply
to all classes of a particular category.
\see{11-naming}

\vstr{2}
<slide id=11-features cap="Feature names for containers">
@ Standard feature names
\zline{\fbox{containers}}
\nl
\indent {\em item} -- access operation \n
\indent {\em count} -- number of items \n
\indent {\em has} -- membership test\n
\indent {\em put} -- insert or replace item\n
\indent {\em force} -- like put\n
\indent {\em remove} -- remove an item\n
\indent {\em wipe\_out} -- remove all items\n
\indent {\em empty} -- absence of items\n
\indent {\em full} -- no more space
</slide>

As an example, consider the development of a collection
of container classes.
Our aim, obviously, is to arrive at an inheritance
hierarchy representing a classification of container types
that allows us to choose the kind of container
that most closely fits our needs with respect
to {\em access}, {\em traversal} and {\em storage requirements}.

The advantage of using standard feature names for the complete
collection
instead of the specific names conventionally used
for the different kinds of containers
is that essentially no code has to be changed
when replacing one kind of container with another.
The standard feature names for container classes
in the Eiffel library are listed in \sliref{11-features},
with a short explanation of their meaning.

%%@ Classification
Underlying the implementation of container classes
in the Eiffel libraries is a taxonomy of data structures,
and a taxonomy of
access and traversal methods, that may be combined
in a variety of ways using multiple inheritance.
From a user's point of view, access methods
are the most important attributes of containers,
as they determine the order of insertion
and retrieval.

<latex>
\newsavebox{\xxcontainers}
\sbox{\xxcontainers}{
\it\footnotesize
\curl{\c{container}}{
   \curl{collection}{set}{\curl{bag}{
				\curl{dispenser}{stack}{queue}
				}{
				\curl{cursor}{list}{tree}
				}
			 }
		}{
   \curl{table}{indexable}{hash}
   }
   \n
}
</latex>

<slide id=11-access cap="Access taxonomy">
:@ Access <em> -- keys, order of insertion and retrieval</em>
<latex>
%%\nl
\usebox{\xxcontainers}%
</latex>
</slide>

The hierarchy associated with access methods
most closely corresponds with the traditional
classification of containers according to
their (mathematical) type.
\see{11-access}
For example, a $stack$ may be regarded as a refinement
of a $bag$, imposing additional order.
See section \ref{refinement}.
Somewhat peculiar in the hierarchy shown above
is that the container type $set$ is not a refinement
of $bag$, which would mathematically be more appropriate.


In addition to the hierarchy corresponding to
access methods, we also have a hierarchy corresponding
to traversal methods
(which are either hierarchical or linear)
and a hierarchy representing the options for
storage requirements.

As an example of the possible interplay between
access methods and traversal methods,
look at the $tree$ container in the access hierarchy.
Similar to a $list$, a $tree$ may be accessed by means 
of a $cursor$,
which is a pointer to a particular element in the structure.
Dependent on the traversal method, the $cursor$
points to the respective elements of the tree
either in a linear (that is sorted) order or
hierarchically (corresponding with the levels of the nodes).

@$=$

\vstr{0.5}
<slide id=11-storage cap="Storage taxonomy">
:@ Storage <em> -- requirements</em>
<latex>
%%\nl
\it\curl{$\bullet$}{infinite (lazy)}{\curl{finite}{unbounded}{\curl{bounded}{resizable}{fixed}}}
%%\n
</latex>
</slide>



With respect to the storage requirements,
we have the choice between finite structures
or potentially infinite structures
(that must be implemented in a $lazy$, that is demand-driven,
fashion).
Bounded structures such as arrays may again be 
subdivided into fixed or resizable structures.
\see{11-storage}

The hierarchies corresponding to access and traversal
methods and storage requirements may be regarded
as the dimensions along which a variety
of container types may be constructed.
The properties chosen with respect to these dimensions
may be used as type annotations serving as
an index for retrieval.


:@ Indexing and retrieval
To support the retrieval of classes from
a repository (a database of software components),
the Eiffel language provides the keyword $index$
by which attributes associated with a class may
be defined.
For instance, a container class may be given the
attributes $fifo$ (first-in first-out)
or $lifo$ (last-in first-out)
that characterize respectively the access behavior of
a queue and a stack.
\see{11-indexing}
The Eiffel system itself, however, does not provide
for a software repository facility.


<slide id=11-indexing cap="Indexing and retrieval">
@ Indexing
{\em -- type annotations}
<I
- Access -- {\em fifo, lifo, key}
- Storage  -- {\em bounded, unbounded}
I>
@ Archival and querying tools 
\zline{\fbox{Eiffel}}
<I
- {\bf flat} -- flattens \c{the} inheritance structure
- {\bf short} -- produces interface
- {\bf good} -- browser for class universe
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The retrieval of classes (and software components
in general) is one of the unresolved problems
in software reuse.
Clearly, a $browser$ allowing traversal of the class
universe does not suffice.
As an anecdote, Alan Kay (the spiritual father of Smalltalk)
once remarked that after an absence of a few months
in which the class structure of the Smalltalk
library had been changed he had considerable difficulty
in locating the appropriate classes.
The anecdote indicates that browsers are only helpful
to refresh our memory, but not to familiarize
ourselves with a class library or to select
previously unknown classes for reuse.

The class browser $good$ that comes with the Eiffel
system is a graphical class browser
which sets (apart from its speed) an example for
a general purpose class browser,
as it allows a graphical display of both inheritance
relations and client-server relations.
In addition to a class browser, the Eiffel system
provides a tool $flat$ that allows one to flatten
a class to include all features inherited
from its ancestors.
Moreover, it provides a tool $short$
that allows one to extract an interface description
from a (possibly flattened)
class implementation.
A tool such as $short$ is an invaluable means
for producing automatically the documentation
specifying the interface of a class.
Invaluable, since reuse ultimately depends
upon understanding the functionality offered 
by a library.


@+ Template classes <em> -- bags and sets</em>

@$=P

Numerous C++ libraries offering
data structures for containers are available.
Yet, many programmers choose to develop
such a library themselves.
The reasons for this may vary from
matters of taste to considerations
with respect to the formal properties
of the types offered.


<slide id=cc-bag cap="The implementation of a bag">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T >
class bag { /// <box>bag< T ></box>
public:
bag() { s = new list<T>; }
bag(const bag< T >& b) { s = b.s; }
~bag() { delete s; }
bag<T>& operator=(const bag< T >& b) { 
	s = b.s; return *this;
	}
virtual void insert(const T& e) { s->insert(e); }
operator iter< T >() const { return *s; }
int count(const T& e) const;
void map(T f(const T& e));
protected:
list<T>* s; ///see section \ref{gen-list}
};
</program>

\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In the following, we will look at some issues
in defining generic classes for the
mathematical data types $bag$, $set$ and $powerset$.
Such data types must, for example, be provided
in a library meant to support the use of
formal methods as outlined in section
\ref{3-design}.
Naturally, in developing real industrial-strength
libraries many more issues play a role.
See, for example, [_BV90].

In \sliref{cc-bag},
a generic definition for the type $bag$ is given.
Mathematically, a $bag$ or {\em multi-set} is
a $set$ which may contain multiple
instances of a particular value.
(With respect to the subtype refinement relation,
however, a $set$ may be considered as
a subtype of $bag$.)

The template class $bag< T >$ offers a default
constructor, a copy constructor and a member
function for assignment, as required
by the canonical class idiom
presented in section \ref{canonical}.
An instance of the (generic) $list< T >$ class, developed
in section \ref{generic},
is used to store the elements of the $bag$.
The destructor of $bag$ simply deletes
the the protected data member $s$.

To insert an element, $bag< T >::insert$ forwards
the call to $list< T >::insert$
and, similarly, when an iterator is requested the list
pointer is converted appropriately.


<slide id=cc-bag-op  cap="Bag operations">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T >  /// <box>bag< T >::count>/box>
int bag< T >::count(const T& e) const {
iter< T > it = *this;
T* p = 0; int cnt = 0;
while ( p = it() )
	if ( (T&) e == *p ) cnt++; /// $ (*) $
return cnt;
}

template< class T >  /// <box>bag< T >::map</box>
void bag< T >::map(T f(const T& e)) {
iter< T > it = *this;
T* p = 0; 
while ( p = it() ) *p = f(*p);
}
</program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The function $bag<T>::count$ and
$bag<T>::map$ are defined in \sliref{cc-bag-op}.
The $count$ function tells how many
instances of a particular element are in
the $bag$.
It employs an iterator to traverse the list
and compares its contents with the element
given as an argument.

The function $map$ may be used to modify
the contents of the $bag$ applying some
mapping function to each element:

<nslide>
\ifsli{\mbox{}\vspace{-1cm}}{}
[P
int S(const int& x) { return x+1; }

bag<int> b;
b.insert(1); b.insert(2); b.insert(1);
b.map(S);
iter<int> it = b; ///// get the iterator
int* p = 0; ///// start
while ( p = it() ) {
	cout << "item;" << *p << endl;  ///// take the value
	}
P]
</nslide>

\noid
In the example above,
the function $S$ is defined as the successor
function for integers.
Also, a bag $b$ of integers is declared,
into which the (integer) elements 1,2 and 1 are inserted.
As a consequence $b.count(1)$ will deliver $2$
and $b.count(2)$ will deliver $1$ as a result.

The $bag< T >::map$ function is used to apply 
the successor function $S$ to each element
of the $bag$.
Then, an iterator is obtained, simply by
assignment with an implicit conversion,
and the contents of the $bag$ are written
to standard output.


<slide id=cc-set cap="Deriving a set class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T >
class set : public bag< T > { /// <box>set< T ></box>
public:
void insert(const T& e) {
	if (!member(e)) bag<T>::insert(e);
	}
bool member(const T& e) {
	return count(e) == 1;
	}
};
</program>

\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Evidently, the class $bag< T >$ lacks many of
the features required for the mathematical
notion of a $bag$,
such as operators for bag union and bag intersection.
Nevertheless, $bag< T >$ may conveniently be used to
define the class $set< T >$ as a derived class.

The $set< T >$ class given in \sliref{cc-set}
defines an additional function $member$ and redefines $insert$
to check that the element is not already a member
of the set
(since, in contrast to a bag,
a set may not contain multiple instances of a value).
The function $member$ delivers $true$ when
the number of occurrences of an element
is precisely one, otherwise
it returns $false$.


<slide id=cc-set-eq cap="Equality for sets">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T > /// <box> set< T > == set< T ></box>
int operator==(const set< T >& s, const set< T >& b) {
iter<T> it = s;
T* p = 0; int eq = 1;
while ( eq && (p = it()) ) 
	if ( s.count(*p) != b.count(*p) ) eq = false;
return eq;
}
</program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Equality between sets may be defined as
in \sliref{cc-set-eq}.
Set equality amounts to element-wise
correspondence, irrespective of the
order in which the elements occur.
The definition given in \sliref{cc-set-eq}
is somewhat more general than necessary,
in that it also applies to bags.
The requirement $! member(*p)$
would have been sufficient.

The definition of equality for sets is necessary
in order to be able to
define instances of set having a
set-valued instantiation parameter,
such as the class $power<T>$
defined in \sliref{cc-power}, which is derived
from $set< set< T > >$.
When instantiating $set< set< T > >$ for some
type $T$, equality is required for $set< T >$
by the line $ (*) $ in the  function
$bag<T>::count$ defined in \sliref{cc-bag-op}.

<slide id=cc-set-op cap="Writing a set to a stream">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T > /// <box> operator<< </box>
ostream& operator<<(ostream& os, const set< T >& s) {
iter< T > it = s;
T* p = 0;
while ( (p = it()) ) {
	cout << *p << endl;
	}
return os;
}
</program>

\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In \sliref{cc-set-op},
a generic operator is defined to write an
arbitrary set to a stream,
by overloading {P operator<< P} for
$const set< T >&$.
To traverse the elements of the set it employs
an iterator, obtained by
assigning the set reference to an
$iter<T>$ instance.

<slide id=cc-power cap="The powerset class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program>
template< class T >
class power : public set< set< T > > {/// <box>power< T ></box>

};
</program>

\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

To define the class $power< T >$
it suffices to derive the class
from $set< set< T > > $.
However, according to the rules
given for the canonical class idiom,
both $power< T >$
and $set< T >$
should be augmented with
a default and copy constructor,
and an assignment operator and destructor as well.

Note that the $map$ function as defined for
$bag< T >$ is potentially unsafe for both $set< T >$
and $power< T >$ instances.
For example, the function given to $map$ may
result in identical values for different
arguments.
As a consequence, the restriction that
the set does not contain multiple instances
of the same value may be violated.


\cleartrue
<nslide>
[P
set<int> s1;
s1.insert(1); s1.insert(2); 
set<int> s2;
s2.insert(2); s2.insert(3);
cout << s1;
power<int> b;
b.insert(s1); b.insert(s2);
cout << (set< set< int > >&) b; /// cast is necessary
P]
</nslide>

\noid
An example of employing the powerset
is given above.
First, two instances of $set< int >$ are created,
which are then inserted in an instance of
$power<int>$.
The {P operator<< P} function (defined
for $set< T >$ in \sliref{cc-set-op})
may be used to write the powerset
to standard output.
For each (set-valued) element of
the powerset,
the {\tt operator<<} function
instantiated for $set< int >$
is called to write the individual
elements to standard output.

</document>
