@+ DIVA - distributed visualisation architecture 

The DIVA architecture is intended as a framework and should be flexible enough to incorporate new components. The
components used in DIVA are generic software components which interact with each other. The architecture is distributed,
meaning that its components can reside on different hosts on the network. Related collaborative architectures can be found
in Bentley et al. (1994) and Reinhard et al. (1994). 

:@- Software Components

<slide cap="Architecture">
%%<image name=@figures/vis/architecture>
</slide>




Figure 2 shows the main components of the architecture. In the following list, the three components that were already
present in an earlier version of the DIVA architecture (Sch&#246;nhage and Eli&#235;ns, 1997; Sch&#246;nhage and
Eli&#235;ns, 1998), are listed first. The last three components in the list extend DIVA. 

<ul>
<li> generator -- embodies the primary model 
<li> shared concept space -- information store that contains the derived model 
<li> presentation component -- the actual information visualization 
<li> DIVA services directory -- central registering of components 
<li> collaborative session manager -- overall coordination of virtual meetings 
<li> local collaboration component -- local collaboration support 
</ul>


In a normal situation, one DIVA services directory and more than one of each of the other components could exist. A
presentation component and a local collaboration component are present at the desktop of each participant. A short
description is given for each of the components. 

The generator embodies the primary model and generates all data needed for the visualization. Examples of generators are
simulations of business processes. The generated raw data is transferred to the shared concept space. 

The shared concept space stores information in an expressive and adaptive way. The information is contained in the form of
concepts that are stored in a hierarchical manner. Each concept has one or more data properties that represent the
information. The data properties are updated with data coming from the generator. The received data can be stored directly
or can be used to compute and store derived information. 

The presentation component actually visualizes concepts from the shared concept space. It makes use of gadgets, which
are generic visualization primitives that present certain types of information. As an example, cone trees are primitives
(gadgets) to visualize hierarchical information (Robertson et al., 1993). 

Examples of gadgets are a rotating object which indicates a certain action or a histogram which displays data. 

The DIVA services directory (DSD) is the central directory component. DIVA components (or services) can register here,
identifying themselves and giving their location. Once they are registered, the DSD can inform other objects about the
availability and whereabouts of these services. 

The collaboration session manager (CSM) coordinates components. It deals with interaction protocols, which means it
knows about the participants and their roles, sharing perspectives, user to user communication and consistency. 

The local collaboration component is directly connected to the session manager. It is present at each participants desktop
and handles interactions and information related to a collaborative session. It may for example display a list of participants
and offer communication facilities. 

@ User Environment 

<slide cap="User Environment">
%%<image name=@figures/vis/userenv>
</slide>

Figure 3 shows a typical user environment. Outside of the user environment, two components are shown. A generator,
which is a simulation in this figure, feeds data into the shared concept space. This is depicted by the fat arrow. These two
components can be situated anywhere on the network. Most of the components in a user environment use the display. The
local collaboration component displays information about the collaborative session. The presentation component displays
the visualization and the controller displays its user interface. 


@ Display agents 

From the shared concept space, information is being sent to display agents. These agents are present at the user
environment and each of them maintains one gadget. The information they receive is transformed into a visualization. As an
example, consider the visualization in Figure 4 on page . The line of three puppets in front of the desk is a visualization
gadget depicting a queue. When the display agent senses that the length of the queue increases by one, it accordingly places
a fourth puppet on the screen. 

The display agents also reside in the perspective repository. When a user requests a certain view, the agents that represent
that view are cloned and moved to the user environment to build the perspective in the VRML world. Enforcing a
perspective onto another user is accomplished by cloning and moving the display agents from one user to another. 

Now why can we call these entities agents? There are a lot of definitions of agents (Franklin and Graesser, 1996), and the
question can be raised why a display agent is an agent. In other words, how does it differ from a standard program or
software component? For one, the agents are autonomous, which means they execute on their own. Second, they react on
certain input and subsequently act on their environment, namely, they sense information from the shared concept space and
act on the VRML world. They have some goals they need to accomplish. Third, the display agents can communicate with
each other, for instance, to discuss how to place the gadgets each of them represents on the screen (this is a future feature).
Fourth, they have a domain which they have knowledge of. This domain is the visualization of information. Fifth, they act on
behalf of a user. Users can give their preferences to a display agent, and the agent will take care of it. All in all, the display
agent fits quite a number of definitions of autonomous agents given by Franklin and Graesser (1996). 

@ Controllers 

Every DIVA component can have a separate mobile controller. It can be moved from one environment to another, so it can
be shared by several participants. The ability to use a controller depends on the role of the user. Participants can request a
controller or the chair could appoint it to one of them. 

Controllers can have several functions. For example, a simulation can be controlled by starting and stopping the simulation
and changing certain parameters. A controller for a shared concept space can be used to create new computed concepts
or to decide which data from the generator is selected. 

@ CORBA and the Web 

DIVA is designed as a distributed object oriented system. The DIVA components are written in C++ and Java, and can run
on different platforms. We use the Common Object Request Broker Architecture (CORBA) to let our distributed objects
communicate with each other. CORBA (Orfali and Harkey, 1997; Siegel, 1996) abstracts from hardware, operating systems
and programming languages. By using the interface definition language (IDL) to describe the interfaces between
components and by making use of the object request broker (ORB), distributed components are able to communicate. 

Voyager (ObjectSpace, 1997) is an agent ORB written purely in Java, which supports CORBA. Voyager allows us to use
mobile objects, a feature which CORBA does not have. We use Voyager to construct the mobile controller components.
These components are able to "dock" at a user environment and can subsequently show their user interface on the screen
to let the user interact with it. 

We use VRML (ISO, 1997) as the main visualization tool. The users are able to navigate through the VRML worlds by using
a VRML-browser. The External Authoring Interface (EAI) makes it possible to control the VRML worlds dynamically via
the Java and Javascript languages. 

The visualization gadgets in the presentation component are represented by mobile display agents. These agents are
constructed using Voyager. Display agents can also "dock" in a user environment and, in addition, get access to the local
VRML world. They collect the needed information from shared concept spaces to build and maintain the 3D visualization. 

The combination of CORBA and the Web enables access to information resources by means of HTML, Java and VRML
(see also Rohrer and Swing, 1997). For example, the simulation and shared concept space can be hosted on a Unix server
while the presentation components are executed in a Web-browser on Windows client machines. 

