<document>
@* Software architectures revisited
\label{Architecture}
<include file=navigate>

The Web is, at the time of writing, still in flux.
Yet it is becoming more and more the standard infrastructure
on which applications are built.
A recurring question is, <em>`how do we build Web applications?'</em>.
There is no definite answer to this question.
There is no body of solutions that may serve to indicate
proven practice.
But there is, definitely, a convergence towards
<em>objectifying</em>, or <em>object-orienting</em> as it
is called in [_OOWeb], the Web and its applications.
Anyway, the following quote, taken from [_Practice], p. 10,  says it all.
<slide id=oo-arch mode=hide>
<c><p></c>
<quote><em>
It is a brave architect who, in today's environment,
does not develop, or at least consider,
an object-oriented design.</em>
%%<c><p align=right>[_Practice] p. 10</c>
</quote>
<c><p></c>
</slide>
\noindent
Clearly, the architecture of the technological infrastructure
of the Web, as well as the architecture of Web applications,
may benefit from an object-oriented approach.
Nevertheless, knowing this, we still do not know how
to build actual Web applications.

<slide id=styles cap="Architectural software styles">
:@ Architectural software styles
///<box>class of architectures</box>
<ul>
<li> <em>component types</em> -- process, event, repository
<li> <em>runtime relations</em> -- topology
<li> <em>semantic constraints</em> -- immutability
<li> <em>communication and coordination</em> -- connectors
</ul>
</slide>

From the perspective of software architectures,
we may ask ourselves what architectural style,
or for that matter which mix of architectural styles,
we may deploy for building such applications.
As a reminder, an architectural style, which characterizes
a class of software architectures,
consists of a description of the
types of components used (processes, events, repositories),
the (runtime) relations between these components 
(for example the network topology),
possible semantic constraints (such as the immutability
of particular components)
and properties concerning communication
and cooperation (such as the connectors or protocols used).

<slide id=arch-themes cap="Themes and variations">
:@ Themes and variations
///<box>technological constraints</box>
<ul>
<li> OO -- simple call and return
<li> CORBA -- independent components
<li> WWW -- data centered
<li> events -- independent components
<li> logic -- virtual machine architecture
</ul>
</slide>

A rather simple-minded categorization of architectural
styles, reflecting obvious technological constraints,
is given in \sliref{arch-themes}.
Each of the styles is characterized by a single phrase
capturing a central feature of the style.
For example, an OO approach may be characterized by the fact that
it embodies a simple call and return mechanism, which, by the way,
gets its power from the fact that it concerns methods or,
in C++ jargon, virtual functions.
Events have proven to be an excellent means to obtain a high degree 
of independence between components.
And logic, as has been argumented in section <ref file=@online/oo/II/7/embed>Logic</ref>,
may be used to promote a clear separation between knowledge-level
and system-level aspects of a system, by embedding
a (virtual) logic machine.
<p>
The categorization is, however,
simple-minded because, as we may observe in retrospect,
most of the applications discussed contain elements
of at least a couple of the styles mentioned.
So, instead of discussing one style,
we need to consider a mix of styles,
and determine what mix of styles may be
effectively used to create the applications we have in mind.

:@- The architecture of the Web
To return to the Web,
why is the notion of software architecture important?
%%\noindent
As indicated in \sliref{web-architecture}, for one,
the Web is still growing at a rapid rate,
and it is becoming increasingly important economically.
So, we are faced with the problem of managing this growth,
and, much sooner than we could have expected,
with the problem of maintaining the applications that populate the Web.

<slide id=web-architecture cap="Architectural issues">
:@ Architectural issues
<ul>
<li> managing growth, maintaining installed base
<li> enhanced functionality -- <em>synchronized multimedia</em>
<li> improved technological infrastructure -- <em>HTTP-NG</em>
</ul>
</slide>
<p>
Secondly, the Web is continuously enhanced with new 
functionality, including, for example,
<em>synchronized multimedia</em> as proposed in the SMIL standard, see [_SMIL].
Consequently, with respect to the technological infrastructure,
we need to be able to accommodate new requirements,
such as <em>quality of service</em>, needed for the timely
delivery of multimedia material.
<p>
And thirdly,
many attempts are being undertaken to improve the quality
of the infrastructure itself,
as exemplified by the HTTP-NG effort,
which aims at higher speeds and a state-full
communication protocol, see [_HTTP-NG].
<p>
As clearly stated by the Web's principal architect,
Tim Berners-Lee,
graceful extensibility has always been one of the primary goals
in developing the architecture for the Web.
In this respect, the Web differs significantly from other
distributed technologies, such as CORBA,
which does not allow for non-compliant extensions.
In contrast, the Web does to a great degree allow
for non-compliant extensions simply by ignoring them,
until they become a standard.
The challenge, then, from an architectural point of view,
is to come up with better standards and better
technologies without sacrificing the extensibility
allowed by non-strict technologies such as HTML and HTTP.

:@- Plugin architectures
To conclude this chapter, I would like to discuss briefly
an extension mechanism that has proved to be invaluable
for developing Web applications, the <em>plugin architecture</em>.
Plugin architectures are becoming more and more
popular, for `ordinary' tools such as Adobe Photoshop
and Macromedia Director.
In a Web context, the most notable examples
are Netscape Navigator and Microsoft Internet Explorer,
which both provide a facility to extend the browser
with new functionality that is available
in a dynamically loadable library.




<slide id=Plugins mode=hide>
<center>
<table><!--{ | l | l | }-->
<tr><td><em>Client NPP/Callbacks</em></td><td><em>Browser NPN/Calls</em></td></tr>
<tr><td>Instantiation and Destruction</td><td>Version Info</td></tr>
<tr><td>Stream Notification</td><td>Stream Creation and Destruction</td></tr>
<tr><td>Reading and Writing Streams</td><td>StreamAsFile</td></tr>
<tr><td>LiveConnect</td><td></td></tr>
</table>
</center>
</slide>

<p>
Plugin architectures are realized by using callbacks,
in the same way as in object-oriented frameworks.
Above, an overview is given of
the callback functions required by the Netscape plugin architecture.
These functions must be implemented by the
(plugin) client, so that the browser can recognize
and activate the plugin.
Such callbacks encompass instantiation and destruction functions,
notification when a stream is ready,
functions for reading and writing streams, and the <em>Live Connect</em>
functions, which enable the (plugin) client to communicate
with Javascript functions and Java applets that are currently active.
The browser, in return,
provides convenience functions to obtain version information,
to create or destroy streams or to store the contents
of a stream in a temporary file.
<p>
It should be noted that the actual API for
the creation of (Netscape) plugins is not object-oriented,
although a partial class library is available to create
plugins in an object-oriented manner. <slide color=red mode=hide id=poosd=ok><c><p></c>
Nevertheless, ignoring details,
plugin architectures indicate what may become
the dominant paradigm of the future,
framework-like environments that are extensible by components
following a clearly defined pattern or protocol;
that is to say, components created according
to the <a href=@online/oo/index.html target=_top>principles of object-oriented software development</a>.
</slide>

</document>
