<document>
@+ <a href=@talk-web-agents.html target=_top>Intelligent agents</a>
%%-- technology or metaphor?

<include file=navigate>

In [_Negro], <em>intelligent agents</em> are characterized
as autonomous, intelligent processes
aiding the user in complex tasks, such as answering email,
gathering information and planning activities.
In practice, agents on the Internet may help
in monitoring changes in Web pages,
collecting information on topics of interest,
or searching based on personal preferences.
See [_Agents], [_Search].
Other types of agents, such as the <em>shopping agents</em>
described in [_Mobile] or even <em>virtual players</em>
as those described in [_VR], might become possible in the future.
However, despite the range of possible examples,
the notion of <em>intelligent agent</em> is
not very clear.
<p>
In [_Survey], two definitions of <em>agent</em> are given,
a soft definition, characterizing agents as autonomous processes
that show some intelligence,
and a hard definition attributing agents with mentalistic
properties such as belief, desire and intentions.
At this stage, the hard definition is clearly no more than a metaphor,
since there is no technology that actually supports it.
Taking the soft definition, one could argue that it is (partly)
realized by modern object technology, as embodied in Java and CORBA,
omitting the intelligence that is.
<p>
Whether or not adopting the agent metaphor,
there is definitely a challenge of
making applications more intelligent, and perhaps even
more human. Cf. [_WebWord] and [_Maes].
To my mind, one fundamental problem that we must solve
to realize this goal is to define the
technology, or the combination of technologies, needed
to support the anthropomorphic metaphor of agents.
<p>
Given the merits of logic programming in a variety
of application areas, encompassing areas such as
diagnostic expert systems,
natural language processing,
and control systems,
it seemed natural to organize a 
workshop called <a href=@online/workshops/www6/contents.html><em>Logic Programming and the Web</em></a>
to investigate how logic programming technology
might be deployed to make the Web more intelligent.
Nevertheless, although the presentations at the workshop
indicated that logic programming could fruitfully
be applied in for example the creation of virtual worlds,
e-commerce applications, and intelligent rental advisors,
it did not shed any light on how
to bridge the gap between the (mentalistic) agent
metaphor and its software realization.
<p>
In the remainder of this section we will discuss 
the <a href=@online/projects/wasp/contents.html><em>Web Agent Support Program</em></a> research project
to delineate the requirements for a framework providing
agent technology support for Web applications.


:@+ Web Agent Support Program

The WASP project, of which an outline is given in [_WASP], concerns the development
of Web Agent Support to enable average users
to keep track of relevant information on the Web.
<p>
The project was envisaged to result in a framework
providing support for:
\label{agent-framework}
<slide mode=hide id=agent-framework caption="Framework requirements">
%%:@ Framework requirements
<ul>
<li> intelligent navigation and information retrieval,
<li> information and document maintenance,
<li> user interfaces for Web-aware applications,
<li> dynamic documents with user-defined applets,
<li> declarative descriptions of agent behavior based
on user preferences,
<li> declarative modeling of coordinated and cooperative
behavior of software agents, and
<li> programming single and multi-agent systems.
</ul>
</slide>


As an target product for the WASP project, 
we envisaged developing
<em>Pamela</em> (Personal Assistant for Maintaining
Electronic Archives),
an application combining the functional and
architectural features mentioned above.


<p>
In summary, our project aims at providing insight in and solutions
for
<slide mode=hide id=aims caption="Our project aims ...">
<ul>
<li> modeling the behavior of cooperating agents,
<li> generic means for realizing actual agents in a Web-aware context,
<li> architectural support for programming agent-based systems.
</ul>
</slide>
The aspects of our research as indicated above
address the problems involved in defining and realizing
the potential of the <i>agent</i> metaphor as a
human--computer interface in the distributed information
system domain, in particular the Web.

<p>
The architectural requirements for realizing agents
in a Web-aware context consist of
(a) high-level support for distribution
to allow for notification and the communication between
agents,
(b) access to the Web both in terms
of server-side and client-side computation,
and (c) support for information retrieval and
data management.


<slide id=agent-components caption="Framework components">
:@ Framework components 
///<box>Web-aware agents</box>
<ul>
<li> a <i>methodology</i> for developing agent-based applications, as well as
<li> a <i>logical foundation</i> for modeling agent behavior; and in addition
<li> <i>guidelines</i> for realizing actual agent applications, and
<li> <i>software components</i> that can be used as building blocks, including
<li> a <i>language</i> for programming agent behavior.
</ul>
</slide>

%%:@ A framework for Web-aware agents
The WASP project is aimed to result in
a framework (in its extended meaning) for the
development of agent-based Web-aware applications.
The components provided by such a framework are listed
in \sliref{agent-components}.
In addition to the proper software components,
the framework includes a methodology,
as well as a logical foundation.
Further we wish to develop guidelines for realizing
actual agent applications, and our hope is to develop
a language for programming agent applications,
based on the language DLP, described in appendix
<ref file=@online/oo/appendix/dlp/dlp>DLP</ref>.

<ignore>
%%<h4>Discussion</h4>
Currently agents are in the focus of interest
of numerous research
groups&nbsp;<cite><a href="index.html#Internet">Internet</a>,<a href="index.html#Softbots">Softbots</a>,<a href="index.html#Software">Software</a>,<a href="index.html#Architecture">Architecture</a>,<a href="
index.html#Survey">Survey</a></cite>.
However, one may observe a divergence between
purely formal approaches and purely pragmatical
approaches, that provide an operational realization
of agents lacking a solid foundation.
<p>
Although many of the ideas and concepts
underlying agents have been a topic of
research for decades, notably issues
concerning distributed systems and
programming language design, the notion of agents
itself hase become a major research topic
only recently.
A survey of agent theories, architectures
and languages is given in &nbsp;[<cite><a href="index.html#Survey">Survey</a
></cite>].
As concerns agent theories, the main focus of
attention lies on a logical analysis
of intentional notions underlying the behavior
o research groups&nbsp;<cite><a href="index.html#Internet">Interne
t</a>,<a href="index.html#Softbots">Softbots</a>,<a href="index.html#Software">Software</a>,<a href="index.html#Architecture">Architecture</a>,<a href="
index.html#Survey">Survey</a></cite>.
However, one may observe a divergence between
purely formal approaches and purely pragmatical
approaches, that provide an operational realization
of agents lacking a solid foundation.
<p>
Although many of the ideas and concepts
underlying agents have been a topic of
research for decades, notably issues
concerning distributed systems and
programming language design, the notion of agents
itself hase become a major research topic
only recently.:u
single agents, whereas for agent architectures
issues of communication and cooperation between agents
have been addressed in more detail.
In &nbsp;[<cite><a href="index.html#Survey">Survey</a></cite>], concurrent l
ogic programming languages
are considered to be the ancestors of agent languages.
Of immediate relevance for our own work is the
Oasis programming language described in &nbsp;[<cite><a href="index.html#Oasis">Oasis</a></cite>],
which bears a close resemblance to the
distributed logic programming language DLP &nbsp;[<cite><a href="index.html#DLP">DLP</a></cite>].


Recently, commercial agent systems have been developed for the Web.
For example, General Magic has launched its
Telescript Agent System, a system that employs
mobile agents defined in General Magic's Telescript language.
Also in &nbsp;[<cite><a href="index.html#Internet">Internet</a></cite>], a number of agent-like programs
are described.
We are reluctant to qualify these systems as 'agents',
however, since they are defined in a purely operational way.
We find, for complex agents anyway, a more explicit
representation of the intentional aspects desirable.
<p>
In a recent issue of the CACM( 37(7), July 1994),
some current trends and applications of agent research
are presented.
From our perspective we consider as important representatives
of ongoing research:
&nbsp;[<cite><a href="index.html#Software">Software</a></cite>],
that describes the agent communication language
KQML (Knowledge Query and Manipulation Language);
&nbsp;[<cite><a href="index.html#Architecture">Architecture</a></cite>], which describes the complexity of integrating
collections of agents;
and &nbsp;[<cite><a href="index.html#Softbots">Softbots</a></cite>], which describes a number of simple agents
for facilitating access to the Web.
</ignore>
</document>
