<document>
@+ <a href=@talk-web-model.html target=_top>Models of computation</a>
<include file=navigate>

The <em>Programming the Web</em> workshop was intended 
to focus on concepts and requirements for high-level APIs
suitable for developing Web-aware applications.
The papers that were submitted, which are available
on the CDROM accompanying this book, covered a wide range of
interests, including computation models,
applications and user requirements,
software architectures and libraries,
as well as heuristics and guidelines for API developers.

<slide color=red id=api-requirements cap="Requirements for APIs">
<c><p></c><br><b>C</b>omplaints
<ul>
<li> lack of referential integrity
<li> undetected failures
<li> no control over <em>quality of service</em>
</ul>
<c><p></c><br><b>O</b>bservations
<ul>
<li> dynamic quality of services
<li> complex interaction
</ul>
<c><p></c><br><b>R</b>equirements
<ul>
<li> uniformity, openness, flexibility,
orthogonality, layered
%%, platform-independent
</ul>
<c><p></c><br><b>B</b>ehavior
<ul>
<li> reliable, configurable, monitoring, notification,
%%triggering,
thread-safe
</ul>
<c><p></c><br><b>A</b>nswers
<ul>
<li> object-oriented, components, virtual APIs, callbacks, plug-ins
</ul>
</slide>

The kickoff for the workshop was given by
Luca Cardelli, who raised the
question <a href=http://www.research.digital.com/SRC/personal/Luca_Cardelli/home.html>`What is the Web's model of computation?'</a>.
This question appeared to be of critical importance
for understanding the requirements for APIs and for
evaluating possible solutions.
In summary, we may observe that there is some notion
of global computation for the Web, but that computation on
the Web is fraught with many obstacles,
such as the lack of referential integrity (e.g. dead links),
unreliable services (both in availability and quality),
failures (due to servers or network congestion).
What we need, in conclusion, is 
some (formal) model of computation that captures these
properties.
In addition, we need to be able to deal
with such properties in our Web programs,
for example we may wish to anticipate on the possible
unavailability of a Web server, and provide an alternative in that case.
%%For example of a language offering such features,
%%see [_Cardelli98].
<p>
In \sliref{api-requirements}, an overview is given of the
complaints about the functionality of the Web,
observations concerning its `nature',
general requirements for open systems development,
a wish-list of desired behavioral characteristics
and potential (technological) answers.


<slide color=red id=api cap="Dimensions of APIs">
<c><p></c><br><b>A</b>ctions
<ul>
<li>define a distributed model of computation
that suits the Web.
<li>define canonical (language-independent?)
object models for ... resources, application domains ...
</ul>
<c><p></c><br><b>P</b>erspectives
<ul>
<li> servers - extensions
<li> browsers - clients, viewers, configuration
<li> agents  - e.g. payment
</ul>
<c><p></c><br><b>I</b>nterests
<ul>
<li> distributed objects
<li> plugin components
<li> formalization of requirements and solutions
</ul>
</slide>

Not surprisingly, there did not seem to be a canonical
approach to the definition and development of APIs
and Web applications,
perhaps not in the last place because the demarcation between
computation models, languages and APIs is not clear-cut.
<p>
Nevertheless, as summarized in \sliref{api},
it seemed clear that we need to define
a suitable computation model
as well as (abstract) object models
that capture the requirements for resources and
application domains (such as for example e-commerce).
In addition we must distinguish between client
and server perspectives, with autonomous (intelligent)
agents as a possible third perspective.
And, naturally, our own (technological) interests
play a role as well, to the extent that it may determine
possible solutions.
<p>
Considering the basic needs for the development of Web-aware
applications, as expressed by the workshop's participants,
which ranged over resolving URLs, billing and payment facilities,
and quality of service constraints,
we may observe that facilities for Web programming
are nowadays as a standard provided (as extensions)
by languages such as Tcl, Perl, Python and Java. 
More domain-specific facilities are being
developed in a CORBA context, or for frameworks such
as San Francisco.

\label{DOM}
:@- <em>Document Object Model</em>

Client-side scripting has been popularized by Dynamic HTML (DHTML)
as originally introduced by Netscape and Microsoft.
Nevertheless, scripting facilities are not standard accross the various
browsers.
To remedy this situation, the W3C has developed a recommendation
for a <em>Document Object Model</em> (DOM),
that provides a standard application programmer interface
to access the structure and content of HTML and XML Web pages.
The DOM allows XML and HTML pages to be treated in an object-oriented
way, providing facilities for access, navigation and manipulation.


<slide type=fig id=dom-structure cap="Hierarchical structure of DOM">
<image name=@figures/web/dom>
</slide>

Since XML is increasingly being used for other applications,
such as Electronic Data Interchange (EDI),
the DOM may in effect provide a foundation for developing Web applications.
<p>
The W3C DOM Recommendation provides interfaces, described 
in a language and platform-independent way in IDL,
for the structural components that may be used in XML
and HTML documents, as indicated in \sliref{dom-structure}.
These interfaces have been refined independently for
both XML and HTML, to allow programmers to access
XML and HTML-specific features.
In addition to the IDL interfaces, a language-binding 
is specified for ECMAscript, which may serve as an example
for similar bindings for Javascript and other languages, such as Java.


</document>
