<document>
<include file=navigate>

:@ Expressions
Again due to its C heritage,
C++ supports many {\em basic types}
(including $int$, $char$ and $float$)
and {\em compound types}
(including arrays, functions, pointer types,
reference types, and user-defined class, union or
struct types).
\see{cc-expr-1}

@$=P

<slide id=cc-expr-1 caption="C++ -- expressions (1)">
:@ Type expressions
<I
- basic types <em> -- int, char, float, ... </em>
- array  -- <em> int $ar[SIZE]$ </em>
- function  -- <em> void $f(int)$ </em>
- pointer  --  $ int* ,  char*, void (*f)(int) $ 
- reference -- $ int&, char*& $
- class, union, struct -- <em> user-defined</em>
I>
\vspace{-0.1cm}
</slide>

Pointer types encompass pointers to basic types
and pointers to user-defined types,
such as functions and classes.
The difference between {\em object},
{\em reference} and
{\em pointer types}
may be succinctly characterized as the difference
between the actual thing, an alias
(that looks like the actual thing but isn't)
and an address of the actual thing
(where you have to go to get it).


<slide id=cc-expr-2 caption="C++ -- expressions (2)">
:@ Expressions
\ifsli{\vspace{-0.5cm}}{}
<I
- operators -- $ + , - ,.., < , <= ,.., == , != ,.., && , || $
- indexing -- <em> $o[ e ]$ </em>
- application  -- <em> $o(...)$ </em>
- access -- <em> $o.m(...)$ </em>
- dereference -- <em> $p->m(...)$ </em>
- in/decrement  -- <em> <tt>o++</tt>, <tt>o--</tt></em>
- conditional -- <em> $ b?e1:e2$ </em>
I>
\vspace{-0.15cm}
:@ Assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- $ var = expression $
- modifying -- $+=$. $-=$, ...
I>
\ifsli{\vspace{-1cm}}{}
\vspace{-0.1cm}
</slide>

Value expressions may be created using
arithmetic and comparison operators
(including <tt> == </tt> for equality and <tt> != </tt>
for inequality).
As logical operators, C++ includes
conjunction (<tt>&amp;&amp;</tt>)
and disjunction (<tt><math> || </math></tt>),
as well as a number of bitwise logical operators.
Also, we have an {\em indexing} operator
(which may be defined for arbitrary types),
an {\em application} operator
(which may also be defined for arbitrary types),
an {\em access} operator
(which is as a standard used for member function
invocation or method calls),
a {\em dereference} operator
(which is used to invoke member functions through a
pointer to an object)
and {\em in-} and {\em decrement}
operations
(that, again, may be defined for arbitrary types).
Needless to say, user-defined operators
must be applied with care.
Also, we have a conditional expression of the form
<em>b?<math>e_1</math>:<math>e_2</math></em> testing the condition <em>b</em> to
deliver <math>e_1</math> when it evaluates to true and <math>e_2</math> otherwise.
Also, C++ allows for sequencing within expressions
of the form <math>(e_1,...,e_n)</math>,
which evaluates <math>e_1,...,e_n</math> in that order
and delivers <math>e_n</math> as its value.
<p>
Assignments in C++, it is important to note,
are written as <em>var = expression</em> with a single <em>=</em>
symbol.
This convention is known to cause mistakes by programmers
raised with languages such as Pascal or Modula-2.
In addition, C++ offers modifying assignments,
which may be used as, for example, in <em>n += 1</em>,
which is identical in meaning to <em>n = n + 1</em>.

</document>
