<document>
<include file=navigate>

:@ Technology
In addition to the elements introduced thus far, C++
offers a number of other features that may profitably
be used in the development of libraries and programs.
\see{cc-tech-1}


<slide id=cc-tech-1 caption="C++ -- techniques (1)">
<!-- \ifsli{}{ -->
:@ Techniques
<!-- } -->
\ifsli{\vspace{0.0cm}}{}
<I
- templates <em> --
\ifsli{\n\mbox{}}{}
{P template&lt;class T> class C { ... } P}</em>
- overloading  -- <em> void $read(int);$ void $read(float)$ </em>
- friends  -- <em> to bypass protection </em>
- type conversions -- <em> by class constructors or type operators</em>
- type coercion -- <em> by explicit casts</em> (is dangerous)
- smart pointers -- <em> by overloading de-reference</em>
%%- Standard Template Library (STL) -- <em>template containers</em>
I>
\ifsli{\vspace{-1.5cm}}{}
</slide>

For instance, C++ offers
{\em templates}
(to define generic classes and functions),
{\em overloading}
(to define a single function for multiple types),
{\em friends}
(to bypass protection),
{\em type conversion}
(which may be defined by class constructors
or type operators),
{\em type coercions}
(or casts, which may be used to resolve ambiguity
or to escape  a too rigid typing regime),
and {\em smart pointers}
(obtained by overloading the dereference operator).
An integral part of standard C++ is the Standard Template Library,
offering a generic collection of containers,
of which a brief description is given in 
section <ref file=@online/oo/I/2/poly-cc>STL</ref>.
%%In section \ref{delegation},
%%examples are given of how these features may be used.
<p>
To get some of the flavor of using C++, look at the definition of
the $ctr$ class in \sliref{cc-tech-2} employing multiple constructors,
operators, default arguments and type conversion.


<slide id=cc-tech-2 caption="C++ - techniques (2)">
\vspace{0.1cm}
<program>
class ctr { /// <box>C++</box>
public:
ctr(int i = 0, char* x = "ctr") { 
	n = i; strcpy(s,x); 
	}
ctr& operator++(int) { n++; return *this; }
int operator()() { return n; }
operator int()   { return n; }
operator char*() { return s; }
private:
int n; char s[64];
};
</program>
\vspace{-0.4cm}
:@ Usage
<program>
ctr c; cout << (char*) c++ << "=" << c();
</program>
\vspace{-0.6cm}
</slide>

The $ctr$ provides a constructor,
with an integer argument (which is by default set to zero,
if omitted) and a string argument
(that expects a name for the counter).
The increment operator is used to define the
function $add$
(which by default increments by one),
and the application operator is used instead of $val$.
Also, a type conversion operator is defined
to deliver the value of the $ctr$ instance anywhere where
an integer is expected.
In addition, a $char*$ type conversion operator
is used to return
the name of the $ctr$.
<p>
Again, the difference is most clearly reflected in how
an instance of $ctr$ is used.
This example illustrates that C++ offers
many of the features that allow us to define
objects which may be used in a (more or less)
natural way.
In the end, this is what software development is about,
to please the user, within reason.
</document>
