<document>
:@+ C++ realization

<include file=navigate>

The C++ realization consists of 
a <a href=broker.h.html><em>broker</em></a>
that gives access to the ORB for 
the <a href=client.c.html><em>client</em></a>
and to both the <em>ORB</em> and the <em>BOA</em> for
the <em>server</em>.
Apart from a <em>client</em> and <em>server</em>,
we need an implementation for the <em>world</em>,
which is given in respectively
the <a href=world-srv.h.html><em>world-srv.h</em></a>
and <a href=world-srv.c.html><em>world-srv.c</em></a> files.

<slide cap="C++ realization">
:@ C++ realization
<ul>
<li> <a href=broker.h.html>broker</a> -- access to ORB and BOA
<li> <a href=client.c.html>client</a> -- needs ORB only
<li> <a href=server.c.html>server</a> -- needs access to BOA
<li> implementation --
	<a href=world-srv.h.html>world-srv.h</a>
	<a href=world-srv.c.html>world-srv.c</a>
</ul>
</slide>


:@ Broker
The <em>broker</em> class in C++  provides a constructor,
an initialization method, the <em>operator</em> that starts
the server's event loop, and methods to convert strings
to object references and methods to read and write
object identifiers.

<slide type=hide cap="C++ broker -- class">
<c><h4><a href=broker.h.html>C++ broker -- class</a></h4></c>
<program>
class broker { /// <box>broker.h</box>
public:

broker();

void init(int& argc, char**& argv, int fl=0); /// 1 = client

int operator()(); /// to run the server

char* ref(corba::object* x);  /// object_to_string
corba::object* ref(const char* s); /// string_to object

corba::object* object(const char* file); /// get object IOR file
void refout(const char* f, corba::object* x); /// IOR to file
};
</program>
</slide>

Below, the implementation is given of the constructor, 
which needs to do nothing, the <em>init</em> method,
that initializes the ORB, and the BOA in the case of
a server.

<slide type=hide cap="C++ broker -- implementation">
<c><h4><a href=broker.h.html>C++ broker -- implementation</a></h4></c>
<program>
broker::broker() { }  /// <box>broker.c</box>

void broker::init(int& argc, char**& argv, int fl=0) {
   _orb = CORBA_ORB_init(argc, argv);
   if (!fl) _boa = _orb -> BOA_init(argc, argv);
   }
}

int broker::operator()() {
   _boa -> impl_is_ready(CORBA_ImplementationDef::_nil());
   }
</program>
</slide>

As said before, the broker class provides only shorthands
and abstractions from vendor-specific code.
<p>

:@ Client
The code snippet below shows how the client
creates and initializes a <em>broker</em> object,
how it obtains an object reference, and also some
calls to the <em>world</em> server, including <em>hello</em>
and a request to be informed about Clinton's affairs,
which certainly had some relevance at the time of writing this.

<slide type=hide cap="client">
<c><h4><a href=client.c.html>C++ client</a></h4></c>
<program>
    int main(int argc, char* argv[], char*[]) { ///<box>client.c</box>

    broker* _broker = new broker(); /// get yourself a broker
    try
    {
	_broker->init(argc,argv); /// initialize broker

	corba::object* obj = _broker->object("world.ref"); 
	universe_world_var world = universe_world::_narrow(obj); 

	{ /// some object invocations
		world -> hello();
	        world -> ask("How are Clinton's affairs?");
	        cout << "client:" << world->tell() << endl;
	}
    } catch(corba::exception ex) { ... }
}
</program>
</slide>

Note that the object that is obtained from <tt>world.ref</tt>
must be dynamically downcasted to its actual type.

:@ Server
The server code is similar as far as the creation and initialization
of the <em>broker</em> is concerned, but instead of creating
an object from a string identifier, it writes the identity of
the server object created to the file <tt>world.ref</tt>.

<slide type=hide cap="server">
<c><h4><a href=server.c.html>C++ server</a></h4></c>
<program>
int main(int argc, char* argv[], char*[]) ///<box>server.c</box>
{
    broker* _broker = new broker(); /// get yourself a broker 

    try
    {
	_broker->init(argc,argv); /// initialize orb and boa
	universe_world_var p = new universe_world_srv; 

	_broker->refout("world.ref", p); /// write identity
	
	_broker->operator()(); /// run the the world
    }
    catch(corba::exception& ex) { ... }
}
</program>
</slide>

The C++ class that implements the <em>world</em> interface
might appear disappointingly trivial.
It is left to the student to think of something less trivial.


<slide type=hide cap="world-srv.c">
<c><h4><a href=world-srv.c.html>C++ <em>world</em> implementation</a></h4></c>
<program>
void universe_world_srv::hello() {  ///<box>world_srv.c</box>
    	cout << "Hello World!" << endl;
	}

void universe_world_srv::ask(const char* s) {
    	cout << "Asking: " << s << endl;
	}

char* universe_world_srv::tell() {
	char result[] = "ok";
	CORBA_String_var s = (const char*) result; 
	return s._retn();
	}

void  universe_world_srv::halt() {
	exit(0);
	}
</program>
</slide>

:@ Discussion
Writing a CORBA application might indeed be simpler than you thought.
Follow the steps taken in this example as a recipe, and
you master the art of writing CORBA applications instantly.
</document>
