<document>
:@+ Java realization

<include file=navigate>

The Java realization of the <em>Hello Universe</em>
example has the same structure as the C++ realization.
See \sliref{java-realization}.

<slide id=java-realization cap="Java realization">
:@ Java realization
<ul>
<li> <a href=broker.java.html>broker</a> -- contact ORB and BOA
<li> <a href=client.java.html>client</a> -- connect to server
<li> <a href=server.java.html>server</a> -- announce server object
<li> implementation -- <a href=world_srv.java.html>world_srv.java</a>
</ul>
</slide>

:@ Broker
The IDL-Java binding seems to be somewhat better standardized,
possibly because it is of a later date.
So the advantage of using a <em>broker</em> abstraction
lies primarily in the shorthands it provides,
and the similarity with the C++ realization.

<slide type=hide cap="Java -- broker">
<program>
import org.omg.CORBA.*; ///<box>broker.java</box>
import java.io.*;


public class broker {
boolean _fl = false;
java.applet.Applet _applet = null;

public broker() { }
public broker(boolean fl) { _fl = fl; }

public broker(java.applet.Applet x) {
	_applet = x;
	init(null);
	}

public void init(String[] args) {
if (_applet == null) {
       	_orb = ORB.init(args,new java.util.Properties());
       	if (!_fl)
	    _boa = _orb.BOA_init(args,new java.util.Properties());
} else 
	_orb = ORB.init(_applet, null);
}


public int operator() { /// run -- server-only
        if (!_fl) _boa.impl_is_ready(null);
	return 0; // OK
	}

 ....
};
</program>
</slide>

The only (real) difference with the C++ broker class
is the possible existence of an <em>applet</em>.
Somewhat unfortunately, when the (client) object is an applet,
the initialization of the ORB must be done in a different way.

:@ Client
The client in Java must, also, create and initialize a
<em>broker</em> object, and create a reference to
the <em>world</em> server object.

<slide type=hide cap="Java -- client">
<c><h4>Java -- client</h4></c>
<display>
package universe;

import org.omg.CORBA.*; ///<box>client.java</box>
import java.io.*;
import hush.broker; /// see <a href=broker.java.html>broker.java</a>

public class client 
{
    public static void main(String args[])
    {
	broker _broker = new broker(true); // true means client only
	try
	{
	    _broker.init(args);  /// init orb

	    org.omg.CORBA.Object obj = _broker.object("world.ref"); 
	    world world = worldHelper.narrow(obj); 
	    if (world == null)
		throw new RuntimeException();
	    

	    System.out.println("Enter 'h' (hello) or 'x' (exit):");
	    ... // do some requests to the world
	} catch(...)  { ... }
    }
};
</display>
</slide>

In Java, an extra object <tt><em>interface</em>Helper</tt>
is created to allow for casting the object that is
created from a string  object identifier to its actual object type,
<em>world</em> in this case.

:@ Server
The server provides access to the (server) object that implements the 
<em>world</em> interface.
After creating and initializing a <em>broker</em> object,
it creates an instance of a <em>world</em> server and
writes its object identifier to the file <tt>world.ref</tt>.
In addition it writes a <tt>HTML</tt> file that contains
an applet that has a (string) reference to the server.
See the online version or the Orbacus documentation for details.

<slide mode=hide cap="Java -- server">
<c><h4>Java -- server</h4></c>
<display>
package universe; ///<box>server.java</box>

import org.omg.CORBA.*;
import java.io.*;
import hush.broker;  // see <a href=broker.java.html>broker.java</a>

public class server
{
    public static void main(String args[])
    {
	broker _broker = new broker();
	try
	{
	    _broker.init(args); /// create orb en boa; 

	    world_srv p = new world_srv(); /// create impl object

	    _broker.refout("world.ref",p); /// write ref

					   /// create world.htm
	    _broker.html("world.htm",p,    
		" code=universe/applet.class " 
			 + "width=500 height=300");


	    _broker.operator(); // run  == boa.impl_is_ready(null);

	} catch(SystemException ex) { 
	    _broker.print(ex);
	    System.exit(1);
	}

	System.exit(0);
    }
}
</display>
</slide>
After creating the object and its reference files, the server
invokes the method <tt>impl_is_ready</tt>,
through the broker's <em>operator</em>.
<p>
The implementation of the actual <em>world</em>
server is as simple as its C++ peer. 

<slide mode=hide cap="Java -- implementation">
<c><h4>Java -- server implementation</h4></c>
<display>
package universe; /// <box>world_srv.java</box>

import org.omg.CORBA.*;

public class world_srv extends _worldImplBase
{
public void hello() {
	System.out.println("Hello World!");
	}

public void ask(String msg) {
	System.out.println(msg);
	}

public String tell() {
	String s = new String("ok");
	return s;
	}  

public void halt() {
	System.exit(0);
	}
}
</display>
</slide>

Note that dealing with strings is significantly simpler in
Java, because of Java's higher-level built-in String.

:@ Discussion
Observe that the Java and C++ realizations are very similar
in structure and also very similar in code.
Nevertheless, what does not become evident in this small example
is the bookkeeping that needs to be done in C++
for managing the objects created. Since Java supports
garbage collection, the programmer needs to do no such bookkeeping,
whereas in C++ one has to use either reference counting
or the special <tt>_var</tt> type, that acts as a smart pointer
which does the reference counting automatically.
See any CORBA documentation for details.
</document>
