<document>

<include file=navigate>

:@ Terminology
In comparison with C++, Java is almost as rich in keywords.
Notably lacking, however, is the keyword <em>virtual</em>.
This is not needed, since in Java every method is by definition
subject to dynamic dispatching.
Also, since Java doesn't allow multiple inheritance,
there is no need to use it for avoiding multiple copies of
inherited base classes.

<slide caption="Java -- terminology (1)">
:@ Keywords:
/// <box>Java overview</box>
*- {\bf new}, {\bf finalize}, {\bf extends}, {\bf implements}, {\bf synchronized}
\vspace{-0.1cm}
:@ Language features:
<I
- no pointers <em> -- references only simplify life</em>
- garbage collection <em> -- no programmers' intervention</em>
- constructors <em> -- to create and initialize</em>
- single inheritance <em> -- for refinement</em>
- interfaces <em> -- abstract classes </em>
- synchronized <em>-- to prevent concurrent invocation</em>
- private, protected, public <em> -- for access protection</em>
I>
\ifsli{\vspace{-1cm}}{}
\vspace{-0.15cm}
</slide>

New is the keyword <em>interface</em> in Java.
Defining an <em>interface</em> is equivalent to defining
an abstract class in C++. One merely lists the methods
provided by the (abstract) claas, without providing an implementation.
A concrete class may then indicate that it <em>implements</em>
the interface. In this way multiple (interface) inheritance is
supported in a nice and clean way. 
<p>
Also new is the keyword <em>synchronized</em>, reflecting the built-in
support for concurrency in Java.
A <em>synchronized</em> method excludes multiple invocations of
that method, which might otherwise occur in a multi-threaded
program.
<p>
The keyword <em>final</em> may be used to indicate that a
particular value may not be changed. 
In this sense it is similar to the C++ keyword <em>const</em>.
It must be noted that Java is even more elaborate in the 
use of the keywords
{\em private}, {\em protected} and {\em public} than C++.
They are used 
to indicate access restrictions for the methods
of an object for objects inside and outside the package in
which the objects' class is defined.



The language features offered by Java
resemble those of C++.
However, in many respects Java is much simpler than C++.
Most notably, the absence of pointers, a sure source of errors,
makes programmers' lives easier.
In particular since Java offers automatic garbage collection,
programmers need not to worry about disposing
objects created dynamically.
Resource management, however, may be done by defining
a method <em>finalize</em>.
The counterpart, however, of that is that all objects
in Java come into existence by explicit dynamic creation.
<p>
The availability of <em>interfaces</em>
compensates for the restriction to single inheritance, which must
be indicated by the keyword <em>extends</em>
instead of the colon.
It has often been argued that multiple inheritance
is not really necessary.
This holds true, however, only for
implementation inheritance.
Multiple (interface) inheritance is a powerful feature
that has interesting applications once one has discovered
how to use it.
%%AE
%%See [XXXX] for the application of a software design pattern
%%that intrinsically requires multiple inheritance.
<ignore>
Java provides a simple, but rather basic threading support.
Programming with threads, however, requires a lot of care.
To avoid the concurrent access of critical resources,
the Java language has the keyword <em>synchronized</em>,
by which concurrent invocations of a method can be prevented.
</ignore>
</document>
