<!DOCTYPE DOCUMENT PUBLIC "-//DejaVu//DTD DVDOC//EN">
   <title>Declarative and Operational Document Encodings</title>
   
    <p>SGML (Standard Generalized Markup Language) [_SGML] is an
     ISO standard which uses tagging to encode the logical structure
     of a document.  
     The set of tags needed to
     describe a specific document structure depends on the type of
     documents involved.  For example, the tags needed to markup a
     mathematical article are in general not suitable to describe the
     structure of a telephone book.  As a consequence, SGML does not
     describe a set of tags, but a way to define an appropriate set of
     tags and the order these tags should appear in the document
     instance. Such a definition is called a <em>document type
     definition</em> or DTD.  Note that the idea of defining a
     fine-tuned markup language for each document type is the exact
     opposite of the ``one size, fits all'' approach of current
     HTML-based web browsers.  Many other advantages of having SGML
     aware Web browsers (and servers) are discussed in [_SperGold94].

    <p>An SGML document essentially consists of two parts: a prolog,
     containing the document type declaration, followed by the
     document instance, containing the data interspersed with markup.
     
    <h4>Document instance</h4>
    
    <p>A document instance is a hierarchical structure of (possibly
     empty) <i>elements</i>, where each non-empty element contains
     other elements or character data. Each element has a name (the
     <i>general identifier</i>) and the start and end of an element
     are indicated by tags (typically
     &lt;name&gt;content&lt;/name&gt). Begin or end tags may be
     mandatory or optional.  Moreover, elements contain zero or more
     <i>attributes</i>.  Consider the example document below.  <!--
     \verbatiminput{memo.sgml} --> The root element <i>memo</i> has
     one attribute, indicating the security level of the document.
     The memo element contains five other elements. A double <i>to</i>
     element stating the addressees, a <i>from</i> element specifying
     the sender, a <i>subject</i> and a <i>body</i> field.  All
     elements of memo contain character data and no other elements.
     Note that the tags emphasize the logical structure of the
     document rather than stating how it should be formatted.
     
    <h4>Document type declaration</h4>
    
    <p>The document instance above must be preceded by a <i>doctype</i>
     declaration.  The main part of the document type declaration is
     the <i>document type definition</i> or DTD. It defines the
     elements of a document and the required order of their
     sub-elements.  The elements and their contents are defined by the
     use of <i>element declarations</i>.  <!--
     \verbatiminput{memo.dtd} --> The second line declares the
     memo element, and defines its content as a sequence of one or
     more <i>to</i> elements, a <i>from</i>, a <i>subject</i> and a
     <i>body</i> element.  The two `O' characters stand for ``omit'',
     indicating that the begin and end tag may be omitted.  The third
     line defines the elements containing character data only.  Their
     start tags are mandatory, indicated by the `-'.  The list of
     attributes of each element is declared by an <i>attlist</i>
     declaration. Attributes can be
     of different types, and be mandatory or optional.  The DTD can
     specify a default value, as is shown in the case of the security
     attribute. The DTD declaration may be contained within the
     <i>doctype</i> declaration, but are typically defined by a
     separate file. In that case, the <i>doctype</i> declaration
     contains a reference to that file.
     
    <h4>Processing instructions</h4>
    
    <p> Processing instructions are used to pass system dependent
     information to the application to tell how the document is to be
     processed. Processing instructions are contained within `&lt?'
     and `&gt;' characters and can appear on arbitrary places within
     the document. In the following examples, we employ processing
     instructions to indicate the URL of a style sheet.
     
    <h4>Entities</h4>
    
    <p>Fragments of markup and character data can be given a name
     using an <i>entity</i> declaration.  The declaration of an entity
     is part of the document type declaration, but the entity may be
     used within the document instance. The contents of an entity may
     be defined by a string, or may be contained in an external file,
     in which case the entity declaration contains a reference to the
     file.  External entities may be referenced by a <i>system
     identifier</i>. Support for these identifiers is system dependent
     and may include filenames, URLs and database queries. Consider a
     variant of the previous example: <!-- \verbatiminput{entity.sgml}
     --> The first line references an external DTD by means of a
     system identifier, and the second line defines an entity for
     later usage. Note that the entity definition is enclosed within
     the square brackets of the <i>doctype</i> declaration.  The third
     line contains a processing instruction to define the location of the
     style sheet. In contrast to the URL of the DTD, which is resolved
     by the parser, the URL of the style sheet is passed to
     the application without further processing by the parser.

     <p>To avoid system dependent identifiers such as filenames, an
     extra indirection is provided by the concept of <i>public
     identifiers</i>.  These identifiers are assumed to be publicly
     known, and the SGML parser of the target application is expected
     to be able to resolve them. Typically, a local catalog file is
     used to map public identifiers onto system dependent ones.
     <i>Formal</i> public identifiers have a standardized and meaningful inner structure,
     to facilitate automatic resolving without the use of catalogs.
     In the following examples, we refer to the HTML 3.0 DTD by means of
     a formal public identifier.
     
