    <h1>DejaVu -- The Framework</h1>
    <!-- \label{dejavu} -->
    
    <p>The problem addressed in the DejaVu project is essentially to
     provide support for the software engineering of (moderately complex)
     applications that require, apart from other functionality, multimedia
     interfaces and associative online help.  Such support is primarily
     offered in the form of class libraries, most of which are written in
     C++.  The language C++ was chosen for pragmatic reasons, since it
     allows for easily wrapping public domain software written in C in
     classes with a more user-friendly interface.
     
    <p>
     Application development generally encompasses a variety of programming
     tasks, including system-level software development (for example for
     networking or multimedia functionality), programming the user
     interface (including the definition of screen layout and the
     responsivity of the interface widgets to user actions), and the
     definition of (high-level) application-specific functionality.  Each
     of these kinds of tasks may require a different approach and possibly
     a different application programming language.  For example, the
     development of the user interface is often more conveniently done
     using a scripting language, to avoid the waiting times involved in
     compiling and linking.  Similarly, defining knowledge-level
     application-specific functionality may benefit from the use of a
     declarative or logic programming language &nbsp;[<cite><a href=refs.html#Eliens92>Eliens92</a></cite>].
     
    <p>
     In our project, we decided from the start to support a multi-paradigm
     approach to software development and consequently we had to define the
     mutual interaction between the various language paradigms, as for
     example the interaction between C++ and a scripting language, such as
     Tcl &nbsp;[<cite><a href=refs.html#Tcl>Tcl</a></cite>].  Current scripting languages, including Python &nbsp;[<cite><a href=refs.html#Python>Python</a></cite>]
     and Tcl, provide facilities for being embedded in C and C++, but
     extending and employing these languages in C or C++ is rather
     cumbersome.  The <i>hush</i> library &nbsp;[<cite><a href=refs.html#Eliens95b>Eliens95b</a></cite>] offers a uniform
     interface to a number of script languages, and in addition, it offers
     a variety of widgets and multimedia extensions, that are accessible
     through any of the script interpreters as well as the C++ interface.
     
    <h2>Idioms and Patterns</h2>
    
    <p>To realize the functionality of <i>hush</i> and its extensions, we
     employed some basic idioms and patterns &nbsp;[<cite><a href=refs.html#GOF94>GOF94</a></cite>]
     derived from the <i>handle/body</i> idiom
     that was originally introduced in &nbsp;[<cite><a href=refs.html#Coplien92>Coplien92</a></cite>], notably the <i>virtual
      self-reference</i> idiom and the <i>dynamic role-switching</i> idiom.
     
    <h4>Virtual self-reference</h4>
    
    <p>The <i>virtual self-reference</i> idiom has been introduced to
     support the development of compound widgets.  It allows for (re)using
     the script and C++ interface of possibly compound widgets, by
     employing explicit redirection to an inner or primary component.
     Every widget can delegate part of its functionality to an
     inner component. It is common practice to derive a compound widget
     from another widget by using interface inheritance only, and to
     delegate functionality to an inner component by explicit redirection.
     
    <h4>Dynamic role-switching</h4>
    
    <p>The <i>dynamic role-switching</i> idiom provides a means to offer a
     multitude of functional modes simultaneously. For example, a single
     kit object gives access to multiple (embedded) script interpreters, as
     well as a remote kit.  The idiom is essential in supporting
     the multi-paradigm nature of the DejaVu framework. In our description
     of the design of the Web components in section <ref><a href=#Design>Design</a></ref>, we
     will show how dynamic role-switching is employed for using various
     network protocols via the same (net)client class. The idiom is also
     used to define a (single) viewer class that is capable of displaying
     documents of various MIME-types (including SGML, HTML and VRML).
     
    <h2>The Components</h2>
    
    <p>The <i>hush</i> library contains the classes providing the
     interface to the embedded script interpreter(s) and the classes
     allowing for a smooth interaction with the underlying window system,
     including the <i>kit</i>, <i>event</i>, <i>handler</i>, <i>widget</i> and
     <i>item</i> classes.  In addition to these, <i>hush</i> provides the class
     <i>session</i>.  Application programmers are minimally required to
     derive a class, say <i>application</i>, from <i>session</i> and
     redefine the function <i>main</i>.  When the program is to be used as
     a windowing shell, the function <i>prelude</i> may be redefined to
     allow for initializing external packages or other components of the
     DejaVu framework.
     The extensions of <i>hush</i> include
    <slide id=frame-1 cap="The <i>hush</i> extensions">
    <ul>
     <li> <i>widgets</i> -- for creating graphical user interfaces
     <li> <i>sim</i>   -- for discrete event simulation
     <li> <i>hymne</i> -- for sound synthesis and music
     <li> <i>video</i> -- for digital video
     <li> <i>vr</i> -- for the OpenGL-based VRML extension
     <li> <i>web</i>   -- for connecting to the Web
    </ul>
    </slide>
    
    
    <h4>User interface widgets</h4>
    
    <p>The <i>hush</i> widget library contains classes for a variety of
     user interface widgets, including the classes <i>button</i>,
     <i>menu</i>, <i>text</i>, <i>canvas</i>, <i>filechooser</i> and many
     more &nbsp;[<cite><a href=refs.html#Eliens95b>Eliens95b</a></cite>].  A widget allows for binding an independently
     defined handler to user actions.  Alternatively, a widget may declare
     itself to be its own handler.  Allowing for delegating to an external
     handler helps to avoid cluttering the class name space.  On the other
     hand, employing inheritance may result in a significant reuse of
     initialization code.
     
    <h4>Simulation modeling support</h4>
    
    <p>Our interest in discrete event simulation was accidentally aroused
     by the need to provide support for Business Mathematics courses
     &nbsp;[<cite><a href=refs.html#BolEl94>BolEl94</a></cite>].  The simulation package we developed is fully integrated
     with <i>hush</i>, which allows the simulation developer to use the
     animation and hypertextual facilities offered by <i>hush</i>.  The
     package provides the following classes
    <slide id=frame-2 cap="Simulation classes">
    <ul>
     <li><i>simulation</i> -- containing the scheduler
     <li><i>event</i> -- representing the events
     <li><i>entity</i> -- representing processes consisting of events
     <li><i>generator</i> -- providing a variety of random
      distribution functions 
     <li><i>resource</i> -- to model passive objects conveniently
     <li><i>histogram</i> -- to plot the results of a simulation
     <li><i>analysis</i> -- to perform statistical analyses on the
      results
      
    </ul>
    </slide>
    The class <i>event</i> (used in the simulation package) contains
    functions to determine the status of an event.  For example, an event
    may be <i>conditional</i> (which means that it will be tried for each
    cycle of the scheduler) or <i>pending</i> (which means that its
    activation will be delayed until it becomes active or conditional).
    The library also contains a class <i>entity</i> representing
    processes, or active objects.  An entity may be regarded as a
    ``structured event'', that is as an active object with phases in its
    life-cycle.  In our approach, however, an explicit switch on the
    entity's phases must be included in the definition of the event.
    
    <h4>Software sound facilities</h4>
    
    <p>Sound facilities are an essential ingredient of a framework
     supporting multimedia.  Sound may be produced from a stored audio file
     or, provided that real-time sound synthesis facilities are available,
     from some high level representation in a sound or music description
     language.  Apart from a reduction in the storage capacity needed (and
     correspondingly, the network traffic load in a distributed
     environment), employing a high level music/sound description language
     has as an additional advantage that improved synchronization and
     linking facilities may be offered.  The DejaVu framework supports both
     MIDI and the sound synthesis facilities offered by Csound &nbsp;[<cite><a href=refs.html#Csound>Csound</a></cite>].
     When creating an instance of the <i>icsound</i> class a process is
     created which is capable of receiving sound events, as defined by the
     Csound numerical score language, and converting these events to data
     for the audio device.  (The <i>icsound</i> class inherits from a
     <i>wrapper</i> class providing the functionality for spawning of a new
     process that communicates with the original process via pipes.)  Care
     is taken that the audio device is connected only once.  However,
     multiple instances of the <i>icsound</i> class may exist.  In addition
     to the <i>icsound</i> class, which accepts only low level sound
     events, instances of <i>player</i> may be used which accept musical
     events defined in the high level music description language Scot, that
     comes with Csound.  The <i>player</i> class is also a <i>handler</i>.
     This allows instances of <i>player</i> to be embedded in scripts, as
     well as being activated by user-defined events.
     
    <h4>Digital video</h4>
    
    <p>Digital video may be considered as another standard ingredient of
     hypermedia and multimedia frameworks.  To employ digital video, our
     framework offers the class <i>video</i>.  Apart from the usual widget
     creation commands, the class <i>video</i> has functions for reading in
     a video file, to play (forward) and to rewind.  In addition, it offers
     a function to bind a frame number to an event.  Binding frame numbers
     to events allows for both synchronization and hyperlinking, albeit in
     a low level fashion.  Our video extension is based on the public
     domain <i>xanim</i> package and includes support for MPEG, AVI and
     Quicktime.
     
    <h4>3D graphics and VRML</h4>
    
    <p>Another exciting development is the introduction of VRML &nbsp;[<cite><a href=refs.html#VRML>VRML</a></cite>] as
     a standard for developing 3D graphics and virtual reality on the Web.
     We have extended <i>hush</i> with widgets for displaying 3D graphics
     (based on the public domain Mesa &nbsp;[<cite><a href=refs.html#Mesa>Mesa</a></cite>] library for OpenGL) and VRML
     scenes (based on an extension of the public domain Qvlib &nbsp;[<cite><a href=refs.html#Qvlib>Qvlib</a></cite>]
     parser).  Due to the extensibility of our Web (browser) components,
     these widgets can easily be incorporated as special (Web-aware)
     viewers in the browser.
     
