<!DOCTYPE DOCUMENT PUBLIC "-//DejaVu//DTD DVDOC//EN">
 <html>
   <title>Declarative and Operational Document Encodings</title>
   
    <h1>Declarative and Operational Document Encodings</h1>
    <!-- \label{encoding} -->
    
    <p>Document markup languages have slowly evolved from procedural to
     declarative markup. Powerful formatters such as troff and TeX merely
     allow the text of the document to be interspersed with formatting
     commands.  If the document has to be formatted by another
     system, or for a different medium, (all) the markup has to be
     changed.  Macro facilities may solve part of the problem because
     part of the (low-level) formatting decisions can be localized in
     the macro definition. Macro packages such as LaTeX provide high
     level markup constructs which allow for a more declarative style
     by emphasizing the structure of the text, instead of the physical
     document layout. 
     
    <h2>Content, Structure and Layout</h2>
    
    <p>To enhance inter-operability, that is the ability to edit, browse and
     typeset documents for different media on different platforms, one
     has to make a clear distinction between a document's contents,
     how these contents are structured,
     and the mapping of structure to format instructions.
     
    <h4>Content data</h4>
    
    <p>Encoding mechanisms used to represent the content data largely
     depend on the medium used.  Text encoding schemes, for example,
     have to be able to deal with multiple (international) character
     sets. Similarly, mechanisms used to represent graphics, images,
     audio and video data differ on the resolution and compression
     technique used. Because continuous media streams are encoded by
     sequences of digital samples, they have an inherent structure
     representing the temporal dimension that static media lack.
          
    <h4>Declarative markup -- structure</h4>

    <p>Mechanisms to encode the logical structure of a document should
     be able to describe several relations between the various (data)
     parts of the document.  Common examples are a part-whole
     relation, used to build a hierarchical structure (e.g article,
     section, subsection, paragraph), references, used to make
     bibliographical references, footnotes, annotations and other
     links.  For text-based applications, <em>tagging</em> has proved
     to be an effective mechanism, but even if other media types are
     involved (e.g. images in HTML) tagging can still be very useful.
          
    <h4>Style sheets -- layout</h4>
    
    <p>The information provided by the content data and declarative
     markup needs to be augmented with information about how the
     various elements should be mapped on an output device.  Rather
     than embedding such information in the document or processing
     application, it is often better to store this kind of information
     independently, in a separate <em>style sheet</em>.
     
    <p>While the concept of a style sheet can be found in many of
     today's word-processors, style sheets are only slowly finding
     their way into the Web.  At the moment, various schemes
     and style sheet languages are being discussed by a W3C
     [_W3-Style] working group in order to find a mechanism which
     meets the needs of both authors and readers.
     
    <p>Style sheets have been an issue for SGML-based document
     processing as well. While the SGML standard was originally
     published back in 1986, there still is no definitive language
     addressing the needs for standardized style sheets. The Document
     Style Semantics and Specification Language [_DSSSL] is still a
     draft international standard.  As a result, many SGML vendors
     developed their own style sheet language, e.g.. SoftQuad's Panorama
     Pro [_Panorama].

    <h2>SGML -- basic concepts</h2>
    <so basics>
     
    <h4>Encoding Hypermedia Documents</h4>
    
    <p>The separation of content, structure and layout has proven to
     be effective for text-oriented document processing, but it is not
     clear whether this approach scales to time-dependent and highly
     interactive hypermedia documents.  Synchronization constraints,
     user interaction, hyperlink traversal are important
     characteristics of such documents, but it depends on the
     application whether these issues are part of the style, structure
     or even the document contents.
     
    <p>While many of these issues can be defined in the structure of a
     document, SGML lacks standard mechanisms to do so.  The
     Hypermedia/Time-Based Structuring Language (HyTime) [_HyTime] is
     a relatively new ISO standard defining syntactic SGML conventions
     representing semantics for hyperlinking, addressing, aligning and
     scheduling of multimedia documents. We will further discuss these
     issues in section <ref>issues</ref>.

