<!DOCTYPE DOCUMENT PUBLIC "-//DejaVu//DTD DVDOC//EN">
 <html>
   <title>Active Web documents -- an example</title>
    <h1>Active Web Documents -- an example</h1>
    <!-- \label{example} -->
    <!-- \newcommand{\codeinput}[1]{\verbatiminput{#1}} -->

    <p>As an example, we will illustrate how to deploy the SGML
     related components used to parse and display SGML encoded
     documents.  The <i>SGMLviewer</i> is based on SP, James Clark's
     conforming SGML parser [_SP].  Web applications
     benefit from the use of such a parser in a number of ways:
    <ul>
     
     <li><i>Validation</i> -- 
      Unlike most Web browsers, the <i>hush</i> browser uses a validating
      parser.  The document instance is validated against a document
      type definition.  As a result, the <i>hush</i> browser prints
      warnings in case of illegal markup.
      
     <li><i>Extra SGML features</i> -- SGML provides some standard
      features such as file inclusion and macro substitution, which
      are processed by the parser in a fully transparent way.
      Moreover, the parser output describes a complete document (that
      is, all optional begin and end tags are added, and default
      attribute values are filled in), which simplifies formatting and
      display.
      
     <li><i>Support for new document types</i> -- New document types
      and extended versions of existing ones can be parsed by defining
      a new document type definition describing the structure of the
      document.  Such document types can be made to fit the
      application domain in a precise manner.
    </ul>
    
    <h2>Extending Document Types</h2> 
    
    <p>We will start by illustrating how to extend the default HTML
     document type definition.  The document instance given below is
     specified in HTML but employs in addition two extensions for
     active documents.  The first extension is a <i>video</i> tag,
     which is used to display inline video fragments. The other is an
     <i>applet</i> tag, used to embed small applications written in a
     scripting language. We use Tcl in the examples.  In the example
     below, the <i>applet</i> displays some notes that are played when
     the user clicks on the image.  <!-- \codeinput{demo.sgml} -->
     The first line of the example defines the type of the document
     (demo).  It is specified in a separate document type definition
     (DTD) that we will describe below.  The parser automatically
     retrieves the DTD from the Web if its location is specified by a
     URL.  The next line illustrates the use of an <i>entity</i>
     declaration, an SGML mechanism used here as a primitive macro
     facility defining the title of the document. The title entity is
     used twice, in the <i>title</i> and <i>h1</i> tag, and will be
     expanded by the parser.  The third line specifies the style sheet
     that is needed to display the contents of the document. The use
     of style sheets will be explained later.  The <i>video</i>
     element requires a <i>src</i> attribute defining the location of
     the video file.  Finally, the <i>applet</i> tag is used to inline
     embedded script code defining a small musical applet. The image
     and the note files are located in the directory specified in the
     <i>data</i> attribute of the <i>applet</i> tag.

    <p>Whether active document elements are to be defined by a new,
     specific tag or by the more general applet mechanism is a matter
     of taste. A new tag requires modification of the DTD and style
     sheet but describes the element in a more declarative way,
     which gives the application more freedom in displaying
     the contents. For example,
     a browser may decide to display a text alternative if the
     local platform does not support video.  

     <!-- \begin{figure}[!htb]\epsfig{file=browser.eps,
     width=\linewidth}\caption{The example document as displayed by
     the <i>hush</i> browser}\label{fig:browser}\end{figure} -->
     
     
    <h4>Extending the DTD</h4> 
    
    <p>
     Recall that a document type definition defines the structure of a
     document by describing the <i>elements</i> and <i>attributes</i>
     that can be interspersed as tags with the document content.  The
     following DTD extends the (draft) HTML 3.0 DTD [_HTML3] with the
     <i>video</i> and <i>applet</i> elements required for the example
     above.  <!-- \codeinput{demo.dtd} --> The first line extends
     the <i>special</i> parameter entity defined by the HTML DTD with
     <i>video</i> and <i>applet</i> elements, enabling the use of
     <i>video</i> and <i>applet</i> tags where image or anchor tags
     are allowed. The second definition of <i>special</i> (in the HTML
     DTD) will be ignored by the parser.  In the second line, the SGML
     parameter entity mechanism is used to include the draft HTML 3.0
     document type definition, which is referenced by a formal public identifier.  
     The <i>demo</i> element consists of
     the sub-elements defined by the HTML DTD.  The two <i>O</i>
     characters in the element declaration indicate that begin and end
     tag of the <i>demo</i> element may be omitted, since the begin
     and end of the document can be derived by the parser.  In
     contrast, the begin tag of a <i>video</i> or <i>applet</i>
     element is mandatory, the end tag optional.  The video and applet
     elements have optional (<i>implied</i>), mandatory
     (<i>required</i>) and default attributes (e.g. the <i>lang</i>
     attribute defaults to <i>Tcl</i>).

    <p>The information contained in the DTD is used by the parser to
     generate a complete and validated document instance. Note that
     this task could be performed by an HTTP-server as well, which
     would significantly simplify the design and implementation of web
     clients. Therefore, there are strong arguments to add SGML
     functionality to servers as well [_SperGold94].

    <h4>Style sheet</h4>
    
    <p>Style sheets define how the various elements should be
     processed.  The <i>hush</i> browser (see figure
     <ref>fig:browser</ref>) defines a default style for HTML
     elements.  However, these styles can be redefined and extended by
     a document instance using a special processing instruction,
     notated as <tt>&lt;?stylesheet&nbsp;url&gt;</tt>.  The browser
     retrieves the URL specified in the processing instruction and
     uses it to display the contents of the document.  The example
     specifies a URL to a style sheet that describes how to process
     the new <i>video</i> tag. Recall that processing instructions are
     application dependent, so the parser passes the text in a processing
     instruction directly to the application.  At the moment, we use
     an experimental style sheet language based on Tcl. An example of
     a style sheet fragment that specifies how the <i>title</i> tag
     should be processed, is given below.  <!--
     \codeinput{demo.style} --> While the style sheet mechanism
     needs some refinement, our approach supports the extension of
     existing document types and allows for extensive experimenting
     with the (many) new tags proposed for the HTML 3.0 standard.
     
    <h2>Creating New Document Types</h2> 
    
    <p>The document instance of the previous example was very similar
     to plain HTML documents, which made it worthwhile to reuse the
     original HTML DTD and style sheet. However, for some applications
     HTML is not suited at all and a completely new document structure
     is needed. The next example shows a document
     instance of a simple musical application.  

     <!-- \codeinput{music.sgml} --> 

     The first line defines the (filename of) a new document type
     definition. The next line contains an entity definition
     describing a G7 chord. 
     An SGML processing instruction is used to specify the filename of
     the style sheet.  The root of the document hierarchy is the
     <i>score</i> element, consisting of several chords. Chords are
     build of notes, which are described by single characters. The
     first two chords use the entity defined before, specifying the
     notes of a G7 chord.  The third occurrence of
     <i>chord</i> describes a C major
     chord. 
     
    <h4>A new DTD</h4>
    
    <p>The DTD corresponding to the simple musical document 
     given above defines the structural elements and their
     attributes. When the application  processes the 
     document, the parser will fill in the
     default duration for all notes, resolve the
     entity definition and add the missing end tags for the notes and
     chords.
     
     <!-- \codeinput{music.dtd} -->
     
    <p>The DTD defines three structural elements: a <i>score</i>
     containing several <i>chords</i>, a <i>chord</i> containing
     several <i>notes</i> and a <i>note</i> consisting of data. Both
     chord and note have three attributes: <i>id</i>, <i>name</i> and
     <i>duration</i>. The first two are optional and the last has a
     default value of 4.  Note the difference between the <i>id</i>
     attribute, which has to be a unique identifier, and the
     <i>name</i> attribute, which can be an arbitrary string.
     
     
    <h4>Style sheet</h4>
    
    <p>Playing the document by a web browser, does not
     necessarily involve displaying the data visually.
     The simple style sheet shown below
     simply collects notes and chords and plays 
     them by using the <i>play</i> command.
     Note that most of the timing relations are implicit
     in the document. For example, the notes within a single chord are
     to be played in parallel, and the chords themselves are to be
     played sequentially. However, this is not explicitly defined by
     the document instance or DTD and can only be intuitively derived
     from the element names.  Even in the style sheet below, these
     timing relations remain implicit.

     <!-- \codeinput{music.style} -->

     The procedures corresponding to the open and close tags build a
     string representation of the score. At the opening of the score
     element the string is initialized with a command defining the tempo
     at 120 beats per minute. During the parsing process the string is
     extended with the parsed notes. After the last chord has been
     parsed the resulting string is:
     <tt>t120 (g&lt;b&lt;f&lt; r)(g&lt;b&lt;f&lt; r)(c&lt;e&lt;g&lt; r)</tt>.
     This string is played after the score end tag has
     been encountered. The Tcl command <i>play</i> used to play the notes is
     provided by the <i>hymne</i> extension of <i>hush</i>
     [_OssEl94,OssEl95].


