<!DOCTYPE DOCUMENT PUBLIC "-//DejaVu//DTD DVDOC//EN">
 <html>
   <title>Animating the Web -- an SGML-based approach (section 3)</title>
   
    <h1>Viewing the Web -- the Web Components</h1>
    <!-- \label{Design} -->
   
    <p>The <i>hush</i> class library offers two ways to connect
     applications with the Web.  Pages on the Web can be used as part
     of an application by employing the <i>web</i> component, for
     example for online help. On the other hand, these pages may
     contain embedded <i>applets</i>, defined by the application.
     Since most <i>hush</i> applications have a script interface as
     well, the application itself may be (recursively) embedded within
     such HTML documents [_DoornEl95].
     
    <h2>Requirements of the Web Components</h2>

    <p>The design of the Web-related components of the <i>hush</i>
     library has been influenced by requirements such as extensibility
     with new data formats, data transfer protocols and support for
     active documents.
     
    <h4>Data formats</h4>

    <p>
     The world of hyper- and multimedia abounds with data formats.
     For each type of media, such as text, sound and video, a few data
     formats play a prominent role.  A Web browser should provide the
     means to show these data formats.  However, viewer applications,
     separate from the browser, have some major drawbacks. For
     example, users have to use different interfaces and following
     links specified in a separate document is more difficult because
     of the loose coupling between the viewer application and the main
     browser.

    <p>A more elegant solution is to provide mechanisms to include the
     viewers into the browser itself, since it allows for a more
     uniform interface and for inlining the display of information, so
     that the user does not have to switch to another toplevel window.
     For example, it is possible to insert video fragments into an
     HTML-page or even to insert an HTML-page into another HTML-page.

    <p>The ability to support multiple data formats has been a major
     influence on the design of our Web components.  At the moment our
     browser supports HTML, SGML, VRML, video (including MPEG,
     Quicktime, AVI) and a variety of image types.  Developing new
     viewers and including them as integrated components into the
     framework is possible without much effort.
     
    <h4>Data transfer protocols</h4>
    
    <p>The dominant communication protocol of the Web is currently
     HTTP, but new or enhanced communication and network transfer
     protocols will undoubtedly be needed. Examples include real-time
     protocols to incorporate continuous multimedia streams or
     protocols supporting safe data transfer.  The DejaVu framework
     provides a testbed to experiment with these new protocols by
     plugging them into a Web environment.

    <h4>Active documents -- client-side computing</h4>

    <p>A simple but powerful approach to active documents is to add
     the ability to run applets (mini-applications).  At the moment
     our browser can execute Tcl-scripts (applets) that can be
     inserted into HTML or SGML documents, and the design is
     sufficiently open to add support for Java or Python applets as
     well.
     
     
    <h2>Architecture of the Web Components</h2>
    
    <p>To satisfy our requirements, we needed a flexible and open
     design of the Web components.  To achieve such flexibility and
     extensibility, we made use of the idioms mentioned in section
     <ref>dejavu</ref>. Recall that the virtual self-reference idiom
     is used to realize nested components and the dynamic
     role-switching idiom allows for multiple modes, for example to
     display a variety of document formats. 
     <slide id=fig1 cap=Overview>
     <control><p align=center>
     <img src=local/overview.gif></control> <!--
     \begin{figure}[!ht] \begin{center}\epsfig{file=local/overview.eps,
     width=0.7\linewidth} \caption{Overview of the Web components}
     \label{fig:overview}\end{center} \end{figure} -->
     </slide>
     Figure <ref>fig:overview</ref> shows an overview of the basic
     Web-related components of the <i>hush</i> library.  The
     <i>browser</i> provides the toplevel user interface for all Web
     components, including a viewer, a scrollbar, navigation buttons
     (back, forward, home, reload) and an entry box to enter URLs.
     The <i>client</i>, <i>web</i> and <i>MIMEviewer</i> components
     form the conceptual base of our approach of connecting to the
     Web:
    <slide id=view-components cap=Components>
    <ul>
     <li><i>client</i> -- the interface to the World Wide Web,
      supporting several protocols
     <li><i>viewer</i> -- a widget to display several MIME types, such
      as HTML and VRML
     <li><i>web</i> -- an extension of the <i>MIMEviewer</i> with
      connection to the Web, history of followed links and caching
    </ul>
    </slide>
    Each component can be extended in a simple way.
    
    <h4>The client component</h4>
    
    <p>The <i>client</i> component builds the bridge between the local
     <i>web</i> widget and the World Wide Web by providing an abstract
     and uniform interface to network (file) access and transport
     protocols. Figure <ref>fig:client</ref> shows the design of the
     <i>client</i> component.  
     <slide id=view-3 cap="The client component">
     <control><hr><p align=center>
     <img src=local/client.gif><hr></control> <!-- \begin{figure}[!ht]
     \begin{center}\epsfig{file=local/client.eps, width=0.7\linewidth}
     \caption{The Client component} \label{fig:client}\end{center}
     \end{figure} --> 
     </slide>
     To be able to add specific clients for new
     protocols, the dynamic role-switching idiom is used to implement
     the <i>client</i> class. The <i>client</i> implicitly changes its
     role, depending on the protocol required.  The <i>web</i> widget
     only knows about the (abstract) <i>client</i> class while the
     actual functionality is implemented in several concrete
     <i>client</i> classes, one per protocol.
     
    <p>
     When the <i>client</i> gets the instruction to retrieve a URL,
     it inspects the URL and chooses the right protocol. In other
     words, it changes its role to the appropriate protocol
     implementation. The addition of new protocols can be done
     without changing the <i>web</i> widget.
     
    <h4>The viewer component</h4>
    
    <p>In the realization of the <i>MIMEviewer</i> components we have
     employed the dynamic role-switching idiom in the same way as in
     the implementation of the <i>client</i> component. The
     <i>MIMEviewer</i> class offers a uniform interface to the
     different viewers (see figure <ref>fig:viewer</ref>).
     <slide id=view-2 cap=Viewers>
     <control><p align=center><img src=local/viewer.gif></control>
     <!-- \begin{figure}[!ht] \begin{center}\epsfig{file=local/viewer.eps,
     width=0.7\linewidth} \caption{The MIMEviewer component}
     \label{fig:viewer}\end{center} \end{figure} -->
     </slide>
     The user creates
     a <i>MIMEviewer</i> object and tells which role it should play
     (e.g. SGMLviewer or VRMLviewer). This role can be changed during
     the lifetime of a single <i>MIMEviewer</i> object by calling a
     method to change its role.  A browser typically uses only one
     <i>MIMEviewer</i> that changes its role according to the type of
     data that should be displayed.

    <p>Since the <i>MIMEviewer</i> provides no network functionality
     at all, it generates an event whenever it needs to retrieve data
     pointed to by a URL. Such events are generated as a response to
     user interaction (e.g. clicking an anchor) or to fetch inline
     data during the parsing process. These events (realized by the
     hidden <i>urlevent</i> class) are typically handled by a
     <i>web</i> component.
     
    <h4>The web component</h4>

    <p>
     A central role in our approach is played by the <i>web</i>
     widget.  It combines the functionality of the <i>MIMEviewer</i>
     and the <i>client</i> components. It is able to follow links by
     retrieving data via several protocols, a task delegated to its
     <i>client</i> component. Additionally, the <i>web</i> component
     adds a history and caching mechanism to the <i>MIMEviewer</i>.
     The <i>web</i> component's behavior is similar to the standard
     widgets of the framework, and can be conveniently used as a part
     of an application's GUI.  Because the <i>web</i> widget has both
     a C++ class interface and a script interface, it is easy to
     create, or extend, applications with Web functionality.  Note
     that the SGML viewer already allows for a rich variety of data
     formats, possibly extending HTML.
