<document>
<!--
\title{Midi Feature Detection in ACOI\\ combining grammars and logic}
\author{Anton Eliens
and Martin Kersten\\
{\footnotesize \sl CWI,}
{\footnotesize \sl Kruislaan ...}\\ 
{\footnotesize \sl email: \{eliens,mk\}@cwi.nl}
} \date{}\maketitle
\def\thefootnote{}
\setcounter{page}{1}

\date{}\maketitle
\def\thefootnote{}
\setcounter{page}{1}
-->

<control>

<slide id=title mode=hide>
<center>
<h1>
<a href=@talk-paper.html target=_top>Musical feature detection in ACOI</a>
</h1>
<h2>
<i>combining grammars and logic</i>
</h2>
</center>

<center>
Anton Eliens &amp;
Martin Kersten
<br>
CWI
email: eliens@cs.vu.nl, M.Kersten@cwi.nl
</center>
</slide>
</control>


<latex><document>
<abstract>
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


The indexing and retrieval problem for musical data
is basically unsolved.
Existing systems rely on hand-crafted solutions,
geared towards a particular group of users,
such as for example composers of film music.
This paper describes an approach to the detection
of musical features which combines the use of feature
grammars (as developed in the ACOI framework)
to describe the structural properties of musical data,
and logic to extract content-related properties of
that data. In this way we hope to be able to improve the
indexing and retrieval of musical information.
This work is part of the ACOI project, which provides
a large-scale experimentation platform to study the indexing
and retrieval of multimedia objects.
Central in the design of the ACOI architecture is
the <em>feature detector engine</em> model (FDE),
which allows for employing black-box feature
detectors for arbitrary multimedia objects.
Our research indicates how to extend feature detector grammars
with content-related knowledge in a transparent manner.
It moreover provides a concrete implementation of feature
detectors for MIDI encoded material.
</abstract>
</document>
</latex>
<p>
<document>
@* Introduction
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


With the growth of information spaces
the retrieval of information, based on indexing schemes,
becomes increasingly important.
As it comes to information embedded in multimedia objects,
we must observe that progress in automatic indexing
is rather limited.
Obviously, taking the World Wide Web as our information space,
manual classification schemes do not suffice, simply
because they do not scale.
<p>
The ACOI project [_ACOI] provides a large scale
experimentation platform to study issues in
the indexing and retrieval of multimedia objects.
The resulting ACOI framework is intended to
provide a sound model for indexing and retrieval based
on feature detection, as well
as an effective system architecture accomodating a variety
of algorithms to extract relevant properties
from multimedia objects.
The ACOI approach to multimedia feature detection
is based on the deployment of high-level feature grammars
augmented with media-specific feature detectors
to describe the structural properties of multimedia objects.
The structured objects that correspond to the parse trees
may be used for the retrieval of information.
Key challenges here are to find sufficiently selective properties
for a broad range of multimedia objects
and realistic similarity measures for the retrieval
of information.
<p>
In this paper, we will look at the indexing and retrieval
of musical fragments.
This study is primarily aimed at establishing the
architectural requirements for the detection of musical features
and to indicate directions for exploring the 
inherently difficult problem of finding proper discriminating
features and similarity measures in the musical domain.
In this study we have limited ourselves to the analysis
of music encoded in MIDI, to avoid the technical difficulties
involved in extracting basic musical properties
from raw sound material.
Currently we have a simple running prototype for
extracting higher level features from MIDI files.
In our approach to musical feature detection, 
we extended the basic grammar-based ACOI framework
with an embedded logic component to facilitate
the formulation of predicates and constraints over
the musical structure obtained from the input.

:@ Structure
The structure of this paper is as follows.
First we will look at the ACOI framework and
the interaction of components supporting grammar-based
feature detection.
Then we will describe a grammar for musical fragments 
and a corresponding feature detector for the extraction
of features from a MIDI file or MIDI fragment.
Finally, we will discuss the options for processing
queries and draw some conclusions.

</document>


<latex>
<document>
@* The ACOI framework
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


<slide id=acoi type=fig cap="The extended ACOI architecture">
<center>
<image name=local/acoi/acoi>
</center>
</slide>


The ACOI framework is intended to accomodate
a broad spectrum of classification schemes,
manual as well as (semi) automatic, for the indexing
and retrieval of multimedia objects.
What is stored are not the actual multimedia objects
themselves, but structural descriptions of
these objects (including their location) that may be used for
retrieval.
<p>
The ACOI model is based on the assumption that
indexing an arbitrary multimedia object
is equivalent to deriving a grammatical structure
that provides a namespace to reason about the object
and to access its components.
However there is an important difference with
ordinary parsing in that the lexical and grammatical items
corresponding to the components of the multimedia object
must be created dynamically by inspecting the actual object.
Moreover, in general, there is not a fixed sequence
of lexicals as in the case of natural or formal languages.
To allow for the dynamic creation of lexical
and grammatical items the ACOI framework supports both
<em>black-box</em> ans <em>white-box</em> (feature)
detectors.
Black-box detectors are algorithms, usually 
developed by a specialist in the media domain,
that extract properties from the media object 
by some form of analysis.
White-box detectors, on the other hand,
are created by defining logical
or mathematical expressions over the grammar itself.
In this paper we will focus on black-box detectors only.
<p>
As an example, look at the (simple) feature grammar below,
specifying the structure of a hypothetical community.

<slide type=hide cap="A sample grammar">
<program>
detector world; /// finds the name of the world
detector people; /// checks name, eliminates institutes
detector company; /// looks if there are at least two persons

atom str name;

community: world people company;

world: name;
people: person*;

person: name;
</program>
</slide>

A <em>community</em> consists of people,
and is a community only if it allows for the people
to be in eachothers company.
<p>
A community has a name.
The actual purpose of this grammar is to select the persons
that belong to a particular community from the input,
which consists of names of potential community members.
Note that the grammar specifies three detectors.
These detectors correspond to functions that are invoked
when expanding the corresponding non-terminal in the grammar.
An example of a detector function is the
<em>personDetector</em> function partially specified below.

<slide type=hide cap="A person detector">
<program>
int personDetector(tree *pt, list *tks ){
...
q = query_query("kit=pl src=check.pl");

while (t = next_token(tks)) {
      sprintf(buf,"person(%s)",t);
      query_eval(q,buf);
      if (query_result(q,0)) ///// put name(person) on tokenstream
                putAtom(tks,"name",t);
      }
...
}
</program>
</slide>

The <em>personDetector</em> function checks for each token
on the input tokenstream $tks$ whether the token corresponds
to the name of a person belonging to the community.
The check is performed by an embedded logic component
that contains the information needed to establish whether
a person is a member of the community.
Note that the query for a single token
may result in adding multiple names to the token stream.
<p>
The <em>companyDetector</em> differs from the <em>personDetector</em>
in that it needs to inspect the complete parse tree
to see whether the (implicit) <em>company</em> predicate is satisfied.
<p>
When parsing succeeds and the <em>company</em> predicate is satisfied
a given input may result in a sequence
of updates of the underlying database, as illustrated below.


<slide type=hide cap="Database updates">
<display>
V0 := newoid();
V1 := newoid();
  community_world.insert(oid(V0),oid(V1));
    world_name.insert(oid(V1),"casa");
  community_people.insert(oid(V0),oid(V1));
V2 := newoid();
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"alice");
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"sebastiaan");
    ...
</display>
</slide>

Evidently, the updates correspond to assigning 
appropriate values to the attributes of a structured object,
relecting the properties of the given community.

<p>
The overall architecture of the ACOI framework is depicted in \sliref{acoi}.
Taking a feature grammar specification, such as the simple
community grammar, as a point of reference, we see
that it is related to an actual feature detector
(possibly containing an embedded logic component)
taht is invoked by the Feature detector Engine
when an appropriate media object is presented for indexing.
The feature grammar and its associated detector
further result in updating respectively the data schemas
and tye actual information stored in the (Monet) database.
The Monet database, which underlies the ACOI framework,
is a customizable, high-performance, main-memory database
developed at the CWI and the University of Amsterdam [_MONET].
<p>
At the user end, a feature grammar is related to
a <em>View</em>, <em>Query</em>
and <em>Report</em> component,
that respectively allow for inspecting a feature grammar,
expressing a query, and delivering a response
to a query.
Some examples of these components are currently implemented as applets
in Java 1.1 with Swing. See [_ACOI].
<p>
:@ Formal specification
Formally, a feature grammar $G$ may be defined
as $G = (V,T,P,S)$, where $V$ is a collection of
variables or non-terminals, 
$T$ a collection of terminals,
$P$ a collection of productions of the form $V -> (V \union T)$
and $S$ a start symbol.
A token sequence $ts$ belongs to the
the language $L(G)$ if $S -*-> ts$.
Sentential token sequences, those belonging to
$L(G)$ or its sublanguages $L(G_v) = (V_v,T_v,P_v,v)$
for $v \e (T \union V)$, correspond to a complex object
$C_v$,, which is the object corresponding to the parse tree 
for $v$, as illustrated in the community example.
The parse tree defines a hierarchical structure
that may be used to access and manipulate the components
of the multimedia object subjected to the detector.

</document>
<document>

@* The anatomy of a MIDI feature detector
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


Automatic indexing for musical data
is an inherently difficult problem.
Existing systems rely on hand-crafted solutions,
geared towards a particular group of users,
such as for example composers of film music [_MM].
In this section, we will look at a simple feature detector for
MIDI encoded musical data.
It provides a skeleton for future experimentation.

<slide id=midi-structure type=fig cap="MIDI features">
<image name=local/acoi/grammar>
</slide>

The hierarchical information structure that we consider
is depicted in \sliref{midi-structure}.
It contains only a limited number of basic
properties and must be extended with information
along the lines of the musical ontology proposed
in [_AI].
However, the detector presented here provides
a skeleton solution that accomodates an extensio with
arbitrary predicates over the musical structure in a transparent manner.
<p>
The grammar given below corresponds in an obvious way with
the structure depicted in \sliref{mid-structure}.

<slide type=hide cap="A simple feature grammar for MIDI files">
<display>

detector song; ### to get the filename
detector lyrics; ### extracts lyrics
detector melody; ### extracts melody
detector check;  ### to walk the tree

atom str name;
atom str text;
atom str note;  

midi: song;

song: file lyrics melody check;

file: name;

lyrics: text*;
melody: note*;

</display>
</slide>

The start symbol is a <em>song</em>.
The detector that is associated with <em>song</em>
reads in a midifile.
The musical information contained in the midifile is then stored
a a collection of Prolog facts.
This translation is very direct.
In effect the midifile header information is stored, and
events are recorded as facts, as illustrated below
for a <em>note_on</em> and <em>note_off</em> event.
<slide mode=hide id=midi-event>
<display>
event('kortjakje',2,time=384, note_on:[chan=2,pitch=72,vol=111]).
event('kortjakje',2,time=768, note_off:[chan=2,pitch=72,vol=100]).
</display>
</slide>

After translating the midifile into a Prolog format, the other detectors
will be invoked, that is the <em>composer</em>,
<em>lyrics</em> and <em>melody</em> detector,
to extract the information related to these properties.

<slide id=midi-processing type=fig cap="Processing MIDI file">
<image name=local/acoi/process>
</slide>

The actual processing is depicted in \sliref{midi-processing}.
The input is a midifile.
As indicated in the top line, the midifile itself may
be generated from a <em>lilypond</em> file.
Lilypond is a \LaTeX-like formatting language for
musical scores that also supports the generation of MIDI [_Lily].
As indicated on the bottom line, processing a midifile
results in a collection of features as well as in a midifile
and lilypond file.
The (result) midifile contains an extract
of the original (input) midifile and the lilypond file
contains a score for this extract,
that may be presented to the (end) user as the result of a query.
This setup allows us to verify whether our extract or
abstraction of the origanal musical structure is effective,
simply by comparing the input (midi or lilypond)
musical structure with the output (midi or lilypond) extract. 
<p>
To extract relevant fragments of the melody we
use the melody detector, of which a partial listing is given below.

<slide type=hide cap="The melody detector">
<program>
int melodyDetector(tree *pt, list *tks ){
char buf[1024]; char* _result;
void* q = _query;
int idq = 0; 

  idq = query_eval(q,"X:melody(X)");
  while ((_result = query_result(q,idq)) ) {
         printf("note: %s\n",_result);
         putAtom(tks,"note",_result);
         }
  return SUCCESS;
} 
</program>
</slide>

The embedded logic component is given the
query {P X:melody(X) P}, which results in the notes
that constitute the (relevant fragment of the) melody.
These notes are then added to the tokenstream.
A similar detector is available for the lyrics.
<p>
Parsing a given midifile, for example <em>kortjakje.mid</em>,
results in updating the Monet database as indicated below.

<ignore>
<slide mode=hide id=token-api cap="The tokenstream API">
<display>
given type AtomName, AtomValue;

interface tokenstream {
	void putAtom(in AtomName, in AtomValue);
};
</display>
</slide>
</ignore>

<slide type=hide cap="Update of Monet database">
<program>
V1 := newoid();
  midi_song.insert(oid(V0),oid(V1));
V2 := newoid();
    song_file.insert(oid(V1),oid(V2));
      file_name.insert(oid(V2),"kortjakje");
    song_lyrics.insert(oid(V1),oid(V2));
      lyrics_text.insert(oid(V2),"e");
      lyrics_text.insert(oid(V2),"per-");
      lyrics_text.insert(oid(V2),"sonne");
      lyrics_text.insert(oid(V2),"Moi");
      lyrics_text.insert(oid(V2),"je");
      lyrics_text.insert(oid(V2),"dis");
      lyrics_text.insert(oid(V2),"que");
      lyrics_text.insert(oid(V2),"les");
      lyrics_text.insert(oid(V2),"bon-");
      lyrics_text.insert(oid(V2),"bons");
      lyrics_text.insert(oid(V2),"Val-");
      lyrics_text.insert(oid(V2),"ent");
    song_melody.insert(oid(V1),oid(V2));
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"a-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"g-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"f-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"d-2");
      melody_note.insert(oid(V2),"e-2");
      melody_note.insert(oid(V2),"c-2");

</program>

</slide>

The updates clearly reflect the structure of
the musical information object that corresponds to
the properties defined in the grammar.

:@ Implementation status
Currently, we have a running prototype of the MIDI
feature detector.
It uses an adapted version of public domain
midi processing software.
The embedded logic component is part of the <em>hush</em>
framework. It uses an object extension of Prolog
that allows for the definition of native objects
to interface with the midi processing software.
A description of the logic component is beyond the
scope of this paper, but will be provided in [_OO].
The logic component, however, allows for
the definition of arbitrary predicates to extract 
the musical information, such as the melody and the lyrics.
As stated before,
the current detector may be regarded as
a skeleton implementation that provides
the basis for further experimentation.

</document>
<document>

@* Queries -- the user interface
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


Assuming that we have an adequate solution
for indexing musical data, we need to define how end users may access these data, 
that is search for musical objects in the information space
represented by the database, for the ACOI project the World Wide Web.

<slide id=midi-keyboard type=fig cap="Keyboard interface">
<image name=local/jamming/keyboard>
</slide>

For a limited category of users, those with some musical skills,
a direct interface such as a keyboard
or a score editor, as provided by the <em>hush</em> framework
[_Jamming], might provide a suitable interface for
quierying the musical database.
Yet, for many others, a textual description, or a form-based
query will be more appropriate.

<slide type=fig cap="User Query Processing">
<image name=local/acoi/query>
</slide>

In processing a query, we may in some cases
derive a partial melody or rhythmic structure from the query,
as well as some additional features or criteria.
As explained in the previous section, the output of indexing midifiles
consists of both information concerning features
as well as a musical rendering of some of these features.
These features can be used to match against
the criteria formulated in the query.
The musical renderings, which include a partial score,
may be presented to the user in response to a query,
to establish whether the result is acceptable.

<ignore>
<slide mode=hide cap="Query interface">
<ul>
<li> fragment of a melody
<li> fragment of the lyrics
<li> other information
</ul>
</slide>
</ignore>




</document>
<document>
@* Conclusions
<c>
<p align=right><a href=guide.html>Reader's Guide</a>
<hr>
<a href=abstract.html>abstract</a>
<a href=0.html>intro</a>
<a href=acoi.html>ACOI</a> 

<a href=detector.html>midi detector</a> 
<a href=user.html>queries</a>
<a href=conclusions.html>conclusions</a>
<hr>
</c>


This paper described an approach to the detection
of musical features which combines the use of feature
grammars (as developed in the ACOI framework)
to describe the structural properties of musical data,
and logic to extract content-related properties of
that data. In this way we hope to be able to improve the
indexing and retrieval of musical information.
<p>
Currently, we have a running prototypes that provides
a skeleton solution for indexing midifiles.
The next step in our research will consist of
creating suitable ways for querying the musical database.
We will also have to explore how to present
a possibly large collection of matches,
and how to assist the end user in refining a
query as to obtain the desired result.

</document>

</latex>

</document>
