<h1>Process Simulation</h1>
<include file=navigate>
The BPSIM library is an extension of the simulation library SIM [_SIM].
SIM is a C++ library offering classes supporting
discrete event simulation, based on standard simulation techniques
[_Watkins93].

In discrete event simulation,
the components of the model consists of events,
which are
activated at certain points in time and in this way affect the overall state
of the system.
The simulation library consists of the following classes:
<ul>
<li><i>simulation</i> -- the scheduler
<li><i>event</i> -- representing the events
<li><i>entity</i> -- process consisting of events
<li><i>generator</i> -- random distributions 
<li><i>resource</i> -- to model passive objects
<li><i>queue</i> -- to hold waiting events 
<li><i>histogram</i> -- to plot the results 
<li><i>analysis</i> -- for statistical analyses
</ul>
The SIM library is integrated with the <i>hush</i> library,
which may be used for defining a script interface to the simulation
package, for developing a graphical user interface
and for visualising simulation models.


@ The library BPSIM
The classes provided by the business process simulation
library BPSIM reflect the entities provided by the
logistics-based business modelling method LBM.
On a somewhat more abstract level,
a business process simulation  consists of <i>data</i>,
flowing through the process and different handlers -- 
<i>datahandlers</i> -- performing some action on the data,
such as transportation or specific operations.
<p>
Accordingly, BPSIM provides two  base classes underlying
the classes corresponding to LBM entities:
<ul>
<li> <i>data</i> -- which represents the product 
or case, i.e.
the information, that flows through the process. 
<li> <i>datahandler</i>  -- which is the basis 
for all classes that
handle information.
</ul>
Note that LBM contains no symbol for data as it only
depicts the objects that handle the information.

The following classes are derived from 
the <i>datahandler</i> class:
<em>operation</em>,
<em>transport</em>,
<em>waitqueue</em>,
<em>choice</em>,
<em>archive</em>, and
<em>external agent</em>,
which realize the corresponding entities in LBM.

<p>
An
<i>operation</i> takes time, and is executed by an employee. Sometimes 
an operation results in more than one outgoing dataflow,
for instance when it 
issues a request for additional information from a different 
department.
<p>
A <em>waitqueue</em> functions as a regular queue if 
it has one incoming dataflow, i.e. transport.
When there are more incoming
flows, it functions as a synchronized queue.
Data from one flow is not passed on to the
next datahandler until the data from the other flow has arrived.
This happens for instance when work on a case cannot continue
until additional information has arrived.
<p>
An external <i>agent</i> can either generate data
according to some random number distribution, or take
data, process it and pass it on to the next datahandler. In the latter 
case the agent functions as a black box: we only
care how long the processing takes, not how it is exactly performed.


<p>
In addition, BPSIM offers
the classes
<i>means</i> (which can be used to model resources 
that are necessary to perform certain operations), and
<em>employee</em> (which models the 
different people that perform the operations).

<p>
As one can see, the entities <em>task</em> and 
<em>organization unit</em> 
from LBM have no specific counterpart in BPSIM.
The reason for this is that having no other <i>datahandler</i>
between two operations already implies those
operations belong to the same task. Consequently they are
executed with no time in between and by the same employee. 
Also, the fact 
that tasks are executed in
different <em>organization units</em> does not add any 
information that changes
the behaviour of the simulation. If it takes time to transport
information between different units, 
then that time can be represented by 
the object of class <em>transport</em> between those units. 
<p>
For the gathering and analyzing of results, the SIM classes 
<em>histogram</em> and
<em>analysis</em> are used.
The class <em>agent</em>, for example, 
can be given a histogram to track the lead time
of data it has generated. 

@ A script interface for BPSIM
Employing the facilities of the <i>hush</i> library
a script interface has been defined for BPSIM
that allows the user, that is the designer of business models,
to construct and run business simulation models
with a short turn around time.
<p>
Each class in the BPSIM library corresponds to
a command in the script language.
As illustrated in the next section,
script commands result in a graphic
representation of model, which may be displayed
in a Web page as an applet.
<include file=navigate>
