<html>
<body bgcolor="#FFFFFF" text="#000000">
<!--\appendix-->
<h1><a name=Appendix>Appendix: The <i>hush</i> widget classes</h1>

<include file=navigate>

<control>
[-<a href=conclusions.html#Conclusions><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=hush.html#top>></a>-]
</control>


The <i>hush</i> widget class library
encapsulates the standard Tk widgets.
In addition, a hypertext widget is offered.
The widget classes are organized as a tree,
with the class <i>widget</i> at the root.



%%FIG: f-widget

<slide id=widgets cap="Widget classes">
<a name=Classes>
<img src=local/figures/f-widget.gif>
<!-- \vspace{0cm}\hspace*{0cm}\figbox{figures/f-widget} -->
</slide>




Each concrete widget class offers the functionality
supported by the (abstract) <i>widget</i> class
and may in addition define functions
specific to the particular widget class.
The member function for a widget class
have usually a straightforward correspondence
with the command interface defined by the
Tcl/Tk toolkit.
See <cite>Ousterhout94</cite> for the most recent description.


Each widget class specifies two constructors,
one with only a path and one which allows
both for a widget and a path.
In the latter case, the actual path consists
of the concatenation of the path of
the widget and the path specified by the
string parameter.
For the concrete widget classes, no
widget will be created when the <i>options</i> parameter
is zero.
This convention is adopted to allow composite
widgets to inherit from the standard widgets,
yet define their own components.

In addition, each widget class has
a destructor, which is omitted for brevity.
The destructor may be used to reclaim the storage for a
widget object.
To remove a widget from the screen, the
function <i>widget::destroy</i> must be used.



<h4><a name=scale>The <i>scale</i> class</h4>
The <i>scale</i> widget may be used to obtain numerical
input from the user.

<slide id=scale-widget cap="The <i>scale</i> class">
<program>
  interface scale : widget { 

    scale(char* p, char* options = "");
    scale(widget* w, char* p, char* options = "");

    void text(char* s);                        // text to display
    void from(int n);                          // begin value
    void to(int n);                            // end value
    int get();                                 // gets the value
    void set(int v);                           // sets the value

  protected:
    install(binding*, char* args);
  };
</program>
</slide>

When a handler is attached to a <i>scale</i>
it is called when the user releases the
slider.
The value of the <i>scale</i> is passed as an additional
parameter when the handler is invoked.
The default binding for the scale is the <i>ButtonRelease</i> event.




<h4><a name=message>The <i>message</i> class</h4>
The <i>message</i> widget may be used to
display a message on the screen.

<slide id=message-widget cap="The <i>message</i> class">
<program>
  interface message : widget { 

    message(char* p, char* options = "" );
    message(widget* w, char* p, char* options = "" );
    
    void text(char* s);                               // the text
  };
</program>
</slide>

The <i>message</i> class does not define default bindings,
but the user is free to associate events to
a message widget by employing <i>widget::bind</i>.



<h4><a name=button>The <i>button</i> class</h4> 
Buttons come in a number of varieties,
such as ordinary (push) buttons, that simply invoke
an action, checkbuttons, that toggle between
an on and off state, and radiobuttons, that
may be used to constrain buttons to
allow the selection of only a single alternative.
Checkbuttons and radiobuttons are implemented as
subclasses of the class <i>button</i>, and will not
be further discussed here.

<slide id=button-widget cap="The <i>button</i> class">
<program>
  interface button : widget { 

    button(char* p, char* options = "");
    button(widget* w, char* p, char* options = "");
    
    void text(char* s);                    // text to display
    void bitmap(char* s);                  // to display a bitmap
    
    void state(char *s);            // to change the buttons state
    
    void flash();
    char* invoke();

  protected:
    install(binding*,char* args);           // default binding
  };
</program>
</slide>

In addition to the constructors, which have the same
format for each widget class,
the <i>button</i> class offers the function $text$
to define the text displayed by the button and the
function <i>bitmap</i>, which takes as argument
the name of a file containing a bitmap,
to have a bitmap displayed instead.
The function <i>state</i> may be used to change the state of the button.
Legal arguments are either <i>normal</i>, $active$ or $disabled$. 
Further, the <i>button</i> class defines the function
<i>flash</i> and <i>invoke</i> that
result respectively in flashing the button and in invoking
the action associated with the button by means of
the <i>widget::handler</i> function.
(Note that <i>button::install</i> is defined, albeit protected.)



<h4><a name=menubutton>The <i>menubutton</i>  class</h4>
The <i>menubutton</i> is a specialization of the
<i>button</i> widget.
It allows for attaching a menu
that will be displayed
when pressing the button.


<slide id=menubutton cap="The <i>menubutton</i>  class">
<program>
  interface menubutton : button { 
    
    menubutton(char* p, char* options = "");
    menubutton(widget* w, char* p, char* options = "");
    
    void menu(char* s);                    // to attach a menu
    void menu(class menu* m);
  };
</program>
</slide>

The <i>menubutton</i> must be used to pack
menus in a <i>menubar</i>.



<h4><a name=menu>The <i>menu</i> class</h4>
A menu consists of a number of button-like
entries, each associated with an action.
A menu entry may also consist of another menu,
that pops up whenever the entry is selected.


<slide id=menu cap="The <i>menu</i> class">
<program>
  interface menu : widget { 
    
    menu(char* p, char* options = "");
    menu(widget* w, char* p, char* options = "");
    
    menu* add(char* s, char* options = "");
    
    menu* entry(char* s, char* args ="", char* options="");
    menu* entry(char* s, binding* ac, char* args="", char* opts="");
    
    menu* cascade(char* s, char* m, char* options = "");
    menu* cascade(char* s, menu* m, char* options = "");
    
    char* entryconfigure(int i, char* options);
    
    int index(char *s);
    int active();                    // returns active index
    
    void del(int i);                 // delete entry with index i
    void del(char* s);               // delete entry with tag s 
    
    char* invoke(int i);             // invoke entry with index i
    char* invoke(char *s );          // invoke entry with tag s
    
    void post(int x = 500, int y = 500); 
    void unpost();

  protected:
    install(binding*, char* args);
  };
</program>
</slide>

The <i>add</i> function is included to allow arbitrary entries
(as defined by Tk) to be added.
We restrict ourselves to simple command and cascade entries.

The <i>entry</i> function
(that is used for adding simple command entries)
may explicitly be given a <i>binding</i> to
be associated with the entry.
Alternatively, if no binding is specified,
the default handler binding installed
by invoking <i>widget::bind</i> will be used.
The string used as a label for the entries
(the first parameter of <i>entry</i>)
will be given as a parameter to the action invoked 
when selecting the entry.
The string given in the <i>args</i> parameter
will be added to the actual parameters for the action invoked.

The <i>cascade</i> function may either be given
a <i>menu</i> or a string,
containing the pathname of the menu.
In any case the cascaded menu must be a child
of the original menu.

The function <i>index</i> returns the integer index
associated with the string describing the entry.
The function <i>active</i> may be used to inquire
which entry has been selected.

Entries may be deleted using the function <i>del</i>
and invoked by using <i>invoke</i>.
For both functions, the entry may be indicated
by its numerical index or a string.
Menus are toplevel widgets, they are mapped
to the screen either by invoking the function <i>post</i>,
or by pressing the <i>menubutton</i> to which the
menu is attached.



<h4><a name=canvas>The <i>canvas</i> class</h4>
Apart from the two standard constructors,
it offers the functions <i>tag</i>, <i>tags</i> and <i>move</i>
that merely repeat the functions offered by the <i>item</i> class,
except that <i>move</i> may also be given a tag to identify
the items to be moved.


<slide id=canvas-widget cap="The <i>canvas</i> class">
<program>
  interface canvas : widget { 

    canvas(char *p, char* options="");
    canvas(widget* w, char *p, char* options="");
    
    void tag(int id, char* tag);
    char* tags(int id);
    
    void move(int id, int x, int y);
    void move(char* id, int x, int y);
    
    item bitmap(int x1, int y1, char* bitmap, char* options="");
    item line(int x1, int y1, int x2, int y2, char* options="");
    item line(char* linespec, char* options="");
    item circle(int x1, int y1, int rad, char* options="");
    item oval(int x1, int y1, int x2, int y2, char* options="");
    item polygon(char* linespec, char* options="");
    item rectangle(int x1, int y1, int x2, int y2, char* options="");
    item text(int x1, int y1, char* txt, char* options=""); 
    item window(int x1, int y1, char* win, char* options="");
    item window(int x1, int y1, widget* win, char* options="");
    
    item current();
    item overlapping(int x, int y);
    
    itemconfigure(int it, char* options);
    itemconfigure(char* tag, char* options);
    itembind(int it, char* s, binding* a, char* args = "" );
    itembind(char* tag, char* s, binding* a, char* args = "" );
    
    void postscript(char* file, char* options="");

  protected:
    install(binding*, char* args);
  };
</program>
</slide>


Currently, the graphic items <i>bitmap</i>, <i>line</i>,
<i>oval</i>, <i>polygon</i> and <i>rectangle</i> may created
and, in addition, <i>text</i> items and <i>window</i> items consisting
of a widget.
The function <i>overlapping</i> may be used to retrieve
the item overlapping a particular position.

In addition, the <i>canvas</i> class offers auxiliary functions
needed to support
the functionality provided by the <i>item</i> class.
The canvas may be written as Postscript to a file
with the function <i>canvas::postscript</i>.




<h4><a name=frame>The <i>frame</i> class</h4>
Frame widgets may used to combine widgets.
A frame has no functionality or bindings of its own.

<slide id=frame-widget cap="The <i>frame</i> class">
<program>
  interface frame : widget { 
    
    frame(char* p, char * options = "");
    frame(widget* w, char* p, char * options = "");
  };
</program>
</slide>

The frame widget class has the <i>toplevel</i>
and <i>menubar</i> as subclasses.

The <i>toplevel</i> widget is used when the widget
must be independently mapped to the screen.

The <i>menubar</i> widget is used as a special
frame to collect <i>button</i> widgets including <i>menubutton</i> 
widgets.



<h4><a name=scrollbar>The <i>scrollbar</i> class</h4>
The <i>scrollbar</i> allows the user to scroll
through widgets
that are only partly displayed.

<slide id=scrollbar cap="The <i>scrollbar</i> class">
<program>
  interface scrollbar : widget { 

    scrollbar(char* p, char* options = "");
    scrollbar(widget* w, char* p, char* options = "");

    void orient(char* opts="vertical");   // orientation
    xview(widget* w);                     // widget to scroll
    yview(widget* w);                     // widget to scroll
  };
</program>
</slide>

The default orientation of a <i>scrollbar</i> is vertical.
A scrollbar must be explicitly attached to a widget <i>w</i>
by calling the <i>scrollbar::yview</i> functions for
vertical scrollbars and <i>scrollbar::xview</i> for
horizontal scrollbars.
To obtain the proper geometrical layout, the
scrollbar and the widget it controls must
usually be packed in a frame.
   


<h4><a name=listbox>The <i>listbox</i> class</h4>
The <i>listbox</i> widget is used to allow the
user to select an item from a list of
alternatives.

<slide id=listbox cap="The <i>listbox</i> class">
<program>
  interface listbox : widget { 
    
    listbox(char* p, char* options = "");
    listbox(widget* w, char* p, char* options = "");
    
    void insert(char* s);
    char* get(int d);             // entry with index d
    
    void singleselect();

  protected:
    install(binding*, char* args);
  };
</program>
</slide>

The <i>listbox</i> may be filled by using <i>insert</i>.
When a handler is attached to the widget, it
is activated when the user double clicks
on an item.
The selected entry is passed as an additional parameter
to the handler.
The entry may also be obtained by either 
<i>kit::selection</i> or <i>listbox::get</i>.




<h4><a name=entry>The <i>entry</i> class</h4>
An <i>entry</i> widget may be used to display
text or allow the user to type a short
text.

<slide id=entry-widget cap="The <i>entry</i> class">
<program>
  interface entry : widget { 

    entry(char* p, char* options = "");
    entry(widget* w,char* p, char* options = "");
    
    void insert(char* s);                    // insert text
    char* get();                         // to get the text

  protected:
    install(binding*, char* args);
  };
</program>
</slide>

When a handler is attached
to the entry widget it is activated
whenever the user double clicks on the widget
or presses the return key.
The contents of the entry are added as an argument
when calling the handler.



<h4><a name=hypertext>The <i>hypertext</i> class</h4>
The <i>hypertext</i> widget may be used to display
text with embedded Tcl code.

<slide id=hypertext cap="The <i>hypertext</i> class">
<program>
  interface hypertext : widget {

    hypertext(char* p, char* options = "");
    hypertext(widget* w, char* p, char* options = "");
    
    void file(char* f);         // to read in hypertext file
  };
</program>
</slide>

Apart from the standard constructors,
it offers the function <i>file</i> to read in a hypertext file.
Such a hypertext file allows to embed widgets
in the text by inserting them in escape sequences.

<include file=navigate>
