<body bgcolor="#FFFFFF" text="#000000">
<h1>
<a name=Graphics>
<a name=Hypertext>
Graphics and hypertext
</h1>

<include file=navigate>

<control>
[-<a href=user.html#User><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=conclusions.html#Conclusions>></a>-]
</control>



The Tk toolkit offers powerful facilities
for graphics and (hyper)text. See <cite>Ousterhout93</cite>.
In this section we discuss the <i>canvas</i>
widget offered by Tk.
Instead of looking at the <i>text</i> widget
provided by Tk, we will (briefly)
look at the <i>hypertext</i> widget,
which presents an alternative approach to
defining hyperstructures.



<h2><a name=Items>The <i>item</i> class</h2>
<!--\label{hush-item}-->
<!--\label{Items}-->

The canvas widget allows the programmer to
create a number of built-in graphic items.
Items are given a numerical index when created and,
in addition, they may be given a (string) tag.
Tags allow items to be manipulated in a group-wise fashion.
To deal with items in a C++ context, the <i>hush</i> library
contains a class <i>item</i> of which the functionality
is shown below.

<slide id=item cap="The <i>item</i> interface">
<program>
  interface item { 
    operator int();                        // returns item index
    
    void configure(char* cmd);    // calls canvas::itemconfigure
    
    void tag(char* s);                      // sets tag for item
    char* tags();              // delivers tags set for the item
    void move(int x, int y);
    
    bind(char *b, handler* h, char* args = "" );
    bind(char *b, binding* ac, char* args = "" );
    
    handler(class handler* h, char* args = "" );
    handler(binding* ac, char* args = "" );
    
  protected:
    virtual install(action&,char* args="");    // default bindings
  };
</program>
</slide>

Instances of <i>item</i> may not be created directly by
the user, but instead are created by the canvas
widget.
For an item, its index may be obtained by casting
the item to <i>int</i>.
If the index does not identify an existing item,
it will be zero.
Existing items may be moved, in a relative way,
by the function <i>move</i>.

In a similar way as for widgets,
items may be associated with events,
either explicitly by using
<i>item::bind</i>, or
implicitly by using <i>item::handler</i>.
The default bindings for <i>items</i> are
identical to the default bindings
for the canvas widget, but these may be overridden
by descendant classes.

Similar as the <i>widget</i> class, the <i>item</i> class
is derived from the <i>handler</i> class.
This allows the user to define possibly compound
shapes that have their own handlers.



<h2><a name=Canvas>The <i>canvas</i> widget</h2>
<!--\label{canvas}-->

The Tk canvas widget offers powerful means for
doing structured graphics.
The <i>hush</i> class $canvas$ provides merely
a simplified interface to
the corresponding Tk widget.



As an example of the use of a canvas, consider the
definition of the <i>move_handler</i>
below.
The <i>move_handler</i> defines one of the modes of the
<i>tablet</i> and allows for moving graphical items
on the canvas.

<slide id=Move cap="Moving items on a canvas">
<a name=Move>
<program>

  class move_handler : public handler { 
  public:
    
    move_handler( canvas* cv ) : c(cv) { dragging = 0; }
    
    void press( event& e ) {     // if overlapping start dragging
    	  x = e.x(); y = e.y();     
    	  id = c->overlapping(x, y);
    	  if (id) dragging = 1;
    }
    
    void motion( event& e ) {                  // if dragging move
      if (dragging) {                
    	id.move( e.x() - x, e.y() - y );
    	x = e.x(); y = e.y();
    	}
    }
    
    void release( event&  ) { dragging = 0; }     // stop dragging
    
  protected:
    canvas* c; int dragging; item id; int x,y;
  };
</program>
</slide>


The <i>move_handler</i> class is derived from the class
<i>handler</i>.
It makes use of the <i>dispatch</i>
and <i>operator()</i> function defined for 
<i>handler</i>,
but redefines the (virtual) functions
<i>press</i>, <i>motion</i> and <i>release</i>.

When creating an instance of <i>move_handler</i>,
a pointer to the canvas must be given to the constructor.
In addition, the class has data members to record
position coordinates
and whether a particular item is being moved.
Actually moving an item occurs by pressing the (left)
mouse button on an item and dragging the item along.
When the mouse button is released, moving stops.
To identify the item, the function <i>overlapping</i>
is used.
The movement is determined by the distance
between the last recorded position and the
current position of the cursor.


In an analogous manner,
a <i>box_handler</i> may be defined,
which is used for drawing rectangles and allows
for rubberbanding.
The <i>box_handler</i> sets dragging
to true when the button is pressed
and creates a rectangle of zero width and height.
Each time the function <i>motion</i> is called,
the item created in the previous round is deleted
and a new rectangle is created:
<slide id=box-handler cap="A box handler">
<program>
  void box_handler::motion( event& e ) {   // if dragging stretch
      if (dragging) {
	id.del();
	id = c->rectangle(x,y,e.x(),e.y());  // x and y are fixed
    	}
    }
</program>
</slide>
where <i>c</i> is a pointer to the canvas
and <i>x</i> and <i>y</i> the button pointer coordinates
stored when dragging began.
For circles and lines, it suffices to replace the call
to rectangle with  a call to the appropriate
figure creation function.



%%FIG: f-help 

<slide id=help cap="Hypertext help">
<a name=Help>
<image src=local/screens/help.gif>
<!-- \vspace{0.5cm}\hspace*{0.0cm}\epsfbox{screens/help.ps} -->
</slide>



<h2><a name=Hypertext>The <i>hypertext</i> widget</h2>
<!--\label{hyper}-->


Both the Tk canvas and text widget allow to bind
actions to particular items
and hence to define dynamically what we may call
<i>hyperstructures</i>.

A different, in a way more static, approach is offered
by the <i>hypertext</i> widget originally developed by
George Howlett. %%<tt>george.howlett@att.com</tt>.


The <i>hypertext</i> widget may be used to display text
files containing embedded Tcl code.
The Tcl code must be placed between
escapes, that take the form of <code> %%</code>
for both the begin and end of the code.
A screen shot of a fragment of the on-line help
for <i>drawtool</i> is given in 
figure <a href=#Help><ref>Help</ref></a>.
Notice, that the on-line help provides a
replica of the <i>drawtool</i> application, surrounded
by text.
When looking at (again a fragment of) the hypertext
file specifying the contents of the on-line help,
given below,
you see that the <i>drawtool</i> command defined
in section <a href=#New><ref>new</ref></a>
is employed to create the embedded widget:


<slide id=rubber cap="Rubber banding">>
<program>
  Rubber banding: press the left mouse button
  and release when the rectangle is of appropriate
  size
      %%
  drawtool $this.draw 
  $this append $this.draw 
  $this.draw create rectangle 20 20 80 80
  $this.draw create rectangle 10 30 70 90
  $this.draw create oval 40 40 90 90
  $this append $this.draw 
  %%
  For additional information click on the %% 
  button $this.goto -text instruction -command end-of-text
  $this append $this.goto
  %%
  button.  Press %%
  button $this.quit -command { destroy . } -text quit -bg pink
  $this append quit
  %% to remove the window.
</program>
</slide>


When specifying the hypertext file,
widgets may be
given a pathname relative to the pathname of the
hypertext widget by using the variable <i>this</i>.
In addition the hypertext widget offers the variables
<i>thisline</i> and <i>thisfile</i> to identify the
current line number and current file name.

Any of the widgets and commands offered by Tcl/Tk or supported
by <i>hush</i> may be included in a hypertext file,
including the ones defined by the program
itself.



<include file=navigate>
