<body bgcolor="#FFFFFF" text="#000000">
<h1> <a name=Introduction>Introduction </h1>

<include file=navigate>

<control>
<ul>
<li> <a href=#available> Availability <a>,
<a href=#structure>Structure</a>
</ul>
</control>

<control>
[-<a href=hush.html#top><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=background.html#Background>></a>-]
</control>

In comparison with ordinary programming
(in C++), programming in a window environment
(in C++) introduces a number of additional
difficulties.
First of all, the programmer must become
acquainted with the various widgets
constituting the (graphical) user interface,
such as buttons, menus, messages, canvasses, etcetera.
And secondly, perhaps the most difficult
aspect of window programming, the programmer
must deal with a rather different control
structure, involving actions and callbacks
in response to events generated by the user
or the window environment, such as mouse 
movements and mouse
button manipulations.
<p>
A number of toolkits for the X11 environment
with an interface to C++ do exist already.
Well-known for example is the InterViews 
library, which offers powerful features for
defining the layout of graphical user interfaces.
See <cite>LVC89</cite>.
However, despite the elegance of its design,
InterViews is slightly cumbersome to use
and lacks a number of the features and widgets
needed for rapidly implementing a graphical
user interface.

Commercial packages for GUI (Graphical User Interface) programming in C++
are available.
The disadvantage of these packages, apart from their price,
is primarily that they do not offer
the flexibility needed in a research environment.
<p>
A rather different approach to GUI programming
has been advocated in <cite>Ousterhout91</cite>,
which describes the Tcl/Tk toolkit.
Tcl is a cshell-like (interpreted) script language that may be embedded
in C or C++.
Tk is a window and graphics toolkit based on X11,
partly implemented in Tcl and partly in C.
Tk offers numerous widgets, including a powerful
canvas and text widget.
Moreover, the Tcl scripting language allows the user to rapidly
prototype rather complex graphical user interfaces
by writing Tcl scripts.
These scripts may be executed by using <i>wish</i>,
the windowing shell interpreter that comes with Tk.
Despite being based on Tcl, the performance of Tk (and <i>wish</i>)
is comparable
with (and in some respects even better than)
C or C++ based toolkits.
<p>
The Tcl/Tk toolkit has become very
popular in a rather short period of
time.
The popularity of Tcl/Tk is partly due
to the extensibility of Tcl.
New functionality, implemented in C,
may easily be added by creating a new
version of the <i>wish</i> interpreter,
incorporating the additional commands.
Numerous extensions to Tcl/Tk and corresponding
interpreters have been made available,
including extensions offering facilities
for distributed programming 
(<i>dp</i>),
extensions offering object oriented
features
($[incr\: tcl]$)\footnote{
$[incr\: tcl]$ may be read as a paraphrase
of tcl++ in Tcl syntax.
},
and extensions offering
additional widgets such as a barchart
and hypertext widget
(<i>blt</i>).\footnote{
These extensions may be obtained from
<tt>harbor.ecn.purdue.edu</tt>.
}
<p>
The possibility of employing interpreted
code and the availability of numerous widgets
makes the Tcl/Tk toolkit (and its extensions)
an ideal vehicle for implementing user
interfaces.

However, Tcl/Tk has its drawbacks as well.
One problem, obviously,
is to manage the large number of extensions.
Ideally, there is one <i>wish</i>-like shell
unifying the various features.
Even better, one should have the opportunity
to create such a shell in a simple manner.

A second problem is that, when an application
grows, script code will 
not always allow for an optimal solution.
Generally, script code is not robust
and may be hard to maintain.
In particular, when an application contains
many components not related to the user interface,
efficiently compiled code
may be more appropriate.

For the latter problem, the obvious solution is
to employ the C API (Application Programmer Interface) offered by Tcl
and to create a new interpreter including
the functionality needed.
In a similar way, the first problem is rather easily
solved by linking the appropriate libraries
into an extended interpreter.

Nevertheless, this is easier said than done.
First of all, the C API offered for
Tcl/Tk is rather demanding for the novice
programmer and does not support a style
of programming that is recommendable from
a software engineering perspective.
Secondly, although not very difficult,
creating a new interpreter with additional
C/C++ code is somewhat cumbersome.

The <i>hush</i> library has been developed
to address the two problems mentioned.
<i>Hush</i> stands for
<i>hyper utility shell</i>.
The standard interpreter associated with
the <i>hush</i> library is a shell,
called <i>hush</i>, including 
a number of the available extensions of
Tcl/Tk and widgets developed by ourselves
(such as a <i>filechooser</i> and an MPEG video widget).
The <i>hush</i> library offers
a C++ interface to the Tcl/Tk toolkit
and its extensions.
It allows the programmer to employ
the functionality of Tcl/Tk in a C++
program.
Moreover, a program created with <i>hush</i>
is itself an interpreter extending the
<i>hush</i> interpreter (and <i>wish</i>).

The <i>hush</i> library is explicitly
intended to support the needs
of both novice and experienced
window programmers.
Its C++ class interface should suffice for
most applications,
yet it allows for employing 
Tcl script code when more is demanded.

The contribution of <i>hush</i>
with respect to the Tcl/Tk toolkit is
essentially that it provides a type-secure
solution for connecting Tcl and C++ code.
As an additional advantage, the <i>hush</i> library
allows the programmer to employ inheritance for
the development of possibly compound
widgets.
In particular, it provides the means to define
composite widgets that behave as the standard
Tk widgets.

The class structure of 
the <i>hush</i> library is reminiscent to
the class structure of the InterViews library.
In comparison with the InterViews
library, 
the widget class interfaces and 
event callbacks are significantly easier to use.
Also, the <i>hush</i> library provides
many more ready-to-use
graphical interface widgets.
However, <i>hush</i> does not offer
resolution-independent graphics and provides
no pre-defined classes for complex
interactions.

Summarizing, <i>hush</i> supports
a multi-paradigm approach to window
programming, allowing to combine
the robustness of compiled C++
code with the flexibility of
interpreted Tcl code.
As such, it offers the best of both
worlds.
Or the worst, for that matter.
<p>
The material presented here requires at least some
knowledge of C++. See for example <cite>Stroustrup91</cite>.
Some familiarity
with Tcl/Tk is also helpful. See <cite>Ousterhout94</cite>.


<a name=available>
<h4>Availability</h4>
The <i>hush</i> library has been in use for student programming 
assignments at the Vrije Universiteit for two years.
It may be obtained by anonymous ftp from
<a href=http://www.cs.vu.nl/~eliens/ftp/hush/><code>ftp.cs.vu.nl</code></a>,
directory <code>eliens/hush</code>.
You may also retrieve it via
<code>http://www.cs.vu.nl/~eliens/hush/</code>.



<a name=structure>
<h4>Structure</h4>
In section <a href=#Background><ref>Background</ref></a>,
some background information concerning Tcl/Tk is given.
Section <a href=#Structure> <ref>Structure</ref> </a>
sketches the structure of a typical 
<i>hush</i> program
and gives an overview of the <i>hush</i>
class library, including the <i>kit</i> and <i>session</i> class.
Section <a href=#Binding><ref>Binding</ref> </a>
describes how handler objects may be defined as
event callbacks.
Next, section <a href=#User> <ref>User</ref> </a>
presents a drawing tool application.
The application illustrates the use
of the various widget classes
and demonstrates how to construct compound
widgets.
In addition, it shows how a widget developed
in C++ may be made available
as a widget command to be used in scripts.
And finally, in section <a href=#Graphics><ref>Graphics</ref></a>,
we will look at the facilities offered
for structured graphics and hypertext.







<include file=navigate>

