<body bgcolor="#FFFFFF" text="#000000">
<h1><a name=User>User interface widgets</h1>
<!--\label{user}-->

<include file=navigate>

<control>
[-<a href=bind.html#Binding><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=graphics.html#Graphics>></a>-]
</control>



The Tk toolkit offers numerous built-in widgets.
The Tk widgets conform to the look-and-feel
of the OSF/Motif standard.
The <i>hush</i> C++ interface for Tk provides
for each Tk widget a class of the same name,
which supports the creation of a widget
and allows the user to access and modify it.
In addition to the standard Tk widgets,
the <i>hush</i> library includes
a number of other widgets, such as a <i>barchart</i>,
<i>hypertext</i>, and <i>photo</i> widget (created by other Tk 
adepts).
Also widgets of our own making are offered,
such as a <i>filechooser</i>, an MPEG video widget, and recently
a <a href=http://www.cs.vu.nl/~thijs/www3paper/>
World Wide Web browser</a> widget.


In this section we will look at an extension of the simple drawing tool
presented in section 
<a href=#Draw><ref>Draw</ref></a>.

<!-- %%FIG f-dtool -->

<slide id=drawtool cap="The <i>drawtool</i> interface">
<a name=Drawtool>
<img src=local/screens/drawtool.gif>
<!-- \vspace{0.8cm}\hspace*{1cm}\epsfbox{screens/drawtool.ps} -->
</slide>



The example illustrates how to use the
<i>hush</i> library widgets.
The example serves to illustrate
in particular how handlers may be attached
to widgets, either by declaration or
by inheritance, and how to construct
compound widgets.

Our approach may be considered object
oriented, in the sense that each component
of the user interface is defined by a class
derived from a widget class.

It must be pointed out beforehand, that the major difficulty
in defining compound or mega widgets is not the construction
of the components themselves, but to delegate the configuration
and binding instructions to the appropriate components. 


In section <a href=#New><ref>New</ref></a> it will be
shown how a compound widget
defined in C++ may be made to correspond
to a widget command that may be used
in a Tcl script.
Ideally, defining a new widget
includes both the definition
of a C++ class interface and
the definition of the corresponding Tcl
command.


<h4>Example -- drawtool</h4>
Our drawing tool consists of a <i>tablet</i>,
which is a canvas with scrollbars
to allow for a large size canvas
of which only a part is displayed,
a <i>menu_bar</i>, having a <i>File</i>
and an <i>Edit</i> menu, and a <i>toolbox</i>,
which is a collection of buttons for
selecting among the drawing facilities.
See figure <a href=#Drawtool><ref>Drawtool</ref></a>.
In addition, a help facility is offered.
The application class for <i>drawtool</i> is defined
as follows:


<slide id=application cap="Application class">
<program>
  
  class application : public session { 
  public:
    application(int argc, char* argv[])
				: session(argc,argv,"drawtool") {}
  
    void main() {
      widget* root = tk->root();
      frame* f = new frame(root,".frame");
  
      tablet* c = new tablet(f);                 // create tablet

      toolbox* b = new toolbox(f,c);
      menubar* m = new menu_bar(root,c,b);

      b->pack("-side left");                       // pack tablet
      c->pack("-side right");

      tk->pack(m)->pack(f);                // pack menu and frame
    }
  };
</program>
</slide>


Before the main event loop is started,
the components of the drawing tool are
created and packed to the root widget.

In addition to the <i>tablet</i>, <i>menu_bar</i>
and <i>toolbox</i>,
a <i>frame</i> widget is created to pack the
<i>toolbox</i> and <i>tablet</i> together.
This is needed to ensure that the geometrical
layout of the widget comes out right.

Each of the component widgets is given
a pointer to the root widget.
In addition, a pointer to the <i>tablet</i>
is given to the <i>toolbox</i> and a pointer to
the <i>toolbox</i> is given to the <i>menu_bar</i>.
The reason for this will become clear
when discussing the <i>toolbox</i> and
<i>menu_bar</i> in sections
<a href=#Buttons><ref>Buttons</ref></a>
and <a href=#Menus><ref>Menus</ref></a>,
respectively.
In the example, no attention will be paid to memory management.


<h2><a name=Configure>Configuring widgets</h2>
<!--\label{hush-widget}-->
<!--\label{widget}-->

Widgets are the elements a GUI is made of.
They appear as windows on the screen to display
text or graphics and may respond to events
such as motioning the mouse or pressing
a key by calling an action associated with that event.

Most often, the various widgets constituting the user
interface are (hierarchically) related to each other,
as for instance in the <i>drawtool</i> application which contains
a canvas to display graphic elements,
a button toolbox for selecting the graphic
items and a menubar offering various options
such as saving the drawing in a file.

<h4>Pathnames</h4>
Widgets in Tk are identified by a <i>pathname</i>.
The pathname of a widget reflects its possible
subordination to another widget.
See figure <a href=#Hierarchy><ref>Hierarchy</ref></a>.


%%FIG f-hier 
<slide id=f-hier cap="The <i>drawtool</i> widget hierarchy">>
<control><a name=Hierarchy></control>
<image src=local/figures/f-hier.gif>
<!-- \vspace{0.5cm}\hspace*{-1.5cm}\figbox{figures/f-hier}\vspace{0.5cm} -->
</slide>



Pathnames consist of strings separated by dots.
The first character of a path must be a dot.
The first letter of a path must be lower case.
The format of a path name may be expressed in BNF form as
<pre>
  path ::= . | .string | path.string
</pre>
where <i>string</i> and <i>path</i> are nonterminals.
For example "." is the pathname of the root widget,
whereas ".quit" is the pathname of a child
of the root widget.
A widget that is a child of another widget
must have the pathname of its parent
as part of its own path name.
For example, the widget ".f.m" may have a widget ".f.m.h"
as a child widget.
Note that the widget hierarchy induced by the pathnames
is completely orthogonal to the widget class inheritance
hierarchy depicted in figures
<a href=#Widgets><ref>Widgets</ref></a>
and
<a href=#Classes><ref>Classes</ref></a>.
Pathnames are treated somewhat more liberally in <i>hush</i>.
For example, widget pathnames may simply be defined
or extended by a string.
The missing dot is then automatically inserted.

<h4>The <i>widget</i> class</i>
When creating a widget, a pathname must be given to
the widget constructor. Pathnames may be defined relative
to a parent widget.
The class interface of <i>widget</i> is given below:


<slide id=widget cap="The <i>widget</i> interface">
<a name=Widget>
<program>
  interface widget : handler { 
    
    widget(char* p); 
    widget(widget& w, char* p); 
    
    char* type();                // returns type of the widget
    char* path();                // returns path of the widget
    
    int eval(char* cmd);         // invokes "thepath() cmd"
    char* result();              // returns the result of eval
    char* evaluate(char* cmd);   // combines eval and result()
    
    virtual void configure(char* cmd);  // invokes Tk configure 
    virtual void geometry(int w, int h); // determines w x h
    
    widget* pack(char* options = "" ); // maps it to the screen
    
    bind(char *b, handler* h, char* args = "" );  // binding

    bind(handler* h, char* args = "" );           // implicit
    
    void xscroll(scrollbar* s);        // to attach scrollbars
    void yscroll(scrollbar* s);
    
    void focus(char* options="");
    void grab(char* options="");
    
    void destroy();             // to remove it from the screen
    
    void* tkwin();  // gives access to Tk_Window implementation
    
    widget* self();            // for constructing mega widgets
    void redirect(widget* w);

  protected:
    char* thepath();                // delivers the virtual path
    void alias( widget* );          // to create widget command
    virtual install(binding*,char* args="");  // default bindings
    virtual direct(char* bnd, binding*, char* args=""); // effect
  };
</program>
</slide>


The <i>widget</i> class is an abstract class.
Calling the constructor <i>widget</i> as in
<pre>
  widget* w = new widget(".awry");
</pre>
does not result in creating an actual
widget but only defines a pointer to the widget
with that particular name.
If a widget with that name exists, it may be treated
as an ordinary widget object, otherwise an error will occur.
The constructor <i>widget(widget* w,char* path)</i> creates
a widget by appending the pathname <i>path</i> to the
pathname of the argument widget <i>w</i>.

The function <i>path</i> delivers the pathname of
a widget object.
Each widget created by Tk actually defines a
Tcl command associated with the
pathname of the widget.
In other words, an actual widget may be regarded
as an object which can be asked to evaluate commands.
For example a widget ".b" may be asked to change
its background color by a Tcl command like
<pre> 
  .b configure -background blue
</pre>
The functions <i>eval</i>, <i>result</i> and <i>evaluate</i>
enable the programmer to apply Tcl commands
to the widget directly, as does the <i>configure</i>
command.
The function <i>geometry</i> sets the width and height of
the widget.

<h4>Packing</h4>
Naming widgets in a hierarchical fashion does not imply
that the widgets behave accordingly.
In particular, to position widgets properly,
they must be packed in relation 
to one another.
Packing results in displaying the widgets on the screen.
The widget class interface offers two
<i>pack</i> functions.
The function <i>widget::pack(char*)</i>
applies to individual widgets.
As options one may specify for example
<tt> -side <i>X</i></tt>, where <i>X</i> is either
<tt>top</tt>, <tt>bottom</tt>, <tt>left</tt>
or <tt>right</tt>,
to pack the widget to the appropriate side
of the cavity specified by the ancestor widget.
Other options are <tt> -fill x</tt> or
<tt> -fill y</tt>, to fill up the space
in the appropriate dimensions
or <tt> -padx <i>N</i></tt> or <tt> -pady <i>N</i></tt>, for
some integer <tt> <i>N</i></tt>,
to surround the widget with some extra space.

As a remark, the <i>kit::pack</i> function may only
be used to pack widgets to the root window.

<h4>Binding events</h4>
Widgets may respond to events.
To associate an event with an action,
an explicit binding must be specified
for that particular widget.
Some widgets provide default bindings.
These may, however, be overruled.

The function <i>bind</i> is used to associate
handlers or bindings with events.
The first string parameter of
<i>bind</i> may be used to specify
the event type.
Common event types are,
for example,
<i>ButtonPress</i>, <i>ButtonRelease</i>
and <i>Motion</i>, which are the
default events for canvas widgets.
Also keystrokes may be defined as events,
as for example <i>Return</i>,
which is the default event for 
the <i>entry</i> widget.

The function <i>widget::bind(handler*, char*)</i> may be
used to associate a handler object
or
action with the default bindings
for the widget.
Concrete widgets may not override the <i>handler</i>
function itself, but must define the protected
virtual function <i>install</i>.
Typically, the install function consists
of calls to <i>bind</i>
for each of the event types that is
relevant to the widget.
Bindings are effected by the virtual function
<i>direct</i> that may be redefined to effect
the binding for multiple widgets, for example.


For both the <i>bind</i> 
functions, the optional <i>args</i>
parameter may be used to specify
the arguments that will be passed
to the handler or action
when it is invoked.
For the <i>button</i> widget for example,
the default <i>install</i> function supplies
the text of the button as an  additional
argument for its handler.


<h4>Compound widgets</h4>
In addition, the widget class offers four
functions that may be used
when defining compound or mega widgets.
The function call <i>redirect(w)</i> must by used
to delegate the invocation
of the <i>eval</i>, <i>configure</i>,
<i>bind</i> and <i>handler</i> functions
to the widget <i>w</i>.
The function <i>self()</i> gives access to the widget
to which the commands are redirected.
After invoking <i>redirect</i>,
the function <i>thepath</i> will deliver
the path that is determined by <code>self()->path()</code>.
In contrast, the function <i>path</i> will still deliver
the pathname of the outer widget.
Calling <i>redirect</i> when creating the compound widget
class suffices for most situations.
However, when the default events must be changed or the 
declaration 
of a handler must take effect for several component widgets,
the virtual function <i>install</i> must be redefined
to handle the delegation explicitly.
The <i>alias</i> function is needed when creating 
widgets that are also used in Tcl scripts.
It creates the command corresponding to the
widget's path name.
How <i>redirect</i> and <i>alias</i> actually work will
hopefully become clear in the examples.


<h2><a name=Buttons>Buttons</h2>
<!--\label{buttons}-->


As the first component of the drawing tool,
we will look at the <i>toolbox</i>.
The <i>toolbox</i> is a collection of buttons
packed in a frame: 


<slide id=toolbutton cap="The <i>toolbutton</i> class">>
<program>
  
  class toolbutton : public button {       // the toolbutton
  public:
    toolbutton(widget* w, char* name) : button(w,name) { 
        text(name);
	bind(w,name);    // the parent becomes the handler
	pack(); 
    }
  };
  	
  class toolbox : public frame {               // the toolbox
  public:
    toolbox(widget* w, tablet* t) : c(t), frame(w,"toolbox") { 
        button* b0 = new toolbutton(this,"draw");
        button* b1 = new toolbutton(this,"move");
        button* b2 = new toolbutton(this,"box");
        button* b3 = new toolbutton(this,"circle");
        button* b4 = new toolbutton(this,"arrow");
    }
  
    int operator()() {
		c->mode( _event->arg(1) );     // transfer to tablet
		return OK;
		} 
  private:
    tablet* c;
  };
</program>
</slide>


Each individual button is an instance of
the class <i>toolbutton</i>.
When a toolbutton is created,
the actual button is given the name of the button
as its path.
Next, the button is given the name as its text,
the parent widget <i>w</i> is declared to be the 
handler for the button and the button is packed.
The function <i>text</i> is a member function
of the class <i>button</i>, whereas both
<i>bind</i> and <i>pack</i> are common widget functions.
Note that the parameter <i>name</i> is used as 
a pathname, as the text to display, and as an argument
for the handler, that will passed
as a parameter when invoking the handler object.


The <i>toolbox</i> class inherits from the <i>frame</i> widget class,
and creates a frame widget with a path relative to the widget parameter
provided by the constructor.
The constructor further creates the toolbuttons.


The <i>toolbox</i> is both the parent widget
and handler for each individual <i>toolbutton</i>.
When the  <i>operator()</i> function of the <i>toolbox</i> is invoked
in response to pressing a button,
the call is delegated to the <i>mode</i> function
of the <i>tablet</i>.
The argument given to <i>mode</i> corresponds to
the name of the button pressed.

<h4>Comments</h4>
The definition of the <i>toolbutton</i> and <i>toolbox</i>
illustrates that a widget need not necessarily be its
own handler.
The decision whether to define a subclass which is made
its own handler or
to install an external handler
depends on what is considered the most convenient
way to access the resources needed.
As a guideline, exploit the regularity of the application!



<h2><a name=Menus>Menus</h2>
<!--\label{menus}-->


The second component of our drawing tool is the <i>menu_bar</i>:

<slide id=menubar cap="Our menubar class">
<program>

  class menu_bar : public menubar {          // row of menubuttons 
  public:
    menu_bar(widget* w, tablet* t, toolbox* b) : menubar(w,"bar") { 
      configure("-relief sunken"); 
  
      menubutton* b1 = new file_menu(this,t); 
      menubutton* b2 = new edit_menu(this,b);
      button* b3 = new help_button(this); 
    }
  };
</program>
</slide>


The class <i>menu_bar</i>
is derived from the <i>hush</i> widget <i>menubar</i>,
which is described in the appendix.
Its constructor requires an ancestor widget,
a <i>tablet</i> and a <i>toolbox</i>.
The <i>tablet</i> is passed as a parameter to the
<i>file_menu</i>,
so that the application can write to a file
and read from a file.
The <i>toolbox</i> is passed as a parameter to the <i>edit_menu</i>
because the <i>toolbox</i> is employed as a handler
for the <i>edit_menu</i>.
In addition, a <i>help_button</i> is created, which provides
on-line help in a hypertext format when pressed.
The help facility will be discussed in section 
<a href=#Hypertext><ref>Hypertext</ref></a>.

The <i>menu_bar</i> consists of menubuttons
to which actual menus are attached.
The <i>menubutton</i> and <i>menu</i> widgets
are built-in widgets.
They are described in the appendix.
Each menu consists of a number of entries,
which may possibly lead to cascaded menus.

The <i>file_menu</i> class
defines a menu, but is derived from <i>menubutton</i>
in order to attach the menu to its <i>menu_bar</i>
parent:



<slide id=filemenu cap="The file_menu class">
<program>

  class file_menu : public menubutton { 
  
  public:
    file_menu(widget* w, tablet* t) : c(t), menubutton(w,"file") { 
  
      configure("-relief sunken"); text("File"); pack("-side left"); 

      f = new file_handler(c);   // create a file_handler
  
      class menu* m = new class menu(this,"menu"); 
      this->menu(m);             // declares it for the menubutton
      m->bind(this);             // installs this as the handler
  
      m->entry("Open");
      m->entry("Save");
      m->entry("Quit");
    }
  
    int operator()() { 

  	if (!strcmp( _event->arg(1),"Quit")) tk->quit(); 
  	else f->dispatch( _event ); // transfer to file_handler
  	return OK;
    }

  protected:
    tablet* c;
    file_handler* f;
  };
</program>
</slide>



The <i>file_menu</i> constructor defines the appearance of
the button and creates a <i>file_handler</i> (which will
be discussed in section 
<a href=#Dialogs><ref>Dialogs</ref></a>).
It then defines the actual menu.
The menu must explicitly be attached to
the <i>menubutton</i> by invoking the 
function <i>menubutton::menu</i>.
For creating the menu, the keyword <i>class</i> is needed
to disambiguate between the creation of an instance of the class
menu and the call of the <i>menubutton::menu</i> function.


Before defining the various entries of the menu,
the <i>file_menu</i> instance is declared
as the handler for the menu entries.
However, except for the entry <i>Quit</i>, which is handled
by calling the <i>kit::quit</i> function,
the calls are delegated to the previously created <i>file_handler</i>.


The second button of the <i>menu_bar</i> is defined by the
<i>edit_menu</i>.
The <i>edit_menu</i> requires a <i>toolbox</i> and creates
a menubutton.
It configures the button and
defines a menu containing two entries,
one of which is a cascaded menu.
Both the main menu and the cascaded menu
are given the <i>toolbox</i> as a handler.
This makes sense only because 
for our simple application,
the functionality offered by
the <i>toolbox</i> and <i>edit_menu</i> coincide.





<h2>Defining actions -- delegation versus inheritance</h2>

The most important component of our <i>drawtool</i>
application is defined by the <i>tablet</i> class:


<slide id=tablet cap="The <i>tablet</i> class">
<program>

  class drawmode {                               // drawing modes
  public: enum { draw, move, box, circle, arrow, lastmode };
  };
    
  class tablet : public canvas {                    // the tablet
  public:
    
    tablet(widget* w, char* options="");  
    
    int operator()() {                      // according to _mode
    	return handlers[_mode]->dispatch( _event );
    }
    
    void mode(char* m);                // to set the drawing mode
    
  protected:
    void init(char* options);           // initializes the tablet
    int _mode; 
    class handler* handlers[drawmode::lastmode];   // keeps modes
    canvas* c;                               // the actual canvas
  };
</program>
</slide>



The various modes supported by the drawing tool are
enumerated in a separate class <i>drawmode</i>.
The <i>tablet</i> class itself inherits from the <i>canvas</i>
widget class.
This has the advantage that it offers the full functionality
of a canvas.
In addition to the constructor and <i>operator()</i> function,
which delegates the incoming event to the appropriate
handler
according to the <i>_mode</i> variable,
it offers a function <i>mode</i>, which sets the mode of the
canvas as indicated by its string argument,
and a function <i>init</i> that determines the creation
and geometrical layout of the component widgets.
As instance variables, it contains the integer <i>_mode</i>
variable and an array of handlers that contains
the handlers corresponding to the modes supported.
See section <a href=#Canvas><ref>Canvas</ref></a> 
for an example of a typical
canvas handler.


<h4>Dispatching</h4>
Although the <i>tablet</i> must act as a canvas,
the actual <i>tablet</i> widget is nothing but a <i>frame</i>
that contains a canvas widget as one of its components.
This is reflected in the definition of the
<i>tablet</i> constructor and the way it invokes
the <i>canvas</i> constructor.


<slide id=tablet-constructor cap="The tablet constructor">>
<program>
  tablet::tablet(widget* w, char* options) : canvas(w,"tablet",0) { 
    widget* top = new frame(path());

    init(options);                          // inialization, layout
    redirect(c);                            // redirect to canvas
    bind(this);                             // this is the handler

    handlers[drawmode::draw] = new draw_handler(this);
    handlers[drawmode::move] = new move_handler(this);
    handlers[drawmode::box] = new box_handler(this);
    handlers[drawmode::circle] = new circle_handler(this);
    handlers[drawmode::arrow] = new arrow_handler(this);
    _mode = drawmode::draw;
  }
</program>
</slide>


By convention, when the options parameter is <i>0</i>
instead of the empty string, no actual widget
is created but only an abstract widget,
as happens when calling the <i>widget</i> class constructor.
Instead of creating a <i>canvas</i> rightaway,
the <i>tablet</i> constructor creates a top frame,
initializes the actual component widgets,
and redirects the <i>eval</i>, <i>configure</i>, <i>bind</i>
and <i>handler</i> invocations to the <i>canvas</i>
child widget.
It then
declares itself to be its own handler,
which results in declaring itself to be the handler for the
canvas component.
Note that reversing the order of calling <i>redirect</i>
and <i>handler</i> would be disastrous,
since the bindings resulting from calling <i>handler</i>
would then be defined not for the <i>canvas</i> but
for the <i>frame</i> containing the <i>canvas</i>.
After that it creates the handlers for the various modes
and sets the initial mode to <i>draw</i>.

The <i>operator()</i> function takes care of dispatching calls
to the appropriate handler.
The <i>dispatch</i> function must be called to pass
the <i>event</i> information.


<h2><a name=New>Creating new widgets</h2>
<!--\label{new}-->


Having taken care of the basic components
of the drawing tool, that is the <i>toolbox</i>,
<i>menu_bar</i> and <i>tablet</i> widgets, all that remains
to be done is to define
a suitable <i>file_handler</i>,
appropriate handlers for the
various drawing modes
and a <i>help_handler</i>.
This will be done in sections
<a href=#Dialogs><ref>Dialogs</ref></a>,
<a href=#Canvas><ref>Canvas</ref></a> and
<a href=#Hypertext><ref>Hypertext</ref></a>,
respectively.

However, before that we will look at how to define
the <i>drawtool</i> widget class such that we may also declare
a corresponding <i>drawtool</i> script command.
The actual declaration of the <i>drawtool</i> command
is done in the application class defined below,
which will by now look familiar, except for the function
<i>prelude</i>:

<drawtool-application cap="The <i>drawtool</i> application class">
<program>
    
  class application : public session {              
  public:
    application(int argc, char* argv[])
				: session(argc,argv,"drawtool") {}
    
    void prelude( ) {
    	tk->bind("drawtool", new drawtool());    // declare
    }
    
    void main( kit* tk, int, char* argv[] ) {
    	drawtool* d = new drawtool(".draw");
    	tk->bind("drawtool",d);                // override
    	d->pack();
    }
  };
</program>
</slide>

In the body of the <i>prelude</i> function,
the Tcl command <i>drawtool</i> is declared,
with an instance of <i>drawtool</i> as its handler.

In this way, the <i>drawtool</i> widget
is made available as a command when
the program is used as an interpreter.
However, in the function <i>main</i>
this declaration is overridden.
Instead, the actual <i>drawtool</i> widget
is made the handler of the command,
in order to allow for a script to
address the <i>drawtool</i> by calling
<i>drawtool self</i>, as will be explained later.
<p>
Since an instance of <i>drawtool</i> may also be used as
simply a handler for the <i>drawtool</i> command,
the <i>drawtool</i> class must offer
a constructor that creates no widget,
in addition to a constructor that does create a <i>drawtool</i>
widget:

<slide id=drawtool-class cap="The <i>drawtool</i> class">
<program>
  class drawtool : public canvas { 
  public:
    drawtool() : canvas() {  }                        // no widget
    drawtool(char* p, char* opts="") : canvas(p,0) {  
	top = new frame(path(),"-class Drawtool");  // outer frame
    	init(opts);
    	redirect(c);                      // redirect to tablet
	alias( top );              // to declare widget command
    }
                                         
    // Define the semantics of the drawtool command

    int operator()(){                     
    	if (!strcmp("self",argv[1]) )                     // self
			   tk->result(self()->path());
    	else if ( !strcmp( "drawtool" ,*argv) )           // create
			   create(--argc,++argv); 
    	else                                              // eval
			   self()->eval( flatten(--argc,++argv) );
    	return OK;
    }
    
  protected:
    wiget* top;                                      // outer frame
    tablet* c;                                   // inner component
    
    void init(char* options);

    // To create a new drawtool widget and corresponding command

    void create(int argc, char* argv[]) {    
      char* name = *argv;
      new drawtool(name, flatten(--argc,++argv));
    }
  };
</program>
</slide>

The <i>drawtool</i> widget constructor redirects itself to
the <i>tablet</i> widget, which is initialized by calling <i>init</i>.

The <i>drawtool::operator()</i> function defines the semantics
of the <i>drawtool</i> script command.
When the first argument of the call is <i>drawtool</i>,
a new <i>drawtool</i> widget is created,
except when the second argument is <i>self</i>.
In that case, 
the virtual
path of the widget is returned, which is actually
the path of the <i>tablet</i>'s canvas.
As an example of a script employing <i>self</i>
consider
<program>
  set x [drawtool self]
  $x create rectangle 100 20 160 80 
  $x create rectangle 90 30 150 90 
  $x create oval 120 40 170 90
</program>
Evaluating the script results in the drawing 
displayed in figure <a href=#Drawtool><ref>Drawtool</ref></a>.
Such a script may be read in 
by using the <i>Open</i> option in the <i>File</i>
menu (see section <a href=#Dialogs><ref>Dialogs</ref></a>).
<p>
If neither of these cases apply,
the function <i>widget::eval</i> is invoked
for <i>self()</i>, with the remaining arguments flattened
to a string.
This makes it possible to
use the <i>drawtool</i> almost as an ordinary
canvas  as illustrated above and
in the example hypertext script shown in
section 
<a href=#Hypertext><ref>Hypertext</ref></a>.

The creation of the actual widget and
declaration of the corresponding Tcl command,
according to the Tk convention,
is somewhat more involved.

Recall that each Tk widget is identified by
its path, which simultaneously
defines a command that may be used
to configure the widget or,
as for a canvas,
to draw figures on the screen.
Hence, the function <i>create</i> must 
create a new widget and declare the widget
to be the handler of the command
corresponding to its pathname.









<h4>Discussion</h4>
By now you may have lost track of
how delegation within a compound
widget takes place.
Hopefully, a brief look at the implementation
will clarify this.

Each <i>eval</i>, <i>configure</i> or <i>bind</i>
function call for a widget
results in a command addressed at
the path of the widget. 
By redirecting the command to a different path,
the instructions may be delegated to
the appropriate (component) widget.
Delegation occurs, in other words,
by directing the commands to the widget's virtual
path, which is obtained by the protected
function <i>thepath()</i>.
In contrast, the function <i>path()</i> delivers
the path of the widget's outer component.
Indirection takes place by invoking the function
<i>self()</i>, which relies on an instance variable
<i>_self</i> that may be set by the <i>redirect</i>
function.


%%FIG f-deref 
<slide id=drefs cap="Dereferencing <i>self()</i>">
<a name=Dereferencing>
<img src=local/figures/f-deref.gif>
<!-- \vspace{0cm}\hspace*{0cm}\figbox{figures/f-deref} -->
</slide>

Figure <a href=#Dereferencing><ref>Dereferencing</ref></a>
represents the evaluation of <i>self()</i> for
<i>drawtool</i> in a pictorial way.
Dereferencing ultimately results in addressing commands
to the <i>canvas</i> widget, because of the
redirections declared for <i>drawtool</i> and <i>tablet</i>.
The implementation of <i>thepath()</i> and <i>self()</i> is 
simply:
<program>
  char* thepath() { return self()->path(); }
  widget* self() { return _self?_self->self():this; }
</program>
Hence, resolving a compound widget's
primary inner component relies on simple pointer chasing,
which may be applied recursively to an
arbitrary depth at acceptable costs.




<h2><a name=Dialogs>Dialogs</h2>
<!--\label{dialogs}-->

Interactive applications may require
the user to type some input
after reading a message or
to select an item from a list
of alternatives.
One of the widgets that may be used in a dialog
with the user is a <i>file_chooser</i>
widget as depicted in figure
<a href=#Filechooser><ref>Filechooser</ref></a>.

<slide id=Filechooser cap="A filechoose">
<a name=Filechooser>
<image src=local/screens/chooser.gif>
<!-- \vspace{0.8cm}\hspace*{3cm}\epsfbox{screens/chooser.ps} -->
</slide>




Despite its simple appearance,
the <i>file_chooser</i> widget
has some subtle complexities.
The <i>file_chooser</i> class is given below:


<slide id=file_chooser calp+The <i>file_chooser</i> class">>
<program>
  class file_chooser : public toplevel {              // toplevel 

    file_chooser() : toplevel( gensym("filechooser") ) { init(); }

    int operator()();
    char* get() { return e->get(); }

  protected:
    button* b; button *c;                          // OK and CANCEL 
    entry* e; listbox* l;
    int install(char* s, binding* a, char* opts);
    void init();
    void list();
  };
</program>
</slide>

The <i>file_chooser</i> widget consists
of a <i>listbox</i> filled with filenames
and an <i>entry</i> widget that contains
the filename selected by
the user (by double clicking on the name)
or which may, alternatively, 
be used to type in a filename directly.
In addition, the <i>file_chooser</i>
has an <i>OK</i> button, to confirm the choice
and a <i>CANCEL</i> button, to break off
the dialog.

Typically, a <i>file_chooser</i>
is a toplevel widget, that is a widget that
is independently mapped to the screen.
To avoid name clashes the function <i>gensym</i>,
which delivers a system-wide unique
name (with <i>filechooser</i> as a prefix),
is used to determine its path.
Apart from the <i>operator()</i> function,
the <i>file_chooser</i> has only
one public function <i>get</i>,
which delivers the name selected or 
typed in by the user.

The widget components of the
<i>file_chooser</i>, two buttons and
the 
<a href=#entry><i>entry</i></a> and
<a href=#listbox><i>listbox</i></a> widgets,
are stored in its instance variables.
Further, we have a function <i>init</i>
to construct the actual <i>file_chooser</i>
widget, a function <i>list</i> to fill
the <i>listbox</i> and the function <i>install</i>,
which is used to install an external handler
for the two button widgets.
The <i>install</i> function is defined as
<pre>
  void file_chooser::install(binding* a, char* args) {
   	b->handler(a,args);
    	c->handler(a,args);
  }
</pre>
Recall, that when declaring a handler for
a button, the name of the button is given
as an additional argument when invoking the handler.
This enables the <i>file_handler</i>
to distinguish between a call due
to pressing the <i>OK</i> button and a call
due to pressing the <i>CANCEL</i> button.

The interplay between the C++ definition
and the underlying Tcl/Tk toolkit is nicely
illustrated by the definition of the <i>list</i> function.
<pre>
 void file_chooser::list() {
   sprintf(buf,"foreach i [glob *.tcl] { %s insert end $i }",
						     l->path());
   tk->eval( buf );
  }
</pre>
Calling <i>list</i> results in filling the
<i>listbox</i> with the filenames in the
current directory.
Its corresponding definition in C++
would, no doubt, be much more involved.

<h4>The <i>file_handler</i> class</h4>
Window-based interactive applications differ from
ordinary interactive applications
by relying on an event-driven flow of control.
The indirection that is typical for
event-driven control is exemplified
in the definition of the
<i>file_handler</i> depicted below
(recall that the <i>file_handler</i>
was employed by the <i>file_menu</i>
described in section <a href=#Menus><ref>Menus</ref></a>):


<slide id=file_handler cap="The <i>file_handler</i> class">>
<program>
    
  class file_handler : public handler {
  public:
    file_handler( canvas* x ) : c(x) {}

    int operator()() {
	char* key = _event->arg(1);
    	if (!strcmp("Open", key)) launch("OPEN");
    	else if (!strcmp("Save", key)) launch("SAVE");
    	else if (!strcmp("OPEN", key)) open();
    	else if (!strcmp("SAVE", key)) save();
    	return OK;
    }

  protected:
    canvas* c;
    file_chooser* f;
    
    void launch(char* args) {       // launch new filechooser
    	f = new file_chooser();
    	f->handler(this, args);
    }

    void open() { tk->source( f->get() ); f->destroy(); }
    void save() { c->postscript( f->get() ); f->destroy(); }
  };
</program>
</slide>

Since the <i>file_handler</i> does not
correspond to an actual widget when created,
its constructor merely stores the
canvas pointer, which is actually a pointer
to the <i>tablet</i>.

In response to the <i>Open</i> or <i>Save</i> 
menu entries, the <i>file_handler</i>
launches a <i>file_chooser</i>
and declares itself to be the
handler (with the appropriate arguments).
For example, when selecting the <i>Open</i>
entry, the <i>file_chooser</i> is launched
which eventually calls the <i>file_$handler::dispatch$</i>
function
with <i>OPEN</i> as its argument.
The <i>file_handler</i> then invokes
the <i>open</i> function,
which results in reading in the file
and destroys the <i>file_chooser</i>.
In a similar way, the menu entry <i>Save</i>
results in writing the canvas to a postscript
file.
(The code for checking whether the
<i>OK</i> or <i>CANCEL</i> button is pressed
is left as an exercise.)


<include file=navigate>
