<document>
@* The ACOI framework
<include file=navigate>

<slide id=acoi type=fig cap="The extended ACOI architecture">
<image pre="\hspace{1.5cm}" name=local/acoi/acoi>
</slide>

<!--
\newcommand{\apijl}[1]{\stackrel{#1}{\longrightarrow}}
-->

The ACOI framework is intended to accomodate
a broad spectrum of classification schemes,
manual as well as (semi) automatic, for the indexing
and retrieval of multimedia objects.
What is stored are not the actual multimedia objects
themselves, but structural descriptions of
these objects (including their location) that may be used for
retrieval.
<p>
The ACOI model is based on the assumption that
indexing an arbitrary multimedia object
is equivalent to deriving a grammatical structure
that provides a namespace to reason about the object
and to access its components.
However there is an important difference with
ordinary parsing in that the lexical and grammatical items
corresponding to the components of the multimedia object
must be created dynamically by inspecting the actual object.
Moreover, in general, there is not a fixed sequence
of lexicals as in the case of natural or formal languages.
To allow for the dynamic creation of lexical
and grammatical items the ACOI framework supports both
<em>black-box</em> and <em>white-box</em> (feature)
detectors.
Black-box detectors are algorithms, usually 
developed by a specialist in the media domain,
that extract properties from the media object 
by some form of analysis.
White-box detectors, on the other hand,
are created by defining logical
or mathematical expressions over the grammar itself.
In this paper we will focus on black-box detectors only.
<p>
<ignore>
As an example, look at the (simple) feature grammar below,
specifying the structure of a hypothetical community.

<slide type=hide cap="A sample grammar">
<program>
detector world; /// finds the name of the world
detector people; /// checks name, eliminates institutes
detector company; /// looks if there are at least two persons

atom str name;

community: world people company;

world: name;
people: person*;

person: name;
</program>
</slide>

A <em>community</em> consists of people,
and is a community only if it allows for the people
to be in eachothers company.
<p>
A community has a name.
The actual purpose of this grammar is to select the persons
that belong to a particular community from the input,
which consists of names of potential community members.
Note that the grammar specifies three detectors.
These detectors correspond to functions that are invoked
when expanding the corresponding non-terminal in the grammar.
An example of a detector function is the
<em>personDetector</em> function partially specified below.

<slide type=hide cap="A person detector">
<program>
int personDetector(tree *pt, list *tks ){
...
q = query_query("kit=pl src=check.pl");

while (t = next_token(tks)) {
      sprintf(buf,"person(\%s)",t);
      query_eval(q,buf);
      if (query_result(q,0)) ///// put name(person) on tokenstream
                putAtom(tks,"name",t);
      }
...
}
</program>
</slide>

The <em>personDetector</em> function checks for each token
on the input tokenstream $tks$ whether the token corresponds
to the name of a person belonging to the community.
The check is performed by an embedded logic component
that contains the information needed to establish whether
a person is a member of the community.
Note that the query for a single token
may result in adding multiple names to the token stream.
<p>
The <em>companyDetector</em> differs from the <em>personDetector</em>
in that it needs to inspect the complete parse tree
to see whether the (implicit) <em>company</em> predicate is satisfied.
<p>
When parsing succeeds and the <em>company</em> predicate is satisfied
a given input may result in a sequence
of updates of the underlying database, as illustrated below.


<slide type=hide cap="Database updates">
<display>
V0 := newoid();
V1 := newoid();
  community_world.insert(oid(V0),oid(V1));
    world_name.insert(oid(V1),"casa");
  community_people.insert(oid(V0),oid(V1));
V2 := newoid();
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"alice");
    people_person.insert(oid(V1),oid(V2));
      person_name.insert(oid(V2),"sebastiaan");
    ...
</display>
</slide>

Evidently, the updates correspond to assigning 
appropriate values to the attributes of a structured object,
reflecting the properties of the given community.

<p>
The overall architecture of the ACOI framework is depicted in \sliref{acoi}.
Taking a feature grammar specification, such as the simple
community grammar, as a point of reference, we see
that it is related to an actual feature detector
(possibly containing an embedded logic component)
that is invoked by the Feature detector Engine
when an appropriate media object is presented for indexing.
</ignore>
The information obtained from parsing
a multimedia object is stored in the Monet database.
The feature grammar and its associated detector
further result in updating  the data schemas
stored in the (Monet) database.
The Monet database, which underlies the ACOI framework,
is a customizable, high-performance, main-memory database
developed at the CWI and the University of Amsterdam [_Monet].
<p>
At the user end, a feature grammar is related to
a <em>View</em>, <em>Query</em>
and <em>Report</em> component,
that respectively allow for inspecting a feature grammar,
expressing a query, and delivering a response
to a query.
Some examples of these components are currently implemented as applets
in Java 1.1 with Swing. See [_ACOI].
<p>
The processing which occurs for a MIDI file,
by using the grammar and associated detectors
described in section <ref file=detector>Detector</ref>
is depicted in \sliref{midi-processing}.

<slide id=midi-processing type=fig cap="Processing MIDI file">
<image pre="\hspace{2.0cm}" name=local/acoi/process>
</slide>

The input is a MIDI file.
As indicated in the top line, the MIDI file itself may
be generated from a <em>score</em>.
%%Lilypond is a \LaTeX-like formatting language for
%%musical scores that also supports the generation of MIDI [_Lily].
As indicated on the bottom line, processing a MIDI file
results in a collection of features as well as in a (simplified) MIDI file
and corresponding score.
In the current prototype, a collection of Prolog facts is used
as an intermediate representation, from which higher level
features are derived by an appropriate collection of rules.
<p>
The (result) MIDI file contains an extract
of the original (input) MIDI file
that may be presented to the (end) user as the result of a query.
This setup allows us to verify whether our extract or
abstraction of the original musical structure is effective,
simply by comparing the input 
musical structure with the output (MIDI) extract. 
<p>
:@ Formal specification
Formally, a feature grammar $G$ may be defined
as $G = (V,T,P,S)$, where $V$ is a collection of
variables or non-terminals, 
$T$ a collection of terminals,
$P$ a collection of productions of the form $V -> (V \union T)$
and $S$ a start symbol.
A token sequence $ts$ belongs to the
language $L(G)$ if $S -*-> ts$.
Sentential token sequences, those belonging to
$L(G)$ or its sublanguages $L(G_v) = (V_v,T_v,P_v,v)$
for $v \e (T \union V)$, correspond to a complex object
$C_v$, which is the object corresponding to the parse tree 
for $v$.
The parse tree defines a hierarchical structure
that may be used to access and manipulate the components
of the multimedia object subjected to the detector.
See [_Features] for further details.

</document>
