<document>
%%@* Similarity matching for musical structures
@* Content-based retrieval -- related work
\label{Match}
<include file=navigate>

@$=raw

<ignore>

The key message is that we can deal with non-trivial feature
detection in an integrated framework. In this respect I consider
the lyric feature as a simple step to beat others, only
if we are sure that those midi search engines do not do the same.

The next step I would immediately consider is to include in
the database an indication of the instruments in the piece
(restricting ourselfs to General midi). It might be interesting
to built an histogram of the time contribution of each instrument.
This is again relatively ease, but also can gives the user
some interesting visual feedback.

In the same line, the distribution in note duration will help
to classify the piece in terms of genre (relax, hotmetal)
</ignore>



<ignore>
%%:@ Possible indexes
<ul>
<li> instruments used
<li> relative contribution per instrument
<li> distribution of note durations
<li> distribution of intervals
<li> melodic contour or pitch profile
</ul>
See [_Beyond], [_Similarity].
</ignore>



The ACOI architecture, as sketched in the previous sections,
allows for musical feature detection in an integrated framework.
We aim at providing content-based retrieval facilities
in the musical domain for a large category of
users over a very large database of musical works.
<p>
As concerns musical content, at least for most genres,
it appears that we should focus primarily on <em>melody</em>,
since, as phrased in [_Concepts]:
<quote><em>"It is melody that makes music memorable:
we are likely to recall a tune long after we have
forgotten its text."</em></quote>
Other features, content-based as well as descriptive,
may however be used as additional filters in the proces
of retrieval.
<p>
Melodic searching and matching has been explored mainly
in the context of bibliographic tools and for the
analysis of (monophonic) repertories [_Similarity].
As described in section <ref file=web>Web</ref>,
many of these efforts have been made available to
the general public through the Web.
Challenges for the near future are, however, to provide
for melodic similarity matching on polyphonic works,
and retrieval over very large databases of musical fragments.
<p>
In this section we will look in somewhat more detail
at the problem of melodic similarity matching.
In particular, we will discuss representational issues,
matching algorithms and additional analysis tools
that may be used for musical information retrieval.

<slide id=Kortjakje type=fig cap="Twinkle, twinkle ...">
\vspace{-1.0cm}
<image pre="\hspace{1.0cm}" name=local/kortjakje>
\vspace{2.0cm}
</slide>

:@ Melodic similarity
Consider the musical fragment depicted in \sliref{Kortjakje},
Twinkle, twinkle little star
(known in the Dutch tradition as <em>"Altijd is Kortjakje ziek"</em>),
which has been used by Mozart for a series of variations [_Mozart].
Now, imagine how you would approach establishing the similarity between 
the original theme and these variations.
As a matter of fact, we discovered that exactly this problem
had been tackled in the study reported in [_Compare], which we will discuss
later.
Before that, we may reflect on what we mean by the concept of a <em>melody</em>.
In the aforementioned variations the original melody is disguised
by, for example, decorations and accompaniments.
In some variations, the melody is distributed among
the various parts (the left and right hand).
In other variations, the melody is only implied by the harmonic structure.
Nevertheless, for the human ear there seems to be,
as it is called in [_Concepts],
a <em>'prototypical'</em> melody that is present in each of the variations.
<p>
When we restrict ourselves to pitch-based comparisons,
melodic similarity may be established by
comparing profiles of pitch-direction (up, down, repeat) or
pitch contours (which may be depicted graphically).
Also, given a suitable representation,
we may compare pitch-event strings (assuming a normalized
pitch representation such as position within a scale)
or intervallic contours (which gives the distance between
notes in for example semitones).
Following [_Concepts], we may observe however that the more general
the system of representation, the longer the (query) <em>string</em>
will need to be to produce meaningful discriminations.
As further discussed in [_Concepts], recent studies in
musical perception indicate that pitch-information without 
durational values does not suffice.

:@ Representational issues
Given a set of musical fragments, we may envisage
several reductions to arrive at the (hypothetical)
prototypical melody.
Such reductions must provide for the elimination of
confounds such as rests, repeated notes and grace notes,
and result in, for example, a pitch-string (in a suitable representation),
a duration profile, and (possibly) accented note profiles
and harmonic reinforcement profiles (which capture notes that
are emphasized by harmonic changes).
Unfortunately, as observed in [_Concepts],
the problem of which reductions to apply
is rather elusive, since it depends to a great extent
on the goals of the query and the repertory at hand.
<p>
As concerns the representation of pitch information,
there is a choice between a base-7 representation,
which corresponds with the position relative to the 
tonic in the major or minor scales,
a base-12 representation, which corresponds
with a division in twelve semitones as in
the chromatic scale,
and more elaborate encodings, 
which also reflect notational differences in identical notes
that arise through the use of accidentals.
For MIDI applications, a base-12 notation is most suitable,
since the MIDI note information is given in semitone steps.
In addition to relative pitch information, octave information is also
important, to establish the rising and falling of melodic contour.
<p>
When we restrict ourselves to directional profiles
(up, down, repeat),
we may include information concerning the slope, or degree of change,
the relation of the current pitch to the original pitch,
possible repetitions,
recurrence of pitches after intervening pitches,
and possible segmentations in the melody.
In addition, however, to support relevant comparisons
it seems important to have information on the rhythmic and harmonic 
structure as well.

:@ Similarity matching
An altogether different approach at establishing melodic similarity
is proposed in [_Compare]. This approach has been followed
in the Meldex system [_Meldex], discussed in
section <ref file=web>Web</ref>.
The approach is different in that it relies on
a (computer science) theory of finite sequence comparison,
instead of musical considerations.
The general approach is, as explained in [_Compare],
to search for an optimal correspondence between elements
of two sequences, based on a distance metric or measure
of dissimilarity,
also known more informally as the <em>edit-distance</em>,
which amounts to the (minimal) number of transformations that need to be applied
to the first sequence in order to obtain the second one.
Typical transformations include <em>deletion</em>,
<em>insertion</em> and <em>replacement</em>.
In the musical domain, we may also apply transformations
such as <em>consolidation</em> (the replacement of several elements
by one element) and <em>fragmentation</em> (which is the reverse of
consolidation).
The metric is even more generally applicable by associating a weight 
with each of the transformations.
Elements of the musical sequences used in [_Compare] are
pitch-duration pairs, encoded in base-12 pitch information
and durations as multiples of 1/16th notes.
<p>
The matching algorithm can be summarized by the following recurrence
relation for the dissimilarity metric.
Given two sequences $A = a_1,...,a_m$ and $B = b_1,...,b_n$
and $d_{ij} = d(a_i,b_j)$, we define the distance as
<raw height=400>
\curlx{$d_{ij} = min$}{
  \mbox{\parbox{12cm}{$d_{i-1,j} + w (a_i,0)$   \zline{deletion}}} \\
  \mbox{\parbox{12cm}{$d_{i,j-1} + w (0,b_j)$     \zline{insertion}}} \\
  \mbox{\parbox{12cm}{$d_{i-1,j-1} + w (a_i,b_j)$     \zline{replacement}}} \\
  \mbox{\parbox{12cm}{$d_{i-k,j-1} + w (a_{i-k+1},...,a_i,b_j)$. $2 <= k <= i$ \zline{consolidation}}} \\
  \mbox{\parbox{12cm}{$d_{i-1,j-k+1} + w (a_i,b_{j-k+1},...b_j)$  $2 <= k <= j$ \zline{fragmentation}}}
} </raw>
<c><p></c>
<br>
\noindent
with
<raw height=200>
<display>
  $d_{i0} = d_{i-1,0} + w(a_i,0)$, $i>=1$            /// deletion
  $d_{0j} = d_{0,j-1} + w(0,b_i)$, $j>=1$            /// insertion
</display>
</raw>
<c><p></c>
and $d_{00} = 0$.
<p>
The weigths $w(\_,\_)$ are determined by the degree of dissonance and the length
of the notes involved.
The actual algorithms for determining the dissimilarity
between two sequences uses dynamic programming techniques.
The algorithm has been generalized to look for matching phrases,
or subsequences, within a sequence.
The complexity of the algorithm is $O(mn)$,
provided that a limit is imposed on the number of notes
involved in consolidation and fragmentation.
<p>
Nevertheless, as indicated in experiments for the
Meldex database, the resulting complexity is still
forbidding when large databases are involved.
The Meldex system offers apart from the (approximate)
dynamic programming algorithm also a state matching algorithm
that is less flexible, but significantly faster.
The Meldex experiments involved a database
of 9400 songs, that were used to investigate six
musical search criteria:
(1) exact interval and rhythm,
(2) exact contour and rhythm,
(3) exact interval,
(4) exact contour,
(5) approximate interval and rhythm, and
(6) approximate contour and rhythm.
Their results indicate that the number of notes needed
to return a reasonable number of songs scales
logarithmically with database size [_Meldex].
It must be noted that the Meldex database contained
a full (monophonic) transcription of the songs.
An obvious solution to manage the complexity of
searching over a large database would seem to be the storage
of prototypical themes or melodies instead of complete songs.

:@ Indexing and analysis
There are several tools available that may assist us in creating
a proper index of musical information.
One of these tools is the Humdrum system,
which offers facilities 
for metric and harmonic analysis,
that have proven their worth in several musicological investigations [_Humdrum].
Another tool that seems to be suitable for our purposes,
moreover since it uses a simple pitch-duration, or <em>piano-roll</em>,
encoding of musical material, is
the system for metric and harmonic analysis described in [_Meter].
Their system derives a metrical structure, encoded
as hierarchical levels of equally spaced beats,
based on preference-rules which determine the overall
likelihood of the resulting metrical structure.
Harmonic analysis further results in (another level of)
<em>chord spans</em> labelled with roots,
which is also determined by preference rules that take
into account the previously derived metrical structure.
As we have observed before, metrical and harmonic analysis
may be used to eliminate confounding information
with regard to the 'prototypical' melodic structure.

:@ Discussion
Sofar our review of related research indicates
that we should rely on approximate
matching for determining musical similarity in response to a query.
Nevertheless, it is clear that we should strive
for reducing the actual information stored,
In addition, we should aim at an hierarchically organized
index of musical information,
for example based on melodic contours 
of thematic fragments, to reduce the actual search 
involving comparisons by approximate matching.
<p>
Since it is not our intention to develop
any new technology, we will very likely rely on the
software that has been developed in the projects mentioned,
in particular the Humdrum suite of tools,
the metric and harmonic analysis tools described in [_Meter],
and the matching software developed for the Meldex system (that has been kindly made
available to us by R. McNabb).
Together with the feature detection architecture described in
section <ref file=detector>Detector</ref>,
this should give us the technology with which to create a powerful
information retrieval system in the musical domain.
In addition we wish to explore
the usage of timbre (i.e. the distribution of instrument usage)
and genre (as indicated by contextual information, and possibly
statistical analysis of the musical information in the database),
to improve on the effectiveness of searching for particular songs
or works.


</document>
