Visualization -- the transformation of data and information into
multimedia including pictures, animation and 3D scenes -- enables
users to understand information more naturally. It reveals patterns
and relations in the information which may otherwise remain hidden. As
a consequence, it can provide a single user with enough
<EM>valuable</EM> information to support decision making. In addition to
this, visualization can also be used to explain information to other
people. In this case, the results of visualization are deployed as
arguments in collaborative decision making.

<P>
This paper discusses a distributed visualization architecture which
supports collaborative decision making. The architecture is designed
with the following consideration in mind: ``multiple users, with
different information needs, require multiple views or perspectives of
the data.'' Additionally, in order to support the cooperation between
users during the decision making process, we extend the architecture
with collaborative aspects including session management, and the
exchange of visualization perspectives.
<H1><A NAME="SECTION00110000000000000000">
Introduction</A>
</H1>

Visualization is used to give better insight into data by showing a visual representation of the information. Visualization is becoming increasingly important because people are suffering from an information overload caused by enormous amounts of data. By using visualization we can first explore a comprehensive overview of the information and later decide to zoom in on the details.

<P>
Currently, people are using visual representations of information for two different purposes. First, visualization is often used to understand information. A visualization gives quick insight into information using humans' remarkable perceptual abilities&nbsp;(<A
 HREF="node22.html#UI:Shneiderman">Shneiderman, 1998</A>). Second, visual representations are used to show information to other people. <A
 HREF="node22.html#UI:Shneiderman">Shneiderman (1998, p. 522)</A> states that the bandwidth of information presentation is potentially higher in the visual domain than it is for media reaching any of the other senses. For example, news papers are full of graphs to show economic growth or the developments on the stock exchange market. In the first case, when using visualization to understand information, we often apply it individually (although it is surely useful to try to understand information in a group process). In the latter case we are communicating with other people because we try to illustrate something, or we want to convince them of our point of view.

<P>
In addition to static visualizations (2D images), current technology enables a new form of visualization: the interactive visualization of dynamic data. In recent years, the desktop computer has evolved from a text/picture based system to a fully multimedia-enabled workstation. This offers a great opportunity to deploy visualization on multimedia desktop computers. Visual representations consisting of interactive 2D or 3D animations enable the visualization of dynamic data coming from, for example, running simulations.
Furthermore, multimedia PCs connected to fast networks allow desktop video conferencing, enabling direct user-to-user communication.

<P>

<H4><A NAME="SECTION00110010000000000000">
Structure</A>
</H4>
The next section illustrates why visualization is useful to support collaborative<A NAME="tex2html1"
 HREF="footnode.html#foot49"><SUP><IMG  ALIGN="BOTTOM" BORDER="1" ALT="[*]"
 SRC="@icons/latex2html/icons.gif/foot_motif.gif"></SUP></A> decision making in a business process re-design project. Section&nbsp;<A HREF="node6.html#diva">3</A> briefly describes the <EM>distributed visualization architecture</EM> ( D<SMALL>IVA</SMALL>), intended for multi-user visualization. After discussing issues in collaborative visualization in Section&nbsp;<A HREF="node7.html#collaboration">4</A>, we will describe our architecture extended with collaborative aspects in Section&nbsp;<A HREF="node14.html#architecture">5</A>. The sixth section illustrates  D<SMALL>IVA</SMALL> from a user's point of view by means of a sample visualization. Finally, in Section&nbsp;<A HREF="node21.html#conclusions">7</A>, we will end up with conclusions.

<P>
<H1><A NAME="SECTION00120000000000000000">&#160;</A>
<A NAME="example">&#160;</A>
<BR>
Re-designing business processes at the GAK
</H1>
Our example concerns the GAK (<EM>Gemeenschappelijk Administratie Kantoor</EM>), which is the largest provider of social security in the Netherlands. The GAK organization's main services are the registration and collection of insurance premiums, and the payment of social security benefits. 

<P>
ASZ, which is the IT company of the GAK Group, builds and maintains the information systems that the GAK is using. Currently, the information infrastructure consists of several large databases, hundreds of separate applications and little integration. To improve this, ASZ is investigating a software architecture that combines the databases, legacy software and new applications into a highly integrated system.

<P>
This development will certainly have an impact on the business processes at the GAK. The company will be able to improve current services and to offer new ones. However, deciding how the new business processes must be organized and what the consequences of some decisions will be is not at all trivial.

<P>
To assist the managers in studying the alternatives we create business-process simulations to <EM>execute</EM> the re-design alternatives (<A
 HREF="node22.html#DV:bpsim">Eli&#235;ns et&nbsp;al., 1996</A>). The managers are now able to run the simulations and experiment with the re-design alternatives themselves. To fully exploit the potential of business simulations we allow the managers to visualize and discuss both the results of the simulation, e.g. the costs and profits, and the running simulation itself, e.g. to illustrate the activities in the re-designed alternative. 

<P>

<H2><A NAME="SECTION00121000000000000000">
Example: registration of new companies</A>
</H2>

<P>
As a concrete example, consider the process of registering the employees of a newly established company for social security. In the past, the employer had to go to a number of counters to fill in the required forms. When the client had forgotten something needed for the registration, he had to go back to get it and start the whole procedure again. 

<P>
As a re-designed alternative we want to explore two options. First, all the paper forms could be combined into a single computer application. All forms could then be filled in at once, in dialogue with a single GAK employee. Second, a GAK employee might be able to go to the newly established company. There, using a laptop, she could fill in all of the needed information by asking it directly to the client on the spot.

<P>
To decide which alternative is preferable, we have to consider a number of aspects including the cost of the alternative, the satisfaction of the clients, and the time needed to register the company. 

<P>
A visualization of the <EM>business process flow</EM> is useful in explaining the business process alternatives, . This illustrates who is performing which tasks and how the information flows through the model. Additionally, a <EM>geographical visualization</EM> shows how far and how often clients and employees of the GAK have to travel. The costs, waiting times and other statistical information of the re-designed alternatives can be presented using <EM>statistical visualizations</EM>, such as charts and histograms.

<P>
The decision makers, who are spread out over the country, plan to make the definitive decision at a meeting. However, before that, they want to prepare and discuss several alternatives. The above mentioned visualizations offer the decision makers (and other interested employees) a common ground for discussion.

<P>
Essentially, we want to support two forms of collaboration: <EM>synchronous distributed</EM> and <EM>face to face</EM> (<A
 HREF="node22.html#UI:Ellis">Ellis et&nbsp;al., 1991</A>). In order to help the participants prepare for the meeting, we first support <EM>synchronous distributed</EM> collaboration where the users cooperate at the same time but in different places. Secondly, at the meeting, where the decisions are made, the decision makers will discuss the selected alternatives <EM>face to face</EM>, i.e. same time, same place. 

<P>

<H1><A NAME="SECTION00130000000000000000">&#160;</A>
<A NAME="diva">&#160;</A>
<BR>
Multi-user visualization
</H1>

As the above example illustrates, it is useful to take visualization from the single user domain into the realm of distributed multi-user systems. This makes it possible to discuss shared information sources. 

<P>
However, multiple users with different backgrounds have different information requirements. In the example above some managers might be interested in the resource allocation (who is using what) of the re-design alternative, while others might be more interested in the financial aspects. To support these different information requirements, multiple perspectives (or views) on the information are required. So, based on the same simulation, we distinguish alternative perspectives that visualize different aspects of the re-designed process.

<P>
The need to have multiple perspectives was the main motivation for designing the <EM>distributed visualization architecture</EM> ( D<SMALL>IVA</SMALL>). Additional requirements were the support for interactive visualization to allow for experimentation, and visualization at the user's desktop by means of a networked or web-based architecture (<A
 HREF="node22.html#Diva:bradford98">Sch&#246;nhage &amp; Eli&#235;ns, 1998</A>).

<P>
We regard the process of visualization as a transition of data through a sequence of models, starting with the generation of data and ending with the presentation of a visualization (<A
 HREF="node22.html#Diva:NPIVM97">Sch&#246;nhage &amp; Eli&#235;ns, 1997</A>). To allow for multiple perspectives on the data, we introduce an intermediate model between the generation and presentation of information. This intermediate model contains information based on the originally generated data, adapted to the information requirements of its users.

<P>
<slide id=concept cap="Conceptual Architecture">
<BR>
<DIV ALIGN="CENTER"><A NAME="fig:concept">&#160;</A><A NAME="110">&#160;</A>
<TABLE WIDTH="50%">
<CAPTION><STRONG>Figure 1:</STRONG>
Conceptual architecture</CAPTION>
<TR><TD>
<image src=concept.gif>
<IGNORE
 WIDTH="153" HEIGHT="34"
 SRC="img1.gif"
 ALT="\begin{figure}
\begin{center}
\epsfig{file=concept.eps, height=\linewidth, angle=-90, clip=} \end{center}\end{figure}"></TD></TR>
</TABLE>
</DIV>
</slide>
<BR>

<P>
Figure&nbsp;<A HREF="node6.html#fig:concept">1</A> depicts our architecture on a conceptual level. The primary model is the source of the information and contains explicitly or implicitly all information available. A conceptual mapping gives us the ability to adapt the raw information in the primary model to our information needs. Consequently, information in the derived model differs from data in the primary model in two ways. Primarily, only valuable information is selected to be present in the derived model and, secondly, information derived from primary data is added in the derived model.

<P>
How we present the derived information is specified in the presentational mapping. Here, information concepts in the derived model are mapped onto generic visualization primitives. The final presentation is the content of the presentation model. For example, when using  D<SMALL>IVA</SMALL> for 3D visualization, the presentation component contains a 3D scene through which end users can navigate. For a more detailed description of  D<SMALL>IVA</SMALL> see <A
 HREF="node22.html#Diva:NPIVM97">Sch&#246;nhage &amp; Eli&#235;ns (1997)</A> and (<A
 HREF="node22.html#Diva:bradford98">1998</A>).

<P>
<H1><A NAME="SECTION00140000000000000000">&#160;</A>
<A NAME="collaboration">&#160;</A>
<BR>
Collaborative visualization
</H1>

In D<SMALL>IVA</SMALL>, multiple users can have their own presentation model 
(perspective) while sharing a common resource. However, in this approach the 
different users have the feeling that they are the only user of the shared 
resource. There is not yet support that allows a user to be aware of other users 
or to interact with them. Our goal is to expand the architecture to support 
users to collaborate with each other. Here, 'to collaborate' means that the 
users are able to discuss visualized information in order to reach a decision.

<P>
The next section will address the issues that are involved in this restrictive 
notion of collaborative visualization. Then, we will discuss the requirements 
for an extended D<SMALL>IVA</SMALL> architecture in Section&nbsp;<A HREF="node13.html#col.reqs">4.2</A>.

<P>
<H2><A NAME="SECTION00141000000000000000">&#160;</A>
<A NAME="col.iss">&#160;</A>
<BR>
Issues
</H2>

<P>
D<SMALL>IVA</SMALL> focuses on visualizing information from different perspectives.
We can distinguish between two distinct phases of activities within this approach.

<P>
The first phase is to define and experiment with the perspectives.
This activity is done mostly in solitude, although multiple users can share
a primary model or a derived model. The purpose is to determine the information
need and the relevant data for that need. 
  
The second phase is that of multiple users collaborating by reviewing and 
discussing the different defined perspectives. This article focuses on the 
latter, the collaboration phase.

<P>
When a group of people collaborates, the group members must share a common 
workspace&nbsp;(<A
 HREF="node22.html#UI:Ellis">Ellis et&nbsp;al., 1991</A>). The task of a group of users is to interact with 
each other and present different views on shared information. Let us assume that 
the goal is to reach a decision, for instance, concerning which model to choose for a 
business process re-design project, as in the example of Section&nbsp;<A HREF="node4.html#example">2</A>. 

<P>

<H4><A NAME="SECTION00141010000000000000">
Sessions</A>
</H4>

<P>
Collaboration normally takes place in some kind of meeting, which can differ in 
interaction protocols, group size, formality, etc. Each participant of the 
collaboration can have one or more <EM>roles</EM> depending on the sort of 
meeting. A role is a set of rights and obligations&nbsp;(<A
 HREF="node22.html#UI:Ellis">Ellis et&nbsp;al., 1991</A>). We 
distinguish the following roles: chair, listener, talker and interactor. The 
chair sets up the session, a listener is a passive participant, a talker is 
actively explaining his arguments and, finally, an interactor is able to 
interact with shared resources. The rights and obligations of the different 
roles are determined by the <EM>interaction protocol</EM>. 
The possibility to switch roles dynamically is important, since a 
listener can change into a talker from one moment to the other.

<P>
Collaborative visualization in D<SMALL>IVA</SMALL> is a virtual meeting, where the 
participants are at different places and their desktops are connected by a 
network. We will call the event of such a virtual meeting a <EM>session</EM>. 
Session management should support several kinds of sessions and thus be 
able to handle changing numbers of participants, their roles and interaction 
protocols.

<P>
The notion of <EM>subgroups</EM> makes it feasible to split the total group of 
participants in (non disjoint) groups. These subgroups can communicate separately or
perform subtasks. Subgroups can come into existence dynamically.

<P>
<H4><A NAME="SECTION00141020000000000000">
Sharing perspectives</A>
</H4>

<P>
It is important that the cooperators can show their personal perspective or view 
to other participants, in order to support their arguments in a discussion. One 
way to share views is for one participant to <EM>enforce</EM> his perspective onto 
another user or group. Views can also be shared by means of a <EM>perspective 
repository</EM>, where participants can select a perspective they would like to 
consider. The perspectives they can choose from, must be deposited by other 
participants. This implies that not every participant should have to create her 
own perspective before joining a session. Obviously, there is a need to maintain 
meta-information, explaining what the perspectives are about.

<P>
<H4><A NAME="SECTION00141030000000000000">
Interference versus non-interference</A>
</H4>

<P>
The common basis for the collaborators is the primary model, for instance, 
embodied by a simulation. Several derived models can depend on the primary 
model, and derived model could be related to a number of presentation models. 
When collaborating, the common basis should be the same for all the cooperators 
at every moment in time. To assure consistency, it is best to have the 
simulation act autonomously without the slightest interference. We will refer to 
this as <EM>non-interference</EM>. Non-interference does not restrict the 
possibility for each user to create his own perspective in any way. It does 
prevent somebody from rewinding, changing parameters or restarting the 
simulation while others do not want or expect this.

<P>
However, the need to stop, rewind or change parameters in a simulation is 
imminent. Considering multiple <EM>what-if</EM> situations, for example, is 
necessary when looking at different re-design alternatives, each with its own 
set of parameters. One way to meet this requirement, while upholding 
non-interference, is to store all past events in a database or to allow copies 
with different parameters of the simulation to be started.

<P>
While interaction with the primary model should be avoided as much as possible, 
derived models can be used in a more flexible manner. Several derived models can 
be created, all depending on one primary model. While the primary model can be 
considered a common basis that should not be interfered with, the derived model 
can be seen as a common workspace that permits <EM>interference</EM>. All 
participants of a session could use the same derived model or multiple derived 
models could be created, depending on the need to share information concepts or 
to be independent of the other users.

<P>
<H4><A NAME="SECTION00141040000000000000">
Communications</A>
</H4>

<P>
Some form of user-to-user communication is necessary to enable collaboration. 
These communications can range from a simple chat tool or whiteboard to 
sophisticated audio/video conferencing tools. 

<P>
Tools of interest for use with D<SMALL>IVA</SMALL> include telepointers, to point out 
things of interest, raising hands, to indicate someone wants to speak, and voting 
tools, to support decision making&nbsp;(<A
 HREF="node22.html#UI:Ellis">Ellis et&nbsp;al., 1991</A>).

<P>

<H2><A NAME="SECTION00142000000000000000">&#160;</A>
<A NAME="col.reqs">&#160;</A>
<BR>
Requirements
</H2>

<P>
Taking into account the issues for collaborative visualization mentioned in 
the previous section, we can summarize the following requirements.

<P>
<UL>
<LI>Session management is needed to control the virtual meetings, including the 
	participants, their roles and interaction protocols.
<LI>The participants must be 
	able to share their perspectives by enforcement and via perspective repositories.
<LI>The primary model should  
	be interfered with as little as possible.
<LI>Additional communication support is 
	necessary, but falls outside the scope of this paper.
</UL>

<P>
<H1><A NAME="SECTION00150000000000000000">&#160;</A>
<A NAME="architecture">&#160;</A>
<BR>
Collaborative multi-user visualization architecture
</H1>

The D<SMALL>IVA</SMALL> architecture is intended as a <EM>framework</EM> and should be 
flexible enough to incorporate new components. The components used in 
D<SMALL>IVA</SMALL> are generic software components which interact with each other. 
The architecture is distributed, meaning that its components can reside on 
different hosts on the network. Related collaborative architectures can be found
in <A
 HREF="node22.html#UI:Bentley94">Bentley et&nbsp;al. (1994)</A> and <A
 HREF="node22.html#UI:Reinhard94">Reinhard et&nbsp;al. (1994)</A>.

<P>
<H2><A NAME="SECTION00151000000000000000">
Software components</A>
</H2>

<P>
<slide id=architecture cap="Architecture>
<BR>
<DIV ALIGN="CENTER"><A NAME="arch.arch">&#160;</A><A NAME="271">&#160;</A>
<TABLE WIDTH="50%">
<CAPTION><STRONG>Figure 2:</STRONG>
The main components of D<SMALL>IVA</SMALL></CAPTION>
<TR><TD>
<DIV ALIGN="CENTER">
<image src=architecture.gif>
</DIV></TD></TR>
</TABLE>
</DIV>
</slide>
<BR>

<P>
Figure <A HREF="node15.html#arch.arch">2</A> shows the main components of the architecture. In the 
following list, the three components that were already present in an earlier 
version of the D<SMALL>IVA</SMALL> architecture (<A NAME="tex2html213"
 HREF="node22.html#Diva:NPIVM97">Sch&#246;nhage &amp; Eli&#235;ns, 1997</A>; <A NAME="tex2html214"
 HREF="node22.html#Diva:bradford98">Sch&#246;nhage &amp; Eli&#235;ns, 1998</A>), are 
listed first. The last three components in the list extend D<SMALL>IVA</SMALL>. 

<P>
<UL>
<LI>generator -- embodies the primary model
<LI>shared concept space -- information store that contains the derived model
<LI>presentation component -- the actual information visualization
<LI>D<SMALL>IVA</SMALL> services directory -- central registering of components
<LI>collaborative session manager -- overall coordination of virtual meetings
<LI>local collaboration component -- local collaboration support
</UL>

<P>
In a normal situation, one D<SMALL>IVA</SMALL> services directory and more than one of 
each of the other components could exist. A presentation component and a local 
collaboration component are present at the desktop of each participant. A short 
description is given for each of the components.

<P>
The <EM>generator</EM> embodies the primary model and generates all data needed 
for the visualization. Examples of generators are simulations of business processes. The 
generated raw data is transferred to the shared concept space.

<P>
The <EM>shared concept space</EM> stores information in an expressive and adaptive 
way. The information is contained in the form of concepts that are stored in a 
hierarchical manner. Each concept has one or more data properties that represent 
the information. The data properties are updated with data coming from the 
generator. The received data can be stored directly or can be used to compute 
and store derived information. 

<P>
The <EM>presentation component</EM> actually visualizes concepts from the shared 
concept space. It makes use of gadgets, which are generic 
visualization primitives that present certain types of information.
As an example, cone trees are primitives (gadgets) to visualize hierarchical 
information&nbsp;(<A
 HREF="node22.html#InfoVis:Robertson">Robertson et&nbsp;al., 1993</A>).

<P>
Examples of gadgets are a rotating object which indicates a certain action or 
a histogram which displays data. 

<P>
The <EM>D<SMALL>IVA</SMALL> services directory (DSD)</EM> is the central directory 
component. D<SMALL>IVA</SMALL> components (or services) can register here, identifying 
themselves and giving their location. Once they are registered, the DSD can 
inform other objects about the availability and whereabouts of these services.

<P>
The <EM>collaboration session manager (CSM)</EM> coordinates components. It deals 
with interaction protocols, which means it knows about the participants and 
their roles, sharing perspectives, user to user communication and consistency.

<P>
The <EM>local collaboration component</EM> is directly connected to the session manager. It is 
present at each participants desktop and handles interactions 
and information related to a collaborative session. It may for example display
a list of participants and offer communication facilities.

<P>
<H2><A NAME="SECTION00152000000000000000">
User Environment</A>
</H2>

<P>
Figure <A HREF="node16.html#arch.userenv">3</A> shows a typical user environment. Outside of the user 
environment, two components are shown. A generator, which is a simulation  in 
this figure, feeds data into the shared concept space. This is depicted by the 
fat arrow. These two components can be situated anywhere on the network. Most of 
the components in a user environment use the display. The local collaboration 
component displays information about the collaborative session. The presentation 
component displays the visualization and the controller displays its user 
interface.   

<P>
<slide id=userenv cap="The user environment">
<BR>
<DIV ALIGN="CENTER"><A NAME="arch.userenv">&#160;</A><A NAME="249">&#160;</A>
<TABLE WIDTH="50%">
<CAPTION><STRONG>Figure 3:</STRONG>
The user environment</CAPTION>
<TR><TD>
<DIV ALIGN="CENTER">
<image src=userenv.gif>
</DIV></TD></TR>
</TABLE>
</DIV>
</slide>
<BR>

<P>

<H4><A NAME="SECTION00152010000000000000">
Display agents</A>
</H4>

<P>
From the shared concept space, information is being sent to <EM>display 
agents</EM>. These agents are present at the user environment and each of them 
maintains one gadget. The information they receive is  transformed into a 
visualization. As an example, consider the visualization in 
Figure&nbsp;<A HREF="node20.html#appl.screenshot">4</A> on page&nbsp;<A HREF="node20.html#appl.screenshot"><IMG  ALIGN="BOTTOM" BORDER="1" ALT="[*]"
 SRC="@icons/latex2html/icons.gif/cross_ref_motif.gif"></A>. The line of 
three puppets in front of the desk is a visualization gadget depicting a queue.
When the display agent senses that the length of the queue increases by 
one, it accordingly places a fourth puppet on the screen. 

<P>
The display agents also reside in the perspective repository.  When a user 
requests a certain view, the agents that represent that view are cloned and 
moved to the user environment to build the perspective in the VRML world. 
Enforcing a perspective onto another user is accomplished by cloning and moving 
the display agents from one user to another.

<P>
Now why can we call these entities agents? There are a lot of definitions of agents&nbsp;(<A
 HREF="node22.html#Agents:Franklin96">Franklin &amp; Graesser, 1996</A>), and the question can be raised why a display agent is an agent. In other words, how does it differ from a standard program or software component? For one, the agents are autonomous, which means they execute on their own. Second, they react on certain input and subsequently act on their environment, namely, they sense information from the shared concept space and act on the VRML world. They have some goals they need to accomplish. Third, the display agents can communicate with each other, for instance, to discuss how to place the gadgets each of them represents on the screen (this is a future feature). Fourth, they have a domain which they have knowledge of. This domain is the visualization of information. Fifth, they act on behalf of a user. Users can give their preferences to a display agent, and the agent will take care of it. All in all, the display agent fits quite a number of definitions of autonomous agents given by <A
 HREF="node22.html#Agents:Franklin96">Franklin &amp; Graesser (1996)</A>.

<P>
<H4><A NAME="SECTION00152020000000000000">
Controllers</A>
</H4>

<P>
Every D<SMALL>IVA</SMALL> component can have a separate mobile <EM>controller</EM>. It 
can be moved from one environment to another, so it can be shared by several 
participants. The ability to use a controller depends on the role of the user. 
Participants can request a controller or the chair could appoint it to one of 
them.

<P>
Controllers can have several functions. For example, a simulation can be 
controlled by starting and stopping the simulation and changing certain 
parameters. A controller for a shared concept space can be used to create new 
computed concepts or to decide which data from the generator is selected. 

<P>
<H2><A NAME="SECTION00153000000000000000">
CORBA and the Web</A>
</H2>

<P>
D<SMALL>IVA</SMALL> is designed as a distributed object oriented system. The 
D<SMALL>IVA</SMALL> components are written in C++ and Java, and can run on different 
platforms. We use the Common Object Request Broker Architecture (CORBA) to let 
our distributed objects communicate with each other. CORBA&nbsp;(<A NAME="tex2html253"
 HREF="node22.html#CORBA:Orfali">Orfali &amp; Harkey, 1997</A>; <A NAME="tex2html254"
 HREF="node22.html#CORBA:Siegel">Siegel, 1996</A>) abstracts from hardware, operating systems and programming 
languages. By using the interface definition language (IDL) to describe the 
interfaces between components and by making use of the object request broker 
(ORB), distributed components are able to communicate.

<P>
Voyager&nbsp;(<A
 HREF="node22.html#CORBA:voyager">ObjectSpace, 1997</A>) is an agent ORB written purely in Java, which 
supports CORBA. Voyager allows us to use mobile objects, a feature which CORBA
does not have. We use Voyager to construct the mobile controller components. 
These components are able to "dock" at a user environment and can subsequently 
show their user interface on the screen to let the user interact with it. 

<P>
We use VRML&nbsp;(<A
 HREF="node22.html#VRML:ISO">ISO, 1997</A>) as the main visualization tool. The users are able to navigate through 
the VRML worlds by using a VRML-browser. The External Authoring Interface (EAI) 
makes it possible to control the VRML worlds dynamically via the Java and Javascript 
languages.

<P>
The visualization gadgets in the presentation component are represented by 
mobile display agents. These agents are constructed using Voyager. Display 
agents can also "dock" in a user environment and, in addition, get access to the 
local VRML world. They collect the needed information from shared concept spaces 
to build and maintain the 3D visualization.

<P>
The combination of CORBA and the Web enables access to information resources
by means of HTML, Java and VRML (see also <A
 HREF="node22.html#InfoVis:Rohrer">Rohrer &amp; Swing, 1997</A>).
For example, the simulation and shared concept
space can be hosted on a Unix server while the presentation components are
executed in a Web-browser on Windows client machines.

<P>

<H1><A NAME="SECTION00160000000000000000">&#160;</A>
<A NAME="application">&#160;</A>
<BR>
Application
</H1>

<slide id=screenshot cap="Screenshot">
<BR>
<DIV ALIGN="CENTER"><A NAME="appl.screenshot">&#160;</A><A NAME="338">&#160;</A>
<TABLE WIDTH="50%">
<CAPTION><STRONG>Figure 4:</STRONG>
A perspective with a remote control</CAPTION>
<TR><TD>
<image src=screenshot.gif>
<IGNORE
 WIDTH="14" HEIGHT="18"
 SRC="img3.gif"
 ALT="\begin{figure}
\begin{center}
\epsfig{file=screenshot.eps, width=\linewidth, clip=} \end{center}\end{figure}"></TD></TR>
</TABLE>
</DIV>
</slide>
<BR>

<P>
Figure&nbsp;<A HREF="node20.html#appl.screenshot">4</A> presents a 
screenshot of the desktop of a decision maker participating in a 
collaborative session as described in the example of Section&nbsp;<A HREF="node4.html#example">2</A>. We 
describe a scenario of how the user gets to this display.

<P>
The decision maker starts a Java and VRML enabled Web-browser and follows a 
link pointing to a D<SMALL>IVA</SMALL> server. The resulting HTML file will setup the 
user environment. First, the user has to log in, making available her name and 
network address, and after that she can choose from one or more sessions to join. 
Once the user enters a session, she will be assigned a role and  
then gets a default or enforced perspective. The VRML world showing her view 
is embedded in the Web page. Two Java applets will contain the session interface 
and the collaboration interface (these are not shown in the screenshot). 

<P>
With a push on a button she is able to request a remote control, which will 
arrive at her environment (assuming that she is allowed to do 
so). Once the remote control arrives in the form of a mobile object, this object 
pops up a remote control user interface on the display. The user is then able to 
control the simulation that is associated with the remote control. In Figure&nbsp;<A HREF="node20.html#appl.screenshot">4</A>
the remote control can be used to run, stop and reset the 
simulation. In addition, the speed of the simulation as well as three 
parameters can be changed. Changes to the simulation will accordingly appear in 
the visualization in the browser window.

<P>

<H1><A NAME="SECTION00170000000000000000">&#160;</A>
<A NAME="conclusions">&#160;</A>
<BR>
Conclusions
</H1>
This paper is based on our belief that (interactive) visualizations are useful arguments in decision making because they provide such quick insight into information by using the human perceptual abilities. By means of collaborative visualization, decision makers are able to discuss a shared information source, such as a business process simulation, to convince other users of their point of view. 

<P>
Based on a discussion of issues in collaborative visualization, we have concluded that the following requirements are needed for our visualization architecture. 

<P>
First, different perspectives are necessary because multiple users, with different information needs, require different views on the data. These perspectives can be created by means of shared concept spaces and presentation components that make use of display agents. 

<P>
Consequently, an important requirement is the exchange of visualization perspectives, for example, by enforcement or by a repository of perspectives. The exchange is achieved by cloning and transporting display agents, which in turn define how and what available information is presented to the users. 

<P>
To manage the cooperative sessions, we have defined two collaboration components that handle the rights and obligations belonging to the roles of the participants. 

<P>
As a final requirement, we have stated that interference should be avoided as much as possible because other participants are involved in the actions taken. On the other hand, interaction with a running simulation to evaluate some <EM>what-if</EM> situations is very powerful. Therefore, we have created remote control components to interact with simulations and other data generators. 

<P>
In further research, we are investigating an extension of the shared concept space to store sessions that can be replayed at a later time. Additionally, we are planning a case study to determine for useful visualization primitives to represent business information. 

<P>

<P>
<H2><A NAME="SECTIONREF">Bibliography</A>
</H2>

<DL COMPACT>
<DT><A NAME="UI:Bentley94"><STRONG>
Bentley, R., Rodden, T., Sawyer, P. &amp; Sommerville, I.
  (1994),</STRONG></A>
<DD> `Architectural support for
  cooperative multiuser interfaces', <EM>IEEE Computer</EM> <B>27</B>(5),&nbsp;37-46.

<P>
<DT><A NAME="DV:bpsim"><STRONG>
Eli&#235;ns, A., Niessink, F., Sch&#246;nhage, S., van Ossenbruggen, J.
  &amp; Nash, P.  (1996),</STRONG></A>
<DD> Support for
  Business Process Redesign: Simulation, Hypermedia and the Web,
  <EM>in</EM> `Euromedia 96: Telematics in a Multimedia Environment, London,
  United Kingdom', The Society for Computer Simulation International,
  pp.&nbsp;193-200.

<P>
<DT><A NAME="UI:Ellis"><STRONG>
Ellis, C., Gibbs, S. &amp; Rein, G.  (
  1991),</STRONG></A>
<DD> `Groupware: some issues and experiences', <EM>
  Communications of the ACM</EM> <B>34</B>(1),&nbsp;680-689.

<P>
<DT><A NAME="Agents:Franklin96"><STRONG>
Franklin, S. &amp; Graesser, A.  (1996)
</STRONG></A>
<DD>  , Is it an Agent, or just a Program?: A Taxonomy for Autonomous
  Agents, <EM>in</EM> `Proceedings of the Third International Workshop on Agent
  Theories, Architectures, and Languages'.
<BR><A NAME="tex2html6"
 HREF="www.msci.memphis.edu/<IMG
 WIDTH="31" HEIGHT="23" ALIGN="BOTTOM" BORDER="0"
 SRC="img4.gif"
 ALT="$\sim$">franklin/AgentProg.html"><b>URL:</b> www.msci.memphis.edu/<IMG
 WIDTH="31" HEIGHT="23" ALIGN="BOTTOM" BORDER="0"
 SRC="img4.gif"
 ALT="$\sim$">franklin/AgentProg.html</A>

<P>
<DT><A NAME="VRML:ISO"><STRONG>
ISO  (1997),</STRONG></A>
<DD> <EM>The Virtual Reality
  Modeling Language</EM>.
<BR> International Standard ISO/IEC IS 14772-1:1997.

<P>
<DT><A NAME="CORBA:voyager"><STRONG>
ObjectSpace  (1997),</STRONG></A>
<DD> <EM>Voyager Core
  Technology User Guide (Version 2.0 beta1)</EM>.
<BR><A NAME="tex2html7"
 HREF="www.objectspace.com/voyager/documentation.html"><b>URL:</b> www.objectspace.com/voyager/documentation.html</A>

<P>
<DT><A NAME="CORBA:Orfali"><STRONG>
Orfali, R. &amp; Harkey, D.  (1997),</STRONG></A>
<DD>
  <EM>Client Server Programming with JAVA and CORBA</EM>,
  John Wiley &amp; Sons.

<P>
<DT><A NAME="UI:Reinhard94"><STRONG>
Reinhard, W., Schweizer, J. &amp; V&#246;lksen, G.  (
  1994),</STRONG></A>
<DD> `CSCW Tools: concepts and architectures',
  <EM>IEEE Computer</EM> <B>27</B>(5),&nbsp;28-36.

<P>
<DT><A NAME="InfoVis:Robertson"><STRONG>
Robertson, G., Card, S. &amp; Mackinlay, J.  (
  1993),</STRONG></A>
<DD> `Information Visualization using 3D
  Interactive Animation', <EM>Communications of the ACM</EM> <B>
  36</B>(4),&nbsp;57-71.

<P>
<DT><A NAME="InfoVis:Rohrer"><STRONG>
Rohrer, R. &amp; Swing, E.  (1997),</STRONG></A>
<DD>
  `Web-based Information Visualization', <EM>IEEE Computer Graphics &amp;
  Applications</EM> <B>17</B>(4),&nbsp;52-59.

<P>
<DT><A NAME="Diva:NPIVM97"><STRONG>
Sch&#246;nhage, S. &amp; Eli&#235;ns, A.  (
  1997),</STRONG></A>
<DD> A flexible architecture for user-adaptable
  visualization, <EM>in</EM> D.&nbsp;S. Ebert &amp; C.&nbsp;K. Nicholas, eds,
  `Workshop on New Paradigms in Information Visualization and Manipulation '97,
  Conference on Information and Knowledge Management, 10 - 14 November 1997,
  Las Vegas, USA', ACM Press.

<P>
<DT><A NAME="Diva:bradford98"><STRONG>
Sch&#246;nhage, S. &amp; Eli&#235;ns, A.  (
  1998),</STRONG></A>
<DD> Multi-user Visualization: a
  CORBA/Web-based approach, <EM>in</EM> `Proceedings of "Digital
  Convergence: the Future of the Internet and WWW", 20-23 April 1998, Bradford,
  United Kingdom', British Computer Society.

<P>
<DT><A NAME="UI:Shneiderman"><STRONG>
Shneiderman, B.  (1998),</STRONG></A>
<DD> <EM>Designing the
  User-Interface, Strategies for Effective Human-Computer
  Interaction</EM>, 3rd edn, Addison-Wesley Publishing Company.

<P>
<DT><A NAME="CORBA:Siegel"><STRONG>
Siegel, J.  (1996),</STRONG></A>
<DD> <EM>CORBA
  Fundamentals and Programming</EM>, John Wiley &amp; Sons.

<P>
</DL>

<P>
<BR><HR>

<A NAME="CHILD_LINKS">&#160;</A>
<UL>
<LI><A NAME="tex2html283"
 HREF="node23.html">Biography</A>
</UL>

<BR><HR>
<ADDRESS>
<I>A Eliens</I>
<BR><I>1998-09-05</I>
</ADDRESS>
</BODY>
</HTML>
<H1><A NAME="SECTION00210000000000000000">
Biography</A>
</H1>
Bastiaan Sch&#246;nhage is PhD-student at the Vrije Universiteit in Amsterdam and ASZ Research &amp; Development. His research comprises the design and exploration of a flexible software architecture for dynamic information visualization on the Web ( D<SMALL>IVA</SMALL>). His research interests include information visualization, and distributed OO software architectures.

<P>
Peter Paul Bakker completed his master thesis on the collaborative
aspects of the  D<SMALL>IVA</SMALL>-project. He is a pre-graduate 
student in Software Engineering at the Vrije Universiteit and he is doing an internship
at ASZ R&amp;D. His research interests include collaboration, VRML and agent-based ORBs. 

<P>
Anton Eli&#235;ns is lecturer at the Software Engineering section of the Computer Science Department of the Vrije Universiteit, Amsterdam.
His research interests include object orientation, hypermedia and distributed logic programming.
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">
<!--Converted with LaTeX2HTML 98.1 release (February 19th, 1998)
originally by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds
* revised and updated by:  Marcus Hennecke, Ross Moore, Herb Swan
* with significant contributions from:
  Jens Lippmann, Marek Rouchal, Martin Wilck and others -->
<HTML>
<HEAD>
<TITLE>About this document ... </TITLE>
<META NAME="description" CONTENT="About this document ... ">
<META NAME="keywords" CONTENT="perspectives">
<META NAME="resource-type" CONTENT="document">
<META NAME="distribution" CONTENT="global">
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">
<LINK REL="STYLESHEET" HREF="perspectives.css">
<LINK REL="previous" HREF="node22.html">
<LINK REL="up" HREF="perspectives.html">
</HEAD>
<BODY >

<IMG WIDTH="37" HEIGHT="24" ALIGN="BOTTOM" BORDER="0" ALT="next"
 SRC="@icons/latex2html/icons.gif/next_motif_gr.gif"> 
<A NAME="tex2html296"
 HREF="perspectives.html">
<IMG WIDTH="26" HEIGHT="24" ALIGN="BOTTOM" BORDER="0" ALT="up"
 SRC="@icons/latex2html/icons.gif/up_motif.gif"></A> 
<A NAME="tex2html292"
 HREF="node23.html">
<IMG WIDTH="63" HEIGHT="24" ALIGN="BOTTOM" BORDER="0" ALT="previous"
 SRC="@icons/latex2html/icons.gif/previous_motif.gif"></A>   
<BR>
<B> Up:</B> <A NAME="tex2html297"
 HREF="perspectives.html">No Title</A>
<B> Previous:</B> <A NAME="tex2html293"
 HREF="node23.html">Biography</A>
<BR>
<BR>


<H1><A NAME="SECTION00300000000000000000">
  About this document ... </A>
</H1> 
 <P>
This document was generated using the
<A HREF="http://www-dsed.llnl.gov/files/programs/unix/latex2html/manual/"><STRONG>LaTeX</STRONG>2<tt>HTML</tt></A> translator Version 98.1 release (February 19th, 1998)
<P>
Copyright &#169; 1993, 1994, 1995, 1996, 1997,
<A HREF="http://cbl.leeds.ac.uk/nikos/personal.html">Nikos Drakos</A>, 
Computer Based Learning Unit, University of Leeds.
<P>
The command line arguments were: <BR>
 <STRONG>latex2html</STRONG> <tt>perspectives</tt>.
<P>
The translation was initiated by A Eliens on 1998-09-05
<BR><HR>
<ADDRESS>
<I>A Eliens</I>
<BR><I>1998-09-05</I>
</ADDRESS>
</BODY>
</HTML>

