\documentstyle[options]{style}\begin{document}

 \chapter{ Abstraction and Complexity}





\section{Introduction}

The short musical phrases appearing at the beginning of each chapter
are taken from {\em The Notebook for Anna Magdalena Bach}
(which are intended primarily for young players).
Despite their apparent simplicity, however, they are acknowledged
by experienced pianists as being hard to play properly,
yet they are among the standard exercises for learner pianists.
In a way this reflects the problem of teaching object-oriented programming.
The concepts underlying object-oriented programming
may at first seem deceptively simple (and not require the complexity of C++ or
a type theoretical analysis).
However, in developing object-oriented models
some intrinsically difficult questions
remain, for which we have no definite answer
and which may even require extensive expertise and technology
to come up with a partial solution.
Returning to the music, I often find myself improvising,
leaving the written music for what it is,
a starting point.


The approach taken in this book may be characterized
as {\em abstract},
in the sense that attention is paid primarily to concepts
rather than particular details of a solution
or implementation language.
By chance, in response to a discussion in my class,
I looked up the meaning of {\em abstract}
in a dictionary, where to my surprise I learned
that one of its meanings 
is {\em to steal}, {\em to take away dishonestly}.
Jokingly, I remarked that this meaning sheds a different
light on the notion of {\em abstract data types},
but at a deeper level I recognized the extent
to which the ideas presented in this book have profited
from the ideas originally developed by others.
My rendering of these ideas in a more abstract form
is, however, not meant to appropriate them in a dishonest way,
but rather to give these ideas the credit they deserve by
fitting them in a context, a framework encompassing
both theoretical and pragmatical aspects of object-oriented computing.
As one of the meanings of the adjective {\em abstract},
the dictionary also lists the word {\em abstruse}
Jokingly, I remarked that this meaning sheds a different
light on the notion of {\em abstract data types},
but at a deeper level I recognized the extent
to which the ideas presented in this book have profited
from the ideas originally developed by others.
My rendering of these ideas in a more abstract form
is, however, not meant to appropriate them in a dishonest way,
but rather to give these ideas the credit they deserve by
fitting them in a context, a framework encompassing
both theoretical and pragmatical aspects of object-oriented computing.
As one of the meanings of the adjective {\em abstract},
the dictionary also lists the word {\em abstruse}
(not easy to understand).
There is no need to say that, within the limits of
my capabilities, I have tried to avoid becoming abstruse.







\section{Empirical Results}




\paragraph{Vitanyi}

Source: Alta Vista

\paragraph{Documents 1-10 of about 500 matching the query, best matches first}

\begin{description}

\item[\label{V0}V0]
Paul M.B. Vitanyi
http://www.cwi.nl/\verb?~?paulv/

\item[\label{V1}V1]
bionet.info-theory: Review of Li and Vitanyi's book:
http://www.krl.caltech.edu/\verb?~?brown/news/info-theory-html/0254.html 

\end{description}







\subsection{ Abstraction}

Source: Alta Vista

\paragraph{Documents 1-10 of about 9000 matching the query, best matches firs}

\begin{description}
\item[\label{A1}A1]
Abstraction
{\it bstraction is a design technique that focuses on the essential aspects of an entity and ignores or conceals less important or non-essential.. }http://actor.cs.vt.edu/\verb?~?kafura/cs2704/abstraction.html

\item[\label{A2}A2]
Branching and Abstraction
{\it Investigates the semantic equavalence of semantics. }http://theory.stanford.edu/\verb?~?rvg/abstraction/node2.html 

\item[\label{A3}A3]
Related Research: Abstraction
{\it Lists related research, for example {\em Planning: Complexity and Expressivity} and {\em Abstraction in Problem Solving and Learning} }http://ai.eecs.umich.edu/people/vanlent/www/related/abstraction.html 

\item[\label{A4}A4]
Functional Abstraction versus Structural Decomposition
{\it The behavior of a system can be analyzed in two ways. One is by decomposition into parts the behavior of which is known in isolation, The behavior of the system is described in terms of cause-and-effect relations. Another is by abstraction to a functional level and separation of the loops of interaction of interest. }http://homer.isye.gatech.edu/\verb?~?tg/courses/6219/rpg/node9.html 

\item[\label{A5}A5]
Procedural Abstraction : procedures as arguments:
{\it In Scheme, all procedures may be used as arguments to other procedures and as values returned by procedures too !  }http://www.cs.tulane.edu/www/Krishnan/Scheme/proc\_args.html 

\item[\label{A6}A6]
SEDAC--IA Thematic Guide--Flexibility, Abstraction, and Accessibility:
{\it Whatever current knowledge is integrated through an assessment project is bound to change. Flexibility to incorporate new, changed information as it becomes available may be the single most important criterion for successful integrated assessment: Among strongly modeled assessments, models with certain characteristics have big advantages in their ability to assimilate such changes in knowledge. }http://www.usf.uni-kassel.de/mirror/www.ciesin.org/sedac-2.3/iamcc.tg/TGsec3-1-4.html

\item[\label{A7}A7 ]
The Tool Abstraction Concept:
{\it The idea of the Tool Abstraction remains unexplained because this is a dead link! }http://iue.tuwien.ac.at/diss/halama/diss/node42.html -

\item[\label{A8}A8]
Painterly Abstraction in Modernist American Poetry:
{\it Infinite Incantations of Ourselves -- It argues that modernist poets have tended to resist the received values of their contemporary culture by finding idealizing principles in modes of pure abstraction. }http://www.fwi.uva.nl/research/neuro/projects/abstraction.papers.html


\end{description}\end{thebibliography}

\paragraph{Other}
\begin{thebibliography}{W1}
\bibitem{A9}
Towards a New Model of Abstraction in Software Engineering:
{\it The view of abstraction on which software engineering is based does not support the reality of practice: it suggests that abstractions hide their implementation, whereas the evidence is that this is not generally possible. This discrepancy between our basic conceptual foundations and practice appears to be at the heart of a number of portability and complexity problems.  }http://www.parc.xerox.com/spl/groups/eca/pubs/abstracts/Kiczales:IMSA92.html 

\bibitem{A10}
CS455 Manual: Abstraction:
{\it Example of Abstraction. Computer Science 455. }http://www.cec.wustl.edu/\verb?~?csm4/a8p2f1.html

\bibitem{A11}
THE LYRICAL ABSTRACTION:
{\it Lyrical abstraction is related to action painting, a style which was in the ascendant in the United States, especially trough Jackson Pollock. His is the utterance: "When I'm in my painting, I'm not aware of what I'm doing". }http://uc2.unicall.be/pmmk/mc-lyr.htm

\bibitem{A12}
Design Patterns : Abstraction and Reuse of Object-Oriented Design
{\it We propose design patterns as a new mechanism for expressing object-oriented design expertise. Design patters identify, name, and abstract common themes in object-oriented design. They capture the intent behind a design by identifying objects, their collaborations, and the distribution of responsibilities. }http://www\_is.cs.utwente.nl:8080/keulen/me/docs/ecoop93-patterns.html









\subsection{ Complexity}

Source: Alta Vista

\paragraph{Documents 1-10 of about 30000 matching the query, best matches first.}



\bibitem{C1}
Journal of Complexity
{\it The Journal of Complexity publishes original research papers that contain substantial mathematical results on complexity as broadly conceived. }http://192.215.52.3/www/catalog/cm.htm

\bibitem{C2}
Algorithms and Complexity:
{\it CS180 Algorithms and Complexity Winter 1996, UCLA (multiple entries). }http://ballade.cs.ucla.edu:8080/\verb?~?abkclass/cs180/slides/s80.html 

\bibitem{C3}
Bibliography of Measures of Complexity:
{\it Hypertext Bibliography of Measures of Complexity. by Bruce Edmonds Last updated on 24 May 1996. his is a bibliography covering some of the philosophical and practical references to the concept and measurement of Complexity. It does not attempt to cover all the new sciences that might come under the "Complexity" banner, unless they are relevant to the idea of Complexity per se. Also, ... }http://www.fmb.mmu.ac.uk/\verb?~?bruce/combib/combib.htm

\bibitem{C4}
Mathematical Complexity of Simple Economics:
{\it Relative to the physical and biological sciences, one tends to think of economics and other social and... }http://www.jhuapl.edu/colloq/math.htm

\bibitem{C5}
Complexity, Self-Organization, Emergence
{\it Complexity International is a refereed journal for scientific papers dealing with any area of complex systems research. The theme of the journal is the field of complex systems, the generation of complex behaviour from the interaction of multiple parallel processes. Relevant topics include (but are not restricted to): artificial life, cellular automata, chaos theory, control theory, evolutionary programming, fractals, genetic algorithms, information systems, neural networks, non-linear dynamics, and parallel computation. }http://www.informatik.uni-leipzig.de/privat/wellner/public\_html/emerg.html 

\bibitem{C6}
Complexity, Self-Organizing Systems, and Urban Phenomena:
{\it Sources on Complexity, Self-Organization, and Emergent Phenomena. Copyright  1995 by Daniel Serda. }http://www.mit.edu:8001/people/dserda/complex.htm

\bibitem{C7}
On the Sample Complexity of Noise-Tolerant Learning
{\it n this paper, we further characterize the complexity of noise-tolerant learning in the PAC model. Specifically, we show a general lower bound of $\Omega(\log(1/\delta)/(\epsilon(1-2\eta)^2))$ on the number of examples required for PAC learning in the presence of classification noise.  }http://theory.lcs.mit.edu/\verb?~?sed/research/IPL96-abs.html 

\bibitem{C8}
Complexity of Methods and Classes (30-Mar-1995):
{\it The choice of an implementation out of several possible alternatives is usually governed by efficiency considerations. Depending on the structure of the problem at hand, different implementations will have different runtime characteristics and space requirements. We refer to these as the (time or space) complexity. Other measures exist. }http://i44s11.info.uni-karlsruhe.de/\verb?~?frick/karla/complexity.html 

\bibitem{C9}
Computational Complexity Faculty:
{\it Members in this group are also known for their work in Numerical Analysis. Borodin, A. (Professor) Carter, M.... }
http://www.cs.utoronto.ca/DCS/Faculty/complexity.html 

\bibitem{C10}
Suzanne Beaulieu, Complexity in Management Sciences:
{\it Complexity, Society \& Liberty Conference UQTR, June 11-12, 1996.  }http://www.uqtr.uquebec.ca/complexity/abbeauli.html 

\bibitem{C11}
Complexity of Japanese Business Culture
{\it Understanding the Business Culture of Japan. Business culture, or corporate culture, is a        difficult and subtle topic to discuss. But, I try to make it a... }http://www.ict.co.jp/japa-bus/Business-culture-1.html - 

\bibitem{C12}
Automata, Formal Languages, and Complexity
{\it Since God created computer science, the standard textbook in this area has been Hopcroft and Ullman. For... }http://www.cs.uiowa.edu/\verb?~?mfleck/vision-html/handbook/automata.html 

\bibitem{C13}
Abstraction-Induced Complexity
{\it Abstraction reduces the apparent complexity of an implementation by hiding all but "the most relevant" details. However, no interface is suitable for all the users of the implementation for exactly the same reason: each user has a slightly different view of what is "most relevant." Thus, although abstractions can reduce complexity, working around their limitations can introduce other complexities. Some abstractions are designed to minimize the added complexity.  }http://www.cs.washington.edu/homes/pardo/papers.d/aic.html 

\bibitem{C14}
\bibitem{Simple Complexity:}
{\it The Little Prince by Antoine de Saint-Exupry. by. Michael Waddell.        "The combination in the book of fantasy, satire, philosophy,. }http://www.uwm.edu/\verb?~?ndell/litprn.htm

\bibitem{C15}
Complexity-related Job Offers abroad:
{\it Stellenangebote auerhalb Europas. UCSD: Mehrere Professorenstellen. Dartmouth        College: Zwei Professorenstellen. National University of Singapore:.. }http://www.informatik.uni-trier.de/\verb?~?bern/GI/BushJobs.html

\bibitem{C16}
Software Complexity
{\it The term software complexity goes back to 1960s. Myers, Yourdon and Constantine discussed and defined the term complexity:  In general, the complexity of an object is a function of the realtionships among the components of the object. }http://irb.cs.uni-magdeburg.de/se/explain/complexity.html 

\bibitem{C17}
Peter Erwin: Chaos \& Complexity:
{\it "I can't define it, but I know it when I see it."  }http://www.astro.wisc.edu/\verb?~?erwin/chaos.html

\bibitem{C18}
Increasing Complexity
{\it Increasing Complexity Of Consumer Applications. Nielsen Consumer Information Services        package analytical tools for every level of complexity -- from... }http://www.nielsen.com/home/consumer/consum12.htm













\end{document}
