<document>

:@ Techniques
In addition to the elements introduced thus far, C++
offers a number of other features that may profitably
be used in the development of libraries and programs.
\see{cc-tech-1}


<slide id=cc-tech-1 caption="C++ -- techniques (1)">
<!-- \ifsli{}{ -->
:@ Techniques
<!-- } -->
\ifsli{\vspace{0.0cm}}{}
<I
- templates <em> --
\ifsli{\n\mbox{}}{}
$template<class \: T > class \: C \: {{ ... }}$</em>
- overloading  -- <em> void $read(int);$ void $read(float)$ </em>
- friends  -- <em> to bypass protection </em>
- type conversions -- <em> by class constructors or type operators</em>
- type coercion -- <em> by explicit casts</em> (is dangerous)
- smart pointers -- <em> by overloading de-reference</em>
I>
\ifsli{\vspace{-1.5cm}}{}
</slide>

For instance, C++ offers
{\em templates}
(to define generic classes and functions),
{\em overloading}
(to define a single function for multiple types),
{\em friends}
(to bypass protection).
{\em type conversion}
(that may be defined by class constructors
or type operators),
{\em type coercions}
(or casts, that may be used to resolve ambiguity
or to escape  a too rigid typing regime),
and {\em smart pointers}
(obtained by overloading the dereference operator).
In section \ref{delegation},
examples are given of how these features may be used.
<p>
To get some of the flavor of using C++, look at the definition of
the $ctr$ class in \sliref{cc-tech-2} employing multiple constructors,
operators, default arguments and type conversion.


<slide id=cc-tech-2 caption="C++ - techniques (2)">
<!--\ifsli{
\vspace{0.1cm}
[P
class ctr { /// \fbox{C++}
public:
ctr() { n = 0; }
void operator++() { n = n + 1; }
int operator()() { return n; }
operator char*() {
	return "aCounter";
	}
private:
int n;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}{ -->
\ifsli{}{\vspace{0.1cm}}
<program>
class ctr { /// \fbox{C++}
public:
ctr() { n = 0; }
void operator++() { n = n + 1; }
int operator()() { return n; }
operator char*() { return "aCounter"; }
private:
int n;
};
</program>
\ifsli{}{\vspace{-0.2cm}}
<!--}-->
:@ Usage
\ifsli{\vspace{-0.5cm}}{}
<program>
ctr c; c++; cout << (char*) c << " is " << c();
</program>
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

The $ctr$ provides two constructors,
one with an integer argument (which is by default set to zero,
if omitted) and one with a string argument
(that expects a valid coding of an integer as a string).
The increment operator is used to define the
function $add$
(which also has a default argument to increment by one),
and the application operator is used instead of $val$.
Also, a type conversion operator is defined
to deliver the value of the $ctr$ instance anywhere where
an integer is expected.
In addition, a type conversion operator
(employing a conversion function from
integers to ($char*$) strings) is used to return
the value of the $ctr$ instance as a string.
<p>
Again, the difference is most clearly reflected in how
an instance of $ctr$ is used.
This example illustrates that C++ offers
many of the features that allow us to define
objects which may be used in a (more or less)
natural way.
In the end, this is what software development is about,
to please the user, within reason.
</document>
