<document>

:@ Terminology
The C++ language is without doubt a large and complex language.
Fortunately, an increasing number of textbooks have
become available which provide an appropriate introduction
to C++ and its use for the realization of abstract
data types, including [_Heading] and [_Weiss93].
<p>
Among the additional keywords introduced in C++
(extending C) we have the
keyword {\em const}
(that may be used to define constants),
the keyword {\em inline}
(that may be used to define inline expanded functions,
that for C have to be defined using macros),
the keyword {\em new}
(to dynamically create objects on the heap),
the keyword {\em delete}
(to destroy dynamically created objects)
and, finally,
the keywords {\em private}, {\em public} and
{\em protected}
(to indicate access restrictions for the instances
of an object class).
\see{cc-term}

<slide id=cc-term caption="C++ -- terminology (1)">
:@ Keywords:
\zline{ \fbox{C++ overview} }
*- {\bf inline}, {\bf new}, {\bf delete}, {\bf private}, {\bf protected}, {\bf public}
:@ Language features:
<I
- constructors {\em -- to create and initialize}
- destructors {\em -- to reclaim resources}
- virtual functions {\em -- dynamic binding}
- (multiple) inheritance {\em -- for refinement}
- type conversions {\em -- to express relations between types}
- private, protected, public {\em -- for access protection}
- friends {\em -- to allow for efficient access}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The language features offered by C++
supporting object-oriented programming
include
{\em constructors}
(that are defined for each class to create
and initialize instances),
{\em destructors}
(that may be used to reclaim resources),
{\em virtual functions}
(that must be used to effect dynamic binding),
{\em multiple inheritance}
(to specify behavioral refinement),
{\em type conversions}
(that allow the user to define coercion relations
between, both system-defined and user-defined,
data types),
and {\em friend declarations}
(that may be used to grant efficient access
to selected functions or classes).


<p>
The annotated reference manual (ARM) is not a book
to be used
to learn the language, but provides an excellent
source of detailed technical explanations
and the motivations underlying particular
design decisions.


<slide id=cc-term-2 caption="C++ -- terminology (2)">
\ifsli{\vspace{0.2cm}}{}
:@ Some basic terminology
\ifsli{}{\zline{[_ES90]}}\nl
%%\footnotesize
{\bf name}
{\em -- denotes an \fbox{object}, a function, set of functions, enumerator, type,
class member, template, a value or a label}<br>
<!-- \indent --> $\bullet$ introduced by a {\em declaration},<br>
<!-- \indent --> $\bullet$ used within a {\em scope}, and<br>
<!-- \indent --> $\bullet$ has a {\em type} which determines its use.<br>l
\fbox{\bf object}
{\em -- region of storage}<br>
<!-- \indent --> $\bullet$ a named object has a {\em storage class} that determines
its {\em lifetime}, and<br>
<!-- \indent --> $\bullet$ the {\em meaning} of the values found in an object
is determined by the {\em type} of the expression used to {\em access} it!
<br>
</slide>

To get an idea of the full set of features offered
by C++, look at the meaning of a name in C++ (as described
in the ARM). 
\see{cc-term-2}
A {\em name} can either denote an object,
a function, a set of functions, an enumerator, a type
(including classes, structs and unions),
a class member, a template (class or function),
a value or a label.
A name is typically introduced by a declaration,
and is used within a scope.
Moreover, each name has a type which determines its use.
An {\em object} in C++ is nothing but a
{\em region of storage},
with a lifetime determined by its storage class
(that is, whether it is created on the stack or on the
heap).
Meaning is given to an object by the type
used to access it, which is determined during compile
time.
The only information needed at runtime in C++
is concerned with virtual functions
(which require a virtual function dispatch table
for dynamic binding).
</document>
