<document>

:@ Expressions
The syntax of Smalltalk needs some
time to get used to.
Since everything is an object, 
expressions may be regarded as being
composed of constants, variables
and method expressions.
<p>
There is a large variety of literal
constants (including numbers, characters,
strings, symbols, byte arrays and
literal arrays), as depicted in \sliref{sm-expr-1}.


<slide id=sm-expr-1 caption="Smalltalk -- expressions (1)">
:@ Literal constants
\ifsli{\vspace{-0.5cm}}{}
<I
- number <em> -- $1$, $34.6$, $8r24$ </em>
- character <em> -- \$a, \$b, ... </em>
- string <em> -- "this is a string" </em>
- symbol <em> -- $\# Float$, $\# '\%\$$</em>
- byte array <em> -- $\# [0 255 2 7]$ </em>
- array of literals
\ifsli{\n\mbox{}\hspace{3cm}}{}
<em> -- $\# (12.1 \: \# (\$a \: \$b))$</em>
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Expressions may be assigned to variables.
Usually, variables are given a name that
betrays their expected type,
as for example $anInteger$.
(In Smalltalk, class names start with
an upper case and variables with a lower case letter.)

<slide id=sm-expr-2 caption="Smalltalk -- expressions (2)">
:@ Assignment
\ifsli{\vspace{-0.5cm}}{}
<I
-  $aVar := 1.$
I>
:@ Variables
\ifsli{\vspace{-0.5cm}}{}
<I
- temporary, instance, <!-- \ifsli{\n}{} --> class, pool, global
I>
@ Block expressions
\ifsli{\vspace{-0.5cm}}{}
<I>- $[ :arg | expression ]$
\ifsli{\vspace{-1cm}}{}
</slide>

We distinguish between
{\em temporary variables}
(having a method procedure scope),
{\em instance variables}
(having an object scope),
{\em class variables}
(having as their scope the
collection of instances of the class),
{\em pool variables}
(that have a category as their scope)
and
{\em global variables}
(that are visible everywhere).
\see{sm-expr-2}
<p>
A special kind of expression is the
{\em block expression} that consists
of a program fragment,
possibly parametrized with an argument.
Block expressions are used to define control structures
employing message expressions.
Block expressions correspond to function literals
(lambda-expressions) in languages such as Lisp and
Smalltalk.
<p>
Message expressions may be characterized
as either {\em unary},
{\em binary} or {\em keyword} messages.
\see{sm-expr-3}


<slide id=sm-expr-3 caption="Smalltalk -- expressions (3)">
@ Message expressions
\ifsli{\vspace{-0.5cm}}{}
<I>- unary, binary, keyword
:@ Unary
\ifsli{\vspace{-0.5cm}}{}
<I>- {\em $1.0 \: sin$, $Random \: new$}
:@ Binary
\ifsli{\vspace{-0.5cm}}{}
<I
- arithmetic {\em -- $ctr + 1$}
- comparison {\em -- $ aVar >= 200 $}
- combination {\em -- $ 100 @ 200 $}
- association {\em -- $\# Two -> 2$}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Unary messages consist of a single
method name addressed at an expression
denoting an object, for example
a constant or a class.
<p>
As binary method expressions, we have the familiar arithmetic and comparison
expressions as well as the less
familiar combination expression
(used for graphics coordinates)
and association expression
(used to define associative maps).
All binary (infix) message selectors have the 
same precedence and bind to the left.
Despite their common appearance,
these are all true message expressions
(which may lead to surprises, for example
in the case of a non-commutative definition
of the arithmetic operations).
Examples of keyword message selectors are given
in \sliref{sm-control}.
</document>
