<document>

:@ Objects
Everything in Smalltalk is an object.
An {\em object} may be regarded as consisting
of instance variables and a collection
of methods.


<slide id=sm-objects-1 caption="Smalltalk -- objects (1)">
:@ Object <em> -- behavior</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- instance variables, methods
:@ Class <em> -- description</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- class variables, class methods
:@ Self <em> -- self reference</em>
\ifsli{\vspace{-0.5cm}}{}
<I>- $super$ for ancestors
\ifsli{\vspace{-1cm}}{}
</slide>

A {\em class} is the description of
a collection of objects, its instances.
Considered as an object, a class may be
said to have class variables
and class methods.
<p>
For self-reference the special expression $self$ may be used.
To invoke methods from the parent class
the expression
$super$ may be be used.
<p>
An example of an object class description
is given in \sliref{sm-objects-2}.
The class $Ctr$ is defined as a
subclass of the class $behavior$.
It supports an {\em initialization}
protocol
(containing the method {\em initialize}),
a protocol for modification
(containing the method {\em add}),
and an {\em inspection} protocol
(containing the method {\em value}).

%%\cleartrue
<slide id=sm-objects-2 caption="Smalltalk -- objects (2)">
:@ Example <em> -- class</em>
\ifsli{\vspace{-0.5cm}}{}
<program>
Behavior subclass: $\#$Ctr
instanceVariableNames: 'value'
Ctr methodsFor: 'initialization'
   initialize
	value := 0.
Ctr methodsFor: 'modifications'
   add: Avalue
	value := value + aValue.
Ctr methodsFor: 'inspection'
   value
	$\hat{\mbox{}}$value
</program>
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Note that $value$ occurs both as
an instance variable and as a method.
Only the method is accessible by the user.

<p>
In addition, we need a class description
defining the object functionality
of Ctr,
which consists of an {\em instance
creation} protocol defining the class
method $new$.
\see{sm-objects-3}
This class description is (implicitly) an
instance of a meta class generated by the Smalltalk
system. See section \ref{meta}.

\ifsli{\clearpage}{}\vfig{1}
<slide id=sm-objects-3 caption="Smalltalk -- objects (3)">
:@ Class description <em> -- meta class</em>
\ifsli{\vspace{-0.5cm}}{}
<program>
Ctr class
instanceVariableNames: ''
Ctr class methodsFor: 'instance creation'
	new
		$\hat{\:}$super new initialize
</program>
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

</document>
