%class lex
%o 5000
%p 7000
%e 3000
%a 5000
%n 1000
%{
#include <hush/shell.h>
#include "grammar/input.h"
#include <iostream.h>
#include <grammar/string.h>
#include <grammar/converter.h>
#include <grammar/attribute.h>

int iswhite(char c) { return ( c == ' ' || c == '\t' || c == '\n' ); } 

char* firstof(char* p) {
while ( *p && iswhite(*p) ) p++;
return p;
}

static int in_ignore = 0;

static int pr=1;
static int bk=0;
static int in_comment = 0;
static int in_dollar = 0;
static int holdpr=0;
static char docstyle[1024];
static char slidestyle[] = "<document type=rawslide>";
static char rawstyle[] = "<document type=raw><format mode=text>";
string slide = "";
string id = "";
string caption = "";
string citefile = "refs";

static int rawcnt = 0;
static char filename[BUFSIZ] = "temp";

static char documents[BUFSIZ] = DOC;
static char manuals[BUFSIZ] = MAN;
static char home[BUFSIZ] = HIT;

static in_li = 0;
static in_a = 0;
static refmode=0;
static nameit=0;
static citit=0;
static refit=0;

static int authorno = 0;
static int bookmode = 0;
static int titlemode = 0;

static char tagfile[BUFSIZ];
static char reffile[BUFSIZ];

char endline[BUFSIZ];
char endbrace[BUFSIZ];

static int in_b = 0;
static int braces = 0;

static int level = 1;
static string droot = "index";
static string up = "index";
static string fwd = "index";
static string bwd = "index";

typedef ostream& OFUN(ostream&);

ostream& bcmt(ostream& os) {
	 if (in_comment==0) os << "<!--";
	 return os;
	 }

ostream& ecmt(ostream& os) {
	 if (in_comment==0) os << "-->";
	 return os;
	 }

class io {
public:
ostream* old;
ostream* o;
io(ostream* x) : o(x), next(0) { }
void cmt(char* s)  {
	print(bcmt);
	print(s);
	print(ecmt);
	}
void print(OFUN f) {
        if (o) *(o) << f;
        if (next) next->print(f);
        }
void print(char* s) {
	if (o) *(o) << s;
	if (next) next->print(s);
	}
void print(int& n) {
	if (o) *(o) << n;
	if (next) next->print(n); 
	}
void print(char& c) { 
	if (o) *(o) << c;
	if (next) next->print(c); 
	}

void flush() { o->flush(); }

void set(io* x) { next = x; }
io* child() { return next; }

void redirect(char* s = "@tempfile");
void hold() {
	o->flush();
	old = o;
	o = 0;
	}
void reset() { 
	/*
	o->flush();
	((ofstream*)o)->close();
	*/
	o = old;
	old = 0;
	}

void push(char* s = "@tempfile");

void pop() {
	cerr << "POP " << endl;
	if (next) {
		io* tmp = next->child();
		((ofstream*) (next->o))->flush();
		((ofstream*) (next->o))->close();
		delete next;
		next = tmp;
		}
	}

int oke() { return pr; /*global!!*/ }
	
private:
io* next;
};
 
inline
void io::push(char* s) {
	cerr << "PUSH " << s << endl;
	ofstream* os = new ofstream();
	os->open(s,ios::out);
	assert(!os->fail());
	io* x = new io(os);
	/*
	x->print("<!-- %%%% Start ");
	x->print(s);
	x->print(" -->");
	*/
	if (!next) next = x;
	else {
	  //cerr << "NEXT " << endl;
	  x->set(next);
	  next = x;
	  }
	}

inline
void io::redirect(char* s) {
	cerr << "RAW " << s << endl;
	o->flush();
	old = o;
	ofstream* os = new ofstream();
	os->open(s,ios::out);
	os->open(s,ios::out);
	o = os;
	}
 
io& operator<<(io& x, OFUN f) { if (x.oke()) x.print(f); return x; }
io& operator<<(io& x, char* s) { if (pr) x.print(s); return x; }
io& operator<<(io& x, int& n) { if (pr) x.print(n); return x; }
io& operator<<(io& x, char& c) { if (pr) x.print(c); return x; }
 
static io out(&cout);
static io* old = &out;
 
extern "C" system(char*);
extern "C" int atoi(char*);

inline void execsystem(char* s) {
	//cerr << "EXEC SHELL: " << s << endl;
	system( s );
	}

enum { htmlmode, textmode, slidemode };

#define ifnat if (yytext[0]!='@')
#define iftext if (mode == textmode)
#define ifslide if ( mode == slidemode )

int tex_mode = 0;

int prog=0;
int in_cmode = 0;
int in_program = 0;
int in_progcmt = 0;
int doc_no=0;
int mode = htmlmode;
int latexmode = 0;
char ext[12] = ".t";
int in_control = 0;

char buf[BUFSIZ];
%}



str \"[^\"]*\"
achar [\#\~\/A-Za-z0-9\.\-\+\=]
achars ({str}|{achar}+)* 
name [A-Za-z][A-Za-z0-9\-\_]*
attr ([ \t]*|{str}|{aname}+)
ixatts [ \t]*{achars}({achars}|[ \t])*
atts [^\>]*
prefix  (http|mailto|ftp|news)
uri {prefix}":"[^\ \t\n\,\;\)\]]*

docdirs (papers|examples|sources|tutorials|interfaces|packages|resources)
includedirs (hush|widgets|dv|sim|web|client|sgml|extern)
hushdirs (papers|projects|talks|ftp)
places   (hush|dejavu|eliens|jrvosse|bastiaan|martijn|oo|se)
orgs     ("cs.vu.nl")

lpath ("../../"|""|"../")
lcfg (resource|hush|orbix|"orbix-1.3"|"orbix-2.0")

cfg (resource|package|examples|hush|orbix|"orbix-1.3"|"orbix-2.0")

%%


"<b>"|"</b>" { out << yytext; }
"[^\\<]" { if (in_program) out << "[^\\&lt;]"; else out << yytext; }
\""<!--"\" { if (in_program) out << "\"&lt;!--\""; else out << yytext; }
\""-->"\" { if (in_program) out << "\"--&gt;\""; else out << yytext; }
\""<!"\" { if (in_program) out << "\"&lt;!\""; else out << yytext; }
\""</"\" { if (in_program) out << "\"&lt;/\""; else out << yytext; }
\"\<\" { if (in_program) out << "\"&lt;\""; else out << yytext; }



^"%/".*   { out << bcmt << yytext << ecmt; }
^"%%%".* { if (in_program) out << yytext; }
"$"[A-Za-z0-9\-\/]+"$" { 
		yytext[yyleng-1]='\0';
		out << "<em>" << yytext+1 << "</em>";
		}


{lpath}"lib.mk" {
	 char buf[BUFSIZ]; strcpy(buf,yytext);
	 buf[strlen(buf)-3]='\0';
	 if (in_program) 
		out << "<a href=\"" 
	            << documents  << "/sources/cfg/lib.mk.html\">"
		    << yytext << "</a>";
	  else out << yytext;
	  }

{lpath}{lcfg}".mk" {
	 char buf[BUFSIZ]; strcpy(buf,yytext);
	 buf[strlen(buf)-3]='\0';
	 if (in_program) 
		out << "<a href=\"" 
	            << buf << ".mk.html\">" << yytext << "</a>";
	  else out << yytext;
	  }


"module"|"interface"|"attribute"|"readonly"|"any"|"enum"|"class"|"public"|"static"|"protected"|"private"|"virtual"|"typedef"|"const"|"extern"|"operator"|"operator()"|"operator()()"|"void"|"int"|"float"|"double" {
	if (in_program) { out << "<i>" << yytext << "</i>"; }
	else out << yytext;
	}

^\%\%.*\n { }
^"\\ifsli{".* { out << bcmt << yytext << ecmt; }
^"\\noid".* { out << bcmt << yytext << ecmt; }
^"\\index".* { out << bcmt << yytext << ecmt; }
^"\\newpage".* { out << bcmt << yytext << ecmt; }
^"\\slyname{".* { out << bcmt << yytext << ecmt; }
^"\\ln".* { out << bcmt << yytext << ecmt; }
^"\\foot".* { out << bcmt << yytext << ecmt; }
^"\\noindent".* { out << bcmt << yytext << ecmt; }
^"\\clear".* { out << bcmt << yytext << ecmt; }
^"\\markit".* { out << bcmt << yytext << ecmt; }
^"\\slilarge".* { out << bcmt << yytext << ecmt; }
"\\setcounter".* { out << bcmt << yytext << ecmt; }
"\\zline".* { out << bcmt << yytext << ecmt; }
"\\vstr{"[0-9\.]+"}" { out << bcmt << yytext << ecmt; }
"\\vfig{"[0-9\.]+"}" { out << bcmt << yytext << ecmt; }
"\\label{"[A-Za-z0-9\-]+"}" {
	yytext[yyleng-1]='\0';
	out << "<a name=" << yytext+7 << ">";
	}
"\\ref{"[A-Za-z0-9\-]+"}" {
	yytext[yyleng-1]='\0';
	out << "<ref><a href=\"#" << yytext+5 <<
	"\">" << yytext+5 << "</a></ref>";
	}
"\\see{"[A-Za-z0-9\-]+"}" {
	yytext[yyleng-1]='\0';
	out << "<ref><a href=\"#Slide-" << yytext+5 <<
	"\">Slide-" << yytext+5 << "</a></ref>";
	}
"\\sliref{"[A-Za-z0-9\-]+"}" {
	yytext[yyleng-1]='\0';
	out << "<ref><a href=\"#Slide-" << yytext+8 <<
	"\">Slide-" << yytext+8 << "</a></ref>";
	}
"\\index{"[A-Za-z0-9\-]+"}" {
	out << bcmt << yytext << ecmt;
	}
"{\\tt "[\ A-Za-z\-\/\,0-9]+"}" { yytext[yyleng-1]='\0'; out << "<tt>" << yytext+5 << "</tt>"; } 
"{\\em "[\ A-Za-z\-\/\,0-9]+"}" { yytext[yyleng-1]='\0'; out << "<em>" << yytext+5 << "</em>"; } 
"{\\it "[\ A-Za-z\-\/\,0-9]+"}" { yytext[yyleng-1]='\0'; out << "<i>" << yytext+5 << "</i>"; } 
"{\\bf "[\ A-Za-z\-\/\,0-9]+"}" { yytext[yyleng-1]='\0'; out << "<b>" << yytext+5 << "</b>"; } 
"{\\rm "[\ A-Za-z\-\/\,0-9]+"}" { yytext[yyleng-1]='\0'; out << "<rm>" << yytext+5 << "</rm>"; } 
^"# "[0-9]+.*\n { ;}
"<!--" { out << bcmt; in_comment++; }
"-->" { in_comment--; out << ecmt; }
"<"[aA]" "[^>]+">" { in_a = 1; out << yytext; }
"</"[aA]">" { in_a = 0; out << yytext; }
{uri} {
	if (! in_a )
          out << "<A HREF=\"" << yytext << "\">" << yytext << "</A>";
	else
	  out << yytext;
        }
"Eliens"[0-9]+ { out << yytext; }
"\\Eliens" { out << yytext; }
"Eliens" { out << "Eli&euml;ns"; }
"Schonhage" { out << "Sch&ouml;nhage"; }
"Vitanyi"  { out << "Vit&aacute;nyi"; }
[A-Za-z0-9\-]+ {
	if (nameit) out << "<a name=" << yytext << ">" << yytext 
		      << "</a>";
	else if (citit) out << "<a href=\"" << tagfile 
		      << "#" << yytext << "\">" << yytext << "</a>";
	else if (refit) out << "<a href=\"" << reffile 
		      << "#" << yytext << "\">" << yytext << "</a>";
	else out << yytext;
	}
"<c>"   { out << bcmt << "\\c{" << ecmt; }
"</c>"  { out << bcmt << "}" << ecmt; }
"<control>" { in_control = 1; out << bcmt <<  yytext << ecmt; }
"</control>" { in_control = 0; out << bcmt <<  yytext << ecmt; }
"<?latex>"|"<latex>" { 
		  tex_mode = 1; 
		  holdpr = pr;
		  if (mode==htmlmode) pr = 0;
		  out << bcmt;
		  }
"<?/latex>"|"</latex>" {
		 tex_mode = 0;
		 out << ecmt;
		 if (mode==htmlmode) pr = holdpr;
		 }

"<ignore>" { in_ignore++; pr--; }
"</ignore>" { in_ignore--; pr++; }

"<base"{atts}">" { out << yytext; }

"<document"{atts}">" {
int tmp = pr;
if (doc_no++ == 0) {
	pr = 1;
	char buf[BUFSIZ];
	if (strlen(yytext) <= 10) {
  	   if (latexmode) 
		sprintf(docstyle,"<document type=style>");
	    else
		sprintf(docstyle,"<document type=style>");

	} else
        	sprintf(docstyle,"%s",yytext);
	out << bcmt << docstyle << ecmt;
	soinc("header","");
	}
pr = tmp;
}
"</document>" {
int tmp = pr;
if (--doc_no == 0) {
	pr = 1;
	soinc("footer","");
	out << bcmt << yytext << ecmt;
	}
pr = tmp;
}

"<abstract>" { out << "<!--<control>--><h4>Abstract</h4><!--</control>-->" << yytext; }

"@so ".*\n {
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else out << yytext;
}

"@ds ".*\n {
  out << "<program><pre>" << endl;
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; files(yytext+4,"");
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,"");
	}
  else out << yytext;
  out << "</pre></program>" << endl;
}

"<so "{atts}">" {
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; files(yytext+4,ext);
	}
  else out << yytext;
}

"<include "{atts}">" {
   int choice = 0;
   yytext[yyleng-1] = '\0';
   string dir = "";
   string ext = "";
   string file = "@temp";
   string cmd = "";
   string_converter x(yytext+8);
   for (int i = 1; i <= x.argc(); i++ ) {
       //cerr << "ATTR:" << x[i] << endl;
       attribute p(x[i]);
           if ( p.matches("file") ) { 
			if (choice !=3 ) choice = 1;
			file = p.value();
			}
           if ( p.matches("files") ) { choice = 1; file = p.value(); }
           if ( p.matches("dir") ) { choice = 2; dir = p.value(); }
           if ( p.matches("dirs") ) { choice = 2; dir = p.value(); }
           if ( p.matches("ext") ) { ext = p.value(); }
           if ( p.matches("cmd") ) { choice = 3; cmd = p.value(); }
           }
   if (choice == 0) file = yytext+9;
   if (choice == 2 && !strcmp(ext,"")) ext = ::ext;
   if (!tex_mode) {
	if (choice==2) directories(dir,ext); 
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		execsystem(buf); files(file,"");
		}
	else files(file,ext);
	}
   else if (mode != htmlmode && !in_control) {
	if (choice==2) directories(dir,ext);
	else if(choice==3) {
		char buf[BUFSIZ];
		sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		execsystem(buf); files(file,"");
		}
	else files(file,ext);
	}
   else out << yytext << ">";
}

"@cd ".*\n {
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else out << yytext;
}

"<cd "{atts}">" {
  if (!tex_mode) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else if (mode != htmlmode && !in_control) {
	yytext[yyleng-1]='\0'; directories(yytext+4,ext);
	}
  else out << yytext;
}

^"@#!".*\n {
	yytext[yyleng-1]='\0';
		execsystem(yytext+3);
			}

"///"|"//" {
	if (in_program) {
		in_progcmt = 1;
		out << yytext << "<i>";
		strcat(endline,"</i>");
		}
	else out << yytext;
	}


"<exec"{atts}">" {
       if (mode != tex_mode && !in_control) { 
		int infile = 0;
		yytext[yyleng-1] = 0;
		string cmd = "date";
		string file = "@temp";
	        string_converter x(yytext+6);
   		for (int i = 1; i <= x.argc(); i++ ) {
       		  attribute p(x[i]);
           	  if ( p.matches("cmd") ) cmd = p.value();
           	  if ( p.matches("file") )  {
				file = p.value();
				infile = 1;
				}
		}
		char buf[BUFSIZ];
		if (infile)
			sprintf(buf,"%s > %s",(char*)cmd,(char*)file);
		else
			sprintf(buf,"%s",(char*) cmd );
       		execsystem(buf);
		if (infile) files((char*)file,"");
       		out.flush();
       }
   }

^"@exec ".* { execsystem(yytext+5); out.flush(); }


^"@navigate"{atts}/("-->"|"}"|\n) |
"<navigate"{atts}">" {
	ifnat yytext[yyleng-1]='\0';
	string_converter s(yytext+9);
	int i = 1;
	for (i = 1; i <= s.argc(); i++ ) {
	  attribute p(s[i]);
	  //cerr << "ATT:" << i << " "  << s[i] << " " << p.key() << " " <<  p.value()  << endl; 
	  if ( p.matches("level") ) { level= atoi(p.value()); }
	  else if ( p.matches("up") ) { up = p.value(); }
	  else if ( p.matches("fwd") ) { fwd = p.value(); }
	  else if ( p.matches("next") ) { fwd = p.value(); }
	  else if ( p.matches("bwd") ) { bwd = p.value(); }
	  else if ( p.matches("back") ) { bwd = p.value(); }
	  else if ( p.matches("top") ) { droot = p.value(); }
	  else if ( p.matches("contents") ) { droot = p.value(); }
	  else if ( p.matches("root") ) { droot = p.value(); }
	  else if ( p.matches("index") ) { droot = p.value(); }
	}
	if (in_comment == 0) {
  	  out << "<!--<control>-->";
  	  for (i=1; i <= level; i++) out << "[";
  	  if ( ((char*) bwd)[0] =='#' )
  	     out << "-<a href=\"" << bwd << "\"><</a>";
  	  else 
  	     out << "-<a href=\"" << bwd <<  ".html\"><</a>";
  	  if ( ((char*)droot)[0] == '#' )
  	     out << "--<a href=\"" << droot << "\">%</a>";
  	  else
  	     out << "--<a href=\"" << droot << ".html\">%</a>";
  	  if ( ((char*)up)[0] == '#' )
  	     out << "--<a href=\"" << up << "\">^</a>";
  	  else
  	     out << "--<a href=\"" << up << ".html\">^</a>";
  	  if ( ((char*)fwd)[0] == '#' )
  	     out << "--<a href=\"" << fwd << "\">></a>-";
  	  else
  	     out << "--<a href=\"" << fwd << ".html\">></a>-";
  	  for (i=1; i <= level; i++) out << "]";
  	  out << "<!--</control>-->";
	}
}


"<slipar>" { ifslide pr = 1; else iftext pr = 0; }
"</slipar>" { ifslide pr = 0; else iftext pr = 1; }
"<text>" { ifslide pr = 0; else iftext pr = 1; }
"</text>" { ifslide pr = 1; else iftext pr = 0; }

"<format"{atts}">" {
	char buf[BUFSIZ];
        yytext[yyleng-1] = '\0';
        string_converter x(yytext+7);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("mode") ) 
			if (!strcmp(p.value(),"text"))
				mode = textmode;
			else if (!strcmp(p.value(),"slide"))
				mode = slidemode;
                }
	cout << "<!-- %% fm " << mode << "-->" << endl;
        }

"<slide"{atts}">" {
    if (!in_ignore) {
	ifslide pr = 1;
	char buf[BUFSIZ];
        yytext[yyleng-1] = '\0';
        string id = "X";
        string caption = "X";
        string file = "tempfile";
        string_converter x(yytext+6);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("id") ) {
				id = p.value();
				file = p.value();
				}
                if ( p.matches("caption") ) caption = p.value();
                if ( p.matches("cap") ) caption = p.value();
                if ( p.matches("file") ) file = p.value();
		if ( p.matches("dir") ) {
		  sprintf(buf,"%s/%s", p.value(),(char*)file);
		  file = buf;
		  }
                }
	sprintf(buf,"sli-%s.sli",(char*) file);
	out.push(buf);
	sprintf(buf,"sli-%s.gif",(char*) file);
	slide = buf;
	::caption = caption;
	::id = id;
	*(out.child()) << "\n" << slidestyle;
	out << "<hr><a name=" << (char*) id << ">";
	out << "<!-- <a name=Slide-" << (char*) id << ">-->";
        out << yytext;
	out << ">";
        }
}

"</slide>" { 
     if (!in_ignore) {
	    if (mode==htmlmode) {
	    	out << "<control><hr><center>"
		  << "Slide: " // << (char*)id << ": "
	          << "<a href=\"" << (char*) slide << "\">"
	          << (char*) caption << "</a></center><hr></control>";
		  }
	    out << yytext; *(out.child()) << "\n</document>"; 
	    out.pop();
	    ifslide pr = 0;
	    }
      }

"<raw>" {
	if (mode==htmlmode) {
		char buf[BUFSIZ];
		sprintf(buf,"raw-%s-%d",filename, rawcnt++);
		char file[BUFSIZ];
		sprintf(file,"%s.raw",buf);
		out << "<img src=" << buf << ".gif>";
		//out.redirect(file);
		out.hold();
		//out << bcmt;
		out.push(file);
		out << rawstyle << endl;
		}
	}

"</raw>" {
	  if (mode==htmlmode) {
	     out << "</document>" << endl;
	     out.reset();
	     out.pop();
	     //out << ecmt;
	     }
	  }

[ \t\n]*"<cite"{atts}">" {
        yytext[yyleng-1] = '\0';
	char* yy = firstof(yytext);
	int fileb = 0;
	string id = "";
        //string file = "";
        string_converter x(yy+5);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("id") ) id = p.value();
                else if ( p.matches("file") ) {
			fileb = 1; citefile = p.value();
			//cerr << "ID: " << (char*) id;
			//cerr << " TAGFILE: " << (char*) file << endl;
			}
                }
	if ( fileb )
		sprintf(tagfile,"%s.html", (char*) citefile);
        out << "&nbsp;" << yy;
	out << ">";
	citit = 1;
        }

"</cite>" { citit = 0; out << yytext; }

"<ref"{atts}">" {
        yytext[yyleng-1] = '\0';
	int fileb = 0;
	string id = "";
        string file = "";
        string_converter x(yytext+5);
        for (int i = 1; i <= x.argc(); i++ ) {
                attribute p(x[i]);
                if ( p.matches("id") ) id = p.value();
                else if ( p.matches("file") ) {
			fileb = 1; file = p.value();
			//cerr << "ID: " << (char*) id;
			//cerr << " TAGFILE: " << (char*) file << endl;
			}
                }
	if ( fileb )
		sprintf(reffile,"%s.html", (char*) file);
        out << yytext;
	out << ">";
	refit = 1;
        }

"</ref>" { refit = 0; out << yytext; }



"<display"{atts}">" { in_program=1; out << yytext << "<pre>"; } 
"</display>" { in_program=0; out << "</pre>" <<  yytext; }

"<program"{atts}">" { 
	in_program=1;
	out << bcmt << yytext << ecmt << "<pre>";
	} 
"</program>" { 
	in_program=0; 
	out << "</pre>" << bcmt << yytext << ecmt; 
	}

"<pre>" { if (!in_program) out << yytext; }
"</pre>" { if (!in_program) out << yytext; }
"<i>"|"</i>"|"<b>"|"</b>" { out << yytext; }


"<bibliography>"|"<references>" { 
        //out << "<control><h4>References</h4></control>";
	refmode = 1; out << yytext << "<dl>";
	} 
"</bibliography>"|"</references>" {
	refmode = 0; out << "</dl>" << yytext;
	} 

"<dt>" { if (refmode) nameit = 1; out << yytext; } 
"<dd>" { nameit = 0; out << yytext; } 


^[\:]?"@"[ 0-9\@\#\*\+\-\!\%].*   {
		int tmp = pr;
		char buf[BUFSIZ];
		int star = 0;
		char *py = yytext;
		char *phold = py;
		char bx[2];
		bx[0] = '\0'; bx[1] = '\0';
		if (py[0]==':') { bx[0]='*'; star=1; py++; }
		//docon=1; ln=0; testip();
		char* s = py + 2;
		switch(py[1]) {
		case '@': sprintf(buf,"<!--\\part%s{%s}-->",bx,s);
			  pr = 1;
                          out << buf;
			  if (!star && bk) {
			      sprintf(buf,"<!--\\markpart{ %s }-->",s);
			      out << buf;
			      }
			  break;
		case '#': pr = 1;
			  out << "<!--<control>--><h1>" << s << "</h1><!--</control>-->";
			  out << "<!-- \\chapter{" << s << "}-->";
			  if (!star && bk) {
			     sprintf(buf,"<!--\\markchapter{ %s }-->",s);
			     out << buf; 
			     }
			  break;
		case '%': 
			break;
		case '0': 
		case '*':
		case '1': pr = 1;
			  out << "<h1>" << s << "</h1>";
			  if (!star && bk) {
			     sprintf(buf,"<!--\\marksection{ %s }-->",s);
			     out << buf;
			     }
			  break;
		case '+':
		case '2': out << "<h2>" << s << "</h2>";
			  if (!star && bk) {
			      sprintf(buf,"<!--\\marksubsection{ %s }-->",s);
			      out << buf;
			      }
			  break;
		case '-':
		case '3': out << "<h3>" << s << "</h3>";
 			  break;
		case '4': 
		case ' ':
		          out << "<h4>" << s << "</h4>";
			  if (!star) sprintf(buf,"<!--\\index{ %s }-->",s);
			  out << buf;
			  break;
		case '!': 
		default : out << "<h4>" << s << "</h4>";
                          break;
		}
	pr = tmp;
	}

"<text>"  { ; }
"</text>" { ; }

^"<I>-" { out << "<ul><li>"; strcat(endline,"</ul>"); }
^"<*>-" { out << "<ul><li>"; strcat(endline,"</ul>"); }
^"*-" { out << "<ul><li>"; strcat(endline,"</ul>"); }
"{I " { out << "<i>"; }
" I}" { out << "</i>"; }
"{P " { out << "<tt>"; }
" P}" { out << "</tt>"; }
^"[P]" { out << "<program>\n"; strcat(endline,"</program>"); }
^"[P" { in_program=1; out << "<program><pre>"; }
^"P]" { in_program=0; out << "</pre></program>"; }
^"<I" { in_li=1; out << "<ul>"; }
^"I>" { in_li=0; out << "</ul>"; }
^"<N" { in_li=1; out << "<ol>"; }
^"N>" { in_li=0; out << "</ol>"; }
^"- "  { if (in_li) out << "<li>"; }


"[_"[A-Za-z][A-Za-z0-9\-\,]+"]" {
    if (!in_program) {
	yytext[yyleng-1] = 0;
	out << "&nbsp;[<cite><a href=\"" << tagfile 
			      << "#" << yytext+2 << "\">" << yytext+2
			      << "</a></cite>]";
    } else { out << yytext; }
}

^\%"K " { nameit = 1;  authorno = 0; out << "<dt>"; }
^\%"A " { if (!authorno++) out << "<dd>"; nameit = 0; }
^\%"T " { if (!authorno++) out << "<dd>"; nameit = 0;
	  bookmode=0; titlemode = 1; }
^\%"B " { if (!authorno++) out << "<dd>"; nameit = 0;
	  bookmode=1; out << "<i>"; titlemode = 1; }
^\%"O " { if (bookmode) out << "</i>"; titlemode = 0; }
^\%"X".* { ; }


"\\$" { out << yytext; }
"$"   {
	if (in_program && !in_progcmt) out << "\\$";
	else {
		if (in_dollar) {
			in_dollar = 0;
			out << "</math>";
		} else {
			in_dollar = 1;
			out << "<math>";
		}
	}
  }

"<" { if (in_program) out << "&lt;"; else out << yytext; }
">" { if (in_program) out << "&gt;"; else out << yytext; }
\t { out << yytext; }


"<"{includedirs}"/"[a-z0-9A-Z\-\/]+"."[hicpx]+">" { 
        char buf[512];
        strcpy(buf,yytext);
        buf[yyleng-1] = '\0';
        out << "<a href=\"" << documents << "/sources/" << buf+1
	    << ".html\">" << "&lt;" << buf+1 << "&gt;" << "</a>";
        }


\"{includedirs}"/"[a-z0-9A-Z\-\/]+"."[hicpx]+\" { 
        char buf[512];
        strcpy(buf,yytext);
        buf[yyleng-1] = '\0';
        out << "<a href=\"" << documents << "/sources/" << buf+1
	    << ".html\">" << "&lt;" << buf+1 << "&gt;" << "</a>";
        }


"<Makefile>" {
        out << "<a href=\"" << documents << "/sources/Makefile.html\">"
	    << "Makefile" << "</a>";
        }

"cfg/"[a-z0-9A-Z\-\/]+".cfg" { 
        char buf[512];
        strcpy(buf,yytext);
        out << "<a href=\"" << documents << "/sources/" << buf
	    << ".html\">" << buf << "</a>";
        }

"cfg/"[a-z0-9A-Z\-\/]+".mk" { 
        char buf[512];
        strcpy(buf,yytext+4);
        out << "<a href=\"" << documents << "/sources/cfg/" << buf
	    << ".html\">" << buf << "</a>";
        }

"<"{includedirs}"/"[a-z0-9A-Z\-\/]+".idl>" { 
        char buf[512];
        strcpy(buf,yytext);
        buf[yyleng-1] = '\0';
        out << "<a href=\"" << documents << "/interfaces/idl/" << buf+1 << ".html\">"
            << "&lt;" << buf+1 << "&gt;" << "</a>";
        }

{docdirs}":"[A-Za-z0-9\-\/\.\*]+ {
        char buf[1024];
        strcpy(buf,yytext);
	char* p = buf;
	if (buf[yyleng-1]=='*') 
		buf[yyleng-1]='\0';
	else if (buf[yyleng-1]=='/') 
		buf[yyleng-1]='\0';
	while ( *p && !(*p == ':') ) p++;
	if (*p==':') *p = '/';
	else cerr << "ERROR p is not /" << endl;
	if (buf[strlen(buf)-1]=='/')
		buf[strlen(buf)-1]='\0';
        out << "<a href=\"" << documents << "/"
	    << buf;
        if (yytext[yyleng-1]=='/')
	    		out << "/index.html";
	else if (yytext[yyleng-1]=='*')
			out << "/index.html";
	else
	    		out << ".html";
	out << "\">" << buf << "</a>";
        }

"//"{places}"/"[A-Za-z0-9\-\/]+ {
        char buf[128];
        strcpy(buf,yytext+2);
        out << "<a href=\"" << home << buf
	    << "" << "\">" << buf << "</a>";
        }

{places}"@cs.vu.nl" {
        out << "<a href=mailto:" << yytext << ">" 
	    << yytext << "</a>";
        }

(man)":"[A-Za-z0-9\-]+"("[1-9ln]")" {
        char n = yytext[yyleng-2];
        char buf[64];
        strcpy(buf,yytext+4);
        buf[yyleng-3-4]='\0';
        out << "<a href=\"" << manuals << "/man" << n << "/"
	    << buf << ".html" << "\">" << yytext+4 << "</a>";
        }



\n { out << endline; strcpy(endline,"");
     if (!titlemode) out << yytext; else out << " ";
     if (in_program) out << "  ";
     nameit = 0;
     if (!in_cmode)
     	in_progcmt = 1;
     }
. { out << yytext; }
%%

static char _argv[24][256];

int main(int argc, char* argv[]) {
int _argc = 0;
//char** _argv=argv;

argc--; argv++;

mode = 0;

strcpy(tagfile,"refs.html");
//strcpy(tagfile,"");
strcpy(endline,"");
strcpy(endbrace,"");

strcpy(documents,DOC);
strcpy(manuals,MAN);
strcpy(home,HIT);

while ((argc > 0) && (argv[0][0] == '-')) {
	if ( argv[0][1] == 'N' ) { 
		strcpy(filename,&argv[0][2]);
		cerr << "NAME = " << filename << endl;
		//argv++; argc--;
		}
	if ( ( argv[0][1] == 'I' ) || ( argv[0][1] == 'R' ) ) { 
		strcpy(_argv[_argc],argv[0]);
		cerr << "Copied " << _argv[_argc] << endl;
		_argc++;
		}
	if ( argv[0][1] == 'D' ) { 
		strcpy(documents,&argv[0][2]);
		cerr << "DOC = " << documents << endl;
		}
	if ( argv[0][1] == 'M' ) { 
		strcpy(manuals,&argv[0][2]);
		cerr << "MAN = " << manuals << endl;
		}
	if ( argv[0][1] == 'H' ) { 
		strcpy(home,&argv[0][2]);
		cerr << "HOME = " << home << endl;
		}
	if ( !strcmp( *argv, "-latex" ) ) { 
		latexmode = 1; 
		cerr << "LaTex = old" << endl;
		}
	else if ( !strcmp( *argv, "-text" ) ) { 
		mode = textmode; 
		cerr << "MODE = text" << endl;
		}
	else if ( !strcmp( *argv, "-slide" ) ) { 
		mode = slidemode; 
		cerr << "MODE = slide" << endl;
		}
	else if ( !strcmp( *argv, "-bk" ) ) { 
		bk = 1;
		cerr << "BK = true" << endl;
		}
	argc--; argv++;
	}

lex l;
//cerr << "Which paths?" << endl;
l.paths(_argc,  _argv);
//l.paths(_argc, _argv);
cerr << "ARG: " << argc << endl;
//cerr << "ht-ml 1.0  -- -I -R temporarily not supported" << endl;
if ( argc == 0 )  l(); 
else
	while (argc-- > 0) 
		l.soinc( *argv++ );
return 0;
}


//////////////////////////////

instr(char c, char *s)
{
while( (*s != '\0') && (c != *s )) s++;
if (*s=='\0') return(0); else return(1);
}
 
firn(char *s) {
int i=0;
//finn=0;
if (*s != '\n') return(0);
else {
while ((*s !='\0') && (*s == '\n') ) { i++; s++; } 
if (*s == '\n') return(i+1);
else return(i);
}
}

char *tol(char *s)
{
char *p = s;
while ( *p == ' ' && *p != '\0') p++;
while (*s != '\0') {
        if (isupper(*s)) *s = tolower(*s);
        s++;
        }
return(p);
}

