<document>
@* <a href=guide/landmark.html>Landmark papers</a>
<?latex><so guide/landmark><?/latex>

The material presented in
this book was originally based on a number of <em>landmark papers</em>.
I consider these papers still as being
important, since they originally expressed 
some important notion or concept.
Moreover, most of these papers are often referred to in other papers and
literature on OOP.
Naturally, other material has been used as well.


:@ Part I: Designing Object-Oriented Systems
In this part an introduction is given to object-oriented programming
in C++ based on [_St88]. An updated version of this paper can be found in [_St91].
As additional literature on C++ I have also consulted [_Coplien92],
which provides an excellent motivation for the use of particular idioms in C++.
[_Snyder86] contributes to the field of OOP by showing the importance of protection
mechanisms, especially in the context of inheritance.
Object-oriented design is treated on the basis of [_BC89] and [_Booch86].
Many of the insights with respect to types
and inheritance can be found in [_HOB87], which also presents
valid arguments for using types as the principle for structuring a design. 

\vfig{0.5}
<slide id=1-I caption="Part I: Designing Object-Oriented Systems">
\footnotesize
:@ Part I: Designing Object-Oriented Systems
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> B. Stroustrup -- <em> What is "Object-Oriented Programming"?</em>
<li> A. Snyder -- <em> Encapsulation and inheritance <c>in object-oriented programming languages</c></em>
<li> K. Beck and W. Cunningham -- <em> A Laboratory for Teaching Object-Oriented Thinking</em>
<li> G. Booch -- <em> Object-Oriented Development</em>
<li> D. Halbert and P. O'Brien -- <em> Using Types and Inheritance <c>in Object-Oriented Programming</c></em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

Also, we will discuss {\em contract theory} proposed originally by [_Meyer88].
In this section the focus will be primarily on the software engineering implications
of contract theory.
A treatment of the formal aspects is postponed until part III.
In addition, I will look at
the validation and testing of object-oriented programs,
and explore some possible metrics for OOP.

:@ Part II: Object-Oriented Languages and Systems
Our discussion of object-oriented languages will take the
well-known paper by [_Wegner87] as its starting point.
An overview will be given of currently existing object-oriented
programming languages and their major design goals.
Next, I will explain the distinction between delegation
and inheritance, and discuss the notion of classless
languages as, for example, exemplified in [_US87].
Further, I will discuss an alternative to the meta-programming facilities
offered by Smalltalk, and describe a general meta-level reflective
architecture of object-oriented languages based on [_Cointe87].
%%\see{1-II}

<slide id=1-II caption="Part II: Object-Oriented Languages and Systems">
:@ Part II: Object-Oriented Languages and Systems
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> P. Wegner -- <em> Dimensions of Object-Based Language Design</em>
<li> P. Cointe -- <em> Metaclasses are First Class: the ObjVlisp Model </em>
<li> D. Ungar and R. Smith -- <em> Self: The power of simplicity </em>
<li> R. Helm <em>et al.</em> -- <em> Contracts: Specifying Behavioral Compositions <c>in Object-Oriented Systems</c></em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

I will also discuss in detail the problems encountered
when extending an object-oriented language with concurrency.
An overview will be given of a number of concurrent extensions of C++.

Finally, the issues involved in specifying the
cooperation between a collection of objects are explored.
This discussion has been inspired by [_HHG90], who present
a formalism (also called {\em contracts}) to describe 
the interaction between objects.
However, I will concentrate primarily on techniques with which to model
the interaction and dependencies between objects.
Of particular relevance in this respect is the event-driven computation
model presented by [_Henderson93], which will be illustrated
by an example.

:@ Part III: Foundations of Object-Oriented Modeling
In part III we study the theoretical foundations of object-oriented 
modeling.
Our starting point will be the treatment of
an algebraic approach to abstract data types and
a discussion of
the divergence between the realization of abstract data types
in modular and object-oriented languages, based on [_Cook90].

\clearfalse
<slide id=1-III caption="Part III:  Foundations of Object-Oriented Modeling">
:@ Part III: Foundations of Object-Oriented Modeling
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> W. Cook -- <em> Object-Oriented Programming versus Abstract Data Types</em>
<li> P. Wegner and S. Zdonik -- <em> Inheritance as an incremental modification mechanism</em>
<li> L. Cardelli and P. Wegner -- <em> On understanding types, data abstraction and polymorphism</em>
<li> W. Cook <em>et al.</em> -- <em> Inheritance is not subtyping</em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

Then, I will clarify the distinction between {\em classes} and {\em types},
following [_WZ88] and, following the same source,
we will investigate the implication of this distinction on the use of inheritance.
Further, a type calculus derived from
[_Pierce93], but originally based on [_CW85], is presented
to explain in a formal setting the mechanisms of encapsulation
and inheritance.
\see{1-III}
\newpage %% XXX

In addition, I will sketch an outline of the proof-theory
needed to establish the correctness of  specifications 
obtained by the modification of a given type by means of inheritance.

:@ Part IV: Object-Oriented Application Frameworks
Taking [_Meyer90] as a starting point, we will discuss the issues
involved in the development of object-oriented libraries.
Further, I will briefly describe a number of the libraries
and toolkits available for C++.
%%\see{1-IV}

\cleartrue
<slide id=1-IV caption="Part IV: Object-Oriented Application Frameworks">
:@ Part IV: Object-Oriented Application Frameworks
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> B. Meyer -- <em>Lessons from the design of the Eiffel Libraries</em>
<li> N. Meyrowitz -- <em>Intermedia: The Architecture and Construction of an Object-Oriented Hypermedia System
and Applications Framework</em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

Hypermedia systems represent an area in which OOP has very successfully
been applied. Also, it has induced interesting research topics for OOP.
To conclude the book, I will discuss some of the recent (OOP-related)
developments in hypermedia research,
and attempt some reflections on the future of OOP.
</document>
