<document>
@* <a href=guide/paradigm.html>Paradigms of programming</a>
\label{paradigms}

<?latex>
<so guide/paradigm>
<?/latex>


Since the introduction of programmable devices, our conception of
programming has undergone radical changes.
New insights with respect to programming have been reflected in a succession of
high level programming languages, such as Algol-60, Algol-68, Simula,
Pascal, C, Modula-2, Modula-3, CLU, Ada, Ada9X and numerous others.

These programming languages were all introduced to support
some specific style of programming.
All of them have in some way set an example of a {\em good} programming style.
This is exactly what we mean by the phrase {\em paradigm of programming},
a style of programming that may somehow be considered mandatory
for developing (a particular category of) applications.
As an example, consider the use of Pascal as a vehicle for teaching
structured programming, or the use of Modula-2 as a vehicle for 
teaching the employment of abstract data types.

\vstr{2}
%%\vfig{1.5}
<slide id=1-styles caption="Styles of programming">
<h4>O{\rm bject}-O{\rm riented} P{\rm rogramming}</h4>
*- {\em high tech synonym for good }
\footnotesize
@ Styles of programming
<I
<li> A language {\em supports} a style of programming
if it provides facilities that make it convenient (easy, safe and efficient)
to use that style
I>
*- compile/runtime checks
*- clean interpretation/ orthogonal / efficient / minimal
</slide>


In a landmark paper with the title <em>"What is object-oriented programming?" </em>
(Stroustrup, 1988), Stroustrup raises the question of
when a language may be considered to
support a particular style of programming.
\see{1-styles}


In general, one can say that a language supports a particular style 
of programming if it provides facilities, both syntactic and semantic,
that makes it convenient (that is easy, safe and efficient) to use that style.
The crucial distinction that must be made
in this context is that between allowing a certain style
and providing support for that style.
Allowing means that it is possible to program in that style.
To support a given style, however, requires in addition that suitable 
compile and runtime checks are provided to enforce a proper
use of the relevant language constructs.
With these considerations in mind, one could question the assertion
that {\em Ada is object-oriented} or that {\em Modula supports abstract data types}.
Naturally, this attitude backfires with C++.
Does C++ support abstract data types
and is it really object-oriented?



<slide id=1-paradigms caption="Paradigms of programming">
@ Procedural programming
*- procedures, use the optimal algorithms
@ Modules
*- hide the data, provide functional abstractions
@ Data abstraction
*- types, provide a sufficiently complete set of operations
<h4>Object-oriented <c>programming</c>
{\em -- organize your types}
</h4>
*- make commonality explicit <c>by using inheritance</c>
<!--\ifsli{\vspace{-1cm}}{}-->
</slide>


It is equally important to establish whether a language allows a clean
interpretation of the constructs introduced, whether the constructs supporting
object orientation are {\em orthogonal} to (that is independent of)
the other constructs of the language, whether an {\em efficient} implementation
of these constructs is possible, and whether the language is kept {\em minimal},
that is without superfluous constructs.


Before establishing in more detail what the main ingredients of
object-orientation are, let us briefly look at some of the styles of programming
that may be considered as leading to an object-oriented style.
\see{1-paradigms}


In his article, Stroustrup (1988) 
stresses the continuity between the respective styles
of programming pictured in \sliref{1-paradigms}.
Each style is captured by a short phrase stating its principal concern,
that is guidelines for developing {\em good} programs.

@+ Procedural programming
The procedural style of programming is most closely related to the 
school of structured programming, of which for instance [_Dijkstra76] and
[_Gries] are important proponents.
The procedural style supports a method of program development that is
known as {\em stepwise refinement}.
Stepwise refinement is an important heuristic for developing
complex algorithms. 
Instead of writing out a complex algorithm in all its detail, the method
allows for refining the elementary steps of the basic algorithm by means
of increasingly detailed procedures.

<slide id=1-art caption="Programming as an art">
<!--\ifsli{-->
:@ Procedural programming
<!--}{\vspace{0.1cm}}-->
<program>
while ( programming == art )  {

	incr( pleasure );
	decr( bugs );
	incr( portability );
	incr( maintainability );
	incr( quality );
	incr( salary );

} // live happily ever after
</program>
\ifsli{\ln\vspace{-1cm}}{\vspace{-0.2cm}}
</slide>

As a playful example of this style of programming, consider the fragment
that may be found on the cover of [_Knuth92].
\see{1-art}
Ignoring the contents, clearly the structure shows an
algorithm that is conceived as the repeated execution of
a number of less complex steps.

@+ Data abstraction
When programs became larger and data more complex, the design of correct algorithms
was no longer the primary concern.
Rather, it became important to provide access to data in a representation
independent manner.
One of the early proponents of data hiding was Parnas (1972a,b),
who introduced a precursor to the notion of {\em data abstraction}
as it has become popular in object-oriented languages such as Smalltalk or C++.

%%@ Modules
As a language that supports data hiding, we may think of Modula-2 that offers
strong support for modules and the specification of import and export relations
between modules.
Also the {\em package construct} of Ada provides support for data hiding.
\see{1-abstraction}

Modules as provided by Modula-2 and Ada give a syntactic means for decomposing a program
into more or less independent components.
It is precisely the purely syntactic nature of modules that may be considered
the principal defect of this approach to data hiding.
Semantically, modules provide no guideline with respect to how
to decompose a program into meaningful components.

<slide id=1-abstraction caption="Data abstraction">
@ Support for data abstraction
*- Abstract Data Types {\em -- encapsulation}
@ Encapsulation
<I
- initialization
- protection
- coercions
I>
\ifsli{\vspace{-1cm}}{}
</slide>


To express the meaning of a module, we need the stronger notion of {\em types},
in the sense of {\em abstract data types} which are characterized
by a set of operations.
The notion of types, as for example supported in CLU
(Liskov and Zilles, 1974, 1974),
enables us to determine whether our decomposition satisfies
certain formal criteria.
For instance, we may ask whether we have defined sufficiently
many
operations for a given type and whether we have correctly done so.
An important advantage of using abstract data types is that we can often
find a mathematical model that formally characterizes the behavior of that type.
From the perspective of formal methods,
data abstraction by means of abstract data types may be considered as one of the
principal means for the specification and verification of complex software
systems. See also sections \ref{adt-modules} and \ref{formal-coop}.

From an implementation perspective,
to support data abstraction a language must
provide constructs to implement {\em concrete realizations}
of abstract data types.
Such support requires that means are provided to
create and initialize elements of a concrete type
in a safe way,
and that vulnerable data is effectively protected.


Very important is
the possibility of defining generic types,
that is types which take a (type) parameter
with which they are instantiated.
For example, the definition of a {\em stack} does not
differ for a stack of integers, a stack of strings
or a stack of elements from an arbitrary user-defined type.


@+ Object-oriented programming
There is a close similarity between the object model as presented earlier
and the notion of abstract data types just described.
Both objects and abstract data types define a set of applicable
operations that completely determine the behavior of an object or an
element of the data type.
To relate an object to an abstract data type we need the notion of {\em class},
that serves as the description on an abstract level of the behavior of (a collection of)
objects. (The objects are called the {\em instances} of the class.)


As noted in [_St88], abstract data types as such, although mathematically satisfying,
are rather inflexible and inconvenient for specifying complex software systems.
To attain such flexibility, we need to be able to organize
our types and express the commonality between them.
The notion of class supports this by a mechanism called {\em inheritance}.
When regarding classes as types, inheritance may be seen
as introducing polymorphic types.
A class that is derived from a particular class (the base class)
may be treated by the compiler as a subtype of (the type of)
that particular class.
\see{1-oo-support}


\cleartrue
<slide id=1-oo-support caption="Support for OOP">
@ Support for OOP
*- Polymorphism {\em -- inheritance}
@ Inheritance
<I
- dynamic binding
- protection
- multiple inheritance
I>
\ifsli{\vspace{-1cm}}{}
</slide>


Operationally, the power of inheritance comes from
message dispatching.
For C++,
this mechanism is called {\em dynamic binding}.
Message dispatching
takes care of selecting the right method in response
to a message or method call.
In a hierarchy of (derived) classes, a method for an object
may be either defined within the class
of the object itself or by one of the classes from which
that class is (directly or indirectly) derived.
Message dispatching is an essential mechanism for supporting
polymorphism, since it allows to choose the most appropriate
behavior for an object of a given type.
This must occur at runtime, since the type of an object
as determined at compile-time may be too general.


An important issue in determining whether a language
supports object-oriented programming is whether
it offers a protection mechanism to shield 
the vulnerable parts of a base class from the classes
that derived from that class.

Another question of interest is whether a language must support
multiple inheritance.
Clearly, there is some disagreement on this issue.
For example, Smalltalk-80 does not support
multiple inheritance.
The Eiffel language, on the other hand supported
multiple inheritance from its first days.
For C++, multiple inheritance was introduced
at a later stage.
At first, it was thought to be expensive
and not really necessary.
Closer analysis, however, revealed that the cost was not
excessive. (See Ellis and Stroustrup, 1990.)
The issue of multiple inheritance is still not
resolved completely.
Generally, it is acknowledged to be a powerful
and at the same time natural extension of single inheritance.
However, the inheritance mechanism itself seems to be
under attack. 
Some doubt remains as to whether inheritance is a suitable
composition mechanism when regarded from the perspective of
reuse and reliability.

</document>
