<document>
@+ Polymorphism
\label{1-poly}

Polymorphism essentially characterizes the type
of a variable, function or object.
Polymorphism may be due to overloading, parametrized types or inheritance.
Polymorphism due to inheritance is often considered
as the greatest contribution of object-oriented languages.
This may be true, but the importance of generic (template) types
and overloading should not be overlooked.

<slide id=1-polymorphism caption="Polymorphic type declarations">
\ifsli{\vspace{0.2cm}}{}
@ Overloading
<program>
extern void print(int);  /// \fbox{print}
extern void print(float);
</program>
\ifsli{\ln\vspace{-1cm}}{}
<h4> Generic class <em> -- templates</em></h4>
<program>
template< class T > class list { ... } /// \fbox{ {P list<T> P} }
list<int>* alist; 
</program>
\ifsli{\ln\vspace{-1cm}}{}
@ Polymorphism by inheritance
<program>
class shape { ... }; /// \fbox{ shape } 
class circle : public shape { ... } 
shape* s = new circle;
</program>
\ifsli{\ln\vspace{-1cm}}{}
</slide>

In \sliref{1-polymorphism} some examples are given of
declarations involving polymorphic types.
The function $print$ is separately defined for $int$ and $float$.
Also, a generic $list$ class is defined by means by employing
$templates$.
The list may be used for any kind of objects, for example integers.
Finally, a $shape$ class is defined from which a $circle$ class
is derived.
An instance of the $circle$ may be referred to by using a $shape$
pointer, because the type $shape$ encompasses $circle$ objects.


@ Inheritance and virtual functions
The power of inheritance in C++ comes from virtual
functions and dynamic binding.
Dynamic binding realizes the polymorphism inherent
in the static type structure of the inheritance 
hierarchy in a dynamic way.
Dynamic binding is illustrated 
by the example shown in \sliref{7-dispatching}.

<slide id=7-dispatching caption="Virtual functions -- dispatching">
<h4> Virtual functions <em> -- dispatching</em></h4>
<program>
class A { /// \fbox{$A$}
public:
virtual void operator()() {
	cout << "A"; 
	}
};

class B : public A { /// \fbox{$B$}
public:
virtual void operator()() {
	 cout << "B";
	 }
};

// A* a = new B(); (*a)();
// produces: B
</program>
</slide>

The class $A$ defines a virtual member function
(that results in printing $A$) which is redefined
by a similar function in class $B$ (which results
in printing $B$).
As an example of using the classes defined above,
look at the following program fragment:

[P
A* a = new B(); (*a)();
P]
In case the function would not have been defined as virtual,
the outcome of applying it to (a pointer to) a $B$ object
would have been $A$, instead of $B$.

Virtual functions that are redefined in derived classes
may still access the original version defined in the base
class, as illustrated in \sliref{7-scoping}.

<slide id=7-scoping caption="Virtual functions -- scoping">
<h4> Scoping <em> -- explicit </em></h4>
<program>
class B : public A {  /// \fbox{$B'$}
public:
virtual void operator()() {
	cout << "B";
	cout << "B";
        A::operator()(); 
	}
};

// A* a = new B(); a->A::operator()(); (*a)();
// produces: ABBA
</program>
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Scoping may be used within a member function of the
class as well as by a client (when invoking a member
function) as illustrated below.

[P
 A* a = new B(); a->A::operator()(); (*a)();
P]
The outcome of this statement is  $ABBA$.

Such a scoping mechanism is certainly not unique
to C++, although the notation for it is.
In Smalltalk, the expression $super$ may be used
to access methods defined in the ancestor class,
and in Eiffel one must use a combination of
redefining and renaming to achieve this.
As a remark, I prefer the use of
<em>operator()()</em> when any other method name would
be arbitrary.
The attractiveness of <em>operator()()</em> is
that it is used as a function application
operator, as in <em>(*a)()</em>.
</document>
