<document>
@+ Analysis

In academic environments software often seems to grow,
without a clear plan or explicit intention of
fulfilling some need or purpose,
except perhaps as a vehicle for research.
In contrast, industrial and business software
projects are usually undertaken
to meet some explicit goal or to satisfy some need.

One of the main problems in such situations,
from the point of view of the developers of the software,
is to extract the needs from the future users of the system
and later to negotiate the solutions proposed by the team.
The problem is primarily a problem of {\em communication},
of bridging the gap between two worlds,
the world of domain expertise on the one hand
and that of expertise in the craft of software
development on the other.

In a number of publications
(Coad and Yourdon, 1991a; Wirfs-Brock <i> et al.</i>, 1990; and Meyer, 1988)
object-oriented analysis has been proposed as
providing a solution to this problem of communication.
According to [_CY90], object-oriented techniques
allow us to capture the system requirements in a model
that directly corresponds with a conceptual
model of the problem domain.
\see{1-analysis}

\clearfalse
\vfig{1.0}%%
\vfig{3.0}
<slide id=1-analysis caption="Object-Oriented Analysis">
%%\sly{analysis}{\footnotesize
:@ O<rm>bject</rm> O<rm>riented</rm> A<rm>nalysis</rm>
*- <em> analysis = extracting the needs</em> 
<p>
The problem domain -- <b>complex reality </b><br>
Communication -- <em>with domain experts</em><br>
Continual change -- <em> user requirements</em><br>
Reuse -- <em>of analysis results</em>
\ifsli{\vspace{-0.5cm}}{}
</slide>


Another claim made by proponents of OOP
is that an object-oriented approach enables
a more seamless transition between the respective phases
of the software life-cycle.
If this claim is really met, this would mean
that changing user requirements could be more easily
discussed in terms of the consequences of these changes
for the system,
and if accepted could in principle be more easily
propagated to the successive phases of development.

One of the basic ideas underlying object-oriented
analysis is that the abstractions arrived at
in developing a conceptual model of the problem domain
will remain stable over time.
Hence, rather than focusing on specific functional
requirements, attention should be given to
modeling the problem domain by means of high level abstractions.
Due to the stability of these abstractions, the results
of analysis are likely candidates for reuse.



The reality to be modeled in analysis is usually very complex.
[_CY90] mention a number of
principles or mechanisms with which to manage complexity.
These show a great similarity to the abstraction
mechanisms mentioned earlier.
\see{1-complexity}

<!--\vfig{1}-->
<slide id=1-complexity caption="Managing complexity">
:@ Principles for managing complexity
<p>
<br>
<!-- \indent --> <b>Abstraction</b> -- procedural / data<br>
<!-- \indent --> <b>Encapsulation</b> -- information hiding<br>
<!-- \indent --> <b>Inheritance</b> -- expression of commonality<br>
<!-- \indent --> <b>Message passing</b> -- consistent interface<br>
<!-- \indent --> <b>Organization</b> -- object / attributes<br>
<!-- \indent --> <b>Scale</b> -- components, frameworks<br>
<!-- \indent --> <b>Behavior</b> -- actors, servers, agents
<!-- \c{\ln\vspace{-0.3cm}} -->
</slide>

Personally, I do not feel entirely comfortable
with the characterization of the analysis phase
given by [_CY90],
since to my mind
user needs and system requirements
are perhaps more conveniently phrased
in terms of functionality and constraints
than in terms of a model that may
simultaneously act as an architectural sketch of
the system that is to be developed.

However, I do agree with [_CY90], and others,
that the products of analysis,
that is the documents describing user needs
and system requirements,
should as far as possible provide a conceptual model
of the domain to which these needs and
requirements are related.

Actually, I do consider the blurring of the distinction
between analysis and design,
and as we will see later, between design and implementation,
as one of the attractive features of an object-oriented
approach.

@ Analysis methods
The phases of {\em analysis} and {\em design}
differ primarily in orientation:
during analysis the focus is on aspects of the problem
domain and the goal is to arrive at a description
of that domain to which the user and system requirements
can be related.
On the other hand, the design phase must result
in an architectural model of the system,
for which we can demonstrate that it fulfills the user
needs and the additional requirements expressed
as the result of analysis.


\vfig{4}
<slide id=1-methods caption="Analysis methods">
<ul>
<li> Functional Decomposition = Functions + Interfaces
<li> Data Flow Approach =  Data Flow + Bubbles
<li> Information  Modeling = Entities + Attributes + Relationships
<li> Object-Oriented = Objects + Inheritance + Message passing
</ul>
\ln\vspace{-0.5cm}
</slide>

[_CY90] discuss a number of methods
that are commonly used in analysis
(see slide \ref{sli-1-methods}).
The choice of a particular method will often depend
upon circumstances of a more sociological nature.
For instance, the experience of a team
with a particular method is often a crucial
factor for success.
For this reason, perhaps, an eclectic method
combining the various approaches may
be preferable
(see, for instance, Rumbaugh {\it et al.}, 1991).
However, it is doubtful whether such an approach
will have the same benefits as a purely
object-oriented approach.
See also section \ref{methods}.


I will briefly characterize the various methods
mentioned by [_CY90]. For a more extensive
description and evaluation the reader is referred
to, for example, [_Jones90].

The method of {\em Functional Decomposition}
aims at characterizing the steps that must be taken to
reach a particular goal.
These steps may be represented by functions
that may take arguments in order to deal
with data that is shared between the
successive steps of the computation.
In general, one can say that this method
is not very good for data hiding.
Another problem is that non-expert users
may not be familiar with viewing their problem
in terms of computation steps.
Also, the method does not result in descriptions
that are easily amenable to change.

The method indicated as the {\em Data Flow Approach}
aims at depicting the information flow
in a particular domain by means of arrows that
represent data and bubbles that represent
processes acting on these data.

{\em Information Modeling}
is a method that has become popular primarily
for developing information systems and applications
involving databases.
As a method, it aims at modeling the application domain
in terms of {\em entities}, that may have attributes,
and relations between entities.

An {\em object-oriented} approach to analysis
is very similar in nature to the information modeling
approach, at least with respect to its aim
of developing a conceptual model of the application domain.
However, in terms of their means, both methods differ
significantly.
The most important distinction between {\em objects},
in the sense of OOP,
and {\em entities}, as used in information modeling,
to my mind lies in the capacity of objects
to embody actual behavior, whereas entities
are of a more passive nature.

:@ Discussion
At this stage it is very difficult to draw any
final conclusions with respect to object-oriented
analysis.
Currently methods are proposed that integrate
the various techniques,
such as for example the Object Modeling Technique
(Rumbaugh {\it et al.}, 1991), which integrates information modeling,
data flow analysis and a technique for modeling
the dynamic behavior of objects.

Returning to my initial remarks,
I think we may safely set as the goal
for every method of analysis
to aim at {\em stable abstractions},
that is a conceptual model that is robust
with respect to evolving user requirements.
Also, we may state a preference for methods
which result in models that have a close
correspondence to the concepts and notions
used by the experts operating in the application
domain.
</document>
