<document>
@+ Design

In an object-oriented approach,
the distinction between {\em analysis}
and {\em design}
is primarily one of emphasis;
emphasis on modeling the reality of
the problem domain versus
emphasis on providing an architectural
model of a system that lends itself
to implementation.

One of the attractive features of such
an approach is the opportunity of a seamless
transition between the respective phases
of the software product in development.
The classical waterfall model can no
longer be considered as appropriate
for such an approach. 
An alternative model, the
{\em fountain model}, is proposed by [_Hend92].
This model allows for a more autonomous
development of software components,
within the constraints of a unifying framework.
The end goal of such a development process
may be viewed as a repository of reusable components.
A similar viewpoint has originally been
proposed by [_Cox86] and [_Meyer88].

In examining the primary goals of design,
[_Meyer88] distinguishes between {\em reusability},
{\em quality} and {\em ease of maintenance}.
Naturally, reusable software presupposes quality,
hence both quality and maintainability
are important design goals.
\see{1-design}

<slide id=1-design caption="Object-oriented design">
:@ O<rm>bject</rm>-O<rm>riented</rm> D<rm>esign</rm>
*- <em> reuse, quality, maintenance</em>
:@ The software life-cycle
[P
		Traditional Object Oriented

Requirements        20         20
Design              30         50 
Implementation      35         20 
Testing             15         10 
P]
</slide>


In [_Meyer88] a rough estimate is given of the shift in effort
between the phases of the software life-cycle,
brought about by an object-oriented approach.
Essentially, these figures show an increase
in the effort needed for design.
This is an immediate consequence
of the observation that the development
of reusable code is intrinsically more difficult.

To my mind, there is yet another reason for the extra
effort involved in design.
In practice it appears to be difficult and time
consuming to arrive at the appropriate
abstract data types for a given application.
The implementation of these structures,
on the other hand, is usually straightforward.

This is another indication that the unit of
reuse should perhaps not be small pieces of code, but
rather (the design of) components that fit into
a larger framework.

@ Software quality
From the perspective of software quality and maintenance,
these mechanisms of {\em encapsulation}
and {\em inheritance} may be characterized as powerful means
to control the complexity of the code needed to realize a system.
\see{1-quality}


<slide id=1-quality caption="Software quality">
@ Software quality
*- correctness, robustness, extendibility, reusability, compatibility
:@ Maintenance <em> -- 70\% of costs</em>
[P
Requirements change --  41.8\% 
Data format change --  17.4\% 
Emergency fixes --  12.4\% 
Routine debugging --  9\% 
Hardware changes --  6.2\%
Documentation --  5.5\% 
Efficiency improvements -- 4\%
P]
\ifsli{\vspace{-1cm}}{} 
</slide>

In [_Meyer88] it is estimated that maintenance accounts for
<math>70 \% </math>
of the actual cost of software.
Moreover, {\em adaptive maintenance},
which is the adaptation to changing requirements,
accounts for a disproportionately large part
of the cost.
Of primary importance for maintenance,
in the sense of the correction of errors,
is the {\em principle of locality}
supported by encapsulation, data abstraction and hiding.
In contrast, inheritance is a feature that may
interfere with maintenance, since it often breaks
down the protection offered by encapsulation.
%%See section 2.
However, to cope with changing requirements,
inheritance provides both a convenient and relatively
safe mechanism.
%%See also the discussion in chapter 7 and 11.
</document>
