<document>
:@ Example -- ATM (2)
\label{ATM-2}
Actually, the ATM example is an interesting example
for comparing the various approaches,
since it is used by many authors to illustrate
their methods.
In [_WWW90] the example is used for spelling out
all the steps that must be taken.
In [_Rum91] it is also extensively described
to illustrate the various 
modeling techniques employed by the method.
Also, in [_BC89] the CRC cards method is illustrated
by sketching the design of an automated teller machine.

The approaches presented in [_BC89] and [_WWW90]
are actually very closely related.
Both may be characterized as {\em responsibility-driven},
in that they concentrate on {\em responsibilities}
and {\em collaboration} relations
to model the interaction between objects.
However, the method described in [_WWW90]
is much more detailed, and to some extent includes
means to formally characterize the behavior of an object
and its interaction with other objects.
To this end it employs an informal notion of
{\em contracts} as originally introduced by [_Meyer88].

In section \ref{ATM-1} a number of candidate
classes have been suggested for our ATM.
Now, with the use of CRC cards, we will delineate
the functionality of (a number of) these classes
more precisely.
Also we will establish how the various object classes
must collaborate to perform their duties.
At the highest level of the design, we may distinguish between
two groups of classes:
the classes representing the  {\em banking model}
(comprising the class $account$ and the class $transaction$),
and the classes that model the interaction with the user
(comprising the class {\em card-reader} and the class
{\em cash-dispenser}, very important indeed).
At a lower level, we also need a class modeling
the {\em database} that provides persistent
storage for the user's account
and the information needed for authorization.
For each of these classes we will use a CRC card
to indicate their responsibilities and the classes
with which they need to collaborate.

The {\em banking model} consists of the classes $account$
and $transaction$.
The class $account$ keeps a record of the actual $balance$
of the $account$ and must allow a user to $deposit$
or $withdraw$ money.
However, for safety reasons, these operations are
never carried out directly, but are performed
by an intermediary {\em transaction} object.
\see{3-atm-2a}

<slide id=3-atm-2a cap="The ATM example (2a)">
:@ Banking model
\zline{\fbox{ATM}}
<latex>
\footnotesize
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf account} & {\it transaction  } \\
keeps balance & {\it database}  \\
deposit money & \\
withdraw money & \\
\hline
\end{tabular}}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf transaction} & {\it card-reader } \\
validation & {\it cash-dispenser} \\
performs transfer & {\it account} \\
keeps audit info & {\it database} \\
\hline
\end{tabular}}
</latex>
</slide>

The responsibilities of the {\em transaction} class
may be summarized as: the validation of user requests
and the execution of money transfers.
The responsibility for maintaining audit information is also
assigned to the {\em transaction} class.
To act as required, a {\em transaction} object
needs to communicate with a number of other objects.
It must acquire information from both the {\em card-reader}
and the {\em database} to check, for example,
whether the user has entered the right PIN code.
To validate a request, it must check whether the
account will be overdrawn or not.
To pay the requested money, it must instruct the
<em>cash-dispenser</em> to do so.
And it must contact the database to log the appropriate
audit information.
In contrast, an {\em account} only needs to respond
to the requests it receives from a {\em transaction}.
Apart from that, it must participate in committing the
transaction to the bank's {\em database}.
Note that the CRC method is non-specific about how
the collaborations are actually realized;
it is unclear which object will take the initiative.
To model these aspects we will need a more precise
notion of {\em control}
that tells us how the potential behavior
(or responsibility) of an object is activated.

The second group of classes may be called
{\em interaction classes},
since these are meant to communicate with
entities in the outside world, outside from
the perspective of the system.
Also the bank's {\em database} may be considered
as belonging to the outside world,
since it stores the information concerning the
account and the authorization of customers
in a system-independent manner.
\see{3-atm-2b}

<slide id=3-atm-2b cap="The ATM example (2b)">
:@ Interaction classes
\zline{\fbox{ATM}}
<latex>
\ifsli{\vspace{-0.5cm}}{}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf cardreader} & {\it event  } \\
signals insertion  & {\it transaction}  \\
decodes strip & \\
\hline
\end{tabular}}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf \c{cash}dispenser} & {\it event  } \\
emits cash  & {\it transaction}  \\
\hline
\end{tabular}}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf database} & {\it event } \\
retrieves account & {\it transaction} \\
records transaction & {\it account} \\
authorization & {\it database} \\
\hline
\end{tabular}}
\n
</latex>
</slide>


Both the {\em card-reader} and the {\em cash-dispenser}
rely on a class called {\em event},
which is needed to model the actions of the user.
For example, when a user inserts a bankcard,
we expect a transaction to start.
For this to happen, we must presuppose an underlying system
that dispatches the event to the {\em card-reader},
which in turn notifies the {\em teller machine}
that a new transaction is to take place.
See also section \ref{cooperation} for a comparative study of
such mechanisms.
The flow of control between a {\em transaction} object
and the {\em cash-dispenser} is far more straightforward,
since a {\em transaction} object only needs to
issue the appropriate instruction.
However, the actual interaction between the
{\em cash-dispenser} and the underlying hardware,
that turns out the money, may be quite intricate.

The {\em database} may either respond directly to the request
coming from the {\em account} or {\em transaction} object
or it may respond to {\em events}
by taking the initiative to call the appropriate methods
of the {\em account} and {\em transaction} objects.
Whether the {\em database} may be accessed directly
or will only react to
events is actually dependent on the
control model we assume when developing the system model.
</document>
