<document>
@+ Responsibilities and collaborations
\label{CRC}

Design is to a large extent a matter of creative thinking.
Heuristics such as performing a linguistic scan
on the requirements document for finding objects
(nouns), methods (verbs) and attributes (adjectives)
may be helpful, but will hopelessly fail
when not applied with good taste and judgement.
Not surprisingly, one of the classical techniques
of creative writing, namely the {\em shoe-box method},
has reappeared in the guise of an object-oriented
development method.
The shoe-box method consists of writing fragments
and ideas on note cards and storing them in a (shoe) box,
so that they may later be retrieved and manipulated
to find a suitable ordering for the
presentation of the material.
To find a proper decomposition into objects,
the method creates for each potential
(object) class a so-called CRC card,
which lists the $Class$ name, the $Responsibilities$
and the possible $Collaborators$ of the proposed class.
In a number of iterations, a collection of cards
will result that more or less reflects the
structure of the intended system.

According to the authors (see Beck and Cunningham, 1989), the method
effectively supports the early stages of design,
especially when working in small groups.
An intrinsic part of the method consists
of what the authors call {\em dynamic simulation}.
To test whether a given collection of cards
adequately characterizes the functionality of the intended
system, the cards may be used to simulate
the behavior of the system.
When working in a group, the cards may be distributed
among the members of the group, who participate
in the simulation game according to their cards.
\see{3-crc}


%%\comment{See [_BC89].}
<slide id=3-crc cap="The CRC method">
:@ O{\rm bject}-O{\rm riented} T{\rm hinking}
\zline{\fbox{CRC}}
<latex>
\nl
\ifsli{\vspace{-0.5cm}}{
{\footnotesize
\fbox{\parbox[t]{8.5cm}{Perspective: \it immerse the reader in the {\em object-ness} of the material.}}\nl
}
$[]$ Give up global knowledge of control.\n
$[]$ Rely on the local knowledge of objects.
}
%%to accomplish their tasks.
%%\n\mbox{}\n
%%\vspace{-1cm}
@ OO-Design with CRC cards
*- {\bf C}lass, {\bf R}esponsibility, {\bf C}ollaborators.
\mbox{}\hspace{1cm}
\ifsli{
\begin{tabular}{|p{4.5cm}|p{4cm}|}
\hline
\fbox{\bf Classname} & {\it Collaborators} \\
Responsibilities & \vspace{1cm} \\ \hline
\end{tabular}
}{
\begin{tabular}{|p{4cm}|p{3cm}|}
\hline
\fbox{\bf Classname} & {\it Collaborators} \\
Responsibilities & \vspace{1cm} \\ \hline
\end{tabular}
}
{\footnotesize
\nln
{\bf Regressions:} global variables, pointers, reliance on the implementation
of objects.
}\n
</latex>
</slide>

A number of authors have adopted the method,
or developed a method very similar,
for identifying objects and characterizing
their functionality in an abstract way.
It is doubtful, however, whether the method has
any significance beyond the early stages of analysis and design.
Without any more formal means to verify whether the
responsibilities listed adequately characterize
the intended functionality of the system,
the method amounts to not much more than brainstorming.
Clearly, the method needs to be complemented
by more formal means to establish whether the (implicit)
protocols of interaction between the objects
satisfy the behavioral requirements
of the system.

Nevertheless, the elegant simplicity of the method
is appealing, and the card format lends itself to
easy incorporation in an on-line documentation system.
Moreover, since the method imposes no strict order,
and has relatively little overhead, it is indeed
a good way in which to get an initial idea of
what objects the system will comprise.


@so atm-2
</document>
