<document>
@+ Modeling heuristics

Following [_Booch86], we have characterized objects as  `crisp'
{\em entities} that {\em suffer} and {\em require} 
actions.
From the perspective of system development,
{\em objects} must primarily be regarded as 
{\em computational entities}, embodying the means by which we may express
a computation.
Modeling a particular problem domain, then,
means defining abstractions in terms of {\em objects},
capturing the functional characteristics 
of that domain.
The question is, how do we arrive at such a model?
\see{3-booch}


<slide id=3-booch cap="The Booch method">
@ Objects <em> -- crisp entities</em>
*- {\em object} = an entity that {\em suffers} and {\em requires} actions
\ifsli{\vspace{-0.5cm}}{}
:@ The method:
\ifsli{\vspace{-1cm}}{}
<I
- $[1]$ Identify the objects and their attributes\n
- $[2]$ Identify operations suffered and required\n
- $[3,4]$ Establish visibility/interface
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In [_Booch86], a straightforward method of object oriented
development is proposed,
which consists of the successive identification of
objects and their attributes, followed by
a precise characterization of the interobject
visibility relations.
In [_Booch91], a shift of emphasis has occurred
towards determining the semantics of an individual object
and the interaction between collections of objects.
[_Booch91] provides extensive examples to illustrate
the method, of which perhaps the most notable
aspect is a graphic notation for describing the 
modular structure of a system.


<slide id=3-heuristics cap="Heuristics for modeling">
:@ Heuristics
\ifsli{\vspace{-0.5cm}}{}
*- {\em model of reality} -- balance {\em nouns} (objects) and {\em verbs} (operations)
:@ Associations
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em directed action} -- drives, instructs
- {\em communication} -- talks-to, tells, instructs
- {\em ownership} -- has, part-of
- {\em resemblance} -- like, is-a
- {\em others} -- works-for, married-to
I>
\ifsli{\vspace{-1cm}}{}
</slide>

As a heuristic to arrive at the proper abstractions
of the problem domain (in terms of object classes),
[_Booch86] proposes scanning the requirements document
for $nouns$, $verbs$ and $adjectives$,
and using these as initial suggestions for respectively
$objects$, and $operations$ and $attributes$
belonging to objects (see \sliref{3-heuristics}).
This technique has been adopted and augmented
by a number of other authors, among which
are [_WWW90] and [_Rum91].
For example [_WWW90] illustrate the technique in fine
detail in several examples, including the design of an
automated teller machine and a document processing
system.

In addition to the interpretation of nouns as
possible objects, verbs as possible operations on objects,
and adjectives as possible attributes of objects,
[_Rum91] suggest this technique to determine
other relations and associations between object classes
as well.
For instance, a model of control and object
interaction may be suggested by phrases
indicating directed action or communication.
Similarly, structural issues, such as whether
an object owns another object or whether
inheritance should be used, may be decided
on the basis of resemblance or 
subordination relations.

@so atm-1
</document>
