<document>
@* <a href=g-lines.html>Design guidelines</a>
\label{3-style}
\label{guidelines}
%%\comment{See [_LH89].}


Computing is a relatively young discipline.
Despite its short history, a number of styles
and schools promoting a particular style have emerged.
However, in contrast to other disciplines
such as the fine arts (including architecture)
and musical composition, there is no
well-established tradition of what is to be considered
as {\em good taste} with respect to software design.
There is even a debate whether software design
must be looked at as an {\em art} or must be promoted
into a {\em science}. See for example [_Knuth92] and [_Gries].
The debate has certainly resulted in new technology
but has not, I am afraid, resulted in 
universally valid design guidelines.


The notion of {\em good design} in the other
disciplines is usually implicitly defined by
a collection of examples of good design,
as preserved in museums or (art or music) historian works.
For software design, we are still a long way from
anything like a museum, setting the standards of good design.
Nevertheless, a compendium of examples of object
oriented applications such as [_Pinson90] and [_Harmon93],
if perhaps not setting the standards
for good design, may certainly be instructive.

<slide id=3-design-criteria cap="Criteria for design">
@ Development process {\em -- cognitive factors}
*- model -->  realize  -->  refine
@ Design criteria <em> -- natural, flexible, reusable</em>
<I
- abstraction {\em -- types}
- modularity {\em -- strong cohesion} (class)
- structure {\em -- subtyping}
- information hiding {\em -- narrow interfaces}
- complexity {\em -- weak coupling}
I>
</slide>

The software engineering literature abounds
with advice and tools to measure the quality of
good design.
In \sliref{3-design-criteria}, a number of the criteria commonly found
in software engineering texts is listed.
In software design, we evidently
strive for a high level of abstraction
(as enabled by a notion of types and a corresponding
notion of {\em contracts}),
a modular structure with strongly cohesive units
(as supported by the class construct),
with units interrelated in a precisely
defined way (for instance by a client/server or subtype
relation).
Other desirable properties are,
a high degree of information hiding (that is narrowly
defined and yet complete interfaces),
and a low level of complexity (which may be achieved
with units that have only weak coupling, as supported
by the client/server model).
An impressive list, indeed.
In practice, however, we see that (object oriented)
software design is more a matter of {\em give and take}.

Design is a human process, in which {\em cognitive factors}
play a critical role.
The role of cognitive factors is reflected
in the so-called {\em fractal design process model}
introduced in [_JF88], which describes object oriented
development as a triangle with bases labeled by
the phrases $model$, $realize$ and $refine$.
This triangle may be iterated at each of the bases,
and so on.
The iterative view of software development
does justice to the importance of human understanding,
since it allows for a simultaneous understanding
of the problem domain and the mechanisms needed
to model the domain and the system architecture.

Good design involves taste.
My personal definition of good design would
certainly also involve cognitive factors {\em --
is the design understandable --}, including
subjective criteria such as {\em -- is it pleasant
to read or study the design --}.
But rather than expressing these criteria explicitly,
I would like to point at a collection of examples
of good (object oriented) software design.

Lacking such a collection, we will instead discuss some
of the issues that may arise in developing a design.
First, we will look at the distinction between
structural and behavioral encapsulation
(which rests primarily on a difference in background
rather than goals).
Next, we will introduce the notion of {\em abstract systems}
and we will present a methodology based on
event-driven programming to employ
abstract systems in modeling interactions
between objects.
And, we will conclude this section by discussing the
tradeoffs involved
in designing for reuse and individual class design.
We will present an example illustrating the tradeoffs
of using either ones of the mechanisms of
inheritance, delegation and templates in developing
(user-defined) abstract data types,
and we will give some guidelines for
detailed class design.

@so structural
@so systems
@so reuse
</document>
