<document>
@+ Perspectives of modeling
\label{perspectives}
\label{3-model}

Understanding a problem domain may be quite demanding.
Understanding is even more difficult when the 
description of the domain is cast in some
representation pertaining to the solution domain.
An object-oriented approach is said to require less
translation from the problem domain to the (software)
solution domain, thus making understanding easier.
Many proponents of an object-oriented approach, however,
seem to be overly optimistic in their conception of
the modeling task.
From an epistemological point of view,
modeling may be regarded as being essentially colored
by the mechanisms that are available to express the model.
Hence, rather than opposing the functional and object-oriented 
approach by claiming that an object-oriented
approach aims at {\em modeling reality},
I would prefer to characterize the distinction in
terms of (modeling from) a different
vernacular, a different perspective due to
different modeling mechanisms.
In other words, a model is meant to capture
some salient aspects of a system or problem domain.
Dependent on what features are considered as most
important, different means will be chosen to construct
a model.

Even within the confines of an object-oriented approach, there appear to be 
radically different perspectives of the modeling required
in the various phases of the software life-cycle.


<slide id=3-perspectives cap="Perspectives of modeling">
@ Modeling reality <em> -- vernacular</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- requirements -- {\em use cases}
- analysis -- {\em domain concepts}
- design -- {\em system architecture}
- implementation -- {\em language support}
>
@ Design model {\em -- system oriented}
\ifsli{\vspace{-0.5cm}}{}
*- provides a justification of the architecture
\ifsli{\vspace{-1cm}}{}
</slide>

An important contribution of [_Jacobs92]
is the notion of {\em use cases}
that describe the situations in which a user actually
interacts with the system.
Such a (use case) model is an important constituent of
the {\em requirements} document, since it precisely
describes what the system is intended for.
For the purpose of analysis,
it may be helpful
to develop a more encompassing (conceptual) model
of the problem domain.
The advantage of such an approach is that the actual system
may later easily be extended due to the generality
of the underlying analysis model.

In contrast to the model used in analysis,
both the design model and the implementation model are more
{\em solution oriented} than {\em domain oriented}.
The {\em implementation} model is clearly dependent on the available
language support.
Within a traditional life-cycle, the {\em design} model may be seen as
a transition from analysis to implementation.
The notion of {\em objects} may act as a unifying factor,
relating the concepts described in the analysis document
to the components around which the design model is built.
However, as we have noted, object-oriented development
does not necessarily follow the course
of a traditional software life-cycle.
Alternatively, we may characterize the function of the 
design document as a {\em justification} of the choices
made in deciding on the final architecture of the system.
This remark holds in so far as an object-oriented approach
is adopted for both design and implementation.
However, see [_Hend90] for a variety of combinations of 
structured, functional and object-oriented techniques.

@ Dimensions of modeling
When restricting ourselves to {\em design models},
we may again distinguish between different modeling
perspectives or, which is perhaps more adequate
in this context, {\em dimensions of modeling}.

In [_Rum91], it is proposed to use three complementary
models for describing the architecture and functionality
of a system.
\see{3-dimensions}

<slide id=3-dimensions cap="The OMT method">
:@ Dimensions of modeling {\em -- OMT}
\ifsli{\vspace{-0.5cm}}{}
<I
- object model {\em -- decomposition into objects}
- dynamic model {\em -- intra-object state changes}
- functional model {\em -- object interaction} (data-flow)
I>
@ Model of control
\ifsli{\vspace{-0.5cm}}{}
*- procedure-driven, event-driven, concurrent
\ifsli{\vspace{-1cm}}{}
</slide>

The OMT method distinguishes between
an {\em object model}, for describing the (static)
structure of object classes and their relations,
a {\em dynamic model}, that describes for each object
class the state changes resulting from performing operations,
and a {\em functional model}, that describes the interaction
between objects in terms of a {\em data-flow} graph.

An important contribution of [_Rum91]
is that it identifies a number of commonly used
{\em control models}, including {\em procedure-driven}
control, {\em event-driven} control and {\em concurrent}
control.
The choice for a particular control model
may deeply affect the design of the system.
See section \ref{duality} for a further discussion.

The OMT approach may be called a {\em hybrid method}
since it employs non object-oriented techniques
for describing intra-object dynamics, namely state-charts,
and a functional approach involving data-flow diagrams,
for describing inter-object communication.

@so coherent
</document>
