<document>
\label{3-mvc}
\label{MVC}
\index{MVC}
:@ Example -- the MVC paradigm
A significant part of the effort of system development
often goes into the design and implementation of
the user interface.
The general feeling with respect to user interface components
is that these clutter up the design.
Nevertheless, as the example of the automated teller machine shows,
interaction with the user may be an intrinsic
part of the system.

In [_BC89], the CRC cards method has also been used
to characterize the so-called {\em Model-View-Controller}
paradigm,
which has become popular as 
an architectural device for developing
user interfaces.
The {\em MVC} paradigm allows us to develop a user
interface in a non-intrusive manner,
that is without cluttering the application code.
The idea is to provide standard (base) classes
from which application-specific classes, that
implement the particular details, may be derived.
\see{3-mvc}


\cleartrue
<slide id=3-mvc cap="The Model-View-Controller paradigm">
:@ Model, View &amp; Control <em> -- user interface design paradigm</em>
<latex>
\footnotesize
\ifsli{\vspace{-1cm}}{}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf Model} & {\it  } \\
Maintain problem related info.\nl Broadcast change information. &  ... \\ \hline
\end{tabular}}
\ifsli{\vspace{-0.5cm}}{}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf View} & {\it  } \\
Render the Model.\nl Transform coordinates. & Controller \n Model \\ \hline
\end{tabular}}
\ifsli{\vspace{-0.5cm}}{}
\ln
{\mbox{}\hspace{1cm}\begin{tabular}{|p{4cm}|p{3cm}|} \hline
\fbox{\bf Controller} & {\it  } \\
Interpret user input.\nl Distribute control. & View \n Model \\ \hline
\end{tabular}}
\ifsli{\vspace{-1.5cm}}{}
\ln
</latex>
</slide>

The class derived from the {\em Model} class
is intended to maintain the problem-related
information.
The only thing a {\em model} object needs to do
is notify the other (dependent) objects when
some of the information it carries has changed.
In response to being notified of a change, the
{\em view} object (an instance of
the class which is derived from {\em View})
may request what it needs to know to
refresh the display of the contents
of the {\em model} object.
A {\em controller} object is employed to interact
with the user.
Any relevant input is transferred to the {\em model} object,
that may in its turn notify the {\em view} object
of a change.

For the automated teller machine, the $account$ class
will be the obvious choice for being derived from
the $Model$ class.
Clearly, the {\em card-reader} class will have
$Controller$ as its base class and the $screen$ class
will be derived from $View$.

An important advantage of using the {\em MVC}-paradigm
is that it allows the programmer to
separate the problem-related code from
the code needed to implement the (often intricate)
details of interacting with the various input and
output devices.
Another advantage lies in the flexibility of the approach.
One $model$ object may be associated with
arbitrary many different {\em view-controller} pairs,
simply by adding another $view$ object to the
list of dependent objects that will be notified of
a change.
Since each $view$ class specifies what it needs
to know of the $model$, the $model$ class
itself will remain unaffected.

Although the {\em MVC}-paradigm was
introduced in the Smalltalk-80 environment,
similar interaction protocols have been used
in user interface libraries for C++, for example
the Interviews library (see Linton {\it et al.}, 1989).
See also sections \ref{cooperation} and \ref{libraries}.
From the perspective of design, the paradigm is
attractive since it may be applied in a cookbook
fashion to implement intricate user interfaces,
often simply by reusing available code.
See [_KP88] for a number of example recipes.
In section \ref{cooperation} we will further explore
the interaction protocol underlying the
{\em MVC}-paradigm.
</document>
