<document>
@+ Object roles and interaction

Objects rarely live in isolation.
In a system of some complexity, 
a number of different kinds of object
classes may usually be distinguished.
Each kind of class may be regarded as playing
a specific role in the system.
For example, when considering our ATM,
classes such as {\em card-reader} and {\em cash-dispenser}
are of a completely different kind, and play a completely
different role, than the classes
{\em account} and {\em database} for instance, or the classes
{\em event} and {\em transaction}.
Often it will take some
experimentation to decide how control must
be distributed among the objects comprising the system.
Although
the framework chosen for the development of the system
may partly determine the control model,
there will usually be ample choice left for the designer of
the system to define the interactions between objects.
}

\clearfalse
<slide id=3-roles cap="Object roles">
@ Object roles
\ifsli{\vspace{-0.5cm}}{}	
<I
- {\em actor} -- operates, \ifsli{\n}{} (suffers no operations)
- {\em server} -- suffers operations
- {\em agent} -- suffers and operates ({\em actor \& server})
I>
\ifsli{\vspace{-1cm}}{}
</slide>

An important function of the design document
is to elucidate the role of each object class
in the system, and to point out
how the objects cooperate to complete the task.
In [_Booch86], a distinction is made between objects
that suffer no operations (actors),
objects that only suffer operations (servers)
and objects that both suffer and require operations
(agents).
Such a characterization in terms of {\em initiative}
may give a first indication of
the role an object plays in the system.
For example, the $account$ class in our ATM example
is best characterized as a {\em server} class,
whereas the {\em transaction class} may be regarded,
in the terminology of [_Booch86], as an actor
class, since it actively controls the computation.
In many cases, the software control model
adopted will also influence the way in which individual
objects are supposed to behave.
\see{3-roles}


With respect to a global view of
the system, it is necessary to ensure that each object class
is completely defined,
that is to establish that each class provides a
sufficiently complete
method interface.
In [_Booch86], a characterization is given of the
kinds of methods that may occur in an interface.
These include methods to create or destroy an
object, methods to modify the state of an object and methods
that only provide information on the state of
an object, or parts thereof.

\clearfalse
<slide id=3-trip cap="Round trip gestalt class design">
\slilarge
@ Class design <em> -- round trip gestalt</em>
\ifsli{\vspace{-1cm}}{}
[D 
analyze a little,
design a little,
implement a little,
test a little...
D]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Before being able to make final decisions 
with respect to the functionality of a class, however,
it is generally necessary to get a clear
overall picture of the system first.
This requires what [_Booch86] characterizes
as {\em round trip gestalt} design, which in other
words expresses the need to {\em
analyze a little,
design a little,
implement a little,
test a little...}
(The notion of {\em gestalt} comes from
perception psychology, where it means a global
perceptual configuration emerging from the background.)

In the next section we will consider 
formal means to verify the correctness of our design.
However, before that we will look at an example
of a design paradigm that allows us to assign
specific roles to object classes and to
specify (in an implicit manner) the protocol
of interaction between instances of these classes.

@so mvc
</document>
