<document>
\label{del-inh}

:@ Example
When developing a class structure,
often the choice will arise as to whether to employ inheritance
or explicit delegation as a means to
utilize the functionality of a particular class.

<slide id=3-deleg cap="Sharing resources">
@ Resource sharing <em> -- delegation or inheritance</em>
<I
- <tt>class line : public list<string> { ... }</tt>
- <tt>class line : private list<string> { ... }</tt>
- <tt>class line { list<string> words; ... }</tt>
I>
</slide>

The choices a designer may be confronted with when
developing a system or a class library may be illustrated
by a simple example.
Suppose you must develop a system that produces 
a {\em kwicindex}.
A {\em kwicindex} contains for each line of text
all possible rotations, so that each word can be
rapidly searched for, together with the context in which 
it occurs.
The example is taken from [_VanVliet], who discusses
a number of alternative solutions from the
perspective of information hiding.
An object-oriented solution appears to be surprisingly
simple, in particular when a suitable library is available.
The two classes that are of interest are, respectively,
a class $line$, to represent each line of text,
and a class $index$ to represent the lines
generated for the {\em kwicindex}. 

<slide id=3-kwic-1 cap="Representing lines">
\ifsli{}{\vspace{0.1cm}}
@ds lib/kwicindex/line.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The class $line$ is simply a list of words (strings)
that can be rotated.
The actual implementation of a $line$ is
provided by an instantiation of
a generic (template) class $list$,
that is contained as a data member,
to which the operations $insert$ and $rotate$
are delegated.

Instead of incorporating a member of type $list<string>$,
the $line$ class could have been made a descendant
of the class $list<string>$, as indicated below
[P
class line : public list<string> {
...
};
P]
However, this would make the type $line$ a subtype
of $list<string>$, which does not seem to be a wise decision.
Alternatively, private inheritance could have been employed
as in
[P
class line : private list<string> {
...
};
P]
But, as we have noted before, private inheritance
may also introduce unwanted dependencies.
Explicit delegation seems to be the best solution,
because the relation between a $line$ object
and a {\em list of strings} is best characterized
as a {\em uses} relation.
}

\cleartrue
<slide id=3-kwic-2 cap="Generating the index">
\ifsli{}{\vspace{0.1cm}}
@ds lib/kwicindex/index.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

For the $index$ class, a similar argument
as for the $line$ class can be given as to
why delegation is used instead of public or private
inheritance.

Actually, the most difficult part of the code
is the function defining how to compare two lines
of text that is needed for sorting the $index$.
Comparison is based on the lexicographical order
as defined below.
@ds kwicindex/lexicographic.s

\noid
The ease of implementing both the $line$ and $index$ 
classes depends largely upon the availability of a
powerful $list$ template class.
In the example, we employed
the functions $list::rotate$ to rotate
the elements in the list, and
$list::sort$ to sort the elements in the list.
The function $list::compare$ was used
to determine the $<=$ relation between elements
of the list.
The actual library employed for this example
is the $splash$ library, which is discussed in
section \ref{libraries}.
</document>
