<document>

@+ Template classes

Template classes are supported only in the later releases of C++
(AT\&T 3.0, Zortech 3.1).
Before the actual support of template classes, and even with the support for template
classes available, programmers have extensively used the macro preprocessor
to define generic classes.
A disadvantage of such an approach is that it easily leads to deviant
notation and the use of non-standard constructs or preprocessor
facilities.
Another important disadvantage of using the preprocessor
is that there is no direct compiler support for checking
whether a template has correctly been defined and instantiated.
And similar objections hold equally as strong for defining constants,
inline functions or template functions as macros.
In \sliref{2-template},
the example of a generic stack has been rewritten 
to employ template classes.

\index{template class}
%%\cleartrue
<slide id=2-template caption="Template class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/stacks/st-1.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

One (subtle) difference between the previous definition, using $void$
pointers, and the current definition that uses templates is that
an instantiated template stack may hold objects (or references to objects),
whereas the other version only allows the storage of pointers to objects.
See section \ref{2-references} for a discussion on the difference between objects, references
and pointers.
<p>
Another, very important, difference is illustrated by the code fragment showing
the use of a (template) stack
<!--\nslide{}{}{-->
@ds lib/stacks/st-1.i
<!--}-->
<p>
When creating the stack, the user must explicitly indicate
what type of elements the stack contains.
Other uses of the stack do not require any explicit type conversions
and are completely type checked by the compiler.
Note that the template construct actually adds to the power
of the language, since a stack of $void$ pointers may easily
be defined by giving the right instantiation parameters.


<so gen-list>
</document>
