<document>
@+ Memory management

Perhaps the most annoying feature of
C++ (or rather absence of it)
is memory management.
Whereas both Smalltalk and Eiffel
offer automatic garbage collection,
the C++ programmer is required
to rely on hand-crafted memory management.

<slide id=2-m-1 caption="Constructors and destructors">
\ifsli{}{\vspace{0.1cm}}
<program><include file=lib/d-1.s></program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Memory management in C++ involves
the use of constructors and destructors.
In the following, we will look
at some examples illustrating the order 
of invocation of constructors and destructors
in relation to single and multiple inheritance.

<p>
The first example, given in \sliref{2-m-1},
defines two classes ($A$ and $B$,
with $B$ derived from $A$),
each having a constructor and destructor
writing the name of the class to standard output.
An example of their use is:
<!--\nslide{}{}{-->
[P
A* a = new B; delete a; ///// ABA
B* b = new B; delete b; ///// ABBA
P]
<!--}-->

\noid
Recall that when creating an instance of
a class, the constructors of the base classes
(if any) are called first.
This is exactly what happens above.
However, contrary to what is expected,
when deleting $a$, the destructor for $B$
is not called, whereas it is invoked when deleting
$b$.


<slide id=2-m-2 caption="Virtual destructors">
\ifsli{}{\vspace{0.1cm}}
<program><include file=lib/d-2.s></program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The remedy to this is to declare the destructor
of $A$ virtual, as in \sliref{2-m-2}, since it dynamically
invokes the destructor declared for the
actual class type $(B)$ of the object referenced.
The program fragment
<!--\noslide{}{}{-->
[P
A* a = new B; delete a; ///// ABBA
B* b = new B; delete b; ///// ABBA
P]
<!--}-->

\noid
now behaves as desired.


<slide id=2-m-3 caption="Multiple inheritance">
\ifsli{}{\vspace{0.1cm}}
<program><include file=lib/d-m.s></program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

@ Multiple inheritance
When employing multiple inheritance,
similar rules are followed,
as depicted in \sliref{2-m-3}.
<p>
However, one problem we may encounter here
is that classes may have a common base class.
Look at the following program fragment:
<!--\noslide{}{}{-->
[P
D* a = new D(); delete a; ///// ABACDDCABA
P]
<!--}-->

\noid
The outcome of creating and deleting
$a$
indicates that an instance of $D$ contains
two copies of $A$.


\cleartrue
<slide id=2-m-4 caption="Virtual inheritance">
\ifsli{}{\vspace{0.1cm}}
<program><include file=lib/d-v.s></program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Again, the remedy is to declare $A$
to be virtually inherited by $B$ and $C$, as depicted in \sliref{2-m-4}.
As reflected in the outcome of
<!--\noslide{}{}{-->
[P
A* a = new D(); delete a; ///// ABCDDCBA
P]
<!--}-->

\noid
instances of the derived class
$D$
then have only one copy of $A$.
</document>
