<document>

@* <a href=g-model.html>The object model</a>


Programming is, put briefly, to provide
a computing device with the instructions it needs
to do a particular computation.
In the words of Dijkstra:
{\em "Programming is the combination of human
reasoning and symbol manipulation skills
used to develop symbol manipulators (programs).
By supplying a computer to such a symbol manipulator
it becomes a {\em concrete} one."}
Although we are by now used to quite fashionable
computing devices, including graphic interfaces and
multimedia peripherals, the abstract meaning
of a computing device has not essentially altered
since the original conception of the
mathematical model
that we know as the Turing machine (see below).

<p>
Despite the fact that our basic mathematical model of
a computing device (and hence our notion of computability)
has not altered significantly, the development of
high level programming languages has meant a drastic change
in our conception of programming.
Within the tradition of imperative programming, the introduction
of objects, and object-oriented programming, may be thought
of as the most radical change of all.
Indeed, at the time of the introduction of Smalltalk, one
spoke of a true revolution in the practice of programming.

<slide id=2-model caption="The object model">
:@ The object model
\ifsli{\vspace{-0.5cm}}{}
*- computation is sending messages between objects
@ Message
\ifsli{\vspace{-0.5cm}}{}
<E
- {\it object method arguments}
E>
@ Encapsulation
\ifsli{\vspace{-0.5cm}}{}
*- objects encapsulate data and procedures
@ Protocol
\ifsli{\vspace{-0.5cm}}{}
*- the collection of messages an object supports
\ifsli{\vspace{-1cm}}{}
</slide>

The object model introduced by Smalltalk somehow
breaks radically 
with our traditional notion of computation.
Instead of regarding a computation as the execution of a sequence
of instructions (changing the state of the machine), object-based
computation must be viewed as sending messages between objects.
Such a notion of computation had already been introduced
in the late 1960s in the programming language Simula
(see Dahl and Nygaard, 1966).
Objects were introduced in Simula to simulate complex
real world events, and to model the interactions between real
world entities.

<p>
In the (ordinary) sequential machine model, the result of a computation
is (represented by) the state of the machine at the end of the computation.
In contrast, computation in the object model is best characterized as
cooperation between objects. The end result then consists, so to speak,
of the collective state of the objects that participated in the computation.
\see{2-model}

Operationally, an object may be regarded as an abstract machine capable of
answering messages.
The collection of messages that may be handled by an object is often
referred to as the {\em protocol} obeyed by the object.
This notion was introduced in the Smalltalk programming environment
originally to provide the means to group the messages
to which an object may respond.
For instance, the distinction between methods for initialization
and methods for modification or processing may be convenient
in developing or using a program.
The notion of {\em protocol} may also be given a more formal
interpretation, as has been done for instance in
the notion of {\em contracts} (introduced in Eiffel)
stating the requirements that must be adhered to 
in communicating with an object.
<p>
Structurally, an object may be regarded as a collection 
of data and procedures.
In principle, the data are invisible from the outside and may be
manipulated only by invoking the right procedure.
In a pure object-oriented language such as Smalltalk and Eiffel,
sending a message to an object is the only way of invoking such
a procedure.
Combined, {\em data-hiding} and {\em message interface abstraction}
will be referred to as {\em encapsulation}.
Actually, object-oriented languages, while in some way supporting
objects as collections of data and procedures, may differ subtly
in the degree and way in which they support data-hiding and 
abstraction.
<p>
In the next section we will introduce C++, which unlike
the purely object-oriented languages Smalltalk and Eiffel may be
called a hybrid language.
Apart from the support of object-oriented features, C++ also
supports ordinary functions and procedures.
Consequently, it does not impose a strictly object-oriented computation model,
but leaves the programmer the choice of what means are used
to express a computation.
Some do consider this freedom to be a disadvantage.
</document>
