<document>

This chapter has introduced the basic
mechanisms of object-oriented programming.
We looked at a general characterization of the object model,
supporting encapsulation and message passing,
and 
observed that an object-oriented approach
may reduce the complexity of the programming
task but does not reduce the space and
time needed to solve a problem.

\vfig{1}
<slide id=2-1 caption="Section 2.1: The object model">
:@ <a href=model.html>The object model</a>
\zline{\fbox{1}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em object model} -- encapsulation,
message protocols
- {\em complexity} -- what is the
contribution of object orientation
I>
\ifsli{\vspace{-1cm}}{}
</slide>


In section 2, we
proceeded with a fairly detailed
discussion of the language constructs
provided by C++,
including constructors, destructors,
the type modifier $const$ and $friend$s.

<slideid=2-2 caption="Section 2.2: Encapsulation and inheritance \c{in C++}">
:@ <a href=encaps.html>Encapsulation and inheritance in C++</a>
\zline{\fbox{2}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em encapsulation} -- constructors, $const$, conversions, friends
- {\em inheritance} -- abstract and concrete shapes
- {\em assertions} -- pre- and post-conditions
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Further, we looked at a simple example 
demonstrating how to use inheritance
for defining a type hierarchy
of graphical shapes.
In addition, it was shown how to develop
robust code by using assertions.


<slideid=2-3 caption="Section 2.3: Canonical classes">
:@ <a href=canonica.html>Canonical classes</a>
\zline{\fbox{3}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em canonical class} -- constructor, destructor, assignment
- <em> letter/envelope</em> -- body and handler class
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In section 3, some idioms were presented
which may be taken as guidelines for the 
implementation of concrete data types.
An instance of a canonical class behaves
as if it is a built-in type.
To provide an efficient implementation
of a concrete data type, the {\em envelope/letter} idiom may be used,
in which a distinction is made
between an interface handler class
and an implementation body class.


\ifsli{\clearpage}{}
\vfig{0.2}
<slide id=2-4 caption="Section 2.4: Generic types">
:@ <a href=generic.html>Generic types</a>
\zline{\fbox{4}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em polymorphic types} -- due to inheritance
- {\em parametrized types} --
corresponding to template classes
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In section 4, we discussed the
realization of generic types in C++.
Both base class hierarchies and template classes
were discussed and illustrated with examples.

<slideid=2-5 caption="Section 2.5: Benefits and pitfalls">
:@ <a href=benefits.html>Benefits and pitfalls</a>
\zline{\fbox{5}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em protection} -- in relation with inheritance
- {\em object, pointers, references} -- the
sneaky example
- {\em inheritance} -- advantages and disadvantages
- {\em memory management} -- constructors and destructors
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In section 5,
we discussed some of the benefits
and pitfalls of employing C++ for object-oriented programming.
In particular, we discussed issues of protection,
the distinction between objects, references
and pointers and the pros and cons of inheritance.
Further, we discussed the role of constructors
and destructors in memory management.

<slide id=2-6 caption="Section 2.6: Extensions to the object model">
:@ <a href=extensio.html>Extensions to the object model</a>
\zline{\fbox{6}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em meta classes} -- dynamic type information
- {\em distribution} -- persistence, active objects
- {\em abnormal events} -- exception handling
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Finally, we discussed a number of extensions
of the basic object model,
such as meta classes, carrying dynamic
type information and features that
are relevant for distribution, including
persistence and active objects.
We also discussed the issues that arise
in dealing with abnormal events, which may
for example occur when a pre-condition is violated.
</document>
