<document>
@$=tt

:@ Type abstraction in C++
As shown in section \ref{generic}
(which discussed generic types),
type abstraction in C++ may occur in various
guises.
One important means of type abstraction
is to employ
what we have called polymorphic base class hierarchies.
For example, the function $move$,
which was somewhat loosely characterized in
\sliref{9-ex-quantification},
may be defined in C++ as follows:
<slide type=control id=c3-x1 cap="example move">
[P
Point* move(Point* p, int d); /// require $int Point::x$
Point* move(Point* p, int d) { p.x += d; return p; }
P]
</slide>

\noid
In effect, the function $move$ accepts a pointer
to an instance of $Point$, or
any class derived from $Point$, satisfying the requirement
that it has a public integer data member $x$.

Similar restrictions generally hold when
instantiating a template class,
but in contrast to base class subtyping requirements,
these restrictions will only be verified at link time.


<slide id=9-cc-abs cap="Type abstraction in C++">
\ifsli{}{\vspace{0.1cm}}
<program>
template< class T > /// requires $T::value()$
class P {
public:
P(T& r) : t(r) {}
int operator==( P<T>& p) {
	return t.value() == p.t.value();
	}
private:
T& t;
};
</program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Consider the template class definition given
in \sliref{9-cc-abs}.
Evidently, for the comparison function to
operate properly, each instantiation type
substituted for the type parameter $T$
must satisfy the requirement that it has a public member function
$value$.

<slide id=9-cc-abs-2 cap="Type instantiation">
\ifsli{}{\vspace{0.1cm}}
[P
template< class T >
class A { /// \fbox{$ A<T> $}
public: 
virtual T value() = 0;
};

class Int : public A<int> { /// \fbox{$ Int <= A<int> $}
public:
Int(int n = 0) : _n(n) {}
int value() { return _n; }
private:
int _n;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Such a requirement may also be expressed by defining
an abstract class $A$ defining a pure
virtual member function $value$.
See \sliref{9-cc-abs-2}.
The restrictions on instantiating $P$
may then be stated informally as
the requirement that each instantiation type
$T$ must be a subtype of $A<X>$ for arbitrary type $X$.
The class $Int$ is an example of a type complying with
the implicit requirements imposed by the definition of $P$.
An example of using $P$ is given below
\cleartrue
<slide id=c3-x-2 type=control cap="Example P">
[P
Int i1, i2;
P<Int> p1(i1), p2(i2);
if ( p1 == p2 ) cout << "OK" << endl; /// OK
P]
</slide>

\noid
Note, however, that
the derivation of $A<int>$ is by no means
necessary or in any way enforced by C++.
</document>
