<document>

@+ A simple type calculus
\label{calculi}

In our first version of a type calculus
we will restrict ourselves to a given set
of basic types (indicated by the 
letter $%r$) and function types (written
$%s -> %t$, where $%s$ stands for the domain
and $%t$ for the range or codomain).
This version of the typed lambda calculus
(with subtyping) is called $%l_{ <= }$ in
[_Pierce93] from which most of the material is taken.
The $%l_{<=}$ calculus is a first order calculus, since it
does not involve quantification over types.
\see{9-c-subtypes}

The structure of type expressions is given by the 
definition
[P]  $ %t ::= %r | %t_1 -> %t_2$
where we use $%t$ as a type identifier and $%r$
as a meta variable for basic types.
The expressions of our language, that we indicate with 
the letter $e$, are similar to lambda terms,
except for the typing of the abstraction variable in
$%l x: %t.e$.


\index{type calculus}
<slide id=9-c-subtypes cap="The subtype calculus">
<raw height=400>
:@ A simple type calculus <em> -- subtypes</em>
\zline{\fbox{ $%l_{ <= }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %t ::= %r | %t_1 -> %t_2$
- $ e ::= x | %l x : %t . e | e_1 e_2 $
I>
\ifsli{\vspace{-0.3cm}}{}
@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$%G |- x : %s$ \hspace{1cm} $%G |- e : %t $}{
    $%G |- %l x : %s . e \e %s -> %t$
    }
- \ffrac{$%G |- e_1 : %s -> %t, e_2 : %s $}{
    $%G |- e_1 e_2 \e %t $
    }
I>
\ifsli{\vspace{-0.2cm}}{}
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{ $ %G |- e : %s$ \hspace{1cm}  $%G |- %s <= %t$}{
   $%G |- e : %t$
   }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>


To determine whether an expression $e$
is correctly typed (with some type expression $%t$)
we need type assignment rules, as given above.
Typing is usually based on a collection
of assumptions $%G$, that contains the typing of
expressions occurring in the expression
for which we are determining the type.
In the type assignment rules and the (subtyping)
refinement rules, the phrase $ %G |- e : %t$ means that
the expression $e$ has type $%t$, under the assumption that
the type assignments in $%G$ are valid.
When $%G$ is empty, as in $ |- e : %t $, the type assignment
holds unconditionally.
Occasionally, we write $ %G |- e \e %t$ instead of
$ %G |- e : %t$ for readability.
These two expressions have identical meaning.

The premises of a type assignment rule
are given above the line. The type assignment
given below the line states the assignment
that may be made on the basis of these premises.

For example, the first type assignment rule states that,
assuming $ %G |- x : %s $ ($x$ has type $%s$) and
$ %G |- e : %t $ ($e$ has type $%t$)
then $ %G |- %l x : %s. e \e %s -> %t$, in other words
the abstraction $ %l x:%s.e$ may be validly typed
as $%t$.

Similarly, the second type assignment rule
states that applying a function $ e_1 : %s -> %t$
to an expression $ e_2 $ of type $%s$ results
in an (application) expression $ e_1 e_2 $ of type $ %t $.

We may assume the basic types denoted by $%r$
to include (integer) subranges, records and variants.
As a consequence, we may employ the subtyping rules
given in section \ref{subtypes} to determine the subtyping
relation between these types.
The (subtyping) refinement rule repeated here
expresses the substitutability property of subtypes,
which allows us to consider an expression
$e$ of type $%s$, with $%s <= %t$, as being
of type $%t$.

In \sliref{9-ex-subtypes}, some examples are given illustrating
the assignment of types to expressions.
Type assignment may to a certain extent be done
automatically, by type inference, as for example in ML
(Milner {\it et al.}, 1990).\index{Milner {\it et al.} (1990)} 
However, in general, typing is not decidable
when we include the more powerful type expressions
treated later.
In those cases the programmer is required to
provide sufficient type information to enable the
type checker to determine the types.

<slide id=9-ex-subtypes cap="Subtypes -- examples">
<raw height=400>
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<I
- $ S = %l x:Int. x + 1$ \n
$S : Int -> Int $
- $ twice = %l f:Int -> Int. %l y : Int . f ( f ( y ) )$ \n
$ twice : ( Int -> Int ) -> Int -> Int$
I>
\ifsli{\vspace{-0.3cm}}{}
:@ Application
\ifsli{\vspace{-0.5cm}}{}
<I
- $ S 0 = 1 \e Int $
- $ twice ( S ) = %l x. S S x \e Int -> Int $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

When we define the successor function $S$ as
$%l x : Int. x + 1$ then we may type $S$
straightforwardly as being of type $Int -> Int$.
Similarly, we may type the (higher order) function
$twice$ as being of type $(Int -> Int) -> Int -> Int$.
Note that the first argument to $twice$ must be a function.
Applying $twice$ to a function argument only results
in a function.
When applied to $S$ it results in a function
of type $Int -> Int$ that results in applying $S$
twice to its (integer) argument.
The subtyping rules (partly imported from section \ref{subtypes})
work as expected.
We may define, for example, a function
$ + : Real \* Real -> Int $ as a subtype of
$ + : Int \* Int -> Int $ (according
to the contra-variant subtyping rule for functions).

@so c-1

@+ Intersection types

We define our second version of the typed lambda calculus
($ %l_{ /\ }$) as an extension of the first version ($%l_{ <= }$),
an extension which provides facilities for
({\it ad hoc}) overloading polymorphism.
Our extension consists of adding a type expression
$ \bigwedge [ %t_1,...,%t_n ]$ which denotes
a finite conjunction of types.
Such a conjunction of types, that we will
also write as $ %t_1 /\ ... /\ %t_n $ is called
an intersection type.
The idea is that an expression $e$ of type
$ \bigwedge [ %t_1,...,%t_n ]$ is correctly typed
if $ e : %t_i $ for some  $i$ in $ 1..n$.
This is expressed in the type assignment rule given 
in \sliref{9-c-intersection}.


<slide id=9-c-intersection cap="The intersection type calculus">
<raw height=400>
@ Intersection types <em> -- overloading</em>
\zline{\fbox{ $%l_{ /\ }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %t ::= %r | %t_1 -> %t_2 | \bigwedge [ %t_1 .. %t_n ]$
I>
:@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$%G |- e : %t_i$   $(i \e 1..n)$}{
    $%G |- e : \bigwedge [ %t_1 .. %t_n ]$
    }
I>
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{ $%G |- %s <= %t_i$ $ (i \e 1..n )$}{
   $%G |- %s <= \bigwedge [ %t_1 .. %t_n ]$
   }
- $\bigwedge [ %t_1 .. %t_n ] <= %t_i$ 
- $ %G |- \bigwedge [ %s -> %t_1 .. %s -> %t_n ] <= %s -> \bigwedge [ %t_1 .. %t_n ]$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The subtyping rule for intersection types states
that any subtype of a type occurring in the
intersection type $ \bigwedge [ %t_1,..., %t_n ]$
is itself a subtype of the intersection type.

In addition we have two subtyping rules without premises,
the first of which says
that the intersection type itself
may be regarded as a subtype of any of its components.
In other words, from a typing perspective
an intersection type is equal (hence may be replaced by)
any of its component types.

Also, we may refine a function, with domain $%s$, which
has an intersection type $ \bigwedge [ %t_1,..., %t_n ]$ as its
range into an intersection type consisting of functions
$%s -> %t_i$ for $i = 1..n$.

Intersection types allow us to express
a limited form of overloading, by enumerating a finite
collection of possible types.
Since the collection of types comprising an
intersection type is finite, we do not need a higher
order calculus here, although we might have used type
abstraction to characterize intersection types.

<slide id=9-ex-intersection cap="Intersection types -- examples">
<raw height=400>
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<I
- $ + : \bigwedge [ Int \* Int -> Int, Real \* Real -> Real]$
- $Int -> Int <= \bigwedge [ Int -> Int,  Real -> Real ] $
- $Msg -> Obj1 /\ Msg -> Obj2 <= Msg -> \bigwedge [ Obj1, Obj2 ]$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

A typical example of an intersection type is presented
by the addition operator, overloaded for integers and reals,
which we may define as
[P
 $ + : \bigwedge [ Int \* Int -> Int, Real \* Real -> Real ] $
P]
According to our refinement rule, we may specialize
an intersection type into any of its components.
For example, when we have an intersection type defining
a mapping for integers and a mapping for reals, we may choose
the one that fits our purposes best.
This example illustrates that intersection types may
be an important tool for realizing optimizations
that depend upon (dynamic) typing.

Similarly, we may refine a generic function working on
objects into a collection of (specialized) functions
by dividing out the range type.
\see{9-ex-intersection}
The resulting intersection type itself may subsequently
be specialized into one of the component
functions.
In [_CGL93], a similar kind of type is used
to model the overloading of methods in objects,
that may but need not necessarily be related by inheritance.
The idea is to regard message passing to objects
as calling a polymorphic function
that dispatches on its first argument.
When the type of the first argument is compatible
with multiple functions (which may happen for
methods that are refined in the inheritance hierarchy) the most
specific function component is chosen,
that is the method with the minimal object type.
A similar idea is encountered in CLOS,
which allows for the definition of {\em multi-methods}
for which dynamic dispatching takes place
for all arguments.
(A problem that occurs in modeling methods as overloaded
functions is that the subtyping relation between
methods no longer holds, due to the domain contravariance requirement.
See [_CGL93] for a possible solution.)

@so c-2

@+ Bounded polymorphism

Our next extension, which we call $F_{ <= }$, involves
(bounded) universal quantification.
For technical reasons we need to introduce  a
primitive type $Top$, which may be considered as
the supertype of all types (including itself).
Also we need type abstraction variables, that we will
write as $%a$ and $%b$.
Our notation for a universally quantified (bounded) type
is $\A %a <= %s . %t$, which denotes the type $%t$
with the type variable $%a$ replaced by any subtype $%s'$ of $%s$.
In a number of cases, we will simply write $\A %a. %t$,
which must be read as $\A %a <= Top. %t$.
Recall that any type is a subtype of $Top$.
Observe that, in contrast to $%l_{ <= }$ and $%l_{ /\ }$,
the calculus $F_{ <= }$ is second order (due to the quantification
over types).

In addition to the (value) expressions found in the two
previous calculi, $F_{ <= }$ introduces a {\em type abstraction}
expression of the form $%L %a <= %t.e$ and a {\em type instantiation}
expression of the form $e[%t]$.
The type abstraction expression $%L %a <= %t.e$
is used in a similar way as the function abstraction expression,
although the abstraction involves types and not values.
Similar to the corresponding type expression,
we write $%L %a.e$ as an abbreviation for $ %L %a <= Top.e $.
The (complementary) type instantiation statement is written
as $e[%t]$, which denotes the expression $e$ in which
the type identifier $%t$ is substituted for the type variable
bound by the first type abstractor.

<slide id=9-c-bounded cap="The bounded type calculus">
<raw height=400>
@ Bounded polymorphism <em> -- abstraction</em>
\zline{\fbox{ $F_{ <= }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ %t ::= Top | %a | %r | %t_1 -> %t_2 | \A %a <= %t_1. %t_2$
- $ e ::= x | %l x:%t.e | e_1  e_2 | %L %a <= %t.e | e [ %t ]$
I>
:@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$%G, %a <= %s  |- e : %t$   }{
    $%G |- %L %a <= %s. e \e \A %a <= %s . %t$
    }
- \ffrac{$%G, e : \A  %a <= %s . %t$ \hspace{1cm} $ %G  |- %s' <= %s $   }{
    $%G |- e [ %s' ] \e %t [ %a := %s' ]$ 
    }
I>
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{ $%G |- %s <= %s'$ \hspace{1cm} $ %G |- %t' <= %t$}{
   $%G |- \A %a <= %s'.%t' <= \A %a <= %s.%t $
   }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The type assignment rule for type abstraction states
that, when we may type an expression $%e$ as being of
type $%t$ (under the assumption that $%a <= %s$),
then we may type $%L %a <= %s.e$ as being of type $\A %a <= %s.%t$.

The type assignment rule for type instantiation
characterizes the relation between type instantiation
and substitution (which is notationally very similar).
When we have an expression $e$ of type $\A %a <= %s$
and we have that $%s' <= %s$, then $e[%s']$ is of type 
$ %t [ %a := %s' ]$, which is $%t$ with $%s'$ substituted for
$%a$.
\see{9-c-bounded}

The refinement rule for bounded types states
the subtyping relation between two bounded types.
We have that $\A %a <= %s' . %t'$ is
a subtype of $\A %a <= %s.%t$ whenever 
$%s <= %s'$ and $%t' <= %t$.
Notice that the relation is contravariant with respect
to the types bounding the abstraction, similar
as for the domains of function subtypes in
the function subtyping rule.

In contrast to the polymorphism due to object type
extensions and overloading, bounded polymorphism
(employing type quantifiers) is an example
of what we have called parametric polymorphism.
In effect, this means that we must explicitly give a type
parameter to instantiate an object or function of
a bounded (parametric) type, similar to when we use
a template in C++.

The examples given in
\sliref{9-ex-parameters}
illustrate how we may define and subsequently
type parametric functions.
In these examples, we employ the convention
that in the absence of a bounding type we assume $Top$
as an upper limit.
The examples are taken from [_CW85].

<slide id=9-ex-parameters cap="Parametrized types -- examples">
<raw height=400>
:@ Examples
\ifsli{\vspace{-0.5cm}}{}
<I
- $id = %L %a. %l x:%a.x$ \n
$id : \A %a. %a -> %a $ 
- $twice1 = %L %a.%l f: %L %b. %b -> %b. %l x:%a. f[%a](f[%a](x))$ \n
$twice1 : \A %a. \A %b. (%b -> %b) -> %a -> %b$ 
- $twice2 = %L %a.%l f: %a -> %a. %l x:%a. f(f(x))$ \n
$twice2 : \A %a. (%a -> %a) -> %a -> %a$ 
I>
:@ Applications
\ifsli{\vspace{-0.5cm}}{}
<I
- $id [ Int ] ( 3 ) = 3$
- $twice1 [ Int ] ( id )( 3 ) = 3$
- $twice1 [ Int ] ( S ) = illegal$
- $twice2 [ Int ] ( S )( 3 ) = 5$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The (generic) identity function $id$
is defined as $ %L %a.%l x:%a.x$, which states
that when we supply a particular type, say $Int$,
then we obtain the function $%l x:Int. x$.
Since the actual type used to instantiate $id$ is not important,
we may type $id$ as being of type $\A %a.%a -> %a$.
In a similar way, we may define and type the two
(generic) variants of the function $twice$.
Notice the difference between the two definitions of $twice$.
The first variant requires the function argument itself
to be of a generic type, and fails (is incorrectly typed)
for the successor function $S$ which is (non generic)
of type $Int -> Int$.
In contrast, the second variant accepts $S$, and we may
rely on the automatic conversion
of $id : \A %a. %a -> %a$ to $id[Int] : Int -> Int$
(based on the second type assignment rule) to accept $id$
as well.

The interplay between {\em parametric} and {\em inclusion}
polymorphism is illustrated in the examples presented
in \sliref{9-ex-quantification}.
Recall that inclusion polymorphism is based
on the subtyping relation between records
(which states that refinement of a record type involves the addition
of components and/or refinement of components that
already belong to the super type).

<slide id=9-ex-quantification cap="Bounded quantification -- examples">
<raw height=400>
@ Bounded quantification
\ifsli{\vspace{-0.5cm}}{}
<I
- $g = %L %a <= {{ one : Int }}. %l x: %a. (x.one)$ \n
$ g : \A %a <= {{ one : int }}. %a -> Int $
- $g' = %L %b . %L %a <= {{ one : %b }}. %l x: %a. (x.one)$ \n
$ g' : \A %b . \A %a <= {{ one : %b }}. %a -> %b $
- $ move = %L %a <= Point. %l p:%a. %l d : Int .(p.x := p.x + d); p$ \n
$move : \A %a <= Point. \, %a -> Int -> %a$
I>
:@ Application
\ifsli{\vspace{-0.5cm}}{}
<I
- $g' [ Int ][ {{ one:Int, two : Bool }}]({{ one = 3, two = true }}) = 3$
- $move [{{ x : Int, y : Int }}]( {{ x = 0, y = 0 }} )(1) = {{ x = 1, y = 0 }}$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The first example defines a function $g$ that works on a record
with at least one component $one$ and delivers as a result
the value of the component $one$ of the argument record.
The function $g'$ is a generalized version of $g$
that abstracts from the particular type of the $one$
component.
Notice that both $g$ and $g'$ may be applied to any record
that conforms to the requirement stated in the bound,
such as the record $ {{ one = 3, two = true }} $.

As another example of employing bounds to impose requirements,
look at the function $move$ that is defined for subtypes of $Point$
(that we assume to be a record containing $x$ and $y$ coordinates).
It expects a record (that is similar to or extends $Point$)
and an (integer) distance, and as a result delivers the modified
record.


:@ Discussion
Parametric polymorphism is an important means to incorporate
subtyping in a coherent fashion.
Apart from [_Pierce93], from which we have taken most of the
material presented here, we may mention
[_PA93] as a reference for further study.
In [_Pierce93] a calculus $F_{ /\ }$ is also introduced
in which intersection polymorphism is expressed by
means of an explicit type variable.
The resulting type may be written as $\A %a \e {{ ... }}$,
where ${{ ... }}$ denotes a finite collection of types.
As already mentioned, intersection types may also be used
to model inclusion polymorphism
(see Castagna {\it et al.}, 1993).\index{Castagna {\it et al.} (1993)}

It is an interesting research issue to explore
the relation between parametric polymorphism and 
inclusion polymorphism further along this line.
However, we will not pursue this line here.
Instead, in the next section we will look
at another application
of parametric polymorphism, namely existential types
that allow us to abstract from hidden component types.
This treatment is based on [_CW85].
In the last section of this chapter, we will look in
more detail at the role of self-reference
in defining (recursive) object types,
following [_CoHC90].
We will conclude this chapter with some
observations concerning the relevance of
such type theories for actual programming languages.
In particular, we will show that Eiffel is
not type consistent.

@so c-3
</document>
