<document>
%%@* <a href=g-ext.html>Existential types <em> -- hiding</em></a>
@* <a href=@talk-existent.html target=_top>Existential types <em> -- hiding</em></a>
\label{existential}

@$=raw

Existential types were introduced in [_CW85] to model
aspects of data abstraction and hiding.
The language introduced in [_CW85] is essentially
a variant of the typed lambda calculi we have looked
at previously. 

Our new calculus, that we call $F_{ \E }$, is an
extension of $F_{ <= }$ with type expressions of the form
$\E %a <= %s. %t$ (to denote existential types)
and expressions of the form $pack[ %a = %s \: in \: %t ]$
(to denote values with hidden types).
Intuitively, the meaning of the expression $pack[ %a = %s \: in \: %t ]$
is that we represent the abstract type $%a$ occurring
in the type expression $%t$ by the actual type $%s$
(in order to realize the value $e$).
Following the type assignment rule, we may actually provide
an instance of a subtype of the bounding type
as the realization of a hidden type.
\see{9-existential}

<slide id=9-existential cap="The existential type calculus">
<raw height=500>
@ Existential types <em> -- hiding</em>
\zline{\fbox{$ F_{ \E }$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- $%t ::= ... | \E %a <= %t_1. %t_2$
- $ e ::= ... | pack[ %a = %s \: in \: %t ]. e$
I>
:@ Type assignment
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$ %G |- %s' <= %s$ \hspace{1cm} $%G |- e : %t$}{
  $pack[ %a = %s' \: in \: %t ].e \e \E %a <= %s. %t$
  }
I>
:@ Refinement
\ifsli{\vspace{-0.5cm}}{}
<I
- \ffrac{$ %G |- %s <= %s' $ \hspace{1cm} $ %G |- %t' <= %t $}{
   $ %G |- \E %a <= %s' . %t' <= \E %a <= %s. %t $
   }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The subtyping refinement rule is similar to the refinement
rule for universally quantified types.
Notice also here the contravariance relation
between the bounding types.

More interesting is what bounding types allow
us to express.
(As before, we will write $\E %a . %t$ to denote
$\E %a <= Top. %t$.)
First, existential types allow us to
indicate that the realization of a particular type exists,
even if we do not indicate how.
The declaration $e : \E %a . %t$ tells us that
there must be some type $%s$ such that $e$  of type $%t$
can be realized.
Apart from claiming that a particular 
type exists, we may also provide information
concerning its structure, while leaving its actual type
undetermined.


<slide id=9-ex-existential cap="Existential types -- examples">
<raw height=400>
:@ Structure <em> -- undeterminacy</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $Top = \E %a.%a$ \zline{(\it the biggest type)}
- $AnyPair = \E %a \E %b.%a  \* %b$ \zline{(\it any pair)}
- $(3,4):\E %a.%a$ {\em --  does not provide sufficient structure!}
- $(3,4):\E %a.%a \* %a$
I>
@ Information hiding
\ifsli{\vspace{-0.5cm}}{}
<I
- $\E %a.%a \* ( %a->Int )$ \zline{\it object, operation}
- $ x : \E %a. %a \* ( %a->Int )$ \zline{ \( \leadsto \) $snd ( x )( fst( x ))$ }
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

For example, the type $\E %a.%a$ (which may clearly be realized
by any type) carries no information whatsoever, hence it may
considered to be equal
to the type $Top$.
More information, for example, is provided by the type 
$\E %a \E %b. %a \* %b$ which defines the product type
consisting of two (possibly distinct) types.
(A product may be regarded as an unlabeled record.)
The type $ \E %a. %a \* %a $ gives even more information
concerning the structure of a product type,
namely that the two components are of the same type.
Hence, for the actual product $ (3,4) $ the latter is the best choice.
\see{9-ex-existential}

Existential types may be used to impose structure on the
contents of a value, 
while hiding its actual representation.
For example, when we have a variable $x$ 
of which we know that it has type $\E %a. %a \* ( %a -> Int )$
then we may use the second component of $x$
to produce an integer value from its first component,
by $snd(x)( fst(x) )$, where $fst$ extracts the first
and $snd$ the second component of a product.
Clearly, we do not need to know the actual
representation type for $%a$.

A similar idea may be employed for (labeled) records.
For example, when we have a record $x$ of type
$\E %a.{{ val : %a, op : %a -> Int }}$ then we may
use the expression $x.op(x.val)$ to apply the
operation $op$ to the value $val$.
Again, no knowledge of the type of $val$ is required in this case.
However, to be able to use an element of an existential type
we must provide an actual representation type,
by instantiating the type parameter in a $pack$
statement.


@$=$

<slide id=9-ADT cap="Packages -- examples">
<raw height=400>
@ Abstract data types {\em -- packages}
\ifsli{\vspace{-0.5cm}}{}
<I
- $ x:\E %a.{{ val : %a, op : %a -> Int }} $
- $ x = pack[%a = Int \: in \: {{ val:%a, op:%a -> Int }} ]((3,S)) $
- $ x.op(x.val) = 4 $
I>
@ Encapsulation
\ifsli{\n\vspace{0.2cm}\mbox{}\hspace{-0.3cm}}{}
\zline{\fbox{$pack [ representation \: in \: interface ]( contents )$}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em interface} -- $type$ $\: \E %a. {{ val : %a, op : %a -> Int }} $
- {\em representation} -- $%a = Int$ \c{\zline{({\it hidden data type})}}
- {\em contents} -- $ (3,S) $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>
\vstr{0.5}

@$=$

The $pack$ statement may be regarded as an encapsulation
construct, allowing us to protect the inner parts
of an abstract data type.
When we look more closely at the $pack$ statement,
we can see three components.
First, we have an {\em interface} specification corresponding
to the existential type associated with the $pack$ expression.
Secondly, we need to provide an actual representation of
the hidden type, $Int$ in the example above.
And finally, we need to provide the actual
contents of the structure.
\see{9-ADT}

@$=raw

In combination with the notion of {\em objects as records},
existential types provide us with a model of abstract data types.
Real objects, however, require a notion of {\em self-reference}
that we have not captured yet.
In the next section we will conclude our exploration
of type theories by discussing the $ F_{%m} $ calculus
that supports recursive (object) types and inheritance.

@so c-4
</document>
