<document>
%%@* <a href=g-fla.html>Flavors of polymorphism</a>
@* <a href=@talk-flavors.html target=_top>Flavors of polymorphism</a>
\label{flavors}

Polymorphism is not a feature exclusive to object-oriented
languages.
For example the ML language is a prime example
of a non object-oriented language
supporting a polymorphic type system 
(see Milner {\it et al.}, 1990).
\index{Milner {\it et al.} (1990)}
Also, most languages, including Fortran and Pascal,
support implicit conversion between integers
and floats, and backwards from floats to integers,
and (in Pacal) from integer subranges to integers.
Polymorphism (including such conversions) is a means
to relieve the programmer from the rigidity imposed
by typing.
Put differently, it's a way in which to increase
the expressivity of the type system.



<slide id=9-typing cap="The nature of types">
:@ Typing <em> -- protection against errors</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em static} -- type checking at compile time
- {\em strong} -- all expressions are type consistent
I>
:@ Untyped <em> -- flexibility</em>
\ifsli{\vspace{-0.5cm}}{}
<I>-bitstrings, sets, $%l$-calculus
:@ Exceptions to monomorphic typing:
\ifsli{\vspace{-0.5cm}}{}
<I>- {\em overloading, coercion, subranging, value-sharing {\rm (nil)}}
\ifsli{\vspace{-1cm}}{}
</slide>

Typing, as we have argued before, is important as
a means to protect against errors.
We must distinguish between {\em static typing}
(which means that type checking takes place at compile time)
and {\em strong typing}
(which means that each expression must be type consistent).
In other words, strong typing allows illegal
operations to be recognized and rejected.
Object-oriented languages (such as Eiffel, and
to a certain extent C++) provide strong typing
which is a mixture of static typing and runtime
checks to effect the dynamic binding of method
invocations.
\see{9-typing}

Typed languages impose rather severe constraints
on the programmer.
It may require considerable effort to arrive at
a consistently typed system and to deal with
the additional notational complexity of defining
the appropriate types.
In practice, many programmers and mathematicians
seem to have a preference for working in
an untyped formalism, like bitstrings, (untyped) sets
or (untyped) lambda calculus.
We may further note that languages such as Lisp,
Prolog and Smalltalk are popular precisely
because of the flexibility due to the absence
of static type checking.

For reliable software development, working in an untyped
setting is often considered as not satisfactory.
However, to make typing practical, we need to relieve
the typing regime by supporting well-understood
exceptions to monomorphic typing, such as overloaded
functions, coercion between data types and value sharing
between types (as provided by a generic nil value).
More importantly, however, we must provide for controlled
forms of polymorphism.



<slide id=9-flavors cap="Flavors of polymorphism">
@ Flavors of polymorphism
\ifsli{\vspace{-1cm}}{}
<latex>
\ln
\curl{\c{polymorphism}}{\curl{universal}{parametric \hspace{1cm} \zline{({\it generics})}}{inclusion \hspace{1cm} \zline{({\it inheritance})}}} {\curl{ad-hoc}{overloading \hspace{1cm} \zline{\fbox{\it intersection}}}{coercion}}
\ln
</latex>
\ifsli{\vspace{-2cm}}{}
<I
- {\em inclusion polymorphism -- } to model subtypes and inheritance
- {\em parametric polymorphism -- } uniformly on a range of types
- {\em intersection types -- } coherent overloading
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In [_CW85], a distinction is made between
{\em ad hoc} polymorphism
(which characterizes the mechanisms mentioned
as common exceptions to monomorphic typing)
and {\em universal} polymorphism
(which allows for theoretically well-founded
means of polymorphism).
Universal polymorphism may take the form of
{\em inclusion polymorphism}
(which is a consequence of derivation by inheritance)
or {\em parametric polymorphism}
(which supports generic types,
as the template mechanism offered
by C++).
\see{9-flavors}
The term {\em inclusion polymorphism} may be understood
by regarding inheritance as a means to
define the properties of a (sub)type incrementally,
and thus (by adding information) delimiting a subset of
the elements corresponding to the supertype.
When overloading is done in a systematic fashion we may
speak of {\em intersection} types, which allows for
polymorphism based on a finite enumeration of types.
See section \ref{calculi}.


@+ Inheritance as incremental modification 

%%\comment{See [_WZ88].}

The notion of inheritance as incremental
modification was originally introduced
in [_WZ88].
Abstractly, we may characterize derivation by inheritance
in a formula as $R = P + M$, where $R$ is the result
obtained by modifying the parent $P$ by (modifier) $M$.
\see{9-inheritance}
In C++ this might have been expressed as something like
<tt>class R : P { M }</tt>.


\cleartrue
\index{incremental modification}
<slide id=9-inheritance cap="Inheritance as incremental modification">
@ Inheritance <em> -- incremental modification</em>
\ifsli{\vspace{-0.5cm}}{}
*- {\bf R}esult = {\bf P}arent + {\bf M}odifier
Example: $R  = {{ a1, a2 }} + {{ a2, a3 }} = {{ a1, a2, a3 }}$
\nl
{\em Independent attributes}: M disjoint from P\n
{\em Overlapping attributes}: M overrules P
@ Dynamic binding
\ifsli{\vspace{-0.5cm}}{}
<I>- $R = {{...,P_i: self ! A,...}} + {{ ...,M_j: self ! B, ... }}$
\ifsli{\vspace{-1cm}}{}
</slide>

For example, we may define the record consisting
of attributes $a_1 ... a_n$ by adding
${{ a_2, a_3 }}$ to the parent ${{ a_1, a_2 }}$.
Clearly, we must make a distinction between
{\em independent} attributes (that occur in either
$P$ or $M$) and {\em overlapping} attributes (that occur
in both $P$ and $M$ and are
taken to be overruled by the definition given in $M$).





An important property of objects,
not taken into account in our interpretation
of {\em object as records} given before, is that
objects (as supported by object-oriented languages)
may be referring to themselves.
For example, both in the parent and the modifier
methods may be defined that refer to a variable $this$ or
$self$ (denoting the object itself).
It is important to note that the variable $self$ is dynamically
bound to the object and not (statically) to
the textual module in which the variable $self$ occurs.
[_WZ88] make a distinction between attributes that
are redefined in $M$, {\em virtual} attributes
(that need to be defined in $M$) and $recursive$
attributes (that are defined in $P$).
Each of these attributes may represent methods
which (implicitly) reference $self$.
(In many object-oriented languages, the variable $self$
or $this$ is implicitly assumed
whenever a method defined within the scope of the
object is invoked.)
Self-reference (implicit or explicit) underlies
dynamic binding and hence is where
the power of inheritance comes from.
Without self-reference 
method calls
would reduce to statically bound function invocation.


@+ Generic abstract data types

Our goal is to arrive at a type theory with sufficient power
to define generic (polymorphic) abstract data types.
In the following section, we will develop a number of type
calculi (following Pierce, 1993)\index{Pierce (1993)} that enable
us to define polymorphic types
by employing {\em type abstraction}.


Type abstraction may be used to define generic
types, data hiding and (inheritance) subtypes.
The idea is that we may characterize generic types
by quantifying over a type variable.
For example, we may define the identity
function $id$ generically as $\A T. id( x:T ) = x$,
stating that for arbitrary type $T$ and element $x$ of
type $T$, the result of applying $id$ to $x$
is $x$. Evidently this holds for any $T$.

In a similar way, we may employ type parameters
to define generic abstract data types.
Further, we may improve on our notion
of {\em objects as records}
by defining a packaging construct that allows for data hiding
by requiring merely that there exists
a particular type implementing the hidden component.

Also, we may characterize the (inheritance) subtyping relation
in terms of bounded quantification,
that is quantification over a restricted collection
of types (restricted by imposing constraints
with respect to the syntactic structure of the type
instantiating the type parameter).


</document>
