<document>

:@ The lambda calculus
The lambda calculus provides a very concise,
yet powerful formalism to reason about
computational abstraction.
The introduction given here has been taken from
[_Barend], which is a standard reference on this subject.

\clearfalse
<slide id=lambda-terms cap="The lambda calculus -- terms">
<raw height=500>
@ Lambda calculus <em> -- very informal</em>
\zline{\fbox{$ %l $}}
\ifsli{\vspace{-0.5cm}}{}
<I
- variables, abstractor $%l$, punctuation $(,)$
I>
@ Lambda terms <em> -- $ %L $</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- $ x \e %L $ \zline{variables}
- $ M \e %L => %l  x. M \e %L $ \zline{abstraction}
- $ M \e %L and N \e %L => M N \e %L $ \c{\zline{application}}
I>
</raw>
\ifsli{\vspace{-1cm}}{}
</slide>

Syntactically, lambda terms are built from a very simple syntax,
figuring variables, the abstractor <raw>$%l$</raw> (that is
used to bind variables in an expression), and punctuation
symbols.
Abstractors may be used to abstract a lambda term <raw>$M$</raw> into
a function <raw>$%l x.M$</raw> with parameter <raw>$x$</raw>.
The expression <raw>$%l x.M$</raw> must be read as denoting the
function with body <raw>$M$</raw> and formal parameter <raw>$x$</raw>.
The variable $x$ is called the bound variable, since it
is bound by the abstractor $%l$.
In addition to function abstraction, we also
have (function) application, which is written
as the juxtaposition of two lambda terms.
\see{lambda-terms}

Behaviorally, lambda terms have a number of properties,
as expressed in the laws given in \sliref{lambda-laws}.


<slide id=lambda-laws cap="The lambda calculus -- laws">
<raw height=300>
:@ Laws
\ifsli{\vspace{-0.5cm}}{}
<I
- $( %l x.M) N = M [ x := N ] $ \zline{beta conversion}
- $ M = N => MZ = NZ and ZM = ZN $
- $ M = N => %l x.M = %l x.N $
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

The most important rule is the {\em beta conversion}
rule, which describes in a manner of speaking how
parameter passing is handled.
In other words function call, that is the application
<raw>$(%l x.M)N$</raw>, results
in the function body $M$ in which $N$ is substituted
for $x$.
Two other laws are the so-called extensionality axioms,
which express how equality of lambda terms
is propagated into application and function abstraction.
These laws impose constraints upon the models
characterizing the meaning of lambda terms.

\cleartrue
<slide id=lambda-substitution cap="The lambda calculus -- substitution">
<raw height=350 width=500>
@ Substitution
\ifsli{\vspace{-0.5cm}}{}
<I
- $x[x:=N] == N$
- $y[x:=N] == y$ if $ x != y $
- $(%l y.M)[x:=N] == %l y.(M[x:=N])$
- $(M_1 M_2 ) [x:=N]$ $ == $ 
\ifsli{\n\mbox{}\hspace{4cm}}{}
  $(M_1 [x:N]) (M_2[x:=N])$
I>
\ifsli{\vspace{-1cm}}{}
</raw>
</slide>

Substitution is defined by induction on the
structure of lambda terms.
A variable $y$ is replaced by $N$ (for a substitution $[x:=N]$)
if $y$ is $x$ and remains $y$ otherwise.
A substitution $[x:=N]$ performed on an abstraction
$%l y.M$ results in substituting $N$ for $x$ in $M$
if $x$ is not $y$.
If $x$ is identical to $y$, then $y$ must first
be replaced by a fresh variable (not occurring in $M$).
A substitution performed on an application simply
results in applying the substitution to both
components of the application.
\see{lambda-substitution}

Some examples of {\em beta conversion} are given 
in \sliref{lambda-examples}.
In the examples, for simplicity we employ
ordinary arithmetical values and operators.
This does not perturb the underlying $%l$-theory,
since both values and operations may be expressed
as proper $%l$-terms.

<slide id=lambda-examples cap="Beta conversion -- examples">
<raw height=500>
:@ Examples
<display>
$(%l x.x) 1 = x[x:=1] = 1$
$(%l x.x+1) 2 = (x+1)[x:=2] = 2 + 1$
$(%l x.x+y+1) 3 = (x+y+1)[x:=3] = 3+y+1$
$(%l y.(%l x.x+y+1) 3) 4) = $
		$((%l x.x+y+1) 3)[y:=4] = 3 + 4 + 1$
</display>
</raw>
</slide>

Note that the result of a substitution may still contain
free variables (as in the third example) that
may be bound in the surrounding environment
(as in the fourth example).

Lambda calculus may be used to state properties
of functions (and other programming constructs)
in a general way.


\vstr{2}
\cleartrue
<slide id=lambda-properties cap="The lambda calculus -- properties">
<raw height=400>
:@ Properties
\ifsli{\vspace{-0.5cm}}{}
<I
- $\A M (%l x.x) M = M$ \zline{identity}
- $\A F \E X. F X = X $ \zline{fixed point} \ln
Proof: take $W = %l x.F ( xx )$ and $X = WW$, then
[P
$X = WW = ( %l x.F ( xx ) ) W = F ( WW ) = FX$
P]
\ifsli{}{\vspace{-0.5cm}}
I>
</raw>
</slide>
\vfig{0}


Consider, for example, the statement
that the identity function works for each lambda term
as expected.
The quantification with respect to $M$ indicates 
that in each possible model for the lambda calculus
(that respects the extensionality axioms given above)
the identity $(%l x.x)M = M$ holds.
\see{lambda-properties}

As another example, consider the statement that each function $F$
has a fixed point, that is a value $X$ for which $FX = X$.
The proof given above, however, does not give us any information
concerning the actual contents of the fixed point,
but merely proves its existence.
In the following (see section \ref{self-reference})
we will write $\Y(F)$ for the fixed
point of a function $F$.

In [_Barend], an extensive account is given of how to
construct mathematical models for the lambda calculus.
A semantics of our type calculus may be given in terms
of such models; however we will not pursue this
any further here.
</document>
