<document>
@* <a href=g-com.html>Specifying behavioral compositions</a>
\label{formal-coop}

@$=tt

The notion of {\em behavioral types}
may be regarded as the formal underpinning of
the notion of {\em contracts} specifying
the interaction between a client and server (object).
Cf. [_Meyer93].
Due to the limited power of the (boolean)
assertion language, contracts as supported by
Eiffel are more limited in what may be specified
than (a general notion of) behavioral types.
However, 
some of the limitations are due,
not to limitations on the assertion language,
but to the local nature of specifying object behavior by
means of contracts.
See also [_Meyer93].

To conclude this chapter,
we will look at an example illustrating the need
to specify global invariants.
Further we will briefly look at alternative formalisms
for specifying the behavior of collections of objects,
and in particular we will
explore the interpretation of $contracts$ as 
behavioral compositions.

\label{global-invariants}
@ Global invariants
Invariants specify the constraints on the state
of a system that must be met for the system
to be consistent.
Clearly, as elementary logic teaches us,
an inconsistent system is totally unreliable.

Some inconsistencies cannot be detected locally,
within the scope of an object,
since they may be caused by actions that do not involve
the object directly.
An example of a situation in which an
externally caused inconsistent object state may occur
is given in \sliref{10-invariants}.
(The example is taken from [_Meyer93], but rephrased in C++.)


<slide id=10-invariants cap="Establishing global invariants">
:@ Problem <em> -- dynamic aliasing</em>
[P
class A {
public:
A() { forward = 0; }
attach(B* b) { forward = b; b->attach(this); }
bool invariant() {
   return !forward || forward->backward == this;
   }
private:
B* forward;
};

class B {
public:
B() { backward = 0; }
attach(A* a) { backward = a; }
bool invariant() {
   return !backward || backward->forward == this;
   }
private:
A* backward;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>


When creating an instance of $A$, the $forward$ pointer
to an instance of $B$ is still empty.
Hence, after creation, the invariant of
the object is satisfied.
Similarly when, after creating an instance of $B$,
this instance is attached to the $forward$ pointer,
and as a consequence the object itself is attached to the
$backward$ pointer of the instance of $B$.
After this, the
invariant is still satisfied.
However, when a second instance of $A$ is created,
for which the same instance of $B$ is attached
to the $forward$ pointer,
the invariant for this object will hold,
but as a result the invariance for the first
instance of $A$ will become violated. See below.
<slide id=composit-x-1 cap="Example" mode=control>
[P
A a1, a2; B b;
a1.attach(b);
a2.attach(b); ///// violates invariant a1
P]
</slide>
\noid
This violation cannot be detected by the object
itself, since it is not involved
in any activity.
Of course, it is possible to check externally
for the objects not directly involved
whether their invariants are still satisfied.
However, the cost of exhaustive checking will
in general be prohibitive.
Selective checking is feasible only
when guided by an adequate specification
of the possible interferences between object states.


\index{interaction}
@ Specifying interaction
Elementary logic and set-theory provide
a powerful vehicle for specifying
the behavior of a system,
including the interaction between its components.
However, taking into account that many
software developers prefer a more
operational mode of thinking 
when dealing with the intricacies of
complex interactions, we will briefly look
at formalisms that allow a more explicit
specification of the operational aspects
of interaction and communication,
yet support to some extent to reason about
such specifications.
\see{10-interaction}

\index{behavioral compositions}
\index{behavioral contracts}
<slide id=10-interaction cap="Specifying interactions">
@ Contracts <em> -- behavioral compositions</em>
\zline{\fbox{interaction}}
\ifsli{\vspace{-0.5cm}}{}
*- specification, refinement, conformance declarations
@ Scripts <em> -- cooperation by enrollment</em>
\ifsli{\vspace{-0.5cm}}{}
*- roles, initialization/termination protocols, critical role set
@ Multiparty interactions <em> -- communication primitive</em>
\ifsli{\vspace{-0.5cm}}{}
*- frozen state, fault-tolerance, weakening synchrony
@ Joint action systems <em> -- action-oriented</em>
\ifsli{\vspace{-0.5cm}}{}
*- state charts, refinement, superposition
\ifsli{\vspace{-1cm}}{}
</slide>

In [_HHG90], a notion of {\em behavioral contracts}
is introduced that allows for characterizing
the behavior of  compositions of objects.
Behavioral contracts fit quite naturally in the
object oriented paradigm, since they allow 
both refinement and (type) conformance declarations.
See below.
Somewhat unclear, yet, is what specification
language the {\em behavioral contracts} formalism 
is intended to support.
On the  other hand, from an implementation perspective
the interactions captured by behavioral contracts
seem to be expressible also within the  confines
of a class system supporting generic classes
and inheritance.

A similar criticism seems to be applicable to the formalism
of (role) {\em scripts} as proposed in [_Francez].
Role scripts allow the developer to specify
the behavior of a system as a set of roles and the
interaction between objects as subscribing to a role.
In contrast to behavioral contracts, the
script formalism may also be applied to describe
the behavior of concurrently active objects.
In particular, the script formalism allows
for the specification of predefined initialization and
termination policies and for the designation of
a so-called {\em critical role set},
specifying the number and kind of participants
minimally required for a successful computation.

Also directed towards the specification of concurrent
systems is the {\em multi-party interactions} formalism
proposed in [_Evangelist],
which is centered around a (synchronous) communication
primitive allowing multiple objects to interact
simultaneously.
The notion of {\em frozen state} (that may be understood as
an invariance requirement that holds during the interaction)
may be useful in particular for the specification of
fault-tolerant systems.
An interesting research issue in this respect is to what extent
the assumption of synchrony  may be weakened
in favor of efficiency.

A rather different orientation towards specifying
the interaction between collections of concurrently active
objects is  embodied by the {\em joint action systems}
approach described in [_Kurki].
Instead of relying on the direct communication
between objects {\em joint action systems} proceed
from the assumption that there exists some global
decision procedure that decides which actions
(and interactions) are appropriate.

<slide id=10-action cap="Specifying actions -- example">
:@ Joint action systems
\ifsli{\vspace{-0.5cm}}{}
[P
action $service()$ by client c; server s is
when c.requesting && s.free do
	<body>
P]
\ifsli{}{\vspace{-0.2cm}}
</slide>

An example of an {\em action} specification is
given in \sliref{10-action}.
Whether the {\em service} is performed depends
upon the state of both the client and the server object
selected by the action manager.
[_Kurki] characterize their approach as
{\em action-oriented} to stress the importance
of specifying actions in an independent manner
(as entities separate from classes and objects).
An interesting feature of the {\em joint action systems}
approach is that the behavior of individual objects
is specified by means of {\em state charts},
a visual specification formalism based on [_Harel87].
The specification formalism adopted gives rise
to interesting variants on the object oriented
repertoire,
such as inheritance and refinement by superposition.
From a pragmatic viewpoint, the assumption of
a global manager seems to impose high demands
on system resources.
Yet, as a specification technique, the concept
of {\em actions} may turn out to be surprisingly
powerful.

In summary, this brief survey of specification formalisms
demonstrates that there is a wide variety
of potentially useful constructs that all bear
some relevance to object oriented modeling,
and as such may enrich the repertoire
of (object oriented) system developers.

@so protocol
</document>
