<document>
@* <a href=g-des.html>Library design</a>
\label{library-design}

One of the great promises of object orientation
is what has been called the {\em industrial reuse of software} 
in [_Meyer90].
The idea of objects as reusable components
was originally put forward in [_Cox86],
who coined the phrase {\em software IC}
to indicate the analogy with building electronic circuits
from {\em off-the-shelf components}.

Software reuse is certainly not an exclusive interest
of object-oriented developers,
as testified for instance in [_Biggerstaff89].
However, object orientation holds strong cards,
as
the mechanisms of encapsulation and
inheritance directly support the reuse of software.
Encapsulation may be used to define
{\em wrapper classes} for existing software,
and inheritance may be used to refine
existing classes to meet specific requirements.
Essentially, a well-designed class library
may act as a repository of
reusable components.
The {\em unit of reuse} is, however,
generally not a single class but
rather a collection of classes providing
a framework for application development.
In this section, we will look
at the design decisions underlying the 
Eiffel libraries, and 
discuss the development of template classes
for abstract data structures in C++.

@+ Taxonomic classification <em> -- the Eiffel libraries</em>



A good example of a well-designed and well-documented
collection of class libraries is given in [_Meyer90],
who describes the design and implementation
of the standard Eiffel libraries.
(See also Meyer, 1994.)

\vfig{0.1}
<slide id=11-eiffel cap="The Eiffel libraries">
@ Library design
{\em -- industrial reuse \c{of software}}
*- {\em unit of reuse} -- class/cluster
:@ The Eiffel libraries
{\em -- contracts}
<I
- {\em Kernel} -- basic system needs\c{, array, strings, io ...}
- {\em Support} -- browsing, \c{persistent} storage\c{, debugging} 
- {\em Data Structures} -- lists, trees\c{, stacks},...
- {\em Lexical/Parsing} -- scanners \c{and parsers}
- {\em Graphics} -- windows, \c{mouse handling,} figures
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The Eiffel libraries include {\em kernel} classes
(for arrays, strings and io),
{\em support} classes (for browsing, persistent storage
and debugging),
{\em data structures} (such as lists, trees and stacks),
classes for {\em lexical analysis} and {\em parsing},
and {\em graphics} classes (for windows, mouse handling
and figures).
\see{11-eiffel}

\vstr{0.5}
<slide id=11-obsolete cap="The problem of consistency">
:@ Consistency
{\em -- support tools}
The great Tempter of Perfection exhorts: ``Correct it here and now,
before it is to late.''
But, here the Guardian Angel of the Installed Base is really a frontman
for the hideous Devil of Eternal Compatibility with the Horrors of the Past,
whose nefarious influence is also visible in Computer Science.
\zline{[_Meyer90]}
\ln
\ifsli{\vspace{-1cm}}{}
</slide>


The Eiffel libraries may serve as an example
of rational and consistent library design.
Each class interface is described by a {\em contract}
stating the class invariant and the
pre- and post-conditions for the services provided.
An interesting feature of the Eiffel libraries
is that the class interface documentation has
been extracted automatically from the class
implementations, thus guaranteeing consistency between
the actual library classes and their description.

Like any software system, libraries tend to evolve
over time.
As the quotation taken from [_Meyer90] in \sliref{11-obsolete}
indicates,
the developer is faced with the dilemma
of keeping the library clean and consistent
or satisfying the user of earlier versions
by maintaining the original class interfaces.
As an intermediate solution, the Eiffel
language provides the keyword {\em obsolete}
which may be used to indicate that particular
methods may eventually be removed from the 
class interface.
The user acquiring a new version of the library
is then urged to adapt the code without
being confronted with a non-functioning system.


\label{eiffel-tax}

Good library design is aimed at the development
of a stable collection of classes with stable interfaces.
Stable interfaces mean that the user of the library
need not worry about any changes in the implementation
of the library, since as long as class interfaces
remain the same user code is not affected.


\vstr{1}
<slide id=11-naming cap="Naming conventions">
:@ Naming
*- favor consistency \c{over specificity}
:@ Conventions 
<I
- {\em verbs} -- procedures
- {\em nouns} -- attributes, functions
- {\em adjectives \c{, question-verbs}} -- boolean queries
I>
\ifsli{\vspace{-1cm}}{}
</slide>

An important aspect of developing library classes
is to arrive at a consistent and understandable
(read predictable) naming scheme for the methods
listed in the class interface.
The advice given in [_Meyer90] is to
use {\em verbs} for (state modifying) procedures,
{\em nouns} for (non-modifying) functions and attributes,
and {\em adjectives} or {\em question verbs} for
boolean queries.
Note that this advice is not without ambiguity,
since words such as {\em empty} and {\em count}
may be used both as nouns and verbs.
In addition, rather than adhering to specific
method names that are conventionally used
(such as $push$ and $pop$ for stacks and $insert$ and $extract$
for queues), [_Meyer90] advises resorting to
standard feature or method names that apply
to all classes of a particular category.
\see{11-naming}

\vstr{2}
<slide id=11-features cap="Feature names for containers">
@ Standard feature names
\zline{\fbox{containers}}
\nl
\indent {\em item} -- access operation \n
\indent {\em count} -- number of items \n
\indent {\em has} -- membership test\n
\indent {\em put} -- insert or replace item\n
\indent {\em force} -- like put\n
\indent {\em remove} -- remove an item\n
\indent {\em wipe\_out} -- remove all items\n
\indent {\em empty} -- absence of items\n
\indent {\em full} -- no more space
</slide>

As an example, consider the development of a collection
of container classes.
Our aim, obviously, is to arrive at an inheritance
hierarchy representing a classification of container types
that allows us to choose the kind of container
that most closely fits our needs with respect
to {\em access}, {\em traversal} and {\em storage requirements}.

The advantage of using standard feature names for the complete
collection
instead of the specific names conventionally used
for the different kinds of containers
is that essentially no code has to be changed
when replacing one kind of container with another.
The standard feature names for container classes
in the Eiffel library are listed in \sliref{11-features},
with a short explanation of their meaning.

%%@ Classification
Underlying the implementation of container classes
in the Eiffel libraries is a taxonomy of data structures,
and a taxonomy of
access and traversal methods, that may be combined
in a variety of ways using multiple inheritance.
From a user's point of view, access methods
are the most important attributes of containers,
as they determine the order of insertion
and retrieval.

\newsavebox{\xxcontainers}
\sbox{\xxcontainers}{
\it\footnotesize
\curl{\c{container}}{
   \curl{collection}{set}{\curl{bag}{
				\curl{dispenser}{stack}{queue}
				}{
				\curl{cursor}{list}{tree}
				}
			 }
		}{
   \curl{table}{indexable}{hash}
   }
   \n
}
<slide id=11-access cap="Access taxonomy">
:@ Access
{\em -- \c{keys,} order of insertion and retrieval}
\nl
\usebox{\xxcontainers}
</slide>

The hierarchy associated with access methods
most closely corresponds with the traditional
classification of containers according to
their (mathematical) type.
\see{11-access}
For example, a $stack$ may be regarded as a refinement
of a $bag$, imposing additional order.
See section \ref{refinement}.
Somewhat peculiar in the hierarchy shown above
is that the container type $set$ is not a refinement
of $bag$, which would mathematically be more appropriate.


In addition to the hierarchy corresponding to
access methods, we also have a hierarchy corresponding
to traversal methods
(which are either hierarchical or linear)
and a hierarchy representing the options for
storage requirements.

As an example of the possible interplay between
access methods and traversal methods,
look at the $tree$ container in the access hierarchy.
Similar to a $list$, a $tree$ may be accessed by means 
of a $cursor$,
which is a pointer to a particular element in the structure.
Dependent on the traversal method, the $cursor$
points to the respective elements of the tree
either in a linear (that is sorted) order or
hierarchically (corresponding with the levels of the nodes).

\vstr{0.5}
<slide id=11-storage cap="Storage taxonomy">
:@ Storage
{\it -- requirements}
\nl\it
\curl{$\bullet$}{infinite (lazy)}{\curl{finite}{unbounded}{\curl{bounded}{resizable}{fixed}}}
\n
</slide>



With respect to the storage requirements,
we have the choice between finite structures
or potentially infinite structures
(that must be implemented in a $lazy$, that is demand-driven,
fashion).
Bounded structures such as arrays may again be 
subdivided into fixed or resizable structures.
\see{11-storage}

The hierarchies corresponding to access and traversal
methods and storage requirements may be regarded
as the dimensions along which a variety
of container types may be constructed.
The properties chosen with respect to these dimensions
may be used as type annotations serving as
an index for retrieval.


:@ Indexing and retrieval
To support the retrieval of classes from
a repository (a database of software components),
the Eiffel language provides the keyword $index$
by which attributes associated with a class may
be defined.
For instance, a container class may be given the
attributes $fifo$ (first-in first-out)
or $lifo$ (last-in first-out)
that characterize respectively the access behavior of
a queue and a stack.
\see{11-indexing}
The Eiffel system itself, however, does not provide
for a software repository facility.


<slide id=11-indexing cap="Indexing and retrieval">
@ Indexing
{\em -- type annotations}
<I
- Access -- {\em fifo, lifo, key}
- Storage  -- {\em bounded, unbounded}
I>
@ Archival and querying tools 
\zline{\fbox{Eiffel}}
<I
- {\bf flat} -- flattens \c{the} inheritance structure
- {\bf short} -- produces interface
- {\bf good} -- browser for class universe
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The retrieval of classes (and software components
in general) is one of the unresolved problems
in software reuse.
Clearly, a $browser$ allowing traversal of the class
universe does not suffice.
As an anecdote, Alan Kay (the spiritual father of Smalltalk)
once remarked that after an absence of a few months
in which the class structure of the Smalltalk
library had been changed he had considerable difficulty
in locating the appropriate classes.
The anecdote indicates that browsers are only helpful
to refresh our memory, but not to familiarize
ourselves with a class library or to select
previously unknown classes for reuse.

The class browser $good$ that comes with the Eiffel
system is a graphical class browser
which sets (apart from its speed) an example for
a general purpose class browser,
as it allows a graphical display of both inheritance
relations and client-server relations.
In addition to a class browser, the Eiffel system
provides a tool $flat$ that allows one to flatten
a class to include all features inherited
from its ancestors.
Moreover, it provides a tool $short$
that allows one to extract an interface description
from a (possibly flattened)
class implementation.
A tool such as $short$ is an invaluable means
for producing automatically the documentation
specifying the interface of a class.
Invaluable, since reuse ultimately depends
upon understanding the functionality offered 
by a library.


@so data
</document>
