<document>
@+ The object/event duality
\label{duality}

One of the major problems in software engineering
that must be resolved for each software development
project is to obtain a proper modular decomposition
of a system.
Object orientation, supporting a component-based
approach, is a big step, as it provides
the mechanisms necessary to address this problem.
However, as argued in our historical aside below,
it provides only part of the solution.
What is still lacking, somehow, is a proper
solution to the problem of characterizing
the interaction between collections of objects.
A (partial) solution to this problem, however,
is suggested by
employing events to define the interaction between
objects.
See also
sections \ref{systems} and \ref{events}.

In the OOPSLA'92 panel, with the title
{\em The heresy of event-orientation in a world of objects},
several speakers observed a discomfort
with respect to the notion of {\em events}
in the object-oriented community.
A reason for this discomfort is the
tension between objects and events,
a tension which is known as the
{\em object/event duality}.
Traditionally, events are actions operating on passive entities,
whereas objects are meant explicitly to
integrate data and operations thereupon.

<slide id=11-events cap="Objects and events">
@ Events <em> -- actions involving objects</em>
<I
- requirements {\em -- user actions}
- design {\em -- object compositions}
- implementation <em> -- <c>action</c> callbacks</em>
- broadcasts <em>-- talker/listener pattern</em>
I>
\ifsli{\vspace{-1cm}}{}
</slide>


Why not abolish events altogether, then?
The title of the panel, obviously indicates
the reluctance of many software developers
to part from events in modeling the behavior of
(object-oriented) systems.
This reluctance may be explained (partly) as
a drive for continuity,
a natural opposition to throwing away techniques that
have proven to be beneficial in the past.
(This is probably also the reason that event-based
techniques also occur in modernized,
read object-oriented, versions of structured
development methods, as examined in section \ref{methods}.)
However, there are also strong arguments in favor
of the position that {\em events}
(which are extraneous to objects)
are an essential ingredient (in modeling the behavior)
of object-oriented systems.
See also sections \ref{perspectives} and \ref{events}.

The notion of events may play a role in requirements
analysis, as well as in design and implementation.
As for requirements analysis,
the first author of [_Jacobs92],
who was one of the speakers of the panel,
observed that {\em use case analysis}
(which is part of Objectory) in essence comes 
down to listing all possible events occurring in
the interaction with the user.
Such an analysis specifies the functional 
properties of a system whose behavior will
be realized by an appropriate object model.

As for design, we may note that events
may be used to characterize the interaction between
objects in a generic fashion.
In section \ref{events}, we discussed an example
demonstrating the use of {\em event-driven control}
as a means to manage the complexity of object compositions.

Event-driven control also underlies the implementation
of many window-based systems.
An event coming from the user or window manager
may trigger an action that is defined as a callback
function associated with the event.
\see{11-events}
In the next chapter we will look in more detail
at the way in which control is handled by objects
in response to events occurring in a window-based
system.

@so history
</document>
