<document>
@* <a href=g-met.html>Methods and tools</a>
\label{env-methods}
\label{methods}

Object-oriented software development is
a relatively new technology.
Consequently, ideas with respect to
methodologies supporting an object-oriented
approach are still in flux.
Nevertheless, a plethora of methods and tools 
does exist supporting object-oriented analysis
and design.
\see{11-methods}

\vfig{1}
<slide id=11-methods cap="Software development methods">
@ Methods
\ifsli{\vspace{-0.5cm}}{}
<I
- OOA/D {\em -- incremental} \zline{[_CY91b]}
- Objectory {\em -- use-case analysis} \zline{[_Jacobs92]}
- OOSA {\em -- model-driven} \zline{[_Kurtz90]}
- OOSD {\em -- structured} \zline{[_Wasserman89]}
- CRC {\em -- cards} \zline{[_BC89]}
- RDD {\em -- responsibility-driven} \zline{[_Wirfs89]}
- OMT {\em -- object modeling} \zline{[_Rum91]}
- OOD {\em -- development} \zline{[_Booch91]}
- Fusion {\em -- lifecycle} \zline{[_Fusion]}
I>
\ifsli{\vspace{-1.0cm}}{}
</slide>

In the list above, we have collected
a number of development methods.
With the exception of OOSA
(which proposes a model-driven approach to analysis), OOSD (which introduces 
a structured design representation method)
and Fusion
(which presents a promising synthesis of ideas
found in a number of other methods, such
as CRC, RDD and OMT)
we have encountered these methods previously
in chapters 1 and 3.

We have based our discussion of the
object-oriented lifecycle in section
\ref{life} on the OOA/D method
proposed in [_CY90] and [_CY91b],
which stresses the gradual transition from
the analysis to the design and implementation phase.
In section \ref{process} we have briefly discussed
the Objectory method, which proposes
employing a {\em use case analysis}
to arrive at a model specifying
system requirements.
In section \ref{process} and \ref{contracts},
we have discussed the responsibility-driven approach proposed in
[_Wirfs89], and 
the various modeling perspectives suggested by [_Rum91].
Our discussion of object-oriented development in section \ref{identifying}
has been based on Booch (1986, 1991, 1994).


Some of these methods (and corresponding tools)
directly stem from a more conventional
approach to software development.
Others are more radical and propose new tools
to support the decomposition principles
underlying object-oriented technology.

Naturally, those who wish to make a gradual
shift from conventional technology to adopting
an object-oriented approach may benefit
from methods that adapt familiar techniques
to the new concepts.

In this section we will look at a variety of existing
methods and the tools they offer.
We do not discuss the tools and diagram techniques
used in any detail.
However,
we will discuss the Fusion method in some detail.
Fusion is a strongly systematic approach to object-oriented
software development that integrates various concepts
and modeling techniques from the other methods,
notably OMT, Booch OOD and CRC.
We will discuss the process view underlying Fusion and sketch
the models it supports in relation to the other methods.
For the reader this section may
supply an overview and references
needed for a more detailed study of a particular
method or tool.


@ Structured methods
Initially, structured methods (which were developed
the beginning of the 1970s)
were primarily concerned with modeling processes
in a modular way.
Based on software engineering principles such
as {\em module coupling} and {\em cohesion}, tools were 
developed to represent the structure of a design
(within what we have previously called the procedural
or modular paradigm). See, for example, [_Yourdon79].
Apart from diagrams to describe the modular 
architecture of a system 
(such as {\em structure charts} and {\em process specifications}),
structured methods
also employ {\em data flow diagrams}
to depict processes and the flow of data
between them,
and {\em hierarchy diagrams} to model
the structure of the data involved.
\see{11-structured}

\vstr{1}
<slide id=11-structured cap="Tools for a structured approach">
:@ Structured methods
\zline{\fbox{tools}}
\ifsli{\vspace{-0.5cm}}{}
<I
- structure chart
- process specification
- dataflow diagrams
- hierarchy diagram
- entity-relationship diagrams
- data dictionary
- state transition diagram
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Later, structured methods were extended to encompass
analysis, and the focus shifted
to modeling the data by means of
{\em entity-relationship diagrams} and {\em data dictionaries}.
Also, {\em state transition diagrams} were employed
to represent the behavioral aspects of a system.

As observed in [_Fichman], {\em in the late 1970s and
early 1980s, planning and modeling of data began to take on a more
central role in system development,
culminating in data oriented methodologies, such as
information engineering}
(which may be regarded as precursors to object-oriented
methods).
Information engineering, however, is
primarily concerned with analysis and strategic planning.
In addition to the modeling techniques mentioned,
tools were developed to model the information needs
of an enterprise and to perform risk analysis.
Also, extensions to the {\em data dictionary} were
proposed in order to have an integrated
repository, serving all phases of
the development.
Currently, repository-based techniques are again
of interest since, in combination
with modern hypermedia technology, they may serve
as the organizational basis for reuse.


@+ Requirements engineering {\em -- Fusion}
\label{Fusion}

The Fusion method is presented in [_Fusion]
as a second generation object-oriented method.
The phrase {\em second generation} is meant
to indicate the method transcends and incorporates
the ideas and techniques that were employed
in the early object-oriented methods.

Above all, the Fusion method focuses on a strongly
systematic approach to object-oriented software
development, with an emphasis on the {\em process}
of development
and the validation of the consistency between the 
models delivered in the various phases of a project.

The software lifecycle model underlying Fusion
is the traditional waterfall model, consisting
of the subsequent phases of analysis, design and
implementation.
Each phase results in a number of models
describing particular aspects of the system.
\see{11-fusion}
A {\em data dictionary} is to be kept
as a means to unify the terminology employed
in the various phases.

<slide id=11-fusion cap="The Fusion method">
:@ Analysis <control> -- Fusion</control>
\zline{\fbox{Fusion}}
<I
- {\em Object Model} -- concepts and relations
- {\em LifeCycle Model} -- sequences of operations
- {\em Operation Model} -- semantics of system operations
I>
:@ Design <control> -- data dictionary</control>
\zline{\fbox{data dictionary}}
<I
- {\em Object Interaction Graph} --
functional dependencies
- {\em Visibility Graphs} --
communication structure
- {\em Class Descriptions} --
attributes and methods
- {\em Inheritance Graphs} --
subtype refinement
I>
:@ Implementation <control> -- validation</control>
\zline{\fbox{validation}}
<I
- {\em System Lifecycle} --
state machines
- {\em Class Descriptions} --
coding,  performance
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The models produced as the result of analysis,
design and implementation serve to document
the decisions made during the development.

%%@ Analysis
Each of the phases covers different aspects of the
system.
Analysis serves to document the system requirements
from a user perspective.
The Fusion method describes how to construct
an {\em Object Model} that captures the basic
concepts of the application domain.
These concepts are represented as entities or objects
and are connected by relations,
similar to {\em entity-relationship} diagrams
employed in semantic modeling.
Analysis also results in an {\em Operation Model},
describing the semantics of the operations that may be performed
by a user by means of pre- and post-conditions, in a formal manner.
In addition, Fusion defines a {\em Lifecycle Model}
that describes, by means of regular expressions,
which sequences of operations are allowed,
as in the JSD method.

%%@ Design
Design may be considered as the transition between
analysis and implementation.
During design, decisions are made with respect to
the realization of the system operations identified
during analysis.
Design according to the Fusion method results
in an {\em Object Interaction Graph},
that for each system operation describes
which objects are involved and which methods
are invoked.
Fusion also allows one to label the arrows
representing method calls in the interaction diagram
with sequencing information.
In addition, design involves the construction
of {\em Visibility Graphs}, indicating the
attribute and method interface for each object,
{\em Class Descriptions},
defining the attributes and methods of objects,
and {\em Inheritance Graphs},
specifying the subtype refinement relation between
classes.

%%@ Implemenation
Implementation is considered in the
Fusion method as a phase in which to work out the
details of the decisions taken during analysis and design.
It results in a {\em System Lifecycle} description for
each object identified in the {\em Object Model},
in the form of a finite state machine,
and precise {\em Class Descriptions},
in the form of (preferably) efficient code.

@ Validation
An important aspect of the Fusion method is the
validation of the completeness and
consistency of the collection of models.
Completeness, obviously, is a relative matter
and can only be established with respect to
explicitly stated user requirements.
However, the models developed in a particular
phase impose additional  requirements
upon the efforts engaged in the later phases
and in the end maintenance.
Consistency involves verifying whether
the various models are not contradictory.
For both development and validation,
the data dictionary plays an important role,
as a common point of reference.


@+ Tools for analysis and design {\em -- a comparative study}

In [_Fichman] a comparative review of a selected number of
object-oriented analysis and design methods is given.
Criteria for selection were the availability
of documentation and acceptance in the object-oriented
community,
measured in terms of refereed articles.

Paraphrasing [_Fichman] again:
<em>As with traditional analysis, the primary goal of
object-oriented analysis is the development of an
accurate and complete description of the problem
domain.</em>

The three analysis models described in [_Fichman]
share a number of diagram techniques
with both structured methods and methods for
object-oriented design.
However, in particular the method proposed
in [_Shlaer88] reflects the domain oriented focus
of analysis.

A similar focus on domain requirements
and analysis may be found in the
Objectory method.
\see{11-compar-1}
Objectory is one of the methods that
has inspired Fusion,
in particular because it presents a systematic
approach to the process of software development.
The Objectory method centers around {\em use case}
analysis.
Use case analysis involves a precise description
of the interaction between the user of a system
and the components representing domain-specific
functionality.
The Objectory method gives precise guidelines
on how to proceed from the identification of
{\em use cases}, which include user interface aspects,
to their realization in the subsequent phases of
design and implementation.
Objects are called blocks in Objectory.
Use case analysis corresponds in a loose way with
the identification of system operations in Fusion.



<slide id=11-compar-1 cap="Comparison of methods (1)">
:@ Objectory <em> -- systematic process</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- requirements
{\em -- use cases, domain object model, user interface}
- analysis
{\em -- subsystems}
- design, implementation
{\em -- block model, interaction diagrams}
I>
:@ OMT <em> -- few rules for <c>discovering</c> inconsistencies</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- analysis
{\em -- object model, dynamic model, functional model}
- design, implementation
{\em -- heuristics to implement analysis models}
I>
:@ Booch <em> -- descriptive</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- diagrams 
{\em -- class, object, timing, state, module, process}
I>
:@ CRC <em> -- exploratory</em>
\ifsli{\vspace{-0.5cm}}{}
<I
- analysis, design
{\em -- class,responsibilities, collaborators}
I>
:@ Formal methods
\ifsli{\vspace{-0.5cm}}{}
<I
- operations
{\em -- pre- and post-conditions}
I>
\ifsli{\vspace{-1cm}}{}
</slide>


There is a close correspondence between the OMT
object model and the analysis object model of Fusion.
Both OMT and Fusion employ extended entity-relationship
diagrams.
Also, the dynamic model of OMT reoccurs in the
Fusion method, albeit in a later phase.
The functional model of OMT, which has the form of a 
dataflow diagram, is generally considered to be 
inappropriate for object-oriented analysis.
Instead, Fusion employs a model in which
the semantics of system operations are captured
by means of formal pre- and post-conditions.
In [_Fusion], OMT is characterized as a very loose
method, giving few rules for discovering
inconsistencies between the various models
and lacking a clear view with respect to the
process of development.
OMT is strongly focused on analysis,
giving nothing but heuristics to implement
the models that result from analysis.
However, what is called the {\em light-weight Fusion method}
almost coincides with OMT.

A lack of detailed guidelines for the
process of software development is also characteristic
for the Booch OOD method.
Booch offers a wealth of descriptive diagrams,
giving detailed information on the various
aspects of a system,
but offers merely heuristics for the actual
process of development.

The CRC method must be regarded primarily as a means
to explore the interaction between the various
objects of a domain.
It is powerful in generating ideas, but offers
poor support for documenting the decisions
with respect to the objects and how they interact.

Formal methods have been another
important source of inspiration for the Fusion method.
The description of system operations
during analysis employs a characterization
of the functionality of operations that
is directly related to
the specification of operations in model-based
specification methods such as VDM and Z. See section \ref{formal-coop}.

The Fusion method may be regarded as being composed of
elements of the methods mentioned above.
It shares  its object model with OMT,
its approach to the characterization of system
operations with formal methods,
its focus on object interaction with CRC
and its explicit description of classes and
their relations with Booch.
\see{11-compar-2}




<slide id=11-compar-2 cap="Comparison of methods (2)">
<control>
:@ Comparison - as a systematic approach
<table border=0 cellpadding=10>
<tr><td></td>
<td>Objectory</td>
<td>OMT</td>
<td>Booch</td>
<td>CRC</td>
<td>Fusion</td>
</tr>
<tr><td>development</td><td align=center>+</td><td align=center>+-</td><td align=center>-</td><td align=center>x</td><td align=center>+</td></tr>
<tr><td align=left>maintenance</td><td align=center>+</td><td align=center>+-</td><td align=center>+</td><td align=center>-</td><td align=center>+</td></tr>
<tr><td align=left>structure</td><td align=center>+-</td><td align=center>+-</td><td align=center>+</td><td align=center>+</td><td align=center>+</td></tr>
<tr><td align=left>management</td><td align=center>+</td><td align=center>+-</td><td align=center>+-</td><td align=center>-</td><td align=center>+</td></tr>
<tr><td align=left>tool support</td><td align=center>+-</td><td align=center>+-</td><td align=center>+-</td><td align=center>-</td><td align=center>+</td></tr>
</table>
</control>
<?latex>
:@ Comparison
{\em -- as a systematic approach}
\c{\zline{\fbox{Fusion}}}
\ifsli{\vspace{-1cm}}{}
\ln
\begin{tabular}{ l c c c c c } 
 & Objectory & OMT  & Booch & CRC & \c{Fusion} \\
 \hline
\c{development} & 
    $ + $      & $\pm$ & $-$   & $\oplus$ & \c{$ + $} \\
\c{maintenance} &
    $ + $      & $\pm$ & $ + $   & $-$ & \c{$ + $} \\
\c{structure}   &
    $\pm$      & $\pm$ & $ + $   & $ + $ & \c{$ + $} \\
\c{management} &
    $ + $      & $\pm$ & $\pm$   & $-$ & \c{$ + $} \\
\c{tool support} &
    $\pm$      & $\pm$ & $\pm$   & $-$ & \c{$ + $} 
\end{tabular}
<?/latex>
</slide>

In comparison with these methods, however,
it provides a much more systematic approach
to the process of development
and, moreover,
is explicitly concerned with issues of validation
and consistency between models.
In addition, [_Fusion] claim
to provide explicit semantics for their various models,
whereas the other methods fail to do so.
However, it must be remarked that the Fusion method
remains somewhat obscure about the nature of system operations.
System operations are characterized as {\em asynchronous}.
Yet, if they are to be taken as methods, such operations
may return a result, which is quite hard to reconcile
with their asynchronous nature.
The claim that the models have a precise semantics,
which is essential for tool support, must be
substantiated by providing an explicit semantics
in a formal manner!

With regard to the process of development,
both Objectory and Fusion provide precise
guidelines.
The CRC method may be valuable
as an additional exploratory device.
For maintenance, the extent to which
a method enforces the documentation of design
decisions is of the utmost importance.
Both the Objectory and Booch method  satisfys
this criterion, as does the Fusion method.
OMT is lacking in this respect, and CRC is clearly inadequate.

Whether a method leads to a {\em good} object-oriented design
of the system architecture, depends to a large extent upon the
ability and experience of the development team.
Apart from Fusion, both the Booch method and CRC
may be characterized as purely object-oriented,
whereas Objectory and OMT are considered to be impure.

A strongly systematic approach to the process of
development is important in particular from
the point of view of project management.
Project management support entails a precise definition
of the deliverables associated with each phase,
as well as an indication of the timing
of their deliverance and validation.
Both the OMT method and Booch are lacking in this
respect, since they primarily provide techniques
to develop descriptive models.
Clearly, CRC lacks any support for project management.

Tool support is dependent on the existence of
a well-defined semantics for the models employed.
For both Objectory and OMT commercial tools
are available, despite their loosely specified semantics.
The Fusion diagraming techniques are also supported,
for example by Paradigm Plus, which also
supports the diagraming techniques of OMT. See section \ref{pde}.
For CRC, tool support is considered
to be useless.
The success of the method depends upon flexibility,
the ease with which new ideas can be tried,
a flexibility which even hypertext cannot offer,
according its authors.

An important question, when considering methods
for software development,
is what means we wish to have to
represent the decisions taken during design.


@ Design representation
As phrased in [_Fichman],
{\em design is the process of mapping system requirements defined
during analysis to an abstract representation of a specific
system-based implementation meeting cost and performance constraints}.
In other words, design is concerned with architectural issues,
whereas analysis is concerned with requirements and constraints.
Although different in objective, in an object-oriented approach
analysis and design may share a common object model.
Both in the method proposed by [_CY91b] and the method proposed in [_Booch91],
the difference between analysis and design lies primarily in the level of detail
in which the respective components of the system are specified.
Many of the diagram techniques employed, however, have no clear
semantics, and rely on an informal description
of common cases only.
In \sliref{11-diagrams}
a number of the diagram types proposed in [_Booch91] are listed.

<slide id=11-diagrams cap="Diagrams for design">
@ Diagrams 
\zline{\fbox{Booch}}
<I
- class diagram/template
- object diagram/template <c> {\em -- dynamics} </c>
- module diagram/template
- process diagram/template <c> {\em -- physical design} </c>
- state transition diagram
- timing diagram
I>
\ifsli{\vspace{-1cm}}{}
</slide>

Apart from {\em class diagrams} and {\em templates}
to describe the functionality
of classes and their (static) relations to other classes,
the methods also uses {\em object diagrams} (and templates) to
characterize the dynamic behavior of (collections of) objects.
Moreover, the method provides for 
{\em module diagrams}
to depict
the relations between modules and to describe the physical architecture
of distributed systems.
As additional techniques to describe the behavioral characteristics of object,
[_Booch91] employs {\em state-transition diagrams} and {\em timing diagrams}
(expressing constraints on the order in which methods may be invoked).

These diagram techniques have as a common objective to represent the
important properties and aspects of a design.
Along the dimensions by which we may classify design representation
technologies, {\em expressiveness} and {\em formality},
object-oriented technology may be ranked as being midway between non-formal,
highly expressive techniques such as hypertext, and highly formal (logic-based)
techniques which require a lot of expertise to express relationships
and properties.
However, object-oriented techniques cover a broad range.
For example, {\em collaboration graphs} (Wirfs-Brock {\it et al.}, 1990) and {\em contracts}
([_Meyer88]) offer (in principle)
a high degree of formality, whereas many of the diagram techniques
seem to be amenable to an extension with hypertext features
to support browsing and the retrieval of information.
See [_CY91b], [_Bigelow88] and [_Webster].

A major problem that is still not satisfactorily solved by any of the
existing methods is to support what may be called the {\em harvesting of reuse},
that is the actual exploitation of the reuse potential that
is intrinsic to object-oriented technology.
Both at the level of analysis and design,
a repository-based approach to managing reuse (in combination with appropriate
retrieval techniques) seems to be the most promising.
Another issue about which opinions diverge is by what principles objects
should be clustered, and what techniques must be provided to support  system partitioning.
Perhaps the solution to these problems is not strictly a matter of
technology, but instead requires us to wait for appropriate conventions
and styles to emerge within each particular application domain.


@so pde
</document>
