<document>
\label{dialogs}

:@ Dialogs
Interactive applications may require
the user to type some input
after reading a message or
to select an item from a list
of alternatives.
One of the widgets that may be used in a dialog
with the user is the {\em filechooser}
widget.
The {\em filechooser} widget consists
of a $listbox$ filled with filenames
and an $entry$ widget that contains
the filename selected by
the user (by double clicking on the name)
or which may, alternatively, 
be used to type in a filename directly.
In addition, the {\em filechooser}
has an $OK$ button, to confirm the choice
and a $cancel$ button, to break off
the dialog.
Below, the construction of a simplified
version of the {\em filechooser} will
be discussed briefly.

Window-based interactive applications differ from
ordinary interactive applications
by relying on an event-driven flow of control.
The indirection that is typical for
event-driven control is exemplified
in the definition of the
{\em file\_handler}
depicted in \sliref{class-filehandler},
that was invoked by the {\em file\_menu}
in section \ref{menus}.

@so sli-file

:@ The {\em file\_handler} widget
Since the {\em file\_handler} does not
correspond to an actual widget when created,
its constructor merely stores the
canvas pointer, which is actually a pointer
to the $tablet$.

In response to the $Open$ or $Save$ 
menu entries, the {\em file\_handler}
launches a {\em file\_chooser}
and declares itself to be the
handler (with the appropriate arguments).
For example, when selecting the $Open$
entry, the {\em file\_chooser} is launched
which eventually calls the {\em file\_$handler::dispatch$}
function
with $OPEN$ as its argument.
The {\em file\_handler} then invokes
the $open$ function,
which results in reading in the file
and destroys the {\em file\_chooser}.
In a similar way, the menu entry $Save$
results in writing the canvas to a postscript
file.
(The code for checking whether the
$OK$ or $cancel$ button is pressed
is left as an exercise.)


:@ The {\em file\_chooser} widget
Despite its simple appearance,
which is left to the imagination of
the reader,
the {\em file\_chooser} widget
has some subtle complexities.

@so sli-choo

A rudimentary {\em file\_chooser} class
is depicted in \sliref{class-filechooser}.
Typically, a {\em filechooser}
is a toplevel widget, that is a widget that
is independently mapped to the screen.
To avoid name clashes the function $gensym$,
which delivers a system-wide unique
name, is used to determine its path.
Apart from the $operator()$ function,
the {\em file\_chooser} has only
one public function $get$,
which delivers the name selected or 
typed in by the user.

The widget components of the
{\em file\_chooser}, two buttons and
the $entry$ and $listbox$ widgets,
are stored in its instance variables.
Further, we have a function $init$
to construct the actual {\em file\_chooser}
widget, a function {\em list} to fill
the $listbox$ and the function $install$,
which is used to install an external handler
for the two button widgets.
The $install$ function is defined as
[P
void file\_chooser::install(action& a, char* args) {
  b->handler(a,args);
  c->handler(a,args);
  }
P]
Recall, that when declaring a handler for
a button, the name of the button is given
as an additional argument when invoking the handler.
This enables the {\em file\_handler}
to distinguish between a call due
to pressing the $OK$ button and a call
due to pressing the $cancel$ button.

The interplay between the C++ definition
and the underlying Tcl/Tk toolkit is nicely
illustrated by the definition of the $list$ function.
[P
void file_chooser::list() {
  sprintf(buf,"foreach i [exec ls] { \%s insert end \$i }", l->path());
  tk->eval( buf );
  }
P]
Calling $list$ results in filling the
$listbox$ with the filenames in the
current directory.
Its corresponding definition in C++
would, no doubt, be much more involved.

The $init$ function constructs the
various component widgets
and an auxiliary frame widget to
obtain the desired layout.
It further defines the appropriate
event binding for the $listbox$, letting
double clicks result in setting the 
$entry$.
</document>
