<document>
\label{draw}

<latex>
\fslide{ex-hush}{Painting a canvas}{
\ln
\hspace*{0cm}\epsfbox{draw.eps}
\ln
}
</latex>


Before looking at the program, think of what
you would like a drawing editor to offer
you.
And, if you have any experience in programming
graphics applications,
how would you approach the implementation
of a drawing editor?

A drawing editor is a typical example of
an interactive program.
As a first
approximation, we will define a drawing editor
that allows the user
to paint a series of black dots by pressing
and moving the mouse button.
See \sliref{ex-hush}.
}

<slide id=draw cap="A simple drawing tool">
@ds lib/draw.c
</slide>

The program realizing our first attempt
is depicted in \sliref{draw}.
Again, the program may be broken up
in four components.

Component [1] consists of simply including
the {\tt hush.h} header file.

Component [2] defines the class {\em drawing\_canvas}.
The class {\em drawing\_canvas} inherits
from the {\em canvas}
widget class and consequently allows for
drawing figures such as a {\em circle}.
See section \ref{canvas} and the appendix for further details
on the {\em canvas} class.

Now, before looking at the constructor of the
{\em drawing\_canvas}, note that the
member functions {\em press},
{\em motion} and {\em release}
expect a reference to an {\em event}.
These are precisely the member functions
corresponding to the event types
for which the canvas is sensitive.
The meaning of these member functions
becomes clear when looking at the role
of the instance variable {\em dragging}.
When 
{\em dragging} 
is non-zero and a {\em motion} event occurs,
a black dot is painted on the canvas.
Drawing starts when pressing a mouse button and
ends when releasing the button.

Turning back to the constructor (a),
we see that it expects a {\em path} string,
which is passed to the {\em canvas}
ancestor class to create an actual
canvas widget.
Further, the body of the constructor
sets the size of the widget to
200 by 100 and initializes
the variable {\em dragging} to zero.
Finally, the {\em drawing\_canvas} widget
is declared to be its own handler.
The member function {\em handler}
is defined by the class {\em widget}
and results in making the widget
sensitive to a number of predefined
events, that may be different for
each concrete widget class.


."@ Discussion
A note on terminology is in place here.
The reader may be a bit astounded by
the fact that we have both a {\em handler}
class and a {\em handler} function,
which is more properly written
as $widget::handler$.
The situation may become even more confusing
when realizing that the {\em widget}
class itself is a descendant of the
{\em handler} class.
Schematically, we have the situation as depicted
in \sliref{h-disc}.

<slide id=h-disc cap="Handler classes and functions">
[P
 class widget : public handler {
 public:
 ...
 void handler(class handler* h) { ... }
 ...
 };
P]
</slide>

Note that there is no ambiguity here.
A {\em handler} object is an object
that may be invoked in response to a Tcl command
or an event.
The {\em handler} function declares
a {\em handler} object to be responsible
for dealing with the events
that are of interest to the widget.
\ln
In other words,
a {\em drawing\_canvas} fulfills
the dual role of being a widget and its
handler.
This must, however, be explicitly
indicated by the programmer,
which explains the occurrence
of the otherwise mysterious
expression $handler(this)$.
The reason not to identify a widget
with a handler is simply that some widgets
need separate handlers.
Another reason is to
avoid the proliferation of the class
name space, which would inevitably occur
when forcing the programmer to define
the response to events by defining
a descendant class of the particular
widget class.

\ln
Before studying the
abstract {\em handler} class in more detail,
we will briefly look at the definition
of the {\em event} class.
Note that in (b), the event reference
is only used to inform after the position
of the mouse pointer.
</document>
