<document>
@* <a href=../g-hush.html>Multi-paradigm programming</a>
\label{hush}

<document>

Hypermedia programming is by its nature complex,
partly because of the variety of content
that must be dealt with,
and partly because the graphical user
interface forms an essential part of the
hypermedia application.

In this section, we will look at
an approach to graphical interface programming
that may best be characterized
as {\em multi-paradigm programming}.
Our approach,
embodied in the $hush$ library (Eliens, 1994),
combines the use of C++
and the script language Tcl.
The advantage of using C++ is its
robustness. The advantage of using
scripts is, clearly, flexibility.
A multi-paradigm approach offers
the best of both worlds.
Or the worst, for that matter.



The {\em hush} 
library offers an interface to the Tcl/Tk 
(window programming) toolkit,
and a
number
of multimedia devices,
including real-time synthesized audio and MPEG (Motion Pictures Experts Group)
software video.

The principal contribution of the approach
embodied in $hush$ is that it offers
a type-secure solution for connecting
script code with C++ (and vice versa).
In particular, it allows one to associate
events with actions by means of
{\em handler} objects.
In addition, the {\em hush} library
allows the programmer to employ inheritance for
the development of possibly compound
widgets.


{\em Hush} stands for
{\em hyper utility shell}.
The standard interpreter associated with
the {\em hush} library is a shell,
called {\em hush}, including 
a number of the available extensions of
Tcl/Tk and widgets developed by ourselves.
\nop{
(such as a {\em filechooser} and an MPEG video widget).
}
The {\em hush} library offers
a C++ interface to the Tcl/Tk toolkit
and its extensions.
Moreover, a program created with {\em hush}
is itself an interpreter extending the
{\em hush} interpreter.

<document>
\label{Tcl/Tk}

\index{Tcl/Tk}
:@ Tcl/Tk
The language Tcl was first presented
in [_Ousterhout90].
Tcl was announced as a flexible cshell-like
language, intended to be used for developing
an X11-based toolkit.
A year later, the Tk toolkit (based on Tcl)
was presented in [_Ousterhout91].
From the start Tcl/Tk has received a lot of attention,
since it provides a flexible and convenient way in which
to develop quite powerful window applications.

The Tcl language offers variables, assignment
and a procedure construct.
Also it provides a number of control constructs,
facilities for manipulating strings 
and built-in primitives giving access to the underlying
operating system.
The basic Tcl language may easily be extended
by associating a function written in C
with a command name.
Arguments given to the command are passed 
as strings to the function defining the command.

The Tk toolkit is an extension of Tcl
with commands to create and configure widgets
for displaying text and graphics,
and providing facilities for window management.
The Tk toolkit, and the $wish$ interpreter
based on Tk, provides a convenient way
to program X-window based applications.


<slide id=tcl-example cap="A Wish example">
:@ Example
\zline{\fbox{\em wish}}
\ln\ifsli{\vspace{-1.0cm}}{}
\hspace*{2.5cm}
\epsfbox{hello.eps}
\ln\ifsli{\vspace{-1.0cm}}{}
:@ Scripts
{\em -- hello world}
[P
button .b -text "Hello, world" -command {
	puts stdout "hello world"
	}

pack .b 
P]
\ifsli{\ln\vspace{-1cm}}{}
</slide>


The $wish$ program is an interpreter for executing
Tcl/Tk scripts.
As an example of a $wish$ script, look
at the {\em hello world} program in \sliref{tcl-example}.
The {\em hello world} script defines a button that
displays {\em Hello, world}, and prints {\em hello world}
to standard output when it is
activated by pressing the left mouse button.
The language used to write this script
is simply Tcl with the commands defined by Tk,
such as the $button$ command (needed to create
a button) and the $pack$ command (that is used to map
the button to the screen).

The $wish$ program actually provides an example
of a simple application based on Tcl/Tk.
It may easily be extended to include, for example,
3D-graphics by linking the appropriate
C libraries and defining the functions making 
this functionality available as (new) Tcl commands.

."@ The Tcl C API
To define Tcl commands in C,
the programmer has to define a command
function
and declare the function
to be a command in Tcl by
invoking the {\em Tcl\_CreateCommand}
function.


Creating a command is done with reference
to an interpreter, which accounts for the
first argument of {\em Tcl\_CreateCommand}.
The name of the command, as may be used 
in a Tcl script must be given as a second argument,
and the C/C++ function defining the command as a third argument.
Finally, when declaring a command, the address
of a structure containing client data 
may be stored, which may
be (the address of) the root window, for example.

When the function is invoked
as the result of executing the Tcl command,
the client data stored at
declaration time is passed as the first
argument to the function.
Since the type {\em ClientData} is actually defined
to be $void*$, the function must first
cast the client data argument to an appropriate
type.
Evidently, casting is error-prone.

Another problem with command functions
as used in the Tcl C API is that
permanent data are possible only
in the form of client data,
global variables or static local
variables.
Both client data and global variables
are unsafe by being too visible
and static local data are simply inelegant.
</document>

</document>

@+ Program structure 
\label{Structure}

\nop{
The {\em hush} library is intended
to provide a convenient way
to program window-based applications in C++.
As a first example of using {\em hush},
we will look at a simple program 
written in C++ that uses a graphical
interface defined by a Tcl/Tk script.
}

\c{
The {\em hush} library is intended to
provide a convenient way to program window-based applications in C++.
Basically, there are two considerations
that may lead you to employ
the {\em hush} library.
When you are familiar with Tcl/Tk and you need
to combine Tcl scripts with C++ code,
you may use {\em handler} classes to
do so in a relatively type-secure way.
On the other hand, when you want to program
graphical user interfaces in C++,
you may employ the {\em hush} widget
classes.
In the latter case, you may
choose to remain ignorant of
the underlying Tcl/Tk implementation
or exploit the Tcl script facility
to the extent you wish.
\nop{
Whether you choose to use Tcl scripts
or not, the structure of the program
is to a large extent the same.
}
}

\nop{
As an illustration of the structure of a program
using {\em hush},
we will look at a simple program written in C++
that uses a graphical interface defined
by a Tcl/Tk script.

After discussing the example,
we will look at
a brief overview of the classes
that constitute the {\em hush} library.
A more detailed description will
be given of the {\em kit} class,
that encapsulates the embedded Tcl interpreter,
and the {\em session} class, that shields
of the details of the window environment.
}

."so example

\c{
%%:@ The hush class library
\nop{
The example given in the previous section
showed what kind of components are
typically used when developing a program
with the {\em hush} library.
However, instead of employing a Tcl script,
the window interface may also be developed entirely
by employing {\em hush} C++ widgets.
}
In this section, a brief overview 
will be given of the classes offered by
the {\em hush} library.
Further, it will be shown how to construct
the {\em hush} interpreter referred
to in the introduction.
In addition, we will take a closer look at
the classes $kit$ and $session$,
which are needed to communicate with the embedded
Tcl interpreter and to initialize the main event loop.
}

\c{
:@ The hush class library
%%:@ The library
The {\em hush} C++ library consists
of three kinds of classes,
namely (a) the widget classes,
which mimic the functionality of Tk,
(b) the handler classes,
which are involved in the
handling of events and the binding of C++
code to Tcl commands,
and (c)
the classes $kit$ and $session$,
which
encapsulate the 
embedded interpreter
and the window management system,
}

.so f_ov

\c{
In the widget class hierarchy depicted
in \sliref{hush-overview} (a), the {\em widget} class
represents an abstract widget,
defining the commands that are valid for
each of the descendant concrete
widget classes.
The $widget$ class, however, is not an abstract class
in C++ terms.
It may be used for creating
pointers to widgets defined in Tcl.
In contrast, employing the constructor
of one of the concrete widget classes
results in actually creating a widget.
A more detailed example showing the functionality
offered by the widget classes will be given
in section \ref{user}.
A description of the individual widget classes
is included in appendix \ref{hush-man}.
}

\c{
The class hierarchy depicted in \sliref{hush-overview} (b)
depicts the {\em handler} class as a subclass
of client.
The reason for this will become clear in section \ref{Binding}.
The {\em handler} class may also be considered
an abstract class, in the sense that
it is intended to be used as the ancestor of
a user-defined handler class.
The {\em handler} class has two
pre-defined descendant classes,
namely the {\em widget} class and
the {\em item} class.
This implies, indeed, that both
the {\em widget} and the {\em item} classes
(the latter of which is discussed in section \ref{hush-item})
may also be used as ancestor handler classes.
The reason for this is that any descendant
of a {\em widget} or {\em item}
class may declare itself to
be its own handler and define the actions
that are invoked in response to
particular events.
This will be illustrated and discussed
in sections \ref{Binding} and \ref{user}.
}


\c{
:@ The hush interpreter
In the introduction, {\em hush}
was announced as both a C++ library
and as an interpreter extending
the {\em wish} interpreter.
The program shown in \sliref{hush-interpreter}
substantiates this claim,
albeit in a perhaps disappointingly simple way.
}

\slide{hush-interpreter}{The {\em hush} interpreter}{
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
\ifsli{
.ds lib/sli-hush.c
}{
.ds lib/hush.c
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}

\c{
The structure of the program is
characteristic for {\em hush}-based applications.
\nop{
similar
to the C++ example of section \ref{hush-example}.
}
Part [1] consists merely of including
the {\tt hush.h} header file
and may possibly define additional functionality.
\nop{
Part [2] is empty.
}
Part [2] consists of an application class,
derived from {\em session},
defining how the {\em hush} interpreter
deals with command-line
arguments (a and d) and the initialization that
takes place when the main event loop is
started (c).
To understand (a) and (d) it is enough
to know that the {\em hush} library
provides a hypertext widget and that
the {\tt -x} option treats the next
argument as the name of a hypertext file.
In section \ref{hyper}, an example will be given that
involves the hypertext widget.
In (b) the {\tt hush.tcl} file is declared to be the initialization
file.
It contains the Tcl code for installing the extensions loaded
in (c).
In (e), a predefined button {\tt .quit} is packed to the root
widget.
Part [3] is needed to initialize the application
and start the main event loop.
}

\c{
The {\em hush} interpreter defined by the program
extends the {\em wish} interpreter
by loading some extensions to Tcl
\nop{discussed in the introduction}
and by allowing for the display of
a hypertext file.
The interpreter accepts any command-line
argument accepted by the {\em wish}
interpreter, in addition to the
{\tt -x} hypertext option.
}
\nop{
The Tcl interface script given
in \sliref{int-script},
for example, may be executed using the
{\em hush} interpreter.
}


\c{
:@ The {\em kit} class
$Hush$ is meant to provide a parsimonious C++
interface to Tcl/Tk.
Nevertheless, as with many a toolkit, some
kind of API shock seems to be unavoidable.
This is especially true for
the $widget$ class (covered in section \ref{hush-widget})
and the class $kit$
defining the C++ interface with the embedded Tcl interpreter.
The functionality of $kit$ can only be completely
understood after reading this section.
However, since an instance of $kit$ is used in almost
any other object (class), it is presented here first.
See \sliref{class-kit}.
The reader will undoubtly gradually learn the
functionality of $kit$ by studying the examples.
}

\c{
To understand why a $kit$ class is needed,
recall that each {\em hush} program contains an embedded
Tcl interpreter.
The $kit$ class encapsulates this interpreter
and provides a collection of member functions
to interact with the embedded interpreter.
}


\nop{
The $kit$ class is derived from the class $hcl$  which
provides access to the lower level details of (an extended version of)
Tcl. \nop{Consult the reference manual.}
}

\c{
The first group of functions ($eval$, $result$,
$evaluate$ and $source$) may be used to execute
commands in the Tcl scripting language directly.
A Tcl command is simply a string conforming to
certain syntactic requirements.
The function $eval$ evaluates a Tcl command.
The function $result()$ may be used to fetch
the result of the last Tcl command.
In contrast, the function $result(char*)$ may be used
to set the result of a Tcl command, when this
command is defined in C++ (as may be done with $kit::action$).
The function $evaluate$ provides a shorthand for
combining $eval$ and $result()$.
The function $source$ may be used to read in
a file containing a Tcl script.
}

\c{
Also, we have the $kit::action$ function
that may be used to associate a Tcl command with a
handler object.
In section \ref{action},
alternative ways of defining an action are discussed.
}

\nop{
The function $after$ may be used to have a
Tcl command evaluated after $n$ milliseconds.
(It makes use of X timer callbacks).
The function $send$ may be used to evaluate
a command in a different application.
An application may be given a name when
creating the $session$ object.
The function $selection$ delivers the current X selection.
The function $update$ may be used to process any
pending events.
These functions are only for experienced users.
See section \ref{tcl}.
}

\c{
The next group of functions is related to widgets
and events that may occur to widgets.
The $event$ function delivers the latest event.
It may only be used in a command that is bound
to some particular event.
See section \ref{hush-events}.
When other events occur before accessing the event object,
the information it contains may be obsolete.
}

\slide{class-kit}{The {\em kit} class}{
."@ The interpreter $kit$
%%\zline{\fbox{kit}}
\ifsli{}{\vspace{0.1cm}}
\ifsli{
[P
interface kit { /// \fbox{kit} 

int eval(char* cmd); 
char* result();     
void result(char* s);   
char* evaluate(char* cmd); 
int source(char* f); 

action& action(char* name, handler* h);

class event event();

widget* root(); 
widget* pack(widget* w, char* opts = "");
widget* pack(char* wp, char* opts = "";

char* selection(char* opts=""); 

void after(int msecs, char* cmd); 
void after(int n, handler* h);

void update(char* opts="");

char* send(char* it, char* cmd);

void trace(int level = 1); 
void notrace(); 
void quit(); 
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}{
[P
interface kit { /// \fbox{kit} 

int eval(char* cmd); ///// to evaluate script command
char* result();     ///// to fetch the result of $eval$
void result(char* s);   ///// to set the result of $eval$
char* evaluate(char* cmd) ///// combines $eval$ and $result$
int source(char* f); ///// to load a script from file

action& action(char* name, handler* h);

class event event(); ///// returns the last event

widget* root(); ///// returns toplevel (root) widget
widget* pack(widget* w, char* options = "");
widget* pack(char* wp, char* options = "";

char* selection(char* options=""); ///// X environment

void after(int msecs, char* cmd); 
void after(int n, handler* h);

void update(char* options="");

char* send(char* it, char* cmd);

void trace(int level = 1); 
void notrace(); 
void quit() ///// to terminate the session
};
P]
}
\ifsli{}{\vspace{-0.2cm}}
}



\c{
The function $root$ gives access to the toplevel root
widget associated with that particular
instance of the $kit$.
The function $pack$ may be used to append
widgets to the root widget, in order to map them to the screen.
Widgets may be identified either by a pointer to
a $widget$ object or by their {\em path name},
which is a string.
See section \ref{widget}.
}

\c{
Next, we have a group of functions related to the
X environment.
The function $selection$ delivers
the current X selection.
The function $after$ may be used to set a timer
callback for a handler.
Setting a timer callback means that the 
handler object will be invoked after
the number of milliseconds given as the first
argument to $after$.

The function $update$ may be used to ensure
that all pending events are processed.
For example, when moving items on a canvas,
an update may be needed to make the changes visible.
Also, we have a $send$ function that may be used
to communicate with other Tcl/Tk
applications.
The first argument of $send$ must be the name of
an application, which may be set when creating
a session.
}



\nop{
The function $action$ may be used to associate a Tcl command to
a command function or handler written in C++.
See section \ref{action} for details.
}

\nop{
A kit object, which may be only created by session(),
provides an interface to the tcl interpreter
and the global properties of the system.
The function eval() may be used to execute
commands in tcl and to retrieve the result.
The function event delivers the latest event.
When other events occur before accessing a particular
event object, the information gotten will be obsolete.
}

\c{
Further, we have 
the functions $trace()$ and $notrace()$, which may be used to turn
on, respectively off, tracing.
The level indicates at what detail information will
be given. Trace level zero is equivalent to notrace().
Finally, the function $quit$ may be used to terminate 
the main event loop.
}


\c{
:@ The {\em session} class
Each program written with {\em hush}
may contain only one embedded
interpreter.
To initialize the {\em kit} object wrapping
the interpreter and to start the main event loop,
an instance of the class {\em session}
must be created.
See \sliref{class-session}.

The preferred way of doing this is by defining
a descendant class of the {\em session} class,
redefining the virtual function
$session::main$ to specify what needs 
to be done before starting the main loop.
In addition, the constructor of the newly
defined class may be used to check
command line arguments and to initialize
application specific data,
as illustrated in \sliref{hush-interpreter}.
}

\slide{class-session}{The {\em session} class}{
."@ The {\em session} class
%%\zline{\fbox{session}}
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
[P
interface session  { /// \fbox{session}
session(int argc, char** argv, char* name = 0);
void init(char* fname);
virtual void prelude(kit* tk, int argc, char* argv[] );
virtual void main(kit* tk, int argc, char* argv[] );
int run( );
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}



\c{
When creating a $session$ object, the $name$ of the application
may be given as the last parameter.
Under this name, the application is known to other
Tk applications, which may communicate with each other
by means of the $send$ command.

The function $init$ may be used to specify a different
initialization script.
This script must include the default $hush$
initialization script, which is an adapted version
of the original $wish$ initialization script.
}

\c{
Apart from the function $main$, 
a function $prelude$ may also be defined.
When the program is used as an interpreter
(by giving {\tt -f $file$} as command line arguments)
only the $prelude$ function will be executed,
otherwise $prelude$ will be executed before 
$main$.
In interpreter mode, the $main$ function will
also be executed when the script contains
the command {\em go-back}.
Finally, the function $run$ must be called
to actually initialize the program and 
start the main loop.
}

<document>
@+ Binding actions to events
\label{Binding}

Handler objects may be bound to Tcl commands or events.
Events are generated by the (X) window environment
in response to actions of the user.
These actions include pressing a mouse button,
releasing a mouse button, moving the mouse, etc.
Instead of explicitly dealing with all incoming
events,
applications delegate control to the environment by
associating a callback function
with each event that is relevant for a particular widget.
This mechanism frees the programmer from the
responsibility of deciding to which widget the event belongs
and what action to take.

Nevertheless, from the perspective of program
design, the proper organization of
the callback functions is not a trivial
matter.
Common practice is to write only a limited number of
callback functions and perform explicit
dispatching according to the type of event.

An object-oriented approach may be advantageous
as a means to organize a collection of
callback functions as member functions of
a single class.
One way of doing this is to define
an abstract event handler class which
provides a virtual member function for
each of the most commonly occurring events.
In effect, such a handler class  hides
the dispatching according to the type
of the event.
A concrete handler class may then
be defined simply by overriding
the member functions corresponding to
the events of interest.

Before studying the
abstract {\em handler} class in more detail,
we will briefly look at the definition
of the {\em event} class.

<document>

\label{hush-events}
:@ Events
Events always belong to a particular widget.
To which widget events are actually directed
depends upon
whether the programmer has defined a binding for the
event type.
When such a binding exists for a widget
and the (toolkit) environment decides that
the event belongs to that widget, then the callback
associated with that event is executed.
Information concerning the event may be retrieved
by asking the kit for the latest event.

\slide{class-event}{The {\em event} class}{
."@ Event
%%\zline{\fbox{event}}
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
\ifsli{
[P
interface event  {  /// \fbox{event}
int type();                    /// \c{ X event type }
char* name();                  /// \c{type as string}

int x();			
int y();

 ...
void* rawevent(); ///// \c{delivers} raw X event
};
P]
}{
[P
interface event  {  /// \fbox{event}
int type();                    /// \c{// X event type }
char* name();                  /// \c{// type as string}

int x();			
int y();

int button(int i = 0);          /// \c{// ButtonPress}
int buttonup(int i = 0);        /// \c{// ButtonRelease}
int motion();                   /// \c{// MotionNotify}

int keyevent();                 /// \c{// KeyPress or KeyRelease}
int buttonevent(int i = 0);     /// \c{// ButtonPress or Release}

int keycode();

void trace();                   /// \c{// prints event information}

void* rawevent(); /// // \c{delivers} raw X event
};
P]
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}



Event objects represent the events generated by
the X-window system.
Each event has a type.
The type of the event can be inspected with $type()$ which
returns an integer value or $name()$ which returns
a string representation of the type.
For some of the common event types, such as $ButtonPress$,
$ButtonRelease$ and $MotionNotify$, member functions
are provided to facilitate testing.
If an integer argument (1, 2 or 3) is given to $button$,
$buttonup$ or $buttonevent$, it is checked whether
the event has occurred for the corresponding button. 

The functions $x$ and $y$ deliver the widget
coordinates of the event, if appropriate.

Calling $trace$ for the event results in printing
the type and coordinate information for the event.
When setting the $kit::trace$ level to 2 this
information is automatically printed.


Programmers not satisfied with this interface can
check the type and access the underlying XEvent at their
own risk.
</document>

\label{handler}

."so sli-dispatch

\c{
:@ Handlers
Handler objects provide a type secure
way in which to deal with local data and global
resources needed when responding to an event.
A handler object may store the information
it needs in instance variables,
when its constructor is called.
See \sliref{class-handler}.
}

\slide{class-handler}{The {\em handler} class}{
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
[P
interface handler : client { /// \c{\fbox{handler}}
virtual int dispatch(kit* _tk, int _argc, char* _argv[]);  
virtual int operator()();
virtual void press( event& ) { }
virtual void release( event& ) { }
virtual void keypress( event& ) { }
virtual void keyrelease( event& ) { }
virtual void motion( event& ) { }
virtual void enter( event& ) { }
virtual void leave( event& ) { }
virtual void other( event& ) { }
protected:
int argc;
char** argv;
kit* tk;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
Activating a handler object in response
to an event or a Tcl command occurs
by calling the $dispatch$ function
of the handler.
The system takes care of this,
provided that the user has bound
the handler object to a Tcl command
or event.
}

\slide{dispatching}{The  {\em dispatch} and $operator()$ function}{
."@ Dispatching
\ifsli{}{\vspace{0.1cm}}
\ifsli{
[P
int handler::dispatch(kit* _tk, int _argc, char* _argv[]) {
tk = _tk; argc = _argc; argv = _argv;
return this->operator()();
}

int handler::operator()() { /// \c{\fbox{$operator()$}}
event e = tk->event();
if (e.type()==ButtonPress) press(e);
else if (e.type()==ButtonRelease) release(e);
else if (e.type()==KeyPress) keypress(e);
else if (e.type()==KeyRelease) keyrelease(e);
else if (e.type()==MotionNotify) motion(e);
else if (e.type()==EnterNotify) enter(e);
else if (e.type()==LeaveNotify) leave(e);
else other(e);
return OK;
}
P]
}{
[P
int handler::dispatch(kit* _tk, int _argc, char* _argv[]) {
	tk = _tk; argc = _argc; argv = _argv;
	return this->operator()();
}

int handler::operator()() { /// \fbox{$operator()$}
	event e = tk->event();
	if ( e.type() == ButtonPress ) press(e);
	else if ( e.type() == ButtonRelease ) release(e);
	else if ( e.type() == KeyPress ) keypress(e);
	else if ( e.type() == KeyRelease ) keyrelease(e);
	else if ( e.type() == MotionNotify ) motion(e);
	else if ( e.type() == EnterNotify ) enter(e);
	else if ( e.type() == LeaveNotify ) leave(e);
	else other(e);
	return OK;
}
P]
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
The $dispatch$ function, in its turn,
calls the $operator()$ function,
after storing the $kit$, $argc$
and $argv$ arguments in the corresponding
instance variables.
See \sliref{dispatching}.
The $handler::operator()$ function
fetches the latest event from the $kit$
object and selects one of the predefined
member functions ({\em press, motion, release}, etc.)
according to the type of the event.
The original handler class knows only
virtual functions.
Each of these function, including
the $dispatch$ and $operator()$
function, may be redefined.
The two-step indirection, via the
$dispatch$ and $operator()$ functions,
is introduced to facilitate 
derivation by inheritance, 
directly from the $handler$ 
or from classes that are themselves
derived from the $handler$ class,
such as the widget classes.
}




<document>
\label{action}

:@ Actions
Handler objects will be activated only
when a binding has been defined,
by using $kit::action$
or implicitly by employing
$widget::bind$ or $widget::handler$.
Such bindings may also be defined
by $kit::after$ or $item::bind$ and
$item::handler$.
Implicit binding results in the creation of
an anonymous action.

Actions, as characterized by the class $action$,
provide the actual means with which to bind C++
code to Tcl commands.
Actions may be defined either trivially
by a Tcl command,
by a C/C++ $command$ function
or by $handler$ objects.
In effect, defining an action by means
of a handler object amounts, implicitly,
to defining a command function with a
privileged client for which, by convention,
the $dispatch$ function is invoked.
}

\slide{class-client}{The {\em client} class}{
:@ Client
\zline{\fbox{client}}
[P
class client {  };  ///// empty class
P]
\ifsli{\ln\vspace{-1cm}}{}
:@ Command
\zline{\fbox{command}}
[P
typedef command(client*, kit*, int argc, char* argv[]);
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}



Data  passed to a $command$ function must be of the type
$client$, which is defined by an empty
class introduced only to please the compiler.
In \sliref{class-client} the type definition of $command$ is given.
Apart from the $client*$ parameter, a command
function must also declare a $kit*$ parameter
and an $argc$ and $argv$ parameter, similar to that for $main$.
The $client*$ data of a command (and similarly for
the $clientdata$ parameter of a $tclcommand$)
can be any kind of class.
However, it is to be preferred that such classes
are made a subclass of $client$.
The $client$ data pointer is declared when creating
an action and passed to the command function when the actual call
is made.
The other parameters ($argc$ and $argv$) depend
on the actual call.
The use of $argc$ and $argv$ comes from the original
C interface of Tcl.
It proves to be a very flexible way of
communicating data, especially in string-oriented
applications.

<slide id=class-action cap="The action class">
%%@ Action \zline{\fbox{action}}
\ifsli{}{\vspace{0.1cm}}
[P
interface action { /// \fbox{action}
action(char* name);

action(char* name, handler* h);
action(char* name, command f, client* data = 0);
action(char* name, tcl_command f, ClientData data = 0);

action( handler* h); /// \ifsli{}{//} anonymous
action( command f, client* data = 0);
action( tcl_command f, ClientData data = 0);

char* name();   /// \c{// returns the name of an action}
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The class $action$ offers no less than seven
constructors.
The first constructor, which takes a ($char*$)
string as a parameter, is merely
for convenience.
It may be used to convert the name of a Tcl script command
into an $action$.
The following three constructors differ from the last
three constructors only by their first string parameter,
which serves to define the name under which the
action will be known by the Tcl script interpreter.
The last three constructors, in contrast,
create anonymous actions, of which the user, however,
can ask the name by invoking the function $name$.

The preferred form of creating an action is by giving it
(apart from a name) a $handler$ as a parameter.
As already noted,
handler objects offer a type-secure way of dealing with client
information.
In contrast, the second constructor of this group,
which takes a $command$ function and possibly a pointer
to $client$ data as parameters, may make (type-insecure) conversions
of the client data necessary.

The constructor taking a {\em tcl\_command} and $ClientData$ as parameters
is incorporated for compatibility reasons only.

When using any of the last six constructors,
as a side-effect an association is created between
the $name$ of the action and a Tcl command.
If such a Tcl command already exists,
the previous association will be overwritten.
This is also the case if it has been defined
as a Tcl script command.
</document>

</document>
@+ User interface widgets
\label{user}

\c{
The Tk toolkit offers numerous built-in widgets.
The Tk widgets conform to the look-and-feel
of the OSF/Motif standard.
The $hush$ C++ interface for Tk provides
for each Tk widget a class of the same name
which supports the creation of a widget
and allows the user to access and modify it.
In addition to the standard Tk widgets,
the $hush$ library includes
a number of other widgets, such as a $barchart$,
$hypertext$ and $photo$ widget (created by other Tk 
adepts).
Also, widgets of our own making are offered, such as a $filechooser$
and MPEG video widget.
}


<document>
\fslide{eps-drawtool}{ The {\em drawtool} interface}{
\ln
\ifsli{\hspace*{2.5cm}}{\hspace*{0cm}}
\epsfbox{drawtool.eps}
\ln\ifsli{\vspace{-1cm}}{}
}


In this section we will look at the realization of simple drawing tool.
The example illustrates how to use the
{\em hush} library widgets, and
serves to illustrate
in particular how handlers may be attached
to widgets, either by declaration or
by inheritance, and how to construct
compound widgets.

Our approach may be considered object-oriented, in the sense that each component
of the user interface is defined by a class
derived from a widget class.
It must be pointed out beforehand that the main difficulty
in defining compound or mega widgets is not the construction
of the components themselves, but delegating the configuration
and binding instructions to the appropriate components. 
Section \ref{new} shows
how a compound widget
defined in C++ may be made to correspond
to a widget command that may be used
in a Tcl script.
Ideally, defining a new widget
includes both the definition
of a C++ class interface and
the definition of the corresponding Tcl
command.


%%@ Example -- drawtool
Our drawing tool consists of a $tablet$,
which is a canvas with scrollbars
to allow for a large size canvas
of which only a part is displayed,
a {\em menu\_bar}, having a {\em File}
and an {\em Edit} menu, and a {\em toolbox},
which is a collection of buttons for
selecting from among the drawing facilities.
In addition, a help facility is offered.
See \sliref{eps-drawtool}.


<slide id=draw-tool-x cap="The drawing tool">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/drawtool.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


In \sliref{draw-tool-x} the application class for {\em drawtool}
is depicted.
Before the main event loop is started,
the components of the drawing tool are
created and packed to the root widget.
In addition to the {\em tablet}, {\em menu\_bar} and {\em toolbox},
a $frame$ widget is created to pack the
menubar and tablet together.
This is needed to ensure that the geometrical
layout of the widget comes out right.
Each of the component widgets is given
a pointer to the root widget.
In addition, a pointer to the tablet
is given to the {\em toolbox} and a pointer to
the {\em toolbox} is given to the {\em menu\_bar}.
The reason for this will become clear
when discussing the {\em toolbox} and
{\em menu\_bar} in sections \ref{buttons} and \ref{menus},
respectively.
In the example, no attention will be paid to memory management.
</document>


\label{hush-widget}

\c{
@ Configuring widgets
\label{widget}
Widgets are the elements from which a GUI is made.
They appear as windows on the screen to display
text or graphics and may respond to events
such as motioning the mouse or pressing
a key by calling an action associated with that event.

Usually, the various widgets constituting the user
interface are (hierarchically) related to each other,
such as in the $drawtool$ application which contains
a canvas to display graphic elements,
a button toolbox for selecting the graphic
items and a menubar offering various options
such as saving the drawing in a file.
}

\c{
."@ Pathnames
Widgets in Tk are identified by a {\em path name}.
The path name of a widget reflects its possible
subordination to another widget.
See \sliref{widget-hierarchy}.
}

.so f_lay

\c{
Pathnames consist of strings separated by dots.
The first character of a path must be a dot.
The first letter of a path must be lower case.
The format of a path name may be expressed in BNF form as
<E>- $<path> ::= '.' | '.'<string> | <path>'.'<string>$
For example `.' is the path name of the root widget,
whereas `.quit' is the path name of a widget
subordinate to the root widget.
A widget subordinate to another widget
must have the path name of that widget
as part of its own path name.
For example, the widget `.f.m' may have a widget `.f.m.h'
as a subordinate widget.
Note that the widget hierarchy induced by the path names
is completely orthogonal to the widget class inheritance
hierarchy depicted in
\sliref{hush-overview} (a) and
\sliref{widget-classes}.
With respect to the path name hierarchy, when speaking of
ancestors we simply mean superordinate widgets.
}


\c{
Pathnames are treated somewhat more liberally in {\em hush}.
For example, widget path names may simply be defined
or extended by a string.
The missing dot is then automatically inserted.
}


\slide{class-widget}{The {\em widget} class}{
%%@ Widget \zline{\fbox{widget}}
\ifsli{}{\vspace{0.1cm}}
\ifsli{
[P
interface widget : handler { /// \c{\fbox{widget}}
widget(char* p); 
widget(widget& w, char* p); 
char* type(); /// \c{returns type of the widget}
char* path(); /// \c{returns path of the widget}
int eval(char* cmd);  /// \c{invokes "thepath() cmd"}
char* result(); /// \c{returns the result of eval}
char* evaluate(char* cmd) /// \c{combines $eval$ and $result()$}
virtual configure(char* cmd); /// \c{uses '$path()$' }
virtual geometry(int xs, int ys); /// \c{width x height}
widget* pack(char* opts = "" );
widget* pack(widget* w, char* = "" );
bind(char *b, handler* h, char* = "" );
bind(char *b, action& ac, char* = "" );
handler(class handler* h, char* = "" );
handler(action& ac, char* = "" );
 ...
void destroy(); /// \c{removes widget from the screen}
void* tkwin();  /// \c{gives access to Tk window}
widget* self(); ///// \c{for constructing} mega widgets
void redirect(widget* w);
protected:
char* thepath(); ///// \c{delivers the} virtual path
virtual install(action&,char* =""); 
};
P]
}{
[P
interface widget : handler { /// \fbox{widget}

widget(char* p); 
widget(widget& w, char* p); 

char* type(); /// \c{// returns type of the widget}
char* path(); /// \c{// returns path of the widget}

int eval(char* cmd);  /// \c{// invokes "thepath() cmd"}
char* result(); /// \c{// returns the result of eval}
char* evaluate(char* cmd) /// \c{// combines $eval$ and $result()$}

virtual void configure(char* cmd); /// \c{// uses '$path()$' }
virtual void geometry(int xs, int ys); /// \c{// width x height}

widget* pack(char* options = "" );
widget* pack(widget* w, char* options = "" );

bind(char *b, handler* h, char* args = "" );
bind(char *b, action& ac, char* args = "" );

handler(class handler* h, char* args = "" );
handler(action& ac, char* args = "" );

void xscroll(scrollbar* s); /// \c{// to attach scrollbars}
void yscroll(scrollbar* s);

void focus(char* options="");
void grab(char* options="");

void destroy(); /// \c{// removes widget from the screen}

void* tkwin();  /// \c{// gives access to Tk window}

widget* self(); /// \c{// for constructing} mega widgets
void redirect(widget* w);
protected:
char* thepath(); /// \c{// delivers} the virtual path
virtual install(action&,char* args=""); 
};
P]
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
Calling the constructor $widget$ as in
[P] widget* w = new widget(".awry");
does not result in creating an actual
widget but only defines a pointer to the widget
with that particular name.
(It is not an abstract class, technically, since it does not
contain any pure virtual functions, see section \ref{abstract-class}.)
If a widget with that name exists, it may be treated
as an ordinary widget object, otherwise an error will occur.
The constructor $widget(widget* w,char* path)$ creates
a widget by appending the path name $path$ to the
path name of the argument widget $w$.

The function $path$ delivers the path name of
a widget object.
Each widget created by Tk actually defines a
Tcl command associated with the
path name of the widget.
In other words, an actual widget may be regarded
as an object which can be asked to evaluate commands.
For example a widget `.b' may be asked to change
its background color by a Tcl command like
[P] .b configure -background blue
The functions $eval$, $result$ and $evaluate$
enable the programmer to apply Tcl commands
to the widget directly, as does the $configure$
command.
The function $geometry$ sets the width and height of
the widget.
}

\c{
."@ Packing
Naming widgets in a hierarchical fashion does not imply
that the widgets behave accordingly.
The widget class interface offers two
$pack$ functions.
The function $widget::pack(char*)$
applies to individual widgets.
As options one may specify, for example,
{\tt -side $X$}, where {\tt $X$} is either
{\tt top}, {\tt bottom}, {\tt left}
or {\tt right},
to pack the widget to the appropriate side
of the cavity specified by the ancestor widget.
Other options are {\tt -fill x} or
{\tt -fill y}, to fill up the space
in the appropriate dimensions,
or {\tt -padx $N$} or {\tt -pady $N$}, for
some integer {\tt $N$},
to surround the widget with some extra space.

Alternatively, the function
$widget::pack(widget*,char*)$
may be used, which allows for the same options
but applies packing to the widget parameter.
This function is convenient when packing
widgets in a $frame$ or $toplevel$ widget.
As a remark, the $kit::pack$ function may only
be used to pack widgets to the root window.
}

\c{
@ Binding events
Widgets may respond to events.
To associate an event with an action,
an explicit binding must be specified
for that particular widget.
Some widgets provide default bindings.
These may, however, be overruled.

The function $bind$ is used to associate
handlers or actions with events.
The first string parameter of
$bind$ may be used to specify
the event type.
Common event types are,
for example,
{\tt $ButtonPress$}, {\tt $ButtonRelease$}
and {\tt $Motion$}, which are the
default events for canvas widgets.
Also keystrokes may be defined as events,
for example {\tt $Return$},
which is the default event for 
the $entry$ widget.

The function $widget::handler$ may be
used to associate a handler object
or
action with the default bindings
for the widget.
Concrete widgets may not override the $handler$
function itself, but must define the protected
virtual function $install$.
Typically, the install function consists
of calls to $bind$
for each of the event types that is
relevant to the widget.

For both the $bind$ and $handler$
functions, the optional $args$
parameter may be used to specify
the arguments that will be passed
to the handler or action
when it is invoked.
For the $button$ widget, for example,
the default $install$ function supplies
the text of the button as an  additional
argument for its handler.
}


\c{
."@ Compound widgets
In addition, the widget class offers four
functions that may be used
when defining compound or mega widgets.
The function $redirect(w)$ must by used
to delegate the invocation
of the $eval$, $configure$, $bind$ and $handler$ functions
to the widget $w$.
The function $self()$ gives access to the widget
to which the commands are redirected.
After invoking $redirect$,
the function $thepath$ will deliver
the path that is determined by $self()->path()$.
In contrast, the function $path$ will still deliver
the path name of the outer widget.
Calling $redirect$ when creating the compound widget
class suffices for most situations.
However, when the default events must be changed or the 
declaration 
of a handler must take effect for several component widgets,
the virtual function $install$ must be redefined
to handle the delegation explicitly.
How $redirect$ and $install$ actually work will
become clear in the examples.
}

\c{
@ Handlers
A note on terminology is in place here.
The reader may be a little astonished by
the fact that we have both a {\em handler}
class and a {\em handler} function,
which is more properly written
as $widget::handler$.
The situation may even seem more confusing
when realizing that the {\em widget}
class itself is a descendant of the
{\em handler} class.
Schematically, we have the situation as depicted
in \sliref{h-disc}.
}

\slide{h-disc}{Handler classes and functions}{
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
[P
 class widget : public handler {
 public:
 ...
 void handler(class handler* h) { ... }
 ...
 };
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}

\c{
Note that there is no ambiguity here.
A {\em handler} object is an object
that may be invoked in response to a Tcl command
or an event.
The {\em handler} function declares
a {\em handler} object to be responsible
for dealing with the events
that are of interest to the widget.
Note that since a widget is a $handler$ instance,
it may declare itself as a handler for the incoming
events.
}



<document>
\label{buttons}

:@ Buttons
As the first component of the drawing tool,
we will look at the {\em toolbox}.
The {\em toolbox} is a collection of buttons
packed in a frame. See \sliref{draw-toolbox}.

<slide id=draw-toolbox cap="The toolbox">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/toolbox.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


Each button is an instance of
the class {\em toolbutton}.
When a toolbutton is created (a),
the actual button is given the name of the button
as its path.
Next, (b) the button is given the name as its text,
the ancestor widget $w$ is declared to be the 
handler for the button and the button is packed.
The function {\em text} is a member function
of the class {\em button}, whereas both
{\em handler} and {\em pack} are common widget functions.
Note that the parameter {\em name} is used as 
a path name, as the text to display, and as an argument
for the handler, that will be passed
as a parameter when invoking the handler object.


The $toolbox$ class inherits from the $frame$ widget class,
and creates a frame widget with a path relative to the widget parameter
provided by the constructor (c).
The constructor further creates the four toolbuttons.


The $toolbox$ is both the superordinate widget
and handler for each $toolbutton$.
When the  $operator()$ function of the $toolbox$ is invoked
in response to pressing a button,
the call is delegated to the $mode$ function
of the $tablet$ (d).
The argument given to $mode$ corresponds to
the name of the button pressed.

The definition of the $toolbutton$ and $toolbox$
illustrates that a widget need not necessarily be its
own handler.
The decision, whether to define a subclass which is made
its own handler or
to install an external handler
depends upon what is considered the most convenient
way in which to access the resources needed.
As a guideline, exploit the regularity of the application.
</document>

\label{menus}


\cleartrue
<slide id=draw-menubar cap="The {\em menu\_bar}">
\ifsli{}{\vspace{0.1cm}}
@ds draw/menu_bar.h
\ifsli{\ln\vspace{-1cm}}{\vspace{-0.2cm}}
</slide>


\c{
:@ Menus
The second component of our drawing tool is the {\em menubar}.
The class {\em menu\_bar}, depicted in \sliref{draw-menubar}
is derived from the {\em hush} widget $menubar$.
Its constructor requires an ancestor widget,
a {\em tablet} and a {\em toolbox}.
The tablet is passed as a parameter to the {\em file\_menu},
and the $toolbox$ to the {\em edit\_menu}.
In addition, a {\em help\_button} is created, which provides
online help in a hypertext format when pressed.
The help facility will be discussed in section \ref{hyper}.
}

\cleartrue
<slide id=draw-file cap="The {\em file\_menu}">
\ifsli{}{\vspace{0.1cm}}
@ds draw/file_menu.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


\c{
A menubar consists of menubuttons to which actual menus are attached.
Each menu consists of a number of entries,
which may possibly lead to cascaded menus.

The {\em file\_menu} class, depicted in \sliref{draw-file},
defines a menu, but is derived from {\em menubutton}
in order to attach the menu to its {\em menubar}
ancestor (a).
Its constructor defines the appearance of
the button and creates a {\em file\_handler} (b).
It then defines the actual menu (c).
The menu must explicitly be attached to
the menubutton by invoking the {\em menubar}
member function {\em menu}.
For creating the menu, the keyword {\em class} is needed
to disambiguate between the creation of an instance of the class
menu and the call of the $menubar::menu$ function.


Before defining the various entries of the menu,
the {\em file\_menu} instance is declared
as the handler for the menu entries (d).
However, except for the entry {\em Quit}, which is handled
by calling the $kit::quit$ function (e),
the calls are delegated to the previously created {\em file\_handler}.
}

."so draw-edit

\c{
The second button of the {\em menu\_bar} is defined by the
{\em edit\_menu}.
The {\em edit\_menu} requires a $toolbox$ and creates
a menubutton.
It configures the button and
defines a menu containing two entries,
one of which is a cascaded menu.
Both the main menu and the cascaded menu
are given the $toolbox$ as a handler.
This makes sense only because 
for our simple application,
the functionality offered by
the $toolbox$ and {\it edit\_menu} coincide.
}


<document>

\vfig{1}
<slide id=draw-tablet cap="The {\em tablet}">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/tablet.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


:@ Defining actions 
The most important component of our {\em drawtool}
application is defined by the {\em tablet} widget class
depicted in \sliref{draw-tablet}.


The various modes supported by the drawing tool are
enumerated in a separate class {\em drawmode}.
The {\em tablet} class itself inherits from the $canvas$
widget class.
This has the advantage that it offers the full functionality
of a canvas.
In addition to the constructor and $operator()$ function,
which delegates the incoming event to the appropriate
handler
according to the {\em \_mode} variable,
it offers a function $mode$, which sets the mode of the
canvas as indicated by its string argument,
and a function {\em init} that determines the creation
and geometrical layout of the component widgets.
As instance variables, it contains an integer {\em \_mode}
variable and an array of handlers that contains
the handlers corresponding to the modes supported.
See section \ref{canvas} for an example of a typical
canvas handler.


@ Dispatching
Although the $tablet$ must act as a canvas,
the actual $tablet$ widget is nothing but a $frame$
that contains a canvas widget as one of its components.
See \sliref{tablet-op}.

<slide id=tablet-op cap="Installing the handlers">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/draw-op.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


This is reflected in the invocation of the
canvas constructor (a).
By convention, when the options parameter is $0$
instead of the empty string, no actual widget
is created but only an abstract widget,
as happens when calling the $widget$ class constructor.
Instead of creating a canvas right away,
the $tablet$ constructor creates a top frame,
initializes the actual component widgets,
and redirects the $eval$, $configure$, $bind$
and $handler$ invocations to the subordinate $canvas$
widget (b).
It then
declares itself to be its own handler,
which results in declaring itself to be the handler for the
canvas component (c).
Note that reversing the order of calling $redirect$
and $handler$ would be disastrous.
After that it creates the handlers for the various modes
and sets the initial mode to $move$.

The $operator()$ function takes care of dispatching calls
to the appropriate handler.
The $dispatch$ function must be called to pass
the $tk$, $argc$ and $argv$ parameters.
</document>

."so scroll
\label{new}

<slide id=draw-new cap="The {\em drawtool} widget command">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/drawtool.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


\c{
:@ Creating new widgets
Having taken care of the basic components
of the drawing tool, that is the {\em toolbox},
{\em menu\_bar} and {\em tablet} widgets, all that remains
to be done is to define
a suitable {\em file\_handler},
appropriate handlers for the
various drawing modes
and a {\em help\_handler}.
We will skip the {\em file\_handler}, but look at the
latter two issues in
sections \ref{canvas} and
\ref{hyper}, respectively.
However, before that it will be shown how we may grant
the {\em drawtool} the status of a veritable
Tk widget, by defining a
$drawtool$ handler class
and a corresponding $drawtool$ widget command.
See \sliref{draw-new}.
}



\c{
Defining a widget command involves three steps: (I) 
the declaration of the binding between
a command and a handler,
(II) the definition of the $operator()$
action function, which actually defines a mini-interpreter,
and (III) the definition of the actual
creation of the widget and its declaration
as a Tcl/Tk command.

Step (I) is straightforward.
We need to define an empty handler, which will
be associated with the $drawtool$ command
when starting the application. See \sliref{new-command} (a).
The functionality offered by the
interpreter defined by the $operator()$
function in (II) is kept quite simple,
but may easily be extended.
When the first argument of the call is $drawtool$,
a new $drawtool$ widget is created as specified in (III),
except when the second argument is $self$.
In that case, 
the virtual
path of the widget is returned, which is actually
the path of the $tablet$'s canvas.
It is the responsibility of the writer of the script
that the $self$ command is not addressed to the
empty handler.
If neither of these cases apply,
the function $widget::eval$ is invoked
for $self()$, with the remaining arguments flattened
to a string.
This allows for using the drawtool almost as an ordinary
canvas. See the example hypertext script shown in
section \ref{hyper}.
}

\c{
The creation of the actual widget and
declaration of the corresponding Tcl command,
according to the Tk convention,
is somewhat more involved (III).


Recall that each Tk widget is identified by
its path, which simultaneously
defines a command that may be used
to configure the widget or,
as for a canvas,
to draw figures on the screen.
Hence, the function $create$ must 
create a new widget and declare the widget
to be the handler of the command
corresponding to its path name.
}


\slide{new-command}{The drawtool application}{
\ifsli{}{\vspace{0.1cm}}
\ifsli{
.ds draw/sl-app.s
}{
.ds draw/applic.s
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
The $application$ class depicted in \sliref{new-command}
will by now look familiar,
except for the function $prelude$.
In the body of the $prelude$ function,
the Tcl command $drawtool$ is declared,
with an instance of {\em drawtool} as its handler (a).

In this way, the $drawtool$ widget
is made available as a command when
the program is used as an interpreter.
However, in the $main$ function
this declaration is overridden (b).
Instead, the actual $drawtool$ widget
is made the handler of the command,
to allow for a script to
address the $drawtool$ by calling
{\em drawtool self}.
}

<document>

@ Delegation
You may by now have lost track of
how delegation within a compound
widget takes place.
Perhaps a brief look at the implementation
will clarify this.

Each $eval$, $configure$ or $bind$
function call for a widget
results in a command addressed to
the path of the widget. 
By redirecting the command to a different path,
the instructions may be delegated to
the appropriate (component) widget.
Delegation occurs, in other words,
by directing the commands to the widget's virtual
path, which is obtained by the protected
function $thepath()$.
In contrast, the function $path()$ delivers
the path of the widget's outer component.
Indirection takes place by invoking the function
$self()$, which relies on an instance variable
{\em \_self} that may be set by the $redirect$
function.
}

@so f_self

The implementation of $thepath()$ and $self()$ is 
simply:
[P
char* thepath() { return self()->path(); }
widget* self() { return _self?_self->self():this; }
P]
Hence, resolving a compound widget's
primary inner component relies on simple pointer chasing,
which may be applied recursively to an
arbitrary depth at acceptable costs.
</document>


."so dialogs
<document>
@+ Graphics and hypertext
\label{Graphics}

\c{
The Tk toolkit offers powerful facilities
for graphics and (hyper)text
(see Ousterhout, 1993).\index{Ousterhout (1993)}
In this section we will discuss only the $canvas$
widget offered by Tk.
Instead of looking at the $text$ widget
provided by Tk, we will (briefly)
look at the $hypertext$ widget,
which presents an alternative approach to
defining hyperstructures.
}


\label{hush-item}

\slide{class-item}{The {\em item} class}{
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
\ifsli{
[P
interface item { /// \c{\fbox{item}}
operator int();  ///// \ifsli{}{returns} item index

void configure(char* cmd); /// \c{uses canvas}

void tag(char* s); ///// sets tag \c{for item}
char* tags(); ///// delivers tags \c{set for the item}
void move(int x, int y);

bind(char *b, handler* h, char* = "" );
bind(char *b, action& ac, char* = "" );

handler(class handler* h, char* = "" );
handler(action& ac, char* = "" );

protected:
virtual install(action&,char* =""); 
};
P]
}{
[P
interface item { /// \fbox{item}
operator int();  ///// \ifsli{}{returns} item index

void configure(char* cmd); ///// \c{uses canvas}

void tag(char* s); ///// sets tag \c{for item}
char* tags(); ///// delivers tags \c{set for the item}
void move(int x, int y);

bind(char *b, handler* h, char* args = "" );
bind(char *b, action& ac, char* args = "" );

handler(class handler* h, char* args = "" );
handler(action& ac, char* args = "" );

protected:
virtual install(action&,char* args=""); 
};
P]
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
:@ The $item$ class
The canvas widget allows the programmer to
create a number of built-in graphic items.
Items are given a numerical index when created and,
in addition, they may be given a (string) tag.
Tags allow items to be manipulated in a group-wise fashion.
To deal with items in a C++ context, the $hush$ library
contains a class $item$ of which the functionality
is shown in \sliref{class-item}.
}


\c{
Instances of $item$ may not be created directly by
the user, but instead are created by the canvas
widget.
For an item, its index may be obtained by casting
the item to $int$.
If the index does not identify an existing item,
it will be zero.
Existing items may be moved, in a relative way,
by the $move$ function.
}

\c{
In a similar way as for widgets,
items may be associated with events,
either explicitly by using
$item::bind$, or
implicitly by using $item::handler$.
The default bindings for $items$ are
identical to the default bindings
for the canvas widget, but these may be overridden
by descendant classes.

Similar to the $widget$ class, the $item$ class
is derived from the $handler$ class.
This allows the user to define possibly compound
shapes defining their own handler.
}

<document>
\label{canvas}

:@ The $canvas$ widget
The Tk canvas widget offers a powerful means for
creating structured graphics.
The $hush$ class $canvas$ provides merely
a simplified interface to
the corresponding Tk widget.
}

<slide id=draw-move cap="The {\em move\_handler} class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds draw/move_handler.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>



As an example of the use of a canvas, consider the
definition of the <em>move_handler</em>
class in \sliref{draw-move}.
The <em>move_handler</em> class is derived from the class
$handler$.
It makes use of the $dispatch$ and $operator()$ function defined for 
$handler$,
but redefines the (virtual) functions
$press$, $motion$ and $release$.


When creating an instance of <em>move_handler</em>,
a pointer to the canvas must be given to the constructor.
In addition, the class has data members to record
position coordinates
and whether a particular item is being moved.
Actually, moving an item occurs by pressing the (left)
mouse button on an item and dragging the item along.
When the mouse button is released, moving stops.
To identify the item, the function $overlapping$
is used.
The movement is determined by the distance
between the last recorded position and the
current position of the cursor.

\fslide{hyper}{Hypertext help}{
."@ Help
\ln
\ifsli{\hspace*{0.8cm}}{\hspace*{-1.0cm}}
\epsfbox{help.eps}
\ln \ifsli{\vspace{-1cm}}{\vspace{-0.5cm}}
}


In an analogous manner,
a <em>box_handler</em> may be defined.
The <em>box_handler</em> sets dragging
to true when the button is pressed
and creates a rectangle of zero width and height.
Each time the function $motion$ is called,
the item created in the previous round is deleted
and a new rectangle is created by calling
[P
 c->rectangle(x,y,e.x(),e.y());
P]
where $c$ is a pointer to the canvas
and $x$ and $y$ the button pointer coordinates
stored when dragging began.
For circles and lines, it suffices to replace the call
to rectangle with  a call to the appropriate
figure creation function.
</document>


."so hyper-eps

\label{hyper}

."so draw-help

\c{
:@ The $hypertext$ widget
Both the Tk canvas and text widget allow the binding of
actions to particular items
and hence defining dynamically what we may call
{\em hyperstructures}.

A different, in a way more static, approach is offered
by the $hypertext$ widget originally developed by
George Howlett. %%{P george.howlett@att.com P}.
}

\cleartrue
\slide{help-text}{ A hypertext help file }{
\ifsli{}{\vspace{0.1cm}}
\ifsli{
[P
Rubber banding: press the left mouse button
and release when the rectangle is of appropriate
size
    \%\%
drawtool \$this.draw 
\$this append \$this.draw 
\$this.draw create rectangle 20 20 80 80
\$this.draw create rectangle 10 30 70 90
\$this.draw create oval 40 40 90 90
\$this append \$this.draw 
\%\%
For additional information click on the \%\% 
button \$this.goto -text instruction -command end-of-text
\$this append \$this.goto
\%\%
button.  Press \%\%
button \$this.quit -command { destroy . } -text quit 
\$this append quit
\%\% to remove the window.
P]
}{
[P
Rubber banding: press the left mouse button
and release when the rectangle is of appropriate
size
    \%\%
drawtool \$this.draw 
\$this append \$this.draw 
\$this.draw create rectangle 20 20 80 80
\$this.draw create rectangle 10 30 70 90
\$this.draw create oval 40 40 90 90
\$this append \$this.draw 
\%\%
For additional information click on the \%\% 
button \$this.goto -text instruction -command end-of-text
\$this append \$this.goto
\%\%
button.  Press \%\%
button \$this.quit -command { destroy . } -text quit -bg pink
\$this append quit
\%\% to remove the window.
P]
}
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
}


\c{
The $hypertext$ widget may be used to display text
files containing embedded Tcl code.
The Tcl code must be placed between
escapes, that take the form of {\tt \%\%}
for both the start and end of the code.
A screen shot of a fragment of the online help
for $drawtool$ is given in \sliref{hyper}.
Notice that the online help provides a
replica of the $drawtool$ application, surrounded
by text.
When looking at (again a fragment of) the hypertext
file specifying the contents of the online help,
given in \sliref{help-text},
you see that the $drawtool$ command defined
in section \ref{new} 
is employed to create the embedded widget.
}


\c{
When specifying the hypertext file,
widgets may be
given a pathname relative to the pathname of the
hypertext widget by using the variable {\em this}.
In addition, the hypertext widget offers the variables
{\em thisline} and {\em thisfile} to identify the
current line number and current file name.
}

\c{
Any of the widgets and commands offered by Tcl/Tk or supported
by $hush$ may be included in a hypertext file,
including the ones defined by the program
itself.
}

</document>

</document>
