<document>

@+ Structure versus presentation -- <em>hypermedia reference models</em>
\label{hypermedia-model}


Hypermedia technology supports the organization of a variety of material into an associative structure.
In addition, hierarchical structuring facilities may be supported.

The basic notions underlying the structuring
facilities of hypertext have been expressed
in the Dexter hypertext model.
See [_Halasz94].

<slide id=12-Dexter cap="The Dexter hypertext reference model">
@ Hypertext model <em> -- documents</em>
\zline{\fbox{Dexter}}
<I
- components, links and anchors
I>
@ Component
<I
- content 
{\em -- text, graphics, video, program}
- attributes
{\em -- semantic description}
- anchors
{\em -- links to other documents}
- presentation
{\em -- display characteristics}
I>
:@ Compound
<I
- children
{\em -- subcomponents}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The Dexter model explains the structure
of hypertext documents in terms
of {\em components}, {\em links} and
{\em anchors}.
The notion of anchors is introduced
to explain how to attach a link
to either a source or destination
component.
An {\em anchor} is the indication
of a particular spot in the document,
or rather a component thereof, usually
identified by some coordinate.

The Dexter model distinguishes
between three layers in a hypertext system,
namely the {\em document} layer
(defining the content and structure 
of documents),
a {\em storage} layer
(handling the storage and retrieval of
components and links)
and a {\em presentation} layer
(handling the display of documents
and the interaction with the user).

A {\em component},
which is a part of a document is
characterized by the following features:
{\em content}
(which may be text, graphics, audio,
video or even a program),
{\em attributes}
(which give a semantic description
that may be used for retrieval or
selective display),
{\em anchors}
(which identify the places to which
a link is attached), and
{\em presentation} characteristics
(which determine the display of the
component).
In addition, for compound components,
a feature {\em children} may be defined, for
storing the list of subcomponents.


@ Multimedia
The original Dexter hypertext model is
strongly oriented towards {\em text},
despite the provision for multimedia
content.
Multimedia, in particular audio and video,
are intrinsically time-based
and require temporal primitives to synchronize
the presentation of material from different 
sources.

In the CMIF multimedia model
described in [_Hardman94], {\em channels}
(which are abstract output devices) are
introduced to allow for the specification
of abstract timing constraints
by means of synchronization arcs between channels.

<slide id=12-multimedia cap="The CMIF multimedia model">
@ Multimedia model <em> -- composition</em>
\zline{\fbox{CMIF}}
<I
- data block {\em -- atomic component}
- channel {\em -- abstract output device}
- synchronization arc {\em -- specifying timing constraints}
- event {\em -- actual presentation}
I>
:@ Views 
{\em -- authoring}
<I
- structure {\em -- sequential and parallel composition}
- channels {\em -- presentation}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

The notion of {\em channels} provides the abstraction
needed to separate the contents of a presentation
from the actual display characteristics.
For example, text may be output through
a text channel while, simultaneously,
video may be output through a video channel.
The screen layout and allocation of these
channels may be determined independently.

The actual presentation is determined by 
{\em events},
that may either arise as the result of a user
action or as the result of the activation of
a synchronization arc.
For example, a synchronization constraint
may specify that an audio fragment containing
speech must be started 10 seconds after the beginning
of a video sequence.
Then, after 10 seconds, the video channel will issue
an event that causes the audio channel to start
presenting its contents.

The CMIF model has been developed to allow for
portable multimedia documents.
In particular, the notion of channels allows
for a platform-independent characterization
of presentation characteristics and timing constraints.
An important characteristic of the model,
from an authoring perspective, is that it supports
a compositional approach to authoring,
since it allows us to compose a channel
(specifying a sequential composition of components)
with arbitrary many other channels, in parallel.

In [_Hardman94], an extension of the CMIF multimedia
model is developed to incorporate the associative
structuring facilities defined by the Dexter
hypertext model.
}

\clearfalse
<slide id=12-model cap=" A hypermedia reference model">
@ Hypermedia model <em> -- components</em>
<I
- contents {\em -- data block}
- attributes {\em -- semantic information}
- anchors {\em -- $(id, value)$}
- presentation {\em -- channel, duration, ...}
I>
:@ Compound
<I
- children {\em -- $($ component, start-time $)$}
- synchronization {\em -- $($ source, \ifsli{dest}{destination} $)$}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

In the combined model, a single component
consists of {\em contents}
containing the actual data blocks of the component,
{\em attributes} 
that specify semantic information,
a list of {\em anchors}
(each specifying a symbolic name and a value,
which in the case of an audio or video
fragment is its time measured from the start),
and {\em presentation} characteristics,
which include
the specification of a channel and the
duration of the component.

As in the Dexter model, {\em compound} components
may have {\em children} attributes,
specifying for each child a component and its
start-time,
and a number of {\em synchronization arcs},
each
specifying
a {\em source} (component and anchor)
and {\em destination} (component and anchor).
Synchronization arcs may cross channel boundaries.

."@ Discussion
The reader is encouraged to specify a
more detailed object model, based on the outline
given above.
Evidently, the incorporation of a variety
of content types and display channels is a serious
challenge.
In particular, the notion of time-based active objects
will probably be difficult to handle.
For an abstract characterization of active time-based
(media) object, the reader is referred to [_Nierstrasz].

@so links

</document>
