@+ The $canvas$ widget

\c{
\label{canvas}
The Tk canvas widget offers powerful means for
doing structured graphics.
The $hush$ class $canvas$ provides merely
a simplified interface to
the corresponding Tk widget.
}

\c{
Apart from the two standard constructors,
it offers the functions $tag$, $tags$ and $move$
that merely repeat the functions offered by the $item$ class,
except that $move$ may also be given a tag to identify
the items to be moved.
Currently, the graphic items $bitmap$, $line$,
$oval$, $polygon$ and $rectangle$ may created
and, in addition, $text$ items and $window$ items consisting
of a widget.
The function $overlapping$ may be used to retrieve
the item overlapping a particular position.
}

\sli{
[P
interface canvas : widget { /// \fbox{canvas}
canvas(char *p, char* options="");
canvas(widget* w, char *p, char* options="");

void tag(int id, char* tag);
char* tags(int id);

void move(int id, int x, int y);
void move(char* id, int x, int y);

item bitmap(int x1, int y1, char* bitmap, char* options="");
item line(int x1, int y1, int x2, int y2, char* options ="");
item line(char* linespec, char* options ="");
item oval(int x1, int y1, int x2, int y2, char* options ="");
item polygon(char* linespec, char* options ="");
item rectangle(int x1, int y1, int x2, int y2, char* options ="");
item text(int x1, int y1, char* txt, char* options=""); 
item window(int x1, int y1, char* win, char* options="");

item current();
item overlapping(int x, int y);

void itemconfigure(int it, char* options);
void itemconfigure(char* tag, char* options);
};
P]
}



\c{
@ Example
As an example of the use of a canvas, consider the
handler class defined below.
The $canvashandler$ class is derived from the class
$handler$ introduced in section \ref{handler}.
It makes use of the $dispatch$ function defined for 
handler,
but redefines the (virtual) functions
$press$, $motion$ and $release$.
}

\sli{
[P
class canvashandler : public handler {
public:

canvashandler( canvas* cv ) { c = cv; moving = 0; }

void press( event& e) {
	x = e.x(); y = e.y();
	id = c->overlapping(x, y);
	if (id) moving = 1;
	}

void motion( event& e ) {
	if (moving) {
		id.move( e.x() - x, e.y() - y );
		x = e.x(); y = e.y();
		}
	}

void release( event& ) { moving = 0; }

protected:
canvas* c;
int x,y;
int moving;
item id;
};
P]
}

\c{
When creating an instance of $canvashandler$,
a pointer to the canvas must be given to the constructor.
In addition, the class has data members to record
position coordinates
and whether a particular item is being moved.
Actually moving an item occurs by pressing the (left)
mouse button on an item and dragging the item along.
When the mouse button is released, moving stops.
To identify the item, the function $overlapping$
is used.
The movement is determined by the distance
between the last recorded position and the
current position of the cursor.
}


\sli{
[P
void program( kit* tk, int, char** ) {

    canvas* c = new canvas(".canvas");
    c->handler(new canvas_handler(c));

    c->oval(120,60,180,180,"-fill blue");
    c->bitmap(30,50,"@bishop.bm");

    tk->pack(c)->append(".quit");
}
P]
}

\c{
To install an instance of $canvashandler$
as a handler for the canvas, the function
$widget::handler$ is used.
Note that the $canvashandler$ is given a reference
to the canvas.
}
