:@+ Dialogs
.":@+ The $entry$ widget

\c{
Interactive applications may require the user to type some input
after reading a message or to select an item from
a list of alternatives.
Among the widgets that may be involved in such
an interaction are the $message$ widget, the $entry$
widget and the $listbox$ widget.
An example of a (compound) widget combining
these widgets is the $filechooser$ widget, that
is discussed in section \ref{new}.

Another example of a compound widget is given
in the function $dialog$ below.
An essential component of (compound) dialog widgets
is usually an $entry$ widget, that may be used
to obtain (typed) input from the user.
}

\sli{
[P
interface entry : widget { /// \fbox{entry}

entry(char* p, char* options = "");
entry(widget* w,char* p, char* options = "");

void insert(char* s);  /// insert text
char* get(); /// to retrieve (edited) text
};
P]
}

\c{
The $entry$ class shown above offers, in addition
to the two (standard) constructors,
the functions $insert$ and $get$, that are used respectively to set
and get the text that will appear in the entry,
The text appearing in the entry may be edited by the user.
}

\c{
@ Example
below an example is given of a function to
create a user-defined compound dialog widget.
The compound widget consists of
a $toplevel$ widget (which makes that the 
dialog widget is directly mapped to the screen
as a toplevel window),
a $message$ widget (to display a message),
an $entry$ widget (to allow the user to enter text)
and two $button$ widgets (one to acknowledge the
answer and the other to cancel the dialog).

The text typed in the $entry$ widget is retrieved
by the function $browse$ defined below.
}

\sli{
@ Browse
{\em -- entry}
[P
int browse( client* data, kit*,  int , char** ) {
	 entry* e = (entry*) data;
	 cout << e->get() << endl;
	 return OK;
	 }
P]
}

\c{
The actual $dialog$ function is defined as
a handler function that may be attached to for
example a button.
The function $dialog$ first creates a $toplevel$ widget,
and subsequently a $message$ widget and an $entry$ widget.
Both the message widget and the entry widget
are made subordinate to the toplevel widget.
Next a $frame$ widget is created, to hold the
two $button$ widgets, $ok$ and $cancel$.
The $ok$ button is given the $browse$ function as
a handler, with the entry widget as client data.
The handler for the $cancel$ button is defined
directly by a Tcl command which results in
destroying the toplevel window.
Both buttons must be explicitly packed in the frame widget.
After that, the frame widget is packed together with
the message widget and entry widget in the toplevel widget.
Before leaving the function, the focus is set to the entry widget,
which effects the cursor to start blinking
in the entry widget.
}

\sli{
@ Dialog  
{\em -- to be attached to a button}
[P
int dialog( client*, kit*,  int , char** ) {

	 toplevel* t = new toplevel(".toplevel");
	 
	 message* m = new message(t,".message");
	 m->text("Type your answer");
         
	 entry* e = new entry(t,".entry","-relief sunken");
	   
         frame* f = new frame(t,".frame");

	 button* ok = new button(f,".ok");
	 ok->text("OK");
	 ok->handler(browse,e);

	 button* cancel = new button(f,".cancel");
	 cancel->text("cancel");
	 cancel->handler("destroy .toplevel");
	  
	 f->pack(cancel,"{left fillx}");
	 f->pack(ok,"{padx 5 right fillx filly}");

         t->pack(m)->pack(e)->pack(f);

	 e->focus();  /// to attract the focus

	 return OK;
	 }
P]
}

\c{
In the example, occasionally the option parameters
are used in order to make the appearance of the
widget nicer.
}
