@+ The interpreter {\em kit}

\c{
$Hush$ is meant to provide a parsimonious C++
interface to Tcl/Tk.
Nevertheless, as with many a toolkit, some
kind of API shock seems to be unavoidable.
This is especially true for the class $kit$
defining the interface.
The functionality of $kit$ can only be completely
understood after reading this article.
However, since an instance of $kit$ is used in almost
any other object (class), it is presented here first.
The reader will undoubtly gradually learn the
functionality of $kit$ by studying the examples.
}


\c{
The $kit$ class is derived from the class $hcl$  which
provides access to the lower level details of (an extended version of)
Tcl. \nop{Consult the reference manual.}
}

\c{
The first group of functions ($eval$, $result$,
$evaluate$ and $source$ may be used to execute
commands in Tcl scripting language directly.
A Tcl command is simply a string conforming to
certain syntactic requirements.
The function $eval$ evaluates a Tcl command.
The function $result()$ may be used to fetch
the result of the last Tcl command.
In contrast, the function $result(char*)$ may be used
to set the result of a Tcl command, when this
command is defined in C++ (as may be done with $kit::action$).
The function $evaluate$ provides a shorthand for
combining $eval$ and $result()$.
The function $source$ may be used to read in
a file containing a Tcl script.
The function $after$ may be used to have a
Tcl command evaluated after $n$ milliseconds.
(It makes use of X timer callbacks).
The function $send$ may be used to evaluate
a command in a different application.
An application may be given a name when
creating the $session$ object.
The function $selection$ delivers the current X selection.
The function $update$ may be used to process any
pending events.
These functions are only for experienced users.
See section \ref{tcl}.
}

\c{
The function $event$ delivers the latest event.
It may only be used in a command that is bound
to some particular event.
See section \ref{events}.
When other event occur before accessing the event object,
the information it contains may be obsolete.
}

\sli{
@ The interpreter $kit$
\zline{\fbox{kit}}
[P
interface kit : hcl { 

int eval(char* cmd); /// to evaluate script command
char* result();     /// to fetch the result of $eval$
void result(char* s);   /// to set the result of $eval$
char* evaluate(char* cmd) /// combines $eval$ and $result$
int source(char* f); /// to load a script from file

void after(int msecs, char* cmd);
char* send(char* it, char* cmd);

char* selection(char* options="");
void update(char* options="");

class event event(); /// returns the last event

widget* root(); /// returns toplevel (root) widget
widget* pack(widget* w, char* options = "{top fillx filly}");
widget* pack(char* wp, char* options = "{top fillx filly}";

action& action(char* name);
action& action(char* name, handler* h);
action& action(char* name, command f, client* data = 0 );
action& action(char* name, tclcommand f, clientdata data = 0 );

void trace(int level = 1); 
void notrace(); 
void quit() /// to terminate the session
};
P]
}


\c{
The function $root$ gives access to the toplevel root
widget associated with that particular
instance of the $kit$.
The function $pack$ may be used to append
widgets to the root widget, in order to map them to the screen.
Widgets may be identified either by a pointer to
a $widget$ object or by their {\em path name},
which is a string.
See section \ref{widget}.

The function $action$ may be used to associate a Tcl command to
a command function or handler written in C++.
See section \ref{action} for details.
}

\nop{
A kit object, which may be only created by session(),
provides an interface to the tcl interpreter
and the global properties of the system.
The function eval() may be used to execute
commands in tcl and to retrieve the result.
The function event delivers the latest event.
When other events occur before accessing a particular
event object, the information gotten will be obsolete.
}

\c{
The functions trace() and notrace() may be used to turn
on, respectively off, tracing.
The level indicates in what detail information will
be given. Trace level zero is equivalent to notrace().
Finally, the function $quit$ may be used to terminate 
the session.
}
