.":@+ The $menu$ widget
:@+ Menus

\c{
Another, frequently occurring, widget is the $menu$ widget.
A menu consists of a number of button-like
entries, each associated with an action.
A menu entry may also consist of another menu,
that pops up whenever the entry is selected.
}

\sli{
[P
interface menu : widget { /// \fbox{menu}

menu(char* p, char* options = "");
menu(widget* w, char* p, char* options = "");

menu* add(char* s, char* options = "");

menu* entry(char* s, char* args ="", char* options="");
menu* entry(char* s, action& ac, char* args ="", char* opts="");

menu* cascade(char* s, char* m, char* options = "");
menu* cascade(char* s, menu* m, char* options = "");

char* entryconfigure(int i, char* options);

int index(char *s);
int active();  /// returns active index

void del(int i);  /// delete entry with index i
void del(char* s); /// delete entry 

char* invoke(int i);  /// invoke entry with index i
char* invoke(char *s ); /// invoke

void post(int x = 500, int y = 500); 
void unpost();
protected:
install(action&, char* args);
};
P]
}

\c{
The $add$ function is included to allow arbitrary entries (as defined by Tk)
to be added.
We restrict ourselves to simple command and cascade entries.

The $entry$ function (that is used for adding simple command entries)
may explicitly be given an $action$ to
be associated with the entry.
Alternatively, if no action is specified,
the default handler action installed
by invoking $widget::handler$ will be used.
The string used as a label for the entries (the first parameter of $entry$)
will be given as a parameter to the action invoked when selecting the entry.
The string given in the $args$ parameter
will be added to the actual parameters for the action invoked.

The $cascade$ function may either be given a $menu$ or a string,
containing the pathname of the menu.
In any case the cascaded menu must be a descendent of the original menu.

The function $index$ returns the integer index
associated with the string describing the entry.
The function $active$ may be used to inquire
which entry has been selected.
See the example below.

Entries may be deleted using the function $del$
and invoked by using $invoke$.
For both functions, the entry may be indicated
by its numerical index or a string.
Menus are toplevel widgets, they are mapped
to the screen either by invoking the function $post$,
or by embedding the menu in a $menubutton$.
}



\c{
@ Example
The function $select$ shown below merely
prints the (index of) the currently active entry
for the menu.
This may be used in a $switch$ statement to
determine the appropriate action.
}

\sli{
[P
int select( client* data, kit*,  int argc, char** argv) {
	 menu* m = (menu*) data;
	 int n = m->active();
	 cout << "entry " << n <<  endl;
	 return OK;
	 }
P]
}

\c{
Note that the $client$ parameter is assumed to be
a menu.
The example below shows how $menus$ embedded
in a $menubutton$ may be organized in a $menubar$.
}

\sli{
[P
void program(kit* tk, int, char** ) {

    widget* bar = new menubar(".bar");

    menubutton* b1 = new menubutton(bar,".b1");
    b1->text("one");

    menubutton* b2 = new menubutton(bar,".b2");
    b2->text("two");

    menu* m1 = new menu(b1,".m");  b1->menu(m1);
    menu* m2 = new menu(b2,".m"); b2->menu(m2);

    m1->handler(select,m1);
    m2->handler(select,m2);

    m1->add("zero"); m1->add("one");
    m2->add("second"); m2->add("third");

    bar->append(b1)->append(b2);

    tk->append(bar); tk->append(".quit");
}
P]
}

\c{
The pathnames of the menubuttons are made dependent
on the pathname of the menubar,
and, analogously, the pathname of each menu
depends on the pathname of the menubutton
they are associated with.
Before adding the entries, the function $select$ is
installed as a handler.
Entries occur in the menu in the order that
they are added.
Finally, the buttons are packed in the menubar and
the bar and quit button are packet in the root
widget.
}
