@* Program structure 

\c{
The $hush$ library is intended to provide a convenient way to program
Tcl/Tk based applications in C++.
We will start by looking at the structure of programs
employing $hush$.
In an ordinary C++ program the
function $main$ is used to start the computation.
Control is effected by creating objects and 
calling the appropriate functions.
When programming in a window based application, at a certain
moment control is delegated to the window environment.
Consequently, there needs to be some kind of main loop
which waits for incoming events, in response to which
the control may be delegated to an appropriate
component of the program.

Since programming an event dispatch loop by hand
is very cumbersome, the main (event dispatch) loop
is usually hidden, and instead the programmer
is required to declare callback functions
that will be activated in response to a particular event.
}

\sli{
[P
#include <hush.h>
void program(kit* tk, int argc, char** argv) {
message* m = new message(".m");
m->text("hello world");
tk->pack(m);
}

int main(int argc, char** argv) {
session* s = new session(argc,argv,"root");
return s->run(program);
}
P]
}

\c{
To hide the details of activating the main loop
and the dispatching of events, $hush$ provides
a class $session$ of which an instance must be created
to start the computation.

Above, an example is given of a simple {\em hello world} program
employing $hush$.
It resembles the example given previously, except that a message
is used instead of a button widget.
In $main$ an instance of $session$ is created
which is then told to run the function $program$.
As a result, $program$ is called with, as
parameters, a pointer to an instance of $kit$
(that is used to communicate with the underlying toolkit)
and the original parameters to $main$.
The function $program$ merely creates a message
saying {\em hello world} and displays this
in a window on the screen by telling the toolkit
to append it to the root window.
}


.so kit
.so session

