:@+ Events

\c{
\label{events}
Events always belong to a particular widget.
To which widget events are actually directed
depends on
whether the programmer has defined a binding for the
event type.
When such a binding exists for a widget
and the (toolkit) environment decides that
the event belongs to that widget, then the callback
associated with that event is executed.
Information concerning the event may be retrieved
by asking the kit for the latest event.
}


\c{
Event objects represent the events generated by
the X-window system.
Each event has a type, which may be one of the types
listed below.
The type of the event can be inspected with $type()$ which
returns an integer value or $name()$ which returns
a string representation of the type.
For some of the common event types, such as $ButtonPress$,
$ButtonRelease$, and $MotionNotify$, member functions
are provided to facilitate testing.
If an integer argument (1,2 or 3) is given to $button()$,
$buttonup()$ or $buttonevent()$, it is checked whether
the event has occurred for the corresponding button. 

The functions $x()$ and $y()$ deliver the widget
coordinates of the event, if appropriate.

Calling $trace$ for the event results in printing
the type and coordinate information for the event.
When setting the $kit::trace$ level to 2 this
information is automatically printed.


Programmers not satisfied with this interface can
check the type and access the underlying XEvent at their
own risk.
}


\clearfalse
\sli{
@ Event
\zline{\fbox{event}}
[P
interface event  { 
int type();                    /// X event type
char* name();                  /// type as string

int x();			
int y();

int x_root();                   /// screen coordinate
int y_root();                   /// screen coordinate

int button(int i = 0);          /// ButtonPress
int buttonup(int i = 0);        /// ButtonRelease
int motion();                   /// MotionNotify

int keyevent();                 /// KeyPress or KeyRelease
int buttonevent(int i = 0);     /// ButtonPress or ButtonRelease

int keycode();

void trace();                   /// prints event information

class widget* widget();
XEvent* xevent();
};
P]
}

\c{
In addition to ordinary event information, an
event also contains a reference to
the widget for which the event occurred.
This information is valid only when the event is 
the last event. It may be lost when other
events occur.
}
