\c{
\label{widget}
Widgets are the elements a GUI is made of.
They appear as windows on the screen to display
text or graphics and may respond to events
such as motioning the mouse or pressing
a key by calling an action associated with that event.

Most often, the various widgets constituting the user
interface are (hierarchically) related to each other,
as for instance in a drawing application containing
a canvas to display graphic elements,
a button toolbox for selecting the graphic
items and a menubar offering various options
such as saving the drawing in a file.

Widgets in Tk are identified by a {\em path name}.
The path name of a widget reflects its possible
subordination to another widget.
The format of a path name may be expressed in BNF form as
<E>- $path ::= <empty> | '.'<string><path>$
where $<empty>$ denotes the empty string.
For example "." is the pathname of the root widget,
whereas ".quit" is the pathname of a widget
subordinate to the root widget.
A widget subordinate to another widget
must have the pathname of that widget
as part of its own path name.
For example, the widget ".f.m" may have a widget ".f.m.h"
as a subordinate widget.

Widgets may be configured in a variety of
ways.
For example most widgets allow specification of 
a foreground and a background color.
The $hush$ interface delegates most of these issues
to the underlying toolkit.
Users wishing to improve on the default
appearance of the widgets must study
the $options$ provided by Tk.

Naming widgets in a hierarchical fashion does not imply
that the widgets behave accordingly.
To group widgets, they must be packed in a common
ancestor widget, that is a $frame$ or $toplevel$
widget.
Alternatively, The Tcl command $pack$ may be used.
To map a widget to the screen it must either
be packed in the $root$ widget, using $kit::pack$,
or to a $toplevel$ widget, using $widget::pack$.\ftn{
The $pack$ command offered by Tcl/Tk is likely to
change in the near future. This may affect the
corresponding member functions for $kit$ and $widget$ as well.
\nop{Consult the reference manual for details.}
}

Widgets may respond to particular events.
To associate an event with an action,
an explicit binding must be defined for that
particular widget.
Some widgets provide default bindings.
These may however be overruled.
}

\sli{
@ Widget
\zline{\fbox{widget}}
[P
interface widget : handler { 

widget(char* p); 
widget(widget& w, char* p); 

char* type(); /// returns type of the widget
char* path(); /// returns path of the widget

int eval(char* cmd);  /// invokes "path() cmd"
char* result(); /// returns the result of eval
char* evaluate(char* cmd) /// combines $eval$ and $reuslt()$

virtual void configure(char* cmd); /// invokes '$path()$ configure cmd'

widget* pack(char* wp, char* options = "{top}" );
widget* pack(widget* w, char* options = "{top}" );

virtual bind(char *b, char* cmd, char* args = "" );
virtual bind(char *b, handler* h, char* args = "" );
virtual bind(char *b, command f, client* c = 0, char* args = "" );
virtual bind(char *b, action& ac, char* args = "" );

handler(char* cmd, char* args = "" );
handler(class handler* h, char* args = "" );
handler(command p, client* c = 0, char* args = "" );
handler(action& ac, char* args = "" );

void source(char* type, char* args, char* options="");
void target(char* type, action& ac, char* args, char* opts="");

void focus(char* options="");
void grab(char* options="");

void destroy(); /// removes widget from the screen

Tk_Window tkwin();  /// gives access to Tk implementation
protected:
virtual install(action&,char* args="");
};
P]
}

\c{
The $widget$ class is an abstract class.
Calling the constructor $widget$ as in
[P] widget* w = new widget(".awry");
does not result in creating an actual
widget but only defines a pointer to the widget
with that particular name.
If a widget with that name exists, it may be treated
as an ordinary widget object, otherwise an error will occur.
The constructor $widget(widget* w,char* path)$ creates
a widget by appending the path name $path$ to the
path name of the argument widget $w$.

The function $path$ delivers the pathname of
a widget object.
Each widget created by Tk actually defines a
Tcl command associated with the
path name of the widget.
In other words, an actual widget may be regarded
as an object which can be asked to evaluate commands.
For example a widget ".b" may be asked to change
its background color by a Tcl command like
[P] .b configure -background blue
The functions $eval$, $result$ and $evaluate$
enable the programmer to apply Tcl commands
to the widget directly, as does the $configure$
command.

The function $pack$ is used to pack
widgets together.
The $options$ parameter of $pack$ may be used
to instruct the geometry manager.
By default, widgets will be appended to the top of
the remaining space in the widget to which it is appended.
\nop{
For more involved geometry management options consult
the manual page for $pack$.
}

The function $bind$ is used to associate actions
with events.
These associations will exist only for
the widget for which they are declared.
The function $bind$ comes in four forms.
The first form associates a Tcl command with the events
specified by $b$.
The second and third form associate an action
either defined by a $command$ or a $handler$
with the events specified by $b$.
The fourth version takes an instance of an action 
as its argument. Actually, the first three forms
provide an abbreviation for the last form.
See section \ref{action}.
For the various bindings allowed consult the manual
page for $bind$.

The function $handler$ has also four forms, analogously to $bind$.
It installs an action for the default bindings of a particular widget.
The protected virtual function $install$ is assumed to
define the default bindings for each widget.
Only for widgets for which $install$ is (re)defined,
invoking $handler$ will have meaning.

The functions $source$ and $target$ may be used to declare
the widget as respectively a source or target
for a $drag\&drop$ operation.
%%See section \ref{tcl} for further details. 

The functions $focus$, $grab$ and $destroy$ correspond
with the Tcl/Tk functions of the same name.

To get hold of the underlying {\em Tk\_Window} object,
the user may employ the function $tkwin$.
This is needed when for example when one
wishes to manipulate bitmaps directly, by using
Tk functions.

The functions $configure$, $pack$, $bind$ and $handler$
are all declared virtual, since user defined widgets
consisting of multiple widgets may need to
redirect these commands to a component widget.
}
