<document>

The power of inheritance comes largely from virtual
functions and dynamic binding.
Dynamic binding realizes the polymorphism inherent
in the static type structure of the inheritance 
hierarchy in a dynamic way.
As a warming-up, dynamic binding is illustrated 
by the example shown in \sliref{7-dispatching}.

<slide id=7-dispatching cap="Virtual functions -- dispatching">
\ifsli{
@ Virtual functions
{\em -- dispatching}
\vspace{-1cm}
[P
class A { /// \fbox{$A$}
public:
virtual void operator()() {
	       cout << "A";
	       }
};

class B : public A { /// \fbox{$B$}
public:
virtual void operator()() {
	cout << "B";
	}
};
P]
\nop{
@ Usage
<I
- {P A* a = new B(); (*a)(); P}
I>
}
}{
@ Virtual functions
{\em -- dispatching}
[P
class A { /// \fbox{$A$}
public:
virtual void operator()() { cout << "A"; }
};

class B : public A { /// \fbox{$B$}
public:
virtual void operator()() { cout << "B"; }
};
P]
\nop{
@ Usage
<I
- {P A* a = new B(); (*a)(); P}
I>
}
}
}

\c{
The class $A$ defines a virtual member function
(that results in printing $A$) which is redefined
by a similar function in class $B$ (which results
in printing $B$).
As an example of using the classes defined above
look at the following program fragment.
[P
A* a = new B(); (*a)();
P]
In case the function would not have been defined as virtual,
the outcome of applying it to (a pointer to) a $B$ object
would have been $A$, instead of $B$.
}

\c{
Virtual functions that are redefined in derived classes
may still access the original version defined in the base
class, as illustated in \sliref{7-scoping}.
}

\slide{7-scoping}{Virtual functions -- scoping}{
\ifsli{
@ Scoping
{\em -- explicit}
\vspace{-1cm}
[P
class B : public A {  /// \fbox{$B'$}
public:
virtual void operator()() {
	cout << "B";
	A::operator()(); 
	}
};
P]
}{
@ Scoping
{\em -- explicit}
[P
class B : public A {  /// \fbox{$B'$}
public:
virtual void operator()() {
	cout << "B"; A::operator()(); 
	}
};
P]
}
}

\ifsli{}{
\c{
Scoping may be used within a member function of the
class as well as by a client (when invoking a member
function) as illustrated below.
[P
 A* a = new B(); a->A::operator()(); (*a)();
P]
The outcome of this statement is  $ABA$.

Such a scoping mechanism is certainly not unique
to C++, although the notation for it is.
In Smalltalk, the expression $super$ may be used
to access methods defined in the ancestor class,
and in Eiffel one must use a combination of
redefining and renaming to achieve this.
As a remark, occasionally I prefer the use of
$operator()()$ when any other method name would
be arbitrary.
The attractiveness of $operator()()$ is
that it is used as a function application
operator, as in $(*a)()$.
}
}
