\c{
@ Abnormal events
Abnormal events do not exclusively occur in distributed
systems.
In (ordinary) sequential systems, abnormal
events such as {\em division-by-zero} or external interrupts
may occur as well.
However, due to the increased (physical) complexity
of distributed systems, the likelihood
of abnormal events (as for example processor failure)
is simply greater.
In general, an abnormal event occurs when an
operation cannot perform its desired computation.
\see{6-abnormal}
}

\clearfalse
\slide{6-abnormal}{Abnormal events}{
@ Abnormal events
\ifsli{\vspace{-0.5cm}}{}
<I
- {\em exceptions} -- to indicate failure
- {\em intervention} -- invocation of a correction routine
I>
\ifsli{\vspace{-1cm}}{}
}

\c{
When an abnormal event occurs, two possible courses
of action can be taken.
See [_BuhrMac92].
Either the operation can fail and raise an exception
(thereby transferring the flow of control to
an exception handler)
or the operation can invoke a correction routine
(that tries to correct the situation that caused
the abnormal event) and continue the computation.
The latter solution is called an {\em intervention}.
In contrast to an exception,
an intervention does not change the flow of control,
it performs the correction and then resumes the operation.
}

\c{
Interventions are familiar from daily computing practice
in the form of interrupts, such as are used
to kill a process.
Under Unix, an interrupt is implemented
as a signal that may be associated with a handler.
The interrupt handler usually invokes the exit function.

Exceptions, on the other hand, are not (yet)
familiar to most programmers.
Only few languages support exceptions at all.
For example, exceptions are part of
the language definition of C++,
but only a limited number of the commonly used compilers
provides support for exceptions.
}

\c{
One of the languages supporting exceptions is Eiffel.
In Eiffel, exceptions are closely linked with the
notion of {\em contracts}.
Abnormal events occur whenever a contract is violated,
that is when either a pre-condition, post-condition
or invariant is not satisfied.
For each of these cases an exception handler may be specified
in the class defining the object for which an exception is raised.
The way abnormal events are dealt with in Eiffel
conforms to what is called the {\em resumption model}
in [_BuhrMac92].
The resumption model allows a handler invoked for an exception
to resume the computation after correcting the faulty situation.
In Eiffel, the {\em retry} statement may be used to try
the operation that resulted in raising an exception again.

The $%m${}System (which is an extension of the runtime
environment for $%m${}C++)
provides a library to deal with abnormal events.
It contains facilities for
termination, intervention, inheritance on exception handlers,
handlers that can take parameters,
various handler binding mechanisms
and asynchronous interrupts.
According to its authors, it is the most general
abnormal event handling package currently available.
}

\c{
In principle,
both exceptions and interventions may be simulated
in an ordinary language,
respectively by setting a global flag which is tested
after the operation
and by adding intervention routine parameters
to function definitions.
However, such simulations are tedious and error-prone
to implement and moreover violate encapsulation
and endanger extensibility.
}
