<document>
@* <a href=@talk-process.html target=_top>Objects and processes</a>
<hr>
<a href=g-proc.html>Instructors' Guide</a>
<hr>

Parallel processing has an impressive history,
both in a practical sense (one may say that processes
existed before objects came into life)
and in terms of the literature it has generated.
(See, for example Andrews, 1991.)
Yet the development of parallel (and distributed)
programs is generally considered to be an expert's job,
due to the inherent complexity of parallel
solutions and the synchronization involved.

Not surprisingly, the object-oriented community has
taken an interest in parallel computing,
hoping that the intrinsic complexity of parallel processing
could be better managed by an object-oriented
approach.

In the following, we will first explore by what means we may introduce
parallelism in an object-oriented system or language
and, subsequently, what encapsulation is needed for processes
and distribution in addition to the encapsulation 
already supported by the object model.

@so support


@+ Object-based concurrency

When it comes to combining {\em objects}
(the building blocks in an object-oriented approach)
with {\em processes} (the building blocks in
parallel computing),
there are three distributed approaches conceivable.
\see{6-o-conc}

\vstr{1}
<slide id=6-o-conc cap="Objects and concurrency">
@ Object-based concurrency
\ifsli{\vspace{-0.5cm}}{}
<I
- add processes {\em -- synchronization}
- multiple active objects {\em -- rendezvous}
- asynchronous communication {\em -- message buffers}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

One can simply add {\em processes} as an additional data
type. Alternatively, one can introduce {\em active objects},
having activity of their own,
or, one can employ {\em asynchronous communication},
allowing the client and server object to proceed
independently.

@ Processes
The first, most straightforward approach,
is to simply add {\em processes}
as a primitive data type, allowing the
creation of independent threads of processing.
An example is Distributed Smalltalk (see Bennett, 1987).
The disadvantage of this somewhat naive approach,
however,
is that the programmer has full responsibility
for the most difficult part of parallel programming,
namely the synchronization between processes and the
avoidance of common errors
such as simultaneously assigning a value to a shared
variable.

Despite the fact that the literature (see Andrews, 1991)
abounds with primitives supporting synchronization
(such as semaphores, conditional sections and monitors),
such an approach is error-prone and means a heavy burden on the
shoulders of the application developer.

@ Active objects
A second, and in my view to be preferred, approach
is to introduce explicitly a notion of {\em active objects}.
Within this approach, parallelism is introduced by having
multiple, simultaneously active objects.
An example of a language supporting active objects
is POOL (see America, 1987).
Communication between active objects occurs
by means of a (synchronous) rendezvous.
To engage in a rendezvous, however,
an active object must interrupt its
own activity by means of an (Ada-like) {\em accept statement}
(or {\em answer statement} as it is called in POOL),
indicating that the object is willing to answer a message.
The advantage of this approach is, clearly, that the encapsulation boundary
of the object (its message interface)
can conveniently be employed
as a monitor-like mechanism to enforce
mutual exclusion between method invocations.

Despite the elegance of this solution, however,
unifying objects and processes in {\em active objects}
is not without problems.
First, one has to decide whether to make
all objects active or allow both
passive and active objects.
Logically, passive objects may be regarded as active
objects that are eternally willing to answer every message listed
in the interface description of the object.
However, this generalization is not without
penalty in terms of runtime efficiency.
Secondly, a much more serious problem is that the
message answering semantics of active objects
is distinctly different from the message
answering semantics of passive objects with respect
to self-invocation.
Namely, to answer a message, an active object
must interrupt its own activity.
Yet, if an active object (in the middle of answering a message)
sends a message to itself, we have a situation of {\em deadlock}.
Direct self-invocation, of course, can be easily detected,
but indirect self-invocations require an analysis of the complete
method invocation graph, which is generally not feasible.

@ Asynchronous communication
Deadlock comes about by synchronous (indirect)
self-invocation.
An immediate solution to this problem is provided
by languages supporting asynchronous communication,
which provide message buffers allowing the caller 
to proceed without waiting for an answer.

Asynchronous message passing, however, 
radically deviates from the (synchronous) message passing
supported by the traditional (passive) object
model.
This has the following consequences.
First, for the programmer, it becomes impossible
to know when a message will be dealt with
and, consequently, when to expect an answer.
Secondly, for the language implementor,
allocating resources for storing
incoming messages and deciding when to deal
with messages waiting in a message buffer
becomes a responsibility for which it is hard to
find a general, yet efficient, solution.

Active objects with asynchronous message passing
constitute the so-called {\em actor} model,
which has 
influenced several
language
designs. See [_Agha].


@+ Process-based encapsulation

In his seminal paper on the dimensions
of object-oriented design, [_Wegner87]
observes that various notions of encapsulation,
as have become popular with object-oriented programming,
already existed in one form or another
in the distributed programming community.
To establish what requirements
an object-oriented system must meet to qualify
as (potentially) distributed,
we once more rely on the analytical work
of [_Wegner87], delineating precisely
the notions of processes, threads and distribution.

%%@ A taxonomy of processes
A {\em process}, according to [_Wegner87], consists
of an {\em interface} (naming the possible transactions,
that is operations, allowed)
and
one or more {\em threads} (that may be active or suspended).
A {\em thread} is characterized by [_Wegner87]
as consisting of a {\em locus of control}
(in other words, a program counter)
and an {\em execution state} (consisting of
a runtime stack and possibly a heap for dynamically
created data).
\see{6-threads}

\clearfalse
\vstr{1}
<slide id=6-threads cap="Processes and threads">
@ Taxonomy of processes
\ifsli{\vspace{-0.5cm}}{}
<ul>
<li> {\bf process}: interface + active/suspended thread(s)
<li> {\bf thread}: locus of control + execution state
<li> {\em sequential process} -- one thread of control 
<li> {\em quasi-concurrent} -- at most one active thread
<li> {\em concurrent} -- multiple active threads 
</ul>
\ifsli{\vspace{-1cm}}{\vspace{-0.3cm}}
</slide>

Dependent on the number of threads, we may
characterize a process as either
{\em sequential} (a single thread of control),
{\em quasi-concurrent} (at most one active thread,
arbitrarily many suspended)
or (truly) {\em concurrent} (multiple active threads,
arbitrarily many suspended).

When considering active objects, a distinction
between {\em single threaded} objects
and {\em multi-threaded} objects seems to be more
appropriate.
The most straightforward approach is to associate
an object with a single thread of control,
that may be divided between answering messages
and the object's own activity.

The model of {\em communicating sequential processes}
(corresponding to single threaded objects)
was first advocated in [_Ho78].
It underlies programming languages such as CSP, occam and Ada,
and is probably the best understood model
of parallel processing that we have
(see Andrews, 1991).

In combination with the assumption that objects
share no variables,
single threaded objects bear a close resemblance
to what [_Wegner87] calls
{\em distributed sequential processes}.
However, with reference to the language DLP
(which employs multi-threaded objects to
support distributed backtracking),
we must note that single-threaded objects
are not the only means to implement active objects.

Distributed sequential processes,
as characterized by [_Wegner87],
provide a combination of characteristics
that make them exemplary as a model for
active objects.
\see{6-seq-proc}

<slide id=6-seq-proc cap="Sequential processes">
@ Distributed sequential processes <em> -- own address space</em>
<ul>
<li> {\em Unit of modularity} -- user interface
<li> {\em Unit of concurrency} -- single thread
<li> {\em Unit of naming} -- name space
</ul>
\ifsli{}{\vspace{-0.3cm}}
</slide>

Perhaps their most important characteristic
is the absence of global data
shared with other processes.
As a consequence, they provide
what may be called {\em strong encapsulation},
since in addition to
the absence of global variables, the
transaction interface of a process
precisely delimits the functionality offered
to potential clients.

In addition to the protection with regard to global variables,
distributed processes must satisfy a number of
other requirements to guarantee correct execution.
These requirements will be dealt with in more detail in section 
\ref{distribution}.

@so active
</document>
