<document>
@* <a href=g-dim.html>Design dimensions of object-oriented languages</a>
\label{dimensions}


Despite the widespread adoption of object-oriented terminology in
the various areas of computer science and software
development practice, there is  considerable confusion about
the precise meaning of the terms employed and the (true)
nature of object-oriented computing.
In an attempt to resolve this confusion, [_Wegner87]
(in the landmark paper {\em Dimensions of object-based language design})
introduces the distinction between {\em object-based}
and {\em object-oriented}.
\see{5-object-based}
This distinction comes down to, roughly, the distinction between
languages providing only encapsulation (object-based) or encapsulation plus
inheritance (object-oriented). See section \ref{object-based}.
Another issue in the debate about object orientation
is the relation between {\em classes}
and {\em types}.
[_Wegner87] concludes that the notions of objects, classes
and inheritance (that constitute the classical object model)
are highly interrelated, and instead
proposes an orthogonal approach by outlining the various dimensions
along which to design an object-oriented language.
These dimensions may be characterized by the phrases: {\em objects},
{\em types}, {\em delegation} and {\em abstraction}.

In this section we will look at the arguments presented in [_Wegner87]
in somewhat more detail.
Also, we will look at the viability of combining seemingly
disparate paradigms (such as the logic programming paradigm)
with the object-oriented language paradigm.
In the sections that follow, we will discuss some alternatives and extensions to
the object model.



\vfig{0.5}
<slide id=5-object-based cap="Object-based versus object-oriented">
@ Object Oriented Language Design
\nl
{\footnotesize
{\bf object}: state + operations\n
{\bf class}: template for object creation\n
{\bf inheritance}: super/base and subclasses
}
[D
object-\=\+oriented = 
      objects + classes + inheritance 
D]
\ifsli{\ln}{}
@ data abstraction
{\em -- state accessible by operations}
@ strong typing
{\em -- compile time checking}
</slide>

@+ Object-based versus object-oriented
\label{object-based}

How would you characterize Ada83? See  [_Barnes94].
Is Ada object-oriented?
And Modula-2? 
See [_Wirth83].
The answer is no and no.
And Ada9X and Modula-3?
See [_Barnes94] and [_Card89].
The answer is yes and yes.
In the past there has been some confusion
as to when to characterize a language as
{\em object-oriented}.
For example, [_Booch86] characterizes Ada as
object-oriented and motivates this by saying that Ada
can be used as an implementation language in
an object-oriented approach to program development.


Clearly, Ada supports some notion of objects (which
are defined as {\em packages}).
However, although Ada supports objects and generic descriptions 
of objects (by generic packages), it does not support
code sharing by inheritance.
In a later work, [_Booch91] revises his original (faulty)
opinion, in response to 
[_Wegner87], who proposed considering {\em inheritance}
as an essential characteristic of {\em object orientation}.

Similarly, despite the support that Modula-2 offers
for defining (object-like) abstract data types, 
consisting of
an interface specification and an implementation
(which may be hidden), Modula-2 does not support
the creation of derived (sub)types that share
the behavior of their base (super)type.
See also section \ref{adt-modules}.

@ Classes versus types
Another confusion that frequently arises
is due to the ill-defined relationship between
the notion of a class and the notion of a type.

The notion of {\em types} is already familiar
from procedural programming languages such as
Pascal and (in an ill-famed way) from C.
The type of variables and functions may be profitably
used to check for (syntactical) errors.
Strong static type checking may prevent errors
ranging from simple typos to using undefined
(or wrongly defined) functions.

The notion of a class originally has a more
operational meaning.
Operationally, a class is a template for object creation.
In other words, a {\em class} is a description of the collection
of its instances, that is the objects that are
created using the class description as a recipe.

Related to this notion of a class, {\em inheritance} was
originally defined as a means to share (parts of)
a description.
Sharing by (inheritance) derivation is, pragmatically,
very convenient.
It provides a more controlled way of code sharing
than, for example, the use of macros and file inclusion
(as were popular in the C community).

Since [_Wegner87] published his original analysis
of the dimensions of object-oriented language design,
the phrase {\em object-oriented} has been commonly understood
as involving {\em objects}, {\em classes} and {\em inheritance}.
This is the traditional object model as embodied
by Smalltalk and, to a large extent, by Eiffel and C++.
However, unlike Smalltalk, both Eiffel and C++ have also been
strongly influenced by the {\em abstract data type} approach
to programming.
Consequently, in Eiffel and C++ classes have been identified
with types and derivation by inheritance with subtyping.

Unfortunately, derivation by inheritance need not necessarily
result
in the creation of proper subtypes,
that is classes whose instances conform to the behavior
specified by their base class.
In effect, derived classes may be only distantly related
to their base classes when inheritance is only
used as a code sharing device.
For example, a window manager class may inherit
from a list container class (an idiom used in Meyer, 1988).

For those inclined to a more formal approach,
the phrase {\em object-oriented} has gradually acquired
the meaning of {\em abstract data types}
and {\em polymorphism}.
From the perspective of language design, however,
this characterization merely serves as a constraint
that (the application of) particular constructs must satisfy
and as an invitation to explore language constructs
that enable the use of abstractions and flexible
behavioral specifications. 



@+ Towards an orthogonal approach {\em -- type extensions}
\label{orthogonal}

According to [_Wegner87], much of the confusion
around the various features of object-oriented
programming languages arises from the fact that
these features are largely interdependent,
as for instance the notion of object and class on 
the one hand,
and the notion of class and inheritance on the other.

To resolve this confusion, [_Wegner87] proposes
a more orthogonal approach to characterize the
various features of object-oriented languages,
according to dimensions that are to a large extent independent.
\see{5-orthogonal}


\vstr{1}
<slide id=5-orthogonal cap="Orthogonal dimensions">
@ Orthogonal approach
\ldeno{}{\footnotesize
\ifsli{\vspace{-0.5cm}}{}
{\bf objects} -- {\em modular computing \c{agents}}\n
{\bf types} -- {\em expression classification}\n
{\bf delegation} -- {\em resource sharing}\n
{\bf abstraction} -- {\em interface \c{specification}}\n
}\nl
{\bf type}: behavior specification\n
{\bf delegation}: ancestors \c{(with dynamic self-reference)}
\nl\ifsli{\vspace{-1cm}}{}
</slide>

The features that constitute an object-oriented
programming language in an orthogonal way are,
according to [_Wegner87]: {\em objects}, {\em types},
{\em delegation} and {\em abstraction}.

@ Objects
are in essence {\em modular computing agents}.
They correspond to the need for encapsulation in design,
that is the construction of modular units to
which a principle of locality applies (due to
combining data and operations).

Object-oriented languages may, however, differ in
the degree to which they support encapsulation.
For example, in a distributed environment a 
high degree of encapsulation must be offered,
prohibiting attempts to alter global variables
(from within an object) or local instance
variables (from without).
Moreover, the runtime object support system must
allow for what may best be called {\em remote method invocation}.

As far as parallel activity is concerned, only a few languages provide
constructs to define concurrently active objects.
See chapter \ref{communication} for a more detailed discussion.

Whether objects support {\em reactiveness},
that is sufficient flexibility to respond safely
to a message, depends largely upon (program) design.
[_Meyer88], for instance, advocates a {\em shopping list
approach} to designing the interface of an object,
to allow for a high degree of (temporal) independence
between method calls.

@ Types
may be understood as a mechanism for {\em expression classification}.
From this perspective, Smalltalk may be regarded
as having a dynamic typing system:
dynamic, in the sense that the inability to evaluate an expression
will lead to a runtime error.
The existence of types obviates the need to have classes,
since a type may be considered as a more abstract description of the
behavior of an object.
Furthermore, subclasses (as may be derived through inheritance)
are more safely defined as subtypes in a polymorphic
type system. See section \ref{flavors}.

At the opposite side of the type dimension we find the statically typed languages,
that allow us to determine the type of 
the designation of a variable at compile-time.
In that case, the runtime support system need not carry any type information,
except a dispatch table to locate virtual functions.

@ Delegation
(in its most generic sense)
is a mechanism for {\em resource sharing}.
As has been shown in [_Lieber], delegation subsumes inheritance,
since the resource sharing effected by inheritance may easily be mimicked
by delegating messages to the object's ancestors
by means of an appropriate dispatching mechanism.

In a narrower sense, delegation is usually understood as a more
dynamic mechanism that allows the redirection of control dynamically.
In addition, languages supporting dynamic delegation
(such as Self, see section \ref{prototypes}) do
not sacrifice dynamic self-reference.
This means that when the object executing
a method refers to itself, the actual context
will be the delegating object.
See section \ref{prototypes} for a more detailed discussion.

In contrast, inheritance (as usually understood in the context of classes)
is a far more static mechanism.
Inheritance may be understood as (statically) copying the code 
from an ancestor class to the inheriting class
(with perhaps some modifications),
whereas delegation is truly dynamic in that messages are dispatched
to objects that have a life-span independent of the dispatching object.

@ Abstraction
(although to some extent related to types) 
is a mechanism that may be independently applied to provide
an {\em interface specification} for an object.
For example, in the presence of active objects
(that may execute in parallel)
we may need to be able to restrict dynamically the interface of an object as specified by its type
in order to maintain the object in a consistent state.
Also for purely sequential objects we may impose a particular protocol of interaction
(as may, for example, be expressed by a {\em contract}) to be able to
guarantee correct behavior.

Another important aspect of abstraction is {\em protection}.
Object-oriented languages may provide (at least) two kinds
of protection.
First, a language may have facilities to
protect the object from illegal access by a client (from without).
This is effected by annotations such as {\em private} and {\em protected}.
And secondly, a language may have facilities to protect the object
(as it were from within) from illegal access through delegation
(that is by instances of derived object classes).
Most languages support the first kind of protection.
Only few languages, however (among which are C++), support the second kind too. 

The independence of abstraction and typing may further
be argued by pointing out that languages
supporting strong typing need not
enforce the use of abstract data types having a well-defined
behavior.



:@ Discussion
Currently there is no single language providing
what in any sense can be called an ideal or
optimal combination of features.
At least there is no consensus as to whether
Smalltalk, Eiffel, C++ or any of the other
languages mentioned is the likely candidate
for this position.
A critique on many of these languages is that they
contain too many features or try to serve too many goals.
Think of Smalltalk's meta classes (see section \ref{meta}),
Eiffel's attempt to be satisfying both from
a practical and formal point of view
(which severely limits the power of the assertion mechanism)
and the intricate type construction and coercion
operators of C++ (which make it flexible, but complex).
An interesting example of what may be called
{\em minimal language design}
is presented by the language Oberon
that features so-called {\em type extensions}
encompassing (in a {\em minimal} way) both the notion
of object and class inheritance.

."@ Type extensions
The Oberon language has been developed
as the (object-oriented) successor of Modula-2.
According to [_BPS89], simplicity was the prime goal
of its design.
Oberon is a procedural language (like Modula-2)
with strong static type checking.
It provides only simple data types, arrays and records.
A class concept is lacking.

Oberon, however, supports records that may
contain (so-called {\em installed}) procedures.
And, more importantly, records can be extended.
Extension works like inheritance, 
in that a record type can be defined that
includes all the components of the extended
(read inherited) record type.
In Oberon a record that is the result
of extending a given record type
is called {\em compatible} with the original
record type,
which means that it may be assigned to a variable holding
a record of that (base) type.
When such an assignment is made, the extended
record is projected onto the original
record (type),
ignoring the fields that are added due to the extension.

In a similar way as Modula-2,
Oberon supports separate compilation by requiring
a definition part (containing an interface specification)
and an implementation part.
Modules may have private classes (that is records),
just by omitting them from the interface specification.
Oberon allows the dynamic creation of (record) objects.
A garbage collector takes care of their elimination.
As far as inheritance is concerned, Oberon only supports single
inheritance.
In other words, record extension only operates 
on a single (base) record.
Moreover, inheritance in Oberon is strict,
due to the restriction that no overriding is allowed.
Viewed from the principle {\em small is beautiful},
Oberon indeed is a very nice language, containing a
minimal set of features to support object-oriented
programming.
However, from a programming perspective,
as a vehicle to implement concrete data types
(see section \ref{canonical}) it leaves
much to be desired.
Not surprising, then, that even purported
{\em object-oriented} extensions to Oberon have been proposed.
See [_BPS89] for a further discussion.

@so multi
</document>
