<document>
@* <a href=g-para.html>The object paradigm</a>


The object paradigm is embodied in numerous programming languages.
[_Sau89] presents a survey of 88 object-oriented languages,
of which 69 are {\em standalone} and 19 incorporated
into either multi-paradigm or database systems.
(A multi-paradigm system, in this context, means a system embedding
an environment for window-programming or knowledge-based
reasoning.)

In this section, we will first look at the classification of
object-oriented languages (as given in Saunders, 1989).
Most of the languages mentioned are based on a distinction
between classes and objects.
However, alternative object models (without classes) are also being 
employed.
Finally, we will review a number of object extensions of the languages
Lisp, C and Prolog.


@so object


@+ A classification of object-oriented languages
\label{classification}

To be characterized as {\em object-oriented} a language must minimally
support an {\em object creation} facility and a {\em message passing} facility
(message-passing in the sense of method invocation).
In addition, many languages provide a mechanism to define {\em classes}
together with some form of {\em inheritance}.
\see{5-classification}

<slide id=5-classification cap="A classification of languages">
@ Objects -- language characteristics
<I
- object creation facility
- message passing capability
- class capability
- inheritance features
I>
:@ Classification
<I
- hybrid -- {\em C, Lisp, Pascal, Prolog}
- frame-based -- {\em knowledge-based reasoning}
- distributed, concurrent, actor -- {\em parallel computing}
- alternative object models -- {\em prototypes, delegation}
I>
</slide>

Actually, as we will see in section \ref{dimensions}, there is
a lively debate on the proper design dimensions of object-oriented
programming languages.
An important issue in this respect is what makes a language 
object-oriented as opposed to object-based.
Other issues in this debate are whether an object-oriented language 
must support classes (in addition to a mechanism to create objects)
and whether (static) inheritance should be preferred above (dynamic) delegation.
This debate is reflected in a number of research efforts investigating
alternative object models and object communication mechanisms.
See section \ref{prototypes}.


."@+ Classification 

Of the 69 (standalone) object-oriented languages surveyed,
53 were research projects and only sixteen were commercial products.
Of these, 14 were extensions of either Lisp (10)
or C (4).
Among the remaining languages, quite a number were derived from 
languages
such as Pascal, Ada or Prolog.

There is a great diversity between the different object-oriented languages.
However, following [_Sau89], they may be divided among subcategories reflecting
their origin or the area of application
for which they were developed (as shown above).

@ Hybrid languages
originating out of (an object-oriented
extension of) an already existing language are likely to be applied
in a similar area as their ancestor.
In practice, this category of languages (which includes C++ and CLOS) seems
to be quite important, partly because their implementation support is
as good as the implementation support for their base languages
and, more importantly, they allow potential software developers a smooth transition 
from a non object-oriented to an object-oriented approach.

@ Frame-based languages
(in contrast to the previous category) were explicitly developed to
deal with one particular application area, in particular
knowledge-based reasoning.
A {\em frame} is a structure consisting of {\em slots} that may either indicate a relation
to other frames or contain the value of an attribute of the frame.
In fact, the early frame-based languages such as FRL
(see Bundy, 1986) and KRL (Bobrow and Winograd, 1977) may be
considered as object-oriented {\em avant la lettre},
that is before object-orientation gained its popularity.
Later frame-based systems, such as KEE (Kunz {\it et al.}, 1984)
and LOOPS (Stefik and Bobrow, 1986),
incorporated explicitly object-oriented notions such
as classes and (multiple) inheritance.

@ Concurrent, distributed and actor languages
To promote the use of parallel processing architectures
a number of parallel object-oriented languages
have been developed, among which are the language {\em Hybrid}
(which supports active objects with their own thread of control,
[_Nierstrasz]),
Concurrent Smalltalk (a concurrent extension of Smalltalk),
Orient-K (a language for parallel knowledge processing)
and POOL-T (which may be characterized as a simplified version
of Ada). (See Yonezawa and Tokoro, 1987.)

POOL-T also supports the notion of {\em active objects}.
Active objects have a body which allows them to execute their 
own activity in parallel with other active objects.
To answer a request to execute a method, an active object must
explicitly interrupt its activity (by means of an answer or
{\em accept} statement as in Ada).
POOL-T is interesting, primarily, because it is complemented
by extensive theoretical research into the semantical foundations
of parallel object-oriented computing.
(See de Bakker {\it et al.}, 1990, and also section
\ref{semantics}.)

The idea of simultaneously active objects leads in a natural way 
to the notion of distributed object-oriented languages
that support objects which may be located on geographically
distinct processors and which communicate by means of (actual)
message passing.
Examples of such languages are {\em Distributed Smalltalk}
(a distributed extension of Smalltalk that introduces so-called
{\em proxy objects} to deal with communication between 
objects residing on different processors, [_Bennett])
and {\em Emerald} (that supports primitives to migrate
objects across a processor network, [_Black]).

All parallel/distributed object-oriented languages introduced
thus far are based on a traditional object model in so far as
an object retains its identity during its lifetime.
In contrast, the so-called {\em actor} languages support
a notion of object whereby the parallel activity of an object
is enabled by {\em self-replacement} of the object in response to a message.
Self-replacement proceeds as follows.
Each {\em actor} object has a mail-queue.
When a message arrives for the {\em actor} object,
the object invokes the appropriate method and subsequently
creates a successor object (which basically is a copy of itself
with some modifications that may depend upon 
the contents of the message).
In other words, message handling occurs asynchronously.
This scheme of asynchronous message passing enables an {\em actor}
system to execute in parallel,
since during the execution of a method the replacement object
may proceed to handle other incoming messages.

In {\em actor} systems, object identity is replaced by what
may be called mail-queue or address identity.
From a theoretical viewpoint this allows us to treat {\em actor}
objects as functions (in a mathematical sense) that
deliver an effect and another object in response 
to a message.
However, pragmatically this leads to a complicated
and quite low-level object model which is hard to
implement in a truly parallel way.

@+ Alternative object models
Since the introduction of Smalltalk, the predominant
notion of objects has been based on the distinction
between classes and objects.
Classes serve to describe the functionality and behavior of objects,
while objects are instance of classes.
In other words, classes serve as templates to create objects.
Inheritance, then, may be regarded
as a means by which to share (descriptions of) object
behavior.
It is generally defined on classes in terms of a derivation mechanism,
that allows one to declare a class to be a subclass of another (super)
class.

The distinction between classes and objects leads to a number
of difficulties, both of a pragmatic and theoretical nature.
(See also sections \ref{dimensions} and \ref{meta} for a discussion of the theoretical
problems.)
For example, the existence of {\em one-of-a-kind} classes,
that is classes which have only one instance, is
often considered unnatural.
An example of a class-less language is the language {\em Self}.
Self has a Smalltalk-like syntax, but in contrast 
to Smalltalk only supports {\em objects} (containing slots)
and {\em messages}, and hence no classes.
Slots may be designated to be parent-slots which means that
messages that cannot be handled by the object itself are delegated to the
parent object.
In contrast to inheritance, which is static since the inherited
functionality is computed at object creation time, delegation
to parent objects as in Self is dynamic, since parent slots
may be changed during the lifetime of an object.

Objects in Self may themselves be used to create other objects
(as copies of the original object) in a similar way as classes
may be used to create instances.
However, the changes made to an object are propagated when cloning
object copies.
Single objects, from which copies are taken, may in other words be regarded
as prototypes, approximating in a dynamic way the functionality of 
their offspring,
whereas classes provide a more static, so to speak universal,
description of their object instances.
Self employs runtime compilation, which is claimed
to result in an efficiency comparable to C.
(See Ungar {\it et al.}, 1992.)
In section \ref{prototypes} we will discuss the
use of prototypes and the distinction between inheritance
and delegation.


Alternative object models may also be encountered 
in object-oriented database managements systems and in systems
embedding objects such as hypertext or hypermedia systems.
See chapter 12.

@+ Object extensions of Lisp, C and Prolog

The notion of object is to a certain extent orthogonal
to, that is independent of, language constructs around
which programming languages may be constructed,
such as expressions, functions and procedures.
Hence, it should come as no surprise that a number of (popular)
object-oriented programming languages were originally
developed as extensions of existing languages or
language implementations.
\see{5-extensions}

<slide id=5-extensions cap="Object-oriented languages">
@ Object extensions
<I
- Lisp -- LOOPS, FLAVORS, CLOS, FOOPS
- C -- Objective C, C++
- Prolog -- SPOOL, VULCAN, DLP
I>
:@ Commercial products 
<em> -- languages</em>
*- Smalltalk, Eiffel, C++, Objective C, Object Pascal
</slide>

The advantage of extending an existing language with
object-oriented constructs, from the point of view
of the user, is that the object-oriented approach can be
gradually learned.
However, at the same time this may be a disadvantage, since a hybrid
approach to software development may give rise to sloppy design.
Many proponents of an object-oriented approach, therefore, believe
that learning to use object-oriented constructs
is best done in an environment as offered by Smalltalk,
where classes and objects are the sole means of developing 
an application.

It is noteworthy that, with the exception of Smalltalk
and Eiffel, many commercially available languages
are actually extensions of existing languages such
as Lisp, C and (to some extent) Prolog. 


@ Lisp-based extensions
In [_Sau89], ten Lisp-based object-oriented languages
are mentioned, among which are
LOOPS (introducing a variety of object-oriented constructs,
see [_LOOPS]),
Flavors (which extends Lisp by adding generic functions
that operate on objects, see [_Moon]),
CLOS (which is actually a standardization effort
of the ANSI X3J13 group to define the Common Lisp Object Standard).
CLOS is a widely used system containing some
nontrivial extensions to the object model
and the way in which polymorphic methods may be defined.
%%See section \ref{adt-theory} for further discussion.

@ C-based extensions
Another very important class of object extensions
is those of C-based object-oriented languages,
of which the most well-known are Objective-C and C++.

The concepts underlying these two extensions
are radically different.
Objective-C introduces objects as an add-on to
the constructs (including {\it struct}'s) available
in C,
whereas C++ realizes a close (and efficient)
coupling between the {\it struct} (record)
notion of C and the concept of a class.

In other words, in Objective-C there is a clear
distinction between conventional C values and data types
such as $int$, $float$ and $struct$ on the one hand, and
objects on the other hand.
Objects have a special data type ($id$) which allows
them to be treated as first class elements.
To define an object class, both an {\em interface}
and {\em implementation} description must be given.
These descriptions are preceded by a special sign 
to designate Objective-C specific code.
Also, method declarations (in the interface description)
and method definitions (that are to be put in the
implementation section) must be preceded by a special sign
to designate them as methods available
for clients of object instances of that class.

The object model of Objective-C closely resembles the
object model of Smalltalk.
In contrast, C++ quite radically departs from this object 
model in order to achieve an as efficient as possible implementation of
objects.
The key to an efficient implementation lies
in the integration of the {\em struct} (record) construct
originally provided by C with the class concept,
by allowing functions to be members of a {\it struct}.

As explained in [_St91], the equivalences depicted in
\sliref{5-structure} hold.


\cleartrue
<slide id=5-structure cap="The equivalence between {\em class} and {\em struct">
@ Object structure
<em> -- efficient mapping</em> 
\zline{\fbox{C++}}
\ifsli{\vspace{-0.5cm}}{}
<I
- {P struct A { ... } $ == $ class A { public: ... } P}
- {P class A { ... } $ == $ struct A { private: ... } P}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

This interpretation allows an efficient mapping
of object structures to the memory of a computer,
provided that the compiler is clever enough.

Nevertheless, the efficiency of C++ comes
at a price. C++ does support micro-efficiency
but does not necessarily lead to the design of
efficient code.
In particular, hand-crafted memory management
will not necessarily offer the most efficient
solution when compared with built-in support,
but is almost certainly detrimental to the quality of the code.

As argued in [_ES90], the design of C++ is intended to
allow an efficient compilation of C++ into C,
that is efficient C code.
Yet, the efficiency of C++ is not
due to its being compiled into C,
but comes from an efficient mapping of object (type)
structures and class member functions to analogous C structures
and functions that require a minimum of overhead.


In effect, C is increasingly being used as a target
language for the compilation of high level languages.
The reason for this is, however, primarily {\em portability},
and not efficiency.
For example, Eiffel compilers (both under Unix and MS-DOS)
produce C code and also allow Eiffel programs to
interface with programs written in C.
However, the overhead due to the exception handling primitives
of Eiffel and the overhead in calling functions defined 
in C (still) results in quite inefficient code,
despite the translation into C.

As portability is concerned,
many authors optimistically claim
the portability of their C++ code
(across platforms and compilers).
However, portability has not been realized
for C++ as it has for ANSI-C.
Currently available compilers (including
AT\&T 3.0 and those of Borland, GNU and Zortech)
definitely disagree on the handling of, for
instance, multiple inheritance, pure virtual functions,
templates and overloading.
Hopefully, the standardization efforts of the C++ ANSI committee
will help to reduce these problems.

@ Prolog-based extensions
A quite different class of object-oriented extensions,
used primarily in research laboratories,
consists of attempts to incorporate object-oriented
features in (high level) logic-based languages, such as Prolog.
Among these are languages such as SPOOL (developed
in the context of the Japanese fifth-generation computing
project, see [_Fukanaga]),
Vulcan (that provides a preprocessor giving syntactic support for embedding objects in
concurrent logic programming languages, see [_Kahn])
and DLP (a language combining logic programming
with object-oriented features and parallelism developed by the author,
see appendix D).
The list of research articles covering the subject of combining
logic programming and object-oriented programming
is quite extensive.
An overview and discussion of the various approaches
is given in [_Davison93] and also in _Eliens92.
</document>
