%{
// (c) A. Eliens (VU)
// html-latex.flt
// html -> latex filter (in a primitive fashion)

#include <iostream.h>
#include <string.h>

int yywrap() { return 1; }    // may be redefined

static int pr=1;     // output yes
static int nu=1;     // chapter numbering
static int pre = 0;  // pre or verbitim environment
static int cmt = 0;  // in comment or not
static int refs = 0; // reference environment
static int fig = 0;  // figure environment
static int math = 0; // math environment
static char* _lastag = "}"; // to allow for </>

static char* lastag() { return _lastag; }

%}
filename XXX
URL [\/\:A-Za-z\.0-9\-\+]+
%%
"<title>".*  { ;}
"</">  { if (pr) cout << lastag(); }
"<nonumbers>" { nu = 0; }
"</nonumbers>" { nu = 1; }
"<h"[0-9]">" {
     if (nu) {
	switch( yytext[2] ) {
	case '0': if (pr) cout << "\\chapter{"; break; 
	case '1': if (pr) cout << "\\section{"; break; 
	case '2': if (pr) cout << "\\subsection{"; break; 
	case '3': if (pr) cout << "\\subsubsection{"; break; 
	case '4': if (pr) cout << "\\paragraph{"; break; 
	default:
	 if (pr) cout << "\\paragraph{"; break; 
	};
     } else {
	switch( yytext[2] ) {
	case '0': if (pr) cout << "\\chapter*{"; break; 
	case '1': if (pr) cout << "\\section*{"; break; 
	case '2': if (pr) cout << "\\subsection*{"; break; 
	case '3': if (pr) cout << "\\subsubsection*{"; break; 
	case '4': if (pr) cout << "\\paragraph{"; break; 
	default:
	 if (pr) cout << "\\paragraph{"; break; 
	};
     }
  }
"</h"[0-9]">" { if (pr) cout << "}"; }
"<html>" { ; }
"</html>" { ; }
"<control>"|"<notex>" { pr=0; } 
"</control>"|"</notex>" { pr=1; }
"<hr>" { if (0) cout << "\\par\\mbox{}\\hrulefill\\par"; }
"<p>"  { if (pr) cout << "\n\n" << endl; }
"</h"[1-9]">" { ;}
"<!--" { cmt = 1; }
"-->" { cmt = 0; }
"<a href=http"[^>]*">" { ;}
"<a href=ftp"[^>]*">" { ;}
"<a href="{filename}">" { ;}
"<a name="[^>]*">" {
	yytext[yyleng-1]='\0';
	if (pr) {
		cout << "\\label{" << yytext+8 << "}";
		}
	}
"<a href=#"{filename}">" { ;}
"<a "[^>]*">" { ;}
"<img src="[^>]+">" { 
	if (0) {  // just ignore
		int p = yyleng - 5;
		yytext[p+1] = 'p';
		yytext[p+2] = 's';
		yytext[p+3] = '\0';
	 cout << "\\epsfbox{" << yytext+9 << "}";  	
	}
   }
"</a>" { ;}
"<pre>" { pre=1; if (pr) cout << "\\begin{verbatim}"; }
"</pre>" { pre=0; if (pr) cout << "\\end{verbatim}"; }
"<math>" { math=1; if (pr) cout << "\\begin{math}"; }
"</math>" { math=0; if (pr) cout << "\\end{math}"; }
"<menu>"|"<ul>" { if (pr) cout << "\\begin{itemize}"; }
"</menu>"|"</ul>" { if (pr) cout << "\\end{itemize}"; }
"<ol>" { if (pr) cout << "\\begin{enumerate}"; }
"</ol>" { if (pr) cout << "\\end{enumerate}"; }
"<dl>" { if (pr && !refs) cout << "\\begin{description}"; }
"</dl>" { if (pr && !refs) cout << "\\end{description}"; }
"<dt> ".* { if (pr)
  if (refs)
	cout << "\\bibitem[" << yytext+5 << "]{" << yytext+5 << "}";
   else
	cout << "\\item[" << yytext+4 << "]";
  }
"<dd>" {;}
"<br>" { if (pr) cout << endl;  /* "\\linebreak[0]";*/ }
"<li>"  { if (pr) cout << "\\item "; }
"<em>"   { if (pr) cout << "{\\em "; }
"</em>"  { if (pr) cout << "}"; }
"<tt>"   { if (pr) cout << "{\\tt "; }
"</tt>"  { if (pr) cout << "}"; }
"<i>"   { if (pr) cout << "{\\it "; }
"</i>"  { if (pr) cout << "}"; }
"<b>"   { if (pr) cout << "{\\bf "; }
"</b>"  { if (pr) cout << "}"; }
"<code>" { pre=1; if (pr) cout << "\\verb?"; }
"</code>"  { pre=0; if (pr) cout << "?"; }
"<ref>" { if (pr) cout << "\\ref{"; }
"</ref>" { if (pr) cout << "}"; }
"<cite>" { if (pr) cout << "\\cite{"; }
"</cite>" { if (pr) cout << "}"; }
"<address>" { ; }
"</address>" { ; }
"<footnote>" { if (pr) cout << "\\footnote{"; }
"</footnote>" { if (pr) cout << "}"; }
"<caption>" { if (pr) cout << "\\caption{"; }
"</caption>" { if (pr) cout << "}"; }
"<footnotetext>" { if (pr) cout << "\\footnotetext{"; }
"</footnotetext>" { if (pr) cout << "}"; }
"<abstract>" { if (pr) cout << "\\begin{abstract}"; }
"</abstract>" { if (pr) cout << "\\end{abstract}"; }
"<figure>" { fig=1; if (pr) cout << "\n\\begin{figure}[htb]"; }
"</figure>" { fig=0; if (pr) cout << "\\end{figure}\n"; }
"<center>" { if (pr) cout << "\\begin{center}"; }
"</center>" { if (pr) cout << "\\end{center}"; }
"<bibliography>"|"<references>" {
	refs=1;
	if (pr) cout << "\\begin{thebibliography}{WWMD91}";
	}
"</bibliography>"|"</references>" {
	if (pr) cout << "\\end{thebibliography}";
	refs=0;
	}
"&lt"(";")? { if (pr) cout << "<"; }
"&gt"(";")? { if (pr) cout << ">"; }
"&amp"(";")? { if (pr) cout << "\\&"; }
"&euml"(";")? { if (pr) cout << "\\\"{e}"; }
"\\_" { if (pr) cout << yytext; }
"_" { if (pr) {
	if (pre || math) cout << "_";
	else cout << "\\_";
	}
   }
\n { if (pr) cout << endl; }
\t { if (pr) cout << "        "; }
. { if (pr) cout << yytext[0]; }
%%

void main(int argc, char* argv[]) { yylex(); }

