# Scot score editor
# written by jrvosse@cs.vu.nl

proc start {} {
	global ed
	set ed [frame .ed]
	pack [ editor $ed ]
}
 
proc init {} {
	global hymne_library
	global old_text old_file scriptfile helpfile
	global info_bm 
	set info_bm  /usr/lib/X11/include/bitmaps/xm_information
	set helpfile "$hymne_library/demo/help.txt"

	set old_text(score) "";		set old_text(orch) ""
	set old_file(score) "";		set old_file(orch) "" 
	set scriptfile(score) ""; 	set scriptfile(orch) ""
	set_options
}

proc set_options {} {
	option add *Text.font  \
		"-adobe-courier-*-r-*-*-16-*-*-*-*-*-*-*"
}

# play the script in the text widget t
proc play_script {t} { play [$t get 1.0 end] }

# play the current selection
proc play_select {} {
	if (![string compare [selection own] ""]) {
		cmd_mesg "Error: no selection made"
	} else { set s [selection get]; play $s }
}

# load a file in the text widget w (type is score or orch)
proc load_script {w type filename} {
	global old_text
	global old_file
	global scriptfile

	if (![file readable "$filename"]) then {
	if {[confirm "Error: $filename not readable. Choose new file?"]} {
		"fchooser Load load_script $type $w"
	}
	} else {
		set old_text($type) [$w.script.text get 1.0 end]
		set old_file($type) $scriptfile($type)
		set scriptfile($type) $filename
		$w.script.name configure -text "$type: $filename"

		set fid [open $filename r]
		set t [read -nonewline  $fid]
		close $fid
		
		$w.script.text delete 1.0 end 
		$w.script.text insert end $t
		$w.script.text mark set insert 1.0
	}
}

proc save_script {w type filename} {
	if ([file exists "$filename"]) then {
		if {![confirm "Warning: $filename exists and will be overwritten"]} {
			"fchooser Save save_script $type $w"
		} else {
			set fid [open $filename w]
			puts $fid [$w.script.text get 1.0 end]
			close $fid
		}
	} else {
		set fid [open $filename w]
		puts $fid [$w.script.text get 1.0 end]
		close $fid
	}
}

proc fchooser {title func w type {suggest ""}} {
	puts "fchooser $title $func $w $type $suggest"
	toplevel .tl
	grab .tl
	wm title .tl "$title File"
	filechooser .tl.fc
	bind .tl.fc.f.e <Key-Return> { .tl.fc.f.b invoke }
	bind .tl.fc.f.e <Enter> 	 { focus .tl.fc.f.e }
	.tl.fc.f.e insert end $suggest
	.tl.fc.f.c configure -command "destroy .tl"
	.tl.fc.f.b configure -command {
		set filename [.tl.fc get]
		if ([string compare $filename ""]) {
			destroy .tl
			$function $widget $script_type $filename
			unset function widget script_type
		}
	}
	pack append .tl .tl.fc top
	global function widget script_type
	set function $func
	set script_type $type
	set widget $w
	dpos .tl
}

proc confirm {mesg} {
	toplevel .tl
	grab .tl
	wm title .tl "confirm"
	global confirm_result
	message .tl.message -text $mesg -relief raised -width 150
	button  .tl.yes -text OK -command { destroy .tl; set confirm_result 1}
	button  .tl.no -text Cancel -command { destroy .tl; set confirm_result 0}
	pack append .tl \
		.tl.message {top} \
	 	.tl.yes {left fillx expand}   \
		.tl.no  {right fillx expand}
	dpos .tl
	tkwait window .tl
	return $confirm_result
}
	
proc undo {w type} {
	global old_text
	global old_file
	global scriptfile

	set t $old_file($type)
	set old_file($type) $scriptfile($type)
	set scriptfile($type) $t
	$w.script.name configure -text "$type: $scriptfile($type)"

	set t $old_text($type)
	set old_text($type) [$w.script.text get 1.0 end]
	$w.script.text delete 1.0 end 
	$w.script.text insert end $t
}

proc clear_script {w type} {
	global scriptfile
	global old_text old_file
	set old_text($type)  [$w.script.text get 1.0 end]
	set old_file($type) $scriptfile($type)
	$w.script.text delete 1.0 end 
	$w.script.name configure -text "$type: Cleared"
	set scriptfile($type) ""	
}

	
proc cmd_mesg mesg_text {
	toplevel .tl
	grab .tl
	dpos .tl
	wm title .tl Info
	wm minsize .tl 100 100
	message .tl.m -text "$mesg_text" -relief raised 
	button  .tl.ok -text OK -relief raised -padx 10 -pady 5 \
			-command {destroy .tl}
	pack append .tl .tl.m top .tl.ok fillx

}

proc cmd_help {} {
	global helpfile 
	if (![file readable $helpfile]) then {
		cmd_mesg "Error. Info file $helpfile not readable"
	} else {
		set fid [open $helpfile r]
		set t [read -nonewline  $fid]
		close $fid
		cmd_mesg $t
	}
}

proc dpos w { wm geometry $w "+100+100" }

proc score_gui {w} {
	global info_bm
	global scriptfile

	frame $w
	# buttons
	set f [frame $w.buttons]
	button $f.info -command cmd_help
	if ([file readable "$info_bm"]) then { 
		$f.info configure -bitmap @$info_bm -width 20 
	} else { 
		$f.info configure -text Info 
	}

	button $f.quit 	-text {Quit} -command exit
	button $f.reset -text {Stop/Reset} 	-command "
		$f.trace invoke
		play -reset; audio -reset
		$f.trace invoke
	"
	button $f.script -text {Play Score} -command "play_script $w.script.text"
	button $f.select -text {Play X-Sel} -command {play_select}
	button $f.clear -text {Clear} -command "clear_script $w score"
	button $f.undo 	-text {Undo} -command "undo $w score"
	button $f.load 	-text {Load} -command "
		fchooser Load load_script $w score \$scriptfile(score) "
	button $f.save 	-text {Save} -command "
		fchooser Save save_script $w score $scriptfile(score) "
	checkbutton $f.trace -var Trace -text Trace -command {
	if {($Trace ==1)} then {play -trace
		} else {play -notrace}
}

	pack append $f \
	$f.script {left fillx expand} \
		$f.select {left fillx expand} \
		$f.reset {left fillx expand} \
		$f.clear {left} \
		$f.undo  {left} \
		$f.load  {left} \
		$f.save  {left} \
		$f.trace {left} \
		$f.info  {right} \
		$f.quit  {right}
	
	# text window  for script
	frame $w.script
	label $w.script.name -text {Score: Not Loaded} -anchor w -width 80 
	text  $w.script.text \
		-relief sunken \
		-setgrid true \
		-borderwidth 10 \
		-width 80 \
		-height 10\
		-yscroll "$w.script.scroll set" 
	scrollbar $w.script.scroll -command "$w.script.text yview"

	pack append $w.script \
		$w.script.name {top frame w} \
		$w.script.text {left fill expand}\
		$w.script.scroll {right fill}

	pack append $w \
		$w.script {fill expand}\
		$f {top fillx}
	
	return $w
}

proc orch_gui {w} {
	frame $w
	# text window  for orch
	frame $w.script
	label $w.script.name -text {Orchestra: Not Loaded} -anchor w -width 80
	text  $w.script.text \
		-relief sunken \
		-setgrid true \
		-borderwidth 10 \
		-width 80\
		-height 10 \
		-yscroll "$w.script.yscroll set"
	scrollbar $w.script.yscroll -command "$w.script.text yview"

	pack append $w.script \
	$w.script.name {top frame w} \
	$w.script.text {left fill expand}\
	$w.script.yscroll {right fill}

	# buttons
	set f [frame $w.buttons]
	button $f.set_orch -text {Compile orchestra} \
		-command {play -orch $scriptfile(orch)}
	button $f.clear -text {Clear} -command "clear_script $w orch"
	button $f.undo 	-text {Undo} -command "undo $w orch"
	button $f.load 	-text {Load file} -command "
		fchooser Load load_script $w orch \$scriptfile(orch) "
	button $f.save 	-text {Save file } -command "
		fchooser Save save_script $w orch \$scriptfile(orch) "
	
	pack append $f \
		$f.set_orch {left fillx expand} \
		$f.clear {left} \
		$f.undo  {left} \
		$f.load  {left} \
		$f.save  {left}
	
	
	pack append $w \
		$w.script {top fill expand} \
		$f {fillx}
	
	return $w
}

proc editor {ed} {
	global hymne
	init
	pack [orch_gui $ed.orch_edit]
	load_script $ed.orch_edit orch $hymne(orch_lib)/default.orc
	
	pack [score_gui $ed.score_edit]
	load_script $ed.score_edit score $hymne(score_lib)/stairway.scot
	focus default $ed.score_edit	
	return $ed
}

pack [ editor [frame $this.ed ] ]
