<html><head><title></title></head>
<body bgcolor=#ffffff>
 
<a href="cover.html">[Top]</a> <a href="corb2prb.html">[Prev]</a> <a href="arch2.html">[Next]</a> <a href="corb2pra.html">[Bottom]</a>
<hr><br>
 
<a name="446107">
<h1> The Object Model</h1>
</a><dl>
<p align=right clear=all>
<b><font size=7>
1 
</font></b>
</p>
<hr></dl>
<a name="446121">
This chapter describes the concrete object model that underlies the CORBA architecture.  The model is derived from the abstract Core Object Model defined by the Object Management Group in the <em>Object Management Architecture Guide</em>. (Information about the <em>OMA</em> <em>Guide</em> and other books in the CORBA documentation set is provided in this document's preface.)<p>
</a><a name="445893">
<h1>	1.1	 Overview</h1>
</a><a name="445894">
The object model provides an organized presentation of object concepts and terminology. It defines a partial model for computation that embodies the key characteristics of objects as realized by the submitted technologies. The OMG object model is <em>abstract</em> in that it is not directly realized by any particular technology. The model described here is a <em>concrete</em> object model. A concrete object model may differ from the abstract object model in several ways:<p>
</a><ul><a name="445896">
<li>It may <em>elaborate</em> the abstract object model by making it more specific, for example, by defining the form of request parameters or the language used to specify types
</a><a name="445897">
<li>It may <em>populate</em> the model by introducing specific instances of entities defined by the model, for example, specific objects, specific operations, or specific types
</a><a name="445898">
<li>It may <em>restrict</em> the model by eliminating entities or placing additional restrictions on their use
</a></ul><a name="445899">
An object system is a collection of objects that isolates the requestors of services (clients) from the providers of services by a well-defined encapsulating interface. In particular, clients are isolated from the implementations of services as data representations and executable code.<p>
</a><a name="445900">
The object model first describes concepts that are meaningful to clients, including such concepts as object creation and identity, requests and operations, types and signatures. It then describes concepts related to object implementations, including such concepts as methods, execution engines, and activation.<p>
</a><a name="445901">
The object model is most specific and prescriptive in defining concepts meaningful to clients. The discussion of object implementation is more suggestive, with the intent of allowing maximal freedom for different object technologies to provide different ways of implementing objects. <p>
</a><a name="446253">
There are some other characteristics of object systems that are outside the scope of the object model. Some of these concepts are aspects of application architecture, some are associated with specific domains to which object technology is applied. Such concepts are more properly dealt with in an architectural reference model. Examples of excluded concepts are compound objects, links, copying of objects, change management, and transactions. Also outside the scope of the object model is the model of control and execution.<p>
</a><a name="445906">
This object model is an example of a <em>classical object model</em>, where a client sends a message to an object. Conceptually, the object interprets the message to decide what service to perform. In the classical model, a message identifies an object and zero or more actual parameters. As in most classical object models, a distinguished first parameter is required, which identifies the operation to be performed; the interpretation of the message by the object involves selecting a method based on the specified operation. Operationally, of course, method selection could be performed either by the object or the ORB.<p>
</a><a name="445907">
<h1>	1.2	 Object Semantics</h1>
</a><a name="445908">
An object system provides services to clients. A <em>client</em> of a service is any entity capable of requesting the service.<p>
</a><a name="445909">
This section defines the concepts associated with object semantics, that is, the concepts relevant to clients.<p>
</a><a name="445910">
<h2>	1.2.1	 Objects</h2>
</a><a name="445911">
An object system includes entities known as objects. An <em>object</em> is an identifiable, encapsulated entity that provides one or more services that can be requested by a client.<p>
</a><a name="445912">
<h2>	1.2.2	 Requests</h2>
</a><a name="445913">
Clients request services by issuing requests. A <em>request</em> is an event, i.e. something that occurs at a particular time. The information associated with a request consists of an operation, a target object, zero or more (actual) parameters, and an optional request context.<p>
</a><a name="445914">
A <em>request form</em> is a description or pattern that can be evaluated or performed multiple times to cause the issuing of requests. As described in the OMG IDL Syntax and Semantics chapter,  request forms are defined by particular language bindings. An alternative request form consists of calls to the dynamic invocation interface to create an invocation structure, add arguments to the invocation structure, and to issue the invocation  (Refer to the C Language Mapping chapter and the Dynamic Invocation Interface chapter for descriptions of these request forms). <p>
</a><a name="445927">
A <em>value</em> is anything that may be a legitimate (actual) parameter in a request. A value may identify an object, for the purpose of performing the request. A value that identifies an object is called an <em>object name</em>. More particularly, a value is an instance of an OMG IDL datatype.<p>
</a><a name="445931">
An <em>object</em> <em>reference </em>is an object name that reliably denotes a particular object. Specifically, an object reference will identify the same object each time the reference is used in a request (subject to certain pragmatic limits of space and time). An object may be denoted by multiple, distinct object references. <p>
</a><a name="445932">
A request may have parameters that are used to pass data to the target object; it may also have a request context which provides additional information about the request. <p>
</a><a name="445933">
A request causes a service to be performed on behalf of the client. One outcome of performing a service is returning to the client the results, if any, defined for the request. <p>
</a><a name="445934">
If an abnormal condition occurs during the performance of a request, an exception is returned. The exception may carry additional return parameters particular to that exception.<p>
</a><a name="445954">
The request parameters are identified by position. A parameter may be an input parameter, an output parameter, or an input-output parameter. A request may also return a single <em>result value</em>, as well as any output parameters.<p>
</a><a name="445955">
The following semantics hold for all requests:<p>
</a><ul><a name="445956">
<li>Any aliasing of parameter values is neither guaranteed removed nor guaranteed to be preserved
</a><a name="445957">
<li>The order in which aliased output parameters are written is not guaranteed
</a><a name="445958">
<li>Any output parameters are undefined if an exception is returned
</a><a name="445959">
<li>The values that can be returned in an input-output parameter may be constrained by the value that was input
</a></ul><a name="445960">
Descriptions of the values and exceptions that are permitted, see  <a href="obmod2.html#445970">Types on page 1-4</a> and <a href="obmod2.html#446041">Exceptions on page 1-6</a>. <p>
</a><a name="445967">
<h2>	1.2.3	 Object Creation and Destruction</h2>
</a><a name="445968">
Objects can be created and destroyed. From a client's point of view, there is no special mechanism for creating or destroying an object. Objects are created and destroyed as an outcome of issuing requests. The outcome of object creation is revealed to the client in the form of an object reference that denotes the new object.<p>
</a><a name="445970">
<h2>	1.2.4	 Types</h2>
</a><a name="445971">
A <em>type</em> is an identifiable entity with an associated predicate (a single-argument mathematical function with a boolean result) defined over values. A value <em>satisfies</em> a type if the predicate is true for that value. A value that satisfies a type is called a <em>member of the type</em>.<p>
</a><a name="445972">
Types are used in signatures to restrict a possible parameter or to characterize a possible result.<p>
</a><a name="445973">
The <em>extension of a type</em> is the set of values that satisfy the type at any particular time.<p>
</a><a name="445974">
An <em>object type</em> is a type whose members are objects (literally, values that identify objects). In other words, an object type is satisfied only by (values that identify) objects.<p>
</a><a name="445975">
Constraints on the data types in this model are shown in this section.<p>
</a><a name="445976">
<strong>Basic types</strong>:<p>
</a><ul><a name="445977">
<li>16-bit and 32-bit signed and unsigned 2's complement integers
</a><a name="445978">
<li>32-bit and 64-bit IEEE floating point numbers
</a><a name="445979">
<li>Characters, as defined in ISO Latin-1 (8859.1) 
</a><a name="445980">
<li>A boolean type taking the values TRUE and FALSE
</a><a name="445981">
<li>An 8-bit opaque datatype, guaranteed to <em>not</em> undergo any conversion during transfer between systems
</a><a name="445982">
<li>Enumerated types consisting of ordered sequences of identifiers
</a><a name="445983">
<li>A string type which consists of a variable-length array of characters; the length of the string is available at runtime
</a><a name="445984">
<li>A type "any" which can represent any possible basic or constructed type
</a></ul><a name="445985">
Constructed types:<p>
</a><ul><a name="445986">
<li>A record type (called struct), consisting of an ordered set of (name,value) pairs
</a><a name="445987">
<li>A discriminated union type, consisting of a discriminator followed by an instance of a type appropriate to the discriminator value
</a><a name="445988">
<li>A sequence type which consists of a variable-length array of a single type; the length of the sequence is available at runtime
</a><a name="445989">
<li>An array type which consists of a fixed-length array of a single type
</a><a name="445990">
<li>An interface type, which specifies the set of operations which an instance of that type must support
</a></ul><a name="446344">
Values in a request are restricted to values that  satisfy these type constraints. The legal values are shown in <a href="obmod2.html#446352">FIG. 1 on page 1-5</a>. No particular representation for values is defined.<p>
</a><a name="446352">
<slide id=obmod-fig-1 cap="legal Values">
<image src="local/images/obmod2a.gif">
<p>
FIG.  1	 Legal Values (does not show in slide mode)
</slide>
</a><a name="446353">
<p>
</a><a name="446012">
<h2>	1.2.5	 Interfaces</h2>
</a><a name="446013">
An <em>interface</em> is a description of a set of possible operations that a client may request of an object. An object <em>satisfies</em> an interface if it can be specified as the target object in each potential request described by the interface.<p>
</a><a name="446014">
An <em>interface type</em> is a type that is satisfied by any object (literally, any value that identifies an object) that satisfies a particular interface.<p>
</a><a name="446015">
Interfaces are specified in OMG IDL. Interface inheritance provides the composition mechanism for permitting an object to support multiple interfaces. The <em>principal interface</em> is simply the most-specific interface that the object supports, and consists of all operations in the transitive closure of the interface inheritance graph.<p>
</a><a name="446016">
<h2>	1.2.6	 Operations</h2>
</a><a name="446017">
An <em>operation</em> is an identifiable entity that denotes a service that can be requested.<p>
</a><a name="446018">
An operation is identified by an <em>operation identifier</em>. An operation is not a value. <p>
</a><a name="446019">
An operation has a signature that describes the legitimate values of request parameters and returned results. In particular, a <em>signature</em> consists of:<p>
</a><ul><a name="446020">
<li> A specification of the parameters required in requests for that operation
</a><a name="446021">
<li>A specification of the result of the operation
</a><a name="446022">
<li>A specification of the exceptions that may be raised by a request for the operation and the types of the parameters accompanying them
</a><a name="446023">
<li>A specification of additional contextual information that may affect the request
</a><a name="446024">
<li>An indication of the execution semantics the client should expect from a request for the operation
</a></ul><a name="446025">
Operations are (potentially) <em>generic</em>, meaning that a single operation can be uniformly requested on objects with different implementations, possibly resulting in observably different behavior. Genericity is achieved in this model via interface inheritance in IDL and the total decoupling of implementation from interface specification.<p>
</a><a name="446026">
The general form for an operation signature is:<p>
</a><a name="446027">
 [oneway] &lt;op_type_spec&gt; &lt;identifier&gt; (param1, ..., paramL)<p>
</a><a name="446028">
 	 					[raises(except1,...,exceptN)] [context(name1, ..., nameM)]<p>
</a><a name="446029">
 where:<p>
</a><ul><a name="446030">
<li>The optional <strong>oneway</strong> keyword indicates that best-effort semantics are expected of requests for this operation; the default semantics are exactly-once if the operation successfully returns results or at-most-once if an exception is returned
</a><a name="446031">
<li>The <strong>&lt;op_type_spec&gt;</strong> is the type of the return result
</a><a name="446032">
<li>The <strong>&lt;identifier&gt;</strong> provides a name for the operation in the interface. 
</a><a name="446033">
<li>The operation parameters needed for the operation; they are flagged with the modifiers <strong>in</strong>,<strong> out</strong>, or <strong>inout</strong> to indicate the direction in which the information flows (with respect to the object performing the request)
</a><a name="446034">
<li>The optional <strong>raises</strong> expression indicates which user-defined exceptions can be signalled to terminate a request for this operation; if such an expression is not provided, no user-defined exceptions will be signalled
</a><a name="446035">
<li>The optional <strong>context</strong> expression indicates which request context information will be available to the object implementation; no other contextual information is required to be transported with the request
</a></ul><a name="446036">
<h3>		 Parameters</h3>
</a><a name="446037">
A parameter is characterized by its mode and its type. The <em>mode</em> indicates whether the value should be passed from client to server (<strong>in</strong>), from server to client (<strong>out</strong>), or both (<strong>inout</strong>). The parameter's type constrains the possible value which may be passed in the directions dictated by the mode.<p>
</a><a name="446038">
<h3>		 Return Result</h3>
</a><a name="446039">
The return result is a distinguished <strong>out</strong> parameter.<p>
</a><a name="446041">
<h3>		 Exceptions</h3>
</a><a name="446042">
An <em>exception</em> is an indication that an operation request was not performed successfully. An exception may be accompanied by additional, exception-specific information.<p>
</a><a name="446043">
The additional, exception-specific information is a specialized form of record. As a record, it may consist of any of the types described in <a href="obmod2.html#445970"> Section 1.2.4</a>.<p>
</a><a name="446050">
All signatures implicitly include the standard exceptions described in <a href="idlsyn.html#690">Section &#32;3.15, "Standard Exceptions," on page &#32;3-33</a>.<p>
</a><a name="446512">
<h3>		 Contexts</h3>
</a><a name="446052">
A <em>request context</em> provides additional, operation-specific information that may affect the performance of a request.<p>
</a><a name="446053">
<h3>		 Execution Semantics</h3>
</a><a name="446054">
Two styles of execution semantics are defined by the object model:<p>
</a><ul><a name="446055">
<li>At-most-once: if an operation request returns successfully, it was performed exactly once; if it returns an exception indication, it was performed at-most-once;
</a><a name="446056">
<li>Best-effort: a best-effort operation is a request-only operation, i.e. it cannot return any results and the requester never synchronizes with the completion, if any, of the request.
</a></ul><a name="446057">
The execution semantics to be expected is associated with an operation. This prevents a client and object implementation from assuming different execution semantics.<p>
</a><a name="446058">
Note that a client is able to invoke an at-most-once operation in a synchronous or deferred-synchronous manner.<p>
</a><a name="446059">
<h2>	1.2.7	 Attributes</h2>
</a><a name="446060">
An interface may have attributes. An attribute is logically equivalent to declaring a pair of accessor functions: one to retrieve the value of the attribute and one to set the value of the attribute.<p>
</a><a name="446061">
An attribute may be read-only, in which case only the retrieval accessor function is defined.<p>
</a><a name="446062">
<h1>	1.3	 Object Implementation</h1>
</a><a name="446063">
This section defines the concepts associated with object implementation, i.e. the concepts relevant to realizing the behavior of objects in a computational system.<p>
</a><a name="446064">
The implementation of an object system carries out the computational activities needed to effect the behavior of requested services. These activities may include computing the result of the request and updating the system state. In the process, additional requests may be issued.<p>
</a><a name="446065">
The implementation model consists of two parts: the execution model and the construction model. The execution model describes how services are performed. The construction model describes how services are defined.<p>
</a><a name="446066">
<h2>	1.3.1	 The Execution Model: Performing Services</h2>
</a><a name="446067">
A requested service is performed in a computational system by executing code that operates upon some data. The data represents a component of the state of the computational system. The code performs the requested service, which may change the state of the system.<p>
</a><a name="446068">
Code that is executed to perform a service is called a <em>method</em>. A method is an immutable description of a computation that can be interpreted by an execution engine. A method has an immutable attribute called a <em>method format</em> that defines the set of execution engines that can interpret the method. An <em>execution engine</em> is an abstract machine (not a program) that can interpret methods of certain formats, causing the described computations to be performed. An execution engine defines a dynamic context for the execution of a method. The execution of a method is called a <em>method activation</em>.<p>
</a><a name="446069">
When a client issues a request, a method of the target object is called. The input parameters passed by the requestor are passed to the method and the output parameters and return value (or exception and its parameters) are passed back to the requestor.<p>
</a><a name="446070">
Performing a requested service causes a method to execute that may operate upon an object's persistent state. If the persistent form of the method or state is not accessible to the execution engine, it may be necessary to first copy the method or state into an execution context. This process is called <em>activation</em>; the reverse process is called <em>deactivation</em>.<p>
</a><a name="446071">
<h2>	1.3.2	 The Construction Model</h2>
</a><a name="446072">
A computational object system must provide mechanisms for realizing behavior of requests. These mechanisms include definitions of object state, definitions of methods, and definitions of how the object infrastructure is to select the methods to execute and to select the relevant portions of object state to be made accessible to the methods. Mechanisms must also be provided to describe the concrete actions associated with object creation, such as association of the new object with appropriate methods.<p>
</a><a name="446073">
An <em>object implementation</em>-or <em>implementation</em>, for short-is a definition that provides the information needed to create an object and to allow the object to participate in providing an appropriate set of services. An implementation typically includes, among other things, definitions of the methods that operate upon the state of an object. It also typically includes information about the intended type of the object.<p>
</a><a name="446518">
<p>
</a>
<hr><br>
 
<a href="cover.html">[Top]</a> <a href="corb2prb.html">[Prev]</a> <a href="arch2.html">[Next]</a> <a href="corb2pra.html">[Bottom]</a>
<hr><br>
 


<address>
<a href="mailto:pubs@omg.org">pubs@omg.org</a>
</address>

<i>Copyright &#169; 1995, <a href="http://www.omg.org">Object Management Group.</a>  All rights
reserved.</i>

<!-- This file was created with Quadralay WebWorks Publisher 3.0.4 -->
<!-- -->
<!-- For more information on how this document, and how the rest of -->
<!-- this server was created, email pubs@omg.org -->
<!-- -->
<!-- Last updated: 06/10/96 08:47:24 -->

</body>
</html>
