<html><head><title></title></head>
<body bgcolor=#ffffff>
 
<a href="cover.html">[Top]</a> <a href="pluspo.html">[Prev]</a> <a href="plusdef.html">[Next]</a> <a href="corb2pra.html">[Bottom]</a>
<hr><br>
 
<a name="99">
<h1> Server-Side Mapping</h1>
</a><dl>
<p align=right clear=all>
<b><font size=7>
18 
</font></b>
</p>
<hr><a name="77">
 <img src="local/images/pluserva.gif">Server-side mapping refers to the portability constraints for an object implementation written in C++. The term <em>server</em> is not meant to restrict implementations to situations in which method invocations cross address space or machine boundaries. This mapping addresses any implementation of an OMG IDL interface.<p>
</a><a name="141">
<img src="local/images/plusera1.gif">The required functionality for a server described here is probably a subset of the functionality an implementor will actually need. As a consequence, in practice, few servers will be completely compliant. However, we expect most of the server code to be portable from one ORB implementation to another. In particular, the body of an operation implementation will usually comply with this mapping.<p>
</a></dl>
<a name="157">
<h1>	18.1	 Implementing Interfaces</h1>
</a><a name="159">
 <img src="local/images/plusera2.gif">To define an implementation in C++, one defines a C++ class with any valid C++ name. For each operation in the interface, the class defines a non-static member function with the mapped name of the operation (the mapped name is the same as the OMG IDL identifier except when the identifier is a C++ keyword, in which case an underscore (`_') is prepended to the identifier, as noted in <a href="pluscon.html#512697">Section &#32;16.1, "Preliminary Information," on page &#32;16-1</a>). Note that the ORB implementation may allow one implementation class to derive from another, so the statement "the class defines a member function" does not mean the class must explicitly define the member function-it could inherit the function.<p>
</a><a name="145">
<img src="local/images/plusera3.gif">The mapping does not specify how the implementation class is related to any other classes, including the generated class for the interface. This approach allows implementations to use either inheritance or delegation and to include other features from the ORB implementation (such as choosing a default transport representation). The examples in this chapter provide sample solutions for defining implementation classes. CORBA-compliant implementations are not required to use these alternatives.<p>
</a><a name="146">
<h1>	18.2	 Implementing Operations</h1>
</a><a name="162">
 <img src="local/images/plusera4.gif">The signature of an implementation member function is the mapped signature of the OMG IDL operation. Unlike the client side, the server-side mapping requires that the function header include the appropriate exception (throw) specification. This requirement allows the compiler to detect when an invalid exception is raised, which is necessary in the case of a local C++-to-C++ library call (otherwise the call would have to go through a wrapper that checked for a valid exception). For example:<p>
</a><a name="163">
<blockquote>// IDL<br>interface A<br>{<br>						exception B {};<br>						void f() raises(B);<br>};</blockquote>
</a><a name="164">
<blockquote></blockquote>
</a><a name="165">
 // C++<br>class MyFavoriteImplementationOfA ...<br>{<br> &#32; &#32;public:<br>	class B : public UserException {};<br>	void f() throw(B);<br>	...<br>};<p>
</a><a name="147">
<img src="local/images/plusera5.gif">The mapping provides two operations that are accessible from within the body of a member function: _this() and _boa(). The _this() function returns an object reference (T_ptr) for the target object. The _boa() function returns a BOA_ptr to the appropriate BOA object. The implementation may not assume where the _boa() function is defined, only that it is available within the member function. The _boa() function could be a member function, a static member function, or a static function defined in a namespace that is accessible from the member functions of the implementation. The return values of _this() and _boa() must be released via CORBA::release().<p>
</a><a name="148">
<img src="local/images/plusera6.gif">Within a member function, the "this" pointer refers to the implementation object's data as defined by the class. In addition to accessing the data, a member function may implicitly call another member function defined by the same class. For example:<p>
</a><a name="149">
<blockquote>// IDL</blockquote>
</a><a name="150">
<blockquote>interface A<br>{<br>						void f();<br>						void g();<br>};</blockquote>
</a><a name="152">
<blockquote></blockquote>
</a><a name="151">
 // C++<br>class MyFavoriteImplementationOfA ...<br>{<br> &#32; &#32;public:<br>	void f();<br>	void g();<br> &#32; &#32;private:<br>	long x_;<br>};<br><br>void MyFavoriteImplementationOfA::f()<br>{<br>	x_ = 3;<br>	g();<br>}<p>
</a><a name="161">
<h1>	18.3	 Examples</h1>
</a><a name="166">
 <img src="local/images/plusera7.gif">As with other examples shown in this mapping, the following examples are not meant to mandate a particular implementation. Rather, they show some of the implementations that are possible in order to help clarify the descriptions of the mapping.<p>
</a><a name="79">
<h2>	18.3.1	 Using C++ Inheritance for Interface Implementation</h2>
</a><a name="82">
 <img src="local/images/plusera8.gif">Implementation classes can be derived from a generated base class based on the OMG IDL interface definition. The generated base classes are known as <em>skeleton classes</em>, and the derived classes are known as <em>implementation classes</em>. Each operation of the interface has a corresponding virtual member function declared in the skeleton class. The signature of the member function is identical to that of the generated client stub class. The implementation class provides implementations for these member functions. The BOA invokes the methods via calls to the skeleton class's virtual functions.<p>
</a><a name="80">
<img src="local/images/plusera9.gif">The following OMG IDL interface will be used in all the examples in this section:<p>
</a><a name="588">
<blockquote>// IDL<br>interface A<br>{<br>						short op1();<br>						void op2(in long l);<br>};</blockquote>
</a><a name="589">
<img src="local/images/plusea10.gif">An IDL compiler generates an interface class A for this interface. This class contains the C++ definitions for the typedefs, constants, exceptions, attributes, and operations in the OMG IDL interface. It has a form similar to the following:<p>
</a><a name="83">
 // C++<br>class A : public virtual CORBA::Object<br>{<br> &#32; &#32;public:<br>	virtual Short op1() = 0;<br>	virtual void op2(Long l) = 0;<br>	...<br>};<p>
</a><a name="84">
<img src="local/images/plusea11.gif">Some ORB implementations might not use public virtual inheritance from CORBA::Object, and might not make the operations pure virtual, but the signatures of the operations will be the same.<p>
</a><a name="86">
<img src="local/images/plusea12.gif">On the server side, a skeleton class can be generated. This class is partially opaque to the programmer, though it will contain a member function corresponding to each operation in the interface.<p>
</a><a name="89">
 // C++<br>class _sk_A : public A<br>{<br> &#32; &#32;public:<br>	// ...server-side implementation-specific detail<br>	// goes here...<br>	virtual Short op1() = 0;<p>
</a><a name="91">
 	virtual void op2(Long l) = 0;<br>	...<br>};<p>
</a><a name="142">
<img src="local/images/plusea13.gif">To implement this interface, a programmer must derive from this skeleton class and implement each of the operations in the OMG IDL interface. An implementation class declaration for interface A would take the form:<p>
</a><a name="271">
 // C++<br>class A_impl : public _sk_A<br>{<br> &#32; &#32;public:<br>	Short op1();<p>
</a><a name="272">
 	void op2(Long l);<br>	...<br>};<p>
</a><a name="94">
<h2>	18.3.2	 Using Delegation for Interface Implementation</h2>
</a><a name="144">
 <img src=local/images/plusea14.gif">Inheritance is not always the best solution for implementing interfaces. Using inheritance from the OMG IDL-generated classes forces a C++ inheritance hierarchy on the implementor. Sometimes, the overhead of such inheritance is too high. For example, implementing OMG IDL interfaces with existing legacy code might be impossible if inheritance from some global class was enforced.<p>
</a><a name="95">
<img src=local/images/plusea15.gif">In some cases delegation can be used to good effect to solve this problem. Rather than inheriting from some global class, the implementation can be coded in any way at all, and some wrapper classes will delegate upcalls to that implementation. This section describes how this can be achieved in a type-safe manner using C++ templates.<p>
</a><a name="97">
<img src=local/images/plusea16.gif">For the examples in this section, the OMG IDL interface from <a href="pluserv.html#79"> Section 18.3.1</a> will again be used:<p>
</a><a name="100">
<blockquote>// IDL<br>interface A<br>{<br>						short op1();<br>						void op2(in long l);<br>};</blockquote>
</a><a name="120">
<img src=local/images/plusea17.gif">An IDL compiler will generate a (possibly abstract) class A in C++ defining this interface.<p>
</a><a name="121">
<img src=local/images/plusea18.gif">Normally, the server implementor will have to derive from this class or some related class to implement a server-side object. However, the an IDL compiler could generate another class, called a <em>tie. </em>This class is partially opaque to the application programmer, though like the skeleton, it provides a method corresponding to each OMG IDL operation.<p>
</a><a name="122">
 // C++<br>template &lt;class T&gt;<br>class _tie_A : public A<br>{<br> &#32; &#32;public:<br>	_tie_A(T &amp;t);<br>	Short op1();<br>	void op2(Long l);<br>	...<br>};<p>
</a><a name="123">
<img src=local/images/plusea19.gif">This class performs the task of delegation. When the template is instantiated with a class that supports the operations of A, then the _tie_A class will delegate all operations to that implementation class. When an instance of this class is created, then a reference to the actual implementation class is passed to the constructor. Typically the implementation will just call the corresponding method in the implementation class via this reference.<p>
</a><a name="124">
 // C++<br>template &lt;class T&gt;<br>class _tie_A : public A<br>{<br> &#32; &#32;public:<br>	_tie_A(T &amp;t) : _ref(t) {}<br>	Short op1() {return _ref.op1();}<br>	void op2(Long l) {_ref.op2(l);}<br><br> &#32; &#32;private:<br>	T &amp;_ref;<br>};<p>
</a><a name="1255">
<h1>	18.4	 Mapping of Dynamic Skeleton Interface to C++</h1>
</a><a name="1263">
<img src=local/images/plusea20.gif"><a href="dynskel.html#686">Section &#32;5.3, "Dynamic Skeleton Interface: Language Mapping," on page &#32;5-3</a> contains general information about mapping the Dynamic Skeleton Interface to programming languages. <p>
</a><a name="1268">
<img src=local/images/plusea21.gif">This section contains the following information:<p>
</a><ul><a name="1269">
<li>Mapping of the Dynamic Skeleton Interface's ServerRequest to C++
</a><a name="1270">
<li>Mapping of the Basic Object Adapter's Dynamic Implementation Routine to C++
</a></ul><a name="1271">
<h2>	18.4.1	 	 Mapping of ServerRequest to C++</h2>
</a><a name="1277">
<img src=local/images/plusea22.gif">The ServerRequest pseudo object maps to a C++ class in the CORBA namespace which supports the following operations and signatures:<p>
</a><a name="1278">
 // C++<br>class ServerRequest<br>{<br>  public:<br>	Identifier op_name() throw(SystemException);<br>	OperationDef_ptr op_def() throw(SystemException);<br>	Context_ptr ctx() throw(SystemException);<br>	void params(NVList_ptr parameters)<br>			throw(SystemException);<br>	void result(Any *value) throw(SystemException);<br>	void exception(Any *value) throw(SystemException);<br>};<p>
</a><a name="1283">
<img src=local/images/plusea23.gif">Note that, as with the rest of the C++ mapping, ORB implementations are free to make such operations virtual and modify the inheritance as needed.<p>
</a><a name="1288">
<img src=local/images/plusea24.gif">All of these operations follow the normal memory management rules for data passed into skeletons by the ORB. That is, the DIR is not allowed to modify or change the string returned by op_name(), <code>in</code> parameters in the NVList, or the context returned by ctx(). Similarly, data allocated by the DIR and handed to the ORB (the NVList parameters, any result value, and exception values) is freed by the ORB rather than by the DIR.<p>
</a><a name="1289">
<h2>	18.4.2	 Handling Operation Parameters and Results</h2>
</a><a name="1294">
<img src=local/images/plusea25.gif">The ServerRequest provides parameter values when the DIR invokes the params() operation. The NVList provided by the DIR to the ORB includes the TypeCodes (inside a NamedValue) for all parameters, including <code>out</code> ones (their values are null pointers at first), for the operation. This allows the ORB to verify that the correct parameter types have been provided before filling their values in, but does not require it to do so. It also relieves the ORB of all responsibility to consult the interface repository, promoting high performance implementations.<p>
</a><a name="1300">
<img src=local/images/plusea26.gif">The NVList provided to the ORB then becomes owned by the ORB. It will not be deallocated until after the DIR returns. This allows the DIR to pass the <code>out</code> values, including the return side of <code>inout</code> values, to the ORB by modifying the NVList after params() has been called.<p>
</a><a name="1305">
<img src=local/images/plusea27.gif">In order to guarantee that the ORB could always verify parameter lists, and to detect errors such as omitted parameters, Dynamic Implementation Routines are always required to call params(), even when the DIR believes that no parameters are used by the operation. When the DIR believes no parameters are used by the operation, it passes an empty NVList.<p>
</a><a name="1310">
<img src=local/images/plusea28.gif">The ServerRequest will not send a response to the invocation until the DIR returns. If a return value is required, the result() operation must be invoked to provide that value to the ORB. Where no return value is required, this need not be invoked.<p>
</a><a name="1315">
<img src=local/images/plusea29.gif">The params() and result() operations may be called only once, and in that exact order.<p>
</a><a name="1316">
<h2>	18.4.3	 Sample Usage</h2>
</a><a name="1321">
<img src=local/images/plusea30.gif">In typical use, the DIR receives an upcall. It will determine the operation signature by using op_name() to consult a private cache of <code>OperationDef</code> information. This allows it to create an NVList and fill in the TypeCodes for all the operation's parameters: the <code>in</code> values, <code>out</code> values, and <code>inout</code> values. Then the DIR calls <code>params()</code> with that NVList. At this point, the value pointers for all <code>in</code> and <code>inout</code> (the input side only) parameters in that NVList are valid.<p>
</a><a name="1326">
<img src=local/images/plusea31.gif">The DIR then performs the work for the request, using the target object reference to determine to which real object the request relates. Next, it stores the value pointers for <strong>out</strong> and <strong>inout</strong> parameters into the NVList, and reports any result() data. It then returns from the DIR upcall, signifying to the ORB that it could send any response message. Finally, the ORB frees the data allocated by the DIR (in the NVList and in the result) after it to the client.<p>
</a><a name="1327">
<h2>	18.4.4	 Reporting Exceptions</h2>
</a><a name="1332">
<img src=local/images/plusea32.gif">To report an exception, rather than provide return values, the DIR provides the exception value inside an Any, and passes that to exception(). As with result data, the data would be freed by the ORB after the DIR returns. (The DIR cannot in general throw exceptions, since in order to "throw" or "catch", C++ systems require type information that can only be generated at compile time. DSI, like DII, cannot rely on such compile-time support.)<p>
</a><a name="1337">
<img src=local/images/plusea33.gif">All exceptions are presented as values embedded in an Any. This is required since the use of C++ catch/throw for user-defined exceptions relies on data generated by a C++ compiler, which will not be available to general bridges (which are constructed without any OMG IDL compiler support).<p>
</a><a name="1342">
<img src=local/images/plusea34.gif">The exception() routine can be called only once, after params() is called. It may not be called if <code>result()</code> has been called.<p>
</a><a name="1343">
<h2>	18.4.5	 Mapping of BOA's Dynamic Implementation Routine</h2>
</a><a name="1349">
<img src=local/images/plusea35.gif">C++ server side mappings, implementation objects are C++ objects. To use the DSI, an object implements a class in the BOA namespace that has a single member function with the following signature:<p>
</a><a name="1350">
 // C++<br>class DynamicImplementation<br>{<br>  public:<br>	virtual void invoke(<br>			CORBA::ServerRequestRef						 request,<br>			CORBA::Environment							&amp;env<br>	) throw (<br>// NO exceptions... uses ServerRequest::	exception()<br>		) = 0;<br>	...<br>};<p>
</a><a name="1355">
<img src=local/images/plusea36.gif">The env parameter is used in the BOA::get_principal() operation. Note that, as with the rest of the C++ mapping, the implementation inherits this interface, and may support other methods as well.<p>
</a><a name="1360">
<img src=local/images/plusea37.gif">As with other C++ based operation implementations, two functions are accessible within the body of methods: _this(), returning an object reference (Object_ptr) for the target object, and _boa(), returning a BOA_ptr to the appropriate BOA. The method code may not assume where these two routines are defined.<p>
</a>
<hr><br>
 
<a href="cover.html">[Top]</a> <a href="pluspo.html">[Prev]</a> <a href="plusdef.html">[Next]</a> <a href="corb2pra.html">[Bottom]</a>
<hr><br>
 


<address>
<a href="mailto:pubs@omg.org">pubs@omg.org</a>
</address>

<i>Copyright &#169; 1995, <a href="http://www.omg.org">Object Management Group.</a>  All rights
reserved.</i>

<!-- This file was created with Quadralay WebWorks Publisher 3.0.4 -->
<!-- -->
<!-- For more information on how this document, and how the rest of -->
<!-- this server was created, email pubs@omg.org -->
<!-- -->
<!-- Last updated: 06/10/96 09:09:09 -->

</body>
</html>
