<html><head><title></title></head>
<body bgcolor=#ffffff>
 
<a href="cover.html">[Top]</a> <a href="pluscon.html">[Prev]</a> <a href="pluserv.html">[Next]</a> <a href="corb2pra.html">[Bottom]</a>
<hr><br>
 
<a name="111">
<h1>  Mapping of Pseudo Objects to C++</h1>
</a><dl>
<p align=right clear=all>
<b><font size=7>
17 
</font></b>
</p>
<hr><a name="89">
 <img src=local/images/pluspoa.gif">CORBA pseudo objects may be implemented either as normal CORBA objects or as <em>serverless objects</em>. In the CORBA specification, the fundamental differences between these strategies are:<p>
</a></dl>
<ul><a name="91">
<li>Serverless object types do not inherit from CORBA::Object
</a><a name="92">
<li>Individual serverless objects are not registered with any ORB
</a><a name="2972">
<li>Serverless objects do not necessarily follow the same memory management rules as for regular IDL types.
</a></ul><a name="93">
<img src=local/images/pluspoa1.gif">References to serverless objects are not necessarily valid across computational contexts; for example, address spaces. Instead, references to serverless objects that are passed as parameters may result in the construction of independent functionally-identical copies of objects used by receivers of these references. To support this, the otherwise hidden representational properties (such as data layout) of serverless objects are made known to the ORB. Specifications for achieving this are not contained in this chapter: making serverless objects known to the ORB is an implementation detail. <p>
</a><a name="2514">
<img src=local/images/pluspoa2.gif">This chapter provides a standard mapping algorithm for all pseudo object types. This avoids the need for piecemeal mappings for each of the nine CORBA pseudo object types, and accommodates any pseudo object types that may be proposed in future revisions of <em>CORBA.</em> It also avoids representation dependence in the C mapping while still allowing implementations that rely on C-compatible representations.<p>
</a><a name="95">
<h1>	17.1	 Usage</h1>
</a><a name="147">
 <img src=local/images/pluspoa3.gif">Rather than C-PIDL, this mapping uses an augmented form of full OMG IDL to describe serverless object types. Interfaces for pseudo object types follow the exact same rules as normal OMG IDL interfaces, with the following exceptions:<p>
</a><ul><a name="96">
<li>They are prefaced by the keyword <strong>pseudo.</strong>
</a><a name="98">
<li>Their declarations may refer to other<a href="#101"><sup>1</sup></a> serverless object types that are not otherwise necessarily allowed in OMG IDL.
</a></ul><a name="103">
<img src=local/images/pluspoa4.gif">As explained in <a href="cmap.html#288">Section &#32;14.23, "Pseudo-objects," on page &#32;14-24</a>, the <strong>pseudo</strong> prefix means that the interface may be implemented in either a normal or serverless fashion. That is, apply either the rules described in the following sections or the normal mapping rules described in <a href="pluscon.html#513080">Chapter 16, "Mapping of OMG IDL to C++"</a>.<p>
</a><a name="1354">
<h1>	17.2	 Mapping Rules</h1>
</a><a name="163">
 <img src=local/images/pluspoa5.gif">Serverless objects are mapped in the same way as normal interfaces, except for the differences outlined in this section.<p>
</a><a name="105">
<img src=local/images/pluspoa6.gif">Classes representing serverless object types are <em>not</em> subclasses of CORBA::Object, and are not necessarily subclasses of any other C++ class. Thus, they do not necessarily support, for example, the Object::create_request operation.<p>
</a><a name="2503">
<img src=local/images/pluspoa7.gif">For each class representing a serverless object type T, overloaded versions of the following functions are provided in the CORBA namespace:<p>
</a><a name="107">
 // C++<br>void release(T_ptr);<br>Boolean is_nil(T_ptr p);<p>
</a><a name="132">
<img src=local/images/pluspoa8.gif">The mapped C++ classes are not guaranteed to be usefully subclassable by users, although subclasses can be provided by implementations. Implementations are allowed to make assumptions about internal representations and transport formats that may not apply to subclasses.<p>
</a><a name="2973">
<img src=local/images/pluspoa9.gif">The member functions of classes representing serverless object types do not necessarily obey the normal memory management rules. This is due to the fact that some serverless objects, such as CORBA::NVList, are essentially just containers for several levels of other serverless objects. Requiring callers to explicitly free the values returned from accessor functions for the contained serverless objects would be counter to their intended usage.<p>
</a><a name="133">
<img src=local/images/pluspa10.gif">All other elements of the mapping are the same. In particular:<p>
</a><a name="1367">
1.	 		The types of references to serverless objects, T_ptr, may or may not simply be a typedef of T*.<p>
</a><a name="1368">
2.	 Each mapped class supports the following static member functions:<p>
</a><a name="2497">
 // C++<br>static T_ptr _duplicate(T_ptr p);<br>static T_ptr _nil();<p>
</a><a name="2498">
 Legal implementations of _duplicate include simply returning the argument or constructing references to a new instance. Individual implementations may provide stronger guarantees about behavior.<p>
</a><a name="138">
3.	 The corresponding C++ classes may or may not be directly instantiable or have other instantiation constraints. For portability, users should invoke the appropriate constructive operations. When none are listed, users cannot depend on any portable means for constructing such objects, and should consult documentation for their implementations. <p>
</a><a name="139">
4.	 As with normal interfaces, assignment operators are not supported.<p>
</a><a name="140">
5.	 Although they can transparently employ "copy-style" rather than "reference-style" mechanics, parameter passing signatures and rules as well as memory management rules are identical to those for normal objects, unless otherwise noted.<p>
</a><a name="141">
<h1>	17.3	 Relation to the C PIDL Mapping</h1>
</a><a name="219">
 <img src=local/images/pluspa11.gif">All serverless object interfaces and declarations that rely on them have direct analogs in the C mapping. The mapped C++ classes can, but need not be, implemented using representations compatible to those chosen for the C mapping. Differences between the pseudo object specifications for C-PIDL and C++ PIDL are as follows:<p>
</a><ul><a name="143">
<li>C++-PIDL calls for removal of representation dependencies through the use of interfaces rather than structs and typedefs.
</a><a name="1426">
<li>C++-PIDL calls for placement of operations on pseudo objects in their interfaces, including a few cases of redesignated functionality as noted.
</a><a name="1661">
<li>In C++-PIDL, the release performs the role of the associated free and delete operations in the C mapping, unless otherwise noted.
</a></ul><a name="1431">
<img src=local/images/pluspa12.gif">Brief descriptions and listings of each pseudo-interface and its C++ mapping are provided in the following sections. Further details, including definitions of types referenced but not defined below, may be found in the relevant sections of this document.<p>
</a><a name="1435">
<h1>	17.4	 Environment</h1>
</a><a name="244">
<img src=local/images/pluspa13.gif"><strong>Environment</strong> provides a vehicle for dealing with exceptions in those cases where true exception mechanics are unavailable or undesirable (for example in the DII). They may be set and inspected using the <strong>exception</strong> attribute.<p>
</a><a name="166">
<img src=local/images/pluspa14.gif">As with normal OMG IDL attributes, the <strong>exception</strong> attribute is mapped into a pair of C++ functions used to set and get the exception. The semantics of the set and get functions, however, are somewhat different than those for normal OMG IDL attributes. The set C++ function assumes ownership of the Exception pointer passed to it. The Environment will eventually call delete on this pointer, so the Exception it points to must be dynamically allocated by the caller. The get function returns a pointer to the Exception, just as an attribute for a variable-length struct would, but the pointer refers to memory owned by the Environment. Once the Environment is destroyed, the pointer is no longer valid. The caller must not call delete on the Exception pointer returned by the get function. The Environment is responsible for deallocating any Exception it holds when it is itself destroyed. If the Environment holds no exception, the get function returns a null pointer.<p>
</a><a name="1000">
<img src=local/images/pluspa15.gif">The clear() function causes the Environment to delete any Exception it is holding. It is not an error to call clear() on an Environment holding no exception. Passing a null pointer to the set exception function is equivalent to calling clear(). If an Environment contains exception information, the caller is responsible for calling clear() on it before passing it to an operation.<p>
</a><a name="167">
<h2>	17.4.1	 Environment Interface</h2>
</a><a name="151">
 // IDL<br>pseudo interface Environment<br>{<br>						attribute exception exception;<br>						void clear();<br>};<p>
</a><a name="228">
<h2>	17.4.2	 Environment C++ Class</h2>
</a><a name="3022">
 // C++<br>class Environment<br>{<br> &#32; &#32;public:<br>	void exception(Exception*);<br>	Exception *exception() const;<br>	void clear();<br>};<p>
</a><a name="839">
<h2>	17.4.3	 Differences from C-PIDL</h2>
</a><a name="1505">
<img src=local/images/pluspa16.gif">The C++-PIDL specification differs from the C-PIDL specification as follows:<p>
</a><ul><a name="840">
<li>Defines an interface rather than a struct.
</a><a name="841">
<li>Supports an attribute allowing operations on exception values as a whole rather than on major numbers and/or identification strings.
</a><a name="2993">
<li>Supports a clear() function that is used to destroy any Exception the Environment may be holding.
</a><a name="3495">
<li>Supports a default constructor that initializes it to hold no exception information.
</a></ul><a name="2994">
<h2>	17.4.4	 Memory Management</h2>
</a><a name="2986">
<img src=local/images/pluspa17.gif">Environment has the following special memory management rules:<p>
</a><ul><a name="2991">
<li>The void exception(Exception*) member function adopts the Exception* given to it.
</a><a name="3005">
<li>Ownership of the return value of the Exception *exception() member function is maintained by the Environment; this return value must not be freed by the caller.
</a></ul><a name="148">
<h1>	17.5	 NamedValue</h1>
</a><a name="274">
<img src=local/images/pluspa18.gif"><strong>NamedValue</strong> is used only as an element of <strong>NVList</strong>, especially in the DII. <strong>NamedValue</strong> maintains an (optional) name, an <strong>any</strong> value, and labelling flags. Legal flag values are <strong>ARG_IN</strong>, <strong>ARG_OUT</strong>, and <strong>ARG_INOUT</strong>.<p>
</a><a name="149">
<img src=local/images/pluspa19.gif">The value in a <strong>NamedValue</strong> may be manipulated via standard operations on <strong>any</strong>.<p>
</a><a name="170">
<h2>	17.5.1	 NamedValue Interface</h2>
</a><a name="152">
 // IDL<br>pseudo interface NamedValue<br>{<br>						readonly attribute Identifier name;<br>						readonly attribute any value;<br>						readonly attribute Flags flags;<br>};<p>
</a><a name="230">
<h2>	17.5.2	 NamedValue C++ Class</h2>
</a><a name="231">
 // C++<br>class NamedValue<br>{<br> &#32; &#32;public:<br>	const char *name() const;<br>	Any *value() const;<br>	Flags flags() const;<br>};<p>
</a><a name="847">
<h2>	17.5.3	 Differences from C-PIDL</h2>
</a><a name="1499">
<img src=local/images/pluspa20.gif">The C++-PIDL specification differs from the C-PIDL specification as follows:<p>
</a><ul><a name="848">
<li>Defines an interface rather than a struct.
</a><a name="849">
<li>Provides no analog of the <strong>len</strong> field.
</a></ul><a name="3037">
<h2>	17.5.4	 Memory Management</h2>
</a><a name="3038">
<img src=local/images/pluspa21.gif">NamedValue has the following special memory management rules:<p>
</a><ul><a name="3043">
<li>Ownership of the return values of the name() and value() functions is maintained by the NamedValue; these return values must not be freed by the caller.
</a></ul><a name="153">
<h1>	17.6	 NVList</h1>
</a><a name="275">
<img src=local/images/pluspa22.gif"><strong>NVList</strong> is a list of <strong>NamedValue</strong>s. A new <strong>NVList</strong> is constructed using the <strong>ORB::create_list</strong> operation (see <a href="pluspo.html#195">Section &#32;17.12, "ORB," on page &#32;17-17</a>). New <strong>NamedValue</strong>s may be constructed as part of an <strong>NVList</strong>, in any of three ways:<p>
</a><ul><a name="154">
<li><strong>add</strong>-creates an unnamed value, initializing only the flags.
</a><a name="155">
<li><strong>add_item</strong>-initializes name and flags.
</a><a name="156">
<li><strong>add_value</strong>-initializes name, value, and flags.
</a><a name="3100">
<li><strong>add_item_consume</strong>-initializes name and flags, taking over memory management responsibilities for the char* name parameter.
</a><a name="3113">
<li><strong>add_value_consume</strong>-initializes name, value, and flags, taking over memory management responsibilities for both the char* name parameter and the Any* value parameter.
</a></ul><a name="157">
<img src=local/images/pluspa23.gif">Each of these operations returns the new item.<p>
</a><a name="3077">
<img src=local/images/pluspa24.gif">Elements may be accessed and deleted via zero-based indexing. The <strong>add</strong>, <strong>add_item</strong>, <strong>add_value</strong>, <strong>add_item_consume</strong>, and <strong>add_value_consume</strong> functions lengthen the <strong>NVList</strong> to hold the new element each time they are called. The <strong>item</strong> function can be used to access existing elements.<p>
</a><a name="3083">
<h2>	17.6.1	 NVList Interface</h2>
</a><a name="160">
 // IDL<br>pseudo interface NVList<br>{<br>						readonly attribute unsigned long count;<br>						NamedValue add(in Flags flags);<br>						NamedValue add_item(in Identifier item_name, in Flags flags);<br>						NamedValue add_value(<br>											in Identifier item_name,<br>											in any val,<br>											in Flags flags<br>						);<br>						NamedValue item(in unsigned long index) raises(Bounds);<br>						Status remove(in unsigned long index) raises(Bounds);<br>};<p>
</a><a name="232">
<h2>	17.6.2	 NVList C++ Class</h2>
</a><a name="233">
 // C++<br>class NVList<br>{<br> &#32; &#32;public:<br>	ULong count() const;<br>	NamedValue_ptr add(Flags);<br>	NamedValue_ptr add_item(const char*, Flags);<br>	NamedValue_ptr add_value(<br>			const char*,<br>			const Any&amp;,<br>																																						Flags<br>	);<br>	NamedValue_ptr add_item_consume(<br>			char*,<br>			Flags<br>	);<br>	NamedValue_ptr add_value_consume(<br>			char*,<br>			Any *,<br>			Flags<br>	);<br>	NamedValue_ptr item(ULong);<br>	Status remove(ULong);<br>};<p>
</a><a name="855">
<h2>	17.6.3	 Differences from C-PIDL</h2>
</a><a name="1494">
<img src=local/images/pluspa25.gif">The C++-PIDL specification differs from the C-PIDL specification as follows:<p>
</a><ul><a name="856">
<li>Defines an interface rather than a typedef
</a><a name="857">
<li>Provides different signatures for operations that add items in order to avoid representation dependencies
</a><a name="858">
<li>Provides indexed access methods
</a></ul><a name="3126">
<h2>	17.6.4	 Memory Management</h2>
</a><a name="3147">
<img src=local/images/pluspa26.gif">NVList has the following special memory management rules:<p>
</a><ul><a name="3148">
<li>Ownership of the return values of the add, add_item, add_value, add_item_consume, and add_value_consume functions is maintained by the NVList; these return values must not be freed by the caller.
</a><a name="3175">
<li>The char* parameters to the add_item_consume and add_value_consume functions and the Any* parameter to the add_value_consume function are consumed by the NVList. The caller may not access these data after they have been passed to these functions because the NVList may copy them and destroy the originals immediately. The caller should use the NamedValue::value() operation in order to modify the value attribute of the underlying NamedValue, if desired.
</a><a name="3216">
<li>The remove function also calls CORBA::release on the removed NamedValue.
</a></ul><a name="3150">
<h1>	17.7	 Request</h1>
</a><a name="277">
<img src=local/images/pluspa27.gif"><strong>Request</strong> provides the primary support for DII. A new request on a particular target object may be constructed using the short version of the request creation operation shown in <a href="pluspo.html#2827">Section &#32;17.13, "Object," on page &#32;17-16</a>:<p>
</a><a name="259">
 // C++<br>Request_ptr Object::_request(Identifier operation);<p>
</a><a name="260">
<img src=local/images/pluspa28.gif">Arguments and contexts may be added after construction via the corresponding attributes in the <strong>Request</strong> interface. Results, output arguments, and exceptions are similarly obtained after invocation. The following C++ code illustrates usage:<p>
</a><a name="261">
 // C++<br>Request_ptr req = anObj-&gt;_request("anOp");<br>*(req-&gt;arguments()-&gt;add(ARG_IN)-&gt;value()) &lt;&lt;= anArg;<br>// ...<br>req-&gt;invoke();<br>if (req-&gt;env()-&gt;exception() == NULL) {<br>	*(req-&gt;result()-&gt;value()) &gt;&gt;= aResult;<br>}<p>
</a><a name="3635">
<img src=local/images/pluspa29.gif">While this example shows the semantics of the attribute-based accessor functions, the following example shows that it is much easier and preferable to use the equivalent argument manipulation helper functions:<p>
</a><a name="3638">
 // C++<br>Request_ptr req = anObj-&gt;_request("anOp");<br>req-&gt;add_in_arg() &lt;&lt;= anArg;<br>// ...<br>req-&gt;invoke();<br>if (req-&gt;env()-&gt;exception() == NULL) {<br>	req-&gt;return_value() &gt;&gt;= aResult;<br>}<p>
</a><a name="262">
<img src=local/images/pluspa30.gif">Alternatively, requests can be constructed using one of the long forms of the creation operation shown in the Object interface in <a href="pluspo.html#2827">Section &#32;17.13, "Object," on page &#32;17-16</a>:<p>
</a><a name="263">
 // C++<br>Status Object::_create_request(<br>							Context_ptr ctx,<br>							const char *operation,<br>							NVList_ptr arg_list,<br>							NamedValue_ptr result,<br>							Request_ptr &amp;request,<br>							Flags req_flags<br>);<br>Status Object::_create_request(<br>			Context_ptr ctx,<br>			const char *operation,<br>			NVList_ptr arg_list,<br>			NamedValue_ptr result,<br>			ExceptionList_ptr,<br>			ContextList_ptr,<br>			Request_ptr &amp;request,<br>			Flags req_flags<br>);<p>
</a><a name="3514">
<img src=local/images/pluspa31.gif">Usage is the same as for the short form except that all invocation parameters are established on construction. Note that the <strong>OUT_LIST_MEMORY</strong> and <strong>IN_COPY_VALUE</strong> flags can be set as flags in the req_flags parameter, but they are meaningless and thus ignored because argument insertion and extraction are done via the Any type.<p>
</a><a name="3519">
<img src=local/images/pluspa32.gif"><strong>Request</strong> also allows the application to supply all information necessary for it to be invoked without requiring the ORB to utilize the Interface Repository. In order to deliver a request and return the response, the ORB requires:<p>
</a><ul><a name="3290">
<li>a target object reference
</a><a name="3291">
<li>an operation name
</a><a name="3292">
<li>a list of arguments (optional)
</a><a name="3293">
<li>a place to put the result (optional)
</a><a name="3297">
<li>a place to put any returned exceptions
</a><a name="3294">
<li>a <strong>Context</strong> (optional)
</a><a name="3295">
<li>a list of the user-defined exceptions that can be thrown (optional)
</a><a name="3296">
<li>a list of <strong>Context</strong> strings that must be sent with the operation (optional)
</a></ul><a name="3298">
<img src=local/images/pluspa33.gif">Since the <strong>Object::create_request</strong> operation allows all of these except the last two to be specified, an ORB may have to utilize the Interface Repository in order to discover them. Some applications, however, may not want the ORB performing potentially expensive Interface Repository lookups during a request invocation, so two new serverless objects have been added to allow the application to specify this information instead:<p>
</a><ul><a name="3302">
<li><strong>ExceptionList</strong>: allows an application to provide a list of <strong>TypeCode</strong>s for all user-defined exceptions that may result when the <strong>Request</strong>is invoke.
</a><a name="3303">
<li><strong>ContextList</strong>: allows an application to provide a list of <strong>Context</strong> strings that must be supplied with the <strong>Request</strong> invocation.
</a></ul><a name="3324">
The <strong>ContextList</strong> differs from the <strong>Context</strong> in that the former supplies only the context strings whose values are to be looked up and sent with the request invocation (if applicable), while the latter is where those values are obtained.<p>
</a><a name="3304">
<img src=local/images/pluspa34.gif">The IDL descriptions for <strong>ExceptionList</strong>, <strong>ContextList</strong>, and <strong>Request</strong> are shown below.<p>
</a><a name="180">
<h2>	17.7.1	 Request Interface</h2>
</a><a name="181">
 // IDL<br>pseudo interface ExceptionList<br>{<br>						readonly attribute unsigned long count;<br>						void add(in TypeCode exc);<br>						TypeCode item(in unsigned long index) raises(Bounds);<br>						Status remove(in unsigned long index) raises(Bounds);<br>};<br><br>pseudo interface ContextList<br>{<br>						readonly attribute unsigned long count;<br>						void add(in string ctxt);<br>						string item(in unsigned long index) raises(Bounds);<br>						Status remove(in unsigned long index) raises(Bounds);<br>};<br><br>pseudo interface Request<br>{<br>						readonly attribute Object target;<br>						readonly attribute Identifier operation;<br>						readonly attribute NVList arguments;<br>						readonly attribute NamedValue result;<br>						readonly attribute Environment env;<br>						readonly attribute ExceptionList exceptions;<br>						readonly attribute ContextList contexts;<br><br>						attribute context ctx;<br><br>						Status invoke();<br>						Status send_oneway();<br>						Status send_deferred();<br>						Status get_response();<br>						boolean poll_response();<br>};<p>
</a><a name="234">
<h2>	17.7.2	 Request C++ Class</h2>
</a><a name="3597">
 // C++<br>class ExceptionList<br>{<br> &#32; &#32;public:<br>	ULong count();<br>	void add(TypeCode_ptr tc);<br>	void add_consume(TypeCode_ptr tc);<br>	TypeCode_ptr item(ULong index);<br>	Status remove(ULong index);<br>};<br><br>class ContextList<br>{<br> &#32; &#32;public:<br>	ULong count();<br>	void add(const char* ctxt);<br>	void add_consume(char* ctxt);<br>	const char* item(ULong index);<br>	Status remove(ULong index);<br>};<br><br>class Request<br>{<br> &#32; &#32;public:<br>	Object_ptr target() const;<br>	const char *operation() const;<br>	NVList_ptr arguments();<br>	NamedValue_ptr result();<br>	Environment_ptr env();<br>	ExceptionList_ptr exceptions();<br>	ContextList_ptr contexts();<br><br>	void ctx(Context_ptr);<br>	Context_ptr ctx() const;<br><br>	// argument manipulation helper functions<br>	Any &amp;add_in_arg();<br>	Any &amp;add_in_arg(const char* name);<br>	Any &amp;add_inout_arg();<br>	Any &amp;add_inout_arg(const char* name);<br>	Any &amp;add_out_arg();<br>	Any &amp;add_out_arg(const char* name);<br>	void set_return_type(TypeCode_ptr tc);<br>	Any &amp;return_value();<p>
</a><a name="3607">
 	Status invoke();<br>	Status send_oneway();<br>	Status send_deferred();<br>	Status get_response();<br>	Boolean poll_response();<br>};<p>
</a><a name="878">
<h2>	17.7.3	 Differences from C-PIDL</h2>
</a><a name="1480">
<img src=local/images/pluspa35.gif">The C++-PIDL specification differs from the C-PIDL specification as follows:<p>
</a><ul><a name="879">
<li>Replacement of <strong>add_argument</strong>, and so forth, with attribute-based accessors.
</a><a name="880">
<li>Use of <strong>env</strong> attribute to access exceptions raised in DII calls.
</a><a name="881">
<li>The <strong>invoke</strong> operation does not take a flag argument, since there are no flag values that are listed as legal in <em>CORBA V2.0</em>.
</a><a name="882">
<li>The <strong>send_oneway</strong> and <strong>send_deferred</strong> operations replace the single <strong>send</strong> operation with flag values, in order to clarify usage.
</a><a name="883">
<li>The <strong>get_response</strong> operation does not take a flag argument, and an operation <strong>poll_response</strong> is defined to immediately return with an indication of whether the operation has completed. This was done because in <em>CORBA V2</em>.<em>0,</em> if the type <strong>Status</strong> is void, the version with <strong>RESP_NO_WAIT</strong> does not enable the caller to determine if the operation has completed.
</a><a name="3634">
<li>The <strong>add_*_arg</strong>, <strong>set_return_type</strong>, and <strong>return_value</strong> member functions are added as shortcuts for using the attribute-based accessors.
</a></ul><a name="3220">
<h2>	17.7.4	 Memory Management</h2>
</a><a name="3221">
<img src=local/images/pluspa36.gif">Request has the following special memory management rules:<p>
</a><ul><a name="3222">
<li>Ownership of the return values of the target, operation, arguments, result, env, exceptions, contexts, and ctx functions is maintained by the Request; these return values must not be freed by the caller.
</a></ul><a name="3798">
<img src=local/images/pluspa37.gif">ExceptionList has the following special memory management rules:<p>
</a><ul><a name="3799">
<li>The add_consume function consumes its TypeCode_ptr argument. The caller may not access the object referred to by the TypeCode_ptr after it has been passed in because the add_consume function may copy it and release the original immediately.
</a><a name="3802">
<li>Ownership of the return value of the item function is maintained by the ExceptionList; this return value must not be released by the caller.
</a></ul><a name="3805">
<img src=local/images/pluspa38.gif">ContextList has the following special memory management rules:<p>
</a><ul><a name="3806">
<li>The add_consume function consumes its char* argument. The caller may not access the memory referred to by the char* after it has been passed in because the add_consume function may copy it and free the original immediately.
</a><a name="3807">
<li>Ownership of the return value of the item function is maintained by the ContextList; this return value must not be released by the caller.
</a></ul><a name="182">
<h1>	17.8	 Context</h1>
</a><a name="279">
<img src=local/images/pluspa39.gif">A <strong>Context</strong> supplies optional context information associated with a method invocation.<p>
</a><a name="282">
<h2>	17.8.1	 Context Interface</h2>
</a><a name="186">
 // IDL<br>pseudo interface Context<br>{<br>						readonly attribute Identifier context_name;<br>						readonly attribute context parent;<br><br>						Status create_child(in Identifier child_ctx_name, out Context child_ctx);<br><br>						Status set_one_value(in Identifier propname, in any propvalue);<br>						Status set_values(in NVList values);<br>						Status delete_values(in Identifier propname);<br>						Status get_values(<br>										in Identifier start_scope,<br>										in Flags op_flags,<br>										in Identifier pattern,<br>										out NVList values<br>						);<br>};<p>
</a><a name="236">
<h2>	17.8.2	 Context C++ Class</h2>
</a><a name="237">
 // C++<br>class Context<br>{<br> &#32; &#32;public:<br>	const char *context_name() const;<br>	Context_ptr parent() const;<br><br>	Status create_child(const char *, Context_ptr&amp;);<br><br>	Status set_one_value(const char *, const Any &amp;);<br>	Status set_values(NVList_ptr);<br>	Status delete_values(const char *);<br>	Status get_values(<br>			const char*,<br>			Flags,<br>								const char*,<br>			NVList_ptr&amp;<br>	);<br>};<p>
</a><a name="889">
<h2>	17.8.3	 Differences from C-PIDL</h2>
</a><a name="1473">
<img src=local/images/pluspa40.gif">The C++-PIDL specification differs from the C-PIDL specification as follows:<p>
</a><ul><a name="890">
<li>Introduction of attributes for context name and parent.
</a><a name="891">
<li>The signatures for values are uniformly set to <strong>any</strong>.
</a><a name="892">
<li>In the C mapping, set_one_value used strings, while others used <strong>NamedValue</strong>s containing <strong>any</strong>. Even though implementations need only support strings as values, the signatures now uniformly allow alternatives.
</a><a name="893">
<li>The <strong>release</strong> operation frees child contexts.
</a></ul><a name="3234">
<h2>	17.8.4	 Memory Management</h2>
</a><a name="3235">
Context has the following special memory management rules:<p>
</a><ul><a name="3236">
<li>Ownership of the return values of the context_name and parent functions is maintained by the Context; these return values must not be freed by the caller.
</a></ul><a name="187">
<h1>	17.9	 Principal</h1>
</a><a name="283">
<img src=local/images/pluspa41.gif">A <strong>Principal</strong> represents information about principals requesting operations. There are no defined operations.<p>
</a><a name="2939">
<img src=local/images/pluspa42.gif">There are no differences from the C-PIDL mapping.<p>
</a><a name="189">
<h2>	17.9.1	 Principal Interface</h2>
</a><a name="284">
 // IDL<br>pseudo interface Principal {};<p>
</a><a name="239">
<h2>	17.9.2	 Principal C++ Class</h2>
</a><a name="241">
 // C++<br>class Principal {};<p>
</a><a name="190">
<h1>	17.10	 TypeCode</h1>
</a><a name="285">
<img src=local/images/pluspa43.gif">A <strong>TypeCode</strong> represents OMG IDL type information.<p>
</a><a name="1450">
<img src=local/images/pluspa44.gif">No constructors for <strong>TypeCode</strong>s are defined. However, in addition to the mapped interface, for each basic and defined OMG IDL type, an implementation provides access to a <strong>TypeCode</strong> pseudo object reference (TypeCode_ptr) of the form _tc_&lt;type&gt; that may be used to set types in Any, as arguments for <strong>equal</strong>, and so on. In the names of these <strong>TypeCode</strong> reference constants, &lt;type&gt; refer to the local name of the type within its defining scope. Each C++ _tc_&lt;type&gt; constant must be defined at the same scoping level as its matching type.<p>
</a><a name="2660">
<img src=local/images/pluspa45.gif">In all C++ <strong>TypeCode</strong> pseudo object reference constants, the prefix "_tc_" should be used instead of the "TC_" prefix prescribed in <a href="intrep2.html#969">Section &#32;6.7, "TypeCodes," on page &#32;6-33</a>. This is to avoid name clashes for CORBA applications that simultaneously use both the C and C++ mappings.<p>
</a><a name="3378">
<img src=local/images/pluspa46.gif">Like all other serverless objects, the C++ mapping for <strong>TypeCode</strong> provides a _nil() operation that returns a nil object reference for a <strong>TypeCode</strong>. This operation can be used to initialize <strong>TypeCode</strong> references embedded within constructed types. However, a nil <strong>TypeCode</strong> reference may never be passed as an argument to an operation, since <strong>TypeCode</strong>s are effectively passed as values, not as object references.<p>
</a><a name="192">
<h2>	17.10.1	 TypeCode Interface</h2>
</a><a name="3419">
 // IDL<br>pseudo interface TypeCode<br>{<br>						exception Bounds {};<br>						exception BadKind {};<br><br>						// for all TypeCode kinds<br>						boolean equal(in TypeCode tc);<br>						TCKind kind();<br><br>						// for tk_objref, tk_struct, tk_union, tk_enum, tk_alias, and tk_except<br>						RepositoryId id() raises(BadKind);<br>						Identifier name() raises(BadKind);<br><br>						// for tk_struct, tk_union, tk_enum, and tk_except<br>						unsigned long member_count() raises(BadKind);<br>						Identifier member_name(in unsigned long index)<br>															raises(BadKind, Bounds);<br><br>						// for tk_struct, tk_union, and tk_except<br>						TypeCode member_type(in unsigned long index)<br>															raises(BadKind, Bounds);<br><br>						// for tk_union<br>						any member_label(in unsigned long index)<br>															raises(BadKind, Bounds);<br>						TypeCode discriminator_type() raises(BadKind);<br>						long default_index() raises(BadKind);<br><br>						// for tk_string, tk_sequence, and tk_array<br>						unsigned long length() raises(BadKind);<br><br>						// for tk_sequence, tk_array, and tk_alias<br>						TypeCode content_type() raises(BadKind);<br><br>						// deprecated interface<br>						long param_count();<br>						any parameter(in long index) raises(bounds);<br>};<p>
</a><a name="3420">
<h2>	17.10.2	 TypeCode C++ Class</h2>
</a><a name="243">
 // C++<br>class TypeCode<br>{<br> &#32; &#32;public:<br>	class Bounds { ... };<br>	class BadKind { ... };<br><br>	Boolean equal(TypeCode_ptr) const;<br>	TCKind kind() const;<br><br>	const char* id() const;<br>	const char* name() const;<br><br>	ULong member_count() const;<br>	const char* member_name(ULong index) const;<br><br>	TypeCode_ptr member_type(ULong index) const;<br><br>	Any *member_label(ULong index) const;<br>	TypeCode_ptr discriminator_type() const;<br>	Long default_index() const;<br><br>	ULong length() const;<br><br>	TypeCode_ptr content_type() const;<br><br>	Long param_count() const;<br>	Any *parameter(Long) const;<br>};<p>
</a><a name="899">
<h2>	17.10.3	 Differences from C-PIDL</h2>
</a><a name="900">
<img src=local/images/pluspa47.gif">For C++, use of prefix "_tc_" instead of "TC_" for constants.<p>
</a><a name="3435">
<h2>	17.10.4	 Memory Management</h2>
</a><a name="3445">
TypeCode has the following special memory management rules:<p>
</a><ul><a name="3448">
<li>Ownership of the return values of the id, name, and member_name functions is maintained by the TypeCode; these return values must not be freed by the caller.
</a></ul><a name="193">
<h1>	17.11	 BOA</h1>
</a><a name="290">
<img src=local/images/pluspa48.gif">A <strong>BOA</strong> mediates between the ORB and object implementations.<p>
</a><a name="194">
<h2>	17.11.1	 BOA Interface</h2>
</a><a name="293">
 // IDL<br>pseudo interface BOA<br>{<br>						Object create(<br>									in ReferenceData id,<br>									in InterfaceDef intf,<br>									in ImplementationDef impl<br>						);<br>						void dispose(in Object obj);<br>						ReferenceData get_id(in Object obj);<br>						void change_implementation(in Object obj, in ImplementationDef impl);<br>						Principal get_principal(in Object obj, in Environment ev);<br>						void impl_is_ready(in ImplementationDef impl);<br>						void deactivate_impl(in ImplementationDef impl);<br>						void obj_is_ready(in Object obj, in ImplementationDef impl);<br>						void deactivate_obj(in Object obj);<br>};<p>
</a><a name="246">
<h2>	17.11.2	 BOA C++ Class</h2>
</a><a name="247">
 // C++<br>class BOA<br>{<br> &#32; &#32;public:<br>	Object_ptr create(<br>							const ReferenceData &amp;,<br>							InterfaceDef_ptr,<br>							ImplementationDef_ptr<br>	);<br>	void dispose(Object_ptr);<br>	ReferenceData *get_id(Object_ptr);<br>	void change_implementation(<br>			Object_ptr,<br>			ImplementationDef_ptr<br>	);<br>	Principal_ptr get_principal(<br>			Object_ptr,<br>			Environment_ptr<br>	);<br>	void impl_is_ready(ImplementationDef_ptr);<br>	void deactivate_impl(ImplementationDef_ptr);<br>	void obj_is_ready(Object_ptr, ImplementationDef_ptr);<br>	void deactivate_obj(Object_ptr);<br>};<p>
</a><a name="906">
<h2>	17.11.3	 Differences from C-PIDL</h2>
</a><a name="907">
<img src=local/images/pluspa49.gif">Means to set exceptions are moved to <strong>Environment</strong>.<p>
</a><a name="195">
<h1>	17.12	 ORB</h1>
</a><a name="294">
<img src=local/images/pluspa50.gif">An <strong>ORB</strong> is the programmer interface to the Object Request Broker.<p>
</a><a name="197">
<h2>	17.12.1	 ORB Interface</h2>
</a><a name="295">
 // IDL<br>pseudo interface ORB<br>{<br>						typedef sequence&lt;Request&gt; RequestSeq;<br>						string object_to_string(in Object obj);<br>						Object string_to_object(in string str);<br>						Status create_list(in long count, out NVList new_list);<br>						Status create_operation_list(in OperationDef oper, out NVList new_list);<br>						Status create_named_value(out NamedValue nmval);<br>						Status create_exception_list(out ExceptionList exclist);<br>						Status create_context_list(out ContextList ctxtlist);<br><br>						Status get_default_context(out Context ctx);<br>						Status create_environment(out Environment new_env);<br><br>						Status send_multiple_requests_oneway(in RequestSeq req);<br>						Status send_multiple_requests_deferred(in RequestSeq req);<br>						boolean poll_next_response();<br>						Status get_next_response(out Request req);<br>};<p>
</a><a name="248">
<h2>	17.12.2	 ORB C++ Class</h2>
</a><a name="249">
 // C++<br>class ORB<br>{<br> &#32; &#32;public:<br>	class RequestSeq {...};<br>	char *object_to_string(Object_ptr);<br>	Object_ptr string_to_object(const char *);<br>	Status create_list(Long, NVList_ptr&amp;);<br>	Status create_operation_list(<br>			OperationDef_ptr,<br>																	NVList_ptr&amp;<br>	);<br>	Status create_named_value(NamedValue_ptr&amp;);<br>	Status create_exception_list(ExceptionList_ptr&amp;);<br>	Status create_context_list(ContextList_ptr&amp;);<br><br>	Status get_default_context(Context_ptr&amp;);<br>	Status create_environment(Environment_ptr&amp;);<br><br>	Status send_multiple_requests_oneway(<br>			const RequestSeq																				&amp;<br>	);<br>	Status send_multiple_requests_deferred(<br>			const 																				RequestSeq &amp;<br>	);<br>	Boolean poll_next_response();<br>	Status get_next_response(Request_ptr&amp;);<br>};<p>
</a><a name="913">
<h2>	17.12.3	 Differences from C-PIDL</h2>
</a><ul><a name="914">
<li>Added <strong>create_environment</strong>. Unlike the struct version, <strong>Environment</strong> requires a construction operation. (Since this is overly constraining for implementations that do not support real C++ exceptions, these implementations may allow Environment to be declared on the stack. See <a href="plusalt.html#102">Section &#32;D.3, "Without Exception Handling," on page &#32;D-1</a> for details.)
</a><a name="918">
<li>Assigned multiple request support to ORB, made usage symmetrical with that in <strong>Request</strong>, and used a sequence type rather than otherwise illegal unbounded arrays in signatures.
</a><a name="3246">
<li>Added <strong>create_named_value</strong>, which is required for creating <strong>NamedValue</strong> objects to be used as return value parameters for the <strong>Object::create_request</strong> operation.
</a><a name="3338">
<li>Added <strong>create_exception_list</strong> and <strong>create_context_list</strong> (see <a href="pluspo.html#3150">Section &#32;17.7, "Request," on page &#32;17-7</a> for more details).
</a></ul><a name="1733">
<h2>	17.12.4	 Mapping of ORB and OA/BOA Initialization Operations </h2>
</a><a name="1745">
<h4> ORB Initialization</h4>
</a><a name="1832">
<img src=local/images/pluspa51.gif">The following PIDL specifies initialization operations for an ORB; this PIDL is part of the CORBA module (not the ORB interface) and is described in <a href="orbif2.html#801">Section &#32;7.4, "ORB Initialization," on page &#32;7-6</a>.<p>
</a><a name="1749">
 // PIDL <br>module CORBA {<br>						typedef string ORBid;<br>						typedef sequence &lt;string&gt; arg_list;<br>						ORB ORB_init (inout arg_list argv, in ORBid orb_identifier);<br>};<p>
</a><a name="1837">
<img src=local/images/pluspa52.gif">The mapping of the preceding PIDL operations to C++ is as follows:<p>
</a><a name="1844">
 // C++<br>namespace CORBA {<br>	typedef char* ORBid;<br>	static ORB_ptr ORB_init(<br>			int&amp; argc,<br>		 						char** argv,<br>										const char* orb_identifier<br>	);<br>};<p>
</a><a name="1839">
<img src=local/images/pluspa53.gif">The C++ mapping for <strong>ORB_init (</strong>and<strong> OA_init, </strong>described in the next section) deviates from the OMG IDL PIDL in its handling of the <strong>arg_list </strong>parameter. This is intended to provide a meaningful PIDL definition of the initialization interface, which has a natural C and C++ binding. To this end, the <strong>arg_list</strong> structure is replaced with argv and argc parameters. <p>
</a><a name="1770">
<img src=local/images/pluspa54.gif">The argv parameter is defined as an unbound array of strings (char **) and the number of strings in the array is passed in the argc (int &amp;) parameter.<p>
</a><a name="1771">
<img src=local/images/pluspa55.gif">If a NULL ORBid is used then argc arguments can be used to determine which ORB should be returned. This is achieved by searching the argc parameters for one tagged ORBid, e.g. -ORBid "ORBid_example." <p>
</a><a name="1772">
<img src=local/images/pluspa56.gif">For C++, the order of consumption of argv parameters may be significant to an application. In order to ensure that applications are not required to handle argv parameters they do not recognize the ORB initialization function must be called before the remainder of the parameters are consumed. Therefore, after the <strong>ORB_init</strong> call the argv and argc parameters will have been modified to remove the ORB understood arguments. It is important to note that the ORB_init call can only reorder or remove references to parameters from the argv list, this restriction is made in order to avoid potential memory management problems caused by trying to free parts of the argv list or extending the argv list of parameters. This is why argv is passed as a char** and not a char**&amp;. <p>
</a><a name="1773">
<h4> OA/BOA Initialization</h4>
</a><a name="1774">
<img src=local/images/pluspa57.gif">The following PIDL specifies the operations (in the ORB interface) that allow applications to get pseudo object references; it is described in detail in <a href="orbif2.html#811">Section &#32;7.5, "OA and BOA Initialization," on page &#32;7-8</a>. <p>
</a><a name="2954">
<blockquote><br>					// PIDL<br>					module CORBA {<br>						interface ORB 			{<br>							typedef sequence &lt;string&gt; arg_list;<br>							typedef string OAid;<br><br>							// Template for OA initialization operations<br>							// &lt;OA&gt; &lt;OA&gt;_init (inout arg_list argv,<br>							//                                     in OAid oa_identifier);<br>							BOA BOA_init (inout arg_list argv,<br>					 							in OAid boa_identifier);<br>				 		};</blockquote>
</a><a name="1796">
 };<p>
</a><a name="1798">
<img src=local/images/pluspa58.gif">The mapping of the <strong>OAinit</strong> (<strong>BOA_ini</strong>t) operation (in the ORB interface) to the C++ programming language is as follows:<p>
</a><a name="1868">
 // C++<br>namespace CORBA {<br>	class ORB<br>	{<br>	 &#32; &#32;public:<br>		typedef string OAid;<br><br>		// Template C++ binding for OA init op<br>		// &lt;OA&gt;_ptr &lt;OA&gt;_init			 					 	(		int * argc,<br>		// 				 									char **argv,<br>		// 		 											OAid oa_identifier);<br>		BOA_ptr BOA_init(<br>			int &amp; argc,<br>			char ** argv,<br>			const char *boa_identifier<br>		);<br>	};<br>}<p>
</a><a name="1819">
<img src=local/images/pluspa59.gif">If a NULL OAid is used then argc arguments can be used to determine which OA should be returned. This is achieved by searching the argc parameters for one tagged OAid, e.g. -OAid "OAid_example". <p>
</a><a name="1820">
<img src=local/images/pluspa60.gif">For C++, the order of consumption of <strong>argv </strong>parameters may be significant to an application. In order to ensure that applications are not required to handle argv parameters they do not recognize the OA initialization function must be called before the remainder of the parameters are consumed by the application. Therefore, after the &lt;OA&gt;_init call the argv and argc parameters will have modified to remove the OA understood arguments. It is important to note that the OA_init call can only reorder or remove references to parameters from the argv list, this restriction is made in order to avoid potential memory management problems caused by trying to free parts of the argv list or extending the argv list of parameters. This is why argv is passed as a char** and not a char**&amp;. <p>
</a><a name="1734">
<h2>	17.12.5	 Mapping of Operations to Obtain Initial Object References</h2>
</a><a name="1982">
<img src=local/images/pluspa61.gif">The following PIDL specifies the operations (in the ORB interface) that allow applications to get pseudo object references for the Interface Repository and Object Services. It is described in detail in <a href="orbif2.html#819">Section &#32;7.6, "Obtaining Initial Object References," on page &#32;7-10</a>.<p>
</a><a name="1959">
 <p>
</a><a name="1990">
<blockquote>					// PIDL<br>					module CORBA {<br>						interface ORB {<br>							typedef string ObjectId;<br>							typedef sequence &lt;ObjectId&gt; ObjectIdList;<br><br>							exception InvalidName {};<br><br>							ObjectIdList list_initial_services ();<br><br>							Object resolve_initial_references (in ObjectId identifier)<br>									raises (InvalidName);<br>						};<br>					};</blockquote>
</a><a name="1957">
<img src=local/images/pluspa62.gif">The mapping of the preceding PIDL to the C++ language is as follows:<p>
</a><a name="2797">
 // C++<br>namespace CORBA {<br>	class ORB {<br>	 &#32; &#32;public:<br>		typedef char* ObjectId;<br>		class ObjectIdList {...};<br>		class InvalidName {...};<br>		ObjectIdList *list_initial_services();<br>		Object_ptr resolve_initial_references(<br>			const char *identifier<br>		);<br>	};<br>}<p>
</a><a name="2827">
<h1>	17.13	 Object</h1>
</a><a name="954">
<img src=local/images/pluspa63.gif">The rules in this section apply to OMG IDL interface <strong>Object</strong>, the base of the OMG IDL interface hierarchy. Interface <strong>Object</strong> defines a normal CORBA object, not a pseudo object. However, it is included here because it references other pseudo objects.<p>
</a><a name="955">
<h2>	17.13.1	 Object Interface</h2>
</a><a name="956">
 // IDL<br>interface Object<br>{<br>						boolean is_nil();<br>						Object duplicate();<br>						void release();<br>						ImplementationDef get_implementation();<br>						InterfaceDef get_interface();<br>						Status create_request(<br>									in Context ctx,<br>									in Identifier operation,<br>									in NVList arg_list,<br>									in NamedValue result,<br>									out Request request,<br>									in Flags req_flags<br>						);<br>						Status create_request2(<br>									in Context ctx,<br>									in Identifier operation,<br>									in NVList arg_list,<br>									in NamedValue result,<br>									in ExceptionList exclist,<br>									in ContextList ctxtlist,<br>									out Request request,<br>									in Flags req_flags<br>						);<br>};<p>
</a><a name="957">
<h2>	17.13.2	 Object C++ Class</h2>
</a><a name="962">
 <img src=local/images/pluspa64.gif">In addition to other rules, all operation names in interface <strong>Object</strong> have leading underscores in the mapped C++ class. Also, the mapping for <strong>create_request</strong> is split into three forms, corresponding to the usage styles described in <a href="dii2.html#328">Section &#32;4.2.1, "create_request," on page &#32;4-4</a> and in <a href="pluspo.html#3150">Section &#32;17.7, "Request," on page &#32;17-6</a> of this document. The <strong>is_nil</strong> and <strong>release</strong> functions are provided in the CORBA namespace, as described in <a href="pluscon.html#510216">Section &#32;16.2.3, "Object Reference Operations," on page &#32;16-5</a>.<p>
</a><a name="969">
 // C++<br>class Object<br>{<br> &#32; &#32;public:<br>	static Object_ptr _duplicate(Object_ptr obj);<br>	static Object_ptr _nil();<br>	ImplementationDef_ptr _get_implementation();<br>	InterfaceDef_ptr _get_interface();<br>	Status _create_request(<br>		Context_ptr ctx,<br>		const char *operation,<br>		NVList_ptr arg_list,<br>		NamedValue_ptr result,<br>		Request_ptr &amp;request,<br>		Flags req_flags<br>	);<br>	Status _create_request(<br>		Context_ptr ctx,<br>		const char *operation,<br>		NVList_ptr arg_list,<br>		NamedValue_ptr result,<br>		ExceptionList_ptr,<br>		ContextList_ptr,<br>		Request_ptr &amp;request,<br>		Flags req_flags<br>	);<br>	Request_ptr _request(const char* operation);<br>};<p>
</a><a name="1670">
<p>
</a>
<hr><br>
 
<a href="cover.htm">[Top]</a> <a href="pluscon.htm">[Prev]</a> <a href="pluserv.htm">[Next]</a> <a href="corb2pra.htm">[Bottom]</a>
<hr><br>
 
<sup>1</sup><a name="101">
 In particular, exception used as a data type and a function name.<p>
</a>

<address>
<a href="mailto:pubs@omg.org">pubs@omg.org</a>
</address>

<i>Copyright &#169; 1995, <a href="http://www.omg.org">Object Management Group.</a>  All rights
reserved.</i>

<!-- This file was created with Quadralay WebWorks Publisher 3.0.4 -->
<!-- -->
<!-- For more information on how this document, and how the rest of -->
<!-- this server was created, email pubs@omg.org -->
<!-- -->
<!-- Last updated: 06/10/96 08:50:21 -->

</body>
</html>
