<document>
@* <a href=@talk-code.html target=_top>Selected source code fragments</a>
<include file=navigate>

This section presents selected samples of the code
implementing server-side code for the CORBA IDL
classes as well as the client-side adaptors for integrating these classes
in the <i>hush</i> class hierarchy.

@+ Handlers

The <i>handler</i> class may be considered the most important
class of <i>hush</i> since almost every other class, including
the <i>event</i>, <i>widget</i> and <i>item</i> classes
are derived from <i>handler</i>. See <cite file=refs>Hush</cite>.
<control>
</slide>
</control>

<slide id=code-server cap="Server code">
@ Server code - handler

<program>
class handler_srv : _crob(hush,handler) {
public:
    handler_srv(const handler* x = 0) : _bdy((handler*)x) { }
protected:
    void  set_body(const handler* x) { _bdy = (handler*) x; } 
    handler* _body() const { 
        return (handler*) _bdy;
        }

    virtual hush::event* dispatch(hush::event* e, ENV);

private:
handler* _bdy;
};
</program>
</slide>

<slide id=code-client cap="Client adaptor">
@ Client adaptor - handler
<program>
class handler_clt : public handler {
protected:
    handler_clt(handler_srv* x) : _bdy(x) { }
    void  set_body(void* x) { _bdy = x; } 
    handler* _body() { 
        if (!_bdy) throw "handler has no body";
        return (handler*) _bdy;
        }
private:
void* _bdy;
};
</program>
</slide>

@+ Events

The implementation of the <i>event_srv</i> class
uses the functionality of <i>handler_srv</i>
of which this class is derived.
The <i>event_clt</i> (adaptor) class on the other hand
is directly derived from the <i>hush</i> <i>event</i> class.

<slide id=code-event-server cap="Server code - event">
@ Server code -- event
<program>
class event_srv : public handler_srv, _crob(hush,event) {
public:
    event_srv(event* e) : handler_srv(e) { }

    event_srv() { _type = 0; _x = 0; _y = 0; } 

    virtual void type( long t, ENV ) { _type = t; }
    virtual long type( ENV ) { return _type; }

    virtual void x( long t, ENV ) { _x = t; }
    virtual long x( ENV ) { return _x; }

    virtual void y( long t, ENV ) { _y = t; }
    virtual long y( ENV ) { return _y; }

protected:
    event* _body() { return (event*) handler_srv::_body(); }
    long _type, _x, _y;
};
</program>
</slide>

<slide id=code-event-client cap="Client adaptor - event">
@ Client adaptor - event
<program>
class event_clt : public event {
public:
    event_clt(hush::event* x) : _bdy(x), event(this)  { }

    hush::event* operator->() { return _body(); }

    virtual int type() const { return (int) _body()->type(); }
    virtual int x() const { return (int) _body()->x(); }
    virtual int y() const { return (int) _body()->y(); }

    virtual void type(int n) { _body()->type( (long) n ); }
    virtual void x(int n) { _body()->x( (long) n ); }
    virtual void y(int n) { _body()->y( (long) n ); }

    /* Orbix 2.0
    virtual void type(int n) { _body()->type( (CORBA::Long) n ); }
    virtual void x(int n) { _body()->x( (CORBA::Long) n ); }
    virtual void y(int n) { _body()->y( (CORBA::Long) n ); }
    */

private:
    void* _bdy;
    hush::event* _body() const { return (hush::event*) _bdy; }
};
</program>
</slide>

@+ The <i>kit</i>

Both the server and client-side implementations of
<i>kit</i> delegate the calls to their embedded objects,
respectively a <i>hush</i> <i>kit</i> object and a CORBA IDL 
<i>kit</i> object.

<slide id=code-kit-server cap="Server code - kit">
@ Server code - kit
<program>
class kit_srv : public handler_srv, _crob(hush,kit) {
public:
    kit_srv(kit* tk) : handler_srv(tk) { }

    virtual void source( const char* s, ENV ) {
        char* p = new char[strlen(s)+1]; strcpy(p,s);
        _body()->source(p);
        }
    virtual void eval( const char* s, ENV ) {
        _body()->eval(s); 
        }
    virtual hush::term* result( ENV ) { 
        term* x = (term*) _body()->result(); 
        hush::term* res = new term_srv(x);
        res->_duplicate(); // orbix 1.3
        return res;
        }
    virtual void update( ENV ) { _body()->update(); }

    virtual hush::widget* root( ENV );

protected:
    kit* _body() { return (kit*) handler_srv::_body(); }
};

</program>
</slide>

<slide id=code-kit-client cap="Client-adaptor - kit">
@ Client adaptor
<program>
class kit_clt : public kit {
public:
    kit_clt(hush::kit* x) : _bdy(x)  { }

    virtual int eval( const char* s ) { _body()->eval(s); return 0; }

    virtual widget* root( ) const; 

private:
    void* _bdy;
    hush::kit* _body() const { return (hush::kit*) _bdy; }
};

</program>
</slide>

@+ Lists

The implementation of lists and iterators as template
C++ classes are taken from the work by John Caspers,
reported in <cite file=refs>Isis</cite>.

<slide id=code-list-server cap="Server code -- list">
@ Server code -- list
<program>
template<class T>
class list_srv : _crob(hush,container) {
public:
    list_srv(list<T>* it = 0) : _bdy(it) {
        this->_duplicate();
        }

    hush::iterator* walk( ENV ) {
        iter<void>* it = ((list<void>*) _body())->walk();
        iter_srv* x = new iter_srv(it);
        x->_duplicate();
        return x;
        }

    long length( ENV ) {
        return (long)  _body()->length();
        }

    CORBA::Object* first( ENV ) {
        T* x = _body()->first();
        //dummy->_duplicate(x); // Orbix 2.0
        x->_duplicate();
        return x;
        }

    CORBA::Object* current( ENV ) {
        T* x = _body()->current();
	x->_duplicate();
	return x;
        }

    CORBA::Object* next( ENV ) {
        T* x = _body()->next();
        x->_duplicate();
        return x;
        }

protected:
    T* dummy; // for duplication in 2.0
    list<T>* _bdy;
    list<T>* _body() const { return (list<T>*) _bdy; }
};

</program>
</slide>

<slide id=code-list-client cap="Client adaptor - list">
@ Client adaptor - list
<program>
template<class T>
class list_clt : public list<T> {
public:
    list_clt(hush::container* it = 0, T* = 0) : _bdy(it) { }

    list_clt<T>* operator->() { return this; }

    void operator=(hush::container* it) { _bdy = it; }

    T* first() {
        CORBA::Object* x = _body()->first();
        if (x)
                return (T*) dummy->_narrow(x);
        else return 0;
        }

    T* next() {
        CORBA::Object* x = _body()->next();
        if (x)
                return (T*) dummy->_narrow(x);
        else return 0;
        }

protected:
    hush::container* _bdy;
    hush::container* _body() const { return (hush::container*) _bdy; }
private:
    static T* dummy;
};

};
</program>
</slide>

<a name=Iter>
@+ Iterators

The implementation of the <i>iter</i> class follows
the same schema as the <i>list</i> implementation.
In particular for narrowing CORBA Objects to the actual
type T of the template client adaptor a static instance variable
dummy has been introduced.
This solution is again due to John Caspers.

<slide id=code-iter-server cap="Server code - iter">
@ Server code - iter
<program>
class iter_srv : _crob(hush,iterator) {
public:
    iter_srv(void* it) : _bdy(it) { }

    CORBA::Object* next( ENV ) {
        return _body()->operator()();
        }

protected:
    void* _bdy;
    iter<CORBA::Object>* _body() const { return (iter<CORBA::Object>*) _bdy; }
};
</program>
</slide>

<slide id=code-iter-client cap="Client adaptor - iter">
@ Client adaptor - iter
<program>
class iter_clt : public iter<T> {
public:
    iter_clt(hush::iterator* it = 0, T* = 0) : _bdy(it) { }

    hush::iterator* operator->() { return _body(); }

    void operator=(hush::iterator* it) { _bdy = it; }

    T* operator()() {
        CORBA::Object* x = _body()->next();
        if (x)
                return (T*) dummy._narrow(x);
        else return 0;
        }

protected:
    hush::iterator* _bdy;
    hush::iterator* _body() const { return (hush::iterator*) _bdy; }
private:
    static T dummy;
};
</program>
</slide>

<include file=navigate>
</document>
