<document class=article packages="a4wide,epsfig,verbatim">
@* <a href=@talk-idl.html target=_top>Interfaces</a>
<include file=navigate>

The IDL interfaces reflect to a large extent the functionality of
the original <i>hush</i> and <i>widgets</i> interfaces.
See <cite>Hush</cite>.
In this section a partial listing of the interfaces will be given.
Note that, in comparison with the corresponding C++ classes,
the IDL interfaces are much more abstract in the sense of omitting
many member functions that are required 
for the implementation of the actual <i>hush</i> framework.

<h2>The <i>hush</i> module</h2>

The <i>hush</i> module contains interfaces corresponding to
the basic <i>hush</i> classes,
<i>handler</i>, <i>event</i>, <i>kit</i>, <i>widget</i>
and <i>item</i>, as well as the auxiliary classes
for iterators and containers.

<slide id=idl-handler cap="handler">
<program>
interface handler {
        event dispatch( in event data );
	};
</program>
</slide>

The <i>handler</i> interface provides only a method
for dispatching events. It may be extended in the
future though.
In <i>hush</i> almost every class is derived from <i>handler</i>.
This is directly reflected in the <i>hush</i> IDL interfaces.

<slide id=idl-event cap="event">
<program>
interface event : handler {

	attribute long type;

	attribute long x;
	attribute long y;
	};
</program>
</slide>

The <i>event</i> interface offers attributes to
determine the type of event and its location.
Also the <i>event</i> interface will very likely be extended in
the future, to allow for a greater variety of events.

<slide id=idl-kit cap="kit">
<program>
interface kit : handler {

	void source(in string file);
	void eval(in string command);
	term result();

	widget root();
	};
</program>
</slide>

In <i>hush</i>, a <i>kit</i> provides an interface to
some embedded interpreter, such as a Tcl interpreter
or a logic engine as provided by BinProlog.
The <i>kit</i> gives also access to the underlying window environment,
in particular it may be asked to provide a reference to
the root window.

<slide id=idl-widget cap="widget">
<program>
interface widget : handler {

   string path();

   void eval( in string cmd );

   void configure( in string options );
   void pack( in string options );
};
</program>
</slide>

A widget is a user interface gadget.
The <i>widget</i> interface collects the functions that
all these gadgets have in common.

<slide id=idl-item cap="item">
<program>
interface item : handler {

        void move( in long x, in long y );
	};
</program>
</slide>

An <i>item</i> is obtained when creating a graphical
object for a canvas. Subsequently, the item reference
suffices to manipulate such objects.
Also the <i>item</i> interface will very likely
be extended in the future.


@ Iterators and lists

As an alternative for
CORBA arrays and sequences, the <i>hush</i>
module offers interfaces for iterators, containers and 
syntactic objects called <i>terms</i>.

<slide id=idl-iter cap="iterator">
<program>
interface iterator {
        Object next();
	};
</program>
</slide>

From a client's perspective, an <i>iterator</i>
is a data generator.
To deal with typed iterators, the <i>hush</i>
C++ library offers template client-side adaptor classes
encapsulating the untyped CORBA iterators.
See section <ref file=examples>Items</ref> and the code
in <ref file=code>Iter</ref>.

<slide id=idl-container cap="container">
<program>
interface container {

        long length();

        Object first();
        Object next();
        Object current();

        iterator walk();
	};
</program>
</slide>

The <i>container</i> interface offers access
to the famous <i>hush</i> list class.
It offers functions for cursor-based list traversal
as well as the <i>walk</i> function that may be used to obtain an iterator.

<slide id=idl-term cap="term">
<program>
interface term {

        string next();

};
</program>
</slide>

Currently, the <i>term</i> interface offers only the
functionality to use a term as an iterator
for strings.
The idea is that one can obtain strings from a term
as long as the term, for example the result of a logic query,
still contains information.

@ Factories and distributed object tables

To obtain references to objects, clients may use
either factory object or distributed object tables.

<slide id=idl-factory cap="factory">
<program>
interface factory {
        hush::kit kit(in string name);
        hush::event event(in long type);
};
</program>
</slide>

The <i>factory</i> interface allows only for creating 
a <i>kit</i> and for creating an <i>event</i>.
Note that <i>handler</i> objects may not be created directly.

<slide id=idl-dot cap="dot">
<program>
interface dot {

        hush::kit kit(in string name);

        hush::container container(in string name);
        hush::iterator iterator(in string name);

        hush::factory hush(in string name);
};
</program>
</slide>

Apart from giving access to a <i>hush</i> factory, 
the <i>dot</i> interface allows for getting access to
a <i>kit</i>, a <i>container</i> and
an <i>iterator</i>.
When obtaining references through a <i>dot</i> object,
these objects are assumed to exist within the server.


<h2>The <i>widgets</i> module</h2>

The <i>widgets</i> module provides the actual user interface
gadgets for <i>hush</i>.
Below we have  included only the (partial) interfaces for
a <i>canvas</i> and a <i>message</i> widget.

<slide id-idl-widgets cap="module widgets">
<program>
module widgets {

interface canvas : hush::widget {

	widgets::canvas create( in hush::widget anc, in string path );

	hush::item circle( in long x, in long y, in long radius, in string options );
	// other items ...
	};


interface message : hush::widget {
	message create( in hush::widget anc, in string path );
	void text(in string txt);
	};


interface factory : hush::factory {

 	widgets::canvas  canvas(in string name, in string options);
 	widgets::message message(in string name, in string options);
	};

interface dot : hush::dot {

	widgets::canvas  canvas(in string name);
 	widgets::message message(in string name);

	widgets::factory widgets(in string name);
	};


};
</program>
</slide>

Note that each widget type has a method <i>create</i>,
with which  an actual widget of that type can be created.
In effect this means that each widget object may act as a factory
for widget objects of that type.
(The server may however refuse to create such objects!)
In addition to the specific gadget interfaces, the <i>widgets</i> module
provides a <i>factory</i> and <i>dot</i> interface,
extending the respective <i>hush</i> interfaces.
<include file=navigate>
</document>
