<HTML>
<HEAD>
<TITLE>3. Widget Implementation</TITLE>
</HEAD>
<BODY>
<A NAME="Top"><H1>3. Widgets - Implementation</H1></A>

The <I>Aws++</I> widgets are built upon the <I>Hush</I> toolkit, using the 
basic 'primitive' widgets offered by it. They do not need any other toolkit
(such as, for example, <A HREF="http://www.xpi.com/tix">Tix</A>.
<P>
<A NAME="aws_balloon"><H2>3.1 aws_balloon</H2></A>

The <B>aws_balloon</B> is composed of a <B>toplevel</B> widget, with in it a 
small <B>label</B> with text. One such toplevel is created per aws_balloon,
not for each bound widget, the label is adjusted for each widget when it 
pops up.
<P>
The toplevel is immediately created together with the label and withdrawn from
the screen, so it's 'there' al the time, because it's properties must be
continuously available through the <B>subLabel()</B> and <B>subFrame()</B>
methods.
<P>
Binding a widget to an <B>aws_balloon</B> simply means that the aws_balloon 
captures the widgets <B>&lt;Any-Enter&gt;</B> and <B>&lt;Any-Leave&gt;</B>
events. If the mouse enters the widget, aws_balloons <B>operator()()</B> is
called with the configured balloon message and status message. The status 
message is immediately displayed, and a timer is set for the balloonmessage
(it's annoying if the balloon appears immediately). The <I>Id</I> of this timer
is stored so it can be canceled if the mouse leaves the widget before the
timer expires.
<P>

<A NAME="aws_combobox"><H2>3.2 aws_combobox</H2></A>

The <B>aws_combobox</B> is composed of a <B>frame</B> containing an <B>entry</B>
and a <B>button</B>. A separate <B>toplevel</B> with <B>listbox</B> is, as with
the <B>aws_balloon</B>, created and immediately withdrawn from the screen.
<P>
The <B>aws_combobox</B> tries to handle the events (<B>&lt;Key-Return&gt;</B> in
the <B>entry</B> and <B>&lt;1&gt;</B> in the <B>listbox</B> itself, simulating
the standard combobox behaviour. However, if a handler is bound to the 
combobox, both events are passed to the handler (the <B>ListSelected</B> and
the <B>EntrySelected</B> are sent to the handler, and the handler will have
to simulate this behaviour itself (this may change in the future - A 
<B>ListSelected</B> event will be handled internally and result in a 
<B>EntrySelected</B> event).
<P>
Implementing this behaviour however, is trivial:
<P>
<CODE><PRE>
        <B>if</B>(cbEvent == "EntrySelected")
        {       
                string  selection = cb-&gt;get();
 
                // Add entry to listbox.
 
                cb-&gt;insert(0, selection);
                cout &lt;&lt; "You selected " &lt;&lt; selection &lt;&lt; endl;
        }
        <B>else if</B>(cbEvent == "ListSelected")
        // Insert in entry, move selection to top
        {
                string selection = cb-&gt;selection();
 
                // Clear entry, insert selection
                cb-&gt;del();
                cb-&gt;insert(selection);
 
                // Remove from listbox, move to top
                cb-&gt;del(cb-&gt;curselection());
                cb-&gt;insert(0, selection);
 
                cout &lt;&lt; "You selected " &lt;&lt; selection &lt;&lt; endl;
	}
</PRE></CODE>
<P>
Handling both events may seem redundant but it allows more flexibility for
the programmer. For example, the handler can check the text typed in the entry
before inserting it in the listbox.
<P>
If the toplevel with the listbox is popped-up, the widget will do a global
grab on the focus. This is tricy, becase another widget (i.e. an
<B>aws_dialog</B>) might be holding the grab. This means that this grabinfo has
to be stored first, and restored after a choice has been made.
<P>
<A NAME="aws_dialog"><H2>3.3 aws_dialog</H2></A>

The <B>aws_dialog</B> is composed of:
<UL>
<LI> A toplevel
<LI> A bunch of frames, one specially designed for adding widgets
<LI> A label containing a bitmap
<LI> A label containing a message
<LI> Zero or more buttons
</UL>
<P>
The implementation of these components is fairly straight-forward - they're
laid out in a standard dialog box manner. More interesting is the frame
returned by the <B>subWidgetFrame()</B> method, which returns a frame in
which the programmer can build his own widgets, makeing it possible to
create dialogs containing <B>checkboxes</B>, <B>entries</B>, etc.
<P>
The programmer can add any number of <B>buttons</B> using the <B>addButton()</B>
method. If no handler is specified, the button will be handled internally and
the specified <I>const char *arg</I> will be returned by the <B>getChoice()</B>
method (very usefull for OK and CANCEL buttons), else the handler will have
to handle the button (usefull for adding a HELP button, or a button to update
the <I>widgetFrame</I> contents.
<P>
The <B>aws_dialog</B> will do a global grab on the mousecursor when it pops up.
Again, the previous grabstate will have to be stored and restored, as this 
dialog might, for example, be a help popup in another dialog window!
<P>
<A NAME="Problems"><H2>3.4 Problems</H2></A>
Most of the problems encountered when implementing these widgets occured
when trying to access <I>Tk</I> internals (and more standard funcitonality)
to which <I>Hush</I> did not provide access.
<P>
Fortunately, <I>Hush</I> provides a rather ugly but effective access to the
<I>Tk</I> toolkits interpreter using 
<P><CODE><PRE>
	<B>int</B>	kit::eval(<B>char *</B>cmd);
	<B>char	*</B>kit::evaluate(<B>char *</B>cmd);
</PRE></CODE>
<P>
(See <A HREF="AppendixC.html#l2">[2]</A> 
<A HREF="@online/hush/manuals/man/man4/kit.html">
The Hush Kit Manualpage)</A>)
<P>
When implementing these <I>AWS</I> Widgets, I notices quite alot of missing
funcitonality, which was sometimes almost trivial to implement.
<P>
For example, <I>Hush</I> provides hardly any access to the <I>Tk</I> <B>wm</B>
command. The <B>wm</B> command (See <A HREF="AppendixC.html#l4">[4]</A>) 
usually works on toplevel windows, so it can be implemented perfectly as
part of the toplevel widget.
<P>
For example, the aws_balloon uses a toplevel as windowtip. The windowtip is
'withdrawn' from the screen when it has to disappear (<B>wm withdraw</B>),
and when it's created, it's made transient (<B>wm transient</B>, which weirdly
enough *is* implemented), should be ignored by the windowmanager 
(<B>wm overrideredirect</B>), and deiconified ('unwithdrawn') when it should
popup again (<B>wm deiconify</B>).
<P>
All of these command's are missing in <I>Hush</I> except for 
<B>wm transient</B>, for which toplevel::transient() *does* exist.
<P>
Another lacking feature is access to the <I>Tk</I> <B>grab</B> command
which can grab the mousepointer locally or globally.
<P> 
Global grabbing is required by the aws_combobox (to make sure the user chooses
from the optionlist when it's pulled down before doing something else, which 
could seriously messup the widget) (<A HREF="#foot">*1</A>)
<P>
The dialog also optionally ilocally grabs the cursor so the user clicks one
of the 'Ok' or 'Cancel' buttons before doing something else with the 
application.
<P>
Finally, it's often usefull to query all sorts of information about a window
using the <I>Tk</I> <B>winfo</B> command. For example, the aws_balloon queries
the active widget's size and position to correctly position the windowtip (just
below the widget exactly in the middle). <I>Hush</I> does not provide access
to this command but it could very well be implemented in the <I>widget</I> 
baseclass.
<P>
See <A HREF="AppendixB.html">Appendix B.</A> for a complete overview of used
but missing commands.
<P>
<A NAME="Bugs"><H2>3.5 Bugs</H2></A>
These bugs are currently known in the <I>AWS</I> implementation:
<UL>
<LI> aws_balloons can be positioned partially outside the visible screen.
<LI> The aws_combobox popup list has a fixed size - it can't be sized to
     it's contents it seems. Scrollbars could fix this.
<LI> The aws_combobox might also be positioned partially outside the visible
     screen.
<LI> The aws_combobox defines a <B>static </B>string, which is not deleted
     before the <CODE>session::statistics()</CODE> are displayed, so this
     might like a leak, but it isn't.
</UL
<HR>
<A HREF="Semantics.html">&lt;&lt; [Prev]</A>
<A HREF="#top"> [Top] </A>
<A HREF="Conclusion.html">[Next] &gt;&gt;</A>
<HR>
*1 <A NAME="foot"><FONT SIZE=2>The pulled down listbox is implemented 
<I>in</I> a toplevel
widget. This would mean that if the user could still manage other windows,
he or she could switch desktops, leaving the toplevel floating in space,
or raise some other window over the listbox, which would look very confusing
</FONT>
</A>
</BODY>
</HTML>
