<!doctype html public "-//W3C//DTD HTML 3.2//EN">
<title>Checklist</title>
<h1><a href=@talk-checklist.html target=_top>Checklist</a></h1>
<slide id=check-1 caption="Data Reference Errors">
@ Data Reference Errors
  <ol>
  <li>Are all variables properly initialized? 
     (are all instance variables initialized in all constructors?)
  <li>Do all pointers have a legal value? (i.e. NULL or pointing
      to a valid location)
  <li>Are pointers tested on being NULL where necessary?
  <li>Alias problems? (e.g. several pointers pointing
      to the same object)
  <li>Are there any off-by-one errors?
      (e.g. strings without terminating EOS byte)
  <li>Could range bound errors occur? Eg: <tt>int array[max]; array[max]=0;</tt>
  <li>Is garbage collection properly implemented? (e.g: are all dynamically
      allocated objects (by new) eventually deallocated (by delete)?)
      If not, does this represent an oversight?
  <li> Are there any scope ambiguities? Do variable declarations hide other 
       declarations? If yes, is this an oversight? E.g:
<pre>class C {
  public: C() { object* o = new object; }
  private: object* o;
};
</pre>
  <li>(C++ experts only:) Do all base classes have a virtual destructor? 
      If not, does this represent an oversight?
  </ol>
</slide>

<slide id=check-2 cap="Computation Errors">
@ Computation Errors
  <ol>
  <li>Are there errors in implicit type casts? 
      (E.g: mixed mode arithmetic (float/int conversion))
  <li>Are underflow/overflow errors possible in math. computations?
  <li>Is the precedence of operators always correct? (E.g: x = 3+4 / 5)
  </ol>
</slide>

<slide id=check-3 cap="Control-flow Errors">
@ Control-flow Errors
  <ol>
  <li>Will every loop eventually terminate ?
  <li>Will the program, module, procedure eventually terminate ?
  <li>Are there any off-by-one errors? (one too many/few) iterations? 
  <li>Are there any non-exhaustive decisions (E.g: does every switch
      statement cover all possible cases?)
      If not, does this represent an oversight?
  <li>Does every if statement have a corresponding else clause?
      If not, does this represent an oversight?
   </ol>

</slide>
<slide id=check-4 cap="File I/O">
@ File I/O
  <ol>
   <li>Are all files used correctly opened/closed?
   <li>Does the program handle failures due to non-existing or 
       inaccessible files?
   </ol>
</slide>

<slide id=check-5 cap="C++ Tcl interface">
@ C++ Tcl interface
  <ol>
   <li>Are all options of new defined Tcl commands recognized
      by properly processing all arguments?
   <li>Does the program give a proper error message if incorrect arguments
       are passed?
   <li>Does the program return a result to the Tcl interpreter each
       time a Tcl command is evoked?
   </ol>
</slide>
<hr>
Adapted from:
G.J Myers,
<em>The Art of Software Testing</em>,
Wiley & Sons, 1979

<p>
See also 
<a href="http://www.cs.vu.nl/~se/tutorials/debug/debug/node2.html">Introduction
to debugging a C/C++ program</a>.
