<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> Defining the interface</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" Defining the interface">
<meta name="keywords" value="class-adt-www">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html43 HREF="node4.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html41 HREF="class-adt-www.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html35 HREF="node2.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>         <BR>
<B> Next:</B> <A NAME=tex2html44 HREF="node4.html"> Defining the implementation</A>
<B>Up:</B> <A NAME=tex2html42 HREF="class-adt-www.html">From Modula-2 ADT's to </A>
<B> Previous:</B> <A NAME=tex2html36 HREF="node2.html"> Separation of interface </A>
<BR> <HR> <P>
<H1><A NAME=SECTION00030000000000000000> Defining the interface</A></H1>
<P>
The header file should, just like in Modula-2, reveal as little as possible
of the implementation. But in contrary to Modula-2, in C++ it's very hard to
hide everything of the implementation, as we'll see later on.
<P>
An example of the interface of a stack as it could be defined in Modula-2:
<P>
<PRE>DEFINITION MODULE IntStacks;

(* Elements: Although the elements could be of a variety of types, for
    simplicity we assume that they are of type INTEGER *)

(* Structure: Linear, Last in first out *)

(* Domain: The number of elements in a stack is bounded to a certain
    maximum. *)

TYPE 
    IntStack;                  (* Opaque type to represent a Stack. *)

PROCEDURE Push(VAR S: IntStack; e: INTEGER);
(* pre  - S is not full.
   post - S includes e as its most recently arrived element. *)

PROCEDURE Pop(VAR S: IntStack; VAR e: INTEGER);
(* pre  - S is not empty.
   post - e is the most recently arrived element of S-pre;
          S no longer contains e. *)

PROCEDURE Empty(S: IntStack): BOOLEAN;
(* post - TRUE : S has no elements
          FALSE: S has at least one element *)

PROCEDURE Full(S: IntStack): BOOLEAN;
(* post - TRUE : S has reached its bound
          FALSE: S can at least contain one more element *)

PROCEDURE Create(VAR S: IntStack): BOOLEAN;
(* pre  - S does not exist
   post - TRUE : S has been created, S is empty
          FALSE: there was not enough memory left to create S *)

PROCEDURE Terminate(VAR S: IntStack);
(* post - S-pre does not exist. *)

END IntStacks.
</PRE>
<P>
A typical header file of the same ADT might look like
<P>
<PRE>// intstack.h

#ifndef _INTSTACK_H_
#define _INTSTACK_H_

class intstack
{
public:
    intstack();
        // post - The stack has been created, the stack is empty
    ~intstack();
        // post - The stack-pre does not exist

    void push(int e);
        // pre  - The stack is not full.
        // post - The stack includes e as its most recently arrived element.
    void pop(int&amp; e);
        // pre  - The stack is not empty.
        // post - e is the most recently arrived element of the stack-pre;
        //        the stack no longer contains e.
    bool empty();
        // post - true : The stack has no elements.
        //        false: The stack has at least one element.
    bool full();
        // post - true : The stack has reached its bound.
        //        false: The stack can at least contain one more element.

private:
    int* stack_;
    int  top_;
};

#endif
</PRE>
<P>
The <code>#ifndef</code>/<code>#define</code> construction is used to prohibit multiple
inclusions of the header file.<A NAME=tex2html2 HREF="footnode.html#25"><IMG ALIGN=BOTTOM ALT="gif" SRC="http://www.cs.vu.nl/images/latex2html//foot_motif.gif"></A>
<P>
The interfaces in Modula-2 and C++ are very similar, but they differ on some
important points. The most noticeable difference is the absence of a
parameter of the stack type in all member functions. This is because
functions do not work <em>on</em> an instance, but are requests <em>to</em> an
instance to perform some action. Compare calling the <code>Full</code> function on
a variable in Modula-2:
<PRE>:
    IF Full(MyStack) THEN
:
</PRE>
to asking a stack-instance if it's full:
<PRE>:
    if (mystack.full())
:
</PRE>
<P>
Another difference is the overall structure of the interface. In Modula-2,
all functions are in fact seperate functions with one common parameter. In a
C++ class, the functions are all member functions of the ADT and are
therefore declared within the <code>class intstack { ... };</code> statement.
<P>
The third difference is the exposure of the internal variables of a class.
The only thing known about the type <code>IntStack</code> in the Modula-2 interface
is the fact that it exists; it's completely opaque. But in C++, all
variables used in the implementation have to be declared in the interface,
thereby making it virtually impossible to create an ADT that's really
abstract. Making these member variables <code>private</code> will shield them
from being used directly by a user.
<P>
The last difference is the names of the functions to create and destroy
instances of the ADT. C++ offers constructors and destructors, so you should
not give them other names like <code>Create</code> or <code>Terminate</code>.
<P>
<BR> <HR><A NAME=tex2html43 HREF="node4.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html41 HREF="class-adt-www.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html35 HREF="node2.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>         <BR>
<B> Next:</B> <A NAME=tex2html44 HREF="node4.html"> Defining the implementation</A>
<B>Up:</B> <A NAME=tex2html42 HREF="class-adt-www.html">From Modula-2 ADT's to </A>
<B> Previous:</B> <A NAME=tex2html36 HREF="node2.html"> Separation of interface </A>
<BR> <HR> <P>
<BR> <HR>
<P><ADDRESS>
<I>SE Praktikum <BR>
Tue Aug 13 13:10:22 MET DST 1996</I>
</ADDRESS>
</BODY>
