<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> An example in C++</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" An example in C++">
<meta name="keywords" value="debug">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html253 HREF="node22.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html251 HREF="node19.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html247 HREF="node20.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>         <BR>
<B> Next:</B> <A NAME=tex2html254 HREF="node22.html"> More information</A>
<B>Up:</B> <A NAME=tex2html252 HREF="node19.html"> Tracing code</A>
<B> Previous:</B> <A NAME=tex2html248 HREF="node20.html"> An example in </A>
<BR> <HR> <P>
<H4><A NAME=SECTION00037320000000000000> An example in C++</A></H4>
<P>
Suppose we've implemented a very simple linked class:
<PRE>#include &lt;iostream.h&gt;


class node
{
public:
    node(int i, node* n) { item_ = i; next_ = n; }

    int item() { return item_; }
    node* next() { return next_; }

protected:
    int item_;
    node* next_;
};


int main()
{
    int i;
    node* current = 0;

// build list
    node* head = new node(1, NULL);
    head = new node(2, head);       // prepend new node
    head = new node(3, head);       // prepend new node

// print list
    current = head;
    for (i = 0; i &lt;= 3; i++)        // since list-size is now 3
    {
        cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
        current = current -&gt; next();
    }

// destroy list
    current = head;
    for (i = 0; i &lt;= 3; i++)
    {
        current = head -&gt; next();
        delete head;
        head = current;
    }

    return 0;
}
</PRE>
It looks quite simple, but it still contains a bug:
<PRE>$ g++ -g -o list list.cc
$ list
item 0 is 3
item 1 is 2
item 2 is 1
Memory fault
$
</PRE>
First, we let GDB pinpoint the exact location of the memory fault:
<PRE>GDB is free software and you are welcome to distribute copies of it
 under certain conditions; type &quot;show copying&quot; to see the conditions.
There is absolutely no warranty for GDB; type &quot;show warranty&quot; for details.
GDB 4.13 (sparc-sun-solaris2.3), 
Copyright 1994 Free Software Foundation, Inc...
$ gdb list
(gdb) run
Starting program: /home/se/doc/debug/source/list 
item 0 is 3
item 1 is 2
item 2 is 1

Program received signal SIGSEGV, Segmentation fault.
node::item (this=0x0) at list.cc:9
9           int item() { return item_; }
(gdb)
</PRE>
If the information provided by GDB is examined closely, it appears that the
<code>this</code>-pointer is 0! This means a member function is called for a
non-existing object and this will indeed cause a memory fault. But how can
this happen? Use the command <code>bt</code> to show the calling sequence that
resulted in the memory fault:
<PRE>(gdb) bt
#0  node::item (this=0x0) at list.cc:9
#1  0x800896e in main () at list.cc:32
(gdb) list list.cc:32
27
28      // print list
29          current = head;
30          for (i = 0; i &lt;= 3; i++)        // since list-size is now 3
31          {
32              cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
33              current = current -&gt; next();
34          }
35
36      // destroy list
(gdb)
</PRE>
It appears something goes wrong while printing all values. Since
<code>current</code> is always pointing to the current object, it follows that
<code>current</code> is at some point set to 0. But <code>current</code> can only be 0
at the end of the list (when <code>next_</code> equals 0). Let's see what happens
when the last item has been printed:
<PRE>(gdb) br list.cc:32
Breakpoint 1 at 0x8008940: file list.cc, line 32.
(gdb) run
The program being debugged has been started already.
Start it from the beginning? (y or n) y
Starting program: /home/se/doc/debug/source/list 

Breakpoint 1, main () at list.cc:32
32              cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
(gdb) cont
Continuing.
item 0 is 3

Breakpoint 1, main () at list.cc:32
32              cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
(gdb) cont
Continuing.
item 1 is 2

Breakpoint 1, main () at list.cc:32
32              cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
(gdb)
</PRE>
So far, so good. This is the last item to be printed (since there are only 3
list-nodes). We now use tracing to see what happens next:
<PRE>(gdb) n
item 2 is 1
33              current = current -&gt; next();
(gdb) n
34          }
(gdb) print current
$1 = (node *) 0x0
(gdb)
</PRE>
As expected, <code>current</code> is now 0. But what happens now?
<PRE>(gdb) n
32              cout &lt;&lt; &quot;item &quot; &lt;&lt; i &lt;&lt; &quot; is &quot; &lt;&lt; current -&gt; item() &lt;&lt; endl;
(gdb)
</PRE>
It appears the loop is not finished yet! How can that be?
<PRE>(gdb) print i
$2 = 3
(gdb)
</PRE>
After some thinking, the problem is clear: the loop is executed four times
instead of three times. The for-loop should be 
<code>for (i = 0; i &lt; 3; i++) { ... }</code> and then it will work (the second
loop should of course be changed also).
<P>
<BR> <HR>
<P><ADDRESS>
<I>SE Praktikum <BR>
Tue Aug 13 11:13:23 MET DST 1996</I>
</ADDRESS>
</BODY>
