<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> C/C++</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" C/C++">
<meta name="keywords" value="debug">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html102 HREF="node8.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html100 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html94 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>         <BR>
<B> Next:</B> <A NAME=tex2html103 HREF="node8.html"> C++</A>
<B>Up:</B> <A NAME=tex2html101 HREF="node6.html"> Semantic errors</A>
<B> Previous:</B> <A NAME=tex2html95 HREF="node6.html"> Semantic errors</A>
<BR> <HR> <P>
<H3><A NAME=SECTION00022100000000000000> C/C++</A></H3>
<P>
<UL><LI> Putting an extra semicolon after the heading of an <code>if</code> or
<code>while</code> statement, for example 
<PRE>if (x &gt; 0);
{
    x++;
    foo(x);
}
</PRE>
will result in <em>always</em> executing the block following the <code>if</code>. 
Something similar can be said for <code>while</code> statements.
<P>
<LI> Writing <code>if (x = 0)</code> instead of <code>if (x == 0)</code>. When compiled
with <code>g++ -Wall</code>, this construction will result in a warning: 'suggest
parentheses around assignment used as truth value'. The same goes for
<code>while</code>.
<P>
<LI> Assuming wrong precedence of operators. For example, the construct
<code>if (mask &amp; 0x80 == 0)</code> looks ok, but it is not, because <code>==</code>
has a higher precedence than <code>&amp;</code>. This construct is in fact equivalent
to <code>if (mask &amp; (0x80 == 0))</code> and will always be false (left as an
exercise for the reader). The obvious solution to this problem is to use
parentheses around (complex) expressions before and after the <code>==</code> as
in <code>if ((mask &amp; 0x80) == 0)</code>.
<P>
<LI> No <code>break</code> at the end of a case-statement; a <code>switch</code> such
as
<PRE>    switch(value)
    {
        case 0:
            printf(&quot;value = 0\n&quot;);
        case 1:
            printf(&quot;value = 1\n&quot;);
        default:
            printf(&quot;value is not 0 or 1\n&quot;);
    }
</PRE>
will, when <code>value</code> equals 0, produce
<PRE>value = 0
value = 1
value is not 0 or 1
</PRE>
instead of the 'expected' output
<PRE>value = 0
</PRE>
The compiler will not generate a warning for this, since it's perfectly
legal.
<P>
<LI> Mixing up the logical operators (<code>&amp;&amp;</code>, <code>||</code>) and the bitwise
operators (<code>&amp;</code>, <code>|</code>).
<P>
<LI> Calling a format-function with too few parameters, e.g.
<code>printf(&quot;debug: x = %d\n&quot;);</code>. This will result in some random value
printed out (depending on the contents of the stack).
Fortunately, when compiling with
<code>-Wall</code>, gcc will warn you: <BR> 
<code>test.cc:5: warning: too few arguments for format</code>.
<P>
<LI> Writing <code>if (function_name)</code> instead of 
<code>if (function_name())</code>. The first is also legal in C/C++ and will not
generate any errors or warnings.
<P>
<LI> Spreading C-comments across multiple lines, thereby putting some
statements in comment:
<PRE>int foo(int x)
{
    int x2, y;

    x2 = x * 3.1415;            /* we're doing some
    y = x2 * x + x * x + 1.1;      pretty extensive
    x = abs(x2) &gt; 1 ? 0 : 1;       calculations here */
    return y * x;
}
</PRE>
This looks very nicely formatted, but it won't return the correct result.
It's much safer to always begin and end a comment on the same line, or use
the <code>//</code> when programming in C++.
<P>
<LI> Using arrays as 1-based instead of 0-based arrays:
<code>int intarray[10];</code> is an array that ranges from 0 to 9, not from 1 to
10. Accessing the 10th element when there is none can result in
unpredictable behaviour, depending on the variables stored in memory that
are immediately following that array. Possible behaviour: memory
faults/segmentation violations<A NAME=tex2html1 HREF="footnode.html#73"><IMG ALIGN=BOTTOM ALT="gif" SRC="http://www.cs.vu.nl/images/latex2html//foot_motif.gif"></A> (memory faults
will happen when a program tries to read from or write to memory that does
not belong to that program), runtime errors in system-functions like
<em>malloc</em>, other variables changing values when they should not.
<P>
<LI> Returning local arrays, for example
<PRE>char* int2string(int i)
{
    char buffer[64];

    sprintf(buffer, &quot;%d&quot;, i);
    return buffer;
}
</PRE>
Local arrays (in C these are called <em>automatic variables</em>) are
allocated on the stack and are removed as soon as the function returns. The
pointer (in this case <code>buffer</code>) will then be invalid. This will work:
<PRE>char* int2string(int i)
{
    char* buffer;

    buffer = new char[64];
    sprintf(buffer, &quot;%d&quot;, i);
    return buffer;
}
</PRE>
<P>
<LI> Passing a variabele to a function where an address is required, e.g.
<code>scanf(&quot;%d&quot;, value)</code> instead of <code>scanf(&quot;%d&quot;, &amp;value)</code>. The first 
will most of the time result in a memory fault.
When compiling with <code>-Wall</code>, gcc will give a warning: <BR> 
<code>test.cc:8: warning: format argument is not a pointer (arg 2)</code>.
<P>
</UL><BR> <HR><A NAME=tex2html102 HREF="node8.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html100 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html94 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>         <BR>
<B> Next:</B> <A NAME=tex2html103 HREF="node8.html"> C++</A>
<B>Up:</B> <A NAME=tex2html101 HREF="node6.html"> Semantic errors</A>
<B> Previous:</B> <A NAME=tex2html95 HREF="node6.html"> Semantic errors</A>
<BR> <HR> <P>
<BR> <HR>
<P><ADDRESS>
<I>SE Praktikum <BR>
Tue Aug 13 11:13:23 MET DST 1996</I>
</ADDRESS>
</BODY>
