<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 3.0//EN">
 <title>Implementation of the fileselector class</title>
   <h2>Implementation of the fileselector class</h2>
   <file>fileselector.c
   <author>bastiaan, martijn, jrvosse.
   <ignore>
   This file contains the the C++ implementation of our fileselector.
   It is annotated with pseudo HTML markup.
   </ignore>
   <p>We need to include some headers first:<pre class=program lang="C++">
  #include &lt;string.h&gt;             <i class=comment>// some standard C and C++ include files</i>
  #include &lt;stdlib.h&gt;
  #include &lt;iostream.h&gt;

  #include &lt;hush/event.h&gt;         <i class=comment>// to handle hush events</i>
  #include &lt;hush/string.h&gt;        <i class=comment>// hush string class</i>
  #include "fileselector.h"      <i class=comment>// The fileselector class header</i>
  </pre> Options to configure the widgets:<pre class=program lang="C++">
  static const char* frameopts= "-relief raised -bd 1";
  static const char* entryopts= "-width 40 -relief sunken -bg white";

  </pre> Options to pack the widgets:<pre class=program lang="C++">
  static const char* label_pack_opts = "-side left -padx 10 -pady 10";
  static const char* entry_pack_opts = "-side right -padx 10 -pady 10 "
                                       "-fill x -expand true";

  </pre><h3>Constructors & destructors</h3>
   The first constructor creates a dummy fileselector object
   which can be used for handling events from the script interface<pre class=program lang="C++">
  fileselector::fileselector() {
      _filename[0] = _pathname[0] = _mask[0] = 0;
  }

  </pre> The 'real' constructor follows below.
   The use of _register() is explained in the tutorial on garbage
   collection.<pre class=program lang="C++">
  fileselector::fileselector(const char *p, const char* options, int istop) : frame(p,0) {
      _filename[0] = _pathname[0] = _mask[0] = 0;

      widget* top;
      if (istop) top = new toplevel(p,options);
      else top = new frame(p,options);
      _register(top);

      build_gui(top, options);  <i class=comment>// set up GUI</i>
      bindings();                <i class=comment>// set up bindings</i>
      mask("*");                 <i class=comment>// set default mask </i>
      dirpath(".");              <i class=comment>// set default directory</i>
  }

  fileselector::fileselector(const widget *parent, const char *p, const char* options, int istop) : frame(parent, p, 0) {
      _filename[0] = _pathname[0] = _mask[0] = 0;

      widget* top=0;      <i class=comment>// top will be deleted automatically by parent</i>
      if (istop) top = new toplevel(parent,p,options);
      else top = new frame(parent,p,options);

      top-&gt;rename();              <i class=comment>// rename top widget (see tutorial)</i>
      tk-&gt;bind(name(), this);     <i class=comment>// bind this to tcl command name</i>
      build_gui(top, options);  <i class=comment>// set up GUI</i>
      bindings();                <i class=comment>// set up bindings</i>
      mask("*");                 <i class=comment>// set default mask </i>
      dirpath(".");              <i class=comment>// set default directory</i>
  }


  </pre> The destructor. Now we have garbage collection for widgets,
   it is no longer needed ... <pre class=program lang="C++">
  fileselector::~fileselector() { }

  </pre> <h3>Common member functions</h3>
   Here come some "ordinary" member functions you
   expect every fileselector to have...
  
   <h4>Retrieving the selected filename</h4>
   This function is used to retrieve the result.
   It grabs the mouse pointer in order to prevent any
   activity on other windows.
   Then it waits untill the fileselector window is destroyed (!)
   which will be the case after pressing Ok or Cancel...<pre class=program lang="C++">
  const char* fileselector::get() {
      grab(); <i class=comment>// grab mouse pointer</i>
      wait(); <i class=comment>// wait until the user has made his choise</i>
      return _filename[0] ? _filename : 0;   <i class=comment>// contains filename or null </i>
  }

  </pre> <h4>The filename mask</h4>
   The following two members can be used to get and set the mask.         
   A mask is quit common concept in fileselector.
   The default mask is "*".<pre class=program lang="C++">
  const char* fileselector::mask() const { return _mask; } 

  void fileselector::mask(const char *m) { <i class=comment>// set mask</i>
      strcpy(_mask,m);            <i class=comment>// save mask</i>
      _maskentry-&gt;del();          <i class=comment>// delete entry</i>
      _maskentry-&gt;insert(_mask);  <i class=comment>// insert new mask </i>
      rescan();
  }

  </pre> <h4>The directory path</h4>
   A get and set pair for the directory path as well.
   <em>
     Side effect: changes applications current working directory
   </em><pre class=program lang="C++">
  const char* fileselector::dirpath() const { return _pathname; }

  int fileselector::dirpath(const char *p) {
      sprintf(buf, "cd %s", p);
      if (tk-&gt;eval(buf) == OK) {                  <i class=comment>// chdir</i>
          strcpy(_pathname, tk-&gt;evaluate("pwd")); <i class=comment>// save path</i>
          _pathentry-&gt;del();                      <i class=comment>// delete entry</i>
          _pathentry-&gt;insert(_pathname);  <i class=comment>// insert new path</i>
          rescan();                       <i class=comment>// rescan directory</i>
          return OK; 
      } else {
          sprintf(buf, "Could not chdir to %s", p);
          tk-&gt;error(buf);
          return ERROR;
      }
  }

  </pre> <h4>Rescanning the directory</h4>
   <em>
        Side effect: Using global Tcl variables $fileselector::fill and $i
   </em><p>
   This function does a rescan, which is needed to update
   the widgets after the directory path or mask has changed.
   <p>
   Most of the work is done by evaluating raw Tcl commands.
   This is a bit tricky since they use (global) Tcl variables ...
   The <tt>glob -nocomplain .* *</tt> command returns a list of
   file and directory names of the current working directory.
   The <tt>foreach</tt> command is used to loop over this list.
   Only the directories are put the listbox, so they will appear
   before all other files.
   <p>
   Another glob command is used to retrieve a list of all filenames matching
   the pattern in _mask. This list is sorted by the lsort command and
   again, we use a foreach command to insert the filenames in the listbox.<pre class=program lang="C++">
  void fileselector::rescan() {
      _listbox  -&gt;del();      <i class=comment>// delete listbox</i>
      
      <i class=comment>// put all directories in listbox </i>
      sprintf(buf,"foreach i [glob -nocomplain .* * ] "
              "{ if [file isdirectory $i] "
              "{%s insert end $i }}",
              _listbox-&gt;path() );
      tk-&gt;eval(buf); 
      
      <i class=comment>// put files in $fileselector::fill variable after mask with glob</i>
      sprintf(buf,"set fileselector::fill [glob -nocomplain %s]", _mask); 
      tk-&gt;eval(buf);
      
      <i class=comment>// put files from variable in listbox</i>
      sprintf(buf,"foreach i [lsort ${fileselector::fill}] "
              "{ if [file isfile $i] {%s insert end $i }}", 
              _listbox-&gt;path() );
      tk-&gt;eval(buf); 
  }

  </pre> <h3>The graphical user interface of the fileselector</h3>
   This function is pretty long. It build the complete GUI.
   Maybe I should have split it up into several smaller functions...
   Note: GUIs like this need to be prototyped interactively!
   <p>
   We will put everything in a widget called "top". This can be a
   toplevel or a frame widget.
   Note: we need access to some of the widgets (the listbox, 
   buttons and entries) in other member functions as well. 
   These widget are defines as instance
   variables in the header file. The other widgets are defined as local
   variables in this function.<pre class=program lang="C++">
  void fileselector::build_gui(widget* top, const char* opts) {
      <i class=comment>// Need some frames to pack things right. All need the same options.</i>
      <i class=comment>// Three frames for the three entries and their labels </i>
      <i class=comment>// (file, path and mask), one frame for the listbox and the </i>
      <i class=comment>// associated scrollbar, and finally one to frame the buttons:</i>
      frame* _fileframe   = new frame(top, "fileframe",  frameopts);
      frame* _maskframe   = new frame(top, "maskframe",  frameopts);
      frame* _pathframe   = new frame(top, "pathframe",  frameopts);
      frame* _boxframe    = new frame(top, "boxframe",   frameopts);
      frame* _buttonframe = new frame(top, "buttonframe",frameopts);
      
      <i class=comment>// Now we can create the file, path and mask entry boxes.</i>
      _fileentry = new entry(_fileframe, "fileentry", entryopts);
      _pathentry = new entry(_pathframe, "pathentry", entryopts);
      _maskentry = new entry(_maskframe, "maskentry", entryopts);
      
      <i class=comment>// Create labels to tell user what all these entries mean:</i>
      label* _filelabel = new label(_fileframe, "filelabel", "-width 5");
      label* _masklabel = new label(_maskframe, "masklabel", "-width 5");
      label* _pathlabel = new label(_pathframe, "pathlabel", "-width 5");

      _filelabel-&gt;text("File:");
      _masklabel-&gt;text("Mask:");
      _pathlabel-&gt;text("Path:");
      
      <i class=comment>// Create a listbox to show the files of the current dir.</i>
      <i class=comment>// We need a scrollbar for large directories.</i>
      _listbox   = new listbox(_boxframe, "box", "-width 40 -height 10");
      scrollbar* _scrollbar = new scrollbar(_boxframe, "scroll");
      _scrollbar-&gt;yview(_listbox); 
      _listbox-&gt;yscroll(_scrollbar);
      
      <i class=comment>// An Ok and Cancel button are pretty basic as well:</i>
      _okbutton = new button(_buttonframe, "ok");
      _okbutton-&gt;text("Ok"); 
      
      _cancelbutton = new button(_buttonframe, "cancel");
      _cancelbutton-&gt;text("Cancel"); 
      
      <i class=comment>// Pack all widgets. </i>
      <i class=comment>// The listbox, scrollbar and the associated frame are a</i>
      <i class=comment>// bit special: they should fill and expand if the</i>
      <i class=comment>// fileselector window is resized by the user.</i>
      <i class=comment>// The entries only expand in the x dimension.</i>
      <i class=comment>// The buttonframe is packed before the boxframe to avoid</i>
      <i class=comment>// that the buttons disappear when the window gets to small.</i>
      <i class=comment>// See packer tutorial for more info...</i>
      _fileframe-&gt;pack(); 
      _maskframe-&gt;pack();
      _pathframe-&gt;pack(); 
      _buttonframe-&gt;pack("-side bottom -fill x"); 
      _boxframe -&gt;pack("-fill both -expand true");
      
      _filelabel-&gt;pack(label_pack_opts);
      _masklabel-&gt;pack(label_pack_opts);
      _pathlabel-&gt;pack(label_pack_opts);
      
      _fileentry-&gt;pack(entry_pack_opts);
      _maskentry-&gt;pack(entry_pack_opts);
      _pathentry-&gt;pack(entry_pack_opts);
      
      _scrollbar-&gt;pack("-side right -fill y -expand false");
      _listbox  -&gt;pack("-side left -fill both -expand true"); 
      
      _cancelbutton-&gt;pack("-side right -padx 10 -pady 10");
      _okbutton    -&gt;pack("-side left  -padx 10 -pady 10");
      
      options(opts);  <i class=comment>// process options</i>
      redirect(top);  <i class=comment>// Redirect self() to top widget</i>
  }

  </pre> <h3>User interaction: Bindings</h3>
   The buttons and entries generate events if they are pressed, hit, etc.
   All we have to do is to bind the event to an object which is 
   capable to process it. Such an object is called a handler.
   The fileselector itself is a pretty good handler, since it is 
   derived (via toplevel and widget) from the handler class.
   So all events will be handled by "this". 
   
   We want to handle events corresponding to pressing the 
   buttons (ok or cancel), selecting a file from the listbox (select),
   and hitting Return in the entry widgets. 
   A Return in the file entry is interpreted to be the same as pressing
   the Ok button. Return in the mask entry should set a new mask
   and update (rescan) the directroy contents in the listbox.
   A Return in the path entry should probably result in a change dir,
   and an update of the filebox. We deal with this actions in the
   handlers application operator()().
  
   In general, it is convenient to supply an extra argument to the event,
   so we can determine what actually happened.<pre class=program lang="C++">
  void fileselector::bindings() {
      _cancelbutton-&gt;bind(this, "Cancel");            <i class=comment>// cancel pressed</i>
      _okbutton-&gt;bind(this, "Ok");                    <i class=comment>// ok button pressed</i>
      _listbox-&gt;bind(this, "Select-Double" );  <i class=comment>// double click is default binding</i>
      _listbox-&gt;bind("&lt;Button-1&gt;", this, "Select" ); <i class=comment>// single click</i>
      _fileentry-&gt;bind("&lt;Return&gt;", this, "Ok") ;      <i class=comment>// like pressing ok</i>
      _maskentry-&gt;bind("&lt;Return&gt;", this, "MaskReturn") ; <i class=comment>// new mask</i>
      _pathentry-&gt;bind("&lt;Return&gt;", this, "PathReturn") ; <i class=comment>// new path</i>
  }

  </pre> <h3>Event types</h3>
   There are two types of events:<ol>
        <li>Events generated by the Tcl interpreter, since our
            fileselector can be used as a Tcl command as well.
        <li>Events generated by the fileselector's components, as defined
            above. 
   </ol>
   As an example of the first event, assume a user feeding the following
   code to the interpreter:
   <pre>
   hush> fileselector .f                # Generates event
   hush> ...
   </pre>
  
   The statement should create a new fileselector called .f
   
   <h4>Creating widgets</h4>
   This event is typical for all widgets and 
   a special member functions is designed to handle this:
   the create() function defined below.
  
   <h4>Dispatching of other event</h4>
   The default dispatch() member of the widget class 
   will call the create() function to handle 
   the event discribed above.
   Additionally, it will check the widget's jump table to see if
   one of the entries matches the current event.
   If both fail, it will call the widgets operator to handle
   the "unknown" event.<pre class=program lang="C++">
  widget* fileselector::create(int argc, char* argv[]) { <i class=comment>// create new widget</i>
      fileselector *f = new fileselector(argv[0]);    
      f-&gt;options(flatten(--argc, ++argv));            <i class=comment>// process options</i>
      tk-&gt;result(f-&gt;path());                          <i class=comment>// return path to tk</i>
      _register(f);                                   <i class=comment>// register for deletion</i>
      return f;
  }

  int fileselector::operator()() {  
      int argc = _event-&gt;argc(); char **argv = _event-&gt;argv();
      string first = argv[0];
      if (first == type()) { 
          create(--argc, ++argv);
          return OK;
      }
      string s = _event-&gt;arg(1);
      if (s == "Select" || s == "Select-Double") {
          <i class=comment>// A select on the listbox:</i>
          _fileentry-&gt;del(); <i class=comment>// delete old filename</i>
          
          int index = _listbox-&gt;selection();
          char* sel = _listbox-&gt;get(index);
          char* selection = new char[strlen(sel) +1];
          strcpy(selection, sel);
          
          <i class=comment>// Perhaps the selection is a directory ...</i>
          sprintf(buf,"file isdirectory \"%s\"", selection);
          int isdir = atoi(tk-&gt;evaluate(buf));
          
          if (isdir) {
              if (s == "Select-Double") dirpath(selection);
          } else {
              _fileentry-&gt;insert(selection); 
              if (s == "Select-Double") {
                  strcpy(_filename, _fileentry-&gt;get());
                  destroy();
              }
          }
          delete selection;
      } else if (s == "Cancel") {
          strcpy(_filename, "");  <i class=comment>// Empty string on cancel</i>
          destroy(); 
      } else if (s == "Ok") {
          strcpy(_filename, _fileentry-&gt;get());
          destroy(); 
      } else if (s == "MaskReturn") {
          <i class=comment>// A [Return] in the mask entry. Update mask and listbox</i>
          mask(_maskentry-&gt;get());
      } else if (s == "PathReturn") {
          <i class=comment>// A [Return] in the path entry. Change directory.</i>
          dirpath(_pathentry-&gt;get());
      } else if (s == "get") {  
          tk-&gt;result(get());          <i class=comment>// Pass selected filename to toolkit</i>
      } else if (s == "rescan") { 
          rescan();
      } else self()-&gt;eval(quote(--argc, ++argv));
      return OK;
  }

  void fileselector::options(const char*) { 
      <i class=comment>// not implemented ...</i>
  }

  </pre> <h4>Error handling</h4>
   An error has occured, probably in the arguments of
   the script command. Print usage and return error.<pre class=program lang="C++">
  int fileselector::usage(const char* msg) const {
      cerr &lt;&lt; type() &lt;&lt; ": " &lt;&lt; msg &lt;&lt; endl;
      cerr &lt;&lt; "Usage: " &lt;&lt; endl;
      cerr &lt;&lt; type() &lt;&lt; " &lt;pathname&gt; [options]" &lt;&lt; endl;
      cerr &lt;&lt; "Or: " &lt;&lt; endl;
      cerr &lt;&lt; "&lt;pathname&gt; get" &lt;&lt; endl;
      cerr &lt;&lt; "&lt;pathname&gt; rescan" &lt;&lt; endl;
      return ERROR;
  }

  </pre>