<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 3.0//EN">
 <h2>Example</h2>
   Example application showing to use of the fileselector widget in C++
   This program is a hush Tcl interpreter extending Tcl with a
   fileselector command.<pre class=program lang="C++">
  #include &lt;iostream.h&gt;           <i class=comment>// for C++ stream I/O</i>
  #include &lt;hush/kit.h&gt;           <i class=comment>// for kit (tk) class</i>
  #include &lt;hush/session.h&gt;       <i class=comment>// for session class</i>
  #include "fileselector.h"       <i class=comment>// for new fileselector widget</i>
  </pre> <h3>Application class</h3>
   This class will model our small example application:<pre class=program lang="C++">
  class application : public session {
    public:
      application(int argc, char* argv[]) : session(argc,argv, "example") {}
      void prelude();  <i class=comment>// do inits here </i>
      int main();      <i class=comment>// main setup here </i>
  };

  </pre> <h3>Prelude</h3>
   <a name=NewCommand>Introduce "fileselector" as a new Tcl command</a>. 
   Note: the prelude is a good place to do this kind of initializations.
   since it is always exectuted before any command line supplied scripts.
   This means that we can use our new command in such scripts as well!
   See also the example script: script.tcl.
   <p>Note the use of _register() to register the new handler for
   deletion. See <a href="http://www.cs.vu.nl/~hush/tutorials/garbage/">Garbage
   collection</a> for more information on _register.<pre class=program lang="C++">
  void application::prelude() {
      fileselector* f = new fileselector; _register(f);
      tk-&gt;bind(f-&gt;type(), f);     <i class=comment>// Create script command </i>
  }

  </pre> <h3>Main</h3>
   We will create a fileselector widget, just to see how this works in C++.
   The title() member is inherited from the toplevel class.
   We will not list the current working directory, but the users
   home directory (~) by using the dirpath() member.
   <p>
   Note: the main routine will not be called if a script
   is supplied on the command line. So the stuff below
   does not appear if we use this program as an interpreter.<pre class=program lang="C++">
  int application::main() {
      widget* root = tk-&gt;root();
      fileselector* f = new fileselector(root, "just_an_example");
      f-&gt;title("Select a file");          
      f-&gt;dirpath("~");                    <i class=comment>// cd ~ ($HOME)</i>
      cout &lt;&lt; "Wait until file selected." &lt;&lt; endl;
      const char* filename =  f-&gt;get();
      if (filename) 
          cout &lt;&lt; "You selected file: " &lt;&lt; filename &lt;&lt; endl;
      else
          cout &lt;&lt; "You canceled fileselection." &lt;&lt;endl;

      return OK;
  }

  </pre> <h3>C main routine</h3>
   This the good old C main function which starts up the whole program.
   The compiler will generate code which will call this function
   parameterized with the command line arguments.<pre class=program lang="C++">
  int main(int argc, char* argv[]) {
      application* a = new application(argc,argv);
      int result = a-&gt;run();
      delete a;
      session::statistics();
      return result;
  }
  </pre>