<!doctype html public "-//W3C//DTD HTML 3.2//EN">
  <title>Garbage collection in hush</title>
   <h1>Garbage collection in hush</h1>
   
   <p>This document describes the garbage collection mechanisms of hush. 
    
   <h2>Introduction</h2> 
   
   <p>It is our firm conviction hat garbage collection should be
    supported by the language environment, as the developers of Java
    have realized.  In C++ this is unfortunately not the case.
    Nevertheless, the <i>hush</i> library is becoming increasingly
    strict in collecting garbage.  It may clean up itself one day.
    Many classes do provide a <i>register</i> method that allows the
    user to declare which (component) objects must be deleted when
    cleaning up.  In most cases you do not have to be very strict in
    cleaning up obsolete objects, except when resource management
    (such as occupying the audio device) is involved.  A hush
    programmer should never, or only very rarely, delete an object
    explicitly.  Rather, one must register each newly created object
    to a (handler) object that provides the most suitable scope.  For
    example some objects live as long as a session, whereas for other
    objects the widget they belong to is a more suitable scope. See
    garbage(4) and handler(4).
    
   <h2>Automatic garbage collection for widgets</h2> 
   
   <p>Widgets that are created as child of an ancestor widget are
    automatically registered for eventual garbage collection.  As a
    result, deleting a toplevel widget will result in the subsequent
    deletion of all its children. Note that this strategy mirrors the
    Tk destroy algorithm.  <p>The root window will be automatically
    destroyed after quitting the program. This means that all direct
    and indirect children of the root window are automatically deleted.
    
   <h4>Examples</h4> 
   
   <p>The following widgets are all deleted automatically:
   <pre>
    widget* root = tk->root();       // Get widget of root window
    frame* f1 = new frame(root, "f1");
    button* b1 = new button(f1, "b1");
    ... etc
   </pre>
   <p>The following widgets are all deleted by deleting top:
   <pre>
    toplevel *top = new toplevel("t1");
    frame* f2 = new frame(top, "f2");
    button* b2 = new button(f2, "b2");
    ... etc
   </pre>
   
   <h2>Implicit garbage collection for other hush objects</h2>
   
   <p>The hush handler class and its descendants have a
    handler::_register function which can be used to register objects
    for deletion. Every object registered by a specific handler will
    be deleted by the destructor of the handler object.  Since every
    widget is derived from handler, you can use the widget's _register
    member for objects which need to be destructed when the widget
    itself is destructed. The kit's _register can be used for objects
    which are needed during a complete hush application.
    
   <h4>Examples</h4>
   <p>Handler will be deleted when kit is deleted:
   <pre>
    handler* h = new myhandler;
    tk->bind("do-something",h);
    tk->_register(h);          
   </pre>
   
   <p>Handler will be deleted when button is deleted:
   <pre>
    button* b = new button(parent, "b");
    handler h = new myhandler;
    b->_register(h);
   </pre>
   
   <h2>Garbage Statistics</h2> 
   
   <p>After destruction of your session, all hush object should have
    been deleted. You can check this by calling the static
    session::statistics member function.
    
   <h4>Example</h4>
   The following main function, for a class app derived from session:
   <pre>
    int main(int argc, char **argv) {
      app *a = new app(argc,argv);  // create a session
      int result = a->run();        // run session
      delete a;                     // delete session
      session::statistics();        // print statistics
      return result;                // return result of run
    }
   </pre>
   
   prints something like:
   
   <pre>
    
    Statistics:   created - deleted = ? 
    session::       2 - 2 = 0
    kit::           2 - 2 = 0
    handler::       24 - 24 = 0
    event::         1 - 1 = 0
    widget::        8 - 8 = 0
    item::          4 - 4 = 0
    binding::       4 - 4 = 0
    string::        0 - 0 = 0
    iter&lt;T&gt;::       0 - 0 = 0
    list&lt;T&gt;::       0 - 0 = 0
    dictionary&lt;T&gt;:: 0 - 0 = 0
    garbage&lt;T&gt;::    0 - 0 = 0
    sublistrep::    0 - 0 = 0
    baselist::      0 - 0 = 0
    cell::          19 - 19 = 0
   </pre>
   
