<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> An M/M/1 Queue</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" An M/M/1 Queue">
<meta name="keywords" value="sim">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html365 HREF="node22.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html363 HREF="node19.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html357 HREF="node20.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html367 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html366 HREF="node22.html"> Queue behavior</A>
<B>Up:</B> <A NAME=tex2html364 HREF="node19.html"> Queuing events</A>
<B> Previous:</B> <A NAME=tex2html358 HREF="node20.html"> Queues and resources</A>
<BR> <HR> <P>
<H2><A NAME=SECTION00042000000000000000> An M/M/1 Queue</A></H2>
<P>
An M/M/1 queue has an exponential inter-arrival and
service time and a single server. 
We are interested in the statistics of the size of the queue and the waiting times of a
customer, with varying means for the inter-arrival times and service times.
Consider the following process-oriented simulation program.
<P>
We first include the library and define the phases for a customer.
The simulation object, the resource, the generators, the queue and the
means for the arrival and service times, as well as
the customer and server entities are declared.
<PRE> 
   #include &lt;sim/sim.h&gt;
   
   enum {ARRIVAL,DEPARTURE};
   
   simulation* sim;
   resource* r;
   queue* q;
   double meanarrival,meanservice;  // mean inter-arrival and service time
   generator* g;
   
   class customer : public entity
   {
   public :
     customer(int ph);
     virtual int operator()();
     int arrival();
     int departure();
   };
 
   class server : public entity
   {
   public :
     server();
     virtual int operator()();
   };
</PRE>
<P>
A customer arrives or departs, so it receives a phase, when created.
The behavior of the entity <i> customer</i> is implemented in its application
operator. When a customer arrives, first a new customer is scheduled
to arrive. We take the inter-arrival time to be exponential, with <i> meanarrival</i> as
the mean inter-arrival time. Then the customer is appended to the queue. When
the customer is served the resource is acquired by the server, a departing
customer then releases the resource and is terminated. The default amount
of 1 is released and acquired each time.
<PRE>   customer::customer(int ph) : entity(ph)
   {
   }
 
   int customer::operator()()
   {
     switch (phase())
     {
       case ARRIVAL :
         return arrival();
       case DEPARTURE :
         return departure();
     }
     return FALSE;
   }
 
   int customer::arrival()
   {
     customer* c = new customer(ARRIVAL);     // schedule new customer
     sim -&gt; schedule(c,(g -&gt; exponential(meanarrival)));
     q -&gt; append(this);             // append to the queue
     return OK;
   }
 
   int customer::departure()
   {
     r -&gt; release();                // release the resource
     sim -&gt; terminate(this);        // terminate this customer
     return OK;
   }
</PRE>
<P>
The behavior of the entity <i> server</i> does not depend on a phase.
Every time it is activated, it takes a
customer from the queue if possible, i.e. if the queue is not empty
and if the required amount of the resource (in this case the default
amount of 1) is available.
If it takes a customer, it acquires the resource, adjusts the phase
of the customer to DEPARTURE and schedules the customer to depart,
after it has been served.
<PRE>   server::server() : entity()
   {
   }
 
   int server::operator()()
   {
     // can we serve a customer ?
     if ( (!(q -&gt; empty())) &amp;&amp; (r -&gt; available()) )
     {
       customer* c = (customer *)q -&gt; removefront();    // take first
       r -&gt; acquire();                        // acquire the resource
       sim -&gt; schedule(c,g -&gt; exponential(meanservice));
       c -&gt; phase(DEPARTURE);    // adjust phase and schedule departing
     }
     return OK;
   }
</PRE>
<P>
The function <i> application::main</i> creates the queue and two histograms,
that are used for generating reports on the size of the queue and the waiting times of 
a customer. A resource with an initial amount of 1 is created.
The simulation is set up by scheduling a customer to arrive immediately
and making the server conditional. The server is never passivated,
so it stays on the conditional list for the entire simulation
and is activated, whenever a customer is.
If the simulation has various queues and servers the server should
be scheduled whenever a customer departs to make the program more
efficient.
<PRE>   class application : public session
   {
   public :
     application(int argc,char** argv);
     int main();
   };
 
   application::application(int argc,char** argv) : session(argc,argv,&quot;M/M/1-queue&quot;)
     {}
 
   int application::main()
   {
     cout &lt;&lt; &quot;mean time between two arrival events&quot; &lt;&lt; endl;
     cin &gt;&gt; meanarrival;
     cout &lt;&lt; &quot;mean time between two service events&quot; &lt;&lt; endl;
     cin &gt;&gt; meanservice;
 
     // create objects and reports
     sim = new simulation();
     g = new generator(8,2,9);
     histogram* h1 = new histogram(&quot;.h1&quot;,&quot;-type weighted -form statistics -title size&quot;);
     histogram* h2 = new histogram(&quot;.h2&quot;,&quot;-type frequency -form statistics -title waitingtimes&quot;);
     tk -&gt; pack(h1);
     tk -&gt; pack(h2);
     tk -&gt; update();
     q = new queue();
     q -&gt; reportsize(h1);    // generate a report on the size of the queue
     q -&gt; reportwaiting(h2); // generate a report on waiting times
     r = new resource(1);
 
     // set up and run the simulation
     customer* c = new customer(ARRIVAL);
     sim -&gt; schedule(c,0.0);         // initially a customer arrives
     server* s = new server();
     sim -&gt; hold(s);                 // the server is conditional
     sim -&gt; run(10000.0);
 
     delete h1;
     delete h2;
     delete q;
     delete sim;
     return 0;
   }
 
   main(int argc,char** argv)
   {
     session *s = new application(argc,argv); // create the session
     return s -&gt; run();                       // start the session
   }
</PRE>
<P>
<BR> <HR><A NAME=tex2html365 HREF="node22.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html363 HREF="node19.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html357 HREF="node20.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html367 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html366 HREF="node22.html"> Queue behavior</A>
<B>Up:</B> <A NAME=tex2html364 HREF="node19.html"> Queuing events</A>
<B> Previous:</B> <A NAME=tex2html358 HREF="node20.html"> Queues and resources</A>
<BR> <HR> <P>
<BR> <HR>
<P><ADDRESS>
<I>A Eliens <BR>
Tue Oct 31 09:27:21 MET 1995</I>
</ADDRESS>
</BODY>
