<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> Queue behavior</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" Queue behavior">
<meta name="keywords" value="sim">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html376 HREF="node23.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html374 HREF="node19.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html368 HREF="node21.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html378 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html377 HREF="node23.html"> The resource class</A>
<B>Up:</B> <A NAME=tex2html375 HREF="node19.html"> Queuing events</A>
<B> Previous:</B> <A NAME=tex2html369 HREF="node21.html"> An M/M/1 Queue</A>
<BR> <HR> <P>
<H2><A NAME=SECTION00043000000000000000> Queue behavior</A></H2>
<P>
In this section we look at some examples of how more complex scenarios
involving queues can be modeled. 
<H4><A NAME=SECTION00043010000000000000> Queue swapping</A></H4>
<P>
This behavior occurs when a member of the queue leaves in preference for an
alternative queue. We can model the swapping of an event <i> e</i> between
the queues <i> q1</i> and <i> q2</i> as follows :
<PRE>   if (condition)
   {
     if (q1 -&gt; extract(e))         // from every position
       q2 -&gt; append(e);            // append to second queue
   }
</PRE>
where condition could be something as the checking of the sizes of the
two queues.
<H4><A NAME=SECTION00043020000000000000> Balking</A></H4>
<P>
If a queue is too long it usually deters further additions. This
phenomenon is called balking and its implementation is straightforward :
<PRE>   if (q -&gt; size() &lt; MAX)
     q -&gt; append(e);                // append if possible
</PRE>
<P>
The event is appended when there is room for it.
<H4><A NAME=SECTION00043030000000000000> Reneging</A></H4>
<P>
Reneging is the premature departure of a customer from a queue before
it has had any service. This is typically the case when its waiting
time exceeded some upper limit. We could model this by checking the
entire queue with the <i> queue::cancel</i> method, but more efficient is the following addition to the <i> arrival</i> event :
<PRE>  
   cancel* cn = new cancel(this);   // give customer to the cancel event
   sim -&gt; schedule(cn,MAX);         // schedule cancel event
</PRE>
and the extracting of the event in the <i> cancel::operator</i> function.
The <i> queue::extract</i> function does nothing if the event is not in
the queue.
<PRE>   int cancel::operator()()
   {
     if (q -&gt; extract(e))         // extract and terminate e if in the queue
       sim -&gt; terminate(e);
     return OK;
   }
</PRE>
<P>
<H4><A NAME=SECTION00043040000000000000> Service pre-emption</A></H4>
<P>
Pre-emption occurs when a customer being served is interrupted by the
server in favor of a new customer that has a higher priority.
The interrupted customer can then wait to complete its service,
wait to be served again or simply leave the system.
The interrupting arrival then takes the place of the interrupted event.
Consider the following behavior of the server entity of the 
M/M/1 queue. Events should then receive a queuing priority when created 
and the event currently being served should be set to NULL when terminated.
<PRE>   int server::operator()()
   {
     if (!(q -&gt; empty()))                // if an event waiting
       if (!current)                     // if nothing being served
         return serve();                 // just serve
       else if (q -&gt; front() -&gt; queuingpriority() &gt; current -&gt; queuingpriority())
         return preempt();               // front has a higher priority
   }
 
   int server::serve()
   {
     customer* c = (customer* )q -&gt; removefront();
     r -&gt; acquire();
     current = c;                          // record as current
     sim -&gt; schedule(c,g -&gt; exponential(meanservice));
     c -&gt; phase(DEPARTURE);
     return OK;
   }
 
   int server::preempt()
   {
     sim -&gt; passivate(current);            // extract departure
     r -&gt; release();                       // and depart
     sim -&gt; terminate(current);
 
     customer* c = (customer* )q -&gt; removefront();
     r -&gt; acquire();
     current = c;                          // record as current
     sim -&gt; schedule(c,g -&gt; exponential(meanservice));
     c -&gt; phase(DEPARTURE);
     return OK;
   }
</PRE>
<P>
If the front can be served we look if there is currently a customer
being served (the current variable is set to NULL in the <i> departure</i>
event). If so the front is served only if its
queuing priority is higher. The front is then recorded as the current.
<H4><A NAME=SECTION00043050000000000000> Time dependencies</A></H4>
<P>
The arrival and service rates have been considered constant so far.
This is not always realistic. The arrival rate at a supermarket shows
a noticeable peak around evening for example. We can easily model
this by constructing an <i> arrivaltime</i> function as follows :
<PRE> 
   double arrivaltime()
   {
     double t = g -&gt; exponential(MEAN1);
     if (sim -&gt; clock() + t &lt; TIME)        // use the appropriate mean
       return t;                           // based on the current
     else                                  // simulation time
       return (g -&gt; exponential(MEAN2));
   }
</PRE>
<P>
and we have different arrival rates at different times, if this function
is used when a new arrival is scheduled.
<P>
<BR> <HR><A NAME=tex2html376 HREF="node23.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html374 HREF="node19.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html368 HREF="node21.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html378 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html377 HREF="node23.html"> The resource class</A>
<B>Up:</B> <A NAME=tex2html375 HREF="node19.html"> Queuing events</A>
<B> Previous:</B> <A NAME=tex2html369 HREF="node21.html"> An M/M/1 Queue</A>
<BR> <HR> <P>
<BR> <HR>
<P><ADDRESS>
<I>A Eliens <BR>
Tue Oct 31 09:27:21 MET 1995</I>
</ADDRESS>
</BODY>
