<!DOCTYPE HTML PUBLIC "-//W3O//DTD W3 HTML 2.0//EN">
<!Converted with LaTeX2HTML 95.1 (Fri Jan 20 1995) by Nikos Drakos (nikos@cbl.leeds.ac.uk), CBLU, University of Leeds >
<HEAD>
<TITLE> The Dining Philosophers</TITLE>
</HEAD>
<BODY>
<meta name="description" value=" The Dining Philosophers">
<meta name="keywords" value="sim">
<meta name="resource-type" value="document">
<meta name="distribution" value="global">
<P>
 <BR> <HR><A NAME=tex2html208 HREF="node8.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html206 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html200 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html210 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html209 HREF="node8.html"> The event class</A>
<B>Up:</B> <A NAME=tex2html207 HREF="node6.html"> The event-based approach</A>
<B> Previous:</B> <A NAME=tex2html201 HREF="node6.html"> The event-based approach</A>
<BR> <HR> <P>
<H3><A NAME=SECTION00033100000000000000> The Dining Philosophers</A></H3>
<P>
The problem is as follows.
Five philosophers sit around a table with five chopsticks in between. They think and if they are hungry and if two chopsticks are available, they eat. If a philosopher gets
hungry and s/he can not acquire a chopstick, the philosopher waits until
s/he can. The philosopher does not
think, if s/he is waiting or eating. We are interested in the fraction
of the time, that a philosopher actually thinks.
<P>
To model this problem, three events can be identified, <i> eat</i>, <i> think</i> and <i> await</i>.
The corresponding classes are derived from the class <i> event</i>. Furthermore we
need a chopstick for every philosopher. These are represented as
a <i> resource</i>. The thinking times are gathered in an instance
of the class <i> histogram</i> and the <i> generator</i> takes care of the
variations in the time needed to think and eat.
<P>
We develop this program in the following steps.
First, the library is included as <i> sim.h</i>. The declarations of the global variables
and constants follow after that. The time unit in this simulation is an
hour, so a philosopher has a mean eating time of two hours and a mean
thinking time of five hours. The duration of the simulation is a year.
Finally, we define the various events.
<PRE>   #include &lt;sim/sim.h&gt;
 
   const double duration = 52*7*24.0;       // a year
   const int number = 5;                    // philosophers
   const int eatingtime = 2;                // 2 hours
   const int thinkingtime = 5;              // 5 hours
   simulation* sim;
   generator* g;
   resource* chopstick[number];
   histogram* thinking;
 
   class eat : public event
   {
   public :
     eat(int i);                 // constructor, taking identity
     virtual int operator()();   // function operator
   private :
     int id;                     // identity of the philosopher
   };
 
   class think : public event
   {
   public : 
     think(int i);               // constructor, taking identity
     virtual int operator()();   // function operator
   private :
     int id;                     // identity of the philosopher
   };
 
   class await : public event
   {
   public :
     await(int i);               // constructor, taking identity
     virtual int operator()();   // function operator
   private :
     int id;                     // identity of the philosopher
   };
</PRE>
<P>
Next, we implement the various events.
An event is given its functionality by deriving it from
the class <i> event</i> and overriding its <i> function operator</i>.
<P>
The logic of the <i> eat</i> event is that the philosopher eats for a
random time, exponentially distributed with a mean eating time.
So, we first determine the actual eating time and schedule a <i> think</i> event
to be activated after this eating time. The <i> eat</i> event can be
terminated.
<PRE>   eat::eat(int i) : event()
   {
     id = i;                                  // set identity
   }
 
   int eat::operator()()
   {
     double t = g -&gt; exponential(eatingtime); // determine eating time
     think* th = new think(id);               // create a thinking event
     sim -&gt; schedule(th,t);                   // schedule thinking
     sim -&gt; terminate(this);                  // terminate this eat event
     return OK;
   }
</PRE>
<P>
If a philosopher starts to think, the philosopher first releases
both chopsticks. The thinking time is determined and
a sample is made of the percentage of this thinking time towards the
total time. An <i> await</i> event is
scheduled and the <i> think</i> event is terminated.
<PRE>   think::think(int i) : event()
   {
     id = i;                                // set identity
   }
 
   int think::operator()()
   { 
     chopstick[id] -&gt; release();            // release left chopstick
     chopstick[(id+1)%number] -&gt; release(); // release right
     double t = g -&gt; exponential(thinkingtime);    // determine thinking time
     thinking -&gt; sample(id,t/duration*100); // add a sample (%)
     await* aw = new await(id);             // create await event
     sim -&gt; schedule(aw,t);                 // schedule waiting
     sim -&gt; terminate(this);                // terminate thinking
     return OK;   
   }
</PRE>
<P>
The <i> await</i> event acquires the left and right chopstick and
schedules an <i> eat</i> event immediately, if both chopsticks are
available. The <i> await</i> event is passivated as it could be on the
conditional list. If no chopsticks are available, the <i> await</i>
event stays on the conditional list or, if it was not conditional
as is the case the first time it is activated, it is added
to the conditional list.
<PRE>   await::await(int i) : event()
   {
     id = i;                                 // set identity
   }
 
   int await::operator()()
   {
     if ( (chopstick[id] -&gt; available()) &amp;&amp;      // available ?
        (chopstick[(id+1)%number] -&gt; available()) )
     {
       chopstick[id] -&gt; acquire();               // acquire left
       chopstick[(id+1)%number] -&gt; acquire();    // acquire right
       eat* e = new eat(id);
       sim -&gt; passivate(this);      // extract from conditional list
       sim -&gt; schedule(e,0);        // schedule eat event immediately
       sim -&gt; terminate(this);      // terminate await event
     }
     else if (!conditional())       // not on conditional list
       sim -&gt; hold(this);           // add to conditional list
     return OK;
   }
</PRE>
<P>
The following step is the definition and implementation of an application,
that is derived from <i> session</i>.
The <i> application::main</i> function first creates the simulation object. Furthermore, 
a frequency histogram and five resources that represent the chopsticks are created.
The histogram is created with its widget path and with its (default) options.
Afterwards it is packed to the display.
The simulation starts with all philosophers waiting and runs for a
year (52*7*24 hours). After running the simulation, the resulting histogram is printed.
<P>
<PRE>   class application : public session
   {
   public :
     application(int argc,char** argv);
     int main();                        // must be defined
   };
 
   application::application(int argc,char** argv) : session(argc,argv,&quot;philosophers&quot;)
     {}
 
   int application::main()   // tk is an instance variable of session
   {
     sim = new simulation(); 
     g = new generator(80,20,19);          // gets three seeds
     thinking = new histogram(&quot;.h&quot;,&quot;-columns 5 -title thinkingtime&quot;);
     tk -&gt; pack(thinking);                 // add to display;
     tk -&gt; update();                       // update display;
     for (int i=0;i&lt;number;i++)
     {
       chopstick[i] = new resource(1);     // create chopsticks
       await* aw = new await(i);           // schedule each
       sim -&gt; schedule(aw,0);              // philosopher waiting
     }
     sim -&gt; run(duration);                 // run for duration time units
     cout &lt;&lt; (*thinking) &lt;&lt; endl;          // print resulting histogram
     delete thinking;
     delete sim;
     return 0;                             // successful termination
   }
</PRE>
<P>
The <i> main</i> function then creates and starts the application.
<P>
<PRE>   main(int argc,char** argv)
   {
     session* s = new application(argc,argv);        // create the session
     return s -&gt; run();                              // start the session
   }
</PRE>
<P>
<BR> <HR><A NAME=tex2html208 HREF="node8.html"><IMG ALIGN=BOTTOM ALT="next" SRC="http://www.cs.vu.nl/images/latex2html//next_motif.gif"></A>   <A NAME=tex2html206 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="up" SRC="http://www.cs.vu.nl/images/latex2html//up_motif.gif"></A>   <A NAME=tex2html200 HREF="node6.html"><IMG ALIGN=BOTTOM ALT="previous" SRC="http://www.cs.vu.nl/images/latex2html//previous_motif.gif"></A>   <A NAME=tex2html210 HREF="node1.html"><IMG ALIGN=BOTTOM ALT="contents" SRC="http://www.cs.vu.nl/images/latex2html//contents_motif.gif"></A>      <BR>
<B> Next:</B> <A NAME=tex2html209 HREF="node8.html"> The event class</A>
<B>Up:</B> <A NAME=tex2html207 HREF="node6.html"> The event-based approach</A>
<B> Previous:</B> <A NAME=tex2html201 HREF="node6.html"> The event-based approach</A>
<BR> <HR> <P>
<BR> <HR>
<P><ADDRESS>
<I>A Eliens <BR>
Tue Oct 31 09:27:21 MET 1995</I>
</ADDRESS>
</BODY>
