<h4>User interface widgets</h4>
<include file=navigate>
The (<i>hush</i>) widget library contains
classes for a variety of user interface widgets,
including the classes <i>button</i>,
<i>menu</i>, <i>text</i>, <i>canvas</i>,
<i>filechooser</i> and many more.
As explained before,
a widget allows for binding an independently defined
handler to user actions.
This may result in a code fragment like:
<pre>
  button* b = new button(".b");
  handler* h = new help_handler(b);
  b->text("Help");
  b->bind(h);
</pre>
We assume that <i>help_handler</i> does something
meaningful, like displaying a hypertext help file.
<p>
Alternatively, a widget may declare itself to be its 
own handler.
In that case we derive a new class from <i>button</i>
and define the functionality (previously defined externally
in the <i>help_handler</i>) in the new class itself,
as illustrated below:
<pre>
  class help_button : public button {
  public:
  help_button( char* path ) : button( path ) {
	this->text("help");
	this->bind(this);
	}
  int operator()(); // does what help_handler did
  };
</pre>
Dependent on circumstances, either one of the approaches
may be the most convenient.
Allowing for delegating to an external handler
helps to avoid cluttering the class name space.
On the other hand, employing inheritance may result
in a significant reuse of initialization code.
<include file=navigate>
