<h4>Simulation modeling support</h4>

Our interest in discrete event simulation was accidentally
aroused by the need to provide support for Business Mathematics
courses.
The simulation package we developed is
fully integrated with <i>hush</i>, which allows the simulation
developer
to use the animation and hypertextual facilities offered
by <i>hush</i>.
<p>
The simulation package provides the following classes:
<i>simulation</i> (which contains the scheduler),
<i>event</i> (representing the events),
<i>entity</i> (representing processes consisting of events),
<i>generator</i> (which provides a variety of random
distribution functions),
<i>resource</i> (to model passive objects conveniently),
<i>histogram</i> (to plot the results of a simulation)
and <i>analysis</i> (to perform statistical analyses on the results).
<p>
The public interface of the class <i>event</i>
(used in the simulation package) looks as follows:
<pre>
  interface event : handler {

    int operator()(); // called when activating the event

    void timestamp();
    double timespent(); // time spent since last stamp
    ...
  };
</pre>
In addition it contains functions to determine
the status of an event.
For example, an event may be <i>conditional</i>
(which means that it will be tried for each cycle of the scheduler)
or <i>pending</i>
(which means that its activation will be delayed until
it becomes active or conditional).
<p>
The interface of the class <i>entity</i> (representing processes)
looks as follows:
<pre>
  interface entity : event {

    int phase();
    void phase( int ph );
    };
</pre>
An entity may be regarded as a 'structured event',
that is as an active object with phases in its
life-cycle.
In our approach, however, an explicit switch on
the entity's phases must be included in the
<i>operator()</i> function.
<p>
From the perspective of our hypermedia framework,
we benefit from the simulation package by
having an efficient scheduler at our disposal
and hence a convenient way to employ application-defined
events.
In addition, we extended the simulation package with a real time option,
that allows us for example to define easily a clock-synchronized
slide show. (See the examples???)


