@ <a href=@talk-ex-account.html target=_top>The account contract</a>
<include file=navigate>
An ordinary account supports methods to <i>deposit</i>
and <i>withdraw</i> money and allows for
inquiring after the <i>balance</i> as well.
As an invariant of an ordinary account we may assume
that the balance must be equal to or greater than zero.
The <i>account</i> class below gives an example
of how to realize a contract in C++ code.


<slide id=account-class cap="The account class">
@ The account class
<program>
class account {
public:
account() { _balance = 0; promise(account::invariant()); }
 
virtual float balance() const { return _balance; }
 
 
void deposit(float x) {
        require( x >= 0 );  ///// check precondition
        hold(); ///// to save the old state
        _balance += x; 
        promise( balance() == old_balance + x && invariant() );
        }
 
void withdraw(float x) {
        require( x <= balance() );  ///// check precondition
        hold(); ///// to save the old state
        _balance -= x; 
        promise( balance() == old_balance - x && invariant() );
        }
 
protected:
   virtual bool invariant() { return balance() >= 0; }
   virtual void hold() { old_balance = balance(); }
private:
   float _balance;
   float old_balance; ///// additional variable
};
</program>
</slide>

Pre- and post-conditions are realized by calling
respectively the <i>require</i> and <i>promise</i> macros,
which are simply variants of the C <i>assert</i>
macro.
To check whether the post-conditions for
(state-changing) methods are satisfied,
a virtual function <i>hold</i>
is introduced that copies the value of (selected parts of)
the state of the object.
We use the prefix <i>old</i> to indicate these
additional instance variables.
To check whether a contract is satisfied runtime,
we must test that the invariant is satisfied
when the object is created.
Further, when a method is invoked,
we must test its pre-condition and establish that
its post-condition is satisfied and that the object invariance
is not violated.
<p>
A <i>credit_account</i> may be considered to be an improvement
over an ordinary account since it allows us to
draw more money from it.
One possible realization is given below.

<slide id=account-credit cap="A Credit Account">
<program>
class credit_account : public account {
public:
credit_account(float x) { _maxcredit = x; _credit = 0; }

float balance() { return _balance + _credit; }

float credit(float x) { 
	require( x >= 0 && x + _credit <= _maxcredit );
	hold();
	_credit += x;
	promise( _credit = old_credit + x );
	promise( balance() == old_balance + x && invariant() );
	}


protected:
  bool invariant() {
	  return _credit <= _maxcredit && account::invariant();
	  }
  void hold() { old_credit = _credit; account::hold(); }
private:
  float _maxcredit, _credit;
  float old_credit;
};
</program>
</display>

We merely introduce one additional method to raise
the credit, and extend the function <i>hold</i>
to keep track of the <i>_credit</i> variable.
Note that the invariant is indeed strengthened
and that the pre-condition of <i>withdraw</i>
has (implicitly) been weakened.
An alternative solution would consist
of redefining <i>deposit</i> and <i>withdraw</i>.
This is left as an exercise.
