<slide id=ex-events cap="Propagating update events">
<h4><a href=@talk-ex-event.html target=_top>Propagating update events</a></h4>
<p>
The <i>Model-View</i> or <i>Observer</i> pattern [_GOF]
provides a general way to deal with information
updates that may affect multiple dependent objects.
Similar functionality can be obtained by employing events.
An <i>event</i> class can be defined as follows:
</slide>

<slide id=events-class cap="The event class">
@ The event class
<program><pre>
  class event {
  public:
  	virtual void operator()() {}      ///// activate
  
  	void dependent(event* e);
  	void update();     ///// to process dependent events                     
  private:
  	set<event*> _dep;              ///// initially empty
  };
</pre></program>

The <i>event</i> class
must provide for a function to activate the event,
and in addition it must allow for defining dependent
events, that may in turn be activated by calling <i>update</i>.
<p>
In the example below, which is adapted from [_Henderson],
events are used to maintain and display
the value of a collection of thermometers in
a consistent fashion.
An abstract thermometer may be defined as:

<slide id=ex-therm cap="The thermometer class">
@ The thermometer class
<program><pre>
  class thermometer {
  friend class Reset;
  public:
  	virtual void set(float v);
  	virtual float get();
  protected:
  	float _temp;                 ///// absolute temperature
  };
</pre></program>
</slide>

The thermometer keeps its value in degrees Kelvin,
and may be refined into a Centigrade and Fahrenheit
thermometer using standard conversion rules.
<p>
For realizing a user interface, that allows us to set and
display the value of a thermometer, we need classes
like <i>prompter</i> and <i>displayer</i>:

<slide id=ex-prompter cap="The promper class">
@ The promper class
<program><pre>
  class prompter : public widget {
  public:
  	prompter(char* msg);
  	float get();
  };
  
  class displayer : public widget {
  public:
  	displayer(char* msg);
  	void put( float v);
  };
</pre></program>
</slide>
These classes may be refined to allow for
textual as well as graphical input and display.
<p>
To manage temperature updates, we introduce the
<i>Get</i>, <i>Reset</i> and <i>Show</i> events:


<slide id=ex-events-get cap="The Get Event">
<program><pre>
  class Get : public event {
  public:
    Get(thermometer* th, prompter* p ) : _th(th), _p(p) {} 
    void operator()() {
  	_th->set ( _p->get() );
  	event::update();
  	}
  private:
    thermometer* _th; prompter* _p;
  };
</pre></program>
</slide>
Activating the <i>Get</i> event results in displaying
a <i>prompter</i> and setting the thermometer's value
with the value obtained from it.
Then its dependent events are activated.
<p>
<i>Reset</i> events are needed to update
the value of other thermometers.
To allow the <i>Reset</i> event access to the stored
temperature value, the class <i>Reset</i> has been made
a friend of thermometer. Alternatively, the <i>_temp</i>
instance variable could have been made a static (class) variable.
<p>
The <i>Show</i> event simply puts the value of
the thermometer in a <i>displayer</i>.
In it defined in a similar way
as the <i>Get</i> events.
<p>
When a <i>Get</i> event occurs for a particular
thermometer, first the other thermometers must be reset,
and then the displays for the various thermometers must be
changed accordingly.
That means that we must create at least one <i>Get</i>
event for each thermometer, and sufficient <i>Reset</i>
events to update the value of every other thermometer.
Also, we must have one or more <i>Show</i> events for each
thermometer.
Both the appropriate <i>Reset</i> and <i>Show</i> events must be 
declared to be dependent upon the occurrence
of a <i>Get</i> event.
Both <i>Get</i> and <i>Show</i> events may be
made available as entries in a menu.
When defining dependencies take care
not to create cycles.
Alternatively you may check dynamically for cycles.
