<h1>Choosing an interpreter</h1>
<include file=navigate>
<include file=rf>
The envelope/letter idiom (originally introduced
in [_Coplien] is increasingly adopted
as a standard approach to separate the definition of
a high level interface and its realization by
an implementation class.
Usually, this involves two classes,
say an abstract class <i>A</i>
and its implementation class <i>AImpl</i>.
A disadvantage of this approach is that an additional
(implementation class) is added to the class (name) space.
Another disadvantage is that refining the behavior of
a class using inheritance becomes more cumbersome.
<p>
To illustrate our approach, look at the definition
of the class <i>kit</i>:
<pre>
  class kit {
  public:
    kit( char* kind = "tcl" );

    virtual int eval(char* cmd) { return _kit?_kit->eval(cmd):0; }
    ...
  protected:
    kit( kit* );
    kit* _kit;
  };
</pre>
The constructor of <i>kit</i> indicates 
as a default a Tcl interpreter which is encapsulated
by the class:
<pre>
  class tcl_kit : public kit {
  public:
    tcl_kit() : kit(0) { ... }
    ...
  };
</pre>
The constructor of <i>kit</i> assigns the <i>tcl_kit</i> 
instance to its <i>_kit</i> variable.
In contrast, the <i>tcl_kit</i> initializes its
(parent) <i>_kit</i> variable to zero.
For the <i>kit</i> instance created by the user,
calls are delegated to the <i>tcl_kit</i>
instance.
However, the user  may still inherit from <i>kit</i>
and selectively override its functions.
<include file=navigate>
