<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 3.0//EN">
<html>
<title>The rules of the game</title>
<body>
<h1>The rules of the game</h1>

<h2>The handle/body idiom -- invocation context</h2>

<p>The handle/body class idiom 
separates the class defining a component's
abstract interface (the handle class) from its
hidden implementation (the body class). All intelligence is
located in the body, and (most) requests to the handle object
are delegated to its implementation.

<h4>Problem</h4>

<p>In order to optimize the creation and destruction of dynamically
allocated objects, <em>representation and resource sharing</em> 
and <em>reference counting</em> are often used techniques, but 
employing these techniques can put a heavy burden on the tasks 
of the application programmer. The handle/body idiom was originally
introduces as a means to hide the use of representation and 
resource sharing from the application programmer.

<p>The static class/object relation found in most
strong typed object-oriented programming languages does not provide 
sufficient flexibility in cases where the actual implementation of
otherwise stable interfaces changes frequently or where the choice for
a particular implementation of a class can only be made at run time.
The flexible relation between an abstract interface and a hidden
implementation can be regarded as the most important virtue of the 
handle/body idiom.

<h4>Background</h4>

<p>This idiom is one of the most often used and described
class idioms [_Coplien92,Eliens95,GOF94,Stroustrup,Pree].
Several other idioms and patterns are based on this idiom
(E.g. the envelope/letter idiom [_Coplien92], the Bridge and Proxy
patterns [_GOF94]).

<p>A major flaw in the realization of the idiom as described in the
literature, is the inability to make calls to member functions which
are refined by subclasses of the (visible) handle class in the
implementation of the (hidden) body class.  We restored the working of
the normal virtual function mechanism by introducing the notion of
<em>explicit invocation context</em>.  In this way, of the handle/body
class idiom can be applied completely transparently, even for
programmers of subclasses of the handle.

<h4>Realization</h4>
The handle class delegates all its functionality to its body implementation.
The handle is both used as the interface class to access functionality
implemented by its body, as well as a semi-abstract base class, defining
the interface of the body classes which are derived from the handle.

<p>The body object is typically instantiated directly by the
constructor of the handle, but this action may be deferred to the
moment of first use, or may be done indirectly by using an abstract
object factory [_GOF94].

<p>To be able to make calls to member functions refined by user defined
subclasses of the handle class, the body class should make such calls
via the handle class as well. 
See <a href="handle.html">appendix A</a> for a detailed code example.

<h4>Usage</h4>

<p>The (improved version of) the idiom is frequently used
in the hush class library. The widget library is build of a stable
interface hierarchy, offering several common GUI widgets classes like
buttons, menus and scrollbars.  The widget (handle) classes are
implemented by a separate, hidden implementation hierarchy, which
allows us to change implementations of the widget library, without the
need to recompile dependent applications.  Additionally, the idioms
helps us to ensure that the various widget implementations are used in
a consistent manner (e.g. simultaneous use of Motif buttons and
OpenLook scrollbars)

<h2>Virtual self-reference</h2>

<p>Another case of delegation, similar to the delegation used by the
handle classes, is common in composite objects, where part of the
functionality of the composite is delegated to an inner component. 

<h4>Problem</h4>

<p>Inheritance is not always a suitable technique for code sharing
and object composition. A familiar example is the combination of a
Text object and two scrollbars into a ScrollableText object.
In that case, most of the functionality of ScrollableText will
be equal to that of the Text object. However, it may be hard to
inherit this functionality directly and add extra functionality
by attaching the scrollbars. 

<h4>Background</h4>

<p>The virtual self-reference pattern is closely
related to the Decorator pattern [_GOF94] and InterViews notion of
MonoGlyph [_???].

[need more].

<h4>Realization</h4>
Key to the implementation of the pattern is the virtual <tt>self()</tt>
member of all components. The self() member returns a reference
to the object itself (e.g. <tt>this</tt> in C++) by default, but
returns the inner component if the outer object explicitly delegated
its functionality by using the <tt>redirect()</tt> member.
Note that the realization of the idiom uses inheritance primarily
for interface sharing, and not for code sharing and object composition.

<p>See <a href="widget.html">appendix C</a> for a detailed code example.

<h4>Usage</h4>

<p>In the hush widget library, every composite widget
can delegate part of its functionality to an (inner) component.
It is common practice to derive a composite widget from another
widget by using interface inheritance only, and further delegating
functionality by explicitly delegate functionality to an inner
component (of the same type as the base class) by using
<tt>redirect()</tt>.

<h2>Dynamic role-switching</h2>
Both the virtual self-reference pattern and the handle/body idiom
can be extended further to provide a completely flexible and dynamically
varying interface/implementation relation.

<h4>Problem</h4>
For many applications static type hierarchies do not provide the
flexibility needed to model dynamically changing roles. For example we
may wish to consider a person as an actor capable of various roles
during his lifetime, some of which may even coexist concurrently.  The
characteristic feature of the dynamic role switching or (actor)
pattern is that it allows us regard a particular entity from multiple
perspectives and that the behavior of that entity changes
accordingly. We will look at a possible realization of the pattern
below.

<h4>Background</h4>

<!--

<h2>The handle/body idiom</h2> 

<h4>When and how to use this idiom?</h4> 

The basic handle/body idiom can be used to implement relatively simple
classes whose instances should share the same (memory) resources or to
let the class interface and implementation vary independently.  The
actual type (class name) of the body class can be hidden by deriving
the body class of the handle. In that case, a recursive dependency
between the constructors of the handle and body can be avoided by the
use of a dummy constructor.

As shown above, the basic handle/body idiom should not be used to implement 
base classes whose member functions make calls to other virtual members.
In the next section we discuss an improved version of the handle/body 
idiom which can be applied to avoid these virtual function anomalies.


<h2>Dynamic invocation context</h2>

<h4>When and how to use this idiom?</h4> Use the explicit invocation
context idiom if a member function needs the services of another
virtual member function, which might be redefined by subclasses of the
handle.  This can be achieved by an explicit reference to the original
invocation context, which can be represented by pointer to the handle
object.  To enable the body object to call such a possibly redefined
function correctly, it makes these calls via the handle.

Subclasses may apply the handle/body idiom in the same way as base
classes do.  However, a (system) programmer implementing the body of
the derived class should be aware of the three access methods to the
body of its handle's base class: via its invocation context (by
calling member functions via the handle pointer), via direct
inheritance (by subclassing directly from {\tt BodyOfA}), and via
indirect inheritance (by subclassing from its handle C).  The latter
method can be avoided by use of dummy constructors.

-->

