<h1><a name=Drawtool>Virtual self-reference</a></h1>
<include file=navigate>
<include file=rf>
A special feature of the <i>hush</i> widget class
library is its support for the definition of
new composite widgets, which provide to the application
programmer the interface of a built-in (or possibly
other composite) widget.
To realize this flexibility, we introduced 
a <i>self()</i> function that adds
another level of indirection to self-reference.
For example, we may define a <i>drawtool</i> widget,
which provides for both the C++ programmer and
the (Tcl) script programmer the interface of an
ordinary (built-in) <i>canvas</i>:
<program><pre>
  class drawtool: public canvas {
    public:
      drawtool() { }                  // to declare drawtool
      drawtool(char* path) {
	initialize(path);             // constructs the components
	redirect(_canvas);            // delegate to _canvas component
      }
      int operator()() {              // interface to Tcl
	if (!strcmp("drawtool",argv[1])) return create(argc, argv);
	else return self()->eval( flatten(argc, argv) );
      }
    private:
      void initialize(char* path); 
      int create(int argc, char* argv[]);
      canvas* _canvas;
  };
</pre></program>
For completeness, the first constructor is only used
to define the script command <i>drawtool</i> as in
<program><pre>
  tk->bind("drawtool", new drawtool);
</pre></program>
which may occur in <i>main</i> or <i>prelude</i>
of the application class.
<p>
The call <i>redirect(_canvas)</i> in the regular constructor
results in delegating both member function calls
and script commands addressed at the <i>drawtool</i> widget
to the <i>canvas</i> widget pointed to by <i>_canvas</i>,
which is assumed to be created in <i>initialize</i>.
The <i>_canvas</i> widget acts, so to speak,
as the primary component of <i>drawtool</i>
in that it receives all bindings and (configuration) commands.
Actual redirection occurs by calling <i>self()</i>.
The definition of <i>widget::self()</i> is simply:
<program><pre>
  widget* self() { return _self?_self->self():this; }
</pre></program>
In other words, it checks the private <i>_self</i>
variable of the widget,
which may be given a value by <i>widget::_redirect</i>,
until <i>_self</i> is undefined (zero),
in which case the <i>this</i> pointer is returned.
In this way primary widget components can be embedded
at arbitrary depth within a composite widget,
without any need to adapt the interface of the
embedding (composite) widget.

Those well-versed in design patterns will recognize the <i>Decorator</i>
patterns (as applied in the Interviews <i>MonoGlyph</i> class
[_Interviews]).
<include file=navigate>
