%%@ objects.pl

object(nil,nil,nil,0,[],[],_). %% the generic object
value(nil,nil,nil).

:- op(700,xfy,['<-']).

%%@ new & delete - object creation and destruction

new(O) :- %%% Class_new
        O =.. [F|A],
        list_concat([F,'_new'],M),
        G =.. [M|A],
        %%cerr(trying(G)),
        clause(G,_) -> call(G),!.

new(O) :- %%% Class_Class
        O =.. [F|A],
        list_concat([F,'_',F],M),
        G =.. [M|A],
        %%cerr(trying(G)),
        clause(G,_) -> call(G),!.

new(O) :- %%% default 
        O =.. [F|_],
	C =.. [F,_],
	eval(C:handler(H)),
	%%cerr(default(new(O))),
	declare(H,new(O),[handler],_),!.

delete(O) :- %%% optional: Class_delete
        O =.. [F,A],
	(eval(O:delete);true),
	((atom(A) -> retractall(value(A,_,_)));true),
	((atom(A) -> retract(object(_,_,A,_,_,_,_)));true),
	cerr(deleted(O)),
	!.

%%@ object - access

object(O) :- object(O,_).
object(O,N) :-
	%%cerr(asking(object(O,N))),
	O =.. [F,A],
	object(_,F,A,N,_,_,_),
	not A = nil,
	not N = 0.
	%%cerr(trying(object(_,F,A,N,_,_,_))).
	
object(O,N,Anc) :- O =.. [F,A], object(_,F,A,N,Anc,_,_).

objectname(C,A,N) :- var(A),!, gensym(C,N),A=N.
objectname(_,N,N).

%%@ ancestors

ancestors(F,P,R) :- 
	ancestors(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

ancestors(F,R):- not var(F), object(_,F,nil,0,R,_,_),!.
ancestors(_,[]).

%%@ attributes

attributes(F,P,R) :- 
	attributes(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

attributes(F,R):- not var(F), object(_,F,nil,0,_,R,_),!.
attributes(_,[]).

%%@ qualifiers

qualifiers(F,P,R) :- 
	qualifiers(F,Anc),
	(var(P) -> R = Anc;
	 P = [_|_] -> append(Anc,P,R);
	 R = Anc).

qualifiers(F,R):- not var(F), object(_,F,nil,0,_,_,R),!.
qualifiers(_,[]).

%%@ declarations

declare(H,class(C)) :- declare(H,class(C),[obscure],[],[]).
declare(H,class(C),A) :- declare(H,class(C),A,[],[]).
declare(H,class(C),A,P) :- declare(H,class(C),A,P,[]).
declare(H,class(C),A,P,Q) :-
	assert(object(H,C,nil,0,A,P,Q)).

%%declare(H,new(O):A,P,R):- not O = (_:_), declare(H,new(O),A,R).
%%declare(H,new(O:A),P,R):- declare(H,new(O),A,P,R).
%%declare(H,new(O),P,R):- not O = (_:_), declare(H,new(O),[],P,R).

declare(H,new(O),R) :- declare(H,new(O),[],[],[],R).
declare(H,new(O),I,R) :- declare(H,new(O),I,[],[],R).
declare(H,new(O),I,P,R) :- declare(H,new(O),I,P,[],R).
declare(H,new(O),I,P,Q,R) :-
	O =.. [F,A|Args],
	%%cerr(declare(H,new(O),I,P,R)),
	objectname(F,A,N),
	ancestors(F,I,Anc),
	attributes(F,P,Att),
	qualifiers(F,Q,Qual),
	%%cerr(declare(H,new(O),I,P,Q,R)),
	(not var(H), var(R) ->
		native(H,N,F:Args,R);R=R),
	assert(object(H,F,N,R,Anc,Att,Qual)),
	cerr(asserted((object(H,F,N,R,Anc,Att,Qual)))).


%%@ method operator overloading

method_translate(var(K)=V,assign(K,V)) :- not var(K),!.
method_translate(value(K)=V,value(K,V)) :- not var(K),!.
method_translate(V=value(K),value(K,V)) :- not var(K),!.

method_translate(X=Y,eq(X,Y)) :- !.
method_translate(X<Y,lt(X,Y)) :- !.
method_translate(X>Y,gt(X,Y)) :- !.
%%method_translate(X<=Y,leq(X,Y)) :- !.
%%method_translate(X>=Y,geq(X,Y)) :- !.

method_translate(X:Y,eval(X:Y)) :- !.

method_translate(X,X) :- !.

%%@ method call

method(self(ID),M,C) :-
	atom(ID),
	(object(_,Class,ID,_,_,_,_);
	 object(_,Class,_,ID,_,_,_)),!,
	O =.. [Class,ID],
	method(O,M,C).

method(O,M,C):-
	object(O),
	O =.. [Class,N],
	method_translate(M,Fun),
	method(Class,N,Fun,C).

method(O,M,C) :-  %%% anonymous
	not object(O),
	O =.. [F,ID|_],
	%%cerr(anonymous(method(O,M,C))),
	method_translate(M,Fun),
	method(F,ID,Fun,C).

method(Class,ID,M,C):- %%% no default handler
	var(ID),
	M = handler(H),
	(object(H,Class,nil,_,_,_,_);true),
	!,
	C = true.


method(_,ID,M,C):- %%% value(K,V)
	M = value(K,V),
	((atom(ID),object(_,_,ID,_,_,_,_)) -> value(ID,K,V); V = nil), 
	!,
	C = true.

method(_,ID,M,C):- %%% assign(K,V)
	M = assign(K,V),
	((atom(ID),value(ID,K,_)) -> retract(value(ID,K,_)); true),
	(atom(ID) -> assert(value(ID,K,V)); true),
	!,
	C = true.
	
method(Class,ID,M,C):-
	M =.. [Method|Args],
	list_concat([Class,'_',Method],Predicate),
	(Args = [] -> C =.. [Predicate,ID];
	append([Predicate,ID],Args,L), C =.. L),
	%%cerr(trying:C),
	(not clause(C,_) -> fail; true,! ).

method(Class,ID,M,C):-
	%%cerr(method(Type,ID,M,C)),
	(object(_,Class,ID,_,Anc,_,_);
	 object(_,Class,nil,_,Anc,_,_)),!,
	member(Type,Anc),
	method(Type,ID,M,C).
	
%%@ eval and its abbreviation

'<-'(O,G) :-
	eval(O:G).

eval(_:[]) :- !.
eval(O:[H|T]) :-
	eval(O:H),
	eval(O:T).

eval(O:G) :-
	not G = [],
	not G = [_|_],
	%%cerr(eval(O:G)),
	method(O,G,B),
	%%cerr(calling(O:G)),
	call(B).
%%</>
