<HTML>
<HEAD>
<!-- This HTML file has been created by texi2html 1.52
     from sicstus.tex on 6 October 1998 -->

<TITLE>SICStus Prolog - Built-In Predicates</TITLE>
</HEAD>
<BODY>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_9.html">previous</A>, <A HREF="sicstus_11.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
<P><HR><P>


<H1><A NAME="SEC92" HREF="sicstus_toc.html#TOC92">Built-In Predicates</A></H1>

<P>
It is not possible to redefine built-in predicates.  An attempt to do so
will give an error message.  See section <A HREF="sicstus_44.html#SEC366">Summary of Built-In Predicates</A>.  
</P>
<P>
SICStus Prolog provides a wide range of built-in predicates to perform the
following tasks:

</P>

<PRE>
Input / Output
     Reading-in Programs
     Term and Goal Expansion
     Input and Output of Terms
     Character I/O
     Stream I/O
     Dec-10 Prolog File I/O
Arithmetic
Comparison of Terms
Control
Error and Exception Handling
Information about the State of the Program
Meta-Logic
Modification of Terms
Modification of the Program
Internal Database
Blackboard Primitives
All Solutions
Coroutining
Debugging
Execution Profiling
Muse Support
Miscellaneous
</PRE>

<P>
The following descriptions of the built-in predicates are grouped according to
the above categorization of their tasks.

</P>

<UL>
<LI><A HREF="sicstus_10.html#SEC93">Input Output</A>
<LI><A HREF="sicstus_10.html#SEC101">Arithmetic</A>
<LI><A HREF="sicstus_10.html#SEC102">Term Compare</A>
<LI><A HREF="sicstus_10.html#SEC103">Control</A>
<LI><A HREF="sicstus_10.html#SEC104">Exception</A>
<LI><A HREF="sicstus_10.html#SEC105">State Info</A>
<LI><A HREF="sicstus_10.html#SEC106">Meta Logic</A>
<LI><A HREF="sicstus_10.html#SEC107">Modify Term</A>
<LI><A HREF="sicstus_10.html#SEC108">Modify Prog</A>
<LI><A HREF="sicstus_10.html#SEC109">Database</A>
<LI><A HREF="sicstus_10.html#SEC110">Blackboard Primitives</A>
<LI><A HREF="sicstus_10.html#SEC111">All Solutions</A>
<LI><A HREF="sicstus_10.html#SEC112">Coroutining</A>
<LI><A HREF="sicstus_10.html#SEC113">Debug Pred</A>
<LI><A HREF="sicstus_10.html#SEC114">Profiling</A>
<LI><A HREF="sicstus_10.html#SEC115">Muse Pred</A>
<LI><A HREF="sicstus_10.html#SEC116">Misc Pred</A>
</UL>



<H2><A NAME="SEC93" HREF="sicstus_toc.html#TOC93">Input / Output</A></H2>
<P>
<A NAME="IDX318"></A>
<A NAME="IDX319"></A>
<A NAME="IDX320"></A>
<A NAME="IDX321"></A>

</P>
<P>
There are two sets of file manipulation predicates in SICStus Prolog.
One set is inherited from DEC-10 Prolog.  These predicates always refer
to a file by name.  The other set of predicates is modeled after
Quintus Prolog and refer to files as streams.  Streams correspond to the
file pointers used at the operating system level.

</P>
<P>
A stream can be opened and connected to a filename or file
descriptor for input or output by calling the predicate <CODE>open/3</CODE>.
<CODE>open/3</CODE> will return a reference to a stream.  The stream may then
be passed as an argument to various I/O predicates.  The predicate
<CODE>close/1</CODE> is used for closing a stream.  The predicate
<CODE>current_stream/3</CODE> is used for retrieving information about a
stream, and for finding the currently existing streams.  
</P>
<P>
Prolog streams can be accessed from C functions as well.
See section <A HREF="sicstus_11.html#SEC139">SICStus Streams</A>, for details.

</P>
<P>
The possible formats of a stream are:

</P>
<DL COMPACT>

<DT><CODE>'$stream'(<VAR>X</VAR>)</CODE>
<DD>
A stream connected to some file.  <VAR>X</VAR> is an integer.

<DT><CODE>user_input</CODE>
<DD>
<A NAME="IDX322"></A>
An alias initially referring to the UNIX <CODE>stdin</CODE> stream.
The alias can be changed with <CODE>prolog_flag/3</CODE> and accessed
by the C variable <CODE>SP_stdin</CODE>.

<DT><CODE>user_output</CODE>
<DD>
<A NAME="IDX323"></A>
An alias initially referring to the UNIX <CODE>stdout</CODE> stream.
The alias can be changed with <CODE>prolog_flag/3</CODE> and accessed
by the C variable <CODE>SP_stdout</CODE>.

<DT><CODE>user_error</CODE>
<DD>
<A NAME="IDX324"></A>
An alias initially referring to the UNIX <CODE>stderr</CODE> stream.
The alias can be changed with <CODE>prolog_flag/3</CODE> and accessed
by the C variable <CODE>SP_stderr</CODE>.

This stream is used by the Prolog top level and debugger,
and for progress and error messages.
</DL>

<P>
<A NAME="IDX325"></A>
<A NAME="IDX326"></A>
<A NAME="IDX327"></A>
<A NAME="IDX328"></A>
The DEC-10 Prolog I/O predicates manipulate streams implicitly, by
maintaining the notion of a <EM>current input stream</EM> and a
<EM>current output stream</EM>.  The current input and output streams are
set to the <CODE>user_input</CODE> and <CODE>user_output</CODE> initially and for
every new break (see section <A HREF="sicstus_4.html#SEC23">Nested Executions--Break and Abort</A>).  The predicate <CODE>see/1</CODE>
(<CODE>tell/1</CODE>) can be used for setting the current input (output)
stream to newly opened streams for particular files.  The predicate
<CODE>seen/0</CODE> (<CODE>told/0</CODE>) close the current input (output) stream,
and resets it to the standard input (output) stream.  The predicate
<CODE>seeing/1</CODE> (<CODE>telling/1</CODE>) is used for retrieving the filename
associated with the current input (output) streams.

</P>
<P>
<A NAME="IDX329"></A>
The possible formats of a filename are:

</P>
<DL COMPACT>

<DT><CODE>user</CODE>
<DD>
This "filename" stands for the standard input or output stream,
depending on context.  Terminal output is only guaranteed to be displayed
if the output stream is explicitly flushed.

<DT><VAR>File</VAR>
<DD>
where <VAR>File</VAR> is any atom other than <CODE>user</CODE>, denotes a file
<VAR>File</VAR> (with an optional <SAMP>`.pl'</SAMP> suffix when consulting or
compiling or an optional <SAMP>`.ql'</SAMP> suffix in <CODE>load/1</CODE>) located
using services provided by the operating system.  On UNIX systems,
Filenames beginning with <TT>`/'</TT> are absolute; other filenames are
looked up in the current working directory.

<DT><CODE><VAR>Alias</VAR>(File)</CODE>
<DD>
<A NAME="IDX330"></A>
<A NAME="IDX331"></A>
where <VAR>File</VAR> is an atom, denotes an alias which must be rewritten
using the user defined predicate <CODE>user:file_search_path/2</CODE> until an
atomic filename is obtained.
</DL>

<P>
Filenames beginning with <TT>`~'</TT> or <TT>`$'</TT> are treated specially.
For example,

</P>
<DL COMPACT>

<DT><TT>`~/sample.pl'</TT>
<DD>
is equivalent to <TT>`/home/sics/al/sample.pl'</TT>, if
<TT>`/home/sics/al'</TT> is the user's home directory.  (This is also
equivalent to <TT>`$HOME/sample.pl'</TT> as explained below.)

<DT><TT>`~clyde/sample.pl'</TT>
<DD>
is equivalent to <TT>`/home/sics/clyde/sample.pl'</TT>, if
<TT>`/home/sics/clyde'</TT> is Clyde's home directory.

<DT><TT>`$UTIL/sample.pl'</TT>
<DD>
is equivalent to <TT>`/usr/local/src/utilities/sample.pl'</TT>, provided
the value of the environment variable <CODE>UTIL</CODE> is
<TT>`/usr/local/src/utilities'</TT>.
</DL>

<P>
Failure to open a file normally causes an exception to be raised.  This
behavior can be turned off and on by of the built-in predicates
<CODE>nofileerrors/0</CODE> and <CODE>fileerrors/0</CODE> described below.  
</P>

<UL>
<LI><A HREF="sicstus_10.html#SEC94">Read In</A>
<LI><A HREF="sicstus_10.html#SEC95">Definite</A>
<LI><A HREF="sicstus_10.html#SEC96">Term I/O</A>
<LI><A HREF="sicstus_10.html#SEC97">Char I/O</A>
<LI><A HREF="sicstus_10.html#SEC98">Stream Pred</A>
<LI><A HREF="sicstus_10.html#SEC99">File Pred</A>
<LI><A HREF="sicstus_10.html#SEC100">I/O Example</A>
</UL>



<H3><A NAME="SEC94" HREF="sicstus_toc.html#TOC94">Reading-in Programs</A></H3>

<P>
When the predicates discussed in this section are invoked, filenames are
relative to the current working directory.  During the load, the current
working directory is temporarily changed to the directory containing the
file being read in.  This has the effect that if one of these predicates
is invoked recursively, the filename of the recursive load is relative
to the directory of the enclosing load.  See section <A HREF="sicstus_8.html#SEC71">Loading Programs</A>, for an
introduction to these predicates.

</P>
<P>
Directives will be executed in order of occurrence.  Be aware of the
changed current working directory as it could have an effect on the
semantics of directives.  Only the first solution of directives is
produced, and variable bindings are not displayed.  Directives that fail
or raise exceptions cause warnings, but do not terminate the load.

</P>
<P>
Most of the predicates listed below take an argument <VAR>Files</VAR> which
is a single file name or a list of file names.  Source files usually end
with an <SAMP>`.pl'</SAMP> suffix and object file names with an <SAMP>`.ql'</SAMP>
suffix.  These suffixes are optional.  Each file name may optionally be
prefixed by a module name.  The module name specifies where to import
the exported predicates of a module file, or where to store the
predicates of a non-module file.  The module is created if it doesn't
exist already.

</P>
<P>
<CODE>absolute_file_name/2</CODE> (see section <A HREF="sicstus_10.html#SEC98">Stream I/O</A>) is used to look up the
files.  The file name <CODE>user</CODE> is reserved and denotes the standard
input stream.

</P>
<P>
These predicates are available in Runtime Systems with the
following limitations:

</P>

<UL>
<LI>

The compiler is not available, so compiling
is replaced by consulting.  

<LI>

The <CODE>redefine_warnings</CODE> and <CODE>single_var_warnings</CODE> Prolog flags
have no effect.

<LI>

Progress and warning messages are not issued.

<LI>

The user is not prompted in the event of name clashes etc.
</UL>

<P>
  

</P>
<P>
<A NAME="IDX332"></A>
<A NAME="IDX333"></A>
<A NAME="IDX334"></A>

</P>
<DL COMPACT>

<DT><CODE>load_files(<VAR>:Files</VAR>)</CODE>
<DD>
<DT><CODE>load_files(<VAR>:Files</VAR>, <VAR>+Options</VAR>)</CODE>
<DD>
<A NAME="IDX335"></A>
<A NAME="IDX336"></A>
A generic predicate for loading files with a list of options to provide
extra control.  This predicate in fact subsumes the other predicates
except <CODE>use_module/3</CODE> which also returns the name of the loaded
module.  
<VAR>Options</VAR> is a list of zero or more of the following:

<DL COMPACT>

<DT><CODE>if(<VAR>X</VAR>)</CODE>
<DD>
<CODE>true</CODE> (the default) to always load, or <CODE>changed</CODE>
to load only if the file has not yet been loaded or if it has been
modified since it was last loaded.  A non-module file is not considered
to have been previously loaded if it was loaded into a different module.
The file <CODE>user</CODE> is never considered to have been previously loaded.

<DT><CODE>when(<VAR>When</VAR>)</CODE>
<DD>
<CODE>always</CODE> (the default) to always load, or <CODE>compile_time</CODE> to
load only if the goal is not in the scope of another
<CODE>load_files/(1,2)</CODE> goal loading object code.

The latter is intended for use when the file only defines predicates
that are needed for proper term or goal expansion during compilation of
other files.

<DT><CODE>load_type(<VAR>LoadType)</VAR></CODE>
<DD>
<CODE>source</CODE> to load source code only, <CODE>object</CODE> to load object
code only, or <CODE>latest</CODE> (the default) to load source or object code,
whichever is newest.  If the file is <CODE>user</CODE>, <CODE>source</CODE> is forced.

<DT><CODE>imports(<VAR>Imports</VAR>)</CODE>
<DD>
<CODE>all</CODE> (the default) to import all exported predicates if the file
is a module file, or a list of predicates to import.

<DT><CODE>compilation_mode(<VAR>Mode</VAR>)</CODE>
<DD>
<CODE>compile</CODE> to translate into compiled code, <CODE>consult</CODE> to
translate into static, interpreted code, or <CODE>assert_all</CODE> to
translate into dynamic, interpreted code.

The default is the compilation mode of any ancestor
<CODE>load_files/(1,2)</CODE> goal, or <CODE>compile</CODE> otherwise. Note that
<VAR>Mode</VAR> has no effect when an object file is loaded, and that it is
recommended to use <CODE>assert_all</CODE> in conjunction with
<CODE>load_type(source)</CODE>, to ensure that the source file will be loaded
even in the presence of an object file.
</DL>

<DT><CODE>consult(<VAR>:Files</VAR>)</CODE>
<DD>
<DT><CODE>reconsult(<VAR>:Files</VAR>)</CODE>
<DD>
<DT><CODE>[]</CODE>
<DD>
<DT><CODE>[<VAR>:File</VAR>|<VAR>+Files</VAR>]</CODE>
<DD>
<A NAME="IDX337"></A>
<A NAME="IDX338"></A>
<A NAME="IDX339"></A>
<A NAME="IDX340"></A>
Consults the source file or list of files specified by <VAR>File</VAR> and
<VAR>Files</VAR>.  Same as <CODE>load_files(<VAR>Files</VAR>, [load_type(source),compilation_mode(consult)])</CODE>. 
<DT><CODE>compile(<VAR>:Files</VAR>)</CODE>
<DD>
<A NAME="IDX341"></A>
Compiles the source file or list of files specified by <VAR>Files</VAR>.  The
compiled code is placed in-core, i.e. is added incrementally to the
Prolog database.  
Same as <CODE>load_files(<VAR>Files</VAR>, [load_type(source),compilation_mode(compile)])</CODE>. 
<DT><CODE>load(<VAR>:Files</VAR>)</CODE>
<DD>
<A NAME="IDX342"></A>
Loads the object file or list of files specified by <VAR>Files</VAR>.  
Same as <CODE>load_files(<VAR>Files</VAR>, [load_type(object)])</CODE>. 
<DT><CODE>ensure_loaded(<VAR>:Files</VAR>)</CODE>
<DD>
<A NAME="IDX343"></A>
Compiles or loads the file or files specified by <VAR>Files</VAR> that
have been modified after the file was last loaded, or that have not
yet been loaded.  The recommended style is to use this predicate for
non-module files only, but if any module files are encountered, their
public predicates are imported.
Same as <CODE>load_files(<VAR>Files</VAR>, [if(changed)])</CODE>. 
<DT><CODE>use_module(<VAR>:File</VAR>)</CODE>
<DD>
<A NAME="IDX344"></A>
Compiles or loads the module file specified by <VAR>File</VAR> if it has been
modified after it was last loaded, or not yet been loaded.  Its public
predicates are imported.  The recommended style is to use this predicate
for module files only, but any non-module files encountered are simply
compiled or loaded.  Same as <CODE>load_files(<VAR>File</VAR>,
[if(changed)])</CODE>. 
<DT><CODE>use_module(<VAR>:File</VAR>, <VAR>+Imports</VAR>)</CODE>
<DD>
<A NAME="IDX345"></A>
Loads the module file <VAR>File</VAR> like <CODE>ensure_loaded/1</CODE> and imports
the predicates in <VAR>Imports</VAR>.  If any of these are not public,
a warning is issued. <VAR>Imports</VAR> may also be set to the atom
<CODE>all</CODE> in which case all public predicates are imported. 
Same as <CODE>load_files(<VAR>File</VAR>, [if(changed),imports(Imports)])</CODE>. 
<DT><CODE>use_module(<VAR>?Module</VAR>, <VAR>:File</VAR>, <VAR>+Imports</VAR>)</CODE>
<DD>
<A NAME="IDX346"></A>
This is equivalent to <CODE>use_module/2</CODE> with the addition
that <VAR>Module</VAR> is unified with the loaded module after the loading.

<DT><CODE>fcompile(<VAR>:Files</VAR>)</CODE>
<DD>
<A NAME="IDX347"></A>
Compiles the source file or list of files specified by <VAR>Files</VAR>.  If
<VAR>Files</VAR> are prefixed by a module name, that module name will be used
for module name expansion during the compilation (see section <A HREF="sicstus_8.html#SEC83">Considerations for File-To-File Compilation</A>).
The suffix <SAMP>`.pl'</SAMP> is added to the given filenames to yield the real
source filenames.  The compiled code is placed on the object file or
list of files formed by adding the suffix <SAMP>`.ql'</SAMP> to the given
filenames.
(This predicate is not available in Runtime Systems.)

<DT><CODE>source_file(<VAR>?File</VAR>)</CODE>
<DD>
<A NAME="IDX348"></A>
<VAR>File</VAR> is the absolute name of a source file currently in the system.

<DT><CODE>source_file(<VAR>:Head</VAR>,<VAR>?File</VAR>)</CODE>
<DD>
<DT><CODE>source_file(<VAR>-Head</VAR>,<VAR>?File</VAR>)</CODE>
<DD>
<A NAME="IDX349"></A>
<VAR>Head</VAR> is the most general goal for a predicate loaded 
from <VAR>File</VAR>.  
<DT><CODE>require(<VAR>:PredSpecOrSpecs</VAR>)</CODE>
<DD>
<A NAME="IDX350"></A>
<VAR>PredSpecOrSpecs</VAR> is a <EM>predicate spec</EM> or a list or a
conjunction of such. The predicate will check if the specified
predicates are loaded and if not, will try to load or import them using
<CODE>use_module/2</CODE>. The file containing the predicate definitions will
be located in the following way:


<UL>
<LI>

The directories specified with <CODE>user:library_directory/1</CODE> are
searched for a file <TT>`INDEX.pl'</TT>. This file is taken to contain
relations between all exported predicates of the module files in the
library directory and its subdirectories. If an <TT>`INDEX.pl'</TT> is not
found, <CODE>require/1</CODE> will try to create one by loading the library
package <CODE>mkindex</CODE> and calling
<CODE>make_index:make_library_index(<VAR>Directory</VAR>)</CODE>
(see section <A HREF="sicstus_14.html#SEC188">The Prolog Library</A>).

<LI>

The first index entry for the requested predicate will be used to
determine the file to load. An exception is raised if the predicate
can't be located.

<LI>

Once an <TT>`INDEX.pl'</TT> is read, it is cached internally for use in
subsequent calls to <CODE>require/1</CODE>.

<LI>

Not available in Runtime Systems.
</UL>

</DL>



<H3><A NAME="SEC95" HREF="sicstus_toc.html#TOC95">Term and Goal Expansion</A></H3>

<P>
When a program is being read in, SICStus Prolog provides hooks that
enable the terms being read in to be source-to-source transformed before
the usual processing of clauses or directives.  The hooks consist in
user-defined predicates that define the transformations.  One
transformation is always available, however: <EM>definite clause
grammars</EM>, a convenient notation for expressing grammar rules.  See
[Colmerauer 75] and [Pereira &#38; Warren 80].

</P>
<P>
Definite clause grammars are an extension of the well-known context-free
grammars.  A grammar rule in Prolog takes the general form

</P>
<P>
<A NAME="IDX351"></A>
<A NAME="IDX352"></A>
<A NAME="IDX353"></A>
<A NAME="IDX354"></A>

</P>

<PRE>
<VAR>head</VAR> --&#62; <VAR>body</VAR>.
</PRE>

<P>
meaning "a possible form for <VAR>head</VAR> is <VAR>body</VAR>".  Both
<VAR>body</VAR> and <VAR>head</VAR> are sequences of one or more items linked by the
standard Prolog conjunction operator <SAMP>`,'</SAMP>.  
</P>
<P>
Definite clause grammars extend context-free grammars in the following ways:

</P>

<OL>
<LI>

A non-terminal symbol may be any Prolog term (other than a variable or
number).  <LI>

A terminal symbol may be any Prolog term.  To distinguish terminals from
non-terminals, a sequence of one or more terminal symbols is written
within a grammar rule as a Prolog list.  An empty sequence is written as
the empty list <SAMP>`[]'</SAMP>.  If the terminal symbols are character codes,
such lists can be written (as elsewhere) as strings.  An empty sequence
is written as the empty list, <SAMP>`[]'</SAMP> or <SAMP>`""'</SAMP>.  <LI>

Extra conditions, in the form of Prolog procedure calls, may be included
in the right-hand side of a grammar rule.  Such procedure calls are
written enclosed in <SAMP>`{}'</SAMP> brackets.  <LI>

The left-hand side of a grammar rule consists of a non-terminal,
optionally followed by a sequence of terminals (again written as a
Prolog list).  <LI>

Disjunction, if-then, if-then-else, and not-provable may be stated
explicitly in the right-hand side of a grammar rule, using the operators
<SAMP>`;'</SAMP> (<SAMP>`|'</SAMP>), <SAMP>`-&#62;'</SAMP>, and <SAMP>`\+'</SAMP> as in a Prolog clause.
<LI>

The cut symbol may be included in the right-hand side of a grammar rule,
as in a Prolog clause.  The cut symbol does not need to be enclosed in
<SAMP>`{}'</SAMP> brackets.  </OL>

<P>
As an example, here is a simple grammar which parses an arithmetic
expression (made up of digits and operators) and computes its value.

</P>

<PRE>
expr(Z) --&#62; term(X), "+", expr(Y), {Z is X + Y}.
expr(Z) --&#62; term(X), "-", expr(Y), {Z is X - Y}.
expr(X) --&#62; term(X).

term(Z) --&#62; number(X), "*", term(Y), {Z is X * Y}.
term(Z) --&#62; number(X), "/", term(Y), {Z is X / Y}.
term(Z) --&#62; number(Z).

number(C) --&#62; "+", number(C).
number(C) --&#62; "-", number(X), {C is -X}.
number(X) --&#62; [C], {"0"=&#60;C, C=&#60;"9", X is C - "0"}.
</PRE>

<P>
In the last rule, <VAR>C</VAR> is the character code of some digit.

</P>
<P>
The query

</P>

<PRE>
| ?- expr(Z, "-2+3*5+1", []).
</PRE>

<P>
will compute <VAR>Z</VAR>=14.  The two extra arguments are explained below.

</P>
<P>
Now, in fact, grammar rules are merely a convenient "syntactic sugar" for
ordinary Prolog clauses.  Each grammar rule takes an input string, analyses
some initial portion, and produces the remaining portion (possibly
enlarged) as output for further analysis.  The arguments required for the
input and output strings are not written explicitly in a grammar rule, but
the syntax implicitly defines them.  We now show how to translate grammar
rules into ordinary clauses by making explicit the extra arguments.

</P>
<P>
A rule such as

</P>

<PRE>
p(X) --&#62; q(X).
</PRE>

<P>
translates into

</P>

<PRE>
p(X, S0, S) :- q(X, S0, S).
</PRE>

<P>
If there is more than one non-terminal on the right-hand side, as in

</P>

<PRE>
p(X, Y) --&#62; 
        q(X), 
        r(X, Y),
        s(Y).
</PRE>

<P>
then corresponding input and output arguments are identified, as in

</P>

<PRE>
p(X, Y, S0, S) :- 
        q(X, S0, S1), 
        r(X, Y, S1, S2), 
        r(Y, S2, S).
</PRE>

<P>
Terminals are translated using the built-in predicate
<CODE>'C'(<VAR>S1</VAR>, <VAR>X</VAR>, <VAR>S2</VAR>)</CODE>, read as "point <VAR>S1</VAR> is
connected by terminal <VAR>X</VAR> to point <VAR>S2</VAR>", and defined by the single
clause 
</P>

<PRE>
'C'([X|S], X, S).
</PRE>

<P>
(This predicate is not normally useful in itself; it has been given the
name upper-case <KBD>c</KBD> simply to avoid using up a more useful name.)
Then, for instance

</P>

<PRE>
p(X) --&#62; [go,to], q(X), [stop].
</PRE>

<P>
is translated by

</P>

<PRE>
p(X, S0, S) :-
        'C'(S0, go, S1), 
        'C'(S1, to, S2), 
        q(X, S2, S3), 
        'C'(S3, stop, S).
</PRE>

<P>
Extra conditions expressed as explicit procedure calls naturally translate
as themselves, e.g.

</P>

<PRE>
p(X) --&#62; [X], {integer(X), X&#62;0}, q(X).
</PRE>

<P>
translates to

</P>

<PRE>
p(X, S0, S) :- 
        'C'(S0, X, S1), 
        integer(X), 
        X&#62;0, 
        q(X, S1, S).
</PRE>

<P>
Similarly, a cut is translated literally.

</P>
<P>
Terminals are translated using the built-in predicate
<CODE>'C'(<VAR>S1</VAR>, <VAR>X</VAR>, <VAR>S2</VAR>)</CODE>, read as "point <VAR>S1</VAR> is
connected by terminal <VAR>X</VAR> to point <VAR>S2</VAR>", and defined by the single
clause 
</P>
<P>
Terminals on the left-hand side of a rule are also translated using
<CODE>'C'/3</CODE>, connecting them to the output argument of the head
non-terminal, e.g.

</P>

<PRE>
is(N), [not] --&#62; [aint].
</PRE>

<P>
becomes

</P>

<PRE>
is(N, S0, S) :-
        'C'(S0, aint, S1),
        'C'(S, not, S1).
</PRE>

<P>
Disjunction has a fairly obvious translation, e.g.

</P>

<PRE>
args(X, Y) --&#62; 
        (   dir(X), [to], indir(Y)
        ;   indir(Y), dir(X)
        ).
</PRE>

<P>
translates to

</P>

<PRE>
args(X, Y, S0, S) :-
        (   dir(X, S0, S1), 
            'C'(S1, to, S2), 
            indir(Y, S2, S)
        ;   indir(Y, S0, S1), 
            dir(X, S1, S)
        ).
</PRE>

<P>
Similarly for if-then, if-then-else, and not-provable.

</P>
<P>
The built-in predicates which are concerned with grammar rules and
other compile/consult time transformations are as follows:

</P>
<DL COMPACT>

<DT><CODE>expand_term(<VAR>+Term1</VAR>,<VAR>?Term2</VAR>)</CODE>
<DD>
<A NAME="IDX355"></A>
If <VAR>Term1</VAR> is a term that can be
transformed, <VAR>Term2</VAR> is the result.  Otherwise <VAR>Term2</VAR> is just
<VAR>Term1</VAR> unchanged.  This transformation takes place automatically
when grammar rules are read in, but sometimes it is useful to be able to
perform it explicitly.  Grammar rule expansion is not the only
transformation available; the user may define clauses for the predicate
<CODE>user:term_expansion/(2,4)</CODE> to perform other transformations.
<CODE>user:term_expansion(<VAR>Term1</VAR>[,<VAR>Layout1</VAR>],<VAR>Term2</VAR>[,<VAR>Layout2</VAR>])</CODE>
is called first, and
only if it fails is the standard expansion used.  
<DT><CODE>term_expansion(<VAR>+Term1</VAR>,<VAR>?TermOrTerms</VAR>)</CODE>
<DD>
<DT><CODE>user:term_expansion(<VAR>+Term1</VAR>,<VAR>?TermOrTerms</VAR>)</CODE>
<DD>
<DT><CODE>term_expansion(<VAR>+Term1</VAR>,<VAR>+Layout1</VAR>,<VAR>?TermOrTerms</VAR>,<VAR>?Layout2</VAR>)</CODE>
<DD>
<DT><CODE>user:term_expansion(<VAR>+Term1</VAR>,<VAR>+Layout1</VAR>,<VAR>?TermOrTerms</VAR>,<VAR>?Layout2</VAR>)</CODE>
<DD>
<A NAME="IDX356"></A>
<A NAME="IDX357"></A>
<A NAME="IDX358"></A>
<A NAME="IDX359"></A>
<EM>A hook predicate</EM>, which defines transformations on terms read
while a program is consulted or compiled.  It is called for every
<VAR>Term1</VAR> read, including at end of file, represented as the term
<CODE>end_of_file</CODE>.  If it succeeds, <VAR>TermOrTerms</VAR> is used for further
processing, otherwise the default grammar rule expansion is attempted.
It is often useful to let a term expand to a list of commands and
clauses, which will then be processed sequentially.

The 4 arguments version also defines transformations on the layout
of the term read, so that the source-linked debugger can display
accurate source code lines if the transformed code needs debugging.
<VAR>Layout1</VAR> is the layout corresponding to <VAR>Term1</VAR>, and <VAR>Layout2</VAR>
should be a valid layout of <VAR>TermOrTerms</VAR> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>).

For accessing aspects of the load context, e.g. the name of the file
being compiled, the predicate <CODE>prolog_load_context/2</CODE> (see section <A HREF="sicstus_10.html#SEC105">Information about the State of the Program</A>) can be used.

<CODE>user:term_expansion/(2,4)</CODE> may also be used to transform queries
entered at the terminal in response to the <SAMP>`| ?- '</SAMP> prompt.  In
this case, it will be called with <VAR>Term1</VAR> = <CODE>?-(<VAR>Query</VAR>)</CODE>
and should succeed with <VAR>TermOrTerms</VAR> = <CODE>?-(<VAR>ExpandedQuery</VAR>)</CODE>.

<DT><CODE>goal_expansion(<VAR>+Goal</VAR>,<VAR>+Module</VAR>,<VAR>?NewGoal</VAR>)</CODE>
<DD>
<DT><CODE>user:goal_expansion(<VAR>+Goal</VAR>,<VAR>+Module</VAR>,<VAR>?NewGoal</VAR>)</CODE>
<DD>
<A NAME="IDX360"></A>
<A NAME="IDX361"></A>
<A NAME="IDX362"></A>
<A NAME="IDX363"></A>
<EM>A hook predicate.</EM> This predicate defines transformations on goals
while clauses are being consulted, compiled or asserted, <EM>after</EM>
any processing by <CODE>user:term_expansion/(2,4)</CODE> of the terms being read
in.  It is called for every simple <VAR>Goal</VAR> encountered in the calling
context <VAR>Module</VAR>.  If it succeeds, <VAR>Goal</VAR> is replaced by
<VAR>NewGoal</VAR>, otherwise <VAR>Goal</VAR> is left unchanged.  <VAR>NewGoal</VAR>
may be an arbitrarily complex goal, and <CODE>user:goal_expansion/3</CODE> is
recursively applied to its subgoals.

This predicate is also used to resolve meta-calls to <VAR>Goal</VAR>
at runtime via the same mechanism.  If the transformation
succeeds, <VAR>NewGoal</VAR> is simply called instead of <VAR>Goal</VAR>.
Otherwise, if <VAR>Goal</VAR> is a goal of an existing predicate,
that predicate is invoked.
Otherwise, error recovery is attempted by
<CODE>user:unknown_predicate_handler/3</CODE> as described below.

<CODE>user:goal_expansion/3</CODE> can be regarded as a macro expansion facility.
It is used for this purpose to support the interface to attributed
variables in <CODE>library(atts)</CODE>, which defines the predicates
<CODE><VAR>M</VAR>:get_atts/2</CODE> and <CODE><VAR>M</VAR>:put_atts/2</CODE> to access
module-specific variable attributes.  These "predicates" are actually
implemented via the <CODE>user:goal_expansion/3</CODE> mechanism.  This has the
effect that calls to the interface predicates are expanded at compile
time to efficient code.

For accessing aspects of the load context, e.g. the name of the file
being compiled, the predicate <CODE>prolog_load_context/2</CODE> (see section <A HREF="sicstus_10.html#SEC105">Information about the State of the Program</A>) can be used.

<DT><CODE>phrase(<VAR>:Phrase</VAR>,<VAR>?List</VAR>)</CODE>
<DD>
<DT><CODE>phrase(<VAR>:Phrase</VAR>,<VAR>?List</VAR>,<VAR>+Remainder</VAR>)</CODE>
<DD>
<A NAME="IDX364"></A>
<A NAME="IDX365"></A>
The list <VAR>List</VAR> is a phrase of type <VAR>Phrase</VAR> (according to the
current grammar rules), where <VAR>Phrase</VAR> is either a non-terminal or
more generally a grammar rule body.  <VAR>Remainder</VAR> is what remains of
the list after a phrase has been found.  If called with 2 arguments, the
remainder has to be the empty list.  
<DT><CODE>'C'(<VAR>?S1</VAR>,<VAR>?Terminal</VAR>,<VAR>?S2</VAR>)</CODE>
<DD>
<A NAME="IDX366"></A>
Not normally of direct use to the user, this built-in predicate is used
in the expansion of grammar rules (see above).  It is defined as if by
the clause <CODE>'C'([X|S], X, S).</CODE> 
</DL>
<P>
  


<H3><A NAME="SEC96" HREF="sicstus_toc.html#TOC96">Input and Output of Terms</A></H3>
<P>
<A NAME="IDX367"></A>

</P>
<P>
Most of the following predicates come in two versions, with or without a
stream argument.  Predicates without a stream argument operate on the
current input or output stream, depending on context.

</P>
<P>
Some of these predicates support a notation for terms containing
multiple occurrences of the same subterm (cycles and DAGs).  The
notation is <CODE>@(<VAR>Template</VAR>,<VAR>Substitution</VAR>)</CODE> where
<VAR>Substitution</VAR> is a list of <VAR>Var</VAR>=<VAR>Term</VAR> pairs where the
<VAR>Var</VAR> occurs in <VAR>Template</VAR> or in one of the <VAR>Terms</VAR>.
This notation stands for the instance of <VAR>Template</VAR> obtained by
binding each <VAR>Var</VAR> to its corresponding <VAR>Term</VAR>.  The purpose of
this notation is to provide a finite printed representation of cyclic
terms.  This notation is not used by default, and <CODE>@/2</CODE> has no
special meaning except in this context.

</P>
<DL COMPACT>

<DT><CODE>read(<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>read(<VAR>+Stream</VAR>,<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX368"></A>
<A NAME="IDX369"></A>
The next term, delimited by a full-stop (i.e. a <KBD>.</KBD>, possibly
followed by layout text), is read from <VAR>Stream</VAR> and is unified with
<VAR>Term</VAR>.  The syntax of the term must agree with current operator
declarations.  If a call <CODE>read(<VAR>Stream, Term</VAR>)</CODE> causes the end
of <VAR>Stream</VAR> to be reached, <VAR>Term</VAR> is unified with the term
<CODE>end_of_file</CODE>.  Further calls to <CODE>read/2</CODE> for the same stream
will then raise an exception, unless the stream is connected to the
terminal.  
<DT><CODE>read_term(<VAR>?Term</VAR>,<VAR>+Options</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>read_term(<VAR>+Stream</VAR>,<VAR>?Term</VAR>,<VAR>+Options</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX370"></A>
<A NAME="IDX371"></A>
Same as <CODE>read/(1-2)</CODE> with a list of options to provide extra
control or information about the term.  <VAR>Options</VAR> is a list of zero
or more of:

<DL COMPACT>

<DT><CODE>syntax_errors(<VAR>+Val</VAR>)</CODE>
<DD>
Controls what action to take on syntax errors.  <VAR>Val</VAR> must be one of
the values allowed for the <CODE>syntax_errors</CODE> Prolog flag.  The
default is set by that flag.

<DT><CODE>variable_names(<VAR>?Names</VAR>)</CODE>
<DD>
<VAR>Names</VAR> is bound to a list of <VAR>Name</VAR>=<VAR>Var</VAR> pairs, where each
<VAR>Name</VAR> is an atom indicating the name of a non-anonymous variable in
the term, and <VAR>Var</VAR> is the corresponding variable.

<DT><CODE>singletons(<VAR>?Names</VAR>)</CODE>
<DD>
<VAR>Names</VAR> is bound to a list of <VAR>Name</VAR>=<VAR>Var</VAR> pairs, one for
each variable appearing only once in the term and whose name does not
begin with <KBD>_</KBD>.

<DT><CODE>cycles(<VAR>+Boolean</VAR>)</CODE>
<DD>
<A NAME="IDX372"></A>
<A NAME="IDX373"></A>
<VAR>Boolean</VAR> must be <CODE>true</CODE> or <CODE>false</CODE>.  If selected, any
occurrences of <CODE>@/2</CODE> in the term read in are replaced by the
potentially cyclic terms they denote as described above.  Otherwise (the
default), <VAR>Term</VAR> is just unified with the term read in.

<DT><CODE>layout(<VAR>?Layout</VAR>)</CODE>
<DD>
<A NAME="IDX374"></A>
<A NAME="IDX375"></A>
<VAR>Layout</VAR> is bound to a <EM>layout term</EM> corresponding to <VAR>Term</VAR>.
The layout <VAR>Y</VAR> of a term <VAR>X</VAR> is one of:


<UL>
<LI>

If <VAR>X</VAR> is a variable or atomic term, <VAR>Y</VAR> is the number of the line
where <VAR>X</VAR> occurs.

<LI>

If <VAR>X</VAR> is a compound term,
<VAR>Y</VAR> is a list whose head is the number of the line where
the first token of <VAR>X</VAR> occurs, and whose remaining elements
are the layouts of the arguments of <VAR>X</VAR>.

<LI>

<CODE>[]</CODE>, if no line number information is available for <VAR>X</VAR>.
</UL>

</DL>


<PRE>
| ?- read_term(T, [layout(L), variable_names(Va), singletons(S)]).
|: [
     foo(X),
     X = Y
     ].

L = [35,[36,36],[36,[37,37,37],38]],
S = ['Y'=_A],
T = [foo(_B),_B=_A],
Va = ['X'=_B,'Y'=_A]
</PRE>

<DT><CODE>write(<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>write(<VAR>+Stream</VAR>,<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX376"></A>
<A NAME="IDX377"></A>
The term <VAR>Term</VAR> is written onto <VAR>Stream</VAR> according to current
operator declarations.  
Same as <CODE>write_term([<VAR>Stream</VAR>,] <VAR>Term</VAR>, [numbervars(true)])</CODE>.

<DT><CODE>display(<VAR>?Term</VAR>)</CODE>
<DD>
<A NAME="IDX378"></A>
The term <VAR>Term</VAR> is displayed <EM>onto the standard output stream</EM>
(which is not necessarily the current output stream) in standard
parenthesized prefix notation.
Same as <CODE>write_term(user, <VAR>Term</VAR>, [ignore_ops(true)])</CODE>.

<DT><CODE>write_canonical(<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>write_canonical(<VAR>+Stream</VAR>,<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX379"></A>
<A NAME="IDX380"></A>
Similar to <CODE>write(<VAR>Stream</VAR>,<VAR>Term</VAR>)</CODE>.  The term will be
written according to the standard syntax.  The output from
<CODE>write_canonical/2</CODE> can be parsed by <CODE>read/2</CODE> even if the term
contains special characters or if operator declarations have changed.
Same as <CODE>write_term([<VAR>Stream</VAR>,] <VAR>Term</VAR>, [quoted(true),ignore_ops(true)])</CODE>.

<DT><CODE>writeq(<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>writeq(<VAR>+Stream</VAR>,<VAR>?Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX381"></A>
<A NAME="IDX382"></A>
Similar to <CODE>write(<VAR>Stream</VAR>,<VAR>Term</VAR>)</CODE>, but the names of atoms
and functors are quoted where necessary to make the result acceptable as
input to <CODE>read/2</CODE>, provided the same operator declarations are in effect. 
Same as <CODE>write_term([<VAR>Stream</VAR>,] <VAR>Term</VAR>, [quoted(true),numbervars(true)])</CODE>.

<DT><CODE>print(<VAR>?Term</VAR>)</CODE>
<DD>
<DT><CODE>print(<VAR>+Stream</VAR>,<VAR>?Term</VAR>)</CODE>
<DD>
<A NAME="IDX383"></A>
<A NAME="IDX384"></A>
<EM>Hookable</EM>.  Prints <VAR>Term</VAR> onto <VAR>Stream</VAR>.  This predicate
provides a handle for user defined pretty printing:


<UL>
<LI>

If <VAR>Term</VAR> is a variable then it is output using
<CODE>write(<VAR>Stream</VAR>,<VAR>Term</VAR>)</CODE>.  
<LI>

If <VAR>Term</VAR> is non-variable then a call is made to the user defined
predicate <CODE>user:portray/1</CODE>.  If this succeeds then it is assumed that
<VAR>Term</VAR> has been output.  
<LI>

Otherwise <CODE>print/2</CODE> is called recursively on the components of
<VAR>Term</VAR>, unless <VAR>Term</VAR> is atomic in which case it is written via
<CODE>write/2</CODE>.  </UL>

In particular, the debugging package prints the goals in the tracing
messages, and the top-level prints the final values of variables.  Thus
you can vary the forms of these messages if you wish.

Note that on lists (<CODE>[_|_]</CODE>), <CODE>print/2</CODE> will first give the
whole list to <CODE>user:portray/1</CODE>, but if this fails it will only give each
of the (top level) elements to <CODE>user:portray/1</CODE>.  That is,
<CODE>user:portray/1</CODE> will not be called on all the tails of the list.

Same as <CODE>write_term([<VAR>Stream</VAR>,] <VAR>Term</VAR>, [portrayed(true),numbervars(true)])</CODE>.

<DT><CODE>portray(<VAR>+Term</VAR>)</CODE>
<DD>
<DT><CODE>user:portray(<VAR>+Term</VAR>)</CODE>
<DD>
<A NAME="IDX385"></A>
<A NAME="IDX386"></A>
<EM>A hook predicate.</EM> This should either print the <VAR>Term</VAR> and
succeed, or do nothing and fail.  In the latter case, the default
printer (<CODE>write/1</CODE>) will print the <VAR>Term</VAR>. 
<DT><CODE>portray_clause(<VAR>+Clause</VAR>)</CODE>
<DD>
<DT><CODE>portray_clause(<VAR>+Stream</VAR>,<VAR>+Clause</VAR>)</CODE>
<DD>
<A NAME="IDX387"></A>
<A NAME="IDX388"></A>
Writes the clause <VAR>Clause</VAR> onto <VAR>Stream</VAR>
exactly as <CODE>listing/(0-1)</CODE> would have written it.  Same as
<CODE>write_term([<VAR>Stream</VAR>,] <VAR>Term</VAR>, [quoted(true),numbervars(true),indented(true)])</CODE> 
followed by a period and a newline, removing redundant module prefixes
and binding variables to terms of the form <CODE>'$VAR'(<VAR>N</VAR>)</CODE>
yielding friendlier variable names.

<DT><CODE>write_term(<VAR>+Term</VAR>,<VAR>+Options</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>write_term(<VAR>+Stream</VAR>,<VAR>+Term</VAR>,<VAR>+Options</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX389"></A>
<A NAME="IDX390"></A>
Same as <CODE>write/(1-2)</CODE> etc. with a list of options to provide
extra control.  This predicate in fact subsumes the above output
predicates except <CODE>portray_clause/(1,2)</CODE> which additionally prints a
period and a newline, and removes module prefixes that are redundant
wrt. the current type-in module. <VAR>Options</VAR> is a list of zero or
more of the following, where <VAR>Boolean</VAR> must be <CODE>true</CODE> or
<CODE>false</CODE> (<CODE>false</CODE> is the default).  
<DL COMPACT>

<DT><CODE>quoted(<VAR>+Boolean</VAR>)</CODE>
<DD>
If selected, functors are quoted where necessary to make the result
acceptable as input to <CODE>read/1</CODE>.  <CODE>write_canonical/1</CODE>,
<CODE>writeq/1</CODE>, and <CODE>portray_clause/1</CODE> select this.

<DT><CODE>ignore_ops(<VAR>+Boolean</VAR>)</CODE>
<DD>
If selected, <VAR>Term</VAR> is written in standard parenthesized notation
instead of using operators.  <CODE>write_canonical/1</CODE> and
<CODE>display/1</CODE> select this.

<DT><CODE>portrayed(<VAR>+Boolean</VAR>)</CODE>
<DD>
If selected, <CODE>user:portray/1</CODE> is called for each subterm.
<CODE>print/1</CODE> selects this.

<DT><CODE>numbervars(<VAR>+Boolean</VAR>)</CODE>
<DD>
If selected, occurrences of <CODE>'$VAR'(<VAR>N</VAR>)</CODE> where <VAR>N</VAR> is an
integer &#62;= 0 are treated specially (see <CODE>numbervars/3</CODE>).
<CODE>print/1</CODE>, <CODE>write/1</CODE>, <CODE>writeq/1</CODE>, and <CODE>portray_clause/1</CODE>
select this.

<DT><CODE>cycles(<VAR>+Boolean</VAR>)</CODE>
<DD>
<A NAME="IDX391"></A>
<A NAME="IDX392"></A>
If selected, the potentially cyclic term is printed in finite <CODE>@/2</CODE>
notation, as discussed above.

<DT><CODE>indented(<VAR>+Boolean</VAR>)</CODE>
<DD>
If selected, the term is printed with the same indentation as is used by
<CODE>portray_clause/1</CODE> and <CODE>listing/(0-1)</CODE>.

<DT><CODE>max_depth(<VAR>N</VAR>)</CODE>
<DD>
Depth limit on printing.  <VAR>N</VAR> is an integer.  0 (the default) means
no limit.
</DL>

<DT><CODE>format(<VAR>+Format</VAR>,<VAR>+Arguments</VAR>)</CODE>
<DD>
<DT><CODE>format(<VAR>+Stream</VAR>,<VAR>+Format</VAR>,<VAR>+Arguments</VAR>)</CODE>
<DD>
<A NAME="IDX393"></A>
<A NAME="IDX394"></A>
Prints <VAR>Arguments</VAR> onto <VAR>Stream</VAR> according to format
<VAR>Format</VAR>.  <VAR>Format</VAR> is a list of formatting characters.  If
<VAR>Format</VAR> is an atom then <CODE>name/2</CODE> (see section <A HREF="sicstus_10.html#SEC106">Meta-Logic</A>) will be
used to translate it into a list of characters.  Thus 

<PRE>
| ?- format("Hello world!", []).
</PRE>

has the same effect as


<PRE>
| ?- format('Hello world!', []).
</PRE>

<CODE>format/2</CODE> and <CODE>format/3</CODE> is a Prolog interface to the C
<CODE>stdio</CODE> function <CODE>printf()</CODE>.  It is modeled after and
compatible with Quintus Prolog.  
<VAR>Arguments</VAR> is a list of items to be printed.  If there are no
items then an empty list should be supplied.  
The default action on a format character is to print it.  The character
<KBD>~</KBD> introduces a control sequence.  To print a <KBD>~</KBD> repeat it:


<PRE>
| ?- format("Hello ~~world!", []).
</PRE>

will result in


<PRE>
Hello ~world!
</PRE>

The escape sequence (see section <A HREF="sicstus_45.html#SEC372">Escape Sequences</A>) <KBD>\c</KBD> (c for continue)
is useful when formatting a string for readability.  It causes all
characters up to, but not including, the next non-layout character to be
ignored.


<PRE>
| ?- format("Hello \c
             world!", []).
</PRE>

will result in


<PRE>
Hello world!
</PRE>

The general format of a control sequence is <SAMP>`~<VAR>N</VAR><VAR>C</VAR>'</SAMP>.  The
character <VAR>C</VAR> determines the type of the control sequence.  <VAR>N</VAR>
is an optional numeric argument.  An alternative form of <VAR>N</VAR> is
<SAMP>`*'</SAMP>.  <SAMP>`*'</SAMP> implies that the next argument in <VAR>Arguments</VAR>
should be used as a numeric argument in the control sequence.  Example:


<PRE>
| ?- format("Hello~4cworld!", [0'x]).
</PRE>

and


<PRE>
| ?- format("Hello~*cworld!", [4,0'x]).
</PRE>

both produce


<PRE>
Helloxxxxworld!
</PRE>

The following control sequences are available.

<DL COMPACT>

<DT><SAMP>`~a'</SAMP>
<DD>
The argument is an atom.  The atom is printed without quoting.  
<DT><SAMP>`~<VAR>N</VAR>c'</SAMP>
<DD>
(Print character.)  The argument is a number that will be interpreted as
a character code.  <VAR>N</VAR> defaults to one and is interpreted as the
number of times to print the character.  
<DT><SAMP>`~<VAR>N</VAR>e'</SAMP>
<DD>
<DT><SAMP>`~<VAR>N</VAR>E'</SAMP>
<DD>
<DT><SAMP>`~<VAR>N</VAR>f'</SAMP>
<DD>
<DT><SAMP>`~<VAR>N</VAR>g'</SAMP>
<DD>
<DT><SAMP>`~<VAR>N</VAR>G'</SAMP>
<DD>
(Print float).  The argument is a float.  The float and <VAR>N</VAR> will be
passed to the C <CODE>printf()</CODE> function as 

<PRE>
printf("%.<VAR>N</VAR>e", <VAR>Arg</VAR>)
printf("%.<VAR>N</VAR>E", <VAR>Arg</VAR>)
printf("%.<VAR>N</VAR>f", <VAR>Arg</VAR>)
printf("%.<VAR>N</VAR>g", <VAR>Arg</VAR>)
printf("%.<VAR>N</VAR>G", <VAR>Arg</VAR>)
</PRE>

respectively.

If <VAR>N</VAR> is not supplied the action defaults to


<PRE>
printf("%e", <VAR>Arg</VAR>)
printf("%E", <VAR>Arg</VAR>)
printf("%f", <VAR>Arg</VAR>)
printf("%g", <VAR>Arg</VAR>)
printf("%G", <VAR>Arg</VAR>)
</PRE>

respectively.

<DT><SAMP>`~<VAR>N</VAR>d'</SAMP>
<DD>
(Print decimal.) The argument is an integer.  <VAR>N</VAR> is interpreted as
the number of digits after the decimal point.  If <VAR>N</VAR> is 0 or
missing, no decimal point will be printed.  Example:


<PRE>
| ?- format("Hello ~1d world!", [42]).
Hello 4.2 world!

| ?- format("Hello ~d world!", [42]).
Hello 42 world!
</PRE>

<DT><SAMP>`~<VAR>N</VAR>D'</SAMP>
<DD>
(Print decimal.) The argument is an integer.  Identical to
<SAMP>`~<VAR>N</VAR>d'</SAMP> except that <SAMP>`,'</SAMP> will separate groups of three
digits to the left of the decimal point.  Example:


<PRE>
| ?- format("Hello ~1D world!", [12345]).
Hello 1,234.5 world!
</PRE>

<DT><SAMP>`~<VAR>N</VAR>r'</SAMP>
<DD>
(Print radix.) The argument is an integer.  <VAR>N</VAR> is interpreted as a
radix, 2 =&#60; <VAR>N</VAR> =&#60; 36.  If <VAR>N</VAR> is missing the
radix defaults to 8.  The letters <SAMP>`a-z'</SAMP> will denote digits larger
than 9.  Example:


<PRE>
| ?- format("Hello ~2r world!", [15]).
Hello 1111 world!

| ?- format("Hello ~16r world!", [15]).
Hello f world!
</PRE>

<DT><SAMP>`~<VAR>N</VAR>R'</SAMP>
<DD>
(Print radix.) The argument is an integer.  Identical to
<SAMP>`~<VAR>N</VAR>r'</SAMP> except that the letters <SAMP>`A-Z'</SAMP> will denote digits
larger than 9.  Example:


<PRE>
| ?- format("Hello ~16R world!", [15]).
Hello F world!
</PRE>

<DT><SAMP>`~<VAR>N</VAR>s'</SAMP>
<DD>
(Print string.) The argument is a list of character codes.  Exactly
<VAR>N</VAR> characters will be printed.  <VAR>N</VAR> defaults to the length of
the string.  Example:


<PRE>
| ?- format("Hello ~4s ~4s!", ["new","world"]).
Hello new  worl!

| ?- format("Hello ~s world!", ["new"]).
Hello new world!
</PRE>

<DT><SAMP>`~i'</SAMP>
<DD>
(Ignore.) The argument, which may be of any type, is ignored.
Example:


<PRE>
| ?- format("Hello ~i~s world!", ["old","new"]).
Hello new world!
</PRE>

<DT><SAMP>`~k'</SAMP>
<DD>
(Print canonical.) The argument may be of any type.  The argument will
be passed to <CODE>write_canonical/1</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>).  Example:


<PRE>
| ?- format("Hello ~k world!", [[a,b,c]]).
Hello .(a,.(b,.(c,[]))) world!
</PRE>

<DT><SAMP>`~p'</SAMP>
<DD>
(Print.) The argument may be of any type.  The argument will be passed
to <CODE>print/1</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>).  Example:


<PRE>
| ?- assert((portray([X|Y]) :- print(cons(X,Y)))).
| ?- format("Hello ~p world!", [[a,b,c]]).
Hello cons(a,cons(b,cons(c,[]))) world!
</PRE>

<DT><SAMP>`~q'</SAMP>
<DD>
(Print quoted.) The argument may be of any type.  The argument will be
passed to <CODE>writeq/1</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>).  Example: 

<PRE>
| ?- format("Hello ~q world!", [['A','B']]).
Hello ['A','B'] world!
</PRE>

<DT><SAMP>`~w'</SAMP>
<DD>
(Write.) The argument may be of any type.  The argument will be passed
to <CODE>write/1</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>).  Example: 

<PRE>
| ?- format("Hello ~w world!", [['A','B']]).
Hello [A,B] world!
</PRE>

<DT><SAMP>`~@'</SAMP>
<DD>
(Call.) The argument is a goal, which will be called and expected
to print on the current output stream.  If the goal is not a built-in
predicate, it should be module prefixed, as <CODE>format/(2-3)</CODE> are not
meta-predicates.  If the goal performs other side-effects or
does not succeed deterministically, the
behavior is undefined.  Example: 

<PRE>
| ?- format("Hello ~@ world!", [write(new)]).
Hello new world!
</PRE>

<DT><SAMP>`~~'</SAMP>
<DD>
(Print tilde.)  Takes no argument.  Prints <SAMP>`~'</SAMP>.  Example:


<PRE>
| ?- format("Hello ~~ world!", []).
Hello ~ world!
</PRE>

<DT><SAMP>`~<VAR>N</VAR>n'</SAMP>
<DD>
(Print newline.) Takes no argument.  Prints <VAR>N</VAR> newlines.  <VAR>N</VAR>
defaults to 1.  Example:


<PRE>
| ?- format("Hello ~n world!", []).
Hello
 world!
</PRE>

<DT><SAMP>`~N'</SAMP>
<DD>
(Print Newline.) Prints a newline if not at the beginning of a line.
</DL>

The following control sequences set column boundaries and specify
padding.  A column is defined as the available space between two
consecutive column boundaries on the same line.  A boundary is
initially assumed at line position 0.  The specifications only apply
to the line currently being written.

When a column boundary is set (<SAMP>`~|'</SAMP> or <SAMP>`~+'</SAMP>) and there are
fewer characters written in the column than its specified width, the
remaining space is divided equally amongst the pad sequences
(<SAMP>`~t'</SAMP>) in the column.  If there are no pad sequences, the column
is space padded at the end.

If <SAMP>`~|'</SAMP> or <SAMP>`~+'</SAMP> specifies a position preceding the
current position, the boundary is set at the current position.

<DL COMPACT>

<DT><SAMP>`~<VAR>N</VAR>|'</SAMP>
<DD>
Set a column boundary at line position <VAR>N</VAR>. <VAR>N</VAR> defaults to the
current position.

<DT><SAMP>`~<VAR>N</VAR>+'</SAMP>
<DD>
Set a column boundary at <VAR>N</VAR> positions past the previous column
boundary. <VAR>N</VAR> defaults to 8.

<DT><SAMP>`~<VAR>N</VAR>t'</SAMP>
<DD>
Specify padding in a column. <VAR>N</VAR> is the fill character code. <VAR>N</VAR>
may also be specified as <CODE>`<VAR>C</VAR></CODE> where <VAR>C</VAR> is the fill
character. The default fill character is <KBD>SPC</KBD>. Any (<SAMP>`~t'</SAMP>)
after the last column boundary on a line is ignored.
</DL>
Example:


<PRE>
| ?-
        format("~`*t NICE TABLE ~`*t~61|~n", []),
        format("*~t*~61|~n", []),
        format("*~t~a~20|~t~a~t~20+~a~t~20+~t*~61|~n",
               ['Right aligned','Centered','Left aligned']),
        format("*~t~d~20|~t~d~t~20+~d~t~20+~t*~61|~n",
               [123,45,678]),
        format("*~t~d~20|~t~d~t~20+~d~t~20+~t*~61|~n",
               [1,2345,6789]),
        format("~`*t~61|~n", []).

************************ NICE TABLE *************************
*                                                           *
*      Right aligned      Centered      Left aligned        *
*                123         45         678                 *
*                  1        2345        6789                *
*************************************************************
</PRE>

</DL>



<H3><A NAME="SEC97" HREF="sicstus_toc.html#TOC97">Character Input/Output</A></H3>
<P>
<A NAME="IDX395"></A>

</P>
<P>
There are two sets of character I/O predicates.  The first set uses the
current input and output streams, while the second set always uses the
standard input and output streams.  The first set is available in an
alternative version where the stream is specified explicitly.  The rule
is that the stream is the first argument, which defaults to the current
input or output stream, depending on context.  
</P>
<DL COMPACT>

<DT><CODE>nl <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>nl(<VAR>+Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX396"></A>
<A NAME="IDX397"></A>
A new line is started on <VAR>Stream</VAR> by printing an <KBD>LFD</KBD>.  If
<VAR>Stream</VAR> is connected to the terminal, its buffer is flushed.

<DT><CODE>get0(<VAR>?N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>get0(<VAR>+Stream</VAR>,<VAR>?N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX398"></A>
<A NAME="IDX399"></A>
<VAR>N</VAR> is the character code of the next character read from
<VAR>Stream</VAR>.  If all characters of <VAR>Stream</VAR> have been read, <VAR>N</VAR>
is -1, and further calls to <CODE>get0/2</CODE> for the same stream
will then raise an exception, unless the stream is connected to the
terminal.

These predicates are called <CODE>get_code/(1-2)</CODE> in the ISO Prolog 
standard.

<DT><CODE>peek_char(<VAR>?N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>peek_char(<VAR>+Stream</VAR>,<VAR>?N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX400"></A>
<A NAME="IDX401"></A>
<VAR>N</VAR> is the character code of the next character from <VAR>Stream</VAR>,
or -1, if all characters of <VAR>Stream</VAR> have been read, <VAR>N</VAR>.
The character is not actually read, it is only looked at and is still
available for subsequent input.  
These predicates are called <CODE>peek_code/(1-2)</CODE> in the ISO Prolog 
standard.

<DT><CODE>get(<VAR>?N</VAR>)</CODE>
<DD>
<DT><CODE>get(<VAR>+Stream</VAR>,<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX402"></A>
<A NAME="IDX403"></A>
Same as <CODE>get0/2</CODE>, except
<VAR>N</VAR> is the character code of the next character that is not
a <VAR>layout-char</VAR> (see section <A HREF="sicstus_45.html#SEC371">Syntax of Tokens as Character Strings</A>)
read from <VAR>Stream</VAR>.  
<DT><CODE>skip(<VAR>+N</VAR>)</CODE>
<DD>
<DT><CODE>skip(<VAR>+Stream</VAR>,<VAR>+N</VAR>)</CODE>
<DD>
<A NAME="IDX404"></A>
<A NAME="IDX405"></A>
Skips just past the next character code <VAR>N</VAR> from <VAR>Stream</VAR>.
<VAR>N</VAR> may be an arithmetic expression.  
<DT><CODE>skip_line</CODE>
<DD>
<DT><CODE>skip_line(<VAR>+Stream</VAR>)</CODE>
<DD>
<A NAME="IDX406"></A>
<A NAME="IDX407"></A>
Skips just past the next <KBD>LFD</KBD> from <VAR>Stream</VAR>.

<DT><CODE>put(<VAR>+N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>put(<VAR>+Stream</VAR>,<VAR>+N</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX408"></A>
<A NAME="IDX409"></A>
Character code <VAR>N</VAR> is output onto <VAR>Stream</VAR>.  <VAR>N</VAR> may
be an arithmetic expression.  
These predicates are called <CODE>put_code/(1-2)</CODE> in the ISO Prolog 
standard.

<DT><CODE>tab(<VAR>+N</VAR>)</CODE>
<DD>
<DT><CODE>tab(<VAR>+Stream</VAR>,<VAR>+N</VAR>)</CODE>
<DD>
<A NAME="IDX410"></A>
<A NAME="IDX411"></A>
<VAR>N</VAR> spaces are output onto <VAR>Stream</VAR>.  <VAR>N</VAR> may be an
arithmetic expression.  </DL>

<P>
The above predicates are the ones which are the most commonly used, as
they can refer to any streams.  The predicates which follow always refer
to the standard input and output streams.  They are provided for
compatibility DEC-10 character I/O, and are actually redundant and
easily recoded in terms of the above predicates.

</P>
<DL COMPACT>

<DT><CODE>ttynl</CODE>
<DD>
<A NAME="IDX412"></A>
Same as <CODE>nl(user)</CODE>.

<DT><CODE>ttyflush</CODE>
<DD>
<A NAME="IDX413"></A>
Same as <CODE>flush_output(user)</CODE>.

<DT><CODE>ttyget0(<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX414"></A>
Same as <CODE>get0(user, <VAR>N</VAR>)</CODE>.

<DT><CODE>ttyget(<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX415"></A>
Same as <CODE>get(user, <VAR>N</VAR>)</CODE>.

<DT><CODE>ttyput(<VAR>+N</VAR>)</CODE>
<DD>
<A NAME="IDX416"></A>
Same as <CODE>put(user, <VAR>N</VAR>)</CODE>.

<DT><CODE>ttyskip(<VAR>+N</VAR>)</CODE>
<DD>
<A NAME="IDX417"></A>
Same as <CODE>skip(user, <VAR>N</VAR>)</CODE>.

<DT><CODE>ttytab(<VAR>+N</VAR>)</CODE>
<DD>
<A NAME="IDX418"></A>
Same as <CODE>tab(user, <VAR>N</VAR>)</CODE>.
</DL>



<H3><A NAME="SEC98" HREF="sicstus_toc.html#TOC98">Stream I/O</A></H3>

<P>
  
The following predicates manipulate streams.  Character and line counts
are maintained per stream.  All streams connected to the terminal,
however, share the same set of counts.  For example, writing to
<CODE>user_output</CODE> will advance the counts for <CODE>user_input</CODE>, if
both are connected to the terminal. Bidirectional streams use the same
counters for input and output.

</P>
<DL COMPACT>

<DT><CODE>open(<VAR>+FileName</VAR>,<VAR>+Mode</VAR>,<VAR>-Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>open(<VAR>+FileName</VAR>,<VAR>+Mode</VAR>,<VAR>-Stream</VAR>,<VAR>+Options</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX419"></A>
<A NAME="IDX420"></A>
If <VAR>FileName</VAR> is a valid file name, the file is opened in mode
<VAR>Mode</VAR> (invoking the UNIX function <CODE>fopen</CODE>) and the resulting
stream is unified with <VAR>Stream</VAR>.  <VAR>Mode</VAR> is one of: <DL COMPACT>

<DT><CODE>read</CODE>
<DD>
Open the file for input.

<DT><CODE>write</CODE>
<DD>
Open the file for output.  The file is created if it does not already
exist, the file will otherwise be truncated.  
<DT><CODE>append</CODE>
<DD>
Open the file for output.  The file is created if it does not already
exist, the file will otherwise be appended to.  </DL>
 
If <VAR>FileName</VAR> is an integer, it is assumed to be a file descriptor
passed to Prolog from C.  The file descriptor is connected to a Prolog
stream (invoking the POSIX function <CODE>fdopen</CODE>) which is unified with
<VAR>Stream</VAR>.  
<VAR>Options</VAR> is a list of zero or more of:

<DL COMPACT>

<DT><CODE>type(<VAR>+T</VAR>)</CODE>
<DD>
Specifies whether the stream is a <CODE>text</CODE> or <CODE>binary</CODE>
stream. Default is <CODE>text</CODE>. 
</DL>

<DT><CODE>close(<VAR>+X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX421"></A>
If <VAR>X</VAR> is a stream, the stream is closed.  If <VAR>X</VAR> is the name of
a file opened by <CODE>see/1</CODE> or <CODE>tell/1</CODE>, the corresponding stream
is closed.  
<DT><CODE>absolute_file_name(<VAR>+RelativeName</VAR>,<VAR>-AbsoluteName</VAR>)</CODE>
<DD>
<A NAME="IDX422"></A>
True if <VAR>RelativeName</VAR> can be expanded to an absolute file name
(an atom) <VAR>AbsoluteName</VAR>, according to the filename syntax rules
(see section <A HREF="sicstus_10.html#SEC93">Input / Output</A>).  This predicate will first search for a file
with the suffix <SAMP>`.pl'</SAMP> added to the name given as an argument.  If
this fails, it will look for a file with no extra suffix added.  If a
file is found <VAR>AbsoluteName</VAR> is its absolute file name is
returned.  Otherwise, <VAR>AbsoluteName</VAR> is a valid expansion of
<VAR>RelativeName</VAR>,

<CODE>absolute_file_name/2</CODE> does not produce alternative expansions via
backtracking.

If <VAR>RelativeName</VAR> is <CODE>user</CODE>, then <VAR>AbsoluteName</VAR> is
also unified with <CODE>user</CODE>; this "filename" stands for the standard
input or output stream, depending on context.

Variants of this predicate are used by all predicates that refer to
filenames for resolving these.  Predicates that load code require that
the specified file exist, possibly with a <SAMP>`.pl'</SAMP> or <SAMP>`.ql'</SAMP>
extension.

<DT><CODE>file_search_path(<VAR>Alias</VAR>,<VAR>?Expansion</VAR>)</CODE>
<DD>
<DT><CODE>user:file_search_path(<VAR>Alias</VAR>,<VAR>?Expansion</VAR>)</CODE>
<DD>
<A NAME="IDX423"></A>
<A NAME="IDX424"></A>
<A NAME="IDX425"></A>
<A NAME="IDX426"></A>
<EM>A hook predicate.</EM> This predicate specifies a set of possible file
name expansions to be tried when a file specification of the form
<CODE><VAR>Alias</VAR>(<VAR>Name</VAR>)</CODE> is used in a predicate that accepts
a filename argument.

<VAR>Alias</VAR> and <VAR>Name</VAR> must be atoms.  
There are two possible cases for <VAR>Expansion</VAR>.


<UL>
<LI>

<VAR>Expansion</VAR> is an atom.  In this case, the filename expands to
<VAR>Expansion</VAR><KBD>/</KBD><VAR>Name</VAR>, which is subject to further expansion if
the resulting atom begins with <TT>`~'</TT> or <TT>`$'</TT> (see section <A HREF="sicstus_10.html#SEC93">Input / Output</A>).

<LI>

<VAR>Expansion</VAR> is a compound term <CODE><VAR>NewAlias</VAR>(NewName)</CODE>.
In this case, the filename expands to
<CODE><VAR>NewAlias</VAR>(<VAR>NewName</VAR><KBD>/</KBD><VAR>Name</VAR>)</CODE>, which is subject to
further expansion via a recursive call to <CODE>user:file_search_path/2</CODE>.
</UL>

Since it is possible to have multiple definitions for the same alias,
predicates such as <CODE>compile/1</CODE> may have to explore several
alternative expansions before they locate the file to compile.
Predicates such as <CODE>compile/1</CODE> look for a file with a <SAMP>`.pl'</SAMP>
suffix as well as for a file without the suffix.  <CODE>load/1</CODE> looks
for a file with a <SAMP>`.ql'</SAMP> suffix.

Aliases are useful in writing portable code, as they minimize the number
of places where a program has to be changed if the program or its environment
is moved to a different location.

<CODE>file_search_path/2</CODE> is always called in the <CODE>user</CODE>
module.  The predicate exists as a dynamic, multifile predicate at startup
with a single clause defining an expansion for the <CODE>library/1</CODE> alias:


<PRE>
file_search_path(library,Path) :- library_directory(Path).
</PRE>

<DT><CODE>library_directory(<VAR>?Directory</VAR>)</CODE>
<DD>
<DT><CODE>user:library_directory(<VAR>?Directory</VAR>)</CODE>
<DD>
<A NAME="IDX427"></A>
<A NAME="IDX428"></A>
<EM>A hook predicate.</EM> This predicate specifies a set of directories
to be searched when a file specification of the form
<CODE>library(<VAR>Name</VAR>)</CODE> is used.  The predicate exists as a
dynamic, multifile predicate at startup.
  
Directories to be searched may be added by using <CODE>asserta/1</CODE> or
<CODE>assertz/1</CODE> (see section <A HREF="sicstus_10.html#SEC108">Modification of the Program</A>):


<PRE>
| ?- assertz(user:file_search_path(home, '$HOME')).
| ?- assertz(user:file_search_path(demo, home('prolog/demo'))).
| ?- assertz(user:library_directory(home('prolog/lib'))).
</PRE>

With these declarations, the file name <CODE>demo(mydemo)</CODE> would expand to
<CODE>'$HOME/prolog/demo/mydemo'</CODE>, where <CODE>'$HOME'</CODE> is interpreted as
an environment variable (the user's home directory).  File names of the form
<CODE>library(mymodule)</CODE> would be looked up in <CODE>'$HOME/prolog/lib'</CODE>
if they cannot be found in the default library directory.

<DT><CODE>current_input(<VAR>?Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX429"></A>
<VAR>Stream</VAR> is the current input stream.
The current input stream is also accessed by the C variable <CODE>SP_curin</CODE>.

<DT><CODE>current_output(<VAR>?Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX430"></A>
<VAR>Stream</VAR> is the current output stream.
The current output stream is also accessed by the C variable <CODE>SP_curout</CODE>.

<DT><CODE>current_stream(<VAR>?FileName</VAR>,<VAR>?Mode</VAR>,<VAR>?Stream</VAR>)</CODE>
<DD>
<A NAME="IDX431"></A>
<VAR>Stream</VAR> is a stream which was opened in mode <VAR>Mode</VAR> and which
is connected to the absolute file name <VAR>Filename</VAR> (an atom) or to
the file descriptor <VAR>Filename</VAR> (an integer).  This predicate can be
used for enumerating all currently open streams through backtracking.

<DT><CODE>set_input(<VAR>+Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX432"></A>
Sets the current input stream to <VAR>Stream</VAR>.

<DT><CODE>set_output(<VAR>+Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX433"></A>
Sets the current output stream to <VAR>Stream</VAR>.

<DT><CODE>flush_output <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>flush_output(<VAR>+Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX434"></A>
<A NAME="IDX435"></A>
Flushes all internally buffered characters for <VAR>Stream</VAR> to the
operating system.  
<DT><CODE>open_null_stream(<VAR>-Stream</VAR>)</CODE>
<DD>
<A NAME="IDX436"></A>
Opens an output stream.  Everything written to this
stream will be thrown away.  
<DT><CODE>character_count(<VAR>+Stream</VAR>,<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX437"></A>
<VAR>N</VAR> is the number of characters read/written on stream <VAR>Stream</VAR>.

<DT><CODE>line_count(<VAR>+Stream</VAR>,<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX438"></A>
<VAR>N</VAR> is the number of lines read/written on stream <VAR>Stream</VAR>.

<DT><CODE>line_position(<VAR>+Stream</VAR>,<VAR>?N</VAR>)</CODE>
<DD>
<A NAME="IDX439"></A>
<VAR>N</VAR> is the number of characters read/written on the current line of <VAR>Stream</VAR>.

<DT><CODE>stream_position(<VAR>+Stream</VAR>,<VAR>?Position</VAR>)</CODE>
<DD>
<A NAME="IDX440"></A>
<VAR>Position</VAR> is a term representing the current position of
<VAR>Stream</VAR>.  The relative order of stream position terms can be tested
with standard term comparison predicates such as <CODE>compare/3</CODE>, but
you should not otherwise rely on their internal representation.  This
operation is available for any Prolog stream.

<DT><CODE>set_stream_position(<VAR>+Stream</VAR>,<VAR>+Position</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX441"></A>
<VAR>Position</VAR> is a term representing a new position of
<VAR>Stream</VAR>, which is then set to the new position.  This operation is
only available for Prolog streams connected to "seekable devices"
(disk files, usually).

<DT><CODE>seek(<VAR>+Stream</VAR>,<VAR>+Offset</VAR>,<VAR>+Method</VAR>,<VAR>-NewLocation</VAR>)</CODE>
<DD>
<A NAME="IDX442"></A>
True if the stream <VAR>Stream</VAR> can be set to the byte offset
<VAR>Offset</VAR> relative to <VAR>Method</VAR>, and <VAR>NewLocation</VAR> is the new
byte offset from the beginning of the file after the operation.
<VAR>Method</VAR> must be one of:

<DL COMPACT>

<DT><CODE>bof</CODE>
<DD>
Seek from the beginning of the file stream.

<DT><CODE>current</CODE>
<DD>
Seek from the current position of the file stream.

<DT><CODE>eof</CODE>
<DD>
Seek from the end of the file stream.
</DL>

This operation is only available for Prolog streams connected to
"seekable devices" (disk files, usually) and is an interface to
the <CODE>stdio</CODE> functions <CODE>fseek</CODE> and <CODE>ftell</CODE>.

<DT><CODE>at_end_of_stream <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>at_end_of_stream(<VAR>+Stream</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX443"></A>
<A NAME="IDX444"></A>
<A NAME="IDX445"></A>
<A NAME="IDX446"></A>
The end of stream has been reached for <VAR>Stream</VAR>.  An input stream
reaches end of stream when all characters (except <KBD>EOF</KBD>) of the
stream have been read.  These predicates peek ahead for next input
character if there is no character available on the buffer of
<VAR>Stream</VAR>.
Unless the stream is to be treated as connected to the terminal
(see <CODE>SP_force_interactive</CODE>, section <A HREF="sicstus_11.html#SEC152">Initializing the Prolog Engine</A>),
a stream remains at end of stream after <KBD>EOF</KBD> has been read, 
and any further attempt to read from the stream will raise an
existence error (see section <A HREF="sicstus_10.html#SEC104">Error and Exception Handling</A>).

<DT><CODE>at_end_of_line</CODE>
<DD>
<DT><CODE>at_end_of_line(<VAR>+Stream</VAR>)</CODE>
<DD>
<A NAME="IDX447"></A>
<A NAME="IDX448"></A>
<A NAME="IDX449"></A>
<A NAME="IDX450"></A>
The end of stream or end of line has been reached for
<VAR>Stream</VAR>.  An input stream reaches end of line when all the
characters except <KBD>LFD</KBD> of the current line have been read.
These predicates peek ahead for next input character if
there is no character available on the buffer of <VAR>Stream</VAR>.

<DT><CODE>fileerrors</CODE>
<DD>
<A NAME="IDX451"></A>
Undoes the effect of <CODE>nofileerrors/0</CODE>.

<DT><CODE>nofileerrors</CODE>
<DD>
<A NAME="IDX452"></A>
After a call to this predicate, failure to locate or open a file will
cause the operation to fail instead of the default action, which is to
raise an exception with an error message.  
<DT><CODE>stream_select(<VAR>+Streams</VAR>,<VAR>+TimeOut</VAR>,<VAR>-ReadStreams</VAR>)</CODE>
<DD>
<A NAME="IDX453"></A>
The list of streams in <VAR>Streams</VAR> is checked for readable characters.
A stream can be any stream associated with an I/O descriptor.  The list
<VAR>ReadStreams</VAR> returns the streams with readable data.  If
<VAR>TimeOut</VAR> is instantiated to <CODE>off</CODE>, the predicate waits until
something is available.  If <VAR>TimeOut</VAR> is <CODE>S:U</CODE> the predicate
waits at most <CODE>S</CODE> seconds and <CODE>U</CODE> microseconds.  Both <CODE>S</CODE>
and <CODE>U</CODE> must be integers &#62;=0.  If there is a timeout,
<VAR>ReadStreams</VAR> is <CODE>[]</CODE>.

Not available in Muse.  Not available in operating systems that do not
support the <CODE>system()</CODE> system call.

<DT><CODE>stream_interrupt(<VAR>?Stream</VAR>,<VAR>?OldHandler</VAR>,<VAR>?NewHandler</VAR>)</CODE>
<DD>
<A NAME="IDX454"></A>
<A NAME="IDX455"></A>
Installs <VAR>NewHandler</VAR> as an interrupt-handler which is invoked when
something is readable on <VAR>Stream</VAR>. <VAR>OldHandler</VAR> is the current
interrupt handler <VAR>Stream</VAR> must be associated with an I/O
descriptor. Interrupt handlers are specified as atoms.  The atom
<CODE>off</CODE> indicates that the interrupt mechanism is turned off for
<VAR>Stream</VAR>.  Any other atom is the name of a predicate invoked when
something is readable on <VAR>Stream</VAR>.  The handler predicate has one
argument, the stream that is readable.  For example,


<PRE>
stream_interrupt(Stream, _, int_handler).
</PRE>

will enable the interrupt mechanism.  Given the predicate

<PRE>
int_handler(Stream) :-
        read(Stream, Data),
        write(Data), nl.
</PRE>

the term read from <VAR>Stream</VAR> will be written to the current output.
<STRONG>Note:</STRONG> there is no guarantee that a complete Prolog term is available
yet.  If not, <CODE>read/2</CODE> will suspend as usual.  
Not available in Muse.  Not available in operating systems that do not
provide the ability to generate signals when new data becomes available
on a file descriptor.

</DL>



<H3><A NAME="SEC99" HREF="sicstus_toc.html#TOC99">DEC-10 Prolog File I/O</A></H3>

<P>
The following predicates manipulate files.

</P>
<DL COMPACT>

<DT><CODE>see(<VAR>+File</VAR>)</CODE>
<DD>
<A NAME="IDX456"></A>
The file <VAR>File</VAR> becomes the current input stream.  <VAR>File</VAR> may be a
stream previously opened by <CODE>see/1</CODE> or a filename.  If it is a
filename, the following action is taken: If there is a stream opened by
<CODE>see/1</CODE> associated with the same file already, then it becomes the
current input stream.  Otherwise, the file <VAR>File</VAR> is opened for
input and made the current input stream.  
<DT><CODE>seeing(<VAR>?FileName</VAR>)</CODE>
<DD>
<A NAME="IDX457"></A>
<VAR>FileName</VAR> is unified with the name of the current input file, if it
was opened by <CODE>see/1</CODE>, with the current input stream, if it is not
<CODE>user_input</CODE>, otherwise with <CODE>user</CODE>.

<DT><CODE>seen</CODE>
<DD>
<A NAME="IDX458"></A>
Closes the current input stream, and resets it to <CODE>user_input</CODE>.

<DT><CODE>tell(<VAR>+File</VAR>)</CODE>
<DD>
<A NAME="IDX459"></A>
The file <VAR>File</VAR> becomes the current output stream.  <VAR>File</VAR> may
be a stream previously opened by <CODE>tell/1</CODE> or a filename.  If it is a
filename, the following action is taken: If there is a stream 
opened by <CODE>tell/1</CODE> associated with the same file already, then it
becomes the current output stream.  Otherwise, the file <VAR>File</VAR>
is opened for output and made the current output stream.

<DT><CODE>telling(<VAR>?FileName</VAR>)</CODE>
<DD>
<A NAME="IDX460"></A>
<VAR>FileName</VAR> is unified with the name of the current output file, if
it was opened by <CODE>tell/1</CODE>, with the current output stream, if
it is not <CODE>user_output</CODE>, otherwise with <CODE>user</CODE>.

<DT><CODE>told</CODE>
<DD>
<A NAME="IDX461"></A>
Closes the current output stream, and resets it to <CODE>user_output</CODE>.
</DL>



<H3><A NAME="SEC100" HREF="sicstus_toc.html#TOC100">An Example</A></H3>

<P>
Here is an example of a common form of file processing:  

</P>

<PRE>
process_file(F) :-
        seeing(OldInput),
        see(F),                 % Open file F
        repeat,
          read(T),              % Read a term
          process_term(T),      % Process it
          T == end_of_file,     % Loop back if not at end of file
        !,
        seen,                   % Close the file
        see(OldInput).
</PRE>

<P>
<A NAME="IDX462"></A>
The above is an example of a <EM>repeat loop</EM>.  Nearly all sensible uses
of <CODE>repeat/0</CODE> follow the above pattern.  Note the use of a cut to
terminate the loop.

</P>


<H2><A NAME="SEC101" HREF="sicstus_toc.html#TOC101">Arithmetic</A></H2>
<P>
<A NAME="IDX463"></A>

</P>
<P>
Arithmetic is performed by built-in predicates which take as arguments
<EM>arithmetic expressions</EM> and evaluate them.  An arithmetic expression
is a term built from numbers, variables, and functors that represent
arithmetic functions.  At the time of evaluation, each variable in an
arithmetic expression must be bound to a non-variable expression.
An expression evaluates to a number, which may be an <EM>integer</EM> or a 
<EM>float</EM>.

</P>
<P>
The range of integers is <CODE>[-2^2147483616, 2^2147483616)</CODE>.  Thus for
all practical purposes, the range of integers can be considered
infinite.

</P>
<P>
The range of floats is the one provided by the C <CODE>double</CODE> type,
typically <CODE>[4.9e-324, 1.8e+308]</CODE> (plus or minus).

</P>
<P>
Only certain functors are permitted in an arithmetic expression.
These are listed below, together with an indication of the functions
they represent.  <VAR>X</VAR> and <VAR>Y</VAR> are assumed to be arithmetic
expressions.  Unless stated otherwise, the arguments of an expression
may be any numbers and its value is a float if any of its arguments is
a float, otherwise the value is an integer.  Any implicit coercions
are performed with the <CODE>integer/1</CODE> and <CODE>float/1</CODE> functions.

</P>
<DL COMPACT>

<DT><CODE>+(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX464"></A>
The value is <VAR>X</VAR>.

<DT><CODE>-<VAR>X</VAR></CODE>
<DD>
<A NAME="IDX465"></A>
The value is the negative of <VAR>X</VAR>.

<DT><CODE><VAR>X</VAR>+<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX466"></A>
The value is the sum of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>-<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX467"></A>
The value is the difference of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>*<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX468"></A>
The value is the product of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>/<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX469"></A>
The value is the <EM>float</EM> quotient of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>//<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX470"></A>
The value is the <EM>integer</EM> quotient of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR> mod <VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX471"></A>
The value is the <EM>integer</EM> remainder after dividing <VAR>X</VAR> by
<VAR>Y</VAR>, i.e.
<CODE>integer(<VAR>X</VAR>)-integer(<VAR>Y</VAR>)*(<VAR>X</VAR>//<VAR>Y</VAR>)</CODE>.

<DT><CODE>integer(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX472"></A>
The value is the closest integer between <VAR>X</VAR> and 0,
if <VAR>X</VAR> is a float, otherwise to <VAR>X</VAR> itself.

<DT><CODE>float(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX473"></A>
The value is the float equivalent of <VAR>X</VAR>, if <VAR>X</VAR> is an
integer, otherwise to <VAR>X</VAR> itself.

<DT><CODE><VAR>X</VAR>/\<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX474"></A>
The value is the bitwise conjunction of the integers <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>\/<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX475"></A>
The value is the bitwise disjunction of the integers <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR>#<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX476"></A>
The value is the bitwise exclusive or of the integers <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE>\(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX477"></A>
The value is the bitwise negation of the integer <VAR>X</VAR>.

<DT><CODE><VAR>X</VAR>&#60;&#60;<VAR>Y</VAR></CODE>
<DD>
<A NAME="IDX478"></A>
The value is the integer <VAR>X</VAR> shifted left by <VAR>Y</VAR> places.

<DT><CODE>X&#62;&#62;Y</CODE>
<DD>
<A NAME="IDX479"></A>
The value is the integer <VAR>X</VAR> shifted right by <VAR>Y</VAR> places.

<DT><CODE>[<VAR>X</VAR>]</CODE>
<DD>
<A NAME="IDX480"></A>
A list of just one number <VAR>X</VAR> evaluates to <VAR>X</VAR>.  Since a quoted
string is just a list of integers, this allows a quoted character to be
used in place of its character code; e.g. <CODE>"A"</CODE> behaves within
arithmetic expressions as the integer 65.  </DL>

<P>
SICStus Prolog also includes an extra set of functions listed below.
These may not be supported by other Prologs.  All trigonometric and
transcendental functions take float arguments and deliver float
values.  The trigonometric functions take arguments or deliver values
in radians.

</P>
<DL COMPACT>

<DT><CODE>abs(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX481"></A>
The value is the absolute value of <VAR>X</VAR>.

<DT><CODE>gcd(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX482"></A>
The value is the greatest common divisor of the two integers <VAR>X</VAR>
and <VAR>Y</VAR>.

<DT><CODE>min(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX483"></A>
The value is the lesser value of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE>max(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX484"></A>
The value is the greater value of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE>msb(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX485"></A>
The value is the most significant bit position of the integer <VAR>X</VAR>.
It is equivalent to, but more efficient than, <CODE>integer(log(2,X))</CODE>.

<DT><CODE>round(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX486"></A>
The value is the float that is the closest integral value to <VAR>X</VAR>.
If <VAR>X</VAR> is exactly half-way between two integers, it must be
rounded to the closest even integral value.

<DT><CODE>truncate(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX487"></A>
The value is the float that is the closest integer between <VAR>X</VAR> and 0.

<DT><CODE>floor(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX488"></A>
The value is the float that is the greatest integral value less or
equal to <VAR>X</VAR>.

<DT><CODE>ceiling(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX489"></A>
The value is the float that is the least integral value greater or
equal to <VAR>X</VAR>.

<DT><CODE>sin(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX490"></A>
The value is the sine of <VAR>X</VAR>.

<DT><CODE>cos(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX491"></A>
The value is the cosine of <VAR>X</VAR>.

<DT><CODE>tan(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX492"></A>
The value is the tangent of <VAR>X</VAR>.

<DT><CODE>cot(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX493"></A>
The value is the cotangent of <VAR>X</VAR>.

<DT><CODE>sinh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX494"></A>
The value is the hyperbolic sine of <VAR>X</VAR>.

<DT><CODE>cosh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX495"></A>
The value is the hyperbolic cosine of <VAR>X</VAR>.

<DT><CODE>tanh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX496"></A>
The value is the hyperbolic tangent of <VAR>X</VAR>.

<DT><CODE>coth(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX497"></A>
The value is the hyperbolic cotangent of <VAR>X</VAR>.

<DT><CODE>asin(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX498"></A>
The value is the arc sine of <VAR>X</VAR>.

<DT><CODE>acos(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX499"></A>
The value is the arc cosine of <VAR>X</VAR>.

<DT><CODE>atan(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX500"></A>
The value is the arc tangent of <VAR>X</VAR>.

<DT><CODE>atan2(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX501"></A>
The value is the four-quadrant arc tangent of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE>acot(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX502"></A>
The value is the arc cotangent of <VAR>X</VAR>.

<DT><CODE>acot2(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX503"></A>
The value is the four-quadrant arc cotangent of <VAR>X</VAR> and <VAR>Y</VAR>.

<DT><CODE>asinh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX504"></A>
The value is the hyperbolic arc sine of <VAR>X</VAR>.

<DT><CODE>acosh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX505"></A>
The value is the hyperbolic arc cosine of <VAR>X</VAR>.

<DT><CODE>atanh(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX506"></A>
The value is the hyperbolic arc tangent of <VAR>X</VAR>.

<DT><CODE>acoth(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX507"></A>
The value is the hyperbolic arc cotangent of <VAR>X</VAR>.

<DT><CODE>sqrt(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX508"></A>
The value is the square root of <VAR>X</VAR>.

<DT><CODE>log(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX509"></A>
The value is the natural logarithm of <VAR>X</VAR>.

<DT><CODE>log(<VAR>Base</VAR>,<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX510"></A>
The value is the logarithm of <VAR>X</VAR> in the base <VAR>Base</VAR>.

<DT><CODE>exp(<VAR>X</VAR>)</CODE>
<DD>
<A NAME="IDX511"></A>
The value is the natural exponent of <VAR>X</VAR>.

<DT><CODE>exp(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<A NAME="IDX512"></A>
The value is <VAR>X</VAR> raised to the power of <VAR>Y</VAR>.

<DT><CODE>inf</CODE>
<DD>
<A NAME="IDX513"></A>
The value is infinity as defined in the IEEE standard.

<DT><CODE>nan</CODE>
<DD>
<A NAME="IDX514"></A>
The value is not-a-number as defined in the IEEE standard.
</DL>

<P>
Variables in an arithmetic expression which is to be evaluated may be bound
to other arithmetic expressions rather than just numbers, e.g.

</P>

<PRE>
evaluate(Expression, Answer) :- Answer is Expression.

| ?- evaluate(24*9, Ans).
Ans = 216 ?

yes
</PRE>

<P>

</P>
<P>
Arithmetic expressions, as described above, are just data structures.
If you want one evaluated you must pass it as an argument to one of the
built-in predicates listed below.  Note that it only evaluates one of
its arguments, whereas all the comparison predicates evaluate both of
theirs.  In the following, <VAR>X</VAR> and <VAR>Y</VAR> stand for arithmetic
expressions, and <VAR>Z</VAR> for some term.  
</P>
<DL COMPACT>

<DT><CODE><VAR>Z</VAR> is <VAR>X</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX515"></A>
<VAR>X</VAR>, which must be an arithmetic expression, is evaluated and the
result is unified with <VAR>Z</VAR>.

<DT><CODE><VAR>X</VAR> =:= <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX516"></A>
The numeric values of <VAR>X</VAR> and <VAR>Y</VAR> are equal.

<DT><CODE><VAR>X</VAR> =\= <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX517"></A>
The numeric values of <VAR>X</VAR> and <VAR>Y</VAR> are not equal.

<DT><CODE><VAR>X</VAR> &#60; <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX518"></A>
The numeric value of <VAR>X</VAR> is less than the numeric value of <VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR> &#62; <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX519"></A>
The numeric value of <VAR>X</VAR> is greater than the numeric value of
<VAR>Y</VAR>.

<DT><CODE><VAR>X</VAR> =&#60; <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX520"></A>
The numeric value of <VAR>X</VAR> is less than or equal to the numeric value
of <VAR>Y</VAR>.  
<DT><CODE><VAR>X</VAR> &#62;= <VAR>Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX521"></A>
The numeric value of <VAR>X</VAR> is greater than or equal to the numeric
value of <VAR>Y</VAR>.  </DL>



<H2><A NAME="SEC102" HREF="sicstus_toc.html#TOC102">Comparison of Terms</A></H2>
<P>
<A NAME="IDX522"></A>
<A NAME="IDX523"></A>
<A NAME="IDX524"></A>

</P>
<P>
These built-in predicates are meta-logical.  They treat uninstantiated
variables as objects with values which may be compared, and they never
instantiate those variables.  They should <EM>not</EM> be used when what
you really want is arithmetic comparison (see section <A HREF="sicstus_10.html#SEC101">Arithmetic</A>) or
unification.  
</P>
<P>
The predicates make reference to a <EM>standard total ordering</EM> of
terms, which is as follows:

</P>

<UL>

<LI>

Variables, by age (oldest first--the order is
<EM>not</EM> related to the names of variables).  
<LI>

Floats, in numeric order (e.g. -1.0 is put before 1.0).
 
<LI>

Integers, in numeric order (e.g. -1 is put before 1).

<LI>

Atoms, in alphabetical (i.e. character code) order.

<LI>

Compound terms, ordered first by arity, then by the name of the
principal functor, then by age for mutables and
by the arguments in left-to-right order for other terms.
Recall that lists are equivalent to compound terms with principal
functor <CODE>./2</CODE>.  </UL>

<P>
For example, here is a list of terms in standard order:

</P>

<PRE>
[ X, -1.0, -9, 1, fie, foe, X = Y, foe(0,2), fie(1,1,1) ]
</PRE>

<P>
<STRONG>Note:</STRONG> the standard order is only well-defined for finite (acyclic)
terms.  There are infinite (cyclic) terms for which no order relation holds.
Furthermore, blocking goals (see section <A HREF="sicstus_5.html#SEC46">Procedural Semantics</A>) on variables or
modifying their attributes (see section <A HREF="sicstus_17.html#SEC191">Attributed Variables</A>) does not preserve their order.

</P>
<P>
These are the basic predicates for comparison of arbitrary terms:

</P>
<DL COMPACT>

<DT><CODE><VAR>Term1</VAR> == <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX525"></A>
The terms currently instantiating <VAR>Term1</VAR> and <VAR>Term2</VAR> are
literally identical (in particular, variables in equivalent positions in
the two terms must be identical).  For example, the query 

<PRE>
| ?- X == Y.
</PRE>

fails (answers <SAMP>`no'</SAMP>) because <VAR>X</VAR> and <VAR>Y</VAR> are distinct
uninstantiated variables.  However, the query 

<PRE>
| ?- X = Y, X == Y.
</PRE>

succeeds because the first goal unifies the two variables
(see section <A HREF="sicstus_10.html#SEC116">Miscellaneous</A>).  
<DT><CODE><VAR>Term1</VAR> \== <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX526"></A>
The terms currently instantiating <VAR>Term1</VAR> and <VAR>Term2</VAR>
are not literally identical.

<DT><CODE><VAR>Term1</VAR> @&#60; <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX527"></A>
The term <VAR>Term1</VAR> is before the term <VAR>Term2</VAR> in the standard order.

<DT><CODE><VAR>Term1</VAR> @&#62; <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX528"></A>
The term <VAR>Term1</VAR> is after the term <VAR>Term2</VAR> in the standard order.

<DT><CODE><VAR>Term1</VAR> @=&#60; <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX529"></A>
The term <VAR>Term1</VAR> is not after the term <VAR>Term2</VAR> in the standard order.

<DT><CODE><VAR>Term1</VAR> @&#62;= <VAR>Term2</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX530"></A>
The term <VAR>Term1</VAR> is not before the term <VAR>Term2</VAR> in the standard order.
</DL>

<P>
Some further predicates involving comparison of terms are:

</P>
<DL COMPACT>

<DT><CODE>?=(<VAR>?X</VAR>,<VAR>?Y</VAR>)</CODE>
<DD>
<A NAME="IDX531"></A>
<VAR>X</VAR> and <VAR>Y</VAR> are either syntactically identical or
syntactically non-unifiable.

<DT><CODE>compare(<VAR>?Op</VAR>,<VAR>?Term1</VAR>,<VAR>?Term2</VAR>)</CODE>
<DD>
<A NAME="IDX532"></A>
The result of comparing terms <VAR>Term1</VAR> and <VAR>Term2</VAR> is <VAR>Op</VAR>,
where the possible values for <VAR>Op</VAR> are:

<DL COMPACT>

<DT><CODE>=</CODE>
<DD>
if <VAR>Term1</VAR> is identical to <VAR>Term2</VAR>,

<DT><CODE>&#60;</CODE>
<DD>
if <VAR>Term1</VAR> is before <VAR>Term2</VAR> in the standard order,

<DT><CODE>&#62;</CODE>
<DD>
if <VAR>Term1</VAR> is after <VAR>Term2</VAR> in the standard order.
</DL>

Thus <CODE>compare(=,Term1,Term2)</CODE> is equivalent to
<CODE>Term1 == Term2</CODE>.  
<DT><CODE>sort(<VAR>+List1</VAR>,<VAR>?List2</VAR>)</CODE>
<DD>
<A NAME="IDX533"></A>
The elements of the list <VAR>List1</VAR> are sorted into the standard order
(see section <A HREF="sicstus_10.html#SEC102">Comparison of Terms</A>) and any identical elements are merged, yielding
the list <VAR>List2</VAR>.  (The time and space complexity of this operation
is at worst <VAR>O(N lg N)</VAR> where <VAR>N</VAR> is the length of <VAR>List1</VAR>.)

<DT><CODE>keysort(<VAR>+List1</VAR>,<VAR>?List2</VAR>)</CODE>
<DD>
<A NAME="IDX534"></A>
The list <VAR>List1</VAR> must consist of items of the form <VAR>Key-Value</VAR>.
These items are sorted into order according to the value of <VAR>Key</VAR>,
yielding the list <VAR>List2</VAR>.  No merging takes place.  This predicate
is <EM>stable</EM>, i.e. if <CODE>K-A</CODE> occurs before <CODE>K-B</CODE> in the
input, then <CODE>K-A</CODE> will occur before <CODE>K-B</CODE> in the output.  (The
time and space complexity of this operation is at worst <VAR>O(N lg N)</VAR>
where <VAR>N</VAR> is the length of <VAR>List1</VAR>.)
</DL>



<H2><A NAME="SEC103" HREF="sicstus_toc.html#TOC103">Control</A></H2>

<DL COMPACT>

<DT><CODE><VAR>+P</VAR> , <VAR>+Q</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX535"></A>
<VAR>P</VAR> and <VAR>Q</VAR>.

<DT><CODE><VAR>+P</VAR> ; <VAR>+Q</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX536"></A>
<VAR>P</VAR> or <VAR>Q</VAR>.

<DT><CODE>! <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX537"></A>
See section <A HREF="sicstus_5.html#SEC48">The Cut Symbol</A>.

<DT><CODE>\+ <VAR>+P</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX538"></A>
Fails if the goal <VAR>P</VAR> has a solution, and succeeds otherwise.  This is not
real negation ("<VAR>P</VAR> is false"), but a kind of pseudo-negation
meaning "<VAR>P</VAR> is not provable".  It is defined as if by 

<PRE>
\+(<VAR>P</VAR>) :- <VAR>P</VAR>, !, fail.
\+(_).
</PRE>

<STRONG>No cuts are allowed in <VAR>P</VAR>.</STRONG>

Remember that with prefix operators such as this one it is necessary to be
careful about spaces if the argument starts with a <KBD>(</KBD>.  For
example: 

<PRE>
| ?- \+ (<VAR>P</VAR>,<VAR>Q</VAR>).
</PRE>

is this operator applied to the conjunction of <VAR>P</VAR> and <VAR>Q</VAR>, but 

<PRE>
| ?- \+(<VAR>P</VAR>,<VAR>Q</VAR>).
</PRE>

would require a predicate <CODE>\+ /2</CODE> for its solution.  The prefix
operator can however be written as a functor of one argument; thus


<PRE>
| ?- \+((<VAR>P</VAR>,<VAR>Q</VAR>)).
</PRE>

is also correct.

<DT><CODE><VAR>+P</VAR> -&#62; <VAR>+Q</VAR> ; <VAR>+R</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX539"></A>
Analogous to


<PRE>
if <VAR>P</VAR> then <VAR>Q</VAR> else <VAR>R</VAR>
</PRE>

and defined as if by 


<PRE>
(<VAR>P</VAR> -&#62; <VAR>Q</VAR>; <VAR>R</VAR>) :- <VAR>P</VAR>, !, <VAR>Q</VAR>.
(<VAR>P</VAR> -&#62; <VAR>Q</VAR>; <VAR>R</VAR>) :- <VAR>R</VAR>.
</PRE>

except the scope of any cut in <VAR>Q</VAR> or <VAR>R</VAR> extends beyond the
if-then-else construct.
<STRONG>No cuts are allowed in <VAR>P</VAR>.</STRONG>

Note that this form of if-then-else only explores <EM>the first</EM>
solution to the goal <VAR>P</VAR>.
  
Note also that the <KBD>;</KBD> is not read as a disjunction operator in this case;
instead, it is part of the if-then-else construction.

The precedence of <KBD>-&#62;</KBD> is less than that of <KBD>;</KBD> (see section <A HREF="sicstus_5.html#SEC49">Operators</A>),
so the expression is read as


<PRE>
;(-&#62;(<VAR>P</VAR>,<VAR>Q</VAR>),<VAR>R</VAR>)
</PRE>

<DT><CODE><VAR>+P</VAR> -&#62; <VAR>+Q</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX540"></A>
When occurring as a goal, this construction is read as equivalent to


<PRE>
(<VAR>P</VAR> -&#62; <VAR>Q</VAR>; fail)
</PRE>

<DT><CODE>if(<VAR>+P</VAR>,<VAR>+Q</VAR>,<VAR>+R</VAR>)</CODE>
<DD>
<A NAME="IDX541"></A>
Analogous to


<PRE>
if <VAR>P</VAR> then <VAR>Q</VAR> else <VAR>R</VAR>
</PRE>

but differs from <CODE>P -&#62; Q ; R</CODE> in that <CODE>if(P, Q, R)</CODE> explores
<EM>all</EM> solutions to the goal <VAR>P</VAR>.  There is a small time penalty
for this--if <VAR>P</VAR> is known to have only one solution of interest, the
form <CODE>P -&#62; Q ; R</CODE> should be preferred.

<STRONG>No cuts are allowed in <VAR>P</VAR>.</STRONG>

<DT><CODE>otherwise</CODE>
<DD>
<DT><CODE>true <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX542"></A>
<A NAME="IDX543"></A>
These always succeed.  Use of <CODE>otherwise/0</CODE> is discouraged, because
it is not as portable as <CODE>true/0</CODE>, and because the former may
suggest a completely different semantics than the latter.  
<DT><CODE>false</CODE>
<DD>
<DT><CODE>fail <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX544"></A>
<A NAME="IDX545"></A>
These always fail.  Use of <CODE>false/0</CODE> is discouraged, because it is
not as portable as <CODE>fail/0</CODE>, and because the latter has a more
procedural flavor to it.  
<DT><CODE>repeat <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX546"></A>
Generates an infinite sequence of backtracking choices.  In sensible code,
<CODE>repeat/0</CODE> is hardly ever used except in <EM>repeat loops</EM>.  A
repeat loop has the structure 

<PRE>
<VAR>Head</VAR> :-
        ...
        <VAR>save</VAR>(OldState),
        repeat,
          <VAR>generate</VAR>(Datum),
          <VAR>action</VAR>(Datum),
          <VAR>test</VAR>(Datum),
        !,
        <VAR>restore</VAR>(OldState),
        ...
</PRE>

The purpose is to repeatedly perform some <VAR>action</VAR> on elements which
are somehow <VAR>generate</VAR>d, e.g. by reading them from a stream, until
some <VAR>test</VAR> becomes true.  Usually, <VAR>generate</VAR>, <VAR>action</VAR>,
and <VAR>test</VAR> are all determinate.  Repeat loops cannot contribute to
the logic of the program.  They are only meaningful if the <VAR>action</VAR>
involves side-effects.  
The only reason for using repeat loops instead of a more natural
tail-recursive formulation is efficiency: when the <VAR>test</VAR> fails
back, the Prolog engine immediately reclaims any working storage
consumed since the call to <CODE>repeat/0</CODE>.  
<DT><CODE>call(<VAR>:Term</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>incore(<VAR>:Term</VAR>)</CODE>
<DD>
<DT><CODE><VAR>:Term</VAR></CODE>
<DD>
<A NAME="IDX547"></A>
<A NAME="IDX548"></A>
<A NAME="IDX549"></A>
If <VAR>Term</VAR> is instantiated to a term which would be acceptable as the
body of a clause, then the goal <CODE>call(<VAR>Term</VAR>)</CODE> is executed
exactly as if that term appeared textually in its place, except that any
cut (<CODE>!</CODE>) occurring in <VAR>Term</VAR> only cuts alternatives in the
execution of <VAR>Term</VAR>.  Use of <CODE>incore/1</CODE> is not recommended.

If <VAR>Term</VAR> is not instantiated as described above, an error message
is printed and the call fails.  
<DT><CODE>call_cleanup(<VAR>:Goal</VAR>,<VAR>:Cleanup</VAR>)</CODE>
<DD>
<A NAME="IDX550"></A>
This construction can be used to ensure that <VAR>Cleanup</VAR> is executed
as soon as <VAR>Goal</VAR> has completed execution, no matter how it
finishes.  In more detail:

When <CODE>call_cleanup/2</CODE> with a continuation <VAR>C</VAR> is called or
backtracked into, first <VAR>Goal</VAR> is called or backtracked into.  Then
there are four possibilities:


<OL>
<LI>

<VAR>Goal</VAR> succeeds deterministically, possibly leaving some blocked subgoals.
<VAR>Cleanup</VAR> is executed with continuation <VAR>C</VAR>.

<LI>

<VAR>Goal</VAR> succeeds with some alternatives outstanding.  Execution
proceeds to <VAR>C</VAR>.  If a cut that removes the outstanding alternatives
is encountered, <VAR>Cleanup</VAR> is executed with continuation to proceed
after the cut.  Also, if an exception <VAR>E</VAR> that will be caught by an
ancestor of the <CODE>call_cleanup/2</CODE> <VAR>Goal</VAR> is raised,
<VAR>Cleanup</VAR> is executed with continuation
<CODE>raise_exception(<VAR>E</VAR>)</CODE>.

<LI>

<VAR>Goal</VAR> fails.  <VAR>Cleanup</VAR> is executed with continuation <CODE>fail</CODE>.

<LI>

<VAR>Goal</VAR> raises an exception <VAR>E</VAR>.  <VAR>Cleanup</VAR> is executed with
continuation <CODE>raise_exception(<VAR>E</VAR>)</CODE>.
</OL>

In a typical use of <CODE>call_cleanup/2</CODE>, <VAR>Cleanup</VAR> succeeds
deterministically after performing some side-effect; otherwise,
unexpected behavior may result.

Note that the Prolog top level operates as a read-execute-fail loop,
which backtracks into or cuts the query when the user types <KBD>;</KBD> or
<KBD>RET</KBD> respectively.  Also, the predicates <CODE>halt/0</CODE>,
<CODE>abort/0</CODE>, and <CODE>reinitialise/0</CODE> are implemented in terms of
exceptions.  All of these circumstances can trigger the execution of 
<VAR>Cleanup</VAR>.

</DL>



<H2><A NAME="SEC104" HREF="sicstus_toc.html#TOC104">Error and Exception Handling</A></H2>
<P>
<A NAME="IDX551"></A>
<A NAME="IDX552"></A>
<A NAME="IDX553"></A>
<A NAME="IDX554"></A>

</P>
<P>
The two built-in predicates <CODE>on_exception/3</CODE> and
<CODE>raise_exception/1</CODE> are used to alter the control flow to meet
exception and error conditions.  The equivalent of a
<CODE>raise_exception/1</CODE> is also executed by the built-in predicates
when errors occur.

</P>
<DL COMPACT>

<DT><CODE>on_exception(<VAR>?Pattern</VAR>,<VAR>:ProtectedGoal</VAR>,<VAR>:Handler</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>raise_exception(<VAR>+Exception</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX555"></A>
<A NAME="IDX556"></A>
<CODE>on_exception/3</CODE> calls <VAR>ProtectedGoal</VAR>.  If this succeeds or
fails, so does the call to <CODE>on_exception/3</CODE>.  If however, during
the execution of <VAR>ProtectedGoal</VAR>, there is a call to
<CODE>raise_exception(<VAR>Exception</VAR>)</CODE>, then <VAR>Exception</VAR> is copied
and the stack is unwound back to the call to <CODE>on_exception/3</CODE>,
whereupon the copy of <VAR>Exception</VAR> is unified with <VAR>Pattern</VAR>.  If
this unification succeeds, then <CODE>on_exception/3</CODE> calls the goal
<VAR>Handler</VAR> in order to determine the success or failure of
<CODE>on_exception/3</CODE>.  Otherwise, the stack keeps unwinding, looking
for an earlier invocation of <CODE>on_exception/3</CODE>.  <VAR>Exception</VAR> may
be any term.

Certain built-in and library predicates rely on the exception mechanism,
so it is usually a bad idea to let <VAR>Pattern</VAR> be a variable,
matching any exception.  If it must be a variable, the <VAR>Handler</VAR>
should examime the exception and pass it on if it is not relevant
to the current invocation.

Instead of the above two predicates, the ISO Prolog standard prescribes
the following two.  They are functionality equivalent to the above two
predicates, but beware of the different argument order:


<PRE>
catch(Goal, Catcher, Recovery) :-
        on_exception(Catcher, Goal, Recovery).

throw(Ball) :-
        raise_exception(Ball).
</PRE>

</DL>

<P>
In a Development System, any previously uncaught exception is caught
and an appropriate error message is printed before returning to the top
level.  In recursive calls to Prolog from C, uncaught exceptions are
returned back to C instead.  The printing of these and
most other messages in a Development System is handled by the
predicate <CODE>print_message/2</CODE>.  The behavior of this predicate can be
overridden by defining <CODE>user:portray_message/2</CODE>, so as to suppress
or alter the format of certain messages.  These predicates work as
follows:

</P>
<DL COMPACT>

<DT><CODE>print_message(<VAR>+Severity</VAR>, <VAR>+Message</VAR>)</CODE>
<DD>
<A NAME="IDX557"></A>
<EM>Hookable</EM>.  Most messages from the system are printed by calling
this predicate.  Before anything is printed, however,
<CODE>print_message/2</CODE> calls <CODE>user:portray_message/2</CODE> with the same
arguments, so as to give the user a means of intercepting the message
before it is actually printed.  If <CODE>user:portray_message/2</CODE>
succeeds, nothing is printed, otherwise <VAR>Message</VAR> is formatted and
printed using the default method.

<VAR>Message</VAR> is a term that encodes the message to be printed.  The
format of message terms is subject to change, but can be inspected in
the file <TT>`Bips/msgs.pl'</TT> of the SICStus Prolog distribution.
<VAR>Severity</VAR> is a term denoting the severity of the message, and is
one of:

<DL COMPACT>

<DT><CODE>force(<VAR>Severity</VAR>)</CODE>
<DD>
<VAR>Message</VAR> should be printed without calling the <CODE>user:portray_message/2</CODE> hook.
This is useful if <CODE>user:portray_message/2</CODE> has intercepted 
the message, and now wants to print a reformatted version of it using
<CODE>print_message/2</CODE>.
<DT><CODE>error</CODE>
<DD>
<VAR>Message</VAR> is an uncaught exception.  The execution will normally be
aborted and return to the top level.  Syntax errors and exceptions that
occur while loading files do not necessarily abort the execution,
however.
<DT><CODE>warning</CODE>
<DD>
<VAR>Message</VAR> is a warning (e.g. singleton variables).
<DT><CODE>informational</CODE>
<DD>
<VAR>Message</VAR> provides information e.g. about files being loaded.
<DT><CODE>help</CODE>
<DD>
<VAR>Message</VAR> is normally a response to a query.
</DL>

<DT><CODE>portray_message(<VAR>+Severity</VAR>, <VAR>+Message</VAR>)</CODE>
<DD>
<DT><CODE>user:portray_message(<VAR>+Severity</VAR>, <VAR>+Message</VAR>)</CODE>
<DD>
<A NAME="IDX558"></A>
<A NAME="IDX559"></A>
<EM>A hook predicate.</EM> Called by <CODE>print_message/2</CODE> before
printing the message.  If this succeeds, the default message for
printing <VAR>Message</VAR> is overridden, and nothing more is printed.
</DL>

<P>
The built-in predicates may raise exceptions as follows on errors.
Usually the goal and the argument number causing the error are
contained in the exception.

</P>
<DL COMPACT>

<DT><CODE>instantiation_error(<VAR>Goal</VAR>,<VAR>ArgNo</VAR>)</CODE>
<DD>
<A NAME="IDX560"></A>
<VAR>Goal</VAR> was called with insufficiently instantiated variables.

<DT><CODE>type_error(<VAR>Goal</VAR>,<VAR>ArgNo</VAR>,<VAR>TypeName</VAR>,<VAR>Culprit</VAR>)</CODE>
<DD>
<A NAME="IDX561"></A>
<VAR>Goal</VAR> was called with the wrong type of argument(s).
<VAR>TypeName</VAR> is the expected type and <VAR>Culprit</VAR> what was actually
found.

<DT><CODE>domain_error(<VAR>Goal</VAR>,<VAR>ArgNo</VAR>,<VAR>Domain</VAR>,<VAR>Culprit</VAR>)</CODE>
<DD>
<A NAME="IDX562"></A>
<VAR>Goal</VAR> was called with argument(s) of the right type but with
illegal value(s).  <VAR>Domain</VAR> is the expected domain and <VAR>Culprit</VAR>
what was actually found.

<DT><CODE>existence_error(<VAR>Goal</VAR>,<VAR>ArgNo</VAR>,<VAR>ObjectType</VAR>,<VAR>Culprit</VAR>,<VAR>Reserved</VAR>)</CODE>
<DD>
<A NAME="IDX563"></A>
Something does not exist as indicated by the arguments.  If the
unknown-flag (see <CODE>prolog_flag/3</CODE>) is set to <CODE>error</CODE>, this
error is raised with <VAR>ArgNo</VAR> set to 0 when an undefined predicate is 
called.

<DT><CODE>permission_error(<VAR>Goal</VAR>,<VAR>Operation</VAR>,<VAR>ObjectType</VAR>,<VAR>Culprit</VAR>,<VAR>Reserved</VAR>)</CODE>
<DD>
<A NAME="IDX564"></A>
The <VAR>Operation</VAR> is not permitted on <VAR>Culprit</VAR> of the
<VAR>ObjectType</VAR>.

<DT><CODE>context_error(<VAR>Goal</VAR>,<VAR>ContextType</VAR>,<VAR>CommandType</VAR>)</CODE>
<DD>
<A NAME="IDX565"></A>
The <VAR>CommandType</VAR> is not permitted in <VAR>ContextType</VAR>.

<DT><CODE>syntax_error(<VAR>Goal</VAR>,<VAR>Position</VAR>,<VAR>Message</VAR>,<VAR>Tokens</VAR>,<VAR>AfterError</VAR>)</CODE>
<DD>
<A NAME="IDX566"></A>
A syntax error was found when reading a term with <CODE>read/(1-2)</CODE>.  This
error is raised only if the <CODE>syntax_errors</CODE> flag (see
<CODE>prolog_flag/3</CODE>) is set to <CODE>error</CODE>.

<DT><CODE>representation_error(<VAR>Goal</VAR>,<VAR>ArgNo</VAR>,<VAR>ErrorType</VAR>)</CODE>
<DD>
<A NAME="IDX567"></A>
A representation error occurs when the program tries to compute
some well-defined value which cannot be represented, such as a compound term
with arity &#62; 255.

<DT><CODE>consistency_error(<VAR>Goal</VAR>,<VAR>Culprit1</VAR>,<VAR>Culprit2</VAR>,<VAR>Message</VAR>)</CODE>
<DD>
<A NAME="IDX568"></A>
A consistency error occurs when two otherwise valid values or operations have
been specified which are inconsistent with each other.  

<DT><CODE>system_error(<VAR>Message</VAR>)</CODE>
<DD>
<A NAME="IDX569"></A>
An error occurred while dealing with the operating system.
</DL>

<P>
It is possible to handle a particular kind of existence errors locally:
calls to undefined predicates.  This can be done by defining clauses for:

</P>
<DL COMPACT>

<DT><CODE>unknown_predicate_handler(<VAR>+Goal</VAR>,<VAR>+Module</VAR>,<VAR>-NewGoal</VAR>)</CODE>
<DD>
<DT><CODE>user:unknown_predicate_handler(<VAR>+Goal</VAR>,<VAR>+Module</VAR>,<VAR>-NewGoal</VAR>)</CODE>
<DD>
<A NAME="IDX570"></A>
<A NAME="IDX571"></A>
<A NAME="IDX572"></A>
<EM>A hook predicate.</EM>  This predicate is called as a result of a call
to an undefined predicate. <VAR>Goal</VAR> is bound to the goal of the
undefined predicate and <VAR>Module</VAR> to the module where the call was
made.  If this predicate succeeds, <VAR>Module:NewGoal</VAR> is called;
otherwise, the action taken is governed by the <CODE>unknown</CODE>
Prolog flag.
</DL>

<P>
The following example shows an auto-loader for library packages:

</P>

<PRE>
user:unknown_predicate_handler(Goal, Module, Goal) :-
        functor(Goal, Name, Arity),
        require(Module:(Name/Arity)).
</PRE>



<H2><A NAME="SEC105" HREF="sicstus_toc.html#TOC105">Information about the State of the Program</A></H2>
<P>
<A NAME="IDX573"></A>
<A NAME="IDX574"></A>

</P>
<DL COMPACT>

<DT><CODE>listing</CODE>
<DD>
<A NAME="IDX575"></A>
Lists onto the current output stream all the clauses in the current
interpreted program (in the type-in module; see section <A HREF="sicstus_7.html#SEC66">Module Prefixing</A>).
Clauses listed onto a file can be consulted back.

<DT><CODE>listing(<VAR>:Spec</VAR>)</CODE>
<DD>
<A NAME="IDX576"></A>
Lists all interpreted predicates covered by <VAR>Spec</VAR>, which has
the same for as for <CODE>spy/1</CODE> (see section <A HREF="sicstus_9.html#SEC87">Spy-points</A>).  For example:


<PRE>
| ?- listing([concatenate/3, reverse, m:go/(2-3), bar:_]).
</PRE>

<DT><CODE>current_atom(<VAR>?Atom</VAR>)</CODE>
<DD>
<A NAME="IDX577"></A>
<VAR>Atom</VAR> is an atom known to SICStus Prolog.
Can be used to enumerate (through backtracking) all currently
known atoms, and return each one as <VAR>Atom</VAR>.  
<DT><CODE>current_predicate(<VAR>?Name</VAR>,<VAR>:Head</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>current_predicate(<VAR>?Name</VAR>,<VAR>-Head</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX578"></A>
<VAR>Name</VAR> is the name of a user defined or library predicate, and
<VAR>Head</VAR> is the most general goal for that predicate, possibly
prefixed by a module name.  This predicate can be used to enumerate
all user defined or library predicates through backtracking. 
<DT><CODE>predicate_property(<VAR>:Head</VAR>,<VAR>?Property</VAR>)</CODE>
<DD>
<DT><CODE>predicate_property(<VAR>-Head</VAR>,<VAR>?Property</VAR>)</CODE>
<DD>
<A NAME="IDX579"></A>
<VAR>Head</VAR> is the most general goal for an existing predicate,
possibly prefixed by a module name,
and <VAR>Property</VAR> is a property of that predicate, where the possible
properties are 

<UL>
<LI>

one of the atoms <CODE>built_in</CODE> (for built-in predicates) or
<CODE>compiled</CODE> or <CODE>interpreted</CODE> (for user defined predicates) or
<CODE>fd_constraint</CODE> for FD predicates see section <A HREF="sicstus_33.html#SEC273">Defining Primitive Constraints</A>.

<LI>

the atom <CODE>dynamic</CODE> for
predicates that have been declared dynamic
(see section <A HREF="sicstus_8.html#SEC75">Dynamic Declarations</A>),

<LI>

the atom <CODE>multifile</CODE> for
predicates that have been declared multifile
(see section <A HREF="sicstus_8.html#SEC74">Multifile Declarations</A>),

<LI>

the atom <CODE>volatile</CODE> for
predicates that have been declared volatile
(see section <A HREF="sicstus_8.html#SEC76">Volatile Declarations</A>),

<LI>

one or more terms <CODE>(block <VAR>Term</VAR>)</CODE> for predicates that have
block declarations (see section <A HREF="sicstus_8.html#SEC77">Block Declarations</A>),

<LI>

the atom <CODE>exported</CODE> or terms <CODE>imported_from(<VAR>ModuleFrom</VAR>)</CODE>
for predicates exported or imported from modules (see section <A HREF="sicstus_7.html#SEC64">The Module System</A>),

<LI>

the term <CODE>(meta_predicate <VAR>Term</VAR>)</CODE> for predicates that have
meta-predicate declarations (see section <A HREF="sicstus_7.html#SEC70">Meta-Predicate Declarations</A>).

<LI>

in Muse only, the atom <CODE>sync</CODE> for predicates that perform side
effects that must be synchronized to preserve standard semantics.

<LI>

in Muse only, the atom <CODE>parallel</CODE> for predicates that may
be executed in parallel.
</UL>

This predicate can be used to enumerate all existing predicates and
their properties through backtracking.

<DT><CODE>current_module(<VAR>?Module</VAR>)</CODE>
<DD>
<A NAME="IDX580"></A>
<VAR>Module</VAR> is a module in the system.  It can be used to backtrack
through all modules present in the system.  
<DT><CODE>current_module(<VAR>?Module</VAR>, <VAR>?File</VAR>)</CODE>
<DD>
<A NAME="IDX581"></A>
<VAR>Module</VAR> is the module defined in <VAR>File</VAR>.

<DT><CODE>module(<VAR>+Module</VAR>)</CODE>
<DD>
<A NAME="IDX582"></A>
The type-in module is set to <VAR>Module</VAR>.

<DT><CODE>prolog_flag(<VAR>+FlagName</VAR>,<VAR>?OldValue</VAR>,<VAR>?NewValue</VAR>)</CODE>
<DD>
<A NAME="IDX583"></A>
<A NAME="IDX584"></A>
<A NAME="IDX585"></A>
<VAR>OldValue</VAR> is the value of the Prolog flag <VAR>FlagName</VAR>,
and the new value of <VAR>FlagName</VAR> is set to <VAR>NewValue</VAR>.  The possible
Prolog flag names and values are: 
<DL COMPACT>

<DT><CODE>agc_margin</CODE>
<DD>
An integer <VAR>Margin</VAR>.  The atoms will be garbage collected when
<VAR>Margin</VAR> new atoms have been created since the last atom garbage
collection.  Initially 10000.  <A NAME="IDX586"></A>

<DT><CODE>argv</CODE>
<DD>
A read-only flag.  The value is a list of atoms of the program arguments
supplied when the current SICStus Prolog process was started.  For example, if
SICStus Prolog were invoked with:


<PRE>
% sicstus -a hello world 2001
</PRE>

then the value will be <CODE>[hello,world,'2001']</CODE>.
<A NAME="IDX587"></A>

<DT><CODE>compiling</CODE>
<DD>
Governs the mode in which <CODE>compile/1</CODE> and <CODE>fcompile/1</CODE>
operate (see section <A HREF="sicstus_8.html#SEC71">Loading Programs</A>).
<A NAME="IDX588"></A>

<DL COMPACT>

<DT><CODE>compactcode</CODE>
<DD>
Compilation produces byte-coded abstract instructions (the default).

<DT><CODE>fastcode</CODE>
<DD>
Compilation produces native machine instructions.  Currently only
available for 680x0, Sparc, and MIPS platforms.

<DT><CODE>profiledcode</CODE>
<DD>
Compilation produces byte-coded abstract instructions instrumented to
produce execution profiling data.  

<DT><CODE>debugcode</CODE>
<DD>
Compiling is replaced by consulting.
</DL>

<DT><CODE>debugging</CODE>
<DD>
Corresponds to the predicates <CODE>debug/0</CODE>, <CODE>nodebug/0</CODE>,
<CODE>trace/0</CODE>, <CODE>notrace/0</CODE>, <CODE>zip/0</CODE>, <CODE>nozip/0</CODE>
(see section <A HREF="sicstus_10.html#SEC113">Debugging</A>).
<A NAME="IDX589"></A>

<DL COMPACT>

<DT><CODE>trace</CODE>
<DD>
Turns on trace mode.

<DT><CODE>debug</CODE>
<DD>
Turns on the debugger.

<DT><CODE>off</CODE>
<DD>
Turns off trace mode and the debugger (the default).
</DL>
(This flag is not available in Runtime Systems.)

<DT><CODE>character_escapes</CODE>
<DD>
<CODE>on</CODE> or <CODE>off</CODE>.  If this flag is <CODE>on</CODE>, 
a backslash occurring inside integers in <SAMP>`0''</SAMP> notation or inside
quoted atoms or strings has special meaning, and indicates the start of
an escape sequence (see section <A HREF="sicstus_45.html#SEC372">Escape Sequences</A>).  This flag is relevant
when reading as well as when writing terms, and is initially <CODE>on</CODE>.
<A NAME="IDX590"></A>
<A NAME="IDX591"></A>
<A NAME="IDX592"></A>

<DT><CODE>fileerrors</CODE>
<DD>
<CODE>on</CODE> or <CODE>off</CODE>.  Turns raising of exception on file errors on or
off.  Equivalent to <CODE>fileerrors/0</CODE> and <CODE>nofileerrors/0</CODE>,
respectively (see section <A HREF="sicstus_10.html#SEC98">Stream I/O</A>).  Initially <CODE>on</CODE>.
<A NAME="IDX593"></A>

<DT><CODE>gc</CODE>
<DD>
<CODE>on</CODE> or <CODE>off</CODE>.  Turns garbage collection of the global stack
on or off.  Initially <CODE>on</CODE>.
<A NAME="IDX594"></A>
<A NAME="IDX595"></A>

<DT><CODE>gc_margin</CODE>
<DD>
<VAR>Margin</VAR>: Number of kilobytes.  If less than <VAR>Margin</VAR> kilobytes
are reclaimed in a garbage collection of the global stack then the size of the global stack should be increased.  Also, no garbage collection is
attempted unless the global stack is at least <VAR>Margin</VAR>
kilobytes.  Initially 500.  <A NAME="IDX596"></A>

<DT><CODE>gc_trace</CODE>
<DD>
Governs global stack garbage collection trace messages.
<A NAME="IDX597"></A>

<DL COMPACT>

<DT><CODE>verbose</CODE>
<DD>
Turn on verbose tracing of garbage collection.

<DT><CODE>terse</CODE>
<DD>
Turn on terse tracing of garbage collection.

<DT><CODE>off</CODE>
<DD>
Turn off tracing of garbage collection (the default).
</DL>

<DT><CODE>redefine_warnings</CODE>
<DD>
<CODE>on</CODE> or <CODE>off</CODE>.  Enable or disable warning messages when a
predicate is being


<UL>
<LI>

redefined from a different file than its previous definition.
<LI>

imported to the <CODE>user</CODE> module and it was previously locally defined.
<LI>

redefined locally and it was previously imported.
<LI>

imported to the <CODE>user</CODE> module from another module than it was
previously imported from.
</UL>

Initially <CODE>on</CODE>.  (This warning is always
disabled in Runtime Systems.)
<A NAME="IDX598"></A>

<DT><CODE>single_var_warnings</CODE>
<DD>
<CODE>on</CODE> or <CODE>off</CODE>.  Enable or disable warning messages when a
clause containing variables not beginning with <KBD>_</KBD> occurring once
only is compiled or consulted.  Initially <CODE>on</CODE>.  <A NAME="IDX599"></A>

<DT><CODE>unknown</CODE>
<DD>
Corresponds to the predicate <CODE>unknown/2</CODE> (see section <A HREF="sicstus_10.html#SEC113">Debugging</A>).
<A NAME="IDX600"></A>
<A NAME="IDX601"></A>
<A NAME="IDX602"></A>

<DL COMPACT>

<DT><CODE>trace</CODE>
<DD>
Causes calls to undefined predicates to be reported and the debugger to
be entered at the earliest opportunity.  (This setting is not possible
in Runtime Systems.)

<DT><CODE>fail</CODE>
<DD>
Causes calls to such predicates to fail.  

<DT><CODE>error</CODE>
<DD>
Causes calls to such predicates to raise an exception (the default).
See section <A HREF="sicstus_10.html#SEC104">Error and Exception Handling</A>.
</DL>

<DT><CODE>syntax_errors</CODE>
<DD>
<A NAME="IDX603"></A>
Controls what action is taken upon syntax errors in <CODE>read/(1-2)</CODE>.

<DL COMPACT>

<DT><CODE>dec10</CODE>
<DD>
The syntax error is reported and the read is repeated.

<DT><CODE>error</CODE>
<DD>
An exception is raised.  See section <A HREF="sicstus_10.html#SEC104">Error and Exception Handling</A>. (the default).

<DT><CODE>fail</CODE>
<DD>
The syntax error is reported and the read fails.  

<DT><CODE>quiet</CODE>
<DD>
The read quietly fails.  
</DL>
 
<DT><CODE>system_type</CODE>
<DD>
<A NAME="IDX604"></A>
A read-only flag.  The value is <CODE>development</CODE> in Development
Systems and <CODE>runtime</CODE> in Runtime Systems.
  
<DT><CODE>typein_module</CODE>
<DD>
<A NAME="IDX605"></A>
Permitted values are atoms.  Controls the current type-in module
(see section <A HREF="sicstus_7.html#SEC66">Module Prefixing</A>).  Corresponds to the predicate <CODE>module/1</CODE>.
<A NAME="IDX606"></A>
<A NAME="IDX607"></A>

<DT><CODE>user_input</CODE>
<DD>
<A NAME="IDX608"></A>
Permitted values are any stream opened for reading. 
Controls which stream is referenced by
<CODE>user_input</CODE> and <CODE>SP_stdin</CODE>. It is initially set to a stream
connected to UNIX <CODE>stdin</CODE>.
 
<DT><CODE>user_output</CODE>
<DD>
<A NAME="IDX609"></A>
Permitted values are any stream opened for writing. 
Controls which stream is referenced by
<CODE>user_output</CODE> and <CODE>SP_stdout</CODE>. It is initially set to a stream
connected to UNIX <CODE>stdout</CODE>.
 
<DT><CODE>user_error</CODE>
<DD>
<A NAME="IDX610"></A>
Permitted values are any stream opened for writing. 
Controls which stream is referenced by
<CODE>user_error</CODE> and <CODE>SP_stderr</CODE>. It is initially set to a stream
connected to UNIX <CODE>stderr</CODE>.
 
<DT><CODE>version</CODE>
<DD>
<A NAME="IDX611"></A>
A read-only flag.  The value is an atom containing the banner text
displayed on startup and reinitialization, such as
<CODE>'SICStus 3 #0: Wed Mar 15 12:29:29 MET 1995'</CODE>.
 
<DT><CODE>toplevel_print_options</CODE>
<DD>
<A NAME="IDX612"></A>
The value is a list of options for <CODE>write_term/3</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>),
to be used when the top level displays variable bindings, answer
constraints, and uncaught exceptions.  Not available in Runtime Systems.
The initial value is
<CODE>[quoted(true),numbervars(true),portrayed(true),max_depth(10)]</CODE>.

<DT><CODE>debugger_print_options</CODE>
<DD>
<A NAME="IDX613"></A>
The value is a list of options for <CODE>write_term/3</CODE> (see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>),
to be used in the debugger's messages.  Not available in Runtime Systems.
The initial value is
<CODE>[quoted(true),numbervars(true),portrayed(true),max_depth(10)]</CODE>.

<DT><CODE>source_info</CODE>
<DD>
<A NAME="IDX614"></A>
<A NAME="IDX615"></A>
<A NAME="IDX616"></A>
<CODE>on</CODE> or <CODE>off</CODE>.  
If <CODE>on</CODE> while code is being loaded, information about line
numbers and filenames are stored with the loaded code.  If <CODE>on</CODE>
while debugging, this information is used to display the source code
location while prompting for a debugger command.  Initially
<CODE>off</CODE>.  Currently, this facility is only useful with the GNU
Emacs interface. 
</DL>

<DT><CODE>prolog_flag(<VAR>?FlagName</VAR>,<VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX617"></A>
<A NAME="IDX618"></A>
<A NAME="IDX619"></A>
<VAR>Value</VAR> is the current value of the Prolog flag
<VAR>FlagName</VAR>.  Can be used to enumerate all Prolog flags and their
values by backtracking.

<DT><CODE>prolog_load_context(<VAR>?Key</VAR>,<VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX620"></A>
<A NAME="IDX621"></A>
<A NAME="IDX622"></A>
<A NAME="IDX623"></A>
This predicate gives access to context variables during compilation and
loading of Prolog files.  It unifies <VAR>Value</VAR> with the value of the
variable identified by <VAR>Key</VAR>. Possible keys are:

<DL COMPACT>

<DT><CODE>file</CODE>
<DD>
The absolute path name of the file being compiled. During loading of a
ql-file, the corresponding source file name is returned.

<DT><CODE>directory</CODE>
<DD>
The absolute path name of the directory of the file being compiled/loaded.

<DT><CODE>module</CODE>
<DD>
The source module (see section <A HREF="sicstus_7.html#SEC69">Module Name Expansion</A>).  This is useful for example if you are
defining clauses for <CODE>user:term_expansion/(2,4)</CODE> and need to access the
source module at compile time.

<DT><CODE>stream</CODE>
<DD>
The stream being compiled or loaded from.

<DT><CODE>term_position</CODE>
<DD>
A term representing the position of the last clause read (see section <A HREF="sicstus_10.html#SEC98">Stream I/O</A>).
</DL>

<DT><CODE>statistics</CODE>
<DD>
<A NAME="IDX624"></A>
Displays on the standard error stream statistics relating to memory
usage, run time, garbage collection of the global stack and stack
shifts.  
<DT><CODE>statistics(<VAR>?Key</VAR>,<VAR>?Value</VAR>)</CODE>
<DD>
<A NAME="IDX625"></A>
This allows a program to gather various execution statistics.  For each of
the possible keys <VAR>Key</VAR>, <VAR>Value</VAR> is unified with a list of values,
as follows: 
<DL COMPACT>

<DT><CODE>global_stack</CODE>
<DD>
<A NAME="IDX626"></A>
<CODE>[<VAR>size used</VAR>,<VAR>free</VAR>]</CODE><BR> 
This refers to the global stack, where compound terms are stored. The
values are gathered before the list holding the answers is allocated.

<DT><CODE>local_stack</CODE>
<DD>
<A NAME="IDX627"></A>
<CODE>[<VAR>size used</VAR>,<VAR>free</VAR>]</CODE><BR> 
This refers to the local stack, where recursive predicate environments
are stored.

<DT><CODE>trail</CODE>
<DD>
<A NAME="IDX628"></A>
<CODE>[<VAR>size used</VAR>,<VAR>free</VAR>]</CODE><BR>
This refers to the trail stack, where conditional variable bindings are
recorded.

<DT><CODE>choice</CODE>
<DD>
<A NAME="IDX629"></A>
<CODE>[<VAR>size used</VAR>,<VAR>free</VAR>]</CODE><BR>
This refers to the choicepoint stack, where partial states are stored
for backtracking purposes.

<DT><CODE>core</CODE>
<DD>
<A NAME="IDX630"></A>
<DT><CODE>memory</CODE>
<DD>
<A NAME="IDX631"></A>
<CODE>[<VAR>size used</VAR>,0]</CODE><BR>
These refer to the amount of memory actually allocated by the
process.

<DT><CODE>heap</CODE>
<DD>
<A NAME="IDX632"></A>
<DT><CODE>program</CODE>
<DD>
<A NAME="IDX633"></A>
<CODE>[<VAR>size used</VAR>,0]</CODE><BR>
These refer to the amount of memory allocated for compiled and
interpreted clauses, symbol tables, and the like.

<DT><CODE>runtime</CODE>
<DD>
<A NAME="IDX634"></A>
<CODE>[<VAR>since start of Prolog</VAR>,<VAR>since previous statistics</VAR>]</CODE>
These refer to CPU time used while executing, excluding time spent
garbage collecting, stack shifting, or in system calls.  In Muse, these
numbers refer to the worker that happens to be executing the call to
<CODE>statistics/2</CODE>, and so normally are not meaningful.

<DT><CODE>walltime</CODE>
<DD>
<A NAME="IDX635"></A>
<CODE>[<VAR>since start of Prolog</VAR>,<VAR>since previous statistics</VAR>]</CODE>
These refer to absolute time elapsed.

<DT><CODE>garbage_collection</CODE>
<DD>
<A NAME="IDX636"></A>
<CODE>[<VAR>no. of GCs</VAR>,<VAR>bytes freed</VAR>,<VAR>time spent</VAR>]</CODE>

<DT><CODE>stack_shifts</CODE>
<DD>
<A NAME="IDX637"></A>
<CODE>[<VAR>no. of local shifts</VAR>,<VAR>no. of trail shifts</VAR>,<VAR>time
spent</CODE>]</VAR>

<DT><CODE>atoms</CODE>
<DD>
<A NAME="IDX638"></A>
<CODE>[<VAR>no. of atoms</VAR>,<VAR>bytes used</VAR>,<VAR>bytes free</VAR>]</CODE>

<DT><CODE>atom_garbage_collection</CODE>
<DD>
<A NAME="IDX639"></A>
<CODE>[<VAR>no. of AGCs</VAR>,<VAR>bytes freed</VAR>,<VAR>time spent</VAR>]</CODE>

</DL>

Times are in milliseconds, sizes of areas in bytes.

</DL>



<H2><A NAME="SEC106" HREF="sicstus_toc.html#TOC106">Meta-Logic</A></H2>
<P>
<A NAME="IDX640"></A>

</P>
<P>
The predicates in this section are meta-logical and perform operations
that require reasoning about the current instantiation of terms or
decomposing terms into their constituents.  Such operations cannot be
expressed using predicate definitions with a finite number of clauses.

</P>
<DL COMPACT>

<DT><CODE>var(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX641"></A>
Tests whether <VAR>X</VAR> is currently uninstantiated (<CODE>var</CODE> is short for
variable).  An uninstantiated variable is one which has not been bound to
anything, except possibly another uninstantiated variable.  Note that a
structure with some components which are uninstantiated is not itself
considered to be uninstantiated.  Thus the directive 

<PRE>
| ?- var(foo(<VAR>X</VAR>, <VAR>Y</VAR>)).
</PRE>

always fails, despite the fact that <VAR>X</VAR> and <VAR>Y</VAR> are
uninstantiated.  
<DT><CODE>nonvar(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX642"></A>
Tests whether <VAR>X</VAR> is currently instantiated.  This is the opposite of
<CODE>var/1</CODE>.  
<DT><CODE>ground(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX643"></A>
Tests whether <VAR>X</VAR> is completely instantiated, i.e. free of
unbound variables.  In this context, mutable terms are treated as
nonground, so as to make <CODE>ground/1</CODE> a monotone predicate.

<DT><CODE>atom(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX644"></A>
Checks that <VAR>X</VAR> is currently instantiated to an atom (i.e. a
non-variable term of arity 0, other than a number).  
<DT><CODE>float(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX645"></A>
Checks that <VAR>X</VAR> is currently instantiated to a float.  
<DT><CODE>integer(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX646"></A>
Checks that <VAR>X</VAR> is currently instantiated to an integer.  
<DT><CODE>number(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX647"></A>
Checks that <VAR>X</VAR> is currently instantiated to a number.  
<DT><CODE>atomic(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX648"></A>
Checks that <VAR>X</VAR> is currently instantiated to an atom or number.

<DT><CODE>simple(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX649"></A>
Checks that <VAR>X</VAR> is currently uninstantiated or instantiated to an
atom or number.

<DT><CODE>compound(<VAR>?X</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX650"></A>
Checks that <VAR>X</VAR> is currently instantiated to a term of arity &#62;
0 i.e. a list or a structure.

<DT><CODE>callable(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX651"></A>
Checks that <VAR>X</VAR> is currently instantiated to a term valid as a goal
i.e. a compound term or an atom.

<DT><CODE>is_mutable(<VAR>?X</VAR>)</CODE>
<DD>
<A NAME="IDX652"></A>
Checks that <VAR>X</VAR> is currently instantiated to a mutable term
(see section <A HREF="sicstus_10.html#SEC107">Modification of Terms</A>).

<DT><CODE>functor(<VAR>+Term</VAR>,<VAR>?Name</VAR>,<VAR>?Arity</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>functor(<VAR>?Term</VAR>,<VAR>+Name</VAR>,<VAR>+Arity</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX653"></A>
The principal functor of term <VAR>Term</VAR> has name <VAR>Name</VAR> and arity
<VAR>Arity</VAR>, where <VAR>Name</VAR> is either an atom or, provided <VAR>Arity</VAR>
is 0, a number.  Initially, either <VAR>Term</VAR> must be instantiated, or
<VAR>Name</VAR> and <VAR>Arity</VAR> must be instantiated to, respectively, either
an atom and an integer in [0,255] or an atomic term and 0.  In the case
where <VAR>Term</VAR> is initially uninstantiated, the result of the call is
to instantiate <VAR>Term</VAR> to the most general term having the principal
functor indicated.  
<DT><CODE>arg(<VAR>+ArgNo</VAR>,<VAR>+Term</VAR>,<VAR>?Arg</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX654"></A>
<VAR>Arg</VAR> is the argument <VAR>ArgNo</VAR> of the compound term <VAR>Term</VAR>.  The
arguments are numbered from 1 upwards,
<VAR>ArgNo</VAR> must be instantiated to a positive integer and
<VAR>Term</VAR> to a compound term.

<DT><CODE><VAR>+Term</VAR> =.. <VAR>?List</VAR> <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE><VAR>?Term</VAR> =.. <VAR>+List</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX655"></A>
<VAR>List</VAR> is a list whose head is the atom corresponding to the principal
functor of <VAR>Term</VAR>, and whose tail is a list of the arguments of
<VAR>Term</VAR>.  e.g. 

<PRE>
| ?- product(0, n, n-1) =.. L.

L = [product,0,n,n-1]

| ?- n-1 =.. L.

L = [-,n,1]

| ?- product =.. L.

L = [product]
</PRE>

If <VAR>Term</VAR> is uninstantiated, then <VAR>List</VAR> must be instantiated
either to a list of determinate length whose head is an atom, or to a
list of length 1 whose head is a number.  Note that this predicate is
not strictly necessary, since its functionality can be provided by
<CODE>arg/3</CODE> and <CODE>functor/3</CODE>, and using the latter two is usually
more efficient.  
<DT><CODE>name(<VAR>+Const</VAR>,<VAR>?CharList</VAR>)</CODE>
<DD>
<DT><CODE>name(<VAR>?Const</VAR>,<VAR>+CharList</VAR>)</CODE>
<DD>
<A NAME="IDX656"></A>
If <VAR>Const</VAR> is an atom or number, <VAR>CharList</VAR> is a list of the
character codes of the characters comprising the name of <VAR>Const</VAR>.
e.g. 

<PRE>
| ?- name(product, L).

L = [112,114,111,100,117,99,116]

| ?- name(product, "product").

| ?- name(1976, L).

L = [49,57,55,54]

| ?- name('1976', L).

L = [49,57,55,54]

| ?- name((:-), L).

L = [58,45]

</PRE>

If <VAR>Const</VAR> is uninstantiated, <VAR>CharList</VAR> must be instantiated to a
list of character codes.  If <VAR>CharList</VAR> can be interpreted as a
number, <VAR>Const</VAR> is unified with that number, otherwise with the atom
whose name is <VAR>CharList</VAR>.  E.g.


<PRE>
| ?- name(X, [58,45]).

X = :-

| ?- name(X, ":-").

X = :-

| ?- name(X, [49,50,51]).

X = 123
</PRE>

Note that there atoms are for which
<CODE>name(<VAR>Const</VAR>,<VAR>CharList</VAR>)</CODE> is true, but which will not be
constructed if <CODE>name/2</CODE> is called with <VAR>Const</VAR> uninstantiated.
One such atom is the atom <CODE>'1976'</CODE>.  It is recommended that new
programs use <CODE>atom_chars/2</CODE> or <CODE>number_chars/2</CODE>, as these
predicates do not have this inconsistency.  
<DT><CODE>atom_chars(<VAR>+Const</VAR>,<VAR>?CharList</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>atom_chars(<VAR>?Const</VAR>,<VAR>+CharList</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX657"></A>
The same as <CODE>name(<VAR>Const</VAR>,<VAR>CharList</VAR>)</CODE>, but <VAR>Const</VAR>
is constrained to be an atom.  
This predicate is called <CODE>atom_codes/2</CODE> in the ISO Prolog 
standard.

<DT><CODE>number_chars(<VAR>+Const</VAR>,<VAR>?CharList</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>number_chars(<VAR>?Const</VAR>,<VAR>+CharList</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX658"></A>
The same as <CODE>name(<VAR>Const</VAR>,<VAR>CharList</VAR>)</CODE>, but <VAR>Const</VAR> is
constrained to be a number.  
This predicate is called <CODE>number_codes/2</CODE> in the ISO Prolog 
standard.

<DT><CODE>copy_term(<VAR>?Term</VAR>,<VAR>?CopyOfTerm</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX659"></A>
<VAR>CopyOfTerm</VAR> is a renaming of <VAR>Term</VAR>, such that brand new
variables have been substituted for all variables in <VAR>Term</VAR>.
If any of the variables of <VAR>Term</VAR> have goals blocked on them,
the copied variables will have copies of the goals blocked on them as well.
Similarly, independent copies are substituted for any mutable terms in
<VAR>term</VAR>.  It behaves as if defined by:


<PRE>
copy_term(X, Y) :-
        assert('copy of'(X)),
        retract('copy of'(Y)).
</PRE>

The implementation of <CODE>copy_term/2</CODE> conserves space by not copying
ground subterms.
</DL>



<H2><A NAME="SEC107" HREF="sicstus_toc.html#TOC107">Modification of Terms</A></H2>

<P>
<A NAME="IDX660"></A>
<A NAME="IDX661"></A>
One of the tenets of logic programming is that terms are immutable
objects of the Herbrand universe, and the only sense in which they can
be modified is by means of instantiating non-ground parts.
There are, however, algorithms where destructive assignment is essential
for performance.  Although alien to the ideals of logic programming,
this feature can be defended on practical grounds.

</P>
<P>
SICStus Prolog provides an abstract datatype and three operations for
efficient <EM>backtrackable</EM> destructive assignment.  In other words,
any destructive assignments are transparently undone on backtracking.
Modifications that are intended to survive backtracking must be done by
asserting or retracting dynamic program clauses instead.  Unlike
previous releases of SICStus Prolog, destructive assignment of arbitrary
terms is not allowed.

</P>
<P>
<A NAME="IDX662"></A>
<A NAME="IDX663"></A>
A <EM>mutable term</EM> is represented as a compound terms with a reserved
functor: <CODE>'$mutable'(<VAR>Value</VAR>,<VAR>Timestamp</VAR>)</CODE> where <VAR>Value</VAR>
is the current value and <VAR>Timestamp</VAR> is reserved for bookkeeping
purposes.

</P>
<P>
Any copy of a mutable term created by <CODE>copy_term/2</CODE>, <CODE>assert</CODE>,
<CODE>retract</CODE>, an internal database predicate, or an all solutions
predicate, is an independent copy of the original mutable term.  Any
destructive assignment done to one of the copies will not affect the
other copy.

</P>
<P>
The following operations are provided:

</P>
<DL COMPACT>

<DT><CODE>create_mutable(<VAR>+Datum</VAR>,<VAR>-Mutable</VAR>)</CODE>
<DD>
<A NAME="IDX664"></A>
<VAR>Mutable</VAR> is a new mutable term with initial value <VAR>Datum</VAR>.
<VAR>Datum</VAR> must not be an unbound variable.

<DT><CODE>get_mutable(<VAR>?Datum</VAR>,<VAR>+Mutable</VAR>)</CODE>
<DD>
<A NAME="IDX665"></A>
<VAR>Datum</VAR> is the current value of the mutable term <VAR>Mutable</VAR>.

<DT><CODE>update_mutable(<VAR>+Datum</VAR>,<VAR>+Mutable</VAR>)</CODE>
<DD>
<A NAME="IDX666"></A>
Updates the current value of the mutable term <VAR>Mutable</VAR> to become
<VAR>Datum</VAR>.  <VAR>Datum</VAR> must not be an unbound variable.

<DT><CODE>is_mutable(<VAR>?Mutable</VAR>)</CODE>
<DD>
<A NAME="IDX667"></A>
Checks that <VAR>Mutable</VAR> is currently instantiated to a mutable term
(see section <A HREF="sicstus_10.html#SEC107">Modification of Terms</A>).
</DL>



<H2><A NAME="SEC108" HREF="sicstus_toc.html#TOC108">Modification of the Program</A></H2>

<P>
The predicates defined in this section allow modification of dynamic
predicates.  Dynamic clauses can be added (<EM>asserted</EM>) or removed
from the program (<EM>retracted</EM>).

</P>
<P>
For these predicates, the argument <VAR>Head</VAR> must be instantiated to an
atom or a compound term, with an optional module prefix.  The argument
<VAR>Clause</VAR> must be instantiated either to a term <CODE><VAR>Head</VAR> :-
<VAR>Body</VAR></CODE> or, if the body part is empty, to <VAR>Head</VAR>, with an
optional module prefix.  An empty body part is represented as
<CODE>true</CODE>.

</P>
<P>
Note that a term <CODE><VAR>Head</VAR> :- <VAR>Body</VAR></CODE> must be enclosed in
parentheses when it occurs as an argument of a compound term, as
<SAMP>`:-'</SAMP> is a standard infix operator with precedence greater than 1000
(see section <A HREF="sicstus_5.html#SEC49">Operators</A>), e.g.: 
</P>

<PRE>
| ?- assert((<VAR>Head</VAR> :- <VAR>Body</VAR>)).
</PRE>

<P>
Like recorded terms (see section <A HREF="sicstus_10.html#SEC109">Internal Database</A>), the clauses of dynamic
predicates have a unique implementation-defined identifier.  Some of
the predicates below have an additional argument which is this
identifier.  This identifier makes it possible to access clauses
directly instead of requiring a normal database (hash-table) lookup.

</P>
<DL COMPACT>

<DT><CODE>assert(<VAR>:Clause</VAR>)</CODE>
<DD>
<DT><CODE>assert(<VAR>:Clause</VAR>,<VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX668"></A>
<A NAME="IDX669"></A>
The current instance of <VAR>Clause</VAR> is interpreted as a clause and is
added to the current interpreted program.  The predicate concerned must
currently be dynamic or undefined and the position of the new clause
within it is implementation-defined.  <VAR>Ref</VAR> is a unique identifier
of the asserted clause.  Any uninstantiated variables in the
<VAR>Clause</VAR> will be replaced by new private variables, along with
copies of any subgoals blocked on these variables (see section <A HREF="sicstus_5.html#SEC46">Procedural Semantics</A>).

<DT><CODE>asserta(<VAR>:Clause</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>asserta(<VAR>:Clause</VAR>,<VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX670"></A>
<A NAME="IDX671"></A>
Like <CODE>assert/2</CODE>, except that the new clause becomes the
<EM>first</EM> clause for the predicate concerned.  
<DT><CODE>assertz(<VAR>:Clause</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>assertz(<VAR>:Clause</VAR>,<VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX672"></A>
<A NAME="IDX673"></A>
Like <CODE>assert/2</CODE>, except that the new clause becomes the <EM>last</EM>
clause for the predicate concerned.  
<DT><CODE>clause(<VAR>:Head</VAR>,<VAR>?Body</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>clause(<VAR>:Head</VAR>,<VAR>?Body</VAR>,<VAR>?Ref</VAR>)</CODE>
<DD>
<DT><CODE>clause(<VAR>?Head</VAR>,<VAR>?Body</VAR>,<VAR>+Ref</VAR>)</CODE>
<DD>
<A NAME="IDX674"></A>
<A NAME="IDX675"></A>
The clause <CODE>(<VAR>Head</VAR> :- <VAR>Body</VAR>)</CODE> exists in the current
interpreted program, and is uniquely identified by <VAR>Ref</VAR>.  The
predicate concerned must currently be dynamic.  At the time of call,
either <VAR>Ref</VAR> must be instantiated to a valid identifier, or
<VAR>Head</VAR> must be instantiated to an atom or a compound term.  Thus
<CODE>clause/3</CODE> can have two different modes of use.  
<DT><CODE>retract(<VAR>:Clause</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX676"></A>
The first clause in the current interpreted program that matches
<VAR>Clause</VAR> is erased.  The predicate concerned must currently be
dynamic.  <CODE>retract/1</CODE> may be used in a non-determinate fashion,
i.e. it will successively retract clauses matching the argument through
backtracking.  If reactivated by backtracking, invocations of the
predicate whose clauses are being retracted will proceed unaffected by
the retracts.  This is also true for invocations of <CODE>clause/(2-3)</CODE> for
the same predicate.  The space occupied by a retracted clause will be
recovered when instances of the clause are no longer in use.  
<DT><CODE>retractall(<VAR>:Head</VAR>)</CODE>
<DD>
<A NAME="IDX677"></A>
Erases all clauses whose head matches <VAR>Head</VAR>, where <VAR>Head</VAR> must
be instantiated to an atom or a compound term.  The predicate concerned
must currently be dynamic.  The predicate definition is retained.
</DL>

<P>
<STRONG>Note:</STRONG> all predicates mentioned above first look for a
predicate that is visible in the module in which the call textually
appears.  If no predicate is found, a new dynamic predicate (with no
clauses) is created automatically.  During OR-parallel execution,
however, predicates must not be defined on the fly like this.  It is
recommended to declare as dynamic predicates for which clauses will be
asserted.

</P>
<DL COMPACT>

<DT><CODE>abolish(<VAR>:Spec</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<DT><CODE>abolish(<VAR>:Name</VAR>,<VAR>+Arity</VAR>)</CODE>
<DD>
<A NAME="IDX678"></A>
<A NAME="IDX679"></A>
Erases all clauses of the predicate specified by <VAR>Spec</VAR> or
<CODE><VAR>Name</VAR>/<VAR>Arity</VAR></CODE>.  <VAR>Spec</VAR> has the same form as for
<CODE>spy/1</CODE> (see section <A HREF="sicstus_9.html#SEC87">Spy-points</A>) and <VAR>Name</VAR> may be prefixed by a
module name (see section <A HREF="sicstus_7.html#SEC66">Module Prefixing</A>).

The predicate definition and all associated information such as
spy-points is also erased.  The predicates concerned must all be user
defined.  
<DT><CODE>erase(<VAR>+Ref</VAR>)</CODE>
<DD>
<A NAME="IDX680"></A>
The dynamic clause or recorded term (see section <A HREF="sicstus_10.html#SEC109">Internal Database</A>) whose
implementation-defined identifier is <VAR>Ref</VAR> is effectively erased
from the internal database or interpreted program.  
<DT><CODE>instance(<VAR>+Ref</VAR>,<VAR>?Term</VAR>)</CODE>
<DD>
<A NAME="IDX681"></A>
A (most general) instance of the dynamic clause or recorded term whose
implementation-defined identifier is <VAR>Ref</VAR> is unified with
<VAR>Term</VAR>.  <VAR>Ref</VAR> must be instantiated to a legal identifier.
</DL>



<H2><A NAME="SEC109" HREF="sicstus_toc.html#TOC109">Internal Database</A></H2>
<P>
<A NAME="IDX682"></A>
  
The predicates described in this section were introduced in early
implementations of Prolog to provide efficient means of performing
operations on large quantities of data.  The introduction of indexed
dynamic predicates have rendered these predicates obsolete, and the sole
purpose of providing them is to support existing code.  There is no
reason whatsoever to use them in new code.

</P>
<P>
These predicates store arbitrary terms in the
database without interfering with the clauses which make up the program.
The terms which are stored in this way can subsequently be retrieved via
the key on which they were stored.  Many terms may be stored on the same
key, and they can be individually accessed by pattern matching.
Alternatively, access can be achieved via a special identifier which
uniquely identifies each recorded term and which is returned when the
term is stored.

</P>
<DL COMPACT>

<DT><CODE>recorded(<VAR>?Key</VAR>,<VAR>?Term</VAR>,<VAR>?Ref</VAR>)</CODE>
<DD>
<A NAME="IDX683"></A>
The internal database is searched for terms recorded under the key
<VAR>Key</VAR>.  These terms are successively unified with <VAR>Term</VAR> in the
order they occur in the database.  At the same time, <VAR>Ref</VAR> is
unified with the implementation-defined identifier uniquely identifying
the recorded item.  If the key is instantiated to a compound term, only
its principal functor is significant.  If the key is uninstantiated, all
terms in the database are successively unified with <VAR>Term</VAR> in the
order they occur.  
<DT><CODE>recorda(<VAR>+Key</VAR>,<VAR>?Term</VAR>,<VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX684"></A>
The term <VAR>Term</VAR> is recorded in the internal database as the first
item for the key <VAR>Key</VAR>, where <VAR>Ref</VAR> is its
implementation-defined identifier.  The key must be given, and only its
principal functor is significant.  Any uninstantiated variables in the
<VAR>Term</VAR> will be replaced by new private variables, along with copies
of any subgoals blocked on these variables (see section <A HREF="sicstus_5.html#SEC46">Procedural Semantics</A>).  
<DT><CODE>recordz(<VAR>+Key</VAR>,<VAR>?Term</VAR>,<VAR>-Ref</VAR>)</CODE>
<DD>
<A NAME="IDX685"></A>
Like <CODE>recorda/3</CODE>, except that the new term becomes the <EM>last</EM>
item for the key <VAR>Key</VAR>.  
<DT><CODE>current_key(<VAR>?KeyName</VAR>,<VAR>?KeyTerm</VAR>)</CODE>
<DD>
<A NAME="IDX686"></A>
<VAR>KeyTerm</VAR> is the most general form of the key for a currently
recorded term, and <VAR>KeyName</VAR> is the name of that key.  This
predicate can be used to enumerate in undefined order all keys for
currently recorded terms through backtracking.  </DL>



<H2><A NAME="SEC110" HREF="sicstus_toc.html#TOC110">Blackboard Primitives</A></H2>
<P>
<A NAME="IDX687"></A>

</P>
<P>
The predicates described in this section store arbitrary terms in a
per-module repository known as the "blackboard".  The main purpose of
the blackboard is to provide a means for communication between branches
executing in parallel, but the blackboard works equally well during
sequential execution.  The blackboard implements a mapping from keys to
values.  Keys are restricted to being atoms or integers in the range
<CODE>[-33554432, 33554431]</CODE>, whereas values are arbitrary
terms.  In contrast to the predicates described in the previous
sections, a given key can map to at most a single term.

</P>
<P>
Each Prolog module maintains its own blackboard, so as to avoid name
clashes if different modules happen to use the same keys.  The "key"
arguments of these predicates are subject to module name expansion, so
the module name does not have to be explicitly given unless multiple
Prolog modules are supposed to share a single blackboard.

</P>
<P>
The predicates below implement atomic blackboard actions.  In Muse,
these predicates are <EM>cavalier</EM> (see section <A HREF="sicstus_6.html#SEC63">Programming Considerations</A>).

</P>
<DL COMPACT>

<DT><CODE>bb_put(<VAR>:Key</VAR>, <VAR>+Term</VAR>)</CODE>
<DD>
<A NAME="IDX688"></A>
A copy of <VAR>Term</VAR> is stored under <VAR>Key</VAR>.  Any previous term stored under
the same <VAR>Key</VAR> is simply deleted.

<DT><CODE>bb_get(<VAR>:Key</VAR>, <VAR>?Term</VAR>)</CODE>
<DD>
<A NAME="IDX689"></A>
If a term is currently stored under <VAR>Key</VAR>, 
a copy of it is unified with <VAR>Term</VAR>.  Otherwise, <CODE>bb_get/2</CODE>
silently fails.

<DT><CODE>bb_delete(<VAR>:Key</VAR>, <VAR>?Term</VAR>)</CODE>
<DD>
<A NAME="IDX690"></A>
If a term is currently stored under <VAR>Key</VAR>, the term is deleted, and
a copy of it is unified with <VAR>Term</VAR>.  Otherwise,
<CODE>bb_delete/2</CODE> silently fails.

<DT><CODE>bb_update(<VAR>:Key</VAR>, <VAR>?OldTerm</VAR>, <VAR>?NewTerm</VAR>)</CODE>
<DD>
<A NAME="IDX691"></A>
If a term is currently stored under <VAR>Key</VAR> and unifies with
<VAR>OldTerm</VAR>, the term is replaced by a copy of <VAR>NewTerm</VAR>.  Otherwise,
<CODE>bb_update/3</CODE> silently fails.  This predicate provides an atomic
swap operation.

</DL>

<P>
The following example illustrates how these primitives may be used to
implement a "maxof" predicate that finds the maximum value computed
by some non-deterministic goal, which may execute in parallel.  
We use a single key <CODE>max</CODE>.  Note the technique of using
<CODE>bb_update/3</CODE> in a repeat-fail loop, since other execution
branches may be competing for updating the value, and we only want
to store a new value if it is greater than the old value.

</P>
<P>
We assume that <VAR>Goal</VAR> does not produce any "false" solutions that
would be eliminated by cuts in a sequential execution.  Thus, <VAR>Goal</VAR> may
need to include redundant checks to ensure that its solutions are valid,
as discussed above.

</P>

<PRE>
maxof(Value, Goal, _) :-
        bb_put(max, -1),                % initialize max-so-far
        call(Goal),
        update_max(Value),
        fail.
maxof(_, _, Max) :-
        bb_delete(max, Max),
        Max &#62; 1.

update_max(New):-
        repeat,
          bb_get(max, Old),
          compare(C, Old, New),
          update_max(C, Old, New), !.

update_max(&#60;, Old, New) :- bb_update(max, Old, New).
update_max(=, _, _).
update_max(&#62;, _, _).
</PRE>



<H2><A NAME="SEC111" HREF="sicstus_toc.html#TOC111">All Solutions</A></H2>
<P>
<A NAME="IDX692"></A>
<A NAME="IDX693"></A>

</P>
<P>
When there are many solutions to a problem, and when all those solutions
are required to be collected together, this can be achieved by repeatedly
backtracking and gradually building up a list of the solutions.  The
following built-in predicates are provided to automate this process.

</P>
<P>
Note that the <VAR>Goal</VAR> argument to the predicates listed below is
called as if by <CODE>call/1</CODE> at runtime.  Thus if <VAR>Goal</VAR> is complex
and if performance is an issue, define an auxiliary predicate which
can then be compiled, and let <VAR>Goal</VAR> call it.

</P>
<DL COMPACT>

<DT><CODE>setof(<VAR>?Template</VAR>,<VAR>:Goal</VAR>,<VAR>?Set</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX694"></A>
Read this as "<VAR>Set</VAR> is the set of all instances of <VAR>Template</VAR>
such that <VAR>Goal</VAR> is satisfied, where that set is non-empty".  The
term <VAR>Goal</VAR> specifies a goal or goals as in <CODE>call(<VAR>Goal</VAR>)</CODE>
(see section <A HREF="sicstus_10.html#SEC103">Control</A>).  <VAR>Set</VAR> is a set of terms represented as a list of
those terms, without duplicates, in the standard order for terms
(see section <A HREF="sicstus_10.html#SEC102">Comparison of Terms</A>).  If there are no instances of <VAR>Template</VAR>
such that <VAR>Goal</VAR> is satisfied then the predicate fails.  
The variables appearing in the term <VAR>Template</VAR> should not appear
anywhere else in the clause except within the term <VAR>Goal</VAR>.
Obviously, the set to be enumerated should be finite, and should be
enumerable by Prolog in finite time.  It is possible for the provable
instances to contain variables, but in this case the list <VAR>Set</VAR> will
only provide an imperfect representation of what is in reality an
infinite set.  
If there are uninstantiated variables in <VAR>Goal</VAR> which do not also
appear in <VAR>Template</VAR>, then a call to this built-in predicate may
backtrack, generating alternative values for <VAR>Set</VAR> corresponding to
different instantiations of the free variables of <VAR>Goal</VAR>.  (It is to
cater for such usage that the set <VAR>Set</VAR> is constrained to be
non-empty.)  Two instantiations are different iff no renaming of
variables can make them literally identical.  For example, given the
clauses: 

<PRE>
likes(bill, cider).
likes(dick, beer).
likes(harry, beer).
likes(jan, cider).
likes(tom, beer).
likes(tom, cider).
</PRE>

the query


<PRE>
| ?- setof(X, likes(X,Y), S).
</PRE>

might produce two alternative solutions via backtracking:


<PRE>
S = [dick,harry,tom],
Y = beer ? ;

S = [bill,jan,tom],
Y = cider ? ;
</PRE>

The query:


<PRE>
| ?- setof((Y,S), setof(X, likes(X,Y), S), SS).
</PRE>

would then produce:  


<PRE>
SS = [(beer,[dick,harry,tom]),(cider,[bill,jan,tom])]
</PRE>

Variables occurring in <VAR>Goal</VAR> will not be treated as free if they are
explicitly bound within <VAR>Goal</VAR> by an existential quantifier.  An
existential quantification is written: 

<PRE>
<VAR>Y</VAR>^<VAR>Q</VAR>
</PRE>

meaning "there exists a <VAR>Y</VAR> such that <VAR>Q</VAR> is true", where
<VAR>Y</VAR> is some Prolog variable.  
For example:  


<PRE>
| ?- setof(X, Y^(likes(X,Y)), S).
</PRE>

would produce the single result:  


<PRE>
S = [bill,dick,harry,jan,tom]
</PRE>

in contrast to the earlier example.

<DT><CODE>bagof(<VAR>?Template</VAR>,<VAR>:Goal</VAR>,<VAR>?Bag</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX695"></A>
This is exactly the same as <CODE>setof/3</CODE> except that the list (or
alternative lists) returned will not be ordered, and may contain
duplicates.  The effect of this relaxation is to save a call to
<CODE>sort/2</CODE>, which is invoked by <CODE>setof/3</CODE> to return an ordered
list.  
<DT><CODE><VAR>?X</VAR>^<VAR>:P</VAR></CODE>
<DD>
<A NAME="IDX696"></A>
The all solution predicates recognize this as meaning "there exists an
<VAR>X</VAR> such that <VAR>P</VAR> is true", and treats it as equivalent to
<CODE><VAR>P</VAR></CODE> (see section <A HREF="sicstus_10.html#SEC103">Control</A>).  The use of this explicit existential
quantifier outside the <CODE>setof/3</CODE> and <CODE>bagof/3</CODE> constructs is
superfluous and discouraged.  
<DT><CODE>findall(<VAR>?Template</VAR>,<VAR>:Goal</VAR>,<VAR>?Bag</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX697"></A>
<VAR>Bag</VAR> is a list of instances of <VAR>Template</VAR> in all proofs of
<VAR>Goal</VAR> found by Prolog.  The order of the list corresponds to the
order in which the proofs are found.  The list may be empty and all
variables are taken as being existentially quantified.  This means that
each invocation of <CODE>findall/3</CODE> succeeds <EM>exactly once</EM>, and
that no variables in <VAR>Goal</VAR> get bound.  Avoiding the management of
universally quantified variables can save considerable time and space.

<DT><CODE>findall(<VAR>?Template</VAR>,<VAR>:Goal</VAR>,<VAR>?Bag</VAR>,<VAR>?Remainder</VAR>)</CODE>
<DD>
<A NAME="IDX698"></A>
Same as <CODE>findall/3</CODE>, except <VAR>Bag</VAR> is the list of solution
instances appended with <VAR>Remainder</VAR>, which is typically unbound.

</DL>



<H2><A NAME="SEC112" HREF="sicstus_toc.html#TOC112">Coroutining</A></H2>

<P>
<A NAME="IDX699"></A>
The coroutining facility can be accessed by a number of built-in
predicates.  This makes it possible to use coroutines in a dynamic way,
without having to rely on block declarations:

</P>
<DL COMPACT>

<DT><CODE>when(<VAR>+Condition</VAR>,<VAR>:Goal</VAR>)</CODE>
<DD>
<A NAME="IDX700"></A>
Blocks <VAR>Goal</VAR> until the <VAR>Condition</VAR> is true, where
<VAR>Condition</VAR> is a Prolog goal with the restricted syntax:

<DL COMPACT>

<DT><CODE>nonvar(<VAR>X</VAR>)</CODE>
<DD>
<DT><CODE>ground(<VAR>X</VAR>)</CODE>
<DD>
<DT><CODE>?=(<VAR>X</VAR>,<VAR>Y</VAR>)</CODE>
<DD>
<DT><CODE><VAR>Condition</VAR>,<VAR>Condition</VAR></CODE>
<DD>
<DT><CODE><VAR>Condition</VAR>;<VAR>Condition</VAR></CODE>
<DD>
</DL>

For example:


<PRE>
| ?- when(((nonvar(X);?=(X,Y)),ground(T)), process(X,Y,T)).
</PRE>

<DT><CODE>freeze(<VAR>?X</VAR>,<VAR>:Goal</VAR>)</CODE>
<DD>
<A NAME="IDX701"></A>
Blocks <VAR>Goal</VAR> until <CODE>nonvar(<VAR>X</VAR>)</CODE> (see section <A HREF="sicstus_10.html#SEC106">Meta-Logic</A>)
holds.  This is defined as if by:


<PRE>
freeze(X, Goal) :- when(nonvar(X), Goal).
</PRE>

or


<PRE>
:- block freeze(-, ?).
freeze(_, Goal) :- Goal.
</PRE>

<DT><CODE>frozen(<VAR>-Var</VAR>,<VAR>?Goal</VAR>)</CODE>
<DD>
<A NAME="IDX702"></A>
If some goal is blocked on the variable <VAR>Var</VAR>, or <VAR>Var</VAR> has
attributes that can be interpreted as a goal (see section <A HREF="sicstus_17.html#SEC191">Attributed Variables</A>),
then that goal is unified with <VAR>Goal</VAR>.  If no goals are blocked,
<VAR>Goal</VAR> is unified with the atom <CODE>true</CODE>.  If more than one
goal is blocked, a conjunction is unified with <VAR>Goal</VAR>.  
<DT><CODE>dif(<VAR>?X</VAR>,<VAR>?Y</VAR>)</CODE>
<DD>
<A NAME="IDX703"></A>
Constrains <VAR>X</VAR> and <VAR>Y</VAR> to represent different terms i.e. to be
non-unifiable.  Calls to <CODE>dif/2</CODE> either succeed, fail, or are
blocked depending on whether <VAR>X</VAR> and <VAR>Y</VAR> are sufficiently
instantiated.  It is defined as if by: 

<PRE>
dif(X, Y) :- when(?=(X,Y), X\==Y).
</PRE>

<DT><CODE>call_residue(<VAR>:Goal</VAR>,<VAR>?Residue</VAR>)</CODE>
<DD>
<A NAME="IDX704"></A>
The <VAR>Goal</VAR> is executed as if by <CODE>call/1</CODE>.  If after the
execution there are still some subgoals of <VAR>Goal</VAR> that are blocked
on some variables, then <VAR>Residue</VAR> is unified with a list of
<VAR>VariableSet</VAR>-<VAR>Goal</VAR> pairs, and those subgoals are no longer blocked
on any variables.  Otherwise, <VAR>Residue</VAR> is unified
with the empty list <CODE>[]</CODE>.

<VAR>VariableSet</VAR> is a set of variables such that when any of the
variables is bound, <VAR>Goal</VAR> gets unblocked.  Usually, a goal is
blocked on a single variable, in which case <VAR>VariableSet</VAR> is a
singleton.

<VAR>Goal</VAR> is an ordinary goal, sometimes module prefixed.
For example:


<PRE>
| ?- call_residue((dif(X,f(Y)), X=f(Z)), Res).

X = f(Z),
Res = [[Y,Z]-(prolog:dif(f(Z),f(Y)))]
</PRE>

</DL>



<H2><A NAME="SEC113" HREF="sicstus_toc.html#TOC113">Debugging</A></H2>

<P>
Debugging predicates are not available in Runtime Systems.

</P>
<DL COMPACT>

<DT><CODE>unknown(<VAR>?OldState</VAR>,<VAR>?NewState</VAR>)</CODE>
<DD>
<A NAME="IDX705"></A>
<VAR>OldState</VAR> is the current state of the "Action on unknown
predicates" flag, and sets the flag to <VAR>NewState</VAR>.  This flag
determines whether or not the system is to catch calls to undefined
predicates (see section <A HREF="sicstus_4.html#SEC20">Undefined Predicates</A>), when 
<CODE>user:unknown_predicate_handler/3</CODE> cannot handle the goal.
The possible states of the
flag are: 
<DL COMPACT>

<DT><CODE>trace</CODE>
<DD>
Causes calls to undefined predicates to be reported and the debugger to
be entered at the earliest opportunity.

<DT><CODE>fail</CODE>
<DD>
Causes calls to such predicates to fail.  
<DT><CODE>error</CODE>
<DD>
Causes calls to such predicates to raise an exception (the default).
See section <A HREF="sicstus_10.html#SEC104">Error and Exception Handling</A>.
</DL>

<DT><CODE>debug</CODE>
<DD>
<A NAME="IDX706"></A>
The debugger is switched on in <EM>leap</EM> mode.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>.  
<DT><CODE>trace</CODE>
<DD>
<A NAME="IDX707"></A>
The debugger is switched on in <EM>creep</EM> mode.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>.  
<DT><CODE>zip</CODE>
<DD>
<A NAME="IDX708"></A>
The debugger is switched on in <EM>zip</EM> mode.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>.  
<DT><CODE>nodebug</CODE>
<DD>
<DT><CODE>notrace</CODE>
<DD>
<DT><CODE>nozip</CODE>
<DD>
<A NAME="IDX709"></A>
<A NAME="IDX710"></A>
<A NAME="IDX711"></A>
The debugger is switched off.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>. 
<DT><CODE>leash(<VAR>+Mode</VAR>)</CODE>
<DD>
<A NAME="IDX712"></A>
Leashing Mode is set to <VAR>Mode</VAR>.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>.  
<DT><CODE>spy <VAR>:Spec</VAR></CODE>
<DD>
<A NAME="IDX713"></A>
Spy-points are placed on all the predicates given by <VAR>Spec</VAR>.
See section <A HREF="sicstus_9.html#SEC87">Spy-points</A>.  
<DT><CODE>nospy <VAR>:Spec</VAR></CODE>
<DD>
<A NAME="IDX714"></A>
Spy-points are removed from all the predicates given by <VAR>Spec</VAR>.
See section <A HREF="sicstus_9.html#SEC87">Spy-points</A>.  
<DT><CODE>nospyall</CODE>
<DD>
<A NAME="IDX715"></A>
Removes all the spy-points that have been set.  
<DT><CODE>debugging</CODE>
<DD>
<A NAME="IDX716"></A>
Displays information about the debugger.  See section <A HREF="sicstus_9.html#SEC86">Basic Debugging Predicates</A>.  </DL>



<H2><A NAME="SEC114" HREF="sicstus_toc.html#TOC114">Execution Profiling</A></H2>

<P>
<A NAME="IDX717"></A>
<A NAME="IDX718"></A>
<A NAME="IDX719"></A>
Execution profiling is a common aid for improving software performance.
The SICStus Prolog compiler has the capability of instrumenting compiled
code with <EM>counters</EM> which are initially zero and incremented
whenever the flow of control passes a given point in the compiled code.
This way the number of calls, backtracks, choicepoints created, etc.,
can be counted for the instrumented predicates, and an estimate of the
time spent in individual clauses and disjuncts can be calculated.

</P>
<P>
Gauge is a graphical user interface for inspecting execution profiles.
It is available as a library module (see section <A HREF="sicstus_38.html#SEC360">The Gauge Profiling Tool</A>).

</P>
<P>
The original version of the profiling package was written by
M.M. Gorlick and C.F. Kesselman at the Aerospace Corporation [Gorlick &#38;
Kesselman 87].

</P>
<P>
Profiling is not available in Muse.

</P>
<P>
Only compiled code can be instrumented.  To get an execution profile of
a program, the compiler must first be told to produce instrumented code.
This is done by issuing the directive:

</P>

<PRE>
| ?- prolog_flag(compiling,_,profiledcode).
</PRE>

<P>
after which the program to be analyzed can be compiled as usual.  Any
new compiled code will be instrumented while the compilation mode flag
has the value <CODE>profiledcode</CODE>.

</P>
<P>
The profiling data is generated by simply running the program.  The
predicate <CODE>profile_data/4</CODE> (see below) makes available a selection
of the data as a Prolog term.  The predicate <CODE>profile_reset/1</CODE>
zeroes the profiling counters for a selection of the currently
instrumented predicates.  
</P>

<DL COMPACT>

<DT><CODE>profile_data(<VAR>:Spec</VAR>,<VAR>?Selection</VAR>,<VAR>?Resolution</VAR>,<VAR>-Data</VAR>)</CODE>
<DD>
<A NAME="IDX720"></A>
<VAR>Data</VAR> is profiling data collected from the
predicates covered by <VAR>Spec</VAR>, which has the same form as for
<CODE>spy/1</CODE> (see section <A HREF="sicstus_9.html#SEC87">Spy-points</A>). 
The <VAR>Selection</VAR> argument determines the kind of profiling data to be
collected.  If uninstantiated, the predicate will backtrack over its
possible values, which are: 
<DL COMPACT>

<DT><CODE>calls</CODE>
<DD>
<A NAME="IDX721"></A>
All instances of entering a clause by a procedure call are counted.
This is equivalent to counting all procedure calls <EM>that have not
been determined to fail by indexing on the first argument</EM>.  
<DT><CODE>backtracks</CODE>
<DD>
<A NAME="IDX722"></A>
All instances of entering a clause by backtracking are counted.

<DT><CODE>choice_points</CODE>
<DD>
<A NAME="IDX723"></A>
All instances of creating a choicepoint are counted.  This occurs,
roughly, when the implementation determines that there are more than one
possibly matching clauses for a procedure call, and when a disjunction
is entered.

<DT><CODE>shallow_fails</CODE>
<DD>
<A NAME="IDX724"></A>
<A NAME="IDX725"></A>
<A NAME="IDX726"></A>
Failures in the "if" part of if-then-else statements, and in the
"guard" part of guarded clauses, are counted as <EM>shallow
failures</EM>.  See section <A HREF="sicstus_13.html#SEC177">If-Then-Else Compilation</A>.

<DT><CODE>deep_fails</CODE>
<DD>
<A NAME="IDX727"></A>
<A NAME="IDX728"></A>
<A NAME="IDX729"></A>
Any failures that do not classify as shallow as above are counted as
<EM>deep failures</EM>.  The reason for distinguishing shallow and deep
failures is that the former are considerably cheaper to execute than the
latter.

<DT><CODE>execution_time</CODE>
<DD>
<A NAME="IDX730"></A>
The execution time for the selected predicates, clauses, or disjuncts is
estimated in artificial units.
</DL>

The <VAR>Resolution</VAR> argument determines the level of resolution of the
profiling data to be collected.  If uninstantiated, the predicate will
backtrack over its possible values, which are: 
<DL COMPACT>

<DT><CODE>predicate</CODE>
<DD>
<VAR>Data</VAR> is a list of <CODE><VAR>Module:PredName</VAR>-<VAR>Count</VAR></CODE>, where
<VAR>Count</VAR> is a sum of the corresponding counts per clause.

<DT><CODE>clause</CODE>
<DD>
<VAR>Data</VAR> is a list of <CODE><VAR>Module:ClauseName</VAR>-<VAR>Count</VAR></CODE>, where
<VAR>Count</VAR> includes counts for any disjunctions occurring inside that
clause.  Note, however, that the selections <CODE>calls</CODE> and
<CODE>backtracks</CODE> do <EM>not</EM> include counts for disjunctions.

<DT><CODE>all</CODE>
<DD>
<VAR>Data</VAR> is a list of <CODE><VAR>Module:InternalName</VAR>-<VAR>Count</VAR></CODE>.
This is the finest resolution level, counting individual clauses and
disjuncts.
</DL>

Above, <VAR>PredName</VAR> is a predicate spec, <VAR>ClauseName</VAR> is a compound
term <CODE><VAR>PredName</VAR>/<VAR>ClauseNumber</VAR></CODE>, and <VAR>InternalName</VAR> is
either<BR>
<VAR>ClauseName</VAR>---corresponding to a clause, or<BR>
<CODE>(<VAR>ClauseName</VAR>-<VAR>DisjNo</VAR>)/<VAR>Arity</VAR>/<VAR>AltNo</VAR></CODE>---corresponding
to a disjunct.

<DT><CODE>profile_reset(<VAR>:Spec</VAR>)</CODE>
<DD>
<A NAME="IDX731"></A>
Zeroes all counters for predicates covered by <VAR>Spec</VAR>,
which has the same form as for <CODE>spy/1</CODE> (see section <A HREF="sicstus_9.html#SEC87">Spy-points</A>). </DL>



<H2><A NAME="SEC115" HREF="sicstus_toc.html#TOC115">Muse</A></H2>

<P>
The following predicates are only defined in the Muse Development System.

</P>
<DL COMPACT>

<DT><CODE>muse_sync</CODE>
<DD>
<A NAME="IDX732"></A>

A built-in predicate that can be used to synchronize for being leftmost
in the parallel search tree.  Its main use is in combination with
foreign language interface calls.

<DT><CODE>muse_flags</CODE>
<DD>
<A NAME="IDX733"></A>

This built-in predicate lists all current Muse flag settings.

<A NAME="IDX734"></A>
<A NAME="IDX735"></A>
<DT><CODE>muse_flag(<VAR>?Flag</VAR>,<VAR>-Old</VAR>)</CODE>
<DD>
<DT><CODE>muse_flag(<VAR>?Flag</VAR>,<VAR>-Old</VAR>,<VAR>+New</VAR>)</CODE>
<DD>
These examine and/or adjust parts of the Muse scheduler state.
The current value is returned in <VAR>Old</VAR>, and a new value may be
specified in <VAR>New</VAR>.

The state cannot be changed when Muse is executing parallel work.
It is therefore recommended to perform such changes
as the first (or best single) goal in a top level query.  If used
elsewhere,  the execution may be aborted if the system cannot perform
the change.

Possible flag values are:

<DL COMPACT>

<DT><CODE>num_workers</CODE>
<DD>
<A NAME="IDX736"></A>
Returns in <VAR>Old</VAR> the current number of workers,
which is adjusted to <VAR>New</VAR>.

<DT><CODE>max_workers</CODE>
<DD>
<A NAME="IDX737"></A>
The maximum number of Muse workers is returned.  
This flag is read-only.

<DT><CODE>worker_id</CODE>
<DD>
<A NAME="IDX738"></A>
The identity of the currently executing Muse workers is returned.  
This flag is read-only.

<DT><CODE>version</CODE>
<DD>
<A NAME="IDX739"></A>
The Muse version is returned.  The version can be <CODE>plain</CODE> or
<CODE>trace</CODE>.  This flag is read-only.
</DL>

The following flags also exist, but understanding them requires deep
knowledge about the Muse scheduler and execution model, so they are
probably not useful for the average user.

<DL COMPACT>

<DT><CODE>vol_susp</CODE>
<DD>
Used to turn voluntary suspension on (1) or off (0).

<DT><CODE>vol_susp_time</CODE>
<DD>
Voluntary suspension interval, in milliseconds,

<DT><CODE>vol_susp_num</CODE>
<DD>
Number of workers to voluntarily suspend.

<DT><CODE>vol_susp_avoid</CODE>
<DD>
Controls the eagerness of voluntary suspension after resumption.

<DT><CODE>eager_cut</CODE>
<DD>
Used to turn eager pruning on (1) or off (0).

<DT><CODE>max_sch_loops</CODE>
<DD>
Maximum number of scheduler loops.

<DT><CODE>max_suspended</CODE>
<DD>
Maximum number of megabytes for suspended branches.

</DL>

<A NAME="IDX740"></A>
<A NAME="IDX741"></A>
<DT><CODE>muse_trace(<VAR>:Goal</VAR>)</CODE>
<DD>
<DT><CODE>muse_trace(<VAR>:Goal</VAR>,<VAR>+FilePrefix</VAR>)</CODE>
<DD>
<A NAME="IDX742"></A>
The trace version generates a trace file for the goal <VAR>Goal</VAR>.  The
file may be prefixed with <VAR>FilePrefix</VAR>.  The file is created in
the current working directory if it can be written there, or otherwise
in the directory that is the value of the <CODE>TMPDIR</CODE> environment
variable.

Cannot be executed in parallel when the execution changes the scheduler
state.

</DL>



<H2><A NAME="SEC116" HREF="sicstus_toc.html#TOC116">Miscellaneous</A></H2>

<DL COMPACT>

<DT><CODE><VAR>?X</VAR> = <VAR>?Y</VAR> <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX743"></A>
Defined as if by the clause <CODE>Z=Z.</CODE>; i.e. <VAR>X</VAR> and <VAR>Y</VAR> are
unified.  
<DT><CODE>length(<VAR>?List</VAR>,<VAR>?Length</VAR>)</CODE>
<DD>
<A NAME="IDX744"></A>
If <VAR>List</VAR> is instantiated to a list of determinate length, then
<VAR>Length</VAR> will be unified with this length.  
If <VAR>List</VAR> is of indeterminate length and <VAR>Length</VAR> is
instantiated to an integer, then <VAR>List</VAR> will be unified with a list
of length <VAR>Length</VAR>.  The list elements are unique variables.  
If <VAR>Length</VAR> is unbound then <VAR>Length</VAR> will be unified with all
possible lengths of <VAR>List</VAR>.  
<DT><CODE>numbervars(<VAR>?Term</VAR>,<VAR>+N</VAR>,<VAR>?M</VAR>)</CODE>
<DD>
<A NAME="IDX745"></A>
Unifies each of the variables in term <VAR>Term</VAR> with a special term, so
that <CODE>write(<VAR>Term</VAR>)</CODE> (or <CODE>writeq(<VAR>Term</VAR>)</CODE>)
(see section <A HREF="sicstus_10.html#SEC96">Input and Output of Terms</A>) prints those variables as (<KBD>A</KBD> + (<VAR>i</VAR> mod
26))(<VAR>i</VAR>/26) where <VAR>i</VAR> ranges from <VAR>N</VAR> to <VAR>M</VAR>-1.
<VAR>N</VAR> must be instantiated to an integer.  If it is 0 you get the
variable names A, B, ..., Z, A1, B1, etc.  This predicate is used by
<CODE>listing/(0-1)</CODE> (see section <A HREF="sicstus_10.html#SEC105">Information about the State of the Program</A>).  
<DT><CODE>halt <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX746"></A>
Causes Prolog to exit back to the shell.  (In recursive calls to Prolog
from C, this predicate will return back to C instead.)

<DT><CODE>halt(<VAR>+Code</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX747"></A>
Causes the Prolog process to immediately exit back to the shell with the
return code <VAR>Code</VAR>, even if it occurs in a recursive call from C.

<DT><CODE>op(<VAR>+Precedence</VAR>,<VAR>+Type</VAR>,<VAR>+Name</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX748"></A>
<A NAME="IDX749"></A>
Declares the atom <VAR>Name</VAR> to be an operator of the stated <VAR>Type</VAR>
and <VAR>Precedence</VAR> (see section <A HREF="sicstus_5.html#SEC49">Operators</A>).  <VAR>Name</VAR> may also be a list
of atoms in which case all of them are declared to be operators.  If
<VAR>Precedence</VAR> is 0 then the operator properties of <VAR>Name</VAR> (if
any) are cancelled.  
<DT><CODE>current_op(<VAR>?Precedence</VAR>,<VAR>?Type</VAR>,<VAR>?Op</VAR>) <EM>[ISO]</EM></CODE>
<DD>
<A NAME="IDX750"></A>
The atom <VAR>Op</VAR> is currently an operator of type <VAR>Type</VAR> and
precedence <VAR>Precedence</VAR>.  Neither <VAR>Op</VAR> nor the other arguments
need be instantiated at the time of the call; i.e. this predicate can
be used to generate as well as to test.  
<DT><CODE>break</CODE>
<DD>
<A NAME="IDX751"></A>
Invokes a recursive top-level.  See section <A HREF="sicstus_4.html#SEC23">Nested Executions--Break and Abort</A>.
(This predicate is not available in Runtime Systems nor in Muse.)

<DT><CODE>abort</CODE>
<DD>
<A NAME="IDX752"></A>
Aborts the current execution.  See section <A HREF="sicstus_4.html#SEC23">Nested Executions--Break and Abort</A>.  (In recursive calls to Prolog
from C, this predicate will return back to C instead.) (In Muse,
will adjust the system to one worker.)

<DT><CODE>save_program(<VAR>+File</VAR>)</CODE>
<DD>
<DT><CODE>save_program(<VAR>+File</VAR>, <VAR>:Goal</VAR>)</CODE>
<DD>
<A NAME="IDX753"></A>
<A NAME="IDX754"></A>
The system saves the program state into file <VAR>File</VAR>.
When the program state is restored, <VAR>Goal</VAR> is executed.
<VAR>Goal</VAR> defaults to <CODE>true</CODE>.
See section <A HREF="sicstus_4.html#SEC24">Saving and Restoring Program States</A>. (Requires Muse to be adjusted to one worker.)

<DT><CODE>restore(<VAR>+File</VAR>)</CODE>
<DD>
<A NAME="IDX755"></A>
The system is returned to the program state previously saved to file
<VAR>File</VAR> with start-up goal <VAR>Goal</VAR>.  <CODE>restore/1</CODE> may succeed,
fail or raise an exception depending on <VAR>Goal</VAR>.  See section <A HREF="sicstus_4.html#SEC24">Saving and Restoring Program States</A>.
(Requires Muse to be adjusted to one worker.)

<DT><CODE>reinitialise</CODE>
<DD>
<A NAME="IDX756"></A>
<A NAME="IDX757"></A>
This predicate can be used to force the reinitialization behavior to take
place at any time.  When SICStus Prolog is reinitialized it:

<UL>
<LI>

calls any user defined C function pointed at by <CODE>SP_reinit_hook</CODE>
(see section <A HREF="sicstus_11.html#SEC150">Hooks</A>).
<LI>

calls <CODE>version/0</CODE> to write banners.
% @item 
% looks for a file <TT>`~/.sicstusrc'</TT> or <TT>`~/.sicstus.ini'</TT> and
% consults it, if it exists.
% @item
% ensures a file <VAR>File</VAR> is loaded if <CODE>-l <VAR>File</VAR></CODE> was supplied as
% program arguments.
<LI>

calls <CODE>initialization/0</CODE> to run user-defined initializations.
</UL>

(In recursive calls to Prolog
from C, this predicate will return back to C instead.)
(In Muse, will adjust the system to one worker.)

<DT><CODE>garbage_collect</CODE>
<DD>
<A NAME="IDX758"></A>
Performs a garbage collection of the global stack immediately.  
<DT><CODE>garbage_collect_atoms</CODE>
<DD>
<A NAME="IDX759"></A>
Performs a garbage collection of the atoms immediately.  
<DT><CODE>gc</CODE>
<DD>
<A NAME="IDX760"></A>
Enables garbage collection of the global stack (the default).  
<DT><CODE>nogc</CODE>
<DD>
<A NAME="IDX761"></A>
Disables garbage collection of the global stack.

<DT><CODE>prompt(<VAR>?Old</VAR>,<VAR>?New</VAR>)</CODE>
<DD>
<A NAME="IDX762"></A>
The sequence of characters (prompt) which indicates that the system is
waiting for user input is represented as an atom, and unified with
<VAR>Old</VAR>; the atom bound to New specifies the new prompt.  In
particular, the goal <CODE>prompt(X, X)</CODE> unifies the current prompt with
<VAR>X</VAR>, without changing it.  Note that this predicate only affects the
prompt given when a user's program is trying to read from the standard input stream
(e.g. by calling <CODE>read/1</CODE>).  Note also that the prompt is reset to
the default <SAMP>`|: '</SAMP> on return to top-level.  
<DT><CODE>version</CODE>
<DD>
<A NAME="IDX763"></A>
<A NAME="IDX764"></A>
Displays the introductory messages for all the component parts of the
current system. 
Prolog will display its own introductory message when initially run and
on reinitialization by calling <CODE>version/0</CODE>.  If this message is
required at some other time it can be obtained using this predicate
which displays a list of introductory messages; initially this list
comprises only one message (Prolog's), but you can add more messages
using <CODE>version/1</CODE>. (This predicate is not available in Runtime
Systems.)

<DT><CODE>version(<VAR>+Message</VAR>)</CODE>
<DD>
<A NAME="IDX765"></A>
Appends <VAR>Message</VAR> to the end of the message list which is output by
<CODE>version/0</CODE>. <VAR>Message</VAR> must be an atom.
(This predicate is not available in Runtime Systems.)

The idea of this message list is that, as systems are constructed on top
of other systems, each can add its own identification to the message
list.  Thus <CODE>version/0</CODE> should always indicate which modules make
up a particular package.  It is not possible to remove messages from the
list.

<DT><CODE>initialization</CODE>
<DD>
<A NAME="IDX766"></A>
<A NAME="IDX767"></A>
Executes goals defined by <CODE>initialization/1</CODE>. Only the first
solution is investigated. <CODE>initialization/0</CODE> is called at system
(re)initialization.  (This predicate is not available in Runtime Systems.)

<DT><CODE>initialization(<VAR>:Goal</VAR>)</CODE>
<DD>
<A NAME="IDX768"></A>
Appends <VAR>Goal</VAR> to the goal list which is executed by
<CODE>initialization/0</CODE>. It is not possible to remove goals from the
list. (This predicate is not available in Runtime Systems.) 
<CODE>initialization/(0-1)</CODE> provide a mechanism which is similar but
more general than <CODE>version/(0-1)</CODE>. It can, for example, be used by
systems constructed on top of SICStus Prolog to load their own
initialization files.

<DT><CODE>help</CODE>
<DD>
<A NAME="IDX769"></A>
<EM>Hookable</EM>, displays basic information, or a user defined help
message.  It first calls <CODE>user:user_help/0</CODE>, and only if that call
fails is a default help message printed on the current output stream.
(This predicate is not available in Runtime Systems.) 
<DT><CODE>user_help</CODE>
<DD>
<DT><CODE>user:user_help</CODE>
<DD>
<A NAME="IDX770"></A>
<A NAME="IDX771"></A>
<EM>A hook predicate.</EM> This may be defined by the user to
print a help message on the current output stream. </DL>

<P><HR><P>
Go to the <A HREF="sicstus_1.html">first</A>, <A HREF="sicstus_9.html">previous</A>, <A HREF="sicstus_11.html">next</A>, <A HREF="sicstus_50.html">last</A> section, <A HREF="sicstus_toc.html">table of contents</A>.
</BODY>
</HTML>
