<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 5.4</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-5.3.html">
<LINK REL=next HREF="sec-5.5.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-5.3.html">Previous</A>
<A HREF="sec-5.5.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:5.4">5.4 Using the library shlib for <CODE>.DLL</CODE> 
and <CODE>.so</CODE> files</A></H2>

<A NAME="sec:DLL"></A>
<A NAME="sec:shlib"></A>

<P>This section discusses the functionality of the (autoload) library
<CODE>shlib.pl</CODE>, providing an interface to shared libraries. 
Currently it supports MS-Windows DLL (<CODE>.DLL</CODE>) libraries and 
Unix
<CODE>.so</CODE> (shared object) files.

<P>

<DL>

<P>
<DT><A NAME="load_foreign_library/1"><STRONG>load_foreign_library</STRONG>(<VAR>+Lib</VAR>)</A><DD>
Equivalent to load_foreign_library(Lib, install).

<P>
<DT><A NAME="load_foreign_library/2"><STRONG>load_foreign_library</STRONG>(<VAR>+Lib, 
+Entry</VAR>)</A><DD>
Search for the given foreign library and link it to the current 
SWI-Prolog instance. The library may be specified with or without the 
extension. First, <A NAME="idx:absolute_file_name/3:591"></A><A HREF="sec-3.35.html#absolute_file_name/3">absolute_file_name/3</A> 
is used to locate the file. If this succeeds, the full path is passed to 
the low-level function to open the library. Otherwise, the plain library 
name is passed, exploiting the operating-system defined search mechanism 
for the shared library. The <A NAME="idx:file_search_path/2:592"></A><A HREF="sec-3.2.html#file_search_path/2">file_search_path/2</A> 
alias mechanism defines the alias <CODE>foreign</CODE>, which refers to 
the directories <CODE>&lt;<VAR>plhome</VAR>&gt;/lib/&lt;<VAR>arch</VAR>&gt;</CODE> 
and
<CODE>&lt;<VAR>plhome</VAR>&gt;/lib</CODE>, in this order.

<P>If the library can be loaded, the function called <VAR>Entry</VAR> 
will be called without arguments. The return value of the function is 
ignored.

<P>The <VAR>Entry</VAR> function will normally call <A HREF="sec-5.6.html#PL_register_foreign()">PL_register_foreign()</A> 
to declare functions in the library as foreign predicates.

<P>
<DT><A NAME="unload_foreign_library/1"><STRONG>unload_foreign_library</STRONG>(<VAR>+Lib</VAR>)</A><DD>
If the foreign library defines the function uninstall(), this function 
will be called without arguments and its return value is ignored. Next, <A NAME="idx:abolish/2:593"></A><A HREF="sec-3.12.html#abolish/2">abolish/2</A> 
is used to remove all known foreign predicates defined in the library. 
Finally the library itself is detached from the process.

<P>
<DT><A NAME="current_foreign_library/2"><STRONG>current_foreign_library</STRONG>(<VAR>-Lib, 
-Predicates</VAR>)</A><DD>
Query the currently loaded foreign libraries and their predicates. </EM><EM>Predicates</EM> 
is a list with elements of the form </EM><EM>Module:Head</EM>, 
indicating the predicates installed with <A HREF="sec-5.6.html#PL_register_foreign()">PL_register_foreign()</A> 
when the entry-point of the library was called.
</DL>

<P><A HREF="sec-5.4.html#fig:msgbox">Figure 5</A> connects a Windows 
message-box using a foreign function. This example was tested using 
Windows&nbsp;NT and Microsoft Visual C++ 2.0.

<P>

<XMP>
#include <windows.h>
#include <SWI-Prolog.h>

static foreign_t
pl_say_hello(term_t to)
{ char *a;

  if ( PL_get_atom_chars(to, &a) )
  { MessageBox(NULL, a, "DLL test", MB_OK|MB_TASKMODAL);

    PL_succeed;
  }

  PL_fail;
}

install_t
install()
{ PL_register_foreign("say_hello", 1, pl_say_hello, 0);
}
</XMP>

<P><CENTER><B>Figure 5 : MessageBox() example in Windows&nbsp;NT</B></CENTER>
<A NAME="fig:msgbox"></A>

<P>

<H3><A NAME="sec:5.4.1">5.4.1 Static Linking</A></H3>

<A NAME="sec:staticl"></A>

<P>Below is an outline of the files structure required for statically 
linking SWI-Prolog with foreign extensions. <CODE>... /pl</CODE> refers 
to the SWI-Prolog home directory (see <A NAME="idx:feature/2:594"></A><A HREF="sec-2.8.html#feature/2">feature/2</A>). <CODE>&lt;<VAR>arch</VAR>&gt;</CODE> 
refers to the architecture identifier that may be obtained using <A NAME="idx:feature/2:595"></A><A HREF="sec-2.8.html#feature/2">feature/2</A>.

<P><CENTER>
<TABLE BORDER=0 COLS=2 FRAME=void RULES=group>
<TR VALIGN=top><TD><CODE>... /pl/runtime/&lt;<VAR>arch</VAR>&gt;/libpl.a</CODE> <TD>SWI-Library </TR>
<TR VALIGN=top><TD><CODE>... /pl/include/SWI-Prolog.h</CODE> <TD>Include 
file </TR>
<TR VALIGN=top><TD><CODE>... /pl/include/SWI-Stream.h</CODE> <TD>Stream 
I/O include file </TR>
<TR VALIGN=top><TD><CODE>... /pl/include/SWI-Exports</CODE> <TD>Export 
declarations (AIX only) </TR>
<TR VALIGN=top><TD><CODE>... /pl/include/stub.c</CODE> <TD>Extension 
stub</TR>
</TABLE>

</CENTER>

<P>The definition of the foreign predicates is the same as for dynamic 
linking. Unlike with dynamic linking however, there is no initialisation 
function. Instead, the file <CODE>... /pl/include/stub.c</CODE> may be 
copied to your project and modified to define the foreign extensions. 
Below is stub.c, modified to link the lowercase example described later 
in this chapter:

<P>

<XMP>
/*  Copyright (c) 1991 Jan Wielemaker. All rights reserved.
    jan@swi.psy.uva.nl

    Purpose: Skeleton for extensions
*/

#include <stdio.h>
#include <SWI-Prolog.h>

extern foreign_t pl_lowercase(term, term);

PL_extension predicates[] =
{
/*{ "name",      arity,  function,      PL_FA_<flags> },*/

  { "lowercase", 2       pl_lowercase,  0 },
  { NULL,        0,      NULL,          0 }     /* terminating line */
};


int
main(int argc, char **argv)
{ PL_register_extensions(predicates);

  if ( !PL_initialise(argc, argv) )
    PL_halt(1);

  PL_install_readline();                /* delete if not required */

  PL_halt(PL_toplevel() ? 0 : 1);
}
</XMP>

<P>Now, a new executable may be created by compiling this file and 
linking it to libpl.a from the runtime directory and the libraries 
required by both the extensions and the SWI-Prolog kernel. This may be 
done by hand, or using the <B>plld</B> utility described in secrefplld.

<P>

<H3><A NAME="sec:5.4.2">5.4.2 Dynamic Linking based on <A NAME="idx:load_foreign/[2,5]:596"></A><A HREF="sec-5.4.html#load_foreign/2">load_foreign/[2,5]</A></A></H3>

<P>The predicates below are considered obsolete. They are briefly 
described here for compatibility purposes. New code should use the 
predicates from the <CODE>library(shlib)</CODE>.

<P>

<DL>

<P>
<DT><A NAME="load_foreign/2"><STRONG>load_foreign</STRONG>(<VAR>+File, 
+Entry</VAR>)</A><DD>
Load a foreign file or list of files specified by <VAR>File</VAR>. The 
files are searched for similar to <A NAME="idx:consult/1:597"></A><A HREF="sec-3.2.html#consult/1">consult/1</A>. 
Except that the `<CODE>.o</CODE>' extension is used rather than `<CODE>.pl</CODE>'.

<P><VAR>Entry</VAR> defines the entry point of the resulting executable. 
The entry point will be called by Prolog to install the foreign 
predicates.

<P>
<DT><A NAME="load_foreign/2"><STRONG>load_foreign</STRONG>(<VAR>+File, 
+Entry, +Options, +Libraries, +Size</VAR>)</A><DD>
The first two arguments are identical to those of <A NAME="idx:load_foreign/2:598"></A><A HREF="sec-5.4.html#load_foreign/2">load_foreign/2</A>. </EM><EM>Options</EM> 
is (a list of) additional option to be given to the loader. The options 
are inserted just before the files. <VAR>Libraries</VAR> is (a list of) 
libraries to be passed to the loader. They are inserted just after the 
files. If <VAR>Size</VAR> is specified Prolog first assumes that the 
resulting executable will fit in <VAR>Size</VAR> bytes and do the 
loading in one pass.

<P>
<DT><A NAME="foreign_file/1"><STRONG>foreign_file</STRONG>(<VAR>?File</VAR>)</A><DD>
Is true if <VAR>File</VAR> is the absolute path name of a file loaded as 
foreign file.
</DL>

