<HTML><HEAD><TITLE>SWI-Prolog Reference Manual: Section 6.3</TITLE><LINK REL=home HREF="Title.html">
<LINK REL=contents HREF="Contents.html">
<LINK REL=index HREF="Index.html">
<LINK REL=summary HREF="sec-B.html">
<LINK REL=previous HREF="sec-6.2.html">
<LINK REL=next HREF="sec-6.4.html">
</HEAD>

<BODY>

<HR><CENTER><A HREF="Title.html">Home</A>
<A HREF="Contents.html">Contents</A>
<A HREF="Index.html">Index</A>
<A HREF="sec-B.html">Summary</A>
<A HREF="sec-6.2.html">Previous</A>
<A HREF="sec-6.4.html">Next</A>
</CENTER>

<HR>

<H2><A NAME="sec:6.3">6.3 Using Program Resources</A></H2>

<P>A <EM>resource</EM> is very similar to a file. Resources however can 
be represented in two different formats: on files, as well as part of 
the resource <EM>archive</EM> of a saved-state (see <A NAME="idx:qsave_program/2:645"></A><A HREF="sec-6.html#qsave_program/2">qsave_program/2</A>).

<P>A resource has a <EM>name</EM> and a <EM>class</EM>. The <EM>source</EM> 
data of the resource is a file. Resources are declared by declaring the 
predicate <A NAME="idx:resource/3:646"></A><A HREF="sec-6.3.html#resource/3">resource/3</A>. 
They are accessed using the predicate
<A NAME="idx:open_resource/3:647"></A><A HREF="sec-6.3.html#open_resource/3">open_resource/3</A>.

<P>Before going into details, let us start with an example. Short texts 
can easily be expressed in Prolog sourcecode, but long texts are 
cumbersome. Assume our application defines a command `help' that prints 
a helptext to the screen. We put the content of the helptext into a file 
called <CODE>help.txt</CODE>. The following code implements our help 
command such that help.txt is incorperated into the runtime executable.

<P>

<XMP>
resource(help, text, 'help.txt').

help :-
        open_resource(help, text, In),
        copy_stream(In, user_output),
        close(In).

copy_stream(In, Out) :-
        get0(In, C),
        copy_stream(C, In, Out).

copy_stream(-1, _, _) :- !.
        copy_stream(C, In, Out) :-
        put(Out, C),
        get0(In, C2),
        copy_stream(C2, In, Out).
</XMP>

<P>The predicate <A NAME="idx:help/0:648"></A><A HREF="sec-2.3.html#help/0">help/0</A> 
opens the resource as a Prolog stream. If we are executing this from the 
development environment, this will actually return a stream to the <CODE>gelp.txt</CODE> 
itself. When executed from the saved-state, the stream will actually be 
a stream opened on the program resource file, taking care of the offset 
and length of the resource.

<P>

<H3><A NAME="sec:6.3.1">6.3.1 Predicates Definitions</A></H3>

<P>

<DL>

<P>
<DT><A NAME="resource/3"><STRONG>resource</STRONG>(<VAR>+Name, +Class, 
+FileSpec</VAR>)</A><DD>
This predicate is defined as a dynamic predicate in the module
<CODE>user</CODE>. Clauses for it may be defined in any module, 
including the user module. <VAR>Name</VAR> is the name of the resource 
(an atom). A resource name may contain all characters, except for $ and 
:, which are reserved for internal usage by the resource library. <VAR>Class</VAR> 
describes the what kind of object we are dealing with. In the current 
implementation, it is just an atom. <VAR>FileSpec</VAR> is a file 
specification that may exploit <A NAME="idx:file_search_path/2:649"></A><A HREF="sec-3.2.html#file_search_path/2">file_search_path/2</A> 
(see
<A NAME="idx:absolute_file_name/2:650"></A><A HREF="sec-3.35.html#absolute_file_name/2">absolute_file_name/2</A>).

<P>Normally, resources are defined as unit clauses (facts), but the 
definition of this predicate can also imply rules. For proper generation 
of the saved state generation, it must be possible to enumerate the 
available resources by calling this predicate with all its arguments 
unbound.

<P>Dynamic rules can be useful to turn all files in a certain directory 
into resources, without specifying a resources for each file. For 
example, assume the <A NAME="idx:file_search_path/2:651"></A><A HREF="sec-3.2.html#file_search_path/2">file_search_path/2</A> <CODE>icons</CODE> 
refers to the resource directory containing (XPM) icons. The following 
definition makes all these images available as resources:

<P>

<XMP>
resource(Name, image, icons(XpmName)) :-
        atom(Name), !,
        file_name_extension(Name, xpm, XpmName).
resource(Name, image, XpmFile) :-
        var(Name),
        absolute_file_name(icons(.), [type(directory)], Dir)
        concat(Dir, '/*.xpm', Pattern),
        expand_file_name(Pattern, XpmFiles),
        member(XpmFile, XpmFiles).
</XMP>

<P>

<P>
<DT><A NAME="open_resource/3"><STRONG>open_resource</STRONG>(<VAR>+Name, 
?Class, -Stream</VAR>)</A><DD>
Opens the resource specified by <VAR>Name</VAR> and <VAR>Class</VAR>. If 
the latter is a variable, it will be unified to the class of the first 
resource found that has tehe specified <VAR>Name</VAR>. If successful,
<VAR>Stream</VAR> becomes a handle to a binary input stream, providing 
access to the content of the resource.

<P>The predicate <A NAME="idx:open_resource/3:652"></A><A HREF="sec-6.3.html#open_resource/3">open_resource/3</A> 
first checks <A NAME="idx:resource/3:653"></A><A HREF="sec-6.3.html#resource/3">resource/3</A>. 
When succesful it will open the returned resource source-file. Otherwise 
it will look in the programs resource database. When creating a 
saved-state, the system normally saves the resource contents into the 
resource archive, but does not save the resource clauses.

<P>This way, the development environment uses the files (and 
modifications to the <A NAME="idx:resource/3:654"></A><A HREF="sec-6.3.html#resource/3">resource/3</A> 
declarations and/or files containing resource info thus immediately 
affect the running environment, while the runtime system quickly 
accesses the system resources.
</DL>

